#ifndef _EVENT_DATA_OBJECTS_H_
#define _EVENT_DATA_OBJECTS_H_
/* Copyright (c) 2004, 2016, Oracle and/or its affiliates. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA */

/**
  @addtogroup Event_Scheduler
  @{

  @file event_data_objects.h
*/

#include "my_global.h"
#include "my_alloc.h"                   // MEM_ROOT
#include "my_time.h"                    // interval_type
#include "mysql/mysql_lex_string.h"     // LEX_STRING

#include "my_thread.h"                  // Needed for psi.h
#include "pfs_stage_provider.h"
#include "mysql/psi/mysql_stage.h"

#include "pfs_statement_provider.h"
#include "mysql/psi/mysql_statement.h"

class Field;
class String;
class THD;
class Time_zone;
struct TABLE;
typedef ulonglong sql_mode_t;
typedef struct st_mysql_lex_string LEX_STRING;
typedef struct st_mysql_const_lex_string LEX_CSTRING;
namespace dd
{
  class Event;
}

void init_scheduler_psi_keys(void);

class Event_queue_element_for_exec
{
public:
  Event_queue_element_for_exec(){};
  ~Event_queue_element_for_exec();

  bool init(LEX_STRING dbname, LEX_STRING name);

  LEX_STRING dbname;
  LEX_STRING name;
  bool dropped;
  THD *thd;

  void claim_memory_ownership();

  /* Prevent use of these */
  Event_queue_element_for_exec(const Event_queue_element_for_exec &)= delete;
  void operator=(Event_queue_element_for_exec &)= delete;

#ifdef HAVE_PSI_INTERFACE
  PSI_statement_info* get_psi_info()
  {
    return & psi_info;
  }

  static PSI_statement_info psi_info;
#endif
};


class Event_basic
{
protected:
  MEM_ROOT mem_root;

public:
  LEX_STRING  m_schema_name;
  LEX_STRING  m_event_name;
  LEX_STRING  m_definer;

  Time_zone  *m_time_zone;
  Event_basic();
  virtual ~Event_basic();
  virtual bool
  fill_event_info(THD *thd, const dd::Event &ev_obj,
                  const char *dbname) = 0;
};



class Event_queue_element : public Event_basic
{
public:
  int m_on_completion;
  int m_status;
  longlong m_originator;

  my_time_t m_last_executed;
  my_time_t m_execute_at;
  my_time_t m_starts;
  my_time_t m_ends;
  my_bool m_starts_null;
  my_bool m_ends_null;
  my_bool m_execute_at_null;

  longlong m_expression;
  interval_type m_interval;

  bool m_dropped;

  uint m_execution_count;

  Event_queue_element();
  virtual ~Event_queue_element();
  virtual bool fill_event_info(THD *thd, const dd::Event &event,
                               const char *dbname);

  bool compute_next_execution_time(THD *thd);

  void mark_last_executed(THD *thd);
};


class Event_timed : public Event_queue_element
{
public:
  LEX_STRING m_definition;

  LEX_CSTRING m_definer_user;
  LEX_CSTRING m_definer_host;

  LEX_STRING m_comment;

  ulonglong m_created;
  ulonglong m_modified;

  sql_mode_t m_sql_mode;

  class Stored_program_creation_ctx *m_creation_ctx;
  LEX_STRING m_definition_utf8;
  Event_timed();
  virtual ~Event_timed();

  void init();

  virtual bool fill_event_info(THD* thd, const dd::Event &event,
                               const char *schema_name);

  int get_create_event(THD *thd, String *buf);

  Event_timed(const Event_timed &)= delete;
  void operator=(Event_timed &)= delete;

};


class Event_job_data : public Event_basic
{
public:
  LEX_STRING  m_definition;
  LEX_CSTRING m_definer_user;
  LEX_CSTRING m_definer_host;

  sql_mode_t  m_sql_mode;

  class Stored_program_creation_ctx *m_creation_ctx;

  Event_job_data();

  bool execute(THD *thd, bool drop);

  Event_job_data(const Event_job_data &)= delete;
  void operator=(Event_job_data &)= delete;

private:
  virtual bool fill_event_info(THD *thd, const dd::Event &event,
                               const char *schema_name);
  bool construct_sp_sql(THD *thd, String *sp_sql);
  bool construct_drop_event_sql(THD *thd, String *sp_sql);
};


/* Compares only the schema part of the identifier */
bool
event_basic_db_equal(LEX_STRING db, Event_basic *et);

/* Compares the whole identifier*/
bool
event_basic_identifier_equal(LEX_STRING db, LEX_STRING name, Event_basic *b);

/**
  @} (End of group Event_Scheduler)
*/

#endif /* _EVENT_DATA_OBJECTS_H_ */
