/* Copyright (c) 2016, Oracle and/or its affiliates. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */

/*
  This header file contains weight tables and weight length table of
  UCA8.0.0, as well as some constant values and table of decomposition.
*/

#ifndef UCA900_DATA_H
#define UCA900_DATA_H

#include "m_ctype.h"
#include "str_uca_type.h"

#define MY_UCA_900_CE_SIZE 3

enum enum_char_category{
  CHAR_CATEGORY_CC= 1, CHAR_CATEGORY_CF, CHAR_CATEGORY_CO, CHAR_CATEGORY_CS,
  CHAR_CATEGORY_LL, CHAR_CATEGORY_LM, CHAR_CATEGORY_LO, CHAR_CATEGORY_LT,
  CHAR_CATEGORY_LU, CHAR_CATEGORY_MC, CHAR_CATEGORY_ME, CHAR_CATEGORY_MN,
  CHAR_CATEGORY_ND, CHAR_CATEGORY_NL, CHAR_CATEGORY_NO, CHAR_CATEGORY_PC,
  CHAR_CATEGORY_PD, CHAR_CATEGORY_PE, CHAR_CATEGORY_PF, CHAR_CATEGORY_PI,
  CHAR_CATEGORY_PO, CHAR_CATEGORY_PS, CHAR_CATEGORY_SC, CHAR_CATEGORY_SK,
  CHAR_CATEGORY_SM, CHAR_CATEGORY_SO, CHAR_CATEGORY_ZL, CHAR_CATEGORY_ZP,
  CHAR_CATEGORY_ZS
};

enum enum_decomp_tag{
  DECOMP_TAG_NONE,      DECOMP_TAG_CIRCLE,   DECOMP_TAG_COMPAT,
  DECOMP_TAG_FINAL,     DECOMP_TAG_FONT,     DECOMP_TAG_FRACTION,
  DECOMP_TAG_INITIAL,   DECOMP_TAG_ISOLATED, DECOMP_TAG_MEDIAL,
  DECOMP_TAG_NARROW,    DECOMP_TAG_NOBREAK,  DECOMP_TAG_SMALL,
  DECOMP_TAG_SQUARE,    DECOMP_TAG_SUB,      DECOMP_TAG_SUPER,
  DECOMP_TAG_VERTICAL,  DECOMP_TAG_WIDE
};

struct Unidata_decomp
{
  my_wc_t charcode;
  enum_char_category category;
  enum_decomp_tag    decomp_tag;
  my_wc_t dec_codes[MY_UCA_MAX_CONTRACTION];
};

Unidata_decomp uni_dec[]= {
  {0x00A0, CHAR_CATEGORY_ZS, DECOMP_TAG_NOBREAK,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00A8, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00AA, CHAR_CATEGORY_LO, DECOMP_TAG_SUPER,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00AF, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00B2, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00B3, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00B4, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00B5, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03BC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00B8, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00B9, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00BA, CHAR_CATEGORY_LO, DECOMP_TAG_SUPER,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00BC, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0034, 0x0000, 0x0000, 0x0000}},
  {0x00BD, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0032, 0x0000, 0x0000, 0x0000}},
  {0x00BE, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0033, 0x2044, 0x0034, 0x0000, 0x0000, 0x0000}},
  {0x00C0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C1, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C3, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C5, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C7, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0043, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00C9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00CA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00CB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00CC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00CD, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00CE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00CF, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D1, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D3, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D5, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00D9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00DA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00DB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00DC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00DD, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0063, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E8, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00E9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00EA, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00EB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00EC, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00ED, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00EE, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00EF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00F9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00FA, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00FB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00FC, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00FD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x00FF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0100, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0101, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0102, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0103, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0104, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0105, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0106, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0043, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0107, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0063, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0108, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0043, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0109, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0063, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x010A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0043, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x010B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0063, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x010C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0043, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x010D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0063, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x010E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0044, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x010F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0064, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0112, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0113, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0114, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0115, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0116, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0117, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0118, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0119, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x011A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x011B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x011C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x011D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x011E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x011F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0120, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0121, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0122, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0123, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0124, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0125, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0128, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0129, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x012A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x012B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x012C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x012D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x012E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x012F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0130, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0132, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x0049, 0x004A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0133, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0069, 0x006A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0134, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004A, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0135, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006A, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0136, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004B, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0137, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006B, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0139, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x013A, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x013B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x013C, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x013D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x013E, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x013F, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x004C, 0x00B7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0140, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x006C, 0x00B7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0143, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0144, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0145, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0146, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0147, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0148, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0149, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x02BC, 0x006E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x014C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x014D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x014E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x014F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0150, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0151, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0154, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0155, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0156, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0157, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0158, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0159, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x015A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x015B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x015C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x015D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x015E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x015F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0160, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0161, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0162, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0163, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0164, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0165, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0168, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0169, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x016A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x016B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x016C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x016D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x016E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x016F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0170, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0171, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0172, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0173, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0174, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0057, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0175, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0176, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0177, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0178, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0179, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x005A, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x017A, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x007A, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x017B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x005A, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x017C, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x007A, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x017D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x005A, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x017E, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x007A, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x017F, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01A0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x031B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01A1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x031B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01AF, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x031B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01B0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x031B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01C4, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x0044, 0x017D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01C5, CHAR_CATEGORY_LT, DECOMP_TAG_COMPAT,
   {0x0044, 0x017E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01C6, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0064, 0x017E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01C7, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x004C, 0x004A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01C8, CHAR_CATEGORY_LT, DECOMP_TAG_COMPAT,
   {0x004C, 0x006A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01C9, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x006C, 0x006A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01CA, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x004E, 0x004A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01CB, CHAR_CATEGORY_LT, DECOMP_TAG_COMPAT,
   {0x004E, 0x006A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01CC, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x006E, 0x006A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01CD, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01CE, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01CF, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D1, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D3, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D5, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00DC, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00FC, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D7, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00DC, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D8, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00FC, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01D9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00DC, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01DA, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00FC, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01DB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00DC, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01DC, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00FC, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01DE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00C4, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01DF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00E4, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0226, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0227, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00C6, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00E6, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004B, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01E9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006B, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01EA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01EB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01EC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01EA, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01ED, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x01EB, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01EE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01B7, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01EF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0292, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006A, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F1, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x0044, 0x005A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F2, CHAR_CATEGORY_LT, DECOMP_TAG_COMPAT,
   {0x0044, 0x007A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F3, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0064, 0x007A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01F9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01FA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00C5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01FB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00E5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01FC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00C6, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01FD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00E6, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01FE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00D8, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x01FF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00F8, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0200, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0201, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0202, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0203, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0204, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0205, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0206, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0207, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0208, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0209, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x020A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x020B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x020C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x020D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x020E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x020F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0210, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0211, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0212, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0213, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0214, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0215, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0216, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0217, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0311, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0218, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0326, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0219, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0326, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x021A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x0326, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x021B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x0326, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x021E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x021F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x030C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0226, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0227, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0228, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0229, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x022A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00D6, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x022B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00F6, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x022C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00D5, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x022D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00F5, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x022E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x022F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0230, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x022E, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0231, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x022F, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0232, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0233, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B0, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B1, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0266, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B2, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B3, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B4, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0279, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B5, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x027B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B6, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0281, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B7, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02B8, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02D8, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02D9, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02DA, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02DB, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0328, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02DC, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02DD, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02E0, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0263, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02E1, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02E2, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02E3, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x02E4, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0295, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0340, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0300, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0341, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0301, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0343, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0313, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0344, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0308, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0374, CHAR_CATEGORY_LM, DECOMP_TAG_NONE,
   {0x02B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x037A, CHAR_CATEGORY_LM, DECOMP_TAG_COMPAT,
   {0x0020, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x037E, CHAR_CATEGORY_PO, DECOMP_TAG_NONE,
   {0x003B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0384, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0385, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x00A8, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0386, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0387, CHAR_CATEGORY_PO, DECOMP_TAG_NONE,
   {0x00B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0388, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0389, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x038A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x038C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x038E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x038F, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0390, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03CA, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03AA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03AB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03AC, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03AD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03AE, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03AF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03B0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03CB, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03CA, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03CB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03CC, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03CD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03CE, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D0, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D1, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D2, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x03A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D3, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03D2, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03D2, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D5, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03D6, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03F0, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03F1, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03F2, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03F4, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x0398, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03F5, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x03B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x03F9, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x03A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0400, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0415, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0401, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0415, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0403, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0413, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0407, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0406, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x040C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x041A, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x040D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0418, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x040E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0423, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0419, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0418, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0439, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0438, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0450, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0435, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0451, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0435, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0453, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0433, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0457, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0456, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x045C, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x043A, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x045D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0438, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x045E, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0443, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0476, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0474, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0477, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0475, 0x030F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04C1, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0416, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04C2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0436, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04D0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0410, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04D1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0430, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04D2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0410, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04D3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0430, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04D6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0415, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04D7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0435, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04DA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x04D8, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04DB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x04D9, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04DC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0416, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04DD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0436, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04DE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0417, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04DF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0437, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04E2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0418, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04E3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0438, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04E4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0418, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04E5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0438, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04E6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x041E, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04E7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x043E, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04EA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x04E8, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04EB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x04E9, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04EC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x042D, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04ED, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x044D, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04EE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0423, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04EF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0443, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0423, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0443, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0423, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0443, 0x030B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0427, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0447, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x042B, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x04F9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x044B, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0587, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0565, 0x0582, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0622, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0627, 0x0653, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0623, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0627, 0x0654, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0624, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0648, 0x0654, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0625, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0627, 0x0655, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0626, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x064A, 0x0654, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0675, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x0627, 0x0674, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0676, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x0648, 0x0674, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0677, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x06C7, 0x0674, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0678, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x064A, 0x0674, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x06C0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x06D5, 0x0654, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x06C2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x06C1, 0x0654, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x06D3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x06D2, 0x0654, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0929, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0928, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0931, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0930, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0934, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0933, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0958, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0915, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0959, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0916, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x095A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0917, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x095B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x091C, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x095C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0921, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x095D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0922, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x095E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x092B, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x095F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x092F, 0x093C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x09CB, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x09C7, 0x09BE, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x09CC, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x09C7, 0x09D7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x09DC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x09A1, 0x09BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x09DD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x09A2, 0x09BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x09DF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x09AF, 0x09BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0A33, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0A32, 0x0A3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0A36, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0A38, 0x0A3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0A59, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0A16, 0x0A3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0A5A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0A17, 0x0A3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0A5B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0A1C, 0x0A3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0A5E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0A2B, 0x0A3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0B48, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0B47, 0x0B56, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0B4B, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0B47, 0x0B3E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0B4C, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0B47, 0x0B57, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0B5C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0B21, 0x0B3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0B5D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0B22, 0x0B3C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0B94, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0B92, 0x0BD7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0BCA, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0BC6, 0x0BBE, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0BCB, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0BC7, 0x0BBE, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0BCC, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0BC6, 0x0BD7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0C48, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0C46, 0x0C56, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0CC0, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0CBF, 0x0CD5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0CC7, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0CC6, 0x0CD5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0CC8, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0CC6, 0x0CD6, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0CCA, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0CC6, 0x0CC2, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0CCB, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0CCA, 0x0CD5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0D4A, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0D46, 0x0D3E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0D4B, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0D47, 0x0D3E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0D4C, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0D46, 0x0D57, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0DDA, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0DD9, 0x0DCA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0DDC, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0DD9, 0x0DCF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0DDD, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0DDC, 0x0DCA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0DDE, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x0DD9, 0x0DDF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0E33, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x0E4D, 0x0E32, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0EB3, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x0ECD, 0x0EB2, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0EDC, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x0EAB, 0x0E99, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0EDD, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x0EAB, 0x0EA1, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F0C, CHAR_CATEGORY_PO, DECOMP_TAG_NOBREAK,
   {0x0F0B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F43, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0F42, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F4D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0F4C, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F52, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0F51, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F57, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0F56, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F5C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0F5B, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F69, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x0F40, 0x0FB5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F73, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0F71, 0x0F72, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F75, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0F71, 0x0F74, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F76, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0FB2, 0x0F80, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F77, CHAR_CATEGORY_MN, DECOMP_TAG_COMPAT,
   {0x0FB2, 0x0F81, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F78, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0FB3, 0x0F80, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F79, CHAR_CATEGORY_MN, DECOMP_TAG_COMPAT,
   {0x0FB3, 0x0F81, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F81, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0F71, 0x0F80, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F93, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0F92, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0F9D, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0F9C, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0FA2, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0FA1, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0FA7, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0FA6, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0FAC, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0FAB, 0x0FB7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x0FB9, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x0F90, 0x0FB5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1026, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1025, 0x102E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x10FC, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x10DC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B06, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1B05, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B08, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1B07, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B0A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1B09, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B0C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1B0B, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B0E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1B0D, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B12, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1B11, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B3B, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x1B3A, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B3D, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x1B3C, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B40, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x1B3E, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B41, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x1B3F, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1B43, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x1B42, 0x1B35, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D2C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D2D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x00C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D2E, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D30, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D31, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D32, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x018E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D33, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D34, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D35, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D36, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D37, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D38, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D39, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D3A, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D3C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D3D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0222, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D3E, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D3F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0250, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0251, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D02, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D49, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4A, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0259, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x025B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x025C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D50, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D51, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x014B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D53, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0254, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D16, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D17, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D1D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x026F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D25, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x03B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x043D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D9B, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0252, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D9C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D9D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0255, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D9E, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x00F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D9F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x025C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA0, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA1, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x025F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA2, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0261, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA3, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0265, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA4, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0268, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA5, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0269, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA6, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x026A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA7, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D7B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA8, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x029D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DA9, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x026D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DAA, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D85, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DAB, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x029F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DAC, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0271, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DAD, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0270, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DAE, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0272, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DAF, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0273, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB0, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0274, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB1, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0275, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB2, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0278, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB3, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0282, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB4, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0283, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB5, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x01AB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB6, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0289, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB7, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x028A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB8, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x1D1C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DB9, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x028B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DBA, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x028C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DBB, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DBC, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0290, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DBD, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0291, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DBE, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0292, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1DBF, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E00, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0325, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E01, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0325, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E02, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0042, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E03, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0062, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E04, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0042, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E05, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0062, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E06, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0042, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E07, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0062, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E08, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00C7, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E09, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00E7, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E0A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0044, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E0B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0064, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E0C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0044, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E0D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0064, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E0E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0044, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E0F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0064, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E10, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0044, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E11, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0064, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E12, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0044, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E13, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0064, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E14, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0112, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E15, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0113, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E16, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0112, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E17, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0113, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E18, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E19, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E1A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E1B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E1C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0228, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E1D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0229, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E1E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0046, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E1F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0066, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E20, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0047, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E21, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0067, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E22, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E23, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E24, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E25, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E26, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E27, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E28, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E29, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x0327, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E2A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0048, 0x032E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E2B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x032E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E2C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E2D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E2E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00CF, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E2F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00EF, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E30, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004B, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E31, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006B, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E32, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004B, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E33, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006B, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E34, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004B, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E35, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006B, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E36, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E37, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E38, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1E36, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E39, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1E37, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E3A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E3B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E3C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004C, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E3D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006C, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E3E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004D, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E3F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006D, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E40, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004D, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E41, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006D, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E42, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004D, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E43, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006D, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E44, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E45, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E46, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E47, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E48, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E49, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E4A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004E, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E4B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006E, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E4C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00D5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E4D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00F5, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E4E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00D5, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E4F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00F5, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E50, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x014C, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E51, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x014D, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E52, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x014C, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E53, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x014D, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E54, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0050, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E55, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0070, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E56, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0050, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E57, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0070, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E58, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E59, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E5A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E5B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E5C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1E5A, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E5D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1E5B, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E5E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0052, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E5F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0072, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E60, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E61, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E62, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0053, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E63, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0073, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E64, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x015A, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E65, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x015B, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E66, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0160, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E67, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0161, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E68, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1E62, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E69, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1E63, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E6A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E6B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E6C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E6D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E6E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E6F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E70, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0054, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E71, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E72, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0324, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E73, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0324, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E74, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E75, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E76, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E77, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x032D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E78, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0168, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E79, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0169, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E7A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x016A, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E7B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x016B, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E7C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0056, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E7D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0076, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E7E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0056, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E7F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0076, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E80, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0057, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E81, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E82, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0057, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E83, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E84, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0057, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E85, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E86, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0057, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E87, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E88, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0057, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E89, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E8A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0058, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E8B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0078, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E8C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0058, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E8D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0078, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E8E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E8F, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E90, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x005A, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E91, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x007A, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E92, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x005A, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E93, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x007A, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E94, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x005A, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E95, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x007A, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E96, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0068, 0x0331, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E97, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0074, 0x0308, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E98, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0077, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E99, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x030A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E9A, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0061, 0x02BE, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1E9B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x017F, 0x0307, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0041, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0061, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00C2, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00E2, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00C2, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00E2, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00C2, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EA9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00E2, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EAA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00C2, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EAB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00E2, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EAC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1EA0, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EAD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1EA1, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EAE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0102, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EAF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0103, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EB0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0102, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EB1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0103, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EB2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0102, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EB3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0103, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EB4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0102, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EB5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0103, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EB6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1EA0, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EB7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1EA1, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EB8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EB9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EBA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EBB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EBC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0045, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EBD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0065, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EBE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00CA, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EBF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00EA, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EC0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00CA, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EC1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00EA, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EC2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00CA, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EC3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00EA, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EC4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00CA, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EC5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00EA, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EC6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1EB8, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EC7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1EB9, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EC8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EC9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ECA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0049, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ECB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0069, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ECC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ECD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ECE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004F, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ECF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x006F, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ED0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00D4, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ED1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00F4, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ED2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00D4, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ED3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00F4, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ED4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00D4, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ED5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00F4, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ED6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00D4, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ED7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x00F4, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ED8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1ECC, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1ED9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1ECD, 0x0302, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EDA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01A0, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EDB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x01A1, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EDC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01A0, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EDD, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x01A1, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EDE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01A0, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EDF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x01A1, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01A0, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x01A1, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01A0, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x01A1, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0055, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0075, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01AF, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x01B0, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01AF, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x01B0, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01AF, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EED, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x01B0, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEE, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01AF, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEF, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x01B0, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF0, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x01AF, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x01B0, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF2, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF4, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0323, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF6, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0309, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0059, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EF9, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0079, 0x0303, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F00, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F01, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F02, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F00, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F03, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F01, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F04, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F00, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F05, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F01, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F06, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F00, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F07, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F01, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F08, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F09, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F0A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F08, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F0B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F09, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F0C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F08, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F0D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F09, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F0E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F08, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F0F, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F09, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F10, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F11, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F12, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F10, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F11, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F14, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F10, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F15, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F11, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F18, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F19, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F1A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F18, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F1B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F19, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F1C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F18, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F1D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F19, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F20, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F20, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F23, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F21, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F24, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F20, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F25, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F21, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F26, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F20, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F27, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F21, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F28, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F29, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F2A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F28, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F2B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F29, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F2C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F28, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F2D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F29, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F2E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F28, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F2F, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F29, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F30, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F31, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F32, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F30, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F33, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F31, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F34, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F30, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F35, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F31, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F36, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F30, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F37, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F31, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F38, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F39, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F3A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F38, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F3B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F39, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F3C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F38, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F3D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F39, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F3E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F38, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F3F, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F39, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F40, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F41, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F42, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F40, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F43, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F41, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F44, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F40, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F45, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F41, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F48, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F49, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F4A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F48, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F4B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F49, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F4C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F48, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F4D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F49, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F50, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F51, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F52, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F50, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F53, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F51, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F54, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F50, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F55, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F51, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F56, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F50, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F57, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F51, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F59, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F5B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F59, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F5D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F59, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F5F, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F59, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F60, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F61, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F62, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F60, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F63, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F61, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F64, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F60, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F65, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F61, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F66, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F60, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F67, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F61, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F68, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F69, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F6A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F68, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F6B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F69, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F6C, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F68, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F6D, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F69, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F6E, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F68, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F6F, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x1F69, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F70, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F71, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F72, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B5, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F73, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F74, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F75, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03AE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F76, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F77, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F78, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03BF, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F79, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F7A, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F7B, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03CD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F7C, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F7D, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F80, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F00, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F81, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F01, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F82, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F02, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F83, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F03, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F84, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F04, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F85, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F05, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F86, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F06, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F87, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F07, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F88, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F08, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F89, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F09, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F8A, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F0A, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F8B, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F0B, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F8C, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F0C, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F8D, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F0D, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F8E, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F0E, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F8F, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F0F, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F90, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F20, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F91, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F21, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F92, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F22, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F93, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F23, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F94, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F24, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F95, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F25, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F96, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F26, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F97, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F27, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F98, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F28, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F99, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F29, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F9A, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F2A, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F9B, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F2B, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F9C, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F2C, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F9D, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F2D, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F9E, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F2E, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F9F, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F2F, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FA0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F60, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FA1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F61, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FA2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F62, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FA3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F63, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FA4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F64, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FA5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F65, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FA6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F66, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FA7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F67, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FA8, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F68, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FA9, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F69, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FAA, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F6A, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FAB, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F6B, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FAC, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F6C, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FAD, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F6D, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FAE, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F6E, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FAF, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x1F6F, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F70, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03AC, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B1, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1FB6, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FB9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FBA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0391, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FBB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0386, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FBC, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0391, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FBD, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FBE, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FBF, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC0, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC1, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x00A8, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F74, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03AE, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B7, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1FC6, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0395, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FC9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0388, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FCA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0397, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FCB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0389, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FCC, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x0397, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FCD, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x1FBF, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FCE, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x1FBF, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FCF, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x1FBF, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03CA, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x0390, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B9, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03CA, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FD9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FDA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x0399, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FDB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x038A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FDD, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x1FFE, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FDE, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x1FFE, 0x0301, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FDF, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x1FFE, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE0, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE1, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03CB, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03B0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C1, 0x0313, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE5, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C1, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C5, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03CB, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0306, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FE9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0304, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FEA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A5, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FEB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x038E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FEC, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A1, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FED, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x00A8, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FEE, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x0385, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FEF, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x0060, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FF2, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1F7C, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FF3, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FF4, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03CE, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FF6, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x03C9, 0x0342, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FF7, CHAR_CATEGORY_LL, DECOMP_TAG_NONE,
   {0x1FF6, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FF8, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x039F, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FF9, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x038C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FFA, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0300, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FFB, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x038F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FFC, CHAR_CATEGORY_LT, DECOMP_TAG_NONE,
   {0x03A9, 0x0345, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FFD, CHAR_CATEGORY_SK, DECOMP_TAG_NONE,
   {0x00B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1FFE, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x0314, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2000, CHAR_CATEGORY_ZS, DECOMP_TAG_NONE,
   {0x2002, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2001, CHAR_CATEGORY_ZS, DECOMP_TAG_NONE,
   {0x2003, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2002, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2003, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2004, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2005, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2006, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2007, CHAR_CATEGORY_ZS, DECOMP_TAG_NOBREAK,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2008, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2009, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x200A, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2011, CHAR_CATEGORY_PD, DECOMP_TAG_NOBREAK,
   {0x2010, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2017, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x0020, 0x0333, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2024, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x002E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2025, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x002E, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2026, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x002E, 0x002E, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x202F, CHAR_CATEGORY_ZS, DECOMP_TAG_NOBREAK,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2033, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x2032, 0x2032, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2034, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x2032, 0x2032, 0x2032, 0x0000, 0x0000, 0x0000}},
  {0x2036, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x2035, 0x2035, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2037, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x2035, 0x2035, 0x2035, 0x0000, 0x0000, 0x0000}},
  {0x203C, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x0021, 0x0021, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x203E, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x0020, 0x0305, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2047, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x003F, 0x003F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2048, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x003F, 0x0021, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2049, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x0021, 0x003F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2057, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x2032, 0x2032, 0x2032, 0x2032, 0x0000, 0x0000}},
  {0x205F, CHAR_CATEGORY_ZS, DECOMP_TAG_COMPAT,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2070, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2071, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2074, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2075, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2076, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2077, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2078, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2079, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x207A, CHAR_CATEGORY_SM, DECOMP_TAG_SUPER,
   {0x002B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x207B, CHAR_CATEGORY_SM, DECOMP_TAG_SUPER,
   {0x2212, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x207C, CHAR_CATEGORY_SM, DECOMP_TAG_SUPER,
   {0x003D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x207D, CHAR_CATEGORY_PS, DECOMP_TAG_SUPER,
   {0x0028, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x207E, CHAR_CATEGORY_PE, DECOMP_TAG_SUPER,
   {0x0029, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x207F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2080, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2081, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2082, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2083, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2084, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2085, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2086, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2087, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2088, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2089, CHAR_CATEGORY_NO, DECOMP_TAG_SUB,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x208A, CHAR_CATEGORY_SM, DECOMP_TAG_SUB,
   {0x002B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x208B, CHAR_CATEGORY_SM, DECOMP_TAG_SUB,
   {0x2212, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x208C, CHAR_CATEGORY_SM, DECOMP_TAG_SUB,
   {0x003D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x208D, CHAR_CATEGORY_PS, DECOMP_TAG_SUB,
   {0x0028, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x208E, CHAR_CATEGORY_PE, DECOMP_TAG_SUB,
   {0x0029, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2090, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2091, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2092, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2093, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2094, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0259, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2095, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2096, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2097, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2098, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2099, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x209A, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x209B, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x209C, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x20A8, CHAR_CATEGORY_SC, DECOMP_TAG_COMPAT,
   {0x0052, 0x0073, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2100, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0061, 0x002F, 0x0063, 0x0000, 0x0000, 0x0000}},
  {0x2101, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0061, 0x002F, 0x0073, 0x0000, 0x0000, 0x0000}},
  {0x2102, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2103, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x00B0, 0x0043, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2105, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0063, 0x002F, 0x006F, 0x0000, 0x0000, 0x0000}},
  {0x2106, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0063, 0x002F, 0x0075, 0x0000, 0x0000, 0x0000}},
  {0x2107, CHAR_CATEGORY_LU, DECOMP_TAG_COMPAT,
   {0x0190, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2109, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x00B0, 0x0046, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x210A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x210B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x210C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x210D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x210E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x210F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0127, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2110, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2111, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2112, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2113, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2115, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2116, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x004E, 0x006F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2119, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x211A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x211B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x211C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x211D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2120, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x0053, 0x004D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2121, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0054, 0x0045, 0x004C, 0x0000, 0x0000, 0x0000}},
  {0x2122, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x0054, 0x004D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2124, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2126, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x03A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2128, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x212A, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x212B, CHAR_CATEGORY_LU, DECOMP_TAG_NONE,
   {0x00C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x212C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x212D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x212F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2130, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2131, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2133, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2134, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2135, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x05D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2136, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x05D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2137, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x05D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2138, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x05D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2139, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x213B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0046, 0x0041, 0x0058, 0x0000, 0x0000, 0x0000}},
  {0x213C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x213D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x213E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x213F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2140, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2211, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2145, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2146, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2147, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2148, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2149, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2150, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0037, 0x0000, 0x0000, 0x0000}},
  {0x2151, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0039, 0x0000, 0x0000, 0x0000}},
  {0x2152, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0031, 0x0030, 0x0000, 0x0000}},
  {0x2153, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0033, 0x0000, 0x0000, 0x0000}},
  {0x2154, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0032, 0x2044, 0x0033, 0x0000, 0x0000, 0x0000}},
  {0x2155, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0035, 0x0000, 0x0000, 0x0000}},
  {0x2156, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0032, 0x2044, 0x0035, 0x0000, 0x0000, 0x0000}},
  {0x2157, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0033, 0x2044, 0x0035, 0x0000, 0x0000, 0x0000}},
  {0x2158, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0034, 0x2044, 0x0035, 0x0000, 0x0000, 0x0000}},
  {0x2159, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0036, 0x0000, 0x0000, 0x0000}},
  {0x215A, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0035, 0x2044, 0x0036, 0x0000, 0x0000, 0x0000}},
  {0x215B, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0038, 0x0000, 0x0000, 0x0000}},
  {0x215C, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0033, 0x2044, 0x0038, 0x0000, 0x0000, 0x0000}},
  {0x215D, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0035, 0x2044, 0x0038, 0x0000, 0x0000, 0x0000}},
  {0x215E, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0037, 0x2044, 0x0038, 0x0000, 0x0000, 0x0000}},
  {0x215F, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0031, 0x2044, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2160, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2161, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0049, 0x0049, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2162, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0049, 0x0049, 0x0049, 0x0000, 0x0000, 0x0000}},
  {0x2163, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0049, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2164, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2165, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0056, 0x0049, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2166, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0056, 0x0049, 0x0049, 0x0000, 0x0000, 0x0000}},
  {0x2167, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0056, 0x0049, 0x0049, 0x0049, 0x0000, 0x0000}},
  {0x2168, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0049, 0x0058, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2169, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x216A, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0058, 0x0049, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x216B, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0058, 0x0049, 0x0049, 0x0000, 0x0000, 0x0000}},
  {0x216C, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x216D, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x216E, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x216F, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2170, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2171, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0069, 0x0069, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2172, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0069, 0x0069, 0x0069, 0x0000, 0x0000, 0x0000}},
  {0x2173, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0069, 0x0076, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2174, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2175, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0076, 0x0069, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2176, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0076, 0x0069, 0x0069, 0x0000, 0x0000, 0x0000}},
  {0x2177, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0076, 0x0069, 0x0069, 0x0069, 0x0000, 0x0000}},
  {0x2178, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0069, 0x0078, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2179, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x217A, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0078, 0x0069, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x217B, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0078, 0x0069, 0x0069, 0x0000, 0x0000, 0x0000}},
  {0x217C, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x217D, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x217E, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x217F, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2189, CHAR_CATEGORY_NO, DECOMP_TAG_FRACTION,
   {0x0030, 0x2044, 0x0033, 0x0000, 0x0000, 0x0000}},
  {0x219A, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2190, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x219B, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2192, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x21AE, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2194, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x21CD, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x21D0, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x21CE, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x21D4, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x21CF, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x21D2, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2204, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2203, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2209, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2208, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x220C, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x220B, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2224, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2223, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2226, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2225, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x222C, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x222B, 0x222B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x222D, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x222B, 0x222B, 0x222B, 0x0000, 0x0000, 0x0000}},
  {0x222F, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x222E, 0x222E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2230, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x222E, 0x222E, 0x222E, 0x0000, 0x0000, 0x0000}},
  {0x2241, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x223C, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2244, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2243, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2247, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2245, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2249, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2248, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2260, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x003D, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2262, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2261, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x226D, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x224D, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x226E, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x003C, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x226F, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x003E, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2270, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2264, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2271, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2265, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2274, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2272, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2275, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2273, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2278, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2276, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2279, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2277, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2280, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x227A, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2281, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x227B, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2284, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2282, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2285, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2283, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2288, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2286, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2289, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2287, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22AC, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22A2, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22AD, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22A8, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22AE, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22A9, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22AF, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22AB, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22E0, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x227C, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22E1, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x227D, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22E2, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2291, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22E3, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2292, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22EA, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22B2, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22EB, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22B3, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22EC, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22B4, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x22ED, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x22B5, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2329, CHAR_CATEGORY_PS, DECOMP_TAG_NONE,
   {0x3008, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x232A, CHAR_CATEGORY_PE, DECOMP_TAG_NONE,
   {0x3009, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2460, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2461, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2462, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2463, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2464, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2465, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2466, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2467, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2468, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2469, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0030, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x246A, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0031, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x246B, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0032, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x246C, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0033, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x246D, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0034, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x246E, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0035, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x246F, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0036, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2470, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0037, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2471, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0038, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2472, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0031, 0x0039, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2473, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0030, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2474, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x2475, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0032, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x2476, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0033, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x2477, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0034, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x2478, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0035, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x2479, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0036, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x247A, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0037, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x247B, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0038, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x247C, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0039, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x247D, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0030, 0x0029, 0x0000, 0x0000}},
  {0x247E, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0031, 0x0029, 0x0000, 0x0000}},
  {0x247F, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0032, 0x0029, 0x0000, 0x0000}},
  {0x2480, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0033, 0x0029, 0x0000, 0x0000}},
  {0x2481, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0034, 0x0029, 0x0000, 0x0000}},
  {0x2482, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0035, 0x0029, 0x0000, 0x0000}},
  {0x2483, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0036, 0x0029, 0x0000, 0x0000}},
  {0x2484, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0037, 0x0029, 0x0000, 0x0000}},
  {0x2485, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0038, 0x0029, 0x0000, 0x0000}},
  {0x2486, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0031, 0x0039, 0x0029, 0x0000, 0x0000}},
  {0x2487, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0032, 0x0030, 0x0029, 0x0000, 0x0000}},
  {0x2488, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2489, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0032, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x248A, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0033, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x248B, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0034, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x248C, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0035, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x248D, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0036, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x248E, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0037, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x248F, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0038, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2490, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0039, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2491, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0030, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2492, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0031, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2493, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0032, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2494, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0033, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2495, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0034, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2496, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0035, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2497, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0036, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2498, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0037, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x2499, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0038, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x249A, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0039, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x249B, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0030, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x249C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0061, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x249D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0062, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x249E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0063, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x249F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0064, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0065, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0066, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0067, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0068, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0069, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x006A, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x006B, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x006C, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x006D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24A9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x006E, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24AA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x006F, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24AB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0070, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24AC, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0071, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24AD, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0072, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24AE, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0073, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24AF, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0074, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0075, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0076, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0077, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0078, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0079, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x007A, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x24B6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24B7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24B8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24B9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24BA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24BB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24BC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24BD, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24BE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24BF, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24C9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24CA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24CB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24CC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24CD, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24CE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24CF, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24D9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24DA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24DB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24DC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24DD, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24DE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24DF, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24E9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x24EA, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2A0C, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x222B, 0x222B, 0x222B, 0x222B, 0x0000, 0x0000}},
  {0x2A74, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x003A, 0x003A, 0x003D, 0x0000, 0x0000, 0x0000}},
  {0x2A75, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x003D, 0x003D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2A76, CHAR_CATEGORY_SM, DECOMP_TAG_COMPAT,
   {0x003D, 0x003D, 0x003D, 0x0000, 0x0000, 0x0000}},
  {0x2ADC, CHAR_CATEGORY_SM, DECOMP_TAG_NONE,
   {0x2ADD, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2C7C, CHAR_CATEGORY_LM, DECOMP_TAG_SUB,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2C7D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2D6F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x2D61, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2E9F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6BCD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2EF3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F9F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F00, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F01, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E28, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F02, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E36, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F03, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E3F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F04, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E59, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F05, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E85, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F06, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4E8C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F07, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4EA0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F08, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x4EBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F09, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x513F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F0A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5165, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F0B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x516B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F0C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5182, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F0D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5196, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F0E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x51AB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F0F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x51E0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F10, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x51F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F11, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5200, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F12, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x529B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F13, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x52F9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F14, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5315, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F15, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x531A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F16, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5338, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F17, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5341, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F18, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x535C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F19, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5369, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F1A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5382, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F1B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x53B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F1C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x53C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F1D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x53E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F1E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x56D7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F1F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x571F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F20, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x58EB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F21, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5902, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F22, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x590A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F23, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5915, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F24, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5927, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F25, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5973, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F26, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5B50, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F27, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5B80, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F28, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5BF8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F29, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5C0F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F2A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5C22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F2B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5C38, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F2C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5C6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F2D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5C71, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F2E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5DDB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F2F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5DE5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F30, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5DF1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F31, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5DFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F32, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5E72, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F33, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5E7A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F34, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5E7F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F35, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5EF4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F36, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5EFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F37, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5F0B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F38, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5F13, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F39, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5F50, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F3A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5F61, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F3B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5F73, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F3C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x5FC3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F3D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6208, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F3E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6236, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F3F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x624B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F40, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x652F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F41, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6534, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F42, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6587, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F43, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6597, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F44, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x65A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F45, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x65B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F46, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x65E0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F47, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x65E5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F48, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x66F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F49, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6708, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F4A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6728, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F4B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6B20, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F4C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6B62, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F4D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6B79, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F4E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6BB3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F4F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6BCB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F50, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6BD4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F51, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6BDB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F52, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6C0F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F53, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6C14, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F54, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x6C34, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F55, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x706B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F56, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x722A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F57, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7236, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F58, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x723B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F59, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x723F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F5A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7247, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F5B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7259, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F5C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x725B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F5D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x72AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F5E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7384, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F5F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7389, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F60, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x74DC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F61, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x74E6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F62, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7518, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F63, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x751F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F64, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7528, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F65, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7530, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F66, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x758B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F67, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7592, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F68, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7676, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F69, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x767D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F6A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x76AE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F6B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x76BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F6C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x76EE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F6D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x77DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F6E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x77E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F6F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x77F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F70, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x793A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F71, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x79B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F72, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x79BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F73, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7A74, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F74, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7ACB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F75, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7AF9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F76, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7C73, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F77, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7CF8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F78, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7F36, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F79, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7F51, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F7A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7F8A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F7B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x7FBD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F7C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F7D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x800C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F7E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8012, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F7F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x807F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8089, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x81E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x81EA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x81F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x81FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x820C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x821B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x821F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x826E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8272, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8278, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x864D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x866B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8840, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x884C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8863, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x897E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x898B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x89D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8A00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8C37, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8C46, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8C55, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8C78, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8C9D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8D64, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8D70, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8DB3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8EAB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8ECA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8F9B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8FB0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x8FB5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9091, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9149, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x91C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x91CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x91D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9577, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9580, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x961C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FAA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x96B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FAB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x96B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FAC, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x96E8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FAD, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9751, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FAE, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x975E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FAF, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9762, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9769, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x97CB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x97ED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x97F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9801, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x98A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x98DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x98DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9996, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FB9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9999, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FBA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x99AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FBB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9AA8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FBC, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9AD8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FBD, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9ADF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FBE, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9B25, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FBF, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9B2F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9B32, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9B3C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9B5A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9CE5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9E75, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9E7F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9EA5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9EBB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9EC3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FC9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9ECD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FCA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9ED1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FCB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9EF9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FCC, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9EFD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FCD, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F0E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FCE, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F13, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FCF, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F20, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FD0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F3B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FD1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F4A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FD2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F52, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FD3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F8D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FD4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9F9C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FD5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x9FA0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3000, CHAR_CATEGORY_ZS, DECOMP_TAG_WIDE,
   {0x0020, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3036, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3012, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3038, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x5341, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3039, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x5344, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x303A, CHAR_CATEGORY_NL, DECOMP_TAG_COMPAT,
   {0x5345, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x304C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x304B, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x304E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x304D, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3050, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x304F, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3052, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3051, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3054, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3053, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3056, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3055, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3058, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3057, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x305A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3059, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x305C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x305B, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x305E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x305D, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3060, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x305F, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3062, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3061, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3065, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3064, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3067, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3066, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3069, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3068, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3070, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x306F, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3071, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x306F, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3073, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3072, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3074, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3072, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3076, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3075, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3077, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3075, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3079, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3078, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x307A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3078, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x307C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x307B, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x307D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x307B, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3094, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3046, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x309B, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x309C, CHAR_CATEGORY_SK, DECOMP_TAG_COMPAT,
   {0x0020, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x309E, CHAR_CATEGORY_LM, DECOMP_TAG_NONE,
   {0x309D, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x309F, CHAR_CATEGORY_LO, DECOMP_TAG_VERTICAL,
   {0x3088, 0x308A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30AC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30AB, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30AE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30AD, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30B0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30AF, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30B2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30B1, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30B4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30B3, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30B6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30B5, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30B8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30B7, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30BA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30B9, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30BC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30BB, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30BE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30BD, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30C0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30BF, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30C2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30C1, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30C5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30C4, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30C7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30C6, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30C9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30C8, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30CF, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30CF, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30D2, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30D2, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30D5, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30D5, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30D9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30D8, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30DA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30D8, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30DC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30DB, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30DD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30DB, 0x309A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30F4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30A6, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30F7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30EF, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30F8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30F0, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30F9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30F1, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30FA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x30F2, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30FE, CHAR_CATEGORY_LM, DECOMP_TAG_NONE,
   {0x30FD, 0x3099, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x30FF, CHAR_CATEGORY_LO, DECOMP_TAG_VERTICAL,
   {0x30B3, 0x30C8, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3131, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1100, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3132, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1101, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3133, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11AA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3134, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1102, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3135, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3136, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3137, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1103, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3138, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1104, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3139, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1105, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x313A, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11B0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x313B, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x313C, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x313D, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x313E, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x313F, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3140, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x111A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3141, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1106, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3142, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1107, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3143, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1108, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3144, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1121, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3145, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1109, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3146, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x110A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3147, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x110B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3148, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x110C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3149, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x110D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x314A, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x110E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x314B, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x110F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x314C, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1110, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x314D, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1111, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x314E, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1112, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x314F, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1161, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3150, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1162, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3151, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1163, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3152, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1164, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3153, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1165, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3154, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1166, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3155, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1167, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3156, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1168, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3157, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1169, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3158, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x116A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3159, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x116B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x315A, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x116C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x315B, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x116D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x315C, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x116E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x315D, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x116F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x315E, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1170, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x315F, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1171, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3160, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1172, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3161, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1173, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3162, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1174, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3163, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1175, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3164, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1160, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3165, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1114, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3166, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1115, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3167, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3168, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3169, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x316A, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x316B, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x316C, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11D7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x316D, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11D9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x316E, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x111C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x316F, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11DD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3170, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3171, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x111D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3172, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x111E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3173, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1120, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3174, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1122, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3175, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1123, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3176, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1127, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3177, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1129, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3178, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x112B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3179, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x112C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x317A, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x112D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x317B, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x112E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x317C, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x112F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x317D, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1132, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x317E, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1136, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x317F, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1140, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3180, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1147, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3181, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x114C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3182, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3183, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3184, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1157, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3185, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1158, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3186, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1159, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3187, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1184, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3188, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1185, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3189, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1188, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x318A, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1191, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x318B, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1192, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x318C, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x1194, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x318D, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x119E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x318E, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x11A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3192, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x4E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3193, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x4E8C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3194, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x4E09, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3195, CHAR_CATEGORY_NO, DECOMP_TAG_SUPER,
   {0x56DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3196, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4E0A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3197, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4E2D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3198, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4E0B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3199, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x7532, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x319A, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4E59, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x319B, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4E19, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x319C, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4E01, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x319D, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x5929, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x319E, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x5730, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x319F, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x4EBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3200, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1100, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3201, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1102, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3202, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1103, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3203, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1105, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3204, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1106, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3205, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1107, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3206, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1109, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3207, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110B, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3208, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110C, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3209, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110E, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x320A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110F, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x320B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1110, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x320C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1111, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x320D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1112, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x320E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1100, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x320F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1102, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3210, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1103, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3211, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1105, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3212, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1106, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3213, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1107, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3214, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1109, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3215, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110B, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3216, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110C, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3217, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110E, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3218, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110F, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x3219, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1110, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x321A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1111, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x321B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x1112, 0x1161, 0x0029, 0x0000, 0x0000}},
  {0x321C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110C, 0x116E, 0x0029, 0x0000, 0x0000}},
  {0x321D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110B, 0x1169, 0x110C, 0x1165, 0x11AB}},
  {0x321E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x110B, 0x1169, 0x1112, 0x116E, 0x0029}},
  {0x3220, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4E00, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3221, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4E8C, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3222, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4E09, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3223, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x56DB, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3224, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4E94, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3225, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x516D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3226, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4E03, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3227, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x516B, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3228, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4E5D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3229, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0028, 0x5341, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x322A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x6708, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x322B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x706B, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x322C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x6C34, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x322D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x6728, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x322E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x91D1, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x322F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x571F, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3230, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x65E5, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3231, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x682A, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3232, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x6709, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3233, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x793E, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3234, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x540D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3235, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x7279, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3236, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x8CA1, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3237, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x795D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3238, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x52B4, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3239, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4EE3, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x323A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x547C, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x323B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x5B66, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x323C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x76E3, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x323D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4F01, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x323E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x8CC7, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x323F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x5354, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3240, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x796D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3241, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x4F11, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3242, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x81EA, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3243, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x81F3, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x3244, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x554F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3245, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5E7C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3246, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6587, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3247, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x7B8F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3250, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0054, 0x0045, 0x0000, 0x0000, 0x0000}},
  {0x3251, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0031, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3252, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0032, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3253, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0033, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3254, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0034, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3255, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0035, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3256, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0036, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3257, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0037, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3258, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0038, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3259, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0032, 0x0039, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x325A, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0030, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x325B, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0031, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x325C, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0032, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x325D, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0033, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x325E, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0034, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x325F, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0035, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3260, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1100, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3261, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1102, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3262, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1103, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3263, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1105, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3264, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1106, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3265, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1107, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3266, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1109, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3267, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3268, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3269, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x326A, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x326B, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1110, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x326C, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1111, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x326D, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1112, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x326E, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1100, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x326F, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1102, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3270, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1103, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3271, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1105, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3272, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1106, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3273, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1107, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3274, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1109, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3275, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110B, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3276, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110C, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3277, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110E, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3278, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110F, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3279, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1110, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x327A, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1111, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x327B, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x1112, 0x1161, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x327C, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110E, 0x1161, 0x11B7, 0x1100, 0x1169, 0x0000}},
  {0x327D, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110C, 0x116E, 0x110B, 0x1174, 0x0000, 0x0000}},
  {0x327E, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x110B, 0x116E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3280, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x4E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3281, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x4E8C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3282, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x4E09, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3283, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x56DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3284, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x4E94, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3285, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x516D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3286, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x4E03, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3287, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x516B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3288, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x4E5D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3289, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x5341, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x328A, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6708, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x328B, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x706B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x328C, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6C34, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x328D, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6728, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x328E, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x91D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x328F, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x571F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3290, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x65E5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3291, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x682A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3292, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6709, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3293, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x793E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3294, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x540D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3295, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x7279, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3296, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x8CA1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3297, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x795D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3298, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x52B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3299, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x79D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x329A, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x7537, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x329B, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5973, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x329C, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x9069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x329D, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x512A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x329E, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5370, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x329F, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6CE8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x9805, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x4F11, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5199, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x6B63, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x4E0A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x4E2D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x4E0B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5DE6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x53F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32A9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x533B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32AA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5B97, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32AB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5B66, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32AC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x76E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32AD, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x4F01, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32AE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x8CC7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32AF, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5354, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x591C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B1, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0036, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B2, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0037, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B3, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0038, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B4, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0033, 0x0039, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B5, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0030, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B6, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0031, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B7, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0032, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B8, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0033, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32B9, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0034, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32BA, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0035, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32BB, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0036, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32BC, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0037, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32BD, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0038, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32BE, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0034, 0x0039, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32BF, CHAR_CATEGORY_NO, DECOMP_TAG_CIRCLE,
   {0x0035, 0x0030, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0033, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0034, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0035, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0036, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0037, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0038, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0039, 0x6708, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32C9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0030, 0x6708, 0x0000, 0x0000, 0x0000}},
  {0x32CA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0031, 0x6708, 0x0000, 0x0000, 0x0000}},
  {0x32CB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0032, 0x6708, 0x0000, 0x0000, 0x0000}},
  {0x32CC, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0048, 0x0067, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32CD, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0065, 0x0072, 0x0067, 0x0000, 0x0000, 0x0000}},
  {0x32CE, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0065, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32CF, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004C, 0x0054, 0x0044, 0x0000, 0x0000, 0x0000}},
  {0x32D0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30A2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30AA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30AB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32D9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32DA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32DB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32DC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32DD, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32DE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32DF, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30CB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30CD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32E9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30CF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32EA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32EB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32EC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32ED, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32EE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32EF, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F0, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F1, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F2, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F3, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F4, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F5, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F6, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30E9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F7, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30EA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F8, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30EB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32F9, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30EC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32FA, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30ED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32FB, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30EF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32FC, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32FD, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x32FE, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x30F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3300, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A2, 0x30D1, 0x30FC, 0x30C8, 0x0000, 0x0000}},
  {0x3301, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A2, 0x30EB, 0x30D5, 0x30A1, 0x0000, 0x0000}},
  {0x3302, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A2, 0x30F3, 0x30DA, 0x30A2, 0x0000, 0x0000}},
  {0x3303, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A2, 0x30FC, 0x30EB, 0x0000, 0x0000, 0x0000}},
  {0x3304, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A4, 0x30CB, 0x30F3, 0x30B0, 0x0000, 0x0000}},
  {0x3305, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A4, 0x30F3, 0x30C1, 0x0000, 0x0000, 0x0000}},
  {0x3306, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A6, 0x30A9, 0x30F3, 0x0000, 0x0000, 0x0000}},
  {0x3307, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A8, 0x30B9, 0x30AF, 0x30FC, 0x30C9, 0x0000}},
  {0x3308, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30A8, 0x30FC, 0x30AB, 0x30FC, 0x0000, 0x0000}},
  {0x3309, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AA, 0x30F3, 0x30B9, 0x0000, 0x0000, 0x0000}},
  {0x330A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AA, 0x30FC, 0x30E0, 0x0000, 0x0000, 0x0000}},
  {0x330B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AB, 0x30A4, 0x30EA, 0x0000, 0x0000, 0x0000}},
  {0x330C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AB, 0x30E9, 0x30C3, 0x30C8, 0x0000, 0x0000}},
  {0x330D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AB, 0x30ED, 0x30EA, 0x30FC, 0x0000, 0x0000}},
  {0x330E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AC, 0x30ED, 0x30F3, 0x0000, 0x0000, 0x0000}},
  {0x330F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AC, 0x30F3, 0x30DE, 0x0000, 0x0000, 0x0000}},
  {0x3310, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AE, 0x30AC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3311, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AE, 0x30CB, 0x30FC, 0x0000, 0x0000, 0x0000}},
  {0x3312, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AD, 0x30E5, 0x30EA, 0x30FC, 0x0000, 0x0000}},
  {0x3313, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AE, 0x30EB, 0x30C0, 0x30FC, 0x0000, 0x0000}},
  {0x3314, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AD, 0x30ED, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3315, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AD, 0x30ED, 0x30B0, 0x30E9, 0x30E0, 0x0000}},
  {0x3316, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AD, 0x30ED, 0x30E1, 0x30FC, 0x30C8, 0x30EB}},
  {0x3317, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AD, 0x30ED, 0x30EF, 0x30C3, 0x30C8, 0x0000}},
  {0x3318, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B0, 0x30E9, 0x30E0, 0x0000, 0x0000, 0x0000}},
  {0x3319, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B0, 0x30E9, 0x30E0, 0x30C8, 0x30F3, 0x0000}},
  {0x331A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AF, 0x30EB, 0x30BC, 0x30A4, 0x30ED, 0x0000}},
  {0x331B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30AF, 0x30ED, 0x30FC, 0x30CD, 0x0000, 0x0000}},
  {0x331C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B1, 0x30FC, 0x30B9, 0x0000, 0x0000, 0x0000}},
  {0x331D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B3, 0x30EB, 0x30CA, 0x0000, 0x0000, 0x0000}},
  {0x331E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B3, 0x30FC, 0x30DD, 0x0000, 0x0000, 0x0000}},
  {0x331F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B5, 0x30A4, 0x30AF, 0x30EB, 0x0000, 0x0000}},
  {0x3320, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B5, 0x30F3, 0x30C1, 0x30FC, 0x30E0, 0x0000}},
  {0x3321, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B7, 0x30EA, 0x30F3, 0x30B0, 0x0000, 0x0000}},
  {0x3322, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30BB, 0x30F3, 0x30C1, 0x0000, 0x0000, 0x0000}},
  {0x3323, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30BB, 0x30F3, 0x30C8, 0x0000, 0x0000, 0x0000}},
  {0x3324, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30C0, 0x30FC, 0x30B9, 0x0000, 0x0000, 0x0000}},
  {0x3325, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30C7, 0x30B7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3326, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30C9, 0x30EB, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3327, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30C8, 0x30F3, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3328, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30CA, 0x30CE, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3329, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30CE, 0x30C3, 0x30C8, 0x0000, 0x0000, 0x0000}},
  {0x332A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30CF, 0x30A4, 0x30C4, 0x0000, 0x0000, 0x0000}},
  {0x332B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D1, 0x30FC, 0x30BB, 0x30F3, 0x30C8, 0x0000}},
  {0x332C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D1, 0x30FC, 0x30C4, 0x0000, 0x0000, 0x0000}},
  {0x332D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D0, 0x30FC, 0x30EC, 0x30EB, 0x0000, 0x0000}},
  {0x332E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D4, 0x30A2, 0x30B9, 0x30C8, 0x30EB, 0x0000}},
  {0x332F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D4, 0x30AF, 0x30EB, 0x0000, 0x0000, 0x0000}},
  {0x3330, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D4, 0x30B3, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3331, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D3, 0x30EB, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3332, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D5, 0x30A1, 0x30E9, 0x30C3, 0x30C9, 0x0000}},
  {0x3333, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D5, 0x30A3, 0x30FC, 0x30C8, 0x0000, 0x0000}},
  {0x3334, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D6, 0x30C3, 0x30B7, 0x30A7, 0x30EB, 0x0000}},
  {0x3335, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D5, 0x30E9, 0x30F3, 0x0000, 0x0000, 0x0000}},
  {0x3336, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D8, 0x30AF, 0x30BF, 0x30FC, 0x30EB, 0x0000}},
  {0x3337, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DA, 0x30BD, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3338, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DA, 0x30CB, 0x30D2, 0x0000, 0x0000, 0x0000}},
  {0x3339, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D8, 0x30EB, 0x30C4, 0x0000, 0x0000, 0x0000}},
  {0x333A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DA, 0x30F3, 0x30B9, 0x0000, 0x0000, 0x0000}},
  {0x333B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DA, 0x30FC, 0x30B8, 0x0000, 0x0000, 0x0000}},
  {0x333C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30D9, 0x30FC, 0x30BF, 0x0000, 0x0000, 0x0000}},
  {0x333D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DD, 0x30A4, 0x30F3, 0x30C8, 0x0000, 0x0000}},
  {0x333E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DC, 0x30EB, 0x30C8, 0x0000, 0x0000, 0x0000}},
  {0x333F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DB, 0x30F3, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3340, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DD, 0x30F3, 0x30C9, 0x0000, 0x0000, 0x0000}},
  {0x3341, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DB, 0x30FC, 0x30EB, 0x0000, 0x0000, 0x0000}},
  {0x3342, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DB, 0x30FC, 0x30F3, 0x0000, 0x0000, 0x0000}},
  {0x3343, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DE, 0x30A4, 0x30AF, 0x30ED, 0x0000, 0x0000}},
  {0x3344, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DE, 0x30A4, 0x30EB, 0x0000, 0x0000, 0x0000}},
  {0x3345, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DE, 0x30C3, 0x30CF, 0x0000, 0x0000, 0x0000}},
  {0x3346, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DE, 0x30EB, 0x30AF, 0x0000, 0x0000, 0x0000}},
  {0x3347, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DE, 0x30F3, 0x30B7, 0x30E7, 0x30F3, 0x0000}},
  {0x3348, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DF, 0x30AF, 0x30ED, 0x30F3, 0x0000, 0x0000}},
  {0x3349, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DF, 0x30EA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x334A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30DF, 0x30EA, 0x30D0, 0x30FC, 0x30EB, 0x0000}},
  {0x334B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30E1, 0x30AC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x334C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30E1, 0x30AC, 0x30C8, 0x30F3, 0x0000, 0x0000}},
  {0x334D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30E1, 0x30FC, 0x30C8, 0x30EB, 0x0000, 0x0000}},
  {0x334E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30E4, 0x30FC, 0x30C9, 0x0000, 0x0000, 0x0000}},
  {0x334F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30E4, 0x30FC, 0x30EB, 0x0000, 0x0000, 0x0000}},
  {0x3350, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30E6, 0x30A2, 0x30F3, 0x0000, 0x0000, 0x0000}},
  {0x3351, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EA, 0x30C3, 0x30C8, 0x30EB, 0x0000, 0x0000}},
  {0x3352, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EA, 0x30E9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3353, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EB, 0x30D4, 0x30FC, 0x0000, 0x0000, 0x0000}},
  {0x3354, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EB, 0x30FC, 0x30D6, 0x30EB, 0x0000, 0x0000}},
  {0x3355, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EC, 0x30E0, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3356, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EC, 0x30F3, 0x30C8, 0x30B2, 0x30F3, 0x0000}},
  {0x3357, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30EF, 0x30C3, 0x30C8, 0x0000, 0x0000, 0x0000}},
  {0x3358, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0030, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3359, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x335A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x335B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0033, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x335C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0034, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x335D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0035, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x335E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0036, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x335F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0037, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3360, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0038, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3361, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0039, 0x70B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3362, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0030, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3363, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0031, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3364, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0032, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3365, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0033, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3366, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0034, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3367, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0035, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3368, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0036, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3369, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0037, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x336A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0038, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x336B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0039, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x336C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0030, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x336D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0031, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x336E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0032, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x336F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0033, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3370, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0034, 0x70B9, 0x0000, 0x0000, 0x0000}},
  {0x3371, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0068, 0x0050, 0x0061, 0x0000, 0x0000, 0x0000}},
  {0x3372, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0064, 0x0061, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3373, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0041, 0x0055, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3374, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0062, 0x0061, 0x0072, 0x0000, 0x0000, 0x0000}},
  {0x3375, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006F, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3376, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x0063, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3377, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0064, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3378, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0064, 0x006D, 0x00B2, 0x0000, 0x0000, 0x0000}},
  {0x3379, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0064, 0x006D, 0x00B3, 0x0000, 0x0000, 0x0000}},
  {0x337A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0049, 0x0055, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x337B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5E73, 0x6210, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x337C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x662D, 0x548C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x337D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5927, 0x6B63, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x337E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x660E, 0x6CBB, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x337F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x682A, 0x5F0F, 0x4F1A, 0x793E, 0x0000, 0x0000}},
  {0x3380, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x0041, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3381, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006E, 0x0041, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3382, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x0041, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3383, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0041, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3384, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0041, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3385, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004B, 0x0042, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3386, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0042, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3387, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0047, 0x0042, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3388, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0063, 0x0061, 0x006C, 0x0000, 0x0000, 0x0000}},
  {0x3389, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0063, 0x0061, 0x006C, 0x0000, 0x0000}},
  {0x338A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x0046, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x338B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006E, 0x0046, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x338C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x0046, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x338D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x0067, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x338E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0067, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x338F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0067, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3390, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0048, 0x007A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3391, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0048, 0x007A, 0x0000, 0x0000, 0x0000}},
  {0x3392, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0048, 0x007A, 0x0000, 0x0000, 0x0000}},
  {0x3393, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0047, 0x0048, 0x007A, 0x0000, 0x0000, 0x0000}},
  {0x3394, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0054, 0x0048, 0x007A, 0x0000, 0x0000, 0x0000}},
  {0x3395, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x2113, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3396, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x2113, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3397, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0064, 0x2113, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3398, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x2113, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x3399, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0066, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x339A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006E, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x339B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x339C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x339D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0063, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x339E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x339F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x006D, 0x00B2, 0x0000, 0x0000, 0x0000}},
  {0x33A0, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0063, 0x006D, 0x00B2, 0x0000, 0x0000, 0x0000}},
  {0x33A1, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x00B2, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33A2, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x006D, 0x00B2, 0x0000, 0x0000, 0x0000}},
  {0x33A3, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x006D, 0x00B3, 0x0000, 0x0000, 0x0000}},
  {0x33A4, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0063, 0x006D, 0x00B3, 0x0000, 0x0000, 0x0000}},
  {0x33A5, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x00B3, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33A6, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x006D, 0x00B3, 0x0000, 0x0000, 0x0000}},
  {0x33A7, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x2215, 0x0073, 0x0000, 0x0000, 0x0000}},
  {0x33A8, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x2215, 0x0073, 0x00B2, 0x0000, 0x0000}},
  {0x33A9, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0061, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33AA, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0050, 0x0061, 0x0000, 0x0000, 0x0000}},
  {0x33AB, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0050, 0x0061, 0x0000, 0x0000, 0x0000}},
  {0x33AC, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0047, 0x0050, 0x0061, 0x0000, 0x0000, 0x0000}},
  {0x33AD, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0072, 0x0061, 0x0064, 0x0000, 0x0000, 0x0000}},
  {0x33AE, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0072, 0x0061, 0x0064, 0x2215, 0x0073, 0x0000}},
  {0x33AF, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0072, 0x0061, 0x0064, 0x2215, 0x0073, 0x00B2}},
  {0x33B0, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x0073, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B1, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006E, 0x0073, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B2, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x0073, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B3, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0073, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B4, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B5, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006E, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B6, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B7, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B8, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33B9, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33BA, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x0057, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33BB, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006E, 0x0057, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33BC, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x03BC, 0x0057, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33BD, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0057, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33BE, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0057, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33BF, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0057, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C0, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x03A9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C1, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x03A9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C2, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0061, 0x002E, 0x006D, 0x002E, 0x0000, 0x0000}},
  {0x33C3, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0042, 0x0071, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C4, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0063, 0x0063, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C5, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0063, 0x0064, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C6, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0043, 0x2215, 0x006B, 0x0067, 0x0000, 0x0000}},
  {0x33C7, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0043, 0x006F, 0x002E, 0x0000, 0x0000, 0x0000}},
  {0x33C8, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0064, 0x0042, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33C9, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0047, 0x0079, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33CA, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0068, 0x0061, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33CB, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0048, 0x0050, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33CC, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0069, 0x006E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33CD, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004B, 0x004B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33CE, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004B, 0x004D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33CF, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006B, 0x0074, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33D0, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006C, 0x006D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33D1, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006C, 0x006E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33D2, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006C, 0x006F, 0x0067, 0x0000, 0x0000, 0x0000}},
  {0x33D3, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006C, 0x0078, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33D4, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0062, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33D5, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x0069, 0x006C, 0x0000, 0x0000, 0x0000}},
  {0x33D6, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x006D, 0x006F, 0x006C, 0x0000, 0x0000, 0x0000}},
  {0x33D7, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0048, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33D8, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0070, 0x002E, 0x006D, 0x002E, 0x0000, 0x0000}},
  {0x33D9, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0050, 0x004D, 0x0000, 0x0000, 0x0000}},
  {0x33DA, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0052, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33DB, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0073, 0x0072, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33DC, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0053, 0x0076, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33DD, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0057, 0x0062, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33DE, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0056, 0x2215, 0x006D, 0x0000, 0x0000, 0x0000}},
  {0x33DF, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0041, 0x2215, 0x006D, 0x0000, 0x0000, 0x0000}},
  {0x33E0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0033, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0034, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0035, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0036, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0037, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0038, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0039, 0x65E5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x33E9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0030, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33EA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0031, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33EB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0032, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33EC, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0033, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33ED, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0034, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33EE, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0035, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33EF, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0036, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F0, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0037, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F1, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0038, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F2, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0031, 0x0039, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F3, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0030, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F4, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0031, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F5, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0032, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F6, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0033, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F7, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0034, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F8, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0035, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33F9, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0036, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33FA, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0037, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33FB, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0038, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33FC, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0032, 0x0039, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33FD, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0033, 0x0030, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33FE, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0033, 0x0031, 0x65E5, 0x0000, 0x0000, 0x0000}},
  {0x33FF, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0067, 0x0061, 0x006C, 0x0000, 0x0000, 0x0000}},
  {0xA69C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x044A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xA69D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x044C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xA770, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0xA76F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xA7F8, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0126, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xA7F9, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x0153, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xAB5C, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0xA727, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xAB5D, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0xAB37, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xAB5E, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0x026B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xAB5F, CHAR_CATEGORY_LM, DECOMP_TAG_SUPER,
   {0xAB52, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF900, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8C48, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF901, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x66F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF902, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8ECA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF903, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8CC8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF904, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6ED1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF905, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E32, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF906, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x53E5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF907, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F9C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF908, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F9C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF909, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5951, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF90A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x91D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF90B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5587, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF90C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5948, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF90D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61F6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF90E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7669, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF90F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F85, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF910, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x863F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF911, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x87BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF912, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88F8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF913, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x908F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF914, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6A02, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF915, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D1B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF916, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x70D9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF917, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x73DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF918, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x843D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF919, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x916A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF91A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x99F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF91B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E82, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF91C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5375, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF91D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B04, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF91E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x721B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF91F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x862D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF920, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9E1E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF921, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5D50, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF922, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6FEB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF923, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x85CD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF924, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8964, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF925, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF926, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x81D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF927, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x881F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF928, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5ECA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF929, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6717, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF92A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D6A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF92B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x72FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF92C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x90CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF92D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4F86, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF92E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF92F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF930, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x64C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF931, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6AD3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF932, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7210, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF933, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x76E7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF934, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF935, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8606, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF936, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x865C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF937, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8DEF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF938, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9732, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF939, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9B6F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF93A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9DFA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF93B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x788C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF93C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x797F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF93D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7DA0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF93E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF93F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9304, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF940, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9E7F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF941, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AD6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF942, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF943, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F04, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF944, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7C60, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF945, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x807E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF946, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7262, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF947, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x78CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF948, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8CC2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF949, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96F7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF94A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF94B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C62, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF94C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6A13, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF94D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6DDA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF94E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6F0F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF94F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7D2F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF950, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7E37, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF951, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x964B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF952, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF953, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x808B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF954, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51DC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF955, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF956, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7A1C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF957, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7DBE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF958, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF959, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9675, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF95A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B80, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF95B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62CF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF95C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6A02, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF95D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF95E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E39, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF95F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BE7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF960, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6012, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF961, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7387, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF962, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7570, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF963, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5317, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF964, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x78FB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF965, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4FBF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF966, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5FA9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF967, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E0D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF968, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6CCC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF969, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6578, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF96A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7D22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF96B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x53C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF96C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x585E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF96D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7701, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF96E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8449, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF96F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AAA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF970, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6BBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF971, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8FB0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF972, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6C88, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF973, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62FE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF974, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x82E5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF975, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x63A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF976, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7565, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF977, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4EAE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF978, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5169, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF979, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF97A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6881, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF97B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7CE7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF97C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x826F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF97D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AD2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF97E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x91CF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF97F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF980, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5442, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF981, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5973, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF982, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EEC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF983, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x65C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF984, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6FFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF985, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x792A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF986, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x95AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF987, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9A6A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF988, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9E97, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF989, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9ECE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF98A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x529B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF98B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x66C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF98C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B77, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF98D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F62, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF98E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5E74, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF98F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6190, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF990, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6200, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF991, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x649A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF992, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6F23, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF993, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7149, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF994, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7489, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF995, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x79CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF996, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7DF4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF997, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x806F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF998, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F26, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF999, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x84EE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF99A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9023, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF99B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x934A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF99C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5217, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF99D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF99E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x54BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF99F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x70C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AAA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EC9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5FF5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x637B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6BAE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7C3E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7375, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4EE4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9A9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x56F9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9AA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BE7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9AB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5DBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9AC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x601C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9AD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x73B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9AE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7469, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9AF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F9A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9234, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96F6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9748, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9818, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4F8B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x79AE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x91B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9B9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x60E1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9BA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E86, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9BB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x50DA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9BC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BEE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9BD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C3F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9BE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6599, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9BF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6A02, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x71CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x84FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x907C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F8D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6688, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x962E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5289, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x677B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9C9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x67F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9CA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D41, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9CB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6E9C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9CC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7409, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9CD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7559, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9CE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x786B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9CF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7D10, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x985E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x516D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x622E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9678, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x502B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5D19, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6DEA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F2A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F8B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9D9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6144, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9DA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6817, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9DB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7387, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9DC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9686, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9DD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5229, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9DE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x540F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9DF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C65, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6613, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x674E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x68A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6CE5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7406, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x75E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F79, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88CF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88E1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9E9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x91CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9EA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9EB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x533F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9EC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6EBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9ED, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x541D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9EE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x71D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9EF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7498, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x85FA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9C57, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9E9F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6797, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6DCB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x81E8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7ACB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7B20, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9F9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7C92, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9FA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x72C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9FB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7099, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9FC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B58, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9FD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4EC0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9FE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8336, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xF9FF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x523A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA00, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA01, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EA6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA02, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA03, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7CD6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA04, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5B85, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA05, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D1E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA06, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x66B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA07, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F3B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA08, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x884C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA09, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x964D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA0A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x898B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA0B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5ED3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA0C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5140, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA0D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x55C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA10, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x585A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA12, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6674, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA15, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA16, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x732A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA17, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x76CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA18, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x793C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA19, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x795E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA1A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7965, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA1B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x798F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA1C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9756, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA1D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7CBE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA1E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7FBD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA20, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8612, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA22, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AF8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA25, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA26, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x90FD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA2A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x98EF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA2B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x98FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA2C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9928, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA2D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9DB4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA2E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x90DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA2F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA30, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4FAE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA31, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x50E7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA32, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x514D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA33, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA34, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52E4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA35, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5351, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA36, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x559D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA37, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5606, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA38, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5668, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA39, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5840, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA3A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA3B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C64, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA3C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA3D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6094, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA3E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6168, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA3F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x618E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA40, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA41, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x654F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA42, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x65E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA43, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6691, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA44, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6885, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA45, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D77, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA46, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6E1A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA47, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6F22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA48, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x716E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA49, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x722B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA4A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7422, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA4B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7891, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA4C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x793E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA4D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7949, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA4E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7948, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA4F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7950, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA50, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7956, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA51, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x795D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA52, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x798D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA53, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x798E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA54, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7A40, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA55, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7A81, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA56, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7BC0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA57, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7DF4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA58, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7E09, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA59, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7E41, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA5A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F72, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA5B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8005, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA5C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x81ED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA5D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8279, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA5E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8279, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA5F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8457, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA60, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8910, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA61, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8996, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA62, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B01, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA63, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B39, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA64, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8CD3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA65, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8D08, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA66, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8FB6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA67, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA68, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA69, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x97FF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA6A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x983B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA6B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA6C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x242EE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA6D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8218, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA70, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E26, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA71, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA72, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5168, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA73, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4F80, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA74, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5145, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA75, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5180, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA76, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA77, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52FA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA78, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x559D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA79, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5555, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA7A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5599, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA7B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x55E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA7C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x585A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA7D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA7E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5944, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA7F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5954, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA80, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5A62, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA81, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5B28, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA82, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5ED2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA83, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5ED9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA84, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F69, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA85, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5FAD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA86, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x60D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA87, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x614E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA88, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6108, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA89, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x618E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA8A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6160, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA8B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA8C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6234, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA8D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x63C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA8E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x641C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA8F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6452, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA90, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6556, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA91, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6674, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA92, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6717, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA93, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x671B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA94, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6756, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA95, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B79, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA96, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6BBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA97, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D41, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA98, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6EDB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA99, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6ECB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA9A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6F22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA9B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x701E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA9C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x716E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA9D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x77A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA9E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7235, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFA9F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x72AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x732A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7471, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7506, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x753B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x761D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x761F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x76CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x76DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x76F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAA9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x774A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAAA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7740, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAAB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x78CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAAC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7AB1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAAD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7BC0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAAE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7C7B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAAF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7D5B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7DF4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F3E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8005, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8352, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83EF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8779, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8941, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8986, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8996, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAB9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8ABF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFABA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AF8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFABB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8ACB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFABC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B01, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFABD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFABE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFABF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B39, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B8A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8D08, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F38, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9199, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9276, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x967C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9756, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAC9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x97DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFACA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x97FF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFACB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x980B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFACC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x983B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFACD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9B12, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFACE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F9C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFACF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2284A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x22844, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x233D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3B9D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4018, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25249, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25CD0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x27ED3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F43, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFAD9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F8E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB00, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0066, 0x0066, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB01, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0066, 0x0069, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB02, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0066, 0x006C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB03, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0066, 0x0066, 0x0069, 0x0000, 0x0000, 0x0000}},
  {0xFB04, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0066, 0x0066, 0x006C, 0x0000, 0x0000, 0x0000}},
  {0xFB05, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x017F, 0x0074, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB06, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0073, 0x0074, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB13, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0574, 0x0576, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB14, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0574, 0x0565, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB15, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0574, 0x056B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB16, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x057E, 0x0576, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB17, CHAR_CATEGORY_LL, DECOMP_TAG_COMPAT,
   {0x0574, 0x056D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB1D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D9, 0x05B4, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB1F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05F2, 0x05B7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB20, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB21, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB22, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB23, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05D4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB24, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB25, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05DC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB26, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05DD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB27, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05E8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB28, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x05EA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB29, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x002B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB2A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E9, 0x05C1, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB2B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E9, 0x05C2, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB2C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0xFB49, 0x05C1, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB2D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0xFB49, 0x05C2, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB2E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D0, 0x05B7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB2F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D0, 0x05B8, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB30, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D0, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB31, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D1, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB32, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D2, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB33, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D3, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB34, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D4, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB35, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D5, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB36, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D6, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB38, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D8, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB39, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D9, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB3A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05DA, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB3B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05DB, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB3C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05DC, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB3E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05DE, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB40, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E0, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB41, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E1, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB43, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E3, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB44, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E4, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB46, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E6, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB47, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E7, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB48, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E8, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB49, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E9, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB4A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05EA, 0x05BC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB4B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D5, 0x05B9, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB4C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05D1, 0x05BF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB4D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05DB, 0x05BF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB4E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x05E4, 0x05BF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB4F, CHAR_CATEGORY_LO, DECOMP_TAG_COMPAT,
   {0x05D0, 0x05DC, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB50, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0671, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB51, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0671, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB52, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x067B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB53, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x067B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB54, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x067B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB55, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x067B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB56, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x067E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB57, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x067E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB58, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x067E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB59, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x067E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB5A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0680, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB5B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0680, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB5C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0680, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB5D, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0680, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB5E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x067A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB5F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x067A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB60, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x067A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB61, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x067A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB62, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x067F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB63, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x067F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB64, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x067F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB65, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x067F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB66, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0679, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB67, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0679, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB68, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0679, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB69, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0679, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB6A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB6B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB6C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB6D, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB6E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB6F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB70, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB71, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB72, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0684, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB73, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0684, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB74, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0684, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB75, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0684, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB76, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0683, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB77, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0683, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB78, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0683, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB79, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0683, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB7A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0686, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB7B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0686, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB7C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0686, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB7D, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0686, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB7E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0687, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB7F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0687, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB80, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0687, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB81, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0687, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB82, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x068D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB83, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x068D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB84, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x068C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB85, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x068C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB86, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x068E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB87, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x068E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB88, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0688, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB89, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0688, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB8A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0698, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB8B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0698, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB8C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0691, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB8D, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0691, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB8E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB8F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB90, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB91, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB92, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB93, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB94, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB95, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB96, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB97, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB98, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB99, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB9A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB9B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB9C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB9D, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB9E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFB9F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA0, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA2, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA3, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA4, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA5, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA6, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA7, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBA9, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBAA, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBAB, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBAC, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBAD, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBAE, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBAF, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBB0, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBB1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD3, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD4, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD6, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD7, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD8, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBD9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBDA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBDB, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBDC, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBDD, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0677, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBDE, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06CB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBDF, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06CB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE0, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE2, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE3, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE4, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE5, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE6, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE7, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06D0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0649, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBE9, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0649, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBEA, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBEB, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBEC, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x06D5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBED, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x06D5, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBEE, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x0648, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBEF, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0648, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF0, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x06C7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x06C7, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF2, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x06C6, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF3, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x06C6, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF4, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x06C8, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF5, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x06C8, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF6, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x06D0, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF7, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x06D0, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x06D0, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBF9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBFA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBFB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBFC, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x06CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBFD, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x06CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBFE, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x06CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFBFF, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x06CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC00, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC01, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC02, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC03, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC04, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC05, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC06, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC07, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC08, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC09, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC0A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC0B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC0C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC0D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC0E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC0F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC10, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC11, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062B, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC12, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062B, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC13, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062B, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC14, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062B, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC15, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062C, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC16, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062C, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC17, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062D, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC18, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062D, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC19, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062E, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC1A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062E, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC1B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062E, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC1C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC1D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC1E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC1F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC20, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC21, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC22, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC23, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC24, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC25, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC26, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0637, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC27, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0637, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC28, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0638, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC29, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0639, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC2A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0639, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC2B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x063A, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC2C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x063A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC2D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC2E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC2F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC30, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC31, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC32, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC33, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0642, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC34, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0642, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC35, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0642, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC36, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0642, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC37, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC38, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC39, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC3A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC3B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x0644, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC3C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC3D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC3E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC3F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC40, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC41, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC42, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC43, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC44, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC45, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC46, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC47, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC48, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC49, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC4A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC4B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC4C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC4D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC4E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC4F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC50, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC51, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0647, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC52, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0647, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC53, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0647, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC54, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0647, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC55, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC56, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC57, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC58, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC59, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC5A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC5B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0630, 0x0670, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC5C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0631, 0x0670, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC5D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0649, 0x0670, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC5E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064C, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFC5F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064D, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFC60, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064E, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFC61, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064F, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFC62, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x0650, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFC63, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x0651, 0x0670, 0x0000, 0x0000, 0x0000}},
  {0xFC64, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC65, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0632, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC66, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC67, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0646, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC68, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC69, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC6A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC6B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x0632, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC6C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC6D, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x0646, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC6E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC6F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC70, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC71, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0632, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC72, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC73, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0646, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC74, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC75, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC76, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC77, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x0632, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC78, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC79, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x0646, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC7A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC7B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC7C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0641, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC7D, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0641, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC7E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0642, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC7F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0642, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC80, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC81, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0644, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC82, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC83, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC84, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC85, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC86, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC87, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC88, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC89, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC8A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC8B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0632, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC8C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC8D, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0646, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC8E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC8F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC90, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0649, 0x0670, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC91, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC92, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0632, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC93, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC94, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0646, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC95, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC96, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC97, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC98, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC99, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC9A, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC9B, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC9C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0628, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC9D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0628, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC9E, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0628, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFC9F, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0628, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA0, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0628, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA1, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA2, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA4, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA6, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062B, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062C, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062C, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCA9, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062D, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCAA, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062D, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCAB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062E, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCAC, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062E, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCAD, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCAE, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCAF, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB0, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB1, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0635, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB2, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0635, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0635, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB4, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0636, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0636, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB6, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0636, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0636, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0637, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCB9, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0638, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCBA, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0639, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCBB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0639, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCBC, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x063A, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCBD, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x063A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCBE, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0641, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCBF, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0641, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC0, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0641, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC1, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0641, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC2, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0642, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0642, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC4, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC6, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x0644, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCC9, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCCA, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCCB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCCC, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCCD, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCCE, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCCF, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD0, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD1, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD2, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD4, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD6, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0647, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0647, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCD9, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0647, 0x0670, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCDA, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCDB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCDC, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCDD, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCDE, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCDF, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0626, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE0, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0626, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE1, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0628, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE2, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0628, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE3, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062A, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE5, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062B, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE6, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062B, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE7, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0633, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE8, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0633, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCE9, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0634, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCEA, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0634, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCEB, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0643, 0x0644, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCEC, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0643, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCED, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0644, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCEE, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0646, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCEF, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0646, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF0, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x064A, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF1, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x064A, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF2, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x064E, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFCF3, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x064F, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFCF4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x0650, 0x0651, 0x0000, 0x0000, 0x0000}},
  {0xFCF5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0637, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF6, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0637, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF7, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0639, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF8, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0639, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCF9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x063A, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCFA, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x063A, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCFB, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCFC, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCFD, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCFE, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFCFF, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062D, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD00, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062D, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD01, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062C, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD02, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062C, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD03, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062E, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD04, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062E, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD05, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD06, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD07, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD08, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD09, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD0A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD0B, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD0C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD0D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD0E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD0F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD10, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD11, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0637, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD12, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0637, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD13, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD14, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD15, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x063A, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD16, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x063A, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD17, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD18, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD19, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD1A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD1B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062D, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD1C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062D, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD1D, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD1E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD1F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062E, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD20, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062E, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD21, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD22, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD23, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x0649, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD24, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x064A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD25, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD26, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD27, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD28, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD29, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD2A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD2B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD2C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x0631, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD2D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD2E, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD2F, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD30, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD31, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD32, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x0647, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD33, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0637, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD34, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0633, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD35, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0633, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD36, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0633, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD37, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0634, 0x062C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD38, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0634, 0x062D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD39, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0634, 0x062E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD3A, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0637, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD3B, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0638, 0x0645, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD3C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0627, 0x064B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD3D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0627, 0x064B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFD50, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD51, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x062D, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD52, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062D, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD53, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD54, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD55, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x0645, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD56, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD57, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x0645, 0x062E, 0x0000, 0x0000, 0x0000}},
  {0xFD58, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD59, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062C, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD5A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062D, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD5B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062D, 0x0645, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD5C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x062D, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD5D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x062C, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD5E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x062C, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD5F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD60, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD61, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x0645, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD62, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD63, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD64, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x062D, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD65, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0635, 0x062D, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD66, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD67, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD68, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD69, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD6A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x0645, 0x062E, 0x0000, 0x0000, 0x0000}},
  {0xFD6B, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x0645, 0x062E, 0x0000, 0x0000, 0x0000}},
  {0xFD6C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD6D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD6E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x062D, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD6F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD70, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0636, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD71, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0637, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD72, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0637, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD73, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0637, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD74, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0637, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD75, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD76, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD77, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0639, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD78, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x0645, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD79, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x063A, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD7A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x063A, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD7B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x063A, 0x0645, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD7C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0641, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD7D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0641, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD7E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0642, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD7F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0642, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD80, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD81, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD82, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062D, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD83, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062C, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD84, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062C, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD85, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD86, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD87, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD88, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD89, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062D, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD8A, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD8B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD8C, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062C, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFD8D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD8E, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062E, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD8F, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062E, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD92, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x062C, 0x062E, 0x0000, 0x0000, 0x0000}},
  {0xFD93, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0647, 0x0645, 0x062C, 0x0000, 0x0000, 0x0000}},
  {0xFD94, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0647, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD95, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD96, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x062D, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD97, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD98, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD99, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x062C, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD9A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD9B, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0645, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFD9C, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD9D, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFD9E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x062E, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFD9F, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDA0, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x062C, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDA1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x062E, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDA2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x062E, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDA3, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDA4, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0645, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDA5, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDA6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x062D, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDA7, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x0645, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDA8, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x062E, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDA9, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDAA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDAB, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDAC, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDAD, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDAE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDAF, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB0, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0642, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB3, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB4, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0642, 0x0645, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFDB5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062D, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDB6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB7, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDB8, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x062C, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFDB9, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x062E, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDBA, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDBB, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDBC, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDBD, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x062C, 0x062D, 0x0000, 0x0000, 0x0000}},
  {0xFDBE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDBF, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062D, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDC0, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDC1, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0641, 0x0645, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDC2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x062D, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDC3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDC4, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0639, 0x062C, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDC5, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0635, 0x0645, 0x0645, 0x0000, 0x0000, 0x0000}},
  {0xFDC6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x062E, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDC7, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x062C, 0x064A, 0x0000, 0x0000, 0x0000}},
  {0xFDF0, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0644, 0x06D2, 0x0000, 0x0000, 0x0000}},
  {0xFDF1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0642, 0x0644, 0x06D2, 0x0000, 0x0000, 0x0000}},
  {0xFDF2, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0627, 0x0644, 0x0644, 0x0647, 0x0000, 0x0000}},
  {0xFDF3, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0627, 0x0643, 0x0628, 0x0631, 0x0000, 0x0000}},
  {0xFDF4, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x062D, 0x0645, 0x062F, 0x0000, 0x0000}},
  {0xFDF5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0644, 0x0639, 0x0645, 0x0000, 0x0000}},
  {0xFDF6, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0631, 0x0633, 0x0648, 0x0644, 0x0000, 0x0000}},
  {0xFDF7, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0639, 0x0644, 0x064A, 0x0647, 0x0000, 0x0000}},
  {0xFDF8, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0648, 0x0633, 0x0644, 0x0645, 0x0000, 0x0000}},
  {0xFDF9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0644, 0x0649, 0x0000, 0x0000, 0x0000}},
  {0xFDFA, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0644, 0x0649, 0x0020, 0x0627, 0x0644}},
  {0xFDFB, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062C, 0x0644, 0x0020, 0x062C, 0x0644, 0x0627}},
  {0xFDFC, CHAR_CATEGORY_SC, DECOMP_TAG_ISOLATED,
   {0x0631, 0x06CC, 0x0627, 0x0644, 0x0000, 0x0000}},
  {0xFE10, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x002C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE11, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x3001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE12, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x3002, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE13, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x003A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE14, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x003B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE15, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x0021, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE16, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x003F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE17, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x3016, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE18, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x3017, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE19, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x2026, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE30, CHAR_CATEGORY_PO, DECOMP_TAG_VERTICAL,
   {0x2025, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE31, CHAR_CATEGORY_PD, DECOMP_TAG_VERTICAL,
   {0x2014, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE32, CHAR_CATEGORY_PD, DECOMP_TAG_VERTICAL,
   {0x2013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE33, CHAR_CATEGORY_PC, DECOMP_TAG_VERTICAL,
   {0x005F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE34, CHAR_CATEGORY_PC, DECOMP_TAG_VERTICAL,
   {0x005F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE35, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x0028, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE36, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x0029, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE37, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x007B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE38, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x007D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE39, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x3014, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE3A, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x3015, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE3B, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x3010, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE3C, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x3011, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE3D, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x300A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE3E, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x300B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE3F, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x3008, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE40, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x3009, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE41, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x300C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE42, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x300D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE43, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x300E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE44, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x300F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE47, CHAR_CATEGORY_PS, DECOMP_TAG_VERTICAL,
   {0x005B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE48, CHAR_CATEGORY_PE, DECOMP_TAG_VERTICAL,
   {0x005D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE49, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x203E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE4A, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x203E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE4B, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x203E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE4C, CHAR_CATEGORY_PO, DECOMP_TAG_COMPAT,
   {0x203E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE4D, CHAR_CATEGORY_PC, DECOMP_TAG_COMPAT,
   {0x005F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE4E, CHAR_CATEGORY_PC, DECOMP_TAG_COMPAT,
   {0x005F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE4F, CHAR_CATEGORY_PC, DECOMP_TAG_COMPAT,
   {0x005F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE50, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x002C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE51, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x3001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE52, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x002E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE54, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x003B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE55, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x003A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE56, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x003F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE57, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x0021, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE58, CHAR_CATEGORY_PD, DECOMP_TAG_SMALL,
   {0x2014, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE59, CHAR_CATEGORY_PS, DECOMP_TAG_SMALL,
   {0x0028, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE5A, CHAR_CATEGORY_PE, DECOMP_TAG_SMALL,
   {0x0029, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE5B, CHAR_CATEGORY_PS, DECOMP_TAG_SMALL,
   {0x007B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE5C, CHAR_CATEGORY_PE, DECOMP_TAG_SMALL,
   {0x007D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE5D, CHAR_CATEGORY_PS, DECOMP_TAG_SMALL,
   {0x3014, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE5E, CHAR_CATEGORY_PE, DECOMP_TAG_SMALL,
   {0x3015, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE5F, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x0023, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE60, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x0026, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE61, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x002A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE62, CHAR_CATEGORY_SM, DECOMP_TAG_SMALL,
   {0x002B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE63, CHAR_CATEGORY_PD, DECOMP_TAG_SMALL,
   {0x002D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE64, CHAR_CATEGORY_SM, DECOMP_TAG_SMALL,
   {0x003C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE65, CHAR_CATEGORY_SM, DECOMP_TAG_SMALL,
   {0x003E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE66, CHAR_CATEGORY_SM, DECOMP_TAG_SMALL,
   {0x003D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE68, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x005C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE69, CHAR_CATEGORY_SC, DECOMP_TAG_SMALL,
   {0x0024, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE6A, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x0025, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE6B, CHAR_CATEGORY_PO, DECOMP_TAG_SMALL,
   {0x0040, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE70, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE71, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x064B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE72, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE74, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064D, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE76, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE77, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x064E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE78, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x064F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE79, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x064F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE7A, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x0650, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE7B, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x0650, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE7C, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x0651, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE7D, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x0651, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE7E, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0020, 0x0652, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE7F, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0640, 0x0652, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE80, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0621, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE81, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0622, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE82, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0622, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE83, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0623, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE84, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0623, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE85, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0624, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE86, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0624, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE87, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0625, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE88, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0625, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE89, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0626, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE8A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0626, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE8B, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0626, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE8C, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0626, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE8D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0627, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE8E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0627, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE8F, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE90, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE91, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE92, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE93, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0629, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE94, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0629, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE95, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE96, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE97, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE98, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE99, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE9A, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE9B, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE9C, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE9D, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE9E, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFE9F, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA0, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA8, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEA9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x062F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEAA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x062F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEAB, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0630, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEAC, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0630, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEAD, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0631, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEAE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0631, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEAF, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0632, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB0, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0632, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB8, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEB9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEBA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEBB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEBC, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEBD, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEBE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEBF, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC0, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC8, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEC9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFECA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFECB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFECC, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFECD, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFECE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFECF, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED0, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED8, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFED9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEDA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEDB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEDC, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEDD, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEDE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEDF, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE0, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE7, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE8, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEE9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEEA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEEB, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEEC, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEED, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0648, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEEE, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0648, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEEF, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0649, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF0, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0649, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF1, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF2, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF3, CHAR_CATEGORY_LO, DECOMP_TAG_INITIAL,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF4, CHAR_CATEGORY_LO, DECOMP_TAG_MEDIAL,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF5, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0622, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF6, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0622, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF7, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0623, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF8, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0623, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEF9, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0625, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEFA, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0625, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEFB, CHAR_CATEGORY_LO, DECOMP_TAG_ISOLATED,
   {0x0644, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFEFC, CHAR_CATEGORY_LO, DECOMP_TAG_FINAL,
   {0x0644, 0x0627, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF01, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0021, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF02, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0022, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF03, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0023, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF04, CHAR_CATEGORY_SC, DECOMP_TAG_WIDE,
   {0x0024, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF05, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0025, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF06, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0026, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF07, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0027, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF08, CHAR_CATEGORY_PS, DECOMP_TAG_WIDE,
   {0x0028, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF09, CHAR_CATEGORY_PE, DECOMP_TAG_WIDE,
   {0x0029, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF0A, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x002A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF0B, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x002B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF0C, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x002C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF0D, CHAR_CATEGORY_PD, DECOMP_TAG_WIDE,
   {0x002D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF0E, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x002E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF0F, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x002F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF10, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF11, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF12, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF13, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF14, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF15, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF16, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF17, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF18, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF19, CHAR_CATEGORY_ND, DECOMP_TAG_WIDE,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF1A, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x003A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF1B, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x003B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF1C, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x003C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF1D, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x003D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF1E, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x003E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF1F, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x003F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF20, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x0040, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF21, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF22, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF23, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF24, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF25, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF26, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF27, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF28, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF29, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF2A, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF2B, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF2C, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF2D, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF2E, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF2F, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF30, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF31, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF32, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF33, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF34, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF35, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF36, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF37, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF38, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF39, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF3A, CHAR_CATEGORY_LU, DECOMP_TAG_WIDE,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF3B, CHAR_CATEGORY_PS, DECOMP_TAG_WIDE,
   {0x005B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF3C, CHAR_CATEGORY_PO, DECOMP_TAG_WIDE,
   {0x005C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF3D, CHAR_CATEGORY_PE, DECOMP_TAG_WIDE,
   {0x005D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF3E, CHAR_CATEGORY_SK, DECOMP_TAG_WIDE,
   {0x005E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF3F, CHAR_CATEGORY_PC, DECOMP_TAG_WIDE,
   {0x005F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF40, CHAR_CATEGORY_SK, DECOMP_TAG_WIDE,
   {0x0060, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF41, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF42, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF43, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF44, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF45, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF46, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF47, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF48, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF49, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF4A, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF4B, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF4C, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF4D, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF4E, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF4F, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF50, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF51, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF52, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF53, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF54, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF55, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF56, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF57, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF58, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF59, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF5A, CHAR_CATEGORY_LL, DECOMP_TAG_WIDE,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF5B, CHAR_CATEGORY_PS, DECOMP_TAG_WIDE,
   {0x007B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF5C, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x007C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF5D, CHAR_CATEGORY_PE, DECOMP_TAG_WIDE,
   {0x007D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF5E, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x007E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF5F, CHAR_CATEGORY_PS, DECOMP_TAG_WIDE,
   {0x2985, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF60, CHAR_CATEGORY_PE, DECOMP_TAG_WIDE,
   {0x2986, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF61, CHAR_CATEGORY_PO, DECOMP_TAG_NARROW,
   {0x3002, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF62, CHAR_CATEGORY_PS, DECOMP_TAG_NARROW,
   {0x300C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF63, CHAR_CATEGORY_PE, DECOMP_TAG_NARROW,
   {0x300D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF64, CHAR_CATEGORY_PO, DECOMP_TAG_NARROW,
   {0x3001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF65, CHAR_CATEGORY_PO, DECOMP_TAG_NARROW,
   {0x30FB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF66, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF67, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF68, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF69, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF6A, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF6B, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF6C, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF6D, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF6E, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF6F, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF70, CHAR_CATEGORY_LM, DECOMP_TAG_NARROW,
   {0x30FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF71, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF72, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF73, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF74, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF75, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30AA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF76, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30AB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF77, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF78, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF79, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF7A, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF7B, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF7C, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF7D, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF7E, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF7F, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF80, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF81, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF82, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF83, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF84, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF85, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF86, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30CB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF87, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF88, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30CD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF89, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF8A, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30CF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF8B, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF8C, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF8D, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF8E, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30DB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF8F, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF90, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF91, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF92, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF93, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF94, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF95, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF96, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF97, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30E9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF98, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30EA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF99, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30EB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF9A, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30EC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF9B, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30ED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF9C, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30EF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF9D, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x30F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF9E, CHAR_CATEGORY_LM, DECOMP_TAG_NARROW,
   {0x3099, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFF9F, CHAR_CATEGORY_LM, DECOMP_TAG_NARROW,
   {0x309A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA0, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3164, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA1, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3131, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA2, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3132, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA3, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3133, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA4, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3134, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA5, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3135, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA6, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3136, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA7, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3137, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA8, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3138, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFA9, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3139, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFAA, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x313A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFAB, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x313B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFAC, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x313C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFAD, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x313D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFAE, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x313E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFAF, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x313F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB0, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3140, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB1, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3141, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB2, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3142, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB3, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3143, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB4, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3144, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB5, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3145, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB6, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3146, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB7, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3147, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB8, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3148, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFB9, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3149, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFBA, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x314A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFBB, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x314B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFBC, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x314C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFBD, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x314D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFBE, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x314E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFC2, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x314F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFC3, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3150, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFC4, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3151, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFC5, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3152, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFC6, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3153, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFC7, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3154, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFCA, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3155, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFCB, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3156, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFCC, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3157, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFCD, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3158, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFCE, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3159, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFCF, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x315A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFD2, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x315B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFD3, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x315C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFD4, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x315D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFD5, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x315E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFD6, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x315F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFD7, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3160, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFDA, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3161, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFDB, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3162, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFDC, CHAR_CATEGORY_LO, DECOMP_TAG_NARROW,
   {0x3163, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE0, CHAR_CATEGORY_SC, DECOMP_TAG_WIDE,
   {0x00A2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE1, CHAR_CATEGORY_SC, DECOMP_TAG_WIDE,
   {0x00A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE2, CHAR_CATEGORY_SM, DECOMP_TAG_WIDE,
   {0x00AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE3, CHAR_CATEGORY_SK, DECOMP_TAG_WIDE,
   {0x00AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE4, CHAR_CATEGORY_SO, DECOMP_TAG_WIDE,
   {0x00A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE5, CHAR_CATEGORY_SC, DECOMP_TAG_WIDE,
   {0x00A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE6, CHAR_CATEGORY_SC, DECOMP_TAG_WIDE,
   {0x20A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE8, CHAR_CATEGORY_SO, DECOMP_TAG_NARROW,
   {0x2502, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFE9, CHAR_CATEGORY_SM, DECOMP_TAG_NARROW,
   {0x2190, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFEA, CHAR_CATEGORY_SM, DECOMP_TAG_NARROW,
   {0x2191, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFEB, CHAR_CATEGORY_SM, DECOMP_TAG_NARROW,
   {0x2192, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFEC, CHAR_CATEGORY_SM, DECOMP_TAG_NARROW,
   {0x2193, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFED, CHAR_CATEGORY_SO, DECOMP_TAG_NARROW,
   {0x25A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0xFFEE, CHAR_CATEGORY_SO, DECOMP_TAG_NARROW,
   {0x25CB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1109A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x11099, 0x110BA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1109C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x1109B, 0x110BA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x110AB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x110A5, 0x110BA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1112E, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x11131, 0x11127, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1112F, CHAR_CATEGORY_MN, DECOMP_TAG_NONE,
   {0x11132, 0x11127, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1134B, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x11347, 0x1133E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1134C, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x11347, 0x11357, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x114BB, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x114B9, 0x114BA, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x114BC, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x114B9, 0x114B0, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x114BE, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x114B9, 0x114BD, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x115BA, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x115B8, 0x115AF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x115BB, CHAR_CATEGORY_MC, DECOMP_TAG_NONE,
   {0x115B9, 0x115AF, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D15E, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D157, 0x1D165, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D15F, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D158, 0x1D165, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D160, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D15F, 0x1D16E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D161, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D15F, 0x1D16F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D162, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D15F, 0x1D170, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D163, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D15F, 0x1D171, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D164, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D15F, 0x1D172, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D1BB, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D1B9, 0x1D165, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D1BC, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D1BA, 0x1D165, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D1BD, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D1BB, 0x1D16E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D1BE, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D1BC, 0x1D16E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D1BF, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D1BB, 0x1D16F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D1C0, CHAR_CATEGORY_SO, DECOMP_TAG_NONE,
   {0x1D1BC, 0x1D16F, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D400, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D401, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D402, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D403, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D404, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D405, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D406, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D407, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D408, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D409, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D40F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D410, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D411, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D412, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D413, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D414, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D415, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D416, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D417, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D418, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D419, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D41F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D420, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D421, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D422, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D423, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D424, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D425, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D426, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D427, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D428, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D429, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D42F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D430, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D431, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D432, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D433, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D434, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D435, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D436, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D437, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D438, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D439, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D43F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D440, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D441, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D442, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D443, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D444, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D445, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D446, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D447, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D448, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D449, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D44F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D450, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D451, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D452, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D453, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D454, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D456, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D457, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D458, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D459, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D45F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D460, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D461, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D462, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D463, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D464, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D465, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D466, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D467, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D468, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D469, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D46F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D470, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D471, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D472, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D473, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D474, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D475, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D476, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D477, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D478, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D479, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D47F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D480, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D481, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D482, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D483, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D484, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D485, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D486, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D487, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D488, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D489, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D48F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D490, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D491, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D492, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D493, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D494, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D495, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D496, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D497, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D498, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D499, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D49A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D49B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D49C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D49E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D49F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4A2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4A5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4A6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4A9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4AA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4AB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4AC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4AE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4AF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4B9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4BB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4BD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4BE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4BF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4C9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4CA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4CB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4CC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4CD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4CE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4CF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4D9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4DA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4DB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4DC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4DD, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4DE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4DF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4E9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4EA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4EB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4EC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4ED, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4EE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4EF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4F9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4FA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4FB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4FC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4FD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4FE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D4FF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D500, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D501, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D502, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D503, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D504, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D505, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D507, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D508, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D509, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D50A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D50D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D50E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D50F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D510, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D511, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D512, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D513, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D514, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D516, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D517, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D518, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D519, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D51A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D51B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D51C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D51E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D51F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D520, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D521, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D522, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D523, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D524, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D525, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D526, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D527, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D528, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D529, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D52F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D530, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D531, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D532, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D533, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D534, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D535, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D536, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D537, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D538, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D539, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D53B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D53C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D53D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D53E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D540, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D541, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D542, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D543, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D544, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D546, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D54F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D550, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D552, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D553, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D554, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D555, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D556, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D557, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D558, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D559, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D55F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D560, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D561, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D562, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D563, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D564, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D565, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D566, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D567, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D568, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D569, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D56F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D570, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D571, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D572, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D573, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D574, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D575, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D576, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D577, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D578, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D579, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D57F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D580, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D581, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D582, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D583, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D584, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D585, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D586, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D587, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D588, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D589, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D58F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D590, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D591, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D592, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D593, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D594, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D595, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D596, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D597, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D598, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D599, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D59F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5A9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5AA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5AB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5AC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5AD, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5AE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5AF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5B9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5BA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5BB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5BC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5BD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5BE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5BF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5C9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5CA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5CB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5CC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5CD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5CE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5CF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5D9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5DA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5DB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5DC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5DD, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5DE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5DF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5E9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5EA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5EB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5EC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5ED, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5EE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5EF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5F9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5FA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5FB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5FC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5FD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5FE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D5FF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D600, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D601, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D602, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D603, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D604, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D605, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D606, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D607, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D608, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D609, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D60F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D610, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D611, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D612, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D613, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D614, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D615, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D616, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D617, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D618, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D619, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D61F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D620, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D621, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D622, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D623, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D624, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D625, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D626, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D627, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D628, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D629, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D62F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D630, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D631, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D632, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D633, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D634, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D635, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D636, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D637, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D638, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D639, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D63F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D640, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D641, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D642, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D643, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D644, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D645, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D646, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D647, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D648, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D649, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D64F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D650, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D651, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D652, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D653, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D654, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D655, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D656, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D657, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D658, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D659, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D65F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D660, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D661, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D662, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D663, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D664, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D665, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D666, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D667, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D668, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D669, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D66F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D670, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D671, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D672, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D673, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D674, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D675, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D676, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D677, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D678, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D679, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D67F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D680, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D681, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D682, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D683, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D684, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D685, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D686, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D687, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D688, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D689, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0061, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0062, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0063, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0064, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0065, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D68F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0066, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D690, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0067, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D691, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0068, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D692, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0069, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D693, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D694, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D695, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D696, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D697, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D698, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x006F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D699, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0071, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0072, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0073, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0074, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0075, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D69F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0076, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0078, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0079, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x007A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0131, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x0237, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0391, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6A9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0392, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6AA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6AB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0394, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6AC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0395, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6AD, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0396, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6AE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0397, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6AF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0398, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0399, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6B9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6BA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6BB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6BC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6BD, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6BE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6BF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C1, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6C9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6CA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6CB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6CC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6CD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6CE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6CF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6D9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6DA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6DB, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2202, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6DC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6DD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6DE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6DF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0391, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0392, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0394, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0395, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0396, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0397, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6E9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0398, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6EA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0399, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6EB, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6EC, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6ED, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6EE, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6EF, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6F9, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6FA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6FB, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6FC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6FD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6FE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D6FF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D700, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D701, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D702, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D703, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D704, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D705, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D706, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D707, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D708, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D709, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D70A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D70B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D70C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D70D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D70E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D70F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D710, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D711, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D712, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D713, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D714, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D715, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2202, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D716, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D717, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D718, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D719, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D71A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D71B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D71C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0391, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D71D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0392, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D71E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D71F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0394, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D720, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0395, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D721, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0396, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D722, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0397, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D723, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0398, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D724, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0399, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D725, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D726, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D727, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D728, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D729, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D72A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D72B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D72C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D72D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D72E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D72F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D730, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D731, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D732, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D733, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D734, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D735, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D736, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D737, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D738, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D739, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D73A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D73B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D73C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D73D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D73E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D73F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D740, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D741, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D742, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D743, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D744, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D745, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D746, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D747, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D748, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D749, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D74A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D74B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D74C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D74D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D74E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D74F, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2202, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D750, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D751, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D752, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D753, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D754, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D755, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D756, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0391, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D757, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0392, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D758, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D759, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0394, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D75A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0395, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D75B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0396, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D75C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0397, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D75D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0398, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D75E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0399, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D75F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D760, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D761, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D762, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D763, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D764, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D765, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D766, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D767, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D768, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D769, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D76A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D76B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D76C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D76D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D76E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D76F, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D770, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D771, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D772, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D773, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D774, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D775, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D776, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D777, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D778, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D779, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D77A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D77B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D77C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D77D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D77E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D77F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D780, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D781, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D782, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D783, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D784, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D785, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D786, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D787, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D788, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D789, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2202, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78A, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78B, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78C, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78D, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78E, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D78F, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D790, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0391, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D791, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0392, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D792, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D793, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0394, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D794, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0395, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D795, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0396, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D796, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0397, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D797, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0398, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D798, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x0399, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D799, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D79A, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D79B, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D79C, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D79D, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D79E, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x039F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D79F, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A0, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A1, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A2, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A3, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A4, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A5, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A6, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A7, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A8, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7A9, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7AA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7AB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7AC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7AD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7AE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7AF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B3, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03BF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7B9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7BA, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7BB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7BC, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7BD, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7BE, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7BF, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C0, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C1, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C2, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C3, CHAR_CATEGORY_SM, DECOMP_TAG_FONT,
   {0x2202, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C4, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C5, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C6, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C7, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C8, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7C9, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03D6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7CA, CHAR_CATEGORY_LU, DECOMP_TAG_FONT,
   {0x03DC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7CB, CHAR_CATEGORY_LL, DECOMP_TAG_FONT,
   {0x03DD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7CE, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7CF, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D0, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D1, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D2, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D3, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D4, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D5, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D6, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D7, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D8, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7D9, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7DA, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7DB, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7DC, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7DD, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7DE, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7DF, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E0, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E1, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E2, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E3, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E4, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E5, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E6, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E7, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E8, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7E9, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7EA, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7EB, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7EC, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7ED, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7EE, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7EF, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F0, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F1, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F2, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F3, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F4, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F5, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F6, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0030, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F7, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0031, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F8, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0032, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7F9, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0033, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7FA, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0034, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7FB, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0035, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7FC, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0036, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7FD, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0037, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7FE, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0038, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1D7FF, CHAR_CATEGORY_ND, DECOMP_TAG_FONT,
   {0x0039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE00, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0627, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE01, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE02, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE03, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE05, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0648, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE06, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0632, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE07, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE08, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE09, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0A, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0C, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE0F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE10, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE11, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE12, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE13, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0631, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE14, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE15, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE16, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE17, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE18, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0630, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE19, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1A, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1C, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x066E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x06BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x06A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE1F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x066F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE21, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE22, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE24, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE27, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE29, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2A, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2C, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE2F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE30, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE31, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE32, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE34, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE35, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE36, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE37, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE39, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE3B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE42, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE47, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE49, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE4B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE4D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE4E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE4F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE51, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE52, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE54, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE57, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE59, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE5B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE5D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x06BA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE5F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x066F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE61, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE62, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE64, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE67, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE68, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE69, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE6A, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0643, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE6C, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE6D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE6E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE6F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE70, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE71, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE72, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE74, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE75, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE76, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE77, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE79, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE7A, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE7B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE7C, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x066E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE7E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x06A1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE80, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0627, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE81, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE82, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE83, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE84, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0647, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE85, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0648, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE86, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0632, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE87, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE88, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE89, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE8B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE8C, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE8D, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE8E, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE8F, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE90, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE91, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE92, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE93, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0631, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE94, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE95, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE96, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE97, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE98, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0630, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE99, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE9A, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EE9B, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA1, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0628, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA2, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA3, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA5, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0648, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA6, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0632, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA7, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA8, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0637, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEA9, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x064A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEAB, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0644, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEAC, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0645, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEAD, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0646, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEAE, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0633, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEAF, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0639, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB0, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0641, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB1, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB2, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0642, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB3, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0631, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB4, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0634, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB5, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB6, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB7, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x062E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB8, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0630, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEB9, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0636, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEBA, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x0638, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1EEBB, CHAR_CATEGORY_LO, DECOMP_TAG_FONT,
   {0x063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F100, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0030, 0x002E, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F101, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0030, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F102, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0031, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F103, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0032, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F104, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0033, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F105, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0034, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F106, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0035, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F107, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0036, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F108, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0037, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F109, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0038, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F10A, CHAR_CATEGORY_NO, DECOMP_TAG_COMPAT,
   {0x0039, 0x002C, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F110, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0041, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F111, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0042, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F112, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0043, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F113, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0044, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F114, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0045, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F115, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0046, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F116, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0047, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F117, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0048, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F118, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0049, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F119, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x004A, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F11A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x004B, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F11B, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x004C, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F11C, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x004D, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F11D, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x004E, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F11E, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x004F, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F11F, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0050, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F120, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0051, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F121, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0052, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F122, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0053, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F123, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0054, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F124, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0055, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F125, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0056, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F126, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0057, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F127, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0058, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F128, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x0059, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F129, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x0028, 0x005A, 0x0029, 0x0000, 0x0000, 0x0000}},
  {0x1F12A, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x0053, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F12B, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F12C, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F12D, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0043, 0x0044, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F12E, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x0057, 0x005A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F130, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0041, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F131, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0042, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F132, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0043, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F133, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F134, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0045, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F135, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F136, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0047, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F137, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0048, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F138, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0049, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F139, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F13F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F140, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0051, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F141, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0052, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F142, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0053, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F143, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0054, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F144, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0055, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F145, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0056, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F146, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0057, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F147, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0058, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F148, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0059, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F149, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x005A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F14A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0048, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F14B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x004D, 0x0056, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F14C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0053, 0x0044, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F14D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0053, 0x0053, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F14E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0050, 0x0050, 0x0056, 0x0000, 0x0000, 0x0000}},
  {0x1F14F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0057, 0x0043, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F16A, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x004D, 0x0043, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F16B, CHAR_CATEGORY_SO, DECOMP_TAG_SUPER,
   {0x004D, 0x0044, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F190, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x0044, 0x004A, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F200, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x307B, 0x304B, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F201, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B3, 0x30B3, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F202, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F210, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x624B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F211, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5B57, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F212, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x53CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F213, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x30C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F214, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x4E8C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F215, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x591A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F216, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x89E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F217, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5929, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F218, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x4EA4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F219, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6620, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x7121, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6599, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x524D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5F8C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x518D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F21F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x65B0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F220, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x521D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F221, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x7D42, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F222, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x751F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F223, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x8CA9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F224, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x58F0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F225, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5439, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F226, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6F14, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F227, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6295, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F228, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6355, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F229, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x4E00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x4E09, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x904A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22C, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5DE6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22D, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x4E2D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22E, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x53F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F22F, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6307, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F230, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x8D70, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F231, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6253, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F232, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x7981, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F233, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x7A7A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F234, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5408, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F235, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6E80, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F236, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6709, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F237, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x6708, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F238, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x7533, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F239, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x5272, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F23A, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x55B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F23B, CHAR_CATEGORY_SO, DECOMP_TAG_SQUARE,
   {0x914D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F240, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x672C, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F241, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x4E09, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F242, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x4E8C, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F243, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x5B89, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F244, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x70B9, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F245, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x6253, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F246, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x76D7, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F247, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x52DD, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F248, CHAR_CATEGORY_SO, DECOMP_TAG_COMPAT,
   {0x3014, 0x6557, 0x3015, 0x0000, 0x0000, 0x0000}},
  {0x1F250, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x5F97, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x1F251, CHAR_CATEGORY_SO, DECOMP_TAG_CIRCLE,
   {0x53EF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F800, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E3D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F801, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E38, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F802, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4E41, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F803, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x20122, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F804, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4F60, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F805, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4FAE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F806, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4FBB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F807, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5002, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F808, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x507A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F809, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5099, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x50E7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x50CF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x349E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2063A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x514D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F80F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5154, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F810, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5164, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F811, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5177, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F812, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2051C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F813, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x34B9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F814, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5167, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F815, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x518D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F816, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2054B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F817, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5197, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F818, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F819, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4ECC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x291DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x51F5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5203, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F81F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x34DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F820, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x523B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F821, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5246, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F822, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5272, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F823, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5277, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F824, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3515, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F825, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F826, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52C9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F827, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52E4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F828, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52FA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F829, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5305, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5306, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5317, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5349, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5351, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x535A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F82F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5373, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F830, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x537D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F831, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x537F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F832, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x537F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F833, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x537F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F834, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x20A2C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F835, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F836, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x53CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F837, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x53DF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F838, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x20B63, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F839, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x53EB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x53F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5406, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x549E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5438, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5448, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F83F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5468, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F840, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x54A2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F841, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x54F6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F842, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5510, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F843, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5553, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F844, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5563, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F845, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5584, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F846, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5584, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F847, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5599, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F848, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x55AB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F849, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x55B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x55C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5716, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5606, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5717, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5651, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F84F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5674, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F850, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5207, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F851, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58EE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F852, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x57CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F853, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x57F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F854, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x580D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F855, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x578B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F856, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5832, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F857, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5831, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F858, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58AC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F859, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x214E4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x58F7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5906, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x591A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5922, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F85F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5962, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F860, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x216A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F861, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x216EA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F862, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x59EC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F863, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5A1B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F864, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5A27, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F865, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x59D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F866, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5A66, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F867, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x36EE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F868, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x36FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F869, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5B08, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5B3E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5B3E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x219C8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BC3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BD8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F86F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BE7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F870, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BF3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F871, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x21B18, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F872, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5BFF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F873, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C06, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F874, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F53, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F875, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F876, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3781, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F877, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C60, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F878, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F879, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5CC0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5C8D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x21DE4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5D43, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x21DE6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5D6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F87F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5D6B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F880, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5D7C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F881, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5DE1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F882, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5DE2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F883, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x382F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F884, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5DFD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F885, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5E28, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F886, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5E3D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F887, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5E69, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F888, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3862, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F889, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x22183, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x387C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EB0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EB3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EB6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5ECA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F88F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2A392, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F890, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5EFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F891, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x22331, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F892, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x22331, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F893, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8201, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F894, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F895, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F22, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F896, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x38C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F897, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x232B8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F898, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x261DA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F899, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F62, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F6B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x38E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5F9A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5FCD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5FD7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F89F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5FF9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6081, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x393A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x391C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6094, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x226D4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x60C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6148, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x614C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x614E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8A9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x614C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8AA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x617A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8AB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x618E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8AC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8AD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61A4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8AE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61AF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8AF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x61F6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6210, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x621B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x625D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62D4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6350, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x22B0C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8B9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x633D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8BA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x62FC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8BB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6368, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8BC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6383, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8BD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x63E4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8BE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x22BF1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8BF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6422, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x63C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x63A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3A2E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6469, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x647E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x649D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6477, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3A6C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x654F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8C9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x656C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8CA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2300A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8CB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x65E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8CC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x66F8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8CD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6649, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8CE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3B19, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8CF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6691, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3B08, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3AE4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5192, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5195, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6700, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x669C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x80AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x43D9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6717, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8D9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x671B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8DA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6721, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8DB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x675E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8DC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6753, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8DD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x233C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8DE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3B49, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8DF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x67FA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6785, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6852, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6885, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2346D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x688E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x681F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6914, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3B9D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6942, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8E9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x69A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8EA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x69EA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8EB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6AA8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8EC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x236A3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8ED, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6ADB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8EE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3C18, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8EF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B21, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x238A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B54, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3C4E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B72, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6B9F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6BBA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6BBB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23A8D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x21D0B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8F9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23AFA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8FA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6C4E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8FB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23CBC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8FC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6CBF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8FD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6CCD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8FE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6C67, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F8FF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D16, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F900, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D3E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F901, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D77, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F902, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D41, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F903, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D69, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F904, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D78, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F905, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D85, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F906, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23D1E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F907, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6D34, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F908, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6E2F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F909, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6E6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3D33, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6ECB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6EC7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23ED1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6DF9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F90F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6F6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F910, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23F5E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F911, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23F8E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F912, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x6FC6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F913, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F914, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x701E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F915, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x701B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F916, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3D96, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F917, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x704A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F918, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x707D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F919, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7077, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x70AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x20525, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7145, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24263, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x719C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F91F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x243AB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F920, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7228, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F921, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7235, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F922, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7250, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F923, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24608, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F924, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7280, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F925, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7295, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F926, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24735, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F927, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24814, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F928, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x737A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F929, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x738B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3EAC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x73A5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3EB8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3EB8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7447, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F92F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x745C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F930, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7471, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F931, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7485, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F932, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x74CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F933, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3F1B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F934, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7524, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F935, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24C36, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F936, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x753E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F937, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24C92, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F938, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7570, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F939, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2219F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7610, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24FA1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x24FB8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25044, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x3FFC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F93F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4008, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F940, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x76F4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F941, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x250F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F942, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x250F2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F943, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25119, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F944, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25133, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F945, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x771E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F946, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x771F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F947, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x771F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F948, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x774A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F949, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4039, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x778B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4046, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4096, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2541D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x784E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F94F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x788C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F950, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x78CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F951, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x40E3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F952, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25626, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F953, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7956, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F954, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2569A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F955, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x256C5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F956, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x798F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F957, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x79EB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F958, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x412F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F959, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7A40, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7A4A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7A4F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2597C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25AA7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25AA7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F95F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7AEE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F960, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4202, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F961, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25BAB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F962, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7BC6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F963, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7BC9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F964, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4227, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F965, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25C80, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F966, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7CD2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F967, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x42A0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F968, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7CE8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F969, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7CE3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7D00, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x25F86, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7D63, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4301, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7DC7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F96F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7E02, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F970, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7E45, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F971, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4334, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F972, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26228, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F973, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26247, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F974, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4359, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F975, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x262D9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F976, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F7A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F977, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2633E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F978, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7F95, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F979, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x7FFA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8005, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x264DA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26523, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8060, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x265A8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F97F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8070, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F980, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2335F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F981, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x43D5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F982, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x80B2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F983, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8103, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F984, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x440B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F985, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x813E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F986, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5AB5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F987, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x267A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F988, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x267B5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F989, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x23393, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2339C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8201, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8204, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F9E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x446B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F98F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8291, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F990, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x828B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F991, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x829D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F992, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x52B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F993, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x82B1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F994, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x82B3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F995, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x82BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F996, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x82E6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F997, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26B3C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F998, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x82E5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F999, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x831D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8363, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83AD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8323, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83BD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83E7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F99F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8457, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8353, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83CC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x83DC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26C36, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26D6B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26CD5, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x452B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x84F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9A9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x84F3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9AA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8516, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9AB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x273CA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9AC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8564, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9AD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26F2C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9AE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x455D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9AF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4561, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x26FB1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x270D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x456B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8650, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x865C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8667, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8669, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x86A9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8688, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9B9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x870E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9BA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x86E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9BB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8779, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9BC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8728, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9BD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x876B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9BE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8786, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9BF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x45D7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x87E1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8801, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x45F9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8860, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8863, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x27667, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88D7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4635, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9C9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x88FA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9CA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x34BB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9CB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x278AE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9CC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x27966, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9CD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x46BE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9CE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x46C7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9CF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AA0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8AED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8B8A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8C55, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x27CA8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8CAB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8CC1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8D1B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8D77, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x27F2F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9D9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x20804, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9DA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8DCB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9DB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8DBC, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9DC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8DF0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9DD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x208DE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9DE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8ED4, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9DF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x8F38, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x285D2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x285ED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9094, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x90F1, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9111, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2872E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x911B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9238, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x92D7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9E9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x92D8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9EA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x927C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9EB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x93F9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9EC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9415, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9ED, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x28BFA, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9EE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x958B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9EF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4995, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F0, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x95B7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F1, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x28D77, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F2, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x49E6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F3, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x96C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F4, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x5DB2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F5, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9723, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F6, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x29145, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F7, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2921A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F8, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4A6E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9F9, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4A76, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9FA, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x97E0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9FB, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2940A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9FC, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4AB2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9FD, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x29496, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9FE, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x980B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2F9FF, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x980B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA00, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9829, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA01, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x295B6, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA02, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x98E2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA03, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4B33, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA04, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9929, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA05, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x99A7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA06, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x99C2, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA07, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x99FE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA08, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4BCE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA09, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x29B30, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9B12, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9C40, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9CFD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4CCE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0E, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4CED, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA0F, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9D67, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA10, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2A0CE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA11, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4CF8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA12, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2A105, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA13, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2A20E, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA14, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2A291, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA15, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9EBB, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA16, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x4D56, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA17, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9EF9, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA18, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9EFE, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA19, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F05, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA1A, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F0F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA1B, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F16, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA1C, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x9F3B, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
  {0x2FA1D, CHAR_CATEGORY_LO, DECOMP_TAG_NONE,
   {0x2A600, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000}},
};

struct Combining_mark
{
  my_wc_t   charcode;
  int       ccc; //Canonical_Combining_Class
};

/*
  All the combining marks whose ccc value is greater than 0.
  Gerneral_Category is Mn.
*/
Combining_mark combining_marks[]= {
  {0x0300, 230}, {0x0301, 230}, {0x0302, 230}, {0x0303, 230},
  {0x0304, 230}, {0x0305, 230}, {0x0306, 230}, {0x0307, 230},
  {0x0308, 230}, {0x0309, 230}, {0x030A, 230}, {0x030B, 230},
  {0x030C, 230}, {0x030D, 230}, {0x030E, 230}, {0x030F, 230},
  {0x0310, 230}, {0x0311, 230}, {0x0312, 230}, {0x0313, 230},
  {0x0314, 230}, {0x0315, 232}, {0x0316, 220}, {0x0317, 220},
  {0x0318, 220}, {0x0319, 220}, {0x031A, 232}, {0x031B, 216},
  {0x031C, 220}, {0x031D, 220}, {0x031E, 220}, {0x031F, 220},
  {0x0320, 220}, {0x0321, 202}, {0x0322, 202}, {0x0323, 220},
  {0x0324, 220}, {0x0325, 220}, {0x0326, 220}, {0x0327, 202},
  {0x0328, 202}, {0x0329, 220}, {0x032A, 220}, {0x032B, 220},
  {0x032C, 220}, {0x032D, 220}, {0x032E, 220}, {0x032F, 220},
  {0x0330, 220}, {0x0331, 220}, {0x0332, 220}, {0x0333, 220},
  {0x0334, 1},   {0x0335, 1},   {0x0336, 1},   {0x0337, 1},
  {0x0338, 1},   {0x0339, 220}, {0x033A, 220}, {0x033B, 220},
  {0x033C, 220}, {0x033D, 230}, {0x033E, 230}, {0x033F, 230},
  {0x0340, 230}, {0x0341, 230}, {0x0342, 230}, {0x0343, 230},
  {0x0344, 230}, {0x0345, 240}, {0x0346, 230}, {0x0347, 220},
  {0x0348, 220}, {0x0349, 220}, {0x034A, 230}, {0x034B, 230},
  {0x034C, 230}, {0x034D, 220}, {0x034E, 220}, {0x0350, 230},
  {0x0351, 230}, {0x0352, 230}, {0x0353, 220}, {0x0354, 220},
  {0x0355, 220}, {0x0356, 220}, {0x0357, 230}, {0x0358, 232},
  {0x0359, 220}, {0x035A, 220}, {0x035B, 230}, {0x035C, 233},
  {0x035D, 234}, {0x035E, 234}, {0x035F, 233}, {0x0360, 234},
  {0x0361, 234}, {0x0362, 233}, {0x0363, 230}, {0x0364, 230},
  {0x0365, 230}, {0x0366, 230}, {0x0367, 230}, {0x0368, 230},
  {0x0369, 230}, {0x036A, 230}, {0x036B, 230}, {0x036C, 230},
  {0x036D, 230}, {0x036E, 230}, {0x036F, 230}, {0x0483, 230},
  {0x0484, 230}, {0x0485, 230}, {0x0486, 230}, {0x0487, 230},
  {0x0591, 220}, {0x0592, 230}, {0x0593, 230}, {0x0594, 230},
  {0x0595, 230}, {0x0596, 220}, {0x0597, 230}, {0x0598, 230},
  {0x0599, 230}, {0x059A, 222}, {0x059B, 220}, {0x059C, 230},
  {0x059D, 230}, {0x059E, 230}, {0x059F, 230}, {0x05A0, 230},
  {0x05A1, 230}, {0x05A2, 220}, {0x05A3, 220}, {0x05A4, 220},
  {0x05A5, 220}, {0x05A6, 220}, {0x05A7, 220}, {0x05A8, 230},
  {0x05A9, 230}, {0x05AA, 220}, {0x05AB, 230}, {0x05AC, 230},
  {0x05AD, 222}, {0x05AE, 228}, {0x05AF, 230}, {0x05B0, 10},
  {0x05B1, 11},  {0x05B2, 12},  {0x05B3, 13},  {0x05B4, 14},
  {0x05B5, 15},  {0x05B6, 16},  {0x05B7, 17},  {0x05B8, 18},
  {0x05B9, 19},  {0x05BA, 19},  {0x05BB, 20},  {0x05BC, 21},
  {0x05BD, 22},  {0x05BF, 23},  {0x05C1, 24},  {0x05C2, 25},
  {0x05C4, 230}, {0x05C5, 220}, {0x05C7, 18},  {0x0610, 230},
  {0x0611, 230}, {0x0612, 230}, {0x0613, 230}, {0x0614, 230},
  {0x0615, 230}, {0x0616, 230}, {0x0617, 230}, {0x0618, 30},
  {0x0619, 31},  {0x061A, 32},  {0x064B, 27},  {0x064C, 28},
  {0x064D, 29},  {0x064E, 30},  {0x064F, 31},  {0x0650, 32},
  {0x0651, 33},  {0x0652, 34},  {0x0653, 230}, {0x0654, 230},
  {0x0655, 220}, {0x0656, 220}, {0x0657, 230}, {0x0658, 230},
  {0x0659, 230}, {0x065A, 230}, {0x065B, 230}, {0x065C, 220},
  {0x065D, 230}, {0x065E, 230}, {0x065F, 220}, {0x0670, 35},
  {0x06D6, 230}, {0x06D7, 230}, {0x06D8, 230}, {0x06D9, 230},
  {0x06DA, 230}, {0x06DB, 230}, {0x06DC, 230}, {0x06DF, 230},
  {0x06E0, 230}, {0x06E1, 230}, {0x06E2, 230}, {0x06E3, 220},
  {0x06E4, 230}, {0x06E7, 230}, {0x06E8, 230}, {0x06EA, 220},
  {0x06EB, 230}, {0x06EC, 230}, {0x06ED, 220}, {0x0711, 36},
  {0x0730, 230}, {0x0731, 220}, {0x0732, 230}, {0x0733, 230},
  {0x0734, 220}, {0x0735, 230}, {0x0736, 230}, {0x0737, 220},
  {0x0738, 220}, {0x0739, 220}, {0x073A, 230}, {0x073B, 220},
  {0x073C, 220}, {0x073D, 230}, {0x073E, 220}, {0x073F, 230},
  {0x0740, 230}, {0x0741, 230}, {0x0742, 220}, {0x0743, 230},
  {0x0744, 220}, {0x0745, 230}, {0x0746, 220}, {0x0747, 230},
  {0x0748, 220}, {0x0749, 230}, {0x074A, 230}, {0x07EB, 230},
  {0x07EC, 230}, {0x07ED, 230}, {0x07EE, 230}, {0x07EF, 230},
  {0x07F0, 230}, {0x07F1, 230}, {0x07F2, 220}, {0x07F3, 230},
  {0x0816, 230}, {0x0817, 230}, {0x0818, 230}, {0x0819, 230},
  {0x081B, 230}, {0x081C, 230}, {0x081D, 230}, {0x081E, 230},
  {0x081F, 230}, {0x0820, 230}, {0x0821, 230}, {0x0822, 230},
  {0x0823, 230}, {0x0825, 230}, {0x0826, 230}, {0x0827, 230},
  {0x0829, 230}, {0x082A, 230}, {0x082B, 230}, {0x082C, 230},
  {0x082D, 230}, {0x0859, 220}, {0x085A, 220}, {0x085B, 220},
  {0x08D4, 230}, {0x08D5, 230}, {0x08D6, 230}, {0x08D7, 230},
  {0x08D8, 230}, {0x08D9, 230}, {0x08DA, 230}, {0x08DB, 230},
  {0x08DC, 230}, {0x08DD, 230}, {0x08DE, 230}, {0x08DF, 230},
  {0x08E0, 230}, {0x08E1, 230}, {0x08E3, 220}, {0x08E4, 230},
  {0x08E5, 230}, {0x08E6, 220}, {0x08E7, 230}, {0x08E8, 230},
  {0x08E9, 220}, {0x08EA, 230}, {0x08EB, 230}, {0x08EC, 230},
  {0x08ED, 220}, {0x08EE, 220}, {0x08EF, 220}, {0x08F0, 27},
  {0x08F1, 28},  {0x08F2, 29},  {0x08F3, 230}, {0x08F4, 230},
  {0x08F5, 230}, {0x08F6, 220}, {0x08F7, 230}, {0x08F8, 230},
  {0x08F9, 220}, {0x08FA, 220}, {0x08FB, 230}, {0x08FC, 230},
  {0x08FD, 230}, {0x08FE, 230}, {0x08FF, 230}, {0x093C, 7},
  {0x094D, 9},   {0x0951, 230}, {0x0952, 220}, {0x0953, 230},
  {0x0954, 230}, {0x09BC, 7},   {0x09CD, 9},   {0x0A3C, 7},
  {0x0A4D, 9},   {0x0ABC, 7},   {0x0ACD, 9},   {0x0B3C, 7},
  {0x0B4D, 9},   {0x0BCD, 9},   {0x0C4D, 9},   {0x0C55, 84},
  {0x0C56, 91},  {0x0CBC, 7},   {0x0CCD, 9},   {0x0D4D, 9},
  {0x0DCA, 9},   {0x0E38, 103}, {0x0E39, 103}, {0x0E3A, 9},
  {0x0E48, 107}, {0x0E49, 107}, {0x0E4A, 107}, {0x0E4B, 107},
  {0x0EB8, 118}, {0x0EB9, 118}, {0x0EC8, 122}, {0x0EC9, 122},
  {0x0ECA, 122}, {0x0ECB, 122}, {0x0F18, 220}, {0x0F19, 220},
  {0x0F35, 220}, {0x0F37, 220}, {0x0F39, 216}, {0x0F71, 129},
  {0x0F72, 130}, {0x0F74, 132}, {0x0F7A, 130}, {0x0F7B, 130},
  {0x0F7C, 130}, {0x0F7D, 130}, {0x0F80, 130}, {0x0F82, 230},
  {0x0F83, 230}, {0x0F84, 9},   {0x0F86, 230}, {0x0F87, 230},
  {0x0FC6, 220}, {0x1037, 7},   {0x1039, 9},   {0x103A, 9},
  {0x108D, 220}, {0x135D, 230}, {0x135E, 230}, {0x135F, 230},
  {0x1714, 9},   {0x1734, 9},   {0x17D2, 9},   {0x17DD, 230},
  {0x18A9, 228}, {0x1939, 222}, {0x193A, 230}, {0x193B, 220},
  {0x1A17, 230}, {0x1A18, 220}, {0x1A60, 9},   {0x1A75, 230},
  {0x1A76, 230}, {0x1A77, 230}, {0x1A78, 230}, {0x1A79, 230},
  {0x1A7A, 230}, {0x1A7B, 230}, {0x1A7C, 230}, {0x1A7F, 220},
  {0x1AB0, 230}, {0x1AB1, 230}, {0x1AB2, 230}, {0x1AB3, 230},
  {0x1AB4, 230}, {0x1AB5, 220}, {0x1AB6, 220}, {0x1AB7, 220},
  {0x1AB8, 220}, {0x1AB9, 220}, {0x1ABA, 220}, {0x1ABB, 230},
  {0x1ABC, 230}, {0x1ABD, 220}, {0x1B34, 7},   {0x1B6B, 230},
  {0x1B6C, 220}, {0x1B6D, 230}, {0x1B6E, 230}, {0x1B6F, 230},
  {0x1B70, 230}, {0x1B71, 230}, {0x1B72, 230}, {0x1B73, 230},
  {0x1BAB, 9},   {0x1BE6, 7},   {0x1C37, 7},   {0x1CD0, 230},
  {0x1CD1, 230}, {0x1CD2, 230}, {0x1CD4, 1},   {0x1CD5, 220},
  {0x1CD6, 220}, {0x1CD7, 220}, {0x1CD8, 220}, {0x1CD9, 220},
  {0x1CDA, 230}, {0x1CDB, 230}, {0x1CDC, 220}, {0x1CDD, 220},
  {0x1CDE, 220}, {0x1CDF, 220}, {0x1CE0, 230}, {0x1CE2, 1},
  {0x1CE3, 1},   {0x1CE4, 1},   {0x1CE5, 1},   {0x1CE6, 1},
  {0x1CE7, 1},   {0x1CE8, 1},   {0x1CED, 220}, {0x1CF4, 230},
  {0x1CF8, 230}, {0x1CF9, 230}, {0x1DC0, 230}, {0x1DC1, 230},
  {0x1DC2, 220}, {0x1DC3, 230}, {0x1DC4, 230}, {0x1DC5, 230},
  {0x1DC6, 230}, {0x1DC7, 230}, {0x1DC8, 230}, {0x1DC9, 230},
  {0x1DCA, 220}, {0x1DCB, 230}, {0x1DCC, 230}, {0x1DCD, 234},
  {0x1DCE, 214}, {0x1DCF, 220}, {0x1DD0, 202}, {0x1DD1, 230},
  {0x1DD2, 230}, {0x1DD3, 230}, {0x1DD4, 230}, {0x1DD5, 230},
  {0x1DD6, 230}, {0x1DD7, 230}, {0x1DD8, 230}, {0x1DD9, 230},
  {0x1DDA, 230}, {0x1DDB, 230}, {0x1DDC, 230}, {0x1DDD, 230},
  {0x1DDE, 230}, {0x1DDF, 230}, {0x1DE0, 230}, {0x1DE1, 230},
  {0x1DE2, 230}, {0x1DE3, 230}, {0x1DE4, 230}, {0x1DE5, 230},
  {0x1DE6, 230}, {0x1DE7, 230}, {0x1DE8, 230}, {0x1DE9, 230},
  {0x1DEA, 230}, {0x1DEB, 230}, {0x1DEC, 230}, {0x1DED, 230},
  {0x1DEE, 230}, {0x1DEF, 230}, {0x1DF0, 230}, {0x1DF1, 230},
  {0x1DF2, 230}, {0x1DF3, 230}, {0x1DF4, 230}, {0x1DF5, 230},
  {0x1DFB, 230}, {0x1DFC, 233}, {0x1DFD, 220}, {0x1DFE, 230},
  {0x1DFF, 220}, {0x20D0, 230}, {0x20D1, 230}, {0x20D2, 1},
  {0x20D3, 1},   {0x20D4, 230}, {0x20D5, 230}, {0x20D6, 230},
  {0x20D7, 230}, {0x20D8, 1},   {0x20D9, 1},   {0x20DA, 1},
  {0x20DB, 230}, {0x20DC, 230}, {0x20E1, 230}, {0x20E5, 1},
  {0x20E6, 1},   {0x20E7, 230}, {0x20E8, 220}, {0x20E9, 230},
  {0x20EA, 1},   {0x20EB, 1},   {0x20EC, 220}, {0x20ED, 220},
  {0x20EE, 220}, {0x20EF, 220}, {0x20F0, 230}, {0x2CEF, 230},
  {0x2CF0, 230}, {0x2CF1, 230}, {0x2D7F, 9},   {0x2DE0, 230},
  {0x2DE1, 230}, {0x2DE2, 230}, {0x2DE3, 230}, {0x2DE4, 230},
  {0x2DE5, 230}, {0x2DE6, 230}, {0x2DE7, 230}, {0x2DE8, 230},
  {0x2DE9, 230}, {0x2DEA, 230}, {0x2DEB, 230}, {0x2DEC, 230},
  {0x2DED, 230}, {0x2DEE, 230}, {0x2DEF, 230}, {0x2DF0, 230},
  {0x2DF1, 230}, {0x2DF2, 230}, {0x2DF3, 230}, {0x2DF4, 230},
  {0x2DF5, 230}, {0x2DF6, 230}, {0x2DF7, 230}, {0x2DF8, 230},
  {0x2DF9, 230}, {0x2DFA, 230}, {0x2DFB, 230}, {0x2DFC, 230},
  {0x2DFD, 230}, {0x2DFE, 230}, {0x2DFF, 230}, {0x302A, 218},
  {0x302B, 228}, {0x302C, 232}, {0x302D, 222}, {0x3099, 8},
  {0x309A, 8},   {0xA66F, 230}, {0xA674, 230}, {0xA675, 230},
  {0xA676, 230}, {0xA677, 230}, {0xA678, 230}, {0xA679, 230},
  {0xA67A, 230}, {0xA67B, 230}, {0xA67C, 230}, {0xA67D, 230},
  {0xA69E, 230}, {0xA69F, 230}, {0xA6F0, 230}, {0xA6F1, 230},
  {0xA806, 9},   {0xA8C4, 9},   {0xA8E0, 230}, {0xA8E1, 230},
  {0xA8E2, 230}, {0xA8E3, 230}, {0xA8E4, 230}, {0xA8E5, 230},
  {0xA8E6, 230}, {0xA8E7, 230}, {0xA8E8, 230}, {0xA8E9, 230},
  {0xA8EA, 230}, {0xA8EB, 230}, {0xA8EC, 230}, {0xA8ED, 230},
  {0xA8EE, 230}, {0xA8EF, 230}, {0xA8F0, 230}, {0xA8F1, 230},
  {0xA92B, 220}, {0xA92C, 220}, {0xA92D, 220}, {0xA9B3, 7},
  {0xAAB0, 230}, {0xAAB2, 230}, {0xAAB3, 230}, {0xAAB4, 220},
  {0xAAB7, 230}, {0xAAB8, 230}, {0xAABE, 230}, {0xAABF, 230},
  {0xAAC1, 230}, {0xAAF6, 9},   {0xABED, 9},   {0xFB1E, 26},
  {0xFE20, 230}, {0xFE21, 230}, {0xFE22, 230}, {0xFE23, 230},
  {0xFE24, 230}, {0xFE25, 230}, {0xFE26, 230}, {0xFE27, 220},
  {0xFE28, 220}, {0xFE29, 220}, {0xFE2A, 220}, {0xFE2B, 220},
  {0xFE2C, 220}, {0xFE2D, 220}, {0xFE2E, 230}, {0xFE2F, 230},
  {0x101FD, 220},{0x102E0, 220},{0x10376, 230},{0x10377, 230},
  {0x10378, 230},{0x10379, 230},{0x1037A, 230},{0x10A0D, 220},
  {0x10A0F, 230},{0x10A38, 230},{0x10A39, 1},  {0x10A3A, 220},
  {0x10A3F, 9},  {0x10AE5, 230},{0x10AE6, 220},{0x11046, 9},
  {0x1107F, 9},  {0x110B9, 9},  {0x110BA, 7},  {0x11100, 230},
  {0x11101, 230},{0x11102, 230},{0x11133, 9},  {0x11134, 9},
  {0x11173, 7},  {0x111CA, 7},  {0x11236, 7},  {0x112E9, 7},
  {0x112EA, 9},  {0x1133C, 7},  {0x11366, 230},{0x11367, 230},
  {0x11368, 230},{0x11369, 230},{0x1136A, 230},{0x1136B, 230},
  {0x1136C, 230},{0x11370, 230},{0x11371, 230},{0x11372, 230},
  {0x11373, 230},{0x11374, 230},{0x11442, 9},  {0x11446, 7},
  {0x114C2, 9},  {0x114C3, 7},  {0x115BF, 9},  {0x115C0, 7},
  {0x1163F, 9},  {0x116B7, 7},  {0x1172B, 9},  {0x11C3F, 9},
  {0x16AF0, 1},  {0x16AF1, 1},  {0x16AF2, 1},  {0x16AF3, 1},
  {0x16AF4, 1},  {0x16B30, 230},{0x16B31, 230},{0x16B32, 230},
  {0x16B33, 230},{0x16B34, 230},{0x16B35, 230},{0x16B36, 230},
  {0x1BC9E, 1},  {0x1D167, 1},  {0x1D168, 1},  {0x1D169, 1},
  {0x1D17B, 220},{0x1D17C, 220},{0x1D17D, 220},{0x1D17E, 220},
  {0x1D17F, 220},{0x1D180, 220},{0x1D181, 220},{0x1D182, 220},
  {0x1D185, 230},{0x1D186, 230},{0x1D187, 230},{0x1D188, 230},
  {0x1D189, 230},{0x1D18A, 220},{0x1D18B, 220},{0x1D1AA, 230},
  {0x1D1AB, 230},{0x1D1AC, 230},{0x1D1AD, 230},{0x1D242, 230},
  {0x1D243, 230},{0x1D244, 230},{0x1E000, 230},{0x1E001, 230},
  {0x1E002, 230},{0x1E003, 230},{0x1E004, 230},{0x1E005, 230},
  {0x1E006, 230},{0x1E008, 230},{0x1E009, 230},{0x1E00A, 230},
  {0x1E00B, 230},{0x1E00C, 230},{0x1E00D, 230},{0x1E00E, 230},
  {0x1E00F, 230},{0x1E010, 230},{0x1E011, 230},{0x1E012, 230},
  {0x1E013, 230},{0x1E014, 230},{0x1E015, 230},{0x1E016, 230},
  {0x1E017, 230},{0x1E018, 230},{0x1E01B, 230},{0x1E01C, 230},
  {0x1E01D, 230},{0x1E01E, 230},{0x1E01F, 230},{0x1E020, 230},
  {0x1E021, 230},{0x1E023, 230},{0x1E024, 230},{0x1E026, 230},
  {0x1E027, 230},{0x1E028, 230},{0x1E029, 230},{0x1E02A, 230},
  {0x1E8D0, 220},{0x1E8D1, 220},{0x1E8D2, 220},{0x1E8D3, 220},
  {0x1E8D4, 220},{0x1E8D5, 220},{0x1E8D6, 220},{0x1E944, 230},
  {0x1E945, 230},{0x1E946, 230},{0x1E947, 230},{0x1E948, 230},
  {0x1E949, 230},{0x1E94A, 7}
};

struct Char_grp_info
{
  enum_char_grp    group;
  Weight_boundary  grp_wt_bdy;
};

const uint16 START_WEIGHT_TO_REORDER= 0x1C47;

Char_grp_info char_grp_infos[]= {
  {CHARGRP_CORE,     {0,      0x1C46}}, //space, punctuation, symbol,
                                        //currency, digit and all
                                        //primary ignorable characters
  {CHARGRP_LATIN,    {0x1C47, 0x1FB5}},
  {CHARGRP_CYRILLIC, {0x2022, 0x21E1}},
  {CHARGRP_ARAB,     {0x22FD, 0x23D6}}
};

/*
 In UCA9.0.0, every collation element has 3 weight data.
 To make one character's weight data aligned with those
 who have more collation elements in same page, we have
 to add zero values. To track the real collation element
 number, add one addtional number to record it.
 For example, character 0x0009 has collation element:
 [.0201.0020.0002]. We make its weight data as:
 0x0201, 0x0020, 0x0002, ... 0x0001.
 The 0x0001 at tail means it has only 1 collation
 element.
*/
uint16 uca900_p000[]= {
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0000*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0001*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0002*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0003*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0004*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0005*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0006*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0007*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0008*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0201, 0x0020, 0x0002, 0x0000, 0x0000, /*0009*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0202, 0x0020, 0x0002, 0x0000, 0x0000, /*000A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0203, 0x0020, 0x0002, 0x0000, 0x0000, /*000B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0204, 0x0020, 0x0002, 0x0000, 0x0000, /*000C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0205, 0x0020, 0x0002, 0x0000, 0x0000, /*000D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*000E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*000F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0010*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0011*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0012*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0013*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0014*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0015*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0016*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0017*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0018*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0019*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*001A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*001B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*001C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*001D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*001E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*001F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x0002, 0x0000, 0x0000, /*0020*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0260, 0x0020, 0x0002, 0x0000, 0x0000, /*0021*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x030C, 0x0020, 0x0002, 0x0000, 0x0000, /*0022*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0398, 0x0020, 0x0002, 0x0000, 0x0000, /*0023*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C12, 0x0020, 0x0002, 0x0000, 0x0000, /*0024*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0399, 0x0020, 0x0002, 0x0000, 0x0000, /*0025*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0396, 0x0020, 0x0002, 0x0000, 0x0000, /*0026*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0305, 0x0020, 0x0002, 0x0000, 0x0000, /*0027*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0317, 0x0020, 0x0002, 0x0000, 0x0000, /*0028*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0318, 0x0020, 0x0002, 0x0000, 0x0000, /*0029*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x038F, 0x0020, 0x0002, 0x0000, 0x0000, /*002A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0616, 0x0020, 0x0002, 0x0000, 0x0000, /*002B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0222, 0x0020, 0x0002, 0x0000, 0x0000, /*002C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020D, 0x0020, 0x0002, 0x0000, 0x0000, /*002D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0277, 0x0020, 0x0002, 0x0000, 0x0000, /*002E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0394, 0x0020, 0x0002, 0x0000, 0x0000, /*002F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, /*0030*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, /*0031*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, /*0032*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, /*0033*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, /*0034*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, /*0035*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, /*0036*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, /*0037*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, /*0038*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, /*0039*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0239, 0x0020, 0x0002, 0x0000, 0x0000, /*003A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0234, 0x0020, 0x0002, 0x0000, 0x0000, /*003B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x061A, 0x0020, 0x0002, 0x0000, 0x0000, /*003C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x061B, 0x0020, 0x0002, 0x0000, 0x0000, /*003D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x061C, 0x0020, 0x0002, 0x0000, 0x0000, /*003E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0266, 0x0020, 0x0002, 0x0000, 0x0000, /*003F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x038E, 0x0020, 0x0002, 0x0000, 0x0000, /*0040*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0000, /*0041*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C60, 0x0020, 0x0008, 0x0000, 0x0000, /*0042*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x0008, 0x0000, 0x0000, /*0043*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x0008, 0x0000, 0x0000, /*0044*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0000, /*0045*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE5, 0x0020, 0x0008, 0x0000, 0x0000, /*0046*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF4, 0x0020, 0x0008, 0x0000, 0x0000, /*0047*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x0008, 0x0000, 0x0000, /*0048*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x0000, /*0049*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D4C, 0x0020, 0x0008, 0x0000, 0x0000, /*004A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D65, 0x0020, 0x0008, 0x0000, 0x0000, /*004B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x0008, 0x0000, 0x0000, /*004C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x0008, 0x0000, 0x0000, /*004D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x0008, 0x0000, 0x0000, /*004E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0000, /*004F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0C, 0x0020, 0x0008, 0x0000, 0x0000, /*0050*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E21, 0x0020, 0x0008, 0x0000, 0x0000, /*0051*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x0008, 0x0000, 0x0000, /*0052*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x0008, 0x0000, 0x0000, /*0053*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x0008, 0x0000, 0x0000, /*0054*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0000, /*0055*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x0008, 0x0000, 0x0000, /*0056*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF5, 0x0020, 0x0008, 0x0000, 0x0000, /*0057*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFF, 0x0020, 0x0008, 0x0000, 0x0000, /*0058*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F0B, 0x0020, 0x0008, 0x0000, 0x0000, /*0059*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F21, 0x0020, 0x0008, 0x0000, 0x0000, /*005A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0319, 0x0020, 0x0002, 0x0000, 0x0000, /*005B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0395, 0x0020, 0x0002, 0x0000, 0x0000, /*005C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x031A, 0x0020, 0x0002, 0x0000, 0x0000, /*005D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0485, 0x0020, 0x0002, 0x0000, 0x0000, /*005E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020B, 0x0020, 0x0002, 0x0000, 0x0000, /*005F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0482, 0x0020, 0x0002, 0x0000, 0x0000, /*0060*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0000, /*0061*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C60, 0x0020, 0x0002, 0x0000, 0x0000, /*0062*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x0002, 0x0000, 0x0000, /*0063*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x0002, 0x0000, 0x0000, /*0064*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0000, /*0065*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE5, 0x0020, 0x0002, 0x0000, 0x0000, /*0066*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF4, 0x0020, 0x0002, 0x0000, 0x0000, /*0067*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x0002, 0x0000, 0x0000, /*0068*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x0000, /*0069*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D4C, 0x0020, 0x0002, 0x0000, 0x0000, /*006A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D65, 0x0020, 0x0002, 0x0000, 0x0000, /*006B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x0002, 0x0000, 0x0000, /*006C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x0002, 0x0000, 0x0000, /*006D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x0002, 0x0000, 0x0000, /*006E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0000, /*006F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0C, 0x0020, 0x0002, 0x0000, 0x0000, /*0070*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E21, 0x0020, 0x0002, 0x0000, 0x0000, /*0071*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x0002, 0x0000, 0x0000, /*0072*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x0002, 0x0000, 0x0000, /*0073*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x0002, 0x0000, 0x0000, /*0074*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0000, /*0075*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x0002, 0x0000, 0x0000, /*0076*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF5, 0x0020, 0x0002, 0x0000, 0x0000, /*0077*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFF, 0x0020, 0x0002, 0x0000, 0x0000, /*0078*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F0B, 0x0020, 0x0002, 0x0000, 0x0000, /*0079*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F21, 0x0020, 0x0002, 0x0000, 0x0000, /*007A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x031B, 0x0020, 0x0002, 0x0000, 0x0000, /*007B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x061E, 0x0020, 0x0002, 0x0000, 0x0000, /*007C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x031C, 0x0020, 0x0002, 0x0000, 0x0000, /*007D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0620, 0x0020, 0x0002, 0x0000, 0x0000, /*007E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*007F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0080*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0081*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0082*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0083*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0084*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0206, 0x0020, 0x0002, 0x0000, 0x0000, /*0085*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0086*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0087*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0088*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0089*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*008A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*008B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*008C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*008D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*008E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*008F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0090*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0091*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0092*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0093*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0094*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0095*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0096*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0097*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0098*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0099*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*009A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*009B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*009C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*009D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*009E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*009F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x001B, 0x0000, 0x0000, /*00A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0261, 0x0020, 0x0002, 0x0000, 0x0000, /*00A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C11, 0x0020, 0x0002, 0x0000, 0x0000, /*00A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C13, 0x0020, 0x0002, 0x0000, 0x0000, /*00A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C10, 0x0020, 0x0002, 0x0000, 0x0000, /*00A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C14, 0x0020, 0x0002, 0x0000, 0x0000, /*00A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x061F, 0x0020, 0x0002, 0x0000, 0x0000, /*00A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0389, 0x0020, 0x0002, 0x0000, 0x0000, /*00A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0489, 0x0020, 0x0002, 0x0000, 0x0000, /*00A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0584, 0x0020, 0x0002, 0x0000, 0x0000, /*00A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0014, 0x0000, 0x0000, /*00AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0315, 0x0020, 0x0002, 0x0000, 0x0000, /*00AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x061D, 0x0020, 0x0002, 0x0000, 0x0000, /*00AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*00AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0585, 0x0020, 0x0002, 0x0000, 0x0000, /*00AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0486, 0x0020, 0x0002, 0x0000, 0x0000, /*00AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04F6, 0x0020, 0x0002, 0x0000, 0x0000, /*00B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0617, 0x0020, 0x0002, 0x0000, 0x0000, /*00B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0014, 0x0000, 0x0000, /*00B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0014, 0x0000, 0x0000, /*00B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0483, 0x0020, 0x0002, 0x0000, 0x0000, /*00B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCB, 0x0020, 0x0004, 0x0000, 0x0000, /*00B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x038B, 0x0020, 0x0002, 0x0000, 0x0000, /*00B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x028B, 0x0020, 0x0002, 0x0000, 0x0000, /*00B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x048C, 0x0020, 0x0002, 0x0000, 0x0000, /*00B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0014, 0x0000, 0x0000, /*00B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0014, 0x0000, 0x0000, /*00BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0316, 0x0020, 0x0002, 0x0000, 0x0000, /*00BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x001E, 0x0626, 0x0020, /*00BC*/
  0x001E, 0x1C41, 0x0020, 0x001E, 0x0003,
  0x1C3E, 0x0020, 0x001E, 0x0626, 0x0020, /*00BD*/
  0x001E, 0x1C3F, 0x0020, 0x001E, 0x0003,
  0x1C40, 0x0020, 0x001E, 0x0626, 0x0020, /*00BE*/
  0x001E, 0x1C41, 0x0020, 0x001E, 0x0003,
  0x0267, 0x0020, 0x0002, 0x0000, 0x0000, /*00BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0025, /*00C0*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0024, /*00C1*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0027, /*00C2*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x002D, /*00C3*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x002B, /*00C4*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0029, /*00C5*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x000A, 0x0000, 0x0110, /*00C6*/
  0x0004, 0x1CAA, 0x0020, 0x000A, 0x0003,
  0x1C7A, 0x0020, 0x0008, 0x0000, 0x0030, /*00C7*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0025, /*00C8*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0024, /*00C9*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0027, /*00CA*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x002B, /*00CB*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x0025, /*00CC*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x0024, /*00CD*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x0027, /*00CE*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x002B, /*00CF*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x000A, 0x0000, 0x0110, /*00D0*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0008, 0x0000, 0x002D, /*00D1*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0025, /*00D2*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0024, /*00D3*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0027, /*00D4*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x002D, /*00D5*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x002B, /*00D6*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0619, 0x0020, 0x0002, 0x0000, 0x0000, /*00D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x002F, /*00D8*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0025, /*00D9*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0024, /*00DA*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0027, /*00DB*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x002B, /*00DC*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0008, 0x0000, 0x0024, /*00DD*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F50, 0x0020, 0x0008, 0x0000, 0x0000, /*00DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x0004, 0x0000, 0x0110, /*00DF*/
  0x0004, 0x1E71, 0x0020, 0x0004, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0025, /*00E0*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0024, /*00E1*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0027, /*00E2*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x002D, /*00E3*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x002B, /*00E4*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0029, /*00E5*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0004, 0x0000, 0x0110, /*00E6*/
  0x0004, 0x1CAA, 0x0020, 0x0004, 0x0003,
  0x1C7A, 0x0020, 0x0002, 0x0000, 0x0030, /*00E7*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0025, /*00E8*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0024, /*00E9*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0027, /*00EA*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x002B, /*00EB*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x0025, /*00EC*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x0024, /*00ED*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x0027, /*00EE*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x002B, /*00EF*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0004, 0x0000, 0x0110, /*00F0*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0002, 0x0000, 0x002D, /*00F1*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0025, /*00F2*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0024, /*00F3*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0027, /*00F4*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x002D, /*00F5*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x002B, /*00F6*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0618, 0x0020, 0x0002, 0x0000, 0x0000, /*00F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x002F, /*00F8*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0025, /*00F9*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0024, /*00FA*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0027, /*00FB*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x002B, /*00FC*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0002, 0x0000, 0x0024, /*00FD*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F50, 0x0020, 0x0002, 0x0000, 0x0000, /*00FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F0B, 0x0020, 0x0002, 0x0000, 0x002B, /*00FF*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002
};

uint16 uca900_p001[]= {
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0032, 0x0002, 0x0000, /*0100*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0032, 0x0002, 0x0000, /*0101*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0000, /*0102*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0000, /*0103*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0031, 0x0002, 0x0000, /*0104*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0031, 0x0002, 0x0000, /*0105*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*0106*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*0107*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x0008, 0x0000, 0x0027, 0x0002, 0x0000, /*0108*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x0002, 0x0000, 0x0027, 0x0002, 0x0000, /*0109*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x0008, 0x0000, 0x002E, 0x0002, 0x0000, /*010A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x0002, 0x0000, 0x002E, 0x0002, 0x0000, /*010B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*010C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*010D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*010E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*010F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0008, 0x0000, 0x0039, 0x0002, 0x0000, /*0110*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0002, 0x0000, 0x0039, 0x0002, 0x0000, /*0111*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0032, 0x0002, 0x0000, /*0112*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0032, 0x0002, 0x0000, /*0113*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0000, /*0114*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0000, /*0115*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x002E, 0x0002, 0x0000, /*0116*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x002E, 0x0002, 0x0000, /*0117*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0031, 0x0002, 0x0000, /*0118*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0031, 0x0002, 0x0000, /*0119*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*011A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*011B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0008, 0x0000, 0x0027, 0x0002, 0x0000, /*011C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0002, 0x0000, 0x0027, 0x0002, 0x0000, /*011D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0000, /*011E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0000, /*011F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0008, 0x0000, 0x002E, 0x0002, 0x0000, /*0120*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0002, 0x0000, 0x002E, 0x0002, 0x0000, /*0121*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0008, 0x0000, 0x0030, 0x0002, 0x0000, /*0122*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0002, 0x0000, 0x0030, 0x0002, 0x0000, /*0123*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0008, 0x0000, 0x0027, 0x0002, 0x0000, /*0124*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0002, 0x0000, 0x0027, 0x0002, 0x0000, /*0125*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0008, 0x0000, 0x0039, 0x0002, 0x0000, /*0126*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0002, 0x0000, 0x0039, 0x0002, 0x0000, /*0127*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x002D, 0x0002, 0x0000, /*0128*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x002D, 0x0002, 0x0000, /*0129*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x0032, 0x0002, 0x0000, /*012A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x0032, 0x0002, 0x0000, /*012B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0000, /*012C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0000, /*012D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x0031, 0x0002, 0x0000, /*012E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x0031, 0x0002, 0x0000, /*012F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x002E, 0x0002, 0x0000, /*0130*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*0131*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x000A, 0x1D4C, 0x0020, 0x000A, 0x0000, /*0132*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0004, 0x1D4C, 0x0020, 0x0004, 0x0000, /*0133*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D4C, 0x0020, 0x0008, 0x0000, 0x0027, 0x0002, 0x0000, /*0134*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D4C, 0x0020, 0x0002, 0x0000, 0x0027, 0x0002, 0x0000, /*0135*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x0008, 0x0000, 0x0030, 0x0002, 0x0000, /*0136*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x0002, 0x0000, 0x0030, 0x0002, 0x0000, /*0137*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*0138*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*0139*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*013A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0008, 0x0000, 0x0030, 0x0002, 0x0000, /*013B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0002, 0x0000, 0x0030, 0x0002, 0x0000, /*013C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*013D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*013E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0008, 0x0000, 0x0110, 0x0002, 0x0000, /*013F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0002, 0x0000, 0x0110, 0x0002, 0x0000, /*0140*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0008, 0x0000, 0x0039, 0x0002, 0x0000, /*0141*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0002, 0x0000, 0x0039, 0x0002, 0x0000, /*0142*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*0143*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*0144*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0008, 0x0000, 0x0030, 0x0002, 0x0000, /*0145*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0002, 0x0000, 0x0030, 0x0002, 0x0000, /*0146*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*0147*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*0148*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F7E, 0x0020, 0x0004, 0x1DB9, 0x0020, 0x0004, 0x0000, /*0149*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DD8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*014A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*014B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0032, 0x0002, 0x0000, /*014C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0032, 0x0002, 0x0000, /*014D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0000, /*014E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0000, /*014F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x002C, 0x0002, 0x0000, /*0150*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x002C, 0x0002, 0x0000, /*0151*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x000A, 0x0000, 0x0110, 0x0004, 0x1CAA, /*0152*/
  0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DDD, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x1CAA, /*0153*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1E33, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*0154*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*0155*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0008, 0x0000, 0x0030, 0x0002, 0x0000, /*0156*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0002, 0x0000, 0x0030, 0x0002, 0x0000, /*0157*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*0158*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*0159*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*015A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*015B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0008, 0x0000, 0x0027, 0x0002, 0x0000, /*015C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0002, 0x0000, 0x0027, 0x0002, 0x0000, /*015D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0008, 0x0000, 0x0030, 0x0002, 0x0000, /*015E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0002, 0x0000, 0x0030, 0x0002, 0x0000, /*015F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*0160*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*0161*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0008, 0x0000, 0x0030, 0x0002, 0x0000, /*0162*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0002, 0x0000, 0x0030, 0x0002, 0x0000, /*0163*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*0164*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*0165*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E9A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0166*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*0167*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x002D, 0x0002, 0x0000, /*0168*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x002D, 0x0002, 0x0000, /*0169*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0032, 0x0002, 0x0000, /*016A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0032, 0x0002, 0x0000, /*016B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0000, /*016C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0000, /*016D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0029, 0x0002, 0x0000, /*016E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0029, 0x0002, 0x0000, /*016F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x002C, 0x0002, 0x0000, /*0170*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x002C, 0x0002, 0x0000, /*0171*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0031, 0x0002, 0x0000, /*0172*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0031, 0x0002, 0x0000, /*0173*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0008, 0x0000, 0x0027, 0x0002, 0x0000, /*0174*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0002, 0x0000, 0x0027, 0x0002, 0x0000, /*0175*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0008, 0x0000, 0x0027, 0x0002, 0x0000, /*0176*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0002, 0x0000, 0x0027, 0x0002, 0x0000, /*0177*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0000, /*0178*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*0179*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*017A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0008, 0x0000, 0x002E, 0x0002, 0x0000, /*017B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0002, 0x0000, 0x002E, 0x0002, 0x0000, /*017C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*017D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*017E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0000, /*017F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*0180*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C71, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0181*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C75, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0182*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*0183*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F71, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0184*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*0185*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DF0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0186*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C85, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0187*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*0188*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C97, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0189*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C9B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*018A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CA0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*018B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*018C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F21, 0x0020, 0x0004, 0x1EF5, 0x0020, 0x0004, 0x0000, /*018D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CB8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*018E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CBD, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*018F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CC2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0190*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CEE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0191*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*0192*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D06, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0193*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D10, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0194*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*0195*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D47, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0196*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D41, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0197*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D6B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*0198*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*0199*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*019A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*019B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1ED4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*019C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DC4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*019D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*019E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DFD, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*019F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x003F, 0x0002, 0x0000, /*01A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x003F, 0x0002, 0x0000, /*01A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D14, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E15, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E38, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F69, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E82, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EA4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EA8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x003F, 0x0002, 0x0000, /*01AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x003F, 0x0002, 0x0000, /*01B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EDE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EEA, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F17, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F26, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F3E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F43, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F6D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x0004, 0x1E71, 0x0020, 0x0004, 0x0000, /*01BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x000A, 0x1F21, 0x0020, 0x000A, 0x0000, /*01C4*/
  0x0028, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C8F, 0x0020, 0x000A, 0x1F21, 0x0020, 0x0004, 0x0000, /*01C5*/
  0x0028, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C8F, 0x0020, 0x0004, 0x1F21, 0x0020, 0x0004, 0x0000, /*01C6*/
  0x0028, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1D77, 0x0020, 0x000A, 0x1D4C, 0x0020, 0x000A, 0x0000, /*01C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x000A, 0x1D4C, 0x0020, 0x0004, 0x0000, /*01C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0004, 0x1D4C, 0x0020, 0x0004, 0x0000, /*01C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x000A, 0x1D4C, 0x0020, 0x000A, 0x0000, /*01CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x000A, 0x1D4C, 0x0020, 0x0004, 0x0000, /*01CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0004, 0x1D4C, 0x0020, 0x0004, 0x0000, /*01CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*01CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*01CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*01CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*01D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*01D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*01D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*01D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*01D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0000, /*01D5*/
  0x0032, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0000, /*01D6*/
  0x0032, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0000, /*01D7*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0000, /*01D8*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0000, /*01D9*/
  0x0028, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0000, /*01DA*/
  0x0028, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0000, /*01DB*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0000, /*01DC*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1CB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0000, /*01DE*/
  0x0032, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0000, /*01DF*/
  0x0032, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x002E, 0x0002, 0x0000, /*01E0*/
  0x0032, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x002E, 0x0002, 0x0000, /*01E1*/
  0x0032, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C47, 0x0020, 0x000A, 0x0000, 0x0110, 0x0004, 0x1CAA, /*01E2*/
  0x0020, 0x000A, 0x0000, 0x0032, 0x0002, 0x0004,
  0x1C47, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x1CAA, /*01E3*/
  0x0020, 0x0004, 0x0000, 0x0032, 0x0002, 0x0004,
  0x1D01, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*01E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF4, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*01E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*01E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*01E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*01E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0031, 0x0002, 0x0000, /*01EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0031, 0x0002, 0x0000, /*01EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0031, 0x0002, 0x0000, /*01EC*/
  0x0032, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0031, 0x0002, 0x0000, /*01ED*/
  0x0032, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1F3E, 0x0020, 0x0008, 0x0000, 0x0028, 0x0002, 0x0000, /*01EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F3E, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*01EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D4C, 0x0020, 0x0002, 0x0000, 0x0028, 0x0002, 0x0000, /*01F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x000A, 0x1F21, 0x0020, 0x000A, 0x0000, /*01F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x000A, 0x1F21, 0x0020, 0x0004, 0x0000, /*01F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0004, 0x1F21, 0x0020, 0x0004, 0x0000, /*01F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*01F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*01F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D20, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F56, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*01F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x0008, 0x0000, 0x0025, 0x0002, 0x0000, /*01F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*01F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0029, 0x0002, 0x0000, /*01FA*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0029, 0x0002, 0x0000, /*01FB*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C47, 0x0020, 0x000A, 0x0000, 0x0110, 0x0004, 0x1CAA, /*01FC*/
  0x0020, 0x000A, 0x0000, 0x0024, 0x0002, 0x0004,
  0x1C47, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x1CAA, /*01FD*/
  0x0020, 0x0004, 0x0000, 0x0024, 0x0002, 0x0004,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x002F, 0x0002, 0x0000, /*01FE*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x002F, 0x0002, 0x0000, /*01FF*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003
};

uint16 uca900_p002[]= {
  0x1C47, 0x0020, 0x0008, 0x0000, 0x003C, /*0200*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x003C, /*0201*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x003E, /*0202*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x003E, /*0203*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x003C, /*0204*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x003C, /*0205*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x003E, /*0206*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x003E, /*0207*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x003C, /*0208*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x003C, /*0209*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x003E, /*020A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x003E, /*020B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x003C, /*020C*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x003C, /*020D*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x003E, /*020E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x003E, /*020F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0008, 0x0000, 0x003C, /*0210*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0002, 0x0000, 0x003C, /*0211*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0008, 0x0000, 0x003E, /*0212*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0002, 0x0000, 0x003E, /*0213*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x003C, /*0214*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x003C, /*0215*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x003E, /*0216*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x003E, /*0217*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0008, 0x0000, 0x0045, /*0218*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0002, 0x0000, 0x0045, /*0219*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0008, 0x0000, 0x0045, /*021A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0002, 0x0000, 0x0045, /*021B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F1D, 0x0020, 0x0008, 0x0000, 0x0000, /*021C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F1D, 0x0020, 0x0002, 0x0000, 0x0000, /*021D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x0008, 0x0000, 0x0028, /*021E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0002, 0x0000, 0x0028, /*021F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DC8, 0x0020, 0x0008, 0x0000, 0x0000, /*0220*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CA4, 0x0020, 0x0002, 0x0000, 0x0000, /*0221*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E07, 0x0020, 0x0008, 0x0000, 0x0000, /*0222*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E07, 0x0020, 0x0002, 0x0000, 0x0000, /*0223*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F2C, 0x0020, 0x0008, 0x0000, 0x0000, /*0224*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F2C, 0x0020, 0x0002, 0x0000, 0x0000, /*0225*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x002E, /*0226*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x002E, /*0227*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0030, /*0228*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0030, /*0229*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x002B, /*022A*/
  0x0002, 0x0000, 0x0032, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x002B, /*022B*/
  0x0002, 0x0000, 0x0032, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x002D, /*022C*/
  0x0002, 0x0000, 0x0032, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x002D, /*022D*/
  0x0002, 0x0000, 0x0032, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x002E, /*022E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x002E, /*022F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x002E, /*0230*/
  0x0002, 0x0000, 0x0032, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x002E, /*0231*/
  0x0002, 0x0000, 0x0032, 0x0002, 0x0003,
  0x1F0B, 0x0020, 0x0008, 0x0000, 0x0032, /*0232*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0002, 0x0000, 0x0032, /*0233*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D98, 0x0020, 0x0002, 0x0000, 0x0000, /*0234*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DD2, 0x0020, 0x0002, 0x0000, 0x0000, /*0235*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EAC, 0x0020, 0x0002, 0x0000, 0x0000, /*0236*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D50, 0x0020, 0x0002, 0x0000, 0x0000, /*0237*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x0004, 0x1C60, 0x0020, /*0238*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E21, 0x0020, 0x0004, 0x1E0C, 0x0020, /*0239*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C4C, 0x0020, 0x0008, 0x0000, 0x0000, /*023A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7F, 0x0020, 0x0008, 0x0000, 0x0000, /*023B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7F, 0x0020, 0x0002, 0x0000, 0x0000, /*023C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D82, 0x0020, 0x0008, 0x0000, 0x0000, /*023D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E9E, 0x0020, 0x0008, 0x0000, 0x0000, /*023E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E7C, 0x0020, 0x0002, 0x0000, 0x0000, /*023F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F38, 0x0020, 0x0002, 0x0000, 0x0000, /*0240*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F79, 0x0020, 0x0008, 0x0000, 0x0000, /*0241*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F79, 0x0020, 0x0002, 0x0000, 0x0000, /*0242*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C68, 0x0020, 0x0008, 0x0000, 0x0000, /*0243*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EC0, 0x0020, 0x0008, 0x0000, 0x0000, /*0244*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF1, 0x0020, 0x0008, 0x0000, 0x0000, /*0245*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CB1, 0x0020, 0x0008, 0x0000, 0x0000, /*0246*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CB1, 0x0020, 0x0002, 0x0000, 0x0000, /*0247*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D55, 0x0020, 0x0008, 0x0000, 0x0000, /*0248*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D55, 0x0020, 0x0002, 0x0000, 0x0000, /*0249*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E2B, 0x0020, 0x0008, 0x0000, 0x0000, /*024A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E2B, 0x0020, 0x0002, 0x0000, 0x0000, /*024B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E3F, 0x0020, 0x0008, 0x0000, 0x0000, /*024C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E3F, 0x0020, 0x0002, 0x0000, 0x0000, /*024D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F13, 0x0020, 0x0008, 0x0000, 0x0000, /*024E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F13, 0x0020, 0x0002, 0x0000, 0x0000, /*024F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C51, 0x0020, 0x0002, 0x0000, 0x0000, /*0250*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C55, 0x0020, 0x0002, 0x0000, 0x0000, /*0251*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C5B, 0x0020, 0x0002, 0x0000, 0x0000, /*0252*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C71, 0x0020, 0x0002, 0x0000, 0x0000, /*0253*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DF0, 0x0020, 0x0002, 0x0000, 0x0000, /*0254*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C89, 0x0020, 0x0002, 0x0000, 0x0000, /*0255*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C97, 0x0020, 0x0002, 0x0000, 0x0000, /*0256*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C9B, 0x0020, 0x0002, 0x0000, 0x0000, /*0257*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CC7, 0x0020, 0x0002, 0x0000, 0x0000, /*0258*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CBD, 0x0020, 0x0002, 0x0000, 0x0000, /*0259*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CCB, 0x0020, 0x0002, 0x0000, 0x0000, /*025A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CC2, 0x0020, 0x0002, 0x0000, 0x0000, /*025B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CCF, 0x0020, 0x0002, 0x0000, 0x0000, /*025C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CD5, 0x0020, 0x0002, 0x0000, 0x0000, /*025D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CD9, 0x0020, 0x0002, 0x0000, 0x0000, /*025E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D5D, 0x0020, 0x0002, 0x0000, 0x0000, /*025F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D06, 0x0020, 0x0002, 0x0000, 0x0000, /*0260*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF8, 0x0020, 0x0002, 0x0000, 0x0000, /*0261*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CFD, 0x0020, 0x0002, 0x0000, 0x0000, /*0262*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D10, 0x0020, 0x0002, 0x0000, 0x0000, /*0263*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE1, 0x0020, 0x0002, 0x0000, 0x0000, /*0264*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EC8, 0x0020, 0x0002, 0x0000, 0x0000, /*0265*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D25, 0x0020, 0x0002, 0x0000, 0x0000, /*0266*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D2C, 0x0020, 0x0002, 0x0000, 0x0000, /*0267*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D41, 0x0020, 0x0002, 0x0000, 0x0000, /*0268*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D47, 0x0020, 0x0002, 0x0000, 0x0000, /*0269*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D3A, 0x0020, 0x0002, 0x0000, 0x0000, /*026A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D87, 0x0020, 0x0002, 0x0000, 0x0000, /*026B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D8D, 0x0020, 0x0002, 0x0000, 0x0000, /*026C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D93, 0x0020, 0x0002, 0x0000, 0x0000, /*026D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D9D, 0x0020, 0x0002, 0x0000, 0x0000, /*026E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1ED4, 0x0020, 0x0002, 0x0000, 0x0000, /*026F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EDA, 0x0020, 0x0002, 0x0000, 0x0000, /*0270*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB1, 0x0020, 0x0002, 0x0000, 0x0000, /*0271*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DC4, 0x0020, 0x0002, 0x0000, 0x0000, /*0272*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DCE, 0x0020, 0x0002, 0x0000, 0x0000, /*0273*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DBD, 0x0020, 0x0002, 0x0000, 0x0000, /*0274*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DFD, 0x0020, 0x0002, 0x0000, 0x0000, /*0275*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DE4, 0x0020, 0x0002, 0x0000, 0x0000, /*0276*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E02, 0x0020, 0x0002, 0x0000, 0x0000, /*0277*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E1C, 0x0020, 0x0002, 0x0000, 0x0000, /*0278*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E44, 0x0020, 0x0002, 0x0000, 0x0000, /*0279*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E49, 0x0020, 0x0002, 0x0000, 0x0000, /*027A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E4E, 0x0020, 0x0002, 0x0000, 0x0000, /*027B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E53, 0x0020, 0x0002, 0x0000, 0x0000, /*027C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E57, 0x0020, 0x0002, 0x0000, 0x0000, /*027D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E5C, 0x0020, 0x0002, 0x0000, 0x0000, /*027E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E61, 0x0020, 0x0002, 0x0000, 0x0000, /*027F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E38, 0x0020, 0x0002, 0x0000, 0x0000, /*0280*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E6A, 0x0020, 0x0002, 0x0000, 0x0000, /*0281*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E78, 0x0020, 0x0002, 0x0000, 0x0000, /*0282*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E82, 0x0020, 0x0002, 0x0000, 0x0000, /*0283*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D61, 0x0020, 0x0002, 0x0000, 0x0000, /*0284*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E8C, 0x0020, 0x0002, 0x0000, 0x0000, /*0285*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E91, 0x0020, 0x0002, 0x0000, 0x0000, /*0286*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB1, 0x0020, 0x0002, 0x0000, 0x0000, /*0287*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EA8, 0x0020, 0x0002, 0x0000, 0x0000, /*0288*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EC0, 0x0020, 0x0002, 0x0000, 0x0000, /*0289*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EDE, 0x0020, 0x0002, 0x0000, 0x0000, /*028A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EEA, 0x0020, 0x0002, 0x0000, 0x0000, /*028B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF1, 0x0020, 0x0002, 0x0000, 0x0000, /*028C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFB, 0x0020, 0x0002, 0x0000, 0x0000, /*028D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DA6, 0x0020, 0x0002, 0x0000, 0x0000, /*028E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F0F, 0x0020, 0x0002, 0x0000, 0x0000, /*028F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F30, 0x0020, 0x0002, 0x0000, 0x0000, /*0290*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F34, 0x0020, 0x0002, 0x0000, 0x0000, /*0291*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F3E, 0x0020, 0x0002, 0x0000, 0x0000, /*0292*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F4C, 0x0020, 0x0002, 0x0000, 0x0000, /*0293*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F75, 0x0020, 0x0002, 0x0000, 0x0000, /*0294*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F84, 0x0020, 0x0002, 0x0000, 0x0000, /*0295*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F95, 0x0020, 0x0002, 0x0000, 0x0000, /*0296*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FA9, 0x0020, 0x0002, 0x0000, 0x0000, /*0297*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FAD, 0x0020, 0x0002, 0x0000, 0x0000, /*0298*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C64, 0x0020, 0x0002, 0x0000, 0x0000, /*0299*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CDD, 0x0020, 0x0002, 0x0000, 0x0000, /*029A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D0A, 0x0020, 0x0002, 0x0000, 0x0000, /*029B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D1C, 0x0020, 0x0002, 0x0000, 0x0000, /*029C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D59, 0x0020, 0x0002, 0x0000, 0x0000, /*029D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D73, 0x0020, 0x0002, 0x0000, 0x0000, /*029E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D7B, 0x0020, 0x0002, 0x0000, 0x0000, /*029F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E27, 0x0020, 0x0002, 0x0000, 0x0000, /*02A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F8D, 0x0020, 0x0002, 0x0000, 0x0000, /*02A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F91, 0x0020, 0x0002, 0x0000, 0x0000, /*02A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x0004, 0x1F21, 0x0020, /*02A3*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0004, 0x1F3E, 0x0020, /*02A4*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0004, 0x1F34, 0x0020, /*02A5*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0004, 0x1E71, 0x0020, /*02A6*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0004, 0x1E82, 0x0020, /*02A7*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0004, 0x1C89, 0x0020, /*02A8*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CE5, 0x0020, 0x0004, 0x1DD8, 0x0020, /*02A9*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0004, 0x1E71, 0x0020, /*02AA*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0004, 0x1F21, 0x0020, /*02AB*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB1, 0x0020, 0x0002, 0x0000, 0x0000, /*02AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FB5, 0x0020, 0x0002, 0x0000, 0x0000, /*02AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1ECC, 0x0020, 0x0002, 0x0000, 0x0000, /*02AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1ED0, 0x0020, 0x0002, 0x0000, 0x0000, /*02AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x0014, 0x0000, 0x0000, /*02B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D25, 0x0020, 0x0014, 0x0000, 0x0000, /*02B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D4C, 0x0020, 0x0014, 0x0000, 0x0000, /*02B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x0014, 0x0000, 0x0000, /*02B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E44, 0x0020, 0x0014, 0x0000, 0x0000, /*02B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E4E, 0x0020, 0x0014, 0x0000, 0x0000, /*02B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E6A, 0x0020, 0x0014, 0x0000, 0x0000, /*02B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF5, 0x0020, 0x0014, 0x0000, 0x0000, /*02B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F0B, 0x0020, 0x0014, 0x0000, 0x0000, /*02B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0493, 0x0020, 0x0002, 0x0000, 0x0000, /*02B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0495, 0x0020, 0x0002, 0x0000, 0x0000, /*02BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D30, 0x0020, 0x0002, 0x0000, 0x0000, /*02BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F7E, 0x0020, 0x0002, 0x0000, 0x0000, /*02BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D31, 0x0020, 0x0002, 0x0000, 0x0000, /*02BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F80, 0x0020, 0x0002, 0x0000, 0x0000, /*02BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F88, 0x0020, 0x0002, 0x0000, 0x0000, /*02BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F7D, 0x0020, 0x0002, 0x0000, 0x0000, /*02C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F89, 0x0020, 0x0002, 0x0000, 0x0000, /*02C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0496, 0x0020, 0x0002, 0x0000, 0x0000, /*02C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0497, 0x0020, 0x0002, 0x0000, 0x0000, /*02C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0498, 0x0020, 0x0002, 0x0000, 0x0000, /*02C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0499, 0x0020, 0x0002, 0x0000, 0x0000, /*02C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x049A, 0x0020, 0x0002, 0x0000, 0x0000, /*02C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x049B, 0x0020, 0x0002, 0x0000, 0x0000, /*02C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x049C, 0x0020, 0x0002, 0x0000, 0x0000, /*02C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x049D, 0x0020, 0x0002, 0x0000, 0x0000, /*02C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x049E, 0x0020, 0x0002, 0x0000, 0x0000, /*02CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x049F, 0x0020, 0x0002, 0x0000, 0x0000, /*02CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04A0, 0x0020, 0x0002, 0x0000, 0x0000, /*02CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04A1, 0x0020, 0x0002, 0x0000, 0x0000, /*02CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04A2, 0x0020, 0x0002, 0x0000, 0x0000, /*02CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04A3, 0x0020, 0x0002, 0x0000, 0x0000, /*02CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1BF8, 0x0020, 0x0002, 0x0000, 0x0000, /*02D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1BF9, 0x0020, 0x0002, 0x0000, 0x0000, /*02D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04A4, 0x0020, 0x0002, 0x0000, 0x0000, /*02D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04A5, 0x0020, 0x0002, 0x0000, 0x0000, /*02D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04A6, 0x0020, 0x0002, 0x0000, 0x0000, /*02D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04A7, 0x0020, 0x0002, 0x0000, 0x0000, /*02D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04A8, 0x0020, 0x0002, 0x0000, 0x0000, /*02D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04A9, 0x0020, 0x0002, 0x0000, 0x0000, /*02D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0487, 0x0020, 0x0002, 0x0000, 0x0000, /*02D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0488, 0x0020, 0x0002, 0x0000, 0x0000, /*02D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x048A, 0x0020, 0x0002, 0x0000, 0x0000, /*02DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x048D, 0x0020, 0x0002, 0x0000, 0x0000, /*02DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0484, 0x0020, 0x0002, 0x0000, 0x0000, /*02DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x048B, 0x0020, 0x0002, 0x0000, 0x0000, /*02DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04AA, 0x0020, 0x0002, 0x0000, 0x0000, /*02DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04AB, 0x0020, 0x0002, 0x0000, 0x0000, /*02DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D10, 0x0020, 0x0014, 0x0000, 0x0000, /*02E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x0014, 0x0000, 0x0000, /*02E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x0014, 0x0000, 0x0000, /*02E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFF, 0x0020, 0x0014, 0x0000, 0x0000, /*02E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F84, 0x0020, 0x0014, 0x0000, 0x0000, /*02E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04AC, 0x0020, 0x0002, 0x0000, 0x0000, /*02E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04AD, 0x0020, 0x0002, 0x0000, 0x0000, /*02E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04AE, 0x0020, 0x0002, 0x0000, 0x0000, /*02E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04AF, 0x0020, 0x0002, 0x0000, 0x0000, /*02E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04B0, 0x0020, 0x0002, 0x0000, 0x0000, /*02E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04B1, 0x0020, 0x0002, 0x0000, 0x0000, /*02EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04B2, 0x0020, 0x0002, 0x0000, 0x0000, /*02EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04B3, 0x0020, 0x0002, 0x0000, 0x0000, /*02EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04B4, 0x0020, 0x0002, 0x0000, 0x0000, /*02ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F7F, 0x0020, 0x0002, 0x0000, 0x0000, /*02EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04B5, 0x0020, 0x0002, 0x0000, 0x0000, /*02EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04B6, 0x0020, 0x0002, 0x0000, 0x0000, /*02F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04B7, 0x0020, 0x0002, 0x0000, 0x0000, /*02F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04B8, 0x0020, 0x0002, 0x0000, 0x0000, /*02F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04B9, 0x0020, 0x0002, 0x0000, 0x0000, /*02F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04BA, 0x0020, 0x0002, 0x0000, 0x0000, /*02F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04BB, 0x0020, 0x0002, 0x0000, 0x0000, /*02F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04BC, 0x0020, 0x0002, 0x0000, 0x0000, /*02F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04BD, 0x0020, 0x0002, 0x0000, 0x0000, /*02F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04BE, 0x0020, 0x0002, 0x0000, 0x0000, /*02F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04BF, 0x0020, 0x0002, 0x0000, 0x0000, /*02F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04C0, 0x0020, 0x0002, 0x0000, 0x0000, /*02FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04C1, 0x0020, 0x0002, 0x0000, 0x0000, /*02FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04C2, 0x0020, 0x0002, 0x0000, 0x0000, /*02FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04C3, 0x0020, 0x0002, 0x0000, 0x0000, /*02FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04C4, 0x0020, 0x0002, 0x0000, 0x0000, /*02FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04C5, 0x0020, 0x0002, 0x0000, 0x0000, /*02FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p003[]= {
  0x0000, 0x0025, 0x0002, 0x0000, 0x0000, /*0300*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0024, 0x0002, 0x0000, 0x0000, /*0301*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0027, 0x0002, 0x0000, 0x0000, /*0302*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x002D, 0x0002, 0x0000, 0x0000, /*0303*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0032, 0x0002, 0x0000, 0x0000, /*0304*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x003A, 0x0002, 0x0000, 0x0000, /*0305*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0026, 0x0002, 0x0000, 0x0000, /*0306*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x002E, 0x0002, 0x0000, 0x0000, /*0307*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x002B, 0x0002, 0x0000, 0x0000, /*0308*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x003B, 0x0002, 0x0000, 0x0000, /*0309*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0029, 0x0002, 0x0000, 0x0000, /*030A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x002C, 0x0002, 0x0000, 0x0000, /*030B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0028, 0x0002, 0x0000, 0x0000, /*030C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*030D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*030E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x003C, 0x0002, 0x0000, 0x0000, /*030F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x003D, 0x0002, 0x0000, 0x0000, /*0310*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x003E, 0x0002, 0x0000, 0x0000, /*0311*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*0312*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0022, 0x0002, 0x0000, 0x0000, /*0313*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0023, 0x0002, 0x0000, 0x0000, /*0314*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*0315*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0316*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0317*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0318*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0319*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*031A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x003F, 0x0002, 0x0000, 0x0000, /*031B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*031C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*031D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*031E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*031F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0320*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0040, 0x0002, 0x0000, 0x0000, /*0321*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0041, 0x0002, 0x0000, 0x0000, /*0322*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0042, 0x0002, 0x0000, 0x0000, /*0323*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0043, 0x0002, 0x0000, 0x0000, /*0324*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0044, 0x0002, 0x0000, 0x0000, /*0325*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0045, 0x0002, 0x0000, 0x0000, /*0326*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0030, 0x0002, 0x0000, 0x0000, /*0327*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0031, 0x0002, 0x0000, 0x0000, /*0328*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0329*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*032A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*032B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*032C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0046, 0x0002, 0x0000, 0x0000, /*032D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0047, 0x0002, 0x0000, 0x0000, /*032E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*032F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0048, 0x0002, 0x0000, 0x0000, /*0330*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0049, 0x0002, 0x0000, 0x0000, /*0331*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0021, 0x0002, 0x0000, 0x0000, /*0332*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0333*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x004A, 0x0002, 0x0000, 0x0000, /*0334*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0039, 0x0002, 0x0000, 0x0000, /*0335*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0035, 0x0002, 0x0000, 0x0000, /*0336*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0035, 0x0002, 0x0000, 0x0000, /*0337*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x002F, 0x0002, 0x0000, 0x0000, /*0338*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x004B, 0x0002, 0x0000, 0x0000, /*0339*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*033A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*033B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*033C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*033D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*033E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*033F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0025, 0x0002, 0x0000, 0x0000, /*0340*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0024, 0x0002, 0x0000, 0x0000, /*0341*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x002A, 0x0002, 0x0000, 0x0000, /*0342*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0022, 0x0002, 0x0000, 0x0000, /*0343*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x002B, 0x0002, 0x0000, 0x0024, /*0344*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0000, 0x004C, 0x0002, 0x0000, 0x0000, /*0345*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*0346*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0347*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0348*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0349*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*034A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*034B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*034C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*034D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*034E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*034F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*0350*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*0351*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*0352*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0353*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0354*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0355*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0356*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*0357*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x004D, 0x0002, 0x0000, 0x0000, /*0358*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0359*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*035A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*035B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*035C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*035D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*035E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*035F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x004E, 0x0002, 0x0000, 0x0000, /*0360*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x004F, 0x0002, 0x0000, 0x0000, /*0361*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*0362*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0004, 0x0000, 0x0000, /*0363*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x0004, 0x0000, 0x0000, /*0364*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x0004, 0x0000, 0x0000, /*0365*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0004, 0x0000, 0x0000, /*0366*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x0004, 0x0000, 0x0000, /*0367*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x0004, 0x0000, 0x0000, /*0368*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x0004, 0x0000, 0x0000, /*0369*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x0004, 0x0000, 0x0000, /*036A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x0004, 0x0000, 0x0000, /*036B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x0004, 0x0000, 0x0000, /*036C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x0004, 0x0000, 0x0000, /*036D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x0004, 0x0000, 0x0000, /*036E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFF, 0x0020, 0x0004, 0x0000, 0x0000, /*036F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC3, 0x0020, 0x0008, 0x0000, 0x0000, /*0370*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC3, 0x0020, 0x0002, 0x0000, 0x0000, /*0371*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE4, 0x0020, 0x0008, 0x0000, 0x0000, /*0372*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE4, 0x0020, 0x0002, 0x0000, 0x0000, /*0373*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0493, 0x0020, 0x0002, 0x0000, 0x0000, /*0374*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0494, 0x0020, 0x0002, 0x0000, 0x0000, /*0375*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC0, 0x0020, 0x0008, 0x0000, 0x0000, /*0376*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC0, 0x0020, 0x0002, 0x0000, 0x0000, /*0377*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x8378, 0x0000, /*0378*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8379, 0x0000, /*0379*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0004, 0x0000, 0x0000, /*037A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD9, 0x0020, 0x0002, 0x0000, 0x0000, /*037B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD8, 0x0020, 0x0002, 0x0000, 0x0000, /*037C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDA, 0x0020, 0x0002, 0x0000, 0x0000, /*037D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0234, 0x0020, 0x0002, 0x0000, 0x0000, /*037E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC7, 0x0020, 0x0008, 0x0000, 0x0000, /*037F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x8380, 0x0000, /*0380*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8381, 0x0000, /*0381*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8382, 0x0000, /*0382*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8383, 0x0000, /*0383*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0483, 0x0020, 0x0002, 0x0000, 0x0000, /*0384*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0489, 0x0020, 0x0002, 0x0000, 0x0024, /*0385*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0024, /*0386*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x028B, 0x0020, 0x0002, 0x0000, 0x0000, /*0387*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBE, 0x0020, 0x0008, 0x0000, 0x0024, /*0388*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0024, /*0389*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0024, /*038A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x838B, 0x0000, /*038B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCE, 0x0020, 0x0008, 0x0000, 0x0024, /*038C*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x838D, 0x0000, /*038D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0008, 0x0000, 0x0024, /*038E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0024, /*038F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x002B, /*0390*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0000, /*0391*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBA, 0x0020, 0x0008, 0x0000, 0x0000, /*0392*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBB, 0x0020, 0x0008, 0x0000, 0x0000, /*0393*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBD, 0x0020, 0x0008, 0x0000, 0x0000, /*0394*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBE, 0x0020, 0x0008, 0x0000, 0x0000, /*0395*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC2, 0x0020, 0x0008, 0x0000, 0x0000, /*0396*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0000, /*0397*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC5, 0x0020, 0x0008, 0x0000, 0x0000, /*0398*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0000, /*0399*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC8, 0x0020, 0x0008, 0x0000, 0x0000, /*039A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC9, 0x0020, 0x0008, 0x0000, 0x0000, /*039B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCB, 0x0020, 0x0008, 0x0000, 0x0000, /*039C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCC, 0x0020, 0x0008, 0x0000, 0x0000, /*039D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCD, 0x0020, 0x0008, 0x0000, 0x0000, /*039E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCE, 0x0020, 0x0008, 0x0000, 0x0000, /*039F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCF, 0x0020, 0x0008, 0x0000, 0x0000, /*03A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD4, 0x0020, 0x0008, 0x0000, 0x0000, /*03A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x83A2, 0x0000, /*03A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FD7, 0x0020, 0x0008, 0x0000, 0x0000, /*03A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDB, 0x0020, 0x0008, 0x0000, 0x0000, /*03A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDC, 0x0020, 0x0008, 0x0000, 0x0000, /*03A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDD, 0x0020, 0x0008, 0x0000, 0x0000, /*03A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDE, 0x0020, 0x0008, 0x0000, 0x0000, /*03A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDF, 0x0020, 0x0008, 0x0000, 0x0000, /*03A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0000, /*03A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x002B, /*03AA*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0008, 0x0000, 0x002B, /*03AB*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0024, /*03AC*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FBE, 0x0020, 0x0002, 0x0000, 0x0024, /*03AD*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0024, /*03AE*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0024, /*03AF*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x002B, /*03B0*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0000, /*03B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBA, 0x0020, 0x0002, 0x0000, 0x0000, /*03B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBB, 0x0020, 0x0002, 0x0000, 0x0000, /*03B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBD, 0x0020, 0x0002, 0x0000, 0x0000, /*03B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBE, 0x0020, 0x0002, 0x0000, 0x0000, /*03B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC2, 0x0020, 0x0002, 0x0000, 0x0000, /*03B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0000, /*03B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC5, 0x0020, 0x0002, 0x0000, 0x0000, /*03B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0000, /*03B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC8, 0x0020, 0x0002, 0x0000, 0x0000, /*03BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC9, 0x0020, 0x0002, 0x0000, 0x0000, /*03BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCB, 0x0020, 0x0002, 0x0000, 0x0000, /*03BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCC, 0x0020, 0x0002, 0x0000, 0x0000, /*03BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCD, 0x0020, 0x0002, 0x0000, 0x0000, /*03BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCE, 0x0020, 0x0002, 0x0000, 0x0000, /*03BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCF, 0x0020, 0x0002, 0x0000, 0x0000, /*03C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD4, 0x0020, 0x0002, 0x0000, 0x0000, /*03C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD7, 0x0020, 0x0019, 0x0000, 0x0000, /*03C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD7, 0x0020, 0x0002, 0x0000, 0x0000, /*03C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDB, 0x0020, 0x0002, 0x0000, 0x0000, /*03C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0000, /*03C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDD, 0x0020, 0x0002, 0x0000, 0x0000, /*03C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDE, 0x0020, 0x0002, 0x0000, 0x0000, /*03C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDF, 0x0020, 0x0002, 0x0000, 0x0000, /*03C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0000, /*03C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x002B, /*03CA*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x002B, /*03CB*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCE, 0x0020, 0x0002, 0x0000, 0x0024, /*03CC*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0024, /*03CD*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0024, /*03CE*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC8, 0x0020, 0x000A, 0x1FB9, 0x0020, /*03CF*/
  0x0004, 0x1FC6, 0x0020, 0x0004, 0x0003,
  0x1FBA, 0x0020, 0x0004, 0x0000, 0x0000, /*03D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC5, 0x0020, 0x0004, 0x0000, 0x0000, /*03D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDC, 0x0020, 0x000A, 0x0000, 0x0000, /*03D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDC, 0x0020, 0x000A, 0x0000, 0x0024, /*03D3*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x000A, 0x0000, 0x002B, /*03D4*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDD, 0x0020, 0x0004, 0x0000, 0x0000, /*03D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCF, 0x0020, 0x0004, 0x0000, 0x0000, /*03D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC8, 0x0020, 0x0004, 0x1FB9, 0x0020, /*03D7*/
  0x0004, 0x1FC6, 0x0020, 0x0004, 0x0003,
  0x1FD3, 0x0020, 0x0008, 0x0000, 0x0000, /*03D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD3, 0x0020, 0x0002, 0x0000, 0x0000, /*03D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC1, 0x0020, 0x0008, 0x0000, 0x0000, /*03DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC1, 0x0020, 0x0002, 0x0000, 0x0000, /*03DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBF, 0x0020, 0x0008, 0x0000, 0x0000, /*03DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBF, 0x0020, 0x0002, 0x0000, 0x0000, /*03DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD2, 0x0020, 0x0008, 0x0000, 0x0000, /*03DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD2, 0x0020, 0x0002, 0x0000, 0x0000, /*03DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE3, 0x0020, 0x0008, 0x0000, 0x0000, /*03E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE3, 0x0020, 0x0002, 0x0000, 0x0000, /*03E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2005, 0x0020, 0x0008, 0x0000, 0x0000, /*03E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2005, 0x0020, 0x0002, 0x0000, 0x0000, /*03E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x200A, 0x0020, 0x0008, 0x0000, 0x0000, /*03E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x200A, 0x0020, 0x0002, 0x0000, 0x0000, /*03E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x200B, 0x0020, 0x0008, 0x0000, 0x0000, /*03E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x200B, 0x0020, 0x0002, 0x0000, 0x0000, /*03E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x200E, 0x0020, 0x0008, 0x0000, 0x0000, /*03E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x200E, 0x0020, 0x0002, 0x0000, 0x0000, /*03E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2015, 0x0020, 0x0008, 0x0000, 0x0000, /*03EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2015, 0x0020, 0x0002, 0x0000, 0x0000, /*03EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2018, 0x0020, 0x0008, 0x0000, 0x0000, /*03EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2018, 0x0020, 0x0002, 0x0000, 0x0000, /*03ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x201C, 0x0020, 0x0008, 0x0000, 0x0000, /*03EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x201C, 0x0020, 0x0002, 0x0000, 0x0000, /*03EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC8, 0x0020, 0x0004, 0x0000, 0x0000, /*03F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD4, 0x0020, 0x0004, 0x0000, 0x0000, /*03F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD7, 0x0020, 0x0004, 0x0000, 0x0000, /*03F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC7, 0x0020, 0x0002, 0x0000, 0x0000, /*03F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC5, 0x0020, 0x000A, 0x0000, 0x0000, /*03F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBE, 0x0020, 0x0004, 0x0000, 0x0000, /*03F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0611, 0x0020, 0x0002, 0x0000, 0x0000, /*03F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE5, 0x0020, 0x0008, 0x0000, 0x0000, /*03F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE5, 0x0020, 0x0002, 0x0000, 0x0000, /*03F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD7, 0x0020, 0x000A, 0x0000, 0x0000, /*03F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD1, 0x0020, 0x0008, 0x0000, 0x0000, /*03FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD1, 0x0020, 0x0002, 0x0000, 0x0000, /*03FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD6, 0x0020, 0x0002, 0x0000, 0x0000, /*03FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD9, 0x0020, 0x0008, 0x0000, 0x0000, /*03FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD8, 0x0020, 0x0008, 0x0000, 0x0000, /*03FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDA, 0x0020, 0x0008, 0x0000, 0x0000, /*03FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p004[]= {
  0x205A, 0x0020, 0x0008, 0x0000, 0x0025, 0x0002, 0x0002, /*0400*/
  0x205A, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*0401*/
  0x2050, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0402*/
  0x2036, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0002, /*0403*/
  0x205E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0404*/
  0x2074, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0405*/
  0x2088, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0406*/
  0x2088, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*0407*/
  0x2091, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0408*/
  0x20BC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0409*/
  0x20E2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*040A*/
  0x2119, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*040B*/
  0x2096, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0002, /*040C*/
  0x2080, 0x0020, 0x0008, 0x0000, 0x0025, 0x0002, 0x0002, /*040D*/
  0x211D, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0002, /*040E*/
  0x217F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*040F*/
  0x2022, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0410*/
  0x202E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0411*/
  0x2032, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0412*/
  0x2036, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0413*/
  0x204A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0414*/
  0x205A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0415*/
  0x2062, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0416*/
  0x206C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0417*/
  0x2080, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0418*/
  0x208D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0419*/
  0x2096, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*041A*/
  0x20B0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*041B*/
  0x20C3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*041C*/
  0x20CC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*041D*/
  0x20E7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*041E*/
  0x20EF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*041F*/
  0x20FC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0420*/
  0x2105, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0421*/
  0x210E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0422*/
  0x211D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0423*/
  0x212E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0424*/
  0x2132, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0425*/
  0x2159, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0426*/
  0x2164, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0427*/
  0x2183, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0428*/
  0x2188, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0429*/
  0x218F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*042A*/
  0x2194, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*042B*/
  0x2198, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*042C*/
  0x21A5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*042D*/
  0x21A9, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*042E*/
  0x21AF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*042F*/
  0x2022, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0430*/
  0x202E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0431*/
  0x2032, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0432*/
  0x2036, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0433*/
  0x204A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0434*/
  0x205A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0435*/
  0x2062, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0436*/
  0x206C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0437*/
  0x2080, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0438*/
  0x208D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0439*/
  0x2096, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*043A*/
  0x20B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*043B*/
  0x20C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*043C*/
  0x20CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*043D*/
  0x20E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*043E*/
  0x20EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*043F*/
  0x20FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0440*/
  0x2105, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0441*/
  0x210E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0442*/
  0x211D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0443*/
  0x212E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0444*/
  0x2132, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0445*/
  0x2159, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0446*/
  0x2164, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0447*/
  0x2183, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0448*/
  0x2188, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0449*/
  0x218F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*044A*/
  0x2194, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*044B*/
  0x2198, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*044C*/
  0x21A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*044D*/
  0x21A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*044E*/
  0x21AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*044F*/
  0x205A, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0002, /*0450*/
  0x205A, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*0451*/
  0x2050, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0452*/
  0x2036, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0002, /*0453*/
  0x205E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0454*/
  0x2074, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0455*/
  0x2088, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0456*/
  0x2088, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*0457*/
  0x2091, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0458*/
  0x20BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0459*/
  0x20E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*045A*/
  0x2119, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*045B*/
  0x2096, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0002, /*045C*/
  0x2080, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0002, /*045D*/
  0x211D, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0002, /*045E*/
  0x217F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*045F*/
  0x2148, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0460*/
  0x2148, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0461*/
  0x21A0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0462*/
  0x21A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0463*/
  0x21B4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0464*/
  0x21B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0465*/
  0x21B8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0466*/
  0x21B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0467*/
  0x21C2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0468*/
  0x21C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0469*/
  0x21BD, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*046A*/
  0x21BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*046B*/
  0x21C7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*046C*/
  0x21C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*046D*/
  0x21CB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*046E*/
  0x21CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*046F*/
  0x21CF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0470*/
  0x21CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0471*/
  0x21D3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0472*/
  0x21D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0473*/
  0x21D7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0474*/
  0x21D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0475*/
  0x21D7, 0x0020, 0x0008, 0x0000, 0x003C, 0x0002, 0x0002, /*0476*/
  0x21D7, 0x0020, 0x0002, 0x0000, 0x003C, 0x0002, 0x0002, /*0477*/
  0x212A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0478*/
  0x212A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0479*/
  0x2155, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*047A*/
  0x2155, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*047B*/
  0x2151, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*047C*/
  0x2151, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*047D*/
  0x214C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*047E*/
  0x214C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*047F*/
  0x20F8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0480*/
  0x20F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0481*/
  0x04F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0482*/
  0x0000, 0x0050, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0483*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0484*/
  0x0000, 0x0023, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0485*/
  0x0000, 0x0022, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0486*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0487*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0488*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0489*/
  0x2084, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*048A*/
  0x2084, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*048B*/
  0x219C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*048C*/
  0x219C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*048D*/
  0x2100, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*048E*/
  0x2100, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*048F*/
  0x2036, 0x0020, 0x000A, 0x0000, 0x0111, 0x0004, 0x0002, /*0490*/
  0x2036, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*0491*/
  0x203A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0492*/
  0x203A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0493*/
  0x2042, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0494*/
  0x2042, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0495*/
  0x2068, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0496*/
  0x2068, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0497*/
  0x2056, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0498*/
  0x2056, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0499*/
  0x209A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*049A*/
  0x209A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*049B*/
  0x20AA, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*049C*/
  0x20AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*049D*/
  0x20A6, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*049E*/
  0x20A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*049F*/
  0x20A2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04A0*/
  0x20A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04A1*/
  0x20D5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04A2*/
  0x20D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04A3*/
  0x20DE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04A4*/
  0x20DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04A5*/
  0x20F4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04A6*/
  0x20F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04A7*/
  0x21DC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04A8*/
  0x21DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04A9*/
  0x210A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04AA*/
  0x210A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04AB*/
  0x2114, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04AC*/
  0x2114, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04AD*/
  0x2121, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04AE*/
  0x2121, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04AF*/
  0x2125, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04B0*/
  0x2125, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04B1*/
  0x213E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04B2*/
  0x213E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04B3*/
  0x215F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04B4*/
  0x215F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04B5*/
  0x216A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04B6*/
  0x216A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04B7*/
  0x2172, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04B8*/
  0x2172, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04B9*/
  0x2142, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04BA*/
  0x2142, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04BB*/
  0x2177, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04BC*/
  0x2177, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04BD*/
  0x217B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04BE*/
  0x217B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04BF*/
  0x21E1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04C0*/
  0x2062, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0002, /*04C1*/
  0x2062, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0002, /*04C2*/
  0x209E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04C3*/
  0x209E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04C4*/
  0x20B5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04C5*/
  0x20B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04C6*/
  0x20D9, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04C7*/
  0x20D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04C8*/
  0x20D1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04C9*/
  0x20D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04CA*/
  0x216E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04CB*/
  0x216E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04CC*/
  0x20C7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04CD*/
  0x20C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04CE*/
  0x21E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04CF*/
  0x2022, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0002, /*04D0*/
  0x2022, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0002, /*04D1*/
  0x2022, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*04D2*/
  0x2022, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*04D3*/
  0x202A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04D4*/
  0x202A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04D5*/
  0x205A, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0002, /*04D6*/
  0x205A, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0002, /*04D7*/
  0x2026, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04D8*/
  0x2026, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04D9*/
  0x2026, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*04DA*/
  0x2026, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*04DB*/
  0x2062, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*04DC*/
  0x2062, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*04DD*/
  0x206C, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*04DE*/
  0x206C, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*04DF*/
  0x2079, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04E0*/
  0x2079, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04E1*/
  0x2080, 0x0020, 0x0008, 0x0000, 0x0032, 0x0002, 0x0002, /*04E2*/
  0x2080, 0x0020, 0x0002, 0x0000, 0x0032, 0x0002, 0x0002, /*04E3*/
  0x2080, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*04E4*/
  0x2080, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*04E5*/
  0x20E7, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*04E6*/
  0x20E7, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*04E7*/
  0x20EB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04E8*/
  0x20EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04E9*/
  0x20EB, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*04EA*/
  0x20EB, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*04EB*/
  0x21A5, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*04EC*/
  0x21A5, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*04ED*/
  0x211D, 0x0020, 0x0008, 0x0000, 0x0032, 0x0002, 0x0002, /*04EE*/
  0x211D, 0x0020, 0x0002, 0x0000, 0x0032, 0x0002, 0x0002, /*04EF*/
  0x211D, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*04F0*/
  0x211D, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*04F1*/
  0x211D, 0x0020, 0x0008, 0x0000, 0x002C, 0x0002, 0x0002, /*04F2*/
  0x211D, 0x0020, 0x0002, 0x0000, 0x002C, 0x0002, 0x0002, /*04F3*/
  0x2164, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*04F4*/
  0x2164, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*04F5*/
  0x2046, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04F6*/
  0x2046, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04F7*/
  0x2194, 0x0020, 0x0008, 0x0000, 0x002B, 0x0002, 0x0002, /*04F8*/
  0x2194, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0002, /*04F9*/
  0x203E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04FA*/
  0x203E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04FB*/
  0x2136, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04FC*/
  0x2136, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*04FD*/
  0x213A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*04FE*/
  0x213A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*04FF*/
};

uint16 uca900_p005[]= {
  0x204E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0500*/
  0x204E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0501*/
  0x2055, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0502*/
  0x2055, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0503*/
  0x2071, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0504*/
  0x2071, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0505*/
  0x207E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0506*/
  0x207E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0507*/
  0x20C1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0508*/
  0x20C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0509*/
  0x20E6, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*050A*/
  0x20E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*050B*/
  0x2109, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*050C*/
  0x2109, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*050D*/
  0x2113, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*050E*/
  0x2113, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*050F*/
  0x2072, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0510*/
  0x2072, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0511*/
  0x20BA, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0512*/
  0x20BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0513*/
  0x20C2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0514*/
  0x20C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0515*/
  0x2104, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0516*/
  0x2104, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0517*/
  0x21B3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0518*/
  0x21B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0519*/
  0x20AF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*051A*/
  0x20AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*051B*/
  0x21E0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*051C*/
  0x21E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*051D*/
  0x20AE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*051E*/
  0x20AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*051F*/
  0x20BB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0520*/
  0x20BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0521*/
  0x20DD, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0522*/
  0x20DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0523*/
  0x20F3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0524*/
  0x20F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0525*/
  0x2146, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0526*/
  0x2146, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0527*/
  0x20D0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0528*/
  0x20D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0529*/
  0x2066, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*052A*/
  0x2066, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*052B*/
  0x2168, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*052C*/
  0x2168, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*052D*/
  0x20B9, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*052E*/
  0x20B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*052F*/
  0xFBC0, 0x0020, 0x0002, 0x8530, 0x0000, 0x0000, 0x0002, /*0530*/
  0x2290, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0531*/
  0x2291, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0532*/
  0x2292, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0533*/
  0x2293, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0534*/
  0x2294, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0535*/
  0x2295, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0536*/
  0x2296, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0537*/
  0x2297, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0538*/
  0x2298, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0539*/
  0x2299, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*053A*/
  0x229A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*053B*/
  0x229B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*053C*/
  0x229C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*053D*/
  0x229D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*053E*/
  0x229E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*053F*/
  0x229F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0540*/
  0x22A0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0541*/
  0x22A1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0542*/
  0x22A2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0543*/
  0x22A3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0544*/
  0x22A4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0545*/
  0x22A5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0546*/
  0x22A6, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0547*/
  0x22A7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0548*/
  0x22A8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0549*/
  0x22A9, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*054A*/
  0x22AA, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*054B*/
  0x22AB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*054C*/
  0x22AC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*054D*/
  0x22AD, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*054E*/
  0x22AE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*054F*/
  0x22AF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0550*/
  0x22B0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0551*/
  0x22B1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0552*/
  0x22B2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0553*/
  0x22B3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0554*/
  0x22B4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0555*/
  0x22B5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*0556*/
  0xFBC0, 0x0020, 0x0002, 0x8557, 0x0000, 0x0000, 0x0002, /*0557*/
  0xFBC0, 0x0020, 0x0002, 0x8558, 0x0000, 0x0000, 0x0002, /*0558*/
  0x22B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0559*/
  0x03CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*055A*/
  0x03CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*055B*/
  0x0262, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*055C*/
  0x0226, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*055D*/
  0x0269, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*055E*/
  0x03CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*055F*/
  0xFBC0, 0x0020, 0x0002, 0x8560, 0x0000, 0x0000, 0x0002, /*0560*/
  0x2290, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0561*/
  0x2291, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0562*/
  0x2292, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0563*/
  0x2293, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0564*/
  0x2294, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0565*/
  0x2295, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0566*/
  0x2296, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0567*/
  0x2297, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0568*/
  0x2298, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0569*/
  0x2299, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*056A*/
  0x229A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*056B*/
  0x229B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*056C*/
  0x229C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*056D*/
  0x229D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*056E*/
  0x229E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*056F*/
  0x229F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0570*/
  0x22A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0571*/
  0x22A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0572*/
  0x22A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0573*/
  0x22A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0574*/
  0x22A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0575*/
  0x22A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0576*/
  0x22A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0577*/
  0x22A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0578*/
  0x22A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0579*/
  0x22A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*057A*/
  0x22AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*057B*/
  0x22AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*057C*/
  0x22AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*057D*/
  0x22AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*057E*/
  0x22AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*057F*/
  0x22AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0580*/
  0x22B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0581*/
  0x22B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0582*/
  0x22B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0583*/
  0x22B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0584*/
  0x22B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0585*/
  0x22B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0586*/
  0x2294, 0x0020, 0x0004, 0x22B1, 0x0020, 0x0004, 0x0002, /*0587*/
  0xFBC0, 0x0020, 0x0002, 0x8588, 0x0000, 0x0000, 0x0002, /*0588*/
  0x023A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0589*/
  0x020E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*058A*/
  0xFBC0, 0x0020, 0x0002, 0x858B, 0x0000, 0x0000, 0x0002, /*058B*/
  0xFBC0, 0x0020, 0x0002, 0x858C, 0x0000, 0x0000, 0x0002, /*058C*/
  0x04F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*058D*/
  0x04F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*058E*/
  0x1C15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*058F*/
  0xFBC0, 0x0020, 0x0002, 0x8590, 0x0000, 0x0000, 0x0002, /*0590*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0591*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0592*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0593*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0594*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0595*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0596*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0597*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0598*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0599*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*059A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*059B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*059C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*059D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*059E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*059F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05AF*/
  0x0000, 0x0052, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05B0*/
  0x0000, 0x0053, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05B1*/
  0x0000, 0x0054, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05B2*/
  0x0000, 0x0055, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05B3*/
  0x0000, 0x0056, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05B4*/
  0x0000, 0x0057, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05B5*/
  0x0000, 0x0058, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05B6*/
  0x0000, 0x0059, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05B7*/
  0x0000, 0x005A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05B8*/
  0x0000, 0x005B, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05B9*/
  0x0000, 0x005B, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05BA*/
  0x0000, 0x005C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05BB*/
  0x0000, 0x005F, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05BD*/
  0x03CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05BE*/
  0x0000, 0x0060, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05BF*/
  0x03CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05C0*/
  0x0000, 0x005E, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05C1*/
  0x0000, 0x005D, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05C2*/
  0x03D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*05C5*/
  0x03D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05C6*/
  0x0000, 0x005A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05C7*/
  0xFBC0, 0x0020, 0x0002, 0x85C8, 0x0000, 0x0000, 0x0002, /*05C8*/
  0xFBC0, 0x0020, 0x0002, 0x85C9, 0x0000, 0x0000, 0x0002, /*05C9*/
  0xFBC0, 0x0020, 0x0002, 0x85CA, 0x0000, 0x0000, 0x0002, /*05CA*/
  0xFBC0, 0x0020, 0x0002, 0x85CB, 0x0000, 0x0000, 0x0002, /*05CB*/
  0xFBC0, 0x0020, 0x0002, 0x85CC, 0x0000, 0x0000, 0x0002, /*05CC*/
  0xFBC0, 0x0020, 0x0002, 0x85CD, 0x0000, 0x0000, 0x0002, /*05CD*/
  0xFBC0, 0x0020, 0x0002, 0x85CE, 0x0000, 0x0000, 0x0002, /*05CE*/
  0xFBC0, 0x0020, 0x0002, 0x85CF, 0x0000, 0x0000, 0x0002, /*05CF*/
  0x22B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05D0*/
  0x22B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05D1*/
  0x22B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05D2*/
  0x22BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05D3*/
  0x22BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05D4*/
  0x22BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05D5*/
  0x22BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05D6*/
  0x22BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05D7*/
  0x22BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05D8*/
  0x22C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05D9*/
  0x22C1, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*05DA*/
  0x22C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05DB*/
  0x22C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05DC*/
  0x22C3, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*05DD*/
  0x22C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05DE*/
  0x22C4, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*05DF*/
  0x22C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05E0*/
  0x22C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05E1*/
  0x22C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05E2*/
  0x22C7, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*05E3*/
  0x22C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05E4*/
  0x22C8, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*05E5*/
  0x22C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05E6*/
  0x22C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05E7*/
  0x22CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05E8*/
  0x22CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05E9*/
  0x22CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05EA*/
  0xFBC0, 0x0020, 0x0002, 0x85EB, 0x0000, 0x0000, 0x0002, /*05EB*/
  0xFBC0, 0x0020, 0x0002, 0x85EC, 0x0000, 0x0000, 0x0002, /*05EC*/
  0xFBC0, 0x0020, 0x0002, 0x85ED, 0x0000, 0x0000, 0x0002, /*05ED*/
  0xFBC0, 0x0020, 0x0002, 0x85EE, 0x0000, 0x0000, 0x0002, /*05EE*/
  0xFBC0, 0x0020, 0x0002, 0x85EF, 0x0000, 0x0000, 0x0002, /*05EF*/
  0x22BC, 0x0020, 0x0004, 0x22BC, 0x0020, 0x0004, 0x0002, /*05F0*/
  0x22BC, 0x0020, 0x0004, 0x22C0, 0x0020, 0x0004, 0x0002, /*05F1*/
  0x22C0, 0x0020, 0x0004, 0x22C0, 0x0020, 0x0004, 0x0002, /*05F2*/
  0x03D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05F3*/
  0x03D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*05F4*/
  0xFBC0, 0x0020, 0x0002, 0x85F5, 0x0000, 0x0000, 0x0002, /*05F5*/
  0xFBC0, 0x0020, 0x0002, 0x85F6, 0x0000, 0x0000, 0x0002, /*05F6*/
  0xFBC0, 0x0020, 0x0002, 0x85F7, 0x0000, 0x0000, 0x0002, /*05F7*/
  0xFBC0, 0x0020, 0x0002, 0x85F8, 0x0000, 0x0000, 0x0002, /*05F8*/
  0xFBC0, 0x0020, 0x0002, 0x85F9, 0x0000, 0x0000, 0x0002, /*05F9*/
  0xFBC0, 0x0020, 0x0002, 0x85FA, 0x0000, 0x0000, 0x0002, /*05FA*/
  0xFBC0, 0x0020, 0x0002, 0x85FB, 0x0000, 0x0000, 0x0002, /*05FB*/
  0xFBC0, 0x0020, 0x0002, 0x85FC, 0x0000, 0x0000, 0x0002, /*05FC*/
  0xFBC0, 0x0020, 0x0002, 0x85FD, 0x0000, 0x0000, 0x0002, /*05FD*/
  0xFBC0, 0x0020, 0x0002, 0x85FE, 0x0000, 0x0000, 0x0002, /*05FE*/
  0xFBC0, 0x0020, 0x0002, 0x85FF, 0x0000, 0x0000, 0x0002  /*05FF*/
};

uint16 uca900_p006[]= {
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0600*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0601*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0602*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0603*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0604*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0605*/
  0x062D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0606*/
  0x062F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0607*/
  0x04FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0608*/
  0x039C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0609*/
  0x039E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*060A*/
  0x1C16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*060B*/
  0x0227, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*060C*/
  0x0228, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*060D*/
  0x04FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*060E*/
  0x04FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*060F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0610*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0611*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0612*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0613*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0614*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0615*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0616*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0617*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0618*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0619*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*061A*/
  0x0235, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*061B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*061C*/
  0xFBC0, 0x0020, 0x0002, 0x861D, 0x0000, 0x0000, 0x0002, /*061D*/
  0x023B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*061E*/
  0x026A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*061F*/
  0x23CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0620*/
  0x22FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0621*/
  0x22FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0622*/
  0x22FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0623*/
  0x2302, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0624*/
  0x2303, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0625*/
  0x2307, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0626*/
  0x230B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0627*/
  0x230D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0628*/
  0x231C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0629*/
  0x231D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*062A*/
  0x231E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*062B*/
  0x2325, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*062C*/
  0x232C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*062D*/
  0x232D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*062E*/
  0x2337, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*062F*/
  0x2338, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0630*/
  0x2346, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0631*/
  0x2347, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0632*/
  0x2359, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0633*/
  0x235A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0634*/
  0x2364, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0635*/
  0x2365, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0636*/
  0x236A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0637*/
  0x236B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0638*/
  0x236E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0639*/
  0x236F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*063A*/
  0x2398, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*063B*/
  0x2399, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*063C*/
  0x23CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*063D*/
  0x23CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*063E*/
  0x23CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*063F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0640*/
  0x2376, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0641*/
  0x2382, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0642*/
  0x2387, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0643*/
  0x239C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0644*/
  0x23A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0645*/
  0x23A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0646*/
  0x23B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0647*/
  0x23B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0648*/
  0x23C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0649*/
  0x23C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*064A*/
  0x0000, 0x006D, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*064B*/
  0x0000, 0x0070, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*064C*/
  0x0000, 0x0073, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*064D*/
  0x0000, 0x0076, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*064E*/
  0x0000, 0x007A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*064F*/
  0x0000, 0x007D, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0650*/
  0x0000, 0x0080, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0651*/
  0x0000, 0x0081, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0652*/
  0x0000, 0x0082, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0653*/
  0x0000, 0x0083, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0654*/
  0x0000, 0x0084, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0655*/
  0x0000, 0x0086, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0656*/
  0x0000, 0x0087, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0657*/
  0x0000, 0x0088, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0658*/
  0x0000, 0x008A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0659*/
  0x0000, 0x008B, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*065A*/
  0x0000, 0x008C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*065B*/
  0x0000, 0x008D, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*065C*/
  0x0000, 0x008E, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*065D*/
  0x0000, 0x008F, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*065E*/
  0x0000, 0x0085, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*065F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0660*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0661*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0662*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0663*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0664*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0665*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0666*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0667*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0668*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0669*/
  0x039A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*066A*/
  0x0229, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*066B*/
  0x022A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*066C*/
  0x0392, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*066D*/
  0x230C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*066E*/
  0x2381, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*066F*/
  0x0000, 0x0098, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0670*/
  0x2301, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0671*/
  0x2300, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0672*/
  0x2304, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0673*/
  0x22FD, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*0674*/
  0x230B, 0x0020, 0x0004, 0x22FD, 0x0020, 0x0004, 0x0002, /*0675*/
  0x23B7, 0x0020, 0x0004, 0x22FD, 0x0020, 0x0004, 0x0002, /*0676*/
  0x23BB, 0x0020, 0x0004, 0x22FD, 0x0020, 0x0004, 0x0002, /*0677*/
  0x23C6, 0x0020, 0x0004, 0x22FD, 0x0020, 0x0004, 0x0002, /*0678*/
  0x231F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0679*/
  0x2320, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*067A*/
  0x230E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*067B*/
  0x2321, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*067C*/
  0x2322, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*067D*/
  0x230F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*067E*/
  0x2323, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*067F*/
  0x2310, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0680*/
  0x232E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0681*/
  0x232F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0682*/
  0x2326, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0683*/
  0x2327, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0684*/
  0x2330, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0685*/
  0x2328, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0686*/
  0x232A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0687*/
  0x2339, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0688*/
  0x233A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0689*/
  0x233B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*068A*/
  0x233C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*068B*/
  0x233D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*068C*/
  0x233E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*068D*/
  0x2340, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*068E*/
  0x2341, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*068F*/
  0x2342, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0690*/
  0x2348, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0691*/
  0x2349, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0692*/
  0x234A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0693*/
  0x234B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0694*/
  0x234C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0695*/
  0x234D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0696*/
  0x234E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0697*/
  0x234F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0698*/
  0x2350, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0699*/
  0x235B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*069A*/
  0x235C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*069B*/
  0x235D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*069C*/
  0x2366, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*069D*/
  0x2368, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*069E*/
  0x236C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*069F*/
  0x2370, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06A0*/
  0x2377, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06A1*/
  0x2378, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06A2*/
  0x237A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06A3*/
  0x237B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06A4*/
  0x237D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06A5*/
  0x237E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06A6*/
  0x2383, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06A7*/
  0x2385, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06A8*/
  0x2388, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06A9*/
  0x2389, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06AA*/
  0x238A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06AB*/
  0x238B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06AC*/
  0x238D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06AD*/
  0x238E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06AE*/
  0x2390, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06AF*/
  0x2392, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06B0*/
  0x2393, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06B1*/
  0x2394, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06B2*/
  0x2395, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06B3*/
  0x2396, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06B4*/
  0x239D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06B5*/
  0x239E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06B6*/
  0x239F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06B7*/
  0x23A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06B8*/
  0x23AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06B9*/
  0x23A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06BA*/
  0x23AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06BB*/
  0x23AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06BC*/
  0x23AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06BD*/
  0x23B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06BE*/
  0x2329, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06BF*/
  0x23B6, 0x0020, 0x0002, 0x0000, 0x0083, 0x0002, 0x0002, /*06C0*/
  0x23B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06C1*/
  0x23B3, 0x0020, 0x0002, 0x0000, 0x0083, 0x0002, 0x0002, /*06C2*/
  0x23B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06C3*/
  0x23B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06C4*/
  0x23B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06C5*/
  0x23BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06C6*/
  0x23BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06C7*/
  0x23BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06C8*/
  0x23BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06C9*/
  0x23BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06CA*/
  0x23BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06CB*/
  0x23C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06CC*/
  0x23C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06CD*/
  0x23C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06CE*/
  0x23C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06CF*/
  0x23CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06D0*/
  0x23CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06D1*/
  0x23D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06D2*/
  0x23D4, 0x0020, 0x0002, 0x0000, 0x0083, 0x0002, 0x0002, /*06D3*/
  0x0279, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06D4*/
  0x23B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06DD*/
  0x04FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06E4*/
  0x23B7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*06E5*/
  0x23C6, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*06E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06E8*/
  0x0500, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*06ED*/
  0x2343, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06EE*/
  0x2351, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06EF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06F0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06F1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06F2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06F3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06F4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06F5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06F6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06F7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06F8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06F9*/
  0x235E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06FA*/
  0x2369, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06FB*/
  0x2371, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*06FC*/
  0x22FD, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*06FD*/
  0x23A3, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*06FE*/
  0x23B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*06FF*/
};

uint16 uca900_p007[]= {
  0x02C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0700*/
  0x027A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0701*/
  0x027B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0702*/
  0x023C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0703*/
  0x023D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0704*/
  0x023E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0705*/
  0x023F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0706*/
  0x0240, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0707*/
  0x0241, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0708*/
  0x026B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0709*/
  0x03D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*070A*/
  0x03D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*070B*/
  0x03D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*070C*/
  0x03D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*070D*/
  0xFBC0, 0x0020, 0x0002, 0x870E, 0x0000, 0x0000, 0x0002, /*070E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*070F*/
  0x23D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0710*/
  0x0000, 0x0099, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0711*/
  0x23D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0712*/
  0x23D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0713*/
  0x23D9, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*0714*/
  0x23DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0715*/
  0x23DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0716*/
  0x23DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0717*/
  0x23DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0718*/
  0x23DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0719*/
  0x23E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*071A*/
  0x23E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*071B*/
  0x23E1, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*071C*/
  0x23E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*071D*/
  0x23E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*071E*/
  0x23E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*071F*/
  0x23E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0720*/
  0x23E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0721*/
  0x23E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0722*/
  0x23E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0723*/
  0x23E9, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*0724*/
  0x23EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0725*/
  0x23EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0726*/
  0x23EB, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*0727*/
  0x23ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0728*/
  0x23EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0729*/
  0x23EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*072A*/
  0x23F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*072B*/
  0x23F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*072C*/
  0x23D8, 0x0020, 0x0004, 0x0000, 0x0112, 0x0004, 0x0002, /*072D*/
  0x23D9, 0x0020, 0x0004, 0x0000, 0x0112, 0x0004, 0x0002, /*072E*/
  0x23DB, 0x0020, 0x0004, 0x0000, 0x0112, 0x0004, 0x0002, /*072F*/
  0x0000, 0x009A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0730*/
  0x0000, 0x009B, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0731*/
  0x0000, 0x009C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0732*/
  0x0000, 0x009D, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0733*/
  0x0000, 0x009E, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0734*/
  0x0000, 0x009F, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0735*/
  0x0000, 0x00A0, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0736*/
  0x0000, 0x00A1, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0737*/
  0x0000, 0x00A2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0738*/
  0x0000, 0x00A3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0739*/
  0x0000, 0x00A4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*073A*/
  0x0000, 0x00A5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*073B*/
  0x0000, 0x00A6, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*073C*/
  0x0000, 0x00A7, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*073D*/
  0x0000, 0x00A8, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*073E*/
  0x0000, 0x00A9, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*073F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0740*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0741*/
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0742*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0743*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0744*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0745*/
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0746*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0747*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0748*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0749*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*074A*/
  0xFBC0, 0x0020, 0x0002, 0x874B, 0x0000, 0x0000, 0x0002, /*074B*/
  0xFBC0, 0x0020, 0x0002, 0x874C, 0x0000, 0x0000, 0x0002, /*074C*/
  0x23DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*074D*/
  0x23E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*074E*/
  0x23EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*074F*/
  0x2311, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0750*/
  0x2312, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0751*/
  0x2313, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0752*/
  0x2314, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0753*/
  0x2315, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0754*/
  0x2316, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0755*/
  0x2318, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0756*/
  0x2331, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0757*/
  0x2332, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0758*/
  0x2344, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0759*/
  0x2345, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*075A*/
  0x2352, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*075B*/
  0x235F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*075C*/
  0x2372, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*075D*/
  0x2373, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*075E*/
  0x2374, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*075F*/
  0x237F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0760*/
  0x2380, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0761*/
  0x2397, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0762*/
  0x239A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0763*/
  0x239B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0764*/
  0x23A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0765*/
  0x23A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0766*/
  0x23AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0767*/
  0x23AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0768*/
  0x23B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0769*/
  0x23A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*076A*/
  0x2353, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*076B*/
  0x2354, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*076C*/
  0x2360, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*076D*/
  0x2333, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*076E*/
  0x2334, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*076F*/
  0x2361, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0770*/
  0x2355, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0771*/
  0x2335, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0772*/
  0x2305, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0773*/
  0x2306, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0774*/
  0x23D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0775*/
  0x23D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0776*/
  0x23D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0777*/
  0x23C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0778*/
  0x23C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0779*/
  0x23D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*077A*/
  0x23D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*077B*/
  0x2336, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*077C*/
  0x2362, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*077D*/
  0x2363, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*077E*/
  0x238C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*077F*/
  0x240B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0780*/
  0x240E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0781*/
  0x240F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0782*/
  0x2410, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0783*/
  0x2412, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0784*/
  0x2413, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0785*/
  0x2414, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0786*/
  0x2415, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0787*/
  0x2418, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0788*/
  0x241A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0789*/
  0x241B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*078A*/
  0x241C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*078B*/
  0x241E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*078C*/
  0x2422, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*078D*/
  0x2423, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*078E*/
  0x2425, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*078F*/
  0x2426, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0790*/
  0x242A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0791*/
  0x242B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0792*/
  0x242C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0793*/
  0x242D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0794*/
  0x242E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0795*/
  0x242F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0796*/
  0x2430, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0797*/
  0x241F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0798*/
  0x240C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0799*/
  0x240D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*079A*/
  0x241D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*079B*/
  0x2411, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*079C*/
  0x2427, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*079D*/
  0x2428, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*079E*/
  0x2429, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*079F*/
  0x2420, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07A0*/
  0x2421, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07A1*/
  0x2416, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07A2*/
  0x2417, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07A3*/
  0x2424, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07A4*/
  0x2419, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07A5*/
  0x2432, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07A6*/
  0x2433, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07A7*/
  0x2434, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07A8*/
  0x2435, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07A9*/
  0x2436, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07AA*/
  0x2437, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07AB*/
  0x2438, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07AC*/
  0x2439, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07AD*/
  0x243A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07AE*/
  0x243B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07AF*/
  0x243C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07B0*/
  0x2431, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07B1*/
  0xFBC0, 0x0020, 0x0002, 0x87B2, 0x0000, 0x0000, 0x0002, /*07B2*/
  0xFBC0, 0x0020, 0x0002, 0x87B3, 0x0000, 0x0000, 0x0002, /*07B3*/
  0xFBC0, 0x0020, 0x0002, 0x87B4, 0x0000, 0x0000, 0x0002, /*07B4*/
  0xFBC0, 0x0020, 0x0002, 0x87B5, 0x0000, 0x0000, 0x0002, /*07B5*/
  0xFBC0, 0x0020, 0x0002, 0x87B6, 0x0000, 0x0000, 0x0002, /*07B6*/
  0xFBC0, 0x0020, 0x0002, 0x87B7, 0x0000, 0x0000, 0x0002, /*07B7*/
  0xFBC0, 0x0020, 0x0002, 0x87B8, 0x0000, 0x0000, 0x0002, /*07B8*/
  0xFBC0, 0x0020, 0x0002, 0x87B9, 0x0000, 0x0000, 0x0002, /*07B9*/
  0xFBC0, 0x0020, 0x0002, 0x87BA, 0x0000, 0x0000, 0x0002, /*07BA*/
  0xFBC0, 0x0020, 0x0002, 0x87BB, 0x0000, 0x0000, 0x0002, /*07BB*/
  0xFBC0, 0x0020, 0x0002, 0x87BC, 0x0000, 0x0000, 0x0002, /*07BC*/
  0xFBC0, 0x0020, 0x0002, 0x87BD, 0x0000, 0x0000, 0x0002, /*07BD*/
  0xFBC0, 0x0020, 0x0002, 0x87BE, 0x0000, 0x0000, 0x0002, /*07BE*/
  0xFBC0, 0x0020, 0x0002, 0x87BF, 0x0000, 0x0000, 0x0002, /*07BF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07C0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07C1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07C2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07C3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07C4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07C5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07C6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07C7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07C8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07C9*/
  0x243D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07CA*/
  0x243E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07CB*/
  0x243F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07CC*/
  0x2440, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07CD*/
  0x2441, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07CE*/
  0x2442, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07CF*/
  0x2443, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07D0*/
  0x2444, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07D1*/
  0x2445, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07D2*/
  0x2446, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07D3*/
  0x2447, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07D4*/
  0x2448, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07D5*/
  0x2449, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07D6*/
  0x244A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07D7*/
  0x244B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07D8*/
  0x244C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07D9*/
  0x244D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07DA*/
  0x244E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07DB*/
  0x244F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07DC*/
  0x2450, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07DD*/
  0x2451, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07DE*/
  0x2452, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07DF*/
  0x2453, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07E0*/
  0x2454, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07E1*/
  0x2455, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07E2*/
  0x2456, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07E3*/
  0x2457, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07E4*/
  0x2458, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07E5*/
  0x2459, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07E6*/
  0x245A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07E7*/
  0x2449, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*07E8*/
  0x244A, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*07E9*/
  0x244C, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*07EA*/
  0x0000, 0x00AA, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07EB*/
  0x0000, 0x00AB, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07EC*/
  0x0000, 0x00AC, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07ED*/
  0x0000, 0x00AD, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07EE*/
  0x0000, 0x00AE, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07EF*/
  0x0000, 0x00AF, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07F0*/
  0x0000, 0x00B0, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07F1*/
  0x0000, 0x00B1, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07F2*/
  0x0000, 0x00B2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07F3*/
  0x245B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07F4*/
  0x245C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07F5*/
  0x0512, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07F6*/
  0x02C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07F7*/
  0x022B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07F8*/
  0x0263, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*07F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*07FA*/
  0xFBC0, 0x0020, 0x0002, 0x87FB, 0x0000, 0x0000, 0x0002, /*07FB*/
  0xFBC0, 0x0020, 0x0002, 0x87FC, 0x0000, 0x0000, 0x0002, /*07FC*/
  0xFBC0, 0x0020, 0x0002, 0x87FD, 0x0000, 0x0000, 0x0002, /*07FD*/
  0xFBC0, 0x0020, 0x0002, 0x87FE, 0x0000, 0x0000, 0x0002, /*07FE*/
  0xFBC0, 0x0020, 0x0002, 0x87FF, 0x0000, 0x0000, 0x0002  /*07FF*/
};

uint16 uca900_p008[]= {
  0x22E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0800*/
  0x22E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0801*/
  0x22E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0802*/
  0x22E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0803*/
  0x22E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0804*/
  0x22E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0805*/
  0x22E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0806*/
  0x22EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0807*/
  0x22EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0808*/
  0x22EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0809*/
  0x22ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*080A*/
  0x22EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*080B*/
  0x22EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*080C*/
  0x22F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*080D*/
  0x22F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*080E*/
  0x22F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*080F*/
  0x22F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0810*/
  0x22F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0811*/
  0x22F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0812*/
  0x22F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0813*/
  0x22F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0814*/
  0x22F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0815*/
  0x22F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0816*/
  0x22FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0817*/
  0x0000, 0x006A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0818*/
  0x0000, 0x006B, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0819*/
  0x22FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*081A*/
  0x22FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*081B*/
  0x0000, 0x0062, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*081C*/
  0x0000, 0x0062, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*081D*/
  0x0000, 0x0063, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*081E*/
  0x0000, 0x0063, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*081F*/
  0x0000, 0x0063, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0820*/
  0x0000, 0x0064, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0821*/
  0x0000, 0x0064, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0822*/
  0x0000, 0x0064, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0823*/
  0x0000, 0x0065, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0824*/
  0x0000, 0x0065, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0825*/
  0x0000, 0x0066, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0826*/
  0x0000, 0x0066, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0827*/
  0x0000, 0x0067, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0828*/
  0x0000, 0x0067, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0829*/
  0x0000, 0x0067, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*082A*/
  0x0000, 0x0068, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*082B*/
  0x0000, 0x0069, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*082C*/
  0x0000, 0x006C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*082D*/
  0xFBC0, 0x0020, 0x0002, 0x882E, 0x0000, 0x0000, 0x0002, /*082E*/
  0xFBC0, 0x0020, 0x0002, 0x882F, 0x0000, 0x0000, 0x0002, /*082F*/
  0x0242, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0830*/
  0x0243, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0831*/
  0x0244, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0832*/
  0x0245, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0833*/
  0x0246, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0834*/
  0x0247, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0835*/
  0x0248, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0836*/
  0x0249, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0837*/
  0x024A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0838*/
  0x024B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0839*/
  0x024C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*083A*/
  0x024D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*083B*/
  0x024E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*083C*/
  0x024F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*083D*/
  0x0250, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*083E*/
  0xFBC0, 0x0020, 0x0002, 0x883F, 0x0000, 0x0000, 0x0002, /*083F*/
  0x23F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0840*/
  0x23F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0841*/
  0x23F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0842*/
  0x23F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0843*/
  0x23F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0844*/
  0x23F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0845*/
  0x23F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0846*/
  0x23F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0847*/
  0x23FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0848*/
  0x23FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0849*/
  0x23FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*084A*/
  0x23FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*084B*/
  0x23FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*084C*/
  0x23FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*084D*/
  0x2400, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*084E*/
  0x2401, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*084F*/
  0x2402, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0850*/
  0x2403, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0851*/
  0x2404, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0852*/
  0x2405, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0853*/
  0x2406, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0854*/
  0x2407, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0855*/
  0x2408, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0856*/
  0x2409, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0857*/
  0x240A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0858*/
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0859*/
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*085A*/
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*085B*/
  0xFBC0, 0x0020, 0x0002, 0x885C, 0x0000, 0x0000, 0x0002, /*085C*/
  0xFBC0, 0x0020, 0x0002, 0x885D, 0x0000, 0x0000, 0x0002, /*085D*/
  0x03D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*085E*/
  0xFBC0, 0x0020, 0x0002, 0x885F, 0x0000, 0x0000, 0x0002, /*085F*/
  0xFBC0, 0x0020, 0x0002, 0x8860, 0x0000, 0x0000, 0x0002, /*0860*/
  0xFBC0, 0x0020, 0x0002, 0x8861, 0x0000, 0x0000, 0x0002, /*0861*/
  0xFBC0, 0x0020, 0x0002, 0x8862, 0x0000, 0x0000, 0x0002, /*0862*/
  0xFBC0, 0x0020, 0x0002, 0x8863, 0x0000, 0x0000, 0x0002, /*0863*/
  0xFBC0, 0x0020, 0x0002, 0x8864, 0x0000, 0x0000, 0x0002, /*0864*/
  0xFBC0, 0x0020, 0x0002, 0x8865, 0x0000, 0x0000, 0x0002, /*0865*/
  0xFBC0, 0x0020, 0x0002, 0x8866, 0x0000, 0x0000, 0x0002, /*0866*/
  0xFBC0, 0x0020, 0x0002, 0x8867, 0x0000, 0x0000, 0x0002, /*0867*/
  0xFBC0, 0x0020, 0x0002, 0x8868, 0x0000, 0x0000, 0x0002, /*0868*/
  0xFBC0, 0x0020, 0x0002, 0x8869, 0x0000, 0x0000, 0x0002, /*0869*/
  0xFBC0, 0x0020, 0x0002, 0x886A, 0x0000, 0x0000, 0x0002, /*086A*/
  0xFBC0, 0x0020, 0x0002, 0x886B, 0x0000, 0x0000, 0x0002, /*086B*/
  0xFBC0, 0x0020, 0x0002, 0x886C, 0x0000, 0x0000, 0x0002, /*086C*/
  0xFBC0, 0x0020, 0x0002, 0x886D, 0x0000, 0x0000, 0x0002, /*086D*/
  0xFBC0, 0x0020, 0x0002, 0x886E, 0x0000, 0x0000, 0x0002, /*086E*/
  0xFBC0, 0x0020, 0x0002, 0x886F, 0x0000, 0x0000, 0x0002, /*086F*/
  0xFBC0, 0x0020, 0x0002, 0x8870, 0x0000, 0x0000, 0x0002, /*0870*/
  0xFBC0, 0x0020, 0x0002, 0x8871, 0x0000, 0x0000, 0x0002, /*0871*/
  0xFBC0, 0x0020, 0x0002, 0x8872, 0x0000, 0x0000, 0x0002, /*0872*/
  0xFBC0, 0x0020, 0x0002, 0x8873, 0x0000, 0x0000, 0x0002, /*0873*/
  0xFBC0, 0x0020, 0x0002, 0x8874, 0x0000, 0x0000, 0x0002, /*0874*/
  0xFBC0, 0x0020, 0x0002, 0x8875, 0x0000, 0x0000, 0x0002, /*0875*/
  0xFBC0, 0x0020, 0x0002, 0x8876, 0x0000, 0x0000, 0x0002, /*0876*/
  0xFBC0, 0x0020, 0x0002, 0x8877, 0x0000, 0x0000, 0x0002, /*0877*/
  0xFBC0, 0x0020, 0x0002, 0x8878, 0x0000, 0x0000, 0x0002, /*0878*/
  0xFBC0, 0x0020, 0x0002, 0x8879, 0x0000, 0x0000, 0x0002, /*0879*/
  0xFBC0, 0x0020, 0x0002, 0x887A, 0x0000, 0x0000, 0x0002, /*087A*/
  0xFBC0, 0x0020, 0x0002, 0x887B, 0x0000, 0x0000, 0x0002, /*087B*/
  0xFBC0, 0x0020, 0x0002, 0x887C, 0x0000, 0x0000, 0x0002, /*087C*/
  0xFBC0, 0x0020, 0x0002, 0x887D, 0x0000, 0x0000, 0x0002, /*087D*/
  0xFBC0, 0x0020, 0x0002, 0x887E, 0x0000, 0x0000, 0x0002, /*087E*/
  0xFBC0, 0x0020, 0x0002, 0x887F, 0x0000, 0x0000, 0x0002, /*087F*/
  0xFBC0, 0x0020, 0x0002, 0x8880, 0x0000, 0x0000, 0x0002, /*0880*/
  0xFBC0, 0x0020, 0x0002, 0x8881, 0x0000, 0x0000, 0x0002, /*0881*/
  0xFBC0, 0x0020, 0x0002, 0x8882, 0x0000, 0x0000, 0x0002, /*0882*/
  0xFBC0, 0x0020, 0x0002, 0x8883, 0x0000, 0x0000, 0x0002, /*0883*/
  0xFBC0, 0x0020, 0x0002, 0x8884, 0x0000, 0x0000, 0x0002, /*0884*/
  0xFBC0, 0x0020, 0x0002, 0x8885, 0x0000, 0x0000, 0x0002, /*0885*/
  0xFBC0, 0x0020, 0x0002, 0x8886, 0x0000, 0x0000, 0x0002, /*0886*/
  0xFBC0, 0x0020, 0x0002, 0x8887, 0x0000, 0x0000, 0x0002, /*0887*/
  0xFBC0, 0x0020, 0x0002, 0x8888, 0x0000, 0x0000, 0x0002, /*0888*/
  0xFBC0, 0x0020, 0x0002, 0x8889, 0x0000, 0x0000, 0x0002, /*0889*/
  0xFBC0, 0x0020, 0x0002, 0x888A, 0x0000, 0x0000, 0x0002, /*088A*/
  0xFBC0, 0x0020, 0x0002, 0x888B, 0x0000, 0x0000, 0x0002, /*088B*/
  0xFBC0, 0x0020, 0x0002, 0x888C, 0x0000, 0x0000, 0x0002, /*088C*/
  0xFBC0, 0x0020, 0x0002, 0x888D, 0x0000, 0x0000, 0x0002, /*088D*/
  0xFBC0, 0x0020, 0x0002, 0x888E, 0x0000, 0x0000, 0x0002, /*088E*/
  0xFBC0, 0x0020, 0x0002, 0x888F, 0x0000, 0x0000, 0x0002, /*088F*/
  0xFBC0, 0x0020, 0x0002, 0x8890, 0x0000, 0x0000, 0x0002, /*0890*/
  0xFBC0, 0x0020, 0x0002, 0x8891, 0x0000, 0x0000, 0x0002, /*0891*/
  0xFBC0, 0x0020, 0x0002, 0x8892, 0x0000, 0x0000, 0x0002, /*0892*/
  0xFBC0, 0x0020, 0x0002, 0x8893, 0x0000, 0x0000, 0x0002, /*0893*/
  0xFBC0, 0x0020, 0x0002, 0x8894, 0x0000, 0x0000, 0x0002, /*0894*/
  0xFBC0, 0x0020, 0x0002, 0x8895, 0x0000, 0x0000, 0x0002, /*0895*/
  0xFBC0, 0x0020, 0x0002, 0x8896, 0x0000, 0x0000, 0x0002, /*0896*/
  0xFBC0, 0x0020, 0x0002, 0x8897, 0x0000, 0x0000, 0x0002, /*0897*/
  0xFBC0, 0x0020, 0x0002, 0x8898, 0x0000, 0x0000, 0x0002, /*0898*/
  0xFBC0, 0x0020, 0x0002, 0x8899, 0x0000, 0x0000, 0x0002, /*0899*/
  0xFBC0, 0x0020, 0x0002, 0x889A, 0x0000, 0x0000, 0x0002, /*089A*/
  0xFBC0, 0x0020, 0x0002, 0x889B, 0x0000, 0x0000, 0x0002, /*089B*/
  0xFBC0, 0x0020, 0x0002, 0x889C, 0x0000, 0x0000, 0x0002, /*089C*/
  0xFBC0, 0x0020, 0x0002, 0x889D, 0x0000, 0x0000, 0x0002, /*089D*/
  0xFBC0, 0x0020, 0x0002, 0x889E, 0x0000, 0x0000, 0x0002, /*089E*/
  0xFBC0, 0x0020, 0x0002, 0x889F, 0x0000, 0x0000, 0x0002, /*089F*/
  0x2317, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08A0*/
  0x2319, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08A1*/
  0x232B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08A2*/
  0x236D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08A3*/
  0x237C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08A4*/
  0x2386, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08A5*/
  0x23A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08A6*/
  0x23A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08A7*/
  0x2308, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08A8*/
  0x2309, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08A9*/
  0x2356, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08AA*/
  0x23C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08AB*/
  0x230A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08AC*/
  0x230B, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*08AD*/
  0x233F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08AE*/
  0x2367, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08AF*/
  0x2391, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08B0*/
  0x23C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08B1*/
  0x2357, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08B2*/
  0x2375, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08B3*/
  0x238F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08B4*/
  0xFBC0, 0x0020, 0x0002, 0x88B5, 0x0000, 0x0000, 0x0002, /*08B5*/
  0x231A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08B6*/
  0x231B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08B7*/
  0x2324, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08B8*/
  0x2358, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08B9*/
  0x23D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08BA*/
  0x2379, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08BB*/
  0x2384, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08BC*/
  0x23A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08BD*/
  0xFBC0, 0x0020, 0x0002, 0x88BE, 0x0000, 0x0000, 0x0002, /*08BE*/
  0xFBC0, 0x0020, 0x0002, 0x88BF, 0x0000, 0x0000, 0x0002, /*08BF*/
  0xFBC0, 0x0020, 0x0002, 0x88C0, 0x0000, 0x0000, 0x0002, /*08C0*/
  0xFBC0, 0x0020, 0x0002, 0x88C1, 0x0000, 0x0000, 0x0002, /*08C1*/
  0xFBC0, 0x0020, 0x0002, 0x88C2, 0x0000, 0x0000, 0x0002, /*08C2*/
  0xFBC0, 0x0020, 0x0002, 0x88C3, 0x0000, 0x0000, 0x0002, /*08C3*/
  0xFBC0, 0x0020, 0x0002, 0x88C4, 0x0000, 0x0000, 0x0002, /*08C4*/
  0xFBC0, 0x0020, 0x0002, 0x88C5, 0x0000, 0x0000, 0x0002, /*08C5*/
  0xFBC0, 0x0020, 0x0002, 0x88C6, 0x0000, 0x0000, 0x0002, /*08C6*/
  0xFBC0, 0x0020, 0x0002, 0x88C7, 0x0000, 0x0000, 0x0002, /*08C7*/
  0xFBC0, 0x0020, 0x0002, 0x88C8, 0x0000, 0x0000, 0x0002, /*08C8*/
  0xFBC0, 0x0020, 0x0002, 0x88C9, 0x0000, 0x0000, 0x0002, /*08C9*/
  0xFBC0, 0x0020, 0x0002, 0x88CA, 0x0000, 0x0000, 0x0002, /*08CA*/
  0xFBC0, 0x0020, 0x0002, 0x88CB, 0x0000, 0x0000, 0x0002, /*08CB*/
  0xFBC0, 0x0020, 0x0002, 0x88CC, 0x0000, 0x0000, 0x0002, /*08CC*/
  0xFBC0, 0x0020, 0x0002, 0x88CD, 0x0000, 0x0000, 0x0002, /*08CD*/
  0xFBC0, 0x0020, 0x0002, 0x88CE, 0x0000, 0x0000, 0x0002, /*08CE*/
  0xFBC0, 0x0020, 0x0002, 0x88CF, 0x0000, 0x0000, 0x0002, /*08CF*/
  0xFBC0, 0x0020, 0x0002, 0x88D0, 0x0000, 0x0000, 0x0002, /*08D0*/
  0xFBC0, 0x0020, 0x0002, 0x88D1, 0x0000, 0x0000, 0x0002, /*08D1*/
  0xFBC0, 0x0020, 0x0002, 0x88D2, 0x0000, 0x0000, 0x0002, /*08D2*/
  0xFBC0, 0x0020, 0x0002, 0x88D3, 0x0000, 0x0000, 0x0002, /*08D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08E2*/
  0x0000, 0x0090, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08E3*/
  0x0000, 0x0077, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08E4*/
  0x0000, 0x007B, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08E5*/
  0x0000, 0x007E, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08E6*/
  0x0000, 0x006F, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08E7*/
  0x0000, 0x0072, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08E8*/
  0x0000, 0x0075, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08EF*/
  0x0000, 0x006E, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08F0*/
  0x0000, 0x0071, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08F1*/
  0x0000, 0x0074, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*08F3*/
  0x0000, 0x0078, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08F4*/
  0x0000, 0x0079, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08F5*/
  0x0000, 0x007F, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08F6*/
  0x0000, 0x0091, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08F7*/
  0x0000, 0x0092, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08F8*/
  0x0000, 0x0096, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08F9*/
  0x0000, 0x0097, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08FA*/
  0x0000, 0x0094, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08FB*/
  0x0000, 0x0095, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08FC*/
  0x0000, 0x0093, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08FD*/
  0x0000, 0x007C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*08FE*/
  0x0000, 0x0089, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*08FF*/
};

uint16 uca900_p009[]= {
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0900*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0901*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0902*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0903*/
  0x265E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0904*/
  0x265F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0905*/
  0x2660, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0906*/
  0x2666, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0907*/
  0x2667, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0908*/
  0x2668, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0909*/
  0x2669, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*090A*/
  0x266A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*090B*/
  0x266C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*090C*/
  0x266E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*090D*/
  0x266F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*090E*/
  0x2670, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*090F*/
  0x2671, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0910*/
  0x2672, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0911*/
  0x2673, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0912*/
  0x2674, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0913*/
  0x2675, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0914*/
  0x2676, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0915*/
  0x2677, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0916*/
  0x2678, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0917*/
  0x267A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0918*/
  0x267B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0919*/
  0x267C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*091A*/
  0x267D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*091B*/
  0x267E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*091C*/
  0x2681, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*091D*/
  0x2682, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*091E*/
  0x2683, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*091F*/
  0x2684, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0920*/
  0x2686, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0921*/
  0x2688, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0922*/
  0x2689, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0923*/
  0x268A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0924*/
  0x268B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0925*/
  0x268C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0926*/
  0x268D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0927*/
  0x268E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0928*/
  0x268E, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0929*/
  0x268F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*092A*/
  0x2690, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*092B*/
  0x2691, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*092C*/
  0x2693, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*092D*/
  0x2694, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*092E*/
  0x2695, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*092F*/
  0x2697, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0930*/
  0x2697, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0931*/
  0x2698, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0932*/
  0x2699, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0933*/
  0x2699, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0934*/
  0x269A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0935*/
  0x269B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0936*/
  0x269C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0937*/
  0x269D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0938*/
  0x269E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0939*/
  0x26A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*093A*/
  0x26A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*093B*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*093C*/
  0x269F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*093D*/
  0x26A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*093E*/
  0x26AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*093F*/
  0x26AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0940*/
  0x26AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0941*/
  0x26AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0942*/
  0x26B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0943*/
  0x26B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0944*/
  0x26B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0945*/
  0x26B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0946*/
  0x26B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0947*/
  0x26B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0948*/
  0x26BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0949*/
  0x26BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*094A*/
  0x26BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*094B*/
  0x26BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*094C*/
  0x26BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*094D*/
  0x26B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*094E*/
  0x26A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*094F*/
  0x265B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0950*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0951*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*0952*/
  0x0000, 0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0953*/
  0x0000, 0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0954*/
  0x26B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0955*/
  0x26AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0956*/
  0x26AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0957*/
  0x2676, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0958*/
  0x2677, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0959*/
  0x2678, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*095A*/
  0x267E, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*095B*/
  0x2686, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*095C*/
  0x2688, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*095D*/
  0x2690, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*095E*/
  0x2695, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*095F*/
  0x266B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0960*/
  0x266D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0961*/
  0x26B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0962*/
  0x26B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0963*/
  0x028E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0964*/
  0x028F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0965*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0966*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0967*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0968*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0969*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*096A*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*096B*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*096C*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*096D*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*096E*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*096F*/
  0x03E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0970*/
  0x1BFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0971*/
  0x265D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0972*/
  0x2661, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0973*/
  0x2662, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0974*/
  0x2663, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0975*/
  0x2664, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0976*/
  0x2665, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0977*/
  0x2685, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0978*/
  0x267F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0979*/
  0x2696, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*097A*/
  0x2679, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*097B*/
  0x2680, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*097C*/
  0x26A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*097D*/
  0x2687, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*097E*/
  0x2692, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*097F*/
  0x26BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0980*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0981*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0982*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0983*/
  0xFBC0, 0x0020, 0x0002, 0x8984, 0x0000, 0x0000, 0x0002, /*0984*/
  0x26C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0985*/
  0x26C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0986*/
  0x26C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0987*/
  0x26C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0988*/
  0x26C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0989*/
  0x26C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*098A*/
  0x26C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*098B*/
  0x26C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*098C*/
  0xFBC0, 0x0020, 0x0002, 0x898D, 0x0000, 0x0000, 0x0002, /*098D*/
  0xFBC0, 0x0020, 0x0002, 0x898E, 0x0000, 0x0000, 0x0002, /*098E*/
  0x26CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*098F*/
  0x26CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0990*/
  0xFBC0, 0x0020, 0x0002, 0x8991, 0x0000, 0x0000, 0x0002, /*0991*/
  0xFBC0, 0x0020, 0x0002, 0x8992, 0x0000, 0x0000, 0x0002, /*0992*/
  0x26CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0993*/
  0x26CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0994*/
  0x26CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0995*/
  0x26CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0996*/
  0x26D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0997*/
  0x26D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0998*/
  0x26D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0999*/
  0x26D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*099A*/
  0x26D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*099B*/
  0x26D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*099C*/
  0x26D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*099D*/
  0x26D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*099E*/
  0x26D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*099F*/
  0x26D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09A0*/
  0x26DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09A1*/
  0x26DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09A2*/
  0x26DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09A3*/
  0x26DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09A4*/
  0x26DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09A5*/
  0x26DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09A6*/
  0x26E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09A7*/
  0x26E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09A8*/
  0xFBC0, 0x0020, 0x0002, 0x89A9, 0x0000, 0x0000, 0x0002, /*09A9*/
  0x26E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09AA*/
  0x26E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09AB*/
  0x26E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09AC*/
  0x26E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09AD*/
  0x26E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09AE*/
  0x26E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09AF*/
  0x26E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09B0*/
  0xFBC0, 0x0020, 0x0002, 0x89B1, 0x0000, 0x0000, 0x0002, /*09B1*/
  0x26EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09B2*/
  0xFBC0, 0x0020, 0x0002, 0x89B3, 0x0000, 0x0000, 0x0002, /*09B3*/
  0xFBC0, 0x0020, 0x0002, 0x89B4, 0x0000, 0x0000, 0x0002, /*09B4*/
  0xFBC0, 0x0020, 0x0002, 0x89B5, 0x0000, 0x0000, 0x0002, /*09B5*/
  0x26EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09B6*/
  0x26ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09B7*/
  0x26EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09B8*/
  0x26EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09B9*/
  0xFBC0, 0x0020, 0x0002, 0x89BA, 0x0000, 0x0000, 0x0002, /*09BA*/
  0xFBC0, 0x0020, 0x0002, 0x89BB, 0x0000, 0x0000, 0x0002, /*09BB*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09BC*/
  0x26F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09BD*/
  0x26F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09BE*/
  0x26F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09BF*/
  0x26F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09C0*/
  0x26F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09C1*/
  0x26F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09C2*/
  0x26F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09C3*/
  0x26F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09C4*/
  0xFBC0, 0x0020, 0x0002, 0x89C5, 0x0000, 0x0000, 0x0002, /*09C5*/
  0xFBC0, 0x0020, 0x0002, 0x89C6, 0x0000, 0x0000, 0x0002, /*09C6*/
  0x26FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09C7*/
  0x26FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09C8*/
  0xFBC0, 0x0020, 0x0002, 0x89C9, 0x0000, 0x0000, 0x0002, /*09C9*/
  0xFBC0, 0x0020, 0x0002, 0x89CA, 0x0000, 0x0000, 0x0002, /*09CA*/
  0x26FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09CB*/
  0x26FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09CC*/
  0x26FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09CD*/
  0x26DD, 0x0020, 0x0004, 0x26FE, 0x0020, 0x0004, 0x0002, /*09CE*/
  0xFBC0, 0x0020, 0x0002, 0x89CF, 0x0000, 0x0000, 0x0002, /*09CF*/
  0xFBC0, 0x0020, 0x0002, 0x89D0, 0x0000, 0x0000, 0x0002, /*09D0*/
  0xFBC0, 0x0020, 0x0002, 0x89D1, 0x0000, 0x0000, 0x0002, /*09D1*/
  0xFBC0, 0x0020, 0x0002, 0x89D2, 0x0000, 0x0000, 0x0002, /*09D2*/
  0xFBC0, 0x0020, 0x0002, 0x89D3, 0x0000, 0x0000, 0x0002, /*09D3*/
  0xFBC0, 0x0020, 0x0002, 0x89D4, 0x0000, 0x0000, 0x0002, /*09D4*/
  0xFBC0, 0x0020, 0x0002, 0x89D5, 0x0000, 0x0000, 0x0002, /*09D5*/
  0xFBC0, 0x0020, 0x0002, 0x89D6, 0x0000, 0x0000, 0x0002, /*09D6*/
  0x26FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09D7*/
  0xFBC0, 0x0020, 0x0002, 0x89D8, 0x0000, 0x0000, 0x0002, /*09D8*/
  0xFBC0, 0x0020, 0x0002, 0x89D9, 0x0000, 0x0000, 0x0002, /*09D9*/
  0xFBC0, 0x0020, 0x0002, 0x89DA, 0x0000, 0x0000, 0x0002, /*09DA*/
  0xFBC0, 0x0020, 0x0002, 0x89DB, 0x0000, 0x0000, 0x0002, /*09DB*/
  0x26DA, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*09DC*/
  0x26DB, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*09DD*/
  0xFBC0, 0x0020, 0x0002, 0x89DE, 0x0000, 0x0000, 0x0002, /*09DE*/
  0x26E7, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*09DF*/
  0x26C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09E0*/
  0x26C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09E1*/
  0x26F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09E2*/
  0x26F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09E3*/
  0xFBC0, 0x0020, 0x0002, 0x89E4, 0x0000, 0x0000, 0x0002, /*09E4*/
  0xFBC0, 0x0020, 0x0002, 0x89E5, 0x0000, 0x0000, 0x0002, /*09E5*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09E6*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09E7*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09E8*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09E9*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09EA*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09EB*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09EC*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09ED*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09EE*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09EF*/
  0x26E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09F0*/
  0x26EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09F1*/
  0x1C17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09F2*/
  0x1C18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09F3*/
  0x1A96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09F4*/
  0x1A97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09F5*/
  0x1A98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09F6*/
  0x1A99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09F7*/
  0x1A9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09F8*/
  0x1A9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09F9*/
  0x0513, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09FA*/
  0x1C19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*09FB*/
  0xFBC0, 0x0020, 0x0002, 0x89FC, 0x0000, 0x0000, 0x0002, /*09FC*/
  0xFBC0, 0x0020, 0x0002, 0x89FD, 0x0000, 0x0000, 0x0002, /*09FD*/
  0xFBC0, 0x0020, 0x0002, 0x89FE, 0x0000, 0x0000, 0x0002, /*09FE*/
  0xFBC0, 0x0020, 0x0002, 0x89FF, 0x0000, 0x0000, 0x0002  /*09FF*/
};

uint16 uca900_p00A[]= {
  0xFBC0, 0x0020, 0x0002, 0x8A00, 0x0000, 0x0000, 0x0002, /*0A00*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A01*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A02*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A03*/
  0xFBC0, 0x0020, 0x0002, 0x8A04, 0x0000, 0x0000, 0x0002, /*0A04*/
  0x2705, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A05*/
  0x2706, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A06*/
  0x270A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A07*/
  0x270B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A08*/
  0x2702, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A09*/
  0x2703, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A0A*/
  0xFBC0, 0x0020, 0x0002, 0x8A0B, 0x0000, 0x0000, 0x0002, /*0A0B*/
  0xFBC0, 0x0020, 0x0002, 0x8A0C, 0x0000, 0x0000, 0x0002, /*0A0C*/
  0xFBC0, 0x0020, 0x0002, 0x8A0D, 0x0000, 0x0000, 0x0002, /*0A0D*/
  0xFBC0, 0x0020, 0x0002, 0x8A0E, 0x0000, 0x0000, 0x0002, /*0A0E*/
  0x270C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A0F*/
  0x2707, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A10*/
  0xFBC0, 0x0020, 0x0002, 0x8A11, 0x0000, 0x0000, 0x0002, /*0A11*/
  0xFBC0, 0x0020, 0x0002, 0x8A12, 0x0000, 0x0000, 0x0002, /*0A12*/
  0x2704, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A13*/
  0x2708, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A14*/
  0x2710, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A15*/
  0x2711, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A16*/
  0x2712, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A17*/
  0x2713, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A18*/
  0x2714, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A19*/
  0x2715, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A1A*/
  0x2716, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A1B*/
  0x2717, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A1C*/
  0x2718, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A1D*/
  0x2719, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A1E*/
  0x271A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A1F*/
  0x271B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A20*/
  0x271C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A21*/
  0x271D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A22*/
  0x271E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A23*/
  0x271F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A24*/
  0x2720, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A25*/
  0x2721, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A26*/
  0x2722, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A27*/
  0x2723, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A28*/
  0xFBC0, 0x0020, 0x0002, 0x8A29, 0x0000, 0x0000, 0x0002, /*0A29*/
  0x2724, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A2A*/
  0x2725, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A2B*/
  0x2726, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A2C*/
  0x2727, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A2D*/
  0x2728, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A2E*/
  0x2729, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A2F*/
  0x272B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A30*/
  0xFBC0, 0x0020, 0x0002, 0x8A31, 0x0000, 0x0000, 0x0002, /*0A31*/
  0x272C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A32*/
  0x272C, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0A33*/
  0xFBC0, 0x0020, 0x0002, 0x8A34, 0x0000, 0x0000, 0x0002, /*0A34*/
  0x272D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A35*/
  0x270D, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0A36*/
  0xFBC0, 0x0020, 0x0002, 0x8A37, 0x0000, 0x0000, 0x0002, /*0A37*/
  0x270D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A38*/
  0x270E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A39*/
  0xFBC0, 0x0020, 0x0002, 0x8A3A, 0x0000, 0x0000, 0x0002, /*0A3A*/
  0xFBC0, 0x0020, 0x0002, 0x8A3B, 0x0000, 0x0000, 0x0002, /*0A3B*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A3C*/
  0xFBC0, 0x0020, 0x0002, 0x8A3D, 0x0000, 0x0000, 0x0002, /*0A3D*/
  0x272F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A3E*/
  0x2730, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A3F*/
  0x2731, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A40*/
  0x2732, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A41*/
  0x2733, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A42*/
  0xFBC0, 0x0020, 0x0002, 0x8A43, 0x0000, 0x0000, 0x0002, /*0A43*/
  0xFBC0, 0x0020, 0x0002, 0x8A44, 0x0000, 0x0000, 0x0002, /*0A44*/
  0xFBC0, 0x0020, 0x0002, 0x8A45, 0x0000, 0x0000, 0x0002, /*0A45*/
  0xFBC0, 0x0020, 0x0002, 0x8A46, 0x0000, 0x0000, 0x0002, /*0A46*/
  0x2734, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A47*/
  0x2735, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A48*/
  0xFBC0, 0x0020, 0x0002, 0x8A49, 0x0000, 0x0000, 0x0002, /*0A49*/
  0xFBC0, 0x0020, 0x0002, 0x8A4A, 0x0000, 0x0000, 0x0002, /*0A4A*/
  0x2736, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A4B*/
  0x2737, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A4C*/
  0x2738, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A4D*/
  0xFBC0, 0x0020, 0x0002, 0x8A4E, 0x0000, 0x0000, 0x0002, /*0A4E*/
  0xFBC0, 0x0020, 0x0002, 0x8A4F, 0x0000, 0x0000, 0x0002, /*0A4F*/
  0xFBC0, 0x0020, 0x0002, 0x8A50, 0x0000, 0x0000, 0x0002, /*0A50*/
  0x270F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A51*/
  0xFBC0, 0x0020, 0x0002, 0x8A52, 0x0000, 0x0000, 0x0002, /*0A52*/
  0xFBC0, 0x0020, 0x0002, 0x8A53, 0x0000, 0x0000, 0x0002, /*0A53*/
  0xFBC0, 0x0020, 0x0002, 0x8A54, 0x0000, 0x0000, 0x0002, /*0A54*/
  0xFBC0, 0x0020, 0x0002, 0x8A55, 0x0000, 0x0000, 0x0002, /*0A55*/
  0xFBC0, 0x0020, 0x0002, 0x8A56, 0x0000, 0x0000, 0x0002, /*0A56*/
  0xFBC0, 0x0020, 0x0002, 0x8A57, 0x0000, 0x0000, 0x0002, /*0A57*/
  0xFBC0, 0x0020, 0x0002, 0x8A58, 0x0000, 0x0000, 0x0002, /*0A58*/
  0x2711, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0A59*/
  0x2712, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0A5A*/
  0x2717, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0A5B*/
  0x272E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A5C*/
  0xFBC0, 0x0020, 0x0002, 0x8A5D, 0x0000, 0x0000, 0x0002, /*0A5D*/
  0x2725, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0A5E*/
  0xFBC0, 0x0020, 0x0002, 0x8A5F, 0x0000, 0x0000, 0x0002, /*0A5F*/
  0xFBC0, 0x0020, 0x0002, 0x8A60, 0x0000, 0x0000, 0x0002, /*0A60*/
  0xFBC0, 0x0020, 0x0002, 0x8A61, 0x0000, 0x0000, 0x0002, /*0A61*/
  0xFBC0, 0x0020, 0x0002, 0x8A62, 0x0000, 0x0000, 0x0002, /*0A62*/
  0xFBC0, 0x0020, 0x0002, 0x8A63, 0x0000, 0x0000, 0x0002, /*0A63*/
  0xFBC0, 0x0020, 0x0002, 0x8A64, 0x0000, 0x0000, 0x0002, /*0A64*/
  0xFBC0, 0x0020, 0x0002, 0x8A65, 0x0000, 0x0000, 0x0002, /*0A65*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A66*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A67*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A68*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A69*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A6A*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A6B*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A6C*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A6D*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A6E*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A6F*/
  0x0000, 0x00C6, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A70*/
  0x0000, 0x00C7, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A71*/
  0x2709, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A72*/
  0x2701, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A73*/
  0x2700, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A74*/
  0x272A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A75*/
  0xFBC0, 0x0020, 0x0002, 0x8A76, 0x0000, 0x0000, 0x0002, /*0A76*/
  0xFBC0, 0x0020, 0x0002, 0x8A77, 0x0000, 0x0000, 0x0002, /*0A77*/
  0xFBC0, 0x0020, 0x0002, 0x8A78, 0x0000, 0x0000, 0x0002, /*0A78*/
  0xFBC0, 0x0020, 0x0002, 0x8A79, 0x0000, 0x0000, 0x0002, /*0A79*/
  0xFBC0, 0x0020, 0x0002, 0x8A7A, 0x0000, 0x0000, 0x0002, /*0A7A*/
  0xFBC0, 0x0020, 0x0002, 0x8A7B, 0x0000, 0x0000, 0x0002, /*0A7B*/
  0xFBC0, 0x0020, 0x0002, 0x8A7C, 0x0000, 0x0000, 0x0002, /*0A7C*/
  0xFBC0, 0x0020, 0x0002, 0x8A7D, 0x0000, 0x0000, 0x0002, /*0A7D*/
  0xFBC0, 0x0020, 0x0002, 0x8A7E, 0x0000, 0x0000, 0x0002, /*0A7E*/
  0xFBC0, 0x0020, 0x0002, 0x8A7F, 0x0000, 0x0000, 0x0002, /*0A7F*/
  0xFBC0, 0x0020, 0x0002, 0x8A80, 0x0000, 0x0000, 0x0002, /*0A80*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A81*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A82*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A83*/
  0xFBC0, 0x0020, 0x0002, 0x8A84, 0x0000, 0x0000, 0x0002, /*0A84*/
  0x273A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A85*/
  0x273B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A86*/
  0x273C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A87*/
  0x273D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A88*/
  0x273E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A89*/
  0x273F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A8A*/
  0x2740, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A8B*/
  0x2742, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A8C*/
  0x2744, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A8D*/
  0xFBC0, 0x0020, 0x0002, 0x8A8E, 0x0000, 0x0000, 0x0002, /*0A8E*/
  0x2745, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A8F*/
  0x2746, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A90*/
  0x2747, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A91*/
  0xFBC0, 0x0020, 0x0002, 0x8A92, 0x0000, 0x0000, 0x0002, /*0A92*/
  0x2748, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A93*/
  0x2749, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A94*/
  0x274A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A95*/
  0x274B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A96*/
  0x274C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A97*/
  0x274D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A98*/
  0x274E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A99*/
  0x274F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A9A*/
  0x2750, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A9B*/
  0x2751, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A9C*/
  0x2753, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A9D*/
  0x2754, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A9E*/
  0x2755, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0A9F*/
  0x2756, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AA0*/
  0x2757, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AA1*/
  0x2758, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AA2*/
  0x2759, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AA3*/
  0x275A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AA4*/
  0x275B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AA5*/
  0x275C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AA6*/
  0x275D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AA7*/
  0x275E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AA8*/
  0xFBC0, 0x0020, 0x0002, 0x8AA9, 0x0000, 0x0000, 0x0002, /*0AA9*/
  0x275F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AAA*/
  0x2760, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AAB*/
  0x2761, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AAC*/
  0x2762, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AAD*/
  0x2763, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AAE*/
  0x2764, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AAF*/
  0x2765, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AB0*/
  0xFBC0, 0x0020, 0x0002, 0x8AB1, 0x0000, 0x0000, 0x0002, /*0AB1*/
  0x2766, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AB2*/
  0x276C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AB3*/
  0xFBC0, 0x0020, 0x0002, 0x8AB4, 0x0000, 0x0000, 0x0002, /*0AB4*/
  0x2767, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AB5*/
  0x2768, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AB6*/
  0x2769, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AB7*/
  0x276A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AB8*/
  0x276B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AB9*/
  0xFBC0, 0x0020, 0x0002, 0x8ABA, 0x0000, 0x0000, 0x0002, /*0ABA*/
  0xFBC0, 0x0020, 0x0002, 0x8ABB, 0x0000, 0x0000, 0x0002, /*0ABB*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ABC*/
  0x276D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ABD*/
  0x276E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ABE*/
  0x276F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ABF*/
  0x2770, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AC0*/
  0x2771, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AC1*/
  0x2772, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AC2*/
  0x2773, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AC3*/
  0x2774, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AC4*/
  0x2777, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AC5*/
  0xFBC0, 0x0020, 0x0002, 0x8AC6, 0x0000, 0x0000, 0x0002, /*0AC6*/
  0x2778, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AC7*/
  0x2779, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AC8*/
  0x277A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AC9*/
  0xFBC0, 0x0020, 0x0002, 0x8ACA, 0x0000, 0x0000, 0x0002, /*0ACA*/
  0x277B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ACB*/
  0x277C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ACC*/
  0x277D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ACD*/
  0xFBC0, 0x0020, 0x0002, 0x8ACE, 0x0000, 0x0000, 0x0002, /*0ACE*/
  0xFBC0, 0x0020, 0x0002, 0x8ACF, 0x0000, 0x0000, 0x0002, /*0ACF*/
  0x2739, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AD0*/
  0xFBC0, 0x0020, 0x0002, 0x8AD1, 0x0000, 0x0000, 0x0002, /*0AD1*/
  0xFBC0, 0x0020, 0x0002, 0x8AD2, 0x0000, 0x0000, 0x0002, /*0AD2*/
  0xFBC0, 0x0020, 0x0002, 0x8AD3, 0x0000, 0x0000, 0x0002, /*0AD3*/
  0xFBC0, 0x0020, 0x0002, 0x8AD4, 0x0000, 0x0000, 0x0002, /*0AD4*/
  0xFBC0, 0x0020, 0x0002, 0x8AD5, 0x0000, 0x0000, 0x0002, /*0AD5*/
  0xFBC0, 0x0020, 0x0002, 0x8AD6, 0x0000, 0x0000, 0x0002, /*0AD6*/
  0xFBC0, 0x0020, 0x0002, 0x8AD7, 0x0000, 0x0000, 0x0002, /*0AD7*/
  0xFBC0, 0x0020, 0x0002, 0x8AD8, 0x0000, 0x0000, 0x0002, /*0AD8*/
  0xFBC0, 0x0020, 0x0002, 0x8AD9, 0x0000, 0x0000, 0x0002, /*0AD9*/
  0xFBC0, 0x0020, 0x0002, 0x8ADA, 0x0000, 0x0000, 0x0002, /*0ADA*/
  0xFBC0, 0x0020, 0x0002, 0x8ADB, 0x0000, 0x0000, 0x0002, /*0ADB*/
  0xFBC0, 0x0020, 0x0002, 0x8ADC, 0x0000, 0x0000, 0x0002, /*0ADC*/
  0xFBC0, 0x0020, 0x0002, 0x8ADD, 0x0000, 0x0000, 0x0002, /*0ADD*/
  0xFBC0, 0x0020, 0x0002, 0x8ADE, 0x0000, 0x0000, 0x0002, /*0ADE*/
  0xFBC0, 0x0020, 0x0002, 0x8ADF, 0x0000, 0x0000, 0x0002, /*0ADF*/
  0x2741, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AE0*/
  0x2743, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AE1*/
  0x2775, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AE2*/
  0x2776, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AE3*/
  0xFBC0, 0x0020, 0x0002, 0x8AE4, 0x0000, 0x0000, 0x0002, /*0AE4*/
  0xFBC0, 0x0020, 0x0002, 0x8AE5, 0x0000, 0x0000, 0x0002, /*0AE5*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AE6*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AE7*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AE8*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AE9*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AEA*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AEB*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AEC*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AED*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AEE*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AEF*/
  0x03EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AF0*/
  0x1C1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AF1*/
  0xFBC0, 0x0020, 0x0002, 0x8AF2, 0x0000, 0x0000, 0x0002, /*0AF2*/
  0xFBC0, 0x0020, 0x0002, 0x8AF3, 0x0000, 0x0000, 0x0002, /*0AF3*/
  0xFBC0, 0x0020, 0x0002, 0x8AF4, 0x0000, 0x0000, 0x0002, /*0AF4*/
  0xFBC0, 0x0020, 0x0002, 0x8AF5, 0x0000, 0x0000, 0x0002, /*0AF5*/
  0xFBC0, 0x0020, 0x0002, 0x8AF6, 0x0000, 0x0000, 0x0002, /*0AF6*/
  0xFBC0, 0x0020, 0x0002, 0x8AF7, 0x0000, 0x0000, 0x0002, /*0AF7*/
  0xFBC0, 0x0020, 0x0002, 0x8AF8, 0x0000, 0x0000, 0x0002, /*0AF8*/
  0x2752, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0AF9*/
  0xFBC0, 0x0020, 0x0002, 0x8AFA, 0x0000, 0x0000, 0x0002, /*0AFA*/
  0xFBC0, 0x0020, 0x0002, 0x8AFB, 0x0000, 0x0000, 0x0002, /*0AFB*/
  0xFBC0, 0x0020, 0x0002, 0x8AFC, 0x0000, 0x0000, 0x0002, /*0AFC*/
  0xFBC0, 0x0020, 0x0002, 0x8AFD, 0x0000, 0x0000, 0x0002, /*0AFD*/
  0xFBC0, 0x0020, 0x0002, 0x8AFE, 0x0000, 0x0000, 0x0002, /*0AFE*/
  0xFBC0, 0x0020, 0x0002, 0x8AFF, 0x0000, 0x0000, 0x0002  /*0AFF*/
};

uint16 uca900_p00B[]= {
  0xFBC0, 0x0020, 0x0002, 0x8B00, 0x0000, 0x0000, 0x0002, /*0B00*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B01*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B02*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B03*/
  0xFBC0, 0x0020, 0x0002, 0x8B04, 0x0000, 0x0000, 0x0002, /*0B04*/
  0x277E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B05*/
  0x277F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B06*/
  0x2780, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B07*/
  0x2781, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B08*/
  0x2782, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B09*/
  0x2783, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B0A*/
  0x2784, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B0B*/
  0x2786, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B0C*/
  0xFBC0, 0x0020, 0x0002, 0x8B0D, 0x0000, 0x0000, 0x0002, /*0B0D*/
  0xFBC0, 0x0020, 0x0002, 0x8B0E, 0x0000, 0x0000, 0x0002, /*0B0E*/
  0x2788, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B0F*/
  0x2789, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B10*/
  0xFBC0, 0x0020, 0x0002, 0x8B11, 0x0000, 0x0000, 0x0002, /*0B11*/
  0xFBC0, 0x0020, 0x0002, 0x8B12, 0x0000, 0x0000, 0x0002, /*0B12*/
  0x278A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B13*/
  0x278B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B14*/
  0x278C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B15*/
  0x278D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B16*/
  0x278E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B17*/
  0x278F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B18*/
  0x2790, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B19*/
  0x2791, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B1A*/
  0x2792, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B1B*/
  0x2793, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B1C*/
  0x2794, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B1D*/
  0x2795, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B1E*/
  0x2796, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B1F*/
  0x2797, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B20*/
  0x2798, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B21*/
  0x2799, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B22*/
  0x279A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B23*/
  0x279B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B24*/
  0x279C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B25*/
  0x279D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B26*/
  0x279E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B27*/
  0x279F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B28*/
  0xFBC0, 0x0020, 0x0002, 0x8B29, 0x0000, 0x0000, 0x0002, /*0B29*/
  0x27A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B2A*/
  0x27A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B2B*/
  0x27A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B2C*/
  0x27A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B2D*/
  0x27A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B2E*/
  0x27A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B2F*/
  0x27A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B30*/
  0xFBC0, 0x0020, 0x0002, 0x8B31, 0x0000, 0x0000, 0x0002, /*0B31*/
  0x27A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B32*/
  0x27A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B33*/
  0xFBC0, 0x0020, 0x0002, 0x8B34, 0x0000, 0x0000, 0x0002, /*0B34*/
  0x27AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B35*/
  0x27AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B36*/
  0x27AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B37*/
  0x27AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B38*/
  0x27AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B39*/
  0xFBC0, 0x0020, 0x0002, 0x8B3A, 0x0000, 0x0000, 0x0002, /*0B3A*/
  0xFBC0, 0x0020, 0x0002, 0x8B3B, 0x0000, 0x0000, 0x0002, /*0B3B*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B3C*/
  0x27B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B3D*/
  0x27B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B3E*/
  0x27B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B3F*/
  0x27B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B40*/
  0x27B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B41*/
  0x27B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B42*/
  0x27B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B43*/
  0x27B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B44*/
  0xFBC0, 0x0020, 0x0002, 0x8B45, 0x0000, 0x0000, 0x0002, /*0B45*/
  0xFBC0, 0x0020, 0x0002, 0x8B46, 0x0000, 0x0000, 0x0002, /*0B46*/
  0x27BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B47*/
  0x27BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B48*/
  0xFBC0, 0x0020, 0x0002, 0x8B49, 0x0000, 0x0000, 0x0002, /*0B49*/
  0xFBC0, 0x0020, 0x0002, 0x8B4A, 0x0000, 0x0000, 0x0002, /*0B4A*/
  0x27BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B4B*/
  0x27BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B4C*/
  0x27BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B4D*/
  0xFBC0, 0x0020, 0x0002, 0x8B4E, 0x0000, 0x0000, 0x0002, /*0B4E*/
  0xFBC0, 0x0020, 0x0002, 0x8B4F, 0x0000, 0x0000, 0x0002, /*0B4F*/
  0xFBC0, 0x0020, 0x0002, 0x8B50, 0x0000, 0x0000, 0x0002, /*0B50*/
  0xFBC0, 0x0020, 0x0002, 0x8B51, 0x0000, 0x0000, 0x0002, /*0B51*/
  0xFBC0, 0x0020, 0x0002, 0x8B52, 0x0000, 0x0000, 0x0002, /*0B52*/
  0xFBC0, 0x0020, 0x0002, 0x8B53, 0x0000, 0x0000, 0x0002, /*0B53*/
  0xFBC0, 0x0020, 0x0002, 0x8B54, 0x0000, 0x0000, 0x0002, /*0B54*/
  0xFBC0, 0x0020, 0x0002, 0x8B55, 0x0000, 0x0000, 0x0002, /*0B55*/
  0x27BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B56*/
  0x27C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B57*/
  0xFBC0, 0x0020, 0x0002, 0x8B58, 0x0000, 0x0000, 0x0002, /*0B58*/
  0xFBC0, 0x0020, 0x0002, 0x8B59, 0x0000, 0x0000, 0x0002, /*0B59*/
  0xFBC0, 0x0020, 0x0002, 0x8B5A, 0x0000, 0x0000, 0x0002, /*0B5A*/
  0xFBC0, 0x0020, 0x0002, 0x8B5B, 0x0000, 0x0000, 0x0002, /*0B5B*/
  0x2798, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0B5C*/
  0x2799, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*0B5D*/
  0xFBC0, 0x0020, 0x0002, 0x8B5E, 0x0000, 0x0000, 0x0002, /*0B5E*/
  0x27A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B5F*/
  0x2785, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B60*/
  0x2787, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B61*/
  0x27B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B62*/
  0x27B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B63*/
  0xFBC0, 0x0020, 0x0002, 0x8B64, 0x0000, 0x0000, 0x0002, /*0B64*/
  0xFBC0, 0x0020, 0x0002, 0x8B65, 0x0000, 0x0000, 0x0002, /*0B65*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B66*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B67*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B68*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B69*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B6A*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B6B*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B6C*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B6D*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B6E*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B6F*/
  0x0514, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B70*/
  0x27AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B71*/
  0x1A9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B72*/
  0x1A9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B73*/
  0x1A9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B74*/
  0x1A9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B75*/
  0x1AA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B76*/
  0x1AA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B77*/
  0xFBC0, 0x0020, 0x0002, 0x8B78, 0x0000, 0x0000, 0x0002, /*0B78*/
  0xFBC0, 0x0020, 0x0002, 0x8B79, 0x0000, 0x0000, 0x0002, /*0B79*/
  0xFBC0, 0x0020, 0x0002, 0x8B7A, 0x0000, 0x0000, 0x0002, /*0B7A*/
  0xFBC0, 0x0020, 0x0002, 0x8B7B, 0x0000, 0x0000, 0x0002, /*0B7B*/
  0xFBC0, 0x0020, 0x0002, 0x8B7C, 0x0000, 0x0000, 0x0002, /*0B7C*/
  0xFBC0, 0x0020, 0x0002, 0x8B7D, 0x0000, 0x0000, 0x0002, /*0B7D*/
  0xFBC0, 0x0020, 0x0002, 0x8B7E, 0x0000, 0x0000, 0x0002, /*0B7E*/
  0xFBC0, 0x0020, 0x0002, 0x8B7F, 0x0000, 0x0000, 0x0002, /*0B7F*/
  0xFBC0, 0x0020, 0x0002, 0x8B80, 0x0000, 0x0000, 0x0002, /*0B80*/
  0xFBC0, 0x0020, 0x0002, 0x8B81, 0x0000, 0x0000, 0x0002, /*0B81*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B82*/
  0x27CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B83*/
  0xFBC0, 0x0020, 0x0002, 0x8B84, 0x0000, 0x0000, 0x0002, /*0B84*/
  0x27C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B85*/
  0x27C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B86*/
  0x27C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B87*/
  0x27C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B88*/
  0x27C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B89*/
  0x27C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B8A*/
  0xFBC0, 0x0020, 0x0002, 0x8B8B, 0x0000, 0x0000, 0x0002, /*0B8B*/
  0xFBC0, 0x0020, 0x0002, 0x8B8C, 0x0000, 0x0000, 0x0002, /*0B8C*/
  0xFBC0, 0x0020, 0x0002, 0x8B8D, 0x0000, 0x0000, 0x0002, /*0B8D*/
  0x27C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B8E*/
  0x27C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B8F*/
  0x27CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B90*/
  0xFBC0, 0x0020, 0x0002, 0x8B91, 0x0000, 0x0000, 0x0002, /*0B91*/
  0x27CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B92*/
  0x27CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B93*/
  0x27CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B94*/
  0x27CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B95*/
  0xFBC0, 0x0020, 0x0002, 0x8B96, 0x0000, 0x0000, 0x0002, /*0B96*/
  0xFBC0, 0x0020, 0x0002, 0x8B97, 0x0000, 0x0000, 0x0002, /*0B97*/
  0xFBC0, 0x0020, 0x0002, 0x8B98, 0x0000, 0x0000, 0x0002, /*0B98*/
  0x27D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B99*/
  0x27D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B9A*/
  0xFBC0, 0x0020, 0x0002, 0x8B9B, 0x0000, 0x0000, 0x0002, /*0B9B*/
  0x27E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B9C*/
  0xFBC0, 0x0020, 0x0002, 0x8B9D, 0x0000, 0x0000, 0x0002, /*0B9D*/
  0x27D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B9E*/
  0x27D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0B9F*/
  0xFBC0, 0x0020, 0x0002, 0x8BA0, 0x0000, 0x0000, 0x0002, /*0BA0*/
  0xFBC0, 0x0020, 0x0002, 0x8BA1, 0x0000, 0x0000, 0x0002, /*0BA1*/
  0xFBC0, 0x0020, 0x0002, 0x8BA2, 0x0000, 0x0000, 0x0002, /*0BA2*/
  0x27D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BA3*/
  0x27D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BA4*/
  0xFBC0, 0x0020, 0x0002, 0x8BA5, 0x0000, 0x0000, 0x0002, /*0BA5*/
  0xFBC0, 0x0020, 0x0002, 0x8BA6, 0x0000, 0x0000, 0x0002, /*0BA6*/
  0xFBC0, 0x0020, 0x0002, 0x8BA7, 0x0000, 0x0000, 0x0002, /*0BA7*/
  0x27D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BA8*/
  0x27E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BA9*/
  0x27D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BAA*/
  0xFBC0, 0x0020, 0x0002, 0x8BAB, 0x0000, 0x0000, 0x0002, /*0BAB*/
  0xFBC0, 0x0020, 0x0002, 0x8BAC, 0x0000, 0x0000, 0x0002, /*0BAC*/
  0xFBC0, 0x0020, 0x0002, 0x8BAD, 0x0000, 0x0000, 0x0002, /*0BAD*/
  0x27D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BAE*/
  0x27D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BAF*/
  0x27DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BB0*/
  0x27DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BB1*/
  0x27DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BB2*/
  0x27DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BB3*/
  0x27DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BB4*/
  0x27DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BB5*/
  0x27E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BB6*/
  0x27E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BB7*/
  0x27E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BB8*/
  0x27E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BB9*/
  0xFBC0, 0x0020, 0x0002, 0x8BBA, 0x0000, 0x0000, 0x0002, /*0BBA*/
  0xFBC0, 0x0020, 0x0002, 0x8BBB, 0x0000, 0x0000, 0x0002, /*0BBB*/
  0xFBC0, 0x0020, 0x0002, 0x8BBC, 0x0000, 0x0000, 0x0002, /*0BBC*/
  0xFBC0, 0x0020, 0x0002, 0x8BBD, 0x0000, 0x0000, 0x0002, /*0BBD*/
  0x27E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BBE*/
  0x27E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BBF*/
  0x27E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BC0*/
  0x27E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BC1*/
  0x27EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BC2*/
  0xFBC0, 0x0020, 0x0002, 0x8BC3, 0x0000, 0x0000, 0x0002, /*0BC3*/
  0xFBC0, 0x0020, 0x0002, 0x8BC4, 0x0000, 0x0000, 0x0002, /*0BC4*/
  0xFBC0, 0x0020, 0x0002, 0x8BC5, 0x0000, 0x0000, 0x0002, /*0BC5*/
  0x27EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BC6*/
  0x27EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BC7*/
  0x27ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BC8*/
  0xFBC0, 0x0020, 0x0002, 0x8BC9, 0x0000, 0x0000, 0x0002, /*0BC9*/
  0x27EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BCA*/
  0x27EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BCB*/
  0x27F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BCC*/
  0x27F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BCD*/
  0xFBC0, 0x0020, 0x0002, 0x8BCE, 0x0000, 0x0000, 0x0002, /*0BCE*/
  0xFBC0, 0x0020, 0x0002, 0x8BCF, 0x0000, 0x0000, 0x0002, /*0BCF*/
  0x27C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BD0*/
  0xFBC0, 0x0020, 0x0002, 0x8BD1, 0x0000, 0x0000, 0x0002, /*0BD1*/
  0xFBC0, 0x0020, 0x0002, 0x8BD2, 0x0000, 0x0000, 0x0002, /*0BD2*/
  0xFBC0, 0x0020, 0x0002, 0x8BD3, 0x0000, 0x0000, 0x0002, /*0BD3*/
  0xFBC0, 0x0020, 0x0002, 0x8BD4, 0x0000, 0x0000, 0x0002, /*0BD4*/
  0xFBC0, 0x0020, 0x0002, 0x8BD5, 0x0000, 0x0000, 0x0002, /*0BD5*/
  0xFBC0, 0x0020, 0x0002, 0x8BD6, 0x0000, 0x0000, 0x0002, /*0BD6*/
  0x27F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BD7*/
  0xFBC0, 0x0020, 0x0002, 0x8BD8, 0x0000, 0x0000, 0x0002, /*0BD8*/
  0xFBC0, 0x0020, 0x0002, 0x8BD9, 0x0000, 0x0000, 0x0002, /*0BD9*/
  0xFBC0, 0x0020, 0x0002, 0x8BDA, 0x0000, 0x0000, 0x0002, /*0BDA*/
  0xFBC0, 0x0020, 0x0002, 0x8BDB, 0x0000, 0x0000, 0x0002, /*0BDB*/
  0xFBC0, 0x0020, 0x0002, 0x8BDC, 0x0000, 0x0000, 0x0002, /*0BDC*/
  0xFBC0, 0x0020, 0x0002, 0x8BDD, 0x0000, 0x0000, 0x0002, /*0BDD*/
  0xFBC0, 0x0020, 0x0002, 0x8BDE, 0x0000, 0x0000, 0x0002, /*0BDE*/
  0xFBC0, 0x0020, 0x0002, 0x8BDF, 0x0000, 0x0000, 0x0002, /*0BDF*/
  0xFBC0, 0x0020, 0x0002, 0x8BE0, 0x0000, 0x0000, 0x0002, /*0BE0*/
  0xFBC0, 0x0020, 0x0002, 0x8BE1, 0x0000, 0x0000, 0x0002, /*0BE1*/
  0xFBC0, 0x0020, 0x0002, 0x8BE2, 0x0000, 0x0000, 0x0002, /*0BE2*/
  0xFBC0, 0x0020, 0x0002, 0x8BE3, 0x0000, 0x0000, 0x0002, /*0BE3*/
  0xFBC0, 0x0020, 0x0002, 0x8BE4, 0x0000, 0x0000, 0x0002, /*0BE4*/
  0xFBC0, 0x0020, 0x0002, 0x8BE5, 0x0000, 0x0000, 0x0002, /*0BE5*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BE6*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BE7*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BE8*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BE9*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BEA*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BEB*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BEC*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BED*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BEE*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BEF*/
  0x1AA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BF0*/
  0x1AA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BF1*/
  0x1AAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BF2*/
  0x0515, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BF3*/
  0x0516, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BF4*/
  0x0517, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BF5*/
  0x0518, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BF6*/
  0x0519, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BF7*/
  0x051A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BF8*/
  0x1C1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BF9*/
  0x051B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0BFA*/
  0xFBC0, 0x0020, 0x0002, 0x8BFB, 0x0000, 0x0000, 0x0002, /*0BFB*/
  0xFBC0, 0x0020, 0x0002, 0x8BFC, 0x0000, 0x0000, 0x0002, /*0BFC*/
  0xFBC0, 0x0020, 0x0002, 0x8BFD, 0x0000, 0x0000, 0x0002, /*0BFD*/
  0xFBC0, 0x0020, 0x0002, 0x8BFE, 0x0000, 0x0000, 0x0002, /*0BFE*/
  0xFBC0, 0x0020, 0x0002, 0x8BFF, 0x0000, 0x0000, 0x0002  /*0BFF*/
};

uint16 uca900_p00C[]= {
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C00*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C01*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C02*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C03*/
  0xFBC0, 0x0020, 0x0002, 0x8C04, 0x0000, 0x0000, 0x0002, /*0C04*/
  0x27F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C05*/
  0x27F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C06*/
  0x27F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C07*/
  0x27F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C08*/
  0x27F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C09*/
  0x27F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C0A*/
  0x27F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C0B*/
  0x27FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C0C*/
  0xFBC0, 0x0020, 0x0002, 0x8C0D, 0x0000, 0x0000, 0x0002, /*0C0D*/
  0x27FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C0E*/
  0x27FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C0F*/
  0x27FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C10*/
  0xFBC0, 0x0020, 0x0002, 0x8C11, 0x0000, 0x0000, 0x0002, /*0C11*/
  0x2800, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C12*/
  0x2801, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C13*/
  0x2802, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C14*/
  0x2803, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C15*/
  0x2804, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C16*/
  0x2805, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C17*/
  0x2806, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C18*/
  0x2807, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C19*/
  0x2808, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C1A*/
  0x280A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C1B*/
  0x280B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C1C*/
  0x280D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C1D*/
  0x280E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C1E*/
  0x280F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C1F*/
  0x2810, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C20*/
  0x2811, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C21*/
  0x2812, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C22*/
  0x2813, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C23*/
  0x2814, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C24*/
  0x2815, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C25*/
  0x2816, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C26*/
  0x2817, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C27*/
  0x2818, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C28*/
  0xFBC0, 0x0020, 0x0002, 0x8C29, 0x0000, 0x0000, 0x0002, /*0C29*/
  0x2819, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C2A*/
  0x281A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C2B*/
  0x281B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C2C*/
  0x281C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C2D*/
  0x281D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C2E*/
  0x281E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C2F*/
  0x281F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C30*/
  0x2820, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C31*/
  0x2821, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C32*/
  0x2827, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C33*/
  0x2828, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C34*/
  0x2822, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C35*/
  0x2823, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C36*/
  0x2824, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C37*/
  0x2825, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C38*/
  0x2826, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C39*/
  0xFBC0, 0x0020, 0x0002, 0x8C3A, 0x0000, 0x0000, 0x0002, /*0C3A*/
  0xFBC0, 0x0020, 0x0002, 0x8C3B, 0x0000, 0x0000, 0x0002, /*0C3B*/
  0xFBC0, 0x0020, 0x0002, 0x8C3C, 0x0000, 0x0000, 0x0002, /*0C3C*/
  0x282A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C3D*/
  0x282B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C3E*/
  0x282C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C3F*/
  0x282D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C40*/
  0x282E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C41*/
  0x282F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C42*/
  0x2830, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C43*/
  0x2831, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C44*/
  0xFBC0, 0x0020, 0x0002, 0x8C45, 0x0000, 0x0000, 0x0002, /*0C45*/
  0x2834, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C46*/
  0x2835, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C47*/
  0x2836, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C48*/
  0xFBC0, 0x0020, 0x0002, 0x8C49, 0x0000, 0x0000, 0x0002, /*0C49*/
  0x2837, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C4A*/
  0x2838, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C4B*/
  0x2839, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C4C*/
  0x283A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C4D*/
  0xFBC0, 0x0020, 0x0002, 0x8C4E, 0x0000, 0x0000, 0x0002, /*0C4E*/
  0xFBC0, 0x0020, 0x0002, 0x8C4F, 0x0000, 0x0000, 0x0002, /*0C4F*/
  0xFBC0, 0x0020, 0x0002, 0x8C50, 0x0000, 0x0000, 0x0002, /*0C50*/
  0xFBC0, 0x0020, 0x0002, 0x8C51, 0x0000, 0x0000, 0x0002, /*0C51*/
  0xFBC0, 0x0020, 0x0002, 0x8C52, 0x0000, 0x0000, 0x0002, /*0C52*/
  0xFBC0, 0x0020, 0x0002, 0x8C53, 0x0000, 0x0000, 0x0002, /*0C53*/
  0xFBC0, 0x0020, 0x0002, 0x8C54, 0x0000, 0x0000, 0x0002, /*0C54*/
  0x283B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C55*/
  0x283C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C56*/
  0xFBC0, 0x0020, 0x0002, 0x8C57, 0x0000, 0x0000, 0x0002, /*0C57*/
  0x2809, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C58*/
  0x280C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C59*/
  0x2829, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C5A*/
  0xFBC0, 0x0020, 0x0002, 0x8C5B, 0x0000, 0x0000, 0x0002, /*0C5B*/
  0xFBC0, 0x0020, 0x0002, 0x8C5C, 0x0000, 0x0000, 0x0002, /*0C5C*/
  0xFBC0, 0x0020, 0x0002, 0x8C5D, 0x0000, 0x0000, 0x0002, /*0C5D*/
  0xFBC0, 0x0020, 0x0002, 0x8C5E, 0x0000, 0x0000, 0x0002, /*0C5E*/
  0xFBC0, 0x0020, 0x0002, 0x8C5F, 0x0000, 0x0000, 0x0002, /*0C5F*/
  0x27FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C60*/
  0x27FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C61*/
  0x2832, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C62*/
  0x2833, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C63*/
  0xFBC0, 0x0020, 0x0002, 0x8C64, 0x0000, 0x0000, 0x0002, /*0C64*/
  0xFBC0, 0x0020, 0x0002, 0x8C65, 0x0000, 0x0000, 0x0002, /*0C65*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C66*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C67*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C68*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C69*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C6A*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C6B*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C6C*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C6D*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C6E*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C6F*/
  0xFBC0, 0x0020, 0x0002, 0x8C70, 0x0000, 0x0000, 0x0002, /*0C70*/
  0xFBC0, 0x0020, 0x0002, 0x8C71, 0x0000, 0x0000, 0x0002, /*0C71*/
  0xFBC0, 0x0020, 0x0002, 0x8C72, 0x0000, 0x0000, 0x0002, /*0C72*/
  0xFBC0, 0x0020, 0x0002, 0x8C73, 0x0000, 0x0000, 0x0002, /*0C73*/
  0xFBC0, 0x0020, 0x0002, 0x8C74, 0x0000, 0x0000, 0x0002, /*0C74*/
  0xFBC0, 0x0020, 0x0002, 0x8C75, 0x0000, 0x0000, 0x0002, /*0C75*/
  0xFBC0, 0x0020, 0x0002, 0x8C76, 0x0000, 0x0000, 0x0002, /*0C76*/
  0xFBC0, 0x0020, 0x0002, 0x8C77, 0x0000, 0x0000, 0x0002, /*0C77*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C78*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C79*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C7A*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C7B*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C7C*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C7D*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C7E*/
  0x051C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C7F*/
  0x2874, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C80*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C81*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C82*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C83*/
  0xFBC0, 0x0020, 0x0002, 0x8C84, 0x0000, 0x0000, 0x0002, /*0C84*/
  0x283D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C85*/
  0x283E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C86*/
  0x283F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C87*/
  0x2840, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C88*/
  0x2841, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C89*/
  0x2842, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C8A*/
  0x2843, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C8B*/
  0x2845, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C8C*/
  0xFBC0, 0x0020, 0x0002, 0x8C8D, 0x0000, 0x0000, 0x0002, /*0C8D*/
  0x2847, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C8E*/
  0x2848, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C8F*/
  0x2849, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C90*/
  0xFBC0, 0x0020, 0x0002, 0x8C91, 0x0000, 0x0000, 0x0002, /*0C91*/
  0x284A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C92*/
  0x284B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C93*/
  0x284C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C94*/
  0x284D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C95*/
  0x284E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C96*/
  0x284F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C97*/
  0x2850, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C98*/
  0x2851, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C99*/
  0x2852, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C9A*/
  0x2853, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C9B*/
  0x2854, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C9C*/
  0x2855, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C9D*/
  0x2856, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C9E*/
  0x2857, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0C9F*/
  0x2858, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CA0*/
  0x2859, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CA1*/
  0x285A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CA2*/
  0x285B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CA3*/
  0x285C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CA4*/
  0x285D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CA5*/
  0x285E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CA6*/
  0x285F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CA7*/
  0x2860, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CA8*/
  0xFBC0, 0x0020, 0x0002, 0x8CA9, 0x0000, 0x0000, 0x0002, /*0CA9*/
  0x2861, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CAA*/
  0x2862, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CAB*/
  0x2863, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CAC*/
  0x2864, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CAD*/
  0x2865, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CAE*/
  0x2866, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CAF*/
  0x2867, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CB0*/
  0x2868, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CB1*/
  0x2869, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CB2*/
  0x286F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CB3*/
  0xFBC0, 0x0020, 0x0002, 0x8CB4, 0x0000, 0x0000, 0x0002, /*0CB4*/
  0x286A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CB5*/
  0x286B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CB6*/
  0x286C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CB7*/
  0x286D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CB8*/
  0x286E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CB9*/
  0xFBC0, 0x0020, 0x0002, 0x8CBA, 0x0000, 0x0000, 0x0002, /*0CBA*/
  0xFBC0, 0x0020, 0x0002, 0x8CBB, 0x0000, 0x0000, 0x0002, /*0CBB*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CBC*/
  0x2871, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CBD*/
  0x2875, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CBE*/
  0x2876, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CBF*/
  0x2877, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CC0*/
  0x2878, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CC1*/
  0x2879, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CC2*/
  0x287A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CC3*/
  0x287B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CC4*/
  0xFBC0, 0x0020, 0x0002, 0x8CC5, 0x0000, 0x0000, 0x0002, /*0CC5*/
  0x287E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CC6*/
  0x287F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CC7*/
  0x2880, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CC8*/
  0xFBC0, 0x0020, 0x0002, 0x8CC9, 0x0000, 0x0000, 0x0002, /*0CC9*/
  0x2881, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CCA*/
  0x2882, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CCB*/
  0x2883, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CCC*/
  0x2884, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CCD*/
  0xFBC0, 0x0020, 0x0002, 0x8CCE, 0x0000, 0x0000, 0x0002, /*0CCE*/
  0xFBC0, 0x0020, 0x0002, 0x8CCF, 0x0000, 0x0000, 0x0002, /*0CCF*/
  0xFBC0, 0x0020, 0x0002, 0x8CD0, 0x0000, 0x0000, 0x0002, /*0CD0*/
  0xFBC0, 0x0020, 0x0002, 0x8CD1, 0x0000, 0x0000, 0x0002, /*0CD1*/
  0xFBC0, 0x0020, 0x0002, 0x8CD2, 0x0000, 0x0000, 0x0002, /*0CD2*/
  0xFBC0, 0x0020, 0x0002, 0x8CD3, 0x0000, 0x0000, 0x0002, /*0CD3*/
  0xFBC0, 0x0020, 0x0002, 0x8CD4, 0x0000, 0x0000, 0x0002, /*0CD4*/
  0x2885, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CD5*/
  0x2886, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CD6*/
  0xFBC0, 0x0020, 0x0002, 0x8CD7, 0x0000, 0x0000, 0x0002, /*0CD7*/
  0xFBC0, 0x0020, 0x0002, 0x8CD8, 0x0000, 0x0000, 0x0002, /*0CD8*/
  0xFBC0, 0x0020, 0x0002, 0x8CD9, 0x0000, 0x0000, 0x0002, /*0CD9*/
  0xFBC0, 0x0020, 0x0002, 0x8CDA, 0x0000, 0x0000, 0x0002, /*0CDA*/
  0xFBC0, 0x0020, 0x0002, 0x8CDB, 0x0000, 0x0000, 0x0002, /*0CDB*/
  0xFBC0, 0x0020, 0x0002, 0x8CDC, 0x0000, 0x0000, 0x0002, /*0CDC*/
  0xFBC0, 0x0020, 0x0002, 0x8CDD, 0x0000, 0x0000, 0x0002, /*0CDD*/
  0x2870, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CDE*/
  0xFBC0, 0x0020, 0x0002, 0x8CDF, 0x0000, 0x0000, 0x0002, /*0CDF*/
  0x2844, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CE0*/
  0x2846, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CE1*/
  0x287C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CE2*/
  0x287D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CE3*/
  0xFBC0, 0x0020, 0x0002, 0x8CE4, 0x0000, 0x0000, 0x0002, /*0CE4*/
  0xFBC0, 0x0020, 0x0002, 0x8CE5, 0x0000, 0x0000, 0x0002, /*0CE5*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CE6*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CE7*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CE8*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CE9*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CEA*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CEB*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CEC*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CED*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CEE*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CEF*/
  0xFBC0, 0x0020, 0x0002, 0x8CF0, 0x0000, 0x0000, 0x0002, /*0CF0*/
  0x2872, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CF1*/
  0x2873, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0CF2*/
  0xFBC0, 0x0020, 0x0002, 0x8CF3, 0x0000, 0x0000, 0x0002, /*0CF3*/
  0xFBC0, 0x0020, 0x0002, 0x8CF4, 0x0000, 0x0000, 0x0002, /*0CF4*/
  0xFBC0, 0x0020, 0x0002, 0x8CF5, 0x0000, 0x0000, 0x0002, /*0CF5*/
  0xFBC0, 0x0020, 0x0002, 0x8CF6, 0x0000, 0x0000, 0x0002, /*0CF6*/
  0xFBC0, 0x0020, 0x0002, 0x8CF7, 0x0000, 0x0000, 0x0002, /*0CF7*/
  0xFBC0, 0x0020, 0x0002, 0x8CF8, 0x0000, 0x0000, 0x0002, /*0CF8*/
  0xFBC0, 0x0020, 0x0002, 0x8CF9, 0x0000, 0x0000, 0x0002, /*0CF9*/
  0xFBC0, 0x0020, 0x0002, 0x8CFA, 0x0000, 0x0000, 0x0002, /*0CFA*/
  0xFBC0, 0x0020, 0x0002, 0x8CFB, 0x0000, 0x0000, 0x0002, /*0CFB*/
  0xFBC0, 0x0020, 0x0002, 0x8CFC, 0x0000, 0x0000, 0x0002, /*0CFC*/
  0xFBC0, 0x0020, 0x0002, 0x8CFD, 0x0000, 0x0000, 0x0002, /*0CFD*/
  0xFBC0, 0x0020, 0x0002, 0x8CFE, 0x0000, 0x0000, 0x0002, /*0CFE*/
  0xFBC0, 0x0020, 0x0002, 0x8CFF, 0x0000, 0x0000, 0x0002  /*0CFF*/
};

uint16 uca900_p00D[]= {
  0xFBC0, 0x0020, 0x0002, 0x8D00, 0x0000, 0x0000, 0x0002, /*0D00*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D01*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D02*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D03*/
  0xFBC0, 0x0020, 0x0002, 0x8D04, 0x0000, 0x0000, 0x0002, /*0D04*/
  0x2887, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D05*/
  0x2888, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D06*/
  0x2889, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D07*/
  0x288A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D08*/
  0x288C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D09*/
  0x288D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D0A*/
  0x288E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D0B*/
  0x2890, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D0C*/
  0xFBC0, 0x0020, 0x0002, 0x8D0D, 0x0000, 0x0000, 0x0002, /*0D0D*/
  0x2892, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D0E*/
  0x2893, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D0F*/
  0x2894, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D10*/
  0xFBC0, 0x0020, 0x0002, 0x8D11, 0x0000, 0x0000, 0x0002, /*0D11*/
  0x2895, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D12*/
  0x2896, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D13*/
  0x2897, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D14*/
  0x2898, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D15*/
  0x2899, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D16*/
  0x289A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D17*/
  0x289B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D18*/
  0x289C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D19*/
  0x289D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D1A*/
  0x289E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D1B*/
  0x289F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D1C*/
  0x28A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D1D*/
  0x28A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D1E*/
  0x28A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D1F*/
  0x28A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D20*/
  0x28A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D21*/
  0x28A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D22*/
  0x28A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D23*/
  0x28A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D24*/
  0x28A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D25*/
  0x28A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D26*/
  0x28AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D27*/
  0x28AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D28*/
  0x28AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D29*/
  0x28AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D2A*/
  0x28AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D2B*/
  0x28AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D2C*/
  0x28B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D2D*/
  0x28B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D2E*/
  0x28B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D2F*/
  0x28B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D30*/
  0x28BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D31*/
  0x28B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D32*/
  0x28BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D33*/
  0x28BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D34*/
  0x28B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D35*/
  0x28B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D36*/
  0x28B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D37*/
  0x28B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D38*/
  0x28B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D39*/
  0x28BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D3A*/
  0xFBC0, 0x0020, 0x0002, 0x8D3B, 0x0000, 0x0000, 0x0002, /*0D3B*/
  0xFBC0, 0x0020, 0x0002, 0x8D3C, 0x0000, 0x0000, 0x0002, /*0D3C*/
  0x28BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D3D*/
  0x28BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D3E*/
  0x28C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D3F*/
  0x28C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D40*/
  0x28C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D41*/
  0x28C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D42*/
  0x28C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D43*/
  0x28C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D44*/
  0xFBC0, 0x0020, 0x0002, 0x8D45, 0x0000, 0x0000, 0x0002, /*0D45*/
  0x28C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D46*/
  0x28C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D47*/
  0x28CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D48*/
  0xFBC0, 0x0020, 0x0002, 0x8D49, 0x0000, 0x0000, 0x0002, /*0D49*/
  0x28CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D4A*/
  0x28CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D4B*/
  0x28CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D4C*/
  0x28CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D4D*/
  0x28B3, 0x0020, 0x0004, 0x28CF, 0x0020, 0x0004, 0x0002, /*0D4E*/
  0x051D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D4F*/
  0xFBC0, 0x0020, 0x0002, 0x8D50, 0x0000, 0x0000, 0x0002, /*0D50*/
  0xFBC0, 0x0020, 0x0002, 0x8D51, 0x0000, 0x0000, 0x0002, /*0D51*/
  0xFBC0, 0x0020, 0x0002, 0x8D52, 0x0000, 0x0000, 0x0002, /*0D52*/
  0xFBC0, 0x0020, 0x0002, 0x8D53, 0x0000, 0x0000, 0x0002, /*0D53*/
  0x28B1, 0x0020, 0x0004, 0x28CF, 0x0020, 0x0004, 0x0002, /*0D54*/
  0x28B2, 0x0020, 0x0004, 0x28CF, 0x0020, 0x0004, 0x0002, /*0D55*/
  0x28BB, 0x0020, 0x0004, 0x28CF, 0x0020, 0x0004, 0x0002, /*0D56*/
  0x28CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D57*/
  0x1AAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D58*/
  0x1AAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D59*/
  0x1AAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D5A*/
  0x1AAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D5B*/
  0x1AAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D5C*/
  0x1AB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D5D*/
  0x1AB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D5E*/
  0x288B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D5F*/
  0x288F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D60*/
  0x2891, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D61*/
  0x28C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D62*/
  0x28C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D63*/
  0xFBC0, 0x0020, 0x0002, 0x8D64, 0x0000, 0x0000, 0x0002, /*0D64*/
  0xFBC0, 0x0020, 0x0002, 0x8D65, 0x0000, 0x0000, 0x0002, /*0D65*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D66*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D67*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D68*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D69*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D6A*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D6B*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D6C*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D6D*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D6E*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D6F*/
  0x1AB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D70*/
  0x1AB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D71*/
  0x1AB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D72*/
  0x1AB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D73*/
  0x1AB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D74*/
  0x1AB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D75*/
  0x1AB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D76*/
  0x1AB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D77*/
  0x1ABA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D78*/
  0x051E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D79*/
  0x28A6, 0x0020, 0x0004, 0x28CF, 0x0020, 0x0004, 0x0002, /*0D7A*/
  0x28AB, 0x0020, 0x0004, 0x28CF, 0x0020, 0x0004, 0x0002, /*0D7B*/
  0x28B3, 0x0020, 0x0004, 0x28CF, 0x0020, 0x0004, 0x0002, /*0D7C*/
  0x28B4, 0x0020, 0x0004, 0x28CF, 0x0020, 0x0004, 0x0002, /*0D7D*/
  0x28BA, 0x0020, 0x0004, 0x28CF, 0x0020, 0x0004, 0x0002, /*0D7E*/
  0x2898, 0x0020, 0x0004, 0x28CF, 0x0020, 0x0004, 0x0002, /*0D7F*/
  0xFBC0, 0x0020, 0x0002, 0x8D80, 0x0000, 0x0000, 0x0002, /*0D80*/
  0xFBC0, 0x0020, 0x0002, 0x8D81, 0x0000, 0x0000, 0x0002, /*0D81*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D82*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D83*/
  0xFBC0, 0x0020, 0x0002, 0x8D84, 0x0000, 0x0000, 0x0002, /*0D84*/
  0x28D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D85*/
  0x28D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D86*/
  0x28D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D87*/
  0x28D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D88*/
  0x28D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D89*/
  0x28D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D8A*/
  0x28D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D8B*/
  0x28D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D8C*/
  0x28D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D8D*/
  0x28D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D8E*/
  0x28DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D8F*/
  0x28DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D90*/
  0x28DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D91*/
  0x28DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D92*/
  0x28DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D93*/
  0x28DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D94*/
  0x28E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D95*/
  0x28E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D96*/
  0xFBC0, 0x0020, 0x0002, 0x8D97, 0x0000, 0x0000, 0x0002, /*0D97*/
  0xFBC0, 0x0020, 0x0002, 0x8D98, 0x0000, 0x0000, 0x0002, /*0D98*/
  0xFBC0, 0x0020, 0x0002, 0x8D99, 0x0000, 0x0000, 0x0002, /*0D99*/
  0x28E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D9A*/
  0x28E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D9B*/
  0x28E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D9C*/
  0x28E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D9D*/
  0x28E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D9E*/
  0x28E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0D9F*/
  0x28E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DA0*/
  0x28E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DA1*/
  0x28EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DA2*/
  0x28EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DA3*/
  0x28EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DA4*/
  0x28ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DA5*/
  0x28EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DA6*/
  0x28EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DA7*/
  0x28F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DA8*/
  0x28F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DA9*/
  0x28F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DAA*/
  0x28F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DAB*/
  0x28F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DAC*/
  0x28F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DAD*/
  0x28F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DAE*/
  0x28F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DAF*/
  0x28F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DB0*/
  0x28F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DB1*/
  0xFBC0, 0x0020, 0x0002, 0x8DB2, 0x0000, 0x0000, 0x0002, /*0DB2*/
  0x28FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DB3*/
  0x28FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DB4*/
  0x28FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DB5*/
  0x28FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DB6*/
  0x28FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DB7*/
  0x28FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DB8*/
  0x2900, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DB9*/
  0x2901, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DBA*/
  0x2902, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DBB*/
  0xFBC0, 0x0020, 0x0002, 0x8DBC, 0x0000, 0x0000, 0x0002, /*0DBC*/
  0x2903, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DBD*/
  0xFBC0, 0x0020, 0x0002, 0x8DBE, 0x0000, 0x0000, 0x0002, /*0DBE*/
  0xFBC0, 0x0020, 0x0002, 0x8DBF, 0x0000, 0x0000, 0x0002, /*0DBF*/
  0x2904, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DC0*/
  0x2905, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DC1*/
  0x2906, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DC2*/
  0x2907, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DC3*/
  0x2908, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DC4*/
  0x2909, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DC5*/
  0x290A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DC6*/
  0xFBC0, 0x0020, 0x0002, 0x8DC7, 0x0000, 0x0000, 0x0002, /*0DC7*/
  0xFBC0, 0x0020, 0x0002, 0x8DC8, 0x0000, 0x0000, 0x0002, /*0DC8*/
  0xFBC0, 0x0020, 0x0002, 0x8DC9, 0x0000, 0x0000, 0x0002, /*0DC9*/
  0x291C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DCA*/
  0xFBC0, 0x0020, 0x0002, 0x8DCB, 0x0000, 0x0000, 0x0002, /*0DCB*/
  0xFBC0, 0x0020, 0x0002, 0x8DCC, 0x0000, 0x0000, 0x0002, /*0DCC*/
  0xFBC0, 0x0020, 0x0002, 0x8DCD, 0x0000, 0x0000, 0x0002, /*0DCD*/
  0xFBC0, 0x0020, 0x0002, 0x8DCE, 0x0000, 0x0000, 0x0002, /*0DCE*/
  0x290B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DCF*/
  0x290C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DD0*/
  0x290D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DD1*/
  0x290E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DD2*/
  0x290F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DD3*/
  0x2910, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DD4*/
  0xFBC0, 0x0020, 0x0002, 0x8DD5, 0x0000, 0x0000, 0x0002, /*0DD5*/
  0x2911, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DD6*/
  0xFBC0, 0x0020, 0x0002, 0x8DD7, 0x0000, 0x0000, 0x0002, /*0DD7*/
  0x2912, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DD8*/
  0x2916, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DD9*/
  0x2917, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DDA*/
  0x2918, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DDB*/
  0x2919, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DDC*/
  0x291A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DDD*/
  0x291B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DDE*/
  0x2914, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DDF*/
  0xFBC0, 0x0020, 0x0002, 0x8DE0, 0x0000, 0x0000, 0x0002, /*0DE0*/
  0xFBC0, 0x0020, 0x0002, 0x8DE1, 0x0000, 0x0000, 0x0002, /*0DE1*/
  0xFBC0, 0x0020, 0x0002, 0x8DE2, 0x0000, 0x0000, 0x0002, /*0DE2*/
  0xFBC0, 0x0020, 0x0002, 0x8DE3, 0x0000, 0x0000, 0x0002, /*0DE3*/
  0xFBC0, 0x0020, 0x0002, 0x8DE4, 0x0000, 0x0000, 0x0002, /*0DE4*/
  0xFBC0, 0x0020, 0x0002, 0x8DE5, 0x0000, 0x0000, 0x0002, /*0DE5*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DE6*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DE7*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DE8*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DE9*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DEA*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DEB*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DEC*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DED*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DEE*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DEF*/
  0xFBC0, 0x0020, 0x0002, 0x8DF0, 0x0000, 0x0000, 0x0002, /*0DF0*/
  0xFBC0, 0x0020, 0x0002, 0x8DF1, 0x0000, 0x0000, 0x0002, /*0DF1*/
  0x2913, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DF2*/
  0x2915, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DF3*/
  0x03ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0DF4*/
  0xFBC0, 0x0020, 0x0002, 0x8DF5, 0x0000, 0x0000, 0x0002, /*0DF5*/
  0xFBC0, 0x0020, 0x0002, 0x8DF6, 0x0000, 0x0000, 0x0002, /*0DF6*/
  0xFBC0, 0x0020, 0x0002, 0x8DF7, 0x0000, 0x0000, 0x0002, /*0DF7*/
  0xFBC0, 0x0020, 0x0002, 0x8DF8, 0x0000, 0x0000, 0x0002, /*0DF8*/
  0xFBC0, 0x0020, 0x0002, 0x8DF9, 0x0000, 0x0000, 0x0002, /*0DF9*/
  0xFBC0, 0x0020, 0x0002, 0x8DFA, 0x0000, 0x0000, 0x0002, /*0DFA*/
  0xFBC0, 0x0020, 0x0002, 0x8DFB, 0x0000, 0x0000, 0x0002, /*0DFB*/
  0xFBC0, 0x0020, 0x0002, 0x8DFC, 0x0000, 0x0000, 0x0002, /*0DFC*/
  0xFBC0, 0x0020, 0x0002, 0x8DFD, 0x0000, 0x0000, 0x0002, /*0DFD*/
  0xFBC0, 0x0020, 0x0002, 0x8DFE, 0x0000, 0x0000, 0x0002, /*0DFE*/
  0xFBC0, 0x0020, 0x0002, 0x8DFF, 0x0000, 0x0000, 0x0002  /*0DFF*/
};

uint16 uca900_p00E[]= {
  0xFBC0, 0x0020, 0x0002, 0x8E00, 0x0000, 0x0000, 0x0002, /*0E00*/
  0x2D73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E01*/
  0x2D74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E02*/
  0x2D75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E03*/
  0x2D76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E04*/
  0x2D77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E05*/
  0x2D78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E06*/
  0x2D79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E07*/
  0x2D7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E08*/
  0x2D7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E09*/
  0x2D7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E0A*/
  0x2D7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E0B*/
  0x2D7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E0C*/
  0x2D7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E0D*/
  0x2D80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E0E*/
  0x2D81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E0F*/
  0x2D82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E10*/
  0x2D83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E11*/
  0x2D84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E12*/
  0x2D85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E13*/
  0x2D86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E14*/
  0x2D87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E15*/
  0x2D88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E16*/
  0x2D89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E17*/
  0x2D8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E18*/
  0x2D8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E19*/
  0x2D8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E1A*/
  0x2D8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E1B*/
  0x2D8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E1C*/
  0x2D8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E1D*/
  0x2D90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E1E*/
  0x2D91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E1F*/
  0x2D92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E20*/
  0x2D93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E21*/
  0x2D94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E22*/
  0x2D95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E23*/
  0x2D96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E24*/
  0x2D97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E25*/
  0x2D98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E26*/
  0x2D99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E27*/
  0x2D9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E28*/
  0x2D9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E29*/
  0x2D9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E2A*/
  0x2D9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E2B*/
  0x2D9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E2C*/
  0x2D9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E2D*/
  0x2DA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E2E*/
  0x2DA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E2F*/
  0x2DA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E30*/
  0x2DA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E31*/
  0x2DA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E32*/
  0x2DA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E33*/
  0x2DA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E34*/
  0x2DA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E35*/
  0x2DA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E36*/
  0x2DA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E37*/
  0x2DAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E38*/
  0x2DAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E39*/
  0x2DAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E3A*/
  0xFBC0, 0x0020, 0x0002, 0x8E3B, 0x0000, 0x0000, 0x0002, /*0E3B*/
  0xFBC0, 0x0020, 0x0002, 0x8E3C, 0x0000, 0x0000, 0x0002, /*0E3C*/
  0xFBC0, 0x0020, 0x0002, 0x8E3D, 0x0000, 0x0000, 0x0002, /*0E3D*/
  0xFBC0, 0x0020, 0x0002, 0x8E3E, 0x0000, 0x0000, 0x0002, /*0E3E*/
  0x1C1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E3F*/
  0x2DAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E40*/
  0x2DAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E41*/
  0x2DAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E42*/
  0x2DB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E43*/
  0x2DB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E44*/
  0x2DB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E45*/
  0x1BFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E46*/
  0x0000, 0x00D2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E47*/
  0x0000, 0x00D3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E48*/
  0x0000, 0x00D4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E49*/
  0x0000, 0x00D5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E4A*/
  0x0000, 0x00D6, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E4B*/
  0x0000, 0x00D7, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E4C*/
  0x0000, 0x00D8, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E4D*/
  0x0000, 0x00D1, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E4E*/
  0x03EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E4F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E50*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E51*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E52*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E53*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E54*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E55*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E56*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E57*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E58*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E59*/
  0x03EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E5A*/
  0x03F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E5B*/
  0xFBC0, 0x0020, 0x0002, 0x8E5C, 0x0000, 0x0000, 0x0002, /*0E5C*/
  0xFBC0, 0x0020, 0x0002, 0x8E5D, 0x0000, 0x0000, 0x0002, /*0E5D*/
  0xFBC0, 0x0020, 0x0002, 0x8E5E, 0x0000, 0x0000, 0x0002, /*0E5E*/
  0xFBC0, 0x0020, 0x0002, 0x8E5F, 0x0000, 0x0000, 0x0002, /*0E5F*/
  0xFBC0, 0x0020, 0x0002, 0x8E60, 0x0000, 0x0000, 0x0002, /*0E60*/
  0xFBC0, 0x0020, 0x0002, 0x8E61, 0x0000, 0x0000, 0x0002, /*0E61*/
  0xFBC0, 0x0020, 0x0002, 0x8E62, 0x0000, 0x0000, 0x0002, /*0E62*/
  0xFBC0, 0x0020, 0x0002, 0x8E63, 0x0000, 0x0000, 0x0002, /*0E63*/
  0xFBC0, 0x0020, 0x0002, 0x8E64, 0x0000, 0x0000, 0x0002, /*0E64*/
  0xFBC0, 0x0020, 0x0002, 0x8E65, 0x0000, 0x0000, 0x0002, /*0E65*/
  0xFBC0, 0x0020, 0x0002, 0x8E66, 0x0000, 0x0000, 0x0002, /*0E66*/
  0xFBC0, 0x0020, 0x0002, 0x8E67, 0x0000, 0x0000, 0x0002, /*0E67*/
  0xFBC0, 0x0020, 0x0002, 0x8E68, 0x0000, 0x0000, 0x0002, /*0E68*/
  0xFBC0, 0x0020, 0x0002, 0x8E69, 0x0000, 0x0000, 0x0002, /*0E69*/
  0xFBC0, 0x0020, 0x0002, 0x8E6A, 0x0000, 0x0000, 0x0002, /*0E6A*/
  0xFBC0, 0x0020, 0x0002, 0x8E6B, 0x0000, 0x0000, 0x0002, /*0E6B*/
  0xFBC0, 0x0020, 0x0002, 0x8E6C, 0x0000, 0x0000, 0x0002, /*0E6C*/
  0xFBC0, 0x0020, 0x0002, 0x8E6D, 0x0000, 0x0000, 0x0002, /*0E6D*/
  0xFBC0, 0x0020, 0x0002, 0x8E6E, 0x0000, 0x0000, 0x0002, /*0E6E*/
  0xFBC0, 0x0020, 0x0002, 0x8E6F, 0x0000, 0x0000, 0x0002, /*0E6F*/
  0xFBC0, 0x0020, 0x0002, 0x8E70, 0x0000, 0x0000, 0x0002, /*0E70*/
  0xFBC0, 0x0020, 0x0002, 0x8E71, 0x0000, 0x0000, 0x0002, /*0E71*/
  0xFBC0, 0x0020, 0x0002, 0x8E72, 0x0000, 0x0000, 0x0002, /*0E72*/
  0xFBC0, 0x0020, 0x0002, 0x8E73, 0x0000, 0x0000, 0x0002, /*0E73*/
  0xFBC0, 0x0020, 0x0002, 0x8E74, 0x0000, 0x0000, 0x0002, /*0E74*/
  0xFBC0, 0x0020, 0x0002, 0x8E75, 0x0000, 0x0000, 0x0002, /*0E75*/
  0xFBC0, 0x0020, 0x0002, 0x8E76, 0x0000, 0x0000, 0x0002, /*0E76*/
  0xFBC0, 0x0020, 0x0002, 0x8E77, 0x0000, 0x0000, 0x0002, /*0E77*/
  0xFBC0, 0x0020, 0x0002, 0x8E78, 0x0000, 0x0000, 0x0002, /*0E78*/
  0xFBC0, 0x0020, 0x0002, 0x8E79, 0x0000, 0x0000, 0x0002, /*0E79*/
  0xFBC0, 0x0020, 0x0002, 0x8E7A, 0x0000, 0x0000, 0x0002, /*0E7A*/
  0xFBC0, 0x0020, 0x0002, 0x8E7B, 0x0000, 0x0000, 0x0002, /*0E7B*/
  0xFBC0, 0x0020, 0x0002, 0x8E7C, 0x0000, 0x0000, 0x0002, /*0E7C*/
  0xFBC0, 0x0020, 0x0002, 0x8E7D, 0x0000, 0x0000, 0x0002, /*0E7D*/
  0xFBC0, 0x0020, 0x0002, 0x8E7E, 0x0000, 0x0000, 0x0002, /*0E7E*/
  0xFBC0, 0x0020, 0x0002, 0x8E7F, 0x0000, 0x0000, 0x0002, /*0E7F*/
  0xFBC0, 0x0020, 0x0002, 0x8E80, 0x0000, 0x0000, 0x0002, /*0E80*/
  0x2DB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E81*/
  0x2DB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E82*/
  0xFBC0, 0x0020, 0x0002, 0x8E83, 0x0000, 0x0000, 0x0002, /*0E83*/
  0x2DB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E84*/
  0xFBC0, 0x0020, 0x0002, 0x8E85, 0x0000, 0x0000, 0x0002, /*0E85*/
  0xFBC0, 0x0020, 0x0002, 0x8E86, 0x0000, 0x0000, 0x0002, /*0E86*/
  0x2DB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E87*/
  0x2DB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E88*/
  0xFBC0, 0x0020, 0x0002, 0x8E89, 0x0000, 0x0000, 0x0002, /*0E89*/
  0x2DBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E8A*/
  0xFBC0, 0x0020, 0x0002, 0x8E8B, 0x0000, 0x0000, 0x0002, /*0E8B*/
  0xFBC0, 0x0020, 0x0002, 0x8E8C, 0x0000, 0x0000, 0x0002, /*0E8C*/
  0x2DBC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E8D*/
  0xFBC0, 0x0020, 0x0002, 0x8E8E, 0x0000, 0x0000, 0x0002, /*0E8E*/
  0xFBC0, 0x0020, 0x0002, 0x8E8F, 0x0000, 0x0000, 0x0002, /*0E8F*/
  0xFBC0, 0x0020, 0x0002, 0x8E90, 0x0000, 0x0000, 0x0002, /*0E90*/
  0xFBC0, 0x0020, 0x0002, 0x8E91, 0x0000, 0x0000, 0x0002, /*0E91*/
  0xFBC0, 0x0020, 0x0002, 0x8E92, 0x0000, 0x0000, 0x0002, /*0E92*/
  0xFBC0, 0x0020, 0x0002, 0x8E93, 0x0000, 0x0000, 0x0002, /*0E93*/
  0x2DBD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E94*/
  0x2DBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E95*/
  0x2DBF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E96*/
  0x2DC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E97*/
  0xFBC0, 0x0020, 0x0002, 0x8E98, 0x0000, 0x0000, 0x0002, /*0E98*/
  0x2DC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E99*/
  0x2DC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E9A*/
  0x2DC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E9B*/
  0x2DC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E9C*/
  0x2DC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E9D*/
  0x2DC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E9E*/
  0x2DC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0E9F*/
  0xFBC0, 0x0020, 0x0002, 0x8EA0, 0x0000, 0x0000, 0x0002, /*0EA0*/
  0x2DC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EA1*/
  0x2DC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EA2*/
  0x2DCA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EA3*/
  0xFBC0, 0x0020, 0x0002, 0x8EA4, 0x0000, 0x0000, 0x0002, /*0EA4*/
  0x2DCB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EA5*/
  0xFBC0, 0x0020, 0x0002, 0x8EA6, 0x0000, 0x0000, 0x0002, /*0EA6*/
  0x2DCC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EA7*/
  0xFBC0, 0x0020, 0x0002, 0x8EA8, 0x0000, 0x0000, 0x0002, /*0EA8*/
  0xFBC0, 0x0020, 0x0002, 0x8EA9, 0x0000, 0x0000, 0x0002, /*0EA9*/
  0x2DB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EAA*/
  0x2DCD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EAB*/
  0xFBC0, 0x0020, 0x0002, 0x8EAC, 0x0000, 0x0000, 0x0002, /*0EAC*/
  0x2DCE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EAD*/
  0x2DCF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EAE*/
  0x2DD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EAF*/
  0x2DD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EB0*/
  0x2DD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EB1*/
  0x2DD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EB2*/
  0x2DD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EB3*/
  0x2DD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EB4*/
  0x2DD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EB5*/
  0x2DD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EB6*/
  0x2DD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EB7*/
  0x2DD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EB8*/
  0x2DDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EB9*/
  0xFBC0, 0x0020, 0x0002, 0x8EBA, 0x0000, 0x0000, 0x0002, /*0EBA*/
  0x2DDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EBB*/
  0x2DDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EBC*/
  0x2DDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EBD*/
  0xFBC0, 0x0020, 0x0002, 0x8EBE, 0x0000, 0x0000, 0x0002, /*0EBE*/
  0xFBC0, 0x0020, 0x0002, 0x8EBF, 0x0000, 0x0000, 0x0002, /*0EBF*/
  0x2DDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EC0*/
  0x2DDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EC1*/
  0x2DE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EC2*/
  0x2DE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EC3*/
  0x2DE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EC4*/
  0xFBC0, 0x0020, 0x0002, 0x8EC5, 0x0000, 0x0000, 0x0002, /*0EC5*/
  0x1BFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EC6*/
  0xFBC0, 0x0020, 0x0002, 0x8EC7, 0x0000, 0x0000, 0x0002, /*0EC7*/
  0x0000, 0x00D9, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EC8*/
  0x0000, 0x00DA, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EC9*/
  0x0000, 0x00DB, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ECA*/
  0x0000, 0x00DC, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ECB*/
  0x0000, 0x00DD, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ECC*/
  0x0000, 0x00DE, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ECD*/
  0xFBC0, 0x0020, 0x0002, 0x8ECE, 0x0000, 0x0000, 0x0002, /*0ECE*/
  0xFBC0, 0x0020, 0x0002, 0x8ECF, 0x0000, 0x0000, 0x0002, /*0ECF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ED0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ED1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ED2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ED3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ED4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ED5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ED6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ED7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ED8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0ED9*/
  0xFBC0, 0x0020, 0x0002, 0x8EDA, 0x0000, 0x0000, 0x0002, /*0EDA*/
  0xFBC0, 0x0020, 0x0002, 0x8EDB, 0x0000, 0x0000, 0x0002, /*0EDB*/
  0x2DCD, 0x0020, 0x0004, 0x2DC1, 0x0020, 0x0004, 0x0002, /*0EDC*/
  0x2DCD, 0x0020, 0x0004, 0x2DC8, 0x0020, 0x0004, 0x0002, /*0EDD*/
  0x2DB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EDE*/
  0x2DBB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*0EDF*/
  0xFBC0, 0x0020, 0x0002, 0x8EE0, 0x0000, 0x0000, 0x0002, /*0EE0*/
  0xFBC0, 0x0020, 0x0002, 0x8EE1, 0x0000, 0x0000, 0x0002, /*0EE1*/
  0xFBC0, 0x0020, 0x0002, 0x8EE2, 0x0000, 0x0000, 0x0002, /*0EE2*/
  0xFBC0, 0x0020, 0x0002, 0x8EE3, 0x0000, 0x0000, 0x0002, /*0EE3*/
  0xFBC0, 0x0020, 0x0002, 0x8EE4, 0x0000, 0x0000, 0x0002, /*0EE4*/
  0xFBC0, 0x0020, 0x0002, 0x8EE5, 0x0000, 0x0000, 0x0002, /*0EE5*/
  0xFBC0, 0x0020, 0x0002, 0x8EE6, 0x0000, 0x0000, 0x0002, /*0EE6*/
  0xFBC0, 0x0020, 0x0002, 0x8EE7, 0x0000, 0x0000, 0x0002, /*0EE7*/
  0xFBC0, 0x0020, 0x0002, 0x8EE8, 0x0000, 0x0000, 0x0002, /*0EE8*/
  0xFBC0, 0x0020, 0x0002, 0x8EE9, 0x0000, 0x0000, 0x0002, /*0EE9*/
  0xFBC0, 0x0020, 0x0002, 0x8EEA, 0x0000, 0x0000, 0x0002, /*0EEA*/
  0xFBC0, 0x0020, 0x0002, 0x8EEB, 0x0000, 0x0000, 0x0002, /*0EEB*/
  0xFBC0, 0x0020, 0x0002, 0x8EEC, 0x0000, 0x0000, 0x0002, /*0EEC*/
  0xFBC0, 0x0020, 0x0002, 0x8EED, 0x0000, 0x0000, 0x0002, /*0EED*/
  0xFBC0, 0x0020, 0x0002, 0x8EEE, 0x0000, 0x0000, 0x0002, /*0EEE*/
  0xFBC0, 0x0020, 0x0002, 0x8EEF, 0x0000, 0x0000, 0x0002, /*0EEF*/
  0xFBC0, 0x0020, 0x0002, 0x8EF0, 0x0000, 0x0000, 0x0002, /*0EF0*/
  0xFBC0, 0x0020, 0x0002, 0x8EF1, 0x0000, 0x0000, 0x0002, /*0EF1*/
  0xFBC0, 0x0020, 0x0002, 0x8EF2, 0x0000, 0x0000, 0x0002, /*0EF2*/
  0xFBC0, 0x0020, 0x0002, 0x8EF3, 0x0000, 0x0000, 0x0002, /*0EF3*/
  0xFBC0, 0x0020, 0x0002, 0x8EF4, 0x0000, 0x0000, 0x0002, /*0EF4*/
  0xFBC0, 0x0020, 0x0002, 0x8EF5, 0x0000, 0x0000, 0x0002, /*0EF5*/
  0xFBC0, 0x0020, 0x0002, 0x8EF6, 0x0000, 0x0000, 0x0002, /*0EF6*/
  0xFBC0, 0x0020, 0x0002, 0x8EF7, 0x0000, 0x0000, 0x0002, /*0EF7*/
  0xFBC0, 0x0020, 0x0002, 0x8EF8, 0x0000, 0x0000, 0x0002, /*0EF8*/
  0xFBC0, 0x0020, 0x0002, 0x8EF9, 0x0000, 0x0000, 0x0002, /*0EF9*/
  0xFBC0, 0x0020, 0x0002, 0x8EFA, 0x0000, 0x0000, 0x0002, /*0EFA*/
  0xFBC0, 0x0020, 0x0002, 0x8EFB, 0x0000, 0x0000, 0x0002, /*0EFB*/
  0xFBC0, 0x0020, 0x0002, 0x8EFC, 0x0000, 0x0000, 0x0002, /*0EFC*/
  0xFBC0, 0x0020, 0x0002, 0x8EFD, 0x0000, 0x0000, 0x0002, /*0EFD*/
  0xFBC0, 0x0020, 0x0002, 0x8EFE, 0x0000, 0x0000, 0x0002, /*0EFE*/
  0xFBC0, 0x0020, 0x0002, 0x8EFF, 0x0000, 0x0000, 0x0002  /*0EFF*/
};

uint16 uca900_p00F[]= {
  0x2E6C, 0x0020, 0x0004, 0x2E83, 0x0020, /*0F00*/
  0x0004, 0x0000, 0x00C4, 0x0004, 0x0003,
  0x0526, 0x0020, 0x0002, 0x0000, 0x0000, /*0F01*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0527, 0x0020, 0x0002, 0x0000, 0x0000, /*0F02*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0528, 0x0020, 0x0002, 0x0000, 0x0000, /*0F03*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03F3, 0x0020, 0x0002, 0x0000, 0x0000, /*0F04*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03F4, 0x0020, 0x0002, 0x0000, 0x0000, /*0F05*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03F5, 0x0020, 0x0002, 0x0000, 0x0000, /*0F06*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03F6, 0x0020, 0x0002, 0x0000, 0x0000, /*0F07*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03F7, 0x0020, 0x0002, 0x0000, 0x0000, /*0F08*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03F8, 0x0020, 0x0002, 0x0000, 0x0000, /*0F09*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03F9, 0x0020, 0x0002, 0x0000, 0x0000, /*0F0A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03FC, 0x0020, 0x0002, 0x0000, 0x0000, /*0F0B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03FC, 0x0020, 0x001B, 0x0000, 0x0000, /*0F0C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03FD, 0x0020, 0x0002, 0x0000, 0x0000, /*0F0D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03FE, 0x0020, 0x0002, 0x0000, 0x0000, /*0F0E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03FF, 0x0020, 0x0002, 0x0000, 0x0000, /*0F0F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0400, 0x0020, 0x0002, 0x0000, 0x0000, /*0F10*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0401, 0x0020, 0x0002, 0x0000, 0x0000, /*0F11*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0402, 0x0020, 0x0002, 0x0000, 0x0000, /*0F12*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0529, 0x0020, 0x0002, 0x0000, 0x0000, /*0F13*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0258, 0x0020, 0x0002, 0x0000, 0x0000, /*0F14*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x052A, 0x0020, 0x0002, 0x0000, 0x0000, /*0F15*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x052B, 0x0020, 0x0002, 0x0000, 0x0000, /*0F16*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x052C, 0x0020, 0x0002, 0x0000, 0x0000, /*0F17*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0F18*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0F19*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x052D, 0x0020, 0x0002, 0x0000, 0x0000, /*0F1A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x052E, 0x0020, 0x0002, 0x0000, 0x0000, /*0F1B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x052F, 0x0020, 0x0002, 0x0000, 0x0000, /*0F1C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0530, 0x0020, 0x0002, 0x0000, 0x0000, /*0F1D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0531, 0x0020, 0x0002, 0x0000, 0x0000, /*0F1E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0532, 0x0020, 0x0002, 0x0000, 0x0000, /*0F1F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, /*0F20*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, /*0F21*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, /*0F22*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, /*0F23*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, /*0F24*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, /*0F25*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, /*0F26*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, /*0F27*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, /*0F28*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, /*0F29*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0004, 0x0000, 0x0000, /*0F2A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0004, 0x0000, 0x0000, /*0F2B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0004, 0x0000, 0x0000, /*0F2C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C41, 0x0020, 0x0004, 0x0000, 0x0000, /*0F2D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0004, 0x0000, 0x0000, /*0F2E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0004, 0x0000, 0x0000, /*0F2F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0004, 0x0000, 0x0000, /*0F30*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0004, 0x0000, 0x0000, /*0F31*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0004, 0x0000, 0x0000, /*0F32*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3D, 0x0020, 0x0004, 0x0000, 0x0000, /*0F33*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0533, 0x0020, 0x0002, 0x0000, 0x0000, /*0F34*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0F35*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0534, 0x0020, 0x0002, 0x0000, 0x0000, /*0F36*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0F37*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0535, 0x0020, 0x0002, 0x0000, 0x0000, /*0F38*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00E1, 0x0002, 0x0000, 0x0000, /*0F39*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x031D, 0x0020, 0x0002, 0x0000, 0x0000, /*0F3A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x031E, 0x0020, 0x0002, 0x0000, 0x0000, /*0F3B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x031F, 0x0020, 0x0002, 0x0000, 0x0000, /*0F3C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0320, 0x0020, 0x0002, 0x0000, 0x0000, /*0F3D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0F3E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0F3F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E26, 0x0020, 0x0002, 0x0000, 0x0000, /*0F40*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E29, 0x0020, 0x0002, 0x0000, 0x0000, /*0F41*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E2B, 0x0020, 0x0002, 0x0000, 0x0000, /*0F42*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E2B, 0x0020, 0x0002, 0x2E6B, 0x0020, /*0F43*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E2D, 0x0020, 0x0002, 0x0000, 0x0000, /*0F44*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E2F, 0x0020, 0x0002, 0x0000, 0x0000, /*0F45*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E31, 0x0020, 0x0002, 0x0000, 0x0000, /*0F46*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E33, 0x0020, 0x0002, 0x0000, 0x0000, /*0F47*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x8F48, 0x0000, /*0F48*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E35, 0x0020, 0x0002, 0x0000, 0x0000, /*0F49*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E37, 0x0020, 0x0002, 0x0000, 0x0000, /*0F4A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E39, 0x0020, 0x0002, 0x0000, 0x0000, /*0F4B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E3B, 0x0020, 0x0002, 0x0000, 0x0000, /*0F4C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E3B, 0x0020, 0x0002, 0x2E6B, 0x0020, /*0F4D*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E3D, 0x0020, 0x0002, 0x0000, 0x0000, /*0F4E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E3F, 0x0020, 0x0002, 0x0000, 0x0000, /*0F4F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E41, 0x0020, 0x0002, 0x0000, 0x0000, /*0F50*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E43, 0x0020, 0x0002, 0x0000, 0x0000, /*0F51*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E43, 0x0020, 0x0002, 0x2E6B, 0x0020, /*0F52*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E45, 0x0020, 0x0002, 0x0000, 0x0000, /*0F53*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E47, 0x0020, 0x0002, 0x0000, 0x0000, /*0F54*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E49, 0x0020, 0x0002, 0x0000, 0x0000, /*0F55*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E4B, 0x0020, 0x0002, 0x0000, 0x0000, /*0F56*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E4B, 0x0020, 0x0002, 0x2E6B, 0x0020, /*0F57*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E4D, 0x0020, 0x0002, 0x0000, 0x0000, /*0F58*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E4F, 0x0020, 0x0002, 0x0000, 0x0000, /*0F59*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E51, 0x0020, 0x0002, 0x0000, 0x0000, /*0F5A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E53, 0x0020, 0x0002, 0x0000, 0x0000, /*0F5B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E53, 0x0020, 0x0002, 0x2E6B, 0x0020, /*0F5C*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E55, 0x0020, 0x0002, 0x0000, 0x0000, /*0F5D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E57, 0x0020, 0x0002, 0x0000, 0x0000, /*0F5E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E59, 0x0020, 0x0002, 0x0000, 0x0000, /*0F5F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E5B, 0x0020, 0x0002, 0x0000, 0x0000, /*0F60*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E5D, 0x0020, 0x0002, 0x0000, 0x0000, /*0F61*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E5F, 0x0020, 0x0002, 0x0000, 0x0000, /*0F62*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E62, 0x0020, 0x0002, 0x0000, 0x0000, /*0F63*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E64, 0x0020, 0x0002, 0x0000, 0x0000, /*0F64*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E66, 0x0020, 0x0002, 0x0000, 0x0000, /*0F65*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E68, 0x0020, 0x0002, 0x0000, 0x0000, /*0F66*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E6A, 0x0020, 0x0002, 0x0000, 0x0000, /*0F67*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E6C, 0x0020, 0x0002, 0x0000, 0x0000, /*0F68*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E26, 0x0020, 0x0002, 0x2E67, 0x0020, /*0F69*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E5F, 0x0020, 0x0004, 0x0000, 0x0111, /*0F6A*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E28, 0x0020, 0x0002, 0x0000, 0x0000, /*0F6B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E61, 0x0020, 0x0002, 0x0000, 0x0000, /*0F6C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x8F6D, 0x0000, /*0F6D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8F6E, 0x0000, /*0F6E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8F6F, 0x0000, /*0F6F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8F70, 0x0000, /*0F70*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E76, 0x0020, 0x0002, 0x0000, 0x0000, /*0F71*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E77, 0x0020, 0x0002, 0x0000, 0x0000, /*0F72*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E78, 0x0020, 0x0002, 0x0000, 0x0000, /*0F73*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E7B, 0x0020, 0x0002, 0x0000, 0x0000, /*0F74*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E7C, 0x0020, 0x0002, 0x0000, 0x0000, /*0F75*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E7D, 0x0020, 0x0002, 0x0000, 0x0000, /*0F76*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E7E, 0x0020, 0x0002, 0x0000, 0x0000, /*0F77*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E7F, 0x0020, 0x0002, 0x0000, 0x0000, /*0F78*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E80, 0x0020, 0x0002, 0x0000, 0x0000, /*0F79*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E81, 0x0020, 0x0002, 0x0000, 0x0000, /*0F7A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E82, 0x0020, 0x0002, 0x0000, 0x0000, /*0F7B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E83, 0x0020, 0x0002, 0x0000, 0x0000, /*0F7C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E84, 0x0020, 0x0002, 0x0000, 0x0000, /*0F7D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, /*0F7E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, /*0F7F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E79, 0x0020, 0x0002, 0x0000, 0x0000, /*0F80*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E7A, 0x0020, 0x0002, 0x0000, 0x0000, /*0F81*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0F82*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0F83*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E85, 0x0020, 0x0002, 0x0000, 0x0000, /*0F84*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0403, 0x0020, 0x0002, 0x0000, 0x0000, /*0F85*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0F86*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0F87*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E6E, 0x0020, 0x0002, 0x0000, 0x0000, /*0F88*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E70, 0x0020, 0x0002, 0x0000, 0x0000, /*0F89*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E74, 0x0020, 0x0002, 0x0000, 0x0000, /*0F8A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E75, 0x0020, 0x0002, 0x0000, 0x0000, /*0F8B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E72, 0x0020, 0x0002, 0x0000, 0x0000, /*0F8C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E6F, 0x0020, 0x0002, 0x0000, 0x0000, /*0F8D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E71, 0x0020, 0x0002, 0x0000, 0x0000, /*0F8E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E73, 0x0020, 0x0002, 0x0000, 0x0000, /*0F8F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E27, 0x0020, 0x0002, 0x0000, 0x0000, /*0F90*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E2A, 0x0020, 0x0002, 0x0000, 0x0000, /*0F91*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E2C, 0x0020, 0x0002, 0x0000, 0x0000, /*0F92*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E2C, 0x0020, 0x0002, 0x2E6B, 0x0020, /*0F93*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E2E, 0x0020, 0x0002, 0x0000, 0x0000, /*0F94*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E30, 0x0020, 0x0002, 0x0000, 0x0000, /*0F95*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E32, 0x0020, 0x0002, 0x0000, 0x0000, /*0F96*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E34, 0x0020, 0x0002, 0x0000, 0x0000, /*0F97*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x8F98, 0x0000, /*0F98*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E36, 0x0020, 0x0002, 0x0000, 0x0000, /*0F99*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E38, 0x0020, 0x0002, 0x0000, 0x0000, /*0F9A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E3A, 0x0020, 0x0002, 0x0000, 0x0000, /*0F9B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E3C, 0x0020, 0x0002, 0x0000, 0x0000, /*0F9C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E3C, 0x0020, 0x0002, 0x2E6B, 0x0020, /*0F9D*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E3E, 0x0020, 0x0002, 0x0000, 0x0000, /*0F9E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E40, 0x0020, 0x0002, 0x0000, 0x0000, /*0F9F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E42, 0x0020, 0x0002, 0x0000, 0x0000, /*0FA0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E44, 0x0020, 0x0002, 0x0000, 0x0000, /*0FA1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E44, 0x0020, 0x0002, 0x2E6B, 0x0020, /*0FA2*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E46, 0x0020, 0x0002, 0x0000, 0x0000, /*0FA3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E48, 0x0020, 0x0002, 0x0000, 0x0000, /*0FA4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E4A, 0x0020, 0x0002, 0x0000, 0x0000, /*0FA5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E4C, 0x0020, 0x0002, 0x0000, 0x0000, /*0FA6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E4C, 0x0020, 0x0002, 0x2E6B, 0x0020, /*0FA7*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E4E, 0x0020, 0x0002, 0x0000, 0x0000, /*0FA8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E50, 0x0020, 0x0002, 0x0000, 0x0000, /*0FA9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E52, 0x0020, 0x0002, 0x0000, 0x0000, /*0FAA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E54, 0x0020, 0x0002, 0x0000, 0x0000, /*0FAB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E54, 0x0020, 0x0002, 0x2E6B, 0x0020, /*0FAC*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E56, 0x0020, 0x0002, 0x0000, 0x0000, /*0FAD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E58, 0x0020, 0x0002, 0x0000, 0x0000, /*0FAE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E5A, 0x0020, 0x0002, 0x0000, 0x0000, /*0FAF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E5C, 0x0020, 0x0002, 0x0000, 0x0000, /*0FB0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E5E, 0x0020, 0x0002, 0x0000, 0x0000, /*0FB1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E60, 0x0020, 0x0002, 0x0000, 0x0000, /*0FB2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E63, 0x0020, 0x0002, 0x0000, 0x0000, /*0FB3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E65, 0x0020, 0x0002, 0x0000, 0x0000, /*0FB4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E67, 0x0020, 0x0002, 0x0000, 0x0000, /*0FB5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E69, 0x0020, 0x0002, 0x0000, 0x0000, /*0FB6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E6B, 0x0020, 0x0002, 0x0000, 0x0000, /*0FB7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E6D, 0x0020, 0x0002, 0x0000, 0x0000, /*0FB8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2E27, 0x0020, 0x0002, 0x2E67, 0x0020, /*0FB9*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E56, 0x0020, 0x0004, 0x0000, 0x0111, /*0FBA*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E5E, 0x0020, 0x0004, 0x0000, 0x0111, /*0FBB*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2E60, 0x0020, 0x0004, 0x0000, 0x0111, /*0FBC*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FBD, 0x0000, /*0FBD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0536, 0x0020, 0x0002, 0x0000, 0x0000, /*0FBE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0537, 0x0020, 0x0002, 0x0000, 0x0000, /*0FBF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0538, 0x0020, 0x0002, 0x0000, 0x0000, /*0FC0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0539, 0x0020, 0x0002, 0x0000, 0x0000, /*0FC1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x053A, 0x0020, 0x0002, 0x0000, 0x0000, /*0FC2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x053B, 0x0020, 0x0002, 0x0000, 0x0000, /*0FC3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x053C, 0x0020, 0x0002, 0x0000, 0x0000, /*0FC4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x053D, 0x0020, 0x0002, 0x0000, 0x0000, /*0FC5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*0FC6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x053E, 0x0020, 0x0002, 0x0000, 0x0000, /*0FC7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x053F, 0x0020, 0x0002, 0x0000, 0x0000, /*0FC8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0540, 0x0020, 0x0002, 0x0000, 0x0000, /*0FC9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0541, 0x0020, 0x0002, 0x0000, 0x0000, /*0FCA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0542, 0x0020, 0x0002, 0x0000, 0x0000, /*0FCB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0543, 0x0020, 0x0002, 0x0000, 0x0000, /*0FCC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x8FCD, 0x0000, /*0FCD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0544, 0x0020, 0x0002, 0x0000, 0x0000, /*0FCE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0545, 0x0020, 0x0002, 0x0000, 0x0000, /*0FCF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03FA, 0x0020, 0x0002, 0x0000, 0x0000, /*0FD0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03FB, 0x0020, 0x0002, 0x0000, 0x0000, /*0FD1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0404, 0x0020, 0x0002, 0x0000, 0x0000, /*0FD2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0405, 0x0020, 0x0002, 0x0000, 0x0000, /*0FD3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0406, 0x0020, 0x0002, 0x0000, 0x0000, /*0FD4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0546, 0x0020, 0x0002, 0x0000, 0x0000, /*0FD5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0547, 0x0020, 0x0002, 0x0000, 0x0000, /*0FD6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0548, 0x0020, 0x0002, 0x0000, 0x0000, /*0FD7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0549, 0x0020, 0x0002, 0x0000, 0x0000, /*0FD8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0407, 0x0020, 0x0002, 0x0000, 0x0000, /*0FD9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0408, 0x0020, 0x0002, 0x0000, 0x0000, /*0FDA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x8FDB, 0x0000, /*0FDB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FDC, 0x0000, /*0FDC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FDD, 0x0000, /*0FDD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FDE, 0x0000, /*0FDE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FDF, 0x0000, /*0FDF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FE0, 0x0000, /*0FE0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FE1, 0x0000, /*0FE1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FE2, 0x0000, /*0FE2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FE3, 0x0000, /*0FE3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FE4, 0x0000, /*0FE4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FE5, 0x0000, /*0FE5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FE6, 0x0000, /*0FE6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FE7, 0x0000, /*0FE7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FE8, 0x0000, /*0FE8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FE9, 0x0000, /*0FE9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FEA, 0x0000, /*0FEA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FEB, 0x0000, /*0FEB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FEC, 0x0000, /*0FEC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FED, 0x0000, /*0FED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FEE, 0x0000, /*0FEE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FEF, 0x0000, /*0FEF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FF0, 0x0000, /*0FF0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FF1, 0x0000, /*0FF1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FF2, 0x0000, /*0FF2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FF3, 0x0000, /*0FF3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FF4, 0x0000, /*0FF4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FF5, 0x0000, /*0FF5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FF6, 0x0000, /*0FF6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FF7, 0x0000, /*0FF7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FF8, 0x0000, /*0FF8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FF9, 0x0000, /*0FF9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FFA, 0x0000, /*0FFA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FFB, 0x0000, /*0FFB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FFC, 0x0000, /*0FFC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FFD, 0x0000, /*0FFD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FFE, 0x0000, /*0FFE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x8FFF, 0x0000, /*0FFF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002
};

uint16 uca900_p010[]= {
  0x3035, 0x0020, 0x0002, 0x0000, 0x0000, /*1000*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3037, 0x0020, 0x0002, 0x0000, 0x0000, /*1001*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3039, 0x0020, 0x0002, 0x0000, 0x0000, /*1002*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x303D, 0x0020, 0x0002, 0x0000, 0x0000, /*1003*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3040, 0x0020, 0x0002, 0x0000, 0x0000, /*1004*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3042, 0x0020, 0x0002, 0x0000, 0x0000, /*1005*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3045, 0x0020, 0x0002, 0x0000, 0x0000, /*1006*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3049, 0x0020, 0x0002, 0x0000, 0x0000, /*1007*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x304E, 0x0020, 0x0002, 0x0000, 0x0000, /*1008*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3055, 0x0020, 0x0002, 0x0000, 0x0000, /*1009*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3059, 0x0020, 0x0002, 0x0000, 0x0000, /*100A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x305A, 0x0020, 0x0002, 0x0000, 0x0000, /*100B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x305C, 0x0020, 0x0002, 0x0000, 0x0000, /*100C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x305E, 0x0020, 0x0002, 0x0000, 0x0000, /*100D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3061, 0x0020, 0x0002, 0x0000, 0x0000, /*100E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3064, 0x0020, 0x0002, 0x0000, 0x0000, /*100F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3068, 0x0020, 0x0002, 0x0000, 0x0000, /*1010*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3069, 0x0020, 0x0002, 0x0000, 0x0000, /*1011*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x306A, 0x0020, 0x0002, 0x0000, 0x0000, /*1012*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x306D, 0x0020, 0x0002, 0x0000, 0x0000, /*1013*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3070, 0x0020, 0x0002, 0x0000, 0x0000, /*1014*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3074, 0x0020, 0x0002, 0x0000, 0x0000, /*1015*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3075, 0x0020, 0x0002, 0x0000, 0x0000, /*1016*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x307B, 0x0020, 0x0002, 0x0000, 0x0000, /*1017*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x307E, 0x0020, 0x0002, 0x0000, 0x0000, /*1018*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3081, 0x0020, 0x0002, 0x0000, 0x0000, /*1019*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3083, 0x0020, 0x0002, 0x0000, 0x0000, /*101A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3085, 0x0020, 0x0002, 0x0000, 0x0000, /*101B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3089, 0x0020, 0x0002, 0x0000, 0x0000, /*101C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x308B, 0x0020, 0x0002, 0x0000, 0x0000, /*101D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3092, 0x0020, 0x0002, 0x0000, 0x0000, /*101E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3094, 0x0020, 0x0002, 0x0000, 0x0000, /*101F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x309A, 0x0020, 0x0002, 0x0000, 0x0000, /*1020*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30A1, 0x0020, 0x0002, 0x0000, 0x0000, /*1021*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30A2, 0x0020, 0x0002, 0x0000, 0x0000, /*1022*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30A3, 0x0020, 0x0002, 0x0000, 0x0000, /*1023*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30A4, 0x0020, 0x0002, 0x0000, 0x0000, /*1024*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30A5, 0x0020, 0x0002, 0x0000, 0x0000, /*1025*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30A6, 0x0020, 0x0002, 0x0000, 0x0000, /*1026*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30AB, 0x0020, 0x0002, 0x0000, 0x0000, /*1027*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30AC, 0x0020, 0x0002, 0x0000, 0x0000, /*1028*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30AD, 0x0020, 0x0002, 0x0000, 0x0000, /*1029*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30AE, 0x0020, 0x0002, 0x0000, 0x0000, /*102A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30AF, 0x0020, 0x0004, 0x0000, 0x0000, /*102B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30AF, 0x0020, 0x0002, 0x0000, 0x0000, /*102C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30B3, 0x0020, 0x0002, 0x0000, 0x0000, /*102D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30B5, 0x0020, 0x0002, 0x0000, 0x0000, /*102E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30B7, 0x0020, 0x0002, 0x0000, 0x0000, /*102F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30BA, 0x0020, 0x0002, 0x0000, 0x0000, /*1030*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30BF, 0x0020, 0x0002, 0x0000, 0x0000, /*1031*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30C3, 0x0020, 0x0002, 0x0000, 0x0000, /*1032*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30B6, 0x0020, 0x0002, 0x0000, 0x0000, /*1033*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30C5, 0x0020, 0x0002, 0x0000, 0x0000, /*1034*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30C1, 0x0020, 0x0002, 0x0000, 0x0000, /*1035*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, /*1036*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00E5, 0x0002, 0x0000, 0x0000, /*1037*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, /*1038*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30CB, 0x0020, 0x0002, 0x0000, 0x0000, /*1039*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30CC, 0x0020, 0x0002, 0x0000, 0x0000, /*103A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3084, 0x0020, 0x0002, 0x0000, 0x0000, /*103B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3088, 0x0020, 0x0002, 0x0000, 0x0000, /*103C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x308C, 0x0020, 0x0002, 0x0000, 0x0000, /*103D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3097, 0x0020, 0x0002, 0x0000, 0x0000, /*103E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3092, 0x0020, 0x0004, 0x30CB, 0x0020, /*103F*/
  0x0004, 0x3092, 0x0020, 0x0004, 0x0003,
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, /*1040*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, /*1041*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, /*1042*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, /*1043*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, /*1044*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, /*1045*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, /*1046*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, /*1047*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, /*1048*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, /*1049*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0299, 0x0020, 0x0002, 0x0000, 0x0000, /*104A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x029A, 0x0020, 0x0002, 0x0000, 0x0000, /*104B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x040E, 0x0020, 0x0002, 0x0000, 0x0000, /*104C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x040F, 0x0020, 0x0002, 0x0000, 0x0000, /*104D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0410, 0x0020, 0x0002, 0x0000, 0x0000, /*104E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0411, 0x0020, 0x0002, 0x0000, 0x0000, /*104F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x308F, 0x0020, 0x0002, 0x0000, 0x0000, /*1050*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3090, 0x0020, 0x0002, 0x0000, 0x0000, /*1051*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30A7, 0x0020, 0x0002, 0x0000, 0x0000, /*1052*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30A8, 0x0020, 0x0002, 0x0000, 0x0000, /*1053*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30A9, 0x0020, 0x0002, 0x0000, 0x0000, /*1054*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30AA, 0x0020, 0x0002, 0x0000, 0x0000, /*1055*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30BB, 0x0020, 0x0002, 0x0000, 0x0000, /*1056*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30BC, 0x0020, 0x0002, 0x0000, 0x0000, /*1057*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30BD, 0x0020, 0x0002, 0x0000, 0x0000, /*1058*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30BE, 0x0020, 0x0002, 0x0000, 0x0000, /*1059*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3041, 0x0020, 0x0002, 0x0000, 0x0000, /*105A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x304F, 0x0020, 0x0002, 0x0000, 0x0000, /*105B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x309C, 0x0020, 0x0002, 0x0000, 0x0000, /*105C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x309D, 0x0020, 0x0002, 0x0000, 0x0000, /*105D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3073, 0x0020, 0x0002, 0x0000, 0x0000, /*105E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3082, 0x0020, 0x0002, 0x0000, 0x0000, /*105F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x308A, 0x0020, 0x0002, 0x0000, 0x0000, /*1060*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3053, 0x0020, 0x0002, 0x0000, 0x0000, /*1061*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30C6, 0x0020, 0x0002, 0x0000, 0x0000, /*1062*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30CD, 0x0020, 0x0002, 0x0000, 0x0000, /*1063*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30CE, 0x0020, 0x0002, 0x0000, 0x0000, /*1064*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3091, 0x0020, 0x0002, 0x0000, 0x0000, /*1065*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30A0, 0x0020, 0x0002, 0x0000, 0x0000, /*1066*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30C7, 0x0020, 0x0002, 0x0000, 0x0000, /*1067*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30C8, 0x0020, 0x0002, 0x0000, 0x0000, /*1068*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30CF, 0x0020, 0x0002, 0x0000, 0x0000, /*1069*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30D0, 0x0020, 0x0002, 0x0000, 0x0000, /*106A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30D1, 0x0020, 0x0002, 0x0000, 0x0000, /*106B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30D2, 0x0020, 0x0002, 0x0000, 0x0000, /*106C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30D3, 0x0020, 0x0002, 0x0000, 0x0000, /*106D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3065, 0x0020, 0x0002, 0x0000, 0x0000, /*106E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x309E, 0x0020, 0x0002, 0x0000, 0x0000, /*106F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x309F, 0x0020, 0x0002, 0x0000, 0x0000, /*1070*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30B4, 0x0020, 0x0002, 0x0000, 0x0000, /*1071*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30B1, 0x0020, 0x0002, 0x0000, 0x0000, /*1072*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30B8, 0x0020, 0x0002, 0x0000, 0x0000, /*1073*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30B9, 0x0020, 0x0002, 0x0000, 0x0000, /*1074*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3036, 0x0020, 0x0002, 0x0000, 0x0000, /*1075*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3038, 0x0020, 0x0002, 0x0000, 0x0000, /*1076*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x303A, 0x0020, 0x0002, 0x0000, 0x0000, /*1077*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3043, 0x0020, 0x0002, 0x0000, 0x0000, /*1078*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x304C, 0x0020, 0x0002, 0x0000, 0x0000, /*1079*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3056, 0x0020, 0x0002, 0x0000, 0x0000, /*107A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x306B, 0x0020, 0x0002, 0x0000, 0x0000, /*107B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3071, 0x0020, 0x0002, 0x0000, 0x0000, /*107C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3076, 0x0020, 0x0002, 0x0000, 0x0000, /*107D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3077, 0x0020, 0x0002, 0x0000, 0x0000, /*107E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x307C, 0x0020, 0x0002, 0x0000, 0x0000, /*107F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x308E, 0x0020, 0x0002, 0x0000, 0x0000, /*1080*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3095, 0x0020, 0x0002, 0x0000, 0x0000, /*1081*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x308D, 0x0020, 0x0002, 0x0000, 0x0000, /*1082*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30B0, 0x0020, 0x0002, 0x0000, 0x0000, /*1083*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30C0, 0x0020, 0x0002, 0x0000, 0x0000, /*1084*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30C2, 0x0020, 0x0002, 0x0000, 0x0000, /*1085*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30CA, 0x0020, 0x0002, 0x0000, 0x0000, /*1086*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30D4, 0x0020, 0x0002, 0x0000, 0x0000, /*1087*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30D6, 0x0020, 0x0002, 0x0000, 0x0000, /*1088*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30D9, 0x0020, 0x0002, 0x0000, 0x0000, /*1089*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30DA, 0x0020, 0x0002, 0x0000, 0x0000, /*108A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30D5, 0x0020, 0x0002, 0x0000, 0x0000, /*108B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30D7, 0x0020, 0x0002, 0x0000, 0x0000, /*108C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30D8, 0x0020, 0x0002, 0x0000, 0x0000, /*108D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3079, 0x0020, 0x0002, 0x0000, 0x0000, /*108E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30DB, 0x0020, 0x0002, 0x0000, 0x0000, /*108F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, /*1090*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, /*1091*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, /*1092*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, /*1093*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, /*1094*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, /*1095*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, /*1096*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, /*1097*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, /*1098*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, /*1099*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30DC, 0x0020, 0x0002, 0x0000, 0x0000, /*109A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30DD, 0x0020, 0x0002, 0x0000, 0x0000, /*109B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30B2, 0x0020, 0x0002, 0x0000, 0x0000, /*109C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x30C4, 0x0020, 0x0002, 0x0000, 0x0000, /*109D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x054B, 0x0020, 0x0002, 0x0000, 0x0000, /*109E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x054C, 0x0020, 0x0002, 0x0000, 0x0000, /*109F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x223B, 0x0020, 0x0008, 0x0000, 0x0000, /*10A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x223D, 0x0020, 0x0008, 0x0000, 0x0000, /*10A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x223F, 0x0020, 0x0008, 0x0000, 0x0000, /*10A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2241, 0x0020, 0x0008, 0x0000, 0x0000, /*10A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2243, 0x0020, 0x0008, 0x0000, 0x0000, /*10A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2245, 0x0020, 0x0008, 0x0000, 0x0000, /*10A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2247, 0x0020, 0x0008, 0x0000, 0x0000, /*10A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x224B, 0x0020, 0x0008, 0x0000, 0x0000, /*10A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x224D, 0x0020, 0x0008, 0x0000, 0x0000, /*10A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x224F, 0x0020, 0x0008, 0x0000, 0x0000, /*10A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2251, 0x0020, 0x0008, 0x0000, 0x0000, /*10AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2253, 0x0020, 0x0008, 0x0000, 0x0000, /*10AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2255, 0x0020, 0x0008, 0x0000, 0x0000, /*10AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2259, 0x0020, 0x0008, 0x0000, 0x0000, /*10AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x225B, 0x0020, 0x0008, 0x0000, 0x0000, /*10AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x225D, 0x0020, 0x0008, 0x0000, 0x0000, /*10AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x225F, 0x0020, 0x0008, 0x0000, 0x0000, /*10B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2261, 0x0020, 0x0008, 0x0000, 0x0000, /*10B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2263, 0x0020, 0x0008, 0x0000, 0x0000, /*10B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2267, 0x0020, 0x0008, 0x0000, 0x0000, /*10B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2269, 0x0020, 0x0008, 0x0000, 0x0000, /*10B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x226B, 0x0020, 0x0008, 0x0000, 0x0000, /*10B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x226D, 0x0020, 0x0008, 0x0000, 0x0000, /*10B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x226F, 0x0020, 0x0008, 0x0000, 0x0000, /*10B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2271, 0x0020, 0x0008, 0x0000, 0x0000, /*10B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2273, 0x0020, 0x0008, 0x0000, 0x0000, /*10B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2275, 0x0020, 0x0008, 0x0000, 0x0000, /*10BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2277, 0x0020, 0x0008, 0x0000, 0x0000, /*10BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2279, 0x0020, 0x0008, 0x0000, 0x0000, /*10BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x227B, 0x0020, 0x0008, 0x0000, 0x0000, /*10BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x227D, 0x0020, 0x0008, 0x0000, 0x0000, /*10BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2281, 0x0020, 0x0008, 0x0000, 0x0000, /*10BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2283, 0x0020, 0x0008, 0x0000, 0x0000, /*10C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2249, 0x0020, 0x0008, 0x0000, 0x0000, /*10C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2257, 0x0020, 0x0008, 0x0000, 0x0000, /*10C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2265, 0x0020, 0x0008, 0x0000, 0x0000, /*10C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x227F, 0x0020, 0x0008, 0x0000, 0x0000, /*10C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2285, 0x0020, 0x0008, 0x0000, 0x0000, /*10C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x90C6, 0x0000, /*10C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2288, 0x0020, 0x0008, 0x0000, 0x0000, /*10C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x90C8, 0x0000, /*10C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x90C9, 0x0000, /*10C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x90CA, 0x0000, /*10CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x90CB, 0x0000, /*10CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x90CC, 0x0000, /*10CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x228D, 0x0020, 0x0008, 0x0000, 0x0000, /*10CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x90CE, 0x0000, /*10CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x90CF, 0x0000, /*10CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x223A, 0x0020, 0x0002, 0x0000, 0x0000, /*10D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x223C, 0x0020, 0x0002, 0x0000, 0x0000, /*10D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x223E, 0x0020, 0x0002, 0x0000, 0x0000, /*10D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2240, 0x0020, 0x0002, 0x0000, 0x0000, /*10D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2242, 0x0020, 0x0002, 0x0000, 0x0000, /*10D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2244, 0x0020, 0x0002, 0x0000, 0x0000, /*10D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2246, 0x0020, 0x0002, 0x0000, 0x0000, /*10D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x224A, 0x0020, 0x0002, 0x0000, 0x0000, /*10D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x224C, 0x0020, 0x0002, 0x0000, 0x0000, /*10D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x224E, 0x0020, 0x0002, 0x0000, 0x0000, /*10D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2250, 0x0020, 0x0002, 0x0000, 0x0000, /*10DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2252, 0x0020, 0x0002, 0x0000, 0x0000, /*10DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2254, 0x0020, 0x0002, 0x0000, 0x0000, /*10DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2258, 0x0020, 0x0002, 0x0000, 0x0000, /*10DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x225A, 0x0020, 0x0002, 0x0000, 0x0000, /*10DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x225C, 0x0020, 0x0002, 0x0000, 0x0000, /*10DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x225E, 0x0020, 0x0002, 0x0000, 0x0000, /*10E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2260, 0x0020, 0x0002, 0x0000, 0x0000, /*10E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2262, 0x0020, 0x0002, 0x0000, 0x0000, /*10E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2266, 0x0020, 0x0002, 0x0000, 0x0000, /*10E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2268, 0x0020, 0x0002, 0x0000, 0x0000, /*10E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x226A, 0x0020, 0x0002, 0x0000, 0x0000, /*10E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x226C, 0x0020, 0x0002, 0x0000, 0x0000, /*10E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x226E, 0x0020, 0x0002, 0x0000, 0x0000, /*10E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2270, 0x0020, 0x0002, 0x0000, 0x0000, /*10E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2272, 0x0020, 0x0002, 0x0000, 0x0000, /*10E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2274, 0x0020, 0x0002, 0x0000, 0x0000, /*10EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2276, 0x0020, 0x0002, 0x0000, 0x0000, /*10EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2278, 0x0020, 0x0002, 0x0000, 0x0000, /*10EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x227A, 0x0020, 0x0002, 0x0000, 0x0000, /*10ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x227C, 0x0020, 0x0002, 0x0000, 0x0000, /*10EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2280, 0x0020, 0x0002, 0x0000, 0x0000, /*10EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2282, 0x0020, 0x0002, 0x0000, 0x0000, /*10F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2248, 0x0020, 0x0002, 0x0000, 0x0000, /*10F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2256, 0x0020, 0x0002, 0x0000, 0x0000, /*10F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2264, 0x0020, 0x0002, 0x0000, 0x0000, /*10F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x227E, 0x0020, 0x0002, 0x0000, 0x0000, /*10F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2284, 0x0020, 0x0002, 0x0000, 0x0000, /*10F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2286, 0x0020, 0x0002, 0x0000, 0x0000, /*10F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2287, 0x0020, 0x0002, 0x0000, 0x0000, /*10F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2289, 0x0020, 0x0002, 0x0000, 0x0000, /*10F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x228A, 0x0020, 0x0002, 0x0000, 0x0000, /*10F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x228B, 0x0020, 0x0002, 0x0000, 0x0000, /*10FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02C6, 0x0020, 0x0002, 0x0000, 0x0000, /*10FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2254, 0x0020, 0x0014, 0x0000, 0x0000, /*10FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x228C, 0x0020, 0x0002, 0x0000, 0x0000, /*10FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x228E, 0x0020, 0x0002, 0x0000, 0x0000, /*10FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x228F, 0x0020, 0x0002, 0x0000, 0x0000, /*10FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p011[]= {
  0x3BF5, 0x0020, 0x0002, 0x0001, /*1100*/
  0x3BF6, 0x0020, 0x0002, 0x0001, /*1101*/
  0x3BF7, 0x0020, 0x0002, 0x0001, /*1102*/
  0x3BF8, 0x0020, 0x0002, 0x0001, /*1103*/
  0x3BF9, 0x0020, 0x0002, 0x0001, /*1104*/
  0x3BFA, 0x0020, 0x0002, 0x0001, /*1105*/
  0x3BFB, 0x0020, 0x0002, 0x0001, /*1106*/
  0x3BFC, 0x0020, 0x0002, 0x0001, /*1107*/
  0x3BFD, 0x0020, 0x0002, 0x0001, /*1108*/
  0x3BFE, 0x0020, 0x0002, 0x0001, /*1109*/
  0x3BFF, 0x0020, 0x0002, 0x0001, /*110A*/
  0x3C00, 0x0020, 0x0002, 0x0001, /*110B*/
  0x3C01, 0x0020, 0x0002, 0x0001, /*110C*/
  0x3C02, 0x0020, 0x0002, 0x0001, /*110D*/
  0x3C03, 0x0020, 0x0002, 0x0001, /*110E*/
  0x3C04, 0x0020, 0x0002, 0x0001, /*110F*/
  0x3C05, 0x0020, 0x0002, 0x0001, /*1110*/
  0x3C06, 0x0020, 0x0002, 0x0001, /*1111*/
  0x3C07, 0x0020, 0x0002, 0x0001, /*1112*/
  0x3C08, 0x0020, 0x0002, 0x0001, /*1113*/
  0x3C09, 0x0020, 0x0002, 0x0001, /*1114*/
  0x3C0A, 0x0020, 0x0002, 0x0001, /*1115*/
  0x3C0B, 0x0020, 0x0002, 0x0001, /*1116*/
  0x3C0C, 0x0020, 0x0002, 0x0001, /*1117*/
  0x3C0D, 0x0020, 0x0002, 0x0001, /*1118*/
  0x3C0E, 0x0020, 0x0002, 0x0001, /*1119*/
  0x3C0F, 0x0020, 0x0002, 0x0001, /*111A*/
  0x3C10, 0x0020, 0x0002, 0x0001, /*111B*/
  0x3C11, 0x0020, 0x0002, 0x0001, /*111C*/
  0x3C12, 0x0020, 0x0002, 0x0001, /*111D*/
  0x3C13, 0x0020, 0x0002, 0x0001, /*111E*/
  0x3C14, 0x0020, 0x0002, 0x0001, /*111F*/
  0x3C15, 0x0020, 0x0002, 0x0001, /*1120*/
  0x3C16, 0x0020, 0x0002, 0x0001, /*1121*/
  0x3C17, 0x0020, 0x0002, 0x0001, /*1122*/
  0x3C18, 0x0020, 0x0002, 0x0001, /*1123*/
  0x3C19, 0x0020, 0x0002, 0x0001, /*1124*/
  0x3C1A, 0x0020, 0x0002, 0x0001, /*1125*/
  0x3C1B, 0x0020, 0x0002, 0x0001, /*1126*/
  0x3C1C, 0x0020, 0x0002, 0x0001, /*1127*/
  0x3C1D, 0x0020, 0x0002, 0x0001, /*1128*/
  0x3C1E, 0x0020, 0x0002, 0x0001, /*1129*/
  0x3C1F, 0x0020, 0x0002, 0x0001, /*112A*/
  0x3C20, 0x0020, 0x0002, 0x0001, /*112B*/
  0x3C21, 0x0020, 0x0002, 0x0001, /*112C*/
  0x3C22, 0x0020, 0x0002, 0x0001, /*112D*/
  0x3C23, 0x0020, 0x0002, 0x0001, /*112E*/
  0x3C24, 0x0020, 0x0002, 0x0001, /*112F*/
  0x3C25, 0x0020, 0x0002, 0x0001, /*1130*/
  0x3C26, 0x0020, 0x0002, 0x0001, /*1131*/
  0x3C27, 0x0020, 0x0002, 0x0001, /*1132*/
  0x3C28, 0x0020, 0x0002, 0x0001, /*1133*/
  0x3C29, 0x0020, 0x0002, 0x0001, /*1134*/
  0x3C2A, 0x0020, 0x0002, 0x0001, /*1135*/
  0x3C2B, 0x0020, 0x0002, 0x0001, /*1136*/
  0x3C2C, 0x0020, 0x0002, 0x0001, /*1137*/
  0x3C2D, 0x0020, 0x0002, 0x0001, /*1138*/
  0x3C2E, 0x0020, 0x0002, 0x0001, /*1139*/
  0x3C2F, 0x0020, 0x0002, 0x0001, /*113A*/
  0x3C30, 0x0020, 0x0002, 0x0001, /*113B*/
  0x3C31, 0x0020, 0x0002, 0x0001, /*113C*/
  0x3C32, 0x0020, 0x0002, 0x0001, /*113D*/
  0x3C33, 0x0020, 0x0002, 0x0001, /*113E*/
  0x3C34, 0x0020, 0x0002, 0x0001, /*113F*/
  0x3C35, 0x0020, 0x0002, 0x0001, /*1140*/
  0x3C36, 0x0020, 0x0002, 0x0001, /*1141*/
  0x3C37, 0x0020, 0x0002, 0x0001, /*1142*/
  0x3C38, 0x0020, 0x0002, 0x0001, /*1143*/
  0x3C39, 0x0020, 0x0002, 0x0001, /*1144*/
  0x3C3A, 0x0020, 0x0002, 0x0001, /*1145*/
  0x3C3B, 0x0020, 0x0002, 0x0001, /*1146*/
  0x3C3C, 0x0020, 0x0002, 0x0001, /*1147*/
  0x3C3D, 0x0020, 0x0002, 0x0001, /*1148*/
  0x3C3E, 0x0020, 0x0002, 0x0001, /*1149*/
  0x3C3F, 0x0020, 0x0002, 0x0001, /*114A*/
  0x3C40, 0x0020, 0x0002, 0x0001, /*114B*/
  0x3C41, 0x0020, 0x0002, 0x0001, /*114C*/
  0x3C42, 0x0020, 0x0002, 0x0001, /*114D*/
  0x3C43, 0x0020, 0x0002, 0x0001, /*114E*/
  0x3C44, 0x0020, 0x0002, 0x0001, /*114F*/
  0x3C45, 0x0020, 0x0002, 0x0001, /*1150*/
  0x3C46, 0x0020, 0x0002, 0x0001, /*1151*/
  0x3C47, 0x0020, 0x0002, 0x0001, /*1152*/
  0x3C48, 0x0020, 0x0002, 0x0001, /*1153*/
  0x3C49, 0x0020, 0x0002, 0x0001, /*1154*/
  0x3C4A, 0x0020, 0x0002, 0x0001, /*1155*/
  0x3C4B, 0x0020, 0x0002, 0x0001, /*1156*/
  0x3C4C, 0x0020, 0x0002, 0x0001, /*1157*/
  0x3C4D, 0x0020, 0x0002, 0x0001, /*1158*/
  0x3C4E, 0x0020, 0x0002, 0x0001, /*1159*/
  0x3C4F, 0x0020, 0x0002, 0x0001, /*115A*/
  0x3C50, 0x0020, 0x0002, 0x0001, /*115B*/
  0x3C51, 0x0020, 0x0002, 0x0001, /*115C*/
  0x3C52, 0x0020, 0x0002, 0x0001, /*115D*/
  0x3C53, 0x0020, 0x0002, 0x0001, /*115E*/
  0x3C71, 0x0020, 0x0002, 0x0001, /*115F*/
  0x3C72, 0x0020, 0x0002, 0x0001, /*1160*/
  0x3C73, 0x0020, 0x0002, 0x0001, /*1161*/
  0x3C74, 0x0020, 0x0002, 0x0001, /*1162*/
  0x3C75, 0x0020, 0x0002, 0x0001, /*1163*/
  0x3C76, 0x0020, 0x0002, 0x0001, /*1164*/
  0x3C77, 0x0020, 0x0002, 0x0001, /*1165*/
  0x3C78, 0x0020, 0x0002, 0x0001, /*1166*/
  0x3C79, 0x0020, 0x0002, 0x0001, /*1167*/
  0x3C7A, 0x0020, 0x0002, 0x0001, /*1168*/
  0x3C7B, 0x0020, 0x0002, 0x0001, /*1169*/
  0x3C7C, 0x0020, 0x0002, 0x0001, /*116A*/
  0x3C7D, 0x0020, 0x0002, 0x0001, /*116B*/
  0x3C7E, 0x0020, 0x0002, 0x0001, /*116C*/
  0x3C7F, 0x0020, 0x0002, 0x0001, /*116D*/
  0x3C80, 0x0020, 0x0002, 0x0001, /*116E*/
  0x3C81, 0x0020, 0x0002, 0x0001, /*116F*/
  0x3C82, 0x0020, 0x0002, 0x0001, /*1170*/
  0x3C83, 0x0020, 0x0002, 0x0001, /*1171*/
  0x3C84, 0x0020, 0x0002, 0x0001, /*1172*/
  0x3C85, 0x0020, 0x0002, 0x0001, /*1173*/
  0x3C86, 0x0020, 0x0002, 0x0001, /*1174*/
  0x3C87, 0x0020, 0x0002, 0x0001, /*1175*/
  0x3C88, 0x0020, 0x0002, 0x0001, /*1176*/
  0x3C89, 0x0020, 0x0002, 0x0001, /*1177*/
  0x3C8A, 0x0020, 0x0002, 0x0001, /*1178*/
  0x3C8B, 0x0020, 0x0002, 0x0001, /*1179*/
  0x3C8C, 0x0020, 0x0002, 0x0001, /*117A*/
  0x3C8D, 0x0020, 0x0002, 0x0001, /*117B*/
  0x3C8E, 0x0020, 0x0002, 0x0001, /*117C*/
  0x3C8F, 0x0020, 0x0002, 0x0001, /*117D*/
  0x3C90, 0x0020, 0x0002, 0x0001, /*117E*/
  0x3C91, 0x0020, 0x0002, 0x0001, /*117F*/
  0x3C92, 0x0020, 0x0002, 0x0001, /*1180*/
  0x3C93, 0x0020, 0x0002, 0x0001, /*1181*/
  0x3C94, 0x0020, 0x0002, 0x0001, /*1182*/
  0x3C95, 0x0020, 0x0002, 0x0001, /*1183*/
  0x3C96, 0x0020, 0x0002, 0x0001, /*1184*/
  0x3C97, 0x0020, 0x0002, 0x0001, /*1185*/
  0x3C98, 0x0020, 0x0002, 0x0001, /*1186*/
  0x3C99, 0x0020, 0x0002, 0x0001, /*1187*/
  0x3C9A, 0x0020, 0x0002, 0x0001, /*1188*/
  0x3C9B, 0x0020, 0x0002, 0x0001, /*1189*/
  0x3C9C, 0x0020, 0x0002, 0x0001, /*118A*/
  0x3C9D, 0x0020, 0x0002, 0x0001, /*118B*/
  0x3C9E, 0x0020, 0x0002, 0x0001, /*118C*/
  0x3C9F, 0x0020, 0x0002, 0x0001, /*118D*/
  0x3CA0, 0x0020, 0x0002, 0x0001, /*118E*/
  0x3CA1, 0x0020, 0x0002, 0x0001, /*118F*/
  0x3CA2, 0x0020, 0x0002, 0x0001, /*1190*/
  0x3CA3, 0x0020, 0x0002, 0x0001, /*1191*/
  0x3CA4, 0x0020, 0x0002, 0x0001, /*1192*/
  0x3CA5, 0x0020, 0x0002, 0x0001, /*1193*/
  0x3CA6, 0x0020, 0x0002, 0x0001, /*1194*/
  0x3CA7, 0x0020, 0x0002, 0x0001, /*1195*/
  0x3CA8, 0x0020, 0x0002, 0x0001, /*1196*/
  0x3CA9, 0x0020, 0x0002, 0x0001, /*1197*/
  0x3CAA, 0x0020, 0x0002, 0x0001, /*1198*/
  0x3CAB, 0x0020, 0x0002, 0x0001, /*1199*/
  0x3CAC, 0x0020, 0x0002, 0x0001, /*119A*/
  0x3CAD, 0x0020, 0x0002, 0x0001, /*119B*/
  0x3CAE, 0x0020, 0x0002, 0x0001, /*119C*/
  0x3CAF, 0x0020, 0x0002, 0x0001, /*119D*/
  0x3CB0, 0x0020, 0x0002, 0x0001, /*119E*/
  0x3CB1, 0x0020, 0x0002, 0x0001, /*119F*/
  0x3CB2, 0x0020, 0x0002, 0x0001, /*11A0*/
  0x3CB3, 0x0020, 0x0002, 0x0001, /*11A1*/
  0x3CB4, 0x0020, 0x0002, 0x0001, /*11A2*/
  0x3CB5, 0x0020, 0x0002, 0x0001, /*11A3*/
  0x3CB6, 0x0020, 0x0002, 0x0001, /*11A4*/
  0x3CB7, 0x0020, 0x0002, 0x0001, /*11A5*/
  0x3CB8, 0x0020, 0x0002, 0x0001, /*11A6*/
  0x3CB9, 0x0020, 0x0002, 0x0001, /*11A7*/
  0x3CD1, 0x0020, 0x0002, 0x0001, /*11A8*/
  0x3CD2, 0x0020, 0x0002, 0x0001, /*11A9*/
  0x3CD3, 0x0020, 0x0002, 0x0001, /*11AA*/
  0x3CD4, 0x0020, 0x0002, 0x0001, /*11AB*/
  0x3CD5, 0x0020, 0x0002, 0x0001, /*11AC*/
  0x3CD6, 0x0020, 0x0002, 0x0001, /*11AD*/
  0x3CD7, 0x0020, 0x0002, 0x0001, /*11AE*/
  0x3CD8, 0x0020, 0x0002, 0x0001, /*11AF*/
  0x3CD9, 0x0020, 0x0002, 0x0001, /*11B0*/
  0x3CDA, 0x0020, 0x0002, 0x0001, /*11B1*/
  0x3CDB, 0x0020, 0x0002, 0x0001, /*11B2*/
  0x3CDC, 0x0020, 0x0002, 0x0001, /*11B3*/
  0x3CDD, 0x0020, 0x0002, 0x0001, /*11B4*/
  0x3CDE, 0x0020, 0x0002, 0x0001, /*11B5*/
  0x3CDF, 0x0020, 0x0002, 0x0001, /*11B6*/
  0x3CE0, 0x0020, 0x0002, 0x0001, /*11B7*/
  0x3CE1, 0x0020, 0x0002, 0x0001, /*11B8*/
  0x3CE2, 0x0020, 0x0002, 0x0001, /*11B9*/
  0x3CE3, 0x0020, 0x0002, 0x0001, /*11BA*/
  0x3CE4, 0x0020, 0x0002, 0x0001, /*11BB*/
  0x3CE5, 0x0020, 0x0002, 0x0001, /*11BC*/
  0x3CE6, 0x0020, 0x0002, 0x0001, /*11BD*/
  0x3CE7, 0x0020, 0x0002, 0x0001, /*11BE*/
  0x3CE8, 0x0020, 0x0002, 0x0001, /*11BF*/
  0x3CE9, 0x0020, 0x0002, 0x0001, /*11C0*/
  0x3CEA, 0x0020, 0x0002, 0x0001, /*11C1*/
  0x3CEB, 0x0020, 0x0002, 0x0001, /*11C2*/
  0x3CEC, 0x0020, 0x0002, 0x0001, /*11C3*/
  0x3CED, 0x0020, 0x0002, 0x0001, /*11C4*/
  0x3CEE, 0x0020, 0x0002, 0x0001, /*11C5*/
  0x3CEF, 0x0020, 0x0002, 0x0001, /*11C6*/
  0x3CF0, 0x0020, 0x0002, 0x0001, /*11C7*/
  0x3CF1, 0x0020, 0x0002, 0x0001, /*11C8*/
  0x3CF2, 0x0020, 0x0002, 0x0001, /*11C9*/
  0x3CF3, 0x0020, 0x0002, 0x0001, /*11CA*/
  0x3CF4, 0x0020, 0x0002, 0x0001, /*11CB*/
  0x3CF5, 0x0020, 0x0002, 0x0001, /*11CC*/
  0x3CF6, 0x0020, 0x0002, 0x0001, /*11CD*/
  0x3CF7, 0x0020, 0x0002, 0x0001, /*11CE*/
  0x3CF8, 0x0020, 0x0002, 0x0001, /*11CF*/
  0x3CF9, 0x0020, 0x0002, 0x0001, /*11D0*/
  0x3CFA, 0x0020, 0x0002, 0x0001, /*11D1*/
  0x3CFB, 0x0020, 0x0002, 0x0001, /*11D2*/
  0x3CFC, 0x0020, 0x0002, 0x0001, /*11D3*/
  0x3CFD, 0x0020, 0x0002, 0x0001, /*11D4*/
  0x3CFE, 0x0020, 0x0002, 0x0001, /*11D5*/
  0x3CFF, 0x0020, 0x0002, 0x0001, /*11D6*/
  0x3D00, 0x0020, 0x0002, 0x0001, /*11D7*/
  0x3D01, 0x0020, 0x0002, 0x0001, /*11D8*/
  0x3D02, 0x0020, 0x0002, 0x0001, /*11D9*/
  0x3D03, 0x0020, 0x0002, 0x0001, /*11DA*/
  0x3D04, 0x0020, 0x0002, 0x0001, /*11DB*/
  0x3D05, 0x0020, 0x0002, 0x0001, /*11DC*/
  0x3D06, 0x0020, 0x0002, 0x0001, /*11DD*/
  0x3D07, 0x0020, 0x0002, 0x0001, /*11DE*/
  0x3D08, 0x0020, 0x0002, 0x0001, /*11DF*/
  0x3D09, 0x0020, 0x0002, 0x0001, /*11E0*/
  0x3D0A, 0x0020, 0x0002, 0x0001, /*11E1*/
  0x3D0B, 0x0020, 0x0002, 0x0001, /*11E2*/
  0x3D0C, 0x0020, 0x0002, 0x0001, /*11E3*/
  0x3D0D, 0x0020, 0x0002, 0x0001, /*11E4*/
  0x3D0E, 0x0020, 0x0002, 0x0001, /*11E5*/
  0x3D0F, 0x0020, 0x0002, 0x0001, /*11E6*/
  0x3D10, 0x0020, 0x0002, 0x0001, /*11E7*/
  0x3D11, 0x0020, 0x0002, 0x0001, /*11E8*/
  0x3D12, 0x0020, 0x0002, 0x0001, /*11E9*/
  0x3D13, 0x0020, 0x0002, 0x0001, /*11EA*/
  0x3D14, 0x0020, 0x0002, 0x0001, /*11EB*/
  0x3D15, 0x0020, 0x0002, 0x0001, /*11EC*/
  0x3D16, 0x0020, 0x0002, 0x0001, /*11ED*/
  0x3D17, 0x0020, 0x0002, 0x0001, /*11EE*/
  0x3D18, 0x0020, 0x0002, 0x0001, /*11EF*/
  0x3D19, 0x0020, 0x0002, 0x0001, /*11F0*/
  0x3D1A, 0x0020, 0x0002, 0x0001, /*11F1*/
  0x3D1B, 0x0020, 0x0002, 0x0001, /*11F2*/
  0x3D1C, 0x0020, 0x0002, 0x0001, /*11F3*/
  0x3D1D, 0x0020, 0x0002, 0x0001, /*11F4*/
  0x3D1E, 0x0020, 0x0002, 0x0001, /*11F5*/
  0x3D1F, 0x0020, 0x0002, 0x0001, /*11F6*/
  0x3D20, 0x0020, 0x0002, 0x0001, /*11F7*/
  0x3D21, 0x0020, 0x0002, 0x0001, /*11F8*/
  0x3D22, 0x0020, 0x0002, 0x0001, /*11F9*/
  0x3D23, 0x0020, 0x0002, 0x0001, /*11FA*/
  0x3D24, 0x0020, 0x0002, 0x0001, /*11FB*/
  0x3D25, 0x0020, 0x0002, 0x0001, /*11FC*/
  0x3D26, 0x0020, 0x0002, 0x0001, /*11FD*/
  0x3D27, 0x0020, 0x0002, 0x0001, /*11FE*/
  0x3D28, 0x0020, 0x0002, 0x0001  /*11FF*/
};

uint16 uca900_p012[]= {
  0x2496, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1200*/
  0x2497, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1201*/
  0x2498, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1202*/
  0x2499, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1203*/
  0x249A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1204*/
  0x249B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1205*/
  0x249C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1206*/
  0x249D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1207*/
  0x249E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1208*/
  0x249F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1209*/
  0x24A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*120A*/
  0x24A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*120B*/
  0x24A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*120C*/
  0x24A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*120D*/
  0x24A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*120E*/
  0x24A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*120F*/
  0x24A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1210*/
  0x24A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1211*/
  0x24A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1212*/
  0x24AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1213*/
  0x24AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1214*/
  0x24AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1215*/
  0x24AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1216*/
  0x24AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1217*/
  0x24AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1218*/
  0x24B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1219*/
  0x24B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*121A*/
  0x24B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*121B*/
  0x24B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*121C*/
  0x24B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*121D*/
  0x24B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*121E*/
  0x24B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*121F*/
  0x24BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1220*/
  0x24BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1221*/
  0x24BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1222*/
  0x24BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1223*/
  0x24C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1224*/
  0x24C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1225*/
  0x24C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1226*/
  0x24C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1227*/
  0x24C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1228*/
  0x24C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1229*/
  0x24C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*122A*/
  0x24C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*122B*/
  0x24C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*122C*/
  0x24C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*122D*/
  0x24CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*122E*/
  0x24CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*122F*/
  0x24CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1230*/
  0x24CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1231*/
  0x24CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1232*/
  0x24D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1233*/
  0x24D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1234*/
  0x24D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1235*/
  0x24D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1236*/
  0x24D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1237*/
  0x24DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1238*/
  0x24DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1239*/
  0x24DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*123A*/
  0x24DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*123B*/
  0x24E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*123C*/
  0x24E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*123D*/
  0x24E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*123E*/
  0x24E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*123F*/
  0x24E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1240*/
  0x24E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1241*/
  0x24E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1242*/
  0x24E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1243*/
  0x24E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1244*/
  0x24EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1245*/
  0x24EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1246*/
  0x24EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1247*/
  0x24ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1248*/
  0xFBC0, 0x0020, 0x0002, 0x9249, 0x0000, 0x0000, 0x0002, /*1249*/
  0x24EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124A*/
  0x24EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124B*/
  0x24F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124C*/
  0x24F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124D*/
  0xFBC0, 0x0020, 0x0002, 0x924E, 0x0000, 0x0000, 0x0002, /*124E*/
  0xFBC0, 0x0020, 0x0002, 0x924F, 0x0000, 0x0000, 0x0002, /*124F*/
  0x24F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1250*/
  0x24F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1251*/
  0x24F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1252*/
  0x24F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1253*/
  0x24F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1254*/
  0x24F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1255*/
  0x24F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1256*/
  0xFBC0, 0x0020, 0x0002, 0x9257, 0x0000, 0x0000, 0x0002, /*1257*/
  0x24F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1258*/
  0xFBC0, 0x0020, 0x0002, 0x9259, 0x0000, 0x0000, 0x0002, /*1259*/
  0x24FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*125A*/
  0x24FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*125B*/
  0x24FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*125C*/
  0x24FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*125D*/
  0xFBC0, 0x0020, 0x0002, 0x925E, 0x0000, 0x0000, 0x0002, /*125E*/
  0xFBC0, 0x0020, 0x0002, 0x925F, 0x0000, 0x0000, 0x0002, /*125F*/
  0x24FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1260*/
  0x24FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1261*/
  0x2500, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1262*/
  0x2501, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1263*/
  0x2502, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1264*/
  0x2503, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1265*/
  0x2504, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1266*/
  0x2505, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1267*/
  0x250B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1268*/
  0x250C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1269*/
  0x250D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*126A*/
  0x250E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*126B*/
  0x250F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*126C*/
  0x2510, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*126D*/
  0x2511, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*126E*/
  0x2512, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*126F*/
  0x2513, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1270*/
  0x2514, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1271*/
  0x2515, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1272*/
  0x2516, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1273*/
  0x2517, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1274*/
  0x2518, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1275*/
  0x2519, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1276*/
  0x251A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1277*/
  0x251C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1278*/
  0x251D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1279*/
  0x251E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*127A*/
  0x251F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*127B*/
  0x2520, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*127C*/
  0x2521, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*127D*/
  0x2522, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*127E*/
  0x2523, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*127F*/
  0x2525, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1280*/
  0x2526, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1281*/
  0x2527, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1282*/
  0x2528, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1283*/
  0x2529, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1284*/
  0x252A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1285*/
  0x252B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1286*/
  0x252C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1287*/
  0x252D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1288*/
  0xFBC0, 0x0020, 0x0002, 0x9289, 0x0000, 0x0000, 0x0002, /*1289*/
  0x252E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*128A*/
  0x252F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*128B*/
  0x2530, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*128C*/
  0x2531, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*128D*/
  0xFBC0, 0x0020, 0x0002, 0x928E, 0x0000, 0x0000, 0x0002, /*128E*/
  0xFBC0, 0x0020, 0x0002, 0x928F, 0x0000, 0x0000, 0x0002, /*128F*/
  0x2532, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1290*/
  0x2533, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1291*/
  0x2534, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1292*/
  0x2535, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1293*/
  0x2536, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1294*/
  0x2537, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1295*/
  0x2538, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1296*/
  0x2539, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1297*/
  0x253B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1298*/
  0x253C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1299*/
  0x253D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*129A*/
  0x253E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*129B*/
  0x253F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*129C*/
  0x2540, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*129D*/
  0x2541, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*129E*/
  0x2542, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*129F*/
  0x2544, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12A0*/
  0x2545, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12A1*/
  0x2546, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12A2*/
  0x2547, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12A3*/
  0x2548, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12A4*/
  0x2549, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12A5*/
  0x254A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12A6*/
  0x254B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12A7*/
  0x254D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12A8*/
  0x254E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12A9*/
  0x254F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12AA*/
  0x2550, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12AB*/
  0x2551, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12AC*/
  0x2552, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12AD*/
  0x2553, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12AE*/
  0x2554, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12AF*/
  0x2555, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12B0*/
  0xFBC0, 0x0020, 0x0002, 0x92B1, 0x0000, 0x0000, 0x0002, /*12B1*/
  0x2556, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12B2*/
  0x2557, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12B3*/
  0x2558, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12B4*/
  0x2559, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12B5*/
  0xFBC0, 0x0020, 0x0002, 0x92B6, 0x0000, 0x0000, 0x0002, /*12B6*/
  0xFBC0, 0x0020, 0x0002, 0x92B7, 0x0000, 0x0000, 0x0002, /*12B7*/
  0x255A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12B8*/
  0x255B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12B9*/
  0x255C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12BA*/
  0x255D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12BB*/
  0x255E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12BC*/
  0x255F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12BD*/
  0x2560, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12BE*/
  0xFBC0, 0x0020, 0x0002, 0x92BF, 0x0000, 0x0000, 0x0002, /*12BF*/
  0x2561, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12C0*/
  0xFBC0, 0x0020, 0x0002, 0x92C1, 0x0000, 0x0000, 0x0002, /*12C1*/
  0x2562, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12C2*/
  0x2563, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12C3*/
  0x2564, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12C4*/
  0x2565, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12C5*/
  0xFBC0, 0x0020, 0x0002, 0x92C6, 0x0000, 0x0000, 0x0002, /*12C6*/
  0xFBC0, 0x0020, 0x0002, 0x92C7, 0x0000, 0x0000, 0x0002, /*12C7*/
  0x2566, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12C8*/
  0x2567, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12C9*/
  0x2568, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12CA*/
  0x2569, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12CB*/
  0x256A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12CC*/
  0x256B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12CD*/
  0x256C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12CE*/
  0x256D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12CF*/
  0x256E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12D0*/
  0x256F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12D1*/
  0x2570, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12D2*/
  0x2571, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12D3*/
  0x2572, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12D4*/
  0x2573, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12D5*/
  0x2574, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12D6*/
  0xFBC0, 0x0020, 0x0002, 0x92D7, 0x0000, 0x0000, 0x0002, /*12D7*/
  0x2575, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12D8*/
  0x2576, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12D9*/
  0x2577, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12DA*/
  0x2578, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12DB*/
  0x2579, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12DC*/
  0x257A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12DD*/
  0x257B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12DE*/
  0x257C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12DF*/
  0x2584, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12E0*/
  0x2585, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12E1*/
  0x2586, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12E2*/
  0x2587, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12E3*/
  0x2588, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12E4*/
  0x2589, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12E5*/
  0x258A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12E6*/
  0x258B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12E7*/
  0x258C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12E8*/
  0x258D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12E9*/
  0x258E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12EA*/
  0x258F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12EB*/
  0x2590, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12EC*/
  0x2591, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12ED*/
  0x2592, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12EE*/
  0x2593, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12EF*/
  0x2594, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12F0*/
  0x2595, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12F1*/
  0x2596, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12F2*/
  0x2597, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12F3*/
  0x2598, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12F4*/
  0x2599, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12F5*/
  0x259A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12F6*/
  0x259B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12F7*/
  0x25A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12F8*/
  0x25A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12F9*/
  0x25A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12FA*/
  0x25A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12FB*/
  0x25A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12FC*/
  0x25A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12FD*/
  0x25A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12FE*/
  0x25AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*12FF*/
};

uint16 uca900_p013[]= {
  0x25AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1300*/
  0x25AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1301*/
  0x25AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1302*/
  0x25AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1303*/
  0x25B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1304*/
  0x25B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1305*/
  0x25B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1306*/
  0x25B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1307*/
  0x25B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1308*/
  0x25B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1309*/
  0x25B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*130A*/
  0x25B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*130B*/
  0x25B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*130C*/
  0x25BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*130D*/
  0x25BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*130E*/
  0x25BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*130F*/
  0x25BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1310*/
  0xFBC0, 0x0020, 0x0002, 0x9311, 0x0000, 0x0000, 0x0002, /*1311*/
  0x25BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1312*/
  0x25BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1313*/
  0x25C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1314*/
  0x25C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1315*/
  0xFBC0, 0x0020, 0x0002, 0x9316, 0x0000, 0x0000, 0x0002, /*1316*/
  0xFBC0, 0x0020, 0x0002, 0x9317, 0x0000, 0x0000, 0x0002, /*1317*/
  0x25C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1318*/
  0x25C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1319*/
  0x25C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*131A*/
  0x25C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*131B*/
  0x25C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*131C*/
  0x25C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*131D*/
  0x25C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*131E*/
  0x25C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*131F*/
  0x25CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1320*/
  0x25CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1321*/
  0x25D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1322*/
  0x25D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1323*/
  0x25D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1324*/
  0x25D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1325*/
  0x25D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1326*/
  0x25D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1327*/
  0x25D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1328*/
  0x25D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1329*/
  0x25D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*132A*/
  0x25DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*132B*/
  0x25DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*132C*/
  0x25DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*132D*/
  0x25DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*132E*/
  0x25DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*132F*/
  0x25E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1330*/
  0x25E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1331*/
  0x25E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1332*/
  0x25EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1333*/
  0x25EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1334*/
  0x25EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1335*/
  0x25ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1336*/
  0x25EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1337*/
  0x25F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1338*/
  0x25F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1339*/
  0x25F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*133A*/
  0x25F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*133B*/
  0x25F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*133C*/
  0x25F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*133D*/
  0x25F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*133E*/
  0x25F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*133F*/
  0x25FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1340*/
  0x2600, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1341*/
  0x2601, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1342*/
  0x2602, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1343*/
  0x2603, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1344*/
  0x2604, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1345*/
  0x2605, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1346*/
  0x2606, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1347*/
  0x2607, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1348*/
  0x2608, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1349*/
  0x2609, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*134A*/
  0x260A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*134B*/
  0x260B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*134C*/
  0x260C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*134D*/
  0x260D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*134E*/
  0x260E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*134F*/
  0x2613, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1350*/
  0x2614, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1351*/
  0x2615, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1352*/
  0x2616, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1353*/
  0x2617, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1354*/
  0x2618, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1355*/
  0x2619, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1356*/
  0x261A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1357*/
  0x2620, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1358*/
  0x2621, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1359*/
  0x2622, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*135A*/
  0xFBC0, 0x0020, 0x0002, 0x935B, 0x0000, 0x0000, 0x0002, /*135B*/
  0xFBC0, 0x0020, 0x0002, 0x935C, 0x0000, 0x0000, 0x0002, /*135C*/
  0x0000, 0x00B5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*135D*/
  0x0000, 0x00B4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*135E*/
  0x0000, 0x00B3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*135F*/
  0x02C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1360*/
  0x0251, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1361*/
  0x027C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1362*/
  0x0252, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1363*/
  0x0253, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1364*/
  0x0254, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1365*/
  0x0255, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1366*/
  0x026C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1367*/
  0x02C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1368*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1369*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*136A*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*136B*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*136C*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*136D*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*136E*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*136F*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1370*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1371*/
  0x1ABB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1372*/
  0x1ABC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1373*/
  0x1ABD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1374*/
  0x1ABE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1375*/
  0x1ABF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1376*/
  0x1AC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1377*/
  0x1AC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1378*/
  0x1AC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1379*/
  0x1AC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*137A*/
  0x1AC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*137B*/
  0x1AC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*137C*/
  0xFBC0, 0x0020, 0x0002, 0x937D, 0x0000, 0x0000, 0x0002, /*137D*/
  0xFBC0, 0x0020, 0x0002, 0x937E, 0x0000, 0x0000, 0x0002, /*137E*/
  0xFBC0, 0x0020, 0x0002, 0x937F, 0x0000, 0x0000, 0x0002, /*137F*/
  0x24B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1380*/
  0x24B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1381*/
  0x24B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1382*/
  0x24BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1383*/
  0x2506, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1384*/
  0x2507, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1385*/
  0x2508, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1386*/
  0x2509, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1387*/
  0x260F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1388*/
  0x2610, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1389*/
  0x2611, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*138A*/
  0x2612, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*138B*/
  0x261B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*138C*/
  0x261C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*138D*/
  0x261D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*138E*/
  0x261E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*138F*/
  0x04C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1390*/
  0x04C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1391*/
  0x04C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1392*/
  0x04C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1393*/
  0x04CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1394*/
  0x04CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1395*/
  0x04CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1396*/
  0x04CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1397*/
  0x04CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1398*/
  0x04CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1399*/
  0xFBC0, 0x0020, 0x0002, 0x939A, 0x0000, 0x0000, 0x0002, /*139A*/
  0xFBC0, 0x0020, 0x0002, 0x939B, 0x0000, 0x0000, 0x0002, /*139B*/
  0xFBC0, 0x0020, 0x0002, 0x939C, 0x0000, 0x0000, 0x0002, /*139C*/
  0xFBC0, 0x0020, 0x0002, 0x939D, 0x0000, 0x0000, 0x0002, /*139D*/
  0xFBC0, 0x0020, 0x0002, 0x939E, 0x0000, 0x0000, 0x0002, /*139E*/
  0xFBC0, 0x0020, 0x0002, 0x939F, 0x0000, 0x0000, 0x0002, /*139F*/
  0x337F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13A0*/
  0x3380, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13A1*/
  0x3381, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13A2*/
  0x3382, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13A3*/
  0x3383, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13A4*/
  0x3384, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13A5*/
  0x3385, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13A6*/
  0x3386, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13A7*/
  0x3387, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13A8*/
  0x3388, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13A9*/
  0x3389, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13AA*/
  0x338A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13AB*/
  0x338B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13AC*/
  0x338C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13AD*/
  0x338D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13AE*/
  0x338E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13AF*/
  0x338F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13B0*/
  0x3390, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13B1*/
  0x3391, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13B2*/
  0x3392, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13B3*/
  0x3393, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13B4*/
  0x3394, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13B5*/
  0x3395, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13B6*/
  0x3396, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13B7*/
  0x3397, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13B8*/
  0x3398, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13B9*/
  0x3399, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13BA*/
  0x339A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13BB*/
  0x339B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13BC*/
  0x339C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13BD*/
  0x339D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13BE*/
  0x339E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13BF*/
  0x339F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13C0*/
  0x33A0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13C1*/
  0x33A1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13C2*/
  0x33A2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13C3*/
  0x33A3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13C4*/
  0x33A4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13C5*/
  0x33A5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13C6*/
  0x33A6, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13C7*/
  0x33A7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13C8*/
  0x33A8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13C9*/
  0x33A9, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13CA*/
  0x33AA, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13CB*/
  0x33AB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13CC*/
  0x33AC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13CD*/
  0x33AD, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13CE*/
  0x33AE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13CF*/
  0x33AF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13D0*/
  0x33B0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13D1*/
  0x33B1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13D2*/
  0x33B2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13D3*/
  0x33B3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13D4*/
  0x33B4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13D5*/
  0x33B5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13D6*/
  0x33B6, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13D7*/
  0x33B7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13D8*/
  0x33B8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13D9*/
  0x33B9, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13DA*/
  0x33BA, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13DB*/
  0x33BB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13DC*/
  0x33BC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13DD*/
  0x33BD, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13DE*/
  0x33BE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13DF*/
  0x33BF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13E0*/
  0x33C0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13E1*/
  0x33C1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13E2*/
  0x33C2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13E3*/
  0x33C3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13E4*/
  0x33C4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13E5*/
  0x33C5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13E6*/
  0x33C6, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13E7*/
  0x33C7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13E8*/
  0x33C8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13E9*/
  0x33C9, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13EA*/
  0x33CA, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13EB*/
  0x33CB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13EC*/
  0x33CC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13ED*/
  0x33CD, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13EE*/
  0x33CE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13EF*/
  0x33CF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13F0*/
  0x33D0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13F1*/
  0x33D1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13F2*/
  0x33D2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13F3*/
  0x33D3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13F4*/
  0x33D4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*13F5*/
  0xFBC0, 0x0020, 0x0002, 0x93F6, 0x0000, 0x0000, 0x0002, /*13F6*/
  0xFBC0, 0x0020, 0x0002, 0x93F7, 0x0000, 0x0000, 0x0002, /*13F7*/
  0x33CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13F8*/
  0x33D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13F9*/
  0x33D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13FA*/
  0x33D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13FB*/
  0x33D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13FC*/
  0x33D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13FD*/
  0xFBC0, 0x0020, 0x0002, 0x93FE, 0x0000, 0x0000, 0x0002, /*13FE*/
  0xFBC0, 0x0020, 0x0002, 0x93FF, 0x0000, 0x0000, 0x0002  /*13FF*/
};

uint16 uca900_p014[]= {
  0x020F, 0x0020, 0x0002, 0x0001, /*1400*/
  0x33F9, 0x0020, 0x0002, 0x0001, /*1401*/
  0x33FA, 0x0020, 0x0002, 0x0001, /*1402*/
  0x33FB, 0x0020, 0x0002, 0x0001, /*1403*/
  0x33FC, 0x0020, 0x0002, 0x0001, /*1404*/
  0x33FD, 0x0020, 0x0002, 0x0001, /*1405*/
  0x33FE, 0x0020, 0x0002, 0x0001, /*1406*/
  0x33FF, 0x0020, 0x0002, 0x0001, /*1407*/
  0x3400, 0x0020, 0x0002, 0x0001, /*1408*/
  0x3401, 0x0020, 0x0002, 0x0001, /*1409*/
  0x3402, 0x0020, 0x0002, 0x0001, /*140A*/
  0x3403, 0x0020, 0x0002, 0x0001, /*140B*/
  0x3404, 0x0020, 0x0002, 0x0001, /*140C*/
  0x3405, 0x0020, 0x0002, 0x0001, /*140D*/
  0x3406, 0x0020, 0x0002, 0x0001, /*140E*/
  0x3407, 0x0020, 0x0002, 0x0001, /*140F*/
  0x3408, 0x0020, 0x0002, 0x0001, /*1410*/
  0x3409, 0x0020, 0x0002, 0x0001, /*1411*/
  0x340A, 0x0020, 0x0002, 0x0001, /*1412*/
  0x340B, 0x0020, 0x0002, 0x0001, /*1413*/
  0x340C, 0x0020, 0x0002, 0x0001, /*1414*/
  0x340D, 0x0020, 0x0002, 0x0001, /*1415*/
  0x340E, 0x0020, 0x0002, 0x0001, /*1416*/
  0x340F, 0x0020, 0x0002, 0x0001, /*1417*/
  0x3410, 0x0020, 0x0002, 0x0001, /*1418*/
  0x3411, 0x0020, 0x0002, 0x0001, /*1419*/
  0x3412, 0x0020, 0x0002, 0x0001, /*141A*/
  0x3413, 0x0020, 0x0002, 0x0001, /*141B*/
  0x3414, 0x0020, 0x0002, 0x0001, /*141C*/
  0x3415, 0x0020, 0x0002, 0x0001, /*141D*/
  0x3416, 0x0020, 0x0002, 0x0001, /*141E*/
  0x3417, 0x0020, 0x0002, 0x0001, /*141F*/
  0x3418, 0x0020, 0x0002, 0x0001, /*1420*/
  0x3419, 0x0020, 0x0002, 0x0001, /*1421*/
  0x341A, 0x0020, 0x0002, 0x0001, /*1422*/
  0x341B, 0x0020, 0x0002, 0x0001, /*1423*/
  0x341C, 0x0020, 0x0002, 0x0001, /*1424*/
  0x341D, 0x0020, 0x0002, 0x0001, /*1425*/
  0x341E, 0x0020, 0x0002, 0x0001, /*1426*/
  0x341F, 0x0020, 0x0002, 0x0001, /*1427*/
  0x3420, 0x0020, 0x0002, 0x0001, /*1428*/
  0x3421, 0x0020, 0x0002, 0x0001, /*1429*/
  0x3422, 0x0020, 0x0002, 0x0001, /*142A*/
  0x3423, 0x0020, 0x0002, 0x0001, /*142B*/
  0x3424, 0x0020, 0x0002, 0x0001, /*142C*/
  0x3425, 0x0020, 0x0002, 0x0001, /*142D*/
  0x3426, 0x0020, 0x0002, 0x0001, /*142E*/
  0x3427, 0x0020, 0x0002, 0x0001, /*142F*/
  0x3428, 0x0020, 0x0002, 0x0001, /*1430*/
  0x3429, 0x0020, 0x0002, 0x0001, /*1431*/
  0x342A, 0x0020, 0x0002, 0x0001, /*1432*/
  0x342B, 0x0020, 0x0002, 0x0001, /*1433*/
  0x342C, 0x0020, 0x0002, 0x0001, /*1434*/
  0x342D, 0x0020, 0x0002, 0x0001, /*1435*/
  0x342E, 0x0020, 0x0002, 0x0001, /*1436*/
  0x342F, 0x0020, 0x0002, 0x0001, /*1437*/
  0x3430, 0x0020, 0x0002, 0x0001, /*1438*/
  0x3431, 0x0020, 0x0002, 0x0001, /*1439*/
  0x3432, 0x0020, 0x0002, 0x0001, /*143A*/
  0x3433, 0x0020, 0x0002, 0x0001, /*143B*/
  0x3434, 0x0020, 0x0002, 0x0001, /*143C*/
  0x3435, 0x0020, 0x0002, 0x0001, /*143D*/
  0x3436, 0x0020, 0x0002, 0x0001, /*143E*/
  0x3437, 0x0020, 0x0002, 0x0001, /*143F*/
  0x3438, 0x0020, 0x0002, 0x0001, /*1440*/
  0x3439, 0x0020, 0x0002, 0x0001, /*1441*/
  0x343A, 0x0020, 0x0002, 0x0001, /*1442*/
  0x343B, 0x0020, 0x0002, 0x0001, /*1443*/
  0x343C, 0x0020, 0x0002, 0x0001, /*1444*/
  0x343D, 0x0020, 0x0002, 0x0001, /*1445*/
  0x343E, 0x0020, 0x0002, 0x0001, /*1446*/
  0x343F, 0x0020, 0x0002, 0x0001, /*1447*/
  0x3440, 0x0020, 0x0002, 0x0001, /*1448*/
  0x3441, 0x0020, 0x0002, 0x0001, /*1449*/
  0x3442, 0x0020, 0x0002, 0x0001, /*144A*/
  0x3443, 0x0020, 0x0002, 0x0001, /*144B*/
  0x3444, 0x0020, 0x0002, 0x0001, /*144C*/
  0x3445, 0x0020, 0x0002, 0x0001, /*144D*/
  0x3446, 0x0020, 0x0002, 0x0001, /*144E*/
  0x3447, 0x0020, 0x0002, 0x0001, /*144F*/
  0x3448, 0x0020, 0x0002, 0x0001, /*1450*/
  0x3449, 0x0020, 0x0002, 0x0001, /*1451*/
  0x344A, 0x0020, 0x0002, 0x0001, /*1452*/
  0x344B, 0x0020, 0x0002, 0x0001, /*1453*/
  0x344C, 0x0020, 0x0002, 0x0001, /*1454*/
  0x344D, 0x0020, 0x0002, 0x0001, /*1455*/
  0x344E, 0x0020, 0x0002, 0x0001, /*1456*/
  0x344F, 0x0020, 0x0002, 0x0001, /*1457*/
  0x3450, 0x0020, 0x0002, 0x0001, /*1458*/
  0x3451, 0x0020, 0x0002, 0x0001, /*1459*/
  0x3452, 0x0020, 0x0002, 0x0001, /*145A*/
  0x3453, 0x0020, 0x0002, 0x0001, /*145B*/
  0x3454, 0x0020, 0x0002, 0x0001, /*145C*/
  0x3455, 0x0020, 0x0002, 0x0001, /*145D*/
  0x3456, 0x0020, 0x0002, 0x0001, /*145E*/
  0x3457, 0x0020, 0x0002, 0x0001, /*145F*/
  0x3458, 0x0020, 0x0002, 0x0001, /*1460*/
  0x3459, 0x0020, 0x0002, 0x0001, /*1461*/
  0x345A, 0x0020, 0x0002, 0x0001, /*1462*/
  0x345B, 0x0020, 0x0002, 0x0001, /*1463*/
  0x345C, 0x0020, 0x0002, 0x0001, /*1464*/
  0x345D, 0x0020, 0x0002, 0x0001, /*1465*/
  0x345E, 0x0020, 0x0002, 0x0001, /*1466*/
  0x345F, 0x0020, 0x0002, 0x0001, /*1467*/
  0x3460, 0x0020, 0x0002, 0x0001, /*1468*/
  0x3461, 0x0020, 0x0002, 0x0001, /*1469*/
  0x3462, 0x0020, 0x0002, 0x0001, /*146A*/
  0x3463, 0x0020, 0x0002, 0x0001, /*146B*/
  0x3464, 0x0020, 0x0002, 0x0001, /*146C*/
  0x3465, 0x0020, 0x0002, 0x0001, /*146D*/
  0x3466, 0x0020, 0x0002, 0x0001, /*146E*/
  0x3467, 0x0020, 0x0002, 0x0001, /*146F*/
  0x3468, 0x0020, 0x0002, 0x0001, /*1470*/
  0x3469, 0x0020, 0x0002, 0x0001, /*1471*/
  0x346A, 0x0020, 0x0002, 0x0001, /*1472*/
  0x346B, 0x0020, 0x0002, 0x0001, /*1473*/
  0x346C, 0x0020, 0x0002, 0x0001, /*1474*/
  0x346D, 0x0020, 0x0002, 0x0001, /*1475*/
  0x346E, 0x0020, 0x0002, 0x0001, /*1476*/
  0x346F, 0x0020, 0x0002, 0x0001, /*1477*/
  0x3470, 0x0020, 0x0002, 0x0001, /*1478*/
  0x3471, 0x0020, 0x0002, 0x0001, /*1479*/
  0x3472, 0x0020, 0x0002, 0x0001, /*147A*/
  0x3473, 0x0020, 0x0002, 0x0001, /*147B*/
  0x3474, 0x0020, 0x0002, 0x0001, /*147C*/
  0x3475, 0x0020, 0x0002, 0x0001, /*147D*/
  0x3476, 0x0020, 0x0002, 0x0001, /*147E*/
  0x3477, 0x0020, 0x0002, 0x0001, /*147F*/
  0x3478, 0x0020, 0x0002, 0x0001, /*1480*/
  0x3479, 0x0020, 0x0002, 0x0001, /*1481*/
  0x347A, 0x0020, 0x0002, 0x0001, /*1482*/
  0x347B, 0x0020, 0x0002, 0x0001, /*1483*/
  0x347C, 0x0020, 0x0002, 0x0001, /*1484*/
  0x347D, 0x0020, 0x0002, 0x0001, /*1485*/
  0x347E, 0x0020, 0x0002, 0x0001, /*1486*/
  0x347F, 0x0020, 0x0002, 0x0001, /*1487*/
  0x3480, 0x0020, 0x0002, 0x0001, /*1488*/
  0x3481, 0x0020, 0x0002, 0x0001, /*1489*/
  0x3482, 0x0020, 0x0002, 0x0001, /*148A*/
  0x3483, 0x0020, 0x0002, 0x0001, /*148B*/
  0x3484, 0x0020, 0x0002, 0x0001, /*148C*/
  0x3485, 0x0020, 0x0002, 0x0001, /*148D*/
  0x3486, 0x0020, 0x0002, 0x0001, /*148E*/
  0x3487, 0x0020, 0x0002, 0x0001, /*148F*/
  0x3488, 0x0020, 0x0002, 0x0001, /*1490*/
  0x3489, 0x0020, 0x0002, 0x0001, /*1491*/
  0x348A, 0x0020, 0x0002, 0x0001, /*1492*/
  0x348B, 0x0020, 0x0002, 0x0001, /*1493*/
  0x348C, 0x0020, 0x0002, 0x0001, /*1494*/
  0x348D, 0x0020, 0x0002, 0x0001, /*1495*/
  0x348E, 0x0020, 0x0002, 0x0001, /*1496*/
  0x348F, 0x0020, 0x0002, 0x0001, /*1497*/
  0x3490, 0x0020, 0x0002, 0x0001, /*1498*/
  0x3491, 0x0020, 0x0002, 0x0001, /*1499*/
  0x3492, 0x0020, 0x0002, 0x0001, /*149A*/
  0x3493, 0x0020, 0x0002, 0x0001, /*149B*/
  0x3494, 0x0020, 0x0002, 0x0001, /*149C*/
  0x3495, 0x0020, 0x0002, 0x0001, /*149D*/
  0x3496, 0x0020, 0x0002, 0x0001, /*149E*/
  0x3497, 0x0020, 0x0002, 0x0001, /*149F*/
  0x3498, 0x0020, 0x0002, 0x0001, /*14A0*/
  0x3499, 0x0020, 0x0002, 0x0001, /*14A1*/
  0x349A, 0x0020, 0x0002, 0x0001, /*14A2*/
  0x349B, 0x0020, 0x0002, 0x0001, /*14A3*/
  0x349C, 0x0020, 0x0002, 0x0001, /*14A4*/
  0x349D, 0x0020, 0x0002, 0x0001, /*14A5*/
  0x349E, 0x0020, 0x0002, 0x0001, /*14A6*/
  0x349F, 0x0020, 0x0002, 0x0001, /*14A7*/
  0x34A0, 0x0020, 0x0002, 0x0001, /*14A8*/
  0x34A1, 0x0020, 0x0002, 0x0001, /*14A9*/
  0x34A2, 0x0020, 0x0002, 0x0001, /*14AA*/
  0x34A3, 0x0020, 0x0002, 0x0001, /*14AB*/
  0x34A4, 0x0020, 0x0002, 0x0001, /*14AC*/
  0x34A5, 0x0020, 0x0002, 0x0001, /*14AD*/
  0x34A6, 0x0020, 0x0002, 0x0001, /*14AE*/
  0x34A7, 0x0020, 0x0002, 0x0001, /*14AF*/
  0x34A8, 0x0020, 0x0002, 0x0001, /*14B0*/
  0x34A9, 0x0020, 0x0002, 0x0001, /*14B1*/
  0x34AA, 0x0020, 0x0002, 0x0001, /*14B2*/
  0x34AB, 0x0020, 0x0002, 0x0001, /*14B3*/
  0x34AC, 0x0020, 0x0002, 0x0001, /*14B4*/
  0x34AD, 0x0020, 0x0002, 0x0001, /*14B5*/
  0x34AE, 0x0020, 0x0002, 0x0001, /*14B6*/
  0x34AF, 0x0020, 0x0002, 0x0001, /*14B7*/
  0x34B0, 0x0020, 0x0002, 0x0001, /*14B8*/
  0x34B1, 0x0020, 0x0002, 0x0001, /*14B9*/
  0x34B2, 0x0020, 0x0002, 0x0001, /*14BA*/
  0x34B3, 0x0020, 0x0002, 0x0001, /*14BB*/
  0x34B4, 0x0020, 0x0002, 0x0001, /*14BC*/
  0x34B5, 0x0020, 0x0002, 0x0001, /*14BD*/
  0x34B6, 0x0020, 0x0002, 0x0001, /*14BE*/
  0x34B7, 0x0020, 0x0002, 0x0001, /*14BF*/
  0x34B8, 0x0020, 0x0002, 0x0001, /*14C0*/
  0x34B9, 0x0020, 0x0002, 0x0001, /*14C1*/
  0x34BA, 0x0020, 0x0002, 0x0001, /*14C2*/
  0x34BB, 0x0020, 0x0002, 0x0001, /*14C3*/
  0x34BC, 0x0020, 0x0002, 0x0001, /*14C4*/
  0x34BD, 0x0020, 0x0002, 0x0001, /*14C5*/
  0x34BE, 0x0020, 0x0002, 0x0001, /*14C6*/
  0x34BF, 0x0020, 0x0002, 0x0001, /*14C7*/
  0x34C0, 0x0020, 0x0002, 0x0001, /*14C8*/
  0x34C1, 0x0020, 0x0002, 0x0001, /*14C9*/
  0x34C2, 0x0020, 0x0002, 0x0001, /*14CA*/
  0x34C3, 0x0020, 0x0002, 0x0001, /*14CB*/
  0x34C4, 0x0020, 0x0002, 0x0001, /*14CC*/
  0x34C5, 0x0020, 0x0002, 0x0001, /*14CD*/
  0x34C6, 0x0020, 0x0002, 0x0001, /*14CE*/
  0x34C7, 0x0020, 0x0002, 0x0001, /*14CF*/
  0x34C8, 0x0020, 0x0002, 0x0001, /*14D0*/
  0x34C9, 0x0020, 0x0002, 0x0001, /*14D1*/
  0x34CA, 0x0020, 0x0002, 0x0001, /*14D2*/
  0x34CB, 0x0020, 0x0002, 0x0001, /*14D3*/
  0x34CC, 0x0020, 0x0002, 0x0001, /*14D4*/
  0x34CD, 0x0020, 0x0002, 0x0001, /*14D5*/
  0x34CE, 0x0020, 0x0002, 0x0001, /*14D6*/
  0x34CF, 0x0020, 0x0002, 0x0001, /*14D7*/
  0x34D0, 0x0020, 0x0002, 0x0001, /*14D8*/
  0x34D1, 0x0020, 0x0002, 0x0001, /*14D9*/
  0x34D2, 0x0020, 0x0002, 0x0001, /*14DA*/
  0x34D3, 0x0020, 0x0002, 0x0001, /*14DB*/
  0x34D4, 0x0020, 0x0002, 0x0001, /*14DC*/
  0x34D5, 0x0020, 0x0002, 0x0001, /*14DD*/
  0x34D6, 0x0020, 0x0002, 0x0001, /*14DE*/
  0x34D7, 0x0020, 0x0002, 0x0001, /*14DF*/
  0x34D8, 0x0020, 0x0002, 0x0001, /*14E0*/
  0x34D9, 0x0020, 0x0002, 0x0001, /*14E1*/
  0x34DA, 0x0020, 0x0002, 0x0001, /*14E2*/
  0x34DB, 0x0020, 0x0002, 0x0001, /*14E3*/
  0x34DC, 0x0020, 0x0002, 0x0001, /*14E4*/
  0x34DD, 0x0020, 0x0002, 0x0001, /*14E5*/
  0x34DE, 0x0020, 0x0002, 0x0001, /*14E6*/
  0x34DF, 0x0020, 0x0002, 0x0001, /*14E7*/
  0x34E0, 0x0020, 0x0002, 0x0001, /*14E8*/
  0x34E1, 0x0020, 0x0002, 0x0001, /*14E9*/
  0x34E2, 0x0020, 0x0002, 0x0001, /*14EA*/
  0x34E3, 0x0020, 0x0002, 0x0001, /*14EB*/
  0x34E4, 0x0020, 0x0002, 0x0001, /*14EC*/
  0x34E5, 0x0020, 0x0002, 0x0001, /*14ED*/
  0x34E6, 0x0020, 0x0002, 0x0001, /*14EE*/
  0x34E7, 0x0020, 0x0002, 0x0001, /*14EF*/
  0x34E8, 0x0020, 0x0002, 0x0001, /*14F0*/
  0x34E9, 0x0020, 0x0002, 0x0001, /*14F1*/
  0x34EA, 0x0020, 0x0002, 0x0001, /*14F2*/
  0x34EB, 0x0020, 0x0002, 0x0001, /*14F3*/
  0x34EC, 0x0020, 0x0002, 0x0001, /*14F4*/
  0x34ED, 0x0020, 0x0002, 0x0001, /*14F5*/
  0x34EE, 0x0020, 0x0002, 0x0001, /*14F6*/
  0x34EF, 0x0020, 0x0002, 0x0001, /*14F7*/
  0x34F0, 0x0020, 0x0002, 0x0001, /*14F8*/
  0x34F1, 0x0020, 0x0002, 0x0001, /*14F9*/
  0x34F2, 0x0020, 0x0002, 0x0001, /*14FA*/
  0x34F3, 0x0020, 0x0002, 0x0001, /*14FB*/
  0x34F4, 0x0020, 0x0002, 0x0001, /*14FC*/
  0x34F5, 0x0020, 0x0002, 0x0001, /*14FD*/
  0x34F6, 0x0020, 0x0002, 0x0001, /*14FE*/
  0x34F7, 0x0020, 0x0002, 0x0001  /*14FF*/
};

uint16 uca900_p015[]= {
  0x34F8, 0x0020, 0x0002, 0x0001, /*1500*/
  0x34F9, 0x0020, 0x0002, 0x0001, /*1501*/
  0x34FA, 0x0020, 0x0002, 0x0001, /*1502*/
  0x34FB, 0x0020, 0x0002, 0x0001, /*1503*/
  0x34FC, 0x0020, 0x0002, 0x0001, /*1504*/
  0x34FD, 0x0020, 0x0002, 0x0001, /*1505*/
  0x34FE, 0x0020, 0x0002, 0x0001, /*1506*/
  0x34FF, 0x0020, 0x0002, 0x0001, /*1507*/
  0x3500, 0x0020, 0x0002, 0x0001, /*1508*/
  0x3501, 0x0020, 0x0002, 0x0001, /*1509*/
  0x3502, 0x0020, 0x0002, 0x0001, /*150A*/
  0x3503, 0x0020, 0x0002, 0x0001, /*150B*/
  0x3504, 0x0020, 0x0002, 0x0001, /*150C*/
  0x3505, 0x0020, 0x0002, 0x0001, /*150D*/
  0x3506, 0x0020, 0x0002, 0x0001, /*150E*/
  0x3507, 0x0020, 0x0002, 0x0001, /*150F*/
  0x3508, 0x0020, 0x0002, 0x0001, /*1510*/
  0x3509, 0x0020, 0x0002, 0x0001, /*1511*/
  0x350A, 0x0020, 0x0002, 0x0001, /*1512*/
  0x350B, 0x0020, 0x0002, 0x0001, /*1513*/
  0x350C, 0x0020, 0x0002, 0x0001, /*1514*/
  0x350D, 0x0020, 0x0002, 0x0001, /*1515*/
  0x350E, 0x0020, 0x0002, 0x0001, /*1516*/
  0x350F, 0x0020, 0x0002, 0x0001, /*1517*/
  0x3510, 0x0020, 0x0002, 0x0001, /*1518*/
  0x3511, 0x0020, 0x0002, 0x0001, /*1519*/
  0x3512, 0x0020, 0x0002, 0x0001, /*151A*/
  0x3513, 0x0020, 0x0002, 0x0001, /*151B*/
  0x3514, 0x0020, 0x0002, 0x0001, /*151C*/
  0x3515, 0x0020, 0x0002, 0x0001, /*151D*/
  0x3516, 0x0020, 0x0002, 0x0001, /*151E*/
  0x3517, 0x0020, 0x0002, 0x0001, /*151F*/
  0x3518, 0x0020, 0x0002, 0x0001, /*1520*/
  0x3519, 0x0020, 0x0002, 0x0001, /*1521*/
  0x351A, 0x0020, 0x0002, 0x0001, /*1522*/
  0x351B, 0x0020, 0x0002, 0x0001, /*1523*/
  0x351C, 0x0020, 0x0002, 0x0001, /*1524*/
  0x351D, 0x0020, 0x0002, 0x0001, /*1525*/
  0x351E, 0x0020, 0x0002, 0x0001, /*1526*/
  0x351F, 0x0020, 0x0002, 0x0001, /*1527*/
  0x3520, 0x0020, 0x0002, 0x0001, /*1528*/
  0x3521, 0x0020, 0x0002, 0x0001, /*1529*/
  0x3522, 0x0020, 0x0002, 0x0001, /*152A*/
  0x3523, 0x0020, 0x0002, 0x0001, /*152B*/
  0x3524, 0x0020, 0x0002, 0x0001, /*152C*/
  0x3525, 0x0020, 0x0002, 0x0001, /*152D*/
  0x3526, 0x0020, 0x0002, 0x0001, /*152E*/
  0x3527, 0x0020, 0x0002, 0x0001, /*152F*/
  0x3528, 0x0020, 0x0002, 0x0001, /*1530*/
  0x3529, 0x0020, 0x0002, 0x0001, /*1531*/
  0x352A, 0x0020, 0x0002, 0x0001, /*1532*/
  0x352B, 0x0020, 0x0002, 0x0001, /*1533*/
  0x352C, 0x0020, 0x0002, 0x0001, /*1534*/
  0x352D, 0x0020, 0x0002, 0x0001, /*1535*/
  0x352E, 0x0020, 0x0002, 0x0001, /*1536*/
  0x352F, 0x0020, 0x0002, 0x0001, /*1537*/
  0x3530, 0x0020, 0x0002, 0x0001, /*1538*/
  0x3531, 0x0020, 0x0002, 0x0001, /*1539*/
  0x3532, 0x0020, 0x0002, 0x0001, /*153A*/
  0x3533, 0x0020, 0x0002, 0x0001, /*153B*/
  0x3534, 0x0020, 0x0002, 0x0001, /*153C*/
  0x3535, 0x0020, 0x0002, 0x0001, /*153D*/
  0x3536, 0x0020, 0x0002, 0x0001, /*153E*/
  0x3537, 0x0020, 0x0002, 0x0001, /*153F*/
  0x3538, 0x0020, 0x0002, 0x0001, /*1540*/
  0x3539, 0x0020, 0x0002, 0x0001, /*1541*/
  0x353A, 0x0020, 0x0002, 0x0001, /*1542*/
  0x353B, 0x0020, 0x0002, 0x0001, /*1543*/
  0x353C, 0x0020, 0x0002, 0x0001, /*1544*/
  0x353D, 0x0020, 0x0002, 0x0001, /*1545*/
  0x353E, 0x0020, 0x0002, 0x0001, /*1546*/
  0x353F, 0x0020, 0x0002, 0x0001, /*1547*/
  0x3540, 0x0020, 0x0002, 0x0001, /*1548*/
  0x3541, 0x0020, 0x0002, 0x0001, /*1549*/
  0x3542, 0x0020, 0x0002, 0x0001, /*154A*/
  0x3543, 0x0020, 0x0002, 0x0001, /*154B*/
  0x3544, 0x0020, 0x0002, 0x0001, /*154C*/
  0x3545, 0x0020, 0x0002, 0x0001, /*154D*/
  0x3546, 0x0020, 0x0002, 0x0001, /*154E*/
  0x3547, 0x0020, 0x0002, 0x0001, /*154F*/
  0x3548, 0x0020, 0x0002, 0x0001, /*1550*/
  0x3549, 0x0020, 0x0002, 0x0001, /*1551*/
  0x354A, 0x0020, 0x0002, 0x0001, /*1552*/
  0x354B, 0x0020, 0x0002, 0x0001, /*1553*/
  0x354C, 0x0020, 0x0002, 0x0001, /*1554*/
  0x354D, 0x0020, 0x0002, 0x0001, /*1555*/
  0x354E, 0x0020, 0x0002, 0x0001, /*1556*/
  0x354F, 0x0020, 0x0002, 0x0001, /*1557*/
  0x3550, 0x0020, 0x0002, 0x0001, /*1558*/
  0x3551, 0x0020, 0x0002, 0x0001, /*1559*/
  0x3552, 0x0020, 0x0002, 0x0001, /*155A*/
  0x3553, 0x0020, 0x0002, 0x0001, /*155B*/
  0x3554, 0x0020, 0x0002, 0x0001, /*155C*/
  0x3555, 0x0020, 0x0002, 0x0001, /*155D*/
  0x3556, 0x0020, 0x0002, 0x0001, /*155E*/
  0x3557, 0x0020, 0x0002, 0x0001, /*155F*/
  0x3558, 0x0020, 0x0002, 0x0001, /*1560*/
  0x3559, 0x0020, 0x0002, 0x0001, /*1561*/
  0x355A, 0x0020, 0x0002, 0x0001, /*1562*/
  0x355B, 0x0020, 0x0002, 0x0001, /*1563*/
  0x355C, 0x0020, 0x0002, 0x0001, /*1564*/
  0x355D, 0x0020, 0x0002, 0x0001, /*1565*/
  0x355E, 0x0020, 0x0002, 0x0001, /*1566*/
  0x355F, 0x0020, 0x0002, 0x0001, /*1567*/
  0x3560, 0x0020, 0x0002, 0x0001, /*1568*/
  0x3561, 0x0020, 0x0002, 0x0001, /*1569*/
  0x3562, 0x0020, 0x0002, 0x0001, /*156A*/
  0x3563, 0x0020, 0x0002, 0x0001, /*156B*/
  0x3564, 0x0020, 0x0002, 0x0001, /*156C*/
  0x3565, 0x0020, 0x0002, 0x0001, /*156D*/
  0x3566, 0x0020, 0x0002, 0x0001, /*156E*/
  0x3567, 0x0020, 0x0002, 0x0001, /*156F*/
  0x3568, 0x0020, 0x0002, 0x0001, /*1570*/
  0x3569, 0x0020, 0x0002, 0x0001, /*1571*/
  0x356A, 0x0020, 0x0002, 0x0001, /*1572*/
  0x356B, 0x0020, 0x0002, 0x0001, /*1573*/
  0x356C, 0x0020, 0x0002, 0x0001, /*1574*/
  0x356D, 0x0020, 0x0002, 0x0001, /*1575*/
  0x356E, 0x0020, 0x0002, 0x0001, /*1576*/
  0x356F, 0x0020, 0x0002, 0x0001, /*1577*/
  0x3570, 0x0020, 0x0002, 0x0001, /*1578*/
  0x3571, 0x0020, 0x0002, 0x0001, /*1579*/
  0x3572, 0x0020, 0x0002, 0x0001, /*157A*/
  0x3573, 0x0020, 0x0002, 0x0001, /*157B*/
  0x35A6, 0x0020, 0x0002, 0x0001, /*157C*/
  0x3574, 0x0020, 0x0002, 0x0001, /*157D*/
  0x3576, 0x0020, 0x0002, 0x0001, /*157E*/
  0x3577, 0x0020, 0x0002, 0x0001, /*157F*/
  0x3578, 0x0020, 0x0002, 0x0001, /*1580*/
  0x3579, 0x0020, 0x0002, 0x0001, /*1581*/
  0x357A, 0x0020, 0x0002, 0x0001, /*1582*/
  0x357B, 0x0020, 0x0002, 0x0001, /*1583*/
  0x357C, 0x0020, 0x0002, 0x0001, /*1584*/
  0x357D, 0x0020, 0x0002, 0x0001, /*1585*/
  0x357E, 0x0020, 0x0002, 0x0001, /*1586*/
  0x357F, 0x0020, 0x0002, 0x0001, /*1587*/
  0x3580, 0x0020, 0x0002, 0x0001, /*1588*/
  0x3581, 0x0020, 0x0002, 0x0001, /*1589*/
  0x3582, 0x0020, 0x0002, 0x0001, /*158A*/
  0x3583, 0x0020, 0x0002, 0x0001, /*158B*/
  0x3584, 0x0020, 0x0002, 0x0001, /*158C*/
  0x3585, 0x0020, 0x0002, 0x0001, /*158D*/
  0x3587, 0x0020, 0x0002, 0x0001, /*158E*/
  0x3588, 0x0020, 0x0002, 0x0001, /*158F*/
  0x3589, 0x0020, 0x0002, 0x0001, /*1590*/
  0x358A, 0x0020, 0x0002, 0x0001, /*1591*/
  0x358B, 0x0020, 0x0002, 0x0001, /*1592*/
  0x358C, 0x0020, 0x0002, 0x0001, /*1593*/
  0x358D, 0x0020, 0x0002, 0x0001, /*1594*/
  0x358E, 0x0020, 0x0002, 0x0001, /*1595*/
  0x3595, 0x0020, 0x0002, 0x0001, /*1596*/
  0x3596, 0x0020, 0x0002, 0x0001, /*1597*/
  0x3597, 0x0020, 0x0002, 0x0001, /*1598*/
  0x3598, 0x0020, 0x0002, 0x0001, /*1599*/
  0x3599, 0x0020, 0x0002, 0x0001, /*159A*/
  0x359A, 0x0020, 0x0002, 0x0001, /*159B*/
  0x359B, 0x0020, 0x0002, 0x0001, /*159C*/
  0x359C, 0x0020, 0x0002, 0x0001, /*159D*/
  0x359D, 0x0020, 0x0002, 0x0001, /*159E*/
  0x359E, 0x0020, 0x0002, 0x0001, /*159F*/
  0x359F, 0x0020, 0x0002, 0x0001, /*15A0*/
  0x35A0, 0x0020, 0x0002, 0x0001, /*15A1*/
  0x35A1, 0x0020, 0x0002, 0x0001, /*15A2*/
  0x35A2, 0x0020, 0x0002, 0x0001, /*15A3*/
  0x35A3, 0x0020, 0x0002, 0x0001, /*15A4*/
  0x35A4, 0x0020, 0x0002, 0x0001, /*15A5*/
  0x35A5, 0x0020, 0x0002, 0x0001, /*15A6*/
  0x35A7, 0x0020, 0x0002, 0x0001, /*15A7*/
  0x35A8, 0x0020, 0x0002, 0x0001, /*15A8*/
  0x35A9, 0x0020, 0x0002, 0x0001, /*15A9*/
  0x35AA, 0x0020, 0x0002, 0x0001, /*15AA*/
  0x35AB, 0x0020, 0x0002, 0x0001, /*15AB*/
  0x35AC, 0x0020, 0x0002, 0x0001, /*15AC*/
  0x35AD, 0x0020, 0x0002, 0x0001, /*15AD*/
  0x35AE, 0x0020, 0x0002, 0x0001, /*15AE*/
  0x35AF, 0x0020, 0x0002, 0x0001, /*15AF*/
  0x35B0, 0x0020, 0x0002, 0x0001, /*15B0*/
  0x35B1, 0x0020, 0x0002, 0x0001, /*15B1*/
  0x35B2, 0x0020, 0x0002, 0x0001, /*15B2*/
  0x35B3, 0x0020, 0x0002, 0x0001, /*15B3*/
  0x35B4, 0x0020, 0x0002, 0x0001, /*15B4*/
  0x35B5, 0x0020, 0x0002, 0x0001, /*15B5*/
  0x35B6, 0x0020, 0x0002, 0x0001, /*15B6*/
  0x35B7, 0x0020, 0x0002, 0x0001, /*15B7*/
  0x35B8, 0x0020, 0x0002, 0x0001, /*15B8*/
  0x35B9, 0x0020, 0x0002, 0x0001, /*15B9*/
  0x35BA, 0x0020, 0x0002, 0x0001, /*15BA*/
  0x35BB, 0x0020, 0x0002, 0x0001, /*15BB*/
  0x35BC, 0x0020, 0x0002, 0x0001, /*15BC*/
  0x35BD, 0x0020, 0x0002, 0x0001, /*15BD*/
  0x35BE, 0x0020, 0x0002, 0x0001, /*15BE*/
  0x35BF, 0x0020, 0x0002, 0x0001, /*15BF*/
  0x35C0, 0x0020, 0x0002, 0x0001, /*15C0*/
  0x35C1, 0x0020, 0x0002, 0x0001, /*15C1*/
  0x35C2, 0x0020, 0x0002, 0x0001, /*15C2*/
  0x35C3, 0x0020, 0x0002, 0x0001, /*15C3*/
  0x35C4, 0x0020, 0x0002, 0x0001, /*15C4*/
  0x35C5, 0x0020, 0x0002, 0x0001, /*15C5*/
  0x35C6, 0x0020, 0x0002, 0x0001, /*15C6*/
  0x35C7, 0x0020, 0x0002, 0x0001, /*15C7*/
  0x35C8, 0x0020, 0x0002, 0x0001, /*15C8*/
  0x35C9, 0x0020, 0x0002, 0x0001, /*15C9*/
  0x35CA, 0x0020, 0x0002, 0x0001, /*15CA*/
  0x35CB, 0x0020, 0x0002, 0x0001, /*15CB*/
  0x35CC, 0x0020, 0x0002, 0x0001, /*15CC*/
  0x35CD, 0x0020, 0x0002, 0x0001, /*15CD*/
  0x35CE, 0x0020, 0x0002, 0x0001, /*15CE*/
  0x35CF, 0x0020, 0x0002, 0x0001, /*15CF*/
  0x35D0, 0x0020, 0x0002, 0x0001, /*15D0*/
  0x35D1, 0x0020, 0x0002, 0x0001, /*15D1*/
  0x35D2, 0x0020, 0x0002, 0x0001, /*15D2*/
  0x35D3, 0x0020, 0x0002, 0x0001, /*15D3*/
  0x35D4, 0x0020, 0x0002, 0x0001, /*15D4*/
  0x35D5, 0x0020, 0x0002, 0x0001, /*15D5*/
  0x35D6, 0x0020, 0x0002, 0x0001, /*15D6*/
  0x35D7, 0x0020, 0x0002, 0x0001, /*15D7*/
  0x35D8, 0x0020, 0x0002, 0x0001, /*15D8*/
  0x35D9, 0x0020, 0x0002, 0x0001, /*15D9*/
  0x35DA, 0x0020, 0x0002, 0x0001, /*15DA*/
  0x35DB, 0x0020, 0x0002, 0x0001, /*15DB*/
  0x35DC, 0x0020, 0x0002, 0x0001, /*15DC*/
  0x35DD, 0x0020, 0x0002, 0x0001, /*15DD*/
  0x35DE, 0x0020, 0x0002, 0x0001, /*15DE*/
  0x35DF, 0x0020, 0x0002, 0x0001, /*15DF*/
  0x35E0, 0x0020, 0x0002, 0x0001, /*15E0*/
  0x35E1, 0x0020, 0x0002, 0x0001, /*15E1*/
  0x35E2, 0x0020, 0x0002, 0x0001, /*15E2*/
  0x35E3, 0x0020, 0x0002, 0x0001, /*15E3*/
  0x35E4, 0x0020, 0x0002, 0x0001, /*15E4*/
  0x35E5, 0x0020, 0x0002, 0x0001, /*15E5*/
  0x35E6, 0x0020, 0x0002, 0x0001, /*15E6*/
  0x35E7, 0x0020, 0x0002, 0x0001, /*15E7*/
  0x35E8, 0x0020, 0x0002, 0x0001, /*15E8*/
  0x35E9, 0x0020, 0x0002, 0x0001, /*15E9*/
  0x35EA, 0x0020, 0x0002, 0x0001, /*15EA*/
  0x35EB, 0x0020, 0x0002, 0x0001, /*15EB*/
  0x35EC, 0x0020, 0x0002, 0x0001, /*15EC*/
  0x35ED, 0x0020, 0x0002, 0x0001, /*15ED*/
  0x35EE, 0x0020, 0x0002, 0x0001, /*15EE*/
  0x35EF, 0x0020, 0x0002, 0x0001, /*15EF*/
  0x35F0, 0x0020, 0x0002, 0x0001, /*15F0*/
  0x35F1, 0x0020, 0x0002, 0x0001, /*15F1*/
  0x35F2, 0x0020, 0x0002, 0x0001, /*15F2*/
  0x35F3, 0x0020, 0x0002, 0x0001, /*15F3*/
  0x35F4, 0x0020, 0x0002, 0x0001, /*15F4*/
  0x35F5, 0x0020, 0x0002, 0x0001, /*15F5*/
  0x35F6, 0x0020, 0x0002, 0x0001, /*15F6*/
  0x35F7, 0x0020, 0x0002, 0x0001, /*15F7*/
  0x35F8, 0x0020, 0x0002, 0x0001, /*15F8*/
  0x35F9, 0x0020, 0x0002, 0x0001, /*15F9*/
  0x35FA, 0x0020, 0x0002, 0x0001, /*15FA*/
  0x35FB, 0x0020, 0x0002, 0x0001, /*15FB*/
  0x35FC, 0x0020, 0x0002, 0x0001, /*15FC*/
  0x35FD, 0x0020, 0x0002, 0x0001, /*15FD*/
  0x35FE, 0x0020, 0x0002, 0x0001, /*15FE*/
  0x35FF, 0x0020, 0x0002, 0x0001  /*15FF*/
};

uint16 uca900_p016[]= {
  0x3600, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1600*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3601, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1601*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3602, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1602*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3603, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1603*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3604, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1604*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3605, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1605*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3606, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1606*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3607, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1607*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3608, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1608*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3609, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1609*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x360A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*160A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x360B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*160B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x360C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*160C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x360D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*160D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x360E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*160E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x360F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*160F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3610, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1610*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3611, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1611*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3612, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1612*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3613, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1613*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3614, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1614*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3615, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1615*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3616, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1616*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3617, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1617*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3618, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1618*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3619, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1619*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x361A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*161A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x361B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*161B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x361C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*161C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x361D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*161D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x361E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*161E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x361F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*161F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3620, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1620*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3621, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1621*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3622, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1622*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3623, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1623*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3624, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1624*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3625, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1625*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3626, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1626*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3627, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1627*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3628, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1628*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3629, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1629*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x362A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*162A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x362B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*162B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x362C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*162C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x362D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*162D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x362E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*162E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x362F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*162F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3630, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1630*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3631, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1631*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3632, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1632*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3633, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1633*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3634, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1634*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3635, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1635*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3636, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1636*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3637, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1637*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3638, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1638*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3639, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1639*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x363A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*163A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x363B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*163B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x363C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*163C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x363D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*163D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x363E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*163E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x363F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*163F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3640, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1640*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3641, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1641*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3642, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1642*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3643, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1643*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3644, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1644*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3645, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1645*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3646, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1646*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3647, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1647*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3648, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1648*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3649, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1649*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x364A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*164A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x364B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*164B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x364C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*164C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x364D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*164D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x364E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*164E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x364F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*164F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3650, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1650*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3651, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1651*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3652, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1652*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3653, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1653*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3654, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1654*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3655, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1655*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3656, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1656*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3657, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1657*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3658, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1658*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3659, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1659*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x365A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*165A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x365B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*165B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x365C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*165C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x365D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*165D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x365E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*165E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x365F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*165F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3660, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1660*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3661, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1661*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3662, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1662*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3663, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1663*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3664, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1664*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3665, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1665*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3666, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1666*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3667, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1667*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3668, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1668*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3669, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1669*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x366A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*166A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x366B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*166B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x366C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*166C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x041E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*166D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x027F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*166E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3575, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*166F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3586, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1670*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x358F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1671*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3590, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1672*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3591, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1673*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3592, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1674*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3593, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1675*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3594, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1676*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x366D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1677*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x366E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1678*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x366F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1679*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3670, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*167A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3671, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*167B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3672, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*167C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3673, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*167D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3674, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*167E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3675, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*167F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*1680*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1681*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1682*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1683*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1684*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1685*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1686*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1687*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1688*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1689*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*168A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*168B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*168C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*168D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*168E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*168F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1690*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1691*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1692*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1693*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1694*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1695*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1696*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1697*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1698*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1699*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*169A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0321, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*169B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0322, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*169C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x969D, 0x0000, 0x0000, 0x0000, /*169D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x969E, 0x0000, 0x0000, 0x0000, /*169E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x969F, 0x0000, 0x0000, 0x0000, /*169F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36D6, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36D7, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36D7, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0000, /*16A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36D8, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36D9, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36D9, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0000, /*16AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36D9, 0x0020, 0x0004, 0x0000, 0x0112, 0x0004, 0x0000, /*16AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36D9, 0x0020, 0x0004, 0x0000, 0x0113, 0x0004, 0x0000, /*16AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36DE, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36DE, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0000, /*16B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36DE, 0x0020, 0x0004, 0x0000, 0x0112, 0x0004, 0x0000, /*16B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36DE, 0x0020, 0x0004, 0x0000, 0x0113, 0x0004, 0x0000, /*16B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36E2, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36E2, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0000, /*16BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36E2, 0x0020, 0x0004, 0x0000, 0x0112, 0x0004, 0x0000, /*16BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36E3, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36E3, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0000, /*16C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36E4, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36E6, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36E7, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36EB, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36EB, 0x0020, 0x0004, 0x0000, 0x0112, 0x0004, 0x0000, /*16CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36EB, 0x0020, 0x0004, 0x0000, 0x0113, 0x0004, 0x0000, /*16CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36EB, 0x0020, 0x0004, 0x0000, 0x0114, 0x0004, 0x0000, /*16CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36ED, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36ED, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0000, /*16D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36EE, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36EE, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0000, /*16D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36E9, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36F1, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36F1, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0000, /*16D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36F2, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36F3, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3700, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3701, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3702, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3703, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3703, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3703, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0000, /*16E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36E1, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0000, /*16E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36EB, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0000, /*16EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x025C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x025D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x025E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36E7, 0x0020, 0x0004, 0x36F2, 0x0020, 0x0004, 0x0000, /*16EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36F1, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x36F1, /*16EF*/
  0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0004,
  0x36D8, 0x0020, 0x0004, 0x36D8, 0x0020, 0x0004, 0x0000, /*16F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x36DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x36FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*16F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x96F9, 0x0000, 0x0000, 0x0000, /*16F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x96FA, 0x0000, 0x0000, 0x0000, /*16FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x96FB, 0x0000, 0x0000, 0x0000, /*16FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x96FC, 0x0000, 0x0000, 0x0000, /*16FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x96FD, 0x0000, 0x0000, 0x0000, /*16FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x96FE, 0x0000, 0x0000, 0x0000, /*16FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x96FF, 0x0000, 0x0000, 0x0000, /*16FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002
};

uint16 uca900_p017[]= {
  0x2F66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1700*/
  0x2F67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1701*/
  0x2F68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1702*/
  0x2F69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1703*/
  0x2F6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1704*/
  0x2F6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1705*/
  0x2F6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1706*/
  0x2F6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1707*/
  0x2F6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1708*/
  0x2F6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1709*/
  0x2F70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*170A*/
  0x2F71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*170B*/
  0x2F72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*170C*/
  0xFBC0, 0x0020, 0x0002, 0x970D, 0x0000, 0x0000, 0x0002, /*170D*/
  0x2F73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*170E*/
  0x2F74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*170F*/
  0x2F75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1710*/
  0x2F76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1711*/
  0x2F77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1712*/
  0x2F78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1713*/
  0x2F79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1714*/
  0xFBC0, 0x0020, 0x0002, 0x9715, 0x0000, 0x0000, 0x0002, /*1715*/
  0xFBC0, 0x0020, 0x0002, 0x9716, 0x0000, 0x0000, 0x0002, /*1716*/
  0xFBC0, 0x0020, 0x0002, 0x9717, 0x0000, 0x0000, 0x0002, /*1717*/
  0xFBC0, 0x0020, 0x0002, 0x9718, 0x0000, 0x0000, 0x0002, /*1718*/
  0xFBC0, 0x0020, 0x0002, 0x9719, 0x0000, 0x0000, 0x0002, /*1719*/
  0xFBC0, 0x0020, 0x0002, 0x971A, 0x0000, 0x0000, 0x0002, /*171A*/
  0xFBC0, 0x0020, 0x0002, 0x971B, 0x0000, 0x0000, 0x0002, /*171B*/
  0xFBC0, 0x0020, 0x0002, 0x971C, 0x0000, 0x0000, 0x0002, /*171C*/
  0xFBC0, 0x0020, 0x0002, 0x971D, 0x0000, 0x0000, 0x0002, /*171D*/
  0xFBC0, 0x0020, 0x0002, 0x971E, 0x0000, 0x0000, 0x0002, /*171E*/
  0xFBC0, 0x0020, 0x0002, 0x971F, 0x0000, 0x0000, 0x0002, /*171F*/
  0x2F7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1720*/
  0x2F7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1721*/
  0x2F7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1722*/
  0x2F7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1723*/
  0x2F7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1724*/
  0x2F7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1725*/
  0x2F80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1726*/
  0x2F81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1727*/
  0x2F82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1728*/
  0x2F83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1729*/
  0x2F84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*172A*/
  0x2F85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*172B*/
  0x2F86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*172C*/
  0x2F87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*172D*/
  0x2F88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*172E*/
  0x2F89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*172F*/
  0x2F8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1730*/
  0x2F8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1731*/
  0x2F8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1732*/
  0x2F8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1733*/
  0x2F8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1734*/
  0x0296, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1735*/
  0x0297, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1736*/
  0xFBC0, 0x0020, 0x0002, 0x9737, 0x0000, 0x0000, 0x0002, /*1737*/
  0xFBC0, 0x0020, 0x0002, 0x9738, 0x0000, 0x0000, 0x0002, /*1738*/
  0xFBC0, 0x0020, 0x0002, 0x9739, 0x0000, 0x0000, 0x0002, /*1739*/
  0xFBC0, 0x0020, 0x0002, 0x973A, 0x0000, 0x0000, 0x0002, /*173A*/
  0xFBC0, 0x0020, 0x0002, 0x973B, 0x0000, 0x0000, 0x0002, /*173B*/
  0xFBC0, 0x0020, 0x0002, 0x973C, 0x0000, 0x0000, 0x0002, /*173C*/
  0xFBC0, 0x0020, 0x0002, 0x973D, 0x0000, 0x0000, 0x0002, /*173D*/
  0xFBC0, 0x0020, 0x0002, 0x973E, 0x0000, 0x0000, 0x0002, /*173E*/
  0xFBC0, 0x0020, 0x0002, 0x973F, 0x0000, 0x0000, 0x0002, /*173F*/
  0x2F8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1740*/
  0x2F90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1741*/
  0x2F91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1742*/
  0x2F92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1743*/
  0x2F93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1744*/
  0x2F94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1745*/
  0x2F95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1746*/
  0x2F96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1747*/
  0x2F97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1748*/
  0x2F98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1749*/
  0x2F99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*174A*/
  0x2F9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*174B*/
  0x2F9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*174C*/
  0x2F9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*174D*/
  0x2F9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*174E*/
  0x2F9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*174F*/
  0x2F9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1750*/
  0x2FA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1751*/
  0x2FA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1752*/
  0x2FA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1753*/
  0xFBC0, 0x0020, 0x0002, 0x9754, 0x0000, 0x0000, 0x0002, /*1754*/
  0xFBC0, 0x0020, 0x0002, 0x9755, 0x0000, 0x0000, 0x0002, /*1755*/
  0xFBC0, 0x0020, 0x0002, 0x9756, 0x0000, 0x0000, 0x0002, /*1756*/
  0xFBC0, 0x0020, 0x0002, 0x9757, 0x0000, 0x0000, 0x0002, /*1757*/
  0xFBC0, 0x0020, 0x0002, 0x9758, 0x0000, 0x0000, 0x0002, /*1758*/
  0xFBC0, 0x0020, 0x0002, 0x9759, 0x0000, 0x0000, 0x0002, /*1759*/
  0xFBC0, 0x0020, 0x0002, 0x975A, 0x0000, 0x0000, 0x0002, /*175A*/
  0xFBC0, 0x0020, 0x0002, 0x975B, 0x0000, 0x0000, 0x0002, /*175B*/
  0xFBC0, 0x0020, 0x0002, 0x975C, 0x0000, 0x0000, 0x0002, /*175C*/
  0xFBC0, 0x0020, 0x0002, 0x975D, 0x0000, 0x0000, 0x0002, /*175D*/
  0xFBC0, 0x0020, 0x0002, 0x975E, 0x0000, 0x0000, 0x0002, /*175E*/
  0xFBC0, 0x0020, 0x0002, 0x975F, 0x0000, 0x0000, 0x0002, /*175F*/
  0x2FA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1760*/
  0x2FA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1761*/
  0x2FA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1762*/
  0x2FA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1763*/
  0x2FA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1764*/
  0x2FA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1765*/
  0x2FA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1766*/
  0x2FAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1767*/
  0x2FAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1768*/
  0x2FAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1769*/
  0x2FAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*176A*/
  0x2FAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*176B*/
  0x2FAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*176C*/
  0xFBC0, 0x0020, 0x0002, 0x976D, 0x0000, 0x0000, 0x0002, /*176D*/
  0x2FB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*176E*/
  0x2FB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*176F*/
  0x2FB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1770*/
  0xFBC0, 0x0020, 0x0002, 0x9771, 0x0000, 0x0000, 0x0002, /*1771*/
  0x2FB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1772*/
  0x2FB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1773*/
  0xFBC0, 0x0020, 0x0002, 0x9774, 0x0000, 0x0000, 0x0002, /*1774*/
  0xFBC0, 0x0020, 0x0002, 0x9775, 0x0000, 0x0000, 0x0002, /*1775*/
  0xFBC0, 0x0020, 0x0002, 0x9776, 0x0000, 0x0000, 0x0002, /*1776*/
  0xFBC0, 0x0020, 0x0002, 0x9777, 0x0000, 0x0000, 0x0002, /*1777*/
  0xFBC0, 0x0020, 0x0002, 0x9778, 0x0000, 0x0000, 0x0002, /*1778*/
  0xFBC0, 0x0020, 0x0002, 0x9779, 0x0000, 0x0000, 0x0002, /*1779*/
  0xFBC0, 0x0020, 0x0002, 0x977A, 0x0000, 0x0000, 0x0002, /*177A*/
  0xFBC0, 0x0020, 0x0002, 0x977B, 0x0000, 0x0000, 0x0002, /*177B*/
  0xFBC0, 0x0020, 0x0002, 0x977C, 0x0000, 0x0000, 0x0002, /*177C*/
  0xFBC0, 0x0020, 0x0002, 0x977D, 0x0000, 0x0000, 0x0002, /*177D*/
  0xFBC0, 0x0020, 0x0002, 0x977E, 0x0000, 0x0000, 0x0002, /*177E*/
  0xFBC0, 0x0020, 0x0002, 0x977F, 0x0000, 0x0000, 0x0002, /*177F*/
  0x3116, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1780*/
  0x3117, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1781*/
  0x3118, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1782*/
  0x3119, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1783*/
  0x311A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1784*/
  0x311B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1785*/
  0x311C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1786*/
  0x311D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1787*/
  0x311E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1788*/
  0x311F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1789*/
  0x3120, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*178A*/
  0x3121, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*178B*/
  0x3122, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*178C*/
  0x3123, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*178D*/
  0x3124, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*178E*/
  0x3125, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*178F*/
  0x3126, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1790*/
  0x3127, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1791*/
  0x3128, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1792*/
  0x3129, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1793*/
  0x312A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1794*/
  0x312B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1795*/
  0x312C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1796*/
  0x312D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1797*/
  0x312E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1798*/
  0x312F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1799*/
  0x3130, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*179A*/
  0x3131, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*179B*/
  0x3132, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*179C*/
  0x3133, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*179D*/
  0x3134, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*179E*/
  0x3135, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*179F*/
  0x3136, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17A0*/
  0x3137, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17A1*/
  0x3138, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17A2*/
  0x313A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17A3*/
  0x313B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17A4*/
  0x313C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17A5*/
  0x313D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17A6*/
  0x313E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17A7*/
  0x313F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17A8*/
  0x3140, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17A9*/
  0x3141, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17AA*/
  0x3142, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17AB*/
  0x3143, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17AC*/
  0x3144, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17AD*/
  0x3145, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17AE*/
  0x3146, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17AF*/
  0x3147, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17B0*/
  0x3148, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17B1*/
  0x3149, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17B2*/
  0x314A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*17B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*17B5*/
  0x314B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17B6*/
  0x314C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17B7*/
  0x314D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17B8*/
  0x314E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17B9*/
  0x314F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17BA*/
  0x3150, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17BB*/
  0x3151, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17BC*/
  0x3152, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17BD*/
  0x3153, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17BE*/
  0x3154, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17BF*/
  0x3155, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17C0*/
  0x3156, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17C1*/
  0x3157, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17C2*/
  0x3158, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17C3*/
  0x3159, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17C4*/
  0x315A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17C5*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17C6*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17C7*/
  0x0000, 0x00E6, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17C8*/
  0x0000, 0x00E7, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17C9*/
  0x0000, 0x00E8, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17CA*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17CB*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17CC*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17CD*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17CE*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17CF*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17D0*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17D1*/
  0x315B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*17D3*/
  0x029B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17D4*/
  0x029C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17D5*/
  0x0259, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17D6*/
  0x1BFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17D7*/
  0x0412, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17D8*/
  0x0413, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17D9*/
  0x0414, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17DA*/
  0x1C1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17DB*/
  0x3139, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17DC*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17DD*/
  0xFBC0, 0x0020, 0x0002, 0x97DE, 0x0000, 0x0000, 0x0002, /*17DE*/
  0xFBC0, 0x0020, 0x0002, 0x97DF, 0x0000, 0x0000, 0x0002, /*17DF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17E0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17E1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17E2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17E3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17E4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17E5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17E6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17E7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17E8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17E9*/
  0xFBC0, 0x0020, 0x0002, 0x97EA, 0x0000, 0x0000, 0x0002, /*17EA*/
  0xFBC0, 0x0020, 0x0002, 0x97EB, 0x0000, 0x0000, 0x0002, /*17EB*/
  0xFBC0, 0x0020, 0x0002, 0x97EC, 0x0000, 0x0000, 0x0002, /*17EC*/
  0xFBC0, 0x0020, 0x0002, 0x97ED, 0x0000, 0x0000, 0x0002, /*17ED*/
  0xFBC0, 0x0020, 0x0002, 0x97EE, 0x0000, 0x0000, 0x0002, /*17EE*/
  0xFBC0, 0x0020, 0x0002, 0x97EF, 0x0000, 0x0000, 0x0002, /*17EF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17F0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17F1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17F2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17F3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17F4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17F5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17F6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17F7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17F8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*17F9*/
  0xFBC0, 0x0020, 0x0002, 0x97FA, 0x0000, 0x0000, 0x0002, /*17FA*/
  0xFBC0, 0x0020, 0x0002, 0x97FB, 0x0000, 0x0000, 0x0002, /*17FB*/
  0xFBC0, 0x0020, 0x0002, 0x97FC, 0x0000, 0x0000, 0x0002, /*17FC*/
  0xFBC0, 0x0020, 0x0002, 0x97FD, 0x0000, 0x0000, 0x0002, /*17FD*/
  0xFBC0, 0x0020, 0x0002, 0x97FE, 0x0000, 0x0000, 0x0002, /*17FE*/
  0xFBC0, 0x0020, 0x0002, 0x97FF, 0x0000, 0x0000, 0x0002  /*17FF*/
};

uint16 uca900_p018[]= {
  0x03D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1800*/
  0x0278, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1801*/
  0x022C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1802*/
  0x027D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1803*/
  0x0256, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1804*/
  0x0257, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1805*/
  0x0211, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1806*/
  0x0212, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1807*/
  0x022D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1808*/
  0x027E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1809*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*180A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*180B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*180C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*180D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*180E*/
  0xFBC0, 0x0020, 0x0002, 0x980F, 0x0000, 0x0000, 0x0002, /*180F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1810*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1811*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1812*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1813*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1814*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1815*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1816*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1817*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1818*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1819*/
  0xFBC0, 0x0020, 0x0002, 0x981A, 0x0000, 0x0000, 0x0002, /*181A*/
  0xFBC0, 0x0020, 0x0002, 0x981B, 0x0000, 0x0000, 0x0002, /*181B*/
  0xFBC0, 0x0020, 0x0002, 0x981C, 0x0000, 0x0000, 0x0002, /*181C*/
  0xFBC0, 0x0020, 0x0002, 0x981D, 0x0000, 0x0000, 0x0002, /*181D*/
  0xFBC0, 0x0020, 0x0002, 0x981E, 0x0000, 0x0000, 0x0002, /*181E*/
  0xFBC0, 0x0020, 0x0002, 0x981F, 0x0000, 0x0000, 0x0002, /*181F*/
  0x32E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1820*/
  0x32E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1821*/
  0x32E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1822*/
  0x32EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1823*/
  0x32ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1824*/
  0x32F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1825*/
  0x32F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1826*/
  0x32F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1827*/
  0x32F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1828*/
  0x32F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1829*/
  0x32FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*182A*/
  0x32FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*182B*/
  0x3301, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*182C*/
  0x3303, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*182D*/
  0x3308, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*182E*/
  0x330A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*182F*/
  0x330B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1830*/
  0x330C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1831*/
  0x3313, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1832*/
  0x3316, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1833*/
  0x3319, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1834*/
  0x331E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1835*/
  0x3322, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1836*/
  0x3325, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1837*/
  0x3327, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1838*/
  0x3329, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1839*/
  0x332C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*183A*/
  0x3331, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*183B*/
  0x3332, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*183C*/
  0x3335, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*183D*/
  0x3339, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*183E*/
  0x333C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*183F*/
  0x333D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1840*/
  0x333E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1841*/
  0x333F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1842*/
  0x32DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1843*/
  0x32E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1844*/
  0x32E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1845*/
  0x32EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1846*/
  0x32EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1847*/
  0x32F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1848*/
  0x32F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1849*/
  0x32F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*184A*/
  0x32FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*184B*/
  0x32FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*184C*/
  0x3302, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*184D*/
  0x3304, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*184E*/
  0x3309, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*184F*/
  0x3314, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1850*/
  0x3317, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1851*/
  0x331A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1852*/
  0x331F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1853*/
  0x3333, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1854*/
  0x3323, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1855*/
  0x3328, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1856*/
  0x332D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1857*/
  0x3337, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1858*/
  0x333A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1859*/
  0x3340, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*185A*/
  0x3341, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*185B*/
  0x331C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*185C*/
  0x32E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*185D*/
  0x32E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*185E*/
  0x32EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*185F*/
  0x32F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1860*/
  0x32EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1861*/
  0x32F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1862*/
  0x332E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1863*/
  0x3305, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1864*/
  0x3307, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1865*/
  0x3300, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1866*/
  0x330D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1867*/
  0x3315, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1868*/
  0x3318, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1869*/
  0x3320, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*186A*/
  0x332A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*186B*/
  0x3338, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*186C*/
  0x333B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*186D*/
  0x3334, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*186E*/
  0x3336, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*186F*/
  0x3342, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1870*/
  0x331B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1871*/
  0x3324, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1872*/
  0x32E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1873*/
  0x332F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1874*/
  0x3326, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1875*/
  0x332B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1876*/
  0x3321, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1877*/
  0xFBC0, 0x0020, 0x0002, 0x9878, 0x0000, 0x0000, 0x0002, /*1878*/
  0xFBC0, 0x0020, 0x0002, 0x9879, 0x0000, 0x0000, 0x0002, /*1879*/
  0xFBC0, 0x0020, 0x0002, 0x987A, 0x0000, 0x0000, 0x0002, /*187A*/
  0xFBC0, 0x0020, 0x0002, 0x987B, 0x0000, 0x0000, 0x0002, /*187B*/
  0xFBC0, 0x0020, 0x0002, 0x987C, 0x0000, 0x0000, 0x0002, /*187C*/
  0xFBC0, 0x0020, 0x0002, 0x987D, 0x0000, 0x0000, 0x0002, /*187D*/
  0xFBC0, 0x0020, 0x0002, 0x987E, 0x0000, 0x0000, 0x0002, /*187E*/
  0xFBC0, 0x0020, 0x0002, 0x987F, 0x0000, 0x0000, 0x0002, /*187F*/
  0x32D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1880*/
  0x32D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1881*/
  0x32DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1882*/
  0x32DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1883*/
  0x32DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1884*/
  0x32DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1885*/
  0x32DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1886*/
  0x32E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1887*/
  0x32E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1888*/
  0x3330, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1889*/
  0x32FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*188A*/
  0x331D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*188B*/
  0x3343, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*188C*/
  0x3345, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*188D*/
  0x3346, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*188E*/
  0x3348, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*188F*/
  0x3349, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1890*/
  0x334C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1891*/
  0x334E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1892*/
  0x334F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1893*/
  0x3351, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1894*/
  0x3353, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1895*/
  0x3355, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1896*/
  0x3356, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1897*/
  0x334A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1898*/
  0x3354, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1899*/
  0x3306, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*189A*/
  0x32FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*189B*/
  0x330E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*189C*/
  0x330F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*189D*/
  0x3344, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*189E*/
  0x3347, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*189F*/
  0x334B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18A0*/
  0x334D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18A1*/
  0x3310, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18A2*/
  0x3352, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18A3*/
  0x3311, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18A4*/
  0x3312, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18A5*/
  0x3357, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18A6*/
  0x3358, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18A7*/
  0x3350, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18A8*/
  0x335A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18A9*/
  0x3359, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18AA*/
  0xFBC0, 0x0020, 0x0002, 0x98AB, 0x0000, 0x0000, 0x0002, /*18AB*/
  0xFBC0, 0x0020, 0x0002, 0x98AC, 0x0000, 0x0000, 0x0002, /*18AC*/
  0xFBC0, 0x0020, 0x0002, 0x98AD, 0x0000, 0x0000, 0x0002, /*18AD*/
  0xFBC0, 0x0020, 0x0002, 0x98AE, 0x0000, 0x0000, 0x0002, /*18AE*/
  0xFBC0, 0x0020, 0x0002, 0x98AF, 0x0000, 0x0000, 0x0002, /*18AF*/
  0x3676, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18B0*/
  0x3677, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18B1*/
  0x3678, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18B2*/
  0x3679, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18B3*/
  0x367A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18B4*/
  0x367B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18B5*/
  0x367C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18B6*/
  0x367D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18B7*/
  0x367E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18B8*/
  0x367F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18B9*/
  0x3680, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18BA*/
  0x3681, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18BB*/
  0x3682, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18BC*/
  0x3683, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18BD*/
  0x3684, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18BE*/
  0x3685, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18BF*/
  0x3686, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18C0*/
  0x3687, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18C1*/
  0x3688, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18C2*/
  0x3689, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18C3*/
  0x368A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18C4*/
  0x368B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18C5*/
  0x368C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18C6*/
  0x368D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18C7*/
  0x368E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18C8*/
  0x368F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18C9*/
  0x3690, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18CA*/
  0x3691, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18CB*/
  0x3692, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18CC*/
  0x3693, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18CD*/
  0x3694, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18CE*/
  0x3695, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18CF*/
  0x3696, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18D0*/
  0x3697, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18D1*/
  0x3698, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18D2*/
  0x3699, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18D3*/
  0x369A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18D4*/
  0x369B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18D5*/
  0x369C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18D6*/
  0x369D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18D7*/
  0x369E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18D8*/
  0x369F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18D9*/
  0x36A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18DA*/
  0x36A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18DB*/
  0x36A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18DC*/
  0x36A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18DD*/
  0x36A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18DE*/
  0x36A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18DF*/
  0x36A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18E0*/
  0x36A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18E1*/
  0x36A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18E2*/
  0x36A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18E3*/
  0x36AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18E4*/
  0x36AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18E5*/
  0x36AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18E6*/
  0x36AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18E7*/
  0x36AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18E8*/
  0x36AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18E9*/
  0x36B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18EA*/
  0x36B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18EB*/
  0x36B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18EC*/
  0x36B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18ED*/
  0x36B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18EE*/
  0x36B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18EF*/
  0x36B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18F0*/
  0x36B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18F1*/
  0x36B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18F2*/
  0x36B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18F3*/
  0x36BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18F4*/
  0x36BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*18F5*/
  0xFBC0, 0x0020, 0x0002, 0x98F6, 0x0000, 0x0000, 0x0002, /*18F6*/
  0xFBC0, 0x0020, 0x0002, 0x98F7, 0x0000, 0x0000, 0x0002, /*18F7*/
  0xFBC0, 0x0020, 0x0002, 0x98F8, 0x0000, 0x0000, 0x0002, /*18F8*/
  0xFBC0, 0x0020, 0x0002, 0x98F9, 0x0000, 0x0000, 0x0002, /*18F9*/
  0xFBC0, 0x0020, 0x0002, 0x98FA, 0x0000, 0x0000, 0x0002, /*18FA*/
  0xFBC0, 0x0020, 0x0002, 0x98FB, 0x0000, 0x0000, 0x0002, /*18FB*/
  0xFBC0, 0x0020, 0x0002, 0x98FC, 0x0000, 0x0000, 0x0002, /*18FC*/
  0xFBC0, 0x0020, 0x0002, 0x98FD, 0x0000, 0x0000, 0x0002, /*18FD*/
  0xFBC0, 0x0020, 0x0002, 0x98FE, 0x0000, 0x0000, 0x0002, /*18FE*/
  0xFBC0, 0x0020, 0x0002, 0x98FF, 0x0000, 0x0000, 0x0002  /*18FF*/
};

uint16 uca900_p019[]= {
  0x2F34, 0x0020, 0x0002, 0x0000, 0x0000, /*1900*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F35, 0x0020, 0x0002, 0x0000, 0x0000, /*1901*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F36, 0x0020, 0x0002, 0x0000, 0x0000, /*1902*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F37, 0x0020, 0x0002, 0x0000, 0x0000, /*1903*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F38, 0x0020, 0x0002, 0x0000, 0x0000, /*1904*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F39, 0x0020, 0x0002, 0x0000, 0x0000, /*1905*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F3A, 0x0020, 0x0002, 0x0000, 0x0000, /*1906*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F3B, 0x0020, 0x0002, 0x0000, 0x0000, /*1907*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F3C, 0x0020, 0x0002, 0x0000, 0x0000, /*1908*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F3D, 0x0020, 0x0002, 0x0000, 0x0000, /*1909*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F3E, 0x0020, 0x0002, 0x0000, 0x0000, /*190A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F3F, 0x0020, 0x0002, 0x0000, 0x0000, /*190B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F40, 0x0020, 0x0002, 0x0000, 0x0000, /*190C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F41, 0x0020, 0x0002, 0x0000, 0x0000, /*190D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F42, 0x0020, 0x0002, 0x0000, 0x0000, /*190E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F43, 0x0020, 0x0002, 0x0000, 0x0000, /*190F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F44, 0x0020, 0x0002, 0x0000, 0x0000, /*1910*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F45, 0x0020, 0x0002, 0x0000, 0x0000, /*1911*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F46, 0x0020, 0x0002, 0x0000, 0x0000, /*1912*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F47, 0x0020, 0x0002, 0x0000, 0x0000, /*1913*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F48, 0x0020, 0x0002, 0x0000, 0x0000, /*1914*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F49, 0x0020, 0x0002, 0x0000, 0x0000, /*1915*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F4A, 0x0020, 0x0002, 0x0000, 0x0000, /*1916*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F4B, 0x0020, 0x0002, 0x0000, 0x0000, /*1917*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F4C, 0x0020, 0x0002, 0x0000, 0x0000, /*1918*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F4D, 0x0020, 0x0002, 0x0000, 0x0000, /*1919*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F4E, 0x0020, 0x0002, 0x0000, 0x0000, /*191A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F4F, 0x0020, 0x0002, 0x0000, 0x0000, /*191B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F50, 0x0020, 0x0002, 0x0000, 0x0000, /*191C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F3C, 0x0020, 0x0004, 0x2F5A, 0x0020, /*191D*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2F3F, 0x0020, 0x0004, 0x2F5B, 0x0020, /*191E*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x991F, 0x0000, /*191F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2F51, 0x0020, 0x0002, 0x0000, 0x0000, /*1920*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F52, 0x0020, 0x0002, 0x0000, 0x0000, /*1921*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F53, 0x0020, 0x0002, 0x0000, 0x0000, /*1922*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F54, 0x0020, 0x0002, 0x0000, 0x0000, /*1923*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F55, 0x0020, 0x0002, 0x0000, 0x0000, /*1924*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F56, 0x0020, 0x0002, 0x0000, 0x0000, /*1925*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F57, 0x0020, 0x0002, 0x0000, 0x0000, /*1926*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F58, 0x0020, 0x0002, 0x0000, 0x0000, /*1927*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F59, 0x0020, 0x0002, 0x0000, 0x0000, /*1928*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F5A, 0x0020, 0x0002, 0x0000, 0x0000, /*1929*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F5B, 0x0020, 0x0002, 0x0000, 0x0000, /*192A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F5C, 0x0020, 0x0002, 0x0000, 0x0000, /*192B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x992C, 0x0000, /*192C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x992D, 0x0000, /*192D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x992E, 0x0000, /*192E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x992F, 0x0000, /*192F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2F5D, 0x0020, 0x0002, 0x0000, 0x0000, /*1930*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F5E, 0x0020, 0x0002, 0x0000, 0x0000, /*1931*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F5F, 0x0020, 0x0002, 0x0000, 0x0000, /*1932*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F60, 0x0020, 0x0002, 0x0000, 0x0000, /*1933*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F61, 0x0020, 0x0002, 0x0000, 0x0000, /*1934*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F62, 0x0020, 0x0002, 0x0000, 0x0000, /*1935*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F63, 0x0020, 0x0002, 0x0000, 0x0000, /*1936*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F64, 0x0020, 0x0002, 0x0000, 0x0000, /*1937*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2F65, 0x0020, 0x0002, 0x0000, 0x0000, /*1938*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00F1, 0x0002, 0x0000, 0x0000, /*1939*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00F2, 0x0002, 0x0000, 0x0000, /*193A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00F3, 0x0002, 0x0000, 0x0000, /*193B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x993C, 0x0000, /*193C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x993D, 0x0000, /*193D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x993E, 0x0000, /*193E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x993F, 0x0000, /*193F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x054A, 0x0020, 0x0002, 0x0000, 0x0000, /*1940*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x9941, 0x0000, /*1941*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9942, 0x0000, /*1942*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9943, 0x0000, /*1943*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0264, 0x0020, 0x0002, 0x0000, 0x0000, /*1944*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x026D, 0x0020, 0x0002, 0x0000, 0x0000, /*1945*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, /*1946*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, /*1947*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, /*1948*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, /*1949*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, /*194A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, /*194B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, /*194C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, /*194D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, /*194E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, /*194F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x315C, 0x0020, 0x0002, 0x0000, 0x0000, /*1950*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x315D, 0x0020, 0x0002, 0x0000, 0x0000, /*1951*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x315E, 0x0020, 0x0002, 0x0000, 0x0000, /*1952*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x315F, 0x0020, 0x0002, 0x0000, 0x0000, /*1953*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3160, 0x0020, 0x0002, 0x0000, 0x0000, /*1954*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3161, 0x0020, 0x0002, 0x0000, 0x0000, /*1955*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3162, 0x0020, 0x0002, 0x0000, 0x0000, /*1956*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3163, 0x0020, 0x0002, 0x0000, 0x0000, /*1957*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3164, 0x0020, 0x0002, 0x0000, 0x0000, /*1958*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3165, 0x0020, 0x0002, 0x0000, 0x0000, /*1959*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3166, 0x0020, 0x0002, 0x0000, 0x0000, /*195A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3167, 0x0020, 0x0002, 0x0000, 0x0000, /*195B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3168, 0x0020, 0x0002, 0x0000, 0x0000, /*195C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3169, 0x0020, 0x0002, 0x0000, 0x0000, /*195D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x316A, 0x0020, 0x0002, 0x0000, 0x0000, /*195E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x316B, 0x0020, 0x0002, 0x0000, 0x0000, /*195F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x316C, 0x0020, 0x0002, 0x0000, 0x0000, /*1960*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x316D, 0x0020, 0x0002, 0x0000, 0x0000, /*1961*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x316E, 0x0020, 0x0002, 0x0000, 0x0000, /*1962*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x316F, 0x0020, 0x0002, 0x0000, 0x0000, /*1963*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3170, 0x0020, 0x0002, 0x0000, 0x0000, /*1964*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3171, 0x0020, 0x0002, 0x0000, 0x0000, /*1965*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3172, 0x0020, 0x0002, 0x0000, 0x0000, /*1966*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3173, 0x0020, 0x0002, 0x0000, 0x0000, /*1967*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3174, 0x0020, 0x0002, 0x0000, 0x0000, /*1968*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3175, 0x0020, 0x0002, 0x0000, 0x0000, /*1969*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3176, 0x0020, 0x0002, 0x0000, 0x0000, /*196A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3177, 0x0020, 0x0002, 0x0000, 0x0000, /*196B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3178, 0x0020, 0x0002, 0x0000, 0x0000, /*196C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3179, 0x0020, 0x0002, 0x0000, 0x0000, /*196D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x996E, 0x0000, /*196E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x996F, 0x0000, /*196F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x317A, 0x0020, 0x0002, 0x0000, 0x0000, /*1970*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x317B, 0x0020, 0x0002, 0x0000, 0x0000, /*1971*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x317C, 0x0020, 0x0002, 0x0000, 0x0000, /*1972*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x317D, 0x0020, 0x0002, 0x0000, 0x0000, /*1973*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x317E, 0x0020, 0x0002, 0x0000, 0x0000, /*1974*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x9975, 0x0000, /*1975*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9976, 0x0000, /*1976*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9977, 0x0000, /*1977*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9978, 0x0000, /*1978*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9979, 0x0000, /*1979*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x997A, 0x0000, /*197A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x997B, 0x0000, /*197B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x997C, 0x0000, /*197C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x997D, 0x0000, /*197D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x997E, 0x0000, /*197E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x997F, 0x0000, /*197F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x317F, 0x0020, 0x0002, 0x0000, 0x0000, /*1980*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3180, 0x0020, 0x0002, 0x0000, 0x0000, /*1981*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3181, 0x0020, 0x0002, 0x0000, 0x0000, /*1982*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3182, 0x0020, 0x0002, 0x0000, 0x0000, /*1983*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3183, 0x0020, 0x0002, 0x0000, 0x0000, /*1984*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3184, 0x0020, 0x0002, 0x0000, 0x0000, /*1985*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3185, 0x0020, 0x0002, 0x0000, 0x0000, /*1986*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3186, 0x0020, 0x0002, 0x0000, 0x0000, /*1987*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3187, 0x0020, 0x0002, 0x0000, 0x0000, /*1988*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3188, 0x0020, 0x0002, 0x0000, 0x0000, /*1989*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3189, 0x0020, 0x0002, 0x0000, 0x0000, /*198A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x318A, 0x0020, 0x0002, 0x0000, 0x0000, /*198B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x318B, 0x0020, 0x0002, 0x0000, 0x0000, /*198C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x318C, 0x0020, 0x0002, 0x0000, 0x0000, /*198D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x318D, 0x0020, 0x0002, 0x0000, 0x0000, /*198E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x318E, 0x0020, 0x0002, 0x0000, 0x0000, /*198F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x318F, 0x0020, 0x0002, 0x0000, 0x0000, /*1990*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3190, 0x0020, 0x0002, 0x0000, 0x0000, /*1991*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3191, 0x0020, 0x0002, 0x0000, 0x0000, /*1992*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3192, 0x0020, 0x0002, 0x0000, 0x0000, /*1993*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3193, 0x0020, 0x0002, 0x0000, 0x0000, /*1994*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3194, 0x0020, 0x0002, 0x0000, 0x0000, /*1995*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3195, 0x0020, 0x0002, 0x0000, 0x0000, /*1996*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3196, 0x0020, 0x0002, 0x0000, 0x0000, /*1997*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3197, 0x0020, 0x0002, 0x0000, 0x0000, /*1998*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3198, 0x0020, 0x0002, 0x0000, 0x0000, /*1999*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3199, 0x0020, 0x0002, 0x0000, 0x0000, /*199A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x319A, 0x0020, 0x0002, 0x0000, 0x0000, /*199B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x319B, 0x0020, 0x0002, 0x0000, 0x0000, /*199C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x319C, 0x0020, 0x0002, 0x0000, 0x0000, /*199D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x319D, 0x0020, 0x0002, 0x0000, 0x0000, /*199E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x319E, 0x0020, 0x0002, 0x0000, 0x0000, /*199F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x319F, 0x0020, 0x0002, 0x0000, 0x0000, /*19A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31A0, 0x0020, 0x0002, 0x0000, 0x0000, /*19A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31A1, 0x0020, 0x0002, 0x0000, 0x0000, /*19A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31A2, 0x0020, 0x0002, 0x0000, 0x0000, /*19A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31A3, 0x0020, 0x0002, 0x0000, 0x0000, /*19A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31A4, 0x0020, 0x0002, 0x0000, 0x0000, /*19A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31A5, 0x0020, 0x0002, 0x0000, 0x0000, /*19A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31A6, 0x0020, 0x0002, 0x0000, 0x0000, /*19A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31A7, 0x0020, 0x0002, 0x0000, 0x0000, /*19A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31A8, 0x0020, 0x0002, 0x0000, 0x0000, /*19A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31A9, 0x0020, 0x0002, 0x0000, 0x0000, /*19AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31AA, 0x0020, 0x0002, 0x0000, 0x0000, /*19AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x99AC, 0x0000, /*19AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x99AD, 0x0000, /*19AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x99AE, 0x0000, /*19AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x99AF, 0x0000, /*19AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x31AB, 0x0020, 0x0002, 0x0000, 0x0000, /*19B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31AC, 0x0020, 0x0002, 0x0000, 0x0000, /*19B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31AD, 0x0020, 0x0002, 0x0000, 0x0000, /*19B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31AE, 0x0020, 0x0002, 0x0000, 0x0000, /*19B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31AF, 0x0020, 0x0002, 0x0000, 0x0000, /*19B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31B0, 0x0020, 0x0002, 0x0000, 0x0000, /*19B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31B1, 0x0020, 0x0002, 0x0000, 0x0000, /*19B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31B2, 0x0020, 0x0002, 0x0000, 0x0000, /*19B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31B3, 0x0020, 0x0002, 0x0000, 0x0000, /*19B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31B4, 0x0020, 0x0002, 0x0000, 0x0000, /*19B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31B5, 0x0020, 0x0002, 0x0000, 0x0000, /*19BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31B6, 0x0020, 0x0002, 0x0000, 0x0000, /*19BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31B7, 0x0020, 0x0002, 0x0000, 0x0000, /*19BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31B8, 0x0020, 0x0002, 0x0000, 0x0000, /*19BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31B9, 0x0020, 0x0002, 0x0000, 0x0000, /*19BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31BA, 0x0020, 0x0002, 0x0000, 0x0000, /*19BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31BB, 0x0020, 0x0002, 0x0000, 0x0000, /*19C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31BC, 0x0020, 0x0002, 0x0000, 0x0000, /*19C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31BD, 0x0020, 0x0002, 0x0000, 0x0000, /*19C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31BE, 0x0020, 0x0002, 0x0000, 0x0000, /*19C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31BF, 0x0020, 0x0002, 0x0000, 0x0000, /*19C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31C0, 0x0020, 0x0002, 0x0000, 0x0000, /*19C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31C1, 0x0020, 0x0002, 0x0000, 0x0000, /*19C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31C2, 0x0020, 0x0002, 0x0000, 0x0000, /*19C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31C3, 0x0020, 0x0002, 0x0000, 0x0000, /*19C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31C4, 0x0020, 0x0002, 0x0000, 0x0000, /*19C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x99CA, 0x0000, /*19CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x99CB, 0x0000, /*19CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x99CC, 0x0000, /*19CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x99CD, 0x0000, /*19CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x99CE, 0x0000, /*19CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x99CF, 0x0000, /*19CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, /*19D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, /*19D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, /*19D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, /*19D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, /*19D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, /*19D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, /*19D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, /*19D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, /*19D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, /*19D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, /*19DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x99DB, 0x0000, /*19DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x99DC, 0x0000, /*19DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x99DD, 0x0000, /*19DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x319B, 0x0020, 0x0004, 0x31B1, 0x0020, /*19DE*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x319B, 0x0020, 0x0004, 0x31B1, 0x0020, /*19DF*/
  0x0004, 0x31BC, 0x0020, 0x0004, 0x0003,
  0x0551, 0x0020, 0x0002, 0x0000, 0x0000, /*19E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0552, 0x0020, 0x0002, 0x0000, 0x0000, /*19E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0553, 0x0020, 0x0002, 0x0000, 0x0000, /*19E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0554, 0x0020, 0x0002, 0x0000, 0x0000, /*19E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0555, 0x0020, 0x0002, 0x0000, 0x0000, /*19E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0556, 0x0020, 0x0002, 0x0000, 0x0000, /*19E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0557, 0x0020, 0x0002, 0x0000, 0x0000, /*19E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0558, 0x0020, 0x0002, 0x0000, 0x0000, /*19E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0559, 0x0020, 0x0002, 0x0000, 0x0000, /*19E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x055A, 0x0020, 0x0002, 0x0000, 0x0000, /*19E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x055B, 0x0020, 0x0002, 0x0000, 0x0000, /*19EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x055C, 0x0020, 0x0002, 0x0000, 0x0000, /*19EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x055D, 0x0020, 0x0002, 0x0000, 0x0000, /*19EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x055E, 0x0020, 0x0002, 0x0000, 0x0000, /*19ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x055F, 0x0020, 0x0002, 0x0000, 0x0000, /*19EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0560, 0x0020, 0x0002, 0x0000, 0x0000, /*19EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0561, 0x0020, 0x0002, 0x0000, 0x0000, /*19F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0562, 0x0020, 0x0002, 0x0000, 0x0000, /*19F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0563, 0x0020, 0x0002, 0x0000, 0x0000, /*19F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0564, 0x0020, 0x0002, 0x0000, 0x0000, /*19F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0565, 0x0020, 0x0002, 0x0000, 0x0000, /*19F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0566, 0x0020, 0x0002, 0x0000, 0x0000, /*19F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0567, 0x0020, 0x0002, 0x0000, 0x0000, /*19F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0568, 0x0020, 0x0002, 0x0000, 0x0000, /*19F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0569, 0x0020, 0x0002, 0x0000, 0x0000, /*19F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x056A, 0x0020, 0x0002, 0x0000, 0x0000, /*19F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x056B, 0x0020, 0x0002, 0x0000, 0x0000, /*19FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x056C, 0x0020, 0x0002, 0x0000, 0x0000, /*19FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x056D, 0x0020, 0x0002, 0x0000, 0x0000, /*19FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x056E, 0x0020, 0x0002, 0x0000, 0x0000, /*19FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x056F, 0x0020, 0x0002, 0x0000, 0x0000, /*19FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0570, 0x0020, 0x0002, 0x0000, 0x0000, /*19FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p01A[]= {
  0x2FB5, 0x0020, 0x0002, 0x0000, 0x0000, /*1A00*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FB6, 0x0020, 0x0002, 0x0000, 0x0000, /*1A01*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FB7, 0x0020, 0x0002, 0x0000, 0x0000, /*1A02*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FB8, 0x0020, 0x0002, 0x0000, 0x0000, /*1A03*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FB9, 0x0020, 0x0002, 0x0000, 0x0000, /*1A04*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FBA, 0x0020, 0x0002, 0x0000, 0x0000, /*1A05*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FBB, 0x0020, 0x0002, 0x0000, 0x0000, /*1A06*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FBC, 0x0020, 0x0002, 0x0000, 0x0000, /*1A07*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FBD, 0x0020, 0x0002, 0x0000, 0x0000, /*1A08*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FBE, 0x0020, 0x0002, 0x0000, 0x0000, /*1A09*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FBF, 0x0020, 0x0002, 0x0000, 0x0000, /*1A0A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FC0, 0x0020, 0x0002, 0x0000, 0x0000, /*1A0B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FC1, 0x0020, 0x0002, 0x0000, 0x0000, /*1A0C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FC2, 0x0020, 0x0002, 0x0000, 0x0000, /*1A0D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FC3, 0x0020, 0x0002, 0x0000, 0x0000, /*1A0E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FC4, 0x0020, 0x0002, 0x0000, 0x0000, /*1A0F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FC5, 0x0020, 0x0002, 0x0000, 0x0000, /*1A10*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FC6, 0x0020, 0x0002, 0x0000, 0x0000, /*1A11*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FC7, 0x0020, 0x0002, 0x0000, 0x0000, /*1A12*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FC8, 0x0020, 0x0002, 0x0000, 0x0000, /*1A13*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FC9, 0x0020, 0x0002, 0x0000, 0x0000, /*1A14*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FCA, 0x0020, 0x0002, 0x0000, 0x0000, /*1A15*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FCB, 0x0020, 0x0002, 0x0000, 0x0000, /*1A16*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FCC, 0x0020, 0x0002, 0x0000, 0x0000, /*1A17*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FCD, 0x0020, 0x0002, 0x0000, 0x0000, /*1A18*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FCE, 0x0020, 0x0002, 0x0000, 0x0000, /*1A19*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FCF, 0x0020, 0x0002, 0x0000, 0x0000, /*1A1A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2FD0, 0x0020, 0x0002, 0x0000, 0x0000, /*1A1B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x9A1C, 0x0000, /*1A1C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9A1D, 0x0000, /*1A1D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x02C9, 0x0020, 0x0002, 0x0000, 0x0000, /*1A1E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02CA, 0x0020, 0x0002, 0x0000, 0x0000, /*1A1F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31C5, 0x0020, 0x0002, 0x0000, 0x0000, /*1A20*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31C6, 0x0020, 0x0002, 0x0000, 0x0000, /*1A21*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31C7, 0x0020, 0x0002, 0x0000, 0x0000, /*1A22*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31C8, 0x0020, 0x0002, 0x0000, 0x0000, /*1A23*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31C9, 0x0020, 0x0002, 0x0000, 0x0000, /*1A24*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31CA, 0x0020, 0x0002, 0x0000, 0x0000, /*1A25*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31CB, 0x0020, 0x0002, 0x0000, 0x0000, /*1A26*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31CC, 0x0020, 0x0002, 0x0000, 0x0000, /*1A27*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31CD, 0x0020, 0x0002, 0x0000, 0x0000, /*1A28*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31CE, 0x0020, 0x0002, 0x0000, 0x0000, /*1A29*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31CF, 0x0020, 0x0002, 0x0000, 0x0000, /*1A2A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31D0, 0x0020, 0x0002, 0x0000, 0x0000, /*1A2B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31D1, 0x0020, 0x0002, 0x0000, 0x0000, /*1A2C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31D2, 0x0020, 0x0002, 0x0000, 0x0000, /*1A2D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31D3, 0x0020, 0x0002, 0x0000, 0x0000, /*1A2E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31D4, 0x0020, 0x0002, 0x0000, 0x0000, /*1A2F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31D5, 0x0020, 0x0002, 0x0000, 0x0000, /*1A30*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31D6, 0x0020, 0x0002, 0x0000, 0x0000, /*1A31*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31D7, 0x0020, 0x0002, 0x0000, 0x0000, /*1A32*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31D8, 0x0020, 0x0002, 0x0000, 0x0000, /*1A33*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31D9, 0x0020, 0x0002, 0x0000, 0x0000, /*1A34*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31DA, 0x0020, 0x0002, 0x0000, 0x0000, /*1A35*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31DB, 0x0020, 0x0002, 0x0000, 0x0000, /*1A36*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31DC, 0x0020, 0x0002, 0x0000, 0x0000, /*1A37*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31DD, 0x0020, 0x0002, 0x0000, 0x0000, /*1A38*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31DE, 0x0020, 0x0002, 0x0000, 0x0000, /*1A39*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31DF, 0x0020, 0x0002, 0x0000, 0x0000, /*1A3A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31E0, 0x0020, 0x0002, 0x0000, 0x0000, /*1A3B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31E1, 0x0020, 0x0002, 0x0000, 0x0000, /*1A3C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31E2, 0x0020, 0x0002, 0x0000, 0x0000, /*1A3D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31E3, 0x0020, 0x0002, 0x0000, 0x0000, /*1A3E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31E4, 0x0020, 0x0002, 0x0000, 0x0000, /*1A3F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31E5, 0x0020, 0x0002, 0x0000, 0x0000, /*1A40*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31E6, 0x0020, 0x0002, 0x0000, 0x0000, /*1A41*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31E7, 0x0020, 0x0002, 0x0000, 0x0000, /*1A42*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31E8, 0x0020, 0x0002, 0x0000, 0x0000, /*1A43*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31E9, 0x0020, 0x0002, 0x0000, 0x0000, /*1A44*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31EA, 0x0020, 0x0002, 0x0000, 0x0000, /*1A45*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31EB, 0x0020, 0x0002, 0x0000, 0x0000, /*1A46*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31EC, 0x0020, 0x0002, 0x0000, 0x0000, /*1A47*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31ED, 0x0020, 0x0002, 0x0000, 0x0000, /*1A48*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31EE, 0x0020, 0x0002, 0x0000, 0x0000, /*1A49*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31EF, 0x0020, 0x0002, 0x0000, 0x0000, /*1A4A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31F0, 0x0020, 0x0002, 0x0000, 0x0000, /*1A4B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31F1, 0x0020, 0x0002, 0x0000, 0x0000, /*1A4C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31FA, 0x0020, 0x0002, 0x0000, 0x0000, /*1A4D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31FB, 0x0020, 0x0002, 0x0000, 0x0000, /*1A4E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31FC, 0x0020, 0x0002, 0x0000, 0x0000, /*1A4F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31FD, 0x0020, 0x0002, 0x0000, 0x0000, /*1A50*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31FE, 0x0020, 0x0002, 0x0000, 0x0000, /*1A51*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31FF, 0x0020, 0x0002, 0x0000, 0x0000, /*1A52*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31F2, 0x0020, 0x0002, 0x0000, 0x0000, /*1A53*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31EB, 0x0020, 0x0004, 0x3211, 0x0020, /*1A54*/
  0x0004, 0x31EB, 0x0020, 0x0004, 0x0003,
  0x31F4, 0x0020, 0x0002, 0x0000, 0x0000, /*1A55*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31F5, 0x0020, 0x0002, 0x0000, 0x0000, /*1A56*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31F6, 0x0020, 0x0002, 0x0000, 0x0000, /*1A57*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31CB, 0x0020, 0x0004, 0x0000, 0x0000, /*1A58*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31CB, 0x0020, 0x0004, 0x0000, 0x0000, /*1A59*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31E0, 0x0020, 0x0004, 0x0000, 0x0000, /*1A5A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31E0, 0x0020, 0x0004, 0x0000, 0x0000, /*1A5B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31F7, 0x0020, 0x0002, 0x0000, 0x0000, /*1A5C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31F8, 0x0020, 0x0002, 0x0000, 0x0000, /*1A5D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31F9, 0x0020, 0x0002, 0x0000, 0x0000, /*1A5E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x9A5F, 0x0000, /*1A5F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3211, 0x0020, 0x0002, 0x0000, 0x0000, /*1A60*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3200, 0x0020, 0x0002, 0x0000, 0x0000, /*1A61*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3202, 0x0020, 0x0002, 0x0000, 0x0000, /*1A62*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3203, 0x0020, 0x0002, 0x0000, 0x0000, /*1A63*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3203, 0x0020, 0x0004, 0x0000, 0x0000, /*1A64*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3204, 0x0020, 0x0002, 0x0000, 0x0000, /*1A65*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3205, 0x0020, 0x0002, 0x0000, 0x0000, /*1A66*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3206, 0x0020, 0x0002, 0x0000, 0x0000, /*1A67*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3207, 0x0020, 0x0002, 0x0000, 0x0000, /*1A68*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3208, 0x0020, 0x0002, 0x0000, 0x0000, /*1A69*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3209, 0x0020, 0x0002, 0x0000, 0x0000, /*1A6A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x31F3, 0x0020, 0x0002, 0x0000, 0x0000, /*1A6B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3201, 0x0020, 0x0002, 0x0000, 0x0000, /*1A6C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3210, 0x0020, 0x0002, 0x0000, 0x0000, /*1A6D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x320A, 0x0020, 0x0002, 0x0000, 0x0000, /*1A6E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x320B, 0x0020, 0x0002, 0x0000, 0x0000, /*1A6F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x320D, 0x0020, 0x0002, 0x0000, 0x0000, /*1A70*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x320E, 0x0020, 0x0002, 0x0000, 0x0000, /*1A71*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x320F, 0x0020, 0x0002, 0x0000, 0x0000, /*1A72*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x320C, 0x0020, 0x0002, 0x0000, 0x0000, /*1A73*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, /*1A74*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00E9, 0x0002, 0x0000, 0x0000, /*1A75*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00EA, 0x0002, 0x0000, 0x0000, /*1A76*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00EB, 0x0002, 0x0000, 0x0000, /*1A77*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00EC, 0x0002, 0x0000, 0x0000, /*1A78*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00ED, 0x0002, 0x0000, 0x0000, /*1A79*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00EE, 0x0002, 0x0000, 0x0000, /*1A7A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00EF, 0x0002, 0x0000, 0x0000, /*1A7B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x00F0, 0x0002, 0x0000, 0x0000, /*1A7C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x9A7D, 0x0000, /*1A7D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9A7E, 0x0000, /*1A7E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1A7F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, /*1A80*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, /*1A81*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, /*1A82*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, /*1A83*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, /*1A84*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, /*1A85*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, /*1A86*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, /*1A87*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, /*1A88*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, /*1A89*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x9A8A, 0x0000, /*1A8A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9A8B, 0x0000, /*1A8B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9A8C, 0x0000, /*1A8C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9A8D, 0x0000, /*1A8D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9A8E, 0x0000, /*1A8E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9A8F, 0x0000, /*1A8F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, /*1A90*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, /*1A91*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, /*1A92*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, /*1A93*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, /*1A94*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, /*1A95*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, /*1A96*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, /*1A97*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, /*1A98*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, /*1A99*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x9A9A, 0x0000, /*1A9A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9A9B, 0x0000, /*1A9B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9A9C, 0x0000, /*1A9C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9A9D, 0x0000, /*1A9D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9A9E, 0x0000, /*1A9E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9A9F, 0x0000, /*1A9F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0415, 0x0020, 0x0002, 0x0000, 0x0000, /*1AA0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0416, 0x0020, 0x0002, 0x0000, 0x0000, /*1AA1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0417, 0x0020, 0x0002, 0x0000, 0x0000, /*1AA2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0418, 0x0020, 0x0002, 0x0000, 0x0000, /*1AA3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0419, 0x0020, 0x0002, 0x0000, 0x0000, /*1AA4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x041A, 0x0020, 0x0002, 0x0000, 0x0000, /*1AA5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x041B, 0x0020, 0x0002, 0x0000, 0x0000, /*1AA6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1BFE, 0x0020, 0x0002, 0x0000, 0x0000, /*1AA7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x029D, 0x0020, 0x0002, 0x0000, 0x0000, /*1AA8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x029E, 0x0020, 0x0002, 0x0000, 0x0000, /*1AA9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x029F, 0x0020, 0x0002, 0x0000, 0x0000, /*1AAA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02A0, 0x0020, 0x0002, 0x0000, 0x0000, /*1AAB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x041C, 0x0020, 0x0002, 0x0000, 0x0000, /*1AAC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x041D, 0x0020, 0x0002, 0x0000, 0x0000, /*1AAD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x9AAE, 0x0000, /*1AAE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AAF, 0x0000, /*1AAF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1AB0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1AB1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1AB2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1AB3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1AB4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1AB5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1AB6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1AB7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1AB8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1AB9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1ABA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1ABB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1ABC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1ABD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0036, 0x0002, 0x0000, 0x0000, /*1ABE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x9ABF, 0x0000, /*1ABF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AC0, 0x0000, /*1AC0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AC1, 0x0000, /*1AC1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AC2, 0x0000, /*1AC2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AC3, 0x0000, /*1AC3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AC4, 0x0000, /*1AC4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AC5, 0x0000, /*1AC5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AC6, 0x0000, /*1AC6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AC7, 0x0000, /*1AC7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AC8, 0x0000, /*1AC8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AC9, 0x0000, /*1AC9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9ACA, 0x0000, /*1ACA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9ACB, 0x0000, /*1ACB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9ACC, 0x0000, /*1ACC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9ACD, 0x0000, /*1ACD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9ACE, 0x0000, /*1ACE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9ACF, 0x0000, /*1ACF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AD0, 0x0000, /*1AD0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AD1, 0x0000, /*1AD1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AD2, 0x0000, /*1AD2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AD3, 0x0000, /*1AD3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AD4, 0x0000, /*1AD4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AD5, 0x0000, /*1AD5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AD6, 0x0000, /*1AD6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AD7, 0x0000, /*1AD7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AD8, 0x0000, /*1AD8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AD9, 0x0000, /*1AD9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9ADA, 0x0000, /*1ADA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9ADB, 0x0000, /*1ADB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9ADC, 0x0000, /*1ADC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9ADD, 0x0000, /*1ADD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9ADE, 0x0000, /*1ADE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9ADF, 0x0000, /*1ADF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AE0, 0x0000, /*1AE0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AE1, 0x0000, /*1AE1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AE2, 0x0000, /*1AE2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AE3, 0x0000, /*1AE3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AE4, 0x0000, /*1AE4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AE5, 0x0000, /*1AE5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AE6, 0x0000, /*1AE6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AE7, 0x0000, /*1AE7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AE8, 0x0000, /*1AE8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AE9, 0x0000, /*1AE9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AEA, 0x0000, /*1AEA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AEB, 0x0000, /*1AEB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AEC, 0x0000, /*1AEC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AED, 0x0000, /*1AED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AEE, 0x0000, /*1AEE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AEF, 0x0000, /*1AEF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AF0, 0x0000, /*1AF0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AF1, 0x0000, /*1AF1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AF2, 0x0000, /*1AF2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AF3, 0x0000, /*1AF3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AF4, 0x0000, /*1AF4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AF5, 0x0000, /*1AF5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AF6, 0x0000, /*1AF6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AF7, 0x0000, /*1AF7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AF8, 0x0000, /*1AF8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AF9, 0x0000, /*1AF9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AFA, 0x0000, /*1AFA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AFB, 0x0000, /*1AFB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AFC, 0x0000, /*1AFC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AFD, 0x0000, /*1AFD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AFE, 0x0000, /*1AFE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9AFF, 0x0000, /*1AFF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002
};

uint16 uca900_p01B[]= {
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B00*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B01*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B02*/
  0x0000, 0x00C8, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B03*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B04*/
  0x3257, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B05*/
  0x3258, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B06*/
  0x3259, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B07*/
  0x325A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B08*/
  0x325B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B09*/
  0x325C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B0A*/
  0x325D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B0B*/
  0x325E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B0C*/
  0x325F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B0D*/
  0x3260, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B0E*/
  0x3261, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B0F*/
  0x3262, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B10*/
  0x3263, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B11*/
  0x3264, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B12*/
  0x3265, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B13*/
  0x3268, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B14*/
  0x3269, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B15*/
  0x326A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B16*/
  0x326B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B17*/
  0x326C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B18*/
  0x326D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B19*/
  0x326E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B1A*/
  0x326F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B1B*/
  0x3270, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B1C*/
  0x3271, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B1D*/
  0x3272, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B1E*/
  0x3273, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B1F*/
  0x3274, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B20*/
  0x3275, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B21*/
  0x3276, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B22*/
  0x3278, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B23*/
  0x3279, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B24*/
  0x327A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B25*/
  0x327B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B26*/
  0x327C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B27*/
  0x327E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B28*/
  0x327F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B29*/
  0x3280, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B2A*/
  0x3281, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B2B*/
  0x3282, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B2C*/
  0x3283, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B2D*/
  0x3284, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B2E*/
  0x3285, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B2F*/
  0x3287, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B30*/
  0x3288, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B31*/
  0x3289, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B32*/
  0x328C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B33*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B34*/
  0x328D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B35*/
  0x328E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B36*/
  0x328F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B37*/
  0x3290, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B38*/
  0x3291, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B39*/
  0x3292, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B3A*/
  0x3293, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B3B*/
  0x3294, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B3C*/
  0x3295, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B3D*/
  0x3296, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B3E*/
  0x3297, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B3F*/
  0x3298, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B40*/
  0x3299, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B41*/
  0x329A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B42*/
  0x329B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B43*/
  0x329C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B44*/
  0x3266, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B45*/
  0x3267, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B46*/
  0x3277, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B47*/
  0x327D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B48*/
  0x3286, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B49*/
  0x328A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B4A*/
  0x328B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B4B*/
  0xFBC0, 0x0020, 0x0002, 0x9B4C, 0x0000, 0x0000, 0x0002, /*1B4C*/
  0xFBC0, 0x0020, 0x0002, 0x9B4D, 0x0000, 0x0000, 0x0002, /*1B4D*/
  0xFBC0, 0x0020, 0x0002, 0x9B4E, 0x0000, 0x0000, 0x0002, /*1B4E*/
  0xFBC0, 0x0020, 0x0002, 0x9B4F, 0x0000, 0x0000, 0x0002, /*1B4F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B50*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B51*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B52*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B53*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B54*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B55*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B56*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B57*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B58*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B59*/
  0x02CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B5A*/
  0x02CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B5B*/
  0x0280, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B5C*/
  0x025A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B5D*/
  0x02A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B5E*/
  0x02A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B5F*/
  0x0210, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B60*/
  0x0571, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B61*/
  0x0572, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B62*/
  0x0573, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B63*/
  0x0574, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B64*/
  0x0575, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B65*/
  0x0576, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B66*/
  0x0577, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B67*/
  0x0578, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B68*/
  0x0579, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B69*/
  0x057A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B6A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1B6B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1B6C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1B6D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1B6E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1B6F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1B70*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1B71*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1B72*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1B73*/
  0x057B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B74*/
  0x057C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B75*/
  0x057D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B76*/
  0x057E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B77*/
  0x057F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B78*/
  0x0580, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B79*/
  0x0581, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B7A*/
  0x0582, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B7B*/
  0x0583, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B7C*/
  0xFBC0, 0x0020, 0x0002, 0x9B7D, 0x0000, 0x0000, 0x0002, /*1B7D*/
  0xFBC0, 0x0020, 0x0002, 0x9B7E, 0x0000, 0x0000, 0x0002, /*1B7E*/
  0xFBC0, 0x0020, 0x0002, 0x9B7F, 0x0000, 0x0000, 0x0002, /*1B7F*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B80*/
  0x0000, 0x00CA, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B81*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B82*/
  0x2C98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B83*/
  0x2C99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B84*/
  0x2C9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B85*/
  0x2C9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B86*/
  0x2C9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B87*/
  0x2C9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B88*/
  0x2C9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B89*/
  0x2C9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B8A*/
  0x2CA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B8B*/
  0x2CA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B8C*/
  0x2CA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B8D*/
  0x2CA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B8E*/
  0x2CA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B8F*/
  0x2CA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B90*/
  0x2CA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B91*/
  0x2CA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B92*/
  0x2CA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B93*/
  0x2CAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B94*/
  0x2CAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B95*/
  0x2CAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B96*/
  0x2CAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B97*/
  0x2CAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B98*/
  0x2CB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B99*/
  0x2CB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B9A*/
  0x2CB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B9B*/
  0x2CB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B9C*/
  0x2CBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B9D*/
  0x2CBC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B9E*/
  0x2CBD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1B9F*/
  0x2CBF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BA0*/
  0x2CB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BA1*/
  0x2CB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BA2*/
  0x2CB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BA3*/
  0x2CC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BA4*/
  0x2CC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BA5*/
  0x2CC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BA6*/
  0x2CC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BA7*/
  0x2CC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BA8*/
  0x2CC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BA9*/
  0x2CC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BAA*/
  0x2CC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BAB*/
  0x2CB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BAC*/
  0x2CBB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BAD*/
  0x2CA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BAE*/
  0x2CBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BAF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BB0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BB1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BB2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BB3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BB4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BB5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BB6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BB7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BB8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BB9*/
  0x2C98, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BBA*/
  0x2CB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BBB*/
  0x2CB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BBC*/
  0x2CAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BBD*/
  0x2C9F, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*1BBE*/
  0x2CB0, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*1BBF*/
  0x2FD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC0*/
  0x2FD1, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC1*/
  0x2FD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC2*/
  0x2FD2, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC3*/
  0x2FD2, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC4*/
  0x2FD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC5*/
  0x2FD3, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC6*/
  0x2FD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC7*/
  0x2FD4, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC8*/
  0x2FD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC9*/
  0x2FD5, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BCA*/
  0x2FD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BCB*/
  0x2FD6, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BCC*/
  0x2FD6, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BCD*/
  0x2FD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BCE*/
  0x2FD7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BCF*/
  0x2FD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BD0*/
  0x2FD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BD1*/
  0x2FDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BD2*/
  0x2FDA, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BD3*/
  0x2FDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BD4*/
  0x2FDB, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BD5*/
  0x2FDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BD6*/
  0x2FDC, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BD7*/
  0x2FDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BD8*/
  0x2FDD, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BD9*/
  0x2FDD, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BDA*/
  0x2FDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BDB*/
  0x2FDE, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BDC*/
  0x2FDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BDD*/
  0x2FE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BDE*/
  0x2FE0, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BDF*/
  0x2FE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BE0*/
  0x2FE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BE1*/
  0x2FE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BE2*/
  0x2FE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BE3*/
  0x2FE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BE4*/
  0x2FE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BE5*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BE6*/
  0x2FE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BE7*/
  0x2FE7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BE8*/
  0x2FE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BE9*/
  0x2FE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BEA*/
  0x2FE9, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BEB*/
  0x2FEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BEC*/
  0x2FEA, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BED*/
  0x2FEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BEE*/
  0x2FEB, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1BEF*/
  0x2FEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BF0*/
  0x2FED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BF1*/
  0x2FEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BF2*/
  0x2FEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BF3*/
  0xFBC0, 0x0020, 0x0002, 0x9BF4, 0x0000, 0x0000, 0x0002, /*1BF4*/
  0xFBC0, 0x0020, 0x0002, 0x9BF5, 0x0000, 0x0000, 0x0002, /*1BF5*/
  0xFBC0, 0x0020, 0x0002, 0x9BF6, 0x0000, 0x0000, 0x0002, /*1BF6*/
  0xFBC0, 0x0020, 0x0002, 0x9BF7, 0x0000, 0x0000, 0x0002, /*1BF7*/
  0xFBC0, 0x0020, 0x0002, 0x9BF8, 0x0000, 0x0000, 0x0002, /*1BF8*/
  0xFBC0, 0x0020, 0x0002, 0x9BF9, 0x0000, 0x0000, 0x0002, /*1BF9*/
  0xFBC0, 0x0020, 0x0002, 0x9BFA, 0x0000, 0x0000, 0x0002, /*1BFA*/
  0xFBC0, 0x0020, 0x0002, 0x9BFB, 0x0000, 0x0000, 0x0002, /*1BFB*/
  0x042A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BFC*/
  0x042B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BFD*/
  0x042C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BFE*/
  0x042D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*1BFF*/
};

uint16 uca900_p01C[]= {
  0x2EC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C00*/
  0x2EC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C01*/
  0x2EC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C02*/
  0x2EC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C03*/
  0x2ECA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C04*/
  0x2ECB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C05*/
  0x2ECC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C06*/
  0x2ECD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C07*/
  0x2ECE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C08*/
  0x2ECF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C09*/
  0x2ED3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C0A*/
  0x2ED4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C0B*/
  0x2ED5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C0C*/
  0x2ED6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C0D*/
  0x2ED7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C0E*/
  0x2ED8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C0F*/
  0x2ED9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C10*/
  0x2EDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C11*/
  0x2EDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C12*/
  0x2EDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C13*/
  0x2EDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C14*/
  0x2EDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C15*/
  0x2EDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C16*/
  0x2EE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C17*/
  0x2EE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C18*/
  0x2EE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C19*/
  0x2EE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C1A*/
  0x2EE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C1B*/
  0x2EE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C1C*/
  0x2EE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C1D*/
  0x2EE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C1E*/
  0x2EEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C1F*/
  0x2EEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C20*/
  0x2EEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C21*/
  0x2EED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C22*/
  0x2EEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C23*/
  0x2EE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C24*/
  0x2EE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C25*/
  0x2EF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C26*/
  0x2EF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C27*/
  0x2EF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C28*/
  0x2EF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C29*/
  0x2EF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C2A*/
  0x2EF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C2B*/
  0x2EF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C2C*/
  0x2EF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C2D*/
  0x2EF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C2E*/
  0x2EF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C2F*/
  0x2EFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C30*/
  0x2EFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C31*/
  0x2EFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C32*/
  0x2EFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C33*/
  0x2EFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C34*/
  0x2EFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C35*/
  0x2EEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C36*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C37*/
  0xFBC0, 0x0020, 0x0002, 0x9C38, 0x0000, 0x0000, 0x0002, /*1C38*/
  0xFBC0, 0x0020, 0x0002, 0x9C39, 0x0000, 0x0000, 0x0002, /*1C39*/
  0xFBC0, 0x0020, 0x0002, 0x9C3A, 0x0000, 0x0000, 0x0002, /*1C3A*/
  0x0292, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C3B*/
  0x0293, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C3C*/
  0x040B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C3D*/
  0x040C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C3E*/
  0x040D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C3F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C40*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C41*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C42*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C43*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C44*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C45*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C46*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C47*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C48*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C49*/
  0xFBC0, 0x0020, 0x0002, 0x9C4A, 0x0000, 0x0000, 0x0002, /*1C4A*/
  0xFBC0, 0x0020, 0x0002, 0x9C4B, 0x0000, 0x0000, 0x0002, /*1C4B*/
  0xFBC0, 0x0020, 0x0002, 0x9C4C, 0x0000, 0x0000, 0x0002, /*1C4C*/
  0x2ED0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C4D*/
  0x2ED1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C4E*/
  0x2ED2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C4F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C50*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C51*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C52*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C53*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C54*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C55*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C56*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C57*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C58*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C59*/
  0x335B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C5A*/
  0x335C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C5B*/
  0x335D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C5C*/
  0x335E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C5D*/
  0x335F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C5E*/
  0x3360, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C5F*/
  0x3361, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C60*/
  0x3362, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C61*/
  0x3363, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C62*/
  0x3364, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C63*/
  0x3365, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C64*/
  0x3366, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C65*/
  0x3367, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C66*/
  0x3368, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C67*/
  0x3369, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C68*/
  0x336A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C69*/
  0x336B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C6A*/
  0x336C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C6B*/
  0x336D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C6C*/
  0x336E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C6D*/
  0x336F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C6E*/
  0x3370, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C6F*/
  0x3371, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C70*/
  0x3372, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C71*/
  0x3373, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C72*/
  0x3374, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C73*/
  0x3375, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C74*/
  0x3376, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C75*/
  0x3377, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C76*/
  0x3378, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C77*/
  0x3379, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C78*/
  0x337A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C79*/
  0x337B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C7A*/
  0x337C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C7B*/
  0x337D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C7C*/
  0x337E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C7D*/
  0x02C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C7E*/
  0x02C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1C7F*/
  0x2032, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1C80*/
  0x204A, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1C81*/
  0x20E7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1C82*/
  0x2105, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1C83*/
  0x210E, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1C84*/
  0x210E, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1C85*/
  0x218F, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1C86*/
  0x21A0, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1C87*/
  0x2129, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1C88*/
  0xFBC0, 0x0020, 0x0002, 0x9C89, 0x0000, 0x0000, 0x0002, /*1C89*/
  0xFBC0, 0x0020, 0x0002, 0x9C8A, 0x0000, 0x0000, 0x0002, /*1C8A*/
  0xFBC0, 0x0020, 0x0002, 0x9C8B, 0x0000, 0x0000, 0x0002, /*1C8B*/
  0xFBC0, 0x0020, 0x0002, 0x9C8C, 0x0000, 0x0000, 0x0002, /*1C8C*/
  0xFBC0, 0x0020, 0x0002, 0x9C8D, 0x0000, 0x0000, 0x0002, /*1C8D*/
  0xFBC0, 0x0020, 0x0002, 0x9C8E, 0x0000, 0x0000, 0x0002, /*1C8E*/
  0xFBC0, 0x0020, 0x0002, 0x9C8F, 0x0000, 0x0000, 0x0002, /*1C8F*/
  0xFBC0, 0x0020, 0x0002, 0x9C90, 0x0000, 0x0000, 0x0002, /*1C90*/
  0xFBC0, 0x0020, 0x0002, 0x9C91, 0x0000, 0x0000, 0x0002, /*1C91*/
  0xFBC0, 0x0020, 0x0002, 0x9C92, 0x0000, 0x0000, 0x0002, /*1C92*/
  0xFBC0, 0x0020, 0x0002, 0x9C93, 0x0000, 0x0000, 0x0002, /*1C93*/
  0xFBC0, 0x0020, 0x0002, 0x9C94, 0x0000, 0x0000, 0x0002, /*1C94*/
  0xFBC0, 0x0020, 0x0002, 0x9C95, 0x0000, 0x0000, 0x0002, /*1C95*/
  0xFBC0, 0x0020, 0x0002, 0x9C96, 0x0000, 0x0000, 0x0002, /*1C96*/
  0xFBC0, 0x0020, 0x0002, 0x9C97, 0x0000, 0x0000, 0x0002, /*1C97*/
  0xFBC0, 0x0020, 0x0002, 0x9C98, 0x0000, 0x0000, 0x0002, /*1C98*/
  0xFBC0, 0x0020, 0x0002, 0x9C99, 0x0000, 0x0000, 0x0002, /*1C99*/
  0xFBC0, 0x0020, 0x0002, 0x9C9A, 0x0000, 0x0000, 0x0002, /*1C9A*/
  0xFBC0, 0x0020, 0x0002, 0x9C9B, 0x0000, 0x0000, 0x0002, /*1C9B*/
  0xFBC0, 0x0020, 0x0002, 0x9C9C, 0x0000, 0x0000, 0x0002, /*1C9C*/
  0xFBC0, 0x0020, 0x0002, 0x9C9D, 0x0000, 0x0000, 0x0002, /*1C9D*/
  0xFBC0, 0x0020, 0x0002, 0x9C9E, 0x0000, 0x0000, 0x0002, /*1C9E*/
  0xFBC0, 0x0020, 0x0002, 0x9C9F, 0x0000, 0x0000, 0x0002, /*1C9F*/
  0xFBC0, 0x0020, 0x0002, 0x9CA0, 0x0000, 0x0000, 0x0002, /*1CA0*/
  0xFBC0, 0x0020, 0x0002, 0x9CA1, 0x0000, 0x0000, 0x0002, /*1CA1*/
  0xFBC0, 0x0020, 0x0002, 0x9CA2, 0x0000, 0x0000, 0x0002, /*1CA2*/
  0xFBC0, 0x0020, 0x0002, 0x9CA3, 0x0000, 0x0000, 0x0002, /*1CA3*/
  0xFBC0, 0x0020, 0x0002, 0x9CA4, 0x0000, 0x0000, 0x0002, /*1CA4*/
  0xFBC0, 0x0020, 0x0002, 0x9CA5, 0x0000, 0x0000, 0x0002, /*1CA5*/
  0xFBC0, 0x0020, 0x0002, 0x9CA6, 0x0000, 0x0000, 0x0002, /*1CA6*/
  0xFBC0, 0x0020, 0x0002, 0x9CA7, 0x0000, 0x0000, 0x0002, /*1CA7*/
  0xFBC0, 0x0020, 0x0002, 0x9CA8, 0x0000, 0x0000, 0x0002, /*1CA8*/
  0xFBC0, 0x0020, 0x0002, 0x9CA9, 0x0000, 0x0000, 0x0002, /*1CA9*/
  0xFBC0, 0x0020, 0x0002, 0x9CAA, 0x0000, 0x0000, 0x0002, /*1CAA*/
  0xFBC0, 0x0020, 0x0002, 0x9CAB, 0x0000, 0x0000, 0x0002, /*1CAB*/
  0xFBC0, 0x0020, 0x0002, 0x9CAC, 0x0000, 0x0000, 0x0002, /*1CAC*/
  0xFBC0, 0x0020, 0x0002, 0x9CAD, 0x0000, 0x0000, 0x0002, /*1CAD*/
  0xFBC0, 0x0020, 0x0002, 0x9CAE, 0x0000, 0x0000, 0x0002, /*1CAE*/
  0xFBC0, 0x0020, 0x0002, 0x9CAF, 0x0000, 0x0000, 0x0002, /*1CAF*/
  0xFBC0, 0x0020, 0x0002, 0x9CB0, 0x0000, 0x0000, 0x0002, /*1CB0*/
  0xFBC0, 0x0020, 0x0002, 0x9CB1, 0x0000, 0x0000, 0x0002, /*1CB1*/
  0xFBC0, 0x0020, 0x0002, 0x9CB2, 0x0000, 0x0000, 0x0002, /*1CB2*/
  0xFBC0, 0x0020, 0x0002, 0x9CB3, 0x0000, 0x0000, 0x0002, /*1CB3*/
  0xFBC0, 0x0020, 0x0002, 0x9CB4, 0x0000, 0x0000, 0x0002, /*1CB4*/
  0xFBC0, 0x0020, 0x0002, 0x9CB5, 0x0000, 0x0000, 0x0002, /*1CB5*/
  0xFBC0, 0x0020, 0x0002, 0x9CB6, 0x0000, 0x0000, 0x0002, /*1CB6*/
  0xFBC0, 0x0020, 0x0002, 0x9CB7, 0x0000, 0x0000, 0x0002, /*1CB7*/
  0xFBC0, 0x0020, 0x0002, 0x9CB8, 0x0000, 0x0000, 0x0002, /*1CB8*/
  0xFBC0, 0x0020, 0x0002, 0x9CB9, 0x0000, 0x0000, 0x0002, /*1CB9*/
  0xFBC0, 0x0020, 0x0002, 0x9CBA, 0x0000, 0x0000, 0x0002, /*1CBA*/
  0xFBC0, 0x0020, 0x0002, 0x9CBB, 0x0000, 0x0000, 0x0002, /*1CBB*/
  0xFBC0, 0x0020, 0x0002, 0x9CBC, 0x0000, 0x0000, 0x0002, /*1CBC*/
  0xFBC0, 0x0020, 0x0002, 0x9CBD, 0x0000, 0x0000, 0x0002, /*1CBD*/
  0xFBC0, 0x0020, 0x0002, 0x9CBE, 0x0000, 0x0000, 0x0002, /*1CBE*/
  0xFBC0, 0x0020, 0x0002, 0x9CBF, 0x0000, 0x0000, 0x0002, /*1CBF*/
  0x041F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CC0*/
  0x0420, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CC1*/
  0x0421, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CC2*/
  0x0422, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CC3*/
  0x0423, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CC4*/
  0x0424, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CC5*/
  0x0425, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CC6*/
  0x0426, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CC7*/
  0xFBC0, 0x0020, 0x0002, 0x9CC8, 0x0000, 0x0000, 0x0002, /*1CC8*/
  0xFBC0, 0x0020, 0x0002, 0x9CC9, 0x0000, 0x0000, 0x0002, /*1CC9*/
  0xFBC0, 0x0020, 0x0002, 0x9CCA, 0x0000, 0x0000, 0x0002, /*1CCA*/
  0xFBC0, 0x0020, 0x0002, 0x9CCB, 0x0000, 0x0000, 0x0002, /*1CCB*/
  0xFBC0, 0x0020, 0x0002, 0x9CCC, 0x0000, 0x0000, 0x0002, /*1CCC*/
  0xFBC0, 0x0020, 0x0002, 0x9CCD, 0x0000, 0x0000, 0x0002, /*1CCD*/
  0xFBC0, 0x0020, 0x0002, 0x9CCE, 0x0000, 0x0000, 0x0002, /*1CCE*/
  0xFBC0, 0x0020, 0x0002, 0x9CCF, 0x0000, 0x0000, 0x0002, /*1CCF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CD0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CD1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CD2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CD3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CD4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CD5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CD6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CD7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CD8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CD9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CDA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CDB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CDC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CDD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CDE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CDF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CE0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CE1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CE2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CE3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CE4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CE5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CE6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CE7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CE8*/
  0x26A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CE9*/
  0x26A1, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1CEA*/
  0x26A1, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1CEB*/
  0x26A1, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1CEC*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CED*/
  0x26A1, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1CEE*/
  0x26A1, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1CEF*/
  0x26A1, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1CF0*/
  0x26A1, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1CF1*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CF2*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CF3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CF4*/
  0x26A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CF5*/
  0x26A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1CF6*/
  0xFBC0, 0x0020, 0x0002, 0x9CF7, 0x0000, 0x0000, 0x0002, /*1CF7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CF8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1CF9*/
  0xFBC0, 0x0020, 0x0002, 0x9CFA, 0x0000, 0x0000, 0x0002, /*1CFA*/
  0xFBC0, 0x0020, 0x0002, 0x9CFB, 0x0000, 0x0000, 0x0002, /*1CFB*/
  0xFBC0, 0x0020, 0x0002, 0x9CFC, 0x0000, 0x0000, 0x0002, /*1CFC*/
  0xFBC0, 0x0020, 0x0002, 0x9CFD, 0x0000, 0x0000, 0x0002, /*1CFD*/
  0xFBC0, 0x0020, 0x0002, 0x9CFE, 0x0000, 0x0000, 0x0002, /*1CFE*/
  0xFBC0, 0x0020, 0x0002, 0x9CFF, 0x0000, 0x0000, 0x0002  /*1CFF*/
};

uint16 uca900_p01D[]= {
  0x1C4B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D00*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C4E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D01*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C4F, 0x0020, 0x0002, 0x0000, 0x0000, /*1D02*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C6D, 0x0020, 0x0002, 0x0000, 0x0000, /*1D03*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D04*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C93, 0x0020, 0x0002, 0x0000, 0x0000, /*1D05*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C94, 0x0020, 0x0002, 0x0000, 0x0000, /*1D06*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAE, 0x0020, 0x0002, 0x0000, 0x0000, /*1D07*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CD4, 0x0020, 0x0002, 0x0000, 0x0000, /*1D08*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D40, 0x0020, 0x0002, 0x0000, 0x0000, /*1D09*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D54, 0x0020, 0x0002, 0x0000, 0x0000, /*1D0A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D69, 0x0020, 0x0002, 0x0000, 0x0000, /*1D0B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D80, 0x0020, 0x0002, 0x0000, 0x0000, /*1D0C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAE, 0x0020, 0x0002, 0x0000, 0x0000, /*1D0D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DC2, 0x0020, 0x0002, 0x0000, 0x0000, /*1D0E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DE1, 0x0020, 0x0002, 0x0000, 0x0000, /*1D0F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DF4, 0x0020, 0x0002, 0x0000, 0x0000, /*1D10*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DE2, 0x0020, 0x0002, 0x0000, 0x0000, /*1D11*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DF5, 0x0020, 0x0002, 0x0000, 0x0000, /*1D12*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DEE, 0x0020, 0x0002, 0x0000, 0x0000, /*1D13*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DE8, 0x0020, 0x0002, 0x0000, 0x0000, /*1D14*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D15*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DFA, 0x0020, 0x0002, 0x0000, 0x0000, /*1D16*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DFB, 0x0020, 0x0002, 0x0000, 0x0000, /*1D17*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E10, 0x0020, 0x0002, 0x0000, 0x0000, /*1D18*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E3E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D19*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E48, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E99, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB9, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EBB, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EBC, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1ED9, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE7, 0x0020, 0x0002, 0x0000, 0x0000, /*1D20*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF9, 0x0020, 0x0002, 0x0000, 0x0000, /*1D21*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F25, 0x0020, 0x0002, 0x0000, 0x0000, /*1D22*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F42, 0x0020, 0x0002, 0x0000, 0x0000, /*1D23*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F8A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D24*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F8B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D25*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBC, 0x0020, 0x0002, 0x0000, 0x0000, /*1D26*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCA, 0x0020, 0x0002, 0x0000, 0x0000, /*1D27*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD0, 0x0020, 0x0002, 0x0000, 0x0000, /*1D28*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD5, 0x0020, 0x0002, 0x0000, 0x0000, /*1D29*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE0, 0x0020, 0x0002, 0x0000, 0x0000, /*1D2A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x20B4, 0x0020, 0x0002, 0x0000, 0x0000, /*1D2B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x001D, 0x0000, 0x0000, /*1D2C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0014, 0x0000, 0x0110, /*1D2D*/
  0x0014, 0x1CAA, 0x0020, 0x0014, 0x0003,
  0x1C60, 0x0020, 0x001D, 0x0000, 0x0000, /*1D2E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C6C, 0x0020, 0x0002, 0x0000, 0x0000, /*1D2F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x001D, 0x0000, 0x0000, /*1D30*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x001D, 0x0000, 0x0000, /*1D31*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CB8, 0x0020, 0x001D, 0x0000, 0x0000, /*1D32*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF4, 0x0020, 0x001D, 0x0000, 0x0000, /*1D33*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x001D, 0x0000, 0x0000, /*1D34*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x001D, 0x0000, 0x0000, /*1D35*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D4C, 0x0020, 0x001D, 0x0000, 0x0000, /*1D36*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D65, 0x0020, 0x001D, 0x0000, 0x0000, /*1D37*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x001D, 0x0000, 0x0000, /*1D38*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x001D, 0x0000, 0x0000, /*1D39*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x001D, 0x0000, 0x0000, /*1D3A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DC1, 0x0020, 0x0002, 0x0000, 0x0000, /*1D3B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x001D, 0x0000, 0x0000, /*1D3C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E07, 0x0020, 0x001D, 0x0000, 0x0000, /*1D3D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0C, 0x0020, 0x001D, 0x0000, 0x0000, /*1D3E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x001D, 0x0000, 0x0000, /*1D3F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x001D, 0x0000, 0x0000, /*1D40*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x001D, 0x0000, 0x0000, /*1D41*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF5, 0x0020, 0x001D, 0x0000, 0x0000, /*1D42*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0014, 0x0000, 0x0000, /*1D43*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C51, 0x0020, 0x0014, 0x0000, 0x0000, /*1D44*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C55, 0x0020, 0x0014, 0x0000, 0x0000, /*1D45*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C4F, 0x0020, 0x0014, 0x0000, 0x0000, /*1D46*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C60, 0x0020, 0x0014, 0x0000, 0x0000, /*1D47*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x0014, 0x0000, 0x0000, /*1D48*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x0014, 0x0000, 0x0000, /*1D49*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CBD, 0x0020, 0x0014, 0x0000, 0x0000, /*1D4A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CC2, 0x0020, 0x0014, 0x0000, 0x0000, /*1D4B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CD4, 0x0020, 0x0014, 0x0000, 0x0000, /*1D4C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF4, 0x0020, 0x0014, 0x0000, 0x0000, /*1D4D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D40, 0x0020, 0x0014, 0x0000, 0x0000, /*1D4E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D65, 0x0020, 0x0014, 0x0000, 0x0000, /*1D4F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x0014, 0x0000, 0x0000, /*1D50*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DD8, 0x0020, 0x0014, 0x0000, 0x0000, /*1D51*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0014, 0x0000, 0x0000, /*1D52*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DF0, 0x0020, 0x0014, 0x0000, 0x0000, /*1D53*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DFA, 0x0020, 0x0014, 0x0000, 0x0000, /*1D54*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DFB, 0x0020, 0x0014, 0x0000, 0x0000, /*1D55*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0C, 0x0020, 0x0014, 0x0000, 0x0000, /*1D56*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x0014, 0x0000, 0x0000, /*1D57*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x0014, 0x0000, 0x0000, /*1D58*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EBB, 0x0020, 0x0014, 0x0000, 0x0000, /*1D59*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1ED4, 0x0020, 0x0014, 0x0000, 0x0000, /*1D5A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x0014, 0x0000, 0x0000, /*1D5B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F8B, 0x0020, 0x0014, 0x0000, 0x0000, /*1D5C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBA, 0x0020, 0x0014, 0x0000, 0x0000, /*1D5D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBB, 0x0020, 0x0014, 0x0000, 0x0000, /*1D5E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBD, 0x0020, 0x0014, 0x0000, 0x0000, /*1D5F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDD, 0x0020, 0x0014, 0x0000, 0x0000, /*1D60*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDE, 0x0020, 0x0014, 0x0000, 0x0000, /*1D61*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x0015, 0x0000, 0x0000, /*1D62*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x0015, 0x0000, 0x0000, /*1D63*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x0015, 0x0000, 0x0000, /*1D64*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x0015, 0x0000, 0x0000, /*1D65*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBA, 0x0020, 0x0015, 0x0000, 0x0000, /*1D66*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBB, 0x0020, 0x0015, 0x0000, 0x0000, /*1D67*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FD4, 0x0020, 0x0015, 0x0000, 0x0000, /*1D68*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDD, 0x0020, 0x0015, 0x0000, 0x0000, /*1D69*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FDE, 0x0020, 0x0015, 0x0000, 0x0000, /*1D6A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EBD, 0x0020, 0x0002, 0x0000, 0x0000, /*1D6B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C6E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D6C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C95, 0x0020, 0x0002, 0x0000, 0x0000, /*1D6D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CEC, 0x0020, 0x0002, 0x0000, 0x0000, /*1D6E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAF, 0x0020, 0x0002, 0x0000, 0x0000, /*1D6F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DC3, 0x0020, 0x0002, 0x0000, 0x0000, /*1D70*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E13, 0x0020, 0x0002, 0x0000, 0x0000, /*1D71*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E43, 0x0020, 0x0002, 0x0000, 0x0000, /*1D72*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E60, 0x0020, 0x0002, 0x0000, 0x0000, /*1D73*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E76, 0x0020, 0x0002, 0x0000, 0x0000, /*1D74*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E9F, 0x0020, 0x0002, 0x0000, 0x0000, /*1D75*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F2A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D76*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D0E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D77*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x20CC, 0x0020, 0x0014, 0x0000, 0x0000, /*1D78*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF4, 0x0020, 0x0004, 0x0000, 0x0111, /*1D79*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0004, 0x0000, 0x0110, /*1D7A*/
  0x0004, 0x1D18, 0x0020, 0x0004, 0x0003,
  0x1D45, 0x0020, 0x0002, 0x0000, 0x0000, /*1D7B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D4B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D7C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E11, 0x0020, 0x0002, 0x0000, 0x0000, /*1D7D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EC5, 0x0020, 0x0002, 0x0000, 0x0000, /*1D7E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE2, 0x0020, 0x0002, 0x0000, 0x0000, /*1D7F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C70, 0x0020, 0x0002, 0x0000, 0x0000, /*1D80*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C96, 0x0020, 0x0002, 0x0000, 0x0000, /*1D81*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CED, 0x0020, 0x0002, 0x0000, 0x0000, /*1D82*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D05, 0x0020, 0x0002, 0x0000, 0x0000, /*1D83*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D6A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D84*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D92, 0x0020, 0x0002, 0x0000, 0x0000, /*1D85*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB0, 0x0020, 0x0002, 0x0000, 0x0000, /*1D86*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DCD, 0x0020, 0x0002, 0x0000, 0x0000, /*1D87*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E14, 0x0020, 0x0002, 0x0000, 0x0000, /*1D88*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E4D, 0x0020, 0x0002, 0x0000, 0x0000, /*1D89*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E77, 0x0020, 0x0002, 0x0000, 0x0000, /*1D8A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E87, 0x0020, 0x0002, 0x0000, 0x0000, /*1D8B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE9, 0x0020, 0x0002, 0x0000, 0x0000, /*1D8C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F03, 0x0020, 0x0002, 0x0000, 0x0000, /*1D8D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F2B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D8E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C4D, 0x0020, 0x0002, 0x0000, 0x0000, /*1D8F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C5A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D90*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C9F, 0x0020, 0x0002, 0x0000, 0x0000, /*1D91*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CB5, 0x0020, 0x0002, 0x0000, 0x0000, /*1D92*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CC6, 0x0020, 0x0002, 0x0000, 0x0000, /*1D93*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CD3, 0x0020, 0x0002, 0x0000, 0x0000, /*1D94*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CC1, 0x0020, 0x0002, 0x0000, 0x0000, /*1D95*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D46, 0x0020, 0x0002, 0x0000, 0x0000, /*1D96*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DF7, 0x0020, 0x0002, 0x0000, 0x0000, /*1D97*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E90, 0x0020, 0x0002, 0x0000, 0x0000, /*1D98*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EC6, 0x0020, 0x0002, 0x0000, 0x0000, /*1D99*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F47, 0x0020, 0x0002, 0x0000, 0x0000, /*1D9A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C5B, 0x0020, 0x0014, 0x0000, 0x0000, /*1D9B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x0014, 0x0000, 0x0000, /*1D9C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C89, 0x0020, 0x0014, 0x0000, 0x0000, /*1D9D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x0014, 0x0000, 0x0110, /*1D9E*/
  0x0014, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CCF, 0x0020, 0x0014, 0x0000, 0x0000, /*1D9F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE5, 0x0020, 0x0014, 0x0000, 0x0000, /*1DA0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D5D, 0x0020, 0x0014, 0x0000, 0x0000, /*1DA1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF8, 0x0020, 0x0014, 0x0000, 0x0000, /*1DA2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EC8, 0x0020, 0x0014, 0x0000, 0x0000, /*1DA3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D41, 0x0020, 0x0014, 0x0000, 0x0000, /*1DA4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D47, 0x0020, 0x0014, 0x0000, 0x0000, /*1DA5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D3A, 0x0020, 0x0014, 0x0000, 0x0000, /*1DA6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D45, 0x0020, 0x0014, 0x0000, 0x0000, /*1DA7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D59, 0x0020, 0x0014, 0x0000, 0x0000, /*1DA8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D93, 0x0020, 0x0014, 0x0000, 0x0000, /*1DA9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D92, 0x0020, 0x0014, 0x0000, 0x0000, /*1DAA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D7B, 0x0020, 0x0014, 0x0000, 0x0000, /*1DAB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB1, 0x0020, 0x0014, 0x0000, 0x0000, /*1DAC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EDA, 0x0020, 0x0014, 0x0000, 0x0000, /*1DAD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DC4, 0x0020, 0x0014, 0x0000, 0x0000, /*1DAE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DCE, 0x0020, 0x0014, 0x0000, 0x0000, /*1DAF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DBD, 0x0020, 0x0014, 0x0000, 0x0000, /*1DB0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DFD, 0x0020, 0x0014, 0x0000, 0x0000, /*1DB1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E1C, 0x0020, 0x0014, 0x0000, 0x0000, /*1DB2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E78, 0x0020, 0x0014, 0x0000, 0x0000, /*1DB3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E82, 0x0020, 0x0014, 0x0000, 0x0000, /*1DB4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EA0, 0x0020, 0x0014, 0x0000, 0x0000, /*1DB5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EC0, 0x0020, 0x0014, 0x0000, 0x0000, /*1DB6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EDE, 0x0020, 0x0014, 0x0000, 0x0000, /*1DB7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB9, 0x0020, 0x0014, 0x0000, 0x0000, /*1DB8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EEA, 0x0020, 0x0014, 0x0000, 0x0000, /*1DB9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF1, 0x0020, 0x0014, 0x0000, 0x0000, /*1DBA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F21, 0x0020, 0x0014, 0x0000, 0x0000, /*1DBB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F30, 0x0020, 0x0014, 0x0000, 0x0000, /*1DBC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F34, 0x0020, 0x0014, 0x0000, 0x0000, /*1DBD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F3E, 0x0020, 0x0014, 0x0000, 0x0000, /*1DBE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC5, 0x0020, 0x0014, 0x0000, 0x0000, /*1DBF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DC0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DC1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1DC2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DC3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DC4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DC5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DC6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DC7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DC8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DC9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x0004, 0x0000, 0x0000, /*1DCA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DCB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DCC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DCD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DCE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1DCF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1DD0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DD1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F60, 0x0020, 0x0004, 0x0000, 0x0000, /*1DD2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0004, 0x0000, 0x0110, /*1DD3*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0004, 0x0000, 0x0110, /*1DD4*/
  0x0004, 0x1CAA, 0x0020, 0x0004, 0x0003,
  0x1C47, 0x0020, 0x0004, 0x1DDD, 0x0020, /*1DD5*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0004, 0x1EE3, 0x0020, /*1DD6*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x0004, 0x0000, 0x0030, /*1DD7*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0004, 0x0000, 0x0111, /*1DD8*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0004, 0x0000, 0x0110, /*1DD9*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0004, 0x0000, 0x0000, /*1DDA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CFD, 0x0020, 0x0004, 0x0000, 0x0000, /*1DDB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D65, 0x0020, 0x0004, 0x0000, 0x0000, /*1DDC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x0004, 0x0000, 0x0000, /*1DDD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D7B, 0x0020, 0x0004, 0x0000, 0x0000, /*1DDE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAE, 0x0020, 0x0004, 0x0000, 0x0000, /*1DDF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x0004, 0x0000, 0x0000, /*1DE0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DBD, 0x0020, 0x0004, 0x0000, 0x0000, /*1DE1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E38, 0x0020, 0x0004, 0x0000, 0x0000, /*1DE2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E3D, 0x0020, 0x0004, 0x0000, 0x0000, /*1DE3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x0004, 0x0000, 0x0000, /*1DE4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x0004, 0x0000, 0x0111, /*1DE5*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0004, 0x0000, 0x0000, /*1DE6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C55, 0x0020, 0x0004, 0x0000, 0x0000, /*1DE7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C60, 0x0020, 0x0004, 0x0000, 0x0000, /*1DE8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C79, 0x0020, 0x0004, 0x0000, 0x0000, /*1DE9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CBD, 0x0020, 0x0004, 0x0000, 0x0000, /*1DEA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE5, 0x0020, 0x0004, 0x0000, 0x0000, /*1DEB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D8B, 0x0020, 0x0004, 0x0000, 0x0000, /*1DEC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0004, 0x0000, 0x0034, /*1DED*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E0C, 0x0020, 0x0004, 0x0000, 0x0000, /*1DEE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E82, 0x0020, 0x0004, 0x0000, 0x0000, /*1DEF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x0004, 0x0000, 0x0034, /*1DF0*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0004, 0x0000, 0x0000, /*1DF1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0004, 0x0000, 0x002B, /*1DF2*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0004, 0x0000, 0x002B, /*1DF3*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0004, 0x0000, 0x002B, /*1DF4*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DF5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x9DF6, 0x0000, /*1DF6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9DF7, 0x0000, /*1DF7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9DF8, 0x0000, /*1DF8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9DF9, 0x0000, /*1DF9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9DFA, 0x0000, /*1DFA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DFB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1DFC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1DFD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*1DFE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*1DFF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p01E[]= {
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0044, /*1E00*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0044, /*1E01*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C60, 0x0020, 0x0008, 0x0000, 0x002E, /*1E02*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C60, 0x0020, 0x0002, 0x0000, 0x002E, /*1E03*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C60, 0x0020, 0x0008, 0x0000, 0x0042, /*1E04*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C60, 0x0020, 0x0002, 0x0000, 0x0042, /*1E05*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C60, 0x0020, 0x0008, 0x0000, 0x0049, /*1E06*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C60, 0x0020, 0x0002, 0x0000, 0x0049, /*1E07*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x0008, 0x0000, 0x0030, /*1E08*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1C7A, 0x0020, 0x0002, 0x0000, 0x0030, /*1E09*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1C8F, 0x0020, 0x0008, 0x0000, 0x002E, /*1E0A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0002, 0x0000, 0x002E, /*1E0B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0008, 0x0000, 0x0042, /*1E0C*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0002, 0x0000, 0x0042, /*1E0D*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0008, 0x0000, 0x0049, /*1E0E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0002, 0x0000, 0x0049, /*1E0F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0008, 0x0000, 0x0030, /*1E10*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0002, 0x0000, 0x0030, /*1E11*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0008, 0x0000, 0x0046, /*1E12*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0002, 0x0000, 0x0046, /*1E13*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0032, /*1E14*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0032, /*1E15*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0032, /*1E16*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0032, /*1E17*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0046, /*1E18*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0046, /*1E19*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0048, /*1E1A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0048, /*1E1B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0030, /*1E1C*/
  0x0002, 0x0000, 0x0026, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0030, /*1E1D*/
  0x0002, 0x0000, 0x0026, 0x0002, 0x0003,
  0x1CE5, 0x0020, 0x0008, 0x0000, 0x002E, /*1E1E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CE5, 0x0020, 0x0002, 0x0000, 0x002E, /*1E1F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0008, 0x0000, 0x0032, /*1E20*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0002, 0x0000, 0x0032, /*1E21*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0008, 0x0000, 0x002E, /*1E22*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0002, 0x0000, 0x002E, /*1E23*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0008, 0x0000, 0x0042, /*1E24*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0002, 0x0000, 0x0042, /*1E25*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0008, 0x0000, 0x002B, /*1E26*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0002, 0x0000, 0x002B, /*1E27*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0008, 0x0000, 0x0030, /*1E28*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0002, 0x0000, 0x0030, /*1E29*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0008, 0x0000, 0x0047, /*1E2A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0002, 0x0000, 0x0047, /*1E2B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x0048, /*1E2C*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x0048, /*1E2D*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x002B, /*1E2E*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x002B, /*1E2F*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1D65, 0x0020, 0x0008, 0x0000, 0x0024, /*1E30*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x0002, 0x0000, 0x0024, /*1E31*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x0008, 0x0000, 0x0042, /*1E32*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x0002, 0x0000, 0x0042, /*1E33*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x0008, 0x0000, 0x0049, /*1E34*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x0002, 0x0000, 0x0049, /*1E35*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0008, 0x0000, 0x0042, /*1E36*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0002, 0x0000, 0x0042, /*1E37*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0008, 0x0000, 0x0042, /*1E38*/
  0x0002, 0x0000, 0x0032, 0x0002, 0x0003,
  0x1D77, 0x0020, 0x0002, 0x0000, 0x0042, /*1E39*/
  0x0002, 0x0000, 0x0032, 0x0002, 0x0003,
  0x1D77, 0x0020, 0x0008, 0x0000, 0x0049, /*1E3A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0002, 0x0000, 0x0049, /*1E3B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0008, 0x0000, 0x0046, /*1E3C*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0002, 0x0000, 0x0046, /*1E3D*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x0008, 0x0000, 0x0024, /*1E3E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x0002, 0x0000, 0x0024, /*1E3F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x0008, 0x0000, 0x002E, /*1E40*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x0002, 0x0000, 0x002E, /*1E41*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x0008, 0x0000, 0x0042, /*1E42*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x0002, 0x0000, 0x0042, /*1E43*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0008, 0x0000, 0x002E, /*1E44*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0002, 0x0000, 0x002E, /*1E45*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0008, 0x0000, 0x0042, /*1E46*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0002, 0x0000, 0x0042, /*1E47*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0008, 0x0000, 0x0049, /*1E48*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0002, 0x0000, 0x0049, /*1E49*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0008, 0x0000, 0x0046, /*1E4A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0002, 0x0000, 0x0046, /*1E4B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x002D, /*1E4C*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x002D, /*1E4D*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x002D, /*1E4E*/
  0x0002, 0x0000, 0x002B, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x002D, /*1E4F*/
  0x0002, 0x0000, 0x002B, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0032, /*1E50*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0032, /*1E51*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0032, /*1E52*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0032, /*1E53*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1E0C, 0x0020, 0x0008, 0x0000, 0x0024, /*1E54*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E0C, 0x0020, 0x0002, 0x0000, 0x0024, /*1E55*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E0C, 0x0020, 0x0008, 0x0000, 0x002E, /*1E56*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E0C, 0x0020, 0x0002, 0x0000, 0x002E, /*1E57*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0008, 0x0000, 0x002E, /*1E58*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0002, 0x0000, 0x002E, /*1E59*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0008, 0x0000, 0x0042, /*1E5A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0002, 0x0000, 0x0042, /*1E5B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0008, 0x0000, 0x0042, /*1E5C*/
  0x0002, 0x0000, 0x0032, 0x0002, 0x0003,
  0x1E33, 0x0020, 0x0002, 0x0000, 0x0042, /*1E5D*/
  0x0002, 0x0000, 0x0032, 0x0002, 0x0003,
  0x1E33, 0x0020, 0x0008, 0x0000, 0x0049, /*1E5E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0002, 0x0000, 0x0049, /*1E5F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0008, 0x0000, 0x002E, /*1E60*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0002, 0x0000, 0x002E, /*1E61*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0008, 0x0000, 0x0042, /*1E62*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0002, 0x0000, 0x0042, /*1E63*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0008, 0x0000, 0x0024, /*1E64*/
  0x0002, 0x0000, 0x002E, 0x0002, 0x0003,
  0x1E71, 0x0020, 0x0002, 0x0000, 0x0024, /*1E65*/
  0x0002, 0x0000, 0x002E, 0x0002, 0x0003,
  0x1E71, 0x0020, 0x0008, 0x0000, 0x0028, /*1E66*/
  0x0002, 0x0000, 0x002E, 0x0002, 0x0003,
  0x1E71, 0x0020, 0x0002, 0x0000, 0x0028, /*1E67*/
  0x0002, 0x0000, 0x002E, 0x0002, 0x0003,
  0x1E71, 0x0020, 0x0008, 0x0000, 0x0042, /*1E68*/
  0x0002, 0x0000, 0x002E, 0x0002, 0x0003,
  0x1E71, 0x0020, 0x0002, 0x0000, 0x0042, /*1E69*/
  0x0002, 0x0000, 0x002E, 0x0002, 0x0003,
  0x1E95, 0x0020, 0x0008, 0x0000, 0x002E, /*1E6A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0002, 0x0000, 0x002E, /*1E6B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0008, 0x0000, 0x0042, /*1E6C*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0002, 0x0000, 0x0042, /*1E6D*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0008, 0x0000, 0x0049, /*1E6E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0002, 0x0000, 0x0049, /*1E6F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0008, 0x0000, 0x0046, /*1E70*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0002, 0x0000, 0x0046, /*1E71*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0043, /*1E72*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0043, /*1E73*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0048, /*1E74*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0048, /*1E75*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0046, /*1E76*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0046, /*1E77*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x002D, /*1E78*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x002D, /*1E79*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0032, /*1E7A*/
  0x0002, 0x0000, 0x002B, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0032, /*1E7B*/
  0x0002, 0x0000, 0x002B, 0x0002, 0x0003,
  0x1EE3, 0x0020, 0x0008, 0x0000, 0x002D, /*1E7C*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EE3, 0x0020, 0x0002, 0x0000, 0x002D, /*1E7D*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EE3, 0x0020, 0x0008, 0x0000, 0x0042, /*1E7E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EE3, 0x0020, 0x0002, 0x0000, 0x0042, /*1E7F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0008, 0x0000, 0x0025, /*1E80*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0002, 0x0000, 0x0025, /*1E81*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0008, 0x0000, 0x0024, /*1E82*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0002, 0x0000, 0x0024, /*1E83*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0008, 0x0000, 0x002B, /*1E84*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0002, 0x0000, 0x002B, /*1E85*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0008, 0x0000, 0x002E, /*1E86*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0002, 0x0000, 0x002E, /*1E87*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0008, 0x0000, 0x0042, /*1E88*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0002, 0x0000, 0x0042, /*1E89*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EFF, 0x0020, 0x0008, 0x0000, 0x002E, /*1E8A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EFF, 0x0020, 0x0002, 0x0000, 0x002E, /*1E8B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EFF, 0x0020, 0x0008, 0x0000, 0x002B, /*1E8C*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EFF, 0x0020, 0x0002, 0x0000, 0x002B, /*1E8D*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0008, 0x0000, 0x002E, /*1E8E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0002, 0x0000, 0x002E, /*1E8F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0008, 0x0000, 0x0027, /*1E90*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0002, 0x0000, 0x0027, /*1E91*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0008, 0x0000, 0x0042, /*1E92*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0002, 0x0000, 0x0042, /*1E93*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0008, 0x0000, 0x0049, /*1E94*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F21, 0x0020, 0x0002, 0x0000, 0x0049, /*1E95*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x0002, 0x0000, 0x0049, /*1E96*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0002, 0x0000, 0x002B, /*1E97*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x0002, 0x0000, 0x0029, /*1E98*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0002, 0x0000, 0x0029, /*1E99*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0004, 0x1F80, 0x0020, /*1E9A*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0004, 0x0000, 0x0111, /*1E9B*/
  0x0004, 0x0000, 0x002E, 0x0002, 0x0003,
  0x1E80, 0x0020, 0x0002, 0x0000, 0x0000, /*1E9C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E81, 0x0020, 0x0002, 0x0000, 0x0000, /*1E9D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x000A, 0x0000, 0x0110, /*1E9E*/
  0x0004, 0x1E71, 0x0020, 0x000A, 0x0003,
  0x1CA9, 0x0020, 0x0002, 0x0000, 0x0000, /*1E9F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0042, /*1EA0*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0042, /*1EA1*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x003B, /*1EA2*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x003B, /*1EA3*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0027, /*1EA4*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0027, /*1EA5*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0027, /*1EA6*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0027, /*1EA7*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0027, /*1EA8*/
  0x0002, 0x0000, 0x003B, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0027, /*1EA9*/
  0x0002, 0x0000, 0x003B, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0027, /*1EAA*/
  0x0002, 0x0000, 0x002D, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0027, /*1EAB*/
  0x0002, 0x0000, 0x002D, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0042, /*1EAC*/
  0x0002, 0x0000, 0x0027, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0042, /*1EAD*/
  0x0002, 0x0000, 0x0027, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0026, /*1EAE*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0026, /*1EAF*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0026, /*1EB0*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0026, /*1EB1*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0026, /*1EB2*/
  0x0002, 0x0000, 0x003B, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0026, /*1EB3*/
  0x0002, 0x0000, 0x003B, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0026, /*1EB4*/
  0x0002, 0x0000, 0x002D, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0026, /*1EB5*/
  0x0002, 0x0000, 0x002D, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0042, /*1EB6*/
  0x0002, 0x0000, 0x0026, 0x0002, 0x0003,
  0x1C47, 0x0020, 0x0002, 0x0000, 0x0042, /*1EB7*/
  0x0002, 0x0000, 0x0026, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0042, /*1EB8*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0042, /*1EB9*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x003B, /*1EBA*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x003B, /*1EBB*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x002D, /*1EBC*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x002D, /*1EBD*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0027, /*1EBE*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0027, /*1EBF*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0027, /*1EC0*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0027, /*1EC1*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0027, /*1EC2*/
  0x0002, 0x0000, 0x003B, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0027, /*1EC3*/
  0x0002, 0x0000, 0x003B, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0027, /*1EC4*/
  0x0002, 0x0000, 0x002D, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0027, /*1EC5*/
  0x0002, 0x0000, 0x002D, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0008, 0x0000, 0x0042, /*1EC6*/
  0x0002, 0x0000, 0x0027, 0x0002, 0x0003,
  0x1CAA, 0x0020, 0x0002, 0x0000, 0x0042, /*1EC7*/
  0x0002, 0x0000, 0x0027, 0x0002, 0x0003,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x003B, /*1EC8*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x003B, /*1EC9*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0008, 0x0000, 0x0042, /*1ECA*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0002, 0x0000, 0x0042, /*1ECB*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0042, /*1ECC*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0042, /*1ECD*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x003B, /*1ECE*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x003B, /*1ECF*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0027, /*1ED0*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0027, /*1ED1*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0027, /*1ED2*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0027, /*1ED3*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0027, /*1ED4*/
  0x0002, 0x0000, 0x003B, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0027, /*1ED5*/
  0x0002, 0x0000, 0x003B, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0027, /*1ED6*/
  0x0002, 0x0000, 0x002D, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0027, /*1ED7*/
  0x0002, 0x0000, 0x002D, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x0042, /*1ED8*/
  0x0002, 0x0000, 0x0027, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x0042, /*1ED9*/
  0x0002, 0x0000, 0x0027, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x003F, /*1EDA*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x003F, /*1EDB*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x003F, /*1EDC*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x003F, /*1EDD*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x003F, /*1EDE*/
  0x0002, 0x0000, 0x003B, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x003F, /*1EDF*/
  0x0002, 0x0000, 0x003B, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x003F, /*1EE0*/
  0x0002, 0x0000, 0x002D, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x003F, /*1EE1*/
  0x0002, 0x0000, 0x002D, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0008, 0x0000, 0x003F, /*1EE2*/
  0x0002, 0x0000, 0x0042, 0x0002, 0x0003,
  0x1DDD, 0x0020, 0x0002, 0x0000, 0x003F, /*1EE3*/
  0x0002, 0x0000, 0x0042, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x0042, /*1EE4*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x0042, /*1EE5*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x003B, /*1EE6*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x003B, /*1EE7*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x003F, /*1EE8*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x003F, /*1EE9*/
  0x0002, 0x0000, 0x0024, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x003F, /*1EEA*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x003F, /*1EEB*/
  0x0002, 0x0000, 0x0025, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x003F, /*1EEC*/
  0x0002, 0x0000, 0x003B, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x003F, /*1EED*/
  0x0002, 0x0000, 0x003B, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x003F, /*1EEE*/
  0x0002, 0x0000, 0x002D, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x003F, /*1EEF*/
  0x0002, 0x0000, 0x002D, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0008, 0x0000, 0x003F, /*1EF0*/
  0x0002, 0x0000, 0x0042, 0x0002, 0x0003,
  0x1EB5, 0x0020, 0x0002, 0x0000, 0x003F, /*1EF1*/
  0x0002, 0x0000, 0x0042, 0x0002, 0x0003,
  0x1F0B, 0x0020, 0x0008, 0x0000, 0x0025, /*1EF2*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0002, 0x0000, 0x0025, /*1EF3*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0008, 0x0000, 0x0042, /*1EF4*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0002, 0x0000, 0x0042, /*1EF5*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0008, 0x0000, 0x003B, /*1EF6*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0002, 0x0000, 0x003B, /*1EF7*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0008, 0x0000, 0x002D, /*1EF8*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F0B, 0x0020, 0x0002, 0x0000, 0x002D, /*1EF9*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x000A, 0x1D77, 0x0020, /*1EFA*/
  0x000A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x0004, 0x1D77, 0x0020, /*1EFB*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF0, 0x0020, 0x0008, 0x0000, 0x0000, /*1EFC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF0, 0x0020, 0x0002, 0x0000, 0x0000, /*1EFD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F1B, 0x0020, 0x0008, 0x0000, 0x0000, /*1EFE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F1B, 0x0020, 0x0002, 0x0000, 0x0000, /*1EFF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p01F[]= {
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F00*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F01*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F02*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F03*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F04*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F05*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F06*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F07*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F08*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F09*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F0A*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F0B*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F0C*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F0D*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F0E*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F0F*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FBE, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F10*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FBE, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F11*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FBE, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F12*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FBE, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F13*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FBE, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F14*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FBE, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F15*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC0, 0x0020, 0x0002, 0x9F16, 0x0000, 0x0000, 0x0000, /*1F16*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9F17, 0x0000, 0x0000, 0x0000, /*1F17*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FBE, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F18*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FBE, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F19*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FBE, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F1A*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FBE, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F1B*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FBE, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F1C*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FBE, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F1D*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC0, 0x0020, 0x0002, 0x9F1E, 0x0000, 0x0000, 0x0000, /*1F1E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9F1F, 0x0000, 0x0000, 0x0000, /*1F1F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F20*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F21*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F22*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F23*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F24*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F25*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F26*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F27*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F28*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F29*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F2A*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F2B*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F2C*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F2D*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F2E*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F2F*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F30*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F31*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F32*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F33*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F34*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F35*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F36*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F37*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F38*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F39*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F3A*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F3B*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F3C*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F3D*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F3E*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F3F*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FCE, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F40*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCE, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F41*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCE, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F42*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FCE, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F43*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FCE, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F44*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FCE, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F45*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC0, 0x0020, 0x0002, 0x9F46, 0x0000, 0x0000, 0x0000, /*1F46*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9F47, 0x0000, 0x0000, 0x0000, /*1F47*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCE, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F48*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCE, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F49*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCE, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F4A*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FCE, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F4B*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FCE, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F4C*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FCE, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F4D*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC0, 0x0020, 0x0002, 0x9F4E, 0x0000, 0x0000, 0x0000, /*1F4E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9F4F, 0x0000, 0x0000, 0x0000, /*1F4F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F50*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F51*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F52*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F53*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F54*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F55*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F56*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F57*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC0, 0x0020, 0x0002, 0x9F58, 0x0000, 0x0000, 0x0000, /*1F58*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F59*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9F5A, 0x0000, 0x0000, 0x0000, /*1F5A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F5B*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC0, 0x0020, 0x0002, 0x9F5C, 0x0000, 0x0000, 0x0000, /*1F5C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F5D*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC0, 0x0020, 0x0002, 0x9F5E, 0x0000, 0x0000, 0x0000, /*1F5E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F5F*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F60*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F61*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F62*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F63*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F64*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F65*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F66*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F67*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F68*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F69*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F6A*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F6B*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F6C*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F6D*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F6E*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F6F*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1F70*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1F71*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FBE, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1F72*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FBE, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1F73*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1F74*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1F75*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1F76*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1F77*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCE, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1F78*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCE, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1F79*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1F7A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1F7B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1F7C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1F7D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9F7E, 0x0000, 0x0000, 0x0000, /*1F7E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9F7F, 0x0000, 0x0000, 0x0000, /*1F7F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F80*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F81*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F82*/
  0x0025, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F83*/
  0x0025, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F84*/
  0x0024, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F85*/
  0x0024, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F86*/
  0x002A, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F87*/
  0x002A, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F88*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F89*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F8A*/
  0x0025, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F8B*/
  0x0025, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F8C*/
  0x0024, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F8D*/
  0x0024, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F8E*/
  0x002A, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F8F*/
  0x002A, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F90*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F91*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F92*/
  0x0025, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F93*/
  0x0025, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F94*/
  0x0024, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F95*/
  0x0024, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1F96*/
  0x002A, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1F97*/
  0x002A, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F98*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F99*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F9A*/
  0x0025, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F9B*/
  0x0025, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F9C*/
  0x0024, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F9D*/
  0x0024, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1F9E*/
  0x002A, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1F9F*/
  0x002A, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1FA0*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1FA1*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1FA2*/
  0x0025, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1FA3*/
  0x0025, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1FA4*/
  0x0024, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1FA5*/
  0x0024, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1FA6*/
  0x002A, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1FA7*/
  0x002A, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1FA8*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1FA9*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1FAA*/
  0x0025, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1FAB*/
  0x0025, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1FAC*/
  0x0024, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1FAD*/
  0x0024, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0022, 0x0002, 0x0000, /*1FAE*/
  0x002A, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1FAF*/
  0x002A, 0x0002, 0x0000, 0x004C, 0x0002, 0x0004,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0000, /*1FB0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0032, 0x0002, 0x0000, /*1FB1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1FB2*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x004C, 0x0002, 0x0000, /*1FB3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1FB4*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC0, 0x0020, 0x0002, 0x9FB5, 0x0000, 0x0000, 0x0000, /*1FB5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x002A, 0x0002, 0x0000, /*1FB6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0002, 0x0000, 0x002A, 0x0002, 0x0000, /*1FB7*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0000, /*1FB8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0032, 0x0002, 0x0000, /*1FB9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0025, 0x0002, 0x0000, /*1FBA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*1FBB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FB9, 0x0020, 0x0008, 0x0000, 0x004C, 0x0002, 0x0000, /*1FBC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x048E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1FBD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1FBE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x048E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1FBF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0490, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1FC0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0489, 0x0020, 0x0002, 0x0000, 0x002A, 0x0002, 0x0000, /*1FC1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1FC2*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x004C, 0x0002, 0x0000, /*1FC3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1FC4*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC0, 0x0020, 0x0002, 0x9FC5, 0x0000, 0x0000, 0x0000, /*1FC5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x002A, 0x0002, 0x0000, /*1FC6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0002, 0x0000, 0x002A, 0x0002, 0x0000, /*1FC7*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FBE, 0x0020, 0x0008, 0x0000, 0x0025, 0x0002, 0x0000, /*1FC8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FBE, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*1FC9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0025, 0x0002, 0x0000, /*1FCA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*1FCB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC4, 0x0020, 0x0008, 0x0000, 0x004C, 0x0002, 0x0000, /*1FCC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x048E, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1FCD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x048E, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1FCE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x048E, 0x0020, 0x0002, 0x0000, 0x002A, 0x0002, 0x0000, /*1FCF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0000, /*1FD0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x0032, 0x0002, 0x0000, /*1FD1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0000, /*1FD2*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0000, /*1FD3*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC0, 0x0020, 0x0002, 0x9FD4, 0x0000, 0x0000, 0x0000, /*1FD4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9FD5, 0x0000, 0x0000, 0x0000, /*1FD5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x002A, 0x0002, 0x0000, /*1FD6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0000, /*1FD7*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0000, /*1FD8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0032, 0x0002, 0x0000, /*1FD9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0025, 0x0002, 0x0000, /*1FDA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FC6, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*1FDB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9FDC, 0x0000, 0x0000, 0x0000, /*1FDC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x048F, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1FDD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x048F, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1FDE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x048F, 0x0020, 0x0002, 0x0000, 0x002A, 0x0002, 0x0000, /*1FDF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0026, 0x0002, 0x0000, /*1FE0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x0032, 0x0002, 0x0000, /*1FE1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0000, /*1FE2*/
  0x0025, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0000, /*1FE3*/
  0x0024, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FD4, 0x0020, 0x0002, 0x0000, 0x0022, 0x0002, 0x0000, /*1FE4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FD4, 0x0020, 0x0002, 0x0000, 0x0023, 0x0002, 0x0000, /*1FE5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x002A, 0x0002, 0x0000, /*1FE6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0002, 0x0000, 0x002B, 0x0002, 0x0000, /*1FE7*/
  0x002A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FDC, 0x0020, 0x0008, 0x0000, 0x0026, 0x0002, 0x0000, /*1FE8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0008, 0x0000, 0x0032, 0x0002, 0x0000, /*1FE9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0008, 0x0000, 0x0025, 0x0002, 0x0000, /*1FEA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FDC, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*1FEB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FD4, 0x0020, 0x0008, 0x0000, 0x0023, 0x0002, 0x0000, /*1FEC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0489, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1FED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0489, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1FEE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0482, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1FEF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x9FF0, 0x0000, 0x0000, 0x0000, /*1FF0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0x9FF1, 0x0000, 0x0000, 0x0000, /*1FF1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0025, 0x0002, 0x0000, /*1FF2*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x004C, 0x0002, 0x0000, /*1FF3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x0024, 0x0002, 0x0000, /*1FF4*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC0, 0x0020, 0x0002, 0x9FF5, 0x0000, 0x0000, 0x0000, /*1FF5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x002A, 0x0002, 0x0000, /*1FF6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0002, 0x0000, 0x002A, 0x0002, 0x0000, /*1FF7*/
  0x004C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FCE, 0x0020, 0x0008, 0x0000, 0x0025, 0x0002, 0x0000, /*1FF8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCE, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*1FF9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0025, 0x0002, 0x0000, /*1FFA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0024, 0x0002, 0x0000, /*1FFB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x004C, 0x0002, 0x0000, /*1FFC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0483, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1FFD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x048F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1FFE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0x9FFF, 0x0000, 0x0000, 0x0000, /*1FFF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002
};

uint16 uca900_p020[]= {
  0x0209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2000*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2001*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2002*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2003*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2004*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2005*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2006*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x001B, 0x0000, 0x0000, 0x0000, 0x0000, /*2007*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2008*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2009*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*200A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*200B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*200C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*200D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*200E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*200F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0213, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2010*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0213, 0x0020, 0x001B, 0x0000, 0x0000, 0x0000, 0x0000, /*2011*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0214, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2012*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0215, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2013*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0216, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2014*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0217, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2015*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0383, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2016*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2017*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0306, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2018*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0307, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2019*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0308, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*201A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0309, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*201B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x030D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*201C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x030E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*201D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x030F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*201E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0310, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*201F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x039F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2020*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2021*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2022*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2023*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0277, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2024*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0277, 0x0020, 0x0004, 0x0277, 0x0020, 0x0004, 0x0000, /*2025*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0277, 0x0020, 0x0004, 0x0277, 0x0020, 0x0004, 0x0277, /*2026*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x03A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2027*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0207, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2028*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0208, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2029*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*202A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*202B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*202C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*202D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*202E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x001B, 0x0000, 0x0000, 0x0000, 0x0000, /*202F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x039B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2030*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x039D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2031*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2032*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03AA, 0x0020, 0x0004, 0x03AA, 0x0020, 0x0004, 0x0000, /*2033*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x03AA, 0x0020, 0x0004, 0x03AA, 0x0020, 0x0004, 0x03AA, /*2034*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x03AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2035*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03AB, 0x0020, 0x0004, 0x03AB, 0x0020, 0x0004, 0x0000, /*2036*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x03AB, 0x0020, 0x0004, 0x03AB, 0x0020, 0x0004, 0x03AB, /*2037*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x03AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2038*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x030A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2039*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x030B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*203A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*203B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0260, 0x0020, 0x0004, 0x0260, 0x0020, 0x0004, 0x0000, /*203C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0275, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*203D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*203E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*203F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2040*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2041*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2042*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2043*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0626, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2044*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0323, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2045*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0324, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2046*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0266, 0x0020, 0x0004, 0x0266, 0x0020, 0x0004, 0x0000, /*2047*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0266, 0x0020, 0x0004, 0x0260, 0x0020, 0x0004, 0x0000, /*2048*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0260, 0x0020, 0x0004, 0x0266, 0x0020, 0x0004, 0x0000, /*2049*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0397, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*204A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x038C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*204B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*204C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*204D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0390, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*204E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0236, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*204F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2050*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0391, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2051*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0622, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2052*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x021A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2053*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2054*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2055*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2056*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03AA, 0x0020, 0x0004, 0x03AA, 0x0020, 0x0004, 0x03AA, /*2057*/
  0x0020, 0x0004, 0x03AA, 0x0020, 0x0004, 0x0004,
  0x02EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2058*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2059*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*205A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*205B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*205C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*205D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*205E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*205F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*2060*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*2061*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*2062*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*2063*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*2064*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xA065, 0x0000, 0x0000, 0x0000, /*2065*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*2066*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*2067*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*2068*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*2069*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*206A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*206B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*206C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*206D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*206E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*206F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3D, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*2070*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*2071*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xA072, 0x0000, 0x0000, 0x0000, /*2072*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA073, 0x0000, 0x0000, 0x0000, /*2073*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*2074*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*2075*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*2076*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*2077*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*2078*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*2079*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0616, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*207A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0621, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*207B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x061B, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*207C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0317, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*207D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0318, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*207E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0000, /*207F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3D, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2080*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2081*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2082*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2083*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C41, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2084*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2085*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2086*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2087*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2088*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2089*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0616, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*208A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0621, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*208B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x061B, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*208C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0317, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*208D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0318, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*208E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xA08F, 0x0000, 0x0000, 0x0000, /*208F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2090*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2091*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2092*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFF, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2093*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CBD, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2094*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2095*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D65, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2096*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2097*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2098*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*2099*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0C, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*209A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*209B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x0015, 0x0000, 0x0000, 0x0000, 0x0000, /*209C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xA09D, 0x0000, 0x0000, 0x0000, /*209D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA09E, 0x0000, 0x0000, 0x0000, /*209E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA09F, 0x0000, 0x0000, 0x0000, /*209F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x000A, 0x1E71, 0x0020, 0x0004, 0x0000, /*20A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C34, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C35, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C38, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C39, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xA0BF, 0x0000, 0x0000, 0x0000, /*20BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0C0, 0x0000, 0x0000, 0x0000, /*20C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0C1, 0x0000, 0x0000, 0x0000, /*20C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0C2, 0x0000, 0x0000, 0x0000, /*20C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0C3, 0x0000, 0x0000, 0x0000, /*20C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0C4, 0x0000, 0x0000, 0x0000, /*20C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0C5, 0x0000, 0x0000, 0x0000, /*20C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0C6, 0x0000, 0x0000, 0x0000, /*20C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0C7, 0x0000, 0x0000, 0x0000, /*20C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0C8, 0x0000, 0x0000, 0x0000, /*20C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0C9, 0x0000, 0x0000, 0x0000, /*20C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0CA, 0x0000, 0x0000, 0x0000, /*20CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0CB, 0x0000, 0x0000, 0x0000, /*20CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0CC, 0x0000, 0x0000, 0x0000, /*20CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0CD, 0x0000, 0x0000, 0x0000, /*20CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0CE, 0x0000, 0x0000, 0x0000, /*20CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0CF, 0x0000, 0x0000, 0x0000, /*20CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0000, 0x0101, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0102, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0103, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0103, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0104, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0105, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0106, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0107, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0035, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0035, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0035, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0108, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0109, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0036, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0036, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0036, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0036, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x010A, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0036, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0036, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0036, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0035, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x010B, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x010C, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x010D, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x010E, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0035, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0035, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*20F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xA0F1, 0x0000, 0x0000, 0x0000, /*20F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0F2, 0x0000, 0x0000, 0x0000, /*20F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0F3, 0x0000, 0x0000, 0x0000, /*20F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0F4, 0x0000, 0x0000, 0x0000, /*20F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0F5, 0x0000, 0x0000, 0x0000, /*20F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0F6, 0x0000, 0x0000, 0x0000, /*20F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0F7, 0x0000, 0x0000, 0x0000, /*20F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0F8, 0x0000, 0x0000, 0x0000, /*20F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0F9, 0x0000, 0x0000, 0x0000, /*20F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0FA, 0x0000, 0x0000, 0x0000, /*20FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0FB, 0x0000, 0x0000, 0x0000, /*20FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0FC, 0x0000, 0x0000, 0x0000, /*20FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0FD, 0x0000, 0x0000, 0x0000, /*20FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0FE, 0x0000, 0x0000, 0x0000, /*20FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA0FF, 0x0000, 0x0000, 0x0000, /*20FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002
};

uint16 uca900_p021[]= {
  0x1C47, 0x0020, 0x0004, 0x0394, 0x0020, 0x0004, 0x1C7A, /*2100*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C47, 0x0020, 0x0004, 0x0394, 0x0020, 0x0004, 0x1E71, /*2101*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*2102*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04F6, 0x0020, 0x0004, 0x1C7A, 0x0020, 0x000A, 0x0000, /*2103*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0586, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2104*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x0004, 0x0394, 0x0020, 0x0004, 0x1DDD, /*2105*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C7A, 0x0020, 0x0004, 0x0394, 0x0020, 0x0004, 0x1EB5, /*2106*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1CC2, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0000, /*2107*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0587, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2108*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04F6, 0x0020, 0x0004, 0x1CE5, 0x0020, 0x000A, 0x0000, /*2109*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*210A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*210B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*210C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*210D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*210E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x0002, 0x0000, 0x0039, 0x0002, 0x0000, /*210F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*2110*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*2111*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*2112*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*2113*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0588, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2114*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*2115*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x000A, 0x1DDD, 0x0020, 0x0004, 0x0000, /*2116*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0589, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2117*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x058A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2118*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*2119*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*211A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*211B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*211C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*211D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x058B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*211E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x058C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*211F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x0014, 0x1DAA, 0x0020, 0x0014, 0x0000, /*2120*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x000A, 0x1CAA, 0x0020, 0x000A, 0x1D77, /*2121*/
  0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1E95, 0x0020, 0x0014, 0x1DAA, 0x0020, 0x0014, 0x0000, /*2122*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x058D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2123*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*2124*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x058E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2125*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*2126*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x058F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2127*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*2128*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0590, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2129*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D65, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*212A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0008, 0x0000, 0x0029, 0x0002, 0x0000, /*212B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*212C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*212D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0591, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*212E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*212F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*2130*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*2131*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*2132*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*2133*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*2134*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22B7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2135*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22B8, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2136*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22B9, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2137*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22BA, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2138*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*2139*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0592, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*213A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE5, 0x0020, 0x000A, 0x1C47, 0x0020, 0x000A, 0x1EFF, /*213B*/
  0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FCF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*213C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*213D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FBB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*213E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FCF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*213F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0615, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*2140*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0593, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2141*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0594, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2142*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0595, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2143*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0596, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2144*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0000, /*2145*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*2146*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*2147*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*2148*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0000, /*2149*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0597, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*214A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*214B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0598, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*214C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x000A, 0x0394, 0x0020, 0x0004, 0x1E71, /*214D*/
  0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1CF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*214E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0599, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*214F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C44, /*2150*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C46, /*2151*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C3E, /*2152*/
  0x0020, 0x001E, 0x1C3D, 0x0020, 0x001E, 0x0004,
  0x1C3E, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C40, /*2153*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3F, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C40, /*2154*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C42, /*2155*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3F, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C42, /*2156*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C40, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C42, /*2157*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C41, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C42, /*2158*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C43, /*2159*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C42, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C43, /*215A*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C45, /*215B*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C40, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C45, /*215C*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C42, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C45, /*215D*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C44, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C45, /*215E*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x0000, /*215F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0000, /*2160*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x000A, 0x1D32, 0x0020, 0x000A, 0x0000, /*2161*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x000A, 0x1D32, 0x0020, 0x000A, 0x1D32, /*2162*/
  0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1D32, 0x0020, 0x000A, 0x1EE3, 0x0020, 0x000A, 0x0000, /*2163*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EE3, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0000, /*2164*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x000A, 0x1D32, 0x0020, 0x000A, 0x0000, /*2165*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EE3, 0x0020, 0x000A, 0x1D32, 0x0020, 0x000A, 0x1D32, /*2166*/
  0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EE3, 0x0020, 0x000A, 0x1D32, 0x0020, 0x000A, 0x1D32, /*2167*/
  0x0020, 0x000A, 0x1D32, 0x0020, 0x000A, 0x0004,
  0x1D32, 0x0020, 0x000A, 0x1EFF, 0x0020, 0x000A, 0x0000, /*2168*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EFF, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0000, /*2169*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFF, 0x0020, 0x000A, 0x1D32, 0x0020, 0x000A, 0x0000, /*216A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EFF, 0x0020, 0x000A, 0x1D32, 0x0020, 0x000A, 0x1D32, /*216B*/
  0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1D77, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0000, /*216C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0000, /*216D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0000, /*216E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0000, /*216F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2170*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x0004, 0x1D32, 0x0020, 0x0004, 0x0000, /*2171*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x0004, 0x1D32, 0x0020, 0x0004, 0x1D32, /*2172*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1D32, 0x0020, 0x0004, 0x1EE3, 0x0020, 0x0004, 0x0000, /*2173*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EE3, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2174*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x0004, 0x1D32, 0x0020, 0x0004, 0x0000, /*2175*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EE3, 0x0020, 0x0004, 0x1D32, 0x0020, 0x0004, 0x1D32, /*2176*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EE3, 0x0020, 0x0004, 0x1D32, 0x0020, 0x0004, 0x1D32, /*2177*/
  0x0020, 0x0004, 0x1D32, 0x0020, 0x0004, 0x0004,
  0x1D32, 0x0020, 0x0004, 0x1EFF, 0x0020, 0x0004, 0x0000, /*2178*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EFF, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*2179*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFF, 0x0020, 0x0004, 0x1D32, 0x0020, 0x0004, 0x0000, /*217A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EFF, 0x0020, 0x0004, 0x1D32, 0x0020, 0x0004, 0x1D32, /*217B*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1D77, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*217C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*217D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*217E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000, /*217F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1AC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2180*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1AC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2181*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1AC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2182*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0000, /*2183*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2184*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2185*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1AC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2186*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1ACA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2187*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1ACB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2188*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3D, 0x0020, 0x001E, 0x0626, 0x0020, 0x001E, 0x1C40, /*2189*/
  0x0020, 0x001E, 0x0000, 0x0000, 0x0000, 0x0003,
  0x059A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*218A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x059B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*218B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xA18C, 0x0000, 0x0000, 0x0000, /*218C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA18D, 0x0000, 0x0000, 0x0000, /*218D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA18E, 0x0000, 0x0000, 0x0000, /*218E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA18F, 0x0000, 0x0000, 0x0000, /*218F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x059C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2190*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x059E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2191*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x059D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2192*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x059F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2193*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2194*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2195*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2196*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2197*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2198*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2199*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x059C, 0x0020, 0x0002, 0x0000, 0x002F, 0x0002, 0x0000, /*219A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x059D, 0x0020, 0x0002, 0x0000, 0x002F, 0x0002, 0x0000, /*219B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x05A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*219C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*219D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*219E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*219F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05A0, 0x0020, 0x0002, 0x0000, 0x002F, 0x0002, 0x0000, /*21AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x05B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05D6, 0x0020, 0x0002, 0x0000, 0x002F, 0x0002, 0x0000, /*21CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x05DA, 0x0020, 0x0002, 0x0000, 0x002F, 0x0002, 0x0000, /*21CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x05D8, 0x0020, 0x0002, 0x0000, 0x002F, 0x0002, 0x0000, /*21CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x05D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x05FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0600, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0601, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0602, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0603, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0604, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0605, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*21FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p022[]= {
  0x0606, 0x0020, 0x0002, 0x0000, 0x0000, /*2200*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0607, 0x0020, 0x0002, 0x0000, 0x0000, /*2201*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0608, 0x0020, 0x0002, 0x0000, 0x0000, /*2202*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0609, 0x0020, 0x0002, 0x0000, 0x0000, /*2203*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0609, 0x0020, 0x0002, 0x0000, 0x002F, /*2204*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x060A, 0x0020, 0x0002, 0x0000, 0x0000, /*2205*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x060B, 0x0020, 0x0002, 0x0000, 0x0000, /*2206*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x060C, 0x0020, 0x0002, 0x0000, 0x0000, /*2207*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x060D, 0x0020, 0x0002, 0x0000, 0x0000, /*2208*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x060D, 0x0020, 0x0002, 0x0000, 0x002F, /*2209*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x060E, 0x0020, 0x0002, 0x0000, 0x0000, /*220A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x060F, 0x0020, 0x0002, 0x0000, 0x0000, /*220B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x060F, 0x0020, 0x0002, 0x0000, 0x002F, /*220C*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0610, 0x0020, 0x0002, 0x0000, 0x0000, /*220D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0612, 0x0020, 0x0002, 0x0000, 0x0000, /*220E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0613, 0x0020, 0x0002, 0x0000, 0x0000, /*220F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0614, 0x0020, 0x0002, 0x0000, 0x0000, /*2210*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0615, 0x0020, 0x0002, 0x0000, 0x0000, /*2211*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0621, 0x0020, 0x0002, 0x0000, 0x0000, /*2212*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0623, 0x0020, 0x0002, 0x0000, 0x0000, /*2213*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0624, 0x0020, 0x0002, 0x0000, 0x0000, /*2214*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0625, 0x0020, 0x0002, 0x0000, 0x0000, /*2215*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0627, 0x0020, 0x0002, 0x0000, 0x0000, /*2216*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0628, 0x0020, 0x0002, 0x0000, 0x0000, /*2217*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0629, 0x0020, 0x0002, 0x0000, 0x0000, /*2218*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x062A, 0x0020, 0x0002, 0x0000, 0x0000, /*2219*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x062B, 0x0020, 0x0002, 0x0000, 0x0000, /*221A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x062C, 0x0020, 0x0002, 0x0000, 0x0000, /*221B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x062E, 0x0020, 0x0002, 0x0000, 0x0000, /*221C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0630, 0x0020, 0x0002, 0x0000, 0x0000, /*221D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0631, 0x0020, 0x0002, 0x0000, 0x0000, /*221E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0632, 0x0020, 0x0002, 0x0000, 0x0000, /*221F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0633, 0x0020, 0x0002, 0x0000, 0x0000, /*2220*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0634, 0x0020, 0x0002, 0x0000, 0x0000, /*2221*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0635, 0x0020, 0x0002, 0x0000, 0x0000, /*2222*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0636, 0x0020, 0x0002, 0x0000, 0x0000, /*2223*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0636, 0x0020, 0x0002, 0x0000, 0x002F, /*2224*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0637, 0x0020, 0x0002, 0x0000, 0x0000, /*2225*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0637, 0x0020, 0x0002, 0x0000, 0x002F, /*2226*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0638, 0x0020, 0x0002, 0x0000, 0x0000, /*2227*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0639, 0x0020, 0x0002, 0x0000, 0x0000, /*2228*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x063A, 0x0020, 0x0002, 0x0000, 0x0000, /*2229*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x063B, 0x0020, 0x0002, 0x0000, 0x0000, /*222A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x063C, 0x0020, 0x0002, 0x0000, 0x0000, /*222B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x063C, 0x0020, 0x0004, 0x063C, 0x0020, /*222C*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x063C, 0x0020, 0x0004, 0x063C, 0x0020, /*222D*/
  0x0004, 0x063C, 0x0020, 0x0004, 0x0003,
  0x063D, 0x0020, 0x0002, 0x0000, 0x0000, /*222E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x063D, 0x0020, 0x0004, 0x063D, 0x0020, /*222F*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x063D, 0x0020, 0x0004, 0x063D, 0x0020, /*2230*/
  0x0004, 0x063D, 0x0020, 0x0004, 0x0003,
  0x063E, 0x0020, 0x0002, 0x0000, 0x0000, /*2231*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x063F, 0x0020, 0x0002, 0x0000, 0x0000, /*2232*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0640, 0x0020, 0x0002, 0x0000, 0x0000, /*2233*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0641, 0x0020, 0x0002, 0x0000, 0x0000, /*2234*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0642, 0x0020, 0x0002, 0x0000, 0x0000, /*2235*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0643, 0x0020, 0x0002, 0x0000, 0x0000, /*2236*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0644, 0x0020, 0x0002, 0x0000, 0x0000, /*2237*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0645, 0x0020, 0x0002, 0x0000, 0x0000, /*2238*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0646, 0x0020, 0x0002, 0x0000, 0x0000, /*2239*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0647, 0x0020, 0x0002, 0x0000, 0x0000, /*223A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0648, 0x0020, 0x0002, 0x0000, 0x0000, /*223B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0649, 0x0020, 0x0002, 0x0000, 0x0000, /*223C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x064A, 0x0020, 0x0002, 0x0000, 0x0000, /*223D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x064B, 0x0020, 0x0002, 0x0000, 0x0000, /*223E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x064C, 0x0020, 0x0002, 0x0000, 0x0000, /*223F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x064D, 0x0020, 0x0002, 0x0000, 0x0000, /*2240*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0649, 0x0020, 0x0002, 0x0000, 0x002F, /*2241*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x064E, 0x0020, 0x0002, 0x0000, 0x0000, /*2242*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x064F, 0x0020, 0x0002, 0x0000, 0x0000, /*2243*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x064F, 0x0020, 0x0002, 0x0000, 0x002F, /*2244*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0650, 0x0020, 0x0002, 0x0000, 0x0000, /*2245*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0651, 0x0020, 0x0002, 0x0000, 0x0000, /*2246*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0650, 0x0020, 0x0002, 0x0000, 0x002F, /*2247*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0652, 0x0020, 0x0002, 0x0000, 0x0000, /*2248*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0652, 0x0020, 0x0002, 0x0000, 0x002F, /*2249*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0653, 0x0020, 0x0002, 0x0000, 0x0000, /*224A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0654, 0x0020, 0x0002, 0x0000, 0x0000, /*224B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0655, 0x0020, 0x0002, 0x0000, 0x0000, /*224C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0656, 0x0020, 0x0002, 0x0000, 0x0000, /*224D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0657, 0x0020, 0x0002, 0x0000, 0x0000, /*224E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0658, 0x0020, 0x0002, 0x0000, 0x0000, /*224F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0659, 0x0020, 0x0002, 0x0000, 0x0000, /*2250*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x065A, 0x0020, 0x0002, 0x0000, 0x0000, /*2251*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x065B, 0x0020, 0x0002, 0x0000, 0x0000, /*2252*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x065C, 0x0020, 0x0002, 0x0000, 0x0000, /*2253*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x065D, 0x0020, 0x0002, 0x0000, 0x0000, /*2254*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x065E, 0x0020, 0x0002, 0x0000, 0x0000, /*2255*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x065F, 0x0020, 0x0002, 0x0000, 0x0000, /*2256*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0660, 0x0020, 0x0002, 0x0000, 0x0000, /*2257*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0661, 0x0020, 0x0002, 0x0000, 0x0000, /*2258*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0662, 0x0020, 0x0002, 0x0000, 0x0000, /*2259*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0663, 0x0020, 0x0002, 0x0000, 0x0000, /*225A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0664, 0x0020, 0x0002, 0x0000, 0x0000, /*225B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0665, 0x0020, 0x0002, 0x0000, 0x0000, /*225C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0666, 0x0020, 0x0002, 0x0000, 0x0000, /*225D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0667, 0x0020, 0x0002, 0x0000, 0x0000, /*225E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0668, 0x0020, 0x0002, 0x0000, 0x0000, /*225F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x061B, 0x0020, 0x0002, 0x0000, 0x002F, /*2260*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0669, 0x0020, 0x0002, 0x0000, 0x0000, /*2261*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0669, 0x0020, 0x0002, 0x0000, 0x002F, /*2262*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x066A, 0x0020, 0x0002, 0x0000, 0x0000, /*2263*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x066B, 0x0020, 0x0002, 0x0000, 0x0000, /*2264*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x066C, 0x0020, 0x0002, 0x0000, 0x0000, /*2265*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x066D, 0x0020, 0x0002, 0x0000, 0x0000, /*2266*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x066E, 0x0020, 0x0002, 0x0000, 0x0000, /*2267*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x066F, 0x0020, 0x0002, 0x0000, 0x0000, /*2268*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0670, 0x0020, 0x0002, 0x0000, 0x0000, /*2269*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0671, 0x0020, 0x0002, 0x0000, 0x0000, /*226A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0672, 0x0020, 0x0002, 0x0000, 0x0000, /*226B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0673, 0x0020, 0x0002, 0x0000, 0x0000, /*226C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0656, 0x0020, 0x0002, 0x0000, 0x002F, /*226D*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x061A, 0x0020, 0x0002, 0x0000, 0x002F, /*226E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x061C, 0x0020, 0x0002, 0x0000, 0x002F, /*226F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x066B, 0x0020, 0x0002, 0x0000, 0x002F, /*2270*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x066C, 0x0020, 0x0002, 0x0000, 0x002F, /*2271*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0674, 0x0020, 0x0002, 0x0000, 0x0000, /*2272*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0675, 0x0020, 0x0002, 0x0000, 0x0000, /*2273*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0674, 0x0020, 0x0002, 0x0000, 0x002F, /*2274*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0675, 0x0020, 0x0002, 0x0000, 0x002F, /*2275*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0676, 0x0020, 0x0002, 0x0000, 0x0000, /*2276*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0677, 0x0020, 0x0002, 0x0000, 0x0000, /*2277*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0676, 0x0020, 0x0002, 0x0000, 0x002F, /*2278*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0677, 0x0020, 0x0002, 0x0000, 0x002F, /*2279*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0678, 0x0020, 0x0002, 0x0000, 0x0000, /*227A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0679, 0x0020, 0x0002, 0x0000, 0x0000, /*227B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x067A, 0x0020, 0x0002, 0x0000, 0x0000, /*227C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x067B, 0x0020, 0x0002, 0x0000, 0x0000, /*227D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x067C, 0x0020, 0x0002, 0x0000, 0x0000, /*227E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x067D, 0x0020, 0x0002, 0x0000, 0x0000, /*227F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0678, 0x0020, 0x0002, 0x0000, 0x002F, /*2280*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0679, 0x0020, 0x0002, 0x0000, 0x002F, /*2281*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x067E, 0x0020, 0x0002, 0x0000, 0x0000, /*2282*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x067F, 0x0020, 0x0002, 0x0000, 0x0000, /*2283*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x067E, 0x0020, 0x0002, 0x0000, 0x002F, /*2284*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x067F, 0x0020, 0x0002, 0x0000, 0x002F, /*2285*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0680, 0x0020, 0x0002, 0x0000, 0x0000, /*2286*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0681, 0x0020, 0x0002, 0x0000, 0x0000, /*2287*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0680, 0x0020, 0x0002, 0x0000, 0x002F, /*2288*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0681, 0x0020, 0x0002, 0x0000, 0x002F, /*2289*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0682, 0x0020, 0x0002, 0x0000, 0x0000, /*228A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0683, 0x0020, 0x0002, 0x0000, 0x0000, /*228B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0684, 0x0020, 0x0002, 0x0000, 0x0000, /*228C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0685, 0x0020, 0x0002, 0x0000, 0x0000, /*228D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0686, 0x0020, 0x0002, 0x0000, 0x0000, /*228E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0687, 0x0020, 0x0002, 0x0000, 0x0000, /*228F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0688, 0x0020, 0x0002, 0x0000, 0x0000, /*2290*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0689, 0x0020, 0x0002, 0x0000, 0x0000, /*2291*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x068A, 0x0020, 0x0002, 0x0000, 0x0000, /*2292*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x068B, 0x0020, 0x0002, 0x0000, 0x0000, /*2293*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x068C, 0x0020, 0x0002, 0x0000, 0x0000, /*2294*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x068D, 0x0020, 0x0002, 0x0000, 0x0000, /*2295*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x068E, 0x0020, 0x0002, 0x0000, 0x0000, /*2296*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x068F, 0x0020, 0x0002, 0x0000, 0x0000, /*2297*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0690, 0x0020, 0x0002, 0x0000, 0x0000, /*2298*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0691, 0x0020, 0x0002, 0x0000, 0x0000, /*2299*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0692, 0x0020, 0x0002, 0x0000, 0x0000, /*229A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0693, 0x0020, 0x0002, 0x0000, 0x0000, /*229B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0694, 0x0020, 0x0002, 0x0000, 0x0000, /*229C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0695, 0x0020, 0x0002, 0x0000, 0x0000, /*229D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0696, 0x0020, 0x0002, 0x0000, 0x0000, /*229E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0697, 0x0020, 0x0002, 0x0000, 0x0000, /*229F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0698, 0x0020, 0x0002, 0x0000, 0x0000, /*22A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0699, 0x0020, 0x0002, 0x0000, 0x0000, /*22A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x069A, 0x0020, 0x0002, 0x0000, 0x0000, /*22A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x069B, 0x0020, 0x0002, 0x0000, 0x0000, /*22A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x069C, 0x0020, 0x0002, 0x0000, 0x0000, /*22A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x069D, 0x0020, 0x0002, 0x0000, 0x0000, /*22A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x069E, 0x0020, 0x0002, 0x0000, 0x0000, /*22A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x069F, 0x0020, 0x0002, 0x0000, 0x0000, /*22A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06A0, 0x0020, 0x0002, 0x0000, 0x0000, /*22A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06A1, 0x0020, 0x0002, 0x0000, 0x0000, /*22A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06A2, 0x0020, 0x0002, 0x0000, 0x0000, /*22AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06A3, 0x0020, 0x0002, 0x0000, 0x0000, /*22AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x069A, 0x0020, 0x0002, 0x0000, 0x002F, /*22AC*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x06A0, 0x0020, 0x0002, 0x0000, 0x002F, /*22AD*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x06A1, 0x0020, 0x0002, 0x0000, 0x002F, /*22AE*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x06A3, 0x0020, 0x0002, 0x0000, 0x002F, /*22AF*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x06A4, 0x0020, 0x0002, 0x0000, 0x0000, /*22B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06A5, 0x0020, 0x0002, 0x0000, 0x0000, /*22B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06A6, 0x0020, 0x0002, 0x0000, 0x0000, /*22B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06A7, 0x0020, 0x0002, 0x0000, 0x0000, /*22B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06A8, 0x0020, 0x0002, 0x0000, 0x0000, /*22B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06A9, 0x0020, 0x0002, 0x0000, 0x0000, /*22B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06AA, 0x0020, 0x0002, 0x0000, 0x0000, /*22B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06AB, 0x0020, 0x0002, 0x0000, 0x0000, /*22B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06AC, 0x0020, 0x0002, 0x0000, 0x0000, /*22B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06AD, 0x0020, 0x0002, 0x0000, 0x0000, /*22B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06AE, 0x0020, 0x0002, 0x0000, 0x0000, /*22BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06AF, 0x0020, 0x0002, 0x0000, 0x0000, /*22BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06B0, 0x0020, 0x0002, 0x0000, 0x0000, /*22BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06B2, 0x0020, 0x0002, 0x0000, 0x0000, /*22BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06B3, 0x0020, 0x0002, 0x0000, 0x0000, /*22BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06B4, 0x0020, 0x0002, 0x0000, 0x0000, /*22BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06B5, 0x0020, 0x0002, 0x0000, 0x0000, /*22C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06B6, 0x0020, 0x0002, 0x0000, 0x0000, /*22C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06B7, 0x0020, 0x0002, 0x0000, 0x0000, /*22C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06B8, 0x0020, 0x0002, 0x0000, 0x0000, /*22C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06B9, 0x0020, 0x0002, 0x0000, 0x0000, /*22C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06BA, 0x0020, 0x0002, 0x0000, 0x0000, /*22C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06BB, 0x0020, 0x0002, 0x0000, 0x0000, /*22C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06BC, 0x0020, 0x0002, 0x0000, 0x0000, /*22C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06BD, 0x0020, 0x0002, 0x0000, 0x0000, /*22C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06BE, 0x0020, 0x0002, 0x0000, 0x0000, /*22C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06BF, 0x0020, 0x0002, 0x0000, 0x0000, /*22CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06C0, 0x0020, 0x0002, 0x0000, 0x0000, /*22CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06C1, 0x0020, 0x0002, 0x0000, 0x0000, /*22CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06C2, 0x0020, 0x0002, 0x0000, 0x0000, /*22CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06C3, 0x0020, 0x0002, 0x0000, 0x0000, /*22CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06C4, 0x0020, 0x0002, 0x0000, 0x0000, /*22CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06C5, 0x0020, 0x0002, 0x0000, 0x0000, /*22D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06C6, 0x0020, 0x0002, 0x0000, 0x0000, /*22D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06C7, 0x0020, 0x0002, 0x0000, 0x0000, /*22D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06C8, 0x0020, 0x0002, 0x0000, 0x0000, /*22D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06C9, 0x0020, 0x0002, 0x0000, 0x0000, /*22D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06CA, 0x0020, 0x0002, 0x0000, 0x0000, /*22D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06CB, 0x0020, 0x0002, 0x0000, 0x0000, /*22D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06CC, 0x0020, 0x0002, 0x0000, 0x0000, /*22D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06CD, 0x0020, 0x0002, 0x0000, 0x0000, /*22D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06CE, 0x0020, 0x0002, 0x0000, 0x0000, /*22D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06CF, 0x0020, 0x0002, 0x0000, 0x0000, /*22DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06D0, 0x0020, 0x0002, 0x0000, 0x0000, /*22DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06D1, 0x0020, 0x0002, 0x0000, 0x0000, /*22DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06D2, 0x0020, 0x0002, 0x0000, 0x0000, /*22DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06D3, 0x0020, 0x0002, 0x0000, 0x0000, /*22DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06D4, 0x0020, 0x0002, 0x0000, 0x0000, /*22DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x067A, 0x0020, 0x0002, 0x0000, 0x002F, /*22E0*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x067B, 0x0020, 0x0002, 0x0000, 0x002F, /*22E1*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0689, 0x0020, 0x0002, 0x0000, 0x002F, /*22E2*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x068A, 0x0020, 0x0002, 0x0000, 0x002F, /*22E3*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x06D5, 0x0020, 0x0002, 0x0000, 0x0000, /*22E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06D6, 0x0020, 0x0002, 0x0000, 0x0000, /*22E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06D7, 0x0020, 0x0002, 0x0000, 0x0000, /*22E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06D8, 0x0020, 0x0002, 0x0000, 0x0000, /*22E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06D9, 0x0020, 0x0002, 0x0000, 0x0000, /*22E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06DA, 0x0020, 0x0002, 0x0000, 0x0000, /*22E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06A6, 0x0020, 0x0002, 0x0000, 0x002F, /*22EA*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x06A7, 0x0020, 0x0002, 0x0000, 0x002F, /*22EB*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x06A8, 0x0020, 0x0002, 0x0000, 0x002F, /*22EC*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x06A9, 0x0020, 0x0002, 0x0000, 0x002F, /*22ED*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x06DB, 0x0020, 0x0002, 0x0000, 0x0000, /*22EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06DC, 0x0020, 0x0002, 0x0000, 0x0000, /*22EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06DD, 0x0020, 0x0002, 0x0000, 0x0000, /*22F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06DE, 0x0020, 0x0002, 0x0000, 0x0000, /*22F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06DF, 0x0020, 0x0002, 0x0000, 0x0000, /*22F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06E0, 0x0020, 0x0002, 0x0000, 0x0000, /*22F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06E1, 0x0020, 0x0002, 0x0000, 0x0000, /*22F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06E2, 0x0020, 0x0002, 0x0000, 0x0000, /*22F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06E3, 0x0020, 0x0002, 0x0000, 0x0000, /*22F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06E4, 0x0020, 0x0002, 0x0000, 0x0000, /*22F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06E5, 0x0020, 0x0002, 0x0000, 0x0000, /*22F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06E6, 0x0020, 0x0002, 0x0000, 0x0000, /*22F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06E7, 0x0020, 0x0002, 0x0000, 0x0000, /*22FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06E8, 0x0020, 0x0002, 0x0000, 0x0000, /*22FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06E9, 0x0020, 0x0002, 0x0000, 0x0000, /*22FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06EA, 0x0020, 0x0002, 0x0000, 0x0000, /*22FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06EB, 0x0020, 0x0002, 0x0000, 0x0000, /*22FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x06EC, 0x0020, 0x0002, 0x0000, 0x0000, /*22FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p023[]= {
  0x06ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2300*/
  0x06EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2301*/
  0x06EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2302*/
  0x06F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2303*/
  0x06F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2304*/
  0x06F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2305*/
  0x06F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2306*/
  0x06F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2307*/
  0x0325, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2308*/
  0x0326, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2309*/
  0x0327, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*230A*/
  0x0328, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*230B*/
  0x06F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*230C*/
  0x06F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*230D*/
  0x06F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*230E*/
  0x06F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*230F*/
  0x06F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2310*/
  0x06FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2311*/
  0x06FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2312*/
  0x06FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2313*/
  0x06FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2314*/
  0x06FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2315*/
  0x06FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2316*/
  0x0700, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2317*/
  0x0701, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2318*/
  0x0702, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2319*/
  0x0703, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*231A*/
  0x0704, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*231B*/
  0x0705, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*231C*/
  0x0706, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*231D*/
  0x0707, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*231E*/
  0x0708, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*231F*/
  0x0709, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2320*/
  0x070A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2321*/
  0x070B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2322*/
  0x070C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2323*/
  0x070D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2324*/
  0x070E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2325*/
  0x070F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2326*/
  0x0710, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2327*/
  0x0711, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2328*/
  0x036F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2329*/
  0x0370, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*232A*/
  0x0712, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*232B*/
  0x0713, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*232C*/
  0x0714, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*232D*/
  0x0715, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*232E*/
  0x0716, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*232F*/
  0x0717, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2330*/
  0x0718, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2331*/
  0x0719, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2332*/
  0x071A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2333*/
  0x071B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2334*/
  0x071C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2335*/
  0x071D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2336*/
  0x071E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2337*/
  0x071F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2338*/
  0x0720, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2339*/
  0x0721, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*233A*/
  0x0722, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*233B*/
  0x0723, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*233C*/
  0x0724, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*233D*/
  0x0725, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*233E*/
  0x0726, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*233F*/
  0x0727, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2340*/
  0x0728, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2341*/
  0x0729, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2342*/
  0x072A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2343*/
  0x072B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2344*/
  0x072C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2345*/
  0x072D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2346*/
  0x072E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2347*/
  0x072F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2348*/
  0x0730, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2349*/
  0x0731, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*234A*/
  0x0732, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*234B*/
  0x0733, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*234C*/
  0x0734, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*234D*/
  0x0735, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*234E*/
  0x0736, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*234F*/
  0x0737, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2350*/
  0x0738, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2351*/
  0x0739, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2352*/
  0x073A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2353*/
  0x073B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2354*/
  0x073C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2355*/
  0x073D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2356*/
  0x073E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2357*/
  0x073F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2358*/
  0x0740, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2359*/
  0x0741, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*235A*/
  0x0742, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*235B*/
  0x0743, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*235C*/
  0x0744, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*235D*/
  0x0745, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*235E*/
  0x0746, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*235F*/
  0x0747, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2360*/
  0x0748, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2361*/
  0x0749, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2362*/
  0x074A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2363*/
  0x074B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2364*/
  0x074C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2365*/
  0x074D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2366*/
  0x074E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2367*/
  0x074F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2368*/
  0x0750, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2369*/
  0x0751, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*236A*/
  0x0752, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*236B*/
  0x0753, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*236C*/
  0x0754, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*236D*/
  0x0755, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*236E*/
  0x0756, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*236F*/
  0x0757, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2370*/
  0x0758, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2371*/
  0x0759, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2372*/
  0x075A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2373*/
  0x075B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2374*/
  0x075C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2375*/
  0x075D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2376*/
  0x075E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2377*/
  0x075F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2378*/
  0x0760, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2379*/
  0x0761, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*237A*/
  0x0762, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*237B*/
  0x0763, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*237C*/
  0x0764, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*237D*/
  0x0765, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*237E*/
  0x0766, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*237F*/
  0x0767, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2380*/
  0x0768, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2381*/
  0x0769, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2382*/
  0x076A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2383*/
  0x076B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2384*/
  0x076C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2385*/
  0x076D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2386*/
  0x076E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2387*/
  0x076F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2388*/
  0x0770, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2389*/
  0x0771, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*238A*/
  0x0772, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*238B*/
  0x0773, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*238C*/
  0x0774, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*238D*/
  0x0775, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*238E*/
  0x0776, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*238F*/
  0x0777, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2390*/
  0x0778, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2391*/
  0x0779, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2392*/
  0x077A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2393*/
  0x077B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2394*/
  0x077C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2395*/
  0x077D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2396*/
  0x077E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2397*/
  0x077F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2398*/
  0x0780, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2399*/
  0x0781, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*239A*/
  0x0782, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*239B*/
  0x0783, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*239C*/
  0x0784, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*239D*/
  0x0785, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*239E*/
  0x0786, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*239F*/
  0x0787, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23A0*/
  0x0788, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23A1*/
  0x0789, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23A2*/
  0x078A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23A3*/
  0x078B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23A4*/
  0x078C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23A5*/
  0x078D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23A6*/
  0x078E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23A7*/
  0x078F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23A8*/
  0x0790, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23A9*/
  0x0791, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23AA*/
  0x0792, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23AB*/
  0x0793, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23AC*/
  0x0794, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23AD*/
  0x0795, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23AE*/
  0x0796, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23AF*/
  0x0797, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23B0*/
  0x0798, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23B1*/
  0x0799, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23B2*/
  0x079A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23B3*/
  0x079B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23B4*/
  0x079C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23B5*/
  0x079D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23B6*/
  0x079E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23B7*/
  0x079F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23B8*/
  0x07A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23B9*/
  0x07A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23BA*/
  0x07A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23BB*/
  0x07A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23BC*/
  0x07A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23BD*/
  0x07A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23BE*/
  0x07A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23BF*/
  0x07A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23C0*/
  0x07A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23C1*/
  0x07A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23C2*/
  0x07AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23C3*/
  0x07AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23C4*/
  0x07AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23C5*/
  0x07AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23C6*/
  0x07AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23C7*/
  0x07AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23C8*/
  0x07B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23C9*/
  0x07B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23CA*/
  0x07B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23CB*/
  0x07B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23CC*/
  0x07B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23CD*/
  0x07B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23CE*/
  0x07B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23CF*/
  0x07B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23D0*/
  0x07B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23D1*/
  0x07B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23D2*/
  0x07BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23D3*/
  0x07BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23D4*/
  0x07BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23D5*/
  0x07BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23D6*/
  0x07BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23D7*/
  0x07BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23D8*/
  0x07C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23D9*/
  0x07C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23DA*/
  0x07C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23DB*/
  0x07C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23DC*/
  0x07C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23DD*/
  0x07C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23DE*/
  0x07C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23DF*/
  0x07C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23E0*/
  0x07C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23E1*/
  0x07C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23E2*/
  0x07CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23E3*/
  0x07CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23E4*/
  0x07CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23E5*/
  0x07CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23E6*/
  0x07CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23E7*/
  0x07CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23E8*/
  0x07D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23E9*/
  0x07D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23EA*/
  0x07D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23EB*/
  0x07D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23EC*/
  0x07D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23ED*/
  0x07D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23EE*/
  0x07D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23EF*/
  0x07D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23F0*/
  0x07D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23F1*/
  0x07D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23F2*/
  0x07DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23F3*/
  0x07DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23F4*/
  0x07DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23F5*/
  0x07DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23F6*/
  0x07DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23F7*/
  0x07DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23F8*/
  0x07E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23F9*/
  0x07E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23FA*/
  0x07E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23FB*/
  0x07E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23FC*/
  0x07E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23FD*/
  0x07E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*23FE*/
  0xFBC0, 0x0020, 0x0002, 0xA3FF, 0x0000, 0x0000, 0x0002  /*23FF*/
};

uint16 uca900_p024[]= {
  0x07E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2400*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2401*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2402*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2403*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2404*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2405*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2406*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2407*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2408*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2409*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*240A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*240B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*240C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*240D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*240E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*240F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2410*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2411*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2412*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2413*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2414*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2415*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2416*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2417*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2418*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x07FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2419*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0800, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*241A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0801, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*241B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0802, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*241C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0803, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*241D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0804, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*241E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0805, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*241F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0806, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2420*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0807, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2421*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0808, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2422*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0809, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2423*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x080A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2424*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x080B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2425*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x080C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2426*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xA427, 0x0000, 0x0000, 0x0000, /*2427*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA428, 0x0000, 0x0000, 0x0000, /*2428*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA429, 0x0000, 0x0000, 0x0000, /*2429*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA42A, 0x0000, 0x0000, 0x0000, /*242A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA42B, 0x0000, 0x0000, 0x0000, /*242B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA42C, 0x0000, 0x0000, 0x0000, /*242C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA42D, 0x0000, 0x0000, 0x0000, /*242D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA42E, 0x0000, 0x0000, 0x0000, /*242E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA42F, 0x0000, 0x0000, 0x0000, /*242F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA430, 0x0000, 0x0000, 0x0000, /*2430*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA431, 0x0000, 0x0000, 0x0000, /*2431*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA432, 0x0000, 0x0000, 0x0000, /*2432*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA433, 0x0000, 0x0000, 0x0000, /*2433*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA434, 0x0000, 0x0000, 0x0000, /*2434*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA435, 0x0000, 0x0000, 0x0000, /*2435*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA436, 0x0000, 0x0000, 0x0000, /*2436*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA437, 0x0000, 0x0000, 0x0000, /*2437*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA438, 0x0000, 0x0000, 0x0000, /*2438*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA439, 0x0000, 0x0000, 0x0000, /*2439*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA43A, 0x0000, 0x0000, 0x0000, /*243A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA43B, 0x0000, 0x0000, 0x0000, /*243B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA43C, 0x0000, 0x0000, 0x0000, /*243C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA43D, 0x0000, 0x0000, 0x0000, /*243D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA43E, 0x0000, 0x0000, 0x0000, /*243E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA43F, 0x0000, 0x0000, 0x0000, /*243F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x080D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2440*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x080E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2441*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x080F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2442*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0810, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2443*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0811, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2444*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0812, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2445*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0813, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2446*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0814, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2447*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0815, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2448*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0816, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2449*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0817, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*244A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xA44B, 0x0000, 0x0000, 0x0000, /*244B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA44C, 0x0000, 0x0000, 0x0000, /*244C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA44D, 0x0000, 0x0000, 0x0000, /*244D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA44E, 0x0000, 0x0000, 0x0000, /*244E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA44F, 0x0000, 0x0000, 0x0000, /*244F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA450, 0x0000, 0x0000, 0x0000, /*2450*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA451, 0x0000, 0x0000, 0x0000, /*2451*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA452, 0x0000, 0x0000, 0x0000, /*2452*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA453, 0x0000, 0x0000, 0x0000, /*2453*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA454, 0x0000, 0x0000, 0x0000, /*2454*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA455, 0x0000, 0x0000, 0x0000, /*2455*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA456, 0x0000, 0x0000, 0x0000, /*2456*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA457, 0x0000, 0x0000, 0x0000, /*2457*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA458, 0x0000, 0x0000, 0x0000, /*2458*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA459, 0x0000, 0x0000, 0x0000, /*2459*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA45A, 0x0000, 0x0000, 0x0000, /*245A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA45B, 0x0000, 0x0000, 0x0000, /*245B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA45C, 0x0000, 0x0000, 0x0000, /*245C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA45D, 0x0000, 0x0000, 0x0000, /*245D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA45E, 0x0000, 0x0000, 0x0000, /*245E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xA45F, 0x0000, 0x0000, 0x0000, /*245F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*2460*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*2461*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*2462*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C41, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*2463*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*2464*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*2465*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*2466*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*2467*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*2468*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, /*2469*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C3E, 0x0020, 0x0006, 0x0000, /*246A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C3F, 0x0020, 0x0006, 0x0000, /*246B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C40, 0x0020, 0x0006, 0x0000, /*246C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C41, 0x0020, 0x0006, 0x0000, /*246D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C42, 0x0020, 0x0006, 0x0000, /*246E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C43, 0x0020, 0x0006, 0x0000, /*246F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C44, 0x0020, 0x0006, 0x0000, /*2470*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C45, 0x0020, 0x0006, 0x0000, /*2471*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C46, 0x0020, 0x0006, 0x0000, /*2472*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, /*2473*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0317, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0x0318, /*2474*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C3F, 0x0020, 0x0004, 0x0318, /*2475*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C40, 0x0020, 0x0004, 0x0318, /*2476*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C41, 0x0020, 0x0004, 0x0318, /*2477*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C42, 0x0020, 0x0004, 0x0318, /*2478*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C43, 0x0020, 0x0004, 0x0318, /*2479*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C44, 0x0020, 0x0004, 0x0318, /*247A*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C45, 0x0020, 0x0004, 0x0318, /*247B*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C46, 0x0020, 0x0004, 0x0318, /*247C*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0x1C3D, /*247D*/
  0x0020, 0x0004, 0x0318, 0x0020, 0x0004, 0x0004,
  0x0317, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0x1C3E, /*247E*/
  0x0020, 0x0004, 0x0318, 0x0020, 0x0004, 0x0004,
  0x0317, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0x1C3F, /*247F*/
  0x0020, 0x0004, 0x0318, 0x0020, 0x0004, 0x0004,
  0x0317, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0x1C40, /*2480*/
  0x0020, 0x0004, 0x0318, 0x0020, 0x0004, 0x0004,
  0x0317, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0x1C41, /*2481*/
  0x0020, 0x0004, 0x0318, 0x0020, 0x0004, 0x0004,
  0x0317, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0x1C42, /*2482*/
  0x0020, 0x0004, 0x0318, 0x0020, 0x0004, 0x0004,
  0x0317, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0x1C43, /*2483*/
  0x0020, 0x0004, 0x0318, 0x0020, 0x0004, 0x0004,
  0x0317, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0x1C44, /*2484*/
  0x0020, 0x0004, 0x0318, 0x0020, 0x0004, 0x0004,
  0x0317, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0x1C45, /*2485*/
  0x0020, 0x0004, 0x0318, 0x0020, 0x0004, 0x0004,
  0x0317, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0x1C46, /*2486*/
  0x0020, 0x0004, 0x0318, 0x0020, 0x0004, 0x0004,
  0x0317, 0x0020, 0x0004, 0x1C3F, 0x0020, 0x0004, 0x1C3D, /*2487*/
  0x0020, 0x0004, 0x0318, 0x0020, 0x0004, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x0277, 0x0020, 0x0004, 0x0000, /*2488*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0004, 0x0277, 0x0020, 0x0004, 0x0000, /*2489*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0004, 0x0277, 0x0020, 0x0004, 0x0000, /*248A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0004, 0x0277, 0x0020, 0x0004, 0x0000, /*248B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C42, 0x0020, 0x0004, 0x0277, 0x0020, 0x0004, 0x0000, /*248C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C43, 0x0020, 0x0004, 0x0277, 0x0020, 0x0004, 0x0000, /*248D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C44, 0x0020, 0x0004, 0x0277, 0x0020, 0x0004, 0x0000, /*248E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C45, 0x0020, 0x0004, 0x0277, 0x0020, 0x0004, 0x0000, /*248F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C46, 0x0020, 0x0004, 0x0277, 0x0020, 0x0004, 0x0000, /*2490*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0004, 0x1C3D, 0x0020, 0x0004, 0x0277, /*2491*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0x0277, /*2492*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0x1C3F, 0x0020, 0x0004, 0x0277, /*2493*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0x1C40, 0x0020, 0x0004, 0x0277, /*2494*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0x1C41, 0x0020, 0x0004, 0x0277, /*2495*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0x1C42, 0x0020, 0x0004, 0x0277, /*2496*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0x1C43, 0x0020, 0x0004, 0x0277, /*2497*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0x1C44, 0x0020, 0x0004, 0x0277, /*2498*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0x1C45, 0x0020, 0x0004, 0x0277, /*2499*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0x1C46, 0x0020, 0x0004, 0x0277, /*249A*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3F, 0x0020, 0x0004, 0x1C3D, 0x0020, 0x0004, 0x0277, /*249B*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C47, 0x0020, 0x0004, 0x0318, /*249C*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C60, 0x0020, 0x0004, 0x0318, /*249D*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C7A, 0x0020, 0x0004, 0x0318, /*249E*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C8F, 0x0020, 0x0004, 0x0318, /*249F*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1CAA, 0x0020, 0x0004, 0x0318, /*24A0*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1CE5, 0x0020, 0x0004, 0x0318, /*24A1*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1CF4, 0x0020, 0x0004, 0x0318, /*24A2*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1D18, 0x0020, 0x0004, 0x0318, /*24A3*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1D32, 0x0020, 0x0004, 0x0318, /*24A4*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1D4C, 0x0020, 0x0004, 0x0318, /*24A5*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1D65, 0x0020, 0x0004, 0x0318, /*24A6*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1D77, 0x0020, 0x0004, 0x0318, /*24A7*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1DAA, 0x0020, 0x0004, 0x0318, /*24A8*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1DB9, 0x0020, 0x0004, 0x0318, /*24A9*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1DDD, 0x0020, 0x0004, 0x0318, /*24AA*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1E0C, 0x0020, 0x0004, 0x0318, /*24AB*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1E21, 0x0020, 0x0004, 0x0318, /*24AC*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1E33, 0x0020, 0x0004, 0x0318, /*24AD*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1E71, 0x0020, 0x0004, 0x0318, /*24AE*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1E95, 0x0020, 0x0004, 0x0318, /*24AF*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1EB5, 0x0020, 0x0004, 0x0318, /*24B0*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1EE3, 0x0020, 0x0004, 0x0318, /*24B1*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1EF5, 0x0020, 0x0004, 0x0318, /*24B2*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1EFF, 0x0020, 0x0004, 0x0318, /*24B3*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1F0B, 0x0020, 0x0004, 0x0318, /*24B4*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1F21, 0x0020, 0x0004, 0x0318, /*24B5*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C47, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C60, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE5, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF4, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D4C, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D65, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0C, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E21, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF5, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFF, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F0B, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F21, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*24CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C60, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE5, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF4, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D4C, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D65, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0C, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E21, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF5, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFF, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F0B, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F21, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0006, 0x1C3E, 0x0020, 0x0006, 0x0000, /*24EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C3F, 0x0020, 0x0006, 0x0000, /*24EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C40, 0x0020, 0x0006, 0x0000, /*24ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C41, 0x0020, 0x0006, 0x0000, /*24EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C42, 0x0020, 0x0006, 0x0000, /*24EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C43, 0x0020, 0x0006, 0x0000, /*24F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C44, 0x0020, 0x0006, 0x0000, /*24F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C45, 0x0020, 0x0006, 0x0000, /*24F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C46, 0x0020, 0x0006, 0x0000, /*24F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, /*24F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3F, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C40, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C41, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C42, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C43, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C44, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C45, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C46, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3E, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, /*24FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*24FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p025[]= {
  0x0818, 0x0020, 0x0002, 0x0001, /*2500*/
  0x0819, 0x0020, 0x0002, 0x0001, /*2501*/
  0x081A, 0x0020, 0x0002, 0x0001, /*2502*/
  0x081B, 0x0020, 0x0002, 0x0001, /*2503*/
  0x081C, 0x0020, 0x0002, 0x0001, /*2504*/
  0x081D, 0x0020, 0x0002, 0x0001, /*2505*/
  0x081E, 0x0020, 0x0002, 0x0001, /*2506*/
  0x081F, 0x0020, 0x0002, 0x0001, /*2507*/
  0x0820, 0x0020, 0x0002, 0x0001, /*2508*/
  0x0821, 0x0020, 0x0002, 0x0001, /*2509*/
  0x0822, 0x0020, 0x0002, 0x0001, /*250A*/
  0x0823, 0x0020, 0x0002, 0x0001, /*250B*/
  0x0824, 0x0020, 0x0002, 0x0001, /*250C*/
  0x0825, 0x0020, 0x0002, 0x0001, /*250D*/
  0x0826, 0x0020, 0x0002, 0x0001, /*250E*/
  0x0827, 0x0020, 0x0002, 0x0001, /*250F*/
  0x0828, 0x0020, 0x0002, 0x0001, /*2510*/
  0x0829, 0x0020, 0x0002, 0x0001, /*2511*/
  0x082A, 0x0020, 0x0002, 0x0001, /*2512*/
  0x082B, 0x0020, 0x0002, 0x0001, /*2513*/
  0x082C, 0x0020, 0x0002, 0x0001, /*2514*/
  0x082D, 0x0020, 0x0002, 0x0001, /*2515*/
  0x082E, 0x0020, 0x0002, 0x0001, /*2516*/
  0x082F, 0x0020, 0x0002, 0x0001, /*2517*/
  0x0830, 0x0020, 0x0002, 0x0001, /*2518*/
  0x0831, 0x0020, 0x0002, 0x0001, /*2519*/
  0x0832, 0x0020, 0x0002, 0x0001, /*251A*/
  0x0833, 0x0020, 0x0002, 0x0001, /*251B*/
  0x0834, 0x0020, 0x0002, 0x0001, /*251C*/
  0x0835, 0x0020, 0x0002, 0x0001, /*251D*/
  0x0836, 0x0020, 0x0002, 0x0001, /*251E*/
  0x0837, 0x0020, 0x0002, 0x0001, /*251F*/
  0x0838, 0x0020, 0x0002, 0x0001, /*2520*/
  0x0839, 0x0020, 0x0002, 0x0001, /*2521*/
  0x083A, 0x0020, 0x0002, 0x0001, /*2522*/
  0x083B, 0x0020, 0x0002, 0x0001, /*2523*/
  0x083C, 0x0020, 0x0002, 0x0001, /*2524*/
  0x083D, 0x0020, 0x0002, 0x0001, /*2525*/
  0x083E, 0x0020, 0x0002, 0x0001, /*2526*/
  0x083F, 0x0020, 0x0002, 0x0001, /*2527*/
  0x0840, 0x0020, 0x0002, 0x0001, /*2528*/
  0x0841, 0x0020, 0x0002, 0x0001, /*2529*/
  0x0842, 0x0020, 0x0002, 0x0001, /*252A*/
  0x0843, 0x0020, 0x0002, 0x0001, /*252B*/
  0x0844, 0x0020, 0x0002, 0x0001, /*252C*/
  0x0845, 0x0020, 0x0002, 0x0001, /*252D*/
  0x0846, 0x0020, 0x0002, 0x0001, /*252E*/
  0x0847, 0x0020, 0x0002, 0x0001, /*252F*/
  0x0848, 0x0020, 0x0002, 0x0001, /*2530*/
  0x0849, 0x0020, 0x0002, 0x0001, /*2531*/
  0x084A, 0x0020, 0x0002, 0x0001, /*2532*/
  0x084B, 0x0020, 0x0002, 0x0001, /*2533*/
  0x084C, 0x0020, 0x0002, 0x0001, /*2534*/
  0x084D, 0x0020, 0x0002, 0x0001, /*2535*/
  0x084E, 0x0020, 0x0002, 0x0001, /*2536*/
  0x084F, 0x0020, 0x0002, 0x0001, /*2537*/
  0x0850, 0x0020, 0x0002, 0x0001, /*2538*/
  0x0851, 0x0020, 0x0002, 0x0001, /*2539*/
  0x0852, 0x0020, 0x0002, 0x0001, /*253A*/
  0x0853, 0x0020, 0x0002, 0x0001, /*253B*/
  0x0854, 0x0020, 0x0002, 0x0001, /*253C*/
  0x0855, 0x0020, 0x0002, 0x0001, /*253D*/
  0x0856, 0x0020, 0x0002, 0x0001, /*253E*/
  0x0857, 0x0020, 0x0002, 0x0001, /*253F*/
  0x0858, 0x0020, 0x0002, 0x0001, /*2540*/
  0x0859, 0x0020, 0x0002, 0x0001, /*2541*/
  0x085A, 0x0020, 0x0002, 0x0001, /*2542*/
  0x085B, 0x0020, 0x0002, 0x0001, /*2543*/
  0x085C, 0x0020, 0x0002, 0x0001, /*2544*/
  0x085D, 0x0020, 0x0002, 0x0001, /*2545*/
  0x085E, 0x0020, 0x0002, 0x0001, /*2546*/
  0x085F, 0x0020, 0x0002, 0x0001, /*2547*/
  0x0860, 0x0020, 0x0002, 0x0001, /*2548*/
  0x0861, 0x0020, 0x0002, 0x0001, /*2549*/
  0x0862, 0x0020, 0x0002, 0x0001, /*254A*/
  0x0863, 0x0020, 0x0002, 0x0001, /*254B*/
  0x0864, 0x0020, 0x0002, 0x0001, /*254C*/
  0x0865, 0x0020, 0x0002, 0x0001, /*254D*/
  0x0866, 0x0020, 0x0002, 0x0001, /*254E*/
  0x0867, 0x0020, 0x0002, 0x0001, /*254F*/
  0x0868, 0x0020, 0x0002, 0x0001, /*2550*/
  0x0869, 0x0020, 0x0002, 0x0001, /*2551*/
  0x086A, 0x0020, 0x0002, 0x0001, /*2552*/
  0x086B, 0x0020, 0x0002, 0x0001, /*2553*/
  0x086C, 0x0020, 0x0002, 0x0001, /*2554*/
  0x086D, 0x0020, 0x0002, 0x0001, /*2555*/
  0x086E, 0x0020, 0x0002, 0x0001, /*2556*/
  0x086F, 0x0020, 0x0002, 0x0001, /*2557*/
  0x0870, 0x0020, 0x0002, 0x0001, /*2558*/
  0x0871, 0x0020, 0x0002, 0x0001, /*2559*/
  0x0872, 0x0020, 0x0002, 0x0001, /*255A*/
  0x0873, 0x0020, 0x0002, 0x0001, /*255B*/
  0x0874, 0x0020, 0x0002, 0x0001, /*255C*/
  0x0875, 0x0020, 0x0002, 0x0001, /*255D*/
  0x0876, 0x0020, 0x0002, 0x0001, /*255E*/
  0x0877, 0x0020, 0x0002, 0x0001, /*255F*/
  0x0878, 0x0020, 0x0002, 0x0001, /*2560*/
  0x0879, 0x0020, 0x0002, 0x0001, /*2561*/
  0x087A, 0x0020, 0x0002, 0x0001, /*2562*/
  0x087B, 0x0020, 0x0002, 0x0001, /*2563*/
  0x087C, 0x0020, 0x0002, 0x0001, /*2564*/
  0x087D, 0x0020, 0x0002, 0x0001, /*2565*/
  0x087E, 0x0020, 0x0002, 0x0001, /*2566*/
  0x087F, 0x0020, 0x0002, 0x0001, /*2567*/
  0x0880, 0x0020, 0x0002, 0x0001, /*2568*/
  0x0881, 0x0020, 0x0002, 0x0001, /*2569*/
  0x0882, 0x0020, 0x0002, 0x0001, /*256A*/
  0x0883, 0x0020, 0x0002, 0x0001, /*256B*/
  0x0884, 0x0020, 0x0002, 0x0001, /*256C*/
  0x0885, 0x0020, 0x0002, 0x0001, /*256D*/
  0x0886, 0x0020, 0x0002, 0x0001, /*256E*/
  0x0887, 0x0020, 0x0002, 0x0001, /*256F*/
  0x0888, 0x0020, 0x0002, 0x0001, /*2570*/
  0x0889, 0x0020, 0x0002, 0x0001, /*2571*/
  0x088A, 0x0020, 0x0002, 0x0001, /*2572*/
  0x088B, 0x0020, 0x0002, 0x0001, /*2573*/
  0x088C, 0x0020, 0x0002, 0x0001, /*2574*/
  0x088D, 0x0020, 0x0002, 0x0001, /*2575*/
  0x088E, 0x0020, 0x0002, 0x0001, /*2576*/
  0x088F, 0x0020, 0x0002, 0x0001, /*2577*/
  0x0890, 0x0020, 0x0002, 0x0001, /*2578*/
  0x0891, 0x0020, 0x0002, 0x0001, /*2579*/
  0x0892, 0x0020, 0x0002, 0x0001, /*257A*/
  0x0893, 0x0020, 0x0002, 0x0001, /*257B*/
  0x0894, 0x0020, 0x0002, 0x0001, /*257C*/
  0x0895, 0x0020, 0x0002, 0x0001, /*257D*/
  0x0896, 0x0020, 0x0002, 0x0001, /*257E*/
  0x0897, 0x0020, 0x0002, 0x0001, /*257F*/
  0x0898, 0x0020, 0x0002, 0x0001, /*2580*/
  0x0899, 0x0020, 0x0002, 0x0001, /*2581*/
  0x089A, 0x0020, 0x0002, 0x0001, /*2582*/
  0x089B, 0x0020, 0x0002, 0x0001, /*2583*/
  0x089C, 0x0020, 0x0002, 0x0001, /*2584*/
  0x089D, 0x0020, 0x0002, 0x0001, /*2585*/
  0x089E, 0x0020, 0x0002, 0x0001, /*2586*/
  0x089F, 0x0020, 0x0002, 0x0001, /*2587*/
  0x08A0, 0x0020, 0x0002, 0x0001, /*2588*/
  0x08A1, 0x0020, 0x0002, 0x0001, /*2589*/
  0x08A2, 0x0020, 0x0002, 0x0001, /*258A*/
  0x08A3, 0x0020, 0x0002, 0x0001, /*258B*/
  0x08A4, 0x0020, 0x0002, 0x0001, /*258C*/
  0x08A5, 0x0020, 0x0002, 0x0001, /*258D*/
  0x08A6, 0x0020, 0x0002, 0x0001, /*258E*/
  0x08A7, 0x0020, 0x0002, 0x0001, /*258F*/
  0x08A8, 0x0020, 0x0002, 0x0001, /*2590*/
  0x08A9, 0x0020, 0x0002, 0x0001, /*2591*/
  0x08AA, 0x0020, 0x0002, 0x0001, /*2592*/
  0x08AB, 0x0020, 0x0002, 0x0001, /*2593*/
  0x08AC, 0x0020, 0x0002, 0x0001, /*2594*/
  0x08AD, 0x0020, 0x0002, 0x0001, /*2595*/
  0x08AE, 0x0020, 0x0002, 0x0001, /*2596*/
  0x08AF, 0x0020, 0x0002, 0x0001, /*2597*/
  0x08B0, 0x0020, 0x0002, 0x0001, /*2598*/
  0x08B1, 0x0020, 0x0002, 0x0001, /*2599*/
  0x08B2, 0x0020, 0x0002, 0x0001, /*259A*/
  0x08B3, 0x0020, 0x0002, 0x0001, /*259B*/
  0x08B4, 0x0020, 0x0002, 0x0001, /*259C*/
  0x08B5, 0x0020, 0x0002, 0x0001, /*259D*/
  0x08B6, 0x0020, 0x0002, 0x0001, /*259E*/
  0x08B7, 0x0020, 0x0002, 0x0001, /*259F*/
  0x08B8, 0x0020, 0x0002, 0x0001, /*25A0*/
  0x08B9, 0x0020, 0x0002, 0x0001, /*25A1*/
  0x08BA, 0x0020, 0x0002, 0x0001, /*25A2*/
  0x08BB, 0x0020, 0x0002, 0x0001, /*25A3*/
  0x08BC, 0x0020, 0x0002, 0x0001, /*25A4*/
  0x08BD, 0x0020, 0x0002, 0x0001, /*25A5*/
  0x08BE, 0x0020, 0x0002, 0x0001, /*25A6*/
  0x08BF, 0x0020, 0x0002, 0x0001, /*25A7*/
  0x08C0, 0x0020, 0x0002, 0x0001, /*25A8*/
  0x08C1, 0x0020, 0x0002, 0x0001, /*25A9*/
  0x08C2, 0x0020, 0x0002, 0x0001, /*25AA*/
  0x08C3, 0x0020, 0x0002, 0x0001, /*25AB*/
  0x08C4, 0x0020, 0x0002, 0x0001, /*25AC*/
  0x08C5, 0x0020, 0x0002, 0x0001, /*25AD*/
  0x08C6, 0x0020, 0x0002, 0x0001, /*25AE*/
  0x08C7, 0x0020, 0x0002, 0x0001, /*25AF*/
  0x08C8, 0x0020, 0x0002, 0x0001, /*25B0*/
  0x08C9, 0x0020, 0x0002, 0x0001, /*25B1*/
  0x08CA, 0x0020, 0x0002, 0x0001, /*25B2*/
  0x08CB, 0x0020, 0x0002, 0x0001, /*25B3*/
  0x08CC, 0x0020, 0x0002, 0x0001, /*25B4*/
  0x08CD, 0x0020, 0x0002, 0x0001, /*25B5*/
  0x08CE, 0x0020, 0x0002, 0x0001, /*25B6*/
  0x08CF, 0x0020, 0x0002, 0x0001, /*25B7*/
  0x08D0, 0x0020, 0x0002, 0x0001, /*25B8*/
  0x08D1, 0x0020, 0x0002, 0x0001, /*25B9*/
  0x08D2, 0x0020, 0x0002, 0x0001, /*25BA*/
  0x08D3, 0x0020, 0x0002, 0x0001, /*25BB*/
  0x08D4, 0x0020, 0x0002, 0x0001, /*25BC*/
  0x08D5, 0x0020, 0x0002, 0x0001, /*25BD*/
  0x08D6, 0x0020, 0x0002, 0x0001, /*25BE*/
  0x08D7, 0x0020, 0x0002, 0x0001, /*25BF*/
  0x08D8, 0x0020, 0x0002, 0x0001, /*25C0*/
  0x08D9, 0x0020, 0x0002, 0x0001, /*25C1*/
  0x08DA, 0x0020, 0x0002, 0x0001, /*25C2*/
  0x08DB, 0x0020, 0x0002, 0x0001, /*25C3*/
  0x08DC, 0x0020, 0x0002, 0x0001, /*25C4*/
  0x08DD, 0x0020, 0x0002, 0x0001, /*25C5*/
  0x08DE, 0x0020, 0x0002, 0x0001, /*25C6*/
  0x08DF, 0x0020, 0x0002, 0x0001, /*25C7*/
  0x08E0, 0x0020, 0x0002, 0x0001, /*25C8*/
  0x08E1, 0x0020, 0x0002, 0x0001, /*25C9*/
  0x08E2, 0x0020, 0x0002, 0x0001, /*25CA*/
  0x08E3, 0x0020, 0x0002, 0x0001, /*25CB*/
  0x08E4, 0x0020, 0x0002, 0x0001, /*25CC*/
  0x08E5, 0x0020, 0x0002, 0x0001, /*25CD*/
  0x08E6, 0x0020, 0x0002, 0x0001, /*25CE*/
  0x08E7, 0x0020, 0x0002, 0x0001, /*25CF*/
  0x08E8, 0x0020, 0x0002, 0x0001, /*25D0*/
  0x08E9, 0x0020, 0x0002, 0x0001, /*25D1*/
  0x08EA, 0x0020, 0x0002, 0x0001, /*25D2*/
  0x08EB, 0x0020, 0x0002, 0x0001, /*25D3*/
  0x08EC, 0x0020, 0x0002, 0x0001, /*25D4*/
  0x08ED, 0x0020, 0x0002, 0x0001, /*25D5*/
  0x08EE, 0x0020, 0x0002, 0x0001, /*25D6*/
  0x08EF, 0x0020, 0x0002, 0x0001, /*25D7*/
  0x08F0, 0x0020, 0x0002, 0x0001, /*25D8*/
  0x08F1, 0x0020, 0x0002, 0x0001, /*25D9*/
  0x08F2, 0x0020, 0x0002, 0x0001, /*25DA*/
  0x08F3, 0x0020, 0x0002, 0x0001, /*25DB*/
  0x08F4, 0x0020, 0x0002, 0x0001, /*25DC*/
  0x08F5, 0x0020, 0x0002, 0x0001, /*25DD*/
  0x08F6, 0x0020, 0x0002, 0x0001, /*25DE*/
  0x08F7, 0x0020, 0x0002, 0x0001, /*25DF*/
  0x08F8, 0x0020, 0x0002, 0x0001, /*25E0*/
  0x08F9, 0x0020, 0x0002, 0x0001, /*25E1*/
  0x08FA, 0x0020, 0x0002, 0x0001, /*25E2*/
  0x08FB, 0x0020, 0x0002, 0x0001, /*25E3*/
  0x08FC, 0x0020, 0x0002, 0x0001, /*25E4*/
  0x08FD, 0x0020, 0x0002, 0x0001, /*25E5*/
  0x08FE, 0x0020, 0x0002, 0x0001, /*25E6*/
  0x08FF, 0x0020, 0x0002, 0x0001, /*25E7*/
  0x0900, 0x0020, 0x0002, 0x0001, /*25E8*/
  0x0901, 0x0020, 0x0002, 0x0001, /*25E9*/
  0x0902, 0x0020, 0x0002, 0x0001, /*25EA*/
  0x0903, 0x0020, 0x0002, 0x0001, /*25EB*/
  0x0904, 0x0020, 0x0002, 0x0001, /*25EC*/
  0x0905, 0x0020, 0x0002, 0x0001, /*25ED*/
  0x0906, 0x0020, 0x0002, 0x0001, /*25EE*/
  0x0907, 0x0020, 0x0002, 0x0001, /*25EF*/
  0x0908, 0x0020, 0x0002, 0x0001, /*25F0*/
  0x0909, 0x0020, 0x0002, 0x0001, /*25F1*/
  0x090A, 0x0020, 0x0002, 0x0001, /*25F2*/
  0x090B, 0x0020, 0x0002, 0x0001, /*25F3*/
  0x090C, 0x0020, 0x0002, 0x0001, /*25F4*/
  0x090D, 0x0020, 0x0002, 0x0001, /*25F5*/
  0x090E, 0x0020, 0x0002, 0x0001, /*25F6*/
  0x090F, 0x0020, 0x0002, 0x0001, /*25F7*/
  0x0910, 0x0020, 0x0002, 0x0001, /*25F8*/
  0x0911, 0x0020, 0x0002, 0x0001, /*25F9*/
  0x0912, 0x0020, 0x0002, 0x0001, /*25FA*/
  0x0913, 0x0020, 0x0002, 0x0001, /*25FB*/
  0x0914, 0x0020, 0x0002, 0x0001, /*25FC*/
  0x0915, 0x0020, 0x0002, 0x0001, /*25FD*/
  0x0916, 0x0020, 0x0002, 0x0001, /*25FE*/
  0x0917, 0x0020, 0x0002, 0x0001  /*25FF*/
};

uint16 uca900_p026[]= {
  0x0918, 0x0020, 0x0002, 0x0001, /*2600*/
  0x0919, 0x0020, 0x0002, 0x0001, /*2601*/
  0x091A, 0x0020, 0x0002, 0x0001, /*2602*/
  0x091B, 0x0020, 0x0002, 0x0001, /*2603*/
  0x091C, 0x0020, 0x0002, 0x0001, /*2604*/
  0x091D, 0x0020, 0x0002, 0x0001, /*2605*/
  0x091E, 0x0020, 0x0002, 0x0001, /*2606*/
  0x091F, 0x0020, 0x0002, 0x0001, /*2607*/
  0x0920, 0x0020, 0x0002, 0x0001, /*2608*/
  0x0921, 0x0020, 0x0002, 0x0001, /*2609*/
  0x0922, 0x0020, 0x0002, 0x0001, /*260A*/
  0x0923, 0x0020, 0x0002, 0x0001, /*260B*/
  0x0924, 0x0020, 0x0002, 0x0001, /*260C*/
  0x0925, 0x0020, 0x0002, 0x0001, /*260D*/
  0x0926, 0x0020, 0x0002, 0x0001, /*260E*/
  0x0927, 0x0020, 0x0002, 0x0001, /*260F*/
  0x0928, 0x0020, 0x0002, 0x0001, /*2610*/
  0x0929, 0x0020, 0x0002, 0x0001, /*2611*/
  0x092A, 0x0020, 0x0002, 0x0001, /*2612*/
  0x092B, 0x0020, 0x0002, 0x0001, /*2613*/
  0x092C, 0x0020, 0x0002, 0x0001, /*2614*/
  0x092D, 0x0020, 0x0002, 0x0001, /*2615*/
  0x092E, 0x0020, 0x0002, 0x0001, /*2616*/
  0x092F, 0x0020, 0x0002, 0x0001, /*2617*/
  0x0930, 0x0020, 0x0002, 0x0001, /*2618*/
  0x0931, 0x0020, 0x0002, 0x0001, /*2619*/
  0x0932, 0x0020, 0x0002, 0x0001, /*261A*/
  0x0933, 0x0020, 0x0002, 0x0001, /*261B*/
  0x0934, 0x0020, 0x0002, 0x0001, /*261C*/
  0x0935, 0x0020, 0x0002, 0x0001, /*261D*/
  0x0936, 0x0020, 0x0002, 0x0001, /*261E*/
  0x0937, 0x0020, 0x0002, 0x0001, /*261F*/
  0x0938, 0x0020, 0x0002, 0x0001, /*2620*/
  0x0939, 0x0020, 0x0002, 0x0001, /*2621*/
  0x093A, 0x0020, 0x0002, 0x0001, /*2622*/
  0x093B, 0x0020, 0x0002, 0x0001, /*2623*/
  0x093C, 0x0020, 0x0002, 0x0001, /*2624*/
  0x093D, 0x0020, 0x0002, 0x0001, /*2625*/
  0x093E, 0x0020, 0x0002, 0x0001, /*2626*/
  0x093F, 0x0020, 0x0002, 0x0001, /*2627*/
  0x0940, 0x0020, 0x0002, 0x0001, /*2628*/
  0x0941, 0x0020, 0x0002, 0x0001, /*2629*/
  0x0942, 0x0020, 0x0002, 0x0001, /*262A*/
  0x0943, 0x0020, 0x0002, 0x0001, /*262B*/
  0x0944, 0x0020, 0x0002, 0x0001, /*262C*/
  0x0945, 0x0020, 0x0002, 0x0001, /*262D*/
  0x0946, 0x0020, 0x0002, 0x0001, /*262E*/
  0x0947, 0x0020, 0x0002, 0x0001, /*262F*/
  0x0EA2, 0x0020, 0x0002, 0x0001, /*2630*/
  0x0EA3, 0x0020, 0x0002, 0x0001, /*2631*/
  0x0EA4, 0x0020, 0x0002, 0x0001, /*2632*/
  0x0EA5, 0x0020, 0x0002, 0x0001, /*2633*/
  0x0EA6, 0x0020, 0x0002, 0x0001, /*2634*/
  0x0EA7, 0x0020, 0x0002, 0x0001, /*2635*/
  0x0EA8, 0x0020, 0x0002, 0x0001, /*2636*/
  0x0EA9, 0x0020, 0x0002, 0x0001, /*2637*/
  0x0948, 0x0020, 0x0002, 0x0001, /*2638*/
  0x0949, 0x0020, 0x0002, 0x0001, /*2639*/
  0x094A, 0x0020, 0x0002, 0x0001, /*263A*/
  0x094B, 0x0020, 0x0002, 0x0001, /*263B*/
  0x094C, 0x0020, 0x0002, 0x0001, /*263C*/
  0x094D, 0x0020, 0x0002, 0x0001, /*263D*/
  0x094E, 0x0020, 0x0002, 0x0001, /*263E*/
  0x094F, 0x0020, 0x0002, 0x0001, /*263F*/
  0x0950, 0x0020, 0x0002, 0x0001, /*2640*/
  0x0951, 0x0020, 0x0002, 0x0001, /*2641*/
  0x0952, 0x0020, 0x0002, 0x0001, /*2642*/
  0x0953, 0x0020, 0x0002, 0x0001, /*2643*/
  0x0954, 0x0020, 0x0002, 0x0001, /*2644*/
  0x0955, 0x0020, 0x0002, 0x0001, /*2645*/
  0x0956, 0x0020, 0x0002, 0x0001, /*2646*/
  0x0957, 0x0020, 0x0002, 0x0001, /*2647*/
  0x0958, 0x0020, 0x0002, 0x0001, /*2648*/
  0x0959, 0x0020, 0x0002, 0x0001, /*2649*/
  0x095A, 0x0020, 0x0002, 0x0001, /*264A*/
  0x095B, 0x0020, 0x0002, 0x0001, /*264B*/
  0x095C, 0x0020, 0x0002, 0x0001, /*264C*/
  0x095D, 0x0020, 0x0002, 0x0001, /*264D*/
  0x095E, 0x0020, 0x0002, 0x0001, /*264E*/
  0x095F, 0x0020, 0x0002, 0x0001, /*264F*/
  0x0960, 0x0020, 0x0002, 0x0001, /*2650*/
  0x0961, 0x0020, 0x0002, 0x0001, /*2651*/
  0x0962, 0x0020, 0x0002, 0x0001, /*2652*/
  0x0963, 0x0020, 0x0002, 0x0001, /*2653*/
  0x0964, 0x0020, 0x0002, 0x0001, /*2654*/
  0x0965, 0x0020, 0x0002, 0x0001, /*2655*/
  0x0966, 0x0020, 0x0002, 0x0001, /*2656*/
  0x0967, 0x0020, 0x0002, 0x0001, /*2657*/
  0x0968, 0x0020, 0x0002, 0x0001, /*2658*/
  0x0969, 0x0020, 0x0002, 0x0001, /*2659*/
  0x096A, 0x0020, 0x0002, 0x0001, /*265A*/
  0x096B, 0x0020, 0x0002, 0x0001, /*265B*/
  0x096C, 0x0020, 0x0002, 0x0001, /*265C*/
  0x096D, 0x0020, 0x0002, 0x0001, /*265D*/
  0x096E, 0x0020, 0x0002, 0x0001, /*265E*/
  0x096F, 0x0020, 0x0002, 0x0001, /*265F*/
  0x0970, 0x0020, 0x0002, 0x0001, /*2660*/
  0x0971, 0x0020, 0x0002, 0x0001, /*2661*/
  0x0972, 0x0020, 0x0002, 0x0001, /*2662*/
  0x0973, 0x0020, 0x0002, 0x0001, /*2663*/
  0x0974, 0x0020, 0x0002, 0x0001, /*2664*/
  0x0975, 0x0020, 0x0002, 0x0001, /*2665*/
  0x0976, 0x0020, 0x0002, 0x0001, /*2666*/
  0x0977, 0x0020, 0x0002, 0x0001, /*2667*/
  0x0978, 0x0020, 0x0002, 0x0001, /*2668*/
  0x0979, 0x0020, 0x0002, 0x0001, /*2669*/
  0x097A, 0x0020, 0x0002, 0x0001, /*266A*/
  0x097B, 0x0020, 0x0002, 0x0001, /*266B*/
  0x097C, 0x0020, 0x0002, 0x0001, /*266C*/
  0x10F3, 0x0020, 0x0002, 0x0001, /*266D*/
  0x10F4, 0x0020, 0x0002, 0x0001, /*266E*/
  0x10F5, 0x0020, 0x0002, 0x0001, /*266F*/
  0x097D, 0x0020, 0x0002, 0x0001, /*2670*/
  0x097E, 0x0020, 0x0002, 0x0001, /*2671*/
  0x097F, 0x0020, 0x0002, 0x0001, /*2672*/
  0x0980, 0x0020, 0x0002, 0x0001, /*2673*/
  0x0981, 0x0020, 0x0002, 0x0001, /*2674*/
  0x0982, 0x0020, 0x0002, 0x0001, /*2675*/
  0x0983, 0x0020, 0x0002, 0x0001, /*2676*/
  0x0984, 0x0020, 0x0002, 0x0001, /*2677*/
  0x0985, 0x0020, 0x0002, 0x0001, /*2678*/
  0x0986, 0x0020, 0x0002, 0x0001, /*2679*/
  0x0987, 0x0020, 0x0002, 0x0001, /*267A*/
  0x0988, 0x0020, 0x0002, 0x0001, /*267B*/
  0x0989, 0x0020, 0x0002, 0x0001, /*267C*/
  0x098A, 0x0020, 0x0002, 0x0001, /*267D*/
  0x098B, 0x0020, 0x0002, 0x0001, /*267E*/
  0x098C, 0x0020, 0x0002, 0x0001, /*267F*/
  0x098D, 0x0020, 0x0002, 0x0001, /*2680*/
  0x098E, 0x0020, 0x0002, 0x0001, /*2681*/
  0x098F, 0x0020, 0x0002, 0x0001, /*2682*/
  0x0990, 0x0020, 0x0002, 0x0001, /*2683*/
  0x0991, 0x0020, 0x0002, 0x0001, /*2684*/
  0x0992, 0x0020, 0x0002, 0x0001, /*2685*/
  0x0993, 0x0020, 0x0002, 0x0001, /*2686*/
  0x0994, 0x0020, 0x0002, 0x0001, /*2687*/
  0x0995, 0x0020, 0x0002, 0x0001, /*2688*/
  0x0996, 0x0020, 0x0002, 0x0001, /*2689*/
  0x0E9C, 0x0020, 0x0002, 0x0001, /*268A*/
  0x0E9D, 0x0020, 0x0002, 0x0001, /*268B*/
  0x0E9E, 0x0020, 0x0002, 0x0001, /*268C*/
  0x0E9F, 0x0020, 0x0002, 0x0001, /*268D*/
  0x0EA0, 0x0020, 0x0002, 0x0001, /*268E*/
  0x0EA1, 0x0020, 0x0002, 0x0001, /*268F*/
  0x0997, 0x0020, 0x0002, 0x0001, /*2690*/
  0x0998, 0x0020, 0x0002, 0x0001, /*2691*/
  0x0999, 0x0020, 0x0002, 0x0001, /*2692*/
  0x099A, 0x0020, 0x0002, 0x0001, /*2693*/
  0x099B, 0x0020, 0x0002, 0x0001, /*2694*/
  0x099C, 0x0020, 0x0002, 0x0001, /*2695*/
  0x099D, 0x0020, 0x0002, 0x0001, /*2696*/
  0x099E, 0x0020, 0x0002, 0x0001, /*2697*/
  0x099F, 0x0020, 0x0002, 0x0001, /*2698*/
  0x09A0, 0x0020, 0x0002, 0x0001, /*2699*/
  0x09A1, 0x0020, 0x0002, 0x0001, /*269A*/
  0x09A2, 0x0020, 0x0002, 0x0001, /*269B*/
  0x09A3, 0x0020, 0x0002, 0x0001, /*269C*/
  0x09A4, 0x0020, 0x0002, 0x0001, /*269D*/
  0x09A5, 0x0020, 0x0002, 0x0001, /*269E*/
  0x09A6, 0x0020, 0x0002, 0x0001, /*269F*/
  0x09A7, 0x0020, 0x0002, 0x0001, /*26A0*/
  0x09A8, 0x0020, 0x0002, 0x0001, /*26A1*/
  0x09A9, 0x0020, 0x0002, 0x0001, /*26A2*/
  0x09AA, 0x0020, 0x0002, 0x0001, /*26A3*/
  0x09AB, 0x0020, 0x0002, 0x0001, /*26A4*/
  0x09AC, 0x0020, 0x0002, 0x0001, /*26A5*/
  0x09AD, 0x0020, 0x0002, 0x0001, /*26A6*/
  0x09AE, 0x0020, 0x0002, 0x0001, /*26A7*/
  0x09AF, 0x0020, 0x0002, 0x0001, /*26A8*/
  0x09B0, 0x0020, 0x0002, 0x0001, /*26A9*/
  0x09B1, 0x0020, 0x0002, 0x0001, /*26AA*/
  0x09B2, 0x0020, 0x0002, 0x0001, /*26AB*/
  0x09B3, 0x0020, 0x0002, 0x0001, /*26AC*/
  0x09B4, 0x0020, 0x0002, 0x0001, /*26AD*/
  0x09B5, 0x0020, 0x0002, 0x0001, /*26AE*/
  0x09B6, 0x0020, 0x0002, 0x0001, /*26AF*/
  0x09B7, 0x0020, 0x0002, 0x0001, /*26B0*/
  0x09B8, 0x0020, 0x0002, 0x0001, /*26B1*/
  0x09B9, 0x0020, 0x0002, 0x0001, /*26B2*/
  0x09BA, 0x0020, 0x0002, 0x0001, /*26B3*/
  0x09BB, 0x0020, 0x0002, 0x0001, /*26B4*/
  0x09BC, 0x0020, 0x0002, 0x0001, /*26B5*/
  0x09BD, 0x0020, 0x0002, 0x0001, /*26B6*/
  0x09BE, 0x0020, 0x0002, 0x0001, /*26B7*/
  0x09BF, 0x0020, 0x0002, 0x0001, /*26B8*/
  0x09C0, 0x0020, 0x0002, 0x0001, /*26B9*/
  0x09C1, 0x0020, 0x0002, 0x0001, /*26BA*/
  0x09C2, 0x0020, 0x0002, 0x0001, /*26BB*/
  0x09C3, 0x0020, 0x0002, 0x0001, /*26BC*/
  0x09C4, 0x0020, 0x0002, 0x0001, /*26BD*/
  0x09C5, 0x0020, 0x0002, 0x0001, /*26BE*/
  0x09C6, 0x0020, 0x0002, 0x0001, /*26BF*/
  0x09C7, 0x0020, 0x0002, 0x0001, /*26C0*/
  0x09C8, 0x0020, 0x0002, 0x0001, /*26C1*/
  0x09C9, 0x0020, 0x0002, 0x0001, /*26C2*/
  0x09CA, 0x0020, 0x0002, 0x0001, /*26C3*/
  0x09CB, 0x0020, 0x0002, 0x0001, /*26C4*/
  0x09CC, 0x0020, 0x0002, 0x0001, /*26C5*/
  0x09CD, 0x0020, 0x0002, 0x0001, /*26C6*/
  0x09CE, 0x0020, 0x0002, 0x0001, /*26C7*/
  0x09CF, 0x0020, 0x0002, 0x0001, /*26C8*/
  0x09D0, 0x0020, 0x0002, 0x0001, /*26C9*/
  0x09D1, 0x0020, 0x0002, 0x0001, /*26CA*/
  0x09D2, 0x0020, 0x0002, 0x0001, /*26CB*/
  0x09D3, 0x0020, 0x0002, 0x0001, /*26CC*/
  0x09D4, 0x0020, 0x0002, 0x0001, /*26CD*/
  0x09D5, 0x0020, 0x0002, 0x0001, /*26CE*/
  0x09D6, 0x0020, 0x0002, 0x0001, /*26CF*/
  0x09D7, 0x0020, 0x0002, 0x0001, /*26D0*/
  0x09D8, 0x0020, 0x0002, 0x0001, /*26D1*/
  0x09D9, 0x0020, 0x0002, 0x0001, /*26D2*/
  0x09DA, 0x0020, 0x0002, 0x0001, /*26D3*/
  0x09DB, 0x0020, 0x0002, 0x0001, /*26D4*/
  0x09DC, 0x0020, 0x0002, 0x0001, /*26D5*/
  0x09DD, 0x0020, 0x0002, 0x0001, /*26D6*/
  0x09DE, 0x0020, 0x0002, 0x0001, /*26D7*/
  0x09DF, 0x0020, 0x0002, 0x0001, /*26D8*/
  0x09E0, 0x0020, 0x0002, 0x0001, /*26D9*/
  0x09E1, 0x0020, 0x0002, 0x0001, /*26DA*/
  0x09E2, 0x0020, 0x0002, 0x0001, /*26DB*/
  0x09E3, 0x0020, 0x0002, 0x0001, /*26DC*/
  0x09E4, 0x0020, 0x0002, 0x0001, /*26DD*/
  0x09E5, 0x0020, 0x0002, 0x0001, /*26DE*/
  0x09E6, 0x0020, 0x0002, 0x0001, /*26DF*/
  0x09E7, 0x0020, 0x0002, 0x0001, /*26E0*/
  0x09E8, 0x0020, 0x0002, 0x0001, /*26E1*/
  0x09E9, 0x0020, 0x0002, 0x0001, /*26E2*/
  0x09EA, 0x0020, 0x0002, 0x0001, /*26E3*/
  0x09EB, 0x0020, 0x0002, 0x0001, /*26E4*/
  0x09EC, 0x0020, 0x0002, 0x0001, /*26E5*/
  0x09ED, 0x0020, 0x0002, 0x0001, /*26E6*/
  0x09EE, 0x0020, 0x0002, 0x0001, /*26E7*/
  0x09EF, 0x0020, 0x0002, 0x0001, /*26E8*/
  0x09F0, 0x0020, 0x0002, 0x0001, /*26E9*/
  0x09F1, 0x0020, 0x0002, 0x0001, /*26EA*/
  0x09F2, 0x0020, 0x0002, 0x0001, /*26EB*/
  0x09F3, 0x0020, 0x0002, 0x0001, /*26EC*/
  0x09F4, 0x0020, 0x0002, 0x0001, /*26ED*/
  0x09F5, 0x0020, 0x0002, 0x0001, /*26EE*/
  0x09F6, 0x0020, 0x0002, 0x0001, /*26EF*/
  0x09F7, 0x0020, 0x0002, 0x0001, /*26F0*/
  0x09F8, 0x0020, 0x0002, 0x0001, /*26F1*/
  0x09F9, 0x0020, 0x0002, 0x0001, /*26F2*/
  0x09FA, 0x0020, 0x0002, 0x0001, /*26F3*/
  0x09FB, 0x0020, 0x0002, 0x0001, /*26F4*/
  0x09FC, 0x0020, 0x0002, 0x0001, /*26F5*/
  0x09FD, 0x0020, 0x0002, 0x0001, /*26F6*/
  0x09FE, 0x0020, 0x0002, 0x0001, /*26F7*/
  0x09FF, 0x0020, 0x0002, 0x0001, /*26F8*/
  0x0A00, 0x0020, 0x0002, 0x0001, /*26F9*/
  0x0A01, 0x0020, 0x0002, 0x0001, /*26FA*/
  0x0A02, 0x0020, 0x0002, 0x0001, /*26FB*/
  0x0A03, 0x0020, 0x0002, 0x0001, /*26FC*/
  0x0A04, 0x0020, 0x0002, 0x0001, /*26FD*/
  0x0A05, 0x0020, 0x0002, 0x0001, /*26FE*/
  0x0A06, 0x0020, 0x0002, 0x0001  /*26FF*/
};

uint16 uca900_p027[]= {
  0x0A21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2700*/
  0x0A22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2701*/
  0x0A23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2702*/
  0x0A24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2703*/
  0x0A25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2704*/
  0x0A26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2705*/
  0x0A27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2706*/
  0x0A28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2707*/
  0x0A29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2708*/
  0x0A2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2709*/
  0x0A2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*270A*/
  0x0A2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*270B*/
  0x0A2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*270C*/
  0x0A2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*270D*/
  0x0A2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*270E*/
  0x0A30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*270F*/
  0x0A31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2710*/
  0x0A32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2711*/
  0x0A33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2712*/
  0x0A34, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2713*/
  0x0A35, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2714*/
  0x0A36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2715*/
  0x0A37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2716*/
  0x0A38, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2717*/
  0x0A39, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2718*/
  0x0A3A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2719*/
  0x0A3B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*271A*/
  0x0A3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*271B*/
  0x0A3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*271C*/
  0x0A3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*271D*/
  0x0A3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*271E*/
  0x0A40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*271F*/
  0x0A41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2720*/
  0x0A42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2721*/
  0x0A43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2722*/
  0x0A44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2723*/
  0x0A45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2724*/
  0x0A46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2725*/
  0x0A47, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2726*/
  0x0A48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2727*/
  0x0A49, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2728*/
  0x0A4A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2729*/
  0x0A4B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*272A*/
  0x0A4C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*272B*/
  0x0A4D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*272C*/
  0x0A4E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*272D*/
  0x0A4F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*272E*/
  0x0A50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*272F*/
  0x0A51, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2730*/
  0x0A52, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2731*/
  0x0A53, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2732*/
  0x0A54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2733*/
  0x0A55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2734*/
  0x0A56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2735*/
  0x0A57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2736*/
  0x0A58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2737*/
  0x0A59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2738*/
  0x0A5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2739*/
  0x0A5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*273A*/
  0x0A5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*273B*/
  0x0A5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*273C*/
  0x0A5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*273D*/
  0x0A5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*273E*/
  0x0A60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*273F*/
  0x0A61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2740*/
  0x0A62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2741*/
  0x0A63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2742*/
  0x0A64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2743*/
  0x0A65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2744*/
  0x0A66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2745*/
  0x0A67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2746*/
  0x0A68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2747*/
  0x0A69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2748*/
  0x0A6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2749*/
  0x0A6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*274A*/
  0x0A6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*274B*/
  0x0A6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*274C*/
  0x0A6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*274D*/
  0x0A6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*274E*/
  0x0A70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*274F*/
  0x0A71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2750*/
  0x0A72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2751*/
  0x0A73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2752*/
  0x0A74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2753*/
  0x0A75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2754*/
  0x0A76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2755*/
  0x0A77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2756*/
  0x0A78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2757*/
  0x0A79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2758*/
  0x0A7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2759*/
  0x0A7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*275A*/
  0x0A7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*275B*/
  0x0A7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*275C*/
  0x0A7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*275D*/
  0x0A7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*275E*/
  0x0A80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*275F*/
  0x0A81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2760*/
  0x0A82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2761*/
  0x0A83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2762*/
  0x0A84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2763*/
  0x0A85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2764*/
  0x0A86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2765*/
  0x0A87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2766*/
  0x0A88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2767*/
  0x034D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2768*/
  0x034E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2769*/
  0x034F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*276A*/
  0x0350, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*276B*/
  0x0351, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*276C*/
  0x0352, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*276D*/
  0x0353, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*276E*/
  0x0354, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*276F*/
  0x0355, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2770*/
  0x0356, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2771*/
  0x0357, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2772*/
  0x0358, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2773*/
  0x0359, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2774*/
  0x035A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2775*/
  0x1C3E, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2776*/
  0x1C3F, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2777*/
  0x1C40, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2778*/
  0x1C41, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2779*/
  0x1C42, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*277A*/
  0x1C43, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*277B*/
  0x1C44, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*277C*/
  0x1C45, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*277D*/
  0x1C46, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*277E*/
  0x1C3E, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0002, /*277F*/
  0x1C3E, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2780*/
  0x1C3F, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2781*/
  0x1C40, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2782*/
  0x1C41, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2783*/
  0x1C42, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2784*/
  0x1C43, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2785*/
  0x1C44, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2786*/
  0x1C45, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2787*/
  0x1C46, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2788*/
  0x1C3E, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0002, /*2789*/
  0x1C3E, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*278A*/
  0x1C3F, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*278B*/
  0x1C40, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*278C*/
  0x1C41, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*278D*/
  0x1C42, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*278E*/
  0x1C43, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*278F*/
  0x1C44, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2790*/
  0x1C45, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2791*/
  0x1C46, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0001, /*2792*/
  0x1C3E, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0002, /*2793*/
  0x0A89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2794*/
  0x0A8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2795*/
  0x0A8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2796*/
  0x0A8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2797*/
  0x0A8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2798*/
  0x0A8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2799*/
  0x0A8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*279A*/
  0x0A90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*279B*/
  0x0A91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*279C*/
  0x0A92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*279D*/
  0x0A93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*279E*/
  0x0A94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*279F*/
  0x0A95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27A0*/
  0x0A96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27A1*/
  0x0A97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27A2*/
  0x0A98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27A3*/
  0x0A99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27A4*/
  0x0A9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27A5*/
  0x0A9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27A6*/
  0x0A9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27A7*/
  0x0A9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27A8*/
  0x0A9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27A9*/
  0x0A9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27AA*/
  0x0AA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27AB*/
  0x0AA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27AC*/
  0x0AA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27AD*/
  0x0AA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27AE*/
  0x0AA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27AF*/
  0x0AA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27B0*/
  0x0AA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27B1*/
  0x0AA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27B2*/
  0x0AA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27B3*/
  0x0AA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27B4*/
  0x0AAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27B5*/
  0x0AAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27B6*/
  0x0AAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27B7*/
  0x0AAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27B8*/
  0x0AAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27B9*/
  0x0AAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27BA*/
  0x0AB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27BB*/
  0x0AB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27BC*/
  0x0AB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27BD*/
  0x0AB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27BE*/
  0x0AB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27BF*/
  0x0AB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27C0*/
  0x0AB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27C1*/
  0x0AB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27C2*/
  0x0AB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27C3*/
  0x0AB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27C4*/
  0x0341, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27C5*/
  0x0342, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27C6*/
  0x0ABA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27C7*/
  0x0ABB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27C8*/
  0x0ABC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27C9*/
  0x0ABD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27CA*/
  0x0ABE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27CB*/
  0x0ABF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27CC*/
  0x0AC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27CD*/
  0x0AC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27CE*/
  0x0AC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27CF*/
  0x0AC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27D0*/
  0x0AC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27D1*/
  0x0AC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27D2*/
  0x0AC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27D3*/
  0x0AC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27D4*/
  0x0AC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27D5*/
  0x0AC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27D6*/
  0x0ACA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27D7*/
  0x0ACB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27D8*/
  0x0ACC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27D9*/
  0x0ACD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27DA*/
  0x0ACE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27DB*/
  0x0ACF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27DC*/
  0x0AD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27DD*/
  0x0AD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27DE*/
  0x0AD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27DF*/
  0x0AD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27E0*/
  0x0AD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27E1*/
  0x0AD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27E2*/
  0x0AD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27E3*/
  0x0AD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27E4*/
  0x0AD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27E5*/
  0x0343, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27E6*/
  0x0344, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27E7*/
  0x0345, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27E8*/
  0x0346, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27E9*/
  0x0347, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27EA*/
  0x0348, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27EB*/
  0x0349, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27EC*/
  0x034A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27ED*/
  0x034B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27EE*/
  0x034C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27EF*/
  0x0AD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27F0*/
  0x0ADA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27F1*/
  0x0ADB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27F2*/
  0x0ADC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27F3*/
  0x0ADD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27F4*/
  0x0ADE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27F5*/
  0x0ADF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27F6*/
  0x0AE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27F7*/
  0x0AE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27F8*/
  0x0AE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27F9*/
  0x0AE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27FA*/
  0x0AE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27FB*/
  0x0AE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27FC*/
  0x0AE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27FD*/
  0x0AE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*27FE*/
  0x0AE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*27FF*/
};

uint16 uca900_p028[]= {
  0x0D9C, 0x0020, 0x0002, 0x0001, /*2800*/
  0x0D9D, 0x0020, 0x0002, 0x0001, /*2801*/
  0x0D9E, 0x0020, 0x0002, 0x0001, /*2802*/
  0x0D9F, 0x0020, 0x0002, 0x0001, /*2803*/
  0x0DA0, 0x0020, 0x0002, 0x0001, /*2804*/
  0x0DA1, 0x0020, 0x0002, 0x0001, /*2805*/
  0x0DA2, 0x0020, 0x0002, 0x0001, /*2806*/
  0x0DA3, 0x0020, 0x0002, 0x0001, /*2807*/
  0x0DA4, 0x0020, 0x0002, 0x0001, /*2808*/
  0x0DA5, 0x0020, 0x0002, 0x0001, /*2809*/
  0x0DA6, 0x0020, 0x0002, 0x0001, /*280A*/
  0x0DA7, 0x0020, 0x0002, 0x0001, /*280B*/
  0x0DA8, 0x0020, 0x0002, 0x0001, /*280C*/
  0x0DA9, 0x0020, 0x0002, 0x0001, /*280D*/
  0x0DAA, 0x0020, 0x0002, 0x0001, /*280E*/
  0x0DAB, 0x0020, 0x0002, 0x0001, /*280F*/
  0x0DAC, 0x0020, 0x0002, 0x0001, /*2810*/
  0x0DAD, 0x0020, 0x0002, 0x0001, /*2811*/
  0x0DAE, 0x0020, 0x0002, 0x0001, /*2812*/
  0x0DAF, 0x0020, 0x0002, 0x0001, /*2813*/
  0x0DB0, 0x0020, 0x0002, 0x0001, /*2814*/
  0x0DB1, 0x0020, 0x0002, 0x0001, /*2815*/
  0x0DB2, 0x0020, 0x0002, 0x0001, /*2816*/
  0x0DB3, 0x0020, 0x0002, 0x0001, /*2817*/
  0x0DB4, 0x0020, 0x0002, 0x0001, /*2818*/
  0x0DB5, 0x0020, 0x0002, 0x0001, /*2819*/
  0x0DB6, 0x0020, 0x0002, 0x0001, /*281A*/
  0x0DB7, 0x0020, 0x0002, 0x0001, /*281B*/
  0x0DB8, 0x0020, 0x0002, 0x0001, /*281C*/
  0x0DB9, 0x0020, 0x0002, 0x0001, /*281D*/
  0x0DBA, 0x0020, 0x0002, 0x0001, /*281E*/
  0x0DBB, 0x0020, 0x0002, 0x0001, /*281F*/
  0x0DBC, 0x0020, 0x0002, 0x0001, /*2820*/
  0x0DBD, 0x0020, 0x0002, 0x0001, /*2821*/
  0x0DBE, 0x0020, 0x0002, 0x0001, /*2822*/
  0x0DBF, 0x0020, 0x0002, 0x0001, /*2823*/
  0x0DC0, 0x0020, 0x0002, 0x0001, /*2824*/
  0x0DC1, 0x0020, 0x0002, 0x0001, /*2825*/
  0x0DC2, 0x0020, 0x0002, 0x0001, /*2826*/
  0x0DC3, 0x0020, 0x0002, 0x0001, /*2827*/
  0x0DC4, 0x0020, 0x0002, 0x0001, /*2828*/
  0x0DC5, 0x0020, 0x0002, 0x0001, /*2829*/
  0x0DC6, 0x0020, 0x0002, 0x0001, /*282A*/
  0x0DC7, 0x0020, 0x0002, 0x0001, /*282B*/
  0x0DC8, 0x0020, 0x0002, 0x0001, /*282C*/
  0x0DC9, 0x0020, 0x0002, 0x0001, /*282D*/
  0x0DCA, 0x0020, 0x0002, 0x0001, /*282E*/
  0x0DCB, 0x0020, 0x0002, 0x0001, /*282F*/
  0x0DCC, 0x0020, 0x0002, 0x0001, /*2830*/
  0x0DCD, 0x0020, 0x0002, 0x0001, /*2831*/
  0x0DCE, 0x0020, 0x0002, 0x0001, /*2832*/
  0x0DCF, 0x0020, 0x0002, 0x0001, /*2833*/
  0x0DD0, 0x0020, 0x0002, 0x0001, /*2834*/
  0x0DD1, 0x0020, 0x0002, 0x0001, /*2835*/
  0x0DD2, 0x0020, 0x0002, 0x0001, /*2836*/
  0x0DD3, 0x0020, 0x0002, 0x0001, /*2837*/
  0x0DD4, 0x0020, 0x0002, 0x0001, /*2838*/
  0x0DD5, 0x0020, 0x0002, 0x0001, /*2839*/
  0x0DD6, 0x0020, 0x0002, 0x0001, /*283A*/
  0x0DD7, 0x0020, 0x0002, 0x0001, /*283B*/
  0x0DD8, 0x0020, 0x0002, 0x0001, /*283C*/
  0x0DD9, 0x0020, 0x0002, 0x0001, /*283D*/
  0x0DDA, 0x0020, 0x0002, 0x0001, /*283E*/
  0x0DDB, 0x0020, 0x0002, 0x0001, /*283F*/
  0x0DDC, 0x0020, 0x0002, 0x0001, /*2840*/
  0x0DDD, 0x0020, 0x0002, 0x0001, /*2841*/
  0x0DDE, 0x0020, 0x0002, 0x0001, /*2842*/
  0x0DDF, 0x0020, 0x0002, 0x0001, /*2843*/
  0x0DE0, 0x0020, 0x0002, 0x0001, /*2844*/
  0x0DE1, 0x0020, 0x0002, 0x0001, /*2845*/
  0x0DE2, 0x0020, 0x0002, 0x0001, /*2846*/
  0x0DE3, 0x0020, 0x0002, 0x0001, /*2847*/
  0x0DE4, 0x0020, 0x0002, 0x0001, /*2848*/
  0x0DE5, 0x0020, 0x0002, 0x0001, /*2849*/
  0x0DE6, 0x0020, 0x0002, 0x0001, /*284A*/
  0x0DE7, 0x0020, 0x0002, 0x0001, /*284B*/
  0x0DE8, 0x0020, 0x0002, 0x0001, /*284C*/
  0x0DE9, 0x0020, 0x0002, 0x0001, /*284D*/
  0x0DEA, 0x0020, 0x0002, 0x0001, /*284E*/
  0x0DEB, 0x0020, 0x0002, 0x0001, /*284F*/
  0x0DEC, 0x0020, 0x0002, 0x0001, /*2850*/
  0x0DED, 0x0020, 0x0002, 0x0001, /*2851*/
  0x0DEE, 0x0020, 0x0002, 0x0001, /*2852*/
  0x0DEF, 0x0020, 0x0002, 0x0001, /*2853*/
  0x0DF0, 0x0020, 0x0002, 0x0001, /*2854*/
  0x0DF1, 0x0020, 0x0002, 0x0001, /*2855*/
  0x0DF2, 0x0020, 0x0002, 0x0001, /*2856*/
  0x0DF3, 0x0020, 0x0002, 0x0001, /*2857*/
  0x0DF4, 0x0020, 0x0002, 0x0001, /*2858*/
  0x0DF5, 0x0020, 0x0002, 0x0001, /*2859*/
  0x0DF6, 0x0020, 0x0002, 0x0001, /*285A*/
  0x0DF7, 0x0020, 0x0002, 0x0001, /*285B*/
  0x0DF8, 0x0020, 0x0002, 0x0001, /*285C*/
  0x0DF9, 0x0020, 0x0002, 0x0001, /*285D*/
  0x0DFA, 0x0020, 0x0002, 0x0001, /*285E*/
  0x0DFB, 0x0020, 0x0002, 0x0001, /*285F*/
  0x0DFC, 0x0020, 0x0002, 0x0001, /*2860*/
  0x0DFD, 0x0020, 0x0002, 0x0001, /*2861*/
  0x0DFE, 0x0020, 0x0002, 0x0001, /*2862*/
  0x0DFF, 0x0020, 0x0002, 0x0001, /*2863*/
  0x0E00, 0x0020, 0x0002, 0x0001, /*2864*/
  0x0E01, 0x0020, 0x0002, 0x0001, /*2865*/
  0x0E02, 0x0020, 0x0002, 0x0001, /*2866*/
  0x0E03, 0x0020, 0x0002, 0x0001, /*2867*/
  0x0E04, 0x0020, 0x0002, 0x0001, /*2868*/
  0x0E05, 0x0020, 0x0002, 0x0001, /*2869*/
  0x0E06, 0x0020, 0x0002, 0x0001, /*286A*/
  0x0E07, 0x0020, 0x0002, 0x0001, /*286B*/
  0x0E08, 0x0020, 0x0002, 0x0001, /*286C*/
  0x0E09, 0x0020, 0x0002, 0x0001, /*286D*/
  0x0E0A, 0x0020, 0x0002, 0x0001, /*286E*/
  0x0E0B, 0x0020, 0x0002, 0x0001, /*286F*/
  0x0E0C, 0x0020, 0x0002, 0x0001, /*2870*/
  0x0E0D, 0x0020, 0x0002, 0x0001, /*2871*/
  0x0E0E, 0x0020, 0x0002, 0x0001, /*2872*/
  0x0E0F, 0x0020, 0x0002, 0x0001, /*2873*/
  0x0E10, 0x0020, 0x0002, 0x0001, /*2874*/
  0x0E11, 0x0020, 0x0002, 0x0001, /*2875*/
  0x0E12, 0x0020, 0x0002, 0x0001, /*2876*/
  0x0E13, 0x0020, 0x0002, 0x0001, /*2877*/
  0x0E14, 0x0020, 0x0002, 0x0001, /*2878*/
  0x0E15, 0x0020, 0x0002, 0x0001, /*2879*/
  0x0E16, 0x0020, 0x0002, 0x0001, /*287A*/
  0x0E17, 0x0020, 0x0002, 0x0001, /*287B*/
  0x0E18, 0x0020, 0x0002, 0x0001, /*287C*/
  0x0E19, 0x0020, 0x0002, 0x0001, /*287D*/
  0x0E1A, 0x0020, 0x0002, 0x0001, /*287E*/
  0x0E1B, 0x0020, 0x0002, 0x0001, /*287F*/
  0x0E1C, 0x0020, 0x0002, 0x0001, /*2880*/
  0x0E1D, 0x0020, 0x0002, 0x0001, /*2881*/
  0x0E1E, 0x0020, 0x0002, 0x0001, /*2882*/
  0x0E1F, 0x0020, 0x0002, 0x0001, /*2883*/
  0x0E20, 0x0020, 0x0002, 0x0001, /*2884*/
  0x0E21, 0x0020, 0x0002, 0x0001, /*2885*/
  0x0E22, 0x0020, 0x0002, 0x0001, /*2886*/
  0x0E23, 0x0020, 0x0002, 0x0001, /*2887*/
  0x0E24, 0x0020, 0x0002, 0x0001, /*2888*/
  0x0E25, 0x0020, 0x0002, 0x0001, /*2889*/
  0x0E26, 0x0020, 0x0002, 0x0001, /*288A*/
  0x0E27, 0x0020, 0x0002, 0x0001, /*288B*/
  0x0E28, 0x0020, 0x0002, 0x0001, /*288C*/
  0x0E29, 0x0020, 0x0002, 0x0001, /*288D*/
  0x0E2A, 0x0020, 0x0002, 0x0001, /*288E*/
  0x0E2B, 0x0020, 0x0002, 0x0001, /*288F*/
  0x0E2C, 0x0020, 0x0002, 0x0001, /*2890*/
  0x0E2D, 0x0020, 0x0002, 0x0001, /*2891*/
  0x0E2E, 0x0020, 0x0002, 0x0001, /*2892*/
  0x0E2F, 0x0020, 0x0002, 0x0001, /*2893*/
  0x0E30, 0x0020, 0x0002, 0x0001, /*2894*/
  0x0E31, 0x0020, 0x0002, 0x0001, /*2895*/
  0x0E32, 0x0020, 0x0002, 0x0001, /*2896*/
  0x0E33, 0x0020, 0x0002, 0x0001, /*2897*/
  0x0E34, 0x0020, 0x0002, 0x0001, /*2898*/
  0x0E35, 0x0020, 0x0002, 0x0001, /*2899*/
  0x0E36, 0x0020, 0x0002, 0x0001, /*289A*/
  0x0E37, 0x0020, 0x0002, 0x0001, /*289B*/
  0x0E38, 0x0020, 0x0002, 0x0001, /*289C*/
  0x0E39, 0x0020, 0x0002, 0x0001, /*289D*/
  0x0E3A, 0x0020, 0x0002, 0x0001, /*289E*/
  0x0E3B, 0x0020, 0x0002, 0x0001, /*289F*/
  0x0E3C, 0x0020, 0x0002, 0x0001, /*28A0*/
  0x0E3D, 0x0020, 0x0002, 0x0001, /*28A1*/
  0x0E3E, 0x0020, 0x0002, 0x0001, /*28A2*/
  0x0E3F, 0x0020, 0x0002, 0x0001, /*28A3*/
  0x0E40, 0x0020, 0x0002, 0x0001, /*28A4*/
  0x0E41, 0x0020, 0x0002, 0x0001, /*28A5*/
  0x0E42, 0x0020, 0x0002, 0x0001, /*28A6*/
  0x0E43, 0x0020, 0x0002, 0x0001, /*28A7*/
  0x0E44, 0x0020, 0x0002, 0x0001, /*28A8*/
  0x0E45, 0x0020, 0x0002, 0x0001, /*28A9*/
  0x0E46, 0x0020, 0x0002, 0x0001, /*28AA*/
  0x0E47, 0x0020, 0x0002, 0x0001, /*28AB*/
  0x0E48, 0x0020, 0x0002, 0x0001, /*28AC*/
  0x0E49, 0x0020, 0x0002, 0x0001, /*28AD*/
  0x0E4A, 0x0020, 0x0002, 0x0001, /*28AE*/
  0x0E4B, 0x0020, 0x0002, 0x0001, /*28AF*/
  0x0E4C, 0x0020, 0x0002, 0x0001, /*28B0*/
  0x0E4D, 0x0020, 0x0002, 0x0001, /*28B1*/
  0x0E4E, 0x0020, 0x0002, 0x0001, /*28B2*/
  0x0E4F, 0x0020, 0x0002, 0x0001, /*28B3*/
  0x0E50, 0x0020, 0x0002, 0x0001, /*28B4*/
  0x0E51, 0x0020, 0x0002, 0x0001, /*28B5*/
  0x0E52, 0x0020, 0x0002, 0x0001, /*28B6*/
  0x0E53, 0x0020, 0x0002, 0x0001, /*28B7*/
  0x0E54, 0x0020, 0x0002, 0x0001, /*28B8*/
  0x0E55, 0x0020, 0x0002, 0x0001, /*28B9*/
  0x0E56, 0x0020, 0x0002, 0x0001, /*28BA*/
  0x0E57, 0x0020, 0x0002, 0x0001, /*28BB*/
  0x0E58, 0x0020, 0x0002, 0x0001, /*28BC*/
  0x0E59, 0x0020, 0x0002, 0x0001, /*28BD*/
  0x0E5A, 0x0020, 0x0002, 0x0001, /*28BE*/
  0x0E5B, 0x0020, 0x0002, 0x0001, /*28BF*/
  0x0E5C, 0x0020, 0x0002, 0x0001, /*28C0*/
  0x0E5D, 0x0020, 0x0002, 0x0001, /*28C1*/
  0x0E5E, 0x0020, 0x0002, 0x0001, /*28C2*/
  0x0E5F, 0x0020, 0x0002, 0x0001, /*28C3*/
  0x0E60, 0x0020, 0x0002, 0x0001, /*28C4*/
  0x0E61, 0x0020, 0x0002, 0x0001, /*28C5*/
  0x0E62, 0x0020, 0x0002, 0x0001, /*28C6*/
  0x0E63, 0x0020, 0x0002, 0x0001, /*28C7*/
  0x0E64, 0x0020, 0x0002, 0x0001, /*28C8*/
  0x0E65, 0x0020, 0x0002, 0x0001, /*28C9*/
  0x0E66, 0x0020, 0x0002, 0x0001, /*28CA*/
  0x0E67, 0x0020, 0x0002, 0x0001, /*28CB*/
  0x0E68, 0x0020, 0x0002, 0x0001, /*28CC*/
  0x0E69, 0x0020, 0x0002, 0x0001, /*28CD*/
  0x0E6A, 0x0020, 0x0002, 0x0001, /*28CE*/
  0x0E6B, 0x0020, 0x0002, 0x0001, /*28CF*/
  0x0E6C, 0x0020, 0x0002, 0x0001, /*28D0*/
  0x0E6D, 0x0020, 0x0002, 0x0001, /*28D1*/
  0x0E6E, 0x0020, 0x0002, 0x0001, /*28D2*/
  0x0E6F, 0x0020, 0x0002, 0x0001, /*28D3*/
  0x0E70, 0x0020, 0x0002, 0x0001, /*28D4*/
  0x0E71, 0x0020, 0x0002, 0x0001, /*28D5*/
  0x0E72, 0x0020, 0x0002, 0x0001, /*28D6*/
  0x0E73, 0x0020, 0x0002, 0x0001, /*28D7*/
  0x0E74, 0x0020, 0x0002, 0x0001, /*28D8*/
  0x0E75, 0x0020, 0x0002, 0x0001, /*28D9*/
  0x0E76, 0x0020, 0x0002, 0x0001, /*28DA*/
  0x0E77, 0x0020, 0x0002, 0x0001, /*28DB*/
  0x0E78, 0x0020, 0x0002, 0x0001, /*28DC*/
  0x0E79, 0x0020, 0x0002, 0x0001, /*28DD*/
  0x0E7A, 0x0020, 0x0002, 0x0001, /*28DE*/
  0x0E7B, 0x0020, 0x0002, 0x0001, /*28DF*/
  0x0E7C, 0x0020, 0x0002, 0x0001, /*28E0*/
  0x0E7D, 0x0020, 0x0002, 0x0001, /*28E1*/
  0x0E7E, 0x0020, 0x0002, 0x0001, /*28E2*/
  0x0E7F, 0x0020, 0x0002, 0x0001, /*28E3*/
  0x0E80, 0x0020, 0x0002, 0x0001, /*28E4*/
  0x0E81, 0x0020, 0x0002, 0x0001, /*28E5*/
  0x0E82, 0x0020, 0x0002, 0x0001, /*28E6*/
  0x0E83, 0x0020, 0x0002, 0x0001, /*28E7*/
  0x0E84, 0x0020, 0x0002, 0x0001, /*28E8*/
  0x0E85, 0x0020, 0x0002, 0x0001, /*28E9*/
  0x0E86, 0x0020, 0x0002, 0x0001, /*28EA*/
  0x0E87, 0x0020, 0x0002, 0x0001, /*28EB*/
  0x0E88, 0x0020, 0x0002, 0x0001, /*28EC*/
  0x0E89, 0x0020, 0x0002, 0x0001, /*28ED*/
  0x0E8A, 0x0020, 0x0002, 0x0001, /*28EE*/
  0x0E8B, 0x0020, 0x0002, 0x0001, /*28EF*/
  0x0E8C, 0x0020, 0x0002, 0x0001, /*28F0*/
  0x0E8D, 0x0020, 0x0002, 0x0001, /*28F1*/
  0x0E8E, 0x0020, 0x0002, 0x0001, /*28F2*/
  0x0E8F, 0x0020, 0x0002, 0x0001, /*28F3*/
  0x0E90, 0x0020, 0x0002, 0x0001, /*28F4*/
  0x0E91, 0x0020, 0x0002, 0x0001, /*28F5*/
  0x0E92, 0x0020, 0x0002, 0x0001, /*28F6*/
  0x0E93, 0x0020, 0x0002, 0x0001, /*28F7*/
  0x0E94, 0x0020, 0x0002, 0x0001, /*28F8*/
  0x0E95, 0x0020, 0x0002, 0x0001, /*28F9*/
  0x0E96, 0x0020, 0x0002, 0x0001, /*28FA*/
  0x0E97, 0x0020, 0x0002, 0x0001, /*28FB*/
  0x0E98, 0x0020, 0x0002, 0x0001, /*28FC*/
  0x0E99, 0x0020, 0x0002, 0x0001, /*28FD*/
  0x0E9A, 0x0020, 0x0002, 0x0001, /*28FE*/
  0x0E9B, 0x0020, 0x0002, 0x0001  /*28FF*/
};

uint16 uca900_p029[]= {
  0x0AE9, 0x0020, 0x0002, 0x0001, /*2900*/
  0x0AEA, 0x0020, 0x0002, 0x0001, /*2901*/
  0x0AEB, 0x0020, 0x0002, 0x0001, /*2902*/
  0x0AEC, 0x0020, 0x0002, 0x0001, /*2903*/
  0x0AED, 0x0020, 0x0002, 0x0001, /*2904*/
  0x0AEE, 0x0020, 0x0002, 0x0001, /*2905*/
  0x0AEF, 0x0020, 0x0002, 0x0001, /*2906*/
  0x0AF0, 0x0020, 0x0002, 0x0001, /*2907*/
  0x0AF1, 0x0020, 0x0002, 0x0001, /*2908*/
  0x0AF2, 0x0020, 0x0002, 0x0001, /*2909*/
  0x0AF3, 0x0020, 0x0002, 0x0001, /*290A*/
  0x0AF4, 0x0020, 0x0002, 0x0001, /*290B*/
  0x0AF5, 0x0020, 0x0002, 0x0001, /*290C*/
  0x0AF6, 0x0020, 0x0002, 0x0001, /*290D*/
  0x0AF7, 0x0020, 0x0002, 0x0001, /*290E*/
  0x0AF8, 0x0020, 0x0002, 0x0001, /*290F*/
  0x0AF9, 0x0020, 0x0002, 0x0001, /*2910*/
  0x0AFA, 0x0020, 0x0002, 0x0001, /*2911*/
  0x0AFB, 0x0020, 0x0002, 0x0001, /*2912*/
  0x0AFC, 0x0020, 0x0002, 0x0001, /*2913*/
  0x0AFD, 0x0020, 0x0002, 0x0001, /*2914*/
  0x0AFE, 0x0020, 0x0002, 0x0001, /*2915*/
  0x0AFF, 0x0020, 0x0002, 0x0001, /*2916*/
  0x0B00, 0x0020, 0x0002, 0x0001, /*2917*/
  0x0B01, 0x0020, 0x0002, 0x0001, /*2918*/
  0x0B02, 0x0020, 0x0002, 0x0001, /*2919*/
  0x0B03, 0x0020, 0x0002, 0x0001, /*291A*/
  0x0B04, 0x0020, 0x0002, 0x0001, /*291B*/
  0x0B05, 0x0020, 0x0002, 0x0001, /*291C*/
  0x0B06, 0x0020, 0x0002, 0x0001, /*291D*/
  0x0B07, 0x0020, 0x0002, 0x0001, /*291E*/
  0x0B08, 0x0020, 0x0002, 0x0001, /*291F*/
  0x0B09, 0x0020, 0x0002, 0x0001, /*2920*/
  0x0B0A, 0x0020, 0x0002, 0x0001, /*2921*/
  0x0B0B, 0x0020, 0x0002, 0x0001, /*2922*/
  0x0B0C, 0x0020, 0x0002, 0x0001, /*2923*/
  0x0B0D, 0x0020, 0x0002, 0x0001, /*2924*/
  0x0B0E, 0x0020, 0x0002, 0x0001, /*2925*/
  0x0B0F, 0x0020, 0x0002, 0x0001, /*2926*/
  0x0B10, 0x0020, 0x0002, 0x0001, /*2927*/
  0x0B11, 0x0020, 0x0002, 0x0001, /*2928*/
  0x0B12, 0x0020, 0x0002, 0x0001, /*2929*/
  0x0B13, 0x0020, 0x0002, 0x0001, /*292A*/
  0x0B14, 0x0020, 0x0002, 0x0001, /*292B*/
  0x0B15, 0x0020, 0x0002, 0x0001, /*292C*/
  0x0B16, 0x0020, 0x0002, 0x0001, /*292D*/
  0x0B17, 0x0020, 0x0002, 0x0001, /*292E*/
  0x0B18, 0x0020, 0x0002, 0x0001, /*292F*/
  0x0B19, 0x0020, 0x0002, 0x0001, /*2930*/
  0x0B1A, 0x0020, 0x0002, 0x0001, /*2931*/
  0x0B1B, 0x0020, 0x0002, 0x0001, /*2932*/
  0x0B1C, 0x0020, 0x0002, 0x0001, /*2933*/
  0x0B1D, 0x0020, 0x0002, 0x0001, /*2934*/
  0x0B1E, 0x0020, 0x0002, 0x0001, /*2935*/
  0x0B1F, 0x0020, 0x0002, 0x0001, /*2936*/
  0x0B20, 0x0020, 0x0002, 0x0001, /*2937*/
  0x0B21, 0x0020, 0x0002, 0x0001, /*2938*/
  0x0B22, 0x0020, 0x0002, 0x0001, /*2939*/
  0x0B23, 0x0020, 0x0002, 0x0001, /*293A*/
  0x0B24, 0x0020, 0x0002, 0x0001, /*293B*/
  0x0B25, 0x0020, 0x0002, 0x0001, /*293C*/
  0x0B26, 0x0020, 0x0002, 0x0001, /*293D*/
  0x0B27, 0x0020, 0x0002, 0x0001, /*293E*/
  0x0B28, 0x0020, 0x0002, 0x0001, /*293F*/
  0x0B29, 0x0020, 0x0002, 0x0001, /*2940*/
  0x0B2A, 0x0020, 0x0002, 0x0001, /*2941*/
  0x0B2B, 0x0020, 0x0002, 0x0001, /*2942*/
  0x0B2C, 0x0020, 0x0002, 0x0001, /*2943*/
  0x0B2D, 0x0020, 0x0002, 0x0001, /*2944*/
  0x0B2E, 0x0020, 0x0002, 0x0001, /*2945*/
  0x0B2F, 0x0020, 0x0002, 0x0001, /*2946*/
  0x0B30, 0x0020, 0x0002, 0x0001, /*2947*/
  0x0B31, 0x0020, 0x0002, 0x0001, /*2948*/
  0x0B32, 0x0020, 0x0002, 0x0001, /*2949*/
  0x0B33, 0x0020, 0x0002, 0x0001, /*294A*/
  0x0B34, 0x0020, 0x0002, 0x0001, /*294B*/
  0x0B35, 0x0020, 0x0002, 0x0001, /*294C*/
  0x0B36, 0x0020, 0x0002, 0x0001, /*294D*/
  0x0B37, 0x0020, 0x0002, 0x0001, /*294E*/
  0x0B38, 0x0020, 0x0002, 0x0001, /*294F*/
  0x0B39, 0x0020, 0x0002, 0x0001, /*2950*/
  0x0B3A, 0x0020, 0x0002, 0x0001, /*2951*/
  0x0B3B, 0x0020, 0x0002, 0x0001, /*2952*/
  0x0B3C, 0x0020, 0x0002, 0x0001, /*2953*/
  0x0B3D, 0x0020, 0x0002, 0x0001, /*2954*/
  0x0B3E, 0x0020, 0x0002, 0x0001, /*2955*/
  0x0B3F, 0x0020, 0x0002, 0x0001, /*2956*/
  0x0B40, 0x0020, 0x0002, 0x0001, /*2957*/
  0x0B41, 0x0020, 0x0002, 0x0001, /*2958*/
  0x0B42, 0x0020, 0x0002, 0x0001, /*2959*/
  0x0B43, 0x0020, 0x0002, 0x0001, /*295A*/
  0x0B44, 0x0020, 0x0002, 0x0001, /*295B*/
  0x0B45, 0x0020, 0x0002, 0x0001, /*295C*/
  0x0B46, 0x0020, 0x0002, 0x0001, /*295D*/
  0x0B47, 0x0020, 0x0002, 0x0001, /*295E*/
  0x0B48, 0x0020, 0x0002, 0x0001, /*295F*/
  0x0B49, 0x0020, 0x0002, 0x0001, /*2960*/
  0x0B4A, 0x0020, 0x0002, 0x0001, /*2961*/
  0x0B4B, 0x0020, 0x0002, 0x0001, /*2962*/
  0x0B4C, 0x0020, 0x0002, 0x0001, /*2963*/
  0x0B4D, 0x0020, 0x0002, 0x0001, /*2964*/
  0x0B4E, 0x0020, 0x0002, 0x0001, /*2965*/
  0x0B4F, 0x0020, 0x0002, 0x0001, /*2966*/
  0x0B50, 0x0020, 0x0002, 0x0001, /*2967*/
  0x0B51, 0x0020, 0x0002, 0x0001, /*2968*/
  0x0B52, 0x0020, 0x0002, 0x0001, /*2969*/
  0x0B53, 0x0020, 0x0002, 0x0001, /*296A*/
  0x0B54, 0x0020, 0x0002, 0x0001, /*296B*/
  0x0B55, 0x0020, 0x0002, 0x0001, /*296C*/
  0x0B56, 0x0020, 0x0002, 0x0001, /*296D*/
  0x0B57, 0x0020, 0x0002, 0x0001, /*296E*/
  0x0B58, 0x0020, 0x0002, 0x0001, /*296F*/
  0x0B59, 0x0020, 0x0002, 0x0001, /*2970*/
  0x0B5A, 0x0020, 0x0002, 0x0001, /*2971*/
  0x0B5B, 0x0020, 0x0002, 0x0001, /*2972*/
  0x0B5C, 0x0020, 0x0002, 0x0001, /*2973*/
  0x0B5D, 0x0020, 0x0002, 0x0001, /*2974*/
  0x0B5E, 0x0020, 0x0002, 0x0001, /*2975*/
  0x0B5F, 0x0020, 0x0002, 0x0001, /*2976*/
  0x0B60, 0x0020, 0x0002, 0x0001, /*2977*/
  0x0B61, 0x0020, 0x0002, 0x0001, /*2978*/
  0x0B62, 0x0020, 0x0002, 0x0001, /*2979*/
  0x0B63, 0x0020, 0x0002, 0x0001, /*297A*/
  0x0B64, 0x0020, 0x0002, 0x0001, /*297B*/
  0x0B65, 0x0020, 0x0002, 0x0001, /*297C*/
  0x0B66, 0x0020, 0x0002, 0x0001, /*297D*/
  0x0B67, 0x0020, 0x0002, 0x0001, /*297E*/
  0x0B68, 0x0020, 0x0002, 0x0001, /*297F*/
  0x0B69, 0x0020, 0x0002, 0x0001, /*2980*/
  0x0B6A, 0x0020, 0x0002, 0x0001, /*2981*/
  0x0B6B, 0x0020, 0x0002, 0x0001, /*2982*/
  0x032B, 0x0020, 0x0002, 0x0001, /*2983*/
  0x032C, 0x0020, 0x0002, 0x0001, /*2984*/
  0x032D, 0x0020, 0x0002, 0x0001, /*2985*/
  0x032E, 0x0020, 0x0002, 0x0001, /*2986*/
  0x032F, 0x0020, 0x0002, 0x0001, /*2987*/
  0x0330, 0x0020, 0x0002, 0x0001, /*2988*/
  0x0331, 0x0020, 0x0002, 0x0001, /*2989*/
  0x0332, 0x0020, 0x0002, 0x0001, /*298A*/
  0x0333, 0x0020, 0x0002, 0x0001, /*298B*/
  0x0334, 0x0020, 0x0002, 0x0001, /*298C*/
  0x0335, 0x0020, 0x0002, 0x0001, /*298D*/
  0x0336, 0x0020, 0x0002, 0x0001, /*298E*/
  0x0337, 0x0020, 0x0002, 0x0001, /*298F*/
  0x0338, 0x0020, 0x0002, 0x0001, /*2990*/
  0x0339, 0x0020, 0x0002, 0x0001, /*2991*/
  0x033A, 0x0020, 0x0002, 0x0001, /*2992*/
  0x033B, 0x0020, 0x0002, 0x0001, /*2993*/
  0x033C, 0x0020, 0x0002, 0x0001, /*2994*/
  0x033D, 0x0020, 0x0002, 0x0001, /*2995*/
  0x033E, 0x0020, 0x0002, 0x0001, /*2996*/
  0x033F, 0x0020, 0x0002, 0x0001, /*2997*/
  0x0340, 0x0020, 0x0002, 0x0001, /*2998*/
  0x0B6C, 0x0020, 0x0002, 0x0001, /*2999*/
  0x0B6D, 0x0020, 0x0002, 0x0001, /*299A*/
  0x0B6E, 0x0020, 0x0002, 0x0001, /*299B*/
  0x0B6F, 0x0020, 0x0002, 0x0001, /*299C*/
  0x0B70, 0x0020, 0x0002, 0x0001, /*299D*/
  0x0B71, 0x0020, 0x0002, 0x0001, /*299E*/
  0x0B72, 0x0020, 0x0002, 0x0001, /*299F*/
  0x0B73, 0x0020, 0x0002, 0x0001, /*29A0*/
  0x0B74, 0x0020, 0x0002, 0x0001, /*29A1*/
  0x0B75, 0x0020, 0x0002, 0x0001, /*29A2*/
  0x0B76, 0x0020, 0x0002, 0x0001, /*29A3*/
  0x0B77, 0x0020, 0x0002, 0x0001, /*29A4*/
  0x0B78, 0x0020, 0x0002, 0x0001, /*29A5*/
  0x0B79, 0x0020, 0x0002, 0x0001, /*29A6*/
  0x0B7A, 0x0020, 0x0002, 0x0001, /*29A7*/
  0x0B7B, 0x0020, 0x0002, 0x0001, /*29A8*/
  0x0B7C, 0x0020, 0x0002, 0x0001, /*29A9*/
  0x0B7D, 0x0020, 0x0002, 0x0001, /*29AA*/
  0x0B7E, 0x0020, 0x0002, 0x0001, /*29AB*/
  0x0B7F, 0x0020, 0x0002, 0x0001, /*29AC*/
  0x0B80, 0x0020, 0x0002, 0x0001, /*29AD*/
  0x0B81, 0x0020, 0x0002, 0x0001, /*29AE*/
  0x0B82, 0x0020, 0x0002, 0x0001, /*29AF*/
  0x0B83, 0x0020, 0x0002, 0x0001, /*29B0*/
  0x0B84, 0x0020, 0x0002, 0x0001, /*29B1*/
  0x0B85, 0x0020, 0x0002, 0x0001, /*29B2*/
  0x0B86, 0x0020, 0x0002, 0x0001, /*29B3*/
  0x0B87, 0x0020, 0x0002, 0x0001, /*29B4*/
  0x0B88, 0x0020, 0x0002, 0x0001, /*29B5*/
  0x0B89, 0x0020, 0x0002, 0x0001, /*29B6*/
  0x0B8A, 0x0020, 0x0002, 0x0001, /*29B7*/
  0x0B8B, 0x0020, 0x0002, 0x0001, /*29B8*/
  0x0B8C, 0x0020, 0x0002, 0x0001, /*29B9*/
  0x0B8D, 0x0020, 0x0002, 0x0001, /*29BA*/
  0x0B8E, 0x0020, 0x0002, 0x0001, /*29BB*/
  0x0B8F, 0x0020, 0x0002, 0x0001, /*29BC*/
  0x0B90, 0x0020, 0x0002, 0x0001, /*29BD*/
  0x0B91, 0x0020, 0x0002, 0x0001, /*29BE*/
  0x0B92, 0x0020, 0x0002, 0x0001, /*29BF*/
  0x0B93, 0x0020, 0x0002, 0x0001, /*29C0*/
  0x0B94, 0x0020, 0x0002, 0x0001, /*29C1*/
  0x0B95, 0x0020, 0x0002, 0x0001, /*29C2*/
  0x0B96, 0x0020, 0x0002, 0x0001, /*29C3*/
  0x0B97, 0x0020, 0x0002, 0x0001, /*29C4*/
  0x0B98, 0x0020, 0x0002, 0x0001, /*29C5*/
  0x0B99, 0x0020, 0x0002, 0x0001, /*29C6*/
  0x0B9A, 0x0020, 0x0002, 0x0001, /*29C7*/
  0x0B9B, 0x0020, 0x0002, 0x0001, /*29C8*/
  0x0B9C, 0x0020, 0x0002, 0x0001, /*29C9*/
  0x0B9D, 0x0020, 0x0002, 0x0001, /*29CA*/
  0x0B9E, 0x0020, 0x0002, 0x0001, /*29CB*/
  0x0B9F, 0x0020, 0x0002, 0x0001, /*29CC*/
  0x0BA0, 0x0020, 0x0002, 0x0001, /*29CD*/
  0x0BA1, 0x0020, 0x0002, 0x0001, /*29CE*/
  0x0BA2, 0x0020, 0x0002, 0x0001, /*29CF*/
  0x0BA3, 0x0020, 0x0002, 0x0001, /*29D0*/
  0x0BA4, 0x0020, 0x0002, 0x0001, /*29D1*/
  0x0BA5, 0x0020, 0x0002, 0x0001, /*29D2*/
  0x0BA6, 0x0020, 0x0002, 0x0001, /*29D3*/
  0x0BA7, 0x0020, 0x0002, 0x0001, /*29D4*/
  0x0BA8, 0x0020, 0x0002, 0x0001, /*29D5*/
  0x0BA9, 0x0020, 0x0002, 0x0001, /*29D6*/
  0x0BAA, 0x0020, 0x0002, 0x0001, /*29D7*/
  0x0385, 0x0020, 0x0002, 0x0001, /*29D8*/
  0x0386, 0x0020, 0x0002, 0x0001, /*29D9*/
  0x0387, 0x0020, 0x0002, 0x0001, /*29DA*/
  0x0388, 0x0020, 0x0002, 0x0001, /*29DB*/
  0x0BAB, 0x0020, 0x0002, 0x0001, /*29DC*/
  0x0BAC, 0x0020, 0x0002, 0x0001, /*29DD*/
  0x0BAD, 0x0020, 0x0002, 0x0001, /*29DE*/
  0x0BAE, 0x0020, 0x0002, 0x0001, /*29DF*/
  0x0BAF, 0x0020, 0x0002, 0x0001, /*29E0*/
  0x0BB0, 0x0020, 0x0002, 0x0001, /*29E1*/
  0x0BB1, 0x0020, 0x0002, 0x0001, /*29E2*/
  0x0BB2, 0x0020, 0x0002, 0x0001, /*29E3*/
  0x0BB3, 0x0020, 0x0002, 0x0001, /*29E4*/
  0x0BB4, 0x0020, 0x0002, 0x0001, /*29E5*/
  0x0BB5, 0x0020, 0x0002, 0x0001, /*29E6*/
  0x0BB6, 0x0020, 0x0002, 0x0001, /*29E7*/
  0x0BB7, 0x0020, 0x0002, 0x0001, /*29E8*/
  0x0BB8, 0x0020, 0x0002, 0x0001, /*29E9*/
  0x0BB9, 0x0020, 0x0002, 0x0001, /*29EA*/
  0x0BBA, 0x0020, 0x0002, 0x0001, /*29EB*/
  0x0BBB, 0x0020, 0x0002, 0x0001, /*29EC*/
  0x0BBC, 0x0020, 0x0002, 0x0001, /*29ED*/
  0x0BBD, 0x0020, 0x0002, 0x0001, /*29EE*/
  0x0BBE, 0x0020, 0x0002, 0x0001, /*29EF*/
  0x0BBF, 0x0020, 0x0002, 0x0001, /*29F0*/
  0x0BC0, 0x0020, 0x0002, 0x0001, /*29F1*/
  0x0BC1, 0x0020, 0x0002, 0x0001, /*29F2*/
  0x0BC2, 0x0020, 0x0002, 0x0001, /*29F3*/
  0x0BC3, 0x0020, 0x0002, 0x0001, /*29F4*/
  0x0BC4, 0x0020, 0x0002, 0x0001, /*29F5*/
  0x0BC5, 0x0020, 0x0002, 0x0001, /*29F6*/
  0x0BC6, 0x0020, 0x0002, 0x0001, /*29F7*/
  0x0BC7, 0x0020, 0x0002, 0x0001, /*29F8*/
  0x0BC8, 0x0020, 0x0002, 0x0001, /*29F9*/
  0x0BC9, 0x0020, 0x0002, 0x0001, /*29FA*/
  0x0BCA, 0x0020, 0x0002, 0x0001, /*29FB*/
  0x0329, 0x0020, 0x0002, 0x0001, /*29FC*/
  0x032A, 0x0020, 0x0002, 0x0001, /*29FD*/
  0x0BCB, 0x0020, 0x0002, 0x0001, /*29FE*/
  0x0BCC, 0x0020, 0x0002, 0x0001  /*29FF*/
};

uint16 uca900_p02A[]= {
  0x0BCD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A00*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BCE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A01*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BCF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A02*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A03*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A04*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A05*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A06*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A07*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A08*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A09*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A0A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A0B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x063C, 0x0020, 0x0004, 0x063C, 0x0020, 0x0004, 0x063C, /*2A0C*/
  0x0020, 0x0004, 0x063C, 0x0020, 0x0004, 0x0004,
  0x0BD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A0D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A0E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A0F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A10*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A11*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A12*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A13*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A14*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A15*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A16*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A17*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A18*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A19*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A1A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A1B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A1C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A1D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A1E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A1F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A20*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A21*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A22*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A23*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A24*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A25*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A26*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A27*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A28*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A29*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A2A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A2B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A2C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A2D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A2E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A2F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A30*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A31*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A32*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0BFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A33*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A34*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A35*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A36*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A37*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A38*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A39*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A3A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A3B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A3C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A3D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A3E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A3F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A40*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A41*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A42*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A43*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A44*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A45*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A46*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A47*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A48*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A49*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A4A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A4B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A4C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A4D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A4E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A4F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A50*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A51*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A52*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A53*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A54*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A55*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A56*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A57*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A58*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A59*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A5A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A5B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A5C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A5D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A5E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A5F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A60*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A61*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A62*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A63*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A64*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A65*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A66*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A67*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C34, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A68*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C35, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A69*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A6A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A6B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C38, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A6C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C39, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A6D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C3A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A6E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C3B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A6F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A70*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A71*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A72*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A73*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0239, 0x0020, 0x0004, 0x0239, 0x0020, 0x0004, 0x061B, /*2A74*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x061B, 0x0020, 0x0004, 0x061B, 0x0020, 0x0004, 0x0000, /*2A75*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x061B, 0x0020, 0x0004, 0x061B, 0x0020, 0x0004, 0x061B, /*2A76*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A77*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A78*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A79*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A7A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A7B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A7C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A7D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C47, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A7E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A7F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C49, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A80*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C4A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A81*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C4B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A82*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C4C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A83*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C4D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A84*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C4E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A85*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C4F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A86*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A87*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C51, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A88*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C52, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A89*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C53, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A8A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A8B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A8C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A8D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A8E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A8F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A90*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A91*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A92*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A93*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A94*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A95*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A96*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A97*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A98*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A99*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A9A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A9B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A9C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A9D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A9E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2A9F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AA0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AA1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AA2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AA3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AA4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AA5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AA6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AA7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AA8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AA9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AAA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AAB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AAC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AAD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AAE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AAF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AB0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AB1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AB2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AB3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AB4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AB5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AB6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AB7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AB8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AB9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ABA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ABB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ABC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ABD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ABE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ABF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AC0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AC1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AC2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AC3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AC4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AC5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AC6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AC7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AC8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AC9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ACA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ACB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ACC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ACD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ACE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ACF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AD0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AD1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AD2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AD3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AD4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AD5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0C9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AD6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AD7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AD8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AD9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ADA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ADB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CA5, 0x0020, 0x0002, 0x0000, 0x002F, 0x0002, 0x0000, /*2ADC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0CA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ADD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ADE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2ADF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AE0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AE1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AE2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AE3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AE4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AE5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AE6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AE7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AE8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AE9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AEA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AEB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AEC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AEE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AEF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AF0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AF1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AF2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CBB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AF3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CBC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AF4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CBD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AF5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AF6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CBF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AF7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AF8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AF9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AFA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AFB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AFC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AFD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AFE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0CC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*2AFF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p02B[]= {
  0x0CC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B00*/
  0x0CC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B01*/
  0x0CCA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B02*/
  0x0CCB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B03*/
  0x0CCC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B04*/
  0x0CCD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B05*/
  0x0CCE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B06*/
  0x0CCF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B07*/
  0x0CD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B08*/
  0x0CD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B09*/
  0x0CD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B0A*/
  0x0CD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B0B*/
  0x0CD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B0C*/
  0x0CD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B0D*/
  0x0CD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B0E*/
  0x0CD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B0F*/
  0x0CD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B10*/
  0x0CD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B11*/
  0x0CDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B12*/
  0x0CDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B13*/
  0x0CDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B14*/
  0x0CDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B15*/
  0x0CDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B16*/
  0x0CDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B17*/
  0x0CE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B18*/
  0x0CE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B19*/
  0x0CE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B1A*/
  0x0CE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B1B*/
  0x0CE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B1C*/
  0x0CE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B1D*/
  0x0CE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B1E*/
  0x0CE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B1F*/
  0x0CE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B20*/
  0x0CE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B21*/
  0x0CEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B22*/
  0x0CEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B23*/
  0x0CEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B24*/
  0x0CED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B25*/
  0x0CEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B26*/
  0x0CEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B27*/
  0x0CF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B28*/
  0x0CF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B29*/
  0x0CF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B2A*/
  0x0CF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B2B*/
  0x0CF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B2C*/
  0x0CF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B2D*/
  0x0CF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B2E*/
  0x0CF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B2F*/
  0x0CF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B30*/
  0x0CF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B31*/
  0x0CFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B32*/
  0x0CFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B33*/
  0x0CFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B34*/
  0x0CFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B35*/
  0x0CFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B36*/
  0x0CFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B37*/
  0x0D00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B38*/
  0x0D01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B39*/
  0x0D02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B3A*/
  0x0D03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B3B*/
  0x0D04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B3C*/
  0x0D05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B3D*/
  0x0D06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B3E*/
  0x0D07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B3F*/
  0x0D08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B40*/
  0x0D09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B41*/
  0x0D0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B42*/
  0x0D0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B43*/
  0x0D0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B44*/
  0x0D0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B45*/
  0x0D0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B46*/
  0x0D0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B47*/
  0x0D10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B48*/
  0x0D11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B49*/
  0x0D12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B4A*/
  0x0D13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B4B*/
  0x0D14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B4C*/
  0x0D15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B4D*/
  0x0D16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B4E*/
  0x0D17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B4F*/
  0x0D18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B50*/
  0x0D19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B51*/
  0x0D1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B52*/
  0x0D1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B53*/
  0x0D1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B54*/
  0x0D1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B55*/
  0x0D1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B56*/
  0x0D1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B57*/
  0x0D20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B58*/
  0x0D21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B59*/
  0x0D22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B5A*/
  0x0D23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B5B*/
  0x0D24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B5C*/
  0x0D25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B5D*/
  0x0D26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B5E*/
  0x0D27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B5F*/
  0x0D28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B60*/
  0x0D29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B61*/
  0x0D2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B62*/
  0x0D2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B63*/
  0x0D2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B64*/
  0x0D2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B65*/
  0x0D2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B66*/
  0x0D2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B67*/
  0x0D30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B68*/
  0x0D31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B69*/
  0x0D32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B6A*/
  0x0D33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B6B*/
  0x0D34, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B6C*/
  0x0D35, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B6D*/
  0x0D36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B6E*/
  0x0D37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B6F*/
  0x0D38, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B70*/
  0x0D39, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B71*/
  0x0D3A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B72*/
  0x0D3B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B73*/
  0xFBC0, 0x0020, 0x0002, 0xAB74, 0x0000, 0x0000, 0x0002, /*2B74*/
  0xFBC0, 0x0020, 0x0002, 0xAB75, 0x0000, 0x0000, 0x0002, /*2B75*/
  0x0D3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B76*/
  0x0D3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B77*/
  0x0D3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B78*/
  0x0D3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B79*/
  0x0D40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B7A*/
  0x0D41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B7B*/
  0x0D42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B7C*/
  0x0D43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B7D*/
  0x0D44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B7E*/
  0x0D45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B7F*/
  0x0D46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B80*/
  0x0D47, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B81*/
  0x0D48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B82*/
  0x0D49, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B83*/
  0x0D4A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B84*/
  0x0D4B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B85*/
  0x0D4C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B86*/
  0x0D4D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B87*/
  0x0D4E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B88*/
  0x0D4F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B89*/
  0x0D50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B8A*/
  0x0D51, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B8B*/
  0x0D52, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B8C*/
  0x0D53, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B8D*/
  0x0D54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B8E*/
  0x0D55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B8F*/
  0x0D56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B90*/
  0x0D57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B91*/
  0x0D58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B92*/
  0x0D59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B93*/
  0x0D5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B94*/
  0x0D5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B95*/
  0xFBC0, 0x0020, 0x0002, 0xAB96, 0x0000, 0x0000, 0x0002, /*2B96*/
  0xFBC0, 0x0020, 0x0002, 0xAB97, 0x0000, 0x0000, 0x0002, /*2B97*/
  0x0D5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B98*/
  0x0D5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B99*/
  0x0D5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B9A*/
  0x0D5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B9B*/
  0x0D60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B9C*/
  0x0D61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B9D*/
  0x0D62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B9E*/
  0x0D63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2B9F*/
  0x0D64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BA0*/
  0x0D65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BA1*/
  0x0D66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BA2*/
  0x0D67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BA3*/
  0x0D68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BA4*/
  0x0D69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BA5*/
  0x0D6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BA6*/
  0x0D6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BA7*/
  0x0D6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BA8*/
  0x0D6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BA9*/
  0x0D6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BAA*/
  0x0D6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BAB*/
  0x0D70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BAC*/
  0x0D71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BAD*/
  0x0D72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BAE*/
  0x0D73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BAF*/
  0x0D74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BB0*/
  0x0D75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BB1*/
  0x0D76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BB2*/
  0x0D77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BB3*/
  0x0D78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BB4*/
  0x0D79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BB5*/
  0x0D7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BB6*/
  0x0D7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BB7*/
  0x0D7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BB8*/
  0x0D7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BB9*/
  0xFBC0, 0x0020, 0x0002, 0xABBA, 0x0000, 0x0000, 0x0002, /*2BBA*/
  0xFBC0, 0x0020, 0x0002, 0xABBB, 0x0000, 0x0000, 0x0002, /*2BBB*/
  0xFBC0, 0x0020, 0x0002, 0xABBC, 0x0000, 0x0000, 0x0002, /*2BBC*/
  0x0D7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BBD*/
  0x0D7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BBE*/
  0x0D80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BBF*/
  0x0D81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BC0*/
  0x0D82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BC1*/
  0x0D83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BC2*/
  0x0D84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BC3*/
  0x0D85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BC4*/
  0x0D86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BC5*/
  0x0D87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BC6*/
  0x0D88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BC7*/
  0x0D89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BC8*/
  0xFBC0, 0x0020, 0x0002, 0xABC9, 0x0000, 0x0000, 0x0002, /*2BC9*/
  0x0D8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BCA*/
  0x0D8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BCB*/
  0x0D8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BCC*/
  0x0D8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BCD*/
  0x0D8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BCE*/
  0x0D8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BCF*/
  0x0D90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BD0*/
  0x0D91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BD1*/
  0xFBC0, 0x0020, 0x0002, 0xABD2, 0x0000, 0x0000, 0x0002, /*2BD2*/
  0xFBC0, 0x0020, 0x0002, 0xABD3, 0x0000, 0x0000, 0x0002, /*2BD3*/
  0xFBC0, 0x0020, 0x0002, 0xABD4, 0x0000, 0x0000, 0x0002, /*2BD4*/
  0xFBC0, 0x0020, 0x0002, 0xABD5, 0x0000, 0x0000, 0x0002, /*2BD5*/
  0xFBC0, 0x0020, 0x0002, 0xABD6, 0x0000, 0x0000, 0x0002, /*2BD6*/
  0xFBC0, 0x0020, 0x0002, 0xABD7, 0x0000, 0x0000, 0x0002, /*2BD7*/
  0xFBC0, 0x0020, 0x0002, 0xABD8, 0x0000, 0x0000, 0x0002, /*2BD8*/
  0xFBC0, 0x0020, 0x0002, 0xABD9, 0x0000, 0x0000, 0x0002, /*2BD9*/
  0xFBC0, 0x0020, 0x0002, 0xABDA, 0x0000, 0x0000, 0x0002, /*2BDA*/
  0xFBC0, 0x0020, 0x0002, 0xABDB, 0x0000, 0x0000, 0x0002, /*2BDB*/
  0xFBC0, 0x0020, 0x0002, 0xABDC, 0x0000, 0x0000, 0x0002, /*2BDC*/
  0xFBC0, 0x0020, 0x0002, 0xABDD, 0x0000, 0x0000, 0x0002, /*2BDD*/
  0xFBC0, 0x0020, 0x0002, 0xABDE, 0x0000, 0x0000, 0x0002, /*2BDE*/
  0xFBC0, 0x0020, 0x0002, 0xABDF, 0x0000, 0x0000, 0x0002, /*2BDF*/
  0xFBC0, 0x0020, 0x0002, 0xABE0, 0x0000, 0x0000, 0x0002, /*2BE0*/
  0xFBC0, 0x0020, 0x0002, 0xABE1, 0x0000, 0x0000, 0x0002, /*2BE1*/
  0xFBC0, 0x0020, 0x0002, 0xABE2, 0x0000, 0x0000, 0x0002, /*2BE2*/
  0xFBC0, 0x0020, 0x0002, 0xABE3, 0x0000, 0x0000, 0x0002, /*2BE3*/
  0xFBC0, 0x0020, 0x0002, 0xABE4, 0x0000, 0x0000, 0x0002, /*2BE4*/
  0xFBC0, 0x0020, 0x0002, 0xABE5, 0x0000, 0x0000, 0x0002, /*2BE5*/
  0xFBC0, 0x0020, 0x0002, 0xABE6, 0x0000, 0x0000, 0x0002, /*2BE6*/
  0xFBC0, 0x0020, 0x0002, 0xABE7, 0x0000, 0x0000, 0x0002, /*2BE7*/
  0xFBC0, 0x0020, 0x0002, 0xABE8, 0x0000, 0x0000, 0x0002, /*2BE8*/
  0xFBC0, 0x0020, 0x0002, 0xABE9, 0x0000, 0x0000, 0x0002, /*2BE9*/
  0xFBC0, 0x0020, 0x0002, 0xABEA, 0x0000, 0x0000, 0x0002, /*2BEA*/
  0xFBC0, 0x0020, 0x0002, 0xABEB, 0x0000, 0x0000, 0x0002, /*2BEB*/
  0x0D92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BEC*/
  0x0D93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BED*/
  0x0D94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BEE*/
  0x0D95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2BEF*/
  0xFBC0, 0x0020, 0x0002, 0xABF0, 0x0000, 0x0000, 0x0002, /*2BF0*/
  0xFBC0, 0x0020, 0x0002, 0xABF1, 0x0000, 0x0000, 0x0002, /*2BF1*/
  0xFBC0, 0x0020, 0x0002, 0xABF2, 0x0000, 0x0000, 0x0002, /*2BF2*/
  0xFBC0, 0x0020, 0x0002, 0xABF3, 0x0000, 0x0000, 0x0002, /*2BF3*/
  0xFBC0, 0x0020, 0x0002, 0xABF4, 0x0000, 0x0000, 0x0002, /*2BF4*/
  0xFBC0, 0x0020, 0x0002, 0xABF5, 0x0000, 0x0000, 0x0002, /*2BF5*/
  0xFBC0, 0x0020, 0x0002, 0xABF6, 0x0000, 0x0000, 0x0002, /*2BF6*/
  0xFBC0, 0x0020, 0x0002, 0xABF7, 0x0000, 0x0000, 0x0002, /*2BF7*/
  0xFBC0, 0x0020, 0x0002, 0xABF8, 0x0000, 0x0000, 0x0002, /*2BF8*/
  0xFBC0, 0x0020, 0x0002, 0xABF9, 0x0000, 0x0000, 0x0002, /*2BF9*/
  0xFBC0, 0x0020, 0x0002, 0xABFA, 0x0000, 0x0000, 0x0002, /*2BFA*/
  0xFBC0, 0x0020, 0x0002, 0xABFB, 0x0000, 0x0000, 0x0002, /*2BFB*/
  0xFBC0, 0x0020, 0x0002, 0xABFC, 0x0000, 0x0000, 0x0002, /*2BFC*/
  0xFBC0, 0x0020, 0x0002, 0xABFD, 0x0000, 0x0000, 0x0002, /*2BFD*/
  0xFBC0, 0x0020, 0x0002, 0xABFE, 0x0000, 0x0000, 0x0002, /*2BFE*/
  0xFBC0, 0x0020, 0x0002, 0xABFF, 0x0000, 0x0000, 0x0002  /*2BFF*/
};

uint16 uca900_p02C[]= {
  0x21E5, 0x0020, 0x0008, 0x0000, 0x0000, /*2C00*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21E6, 0x0020, 0x0008, 0x0000, 0x0000, /*2C01*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21E7, 0x0020, 0x0008, 0x0000, 0x0000, /*2C02*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21E8, 0x0020, 0x0008, 0x0000, 0x0000, /*2C03*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21E9, 0x0020, 0x0008, 0x0000, 0x0000, /*2C04*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21EA, 0x0020, 0x0008, 0x0000, 0x0000, /*2C05*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21EB, 0x0020, 0x0008, 0x0000, 0x0000, /*2C06*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21EC, 0x0020, 0x0008, 0x0000, 0x0000, /*2C07*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21ED, 0x0020, 0x0008, 0x0000, 0x0000, /*2C08*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21EE, 0x0020, 0x0008, 0x0000, 0x0000, /*2C09*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21EF, 0x0020, 0x0008, 0x0000, 0x0000, /*2C0A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F0, 0x0020, 0x0008, 0x0000, 0x0000, /*2C0B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F1, 0x0020, 0x0008, 0x0000, 0x0000, /*2C0C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F2, 0x0020, 0x0008, 0x0000, 0x0000, /*2C0D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F3, 0x0020, 0x0008, 0x0000, 0x0000, /*2C0E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F4, 0x0020, 0x0008, 0x0000, 0x0000, /*2C0F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F5, 0x0020, 0x0008, 0x0000, 0x0000, /*2C10*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F6, 0x0020, 0x0008, 0x0000, 0x0000, /*2C11*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F7, 0x0020, 0x0008, 0x0000, 0x0000, /*2C12*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F8, 0x0020, 0x0008, 0x0000, 0x0000, /*2C13*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F9, 0x0020, 0x0008, 0x0000, 0x0000, /*2C14*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21FA, 0x0020, 0x0008, 0x0000, 0x0000, /*2C15*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21FB, 0x0020, 0x0008, 0x0000, 0x0000, /*2C16*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21FC, 0x0020, 0x0008, 0x0000, 0x0000, /*2C17*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21FD, 0x0020, 0x0008, 0x0000, 0x0000, /*2C18*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21FE, 0x0020, 0x0008, 0x0000, 0x0000, /*2C19*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21FF, 0x0020, 0x0008, 0x0000, 0x0000, /*2C1A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2200, 0x0020, 0x0008, 0x0000, 0x0000, /*2C1B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2201, 0x0020, 0x0008, 0x0000, 0x0000, /*2C1C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2202, 0x0020, 0x0008, 0x0000, 0x0000, /*2C1D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2203, 0x0020, 0x0008, 0x0000, 0x0000, /*2C1E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2204, 0x0020, 0x0008, 0x0000, 0x0000, /*2C1F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2205, 0x0020, 0x0008, 0x0000, 0x0000, /*2C20*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2206, 0x0020, 0x0008, 0x0000, 0x0000, /*2C21*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2207, 0x0020, 0x0008, 0x0000, 0x0000, /*2C22*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2208, 0x0020, 0x0008, 0x0000, 0x0000, /*2C23*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2209, 0x0020, 0x0008, 0x0000, 0x0000, /*2C24*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x220A, 0x0020, 0x0008, 0x0000, 0x0000, /*2C25*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x220B, 0x0020, 0x0008, 0x0000, 0x0000, /*2C26*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x220C, 0x0020, 0x0008, 0x0000, 0x0000, /*2C27*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x220D, 0x0020, 0x0008, 0x0000, 0x0000, /*2C28*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x220E, 0x0020, 0x0008, 0x0000, 0x0000, /*2C29*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x220F, 0x0020, 0x0008, 0x0000, 0x0000, /*2C2A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2210, 0x0020, 0x0008, 0x0000, 0x0000, /*2C2B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2211, 0x0020, 0x0008, 0x0000, 0x0000, /*2C2C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2212, 0x0020, 0x0008, 0x0000, 0x0000, /*2C2D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2213, 0x0020, 0x0008, 0x0000, 0x0000, /*2C2E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xAC2F, 0x0000, /*2C2F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x21E5, 0x0020, 0x0002, 0x0000, 0x0000, /*2C30*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21E6, 0x0020, 0x0002, 0x0000, 0x0000, /*2C31*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21E7, 0x0020, 0x0002, 0x0000, 0x0000, /*2C32*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21E8, 0x0020, 0x0002, 0x0000, 0x0000, /*2C33*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21E9, 0x0020, 0x0002, 0x0000, 0x0000, /*2C34*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21EA, 0x0020, 0x0002, 0x0000, 0x0000, /*2C35*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21EB, 0x0020, 0x0002, 0x0000, 0x0000, /*2C36*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21EC, 0x0020, 0x0002, 0x0000, 0x0000, /*2C37*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21ED, 0x0020, 0x0002, 0x0000, 0x0000, /*2C38*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21EE, 0x0020, 0x0002, 0x0000, 0x0000, /*2C39*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21EF, 0x0020, 0x0002, 0x0000, 0x0000, /*2C3A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F0, 0x0020, 0x0002, 0x0000, 0x0000, /*2C3B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F1, 0x0020, 0x0002, 0x0000, 0x0000, /*2C3C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F2, 0x0020, 0x0002, 0x0000, 0x0000, /*2C3D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F3, 0x0020, 0x0002, 0x0000, 0x0000, /*2C3E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F4, 0x0020, 0x0002, 0x0000, 0x0000, /*2C3F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F5, 0x0020, 0x0002, 0x0000, 0x0000, /*2C40*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F6, 0x0020, 0x0002, 0x0000, 0x0000, /*2C41*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F7, 0x0020, 0x0002, 0x0000, 0x0000, /*2C42*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F8, 0x0020, 0x0002, 0x0000, 0x0000, /*2C43*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21F9, 0x0020, 0x0002, 0x0000, 0x0000, /*2C44*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21FA, 0x0020, 0x0002, 0x0000, 0x0000, /*2C45*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21FB, 0x0020, 0x0002, 0x0000, 0x0000, /*2C46*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21FC, 0x0020, 0x0002, 0x0000, 0x0000, /*2C47*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21FD, 0x0020, 0x0002, 0x0000, 0x0000, /*2C48*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21FE, 0x0020, 0x0002, 0x0000, 0x0000, /*2C49*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x21FF, 0x0020, 0x0002, 0x0000, 0x0000, /*2C4A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2200, 0x0020, 0x0002, 0x0000, 0x0000, /*2C4B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2201, 0x0020, 0x0002, 0x0000, 0x0000, /*2C4C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2202, 0x0020, 0x0002, 0x0000, 0x0000, /*2C4D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2203, 0x0020, 0x0002, 0x0000, 0x0000, /*2C4E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2204, 0x0020, 0x0002, 0x0000, 0x0000, /*2C4F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2205, 0x0020, 0x0002, 0x0000, 0x0000, /*2C50*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2206, 0x0020, 0x0002, 0x0000, 0x0000, /*2C51*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2207, 0x0020, 0x0002, 0x0000, 0x0000, /*2C52*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2208, 0x0020, 0x0002, 0x0000, 0x0000, /*2C53*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2209, 0x0020, 0x0002, 0x0000, 0x0000, /*2C54*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x220A, 0x0020, 0x0002, 0x0000, 0x0000, /*2C55*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x220B, 0x0020, 0x0002, 0x0000, 0x0000, /*2C56*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x220C, 0x0020, 0x0002, 0x0000, 0x0000, /*2C57*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x220D, 0x0020, 0x0002, 0x0000, 0x0000, /*2C58*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x220E, 0x0020, 0x0002, 0x0000, 0x0000, /*2C59*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x220F, 0x0020, 0x0002, 0x0000, 0x0000, /*2C5A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2210, 0x0020, 0x0002, 0x0000, 0x0000, /*2C5B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2211, 0x0020, 0x0002, 0x0000, 0x0000, /*2C5C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2212, 0x0020, 0x0002, 0x0000, 0x0000, /*2C5D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2213, 0x0020, 0x0002, 0x0000, 0x0000, /*2C5E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xAC5F, 0x0000, /*2C5F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D86, 0x0020, 0x0008, 0x0000, 0x0000, /*2C60*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D86, 0x0020, 0x0002, 0x0000, 0x0000, /*2C61*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D87, 0x0020, 0x0008, 0x0000, 0x0000, /*2C62*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E11, 0x0020, 0x0008, 0x0000, 0x0000, /*2C63*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E57, 0x0020, 0x0008, 0x0000, 0x0000, /*2C64*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C4C, 0x0020, 0x0002, 0x0000, 0x0000, /*2C65*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E9E, 0x0020, 0x0002, 0x0000, 0x0000, /*2C66*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D29, 0x0020, 0x0008, 0x0000, 0x0000, /*2C67*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D29, 0x0020, 0x0002, 0x0000, 0x0000, /*2C68*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D6F, 0x0020, 0x0008, 0x0000, 0x0000, /*2C69*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D6F, 0x0020, 0x0002, 0x0000, 0x0000, /*2C6A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F3C, 0x0020, 0x0008, 0x0000, 0x0000, /*2C6B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F3C, 0x0020, 0x0002, 0x0000, 0x0000, /*2C6C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C55, 0x0020, 0x0008, 0x0000, 0x0000, /*2C6D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB1, 0x0020, 0x0008, 0x0000, 0x0000, /*2C6E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C51, 0x0020, 0x0008, 0x0000, 0x0000, /*2C6F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C5B, 0x0020, 0x0008, 0x0000, 0x0000, /*2C70*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EEE, 0x0020, 0x0002, 0x0000, 0x0000, /*2C71*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFA, 0x0020, 0x0008, 0x0000, 0x0000, /*2C72*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFA, 0x0020, 0x0002, 0x0000, 0x0000, /*2C73*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EEF, 0x0020, 0x0002, 0x0000, 0x0000, /*2C74*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D2A, 0x0020, 0x0008, 0x0000, 0x0000, /*2C75*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D2A, 0x0020, 0x0002, 0x0000, 0x0000, /*2C76*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E20, 0x0020, 0x0002, 0x0000, 0x0000, /*2C77*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CB7, 0x0020, 0x0002, 0x0000, 0x0000, /*2C78*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E52, 0x0020, 0x0002, 0x0000, 0x0000, /*2C79*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DFC, 0x0020, 0x0002, 0x0000, 0x0000, /*2C7A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CBC, 0x0020, 0x0002, 0x0000, 0x0000, /*2C7B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D4C, 0x0020, 0x0015, 0x0000, 0x0000, /*2C7C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x001D, 0x0000, 0x0000, /*2C7D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E7C, 0x0020, 0x0008, 0x0000, 0x0000, /*2C7E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F38, 0x0020, 0x0008, 0x0000, 0x0000, /*2C7F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE6, 0x0020, 0x0008, 0x0000, 0x0000, /*2C80*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE6, 0x0020, 0x0002, 0x0000, 0x0000, /*2C81*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE7, 0x0020, 0x0008, 0x0000, 0x0000, /*2C82*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE7, 0x0020, 0x0002, 0x0000, 0x0000, /*2C83*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE8, 0x0020, 0x0008, 0x0000, 0x0000, /*2C84*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE8, 0x0020, 0x0002, 0x0000, 0x0000, /*2C85*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE9, 0x0020, 0x0008, 0x0000, 0x0000, /*2C86*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FE9, 0x0020, 0x0002, 0x0000, 0x0000, /*2C87*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FEA, 0x0020, 0x0008, 0x0000, 0x0000, /*2C88*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FEA, 0x0020, 0x0002, 0x0000, 0x0000, /*2C89*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FEC, 0x0020, 0x0008, 0x0000, 0x0000, /*2C8A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FEC, 0x0020, 0x0002, 0x0000, 0x0000, /*2C8B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FED, 0x0020, 0x0008, 0x0000, 0x0000, /*2C8C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FED, 0x0020, 0x0002, 0x0000, 0x0000, /*2C8D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FEE, 0x0020, 0x0008, 0x0000, 0x0000, /*2C8E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FEE, 0x0020, 0x0002, 0x0000, 0x0000, /*2C8F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FEF, 0x0020, 0x0008, 0x0000, 0x0000, /*2C90*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FEF, 0x0020, 0x0002, 0x0000, 0x0000, /*2C91*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF0, 0x0020, 0x0008, 0x0000, 0x0000, /*2C92*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF0, 0x0020, 0x0002, 0x0000, 0x0000, /*2C93*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF1, 0x0020, 0x0008, 0x0000, 0x0000, /*2C94*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF1, 0x0020, 0x0002, 0x0000, 0x0000, /*2C95*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF3, 0x0020, 0x0008, 0x0000, 0x0000, /*2C96*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF3, 0x0020, 0x0002, 0x0000, 0x0000, /*2C97*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF4, 0x0020, 0x0008, 0x0000, 0x0000, /*2C98*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF4, 0x0020, 0x0002, 0x0000, 0x0000, /*2C99*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF5, 0x0020, 0x0008, 0x0000, 0x0000, /*2C9A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF5, 0x0020, 0x0002, 0x0000, 0x0000, /*2C9B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF8, 0x0020, 0x0008, 0x0000, 0x0000, /*2C9C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF8, 0x0020, 0x0002, 0x0000, 0x0000, /*2C9D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF9, 0x0020, 0x0008, 0x0000, 0x0000, /*2C9E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF9, 0x0020, 0x0002, 0x0000, 0x0000, /*2C9F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FFA, 0x0020, 0x0008, 0x0000, 0x0000, /*2CA0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FFA, 0x0020, 0x0002, 0x0000, 0x0000, /*2CA1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FFB, 0x0020, 0x0008, 0x0000, 0x0000, /*2CA2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FFB, 0x0020, 0x0002, 0x0000, 0x0000, /*2CA3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FFC, 0x0020, 0x0008, 0x0000, 0x0000, /*2CA4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FFC, 0x0020, 0x0002, 0x0000, 0x0000, /*2CA5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FFD, 0x0020, 0x0008, 0x0000, 0x0000, /*2CA6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FFD, 0x0020, 0x0002, 0x0000, 0x0000, /*2CA7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FFE, 0x0020, 0x0008, 0x0000, 0x0000, /*2CA8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FFE, 0x0020, 0x0002, 0x0000, 0x0000, /*2CA9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FFF, 0x0020, 0x0008, 0x0000, 0x0000, /*2CAA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FFF, 0x0020, 0x0002, 0x0000, 0x0000, /*2CAB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2000, 0x0020, 0x0008, 0x0000, 0x0000, /*2CAC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2000, 0x0020, 0x0002, 0x0000, 0x0000, /*2CAD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2001, 0x0020, 0x0008, 0x0000, 0x0000, /*2CAE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2001, 0x0020, 0x0002, 0x0000, 0x0000, /*2CAF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2002, 0x0020, 0x0008, 0x0000, 0x0000, /*2CB0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2002, 0x0020, 0x0002, 0x0000, 0x0000, /*2CB1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x201D, 0x0020, 0x0008, 0x0000, 0x0000, /*2CB2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x201D, 0x0020, 0x0002, 0x0000, 0x0000, /*2CB3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x201E, 0x0020, 0x0008, 0x0000, 0x0000, /*2CB4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x201E, 0x0020, 0x0002, 0x0000, 0x0000, /*2CB5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FEB, 0x0020, 0x0008, 0x0000, 0x0000, /*2CB6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FEB, 0x0020, 0x0002, 0x0000, 0x0000, /*2CB7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF2, 0x0020, 0x0008, 0x0000, 0x0000, /*2CB8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF2, 0x0020, 0x0002, 0x0000, 0x0000, /*2CB9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF6, 0x0020, 0x0008, 0x0000, 0x0000, /*2CBA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF6, 0x0020, 0x0002, 0x0000, 0x0000, /*2CBB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF7, 0x0020, 0x0008, 0x0000, 0x0000, /*2CBC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF7, 0x0020, 0x0002, 0x0000, 0x0000, /*2CBD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2003, 0x0020, 0x0008, 0x0000, 0x0000, /*2CBE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2003, 0x0020, 0x0002, 0x0000, 0x0000, /*2CBF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2004, 0x0020, 0x0008, 0x0000, 0x0000, /*2CC0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2004, 0x0020, 0x0002, 0x0000, 0x0000, /*2CC1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2007, 0x0020, 0x0008, 0x0000, 0x0000, /*2CC2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2007, 0x0020, 0x0002, 0x0000, 0x0000, /*2CC3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2008, 0x0020, 0x0008, 0x0000, 0x0000, /*2CC4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2008, 0x0020, 0x0002, 0x0000, 0x0000, /*2CC5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2009, 0x0020, 0x0008, 0x0000, 0x0000, /*2CC6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2009, 0x0020, 0x0002, 0x0000, 0x0000, /*2CC7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x200D, 0x0020, 0x0008, 0x0000, 0x0000, /*2CC8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x200D, 0x0020, 0x0002, 0x0000, 0x0000, /*2CC9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x200F, 0x0020, 0x0008, 0x0000, 0x0000, /*2CCA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x200F, 0x0020, 0x0002, 0x0000, 0x0000, /*2CCB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2010, 0x0020, 0x0008, 0x0000, 0x0000, /*2CCC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2010, 0x0020, 0x0002, 0x0000, 0x0000, /*2CCD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2011, 0x0020, 0x0008, 0x0000, 0x0000, /*2CCE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2011, 0x0020, 0x0002, 0x0000, 0x0000, /*2CCF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2012, 0x0020, 0x0008, 0x0000, 0x0000, /*2CD0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2012, 0x0020, 0x0002, 0x0000, 0x0000, /*2CD1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2013, 0x0020, 0x0008, 0x0000, 0x0000, /*2CD2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2013, 0x0020, 0x0002, 0x0000, 0x0000, /*2CD3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2014, 0x0020, 0x0008, 0x0000, 0x0000, /*2CD4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2014, 0x0020, 0x0002, 0x0000, 0x0000, /*2CD5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2017, 0x0020, 0x0008, 0x0000, 0x0000, /*2CD6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2017, 0x0020, 0x0002, 0x0000, 0x0000, /*2CD7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2019, 0x0020, 0x0008, 0x0000, 0x0000, /*2CD8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2019, 0x0020, 0x0002, 0x0000, 0x0000, /*2CD9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x201A, 0x0020, 0x0008, 0x0000, 0x0000, /*2CDA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x201A, 0x0020, 0x0002, 0x0000, 0x0000, /*2CDB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x201B, 0x0020, 0x0008, 0x0000, 0x0000, /*2CDC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x201B, 0x0020, 0x0002, 0x0000, 0x0000, /*2CDD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x201F, 0x0020, 0x0008, 0x0000, 0x0000, /*2CDE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x201F, 0x0020, 0x0002, 0x0000, 0x0000, /*2CDF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2020, 0x0020, 0x0008, 0x0000, 0x0000, /*2CE0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2020, 0x0020, 0x0002, 0x0000, 0x0000, /*2CE1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2021, 0x0020, 0x0008, 0x0000, 0x0000, /*2CE2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2021, 0x0020, 0x0002, 0x0000, 0x0000, /*2CE3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1FF1, 0x0020, 0x0004, 0x1FE6, 0x0020, /*2CE4*/
  0x0004, 0x1FF0, 0x0020, 0x0004, 0x0003,
  0x0D96, 0x0020, 0x0002, 0x0000, 0x0000, /*2CE5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0D97, 0x0020, 0x0002, 0x0000, 0x0000, /*2CE6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0D98, 0x0020, 0x0002, 0x0000, 0x0000, /*2CE7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0D99, 0x0020, 0x0002, 0x0000, 0x0000, /*2CE8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0D9A, 0x0020, 0x0002, 0x0000, 0x0000, /*2CE9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0D9B, 0x0020, 0x0002, 0x0000, 0x0000, /*2CEA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2006, 0x0020, 0x0008, 0x0000, 0x0000, /*2CEB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2006, 0x0020, 0x0002, 0x0000, 0x0000, /*2CEC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2016, 0x0020, 0x0008, 0x0000, 0x0000, /*2CED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2016, 0x0020, 0x0002, 0x0000, 0x0000, /*2CEE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, /*2CEF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0023, 0x0002, 0x0000, 0x0000, /*2CF0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0022, 0x0002, 0x0000, 0x0000, /*2CF1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x200C, 0x0020, 0x0008, 0x0000, 0x0000, /*2CF2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x200C, 0x0020, 0x0002, 0x0000, 0x0000, /*2CF3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xACF4, 0x0000, /*2CF4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xACF5, 0x0000, /*2CF5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xACF6, 0x0000, /*2CF6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xACF7, 0x0000, /*2CF7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xACF8, 0x0000, /*2CF8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0281, 0x0020, 0x0002, 0x0000, 0x0000, /*2CF9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x026E, 0x0020, 0x0002, 0x0000, 0x0000, /*2CFA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x026F, 0x0020, 0x0002, 0x0000, 0x0000, /*2CFB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02F6, 0x0020, 0x0002, 0x0000, 0x0000, /*2CFC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1AE2, 0x0020, 0x0002, 0x0000, 0x0000, /*2CFD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0282, 0x0020, 0x0002, 0x0000, 0x0000, /*2CFE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02F7, 0x0020, 0x0002, 0x0000, 0x0000, /*2CFF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p02D[]= {
  0x223B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D00*/
  0x223D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D01*/
  0x223F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D02*/
  0x2241, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D03*/
  0x2243, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D04*/
  0x2245, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D05*/
  0x2247, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D06*/
  0x224B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D07*/
  0x224D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D08*/
  0x224F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D09*/
  0x2251, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D0A*/
  0x2253, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D0B*/
  0x2255, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D0C*/
  0x2259, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D0D*/
  0x225B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D0E*/
  0x225D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D0F*/
  0x225F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D10*/
  0x2261, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D11*/
  0x2263, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D12*/
  0x2267, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D13*/
  0x2269, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D14*/
  0x226B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D15*/
  0x226D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D16*/
  0x226F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D17*/
  0x2271, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D18*/
  0x2273, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D19*/
  0x2275, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D1A*/
  0x2277, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D1B*/
  0x2279, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D1C*/
  0x227B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D1D*/
  0x227D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D1E*/
  0x2281, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D1F*/
  0x2283, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D20*/
  0x2249, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D21*/
  0x2257, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D22*/
  0x2265, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D23*/
  0x227F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D24*/
  0x2285, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D25*/
  0xFBC0, 0x0020, 0x0002, 0xAD26, 0x0000, 0x0000, 0x0002, /*2D26*/
  0x2288, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D27*/
  0xFBC0, 0x0020, 0x0002, 0xAD28, 0x0000, 0x0000, 0x0002, /*2D28*/
  0xFBC0, 0x0020, 0x0002, 0xAD29, 0x0000, 0x0000, 0x0002, /*2D29*/
  0xFBC0, 0x0020, 0x0002, 0xAD2A, 0x0000, 0x0000, 0x0002, /*2D2A*/
  0xFBC0, 0x0020, 0x0002, 0xAD2B, 0x0000, 0x0000, 0x0002, /*2D2B*/
  0xFBC0, 0x0020, 0x0002, 0xAD2C, 0x0000, 0x0000, 0x0002, /*2D2C*/
  0x228D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D2D*/
  0xFBC0, 0x0020, 0x0002, 0xAD2E, 0x0000, 0x0000, 0x0002, /*2D2E*/
  0xFBC0, 0x0020, 0x0002, 0xAD2F, 0x0000, 0x0000, 0x0002, /*2D2F*/
  0x245D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D30*/
  0x245E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D31*/
  0x245F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D32*/
  0x2460, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D33*/
  0x2461, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D34*/
  0x2462, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D35*/
  0x2463, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D36*/
  0x2464, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D37*/
  0x2465, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D38*/
  0x2466, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D39*/
  0x2467, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D3A*/
  0x2468, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D3B*/
  0x246A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D3C*/
  0x246B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D3D*/
  0x246C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D3E*/
  0x246D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D3F*/
  0x246E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D40*/
  0x246F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D41*/
  0x2470, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D42*/
  0x2471, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D43*/
  0x2472, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D44*/
  0x2473, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D45*/
  0x2474, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D46*/
  0x2475, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D47*/
  0x2476, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D48*/
  0x2477, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D49*/
  0x2478, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D4A*/
  0x2479, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D4B*/
  0x247A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D4C*/
  0x247B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D4D*/
  0x247C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D4E*/
  0x247D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D4F*/
  0x247E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D50*/
  0x247F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D51*/
  0x2480, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D52*/
  0x2481, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D53*/
  0x2483, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D54*/
  0x2484, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D55*/
  0x2485, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D56*/
  0x2486, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D57*/
  0x2487, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D58*/
  0x2488, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D59*/
  0x2489, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D5A*/
  0x248A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D5B*/
  0x248B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D5C*/
  0x248C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D5D*/
  0x248D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D5E*/
  0x248E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D5F*/
  0x248F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D60*/
  0x2490, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D61*/
  0x2491, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D62*/
  0x2492, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D63*/
  0x2493, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D64*/
  0x2494, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D65*/
  0x2469, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D66*/
  0x2482, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D67*/
  0xFBC0, 0x0020, 0x0002, 0xAD68, 0x0000, 0x0000, 0x0002, /*2D68*/
  0xFBC0, 0x0020, 0x0002, 0xAD69, 0x0000, 0x0000, 0x0002, /*2D69*/
  0xFBC0, 0x0020, 0x0002, 0xAD6A, 0x0000, 0x0000, 0x0002, /*2D6A*/
  0xFBC0, 0x0020, 0x0002, 0xAD6B, 0x0000, 0x0000, 0x0002, /*2D6B*/
  0xFBC0, 0x0020, 0x0002, 0xAD6C, 0x0000, 0x0000, 0x0002, /*2D6C*/
  0xFBC0, 0x0020, 0x0002, 0xAD6D, 0x0000, 0x0000, 0x0002, /*2D6D*/
  0xFBC0, 0x0020, 0x0002, 0xAD6E, 0x0000, 0x0000, 0x0002, /*2D6E*/
  0x2495, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D6F*/
  0x0427, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D70*/
  0xFBC0, 0x0020, 0x0002, 0xAD71, 0x0000, 0x0000, 0x0002, /*2D71*/
  0xFBC0, 0x0020, 0x0002, 0xAD72, 0x0000, 0x0000, 0x0002, /*2D72*/
  0xFBC0, 0x0020, 0x0002, 0xAD73, 0x0000, 0x0000, 0x0002, /*2D73*/
  0xFBC0, 0x0020, 0x0002, 0xAD74, 0x0000, 0x0000, 0x0002, /*2D74*/
  0xFBC0, 0x0020, 0x0002, 0xAD75, 0x0000, 0x0000, 0x0002, /*2D75*/
  0xFBC0, 0x0020, 0x0002, 0xAD76, 0x0000, 0x0000, 0x0002, /*2D76*/
  0xFBC0, 0x0020, 0x0002, 0xAD77, 0x0000, 0x0000, 0x0002, /*2D77*/
  0xFBC0, 0x0020, 0x0002, 0xAD78, 0x0000, 0x0000, 0x0002, /*2D78*/
  0xFBC0, 0x0020, 0x0002, 0xAD79, 0x0000, 0x0000, 0x0002, /*2D79*/
  0xFBC0, 0x0020, 0x0002, 0xAD7A, 0x0000, 0x0000, 0x0002, /*2D7A*/
  0xFBC0, 0x0020, 0x0002, 0xAD7B, 0x0000, 0x0000, 0x0002, /*2D7B*/
  0xFBC0, 0x0020, 0x0002, 0xAD7C, 0x0000, 0x0000, 0x0002, /*2D7C*/
  0xFBC0, 0x0020, 0x0002, 0xAD7D, 0x0000, 0x0000, 0x0002, /*2D7D*/
  0xFBC0, 0x0020, 0x0002, 0xAD7E, 0x0000, 0x0000, 0x0002, /*2D7E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*2D7F*/
  0x24A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D80*/
  0x24BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D81*/
  0x24CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D82*/
  0x24D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D83*/
  0x24E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D84*/
  0x250A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D85*/
  0x251B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D86*/
  0x2524, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D87*/
  0x253A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D88*/
  0x2543, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D89*/
  0x254C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D8A*/
  0x257D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D8B*/
  0x259C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D8C*/
  0x25AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D8D*/
  0x25B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D8E*/
  0x25D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D8F*/
  0x25DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D90*/
  0x25EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D91*/
  0x261F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D92*/
  0x25CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D93*/
  0x25CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D94*/
  0x25CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D95*/
  0x25CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2D96*/
  0xFBC0, 0x0020, 0x0002, 0xAD97, 0x0000, 0x0000, 0x0002, /*2D97*/
  0xFBC0, 0x0020, 0x0002, 0xAD98, 0x0000, 0x0000, 0x0002, /*2D98*/
  0xFBC0, 0x0020, 0x0002, 0xAD99, 0x0000, 0x0000, 0x0002, /*2D99*/
  0xFBC0, 0x0020, 0x0002, 0xAD9A, 0x0000, 0x0000, 0x0002, /*2D9A*/
  0xFBC0, 0x0020, 0x0002, 0xAD9B, 0x0000, 0x0000, 0x0002, /*2D9B*/
  0xFBC0, 0x0020, 0x0002, 0xAD9C, 0x0000, 0x0000, 0x0002, /*2D9C*/
  0xFBC0, 0x0020, 0x0002, 0xAD9D, 0x0000, 0x0000, 0x0002, /*2D9D*/
  0xFBC0, 0x0020, 0x0002, 0xAD9E, 0x0000, 0x0000, 0x0002, /*2D9E*/
  0xFBC0, 0x0020, 0x0002, 0xAD9F, 0x0000, 0x0000, 0x0002, /*2D9F*/
  0x2623, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DA0*/
  0x2624, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DA1*/
  0x2625, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DA2*/
  0x2626, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DA3*/
  0x2627, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DA4*/
  0x2628, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DA5*/
  0x2629, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DA6*/
  0xFBC0, 0x0020, 0x0002, 0xADA7, 0x0000, 0x0000, 0x0002, /*2DA7*/
  0x262A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DA8*/
  0x262B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DA9*/
  0x262C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DAA*/
  0x262D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DAB*/
  0x262E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DAC*/
  0x262F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DAD*/
  0x2630, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DAE*/
  0xFBC0, 0x0020, 0x0002, 0xADAF, 0x0000, 0x0000, 0x0002, /*2DAF*/
  0x2631, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DB0*/
  0x2632, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DB1*/
  0x2633, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DB2*/
  0x2634, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DB3*/
  0x2635, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DB4*/
  0x2636, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DB5*/
  0x2637, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DB6*/
  0xFBC0, 0x0020, 0x0002, 0xADB7, 0x0000, 0x0000, 0x0002, /*2DB7*/
  0x2638, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DB8*/
  0x2639, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DB9*/
  0x263A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DBA*/
  0x263B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DBB*/
  0x263C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DBC*/
  0x263D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DBD*/
  0x263E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DBE*/
  0xFBC0, 0x0020, 0x0002, 0xADBF, 0x0000, 0x0000, 0x0002, /*2DBF*/
  0x263F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DC0*/
  0x2640, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DC1*/
  0x2641, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DC2*/
  0x2642, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DC3*/
  0x2643, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DC4*/
  0x2644, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DC5*/
  0x2645, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DC6*/
  0xFBC0, 0x0020, 0x0002, 0xADC7, 0x0000, 0x0000, 0x0002, /*2DC7*/
  0x2646, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DC8*/
  0x2647, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DC9*/
  0x2648, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DCA*/
  0x2649, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DCB*/
  0x264A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DCC*/
  0x264B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DCD*/
  0x264C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DCE*/
  0xFBC0, 0x0020, 0x0002, 0xADCF, 0x0000, 0x0000, 0x0002, /*2DCF*/
  0x264D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DD0*/
  0x264E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DD1*/
  0x264F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DD2*/
  0x2650, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DD3*/
  0x2651, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DD4*/
  0x2652, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DD5*/
  0x2653, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DD6*/
  0xFBC0, 0x0020, 0x0002, 0xADD7, 0x0000, 0x0000, 0x0002, /*2DD7*/
  0x2654, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DD8*/
  0x2655, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DD9*/
  0x2656, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DDA*/
  0x2657, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DDB*/
  0x2658, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DDC*/
  0x2659, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DDD*/
  0x265A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2DDE*/
  0xFBC0, 0x0020, 0x0002, 0xADDF, 0x0000, 0x0000, 0x0002, /*2DDF*/
  0x202E, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DE0*/
  0x2032, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DE1*/
  0x2036, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DE2*/
  0x204A, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DE3*/
  0x2062, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DE4*/
  0x206C, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DE5*/
  0x2096, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DE6*/
  0x20B0, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DE7*/
  0x20C3, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DE8*/
  0x20CC, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DE9*/
  0x20E7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DEA*/
  0x20EF, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DEB*/
  0x20FC, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DEC*/
  0x2105, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DED*/
  0x210E, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DEE*/
  0x2132, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DEF*/
  0x2159, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DF0*/
  0x2164, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DF1*/
  0x2183, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DF2*/
  0x2188, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DF3*/
  0x21D3, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DF4*/
  0x2105, 0x0020, 0x0004, 0x210E, 0x0020, 0x0004, 0x0002, /*2DF5*/
  0x2022, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DF6*/
  0x205A, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DF7*/
  0x2095, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DF8*/
  0x2129, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DF9*/
  0x21A0, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DFA*/
  0x21A9, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DFB*/
  0x21AE, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DFC*/
  0x21B8, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DFD*/
  0x21BD, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*2DFE*/
  0x21C7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001  /*2DFF*/
};

uint16 uca900_p02E[]= {
  0x03B6, 0x0020, 0x0002, 0x0000, 0x0000, /*2E00*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03B7, 0x0020, 0x0002, 0x0000, 0x0000, /*2E01*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x035B, 0x0020, 0x0002, 0x0000, 0x0000, /*2E02*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x035C, 0x0020, 0x0002, 0x0000, 0x0000, /*2E03*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x035D, 0x0020, 0x0002, 0x0000, 0x0000, /*2E04*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x035E, 0x0020, 0x0002, 0x0000, 0x0000, /*2E05*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03B8, 0x0020, 0x0002, 0x0000, 0x0000, /*2E06*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03B9, 0x0020, 0x0002, 0x0000, 0x0000, /*2E07*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03BA, 0x0020, 0x0002, 0x0000, 0x0000, /*2E08*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x035F, 0x0020, 0x0002, 0x0000, 0x0000, /*2E09*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0360, 0x0020, 0x0002, 0x0000, 0x0000, /*2E0A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03BB, 0x0020, 0x0002, 0x0000, 0x0000, /*2E0B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0361, 0x0020, 0x0002, 0x0000, 0x0000, /*2E0C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0362, 0x0020, 0x0002, 0x0000, 0x0000, /*2E0D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03BC, 0x0020, 0x0002, 0x0000, 0x0000, /*2E0E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03BD, 0x0020, 0x0002, 0x0000, 0x0000, /*2E0F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03BE, 0x0020, 0x0002, 0x0000, 0x0000, /*2E10*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03BF, 0x0020, 0x0002, 0x0000, 0x0000, /*2E11*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03C0, 0x0020, 0x0002, 0x0000, 0x0000, /*2E12*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03C1, 0x0020, 0x0002, 0x0000, 0x0000, /*2E13*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03C2, 0x0020, 0x0002, 0x0000, 0x0000, /*2E14*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03C3, 0x0020, 0x0002, 0x0000, 0x0000, /*2E15*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03C4, 0x0020, 0x0002, 0x0000, 0x0000, /*2E16*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x021C, 0x0020, 0x0002, 0x0000, 0x0000, /*2E17*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0276, 0x0020, 0x0002, 0x0000, 0x0000, /*2E18*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02F8, 0x0020, 0x0002, 0x0000, 0x0000, /*2E19*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03C5, 0x0020, 0x0002, 0x0000, 0x0000, /*2E1A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03C6, 0x0020, 0x0002, 0x0000, 0x0000, /*2E1B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0363, 0x0020, 0x0002, 0x0000, 0x0000, /*2E1C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0364, 0x0020, 0x0002, 0x0000, 0x0000, /*2E1D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03C7, 0x0020, 0x0002, 0x0000, 0x0000, /*2E1E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03C8, 0x0020, 0x0002, 0x0000, 0x0000, /*2E1F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0365, 0x0020, 0x0002, 0x0000, 0x0000, /*2E20*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0366, 0x0020, 0x0002, 0x0000, 0x0000, /*2E21*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0367, 0x0020, 0x0002, 0x0000, 0x0000, /*2E22*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0368, 0x0020, 0x0002, 0x0000, 0x0000, /*2E23*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0369, 0x0020, 0x0002, 0x0000, 0x0000, /*2E24*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x036A, 0x0020, 0x0002, 0x0000, 0x0000, /*2E25*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x036B, 0x0020, 0x0002, 0x0000, 0x0000, /*2E26*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x036C, 0x0020, 0x0002, 0x0000, 0x0000, /*2E27*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x036D, 0x0020, 0x0002, 0x0000, 0x0000, /*2E28*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x036E, 0x0020, 0x0002, 0x0000, 0x0000, /*2E29*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02F1, 0x0020, 0x0002, 0x0000, 0x0000, /*2E2A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02F2, 0x0020, 0x0002, 0x0000, 0x0000, /*2E2B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02F3, 0x0020, 0x0002, 0x0000, 0x0000, /*2E2C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02F4, 0x0020, 0x0002, 0x0000, 0x0000, /*2E2D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0268, 0x0020, 0x0002, 0x0000, 0x0000, /*2E2E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x218D, 0x0020, 0x0002, 0x0000, 0x0000, /*2E2F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0283, 0x0020, 0x0002, 0x0000, 0x0000, /*2E30*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x028C, 0x0020, 0x0002, 0x0000, 0x0000, /*2E31*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0224, 0x0020, 0x0002, 0x0000, 0x0000, /*2E32*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x028D, 0x0020, 0x0002, 0x0000, 0x0000, /*2E33*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0223, 0x0020, 0x0002, 0x0000, 0x0000, /*2E34*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0237, 0x0020, 0x0002, 0x0000, 0x0000, /*2E35*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03A1, 0x0020, 0x0002, 0x0000, 0x0000, /*2E36*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03A2, 0x0020, 0x0002, 0x0000, 0x0000, /*2E37*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03A3, 0x0020, 0x0002, 0x0000, 0x0000, /*2E38*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x038A, 0x0020, 0x0002, 0x0000, 0x0000, /*2E39*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0218, 0x0020, 0x0002, 0x0000, 0x0000, /*2E3A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0219, 0x0020, 0x0002, 0x0000, 0x0000, /*2E3B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0284, 0x0020, 0x0002, 0x0000, 0x0000, /*2E3C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x02F5, 0x0020, 0x0002, 0x0000, 0x0000, /*2E3D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0384, 0x0020, 0x0002, 0x0000, 0x0000, /*2E3E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x038D, 0x0020, 0x0002, 0x0000, 0x0000, /*2E3F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x021D, 0x0020, 0x0002, 0x0000, 0x0000, /*2E40*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0225, 0x0020, 0x0002, 0x0000, 0x0000, /*2E41*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0311, 0x0020, 0x0002, 0x0000, 0x0000, /*2E42*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x021B, 0x0020, 0x0002, 0x0000, 0x0000, /*2E43*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x03C9, 0x0020, 0x0002, 0x0000, 0x0000, /*2E44*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xAE45, 0x0000, /*2E45*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE46, 0x0000, /*2E46*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE47, 0x0000, /*2E47*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE48, 0x0000, /*2E48*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE49, 0x0000, /*2E49*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE4A, 0x0000, /*2E4A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE4B, 0x0000, /*2E4B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE4C, 0x0000, /*2E4C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE4D, 0x0000, /*2E4D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE4E, 0x0000, /*2E4E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE4F, 0x0000, /*2E4F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE50, 0x0000, /*2E50*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE51, 0x0000, /*2E51*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE52, 0x0000, /*2E52*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE53, 0x0000, /*2E53*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE54, 0x0000, /*2E54*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE55, 0x0000, /*2E55*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE56, 0x0000, /*2E56*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE57, 0x0000, /*2E57*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE58, 0x0000, /*2E58*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE59, 0x0000, /*2E59*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE5A, 0x0000, /*2E5A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE5B, 0x0000, /*2E5B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE5C, 0x0000, /*2E5C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE5D, 0x0000, /*2E5D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE5E, 0x0000, /*2E5E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE5F, 0x0000, /*2E5F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE60, 0x0000, /*2E60*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE61, 0x0000, /*2E61*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE62, 0x0000, /*2E62*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE63, 0x0000, /*2E63*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE64, 0x0000, /*2E64*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE65, 0x0000, /*2E65*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE66, 0x0000, /*2E66*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE67, 0x0000, /*2E67*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE68, 0x0000, /*2E68*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE69, 0x0000, /*2E69*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE6A, 0x0000, /*2E6A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE6B, 0x0000, /*2E6B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE6C, 0x0000, /*2E6C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE6D, 0x0000, /*2E6D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE6E, 0x0000, /*2E6E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE6F, 0x0000, /*2E6F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE70, 0x0000, /*2E70*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE71, 0x0000, /*2E71*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE72, 0x0000, /*2E72*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE73, 0x0000, /*2E73*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE74, 0x0000, /*2E74*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE75, 0x0000, /*2E75*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE76, 0x0000, /*2E76*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE77, 0x0000, /*2E77*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE78, 0x0000, /*2E78*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE79, 0x0000, /*2E79*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE7A, 0x0000, /*2E7A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE7B, 0x0000, /*2E7B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE7C, 0x0000, /*2E7C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE7D, 0x0000, /*2E7D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE7E, 0x0000, /*2E7E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE7F, 0x0000, /*2E7F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xCE36, 0x0000, /*2E80*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xD382, 0x0000, /*2E81*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xCE5B, 0x0000, /*2E82*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xCE5A, 0x0000, /*2E83*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xCE59, 0x0000, /*2E84*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xCEBB, 0x0000, /*2E85*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xD182, 0x0000, /*2E86*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xD1E0, 0x0000, /*2E87*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xD200, 0x0000, /*2E88*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xD202, 0x0000, /*2E89*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xD35C, 0x0000, /*2E8A*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xD369, 0x0000, /*2E8B*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xDC0F, 0x0000, /*2E8C*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xDC0F, 0x0000, /*2E8D*/
  0x0000, 0x0000, 0x0111, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xDC22, 0x0000, /*2E8E*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xDC23, 0x0000, /*2E8F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xDC22, 0x0000, /*2E90*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xDC23, 0x0000, /*2E91*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xDDF3, 0x0000, /*2E92*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xDE7A, 0x0000, /*2E93*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xDF51, 0x0000, /*2E94*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xDF50, 0x0000, /*2E95*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xDFC4, 0x0000, /*2E96*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xDFC3, 0x0000, /*2E97*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xE24C, 0x0000, /*2E98*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xE535, 0x0000, /*2E99*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAE9A, 0x0000, /*2E9A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xE5E1, 0x0000, /*2E9B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xE5E5, 0x0000, /*2E9C*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xE708, 0x0000, /*2E9D*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xEB7A, 0x0000, /*2E9E*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xEBCD, 0x0000, /*2E9F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xEC11, 0x0000, /*2EA0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xEC35, 0x0000, /*2EA1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xEC3A, 0x0000, /*2EA2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xF06C, 0x0000, /*2EA3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xF22B, 0x0000, /*2EA4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xF22B, 0x0000, /*2EA5*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xCE2C, 0x0000, /*2EA6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xF25B, 0x0000, /*2EA7*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xF2AD, 0x0000, /*2EA8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xF38B, 0x0000, /*2EA9*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xF58B, 0x0000, /*2EAA*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xF6EE, 0x0000, /*2EAB*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xF93A, 0x0000, /*2EAC*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xF93B, 0x0000, /*2EAD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xFAF9, 0x0000, /*2EAE*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xFCF9, 0x0000, /*2EAF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xFE9F, 0x0000, /*2EB0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xFF53, 0x0000, /*2EB1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xFF52, 0x0000, /*2EB2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0004, 0xFF53, 0x0000, /*2EB3*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xFF53, 0x0000, /*2EB4*/
  0x0000, 0x0000, 0x0111, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xFF52, 0x0000, /*2EB5*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xFF8A, 0x0000, /*2EB6*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xFF8A, 0x0000, /*2EB7*/
  0x0000, 0x0000, 0x0111, 0x0004, 0x0003,
  0xFB40, 0x0020, 0x0004, 0xFF8B, 0x0000, /*2EB8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x8002, 0x0000, /*2EB9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x8080, 0x0000, /*2EBA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x807F, 0x0000, /*2EBB*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x8089, 0x0000, /*2EBC*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x81FC, 0x0000, /*2EBD*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x8279, 0x0000, /*2EBE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x8279, 0x0000, /*2EBF*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x8279, 0x0000, /*2EC0*/
  0x0000, 0x0000, 0x0111, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x864E, 0x0000, /*2EC1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x8864, 0x0000, /*2EC2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x8980, 0x0000, /*2EC3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x897F, 0x0000, /*2EC4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x89C1, 0x0000, /*2EC5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x89D2, 0x0000, /*2EC6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x89D2, 0x0000, /*2EC7*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x8BA0, 0x0000, /*2EC8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x8D1D, 0x0000, /*2EC9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x8DB3, 0x0000, /*2ECA*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x8F66, 0x0000, /*2ECB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x8FB6, 0x0000, /*2ECC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x8FB6, 0x0000, /*2ECD*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x8FB6, 0x0000, /*2ECE*/
  0x0000, 0x0000, 0x0111, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x9091, 0x0000, /*2ECF*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x9485, 0x0000, /*2ED0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9577, 0x0000, /*2ED1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9578, 0x0000, /*2ED2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x957F, 0x0000, /*2ED3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x95E8, 0x0000, /*2ED4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x961C, 0x0000, /*2ED5*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x961D, 0x0000, /*2ED6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x96E8, 0x0000, /*2ED7*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x9752, 0x0000, /*2ED8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x97E6, 0x0000, /*2ED9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9875, 0x0000, /*2EDA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x98CE, 0x0000, /*2EDB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x98DE, 0x0000, /*2EDC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x98DF, 0x0000, /*2EDD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x98E0, 0x0000, /*2EDE*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x98E0, 0x0000, /*2EDF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9963, 0x0000, /*2EE0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9996, 0x0000, /*2EE1*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x9A6C, 0x0000, /*2EE2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9AA8, 0x0000, /*2EE3*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x9B3C, 0x0000, /*2EE4*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x9C7C, 0x0000, /*2EE5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9E1F, 0x0000, /*2EE6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9E75, 0x0000, /*2EE7*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x9EA6, 0x0000, /*2EE8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9EC4, 0x0000, /*2EE9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9EFE, 0x0000, /*2EEA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9F4A, 0x0000, /*2EEB*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x9F50, 0x0000, /*2EEC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9F52, 0x0000, /*2EED*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x9F7F, 0x0000, /*2EEE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9F8D, 0x0000, /*2EEF*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x9F99, 0x0000, /*2EF0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0004, 0x9F9C, 0x0000, /*2EF1*/
  0x0000, 0x0000, 0x0110, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x9F9C, 0x0000, /*2EF2*/
  0x0000, 0x0000, 0x0111, 0x0004, 0x0003,
  0xFB41, 0x0020, 0x0004, 0x9F9F, 0x0000, /*2EF3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAEF4, 0x0000, /*2EF4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAEF5, 0x0000, /*2EF5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAEF6, 0x0000, /*2EF6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAEF7, 0x0000, /*2EF7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAEF8, 0x0000, /*2EF8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAEF9, 0x0000, /*2EF9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAEFA, 0x0000, /*2EFA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAEFB, 0x0000, /*2EFB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAEFC, 0x0000, /*2EFC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAEFD, 0x0000, /*2EFD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAEFE, 0x0000, /*2EFE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC0, 0x0020, 0x0002, 0xAEFF, 0x0000, /*2EFF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002
};

uint16 uca900_p02F[]= {
  0xFB40, 0x0020, 0x0004, 0xCE00, 0x0000, 0x0000, 0x0002, /*2F00*/
  0xFB40, 0x0020, 0x0004, 0xCE28, 0x0000, 0x0000, 0x0002, /*2F01*/
  0xFB40, 0x0020, 0x0004, 0xCE36, 0x0000, 0x0000, 0x0002, /*2F02*/
  0xFB40, 0x0020, 0x0004, 0xCE3F, 0x0000, 0x0000, 0x0002, /*2F03*/
  0xFB40, 0x0020, 0x0004, 0xCE59, 0x0000, 0x0000, 0x0002, /*2F04*/
  0xFB40, 0x0020, 0x0004, 0xCE85, 0x0000, 0x0000, 0x0002, /*2F05*/
  0xFB40, 0x0020, 0x0004, 0xCE8C, 0x0000, 0x0000, 0x0002, /*2F06*/
  0xFB40, 0x0020, 0x0004, 0xCEA0, 0x0000, 0x0000, 0x0002, /*2F07*/
  0xFB40, 0x0020, 0x0004, 0xCEBA, 0x0000, 0x0000, 0x0002, /*2F08*/
  0xFB40, 0x0020, 0x0004, 0xD13F, 0x0000, 0x0000, 0x0002, /*2F09*/
  0xFB40, 0x0020, 0x0004, 0xD165, 0x0000, 0x0000, 0x0002, /*2F0A*/
  0xFB40, 0x0020, 0x0004, 0xD16B, 0x0000, 0x0000, 0x0002, /*2F0B*/
  0xFB40, 0x0020, 0x0004, 0xD182, 0x0000, 0x0000, 0x0002, /*2F0C*/
  0xFB40, 0x0020, 0x0004, 0xD196, 0x0000, 0x0000, 0x0002, /*2F0D*/
  0xFB40, 0x0020, 0x0004, 0xD1AB, 0x0000, 0x0000, 0x0002, /*2F0E*/
  0xFB40, 0x0020, 0x0004, 0xD1E0, 0x0000, 0x0000, 0x0002, /*2F0F*/
  0xFB40, 0x0020, 0x0004, 0xD1F5, 0x0000, 0x0000, 0x0002, /*2F10*/
  0xFB40, 0x0020, 0x0004, 0xD200, 0x0000, 0x0000, 0x0002, /*2F11*/
  0xFB40, 0x0020, 0x0004, 0xD29B, 0x0000, 0x0000, 0x0002, /*2F12*/
  0xFB40, 0x0020, 0x0004, 0xD2F9, 0x0000, 0x0000, 0x0002, /*2F13*/
  0xFB40, 0x0020, 0x0004, 0xD315, 0x0000, 0x0000, 0x0002, /*2F14*/
  0xFB40, 0x0020, 0x0004, 0xD31A, 0x0000, 0x0000, 0x0002, /*2F15*/
  0xFB40, 0x0020, 0x0004, 0xD338, 0x0000, 0x0000, 0x0002, /*2F16*/
  0xFB40, 0x0020, 0x0004, 0xD341, 0x0000, 0x0000, 0x0002, /*2F17*/
  0xFB40, 0x0020, 0x0004, 0xD35C, 0x0000, 0x0000, 0x0002, /*2F18*/
  0xFB40, 0x0020, 0x0004, 0xD369, 0x0000, 0x0000, 0x0002, /*2F19*/
  0xFB40, 0x0020, 0x0004, 0xD382, 0x0000, 0x0000, 0x0002, /*2F1A*/
  0xFB40, 0x0020, 0x0004, 0xD3B6, 0x0000, 0x0000, 0x0002, /*2F1B*/
  0xFB40, 0x0020, 0x0004, 0xD3C8, 0x0000, 0x0000, 0x0002, /*2F1C*/
  0xFB40, 0x0020, 0x0004, 0xD3E3, 0x0000, 0x0000, 0x0002, /*2F1D*/
  0xFB40, 0x0020, 0x0004, 0xD6D7, 0x0000, 0x0000, 0x0002, /*2F1E*/
  0xFB40, 0x0020, 0x0004, 0xD71F, 0x0000, 0x0000, 0x0002, /*2F1F*/
  0xFB40, 0x0020, 0x0004, 0xD8EB, 0x0000, 0x0000, 0x0002, /*2F20*/
  0xFB40, 0x0020, 0x0004, 0xD902, 0x0000, 0x0000, 0x0002, /*2F21*/
  0xFB40, 0x0020, 0x0004, 0xD90A, 0x0000, 0x0000, 0x0002, /*2F22*/
  0xFB40, 0x0020, 0x0004, 0xD915, 0x0000, 0x0000, 0x0002, /*2F23*/
  0xFB40, 0x0020, 0x0004, 0xD927, 0x0000, 0x0000, 0x0002, /*2F24*/
  0xFB40, 0x0020, 0x0004, 0xD973, 0x0000, 0x0000, 0x0002, /*2F25*/
  0xFB40, 0x0020, 0x0004, 0xDB50, 0x0000, 0x0000, 0x0002, /*2F26*/
  0xFB40, 0x0020, 0x0004, 0xDB80, 0x0000, 0x0000, 0x0002, /*2F27*/
  0xFB40, 0x0020, 0x0004, 0xDBF8, 0x0000, 0x0000, 0x0002, /*2F28*/
  0xFB40, 0x0020, 0x0004, 0xDC0F, 0x0000, 0x0000, 0x0002, /*2F29*/
  0xFB40, 0x0020, 0x0004, 0xDC22, 0x0000, 0x0000, 0x0002, /*2F2A*/
  0xFB40, 0x0020, 0x0004, 0xDC38, 0x0000, 0x0000, 0x0002, /*2F2B*/
  0xFB40, 0x0020, 0x0004, 0xDC6E, 0x0000, 0x0000, 0x0002, /*2F2C*/
  0xFB40, 0x0020, 0x0004, 0xDC71, 0x0000, 0x0000, 0x0002, /*2F2D*/
  0xFB40, 0x0020, 0x0004, 0xDDDB, 0x0000, 0x0000, 0x0002, /*2F2E*/
  0xFB40, 0x0020, 0x0004, 0xDDE5, 0x0000, 0x0000, 0x0002, /*2F2F*/
  0xFB40, 0x0020, 0x0004, 0xDDF1, 0x0000, 0x0000, 0x0002, /*2F30*/
  0xFB40, 0x0020, 0x0004, 0xDDFE, 0x0000, 0x0000, 0x0002, /*2F31*/
  0xFB40, 0x0020, 0x0004, 0xDE72, 0x0000, 0x0000, 0x0002, /*2F32*/
  0xFB40, 0x0020, 0x0004, 0xDE7A, 0x0000, 0x0000, 0x0002, /*2F33*/
  0xFB40, 0x0020, 0x0004, 0xDE7F, 0x0000, 0x0000, 0x0002, /*2F34*/
  0xFB40, 0x0020, 0x0004, 0xDEF4, 0x0000, 0x0000, 0x0002, /*2F35*/
  0xFB40, 0x0020, 0x0004, 0xDEFE, 0x0000, 0x0000, 0x0002, /*2F36*/
  0xFB40, 0x0020, 0x0004, 0xDF0B, 0x0000, 0x0000, 0x0002, /*2F37*/
  0xFB40, 0x0020, 0x0004, 0xDF13, 0x0000, 0x0000, 0x0002, /*2F38*/
  0xFB40, 0x0020, 0x0004, 0xDF50, 0x0000, 0x0000, 0x0002, /*2F39*/
  0xFB40, 0x0020, 0x0004, 0xDF61, 0x0000, 0x0000, 0x0002, /*2F3A*/
  0xFB40, 0x0020, 0x0004, 0xDF73, 0x0000, 0x0000, 0x0002, /*2F3B*/
  0xFB40, 0x0020, 0x0004, 0xDFC3, 0x0000, 0x0000, 0x0002, /*2F3C*/
  0xFB40, 0x0020, 0x0004, 0xE208, 0x0000, 0x0000, 0x0002, /*2F3D*/
  0xFB40, 0x0020, 0x0004, 0xE236, 0x0000, 0x0000, 0x0002, /*2F3E*/
  0xFB40, 0x0020, 0x0004, 0xE24B, 0x0000, 0x0000, 0x0002, /*2F3F*/
  0xFB40, 0x0020, 0x0004, 0xE52F, 0x0000, 0x0000, 0x0002, /*2F40*/
  0xFB40, 0x0020, 0x0004, 0xE534, 0x0000, 0x0000, 0x0002, /*2F41*/
  0xFB40, 0x0020, 0x0004, 0xE587, 0x0000, 0x0000, 0x0002, /*2F42*/
  0xFB40, 0x0020, 0x0004, 0xE597, 0x0000, 0x0000, 0x0002, /*2F43*/
  0xFB40, 0x0020, 0x0004, 0xE5A4, 0x0000, 0x0000, 0x0002, /*2F44*/
  0xFB40, 0x0020, 0x0004, 0xE5B9, 0x0000, 0x0000, 0x0002, /*2F45*/
  0xFB40, 0x0020, 0x0004, 0xE5E0, 0x0000, 0x0000, 0x0002, /*2F46*/
  0xFB40, 0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0002, /*2F47*/
  0xFB40, 0x0020, 0x0004, 0xE6F0, 0x0000, 0x0000, 0x0002, /*2F48*/
  0xFB40, 0x0020, 0x0004, 0xE708, 0x0000, 0x0000, 0x0002, /*2F49*/
  0xFB40, 0x0020, 0x0004, 0xE728, 0x0000, 0x0000, 0x0002, /*2F4A*/
  0xFB40, 0x0020, 0x0004, 0xEB20, 0x0000, 0x0000, 0x0002, /*2F4B*/
  0xFB40, 0x0020, 0x0004, 0xEB62, 0x0000, 0x0000, 0x0002, /*2F4C*/
  0xFB40, 0x0020, 0x0004, 0xEB79, 0x0000, 0x0000, 0x0002, /*2F4D*/
  0xFB40, 0x0020, 0x0004, 0xEBB3, 0x0000, 0x0000, 0x0002, /*2F4E*/
  0xFB40, 0x0020, 0x0004, 0xEBCB, 0x0000, 0x0000, 0x0002, /*2F4F*/
  0xFB40, 0x0020, 0x0004, 0xEBD4, 0x0000, 0x0000, 0x0002, /*2F50*/
  0xFB40, 0x0020, 0x0004, 0xEBDB, 0x0000, 0x0000, 0x0002, /*2F51*/
  0xFB40, 0x0020, 0x0004, 0xEC0F, 0x0000, 0x0000, 0x0002, /*2F52*/
  0xFB40, 0x0020, 0x0004, 0xEC14, 0x0000, 0x0000, 0x0002, /*2F53*/
  0xFB40, 0x0020, 0x0004, 0xEC34, 0x0000, 0x0000, 0x0002, /*2F54*/
  0xFB40, 0x0020, 0x0004, 0xF06B, 0x0000, 0x0000, 0x0002, /*2F55*/
  0xFB40, 0x0020, 0x0004, 0xF22A, 0x0000, 0x0000, 0x0002, /*2F56*/
  0xFB40, 0x0020, 0x0004, 0xF236, 0x0000, 0x0000, 0x0002, /*2F57*/
  0xFB40, 0x0020, 0x0004, 0xF23B, 0x0000, 0x0000, 0x0002, /*2F58*/
  0xFB40, 0x0020, 0x0004, 0xF23F, 0x0000, 0x0000, 0x0002, /*2F59*/
  0xFB40, 0x0020, 0x0004, 0xF247, 0x0000, 0x0000, 0x0002, /*2F5A*/
  0xFB40, 0x0020, 0x0004, 0xF259, 0x0000, 0x0000, 0x0002, /*2F5B*/
  0xFB40, 0x0020, 0x0004, 0xF25B, 0x0000, 0x0000, 0x0002, /*2F5C*/
  0xFB40, 0x0020, 0x0004, 0xF2AC, 0x0000, 0x0000, 0x0002, /*2F5D*/
  0xFB40, 0x0020, 0x0004, 0xF384, 0x0000, 0x0000, 0x0002, /*2F5E*/
  0xFB40, 0x0020, 0x0004, 0xF389, 0x0000, 0x0000, 0x0002, /*2F5F*/
  0xFB40, 0x0020, 0x0004, 0xF4DC, 0x0000, 0x0000, 0x0002, /*2F60*/
  0xFB40, 0x0020, 0x0004, 0xF4E6, 0x0000, 0x0000, 0x0002, /*2F61*/
  0xFB40, 0x0020, 0x0004, 0xF518, 0x0000, 0x0000, 0x0002, /*2F62*/
  0xFB40, 0x0020, 0x0004, 0xF51F, 0x0000, 0x0000, 0x0002, /*2F63*/
  0xFB40, 0x0020, 0x0004, 0xF528, 0x0000, 0x0000, 0x0002, /*2F64*/
  0xFB40, 0x0020, 0x0004, 0xF530, 0x0000, 0x0000, 0x0002, /*2F65*/
  0xFB40, 0x0020, 0x0004, 0xF58B, 0x0000, 0x0000, 0x0002, /*2F66*/
  0xFB40, 0x0020, 0x0004, 0xF592, 0x0000, 0x0000, 0x0002, /*2F67*/
  0xFB40, 0x0020, 0x0004, 0xF676, 0x0000, 0x0000, 0x0002, /*2F68*/
  0xFB40, 0x0020, 0x0004, 0xF67D, 0x0000, 0x0000, 0x0002, /*2F69*/
  0xFB40, 0x0020, 0x0004, 0xF6AE, 0x0000, 0x0000, 0x0002, /*2F6A*/
  0xFB40, 0x0020, 0x0004, 0xF6BF, 0x0000, 0x0000, 0x0002, /*2F6B*/
  0xFB40, 0x0020, 0x0004, 0xF6EE, 0x0000, 0x0000, 0x0002, /*2F6C*/
  0xFB40, 0x0020, 0x0004, 0xF7DB, 0x0000, 0x0000, 0x0002, /*2F6D*/
  0xFB40, 0x0020, 0x0004, 0xF7E2, 0x0000, 0x0000, 0x0002, /*2F6E*/
  0xFB40, 0x0020, 0x0004, 0xF7F3, 0x0000, 0x0000, 0x0002, /*2F6F*/
  0xFB40, 0x0020, 0x0004, 0xF93A, 0x0000, 0x0000, 0x0002, /*2F70*/
  0xFB40, 0x0020, 0x0004, 0xF9B8, 0x0000, 0x0000, 0x0002, /*2F71*/
  0xFB40, 0x0020, 0x0004, 0xF9BE, 0x0000, 0x0000, 0x0002, /*2F72*/
  0xFB40, 0x0020, 0x0004, 0xFA74, 0x0000, 0x0000, 0x0002, /*2F73*/
  0xFB40, 0x0020, 0x0004, 0xFACB, 0x0000, 0x0000, 0x0002, /*2F74*/
  0xFB40, 0x0020, 0x0004, 0xFAF9, 0x0000, 0x0000, 0x0002, /*2F75*/
  0xFB40, 0x0020, 0x0004, 0xFC73, 0x0000, 0x0000, 0x0002, /*2F76*/
  0xFB40, 0x0020, 0x0004, 0xFCF8, 0x0000, 0x0000, 0x0002, /*2F77*/
  0xFB40, 0x0020, 0x0004, 0xFF36, 0x0000, 0x0000, 0x0002, /*2F78*/
  0xFB40, 0x0020, 0x0004, 0xFF51, 0x0000, 0x0000, 0x0002, /*2F79*/
  0xFB40, 0x0020, 0x0004, 0xFF8A, 0x0000, 0x0000, 0x0002, /*2F7A*/
  0xFB40, 0x0020, 0x0004, 0xFFBD, 0x0000, 0x0000, 0x0002, /*2F7B*/
  0xFB41, 0x0020, 0x0004, 0x8001, 0x0000, 0x0000, 0x0002, /*2F7C*/
  0xFB41, 0x0020, 0x0004, 0x800C, 0x0000, 0x0000, 0x0002, /*2F7D*/
  0xFB41, 0x0020, 0x0004, 0x8012, 0x0000, 0x0000, 0x0002, /*2F7E*/
  0xFB41, 0x0020, 0x0004, 0x8033, 0x0000, 0x0000, 0x0002, /*2F7F*/
  0xFB41, 0x0020, 0x0004, 0x807F, 0x0000, 0x0000, 0x0002, /*2F80*/
  0xFB41, 0x0020, 0x0004, 0x8089, 0x0000, 0x0000, 0x0002, /*2F81*/
  0xFB41, 0x0020, 0x0004, 0x81E3, 0x0000, 0x0000, 0x0002, /*2F82*/
  0xFB41, 0x0020, 0x0004, 0x81EA, 0x0000, 0x0000, 0x0002, /*2F83*/
  0xFB41, 0x0020, 0x0004, 0x81F3, 0x0000, 0x0000, 0x0002, /*2F84*/
  0xFB41, 0x0020, 0x0004, 0x81FC, 0x0000, 0x0000, 0x0002, /*2F85*/
  0xFB41, 0x0020, 0x0004, 0x820C, 0x0000, 0x0000, 0x0002, /*2F86*/
  0xFB41, 0x0020, 0x0004, 0x821B, 0x0000, 0x0000, 0x0002, /*2F87*/
  0xFB41, 0x0020, 0x0004, 0x821F, 0x0000, 0x0000, 0x0002, /*2F88*/
  0xFB41, 0x0020, 0x0004, 0x826E, 0x0000, 0x0000, 0x0002, /*2F89*/
  0xFB41, 0x0020, 0x0004, 0x8272, 0x0000, 0x0000, 0x0002, /*2F8A*/
  0xFB41, 0x0020, 0x0004, 0x8278, 0x0000, 0x0000, 0x0002, /*2F8B*/
  0xFB41, 0x0020, 0x0004, 0x864D, 0x0000, 0x0000, 0x0002, /*2F8C*/
  0xFB41, 0x0020, 0x0004, 0x866B, 0x0000, 0x0000, 0x0002, /*2F8D*/
  0xFB41, 0x0020, 0x0004, 0x8840, 0x0000, 0x0000, 0x0002, /*2F8E*/
  0xFB41, 0x0020, 0x0004, 0x884C, 0x0000, 0x0000, 0x0002, /*2F8F*/
  0xFB41, 0x0020, 0x0004, 0x8863, 0x0000, 0x0000, 0x0002, /*2F90*/
  0xFB41, 0x0020, 0x0004, 0x897E, 0x0000, 0x0000, 0x0002, /*2F91*/
  0xFB41, 0x0020, 0x0004, 0x898B, 0x0000, 0x0000, 0x0002, /*2F92*/
  0xFB41, 0x0020, 0x0004, 0x89D2, 0x0000, 0x0000, 0x0002, /*2F93*/
  0xFB41, 0x0020, 0x0004, 0x8A00, 0x0000, 0x0000, 0x0002, /*2F94*/
  0xFB41, 0x0020, 0x0004, 0x8C37, 0x0000, 0x0000, 0x0002, /*2F95*/
  0xFB41, 0x0020, 0x0004, 0x8C46, 0x0000, 0x0000, 0x0002, /*2F96*/
  0xFB41, 0x0020, 0x0004, 0x8C55, 0x0000, 0x0000, 0x0002, /*2F97*/
  0xFB41, 0x0020, 0x0004, 0x8C78, 0x0000, 0x0000, 0x0002, /*2F98*/
  0xFB41, 0x0020, 0x0004, 0x8C9D, 0x0000, 0x0000, 0x0002, /*2F99*/
  0xFB41, 0x0020, 0x0004, 0x8D64, 0x0000, 0x0000, 0x0002, /*2F9A*/
  0xFB41, 0x0020, 0x0004, 0x8D70, 0x0000, 0x0000, 0x0002, /*2F9B*/
  0xFB41, 0x0020, 0x0004, 0x8DB3, 0x0000, 0x0000, 0x0002, /*2F9C*/
  0xFB41, 0x0020, 0x0004, 0x8EAB, 0x0000, 0x0000, 0x0002, /*2F9D*/
  0xFB41, 0x0020, 0x0004, 0x8ECA, 0x0000, 0x0000, 0x0002, /*2F9E*/
  0xFB41, 0x0020, 0x0004, 0x8F9B, 0x0000, 0x0000, 0x0002, /*2F9F*/
  0xFB41, 0x0020, 0x0004, 0x8FB0, 0x0000, 0x0000, 0x0002, /*2FA0*/
  0xFB41, 0x0020, 0x0004, 0x8FB5, 0x0000, 0x0000, 0x0002, /*2FA1*/
  0xFB41, 0x0020, 0x0004, 0x9091, 0x0000, 0x0000, 0x0002, /*2FA2*/
  0xFB41, 0x0020, 0x0004, 0x9149, 0x0000, 0x0000, 0x0002, /*2FA3*/
  0xFB41, 0x0020, 0x0004, 0x91C6, 0x0000, 0x0000, 0x0002, /*2FA4*/
  0xFB41, 0x0020, 0x0004, 0x91CC, 0x0000, 0x0000, 0x0002, /*2FA5*/
  0xFB41, 0x0020, 0x0004, 0x91D1, 0x0000, 0x0000, 0x0002, /*2FA6*/
  0xFB41, 0x0020, 0x0004, 0x9577, 0x0000, 0x0000, 0x0002, /*2FA7*/
  0xFB41, 0x0020, 0x0004, 0x9580, 0x0000, 0x0000, 0x0002, /*2FA8*/
  0xFB41, 0x0020, 0x0004, 0x961C, 0x0000, 0x0000, 0x0002, /*2FA9*/
  0xFB41, 0x0020, 0x0004, 0x96B6, 0x0000, 0x0000, 0x0002, /*2FAA*/
  0xFB41, 0x0020, 0x0004, 0x96B9, 0x0000, 0x0000, 0x0002, /*2FAB*/
  0xFB41, 0x0020, 0x0004, 0x96E8, 0x0000, 0x0000, 0x0002, /*2FAC*/
  0xFB41, 0x0020, 0x0004, 0x9751, 0x0000, 0x0000, 0x0002, /*2FAD*/
  0xFB41, 0x0020, 0x0004, 0x975E, 0x0000, 0x0000, 0x0002, /*2FAE*/
  0xFB41, 0x0020, 0x0004, 0x9762, 0x0000, 0x0000, 0x0002, /*2FAF*/
  0xFB41, 0x0020, 0x0004, 0x9769, 0x0000, 0x0000, 0x0002, /*2FB0*/
  0xFB41, 0x0020, 0x0004, 0x97CB, 0x0000, 0x0000, 0x0002, /*2FB1*/
  0xFB41, 0x0020, 0x0004, 0x97ED, 0x0000, 0x0000, 0x0002, /*2FB2*/
  0xFB41, 0x0020, 0x0004, 0x97F3, 0x0000, 0x0000, 0x0002, /*2FB3*/
  0xFB41, 0x0020, 0x0004, 0x9801, 0x0000, 0x0000, 0x0002, /*2FB4*/
  0xFB41, 0x0020, 0x0004, 0x98A8, 0x0000, 0x0000, 0x0002, /*2FB5*/
  0xFB41, 0x0020, 0x0004, 0x98DB, 0x0000, 0x0000, 0x0002, /*2FB6*/
  0xFB41, 0x0020, 0x0004, 0x98DF, 0x0000, 0x0000, 0x0002, /*2FB7*/
  0xFB41, 0x0020, 0x0004, 0x9996, 0x0000, 0x0000, 0x0002, /*2FB8*/
  0xFB41, 0x0020, 0x0004, 0x9999, 0x0000, 0x0000, 0x0002, /*2FB9*/
  0xFB41, 0x0020, 0x0004, 0x99AC, 0x0000, 0x0000, 0x0002, /*2FBA*/
  0xFB41, 0x0020, 0x0004, 0x9AA8, 0x0000, 0x0000, 0x0002, /*2FBB*/
  0xFB41, 0x0020, 0x0004, 0x9AD8, 0x0000, 0x0000, 0x0002, /*2FBC*/
  0xFB41, 0x0020, 0x0004, 0x9ADF, 0x0000, 0x0000, 0x0002, /*2FBD*/
  0xFB41, 0x0020, 0x0004, 0x9B25, 0x0000, 0x0000, 0x0002, /*2FBE*/
  0xFB41, 0x0020, 0x0004, 0x9B2F, 0x0000, 0x0000, 0x0002, /*2FBF*/
  0xFB41, 0x0020, 0x0004, 0x9B32, 0x0000, 0x0000, 0x0002, /*2FC0*/
  0xFB41, 0x0020, 0x0004, 0x9B3C, 0x0000, 0x0000, 0x0002, /*2FC1*/
  0xFB41, 0x0020, 0x0004, 0x9B5A, 0x0000, 0x0000, 0x0002, /*2FC2*/
  0xFB41, 0x0020, 0x0004, 0x9CE5, 0x0000, 0x0000, 0x0002, /*2FC3*/
  0xFB41, 0x0020, 0x0004, 0x9E75, 0x0000, 0x0000, 0x0002, /*2FC4*/
  0xFB41, 0x0020, 0x0004, 0x9E7F, 0x0000, 0x0000, 0x0002, /*2FC5*/
  0xFB41, 0x0020, 0x0004, 0x9EA5, 0x0000, 0x0000, 0x0002, /*2FC6*/
  0xFB41, 0x0020, 0x0004, 0x9EBB, 0x0000, 0x0000, 0x0002, /*2FC7*/
  0xFB41, 0x0020, 0x0004, 0x9EC3, 0x0000, 0x0000, 0x0002, /*2FC8*/
  0xFB41, 0x0020, 0x0004, 0x9ECD, 0x0000, 0x0000, 0x0002, /*2FC9*/
  0xFB41, 0x0020, 0x0004, 0x9ED1, 0x0000, 0x0000, 0x0002, /*2FCA*/
  0xFB41, 0x0020, 0x0004, 0x9EF9, 0x0000, 0x0000, 0x0002, /*2FCB*/
  0xFB41, 0x0020, 0x0004, 0x9EFD, 0x0000, 0x0000, 0x0002, /*2FCC*/
  0xFB41, 0x0020, 0x0004, 0x9F0E, 0x0000, 0x0000, 0x0002, /*2FCD*/
  0xFB41, 0x0020, 0x0004, 0x9F13, 0x0000, 0x0000, 0x0002, /*2FCE*/
  0xFB41, 0x0020, 0x0004, 0x9F20, 0x0000, 0x0000, 0x0002, /*2FCF*/
  0xFB41, 0x0020, 0x0004, 0x9F3B, 0x0000, 0x0000, 0x0002, /*2FD0*/
  0xFB41, 0x0020, 0x0004, 0x9F4A, 0x0000, 0x0000, 0x0002, /*2FD1*/
  0xFB41, 0x0020, 0x0004, 0x9F52, 0x0000, 0x0000, 0x0002, /*2FD2*/
  0xFB41, 0x0020, 0x0004, 0x9F8D, 0x0000, 0x0000, 0x0002, /*2FD3*/
  0xFB41, 0x0020, 0x0004, 0x9F9C, 0x0000, 0x0000, 0x0002, /*2FD4*/
  0xFB41, 0x0020, 0x0004, 0x9FA0, 0x0000, 0x0000, 0x0002, /*2FD5*/
  0xFBC0, 0x0020, 0x0002, 0xAFD6, 0x0000, 0x0000, 0x0002, /*2FD6*/
  0xFBC0, 0x0020, 0x0002, 0xAFD7, 0x0000, 0x0000, 0x0002, /*2FD7*/
  0xFBC0, 0x0020, 0x0002, 0xAFD8, 0x0000, 0x0000, 0x0002, /*2FD8*/
  0xFBC0, 0x0020, 0x0002, 0xAFD9, 0x0000, 0x0000, 0x0002, /*2FD9*/
  0xFBC0, 0x0020, 0x0002, 0xAFDA, 0x0000, 0x0000, 0x0002, /*2FDA*/
  0xFBC0, 0x0020, 0x0002, 0xAFDB, 0x0000, 0x0000, 0x0002, /*2FDB*/
  0xFBC0, 0x0020, 0x0002, 0xAFDC, 0x0000, 0x0000, 0x0002, /*2FDC*/
  0xFBC0, 0x0020, 0x0002, 0xAFDD, 0x0000, 0x0000, 0x0002, /*2FDD*/
  0xFBC0, 0x0020, 0x0002, 0xAFDE, 0x0000, 0x0000, 0x0002, /*2FDE*/
  0xFBC0, 0x0020, 0x0002, 0xAFDF, 0x0000, 0x0000, 0x0002, /*2FDF*/
  0xFBC0, 0x0020, 0x0002, 0xAFE0, 0x0000, 0x0000, 0x0002, /*2FE0*/
  0xFBC0, 0x0020, 0x0002, 0xAFE1, 0x0000, 0x0000, 0x0002, /*2FE1*/
  0xFBC0, 0x0020, 0x0002, 0xAFE2, 0x0000, 0x0000, 0x0002, /*2FE2*/
  0xFBC0, 0x0020, 0x0002, 0xAFE3, 0x0000, 0x0000, 0x0002, /*2FE3*/
  0xFBC0, 0x0020, 0x0002, 0xAFE4, 0x0000, 0x0000, 0x0002, /*2FE4*/
  0xFBC0, 0x0020, 0x0002, 0xAFE5, 0x0000, 0x0000, 0x0002, /*2FE5*/
  0xFBC0, 0x0020, 0x0002, 0xAFE6, 0x0000, 0x0000, 0x0002, /*2FE6*/
  0xFBC0, 0x0020, 0x0002, 0xAFE7, 0x0000, 0x0000, 0x0002, /*2FE7*/
  0xFBC0, 0x0020, 0x0002, 0xAFE8, 0x0000, 0x0000, 0x0002, /*2FE8*/
  0xFBC0, 0x0020, 0x0002, 0xAFE9, 0x0000, 0x0000, 0x0002, /*2FE9*/
  0xFBC0, 0x0020, 0x0002, 0xAFEA, 0x0000, 0x0000, 0x0002, /*2FEA*/
  0xFBC0, 0x0020, 0x0002, 0xAFEB, 0x0000, 0x0000, 0x0002, /*2FEB*/
  0xFBC0, 0x0020, 0x0002, 0xAFEC, 0x0000, 0x0000, 0x0002, /*2FEC*/
  0xFBC0, 0x0020, 0x0002, 0xAFED, 0x0000, 0x0000, 0x0002, /*2FED*/
  0xFBC0, 0x0020, 0x0002, 0xAFEE, 0x0000, 0x0000, 0x0002, /*2FEE*/
  0xFBC0, 0x0020, 0x0002, 0xAFEF, 0x0000, 0x0000, 0x0002, /*2FEF*/
  0x1A5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2FF0*/
  0x1A5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2FF1*/
  0x1A5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2FF2*/
  0x1A5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2FF3*/
  0x1A5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2FF4*/
  0x1A60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2FF5*/
  0x1A61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2FF6*/
  0x1A62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2FF7*/
  0x1A63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2FF8*/
  0x1A64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2FF9*/
  0x1A65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2FFA*/
  0x1A66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*2FFB*/
  0xFBC0, 0x0020, 0x0002, 0xAFFC, 0x0000, 0x0000, 0x0002, /*2FFC*/
  0xFBC0, 0x0020, 0x0002, 0xAFFD, 0x0000, 0x0000, 0x0002, /*2FFD*/
  0xFBC0, 0x0020, 0x0002, 0xAFFE, 0x0000, 0x0000, 0x0002, /*2FFE*/
  0xFBC0, 0x0020, 0x0002, 0xAFFF, 0x0000, 0x0000, 0x0002  /*2FFF*/
};

uint16 uca900_p030[]= {
  0x0209, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*3000*/
  0x0231, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3001*/
  0x028A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3002*/
  0x03AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3003*/
  0x1A8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3004*/
  0x1C07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3005*/
  0x3D66, 0x0020, 0x0004, 0x3D7C, 0x0020, 0x0004, 0x0002, /*3006*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3007*/
  0x036F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3008*/
  0x0370, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3009*/
  0x0371, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*300A*/
  0x0372, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*300B*/
  0x0373, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*300C*/
  0x0374, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*300D*/
  0x0375, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*300E*/
  0x0376, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*300F*/
  0x0377, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3010*/
  0x0378, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3011*/
  0x1A8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3012*/
  0x1A8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3013*/
  0x0379, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3014*/
  0x037A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3015*/
  0x037B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3016*/
  0x037C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3017*/
  0x037D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3018*/
  0x037E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3019*/
  0x037F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*301A*/
  0x0380, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*301B*/
  0x021E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*301C*/
  0x0312, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*301D*/
  0x0313, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*301E*/
  0x0314, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*301F*/
  0x1A8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3020*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3021*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3022*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3023*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3024*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3025*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3026*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3027*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3028*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3029*/
  0x0000, 0x00FB, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*302A*/
  0x0000, 0x00FC, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*302B*/
  0x0000, 0x00FD, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*302C*/
  0x0000, 0x00FE, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*302D*/
  0x0000, 0x00FF, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*302E*/
  0x0000, 0x0100, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*302F*/
  0x021F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3030*/
  0x1C0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3031*/
  0x1C0A, 0x0020, 0x0002, 0x0000, 0x0037, 0x0002, 0x0002, /*3032*/
  0x1C0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3033*/
  0x1C0B, 0x0020, 0x0002, 0x0000, 0x0037, 0x0002, 0x0002, /*3034*/
  0x1C0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3035*/
  0x1A8C, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3036*/
  0x1A8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3037*/
  0xFB40, 0x0020, 0x0004, 0xD341, 0x0000, 0x0000, 0x0002, /*3038*/
  0xFB40, 0x0020, 0x0004, 0xD344, 0x0000, 0x0000, 0x0002, /*3039*/
  0xFB40, 0x0020, 0x0004, 0xD345, 0x0000, 0x0000, 0x0002, /*303A*/
  0x1C08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*303B*/
  0x3D79, 0x0020, 0x0004, 0x3D67, 0x0020, 0x0004, 0x0002, /*303C*/
  0x03AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*303D*/
  0x1A90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*303E*/
  0x1A91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*303F*/
  0xFBC0, 0x0020, 0x0002, 0xB040, 0x0000, 0x0000, 0x0002, /*3040*/
  0x3D5A, 0x0020, 0x000D, 0x0000, 0x0000, 0x0000, 0x0001, /*3041*/
  0x3D5A, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3042*/
  0x3D5B, 0x0020, 0x000D, 0x0000, 0x0000, 0x0000, 0x0001, /*3043*/
  0x3D5B, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3044*/
  0x3D5C, 0x0020, 0x000D, 0x0000, 0x0000, 0x0000, 0x0001, /*3045*/
  0x3D5C, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3046*/
  0x3D5E, 0x0020, 0x000D, 0x0000, 0x0000, 0x0000, 0x0001, /*3047*/
  0x3D5E, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3048*/
  0x3D5F, 0x0020, 0x000D, 0x0000, 0x0000, 0x0000, 0x0001, /*3049*/
  0x3D5F, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*304A*/
  0x3D60, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*304B*/
  0x3D60, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*304C*/
  0x3D61, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*304D*/
  0x3D61, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*304E*/
  0x3D62, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*304F*/
  0x3D62, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3050*/
  0x3D63, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3051*/
  0x3D63, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3052*/
  0x3D64, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3053*/
  0x3D64, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3054*/
  0x3D65, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3055*/
  0x3D65, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3056*/
  0x3D66, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3057*/
  0x3D66, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3058*/
  0x3D67, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3059*/
  0x3D67, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*305A*/
  0x3D68, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*305B*/
  0x3D68, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*305C*/
  0x3D69, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*305D*/
  0x3D69, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*305E*/
  0x3D6A, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*305F*/
  0x3D6A, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3060*/
  0x3D6B, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3061*/
  0x3D6B, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3062*/
  0x3D6C, 0x0020, 0x000D, 0x0000, 0x0000, 0x0000, 0x0001, /*3063*/
  0x3D6C, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3064*/
  0x3D6C, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3065*/
  0x3D6D, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3066*/
  0x3D6D, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3067*/
  0x3D6E, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3068*/
  0x3D6E, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3069*/
  0x3D6F, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*306A*/
  0x3D70, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*306B*/
  0x3D71, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*306C*/
  0x3D72, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*306D*/
  0x3D73, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*306E*/
  0x3D74, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*306F*/
  0x3D74, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3070*/
  0x3D74, 0x0020, 0x000E, 0x0000, 0x0038, 0x0002, 0x0002, /*3071*/
  0x3D75, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3072*/
  0x3D75, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3073*/
  0x3D75, 0x0020, 0x000E, 0x0000, 0x0038, 0x0002, 0x0002, /*3074*/
  0x3D76, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3075*/
  0x3D76, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3076*/
  0x3D76, 0x0020, 0x000E, 0x0000, 0x0038, 0x0002, 0x0002, /*3077*/
  0x3D77, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3078*/
  0x3D77, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3079*/
  0x3D77, 0x0020, 0x000E, 0x0000, 0x0038, 0x0002, 0x0002, /*307A*/
  0x3D78, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*307B*/
  0x3D78, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*307C*/
  0x3D78, 0x0020, 0x000E, 0x0000, 0x0038, 0x0002, 0x0002, /*307D*/
  0x3D79, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*307E*/
  0x3D7A, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*307F*/
  0x3D7B, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3080*/
  0x3D7C, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3081*/
  0x3D7D, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3082*/
  0x3D7E, 0x0020, 0x000D, 0x0000, 0x0000, 0x0000, 0x0001, /*3083*/
  0x3D7E, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3084*/
  0x3D7F, 0x0020, 0x000D, 0x0000, 0x0000, 0x0000, 0x0001, /*3085*/
  0x3D7F, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3086*/
  0x3D81, 0x0020, 0x000D, 0x0000, 0x0000, 0x0000, 0x0001, /*3087*/
  0x3D81, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3088*/
  0x3D82, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3089*/
  0x3D83, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*308A*/
  0x3D84, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*308B*/
  0x3D85, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*308C*/
  0x3D86, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*308D*/
  0x3D87, 0x0020, 0x000D, 0x0000, 0x0000, 0x0000, 0x0001, /*308E*/
  0x3D87, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*308F*/
  0x3D88, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3090*/
  0x3D89, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3091*/
  0x3D8A, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3092*/
  0x3D8B, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*3093*/
  0x3D5C, 0x0020, 0x000E, 0x0000, 0x0037, 0x0002, 0x0002, /*3094*/
  0x3D60, 0x0020, 0x000D, 0x0000, 0x0000, 0x0000, 0x0001, /*3095*/
  0x3D63, 0x0020, 0x000D, 0x0000, 0x0000, 0x0000, 0x0001, /*3096*/
  0xFBC0, 0x0020, 0x0002, 0xB097, 0x0000, 0x0000, 0x0002, /*3097*/
  0xFBC0, 0x0020, 0x0002, 0xB098, 0x0000, 0x0000, 0x0002, /*3098*/
  0x0000, 0x0037, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3099*/
  0x0000, 0x0038, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*309A*/
  0x0491, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*309B*/
  0x0492, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*309C*/
  0x1C0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*309D*/
  0x1C0D, 0x0020, 0x0002, 0x0000, 0x0037, 0x0002, 0x0002, /*309E*/
  0x3D81, 0x0020, 0x0016, 0x3D83, 0x0020, 0x0016, 0x0002, /*309F*/
  0x0220, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*30A0*/
  0x3D5A, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*30A1*/
  0x3D5A, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30A2*/
  0x3D5B, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*30A3*/
  0x3D5B, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30A4*/
  0x3D5C, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*30A5*/
  0x3D5C, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30A6*/
  0x3D5E, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*30A7*/
  0x3D5E, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30A8*/
  0x3D5F, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*30A9*/
  0x3D5F, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30AA*/
  0x3D60, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30AB*/
  0x3D60, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30AC*/
  0x3D61, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30AD*/
  0x3D61, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30AE*/
  0x3D62, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30AF*/
  0x3D62, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30B0*/
  0x3D63, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30B1*/
  0x3D63, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30B2*/
  0x3D64, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30B3*/
  0x3D64, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30B4*/
  0x3D65, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30B5*/
  0x3D65, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30B6*/
  0x3D66, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30B7*/
  0x3D66, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30B8*/
  0x3D67, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30B9*/
  0x3D67, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30BA*/
  0x3D68, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30BB*/
  0x3D68, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30BC*/
  0x3D69, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30BD*/
  0x3D69, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30BE*/
  0x3D6A, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30BF*/
  0x3D6A, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30C0*/
  0x3D6B, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30C1*/
  0x3D6B, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30C2*/
  0x3D6C, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*30C3*/
  0x3D6C, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30C4*/
  0x3D6C, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30C5*/
  0x3D6D, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30C6*/
  0x3D6D, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30C7*/
  0x3D6E, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30C8*/
  0x3D6E, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30C9*/
  0x3D6F, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30CA*/
  0x3D70, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30CB*/
  0x3D71, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30CC*/
  0x3D72, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30CD*/
  0x3D73, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30CE*/
  0x3D74, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30CF*/
  0x3D74, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30D0*/
  0x3D74, 0x0020, 0x0011, 0x0000, 0x0038, 0x0002, 0x0002, /*30D1*/
  0x3D75, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30D2*/
  0x3D75, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30D3*/
  0x3D75, 0x0020, 0x0011, 0x0000, 0x0038, 0x0002, 0x0002, /*30D4*/
  0x3D76, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30D5*/
  0x3D76, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30D6*/
  0x3D76, 0x0020, 0x0011, 0x0000, 0x0038, 0x0002, 0x0002, /*30D7*/
  0x3D77, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30D8*/
  0x3D77, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30D9*/
  0x3D77, 0x0020, 0x0011, 0x0000, 0x0038, 0x0002, 0x0002, /*30DA*/
  0x3D78, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30DB*/
  0x3D78, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30DC*/
  0x3D78, 0x0020, 0x0011, 0x0000, 0x0038, 0x0002, 0x0002, /*30DD*/
  0x3D79, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30DE*/
  0x3D7A, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30DF*/
  0x3D7B, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30E0*/
  0x3D7C, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30E1*/
  0x3D7D, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30E2*/
  0x3D7E, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*30E3*/
  0x3D7E, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30E4*/
  0x3D7F, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*30E5*/
  0x3D7F, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30E6*/
  0x3D81, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*30E7*/
  0x3D81, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30E8*/
  0x3D82, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30E9*/
  0x3D83, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30EA*/
  0x3D84, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30EB*/
  0x3D85, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30EC*/
  0x3D86, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30ED*/
  0x3D87, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*30EE*/
  0x3D87, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30EF*/
  0x3D88, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30F0*/
  0x3D89, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30F1*/
  0x3D8A, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30F2*/
  0x3D8B, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*30F3*/
  0x3D5C, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30F4*/
  0x3D60, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*30F5*/
  0x3D63, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*30F6*/
  0x3D87, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30F7*/
  0x3D88, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30F8*/
  0x3D89, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30F9*/
  0x3D8A, 0x0020, 0x0011, 0x0000, 0x0037, 0x0002, 0x0002, /*30FA*/
  0x0221, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*30FB*/
  0x1C0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*30FC*/
  0x1C0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*30FD*/
  0x1C0F, 0x0020, 0x0002, 0x0000, 0x0037, 0x0002, 0x0002, /*30FE*/
  0x3D64, 0x0020, 0x0016, 0x3D6E, 0x0020, 0x0016, 0x0002  /*30FF*/
};

uint16 uca900_p031[]= {
  0xFBC0, 0x0020, 0x0002, 0xB100, 0x0000, 0x0000, 0x0002, /*3100*/
  0xFBC0, 0x0020, 0x0002, 0xB101, 0x0000, 0x0000, 0x0002, /*3101*/
  0xFBC0, 0x0020, 0x0002, 0xB102, 0x0000, 0x0000, 0x0002, /*3102*/
  0xFBC0, 0x0020, 0x0002, 0xB103, 0x0000, 0x0000, 0x0002, /*3103*/
  0xFBC0, 0x0020, 0x0002, 0xB104, 0x0000, 0x0000, 0x0002, /*3104*/
  0x3D8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3105*/
  0x3D8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3106*/
  0x3D8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3107*/
  0x3D8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3108*/
  0x3D91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3109*/
  0x3D92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*310A*/
  0x3D93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*310B*/
  0x3D94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*310C*/
  0x3D95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*310D*/
  0x3D96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*310E*/
  0x3D99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*310F*/
  0x3D9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3110*/
  0x3D9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3111*/
  0x3D9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3112*/
  0x3D9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3113*/
  0x3D9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3114*/
  0x3DA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3115*/
  0x3DA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3116*/
  0x3DA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3117*/
  0x3DA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3118*/
  0x3DA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3119*/
  0x3DA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*311A*/
  0x3DA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*311B*/
  0x3DAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*311C*/
  0x3DAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*311D*/
  0x3DAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*311E*/
  0x3DAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*311F*/
  0x3DB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3120*/
  0x3DB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3121*/
  0x3DB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3122*/
  0x3DB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3123*/
  0x3DB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3124*/
  0x3DB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3125*/
  0x3DBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3126*/
  0x3DBB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3127*/
  0x3DBC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3128*/
  0x3DBD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3129*/
  0x3D90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*312A*/
  0x3D97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*312B*/
  0x3D9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*312C*/
  0x3DBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*312D*/
  0xFBC0, 0x0020, 0x0002, 0xB12E, 0x0000, 0x0000, 0x0002, /*312E*/
  0xFBC0, 0x0020, 0x0002, 0xB12F, 0x0000, 0x0000, 0x0002, /*312F*/
  0xFBC0, 0x0020, 0x0002, 0xB130, 0x0000, 0x0000, 0x0002, /*3130*/
  0x3BF5, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3131*/
  0x3BF6, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3132*/
  0x3CD3, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3133*/
  0x3BF7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3134*/
  0x3CD5, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3135*/
  0x3CD6, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3136*/
  0x3BF8, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3137*/
  0x3BF9, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3138*/
  0x3BFA, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3139*/
  0x3CD9, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*313A*/
  0x3CDA, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*313B*/
  0x3CDB, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*313C*/
  0x3CDC, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*313D*/
  0x3CDD, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*313E*/
  0x3CDE, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*313F*/
  0x3C0F, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3140*/
  0x3BFB, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3141*/
  0x3BFC, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3142*/
  0x3BFD, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3143*/
  0x3C16, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3144*/
  0x3BFE, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3145*/
  0x3BFF, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3146*/
  0x3C00, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3147*/
  0x3C01, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3148*/
  0x3C02, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3149*/
  0x3C03, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*314A*/
  0x3C04, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*314B*/
  0x3C05, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*314C*/
  0x3C06, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*314D*/
  0x3C07, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*314E*/
  0x3C73, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*314F*/
  0x3C74, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3150*/
  0x3C75, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3151*/
  0x3C76, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3152*/
  0x3C77, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3153*/
  0x3C78, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3154*/
  0x3C79, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3155*/
  0x3C7A, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3156*/
  0x3C7B, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3157*/
  0x3C7C, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3158*/
  0x3C7D, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3159*/
  0x3C7E, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*315A*/
  0x3C7F, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*315B*/
  0x3C80, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*315C*/
  0x3C81, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*315D*/
  0x3C82, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*315E*/
  0x3C83, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*315F*/
  0x3C84, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3160*/
  0x3C85, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3161*/
  0x3C86, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3162*/
  0x3C87, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3163*/
  0x3C72, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3164*/
  0x3C09, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3165*/
  0x3C0A, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3166*/
  0x3CF0, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3167*/
  0x3CF1, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3168*/
  0x3CF5, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3169*/
  0x3CF7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*316A*/
  0x3CFC, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*316B*/
  0x3D00, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*316C*/
  0x3D02, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*316D*/
  0x3C11, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*316E*/
  0x3D06, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*316F*/
  0x3D08, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3170*/
  0x3C12, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3171*/
  0x3C13, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3172*/
  0x3C15, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3173*/
  0x3C17, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3174*/
  0x3C18, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3175*/
  0x3C1C, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3176*/
  0x3C1E, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3177*/
  0x3C20, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3178*/
  0x3C21, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3179*/
  0x3C22, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*317A*/
  0x3C23, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*317B*/
  0x3C24, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*317C*/
  0x3C27, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*317D*/
  0x3C2B, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*317E*/
  0x3C35, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*317F*/
  0x3C3C, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3180*/
  0x3C41, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3181*/
  0x3D1A, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3182*/
  0x3D1B, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3183*/
  0x3C4C, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3184*/
  0x3C4D, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3185*/
  0x3C4E, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3186*/
  0x3C96, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3187*/
  0x3C97, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3188*/
  0x3C9A, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*3189*/
  0x3CA3, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*318A*/
  0x3CA4, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*318B*/
  0x3CA6, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*318C*/
  0x3CB0, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*318D*/
  0x3CB3, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*318E*/
  0xFBC0, 0x0020, 0x0002, 0xB18F, 0x0000, 0x0000, 0x0002, /*318F*/
  0x1A92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3190*/
  0x1A93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*3191*/
  0xFB40, 0x0020, 0x0014, 0xCE00, 0x0000, 0x0000, 0x0002, /*3192*/
  0xFB40, 0x0020, 0x0014, 0xCE8C, 0x0000, 0x0000, 0x0002, /*3193*/
  0xFB40, 0x0020, 0x0014, 0xCE09, 0x0000, 0x0000, 0x0002, /*3194*/
  0xFB40, 0x0020, 0x0014, 0xD6DB, 0x0000, 0x0000, 0x0002, /*3195*/
  0xFB40, 0x0020, 0x0014, 0xCE0A, 0x0000, 0x0000, 0x0002, /*3196*/
  0xFB40, 0x0020, 0x0014, 0xCE2D, 0x0000, 0x0000, 0x0002, /*3197*/
  0xFB40, 0x0020, 0x0014, 0xCE0B, 0x0000, 0x0000, 0x0002, /*3198*/
  0xFB40, 0x0020, 0x0014, 0xF532, 0x0000, 0x0000, 0x0002, /*3199*/
  0xFB40, 0x0020, 0x0014, 0xCE59, 0x0000, 0x0000, 0x0002, /*319A*/
  0xFB40, 0x0020, 0x0014, 0xCE19, 0x0000, 0x0000, 0x0002, /*319B*/
  0xFB40, 0x0020, 0x0014, 0xCE01, 0x0000, 0x0000, 0x0002, /*319C*/
  0xFB40, 0x0020, 0x0014, 0xD929, 0x0000, 0x0000, 0x0002, /*319D*/
  0xFB40, 0x0020, 0x0014, 0xD730, 0x0000, 0x0000, 0x0002, /*319E*/
  0xFB40, 0x0020, 0x0014, 0xCEBA, 0x0000, 0x0000, 0x0002, /*319F*/
  0x3D8C, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*31A0*/
  0x3DA2, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*31A1*/
  0x3D9A, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*31A2*/
  0x3D95, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*31A3*/
  0x3DAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31A4*/
  0x3DAD, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*31A5*/
  0x3DAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31A6*/
  0x3DA9, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*31A7*/
  0x3DBC, 0x0020, 0x0004, 0x0000, 0x0112, 0x0004, 0x0002, /*31A8*/
  0x3DA8, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*31A9*/
  0x3DBB, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*31AA*/
  0x3DBC, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*31AB*/
  0x3DB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31AC*/
  0x3D98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31AD*/
  0x3DAE, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*31AE*/
  0x3DB0, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*31AF*/
  0x3DB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31B0*/
  0x3DB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31B1*/
  0x3DB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31B2*/
  0x3DBB, 0x0020, 0x0016, 0x0000, 0x0111, 0x0016, 0x0002, /*31B3*/
  0x3D8D, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*31B4*/
  0x3D92, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*31B5*/
  0x3D96, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*31B6*/
  0x3D99, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*31B7*/
  0x3DA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31B8*/
  0x3DA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31B9*/
  0x3DA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31BA*/
  0xFBC0, 0x0020, 0x0002, 0xB1BB, 0x0000, 0x0000, 0x0002, /*31BB*/
  0xFBC0, 0x0020, 0x0002, 0xB1BC, 0x0000, 0x0000, 0x0002, /*31BC*/
  0xFBC0, 0x0020, 0x0002, 0xB1BD, 0x0000, 0x0000, 0x0002, /*31BD*/
  0xFBC0, 0x0020, 0x0002, 0xB1BE, 0x0000, 0x0000, 0x0002, /*31BE*/
  0xFBC0, 0x0020, 0x0002, 0xB1BF, 0x0000, 0x0000, 0x0002, /*31BF*/
  0x1A67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31C0*/
  0x1A68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31C1*/
  0x1A69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31C2*/
  0x1A6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31C3*/
  0x1A6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31C4*/
  0x1A6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31C5*/
  0x1A6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31C6*/
  0x1A6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31C7*/
  0x1A6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31C8*/
  0x1A70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31C9*/
  0x1A71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31CA*/
  0x1A72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31CB*/
  0x1A73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31CC*/
  0x1A74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31CD*/
  0x1A75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31CE*/
  0x1A76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31CF*/
  0x1A77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31D0*/
  0x1A78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31D1*/
  0x1A79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31D2*/
  0x1A7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31D3*/
  0x1A7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31D4*/
  0x1A7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31D5*/
  0x1A7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31D6*/
  0x1A7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31D7*/
  0x1A7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31D8*/
  0x1A80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31D9*/
  0x1A81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31DA*/
  0x1A82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31DB*/
  0x1A83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31DC*/
  0x1A84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31DD*/
  0x1A85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31DE*/
  0x1A86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31DF*/
  0x1A87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31E0*/
  0x1A88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31E1*/
  0x1A89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31E2*/
  0x1A8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*31E3*/
  0xFBC0, 0x0020, 0x0002, 0xB1E4, 0x0000, 0x0000, 0x0002, /*31E4*/
  0xFBC0, 0x0020, 0x0002, 0xB1E5, 0x0000, 0x0000, 0x0002, /*31E5*/
  0xFBC0, 0x0020, 0x0002, 0xB1E6, 0x0000, 0x0000, 0x0002, /*31E6*/
  0xFBC0, 0x0020, 0x0002, 0xB1E7, 0x0000, 0x0000, 0x0002, /*31E7*/
  0xFBC0, 0x0020, 0x0002, 0xB1E8, 0x0000, 0x0000, 0x0002, /*31E8*/
  0xFBC0, 0x0020, 0x0002, 0xB1E9, 0x0000, 0x0000, 0x0002, /*31E9*/
  0xFBC0, 0x0020, 0x0002, 0xB1EA, 0x0000, 0x0000, 0x0002, /*31EA*/
  0xFBC0, 0x0020, 0x0002, 0xB1EB, 0x0000, 0x0000, 0x0002, /*31EB*/
  0xFBC0, 0x0020, 0x0002, 0xB1EC, 0x0000, 0x0000, 0x0002, /*31EC*/
  0xFBC0, 0x0020, 0x0002, 0xB1ED, 0x0000, 0x0000, 0x0002, /*31ED*/
  0xFBC0, 0x0020, 0x0002, 0xB1EE, 0x0000, 0x0000, 0x0002, /*31EE*/
  0xFBC0, 0x0020, 0x0002, 0xB1EF, 0x0000, 0x0000, 0x0002, /*31EF*/
  0x3D62, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31F0*/
  0x3D66, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31F1*/
  0x3D67, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31F2*/
  0x3D6E, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31F3*/
  0x3D71, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31F4*/
  0x3D74, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31F5*/
  0x3D75, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31F6*/
  0x3D76, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31F7*/
  0x3D77, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31F8*/
  0x3D78, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31F9*/
  0x3D7B, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31FA*/
  0x3D82, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31FB*/
  0x3D83, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31FC*/
  0x3D84, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31FD*/
  0x3D85, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001, /*31FE*/
  0x3D86, 0x0020, 0x000F, 0x0000, 0x0000, 0x0000, 0x0001  /*31FF*/
};

uint16 uca900_p032[]= {
  0x0317, 0x0020, 0x0004, 0x3BF5, 0x0020, 0x0004, 0x0318, 0x0020, /*3200*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3BF7, 0x0020, 0x0004, 0x0318, 0x0020, /*3201*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3BF8, 0x0020, 0x0004, 0x0318, 0x0020, /*3202*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3BFA, 0x0020, 0x0004, 0x0318, 0x0020, /*3203*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3BFB, 0x0020, 0x0004, 0x0318, 0x0020, /*3204*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3BFC, 0x0020, 0x0004, 0x0318, 0x0020, /*3205*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3BFE, 0x0020, 0x0004, 0x0318, 0x0020, /*3206*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3C00, 0x0020, 0x0004, 0x0318, 0x0020, /*3207*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3C01, 0x0020, 0x0004, 0x0318, 0x0020, /*3208*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3C03, 0x0020, 0x0004, 0x0318, 0x0020, /*3209*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3C04, 0x0020, 0x0004, 0x0318, 0x0020, /*320A*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3C05, 0x0020, 0x0004, 0x0318, 0x0020, /*320B*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3C06, 0x0020, 0x0004, 0x0318, 0x0020, /*320C*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3C07, 0x0020, 0x0004, 0x0318, 0x0020, /*320D*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x3BF5, 0x0020, 0x0004, 0x3C73, 0x0020, /*320E*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3BF7, 0x0020, 0x0004, 0x3C73, 0x0020, /*320F*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3BF8, 0x0020, 0x0004, 0x3C73, 0x0020, /*3210*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3BFA, 0x0020, 0x0004, 0x3C73, 0x0020, /*3211*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3BFB, 0x0020, 0x0004, 0x3C73, 0x0020, /*3212*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3BFC, 0x0020, 0x0004, 0x3C73, 0x0020, /*3213*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3BFE, 0x0020, 0x0004, 0x3C73, 0x0020, /*3214*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3C00, 0x0020, 0x0004, 0x3C73, 0x0020, /*3215*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3C01, 0x0020, 0x0004, 0x3C73, 0x0020, /*3216*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3C03, 0x0020, 0x0004, 0x3C73, 0x0020, /*3217*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3C04, 0x0020, 0x0004, 0x3C73, 0x0020, /*3218*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3C05, 0x0020, 0x0004, 0x3C73, 0x0020, /*3219*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3C06, 0x0020, 0x0004, 0x3C73, 0x0020, /*321A*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3C07, 0x0020, 0x0004, 0x3C73, 0x0020, /*321B*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3C01, 0x0020, 0x0004, 0x3C80, 0x0020, /*321C*/
  0x0004, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0x3C00, 0x0020, 0x0004, 0x3C7B, 0x0020, /*321D*/
  0x0004, 0x3C01, 0x0020, 0x0004, 0x3C77, 0x0020, 0x0004,
  0x3CD4, 0x0020, 0x0004, 0x0318, 0x0020, 0x0004, 0x0007,
  0x0317, 0x0020, 0x0004, 0x3C00, 0x0020, 0x0004, 0x3C7B, 0x0020, /*321E*/
  0x0004, 0x3C07, 0x0020, 0x0004, 0x3C80, 0x0020, 0x0004,
  0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0006,
  0xFBC0, 0x0020, 0x0002, 0xB21F, 0x0000, 0x0000, 0x0000, 0x0000, /*321F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xCE00, 0x0000, /*3220*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xCE8C, 0x0000, /*3221*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xCE09, 0x0000, /*3222*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xD6DB, 0x0000, /*3223*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xCE94, 0x0000, /*3224*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xD16D, 0x0000, /*3225*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xCE03, 0x0000, /*3226*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xD16B, 0x0000, /*3227*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xCE5D, 0x0000, /*3228*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xD341, 0x0000, /*3229*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE708, 0x0000, /*322A*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF06B, 0x0000, /*322B*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xEC34, 0x0000, /*322C*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE728, 0x0000, /*322D*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB41, 0x0020, 0x0004, 0x91D1, 0x0000, /*322E*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xD71F, 0x0000, /*322F*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE5E5, 0x0000, /*3230*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE82A, 0x0000, /*3231*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE709, 0x0000, /*3232*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF93E, 0x0000, /*3233*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xD40D, 0x0000, /*3234*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF279, 0x0000, /*3235*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB41, 0x0020, 0x0004, 0x8CA1, 0x0000, /*3236*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF95D, 0x0000, /*3237*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xD2B4, 0x0000, /*3238*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xCEE3, 0x0000, /*3239*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xD47C, 0x0000, /*323A*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xDB66, 0x0000, /*323B*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF6E3, 0x0000, /*323C*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xCF01, 0x0000, /*323D*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB41, 0x0020, 0x0004, 0x8CC7, 0x0000, /*323E*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xD354, 0x0000, /*323F*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF96D, 0x0000, /*3240*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xCF11, 0x0000, /*3241*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB41, 0x0020, 0x0004, 0x81EA, 0x0000, /*3242*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0317, 0x0020, 0x0004, 0xFB41, 0x0020, 0x0004, 0x81F3, 0x0000, /*3243*/
  0x0000, 0x0318, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0xFB40, 0x0020, 0x0006, 0xD54F, 0x0000, 0x0000, 0x0000, 0x0000, /*3244*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xDE7C, 0x0000, 0x0000, 0x0000, 0x0000, /*3245*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xE587, 0x0000, 0x0000, 0x0000, 0x0000, /*3246*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xFB8F, 0x0000, 0x0000, 0x0000, 0x0000, /*3247*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, /*3248*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, /*3249*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, /*324A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, /*324B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C42, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, /*324C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C43, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, /*324D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C44, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, /*324E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C45, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, /*324F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E0C, 0x0020, 0x001D, 0x1E95, 0x0020, 0x001D, 0x1CAA, 0x0020, /*3250*/
  0x001D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3F, 0x0020, 0x0006, 0x1C3E, 0x0020, 0x0006, 0x0000, 0x0000, /*3251*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0006, 0x1C3F, 0x0020, 0x0006, 0x0000, 0x0000, /*3252*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0006, 0x1C40, 0x0020, 0x0006, 0x0000, 0x0000, /*3253*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0006, 0x1C41, 0x0020, 0x0006, 0x0000, 0x0000, /*3254*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0006, 0x1C42, 0x0020, 0x0006, 0x0000, 0x0000, /*3255*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0006, 0x1C43, 0x0020, 0x0006, 0x0000, 0x0000, /*3256*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0006, 0x1C44, 0x0020, 0x0006, 0x0000, 0x0000, /*3257*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0006, 0x1C45, 0x0020, 0x0006, 0x0000, 0x0000, /*3258*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0006, 0x1C46, 0x0020, 0x0006, 0x0000, 0x0000, /*3259*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, /*325A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0006, 0x1C3E, 0x0020, 0x0006, 0x0000, 0x0000, /*325B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0006, 0x1C3F, 0x0020, 0x0006, 0x0000, 0x0000, /*325C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0006, 0x1C40, 0x0020, 0x0006, 0x0000, 0x0000, /*325D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0006, 0x1C41, 0x0020, 0x0006, 0x0000, 0x0000, /*325E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0006, 0x1C42, 0x0020, 0x0006, 0x0000, 0x0000, /*325F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3BF5, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*3260*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3BF7, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*3261*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3BF8, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*3262*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3BFA, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*3263*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3BFB, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*3264*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3BFC, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*3265*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3BFE, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*3266*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3C00, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*3267*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3C01, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*3268*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3C03, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*3269*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3C04, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*326A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3C05, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*326B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3C06, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*326C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3C07, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*326D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3BF5, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*326E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3BF7, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*326F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3BF8, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*3270*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3BFA, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*3271*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3BFB, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*3272*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3BFC, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*3273*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3BFE, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*3274*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C00, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*3275*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C01, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*3276*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C03, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*3277*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C04, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*3278*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C05, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*3279*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C06, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*327A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C07, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x0000, 0x0000, /*327B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C03, 0x0020, 0x0006, 0x3C73, 0x0020, 0x0006, 0x3CE0, 0x0020, /*327C*/
  0x0006, 0x3BF5, 0x0020, 0x0006, 0x3C7B, 0x0020, 0x0006,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3C01, 0x0020, 0x0006, 0x3C80, 0x0020, 0x0006, 0x3C00, 0x0020, /*327D*/
  0x0006, 0x3C86, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3C00, 0x0020, 0x0006, 0x3C80, 0x0020, 0x0006, 0x0000, 0x0000, /*327E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1A94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*327F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFB40, 0x0020, 0x0006, 0xCE00, 0x0000, 0x0000, 0x0000, 0x0000, /*3280*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xCE8C, 0x0000, 0x0000, 0x0000, 0x0000, /*3281*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xCE09, 0x0000, 0x0000, 0x0000, 0x0000, /*3282*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD6DB, 0x0000, 0x0000, 0x0000, 0x0000, /*3283*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xCE94, 0x0000, 0x0000, 0x0000, 0x0000, /*3284*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD16D, 0x0000, 0x0000, 0x0000, 0x0000, /*3285*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xCE03, 0x0000, 0x0000, 0x0000, 0x0000, /*3286*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD16B, 0x0000, 0x0000, 0x0000, 0x0000, /*3287*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xCE5D, 0x0000, 0x0000, 0x0000, 0x0000, /*3288*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD341, 0x0000, 0x0000, 0x0000, 0x0000, /*3289*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xE708, 0x0000, 0x0000, 0x0000, 0x0000, /*328A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xF06B, 0x0000, 0x0000, 0x0000, 0x0000, /*328B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xEC34, 0x0000, 0x0000, 0x0000, 0x0000, /*328C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xE728, 0x0000, 0x0000, 0x0000, 0x0000, /*328D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0006, 0x91D1, 0x0000, 0x0000, 0x0000, 0x0000, /*328E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD71F, 0x0000, 0x0000, 0x0000, 0x0000, /*328F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xE5E5, 0x0000, 0x0000, 0x0000, 0x0000, /*3290*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xE82A, 0x0000, 0x0000, 0x0000, 0x0000, /*3291*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xE709, 0x0000, 0x0000, 0x0000, 0x0000, /*3292*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xF93E, 0x0000, 0x0000, 0x0000, 0x0000, /*3293*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD40D, 0x0000, 0x0000, 0x0000, 0x0000, /*3294*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xF279, 0x0000, 0x0000, 0x0000, 0x0000, /*3295*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0006, 0x8CA1, 0x0000, 0x0000, 0x0000, 0x0000, /*3296*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xF95D, 0x0000, 0x0000, 0x0000, 0x0000, /*3297*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD2B4, 0x0000, 0x0000, 0x0000, 0x0000, /*3298*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xF9D8, 0x0000, 0x0000, 0x0000, 0x0000, /*3299*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xF537, 0x0000, 0x0000, 0x0000, 0x0000, /*329A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD973, 0x0000, 0x0000, 0x0000, 0x0000, /*329B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0006, 0x9069, 0x0000, 0x0000, 0x0000, 0x0000, /*329C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD12A, 0x0000, 0x0000, 0x0000, 0x0000, /*329D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD370, 0x0000, 0x0000, 0x0000, 0x0000, /*329E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xECE8, 0x0000, 0x0000, 0x0000, 0x0000, /*329F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0006, 0x9805, 0x0000, 0x0000, 0x0000, 0x0000, /*32A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xCF11, 0x0000, 0x0000, 0x0000, 0x0000, /*32A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD199, 0x0000, 0x0000, 0x0000, 0x0000, /*32A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xEB63, 0x0000, 0x0000, 0x0000, 0x0000, /*32A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xCE0A, 0x0000, 0x0000, 0x0000, 0x0000, /*32A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xCE2D, 0x0000, 0x0000, 0x0000, 0x0000, /*32A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xCE0B, 0x0000, 0x0000, 0x0000, 0x0000, /*32A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xDDE6, 0x0000, 0x0000, 0x0000, 0x0000, /*32A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD3F3, 0x0000, 0x0000, 0x0000, 0x0000, /*32A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD33B, 0x0000, 0x0000, 0x0000, 0x0000, /*32A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xDB97, 0x0000, 0x0000, 0x0000, 0x0000, /*32AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xDB66, 0x0000, 0x0000, 0x0000, 0x0000, /*32AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xF6E3, 0x0000, 0x0000, 0x0000, 0x0000, /*32AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xCF01, 0x0000, 0x0000, 0x0000, 0x0000, /*32AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x0006, 0x8CC7, 0x0000, 0x0000, 0x0000, 0x0000, /*32AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD354, 0x0000, 0x0000, 0x0000, 0x0000, /*32AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD91C, 0x0000, 0x0000, 0x0000, 0x0000, /*32B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0006, 0x1C43, 0x0020, 0x0006, 0x0000, 0x0000, /*32B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0006, 0x1C44, 0x0020, 0x0006, 0x0000, 0x0000, /*32B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0006, 0x1C45, 0x0020, 0x0006, 0x0000, 0x0000, /*32B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0006, 0x1C46, 0x0020, 0x0006, 0x0000, 0x0000, /*32B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, /*32B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0006, 0x1C3E, 0x0020, 0x0006, 0x0000, 0x0000, /*32B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0006, 0x1C3F, 0x0020, 0x0006, 0x0000, 0x0000, /*32B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0006, 0x1C40, 0x0020, 0x0006, 0x0000, 0x0000, /*32B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0006, 0x1C41, 0x0020, 0x0006, 0x0000, 0x0000, /*32B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0006, 0x1C42, 0x0020, 0x0006, 0x0000, 0x0000, /*32BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0006, 0x1C43, 0x0020, 0x0006, 0x0000, 0x0000, /*32BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0006, 0x1C44, 0x0020, 0x0006, 0x0000, 0x0000, /*32BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0006, 0x1C45, 0x0020, 0x0006, 0x0000, 0x0000, /*32BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0006, 0x1C46, 0x0020, 0x0006, 0x0000, 0x0000, /*32BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C42, 0x0020, 0x0006, 0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, /*32BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE708, 0x0000, /*32C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3F, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE708, 0x0000, /*32C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C40, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE708, 0x0000, /*32C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C41, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE708, 0x0000, /*32C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C42, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE708, 0x0000, /*32C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C43, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE708, 0x0000, /*32C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C44, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE708, 0x0000, /*32C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C45, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE708, 0x0000, /*32C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C46, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE708, 0x0000, /*32C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0x1C3D, 0x0020, 0x0004, 0xFB40, 0x0020, /*32C9*/
  0x0004, 0xE708, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0xFB40, 0x0020, /*32CA*/
  0x0004, 0xE708, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C3F, 0x0020, 0x0004, 0xFB40, 0x0020, /*32CB*/
  0x0004, 0xE708, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1D18, 0x0020, 0x001D, 0x1CF4, 0x0020, 0x001C, 0x0000, 0x0000, /*32CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CAA, 0x0020, 0x001C, 0x1E33, 0x0020, 0x001C, 0x1CF4, 0x0020, /*32CD*/
  0x001C, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1CAA, 0x0020, 0x001C, 0x1EE3, 0x0020, 0x001D, 0x0000, 0x0000, /*32CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x001D, 0x1E95, 0x0020, 0x001D, 0x1C8F, 0x0020, /*32CF*/
  0x001D, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D5A, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D5B, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D5C, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D5E, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D5F, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D60, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D61, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D62, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D63, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D64, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D65, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D66, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D67, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D68, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D69, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D6A, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D6B, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D6C, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D6D, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D6E, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D6F, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D70, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D71, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D72, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D73, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D74, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D75, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D76, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D77, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D78, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D79, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D7A, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D7B, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D7C, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D7D, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D7E, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D7F, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D81, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D82, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D83, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D84, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D85, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D86, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D87, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D88, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D89, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D8A, 0x0020, 0x0013, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*32FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC0, 0x0020, 0x0002, 0xB2FF, 0x0000, 0x0000, 0x0000, 0x0000, /*32FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002
};

uint16 uca900_p033[]= {
  0x3D5A, 0x0020, 0x001C, 0x3D74, 0x0020, 0x001C, 0x0000, /*3300*/
  0x0038, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D6E,
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D5A, 0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x3D76, /*3301*/
  0x0020, 0x001C, 0x3D5A, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D5A, 0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x3D77, /*3302*/
  0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x3D5A,
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D5A, 0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D84, /*3303*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D5B, 0x0020, 0x001C, 0x3D70, 0x0020, 0x001C, 0x3D8B, /*3304*/
  0x0020, 0x001C, 0x3D62, 0x0020, 0x001C, 0x0000,
  0x0037, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D5B, 0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x3D6B, /*3305*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D5C, 0x0020, 0x001C, 0x3D5F, 0x0020, 0x001C, 0x3D8B, /*3306*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D5E, 0x0020, 0x001C, 0x3D67, 0x0020, 0x001C, 0x3D62, /*3307*/
  0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D6E,
  0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x3D5E, 0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D60, /*3308*/
  0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D5F, 0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x3D67, /*3309*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D5F, 0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D7B, /*330A*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D60, 0x0020, 0x001C, 0x3D5B, 0x0020, 0x001C, 0x3D83, /*330B*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D60, 0x0020, 0x001C, 0x3D82, 0x0020, 0x001C, 0x3D6C, /*330C*/
  0x0020, 0x001C, 0x3D6E, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D60, 0x0020, 0x001C, 0x3D86, 0x0020, 0x001C, 0x3D83, /*330D*/
  0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D60, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D86, /*330E*/
  0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D60, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D8B, /*330F*/
  0x0020, 0x001C, 0x3D79, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D61, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D60, /*3310*/
  0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D61, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D70, /*3311*/
  0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D61, 0x0020, 0x001C, 0x3D7F, 0x0020, 0x001C, 0x3D83, /*3312*/
  0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D61, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D84, /*3313*/
  0x0020, 0x001C, 0x3D6A, 0x0020, 0x001C, 0x0000,
  0x0037, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x3D61, 0x0020, 0x001C, 0x3D86, 0x0020, 0x001C, 0x0000, /*3314*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3D61, 0x0020, 0x001C, 0x3D86, 0x0020, 0x001C, 0x3D62, /*3315*/
  0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D82,
  0x0020, 0x001C, 0x3D7B, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x3D61, 0x0020, 0x001C, 0x3D86, 0x0020, 0x001C, 0x3D7C, /*3316*/
  0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D6E,
  0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x3D61, 0x0020, 0x001C, 0x3D86, 0x0020, 0x001C, 0x3D87, /*3317*/
  0x0020, 0x001C, 0x3D6C, 0x0020, 0x001C, 0x3D6E,
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D62, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D82, /*3318*/
  0x0020, 0x001C, 0x3D7B, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D62, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D82, /*3319*/
  0x0020, 0x001C, 0x3D7B, 0x0020, 0x001C, 0x3D6E,
  0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x3D62, 0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x3D68, /*331A*/
  0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D5B,
  0x0020, 0x001C, 0x3D86, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x3D62, 0x0020, 0x001C, 0x3D86, 0x0020, 0x001C, 0x1C0E, /*331B*/
  0x0020, 0x001C, 0x3D72, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D63, 0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D67, /*331C*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D64, 0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x3D6F, /*331D*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D64, 0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D78, /*331E*/
  0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D65, 0x0020, 0x001C, 0x3D5B, 0x0020, 0x001C, 0x3D62, /*331F*/
  0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D65, 0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x3D6B, /*3320*/
  0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D7B,
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D66, 0x0020, 0x001C, 0x3D83, 0x0020, 0x001C, 0x3D8B, /*3321*/
  0x0020, 0x001C, 0x3D62, 0x0020, 0x001C, 0x0000,
  0x0037, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D68, 0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x3D6B, /*3322*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D68, 0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x3D6E, /*3323*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D6A, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x1C0E, /*3324*/
  0x0020, 0x001C, 0x3D67, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D6D, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D66, /*3325*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D6E, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D84, /*3326*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D6E, 0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x0000, /*3327*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3D6F, 0x0020, 0x001C, 0x3D73, 0x0020, 0x001C, 0x0000, /*3328*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3D73, 0x0020, 0x001C, 0x3D6C, 0x0020, 0x001C, 0x3D6E, /*3329*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D74, 0x0020, 0x001C, 0x3D5B, 0x0020, 0x001C, 0x3D6C, /*332A*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D74, 0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x1C0E, /*332B*/
  0x0020, 0x001C, 0x3D68, 0x0020, 0x001C, 0x3D8B,
  0x0020, 0x001C, 0x3D6E, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x3D74, 0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x1C0E, /*332C*/
  0x0020, 0x001C, 0x3D6C, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D74, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x1C0E, /*332D*/
  0x0020, 0x001C, 0x3D85, 0x0020, 0x001C, 0x3D84,
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D75, 0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x3D5A, /*332E*/
  0x0020, 0x001C, 0x3D67, 0x0020, 0x001C, 0x3D6E,
  0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x3D75, 0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x3D62, /*332F*/
  0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D75, 0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x3D64, /*3330*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D75, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D84, /*3331*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D76, 0x0020, 0x001C, 0x3D5A, 0x0020, 0x001C, 0x3D82, /*3332*/
  0x0020, 0x001C, 0x3D6C, 0x0020, 0x001C, 0x3D6E,
  0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x3D76, 0x0020, 0x001C, 0x3D5B, 0x0020, 0x001C, 0x1C0E, /*3333*/
  0x0020, 0x001C, 0x3D6E, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D76, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D6C, /*3334*/
  0x0020, 0x001C, 0x3D66, 0x0020, 0x001C, 0x3D5E,
  0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x3D76, 0x0020, 0x001C, 0x3D82, 0x0020, 0x001C, 0x3D8B, /*3335*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D77, 0x0020, 0x001C, 0x3D62, 0x0020, 0x001C, 0x3D6A, /*3336*/
  0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D84,
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D77, 0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x3D69, /*3337*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D77, 0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x3D70, /*3338*/
  0x0020, 0x001C, 0x3D75, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D77, 0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x3D6C, /*3339*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D77, 0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x3D8B, /*333A*/
  0x0020, 0x001C, 0x3D67, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D77, 0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x1C0E, /*333B*/
  0x0020, 0x001C, 0x3D66, 0x0020, 0x001C, 0x0000,
  0x0037, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D77, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x1C0E, /*333C*/
  0x0020, 0x001C, 0x3D6A, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D78, 0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x3D5B, /*333D*/
  0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x3D6E,
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D78, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D84, /*333E*/
  0x0020, 0x001C, 0x3D6E, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D78, 0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x0000, /*333F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3D78, 0x0020, 0x001C, 0x0000, 0x0038, 0x001C, 0x3D8B, /*3340*/
  0x0020, 0x001C, 0x3D6E, 0x0020, 0x001C, 0x0000,
  0x0037, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D78, 0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D84, /*3341*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D78, 0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D8B, /*3342*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D79, 0x0020, 0x001C, 0x3D5B, 0x0020, 0x001C, 0x3D62, /*3343*/
  0x0020, 0x001C, 0x3D86, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D79, 0x0020, 0x001C, 0x3D5B, 0x0020, 0x001C, 0x3D84, /*3344*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D79, 0x0020, 0x001C, 0x3D6C, 0x0020, 0x001C, 0x3D74, /*3345*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D79, 0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x3D62, /*3346*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D79, 0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x3D66, /*3347*/
  0x0020, 0x001C, 0x3D81, 0x0020, 0x001C, 0x3D8B,
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D7A, 0x0020, 0x001C, 0x3D62, 0x0020, 0x001C, 0x3D86, /*3348*/
  0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D7A, 0x0020, 0x001C, 0x3D83, 0x0020, 0x001C, 0x0000, /*3349*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3D7A, 0x0020, 0x001C, 0x3D83, 0x0020, 0x001C, 0x3D74, /*334A*/
  0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x1C0E,
  0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x3D7C, 0x0020, 0x001C, 0x3D60, 0x0020, 0x001C, 0x0000, /*334B*/
  0x0037, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D7C, 0x0020, 0x001C, 0x3D60, 0x0020, 0x001C, 0x0000, /*334C*/
  0x0037, 0x001C, 0x3D6E, 0x0020, 0x001C, 0x3D8B,
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D7C, 0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D6E, /*334D*/
  0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D7E, 0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D6E, /*334E*/
  0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D7E, 0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D84, /*334F*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D7F, 0x0020, 0x001C, 0x3D5A, 0x0020, 0x001C, 0x3D8B, /*3350*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x3D83, 0x0020, 0x001C, 0x3D6C, 0x0020, 0x001C, 0x3D6E, /*3351*/
  0x0020, 0x001C, 0x3D84, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D83, 0x0020, 0x001C, 0x3D82, 0x0020, 0x001C, 0x0000, /*3352*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3D84, 0x0020, 0x001C, 0x3D75, 0x0020, 0x001C, 0x0000, /*3353*/
  0x0038, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x3D84, 0x0020, 0x001C, 0x1C0E, 0x0020, 0x001C, 0x3D76, /*3354*/
  0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x3D84,
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x3D85, 0x0020, 0x001C, 0x3D7B, 0x0020, 0x001C, 0x0000, /*3355*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3D85, 0x0020, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x3D6E, /*3356*/
  0x0020, 0x001C, 0x3D63, 0x0020, 0x001C, 0x0000,
  0x0037, 0x001C, 0x3D8B, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x3D87, 0x0020, 0x001C, 0x3D6C, 0x0020, 0x001C, 0x3D6E, /*3357*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3D, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF0B9, /*3358*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF0B9, /*3359*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3F, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF0B9, /*335A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C40, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF0B9, /*335B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C41, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF0B9, /*335C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C42, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF0B9, /*335D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C43, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF0B9, /*335E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C44, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF0B9, /*335F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C45, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF0B9, /*3360*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C46, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF0B9, /*3361*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0x1C3D, 0x0020, 0x0004, 0xFB40, /*3362*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0xFB40, /*3363*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C3F, 0x0020, 0x0004, 0xFB40, /*3364*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C40, 0x0020, 0x0004, 0xFB40, /*3365*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C41, 0x0020, 0x0004, 0xFB40, /*3366*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C42, 0x0020, 0x0004, 0xFB40, /*3367*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C43, 0x0020, 0x0004, 0xFB40, /*3368*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C44, 0x0020, 0x0004, 0xFB40, /*3369*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C45, 0x0020, 0x0004, 0xFB40, /*336A*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C46, 0x0020, 0x0004, 0xFB40, /*336B*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C3D, 0x0020, 0x0004, 0xFB40, /*336C*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0xFB40, /*336D*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C3F, 0x0020, 0x0004, 0xFB40, /*336E*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C40, 0x0020, 0x0004, 0xFB40, /*336F*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C41, 0x0020, 0x0004, 0xFB40, /*3370*/
  0x0020, 0x0004, 0xF0B9, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1D18, 0x0020, 0x001C, 0x1E0C, 0x0020, 0x001D, 0x1C47, /*3371*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C8F, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001C, 0x0000, /*3372*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x001D, 0x1EB5, 0x0020, 0x001D, 0x0000, /*3373*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C60, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001C, 0x1E33, /*3374*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DDD, 0x0020, 0x001C, 0x1EE3, 0x0020, 0x001D, 0x0000, /*3375*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E0C, 0x0020, 0x001C, 0x1C7A, 0x0020, 0x001C, 0x0000, /*3376*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x0000, /*3377*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x1C3F, /*3378*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C8F, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x1C40, /*3379*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1D32, 0x0020, 0x001D, 0x1EB5, 0x0020, 0x001D, 0x0000, /*337A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xDE73, 0x0000, 0x0000, 0xFB40, /*337B*/
  0x0020, 0x001C, 0xE210, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0xFB40, 0x0020, 0x001C, 0xE62D, 0x0000, 0x0000, 0xFB40, /*337C*/
  0x0020, 0x001C, 0xD48C, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0xFB40, 0x0020, 0x001C, 0xD927, 0x0000, 0x0000, 0xFB40, /*337D*/
  0x0020, 0x001C, 0xEB63, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0xFB40, 0x0020, 0x001C, 0xE60E, 0x0000, 0x0000, 0xFB40, /*337E*/
  0x0020, 0x001C, 0xECBB, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0xFB40, 0x0020, 0x001C, 0xE82A, 0x0000, 0x0000, 0xFB40, /*337F*/
  0x0020, 0x001C, 0xDF0F, 0x0000, 0x0000, 0xFB40,
  0x0020, 0x001C, 0xCF1A, 0x0000, 0x0000, 0xFB40,
  0x0020, 0x001C, 0xF93E, 0x0000, 0x0000, 0x0008,
  0x1E0C, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001D, 0x0000, /*3380*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001D, 0x0000, /*3381*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCB, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001D, 0x0000, /*3382*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001D, 0x0000, /*3383*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001D, 0x0000, /*3384*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001D, 0x1C60, 0x0020, 0x001D, 0x0000, /*3385*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001D, 0x1C60, 0x0020, 0x001D, 0x0000, /*3386*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x001D, 0x1C60, 0x0020, 0x001D, 0x0000, /*3387*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001C, 0x1D77, /*3388*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1D65, 0x0020, 0x001C, 0x1C7A, 0x0020, 0x001C, 0x1C47, /*3389*/
  0x0020, 0x001C, 0x1D77, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1E0C, 0x0020, 0x001C, 0x1CE5, 0x0020, 0x001D, 0x0000, /*338A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x001C, 0x1CE5, 0x0020, 0x001D, 0x0000, /*338B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCB, 0x0020, 0x001C, 0x1CE5, 0x0020, 0x001D, 0x0000, /*338C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCB, 0x0020, 0x001C, 0x1CF4, 0x0020, 0x001C, 0x0000, /*338D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001C, 0x1CF4, 0x0020, 0x001C, 0x0000, /*338E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001C, 0x1CF4, 0x0020, 0x001C, 0x0000, /*338F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x001D, 0x1F21, 0x0020, 0x001C, 0x0000, /*3390*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001C, 0x1D18, 0x0020, 0x001D, 0x1F21, /*3391*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DAA, 0x0020, 0x001D, 0x1D18, 0x0020, 0x001D, 0x1F21, /*3392*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1CF4, 0x0020, 0x001D, 0x1D18, 0x0020, 0x001D, 0x1F21, /*3393*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1E95, 0x0020, 0x001D, 0x1D18, 0x0020, 0x001D, 0x1F21, /*3394*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1FCB, 0x0020, 0x001C, 0x1D77, 0x0020, 0x001C, 0x0000, /*3395*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001C, 0x1D77, 0x0020, 0x001C, 0x0000, /*3396*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x001C, 0x1D77, 0x0020, 0x001C, 0x0000, /*3397*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001C, 0x1D77, 0x0020, 0x001C, 0x0000, /*3398*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CE5, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x0000, /*3399*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x0000, /*339A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCB, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x0000, /*339B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x0000, /*339C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x0000, /*339D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x0000, /*339E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x1C3F, /*339F*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C7A, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x1C3F, /*33A0*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DAA, 0x0020, 0x001C, 0x1C3F, 0x0020, 0x001C, 0x0000, /*33A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x1C3F, /*33A2*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DAA, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x1C40, /*33A3*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C7A, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x1C40, /*33A4*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DAA, 0x0020, 0x001C, 0x1C40, 0x0020, 0x001C, 0x0000, /*33A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x1C40, /*33A6*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DAA, 0x0020, 0x001C, 0x0625, 0x0020, 0x001C, 0x1E71, /*33A7*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DAA, 0x0020, 0x001C, 0x0625, 0x0020, 0x001C, 0x1E71, /*33A8*/
  0x0020, 0x001C, 0x1C3F, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1E0C, 0x0020, 0x001D, 0x1C47, 0x0020, 0x001C, 0x0000, /*33A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001C, 0x1E0C, 0x0020, 0x001D, 0x1C47, /*33AA*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DAA, 0x0020, 0x001D, 0x1E0C, 0x0020, 0x001D, 0x1C47, /*33AB*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1CF4, 0x0020, 0x001D, 0x1E0C, 0x0020, 0x001D, 0x1C47, /*33AC*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1E33, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001C, 0x1C8F, /*33AD*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1E33, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001C, 0x1C8F, /*33AE*/
  0x0020, 0x001C, 0x0625, 0x0020, 0x001C, 0x1E71,
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0005,
  0x1E33, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001C, 0x1C8F, /*33AF*/
  0x0020, 0x001C, 0x0625, 0x0020, 0x001C, 0x1E71,
  0x0020, 0x001C, 0x1C3F, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x1E0C, 0x0020, 0x001C, 0x1E71, 0x0020, 0x001C, 0x0000, /*33B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x001C, 0x1E71, 0x0020, 0x001C, 0x0000, /*33B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCB, 0x0020, 0x001C, 0x1E71, 0x0020, 0x001C, 0x0000, /*33B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001C, 0x1E71, 0x0020, 0x001C, 0x0000, /*33B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E0C, 0x0020, 0x001C, 0x1EE3, 0x0020, 0x001D, 0x0000, /*33B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x001C, 0x1EE3, 0x0020, 0x001D, 0x0000, /*33B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCB, 0x0020, 0x001C, 0x1EE3, 0x0020, 0x001D, 0x0000, /*33B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001C, 0x1EE3, 0x0020, 0x001D, 0x0000, /*33B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001C, 0x1EE3, 0x0020, 0x001D, 0x0000, /*33B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001D, 0x1EE3, 0x0020, 0x001D, 0x0000, /*33B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E0C, 0x0020, 0x001C, 0x1EF5, 0x0020, 0x001D, 0x0000, /*33BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x001C, 0x1EF5, 0x0020, 0x001D, 0x0000, /*33BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1FCB, 0x0020, 0x001C, 0x1EF5, 0x0020, 0x001D, 0x0000, /*33BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001C, 0x1EF5, 0x0020, 0x001D, 0x0000, /*33BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001C, 0x1EF5, 0x0020, 0x001D, 0x0000, /*33BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001D, 0x1EF5, 0x0020, 0x001D, 0x0000, /*33BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001C, 0x1FE1, 0x0020, 0x001D, 0x0000, /*33C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001D, 0x1FE1, 0x0020, 0x001D, 0x0000, /*33C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x001C, 0x0277, 0x0020, 0x001C, 0x1DAA, /*33C2*/
  0x0020, 0x001C, 0x0277, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C60, 0x0020, 0x001D, 0x1E21, 0x0020, 0x001C, 0x0000, /*33C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x001C, 0x1C7A, 0x0020, 0x001C, 0x0000, /*33C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x001C, 0x1C8F, 0x0020, 0x001C, 0x0000, /*33C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x001D, 0x0625, 0x0020, 0x001C, 0x1D65, /*33C6*/
  0x0020, 0x001C, 0x1CF4, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C7A, 0x0020, 0x001D, 0x1DDD, 0x0020, 0x001C, 0x0277, /*33C7*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C8F, 0x0020, 0x001C, 0x1C60, 0x0020, 0x001D, 0x0000, /*33C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x001D, 0x1F0B, 0x0020, 0x001C, 0x0000, /*33C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001C, 0x0000, /*33CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x001D, 0x1E0C, 0x0020, 0x001D, 0x0000, /*33CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D32, 0x0020, 0x001C, 0x1DB9, 0x0020, 0x001C, 0x0000, /*33CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001D, 0x1D65, 0x0020, 0x001D, 0x0000, /*33CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001D, 0x1DAA, 0x0020, 0x001D, 0x0000, /*33CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x001C, 0x1E95, 0x0020, 0x001C, 0x0000, /*33CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x001C, 0x1DAA, 0x0020, 0x001C, 0x0000, /*33D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x001C, 0x1DB9, 0x0020, 0x001C, 0x0000, /*33D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D77, 0x0020, 0x001C, 0x1DDD, 0x0020, 0x001C, 0x1CF4, /*33D2*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1D77, 0x0020, 0x001C, 0x1EFF, 0x0020, 0x001C, 0x0000, /*33D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001C, 0x1C60, 0x0020, 0x001C, 0x0000, /*33D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001C, 0x1D32, 0x0020, 0x001C, 0x1D77, /*33D5*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DAA, 0x0020, 0x001C, 0x1DDD, 0x0020, 0x001C, 0x1D77, /*33D6*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1E0C, 0x0020, 0x001D, 0x1D18, 0x0020, 0x001D, 0x0000, /*33D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E0C, 0x0020, 0x001C, 0x0277, 0x0020, 0x001C, 0x1DAA, /*33D8*/
  0x0020, 0x001C, 0x0277, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1E0C, 0x0020, 0x001D, 0x1E0C, 0x0020, 0x001D, 0x1DAA, /*33D9*/
  0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1E0C, 0x0020, 0x001D, 0x1E33, 0x0020, 0x001D, 0x0000, /*33DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x001C, 0x1E33, 0x0020, 0x001C, 0x0000, /*33DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x001D, 0x1EE3, 0x0020, 0x001C, 0x0000, /*33DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x001D, 0x1C60, 0x0020, 0x001C, 0x0000, /*33DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EE3, 0x0020, 0x001D, 0x0625, 0x0020, 0x001C, 0x1DAA, /*33DE*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C47, 0x0020, 0x001D, 0x0625, 0x0020, 0x001C, 0x1DAA, /*33DF*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE5E5, /*33E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3F, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE5E5, /*33E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C40, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE5E5, /*33E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C41, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE5E5, /*33E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C42, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE5E5, /*33E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C43, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE5E5, /*33E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C44, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE5E5, /*33E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C45, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE5E5, /*33E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C46, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE5E5, /*33E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x0004, 0x1C3D, 0x0020, 0x0004, 0xFB40, /*33E9*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0xFB40, /*33EA*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C3F, 0x0020, 0x0004, 0xFB40, /*33EB*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C40, 0x0020, 0x0004, 0xFB40, /*33EC*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C41, 0x0020, 0x0004, 0xFB40, /*33ED*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C42, 0x0020, 0x0004, 0xFB40, /*33EE*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C43, 0x0020, 0x0004, 0xFB40, /*33EF*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C44, 0x0020, 0x0004, 0xFB40, /*33F0*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C45, 0x0020, 0x0004, 0xFB40, /*33F1*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3E, 0x0020, 0x0004, 0x1C46, 0x0020, 0x0004, 0xFB40, /*33F2*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C3D, 0x0020, 0x0004, 0xFB40, /*33F3*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0xFB40, /*33F4*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C3F, 0x0020, 0x0004, 0xFB40, /*33F5*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C40, 0x0020, 0x0004, 0xFB40, /*33F6*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C41, 0x0020, 0x0004, 0xFB40, /*33F7*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C42, 0x0020, 0x0004, 0xFB40, /*33F8*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C43, 0x0020, 0x0004, 0xFB40, /*33F9*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C44, 0x0020, 0x0004, 0xFB40, /*33FA*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C45, 0x0020, 0x0004, 0xFB40, /*33FB*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C3F, 0x0020, 0x0004, 0x1C46, 0x0020, 0x0004, 0xFB40, /*33FC*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C40, 0x0020, 0x0004, 0x1C3D, 0x0020, 0x0004, 0xFB40, /*33FD*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C40, 0x0020, 0x0004, 0x1C3E, 0x0020, 0x0004, 0xFB40, /*33FE*/
  0x0020, 0x0004, 0xE5E5, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1CF4, 0x0020, 0x001C, 0x1C47, 0x0020, 0x001C, 0x1D77, /*33FF*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003
};

uint16 uca900_p04D[]= {
  0xFB80, 0x0020, 0x0002, 0xCD00, 0x0000, 0x0000, 0x0002, /*4D00*/
  0xFB80, 0x0020, 0x0002, 0xCD01, 0x0000, 0x0000, 0x0002, /*4D01*/
  0xFB80, 0x0020, 0x0002, 0xCD02, 0x0000, 0x0000, 0x0002, /*4D02*/
  0xFB80, 0x0020, 0x0002, 0xCD03, 0x0000, 0x0000, 0x0002, /*4D03*/
  0xFB80, 0x0020, 0x0002, 0xCD04, 0x0000, 0x0000, 0x0002, /*4D04*/
  0xFB80, 0x0020, 0x0002, 0xCD05, 0x0000, 0x0000, 0x0002, /*4D05*/
  0xFB80, 0x0020, 0x0002, 0xCD06, 0x0000, 0x0000, 0x0002, /*4D06*/
  0xFB80, 0x0020, 0x0002, 0xCD07, 0x0000, 0x0000, 0x0002, /*4D07*/
  0xFB80, 0x0020, 0x0002, 0xCD08, 0x0000, 0x0000, 0x0002, /*4D08*/
  0xFB80, 0x0020, 0x0002, 0xCD09, 0x0000, 0x0000, 0x0002, /*4D09*/
  0xFB80, 0x0020, 0x0002, 0xCD0A, 0x0000, 0x0000, 0x0002, /*4D0A*/
  0xFB80, 0x0020, 0x0002, 0xCD0B, 0x0000, 0x0000, 0x0002, /*4D0B*/
  0xFB80, 0x0020, 0x0002, 0xCD0C, 0x0000, 0x0000, 0x0002, /*4D0C*/
  0xFB80, 0x0020, 0x0002, 0xCD0D, 0x0000, 0x0000, 0x0002, /*4D0D*/
  0xFB80, 0x0020, 0x0002, 0xCD0E, 0x0000, 0x0000, 0x0002, /*4D0E*/
  0xFB80, 0x0020, 0x0002, 0xCD0F, 0x0000, 0x0000, 0x0002, /*4D0F*/
  0xFB80, 0x0020, 0x0002, 0xCD10, 0x0000, 0x0000, 0x0002, /*4D10*/
  0xFB80, 0x0020, 0x0002, 0xCD11, 0x0000, 0x0000, 0x0002, /*4D11*/
  0xFB80, 0x0020, 0x0002, 0xCD12, 0x0000, 0x0000, 0x0002, /*4D12*/
  0xFB80, 0x0020, 0x0002, 0xCD13, 0x0000, 0x0000, 0x0002, /*4D13*/
  0xFB80, 0x0020, 0x0002, 0xCD14, 0x0000, 0x0000, 0x0002, /*4D14*/
  0xFB80, 0x0020, 0x0002, 0xCD15, 0x0000, 0x0000, 0x0002, /*4D15*/
  0xFB80, 0x0020, 0x0002, 0xCD16, 0x0000, 0x0000, 0x0002, /*4D16*/
  0xFB80, 0x0020, 0x0002, 0xCD17, 0x0000, 0x0000, 0x0002, /*4D17*/
  0xFB80, 0x0020, 0x0002, 0xCD18, 0x0000, 0x0000, 0x0002, /*4D18*/
  0xFB80, 0x0020, 0x0002, 0xCD19, 0x0000, 0x0000, 0x0002, /*4D19*/
  0xFB80, 0x0020, 0x0002, 0xCD1A, 0x0000, 0x0000, 0x0002, /*4D1A*/
  0xFB80, 0x0020, 0x0002, 0xCD1B, 0x0000, 0x0000, 0x0002, /*4D1B*/
  0xFB80, 0x0020, 0x0002, 0xCD1C, 0x0000, 0x0000, 0x0002, /*4D1C*/
  0xFB80, 0x0020, 0x0002, 0xCD1D, 0x0000, 0x0000, 0x0002, /*4D1D*/
  0xFB80, 0x0020, 0x0002, 0xCD1E, 0x0000, 0x0000, 0x0002, /*4D1E*/
  0xFB80, 0x0020, 0x0002, 0xCD1F, 0x0000, 0x0000, 0x0002, /*4D1F*/
  0xFB80, 0x0020, 0x0002, 0xCD20, 0x0000, 0x0000, 0x0002, /*4D20*/
  0xFB80, 0x0020, 0x0002, 0xCD21, 0x0000, 0x0000, 0x0002, /*4D21*/
  0xFB80, 0x0020, 0x0002, 0xCD22, 0x0000, 0x0000, 0x0002, /*4D22*/
  0xFB80, 0x0020, 0x0002, 0xCD23, 0x0000, 0x0000, 0x0002, /*4D23*/
  0xFB80, 0x0020, 0x0002, 0xCD24, 0x0000, 0x0000, 0x0002, /*4D24*/
  0xFB80, 0x0020, 0x0002, 0xCD25, 0x0000, 0x0000, 0x0002, /*4D25*/
  0xFB80, 0x0020, 0x0002, 0xCD26, 0x0000, 0x0000, 0x0002, /*4D26*/
  0xFB80, 0x0020, 0x0002, 0xCD27, 0x0000, 0x0000, 0x0002, /*4D27*/
  0xFB80, 0x0020, 0x0002, 0xCD28, 0x0000, 0x0000, 0x0002, /*4D28*/
  0xFB80, 0x0020, 0x0002, 0xCD29, 0x0000, 0x0000, 0x0002, /*4D29*/
  0xFB80, 0x0020, 0x0002, 0xCD2A, 0x0000, 0x0000, 0x0002, /*4D2A*/
  0xFB80, 0x0020, 0x0002, 0xCD2B, 0x0000, 0x0000, 0x0002, /*4D2B*/
  0xFB80, 0x0020, 0x0002, 0xCD2C, 0x0000, 0x0000, 0x0002, /*4D2C*/
  0xFB80, 0x0020, 0x0002, 0xCD2D, 0x0000, 0x0000, 0x0002, /*4D2D*/
  0xFB80, 0x0020, 0x0002, 0xCD2E, 0x0000, 0x0000, 0x0002, /*4D2E*/
  0xFB80, 0x0020, 0x0002, 0xCD2F, 0x0000, 0x0000, 0x0002, /*4D2F*/
  0xFB80, 0x0020, 0x0002, 0xCD30, 0x0000, 0x0000, 0x0002, /*4D30*/
  0xFB80, 0x0020, 0x0002, 0xCD31, 0x0000, 0x0000, 0x0002, /*4D31*/
  0xFB80, 0x0020, 0x0002, 0xCD32, 0x0000, 0x0000, 0x0002, /*4D32*/
  0xFB80, 0x0020, 0x0002, 0xCD33, 0x0000, 0x0000, 0x0002, /*4D33*/
  0xFB80, 0x0020, 0x0002, 0xCD34, 0x0000, 0x0000, 0x0002, /*4D34*/
  0xFB80, 0x0020, 0x0002, 0xCD35, 0x0000, 0x0000, 0x0002, /*4D35*/
  0xFB80, 0x0020, 0x0002, 0xCD36, 0x0000, 0x0000, 0x0002, /*4D36*/
  0xFB80, 0x0020, 0x0002, 0xCD37, 0x0000, 0x0000, 0x0002, /*4D37*/
  0xFB80, 0x0020, 0x0002, 0xCD38, 0x0000, 0x0000, 0x0002, /*4D38*/
  0xFB80, 0x0020, 0x0002, 0xCD39, 0x0000, 0x0000, 0x0002, /*4D39*/
  0xFB80, 0x0020, 0x0002, 0xCD3A, 0x0000, 0x0000, 0x0002, /*4D3A*/
  0xFB80, 0x0020, 0x0002, 0xCD3B, 0x0000, 0x0000, 0x0002, /*4D3B*/
  0xFB80, 0x0020, 0x0002, 0xCD3C, 0x0000, 0x0000, 0x0002, /*4D3C*/
  0xFB80, 0x0020, 0x0002, 0xCD3D, 0x0000, 0x0000, 0x0002, /*4D3D*/
  0xFB80, 0x0020, 0x0002, 0xCD3E, 0x0000, 0x0000, 0x0002, /*4D3E*/
  0xFB80, 0x0020, 0x0002, 0xCD3F, 0x0000, 0x0000, 0x0002, /*4D3F*/
  0xFB80, 0x0020, 0x0002, 0xCD40, 0x0000, 0x0000, 0x0002, /*4D40*/
  0xFB80, 0x0020, 0x0002, 0xCD41, 0x0000, 0x0000, 0x0002, /*4D41*/
  0xFB80, 0x0020, 0x0002, 0xCD42, 0x0000, 0x0000, 0x0002, /*4D42*/
  0xFB80, 0x0020, 0x0002, 0xCD43, 0x0000, 0x0000, 0x0002, /*4D43*/
  0xFB80, 0x0020, 0x0002, 0xCD44, 0x0000, 0x0000, 0x0002, /*4D44*/
  0xFB80, 0x0020, 0x0002, 0xCD45, 0x0000, 0x0000, 0x0002, /*4D45*/
  0xFB80, 0x0020, 0x0002, 0xCD46, 0x0000, 0x0000, 0x0002, /*4D46*/
  0xFB80, 0x0020, 0x0002, 0xCD47, 0x0000, 0x0000, 0x0002, /*4D47*/
  0xFB80, 0x0020, 0x0002, 0xCD48, 0x0000, 0x0000, 0x0002, /*4D48*/
  0xFB80, 0x0020, 0x0002, 0xCD49, 0x0000, 0x0000, 0x0002, /*4D49*/
  0xFB80, 0x0020, 0x0002, 0xCD4A, 0x0000, 0x0000, 0x0002, /*4D4A*/
  0xFB80, 0x0020, 0x0002, 0xCD4B, 0x0000, 0x0000, 0x0002, /*4D4B*/
  0xFB80, 0x0020, 0x0002, 0xCD4C, 0x0000, 0x0000, 0x0002, /*4D4C*/
  0xFB80, 0x0020, 0x0002, 0xCD4D, 0x0000, 0x0000, 0x0002, /*4D4D*/
  0xFB80, 0x0020, 0x0002, 0xCD4E, 0x0000, 0x0000, 0x0002, /*4D4E*/
  0xFB80, 0x0020, 0x0002, 0xCD4F, 0x0000, 0x0000, 0x0002, /*4D4F*/
  0xFB80, 0x0020, 0x0002, 0xCD50, 0x0000, 0x0000, 0x0002, /*4D50*/
  0xFB80, 0x0020, 0x0002, 0xCD51, 0x0000, 0x0000, 0x0002, /*4D51*/
  0xFB80, 0x0020, 0x0002, 0xCD52, 0x0000, 0x0000, 0x0002, /*4D52*/
  0xFB80, 0x0020, 0x0002, 0xCD53, 0x0000, 0x0000, 0x0002, /*4D53*/
  0xFB80, 0x0020, 0x0002, 0xCD54, 0x0000, 0x0000, 0x0002, /*4D54*/
  0xFB80, 0x0020, 0x0002, 0xCD55, 0x0000, 0x0000, 0x0002, /*4D55*/
  0xFB80, 0x0020, 0x0002, 0xCD56, 0x0000, 0x0000, 0x0002, /*4D56*/
  0xFB80, 0x0020, 0x0002, 0xCD57, 0x0000, 0x0000, 0x0002, /*4D57*/
  0xFB80, 0x0020, 0x0002, 0xCD58, 0x0000, 0x0000, 0x0002, /*4D58*/
  0xFB80, 0x0020, 0x0002, 0xCD59, 0x0000, 0x0000, 0x0002, /*4D59*/
  0xFB80, 0x0020, 0x0002, 0xCD5A, 0x0000, 0x0000, 0x0002, /*4D5A*/
  0xFB80, 0x0020, 0x0002, 0xCD5B, 0x0000, 0x0000, 0x0002, /*4D5B*/
  0xFB80, 0x0020, 0x0002, 0xCD5C, 0x0000, 0x0000, 0x0002, /*4D5C*/
  0xFB80, 0x0020, 0x0002, 0xCD5D, 0x0000, 0x0000, 0x0002, /*4D5D*/
  0xFB80, 0x0020, 0x0002, 0xCD5E, 0x0000, 0x0000, 0x0002, /*4D5E*/
  0xFB80, 0x0020, 0x0002, 0xCD5F, 0x0000, 0x0000, 0x0002, /*4D5F*/
  0xFB80, 0x0020, 0x0002, 0xCD60, 0x0000, 0x0000, 0x0002, /*4D60*/
  0xFB80, 0x0020, 0x0002, 0xCD61, 0x0000, 0x0000, 0x0002, /*4D61*/
  0xFB80, 0x0020, 0x0002, 0xCD62, 0x0000, 0x0000, 0x0002, /*4D62*/
  0xFB80, 0x0020, 0x0002, 0xCD63, 0x0000, 0x0000, 0x0002, /*4D63*/
  0xFB80, 0x0020, 0x0002, 0xCD64, 0x0000, 0x0000, 0x0002, /*4D64*/
  0xFB80, 0x0020, 0x0002, 0xCD65, 0x0000, 0x0000, 0x0002, /*4D65*/
  0xFB80, 0x0020, 0x0002, 0xCD66, 0x0000, 0x0000, 0x0002, /*4D66*/
  0xFB80, 0x0020, 0x0002, 0xCD67, 0x0000, 0x0000, 0x0002, /*4D67*/
  0xFB80, 0x0020, 0x0002, 0xCD68, 0x0000, 0x0000, 0x0002, /*4D68*/
  0xFB80, 0x0020, 0x0002, 0xCD69, 0x0000, 0x0000, 0x0002, /*4D69*/
  0xFB80, 0x0020, 0x0002, 0xCD6A, 0x0000, 0x0000, 0x0002, /*4D6A*/
  0xFB80, 0x0020, 0x0002, 0xCD6B, 0x0000, 0x0000, 0x0002, /*4D6B*/
  0xFB80, 0x0020, 0x0002, 0xCD6C, 0x0000, 0x0000, 0x0002, /*4D6C*/
  0xFB80, 0x0020, 0x0002, 0xCD6D, 0x0000, 0x0000, 0x0002, /*4D6D*/
  0xFB80, 0x0020, 0x0002, 0xCD6E, 0x0000, 0x0000, 0x0002, /*4D6E*/
  0xFB80, 0x0020, 0x0002, 0xCD6F, 0x0000, 0x0000, 0x0002, /*4D6F*/
  0xFB80, 0x0020, 0x0002, 0xCD70, 0x0000, 0x0000, 0x0002, /*4D70*/
  0xFB80, 0x0020, 0x0002, 0xCD71, 0x0000, 0x0000, 0x0002, /*4D71*/
  0xFB80, 0x0020, 0x0002, 0xCD72, 0x0000, 0x0000, 0x0002, /*4D72*/
  0xFB80, 0x0020, 0x0002, 0xCD73, 0x0000, 0x0000, 0x0002, /*4D73*/
  0xFB80, 0x0020, 0x0002, 0xCD74, 0x0000, 0x0000, 0x0002, /*4D74*/
  0xFB80, 0x0020, 0x0002, 0xCD75, 0x0000, 0x0000, 0x0002, /*4D75*/
  0xFB80, 0x0020, 0x0002, 0xCD76, 0x0000, 0x0000, 0x0002, /*4D76*/
  0xFB80, 0x0020, 0x0002, 0xCD77, 0x0000, 0x0000, 0x0002, /*4D77*/
  0xFB80, 0x0020, 0x0002, 0xCD78, 0x0000, 0x0000, 0x0002, /*4D78*/
  0xFB80, 0x0020, 0x0002, 0xCD79, 0x0000, 0x0000, 0x0002, /*4D79*/
  0xFB80, 0x0020, 0x0002, 0xCD7A, 0x0000, 0x0000, 0x0002, /*4D7A*/
  0xFB80, 0x0020, 0x0002, 0xCD7B, 0x0000, 0x0000, 0x0002, /*4D7B*/
  0xFB80, 0x0020, 0x0002, 0xCD7C, 0x0000, 0x0000, 0x0002, /*4D7C*/
  0xFB80, 0x0020, 0x0002, 0xCD7D, 0x0000, 0x0000, 0x0002, /*4D7D*/
  0xFB80, 0x0020, 0x0002, 0xCD7E, 0x0000, 0x0000, 0x0002, /*4D7E*/
  0xFB80, 0x0020, 0x0002, 0xCD7F, 0x0000, 0x0000, 0x0002, /*4D7F*/
  0xFB80, 0x0020, 0x0002, 0xCD80, 0x0000, 0x0000, 0x0002, /*4D80*/
  0xFB80, 0x0020, 0x0002, 0xCD81, 0x0000, 0x0000, 0x0002, /*4D81*/
  0xFB80, 0x0020, 0x0002, 0xCD82, 0x0000, 0x0000, 0x0002, /*4D82*/
  0xFB80, 0x0020, 0x0002, 0xCD83, 0x0000, 0x0000, 0x0002, /*4D83*/
  0xFB80, 0x0020, 0x0002, 0xCD84, 0x0000, 0x0000, 0x0002, /*4D84*/
  0xFB80, 0x0020, 0x0002, 0xCD85, 0x0000, 0x0000, 0x0002, /*4D85*/
  0xFB80, 0x0020, 0x0002, 0xCD86, 0x0000, 0x0000, 0x0002, /*4D86*/
  0xFB80, 0x0020, 0x0002, 0xCD87, 0x0000, 0x0000, 0x0002, /*4D87*/
  0xFB80, 0x0020, 0x0002, 0xCD88, 0x0000, 0x0000, 0x0002, /*4D88*/
  0xFB80, 0x0020, 0x0002, 0xCD89, 0x0000, 0x0000, 0x0002, /*4D89*/
  0xFB80, 0x0020, 0x0002, 0xCD8A, 0x0000, 0x0000, 0x0002, /*4D8A*/
  0xFB80, 0x0020, 0x0002, 0xCD8B, 0x0000, 0x0000, 0x0002, /*4D8B*/
  0xFB80, 0x0020, 0x0002, 0xCD8C, 0x0000, 0x0000, 0x0002, /*4D8C*/
  0xFB80, 0x0020, 0x0002, 0xCD8D, 0x0000, 0x0000, 0x0002, /*4D8D*/
  0xFB80, 0x0020, 0x0002, 0xCD8E, 0x0000, 0x0000, 0x0002, /*4D8E*/
  0xFB80, 0x0020, 0x0002, 0xCD8F, 0x0000, 0x0000, 0x0002, /*4D8F*/
  0xFB80, 0x0020, 0x0002, 0xCD90, 0x0000, 0x0000, 0x0002, /*4D90*/
  0xFB80, 0x0020, 0x0002, 0xCD91, 0x0000, 0x0000, 0x0002, /*4D91*/
  0xFB80, 0x0020, 0x0002, 0xCD92, 0x0000, 0x0000, 0x0002, /*4D92*/
  0xFB80, 0x0020, 0x0002, 0xCD93, 0x0000, 0x0000, 0x0002, /*4D93*/
  0xFB80, 0x0020, 0x0002, 0xCD94, 0x0000, 0x0000, 0x0002, /*4D94*/
  0xFB80, 0x0020, 0x0002, 0xCD95, 0x0000, 0x0000, 0x0002, /*4D95*/
  0xFB80, 0x0020, 0x0002, 0xCD96, 0x0000, 0x0000, 0x0002, /*4D96*/
  0xFB80, 0x0020, 0x0002, 0xCD97, 0x0000, 0x0000, 0x0002, /*4D97*/
  0xFB80, 0x0020, 0x0002, 0xCD98, 0x0000, 0x0000, 0x0002, /*4D98*/
  0xFB80, 0x0020, 0x0002, 0xCD99, 0x0000, 0x0000, 0x0002, /*4D99*/
  0xFB80, 0x0020, 0x0002, 0xCD9A, 0x0000, 0x0000, 0x0002, /*4D9A*/
  0xFB80, 0x0020, 0x0002, 0xCD9B, 0x0000, 0x0000, 0x0002, /*4D9B*/
  0xFB80, 0x0020, 0x0002, 0xCD9C, 0x0000, 0x0000, 0x0002, /*4D9C*/
  0xFB80, 0x0020, 0x0002, 0xCD9D, 0x0000, 0x0000, 0x0002, /*4D9D*/
  0xFB80, 0x0020, 0x0002, 0xCD9E, 0x0000, 0x0000, 0x0002, /*4D9E*/
  0xFB80, 0x0020, 0x0002, 0xCD9F, 0x0000, 0x0000, 0x0002, /*4D9F*/
  0xFB80, 0x0020, 0x0002, 0xCDA0, 0x0000, 0x0000, 0x0002, /*4DA0*/
  0xFB80, 0x0020, 0x0002, 0xCDA1, 0x0000, 0x0000, 0x0002, /*4DA1*/
  0xFB80, 0x0020, 0x0002, 0xCDA2, 0x0000, 0x0000, 0x0002, /*4DA2*/
  0xFB80, 0x0020, 0x0002, 0xCDA3, 0x0000, 0x0000, 0x0002, /*4DA3*/
  0xFB80, 0x0020, 0x0002, 0xCDA4, 0x0000, 0x0000, 0x0002, /*4DA4*/
  0xFB80, 0x0020, 0x0002, 0xCDA5, 0x0000, 0x0000, 0x0002, /*4DA5*/
  0xFB80, 0x0020, 0x0002, 0xCDA6, 0x0000, 0x0000, 0x0002, /*4DA6*/
  0xFB80, 0x0020, 0x0002, 0xCDA7, 0x0000, 0x0000, 0x0002, /*4DA7*/
  0xFB80, 0x0020, 0x0002, 0xCDA8, 0x0000, 0x0000, 0x0002, /*4DA8*/
  0xFB80, 0x0020, 0x0002, 0xCDA9, 0x0000, 0x0000, 0x0002, /*4DA9*/
  0xFB80, 0x0020, 0x0002, 0xCDAA, 0x0000, 0x0000, 0x0002, /*4DAA*/
  0xFB80, 0x0020, 0x0002, 0xCDAB, 0x0000, 0x0000, 0x0002, /*4DAB*/
  0xFB80, 0x0020, 0x0002, 0xCDAC, 0x0000, 0x0000, 0x0002, /*4DAC*/
  0xFB80, 0x0020, 0x0002, 0xCDAD, 0x0000, 0x0000, 0x0002, /*4DAD*/
  0xFB80, 0x0020, 0x0002, 0xCDAE, 0x0000, 0x0000, 0x0002, /*4DAE*/
  0xFB80, 0x0020, 0x0002, 0xCDAF, 0x0000, 0x0000, 0x0002, /*4DAF*/
  0xFB80, 0x0020, 0x0002, 0xCDB0, 0x0000, 0x0000, 0x0002, /*4DB0*/
  0xFB80, 0x0020, 0x0002, 0xCDB1, 0x0000, 0x0000, 0x0002, /*4DB1*/
  0xFB80, 0x0020, 0x0002, 0xCDB2, 0x0000, 0x0000, 0x0002, /*4DB2*/
  0xFB80, 0x0020, 0x0002, 0xCDB3, 0x0000, 0x0000, 0x0002, /*4DB3*/
  0xFB80, 0x0020, 0x0002, 0xCDB4, 0x0000, 0x0000, 0x0002, /*4DB4*/
  0xFB80, 0x0020, 0x0002, 0xCDB5, 0x0000, 0x0000, 0x0002, /*4DB5*/
  0xFBC0, 0x0020, 0x0002, 0xCDB6, 0x0000, 0x0000, 0x0002, /*4DB6*/
  0xFBC0, 0x0020, 0x0002, 0xCDB7, 0x0000, 0x0000, 0x0002, /*4DB7*/
  0xFBC0, 0x0020, 0x0002, 0xCDB8, 0x0000, 0x0000, 0x0002, /*4DB8*/
  0xFBC0, 0x0020, 0x0002, 0xCDB9, 0x0000, 0x0000, 0x0002, /*4DB9*/
  0xFBC0, 0x0020, 0x0002, 0xCDBA, 0x0000, 0x0000, 0x0002, /*4DBA*/
  0xFBC0, 0x0020, 0x0002, 0xCDBB, 0x0000, 0x0000, 0x0002, /*4DBB*/
  0xFBC0, 0x0020, 0x0002, 0xCDBC, 0x0000, 0x0000, 0x0002, /*4DBC*/
  0xFBC0, 0x0020, 0x0002, 0xCDBD, 0x0000, 0x0000, 0x0002, /*4DBD*/
  0xFBC0, 0x0020, 0x0002, 0xCDBE, 0x0000, 0x0000, 0x0002, /*4DBE*/
  0xFBC0, 0x0020, 0x0002, 0xCDBF, 0x0000, 0x0000, 0x0002, /*4DBF*/
  0x0EAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DC0*/
  0x0EAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DC1*/
  0x0EAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DC2*/
  0x0EAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DC3*/
  0x0EAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DC4*/
  0x0EAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DC5*/
  0x0EB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DC6*/
  0x0EB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DC7*/
  0x0EB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DC8*/
  0x0EB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DC9*/
  0x0EB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DCA*/
  0x0EB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DCB*/
  0x0EB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DCC*/
  0x0EB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DCD*/
  0x0EB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DCE*/
  0x0EB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DCF*/
  0x0EBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DD0*/
  0x0EBB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DD1*/
  0x0EBC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DD2*/
  0x0EBD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DD3*/
  0x0EBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DD4*/
  0x0EBF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DD5*/
  0x0EC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DD6*/
  0x0EC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DD7*/
  0x0EC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DD8*/
  0x0EC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DD9*/
  0x0EC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DDA*/
  0x0EC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DDB*/
  0x0EC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DDC*/
  0x0EC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DDD*/
  0x0EC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DDE*/
  0x0EC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DDF*/
  0x0ECA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DE0*/
  0x0ECB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DE1*/
  0x0ECC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DE2*/
  0x0ECD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DE3*/
  0x0ECE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DE4*/
  0x0ECF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DE5*/
  0x0ED0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DE6*/
  0x0ED1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DE7*/
  0x0ED2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DE8*/
  0x0ED3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DE9*/
  0x0ED4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DEA*/
  0x0ED5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DEB*/
  0x0ED6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DEC*/
  0x0ED7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DED*/
  0x0ED8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DEE*/
  0x0ED9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DEF*/
  0x0EDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DF0*/
  0x0EDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DF1*/
  0x0EDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DF2*/
  0x0EDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DF3*/
  0x0EDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DF4*/
  0x0EDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DF5*/
  0x0EE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DF6*/
  0x0EE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DF7*/
  0x0EE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DF8*/
  0x0EE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DF9*/
  0x0EE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DFA*/
  0x0EE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DFB*/
  0x0EE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DFC*/
  0x0EE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DFD*/
  0x0EE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*4DFE*/
  0x0EE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*4DFF*/
};

uint16 uca900_p0A0[]= {
  0x3DBF, 0x0020, 0x0002, 0x0001, /*A000*/
  0x3DC0, 0x0020, 0x0002, 0x0001, /*A001*/
  0x3DC1, 0x0020, 0x0002, 0x0001, /*A002*/
  0x3DC2, 0x0020, 0x0002, 0x0001, /*A003*/
  0x3DC3, 0x0020, 0x0002, 0x0001, /*A004*/
  0x3DC4, 0x0020, 0x0002, 0x0001, /*A005*/
  0x3DC5, 0x0020, 0x0002, 0x0001, /*A006*/
  0x3DC6, 0x0020, 0x0002, 0x0001, /*A007*/
  0x3DC7, 0x0020, 0x0002, 0x0001, /*A008*/
  0x3DC8, 0x0020, 0x0002, 0x0001, /*A009*/
  0x3DC9, 0x0020, 0x0002, 0x0001, /*A00A*/
  0x3DCA, 0x0020, 0x0002, 0x0001, /*A00B*/
  0x3DCB, 0x0020, 0x0002, 0x0001, /*A00C*/
  0x3DCC, 0x0020, 0x0002, 0x0001, /*A00D*/
  0x3DCD, 0x0020, 0x0002, 0x0001, /*A00E*/
  0x3DCE, 0x0020, 0x0002, 0x0001, /*A00F*/
  0x3DCF, 0x0020, 0x0002, 0x0001, /*A010*/
  0x3DD0, 0x0020, 0x0002, 0x0001, /*A011*/
  0x3DD1, 0x0020, 0x0002, 0x0001, /*A012*/
  0x3DD2, 0x0020, 0x0002, 0x0001, /*A013*/
  0x3DD3, 0x0020, 0x0002, 0x0001, /*A014*/
  0x3DD4, 0x0020, 0x0002, 0x0001, /*A015*/
  0x3DD5, 0x0020, 0x0002, 0x0001, /*A016*/
  0x3DD6, 0x0020, 0x0002, 0x0001, /*A017*/
  0x3DD7, 0x0020, 0x0002, 0x0001, /*A018*/
  0x3DD8, 0x0020, 0x0002, 0x0001, /*A019*/
  0x3DD9, 0x0020, 0x0002, 0x0001, /*A01A*/
  0x3DDA, 0x0020, 0x0002, 0x0001, /*A01B*/
  0x3DDB, 0x0020, 0x0002, 0x0001, /*A01C*/
  0x3DDC, 0x0020, 0x0002, 0x0001, /*A01D*/
  0x3DDD, 0x0020, 0x0002, 0x0001, /*A01E*/
  0x3DDE, 0x0020, 0x0002, 0x0001, /*A01F*/
  0x3DDF, 0x0020, 0x0002, 0x0001, /*A020*/
  0x3DE0, 0x0020, 0x0002, 0x0001, /*A021*/
  0x3DE1, 0x0020, 0x0002, 0x0001, /*A022*/
  0x3DE2, 0x0020, 0x0002, 0x0001, /*A023*/
  0x3DE3, 0x0020, 0x0002, 0x0001, /*A024*/
  0x3DE4, 0x0020, 0x0002, 0x0001, /*A025*/
  0x3DE5, 0x0020, 0x0002, 0x0001, /*A026*/
  0x3DE6, 0x0020, 0x0002, 0x0001, /*A027*/
  0x3DE7, 0x0020, 0x0002, 0x0001, /*A028*/
  0x3DE8, 0x0020, 0x0002, 0x0001, /*A029*/
  0x3DE9, 0x0020, 0x0002, 0x0001, /*A02A*/
  0x3DEA, 0x0020, 0x0002, 0x0001, /*A02B*/
  0x3DEB, 0x0020, 0x0002, 0x0001, /*A02C*/
  0x3DEC, 0x0020, 0x0002, 0x0001, /*A02D*/
  0x3DED, 0x0020, 0x0002, 0x0001, /*A02E*/
  0x3DEE, 0x0020, 0x0002, 0x0001, /*A02F*/
  0x3DEF, 0x0020, 0x0002, 0x0001, /*A030*/
  0x3DF0, 0x0020, 0x0002, 0x0001, /*A031*/
  0x3DF1, 0x0020, 0x0002, 0x0001, /*A032*/
  0x3DF2, 0x0020, 0x0002, 0x0001, /*A033*/
  0x3DF3, 0x0020, 0x0002, 0x0001, /*A034*/
  0x3DF4, 0x0020, 0x0002, 0x0001, /*A035*/
  0x3DF5, 0x0020, 0x0002, 0x0001, /*A036*/
  0x3DF6, 0x0020, 0x0002, 0x0001, /*A037*/
  0x3DF7, 0x0020, 0x0002, 0x0001, /*A038*/
  0x3DF8, 0x0020, 0x0002, 0x0001, /*A039*/
  0x3DF9, 0x0020, 0x0002, 0x0001, /*A03A*/
  0x3DFA, 0x0020, 0x0002, 0x0001, /*A03B*/
  0x3DFB, 0x0020, 0x0002, 0x0001, /*A03C*/
  0x3DFC, 0x0020, 0x0002, 0x0001, /*A03D*/
  0x3DFD, 0x0020, 0x0002, 0x0001, /*A03E*/
  0x3DFE, 0x0020, 0x0002, 0x0001, /*A03F*/
  0x3DFF, 0x0020, 0x0002, 0x0001, /*A040*/
  0x3E00, 0x0020, 0x0002, 0x0001, /*A041*/
  0x3E01, 0x0020, 0x0002, 0x0001, /*A042*/
  0x3E02, 0x0020, 0x0002, 0x0001, /*A043*/
  0x3E03, 0x0020, 0x0002, 0x0001, /*A044*/
  0x3E04, 0x0020, 0x0002, 0x0001, /*A045*/
  0x3E05, 0x0020, 0x0002, 0x0001, /*A046*/
  0x3E06, 0x0020, 0x0002, 0x0001, /*A047*/
  0x3E07, 0x0020, 0x0002, 0x0001, /*A048*/
  0x3E08, 0x0020, 0x0002, 0x0001, /*A049*/
  0x3E09, 0x0020, 0x0002, 0x0001, /*A04A*/
  0x3E0A, 0x0020, 0x0002, 0x0001, /*A04B*/
  0x3E0B, 0x0020, 0x0002, 0x0001, /*A04C*/
  0x3E0C, 0x0020, 0x0002, 0x0001, /*A04D*/
  0x3E0D, 0x0020, 0x0002, 0x0001, /*A04E*/
  0x3E0E, 0x0020, 0x0002, 0x0001, /*A04F*/
  0x3E0F, 0x0020, 0x0002, 0x0001, /*A050*/
  0x3E10, 0x0020, 0x0002, 0x0001, /*A051*/
  0x3E11, 0x0020, 0x0002, 0x0001, /*A052*/
  0x3E12, 0x0020, 0x0002, 0x0001, /*A053*/
  0x3E13, 0x0020, 0x0002, 0x0001, /*A054*/
  0x3E14, 0x0020, 0x0002, 0x0001, /*A055*/
  0x3E15, 0x0020, 0x0002, 0x0001, /*A056*/
  0x3E16, 0x0020, 0x0002, 0x0001, /*A057*/
  0x3E17, 0x0020, 0x0002, 0x0001, /*A058*/
  0x3E18, 0x0020, 0x0002, 0x0001, /*A059*/
  0x3E19, 0x0020, 0x0002, 0x0001, /*A05A*/
  0x3E1A, 0x0020, 0x0002, 0x0001, /*A05B*/
  0x3E1B, 0x0020, 0x0002, 0x0001, /*A05C*/
  0x3E1C, 0x0020, 0x0002, 0x0001, /*A05D*/
  0x3E1D, 0x0020, 0x0002, 0x0001, /*A05E*/
  0x3E1E, 0x0020, 0x0002, 0x0001, /*A05F*/
  0x3E1F, 0x0020, 0x0002, 0x0001, /*A060*/
  0x3E20, 0x0020, 0x0002, 0x0001, /*A061*/
  0x3E21, 0x0020, 0x0002, 0x0001, /*A062*/
  0x3E22, 0x0020, 0x0002, 0x0001, /*A063*/
  0x3E23, 0x0020, 0x0002, 0x0001, /*A064*/
  0x3E24, 0x0020, 0x0002, 0x0001, /*A065*/
  0x3E25, 0x0020, 0x0002, 0x0001, /*A066*/
  0x3E26, 0x0020, 0x0002, 0x0001, /*A067*/
  0x3E27, 0x0020, 0x0002, 0x0001, /*A068*/
  0x3E28, 0x0020, 0x0002, 0x0001, /*A069*/
  0x3E29, 0x0020, 0x0002, 0x0001, /*A06A*/
  0x3E2A, 0x0020, 0x0002, 0x0001, /*A06B*/
  0x3E2B, 0x0020, 0x0002, 0x0001, /*A06C*/
  0x3E2C, 0x0020, 0x0002, 0x0001, /*A06D*/
  0x3E2D, 0x0020, 0x0002, 0x0001, /*A06E*/
  0x3E2E, 0x0020, 0x0002, 0x0001, /*A06F*/
  0x3E2F, 0x0020, 0x0002, 0x0001, /*A070*/
  0x3E30, 0x0020, 0x0002, 0x0001, /*A071*/
  0x3E31, 0x0020, 0x0002, 0x0001, /*A072*/
  0x3E32, 0x0020, 0x0002, 0x0001, /*A073*/
  0x3E33, 0x0020, 0x0002, 0x0001, /*A074*/
  0x3E34, 0x0020, 0x0002, 0x0001, /*A075*/
  0x3E35, 0x0020, 0x0002, 0x0001, /*A076*/
  0x3E36, 0x0020, 0x0002, 0x0001, /*A077*/
  0x3E37, 0x0020, 0x0002, 0x0001, /*A078*/
  0x3E38, 0x0020, 0x0002, 0x0001, /*A079*/
  0x3E39, 0x0020, 0x0002, 0x0001, /*A07A*/
  0x3E3A, 0x0020, 0x0002, 0x0001, /*A07B*/
  0x3E3B, 0x0020, 0x0002, 0x0001, /*A07C*/
  0x3E3C, 0x0020, 0x0002, 0x0001, /*A07D*/
  0x3E3D, 0x0020, 0x0002, 0x0001, /*A07E*/
  0x3E3E, 0x0020, 0x0002, 0x0001, /*A07F*/
  0x3E3F, 0x0020, 0x0002, 0x0001, /*A080*/
  0x3E40, 0x0020, 0x0002, 0x0001, /*A081*/
  0x3E41, 0x0020, 0x0002, 0x0001, /*A082*/
  0x3E42, 0x0020, 0x0002, 0x0001, /*A083*/
  0x3E43, 0x0020, 0x0002, 0x0001, /*A084*/
  0x3E44, 0x0020, 0x0002, 0x0001, /*A085*/
  0x3E45, 0x0020, 0x0002, 0x0001, /*A086*/
  0x3E46, 0x0020, 0x0002, 0x0001, /*A087*/
  0x3E47, 0x0020, 0x0002, 0x0001, /*A088*/
  0x3E48, 0x0020, 0x0002, 0x0001, /*A089*/
  0x3E49, 0x0020, 0x0002, 0x0001, /*A08A*/
  0x3E4A, 0x0020, 0x0002, 0x0001, /*A08B*/
  0x3E4B, 0x0020, 0x0002, 0x0001, /*A08C*/
  0x3E4C, 0x0020, 0x0002, 0x0001, /*A08D*/
  0x3E4D, 0x0020, 0x0002, 0x0001, /*A08E*/
  0x3E4E, 0x0020, 0x0002, 0x0001, /*A08F*/
  0x3E4F, 0x0020, 0x0002, 0x0001, /*A090*/
  0x3E50, 0x0020, 0x0002, 0x0001, /*A091*/
  0x3E51, 0x0020, 0x0002, 0x0001, /*A092*/
  0x3E52, 0x0020, 0x0002, 0x0001, /*A093*/
  0x3E53, 0x0020, 0x0002, 0x0001, /*A094*/
  0x3E54, 0x0020, 0x0002, 0x0001, /*A095*/
  0x3E55, 0x0020, 0x0002, 0x0001, /*A096*/
  0x3E56, 0x0020, 0x0002, 0x0001, /*A097*/
  0x3E57, 0x0020, 0x0002, 0x0001, /*A098*/
  0x3E58, 0x0020, 0x0002, 0x0001, /*A099*/
  0x3E59, 0x0020, 0x0002, 0x0001, /*A09A*/
  0x3E5A, 0x0020, 0x0002, 0x0001, /*A09B*/
  0x3E5B, 0x0020, 0x0002, 0x0001, /*A09C*/
  0x3E5C, 0x0020, 0x0002, 0x0001, /*A09D*/
  0x3E5D, 0x0020, 0x0002, 0x0001, /*A09E*/
  0x3E5E, 0x0020, 0x0002, 0x0001, /*A09F*/
  0x3E5F, 0x0020, 0x0002, 0x0001, /*A0A0*/
  0x3E60, 0x0020, 0x0002, 0x0001, /*A0A1*/
  0x3E61, 0x0020, 0x0002, 0x0001, /*A0A2*/
  0x3E62, 0x0020, 0x0002, 0x0001, /*A0A3*/
  0x3E63, 0x0020, 0x0002, 0x0001, /*A0A4*/
  0x3E64, 0x0020, 0x0002, 0x0001, /*A0A5*/
  0x3E65, 0x0020, 0x0002, 0x0001, /*A0A6*/
  0x3E66, 0x0020, 0x0002, 0x0001, /*A0A7*/
  0x3E67, 0x0020, 0x0002, 0x0001, /*A0A8*/
  0x3E68, 0x0020, 0x0002, 0x0001, /*A0A9*/
  0x3E69, 0x0020, 0x0002, 0x0001, /*A0AA*/
  0x3E6A, 0x0020, 0x0002, 0x0001, /*A0AB*/
  0x3E6B, 0x0020, 0x0002, 0x0001, /*A0AC*/
  0x3E6C, 0x0020, 0x0002, 0x0001, /*A0AD*/
  0x3E6D, 0x0020, 0x0002, 0x0001, /*A0AE*/
  0x3E6E, 0x0020, 0x0002, 0x0001, /*A0AF*/
  0x3E6F, 0x0020, 0x0002, 0x0001, /*A0B0*/
  0x3E70, 0x0020, 0x0002, 0x0001, /*A0B1*/
  0x3E71, 0x0020, 0x0002, 0x0001, /*A0B2*/
  0x3E72, 0x0020, 0x0002, 0x0001, /*A0B3*/
  0x3E73, 0x0020, 0x0002, 0x0001, /*A0B4*/
  0x3E74, 0x0020, 0x0002, 0x0001, /*A0B5*/
  0x3E75, 0x0020, 0x0002, 0x0001, /*A0B6*/
  0x3E76, 0x0020, 0x0002, 0x0001, /*A0B7*/
  0x3E77, 0x0020, 0x0002, 0x0001, /*A0B8*/
  0x3E78, 0x0020, 0x0002, 0x0001, /*A0B9*/
  0x3E79, 0x0020, 0x0002, 0x0001, /*A0BA*/
  0x3E7A, 0x0020, 0x0002, 0x0001, /*A0BB*/
  0x3E7B, 0x0020, 0x0002, 0x0001, /*A0BC*/
  0x3E7C, 0x0020, 0x0002, 0x0001, /*A0BD*/
  0x3E7D, 0x0020, 0x0002, 0x0001, /*A0BE*/
  0x3E7E, 0x0020, 0x0002, 0x0001, /*A0BF*/
  0x3E7F, 0x0020, 0x0002, 0x0001, /*A0C0*/
  0x3E80, 0x0020, 0x0002, 0x0001, /*A0C1*/
  0x3E81, 0x0020, 0x0002, 0x0001, /*A0C2*/
  0x3E82, 0x0020, 0x0002, 0x0001, /*A0C3*/
  0x3E83, 0x0020, 0x0002, 0x0001, /*A0C4*/
  0x3E84, 0x0020, 0x0002, 0x0001, /*A0C5*/
  0x3E85, 0x0020, 0x0002, 0x0001, /*A0C6*/
  0x3E86, 0x0020, 0x0002, 0x0001, /*A0C7*/
  0x3E87, 0x0020, 0x0002, 0x0001, /*A0C8*/
  0x3E88, 0x0020, 0x0002, 0x0001, /*A0C9*/
  0x3E89, 0x0020, 0x0002, 0x0001, /*A0CA*/
  0x3E8A, 0x0020, 0x0002, 0x0001, /*A0CB*/
  0x3E8B, 0x0020, 0x0002, 0x0001, /*A0CC*/
  0x3E8C, 0x0020, 0x0002, 0x0001, /*A0CD*/
  0x3E8D, 0x0020, 0x0002, 0x0001, /*A0CE*/
  0x3E8E, 0x0020, 0x0002, 0x0001, /*A0CF*/
  0x3E8F, 0x0020, 0x0002, 0x0001, /*A0D0*/
  0x3E90, 0x0020, 0x0002, 0x0001, /*A0D1*/
  0x3E91, 0x0020, 0x0002, 0x0001, /*A0D2*/
  0x3E92, 0x0020, 0x0002, 0x0001, /*A0D3*/
  0x3E93, 0x0020, 0x0002, 0x0001, /*A0D4*/
  0x3E94, 0x0020, 0x0002, 0x0001, /*A0D5*/
  0x3E95, 0x0020, 0x0002, 0x0001, /*A0D6*/
  0x3E96, 0x0020, 0x0002, 0x0001, /*A0D7*/
  0x3E97, 0x0020, 0x0002, 0x0001, /*A0D8*/
  0x3E98, 0x0020, 0x0002, 0x0001, /*A0D9*/
  0x3E99, 0x0020, 0x0002, 0x0001, /*A0DA*/
  0x3E9A, 0x0020, 0x0002, 0x0001, /*A0DB*/
  0x3E9B, 0x0020, 0x0002, 0x0001, /*A0DC*/
  0x3E9C, 0x0020, 0x0002, 0x0001, /*A0DD*/
  0x3E9D, 0x0020, 0x0002, 0x0001, /*A0DE*/
  0x3E9E, 0x0020, 0x0002, 0x0001, /*A0DF*/
  0x3E9F, 0x0020, 0x0002, 0x0001, /*A0E0*/
  0x3EA0, 0x0020, 0x0002, 0x0001, /*A0E1*/
  0x3EA1, 0x0020, 0x0002, 0x0001, /*A0E2*/
  0x3EA2, 0x0020, 0x0002, 0x0001, /*A0E3*/
  0x3EA3, 0x0020, 0x0002, 0x0001, /*A0E4*/
  0x3EA4, 0x0020, 0x0002, 0x0001, /*A0E5*/
  0x3EA5, 0x0020, 0x0002, 0x0001, /*A0E6*/
  0x3EA6, 0x0020, 0x0002, 0x0001, /*A0E7*/
  0x3EA7, 0x0020, 0x0002, 0x0001, /*A0E8*/
  0x3EA8, 0x0020, 0x0002, 0x0001, /*A0E9*/
  0x3EA9, 0x0020, 0x0002, 0x0001, /*A0EA*/
  0x3EAA, 0x0020, 0x0002, 0x0001, /*A0EB*/
  0x3EAB, 0x0020, 0x0002, 0x0001, /*A0EC*/
  0x3EAC, 0x0020, 0x0002, 0x0001, /*A0ED*/
  0x3EAD, 0x0020, 0x0002, 0x0001, /*A0EE*/
  0x3EAE, 0x0020, 0x0002, 0x0001, /*A0EF*/
  0x3EAF, 0x0020, 0x0002, 0x0001, /*A0F0*/
  0x3EB0, 0x0020, 0x0002, 0x0001, /*A0F1*/
  0x3EB1, 0x0020, 0x0002, 0x0001, /*A0F2*/
  0x3EB2, 0x0020, 0x0002, 0x0001, /*A0F3*/
  0x3EB3, 0x0020, 0x0002, 0x0001, /*A0F4*/
  0x3EB4, 0x0020, 0x0002, 0x0001, /*A0F5*/
  0x3EB5, 0x0020, 0x0002, 0x0001, /*A0F6*/
  0x3EB6, 0x0020, 0x0002, 0x0001, /*A0F7*/
  0x3EB7, 0x0020, 0x0002, 0x0001, /*A0F8*/
  0x3EB8, 0x0020, 0x0002, 0x0001, /*A0F9*/
  0x3EB9, 0x0020, 0x0002, 0x0001, /*A0FA*/
  0x3EBA, 0x0020, 0x0002, 0x0001, /*A0FB*/
  0x3EBB, 0x0020, 0x0002, 0x0001, /*A0FC*/
  0x3EBC, 0x0020, 0x0002, 0x0001, /*A0FD*/
  0x3EBD, 0x0020, 0x0002, 0x0001, /*A0FE*/
  0x3EBE, 0x0020, 0x0002, 0x0001  /*A0FF*/
};

uint16 uca900_p0A1[]= {
  0x3EBF, 0x0020, 0x0002, 0x0001, /*A100*/
  0x3EC0, 0x0020, 0x0002, 0x0001, /*A101*/
  0x3EC1, 0x0020, 0x0002, 0x0001, /*A102*/
  0x3EC2, 0x0020, 0x0002, 0x0001, /*A103*/
  0x3EC3, 0x0020, 0x0002, 0x0001, /*A104*/
  0x3EC4, 0x0020, 0x0002, 0x0001, /*A105*/
  0x3EC5, 0x0020, 0x0002, 0x0001, /*A106*/
  0x3EC6, 0x0020, 0x0002, 0x0001, /*A107*/
  0x3EC7, 0x0020, 0x0002, 0x0001, /*A108*/
  0x3EC8, 0x0020, 0x0002, 0x0001, /*A109*/
  0x3EC9, 0x0020, 0x0002, 0x0001, /*A10A*/
  0x3ECA, 0x0020, 0x0002, 0x0001, /*A10B*/
  0x3ECB, 0x0020, 0x0002, 0x0001, /*A10C*/
  0x3ECC, 0x0020, 0x0002, 0x0001, /*A10D*/
  0x3ECD, 0x0020, 0x0002, 0x0001, /*A10E*/
  0x3ECE, 0x0020, 0x0002, 0x0001, /*A10F*/
  0x3ECF, 0x0020, 0x0002, 0x0001, /*A110*/
  0x3ED0, 0x0020, 0x0002, 0x0001, /*A111*/
  0x3ED1, 0x0020, 0x0002, 0x0001, /*A112*/
  0x3ED2, 0x0020, 0x0002, 0x0001, /*A113*/
  0x3ED3, 0x0020, 0x0002, 0x0001, /*A114*/
  0x3ED4, 0x0020, 0x0002, 0x0001, /*A115*/
  0x3ED5, 0x0020, 0x0002, 0x0001, /*A116*/
  0x3ED6, 0x0020, 0x0002, 0x0001, /*A117*/
  0x3ED7, 0x0020, 0x0002, 0x0001, /*A118*/
  0x3ED8, 0x0020, 0x0002, 0x0001, /*A119*/
  0x3ED9, 0x0020, 0x0002, 0x0001, /*A11A*/
  0x3EDA, 0x0020, 0x0002, 0x0001, /*A11B*/
  0x3EDB, 0x0020, 0x0002, 0x0001, /*A11C*/
  0x3EDC, 0x0020, 0x0002, 0x0001, /*A11D*/
  0x3EDD, 0x0020, 0x0002, 0x0001, /*A11E*/
  0x3EDE, 0x0020, 0x0002, 0x0001, /*A11F*/
  0x3EDF, 0x0020, 0x0002, 0x0001, /*A120*/
  0x3EE0, 0x0020, 0x0002, 0x0001, /*A121*/
  0x3EE1, 0x0020, 0x0002, 0x0001, /*A122*/
  0x3EE2, 0x0020, 0x0002, 0x0001, /*A123*/
  0x3EE3, 0x0020, 0x0002, 0x0001, /*A124*/
  0x3EE4, 0x0020, 0x0002, 0x0001, /*A125*/
  0x3EE5, 0x0020, 0x0002, 0x0001, /*A126*/
  0x3EE6, 0x0020, 0x0002, 0x0001, /*A127*/
  0x3EE7, 0x0020, 0x0002, 0x0001, /*A128*/
  0x3EE8, 0x0020, 0x0002, 0x0001, /*A129*/
  0x3EE9, 0x0020, 0x0002, 0x0001, /*A12A*/
  0x3EEA, 0x0020, 0x0002, 0x0001, /*A12B*/
  0x3EEB, 0x0020, 0x0002, 0x0001, /*A12C*/
  0x3EEC, 0x0020, 0x0002, 0x0001, /*A12D*/
  0x3EED, 0x0020, 0x0002, 0x0001, /*A12E*/
  0x3EEE, 0x0020, 0x0002, 0x0001, /*A12F*/
  0x3EEF, 0x0020, 0x0002, 0x0001, /*A130*/
  0x3EF0, 0x0020, 0x0002, 0x0001, /*A131*/
  0x3EF1, 0x0020, 0x0002, 0x0001, /*A132*/
  0x3EF2, 0x0020, 0x0002, 0x0001, /*A133*/
  0x3EF3, 0x0020, 0x0002, 0x0001, /*A134*/
  0x3EF4, 0x0020, 0x0002, 0x0001, /*A135*/
  0x3EF5, 0x0020, 0x0002, 0x0001, /*A136*/
  0x3EF6, 0x0020, 0x0002, 0x0001, /*A137*/
  0x3EF7, 0x0020, 0x0002, 0x0001, /*A138*/
  0x3EF8, 0x0020, 0x0002, 0x0001, /*A139*/
  0x3EF9, 0x0020, 0x0002, 0x0001, /*A13A*/
  0x3EFA, 0x0020, 0x0002, 0x0001, /*A13B*/
  0x3EFB, 0x0020, 0x0002, 0x0001, /*A13C*/
  0x3EFC, 0x0020, 0x0002, 0x0001, /*A13D*/
  0x3EFD, 0x0020, 0x0002, 0x0001, /*A13E*/
  0x3EFE, 0x0020, 0x0002, 0x0001, /*A13F*/
  0x3EFF, 0x0020, 0x0002, 0x0001, /*A140*/
  0x3F00, 0x0020, 0x0002, 0x0001, /*A141*/
  0x3F01, 0x0020, 0x0002, 0x0001, /*A142*/
  0x3F02, 0x0020, 0x0002, 0x0001, /*A143*/
  0x3F03, 0x0020, 0x0002, 0x0001, /*A144*/
  0x3F04, 0x0020, 0x0002, 0x0001, /*A145*/
  0x3F05, 0x0020, 0x0002, 0x0001, /*A146*/
  0x3F06, 0x0020, 0x0002, 0x0001, /*A147*/
  0x3F07, 0x0020, 0x0002, 0x0001, /*A148*/
  0x3F08, 0x0020, 0x0002, 0x0001, /*A149*/
  0x3F09, 0x0020, 0x0002, 0x0001, /*A14A*/
  0x3F0A, 0x0020, 0x0002, 0x0001, /*A14B*/
  0x3F0B, 0x0020, 0x0002, 0x0001, /*A14C*/
  0x3F0C, 0x0020, 0x0002, 0x0001, /*A14D*/
  0x3F0D, 0x0020, 0x0002, 0x0001, /*A14E*/
  0x3F0E, 0x0020, 0x0002, 0x0001, /*A14F*/
  0x3F0F, 0x0020, 0x0002, 0x0001, /*A150*/
  0x3F10, 0x0020, 0x0002, 0x0001, /*A151*/
  0x3F11, 0x0020, 0x0002, 0x0001, /*A152*/
  0x3F12, 0x0020, 0x0002, 0x0001, /*A153*/
  0x3F13, 0x0020, 0x0002, 0x0001, /*A154*/
  0x3F14, 0x0020, 0x0002, 0x0001, /*A155*/
  0x3F15, 0x0020, 0x0002, 0x0001, /*A156*/
  0x3F16, 0x0020, 0x0002, 0x0001, /*A157*/
  0x3F17, 0x0020, 0x0002, 0x0001, /*A158*/
  0x3F18, 0x0020, 0x0002, 0x0001, /*A159*/
  0x3F19, 0x0020, 0x0002, 0x0001, /*A15A*/
  0x3F1A, 0x0020, 0x0002, 0x0001, /*A15B*/
  0x3F1B, 0x0020, 0x0002, 0x0001, /*A15C*/
  0x3F1C, 0x0020, 0x0002, 0x0001, /*A15D*/
  0x3F1D, 0x0020, 0x0002, 0x0001, /*A15E*/
  0x3F1E, 0x0020, 0x0002, 0x0001, /*A15F*/
  0x3F1F, 0x0020, 0x0002, 0x0001, /*A160*/
  0x3F20, 0x0020, 0x0002, 0x0001, /*A161*/
  0x3F21, 0x0020, 0x0002, 0x0001, /*A162*/
  0x3F22, 0x0020, 0x0002, 0x0001, /*A163*/
  0x3F23, 0x0020, 0x0002, 0x0001, /*A164*/
  0x3F24, 0x0020, 0x0002, 0x0001, /*A165*/
  0x3F25, 0x0020, 0x0002, 0x0001, /*A166*/
  0x3F26, 0x0020, 0x0002, 0x0001, /*A167*/
  0x3F27, 0x0020, 0x0002, 0x0001, /*A168*/
  0x3F28, 0x0020, 0x0002, 0x0001, /*A169*/
  0x3F29, 0x0020, 0x0002, 0x0001, /*A16A*/
  0x3F2A, 0x0020, 0x0002, 0x0001, /*A16B*/
  0x3F2B, 0x0020, 0x0002, 0x0001, /*A16C*/
  0x3F2C, 0x0020, 0x0002, 0x0001, /*A16D*/
  0x3F2D, 0x0020, 0x0002, 0x0001, /*A16E*/
  0x3F2E, 0x0020, 0x0002, 0x0001, /*A16F*/
  0x3F2F, 0x0020, 0x0002, 0x0001, /*A170*/
  0x3F30, 0x0020, 0x0002, 0x0001, /*A171*/
  0x3F31, 0x0020, 0x0002, 0x0001, /*A172*/
  0x3F32, 0x0020, 0x0002, 0x0001, /*A173*/
  0x3F33, 0x0020, 0x0002, 0x0001, /*A174*/
  0x3F34, 0x0020, 0x0002, 0x0001, /*A175*/
  0x3F35, 0x0020, 0x0002, 0x0001, /*A176*/
  0x3F36, 0x0020, 0x0002, 0x0001, /*A177*/
  0x3F37, 0x0020, 0x0002, 0x0001, /*A178*/
  0x3F38, 0x0020, 0x0002, 0x0001, /*A179*/
  0x3F39, 0x0020, 0x0002, 0x0001, /*A17A*/
  0x3F3A, 0x0020, 0x0002, 0x0001, /*A17B*/
  0x3F3B, 0x0020, 0x0002, 0x0001, /*A17C*/
  0x3F3C, 0x0020, 0x0002, 0x0001, /*A17D*/
  0x3F3D, 0x0020, 0x0002, 0x0001, /*A17E*/
  0x3F3E, 0x0020, 0x0002, 0x0001, /*A17F*/
  0x3F3F, 0x0020, 0x0002, 0x0001, /*A180*/
  0x3F40, 0x0020, 0x0002, 0x0001, /*A181*/
  0x3F41, 0x0020, 0x0002, 0x0001, /*A182*/
  0x3F42, 0x0020, 0x0002, 0x0001, /*A183*/
  0x3F43, 0x0020, 0x0002, 0x0001, /*A184*/
  0x3F44, 0x0020, 0x0002, 0x0001, /*A185*/
  0x3F45, 0x0020, 0x0002, 0x0001, /*A186*/
  0x3F46, 0x0020, 0x0002, 0x0001, /*A187*/
  0x3F47, 0x0020, 0x0002, 0x0001, /*A188*/
  0x3F48, 0x0020, 0x0002, 0x0001, /*A189*/
  0x3F49, 0x0020, 0x0002, 0x0001, /*A18A*/
  0x3F4A, 0x0020, 0x0002, 0x0001, /*A18B*/
  0x3F4B, 0x0020, 0x0002, 0x0001, /*A18C*/
  0x3F4C, 0x0020, 0x0002, 0x0001, /*A18D*/
  0x3F4D, 0x0020, 0x0002, 0x0001, /*A18E*/
  0x3F4E, 0x0020, 0x0002, 0x0001, /*A18F*/
  0x3F4F, 0x0020, 0x0002, 0x0001, /*A190*/
  0x3F50, 0x0020, 0x0002, 0x0001, /*A191*/
  0x3F51, 0x0020, 0x0002, 0x0001, /*A192*/
  0x3F52, 0x0020, 0x0002, 0x0001, /*A193*/
  0x3F53, 0x0020, 0x0002, 0x0001, /*A194*/
  0x3F54, 0x0020, 0x0002, 0x0001, /*A195*/
  0x3F55, 0x0020, 0x0002, 0x0001, /*A196*/
  0x3F56, 0x0020, 0x0002, 0x0001, /*A197*/
  0x3F57, 0x0020, 0x0002, 0x0001, /*A198*/
  0x3F58, 0x0020, 0x0002, 0x0001, /*A199*/
  0x3F59, 0x0020, 0x0002, 0x0001, /*A19A*/
  0x3F5A, 0x0020, 0x0002, 0x0001, /*A19B*/
  0x3F5B, 0x0020, 0x0002, 0x0001, /*A19C*/
  0x3F5C, 0x0020, 0x0002, 0x0001, /*A19D*/
  0x3F5D, 0x0020, 0x0002, 0x0001, /*A19E*/
  0x3F5E, 0x0020, 0x0002, 0x0001, /*A19F*/
  0x3F5F, 0x0020, 0x0002, 0x0001, /*A1A0*/
  0x3F60, 0x0020, 0x0002, 0x0001, /*A1A1*/
  0x3F61, 0x0020, 0x0002, 0x0001, /*A1A2*/
  0x3F62, 0x0020, 0x0002, 0x0001, /*A1A3*/
  0x3F63, 0x0020, 0x0002, 0x0001, /*A1A4*/
  0x3F64, 0x0020, 0x0002, 0x0001, /*A1A5*/
  0x3F65, 0x0020, 0x0002, 0x0001, /*A1A6*/
  0x3F66, 0x0020, 0x0002, 0x0001, /*A1A7*/
  0x3F67, 0x0020, 0x0002, 0x0001, /*A1A8*/
  0x3F68, 0x0020, 0x0002, 0x0001, /*A1A9*/
  0x3F69, 0x0020, 0x0002, 0x0001, /*A1AA*/
  0x3F6A, 0x0020, 0x0002, 0x0001, /*A1AB*/
  0x3F6B, 0x0020, 0x0002, 0x0001, /*A1AC*/
  0x3F6C, 0x0020, 0x0002, 0x0001, /*A1AD*/
  0x3F6D, 0x0020, 0x0002, 0x0001, /*A1AE*/
  0x3F6E, 0x0020, 0x0002, 0x0001, /*A1AF*/
  0x3F6F, 0x0020, 0x0002, 0x0001, /*A1B0*/
  0x3F70, 0x0020, 0x0002, 0x0001, /*A1B1*/
  0x3F71, 0x0020, 0x0002, 0x0001, /*A1B2*/
  0x3F72, 0x0020, 0x0002, 0x0001, /*A1B3*/
  0x3F73, 0x0020, 0x0002, 0x0001, /*A1B4*/
  0x3F74, 0x0020, 0x0002, 0x0001, /*A1B5*/
  0x3F75, 0x0020, 0x0002, 0x0001, /*A1B6*/
  0x3F76, 0x0020, 0x0002, 0x0001, /*A1B7*/
  0x3F77, 0x0020, 0x0002, 0x0001, /*A1B8*/
  0x3F78, 0x0020, 0x0002, 0x0001, /*A1B9*/
  0x3F79, 0x0020, 0x0002, 0x0001, /*A1BA*/
  0x3F7A, 0x0020, 0x0002, 0x0001, /*A1BB*/
  0x3F7B, 0x0020, 0x0002, 0x0001, /*A1BC*/
  0x3F7C, 0x0020, 0x0002, 0x0001, /*A1BD*/
  0x3F7D, 0x0020, 0x0002, 0x0001, /*A1BE*/
  0x3F7E, 0x0020, 0x0002, 0x0001, /*A1BF*/
  0x3F7F, 0x0020, 0x0002, 0x0001, /*A1C0*/
  0x3F80, 0x0020, 0x0002, 0x0001, /*A1C1*/
  0x3F81, 0x0020, 0x0002, 0x0001, /*A1C2*/
  0x3F82, 0x0020, 0x0002, 0x0001, /*A1C3*/
  0x3F83, 0x0020, 0x0002, 0x0001, /*A1C4*/
  0x3F84, 0x0020, 0x0002, 0x0001, /*A1C5*/
  0x3F85, 0x0020, 0x0002, 0x0001, /*A1C6*/
  0x3F86, 0x0020, 0x0002, 0x0001, /*A1C7*/
  0x3F87, 0x0020, 0x0002, 0x0001, /*A1C8*/
  0x3F88, 0x0020, 0x0002, 0x0001, /*A1C9*/
  0x3F89, 0x0020, 0x0002, 0x0001, /*A1CA*/
  0x3F8A, 0x0020, 0x0002, 0x0001, /*A1CB*/
  0x3F8B, 0x0020, 0x0002, 0x0001, /*A1CC*/
  0x3F8C, 0x0020, 0x0002, 0x0001, /*A1CD*/
  0x3F8D, 0x0020, 0x0002, 0x0001, /*A1CE*/
  0x3F8E, 0x0020, 0x0002, 0x0001, /*A1CF*/
  0x3F8F, 0x0020, 0x0002, 0x0001, /*A1D0*/
  0x3F90, 0x0020, 0x0002, 0x0001, /*A1D1*/
  0x3F91, 0x0020, 0x0002, 0x0001, /*A1D2*/
  0x3F92, 0x0020, 0x0002, 0x0001, /*A1D3*/
  0x3F93, 0x0020, 0x0002, 0x0001, /*A1D4*/
  0x3F94, 0x0020, 0x0002, 0x0001, /*A1D5*/
  0x3F95, 0x0020, 0x0002, 0x0001, /*A1D6*/
  0x3F96, 0x0020, 0x0002, 0x0001, /*A1D7*/
  0x3F97, 0x0020, 0x0002, 0x0001, /*A1D8*/
  0x3F98, 0x0020, 0x0002, 0x0001, /*A1D9*/
  0x3F99, 0x0020, 0x0002, 0x0001, /*A1DA*/
  0x3F9A, 0x0020, 0x0002, 0x0001, /*A1DB*/
  0x3F9B, 0x0020, 0x0002, 0x0001, /*A1DC*/
  0x3F9C, 0x0020, 0x0002, 0x0001, /*A1DD*/
  0x3F9D, 0x0020, 0x0002, 0x0001, /*A1DE*/
  0x3F9E, 0x0020, 0x0002, 0x0001, /*A1DF*/
  0x3F9F, 0x0020, 0x0002, 0x0001, /*A1E0*/
  0x3FA0, 0x0020, 0x0002, 0x0001, /*A1E1*/
  0x3FA1, 0x0020, 0x0002, 0x0001, /*A1E2*/
  0x3FA2, 0x0020, 0x0002, 0x0001, /*A1E3*/
  0x3FA3, 0x0020, 0x0002, 0x0001, /*A1E4*/
  0x3FA4, 0x0020, 0x0002, 0x0001, /*A1E5*/
  0x3FA5, 0x0020, 0x0002, 0x0001, /*A1E6*/
  0x3FA6, 0x0020, 0x0002, 0x0001, /*A1E7*/
  0x3FA7, 0x0020, 0x0002, 0x0001, /*A1E8*/
  0x3FA8, 0x0020, 0x0002, 0x0001, /*A1E9*/
  0x3FA9, 0x0020, 0x0002, 0x0001, /*A1EA*/
  0x3FAA, 0x0020, 0x0002, 0x0001, /*A1EB*/
  0x3FAB, 0x0020, 0x0002, 0x0001, /*A1EC*/
  0x3FAC, 0x0020, 0x0002, 0x0001, /*A1ED*/
  0x3FAD, 0x0020, 0x0002, 0x0001, /*A1EE*/
  0x3FAE, 0x0020, 0x0002, 0x0001, /*A1EF*/
  0x3FAF, 0x0020, 0x0002, 0x0001, /*A1F0*/
  0x3FB0, 0x0020, 0x0002, 0x0001, /*A1F1*/
  0x3FB1, 0x0020, 0x0002, 0x0001, /*A1F2*/
  0x3FB2, 0x0020, 0x0002, 0x0001, /*A1F3*/
  0x3FB3, 0x0020, 0x0002, 0x0001, /*A1F4*/
  0x3FB4, 0x0020, 0x0002, 0x0001, /*A1F5*/
  0x3FB5, 0x0020, 0x0002, 0x0001, /*A1F6*/
  0x3FB6, 0x0020, 0x0002, 0x0001, /*A1F7*/
  0x3FB7, 0x0020, 0x0002, 0x0001, /*A1F8*/
  0x3FB8, 0x0020, 0x0002, 0x0001, /*A1F9*/
  0x3FB9, 0x0020, 0x0002, 0x0001, /*A1FA*/
  0x3FBA, 0x0020, 0x0002, 0x0001, /*A1FB*/
  0x3FBB, 0x0020, 0x0002, 0x0001, /*A1FC*/
  0x3FBC, 0x0020, 0x0002, 0x0001, /*A1FD*/
  0x3FBD, 0x0020, 0x0002, 0x0001, /*A1FE*/
  0x3FBE, 0x0020, 0x0002, 0x0001  /*A1FF*/
};

uint16 uca900_p0A2[]= {
  0x3FBF, 0x0020, 0x0002, 0x0001, /*A200*/
  0x3FC0, 0x0020, 0x0002, 0x0001, /*A201*/
  0x3FC1, 0x0020, 0x0002, 0x0001, /*A202*/
  0x3FC2, 0x0020, 0x0002, 0x0001, /*A203*/
  0x3FC3, 0x0020, 0x0002, 0x0001, /*A204*/
  0x3FC4, 0x0020, 0x0002, 0x0001, /*A205*/
  0x3FC5, 0x0020, 0x0002, 0x0001, /*A206*/
  0x3FC6, 0x0020, 0x0002, 0x0001, /*A207*/
  0x3FC7, 0x0020, 0x0002, 0x0001, /*A208*/
  0x3FC8, 0x0020, 0x0002, 0x0001, /*A209*/
  0x3FC9, 0x0020, 0x0002, 0x0001, /*A20A*/
  0x3FCA, 0x0020, 0x0002, 0x0001, /*A20B*/
  0x3FCB, 0x0020, 0x0002, 0x0001, /*A20C*/
  0x3FCC, 0x0020, 0x0002, 0x0001, /*A20D*/
  0x3FCD, 0x0020, 0x0002, 0x0001, /*A20E*/
  0x3FCE, 0x0020, 0x0002, 0x0001, /*A20F*/
  0x3FCF, 0x0020, 0x0002, 0x0001, /*A210*/
  0x3FD0, 0x0020, 0x0002, 0x0001, /*A211*/
  0x3FD1, 0x0020, 0x0002, 0x0001, /*A212*/
  0x3FD2, 0x0020, 0x0002, 0x0001, /*A213*/
  0x3FD3, 0x0020, 0x0002, 0x0001, /*A214*/
  0x3FD4, 0x0020, 0x0002, 0x0001, /*A215*/
  0x3FD5, 0x0020, 0x0002, 0x0001, /*A216*/
  0x3FD6, 0x0020, 0x0002, 0x0001, /*A217*/
  0x3FD7, 0x0020, 0x0002, 0x0001, /*A218*/
  0x3FD8, 0x0020, 0x0002, 0x0001, /*A219*/
  0x3FD9, 0x0020, 0x0002, 0x0001, /*A21A*/
  0x3FDA, 0x0020, 0x0002, 0x0001, /*A21B*/
  0x3FDB, 0x0020, 0x0002, 0x0001, /*A21C*/
  0x3FDC, 0x0020, 0x0002, 0x0001, /*A21D*/
  0x3FDD, 0x0020, 0x0002, 0x0001, /*A21E*/
  0x3FDE, 0x0020, 0x0002, 0x0001, /*A21F*/
  0x3FDF, 0x0020, 0x0002, 0x0001, /*A220*/
  0x3FE0, 0x0020, 0x0002, 0x0001, /*A221*/
  0x3FE1, 0x0020, 0x0002, 0x0001, /*A222*/
  0x3FE2, 0x0020, 0x0002, 0x0001, /*A223*/
  0x3FE3, 0x0020, 0x0002, 0x0001, /*A224*/
  0x3FE4, 0x0020, 0x0002, 0x0001, /*A225*/
  0x3FE5, 0x0020, 0x0002, 0x0001, /*A226*/
  0x3FE6, 0x0020, 0x0002, 0x0001, /*A227*/
  0x3FE7, 0x0020, 0x0002, 0x0001, /*A228*/
  0x3FE8, 0x0020, 0x0002, 0x0001, /*A229*/
  0x3FE9, 0x0020, 0x0002, 0x0001, /*A22A*/
  0x3FEA, 0x0020, 0x0002, 0x0001, /*A22B*/
  0x3FEB, 0x0020, 0x0002, 0x0001, /*A22C*/
  0x3FEC, 0x0020, 0x0002, 0x0001, /*A22D*/
  0x3FED, 0x0020, 0x0002, 0x0001, /*A22E*/
  0x3FEE, 0x0020, 0x0002, 0x0001, /*A22F*/
  0x3FEF, 0x0020, 0x0002, 0x0001, /*A230*/
  0x3FF0, 0x0020, 0x0002, 0x0001, /*A231*/
  0x3FF1, 0x0020, 0x0002, 0x0001, /*A232*/
  0x3FF2, 0x0020, 0x0002, 0x0001, /*A233*/
  0x3FF3, 0x0020, 0x0002, 0x0001, /*A234*/
  0x3FF4, 0x0020, 0x0002, 0x0001, /*A235*/
  0x3FF5, 0x0020, 0x0002, 0x0001, /*A236*/
  0x3FF6, 0x0020, 0x0002, 0x0001, /*A237*/
  0x3FF7, 0x0020, 0x0002, 0x0001, /*A238*/
  0x3FF8, 0x0020, 0x0002, 0x0001, /*A239*/
  0x3FF9, 0x0020, 0x0002, 0x0001, /*A23A*/
  0x3FFA, 0x0020, 0x0002, 0x0001, /*A23B*/
  0x3FFB, 0x0020, 0x0002, 0x0001, /*A23C*/
  0x3FFC, 0x0020, 0x0002, 0x0001, /*A23D*/
  0x3FFD, 0x0020, 0x0002, 0x0001, /*A23E*/
  0x3FFE, 0x0020, 0x0002, 0x0001, /*A23F*/
  0x3FFF, 0x0020, 0x0002, 0x0001, /*A240*/
  0x4000, 0x0020, 0x0002, 0x0001, /*A241*/
  0x4001, 0x0020, 0x0002, 0x0001, /*A242*/
  0x4002, 0x0020, 0x0002, 0x0001, /*A243*/
  0x4003, 0x0020, 0x0002, 0x0001, /*A244*/
  0x4004, 0x0020, 0x0002, 0x0001, /*A245*/
  0x4005, 0x0020, 0x0002, 0x0001, /*A246*/
  0x4006, 0x0020, 0x0002, 0x0001, /*A247*/
  0x4007, 0x0020, 0x0002, 0x0001, /*A248*/
  0x4008, 0x0020, 0x0002, 0x0001, /*A249*/
  0x4009, 0x0020, 0x0002, 0x0001, /*A24A*/
  0x400A, 0x0020, 0x0002, 0x0001, /*A24B*/
  0x400B, 0x0020, 0x0002, 0x0001, /*A24C*/
  0x400C, 0x0020, 0x0002, 0x0001, /*A24D*/
  0x400D, 0x0020, 0x0002, 0x0001, /*A24E*/
  0x400E, 0x0020, 0x0002, 0x0001, /*A24F*/
  0x400F, 0x0020, 0x0002, 0x0001, /*A250*/
  0x4010, 0x0020, 0x0002, 0x0001, /*A251*/
  0x4011, 0x0020, 0x0002, 0x0001, /*A252*/
  0x4012, 0x0020, 0x0002, 0x0001, /*A253*/
  0x4013, 0x0020, 0x0002, 0x0001, /*A254*/
  0x4014, 0x0020, 0x0002, 0x0001, /*A255*/
  0x4015, 0x0020, 0x0002, 0x0001, /*A256*/
  0x4016, 0x0020, 0x0002, 0x0001, /*A257*/
  0x4017, 0x0020, 0x0002, 0x0001, /*A258*/
  0x4018, 0x0020, 0x0002, 0x0001, /*A259*/
  0x4019, 0x0020, 0x0002, 0x0001, /*A25A*/
  0x401A, 0x0020, 0x0002, 0x0001, /*A25B*/
  0x401B, 0x0020, 0x0002, 0x0001, /*A25C*/
  0x401C, 0x0020, 0x0002, 0x0001, /*A25D*/
  0x401D, 0x0020, 0x0002, 0x0001, /*A25E*/
  0x401E, 0x0020, 0x0002, 0x0001, /*A25F*/
  0x401F, 0x0020, 0x0002, 0x0001, /*A260*/
  0x4020, 0x0020, 0x0002, 0x0001, /*A261*/
  0x4021, 0x0020, 0x0002, 0x0001, /*A262*/
  0x4022, 0x0020, 0x0002, 0x0001, /*A263*/
  0x4023, 0x0020, 0x0002, 0x0001, /*A264*/
  0x4024, 0x0020, 0x0002, 0x0001, /*A265*/
  0x4025, 0x0020, 0x0002, 0x0001, /*A266*/
  0x4026, 0x0020, 0x0002, 0x0001, /*A267*/
  0x4027, 0x0020, 0x0002, 0x0001, /*A268*/
  0x4028, 0x0020, 0x0002, 0x0001, /*A269*/
  0x4029, 0x0020, 0x0002, 0x0001, /*A26A*/
  0x402A, 0x0020, 0x0002, 0x0001, /*A26B*/
  0x402B, 0x0020, 0x0002, 0x0001, /*A26C*/
  0x402C, 0x0020, 0x0002, 0x0001, /*A26D*/
  0x402D, 0x0020, 0x0002, 0x0001, /*A26E*/
  0x402E, 0x0020, 0x0002, 0x0001, /*A26F*/
  0x402F, 0x0020, 0x0002, 0x0001, /*A270*/
  0x4030, 0x0020, 0x0002, 0x0001, /*A271*/
  0x4031, 0x0020, 0x0002, 0x0001, /*A272*/
  0x4032, 0x0020, 0x0002, 0x0001, /*A273*/
  0x4033, 0x0020, 0x0002, 0x0001, /*A274*/
  0x4034, 0x0020, 0x0002, 0x0001, /*A275*/
  0x4035, 0x0020, 0x0002, 0x0001, /*A276*/
  0x4036, 0x0020, 0x0002, 0x0001, /*A277*/
  0x4037, 0x0020, 0x0002, 0x0001, /*A278*/
  0x4038, 0x0020, 0x0002, 0x0001, /*A279*/
  0x4039, 0x0020, 0x0002, 0x0001, /*A27A*/
  0x403A, 0x0020, 0x0002, 0x0001, /*A27B*/
  0x403B, 0x0020, 0x0002, 0x0001, /*A27C*/
  0x403C, 0x0020, 0x0002, 0x0001, /*A27D*/
  0x403D, 0x0020, 0x0002, 0x0001, /*A27E*/
  0x403E, 0x0020, 0x0002, 0x0001, /*A27F*/
  0x403F, 0x0020, 0x0002, 0x0001, /*A280*/
  0x4040, 0x0020, 0x0002, 0x0001, /*A281*/
  0x4041, 0x0020, 0x0002, 0x0001, /*A282*/
  0x4042, 0x0020, 0x0002, 0x0001, /*A283*/
  0x4043, 0x0020, 0x0002, 0x0001, /*A284*/
  0x4044, 0x0020, 0x0002, 0x0001, /*A285*/
  0x4045, 0x0020, 0x0002, 0x0001, /*A286*/
  0x4046, 0x0020, 0x0002, 0x0001, /*A287*/
  0x4047, 0x0020, 0x0002, 0x0001, /*A288*/
  0x4048, 0x0020, 0x0002, 0x0001, /*A289*/
  0x4049, 0x0020, 0x0002, 0x0001, /*A28A*/
  0x404A, 0x0020, 0x0002, 0x0001, /*A28B*/
  0x404B, 0x0020, 0x0002, 0x0001, /*A28C*/
  0x404C, 0x0020, 0x0002, 0x0001, /*A28D*/
  0x404D, 0x0020, 0x0002, 0x0001, /*A28E*/
  0x404E, 0x0020, 0x0002, 0x0001, /*A28F*/
  0x404F, 0x0020, 0x0002, 0x0001, /*A290*/
  0x4050, 0x0020, 0x0002, 0x0001, /*A291*/
  0x4051, 0x0020, 0x0002, 0x0001, /*A292*/
  0x4052, 0x0020, 0x0002, 0x0001, /*A293*/
  0x4053, 0x0020, 0x0002, 0x0001, /*A294*/
  0x4054, 0x0020, 0x0002, 0x0001, /*A295*/
  0x4055, 0x0020, 0x0002, 0x0001, /*A296*/
  0x4056, 0x0020, 0x0002, 0x0001, /*A297*/
  0x4057, 0x0020, 0x0002, 0x0001, /*A298*/
  0x4058, 0x0020, 0x0002, 0x0001, /*A299*/
  0x4059, 0x0020, 0x0002, 0x0001, /*A29A*/
  0x405A, 0x0020, 0x0002, 0x0001, /*A29B*/
  0x405B, 0x0020, 0x0002, 0x0001, /*A29C*/
  0x405C, 0x0020, 0x0002, 0x0001, /*A29D*/
  0x405D, 0x0020, 0x0002, 0x0001, /*A29E*/
  0x405E, 0x0020, 0x0002, 0x0001, /*A29F*/
  0x405F, 0x0020, 0x0002, 0x0001, /*A2A0*/
  0x4060, 0x0020, 0x0002, 0x0001, /*A2A1*/
  0x4061, 0x0020, 0x0002, 0x0001, /*A2A2*/
  0x4062, 0x0020, 0x0002, 0x0001, /*A2A3*/
  0x4063, 0x0020, 0x0002, 0x0001, /*A2A4*/
  0x4064, 0x0020, 0x0002, 0x0001, /*A2A5*/
  0x4065, 0x0020, 0x0002, 0x0001, /*A2A6*/
  0x4066, 0x0020, 0x0002, 0x0001, /*A2A7*/
  0x4067, 0x0020, 0x0002, 0x0001, /*A2A8*/
  0x4068, 0x0020, 0x0002, 0x0001, /*A2A9*/
  0x4069, 0x0020, 0x0002, 0x0001, /*A2AA*/
  0x406A, 0x0020, 0x0002, 0x0001, /*A2AB*/
  0x406B, 0x0020, 0x0002, 0x0001, /*A2AC*/
  0x406C, 0x0020, 0x0002, 0x0001, /*A2AD*/
  0x406D, 0x0020, 0x0002, 0x0001, /*A2AE*/
  0x406E, 0x0020, 0x0002, 0x0001, /*A2AF*/
  0x406F, 0x0020, 0x0002, 0x0001, /*A2B0*/
  0x4070, 0x0020, 0x0002, 0x0001, /*A2B1*/
  0x4071, 0x0020, 0x0002, 0x0001, /*A2B2*/
  0x4072, 0x0020, 0x0002, 0x0001, /*A2B3*/
  0x4073, 0x0020, 0x0002, 0x0001, /*A2B4*/
  0x4074, 0x0020, 0x0002, 0x0001, /*A2B5*/
  0x4075, 0x0020, 0x0002, 0x0001, /*A2B6*/
  0x4076, 0x0020, 0x0002, 0x0001, /*A2B7*/
  0x4077, 0x0020, 0x0002, 0x0001, /*A2B8*/
  0x4078, 0x0020, 0x0002, 0x0001, /*A2B9*/
  0x4079, 0x0020, 0x0002, 0x0001, /*A2BA*/
  0x407A, 0x0020, 0x0002, 0x0001, /*A2BB*/
  0x407B, 0x0020, 0x0002, 0x0001, /*A2BC*/
  0x407C, 0x0020, 0x0002, 0x0001, /*A2BD*/
  0x407D, 0x0020, 0x0002, 0x0001, /*A2BE*/
  0x407E, 0x0020, 0x0002, 0x0001, /*A2BF*/
  0x407F, 0x0020, 0x0002, 0x0001, /*A2C0*/
  0x4080, 0x0020, 0x0002, 0x0001, /*A2C1*/
  0x4081, 0x0020, 0x0002, 0x0001, /*A2C2*/
  0x4082, 0x0020, 0x0002, 0x0001, /*A2C3*/
  0x4083, 0x0020, 0x0002, 0x0001, /*A2C4*/
  0x4084, 0x0020, 0x0002, 0x0001, /*A2C5*/
  0x4085, 0x0020, 0x0002, 0x0001, /*A2C6*/
  0x4086, 0x0020, 0x0002, 0x0001, /*A2C7*/
  0x4087, 0x0020, 0x0002, 0x0001, /*A2C8*/
  0x4088, 0x0020, 0x0002, 0x0001, /*A2C9*/
  0x4089, 0x0020, 0x0002, 0x0001, /*A2CA*/
  0x408A, 0x0020, 0x0002, 0x0001, /*A2CB*/
  0x408B, 0x0020, 0x0002, 0x0001, /*A2CC*/
  0x408C, 0x0020, 0x0002, 0x0001, /*A2CD*/
  0x408D, 0x0020, 0x0002, 0x0001, /*A2CE*/
  0x408E, 0x0020, 0x0002, 0x0001, /*A2CF*/
  0x408F, 0x0020, 0x0002, 0x0001, /*A2D0*/
  0x4090, 0x0020, 0x0002, 0x0001, /*A2D1*/
  0x4091, 0x0020, 0x0002, 0x0001, /*A2D2*/
  0x4092, 0x0020, 0x0002, 0x0001, /*A2D3*/
  0x4093, 0x0020, 0x0002, 0x0001, /*A2D4*/
  0x4094, 0x0020, 0x0002, 0x0001, /*A2D5*/
  0x4095, 0x0020, 0x0002, 0x0001, /*A2D6*/
  0x4096, 0x0020, 0x0002, 0x0001, /*A2D7*/
  0x4097, 0x0020, 0x0002, 0x0001, /*A2D8*/
  0x4098, 0x0020, 0x0002, 0x0001, /*A2D9*/
  0x4099, 0x0020, 0x0002, 0x0001, /*A2DA*/
  0x409A, 0x0020, 0x0002, 0x0001, /*A2DB*/
  0x409B, 0x0020, 0x0002, 0x0001, /*A2DC*/
  0x409C, 0x0020, 0x0002, 0x0001, /*A2DD*/
  0x409D, 0x0020, 0x0002, 0x0001, /*A2DE*/
  0x409E, 0x0020, 0x0002, 0x0001, /*A2DF*/
  0x409F, 0x0020, 0x0002, 0x0001, /*A2E0*/
  0x40A0, 0x0020, 0x0002, 0x0001, /*A2E1*/
  0x40A1, 0x0020, 0x0002, 0x0001, /*A2E2*/
  0x40A2, 0x0020, 0x0002, 0x0001, /*A2E3*/
  0x40A3, 0x0020, 0x0002, 0x0001, /*A2E4*/
  0x40A4, 0x0020, 0x0002, 0x0001, /*A2E5*/
  0x40A5, 0x0020, 0x0002, 0x0001, /*A2E6*/
  0x40A6, 0x0020, 0x0002, 0x0001, /*A2E7*/
  0x40A7, 0x0020, 0x0002, 0x0001, /*A2E8*/
  0x40A8, 0x0020, 0x0002, 0x0001, /*A2E9*/
  0x40A9, 0x0020, 0x0002, 0x0001, /*A2EA*/
  0x40AA, 0x0020, 0x0002, 0x0001, /*A2EB*/
  0x40AB, 0x0020, 0x0002, 0x0001, /*A2EC*/
  0x40AC, 0x0020, 0x0002, 0x0001, /*A2ED*/
  0x40AD, 0x0020, 0x0002, 0x0001, /*A2EE*/
  0x40AE, 0x0020, 0x0002, 0x0001, /*A2EF*/
  0x40AF, 0x0020, 0x0002, 0x0001, /*A2F0*/
  0x40B0, 0x0020, 0x0002, 0x0001, /*A2F1*/
  0x40B1, 0x0020, 0x0002, 0x0001, /*A2F2*/
  0x40B2, 0x0020, 0x0002, 0x0001, /*A2F3*/
  0x40B3, 0x0020, 0x0002, 0x0001, /*A2F4*/
  0x40B4, 0x0020, 0x0002, 0x0001, /*A2F5*/
  0x40B5, 0x0020, 0x0002, 0x0001, /*A2F6*/
  0x40B6, 0x0020, 0x0002, 0x0001, /*A2F7*/
  0x40B7, 0x0020, 0x0002, 0x0001, /*A2F8*/
  0x40B8, 0x0020, 0x0002, 0x0001, /*A2F9*/
  0x40B9, 0x0020, 0x0002, 0x0001, /*A2FA*/
  0x40BA, 0x0020, 0x0002, 0x0001, /*A2FB*/
  0x40BB, 0x0020, 0x0002, 0x0001, /*A2FC*/
  0x40BC, 0x0020, 0x0002, 0x0001, /*A2FD*/
  0x40BD, 0x0020, 0x0002, 0x0001, /*A2FE*/
  0x40BE, 0x0020, 0x0002, 0x0001  /*A2FF*/
};

uint16 uca900_p0A3[]= {
  0x40BF, 0x0020, 0x0002, 0x0001, /*A300*/
  0x40C0, 0x0020, 0x0002, 0x0001, /*A301*/
  0x40C1, 0x0020, 0x0002, 0x0001, /*A302*/
  0x40C2, 0x0020, 0x0002, 0x0001, /*A303*/
  0x40C3, 0x0020, 0x0002, 0x0001, /*A304*/
  0x40C4, 0x0020, 0x0002, 0x0001, /*A305*/
  0x40C5, 0x0020, 0x0002, 0x0001, /*A306*/
  0x40C6, 0x0020, 0x0002, 0x0001, /*A307*/
  0x40C7, 0x0020, 0x0002, 0x0001, /*A308*/
  0x40C8, 0x0020, 0x0002, 0x0001, /*A309*/
  0x40C9, 0x0020, 0x0002, 0x0001, /*A30A*/
  0x40CA, 0x0020, 0x0002, 0x0001, /*A30B*/
  0x40CB, 0x0020, 0x0002, 0x0001, /*A30C*/
  0x40CC, 0x0020, 0x0002, 0x0001, /*A30D*/
  0x40CD, 0x0020, 0x0002, 0x0001, /*A30E*/
  0x40CE, 0x0020, 0x0002, 0x0001, /*A30F*/
  0x40CF, 0x0020, 0x0002, 0x0001, /*A310*/
  0x40D0, 0x0020, 0x0002, 0x0001, /*A311*/
  0x40D1, 0x0020, 0x0002, 0x0001, /*A312*/
  0x40D2, 0x0020, 0x0002, 0x0001, /*A313*/
  0x40D3, 0x0020, 0x0002, 0x0001, /*A314*/
  0x40D4, 0x0020, 0x0002, 0x0001, /*A315*/
  0x40D5, 0x0020, 0x0002, 0x0001, /*A316*/
  0x40D6, 0x0020, 0x0002, 0x0001, /*A317*/
  0x40D7, 0x0020, 0x0002, 0x0001, /*A318*/
  0x40D8, 0x0020, 0x0002, 0x0001, /*A319*/
  0x40D9, 0x0020, 0x0002, 0x0001, /*A31A*/
  0x40DA, 0x0020, 0x0002, 0x0001, /*A31B*/
  0x40DB, 0x0020, 0x0002, 0x0001, /*A31C*/
  0x40DC, 0x0020, 0x0002, 0x0001, /*A31D*/
  0x40DD, 0x0020, 0x0002, 0x0001, /*A31E*/
  0x40DE, 0x0020, 0x0002, 0x0001, /*A31F*/
  0x40DF, 0x0020, 0x0002, 0x0001, /*A320*/
  0x40E0, 0x0020, 0x0002, 0x0001, /*A321*/
  0x40E1, 0x0020, 0x0002, 0x0001, /*A322*/
  0x40E2, 0x0020, 0x0002, 0x0001, /*A323*/
  0x40E3, 0x0020, 0x0002, 0x0001, /*A324*/
  0x40E4, 0x0020, 0x0002, 0x0001, /*A325*/
  0x40E5, 0x0020, 0x0002, 0x0001, /*A326*/
  0x40E6, 0x0020, 0x0002, 0x0001, /*A327*/
  0x40E7, 0x0020, 0x0002, 0x0001, /*A328*/
  0x40E8, 0x0020, 0x0002, 0x0001, /*A329*/
  0x40E9, 0x0020, 0x0002, 0x0001, /*A32A*/
  0x40EA, 0x0020, 0x0002, 0x0001, /*A32B*/
  0x40EB, 0x0020, 0x0002, 0x0001, /*A32C*/
  0x40EC, 0x0020, 0x0002, 0x0001, /*A32D*/
  0x40ED, 0x0020, 0x0002, 0x0001, /*A32E*/
  0x40EE, 0x0020, 0x0002, 0x0001, /*A32F*/
  0x40EF, 0x0020, 0x0002, 0x0001, /*A330*/
  0x40F0, 0x0020, 0x0002, 0x0001, /*A331*/
  0x40F1, 0x0020, 0x0002, 0x0001, /*A332*/
  0x40F2, 0x0020, 0x0002, 0x0001, /*A333*/
  0x40F3, 0x0020, 0x0002, 0x0001, /*A334*/
  0x40F4, 0x0020, 0x0002, 0x0001, /*A335*/
  0x40F5, 0x0020, 0x0002, 0x0001, /*A336*/
  0x40F6, 0x0020, 0x0002, 0x0001, /*A337*/
  0x40F7, 0x0020, 0x0002, 0x0001, /*A338*/
  0x40F8, 0x0020, 0x0002, 0x0001, /*A339*/
  0x40F9, 0x0020, 0x0002, 0x0001, /*A33A*/
  0x40FA, 0x0020, 0x0002, 0x0001, /*A33B*/
  0x40FB, 0x0020, 0x0002, 0x0001, /*A33C*/
  0x40FC, 0x0020, 0x0002, 0x0001, /*A33D*/
  0x40FD, 0x0020, 0x0002, 0x0001, /*A33E*/
  0x40FE, 0x0020, 0x0002, 0x0001, /*A33F*/
  0x40FF, 0x0020, 0x0002, 0x0001, /*A340*/
  0x4100, 0x0020, 0x0002, 0x0001, /*A341*/
  0x4101, 0x0020, 0x0002, 0x0001, /*A342*/
  0x4102, 0x0020, 0x0002, 0x0001, /*A343*/
  0x4103, 0x0020, 0x0002, 0x0001, /*A344*/
  0x4104, 0x0020, 0x0002, 0x0001, /*A345*/
  0x4105, 0x0020, 0x0002, 0x0001, /*A346*/
  0x4106, 0x0020, 0x0002, 0x0001, /*A347*/
  0x4107, 0x0020, 0x0002, 0x0001, /*A348*/
  0x4108, 0x0020, 0x0002, 0x0001, /*A349*/
  0x4109, 0x0020, 0x0002, 0x0001, /*A34A*/
  0x410A, 0x0020, 0x0002, 0x0001, /*A34B*/
  0x410B, 0x0020, 0x0002, 0x0001, /*A34C*/
  0x410C, 0x0020, 0x0002, 0x0001, /*A34D*/
  0x410D, 0x0020, 0x0002, 0x0001, /*A34E*/
  0x410E, 0x0020, 0x0002, 0x0001, /*A34F*/
  0x410F, 0x0020, 0x0002, 0x0001, /*A350*/
  0x4110, 0x0020, 0x0002, 0x0001, /*A351*/
  0x4111, 0x0020, 0x0002, 0x0001, /*A352*/
  0x4112, 0x0020, 0x0002, 0x0001, /*A353*/
  0x4113, 0x0020, 0x0002, 0x0001, /*A354*/
  0x4114, 0x0020, 0x0002, 0x0001, /*A355*/
  0x4115, 0x0020, 0x0002, 0x0001, /*A356*/
  0x4116, 0x0020, 0x0002, 0x0001, /*A357*/
  0x4117, 0x0020, 0x0002, 0x0001, /*A358*/
  0x4118, 0x0020, 0x0002, 0x0001, /*A359*/
  0x4119, 0x0020, 0x0002, 0x0001, /*A35A*/
  0x411A, 0x0020, 0x0002, 0x0001, /*A35B*/
  0x411B, 0x0020, 0x0002, 0x0001, /*A35C*/
  0x411C, 0x0020, 0x0002, 0x0001, /*A35D*/
  0x411D, 0x0020, 0x0002, 0x0001, /*A35E*/
  0x411E, 0x0020, 0x0002, 0x0001, /*A35F*/
  0x411F, 0x0020, 0x0002, 0x0001, /*A360*/
  0x4120, 0x0020, 0x0002, 0x0001, /*A361*/
  0x4121, 0x0020, 0x0002, 0x0001, /*A362*/
  0x4122, 0x0020, 0x0002, 0x0001, /*A363*/
  0x4123, 0x0020, 0x0002, 0x0001, /*A364*/
  0x4124, 0x0020, 0x0002, 0x0001, /*A365*/
  0x4125, 0x0020, 0x0002, 0x0001, /*A366*/
  0x4126, 0x0020, 0x0002, 0x0001, /*A367*/
  0x4127, 0x0020, 0x0002, 0x0001, /*A368*/
  0x4128, 0x0020, 0x0002, 0x0001, /*A369*/
  0x4129, 0x0020, 0x0002, 0x0001, /*A36A*/
  0x412A, 0x0020, 0x0002, 0x0001, /*A36B*/
  0x412B, 0x0020, 0x0002, 0x0001, /*A36C*/
  0x412C, 0x0020, 0x0002, 0x0001, /*A36D*/
  0x412D, 0x0020, 0x0002, 0x0001, /*A36E*/
  0x412E, 0x0020, 0x0002, 0x0001, /*A36F*/
  0x412F, 0x0020, 0x0002, 0x0001, /*A370*/
  0x4130, 0x0020, 0x0002, 0x0001, /*A371*/
  0x4131, 0x0020, 0x0002, 0x0001, /*A372*/
  0x4132, 0x0020, 0x0002, 0x0001, /*A373*/
  0x4133, 0x0020, 0x0002, 0x0001, /*A374*/
  0x4134, 0x0020, 0x0002, 0x0001, /*A375*/
  0x4135, 0x0020, 0x0002, 0x0001, /*A376*/
  0x4136, 0x0020, 0x0002, 0x0001, /*A377*/
  0x4137, 0x0020, 0x0002, 0x0001, /*A378*/
  0x4138, 0x0020, 0x0002, 0x0001, /*A379*/
  0x4139, 0x0020, 0x0002, 0x0001, /*A37A*/
  0x413A, 0x0020, 0x0002, 0x0001, /*A37B*/
  0x413B, 0x0020, 0x0002, 0x0001, /*A37C*/
  0x413C, 0x0020, 0x0002, 0x0001, /*A37D*/
  0x413D, 0x0020, 0x0002, 0x0001, /*A37E*/
  0x413E, 0x0020, 0x0002, 0x0001, /*A37F*/
  0x413F, 0x0020, 0x0002, 0x0001, /*A380*/
  0x4140, 0x0020, 0x0002, 0x0001, /*A381*/
  0x4141, 0x0020, 0x0002, 0x0001, /*A382*/
  0x4142, 0x0020, 0x0002, 0x0001, /*A383*/
  0x4143, 0x0020, 0x0002, 0x0001, /*A384*/
  0x4144, 0x0020, 0x0002, 0x0001, /*A385*/
  0x4145, 0x0020, 0x0002, 0x0001, /*A386*/
  0x4146, 0x0020, 0x0002, 0x0001, /*A387*/
  0x4147, 0x0020, 0x0002, 0x0001, /*A388*/
  0x4148, 0x0020, 0x0002, 0x0001, /*A389*/
  0x4149, 0x0020, 0x0002, 0x0001, /*A38A*/
  0x414A, 0x0020, 0x0002, 0x0001, /*A38B*/
  0x414B, 0x0020, 0x0002, 0x0001, /*A38C*/
  0x414C, 0x0020, 0x0002, 0x0001, /*A38D*/
  0x414D, 0x0020, 0x0002, 0x0001, /*A38E*/
  0x414E, 0x0020, 0x0002, 0x0001, /*A38F*/
  0x414F, 0x0020, 0x0002, 0x0001, /*A390*/
  0x4150, 0x0020, 0x0002, 0x0001, /*A391*/
  0x4151, 0x0020, 0x0002, 0x0001, /*A392*/
  0x4152, 0x0020, 0x0002, 0x0001, /*A393*/
  0x4153, 0x0020, 0x0002, 0x0001, /*A394*/
  0x4154, 0x0020, 0x0002, 0x0001, /*A395*/
  0x4155, 0x0020, 0x0002, 0x0001, /*A396*/
  0x4156, 0x0020, 0x0002, 0x0001, /*A397*/
  0x4157, 0x0020, 0x0002, 0x0001, /*A398*/
  0x4158, 0x0020, 0x0002, 0x0001, /*A399*/
  0x4159, 0x0020, 0x0002, 0x0001, /*A39A*/
  0x415A, 0x0020, 0x0002, 0x0001, /*A39B*/
  0x415B, 0x0020, 0x0002, 0x0001, /*A39C*/
  0x415C, 0x0020, 0x0002, 0x0001, /*A39D*/
  0x415D, 0x0020, 0x0002, 0x0001, /*A39E*/
  0x415E, 0x0020, 0x0002, 0x0001, /*A39F*/
  0x415F, 0x0020, 0x0002, 0x0001, /*A3A0*/
  0x4160, 0x0020, 0x0002, 0x0001, /*A3A1*/
  0x4161, 0x0020, 0x0002, 0x0001, /*A3A2*/
  0x4162, 0x0020, 0x0002, 0x0001, /*A3A3*/
  0x4163, 0x0020, 0x0002, 0x0001, /*A3A4*/
  0x4164, 0x0020, 0x0002, 0x0001, /*A3A5*/
  0x4165, 0x0020, 0x0002, 0x0001, /*A3A6*/
  0x4166, 0x0020, 0x0002, 0x0001, /*A3A7*/
  0x4167, 0x0020, 0x0002, 0x0001, /*A3A8*/
  0x4168, 0x0020, 0x0002, 0x0001, /*A3A9*/
  0x4169, 0x0020, 0x0002, 0x0001, /*A3AA*/
  0x416A, 0x0020, 0x0002, 0x0001, /*A3AB*/
  0x416B, 0x0020, 0x0002, 0x0001, /*A3AC*/
  0x416C, 0x0020, 0x0002, 0x0001, /*A3AD*/
  0x416D, 0x0020, 0x0002, 0x0001, /*A3AE*/
  0x416E, 0x0020, 0x0002, 0x0001, /*A3AF*/
  0x416F, 0x0020, 0x0002, 0x0001, /*A3B0*/
  0x4170, 0x0020, 0x0002, 0x0001, /*A3B1*/
  0x4171, 0x0020, 0x0002, 0x0001, /*A3B2*/
  0x4172, 0x0020, 0x0002, 0x0001, /*A3B3*/
  0x4173, 0x0020, 0x0002, 0x0001, /*A3B4*/
  0x4174, 0x0020, 0x0002, 0x0001, /*A3B5*/
  0x4175, 0x0020, 0x0002, 0x0001, /*A3B6*/
  0x4176, 0x0020, 0x0002, 0x0001, /*A3B7*/
  0x4177, 0x0020, 0x0002, 0x0001, /*A3B8*/
  0x4178, 0x0020, 0x0002, 0x0001, /*A3B9*/
  0x4179, 0x0020, 0x0002, 0x0001, /*A3BA*/
  0x417A, 0x0020, 0x0002, 0x0001, /*A3BB*/
  0x417B, 0x0020, 0x0002, 0x0001, /*A3BC*/
  0x417C, 0x0020, 0x0002, 0x0001, /*A3BD*/
  0x417D, 0x0020, 0x0002, 0x0001, /*A3BE*/
  0x417E, 0x0020, 0x0002, 0x0001, /*A3BF*/
  0x417F, 0x0020, 0x0002, 0x0001, /*A3C0*/
  0x4180, 0x0020, 0x0002, 0x0001, /*A3C1*/
  0x4181, 0x0020, 0x0002, 0x0001, /*A3C2*/
  0x4182, 0x0020, 0x0002, 0x0001, /*A3C3*/
  0x4183, 0x0020, 0x0002, 0x0001, /*A3C4*/
  0x4184, 0x0020, 0x0002, 0x0001, /*A3C5*/
  0x4185, 0x0020, 0x0002, 0x0001, /*A3C6*/
  0x4186, 0x0020, 0x0002, 0x0001, /*A3C7*/
  0x4187, 0x0020, 0x0002, 0x0001, /*A3C8*/
  0x4188, 0x0020, 0x0002, 0x0001, /*A3C9*/
  0x4189, 0x0020, 0x0002, 0x0001, /*A3CA*/
  0x418A, 0x0020, 0x0002, 0x0001, /*A3CB*/
  0x418B, 0x0020, 0x0002, 0x0001, /*A3CC*/
  0x418C, 0x0020, 0x0002, 0x0001, /*A3CD*/
  0x418D, 0x0020, 0x0002, 0x0001, /*A3CE*/
  0x418E, 0x0020, 0x0002, 0x0001, /*A3CF*/
  0x418F, 0x0020, 0x0002, 0x0001, /*A3D0*/
  0x4190, 0x0020, 0x0002, 0x0001, /*A3D1*/
  0x4191, 0x0020, 0x0002, 0x0001, /*A3D2*/
  0x4192, 0x0020, 0x0002, 0x0001, /*A3D3*/
  0x4193, 0x0020, 0x0002, 0x0001, /*A3D4*/
  0x4194, 0x0020, 0x0002, 0x0001, /*A3D5*/
  0x4195, 0x0020, 0x0002, 0x0001, /*A3D6*/
  0x4196, 0x0020, 0x0002, 0x0001, /*A3D7*/
  0x4197, 0x0020, 0x0002, 0x0001, /*A3D8*/
  0x4198, 0x0020, 0x0002, 0x0001, /*A3D9*/
  0x4199, 0x0020, 0x0002, 0x0001, /*A3DA*/
  0x419A, 0x0020, 0x0002, 0x0001, /*A3DB*/
  0x419B, 0x0020, 0x0002, 0x0001, /*A3DC*/
  0x419C, 0x0020, 0x0002, 0x0001, /*A3DD*/
  0x419D, 0x0020, 0x0002, 0x0001, /*A3DE*/
  0x419E, 0x0020, 0x0002, 0x0001, /*A3DF*/
  0x419F, 0x0020, 0x0002, 0x0001, /*A3E0*/
  0x41A0, 0x0020, 0x0002, 0x0001, /*A3E1*/
  0x41A1, 0x0020, 0x0002, 0x0001, /*A3E2*/
  0x41A2, 0x0020, 0x0002, 0x0001, /*A3E3*/
  0x41A3, 0x0020, 0x0002, 0x0001, /*A3E4*/
  0x41A4, 0x0020, 0x0002, 0x0001, /*A3E5*/
  0x41A5, 0x0020, 0x0002, 0x0001, /*A3E6*/
  0x41A6, 0x0020, 0x0002, 0x0001, /*A3E7*/
  0x41A7, 0x0020, 0x0002, 0x0001, /*A3E8*/
  0x41A8, 0x0020, 0x0002, 0x0001, /*A3E9*/
  0x41A9, 0x0020, 0x0002, 0x0001, /*A3EA*/
  0x41AA, 0x0020, 0x0002, 0x0001, /*A3EB*/
  0x41AB, 0x0020, 0x0002, 0x0001, /*A3EC*/
  0x41AC, 0x0020, 0x0002, 0x0001, /*A3ED*/
  0x41AD, 0x0020, 0x0002, 0x0001, /*A3EE*/
  0x41AE, 0x0020, 0x0002, 0x0001, /*A3EF*/
  0x41AF, 0x0020, 0x0002, 0x0001, /*A3F0*/
  0x41B0, 0x0020, 0x0002, 0x0001, /*A3F1*/
  0x41B1, 0x0020, 0x0002, 0x0001, /*A3F2*/
  0x41B2, 0x0020, 0x0002, 0x0001, /*A3F3*/
  0x41B3, 0x0020, 0x0002, 0x0001, /*A3F4*/
  0x41B4, 0x0020, 0x0002, 0x0001, /*A3F5*/
  0x41B5, 0x0020, 0x0002, 0x0001, /*A3F6*/
  0x41B6, 0x0020, 0x0002, 0x0001, /*A3F7*/
  0x41B7, 0x0020, 0x0002, 0x0001, /*A3F8*/
  0x41B8, 0x0020, 0x0002, 0x0001, /*A3F9*/
  0x41B9, 0x0020, 0x0002, 0x0001, /*A3FA*/
  0x41BA, 0x0020, 0x0002, 0x0001, /*A3FB*/
  0x41BB, 0x0020, 0x0002, 0x0001, /*A3FC*/
  0x41BC, 0x0020, 0x0002, 0x0001, /*A3FD*/
  0x41BD, 0x0020, 0x0002, 0x0001, /*A3FE*/
  0x41BE, 0x0020, 0x0002, 0x0001  /*A3FF*/
};

uint16 uca900_p0A4[]= {
  0x41BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A400*/
  0x41C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A401*/
  0x41C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A402*/
  0x41C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A403*/
  0x41C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A404*/
  0x41C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A405*/
  0x41C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A406*/
  0x41C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A407*/
  0x41C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A408*/
  0x41C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A409*/
  0x41C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A40A*/
  0x41CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A40B*/
  0x41CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A40C*/
  0x41CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A40D*/
  0x41CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A40E*/
  0x41CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A40F*/
  0x41CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A410*/
  0x41D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A411*/
  0x41D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A412*/
  0x41D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A413*/
  0x41D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A414*/
  0x41D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A415*/
  0x41D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A416*/
  0x41D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A417*/
  0x41D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A418*/
  0x41D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A419*/
  0x41D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A41A*/
  0x41DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A41B*/
  0x41DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A41C*/
  0x41DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A41D*/
  0x41DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A41E*/
  0x41DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A41F*/
  0x41DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A420*/
  0x41E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A421*/
  0x41E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A422*/
  0x41E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A423*/
  0x41E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A424*/
  0x41E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A425*/
  0x41E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A426*/
  0x41E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A427*/
  0x41E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A428*/
  0x41E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A429*/
  0x41E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A42A*/
  0x41EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A42B*/
  0x41EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A42C*/
  0x41EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A42D*/
  0x41ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A42E*/
  0x41EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A42F*/
  0x41EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A430*/
  0x41F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A431*/
  0x41F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A432*/
  0x41F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A433*/
  0x41F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A434*/
  0x41F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A435*/
  0x41F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A436*/
  0x41F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A437*/
  0x41F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A438*/
  0x41F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A439*/
  0x41F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A43A*/
  0x41FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A43B*/
  0x41FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A43C*/
  0x41FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A43D*/
  0x41FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A43E*/
  0x41FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A43F*/
  0x41FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A440*/
  0x4200, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A441*/
  0x4201, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A442*/
  0x4202, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A443*/
  0x4203, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A444*/
  0x4204, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A445*/
  0x4205, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A446*/
  0x4206, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A447*/
  0x4207, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A448*/
  0x4208, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A449*/
  0x4209, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A44A*/
  0x420A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A44B*/
  0x420B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A44C*/
  0x420C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A44D*/
  0x420D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A44E*/
  0x420E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A44F*/
  0x420F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A450*/
  0x4210, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A451*/
  0x4211, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A452*/
  0x4212, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A453*/
  0x4213, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A454*/
  0x4214, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A455*/
  0x4215, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A456*/
  0x4216, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A457*/
  0x4217, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A458*/
  0x4218, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A459*/
  0x4219, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A45A*/
  0x421A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A45B*/
  0x421B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A45C*/
  0x421C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A45D*/
  0x421D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A45E*/
  0x421E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A45F*/
  0x421F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A460*/
  0x4220, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A461*/
  0x4221, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A462*/
  0x4222, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A463*/
  0x4223, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A464*/
  0x4224, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A465*/
  0x4225, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A466*/
  0x4226, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A467*/
  0x4227, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A468*/
  0x4228, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A469*/
  0x4229, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A46A*/
  0x422A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A46B*/
  0x422B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A46C*/
  0x422C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A46D*/
  0x422D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A46E*/
  0x422E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A46F*/
  0x422F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A470*/
  0x4230, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A471*/
  0x4231, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A472*/
  0x4232, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A473*/
  0x4233, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A474*/
  0x4234, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A475*/
  0x4235, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A476*/
  0x4236, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A477*/
  0x4237, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A478*/
  0x4238, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A479*/
  0x4239, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A47A*/
  0x423A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A47B*/
  0x423B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A47C*/
  0x423C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A47D*/
  0x423D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A47E*/
  0x423E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A47F*/
  0x423F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A480*/
  0x4240, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A481*/
  0x4241, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A482*/
  0x4242, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A483*/
  0x4243, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A484*/
  0x4244, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A485*/
  0x4245, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A486*/
  0x4246, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A487*/
  0x4247, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A488*/
  0x4248, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A489*/
  0x4249, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A48A*/
  0x424A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A48B*/
  0x424B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A48C*/
  0xFBC1, 0x0020, 0x0002, 0xA48D, 0x0000, 0x0000, 0x0002, /*A48D*/
  0xFBC1, 0x0020, 0x0002, 0xA48E, 0x0000, 0x0000, 0x0002, /*A48E*/
  0xFBC1, 0x0020, 0x0002, 0xA48F, 0x0000, 0x0000, 0x0002, /*A48F*/
  0x0F41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A490*/
  0x0F42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A491*/
  0x0F43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A492*/
  0x0F44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A493*/
  0x0F45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A494*/
  0x0F46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A495*/
  0x0F47, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A496*/
  0x0F48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A497*/
  0x0F49, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A498*/
  0x0F4A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A499*/
  0x0F4B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A49A*/
  0x0F4C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A49B*/
  0x0F4D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A49C*/
  0x0F4E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A49D*/
  0x0F4F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A49E*/
  0x0F50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A49F*/
  0x0F51, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4A0*/
  0x0F52, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4A1*/
  0x0F53, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4A2*/
  0x0F54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4A3*/
  0x0F55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4A4*/
  0x0F56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4A5*/
  0x0F57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4A6*/
  0x0F58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4A7*/
  0x0F59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4A8*/
  0x0F5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4A9*/
  0x0F5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4AA*/
  0x0F5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4AB*/
  0x0F5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4AC*/
  0x0F5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4AD*/
  0x0F5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4AE*/
  0x0F60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4AF*/
  0x0F61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4B0*/
  0x0F62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4B1*/
  0x0F63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4B2*/
  0x0F64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4B3*/
  0x0F65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4B4*/
  0x0F66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4B5*/
  0x0F67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4B6*/
  0x0F68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4B7*/
  0x0F69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4B8*/
  0x0F6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4B9*/
  0x0F6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4BA*/
  0x0F6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4BB*/
  0x0F6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4BC*/
  0x0F6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4BD*/
  0x0F6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4BE*/
  0x0F70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4BF*/
  0x0F71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4C0*/
  0x0F72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4C1*/
  0x0F73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4C2*/
  0x0F74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4C3*/
  0x0F75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4C4*/
  0x0F76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4C5*/
  0x0F77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4C6*/
  0xFBC1, 0x0020, 0x0002, 0xA4C7, 0x0000, 0x0000, 0x0002, /*A4C7*/
  0xFBC1, 0x0020, 0x0002, 0xA4C8, 0x0000, 0x0000, 0x0002, /*A4C8*/
  0xFBC1, 0x0020, 0x0002, 0xA4C9, 0x0000, 0x0000, 0x0002, /*A4C9*/
  0xFBC1, 0x0020, 0x0002, 0xA4CA, 0x0000, 0x0000, 0x0002, /*A4CA*/
  0xFBC1, 0x0020, 0x0002, 0xA4CB, 0x0000, 0x0000, 0x0002, /*A4CB*/
  0xFBC1, 0x0020, 0x0002, 0xA4CC, 0x0000, 0x0000, 0x0002, /*A4CC*/
  0xFBC1, 0x0020, 0x0002, 0xA4CD, 0x0000, 0x0000, 0x0002, /*A4CD*/
  0xFBC1, 0x0020, 0x0002, 0xA4CE, 0x0000, 0x0000, 0x0002, /*A4CE*/
  0xFBC1, 0x0020, 0x0002, 0xA4CF, 0x0000, 0x0000, 0x0002, /*A4CF*/
  0x4252, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4D0*/
  0x4253, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4D1*/
  0x4254, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4D2*/
  0x4255, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4D3*/
  0x4256, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4D4*/
  0x4257, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4D5*/
  0x4258, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4D6*/
  0x4259, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4D7*/
  0x425A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4D8*/
  0x425B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4D9*/
  0x425C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4DA*/
  0x425D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4DB*/
  0x425E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4DC*/
  0x425F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4DD*/
  0x4260, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4DE*/
  0x4261, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4DF*/
  0x4262, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4E0*/
  0x4263, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4E1*/
  0x4264, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4E2*/
  0x4265, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4E3*/
  0x4266, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4E4*/
  0x4267, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4E5*/
  0x4268, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4E6*/
  0x4269, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4E7*/
  0x426A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4E8*/
  0x426B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4E9*/
  0x426E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4EA*/
  0x426C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4EB*/
  0x426F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4EC*/
  0x426D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4ED*/
  0x4270, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4EE*/
  0x4271, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4EF*/
  0x4272, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4F0*/
  0x4273, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4F1*/
  0x4274, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4F2*/
  0x4275, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4F3*/
  0x4276, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4F4*/
  0x4277, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4F5*/
  0x4278, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4F6*/
  0x4279, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4F7*/
  0x424C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4F8*/
  0x424D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4F9*/
  0x424E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4FA*/
  0x424F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4FB*/
  0x4251, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4FC*/
  0x4250, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4FD*/
  0x022E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A4FE*/
  0x0285, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*A4FF*/
};

uint16 uca900_p0A5[]= {
  0x375A, 0x0020, 0x0002, 0x0001, /*A500*/
  0x375B, 0x0020, 0x0002, 0x0001, /*A501*/
  0x375C, 0x0020, 0x0002, 0x0001, /*A502*/
  0x375D, 0x0020, 0x0002, 0x0001, /*A503*/
  0x375E, 0x0020, 0x0002, 0x0001, /*A504*/
  0x375F, 0x0020, 0x0002, 0x0001, /*A505*/
  0x3760, 0x0020, 0x0002, 0x0001, /*A506*/
  0x3761, 0x0020, 0x0002, 0x0001, /*A507*/
  0x3762, 0x0020, 0x0002, 0x0001, /*A508*/
  0x3763, 0x0020, 0x0002, 0x0001, /*A509*/
  0x3764, 0x0020, 0x0002, 0x0001, /*A50A*/
  0x3765, 0x0020, 0x0002, 0x0001, /*A50B*/
  0x3766, 0x0020, 0x0002, 0x0001, /*A50C*/
  0x3767, 0x0020, 0x0002, 0x0001, /*A50D*/
  0x3768, 0x0020, 0x0002, 0x0001, /*A50E*/
  0x3769, 0x0020, 0x0002, 0x0001, /*A50F*/
  0x376A, 0x0020, 0x0002, 0x0001, /*A510*/
  0x376B, 0x0020, 0x0002, 0x0001, /*A511*/
  0x376C, 0x0020, 0x0002, 0x0001, /*A512*/
  0x376D, 0x0020, 0x0002, 0x0001, /*A513*/
  0x376E, 0x0020, 0x0002, 0x0001, /*A514*/
  0x376F, 0x0020, 0x0002, 0x0001, /*A515*/
  0x3770, 0x0020, 0x0002, 0x0001, /*A516*/
  0x3771, 0x0020, 0x0002, 0x0001, /*A517*/
  0x3772, 0x0020, 0x0002, 0x0001, /*A518*/
  0x3773, 0x0020, 0x0002, 0x0001, /*A519*/
  0x3774, 0x0020, 0x0002, 0x0001, /*A51A*/
  0x3775, 0x0020, 0x0002, 0x0001, /*A51B*/
  0x3776, 0x0020, 0x0002, 0x0001, /*A51C*/
  0x3777, 0x0020, 0x0002, 0x0001, /*A51D*/
  0x3778, 0x0020, 0x0002, 0x0001, /*A51E*/
  0x3779, 0x0020, 0x0002, 0x0001, /*A51F*/
  0x377A, 0x0020, 0x0002, 0x0001, /*A520*/
  0x377B, 0x0020, 0x0002, 0x0001, /*A521*/
  0x377C, 0x0020, 0x0002, 0x0001, /*A522*/
  0x377D, 0x0020, 0x0002, 0x0001, /*A523*/
  0x377E, 0x0020, 0x0002, 0x0001, /*A524*/
  0x377F, 0x0020, 0x0002, 0x0001, /*A525*/
  0x3780, 0x0020, 0x0002, 0x0001, /*A526*/
  0x3781, 0x0020, 0x0002, 0x0001, /*A527*/
  0x3782, 0x0020, 0x0002, 0x0001, /*A528*/
  0x3783, 0x0020, 0x0002, 0x0001, /*A529*/
  0x3784, 0x0020, 0x0002, 0x0001, /*A52A*/
  0x3785, 0x0020, 0x0002, 0x0001, /*A52B*/
  0x3786, 0x0020, 0x0002, 0x0001, /*A52C*/
  0x3787, 0x0020, 0x0002, 0x0001, /*A52D*/
  0x3788, 0x0020, 0x0002, 0x0001, /*A52E*/
  0x3789, 0x0020, 0x0002, 0x0001, /*A52F*/
  0x378A, 0x0020, 0x0002, 0x0001, /*A530*/
  0x378B, 0x0020, 0x0002, 0x0001, /*A531*/
  0x378C, 0x0020, 0x0002, 0x0001, /*A532*/
  0x378D, 0x0020, 0x0002, 0x0001, /*A533*/
  0x378E, 0x0020, 0x0002, 0x0001, /*A534*/
  0x378F, 0x0020, 0x0002, 0x0001, /*A535*/
  0x3790, 0x0020, 0x0002, 0x0001, /*A536*/
  0x3791, 0x0020, 0x0002, 0x0001, /*A537*/
  0x3792, 0x0020, 0x0002, 0x0001, /*A538*/
  0x3793, 0x0020, 0x0002, 0x0001, /*A539*/
  0x3794, 0x0020, 0x0002, 0x0001, /*A53A*/
  0x3795, 0x0020, 0x0002, 0x0001, /*A53B*/
  0x3796, 0x0020, 0x0002, 0x0001, /*A53C*/
  0x3797, 0x0020, 0x0002, 0x0001, /*A53D*/
  0x3798, 0x0020, 0x0002, 0x0001, /*A53E*/
  0x3799, 0x0020, 0x0002, 0x0001, /*A53F*/
  0x379A, 0x0020, 0x0002, 0x0001, /*A540*/
  0x379B, 0x0020, 0x0002, 0x0001, /*A541*/
  0x379C, 0x0020, 0x0002, 0x0001, /*A542*/
  0x379D, 0x0020, 0x0002, 0x0001, /*A543*/
  0x379E, 0x0020, 0x0002, 0x0001, /*A544*/
  0x379F, 0x0020, 0x0002, 0x0001, /*A545*/
  0x37A0, 0x0020, 0x0002, 0x0001, /*A546*/
  0x37A1, 0x0020, 0x0002, 0x0001, /*A547*/
  0x37A2, 0x0020, 0x0002, 0x0001, /*A548*/
  0x37A3, 0x0020, 0x0002, 0x0001, /*A549*/
  0x37A4, 0x0020, 0x0002, 0x0001, /*A54A*/
  0x37A5, 0x0020, 0x0002, 0x0001, /*A54B*/
  0x37A6, 0x0020, 0x0002, 0x0001, /*A54C*/
  0x37A7, 0x0020, 0x0002, 0x0001, /*A54D*/
  0x37A8, 0x0020, 0x0002, 0x0001, /*A54E*/
  0x37A9, 0x0020, 0x0002, 0x0001, /*A54F*/
  0x37AA, 0x0020, 0x0002, 0x0001, /*A550*/
  0x37AB, 0x0020, 0x0002, 0x0001, /*A551*/
  0x37AC, 0x0020, 0x0002, 0x0001, /*A552*/
  0x37AD, 0x0020, 0x0002, 0x0001, /*A553*/
  0x37AE, 0x0020, 0x0002, 0x0001, /*A554*/
  0x37AF, 0x0020, 0x0002, 0x0001, /*A555*/
  0x37B0, 0x0020, 0x0002, 0x0001, /*A556*/
  0x37B1, 0x0020, 0x0002, 0x0001, /*A557*/
  0x37B2, 0x0020, 0x0002, 0x0001, /*A558*/
  0x37B3, 0x0020, 0x0002, 0x0001, /*A559*/
  0x37B4, 0x0020, 0x0002, 0x0001, /*A55A*/
  0x37B5, 0x0020, 0x0002, 0x0001, /*A55B*/
  0x37B6, 0x0020, 0x0002, 0x0001, /*A55C*/
  0x37B7, 0x0020, 0x0002, 0x0001, /*A55D*/
  0x37B8, 0x0020, 0x0002, 0x0001, /*A55E*/
  0x37B9, 0x0020, 0x0002, 0x0001, /*A55F*/
  0x37BA, 0x0020, 0x0002, 0x0001, /*A560*/
  0x37BB, 0x0020, 0x0002, 0x0001, /*A561*/
  0x37BC, 0x0020, 0x0002, 0x0001, /*A562*/
  0x37BD, 0x0020, 0x0002, 0x0001, /*A563*/
  0x37BE, 0x0020, 0x0002, 0x0001, /*A564*/
  0x37BF, 0x0020, 0x0002, 0x0001, /*A565*/
  0x37C0, 0x0020, 0x0002, 0x0001, /*A566*/
  0x37C1, 0x0020, 0x0002, 0x0001, /*A567*/
  0x37C2, 0x0020, 0x0002, 0x0001, /*A568*/
  0x37C3, 0x0020, 0x0002, 0x0001, /*A569*/
  0x37C4, 0x0020, 0x0002, 0x0001, /*A56A*/
  0x37C5, 0x0020, 0x0002, 0x0001, /*A56B*/
  0x37C6, 0x0020, 0x0002, 0x0001, /*A56C*/
  0x37C7, 0x0020, 0x0002, 0x0001, /*A56D*/
  0x37C8, 0x0020, 0x0002, 0x0001, /*A56E*/
  0x37C9, 0x0020, 0x0002, 0x0001, /*A56F*/
  0x37CA, 0x0020, 0x0002, 0x0001, /*A570*/
  0x37CB, 0x0020, 0x0002, 0x0001, /*A571*/
  0x37CC, 0x0020, 0x0002, 0x0001, /*A572*/
  0x37CD, 0x0020, 0x0002, 0x0001, /*A573*/
  0x37CE, 0x0020, 0x0002, 0x0001, /*A574*/
  0x37CF, 0x0020, 0x0002, 0x0001, /*A575*/
  0x37D0, 0x0020, 0x0002, 0x0001, /*A576*/
  0x37D1, 0x0020, 0x0002, 0x0001, /*A577*/
  0x37D2, 0x0020, 0x0002, 0x0001, /*A578*/
  0x37D3, 0x0020, 0x0002, 0x0001, /*A579*/
  0x37D4, 0x0020, 0x0002, 0x0001, /*A57A*/
  0x37D5, 0x0020, 0x0002, 0x0001, /*A57B*/
  0x37D6, 0x0020, 0x0002, 0x0001, /*A57C*/
  0x37D7, 0x0020, 0x0002, 0x0001, /*A57D*/
  0x37D8, 0x0020, 0x0002, 0x0001, /*A57E*/
  0x37D9, 0x0020, 0x0002, 0x0001, /*A57F*/
  0x37DA, 0x0020, 0x0002, 0x0001, /*A580*/
  0x37DB, 0x0020, 0x0002, 0x0001, /*A581*/
  0x37DC, 0x0020, 0x0002, 0x0001, /*A582*/
  0x37DD, 0x0020, 0x0002, 0x0001, /*A583*/
  0x37DE, 0x0020, 0x0002, 0x0001, /*A584*/
  0x37DF, 0x0020, 0x0002, 0x0001, /*A585*/
  0x37E0, 0x0020, 0x0002, 0x0001, /*A586*/
  0x37E1, 0x0020, 0x0002, 0x0001, /*A587*/
  0x37E2, 0x0020, 0x0002, 0x0001, /*A588*/
  0x37E3, 0x0020, 0x0002, 0x0001, /*A589*/
  0x37E4, 0x0020, 0x0002, 0x0001, /*A58A*/
  0x37E5, 0x0020, 0x0002, 0x0001, /*A58B*/
  0x37E6, 0x0020, 0x0002, 0x0001, /*A58C*/
  0x37E7, 0x0020, 0x0002, 0x0001, /*A58D*/
  0x37E8, 0x0020, 0x0002, 0x0001, /*A58E*/
  0x37E9, 0x0020, 0x0002, 0x0001, /*A58F*/
  0x37EA, 0x0020, 0x0002, 0x0001, /*A590*/
  0x37EB, 0x0020, 0x0002, 0x0001, /*A591*/
  0x37EC, 0x0020, 0x0002, 0x0001, /*A592*/
  0x37ED, 0x0020, 0x0002, 0x0001, /*A593*/
  0x37EE, 0x0020, 0x0002, 0x0001, /*A594*/
  0x37EF, 0x0020, 0x0002, 0x0001, /*A595*/
  0x37F0, 0x0020, 0x0002, 0x0001, /*A596*/
  0x37F1, 0x0020, 0x0002, 0x0001, /*A597*/
  0x37F2, 0x0020, 0x0002, 0x0001, /*A598*/
  0x37F3, 0x0020, 0x0002, 0x0001, /*A599*/
  0x37F4, 0x0020, 0x0002, 0x0001, /*A59A*/
  0x37F5, 0x0020, 0x0002, 0x0001, /*A59B*/
  0x37F6, 0x0020, 0x0002, 0x0001, /*A59C*/
  0x37F7, 0x0020, 0x0002, 0x0001, /*A59D*/
  0x37F8, 0x0020, 0x0002, 0x0001, /*A59E*/
  0x37F9, 0x0020, 0x0002, 0x0001, /*A59F*/
  0x37FA, 0x0020, 0x0002, 0x0001, /*A5A0*/
  0x37FB, 0x0020, 0x0002, 0x0001, /*A5A1*/
  0x37FC, 0x0020, 0x0002, 0x0001, /*A5A2*/
  0x37FD, 0x0020, 0x0002, 0x0001, /*A5A3*/
  0x37FE, 0x0020, 0x0002, 0x0001, /*A5A4*/
  0x37FF, 0x0020, 0x0002, 0x0001, /*A5A5*/
  0x3800, 0x0020, 0x0002, 0x0001, /*A5A6*/
  0x3801, 0x0020, 0x0002, 0x0001, /*A5A7*/
  0x3802, 0x0020, 0x0002, 0x0001, /*A5A8*/
  0x3803, 0x0020, 0x0002, 0x0001, /*A5A9*/
  0x3804, 0x0020, 0x0002, 0x0001, /*A5AA*/
  0x3805, 0x0020, 0x0002, 0x0001, /*A5AB*/
  0x3806, 0x0020, 0x0002, 0x0001, /*A5AC*/
  0x3807, 0x0020, 0x0002, 0x0001, /*A5AD*/
  0x3808, 0x0020, 0x0002, 0x0001, /*A5AE*/
  0x3809, 0x0020, 0x0002, 0x0001, /*A5AF*/
  0x380A, 0x0020, 0x0002, 0x0001, /*A5B0*/
  0x380B, 0x0020, 0x0002, 0x0001, /*A5B1*/
  0x380C, 0x0020, 0x0002, 0x0001, /*A5B2*/
  0x380D, 0x0020, 0x0002, 0x0001, /*A5B3*/
  0x380E, 0x0020, 0x0002, 0x0001, /*A5B4*/
  0x380F, 0x0020, 0x0002, 0x0001, /*A5B5*/
  0x3810, 0x0020, 0x0002, 0x0001, /*A5B6*/
  0x3811, 0x0020, 0x0002, 0x0001, /*A5B7*/
  0x3812, 0x0020, 0x0002, 0x0001, /*A5B8*/
  0x3813, 0x0020, 0x0002, 0x0001, /*A5B9*/
  0x3814, 0x0020, 0x0002, 0x0001, /*A5BA*/
  0x3815, 0x0020, 0x0002, 0x0001, /*A5BB*/
  0x3816, 0x0020, 0x0002, 0x0001, /*A5BC*/
  0x3817, 0x0020, 0x0002, 0x0001, /*A5BD*/
  0x3818, 0x0020, 0x0002, 0x0001, /*A5BE*/
  0x3819, 0x0020, 0x0002, 0x0001, /*A5BF*/
  0x381A, 0x0020, 0x0002, 0x0001, /*A5C0*/
  0x381B, 0x0020, 0x0002, 0x0001, /*A5C1*/
  0x381C, 0x0020, 0x0002, 0x0001, /*A5C2*/
  0x381D, 0x0020, 0x0002, 0x0001, /*A5C3*/
  0x381E, 0x0020, 0x0002, 0x0001, /*A5C4*/
  0x381F, 0x0020, 0x0002, 0x0001, /*A5C5*/
  0x3820, 0x0020, 0x0002, 0x0001, /*A5C6*/
  0x3821, 0x0020, 0x0002, 0x0001, /*A5C7*/
  0x3822, 0x0020, 0x0002, 0x0001, /*A5C8*/
  0x3823, 0x0020, 0x0002, 0x0001, /*A5C9*/
  0x3824, 0x0020, 0x0002, 0x0001, /*A5CA*/
  0x3825, 0x0020, 0x0002, 0x0001, /*A5CB*/
  0x3826, 0x0020, 0x0002, 0x0001, /*A5CC*/
  0x3827, 0x0020, 0x0002, 0x0001, /*A5CD*/
  0x3828, 0x0020, 0x0002, 0x0001, /*A5CE*/
  0x3829, 0x0020, 0x0002, 0x0001, /*A5CF*/
  0x382A, 0x0020, 0x0002, 0x0001, /*A5D0*/
  0x382B, 0x0020, 0x0002, 0x0001, /*A5D1*/
  0x382C, 0x0020, 0x0002, 0x0001, /*A5D2*/
  0x382D, 0x0020, 0x0002, 0x0001, /*A5D3*/
  0x382E, 0x0020, 0x0002, 0x0001, /*A5D4*/
  0x382F, 0x0020, 0x0002, 0x0001, /*A5D5*/
  0x3830, 0x0020, 0x0002, 0x0001, /*A5D6*/
  0x3831, 0x0020, 0x0002, 0x0001, /*A5D7*/
  0x3832, 0x0020, 0x0002, 0x0001, /*A5D8*/
  0x3833, 0x0020, 0x0002, 0x0001, /*A5D9*/
  0x3834, 0x0020, 0x0002, 0x0001, /*A5DA*/
  0x3835, 0x0020, 0x0002, 0x0001, /*A5DB*/
  0x3836, 0x0020, 0x0002, 0x0001, /*A5DC*/
  0x3837, 0x0020, 0x0002, 0x0001, /*A5DD*/
  0x3838, 0x0020, 0x0002, 0x0001, /*A5DE*/
  0x3839, 0x0020, 0x0002, 0x0001, /*A5DF*/
  0x383A, 0x0020, 0x0002, 0x0001, /*A5E0*/
  0x383B, 0x0020, 0x0002, 0x0001, /*A5E1*/
  0x383C, 0x0020, 0x0002, 0x0001, /*A5E2*/
  0x383D, 0x0020, 0x0002, 0x0001, /*A5E3*/
  0x383E, 0x0020, 0x0002, 0x0001, /*A5E4*/
  0x383F, 0x0020, 0x0002, 0x0001, /*A5E5*/
  0x3840, 0x0020, 0x0002, 0x0001, /*A5E6*/
  0x3841, 0x0020, 0x0002, 0x0001, /*A5E7*/
  0x3842, 0x0020, 0x0002, 0x0001, /*A5E8*/
  0x3843, 0x0020, 0x0002, 0x0001, /*A5E9*/
  0x3844, 0x0020, 0x0002, 0x0001, /*A5EA*/
  0x3845, 0x0020, 0x0002, 0x0001, /*A5EB*/
  0x3846, 0x0020, 0x0002, 0x0001, /*A5EC*/
  0x3847, 0x0020, 0x0002, 0x0001, /*A5ED*/
  0x3848, 0x0020, 0x0002, 0x0001, /*A5EE*/
  0x3849, 0x0020, 0x0002, 0x0001, /*A5EF*/
  0x384A, 0x0020, 0x0002, 0x0001, /*A5F0*/
  0x384B, 0x0020, 0x0002, 0x0001, /*A5F1*/
  0x384C, 0x0020, 0x0002, 0x0001, /*A5F2*/
  0x384D, 0x0020, 0x0002, 0x0001, /*A5F3*/
  0x384E, 0x0020, 0x0002, 0x0001, /*A5F4*/
  0x384F, 0x0020, 0x0002, 0x0001, /*A5F5*/
  0x3850, 0x0020, 0x0002, 0x0001, /*A5F6*/
  0x3851, 0x0020, 0x0002, 0x0001, /*A5F7*/
  0x3852, 0x0020, 0x0002, 0x0001, /*A5F8*/
  0x3853, 0x0020, 0x0002, 0x0001, /*A5F9*/
  0x3854, 0x0020, 0x0002, 0x0001, /*A5FA*/
  0x3855, 0x0020, 0x0002, 0x0001, /*A5FB*/
  0x3856, 0x0020, 0x0002, 0x0001, /*A5FC*/
  0x3857, 0x0020, 0x0002, 0x0001, /*A5FD*/
  0x3858, 0x0020, 0x0002, 0x0001, /*A5FE*/
  0x3859, 0x0020, 0x0002, 0x0001  /*A5FF*/
};

uint16 uca900_p0A6[]= {
  0x385A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A600*/
  0x385B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A601*/
  0x385C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A602*/
  0x385D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A603*/
  0x385E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A604*/
  0x385F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A605*/
  0x3860, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A606*/
  0x3861, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A607*/
  0x3862, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A608*/
  0x3863, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A609*/
  0x3864, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A60A*/
  0x3865, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A60B*/
  0x3866, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A60C*/
  0x022F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A60D*/
  0x0286, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A60E*/
  0x0270, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A60F*/
  0x37B2, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A610*/
  0x37C4, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A611*/
  0x37E1, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A612*/
  0x3766, 0x0020, 0x0004, 0x3865, 0x0020, 0x0004, 0x0002, /*A613*/
  0x3778, 0x0020, 0x0004, 0x3865, 0x0020, 0x0004, 0x0002, /*A614*/
  0x378D, 0x0020, 0x0004, 0x3865, 0x0020, 0x0004, 0x0002, /*A615*/
  0x37A1, 0x0020, 0x0004, 0x3866, 0x0020, 0x0004, 0x0002, /*A616*/
  0x37AC, 0x0020, 0x0004, 0x3865, 0x0020, 0x0004, 0x0002, /*A617*/
  0x37B2, 0x0020, 0x0004, 0x3866, 0x0020, 0x0004, 0x0002, /*A618*/
  0x37B4, 0x0020, 0x0004, 0x3866, 0x0020, 0x0004, 0x0002, /*A619*/
  0x37BA, 0x0020, 0x0004, 0x3865, 0x0020, 0x0004, 0x0002, /*A61A*/
  0x37DF, 0x0020, 0x0004, 0x3865, 0x0020, 0x0004, 0x0002, /*A61B*/
  0x380E, 0x0020, 0x0004, 0x3865, 0x0020, 0x0004, 0x0002, /*A61C*/
  0x3825, 0x0020, 0x0004, 0x3865, 0x0020, 0x0004, 0x0002, /*A61D*/
  0x382B, 0x0020, 0x0004, 0x3866, 0x0020, 0x0004, 0x0002, /*A61E*/
  0x3832, 0x0020, 0x0004, 0x3865, 0x0020, 0x0004, 0x0002, /*A61F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A620*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A621*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A622*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A623*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A624*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A625*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A626*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A627*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A628*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A629*/
  0x37C8, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A62A*/
  0x382B, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A62B*/
  0xFBC1, 0x0020, 0x0002, 0xA62C, 0x0000, 0x0000, 0x0002, /*A62C*/
  0xFBC1, 0x0020, 0x0002, 0xA62D, 0x0000, 0x0000, 0x0002, /*A62D*/
  0xFBC1, 0x0020, 0x0002, 0xA62E, 0x0000, 0x0000, 0x0002, /*A62E*/
  0xFBC1, 0x0020, 0x0002, 0xA62F, 0x0000, 0x0000, 0x0002, /*A62F*/
  0xFBC1, 0x0020, 0x0002, 0xA630, 0x0000, 0x0000, 0x0002, /*A630*/
  0xFBC1, 0x0020, 0x0002, 0xA631, 0x0000, 0x0000, 0x0002, /*A631*/
  0xFBC1, 0x0020, 0x0002, 0xA632, 0x0000, 0x0000, 0x0002, /*A632*/
  0xFBC1, 0x0020, 0x0002, 0xA633, 0x0000, 0x0000, 0x0002, /*A633*/
  0xFBC1, 0x0020, 0x0002, 0xA634, 0x0000, 0x0000, 0x0002, /*A634*/
  0xFBC1, 0x0020, 0x0002, 0xA635, 0x0000, 0x0000, 0x0002, /*A635*/
  0xFBC1, 0x0020, 0x0002, 0xA636, 0x0000, 0x0000, 0x0002, /*A636*/
  0xFBC1, 0x0020, 0x0002, 0xA637, 0x0000, 0x0000, 0x0002, /*A637*/
  0xFBC1, 0x0020, 0x0002, 0xA638, 0x0000, 0x0000, 0x0002, /*A638*/
  0xFBC1, 0x0020, 0x0002, 0xA639, 0x0000, 0x0000, 0x0002, /*A639*/
  0xFBC1, 0x0020, 0x0002, 0xA63A, 0x0000, 0x0000, 0x0002, /*A63A*/
  0xFBC1, 0x0020, 0x0002, 0xA63B, 0x0000, 0x0000, 0x0002, /*A63B*/
  0xFBC1, 0x0020, 0x0002, 0xA63C, 0x0000, 0x0000, 0x0002, /*A63C*/
  0xFBC1, 0x0020, 0x0002, 0xA63D, 0x0000, 0x0000, 0x0002, /*A63D*/
  0xFBC1, 0x0020, 0x0002, 0xA63E, 0x0000, 0x0000, 0x0002, /*A63E*/
  0xFBC1, 0x0020, 0x0002, 0xA63F, 0x0000, 0x0000, 0x0002, /*A63F*/
  0x2070, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A640*/
  0x2070, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A641*/
  0x2073, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A642*/
  0x2073, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A643*/
  0x2078, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A644*/
  0x2078, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A645*/
  0x208C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A646*/
  0x208C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A647*/
  0x2095, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A648*/
  0x2095, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A649*/
  0x2129, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A64A*/
  0x2129, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A64B*/
  0x2150, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A64C*/
  0x2150, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A64D*/
  0x218C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A64E*/
  0x218C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A64F*/
  0x2193, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A650*/
  0x2193, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A651*/
  0x21A4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A652*/
  0x21A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A653*/
  0x21AD, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A654*/
  0x21AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A655*/
  0x21AE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A656*/
  0x21AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A657*/
  0x21BC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A658*/
  0x21BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A659*/
  0x21C1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A65A*/
  0x21C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A65B*/
  0x21C6, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A65C*/
  0x21C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A65D*/
  0x21DB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A65E*/
  0x21DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A65F*/
  0x215D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A660*/
  0x215D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A661*/
  0x2054, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A662*/
  0x2054, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A663*/
  0x20C0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A664*/
  0x20C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A665*/
  0x20CB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A666*/
  0x20CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A667*/
  0x20E7, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0001, /*A668*/
  0x20E7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A669*/
  0x20E7, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0001, /*A66A*/
  0x20E7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A66B*/
  0x20E7, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0001, /*A66C*/
  0x20E7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A66D*/
  0x20E7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A66E*/
  0x0000, 0x0051, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A66F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A670*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A671*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A672*/
  0x0393, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A673*/
  0x205E, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A674*/
  0x2080, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A675*/
  0x2088, 0x0020, 0x0004, 0x0000, 0x002B, 0x0004, 0x0002, /*A676*/
  0x211D, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A677*/
  0x218F, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A678*/
  0x2194, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A679*/
  0x2198, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A67A*/
  0x2148, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A67B*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A67C*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A67D*/
  0x03CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A67E*/
  0x218E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A67F*/
  0x204F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A680*/
  0x204F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A681*/
  0x207F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A682*/
  0x207F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A683*/
  0x2067, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A684*/
  0x2067, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A685*/
  0x2176, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A686*/
  0x2176, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A687*/
  0x207D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A688*/
  0x207D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A689*/
  0x2118, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A68A*/
  0x2118, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A68B*/
  0x2112, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A68C*/
  0x2112, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A68D*/
  0x215E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A68E*/
  0x215E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A68F*/
  0x2163, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A690*/
  0x2163, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A691*/
  0x2169, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A692*/
  0x2169, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A693*/
  0x2147, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A694*/
  0x2147, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A695*/
  0x2187, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*A696*/
  0x2187, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A697*/
  0x20E7, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0001, /*A698*/
  0x20E7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A699*/
  0x20E7, 0x0020, 0x000A, 0x0000, 0x0000, 0x0000, 0x0001, /*A69A*/
  0x20E7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A69B*/
  0x218F, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0001, /*A69C*/
  0x2198, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0001, /*A69D*/
  0x212E, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A69E*/
  0x21B4, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A69F*/
  0x3867, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6A0*/
  0x3868, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6A1*/
  0x3869, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6A2*/
  0x386A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6A3*/
  0x386B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6A4*/
  0x386C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6A5*/
  0x386D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6A6*/
  0x386E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6A7*/
  0x386F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6A8*/
  0x3870, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6A9*/
  0x3871, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6AA*/
  0x3872, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6AB*/
  0x3873, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6AC*/
  0x3874, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6AD*/
  0x3875, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6AE*/
  0x3876, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6AF*/
  0x3877, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6B0*/
  0x3878, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6B1*/
  0x3879, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6B2*/
  0x387A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6B3*/
  0x387B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6B4*/
  0x387C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6B5*/
  0x387D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6B6*/
  0x387E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6B7*/
  0x387F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6B8*/
  0x3880, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6B9*/
  0x3881, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6BA*/
  0x3882, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6BB*/
  0x3883, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6BC*/
  0x3884, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6BD*/
  0x3885, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6BE*/
  0x3886, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6BF*/
  0x3887, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6C0*/
  0x3888, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6C1*/
  0x3889, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6C2*/
  0x388A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6C3*/
  0x388B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6C4*/
  0x388C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6C5*/
  0x388D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6C6*/
  0x388E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6C7*/
  0x388F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6C8*/
  0x3890, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6C9*/
  0x3891, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6CA*/
  0x3892, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6CB*/
  0x3893, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6CC*/
  0x3894, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6CD*/
  0x3895, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6CE*/
  0x3896, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6CF*/
  0x3897, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6D0*/
  0x3898, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6D1*/
  0x3899, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6D2*/
  0x389A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6D3*/
  0x389B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6D4*/
  0x389C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6D5*/
  0x389D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6D6*/
  0x389E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6D7*/
  0x389F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6D8*/
  0x38A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6D9*/
  0x38A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6DA*/
  0x38A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6DB*/
  0x38A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6DC*/
  0x38A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6DD*/
  0x38A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6DE*/
  0x38A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6DF*/
  0x38A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6E0*/
  0x38A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6E1*/
  0x38A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6E2*/
  0x38AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6E3*/
  0x38AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6E4*/
  0x38AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6E5*/
  0x38AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6E6*/
  0x38AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6E7*/
  0x38AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6E8*/
  0x38B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6E9*/
  0x38B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6EA*/
  0x38B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6EB*/
  0x38B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6EC*/
  0x38B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6ED*/
  0x38B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6EE*/
  0x38B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6EF*/
  0x0000, 0x00B6, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6F0*/
  0x0000, 0x00B7, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6F1*/
  0x02D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6F2*/
  0x0287, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6F3*/
  0x025F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6F4*/
  0x0230, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6F5*/
  0x0238, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6F6*/
  0x0271, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A6F7*/
  0xFBC1, 0x0020, 0x0002, 0xA6F8, 0x0000, 0x0000, 0x0002, /*A6F8*/
  0xFBC1, 0x0020, 0x0002, 0xA6F9, 0x0000, 0x0000, 0x0002, /*A6F9*/
  0xFBC1, 0x0020, 0x0002, 0xA6FA, 0x0000, 0x0000, 0x0002, /*A6FA*/
  0xFBC1, 0x0020, 0x0002, 0xA6FB, 0x0000, 0x0000, 0x0002, /*A6FB*/
  0xFBC1, 0x0020, 0x0002, 0xA6FC, 0x0000, 0x0000, 0x0002, /*A6FC*/
  0xFBC1, 0x0020, 0x0002, 0xA6FD, 0x0000, 0x0000, 0x0002, /*A6FD*/
  0xFBC1, 0x0020, 0x0002, 0xA6FE, 0x0000, 0x0000, 0x0002, /*A6FE*/
  0xFBC1, 0x0020, 0x0002, 0xA6FF, 0x0000, 0x0000, 0x0002  /*A6FF*/
};

uint16 uca900_p0A7[]= {
  0x04D0, 0x0020, 0x0002, 0x0000, 0x0000, /*A700*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04D1, 0x0020, 0x0002, 0x0000, 0x0000, /*A701*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04D2, 0x0020, 0x0002, 0x0000, 0x0000, /*A702*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04D3, 0x0020, 0x0002, 0x0000, 0x0000, /*A703*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04D4, 0x0020, 0x0002, 0x0000, 0x0000, /*A704*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04D5, 0x0020, 0x0002, 0x0000, 0x0000, /*A705*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04D6, 0x0020, 0x0002, 0x0000, 0x0000, /*A706*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04D7, 0x0020, 0x0002, 0x0000, 0x0000, /*A707*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04D8, 0x0020, 0x0002, 0x0000, 0x0000, /*A708*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04D9, 0x0020, 0x0002, 0x0000, 0x0000, /*A709*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04DA, 0x0020, 0x0002, 0x0000, 0x0000, /*A70A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04DB, 0x0020, 0x0002, 0x0000, 0x0000, /*A70B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04DC, 0x0020, 0x0002, 0x0000, 0x0000, /*A70C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04DD, 0x0020, 0x0002, 0x0000, 0x0000, /*A70D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04DE, 0x0020, 0x0002, 0x0000, 0x0000, /*A70E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04DF, 0x0020, 0x0002, 0x0000, 0x0000, /*A70F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04E0, 0x0020, 0x0002, 0x0000, 0x0000, /*A710*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04E1, 0x0020, 0x0002, 0x0000, 0x0000, /*A711*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04E2, 0x0020, 0x0002, 0x0000, 0x0000, /*A712*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04E3, 0x0020, 0x0002, 0x0000, 0x0000, /*A713*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04E4, 0x0020, 0x0002, 0x0000, 0x0000, /*A714*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04E5, 0x0020, 0x0002, 0x0000, 0x0000, /*A715*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04E6, 0x0020, 0x0002, 0x0000, 0x0000, /*A716*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04E7, 0x0020, 0x0002, 0x0000, 0x0000, /*A717*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04E8, 0x0020, 0x0002, 0x0000, 0x0000, /*A718*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04E9, 0x0020, 0x0002, 0x0000, 0x0000, /*A719*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04EA, 0x0020, 0x0002, 0x0000, 0x0000, /*A71A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04EB, 0x0020, 0x0002, 0x0000, 0x0000, /*A71B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04EC, 0x0020, 0x0002, 0x0000, 0x0000, /*A71C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04ED, 0x0020, 0x0002, 0x0000, 0x0000, /*A71D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04EE, 0x0020, 0x0002, 0x0000, 0x0000, /*A71E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04EF, 0x0020, 0x0002, 0x0000, 0x0000, /*A71F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04F0, 0x0020, 0x0002, 0x0000, 0x0000, /*A720*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04F1, 0x0020, 0x0002, 0x0000, 0x0000, /*A721*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F81, 0x0020, 0x0008, 0x0000, 0x0000, /*A722*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F81, 0x0020, 0x0002, 0x0000, 0x0000, /*A723*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F8C, 0x0020, 0x0008, 0x0000, 0x0000, /*A724*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F8C, 0x0020, 0x0002, 0x0000, 0x0000, /*A725*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D2B, 0x0020, 0x0008, 0x0000, 0x0000, /*A726*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D2B, 0x0020, 0x0002, 0x0000, 0x0000, /*A727*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x000A, 0x1F21, 0x0020, /*A728*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0004, 0x1F21, 0x0020, /*A729*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F66, 0x0020, 0x0008, 0x0000, 0x0000, /*A72A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F66, 0x0020, 0x0002, 0x0000, 0x0000, /*A72B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F67, 0x0020, 0x0008, 0x0000, 0x0000, /*A72C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F67, 0x0020, 0x0002, 0x0000, 0x0000, /*A72D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F68, 0x0020, 0x0008, 0x0000, 0x0000, /*A72E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F68, 0x0020, 0x0002, 0x0000, 0x0000, /*A72F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE9, 0x0020, 0x0002, 0x0000, 0x0000, /*A730*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E75, 0x0020, 0x0002, 0x0000, 0x0000, /*A731*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x000A, 0x1C47, 0x0020, /*A732*/
  0x000A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0004, 0x1C47, 0x0020, /*A733*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x000A, 0x1DDD, 0x0020, /*A734*/
  0x000A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0004, 0x1DDD, 0x0020, /*A735*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x000A, 0x1EB5, 0x0020, /*A736*/
  0x000A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0004, 0x1EB5, 0x0020, /*A737*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x000A, 0x1EE3, 0x0020, /*A738*/
  0x000A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0004, 0x1EE3, 0x0020, /*A739*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x000A, 0x0000, 0x0110, /*A73A*/
  0x0004, 0x1EE3, 0x0020, 0x000A, 0x0003,
  0x1C47, 0x0020, 0x0004, 0x0000, 0x0110, /*A73B*/
  0x0004, 0x1EE3, 0x0020, 0x0004, 0x0003,
  0x1C47, 0x0020, 0x000A, 0x1F0B, 0x0020, /*A73C*/
  0x000A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0004, 0x1F0B, 0x0020, /*A73D*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8E, 0x0020, 0x0008, 0x0000, 0x0000, /*A73E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8E, 0x0020, 0x0002, 0x0000, 0x0000, /*A73F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D70, 0x0020, 0x0008, 0x0000, 0x0000, /*A740*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D70, 0x0020, 0x0002, 0x0000, 0x0000, /*A741*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D71, 0x0020, 0x0008, 0x0000, 0x0000, /*A742*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D71, 0x0020, 0x0002, 0x0000, 0x0000, /*A743*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D72, 0x0020, 0x0008, 0x0000, 0x0000, /*A744*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D72, 0x0020, 0x0002, 0x0000, 0x0000, /*A745*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D7F, 0x0020, 0x0008, 0x0000, 0x0000, /*A746*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D7F, 0x0020, 0x0002, 0x0000, 0x0000, /*A747*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D81, 0x0020, 0x0008, 0x0000, 0x0000, /*A748*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D81, 0x0020, 0x0002, 0x0000, 0x0000, /*A749*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E01, 0x0020, 0x0008, 0x0000, 0x0000, /*A74A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E01, 0x0020, 0x0002, 0x0000, 0x0000, /*A74B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DF9, 0x0020, 0x0008, 0x0000, 0x0000, /*A74C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DF9, 0x0020, 0x0002, 0x0000, 0x0000, /*A74D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x000A, 0x1DDD, 0x0020, /*A74E*/
  0x000A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0004, 0x1DDD, 0x0020, /*A74F*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E12, 0x0020, 0x0008, 0x0000, 0x0000, /*A750*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E12, 0x0020, 0x0002, 0x0000, 0x0000, /*A751*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E19, 0x0020, 0x0008, 0x0000, 0x0000, /*A752*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E19, 0x0020, 0x0002, 0x0000, 0x0000, /*A753*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E1A, 0x0020, 0x0008, 0x0000, 0x0000, /*A754*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E1A, 0x0020, 0x0002, 0x0000, 0x0000, /*A755*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E25, 0x0020, 0x0008, 0x0000, 0x0000, /*A756*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E25, 0x0020, 0x0002, 0x0000, 0x0000, /*A757*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E26, 0x0020, 0x0008, 0x0000, 0x0000, /*A758*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E26, 0x0020, 0x0002, 0x0000, 0x0000, /*A759*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E3D, 0x0020, 0x0008, 0x0000, 0x0000, /*A75A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E3D, 0x0020, 0x0002, 0x0000, 0x0000, /*A75B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E70, 0x0020, 0x0008, 0x0000, 0x0000, /*A75C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E70, 0x0020, 0x0002, 0x0000, 0x0000, /*A75D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE8, 0x0020, 0x0008, 0x0000, 0x0000, /*A75E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE8, 0x0020, 0x0002, 0x0000, 0x0000, /*A75F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x000A, 0x1F0B, 0x0020, /*A760*/
  0x000A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EE3, 0x0020, 0x0004, 0x1F0B, 0x0020, /*A761*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1F3D, 0x0020, 0x0008, 0x0000, 0x0000, /*A762*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F3D, 0x0020, 0x0002, 0x0000, 0x0000, /*A763*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F54, 0x0020, 0x0008, 0x0000, 0x0000, /*A764*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F54, 0x0020, 0x0002, 0x0000, 0x0000, /*A765*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F55, 0x0020, 0x0008, 0x0000, 0x0000, /*A766*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F55, 0x0020, 0x0002, 0x0000, 0x0000, /*A767*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F5A, 0x0020, 0x0008, 0x0000, 0x0000, /*A768*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F5A, 0x0020, 0x0002, 0x0000, 0x0000, /*A769*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F5E, 0x0020, 0x0008, 0x0000, 0x0000, /*A76A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F5E, 0x0020, 0x0002, 0x0000, 0x0000, /*A76B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F5F, 0x0020, 0x0008, 0x0000, 0x0000, /*A76C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F5F, 0x0020, 0x0002, 0x0000, 0x0000, /*A76D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F60, 0x0020, 0x0008, 0x0000, 0x0000, /*A76E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F60, 0x0020, 0x0002, 0x0000, 0x0000, /*A76F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F60, 0x0020, 0x0014, 0x0000, 0x0000, /*A770*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CA8, 0x0020, 0x0002, 0x0000, 0x0000, /*A771*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D9C, 0x0020, 0x0002, 0x0000, 0x0000, /*A772*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB8, 0x0020, 0x0002, 0x0000, 0x0000, /*A773*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DD7, 0x0020, 0x0002, 0x0000, 0x0000, /*A774*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E6E, 0x0020, 0x0002, 0x0000, 0x0000, /*A775*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E6F, 0x0020, 0x0002, 0x0000, 0x0000, /*A776*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB0, 0x0020, 0x0002, 0x0000, 0x0000, /*A777*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F61, 0x0020, 0x0002, 0x0000, 0x0000, /*A778*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x000A, 0x0000, 0x0111, /*A779*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x0004, 0x0000, 0x0111, /*A77A*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CE5, 0x0020, 0x000A, 0x0000, 0x0111, /*A77B*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CE5, 0x0020, 0x0004, 0x0000, 0x0111, /*A77C*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x000A, 0x0000, 0x0111, /*A77D*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D0F, 0x0020, 0x0008, 0x0000, 0x0000, /*A77E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D0F, 0x0020, 0x0002, 0x0000, 0x0000, /*A77F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DA1, 0x0020, 0x0008, 0x0000, 0x0000, /*A780*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DA1, 0x0020, 0x0002, 0x0000, 0x0000, /*A781*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x000A, 0x0000, 0x0111, /*A782*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0004, 0x0000, 0x0111, /*A783*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x000A, 0x0000, 0x0111, /*A784*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0004, 0x0000, 0x0111, /*A785*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x000A, 0x0000, 0x0111, /*A786*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E95, 0x0020, 0x0004, 0x0000, 0x0111, /*A787*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x04F2, 0x0020, 0x0002, 0x0000, 0x0000, /*A788*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04F3, 0x0020, 0x0002, 0x0000, 0x0000, /*A789*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x04F4, 0x0020, 0x0002, 0x0000, 0x0000, /*A78A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F82, 0x0020, 0x0008, 0x0000, 0x0000, /*A78B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F82, 0x0020, 0x0002, 0x0000, 0x0000, /*A78C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EC8, 0x0020, 0x0008, 0x0000, 0x0000, /*A78D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D97, 0x0020, 0x0002, 0x0000, 0x0000, /*A78E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F83, 0x0020, 0x0002, 0x0000, 0x0000, /*A78F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DCC, 0x0020, 0x0008, 0x0000, 0x0000, /*A790*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DCC, 0x0020, 0x0002, 0x0000, 0x0000, /*A791*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C83, 0x0020, 0x0008, 0x0000, 0x0000, /*A792*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C83, 0x0020, 0x0002, 0x0000, 0x0000, /*A793*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C84, 0x0020, 0x0002, 0x0000, 0x0000, /*A794*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D24, 0x0020, 0x0002, 0x0000, 0x0000, /*A795*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C6F, 0x0020, 0x0008, 0x0000, 0x0000, /*A796*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C6F, 0x0020, 0x0002, 0x0000, 0x0000, /*A797*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CEB, 0x0020, 0x0008, 0x0000, 0x0000, /*A798*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CEB, 0x0020, 0x0002, 0x0000, 0x0000, /*A799*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C47, 0x0020, 0x000A, 0x0000, 0x002B, /*A79A*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x0004, 0x0000, 0x002B, /*A79B*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x000A, 0x0000, 0x002B, /*A79C*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0004, 0x0000, 0x002B, /*A79D*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x000A, 0x0000, 0x002B, /*A79E*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EB5, 0x0020, 0x0004, 0x0000, 0x002B, /*A79F*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x000A, 0x0000, 0x0035, /*A7A0*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CF4, 0x0020, 0x0004, 0x0000, 0x0035, /*A7A1*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x000A, 0x0000, 0x0035, /*A7A2*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D65, 0x0020, 0x0004, 0x0000, 0x0035, /*A7A3*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x000A, 0x0000, 0x0035, /*A7A4*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x0004, 0x0000, 0x0035, /*A7A5*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x000A, 0x0000, 0x0035, /*A7A6*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E33, 0x0020, 0x0004, 0x0000, 0x0035, /*A7A7*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x000A, 0x0000, 0x0035, /*A7A8*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x0004, 0x0000, 0x0035, /*A7A9*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D25, 0x0020, 0x0008, 0x0000, 0x0000, /*A7AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CCF, 0x0020, 0x0008, 0x0000, 0x0000, /*A7AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF8, 0x0020, 0x0008, 0x0000, 0x0000, /*A7AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D8D, 0x0020, 0x0008, 0x0000, 0x0000, /*A7AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D3A, 0x0020, 0x0008, 0x0000, 0x0000, /*A7AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC1, 0x0020, 0x0002, 0xA7AF, 0x0000, /*A7AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D73, 0x0020, 0x0008, 0x0000, 0x0000, /*A7B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB1, 0x0020, 0x0008, 0x0000, 0x0000, /*A7B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D59, 0x0020, 0x0008, 0x0000, 0x0000, /*A7B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F08, 0x0020, 0x0008, 0x0000, 0x0000, /*A7B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C79, 0x0020, 0x0008, 0x0000, 0x0000, /*A7B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C79, 0x0020, 0x0002, 0x0000, 0x0000, /*A7B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E06, 0x0020, 0x0008, 0x0000, 0x0000, /*A7B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E06, 0x0020, 0x0002, 0x0000, 0x0000, /*A7B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC1, 0x0020, 0x0002, 0xA7B8, 0x0000, /*A7B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7B9, 0x0000, /*A7B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7BA, 0x0000, /*A7BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7BB, 0x0000, /*A7BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7BC, 0x0000, /*A7BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7BD, 0x0000, /*A7BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7BE, 0x0000, /*A7BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7BF, 0x0000, /*A7BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7C0, 0x0000, /*A7C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7C1, 0x0000, /*A7C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7C2, 0x0000, /*A7C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7C3, 0x0000, /*A7C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7C4, 0x0000, /*A7C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7C5, 0x0000, /*A7C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7C6, 0x0000, /*A7C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7C7, 0x0000, /*A7C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7C8, 0x0000, /*A7C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7C9, 0x0000, /*A7C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7CA, 0x0000, /*A7CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7CB, 0x0000, /*A7CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7CC, 0x0000, /*A7CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7CD, 0x0000, /*A7CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7CE, 0x0000, /*A7CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7CF, 0x0000, /*A7CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7D0, 0x0000, /*A7D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7D1, 0x0000, /*A7D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7D2, 0x0000, /*A7D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7D3, 0x0000, /*A7D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7D4, 0x0000, /*A7D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7D5, 0x0000, /*A7D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7D6, 0x0000, /*A7D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7D7, 0x0000, /*A7D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7D8, 0x0000, /*A7D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7D9, 0x0000, /*A7D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7DA, 0x0000, /*A7DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7DB, 0x0000, /*A7DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7DC, 0x0000, /*A7DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7DD, 0x0000, /*A7DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7DE, 0x0000, /*A7DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7DF, 0x0000, /*A7DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7E0, 0x0000, /*A7E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7E1, 0x0000, /*A7E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7E2, 0x0000, /*A7E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7E3, 0x0000, /*A7E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7E4, 0x0000, /*A7E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7E5, 0x0000, /*A7E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7E6, 0x0000, /*A7E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7E7, 0x0000, /*A7E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7E8, 0x0000, /*A7E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7E9, 0x0000, /*A7E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7EA, 0x0000, /*A7EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7EB, 0x0000, /*A7EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7EC, 0x0000, /*A7EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7ED, 0x0000, /*A7ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7EE, 0x0000, /*A7EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7EF, 0x0000, /*A7EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7F0, 0x0000, /*A7F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7F1, 0x0000, /*A7F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7F2, 0x0000, /*A7F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7F3, 0x0000, /*A7F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7F4, 0x0000, /*A7F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7F5, 0x0000, /*A7F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xA7F6, 0x0000, /*A7F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D3F, 0x0020, 0x0002, 0x0000, 0x0000, /*A7F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x0014, 0x0000, 0x0039, /*A7F8*/
  0x0014, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x0014, 0x0000, 0x0110, /*A7F9*/
  0x0014, 0x1CAA, 0x0020, 0x0014, 0x0003,
  0x1ED8, 0x0020, 0x0002, 0x0000, 0x0000, /*A7FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF3, 0x0020, 0x0002, 0x0000, 0x0000, /*A7FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E1B, 0x0020, 0x0002, 0x0000, 0x0000, /*A7FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB6, 0x0020, 0x0002, 0x0000, 0x0000, /*A7FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D3E, 0x0020, 0x0002, 0x0000, 0x0000, /*A7FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB7, 0x0020, 0x0002, 0x0000, 0x0000, /*A7FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p0A8[]= {
  0x295C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A800*/
  0x295D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A801*/
  0x295E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A802*/
  0x295F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A803*/
  0x2960, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A804*/
  0x2961, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A805*/
  0x2962, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A806*/
  0x2963, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A807*/
  0x2964, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A808*/
  0x2965, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A809*/
  0x2966, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A80A*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A80B*/
  0x2967, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A80C*/
  0x2968, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A80D*/
  0x2969, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A80E*/
  0x296A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A80F*/
  0x296B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A810*/
  0x296C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A811*/
  0x296D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A812*/
  0x296E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A813*/
  0x296F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A814*/
  0x2970, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A815*/
  0x2971, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A816*/
  0x2972, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A817*/
  0x2973, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A818*/
  0x2974, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A819*/
  0x2975, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A81A*/
  0x2976, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A81B*/
  0x2977, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A81C*/
  0x2978, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A81D*/
  0x2979, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A81E*/
  0x297A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A81F*/
  0x297B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A820*/
  0x297C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A821*/
  0x297D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A822*/
  0x297E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A823*/
  0x297F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A824*/
  0x2980, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A825*/
  0x2981, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A826*/
  0x2982, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A827*/
  0x051F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A828*/
  0x0520, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A829*/
  0x0521, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A82A*/
  0x0522, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A82B*/
  0xFBC1, 0x0020, 0x0002, 0xA82C, 0x0000, 0x0000, 0x0002, /*A82C*/
  0xFBC1, 0x0020, 0x0002, 0xA82D, 0x0000, 0x0000, 0x0002, /*A82D*/
  0xFBC1, 0x0020, 0x0002, 0xA82E, 0x0000, 0x0000, 0x0002, /*A82E*/
  0xFBC1, 0x0020, 0x0002, 0xA82F, 0x0000, 0x0000, 0x0002, /*A82F*/
  0x1AA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A830*/
  0x1AA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A831*/
  0x1AA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A832*/
  0x1AA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A833*/
  0x1AA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A834*/
  0x1AA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A835*/
  0x0523, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A836*/
  0x0524, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A837*/
  0x1C1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A838*/
  0x0525, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A839*/
  0xFBC1, 0x0020, 0x0002, 0xA83A, 0x0000, 0x0000, 0x0002, /*A83A*/
  0xFBC1, 0x0020, 0x0002, 0xA83B, 0x0000, 0x0000, 0x0002, /*A83B*/
  0xFBC1, 0x0020, 0x0002, 0xA83C, 0x0000, 0x0000, 0x0002, /*A83C*/
  0xFBC1, 0x0020, 0x0002, 0xA83D, 0x0000, 0x0000, 0x0002, /*A83D*/
  0xFBC1, 0x0020, 0x0002, 0xA83E, 0x0000, 0x0000, 0x0002, /*A83E*/
  0xFBC1, 0x0020, 0x0002, 0xA83F, 0x0000, 0x0000, 0x0002, /*A83F*/
  0x2F00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A840*/
  0x2F01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A841*/
  0x2F02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A842*/
  0x2F03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A843*/
  0x2F04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A844*/
  0x2F05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A845*/
  0x2F06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A846*/
  0x2F07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A847*/
  0x2F0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A848*/
  0x2F0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A849*/
  0x2F0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A84A*/
  0x2F0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A84B*/
  0x2F10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A84C*/
  0x2F11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A84D*/
  0x2F12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A84E*/
  0x2F13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A84F*/
  0x2F14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A850*/
  0x2F15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A851*/
  0x2F16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A852*/
  0x2F17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A853*/
  0x2F19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A854*/
  0x2F1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A855*/
  0x2F1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A856*/
  0x2F1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A857*/
  0x2F1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A858*/
  0x2F22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A859*/
  0x2F23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A85A*/
  0x2F25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A85B*/
  0x2F26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A85C*/
  0x2F29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A85D*/
  0x2F2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A85E*/
  0x2F2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A85F*/
  0x2F30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A860*/
  0x2F31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A861*/
  0x2F2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A862*/
  0x2F2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A863*/
  0x2F2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A864*/
  0x2F2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A865*/
  0x2F32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A866*/
  0x2F18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A867*/
  0x2F1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A868*/
  0x2F08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A869*/
  0x2F09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A86A*/
  0x2F0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A86B*/
  0x2F0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A86C*/
  0x2F1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A86D*/
  0x2F24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A86E*/
  0x2F27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A86F*/
  0x2F28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A870*/
  0x2F20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A871*/
  0x2F21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A872*/
  0x2F33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A873*/
  0x0428, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A874*/
  0x0429, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A875*/
  0x0294, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A876*/
  0x0295, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A877*/
  0xFBC1, 0x0020, 0x0002, 0xA878, 0x0000, 0x0000, 0x0002, /*A878*/
  0xFBC1, 0x0020, 0x0002, 0xA879, 0x0000, 0x0000, 0x0002, /*A879*/
  0xFBC1, 0x0020, 0x0002, 0xA87A, 0x0000, 0x0000, 0x0002, /*A87A*/
  0xFBC1, 0x0020, 0x0002, 0xA87B, 0x0000, 0x0000, 0x0002, /*A87B*/
  0xFBC1, 0x0020, 0x0002, 0xA87C, 0x0000, 0x0000, 0x0002, /*A87C*/
  0xFBC1, 0x0020, 0x0002, 0xA87D, 0x0000, 0x0000, 0x0002, /*A87D*/
  0xFBC1, 0x0020, 0x0002, 0xA87E, 0x0000, 0x0000, 0x0002, /*A87E*/
  0xFBC1, 0x0020, 0x0002, 0xA87F, 0x0000, 0x0000, 0x0002, /*A87F*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A880*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A881*/
  0x2983, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A882*/
  0x2984, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A883*/
  0x2985, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A884*/
  0x2986, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A885*/
  0x2987, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A886*/
  0x2988, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A887*/
  0x2989, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A888*/
  0x298A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A889*/
  0x298B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A88A*/
  0x298C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A88B*/
  0x298D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A88C*/
  0x298E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A88D*/
  0x298F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A88E*/
  0x2990, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A88F*/
  0x2991, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A890*/
  0x2992, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A891*/
  0x2993, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A892*/
  0x2994, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A893*/
  0x2995, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A894*/
  0x2996, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A895*/
  0x2997, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A896*/
  0x2998, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A897*/
  0x2999, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A898*/
  0x299A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A899*/
  0x299B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A89A*/
  0x299C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A89B*/
  0x299D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A89C*/
  0x299E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A89D*/
  0x299F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A89E*/
  0x29A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A89F*/
  0x29A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8A0*/
  0x29A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8A1*/
  0x29A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8A2*/
  0x29A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8A3*/
  0x29A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8A4*/
  0x29A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8A5*/
  0x29A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8A6*/
  0x29A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8A7*/
  0x29A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8A8*/
  0x29AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8A9*/
  0x29AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8AA*/
  0x29AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8AB*/
  0x29AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8AC*/
  0x29AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8AD*/
  0x29AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8AE*/
  0x29B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8AF*/
  0x29B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8B0*/
  0x29B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8B1*/
  0x29B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8B2*/
  0x29B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8B3*/
  0x29B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8B4*/
  0x29B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8B5*/
  0x29B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8B6*/
  0x29B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8B7*/
  0x29B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8B8*/
  0x29BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8B9*/
  0x29BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8BA*/
  0x29BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8BB*/
  0x29BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8BC*/
  0x29BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8BD*/
  0x29BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8BE*/
  0x29C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8BF*/
  0x29C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8C0*/
  0x29C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8C1*/
  0x29C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8C2*/
  0x29C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8C3*/
  0x29C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8C4*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8C5*/
  0xFBC1, 0x0020, 0x0002, 0xA8C6, 0x0000, 0x0000, 0x0002, /*A8C6*/
  0xFBC1, 0x0020, 0x0002, 0xA8C7, 0x0000, 0x0000, 0x0002, /*A8C7*/
  0xFBC1, 0x0020, 0x0002, 0xA8C8, 0x0000, 0x0000, 0x0002, /*A8C8*/
  0xFBC1, 0x0020, 0x0002, 0xA8C9, 0x0000, 0x0000, 0x0002, /*A8C9*/
  0xFBC1, 0x0020, 0x0002, 0xA8CA, 0x0000, 0x0000, 0x0002, /*A8CA*/
  0xFBC1, 0x0020, 0x0002, 0xA8CB, 0x0000, 0x0000, 0x0002, /*A8CB*/
  0xFBC1, 0x0020, 0x0002, 0xA8CC, 0x0000, 0x0000, 0x0002, /*A8CC*/
  0xFBC1, 0x0020, 0x0002, 0xA8CD, 0x0000, 0x0000, 0x0002, /*A8CD*/
  0x0290, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8CE*/
  0x0291, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8CF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8D0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8D1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8D2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8D3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8D4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8D5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8D6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8D7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8D8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8D9*/
  0xFBC1, 0x0020, 0x0002, 0xA8DA, 0x0000, 0x0000, 0x0002, /*A8DA*/
  0xFBC1, 0x0020, 0x0002, 0xA8DB, 0x0000, 0x0000, 0x0002, /*A8DB*/
  0xFBC1, 0x0020, 0x0002, 0xA8DC, 0x0000, 0x0000, 0x0002, /*A8DC*/
  0xFBC1, 0x0020, 0x0002, 0xA8DD, 0x0000, 0x0000, 0x0002, /*A8DD*/
  0xFBC1, 0x0020, 0x0002, 0xA8DE, 0x0000, 0x0000, 0x0002, /*A8DE*/
  0xFBC1, 0x0020, 0x0002, 0xA8DF, 0x0000, 0x0000, 0x0002, /*A8DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*A8F1*/
  0x26A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8F2*/
  0x26A4, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A8F3*/
  0x26A4, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A8F4*/
  0x26A4, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A8F5*/
  0x26A4, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A8F6*/
  0x26A4, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A8F7*/
  0x03E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8F8*/
  0x03E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8F9*/
  0x03EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8FA*/
  0x26A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8FB*/
  0x03EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8FC*/
  0x265C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A8FD*/
  0xFBC1, 0x0020, 0x0002, 0xA8FE, 0x0000, 0x0000, 0x0002, /*A8FE*/
  0xFBC1, 0x0020, 0x0002, 0xA8FF, 0x0000, 0x0000, 0x0002  /*A8FF*/
};

uint16 uca900_p0A9[]= {
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A900*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A901*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A902*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A903*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A904*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A905*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A906*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A907*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A908*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A909*/
  0x3014, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A90A*/
  0x3015, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A90B*/
  0x3016, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A90C*/
  0x3017, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A90D*/
  0x3018, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A90E*/
  0x3019, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A90F*/
  0x301A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A910*/
  0x301B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A911*/
  0x301C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A912*/
  0x301D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A913*/
  0x301E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A914*/
  0x301F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A915*/
  0x3020, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A916*/
  0x3021, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A917*/
  0x3022, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A918*/
  0x3023, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A919*/
  0x3024, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A91A*/
  0x3025, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A91B*/
  0x3026, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A91C*/
  0x3027, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A91D*/
  0x3028, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A91E*/
  0x3029, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A91F*/
  0x302A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A920*/
  0x302B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A921*/
  0x302C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A922*/
  0x302D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A923*/
  0x302E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A924*/
  0x302F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A925*/
  0x3030, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A926*/
  0x3031, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A927*/
  0x3032, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A928*/
  0x3033, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A929*/
  0x3034, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A92A*/
  0x0000, 0x00E2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A92B*/
  0x0000, 0x00E3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A92C*/
  0x0000, 0x00E4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A92D*/
  0x042E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A92E*/
  0x0298, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A92F*/
  0x2FF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A930*/
  0x2FF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A931*/
  0x2FF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A932*/
  0x2FF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A933*/
  0x2FF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A934*/
  0x2FF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A935*/
  0x2FF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A936*/
  0x2FF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A937*/
  0x2FF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A938*/
  0x2FF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A939*/
  0x2FFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A93A*/
  0x2FFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A93B*/
  0x2FFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A93C*/
  0x2FFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A93D*/
  0x2FFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A93E*/
  0x2FFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A93F*/
  0x3000, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A940*/
  0x3001, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A941*/
  0x3002, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A942*/
  0x3003, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A943*/
  0x3004, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A944*/
  0x3005, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A945*/
  0x3006, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A946*/
  0x3007, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A947*/
  0x3008, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A948*/
  0x3009, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A949*/
  0x300A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A94A*/
  0x300B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A94B*/
  0x300C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A94C*/
  0x300D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A94D*/
  0x300E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A94E*/
  0x300F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A94F*/
  0x3010, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A950*/
  0x3011, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A951*/
  0x3012, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A952*/
  0x3013, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A953*/
  0xFBC1, 0x0020, 0x0002, 0xA954, 0x0000, 0x0000, 0x0002, /*A954*/
  0xFBC1, 0x0020, 0x0002, 0xA955, 0x0000, 0x0000, 0x0002, /*A955*/
  0xFBC1, 0x0020, 0x0002, 0xA956, 0x0000, 0x0000, 0x0002, /*A956*/
  0xFBC1, 0x0020, 0x0002, 0xA957, 0x0000, 0x0000, 0x0002, /*A957*/
  0xFBC1, 0x0020, 0x0002, 0xA958, 0x0000, 0x0000, 0x0002, /*A958*/
  0xFBC1, 0x0020, 0x0002, 0xA959, 0x0000, 0x0000, 0x0002, /*A959*/
  0xFBC1, 0x0020, 0x0002, 0xA95A, 0x0000, 0x0000, 0x0002, /*A95A*/
  0xFBC1, 0x0020, 0x0002, 0xA95B, 0x0000, 0x0000, 0x0002, /*A95B*/
  0xFBC1, 0x0020, 0x0002, 0xA95C, 0x0000, 0x0000, 0x0002, /*A95C*/
  0xFBC1, 0x0020, 0x0002, 0xA95D, 0x0000, 0x0000, 0x0002, /*A95D*/
  0xFBC1, 0x0020, 0x0002, 0xA95E, 0x0000, 0x0000, 0x0002, /*A95E*/
  0x02D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A95F*/
  0x3C54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A960*/
  0x3C55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A961*/
  0x3C56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A962*/
  0x3C57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A963*/
  0x3C58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A964*/
  0x3C59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A965*/
  0x3C5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A966*/
  0x3C5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A967*/
  0x3C5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A968*/
  0x3C5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A969*/
  0x3C5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A96A*/
  0x3C5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A96B*/
  0x3C60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A96C*/
  0x3C61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A96D*/
  0x3C62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A96E*/
  0x3C63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A96F*/
  0x3C64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A970*/
  0x3C65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A971*/
  0x3C66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A972*/
  0x3C67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A973*/
  0x3C68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A974*/
  0x3C69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A975*/
  0x3C6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A976*/
  0x3C6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A977*/
  0x3C6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A978*/
  0x3C6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A979*/
  0x3C6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A97A*/
  0x3C6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A97B*/
  0x3C70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A97C*/
  0xFBC1, 0x0020, 0x0002, 0xA97D, 0x0000, 0x0000, 0x0002, /*A97D*/
  0xFBC1, 0x0020, 0x0002, 0xA97E, 0x0000, 0x0000, 0x0002, /*A97E*/
  0xFBC1, 0x0020, 0x0002, 0xA97F, 0x0000, 0x0000, 0x0002, /*A97F*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A980*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A981*/
  0x0000, 0x00C9, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A982*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A983*/
  0x329D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A984*/
  0x329E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A985*/
  0x329F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A986*/
  0x32A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A987*/
  0x32A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A988*/
  0x32A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A989*/
  0x32A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A98A*/
  0x32A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A98B*/
  0x32A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A98C*/
  0x32A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A98D*/
  0x32A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A98E*/
  0x32A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A98F*/
  0x32A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A990*/
  0x32AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A991*/
  0x32AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A992*/
  0x32AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A993*/
  0x32AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A994*/
  0x32AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A995*/
  0x32AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A996*/
  0x32B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A997*/
  0x32B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A998*/
  0x32B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A999*/
  0x32B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A99A*/
  0x32B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A99B*/
  0x32B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A99C*/
  0x32B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A99D*/
  0x32B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A99E*/
  0x32B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A99F*/
  0x32B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9A0*/
  0x32BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9A1*/
  0x32BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9A2*/
  0x32BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9A3*/
  0x32BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9A4*/
  0x32BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9A5*/
  0x32BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9A6*/
  0x32C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9A7*/
  0x32C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9A8*/
  0x32C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9A9*/
  0x32C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9AA*/
  0x32C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9AB*/
  0x32C5, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*A9AC*/
  0x32C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9AD*/
  0x32C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9AE*/
  0x32C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9AF*/
  0x32CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9B0*/
  0x32CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9B1*/
  0x32CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9B2*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9B3*/
  0x32CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9B4*/
  0x32D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9B5*/
  0x32CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9B6*/
  0x32D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9B7*/
  0x32D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9B8*/
  0x32D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9B9*/
  0x32D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9BA*/
  0x32D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9BB*/
  0x32CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9BC*/
  0x32D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9BD*/
  0x32C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9BE*/
  0x32C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9BF*/
  0x32D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9C0*/
  0x02CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9C1*/
  0x02CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9C2*/
  0x02CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9C3*/
  0x02D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9C4*/
  0x02D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9C5*/
  0x02D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9C6*/
  0x025B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9C7*/
  0x02A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9C8*/
  0x02A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9C9*/
  0x02D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9CA*/
  0x02D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9CB*/
  0x02D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9CC*/
  0x02D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9CD*/
  0xFBC1, 0x0020, 0x0002, 0xA9CE, 0x0000, 0x0000, 0x0002, /*A9CE*/
  0x1BFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9CF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9D0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9D1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9D2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9D3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9D4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9D5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9D6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9D7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9D8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9D9*/
  0xFBC1, 0x0020, 0x0002, 0xA9DA, 0x0000, 0x0000, 0x0002, /*A9DA*/
  0xFBC1, 0x0020, 0x0002, 0xA9DB, 0x0000, 0x0000, 0x0002, /*A9DB*/
  0xFBC1, 0x0020, 0x0002, 0xA9DC, 0x0000, 0x0000, 0x0002, /*A9DC*/
  0xFBC1, 0x0020, 0x0002, 0xA9DD, 0x0000, 0x0000, 0x0002, /*A9DD*/
  0x042F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9DE*/
  0x0430, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9DF*/
  0x303E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9E0*/
  0x3046, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9E1*/
  0x3050, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9E2*/
  0x3066, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9E3*/
  0x307F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9E4*/
  0x30C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9E5*/
  0x1C00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9E6*/
  0x3058, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9E7*/
  0x307A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9E8*/
  0x303C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9E9*/
  0x303F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9EA*/
  0x304B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9EB*/
  0x3052, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9EC*/
  0x3060, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9ED*/
  0x3063, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9EE*/
  0x3067, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9EF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9F0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9F1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9F2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9F3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9F4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9F5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9F6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9F7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9F8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9F9*/
  0x309B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9FA*/
  0x306C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9FB*/
  0x306F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9FC*/
  0x307D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9FD*/
  0x3080, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*A9FE*/
  0xFBC1, 0x0020, 0x0002, 0xA9FF, 0x0000, 0x0000, 0x0002  /*A9FF*/
};

uint16 uca900_p0AA[]= {
  0x3212, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA00*/
  0x3213, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA01*/
  0x3214, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA02*/
  0x3215, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA03*/
  0x3216, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA04*/
  0x3217, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA05*/
  0x3218, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA06*/
  0x3219, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA07*/
  0x321A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA08*/
  0x321B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA09*/
  0x321C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA0A*/
  0x321D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA0B*/
  0x321E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA0C*/
  0x321F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA0D*/
  0x3220, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA0E*/
  0x3221, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA0F*/
  0x3222, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA10*/
  0x3223, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA11*/
  0x3224, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA12*/
  0x3225, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA13*/
  0x3226, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA14*/
  0x3227, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA15*/
  0x3228, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA16*/
  0x3229, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA17*/
  0x322A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA18*/
  0x322B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA19*/
  0x322C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA1A*/
  0x322D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA1B*/
  0x322E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA1C*/
  0x322F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA1D*/
  0x3230, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA1E*/
  0x3231, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA1F*/
  0x3232, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA20*/
  0x3233, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA21*/
  0x3234, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA22*/
  0x3235, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA23*/
  0x3236, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA24*/
  0x3237, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA25*/
  0x3238, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA26*/
  0x3239, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA27*/
  0x323A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA28*/
  0x323F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA29*/
  0x3240, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA2A*/
  0x3241, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA2B*/
  0x3242, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA2C*/
  0x3243, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA2D*/
  0x3244, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA2E*/
  0x3245, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA2F*/
  0x3246, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA30*/
  0x3247, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA31*/
  0x3248, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA32*/
  0x323B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA33*/
  0x323C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA34*/
  0x323D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA35*/
  0x323E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA36*/
  0xFBC1, 0x0020, 0x0002, 0xAA37, 0x0000, 0x0000, 0x0002, /*AA37*/
  0xFBC1, 0x0020, 0x0002, 0xAA38, 0x0000, 0x0000, 0x0002, /*AA38*/
  0xFBC1, 0x0020, 0x0002, 0xAA39, 0x0000, 0x0000, 0x0002, /*AA39*/
  0xFBC1, 0x0020, 0x0002, 0xAA3A, 0x0000, 0x0000, 0x0002, /*AA3A*/
  0xFBC1, 0x0020, 0x0002, 0xAA3B, 0x0000, 0x0000, 0x0002, /*AA3B*/
  0xFBC1, 0x0020, 0x0002, 0xAA3C, 0x0000, 0x0000, 0x0002, /*AA3C*/
  0xFBC1, 0x0020, 0x0002, 0xAA3D, 0x0000, 0x0000, 0x0002, /*AA3D*/
  0xFBC1, 0x0020, 0x0002, 0xAA3E, 0x0000, 0x0000, 0x0002, /*AA3E*/
  0xFBC1, 0x0020, 0x0002, 0xAA3F, 0x0000, 0x0000, 0x0002, /*AA3F*/
  0x3249, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA40*/
  0x324A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA41*/
  0x324B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA42*/
  0x324C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA43*/
  0x324D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA44*/
  0x324E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA45*/
  0x324F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA46*/
  0x3250, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA47*/
  0x3251, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA48*/
  0x3252, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA49*/
  0x3253, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA4A*/
  0x3254, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA4B*/
  0x3255, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA4C*/
  0x3256, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA4D*/
  0xFBC1, 0x0020, 0x0002, 0xAA4E, 0x0000, 0x0000, 0x0002, /*AA4E*/
  0xFBC1, 0x0020, 0x0002, 0xAA4F, 0x0000, 0x0000, 0x0002, /*AA4F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA50*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA51*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA52*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA53*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA54*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA55*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA56*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA57*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA58*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA59*/
  0xFBC1, 0x0020, 0x0002, 0xAA5A, 0x0000, 0x0000, 0x0002, /*AA5A*/
  0xFBC1, 0x0020, 0x0002, 0xAA5B, 0x0000, 0x0000, 0x0002, /*AA5B*/
  0x0431, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA5C*/
  0x02A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA5D*/
  0x02A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA5E*/
  0x02A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA5F*/
  0x303B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA60*/
  0x3044, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA61*/
  0x3047, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA62*/
  0x304A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA63*/
  0x3051, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA64*/
  0x3057, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA65*/
  0x305B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA66*/
  0x305D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA67*/
  0x305F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA68*/
  0x3062, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA69*/
  0x306E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA6A*/
  0x3072, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA6B*/
  0x3093, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA6C*/
  0x3096, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA6D*/
  0x3098, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA6E*/
  0x3078, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA6F*/
  0x1C01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA70*/
  0x3099, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA71*/
  0x304D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA72*/
  0x3086, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA73*/
  0x30E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA74*/
  0x30E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA75*/
  0x30E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA76*/
  0x054D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA77*/
  0x054E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA78*/
  0x054F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA79*/
  0x3087, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA7A*/
  0x30DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA7B*/
  0x30DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA7C*/
  0x30E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA7D*/
  0x3048, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA7E*/
  0x3054, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA7F*/
  0x2DE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA80*/
  0x2DE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA81*/
  0x2DE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA82*/
  0x2DE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA83*/
  0x2DE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA84*/
  0x2DE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA85*/
  0x2DE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA86*/
  0x2DEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA87*/
  0x2DEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA88*/
  0x2DEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA89*/
  0x2DED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA8A*/
  0x2DEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA8B*/
  0x2DEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA8C*/
  0x2DF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA8D*/
  0x2DF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA8E*/
  0x2DF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA8F*/
  0x2DF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA90*/
  0x2DF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA91*/
  0x2DF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA92*/
  0x2DF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA93*/
  0x2DF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA94*/
  0x2DF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA95*/
  0x2DF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA96*/
  0x2DFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA97*/
  0x2DFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA98*/
  0x2DFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA99*/
  0x2DFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA9A*/
  0x2DFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA9B*/
  0x2DFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA9C*/
  0x2E00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA9D*/
  0x2E01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA9E*/
  0x2E02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AA9F*/
  0x2E03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAA0*/
  0x2E04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAA1*/
  0x2E05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAA2*/
  0x2E06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAA3*/
  0x2E07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAA4*/
  0x2E08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAA5*/
  0x2E09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAA6*/
  0x2E0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAA7*/
  0x2E0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAA8*/
  0x2E0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAA9*/
  0x2E0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAAA*/
  0x2E0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAAB*/
  0x2E0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAAC*/
  0x2E10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAAD*/
  0x2E11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAAE*/
  0x2E12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAAF*/
  0x2E13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAB0*/
  0x2E14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAB1*/
  0x2E15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAB2*/
  0x2E16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAB3*/
  0x2E17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAB4*/
  0x2E18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAB5*/
  0x2E19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAB6*/
  0x2E1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAB7*/
  0x2E1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAB8*/
  0x2E1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAB9*/
  0x2E1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AABA*/
  0x2E1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AABB*/
  0x2E1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AABC*/
  0x2E20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AABD*/
  0x2E21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AABE*/
  0x0000, 0x00DF, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AABF*/
  0x2E22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAC0*/
  0x0000, 0x00E0, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAC1*/
  0x2E23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAC2*/
  0xFBC1, 0x0020, 0x0002, 0xAAC3, 0x0000, 0x0000, 0x0002, /*AAC3*/
  0xFBC1, 0x0020, 0x0002, 0xAAC4, 0x0000, 0x0000, 0x0002, /*AAC4*/
  0xFBC1, 0x0020, 0x0002, 0xAAC5, 0x0000, 0x0000, 0x0002, /*AAC5*/
  0xFBC1, 0x0020, 0x0002, 0xAAC6, 0x0000, 0x0000, 0x0002, /*AAC6*/
  0xFBC1, 0x0020, 0x0002, 0xAAC7, 0x0000, 0x0000, 0x0002, /*AAC7*/
  0xFBC1, 0x0020, 0x0002, 0xAAC8, 0x0000, 0x0000, 0x0002, /*AAC8*/
  0xFBC1, 0x0020, 0x0002, 0xAAC9, 0x0000, 0x0000, 0x0002, /*AAC9*/
  0xFBC1, 0x0020, 0x0002, 0xAACA, 0x0000, 0x0000, 0x0002, /*AACA*/
  0xFBC1, 0x0020, 0x0002, 0xAACB, 0x0000, 0x0000, 0x0002, /*AACB*/
  0xFBC1, 0x0020, 0x0002, 0xAACC, 0x0000, 0x0000, 0x0002, /*AACC*/
  0xFBC1, 0x0020, 0x0002, 0xAACD, 0x0000, 0x0000, 0x0002, /*AACD*/
  0xFBC1, 0x0020, 0x0002, 0xAACE, 0x0000, 0x0000, 0x0002, /*AACE*/
  0xFBC1, 0x0020, 0x0002, 0xAACF, 0x0000, 0x0000, 0x0002, /*AACF*/
  0xFBC1, 0x0020, 0x0002, 0xAAD0, 0x0000, 0x0000, 0x0002, /*AAD0*/
  0xFBC1, 0x0020, 0x0002, 0xAAD1, 0x0000, 0x0000, 0x0002, /*AAD1*/
  0xFBC1, 0x0020, 0x0002, 0xAAD2, 0x0000, 0x0000, 0x0002, /*AAD2*/
  0xFBC1, 0x0020, 0x0002, 0xAAD3, 0x0000, 0x0000, 0x0002, /*AAD3*/
  0xFBC1, 0x0020, 0x0002, 0xAAD4, 0x0000, 0x0000, 0x0002, /*AAD4*/
  0xFBC1, 0x0020, 0x0002, 0xAAD5, 0x0000, 0x0000, 0x0002, /*AAD5*/
  0xFBC1, 0x0020, 0x0002, 0xAAD6, 0x0000, 0x0000, 0x0002, /*AAD6*/
  0xFBC1, 0x0020, 0x0002, 0xAAD7, 0x0000, 0x0000, 0x0002, /*AAD7*/
  0xFBC1, 0x0020, 0x0002, 0xAAD8, 0x0000, 0x0000, 0x0002, /*AAD8*/
  0xFBC1, 0x0020, 0x0002, 0xAAD9, 0x0000, 0x0000, 0x0002, /*AAD9*/
  0xFBC1, 0x0020, 0x0002, 0xAADA, 0x0000, 0x0000, 0x0002, /*AADA*/
  0x2E24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AADB*/
  0x2E25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AADC*/
  0x1C02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AADD*/
  0x03F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AADE*/
  0x03F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AADF*/
  0x2939, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAE0*/
  0x293A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAE1*/
  0x293B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAE2*/
  0x293C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAE3*/
  0x293D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAE4*/
  0x293E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAE5*/
  0x293F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAE6*/
  0x2940, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAE7*/
  0x2941, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAE8*/
  0x2942, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAE9*/
  0x2943, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAEA*/
  0x294C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAEB*/
  0x294D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAEC*/
  0x294E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAED*/
  0x294F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAEE*/
  0x2950, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAEF*/
  0x02A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAF0*/
  0x0272, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAF1*/
  0x291D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAF2*/
  0x1C03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAF3*/
  0x1C04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAF4*/
  0x2951, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAF5*/
  0x295B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AAF6*/
  0xFBC1, 0x0020, 0x0002, 0xAAF7, 0x0000, 0x0000, 0x0002, /*AAF7*/
  0xFBC1, 0x0020, 0x0002, 0xAAF8, 0x0000, 0x0000, 0x0002, /*AAF8*/
  0xFBC1, 0x0020, 0x0002, 0xAAF9, 0x0000, 0x0000, 0x0002, /*AAF9*/
  0xFBC1, 0x0020, 0x0002, 0xAAFA, 0x0000, 0x0000, 0x0002, /*AAFA*/
  0xFBC1, 0x0020, 0x0002, 0xAAFB, 0x0000, 0x0000, 0x0002, /*AAFB*/
  0xFBC1, 0x0020, 0x0002, 0xAAFC, 0x0000, 0x0000, 0x0002, /*AAFC*/
  0xFBC1, 0x0020, 0x0002, 0xAAFD, 0x0000, 0x0000, 0x0002, /*AAFD*/
  0xFBC1, 0x0020, 0x0002, 0xAAFE, 0x0000, 0x0000, 0x0002, /*AAFE*/
  0xFBC1, 0x0020, 0x0002, 0xAAFF, 0x0000, 0x0000, 0x0002  /*AAFF*/
};

uint16 uca900_p0AB[]= {
  0xFBC1, 0x0020, 0x0002, 0xAB00, 0x0000, 0x0000, 0x0002, /*AB00*/
  0x24D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB01*/
  0x24D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB02*/
  0x24D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB03*/
  0x24D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB04*/
  0x24DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB05*/
  0x24DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB06*/
  0xFBC1, 0x0020, 0x0002, 0xAB07, 0x0000, 0x0000, 0x0002, /*AB07*/
  0xFBC1, 0x0020, 0x0002, 0xAB08, 0x0000, 0x0000, 0x0002, /*AB08*/
  0x259D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB09*/
  0x259E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB0A*/
  0x259F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB0B*/
  0x25A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB0C*/
  0x25A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB0D*/
  0x25A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB0E*/
  0xFBC1, 0x0020, 0x0002, 0xAB0F, 0x0000, 0x0000, 0x0002, /*AB0F*/
  0xFBC1, 0x0020, 0x0002, 0xAB10, 0x0000, 0x0000, 0x0002, /*AB10*/
  0x257E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB11*/
  0x257F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB12*/
  0x2580, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB13*/
  0x2581, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB14*/
  0x2582, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB15*/
  0x2583, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB16*/
  0xFBC1, 0x0020, 0x0002, 0xAB17, 0x0000, 0x0000, 0x0002, /*AB17*/
  0xFBC1, 0x0020, 0x0002, 0xAB18, 0x0000, 0x0000, 0x0002, /*AB18*/
  0xFBC1, 0x0020, 0x0002, 0xAB19, 0x0000, 0x0000, 0x0002, /*AB19*/
  0xFBC1, 0x0020, 0x0002, 0xAB1A, 0x0000, 0x0000, 0x0002, /*AB1A*/
  0xFBC1, 0x0020, 0x0002, 0xAB1B, 0x0000, 0x0000, 0x0002, /*AB1B*/
  0xFBC1, 0x0020, 0x0002, 0xAB1C, 0x0000, 0x0000, 0x0002, /*AB1C*/
  0xFBC1, 0x0020, 0x0002, 0xAB1D, 0x0000, 0x0000, 0x0002, /*AB1D*/
  0xFBC1, 0x0020, 0x0002, 0xAB1E, 0x0000, 0x0000, 0x0002, /*AB1E*/
  0xFBC1, 0x0020, 0x0002, 0xAB1F, 0x0000, 0x0000, 0x0002, /*AB1F*/
  0x25E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB20*/
  0x25E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB21*/
  0x25E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB22*/
  0x25E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB23*/
  0x25E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB24*/
  0x25E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB25*/
  0x25E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB26*/
  0xFBC1, 0x0020, 0x0002, 0xAB27, 0x0000, 0x0000, 0x0002, /*AB27*/
  0x25F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB28*/
  0x25F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB29*/
  0x25FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB2A*/
  0x25FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB2B*/
  0x25FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB2C*/
  0x25FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB2D*/
  0x25FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB2E*/
  0xFBC1, 0x0020, 0x0002, 0xAB2F, 0x0000, 0x0000, 0x0002, /*AB2F*/
  0x1C59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB30*/
  0x1C50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB31*/
  0x1CAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB32*/
  0x1CB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB33*/
  0x1CB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB34*/
  0x1CEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB35*/
  0x1CFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB36*/
  0x1D91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB37*/
  0x1D8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB38*/
  0x1D8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB39*/
  0x1DB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB3A*/
  0x1DD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB3B*/
  0x1DDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB3C*/
  0x1DE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB3D*/
  0x1DEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB3E*/
  0x1DF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB3F*/
  0x1DEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB40*/
  0x1DE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB41*/
  0x1DEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB42*/
  0x1DEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB43*/
  0x1DED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB44*/
  0x1E37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB45*/
  0x1E3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB46*/
  0x1E65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB47*/
  0x1E66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB48*/
  0x1E5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB49*/
  0x1E67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB4A*/
  0x1E68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB4B*/
  0x1E69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB4C*/
  0x1E86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB4D*/
  0x1EBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB4E*/
  0x1EC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB4F*/
  0x1EBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB50*/
  0x1EBF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB51*/
  0x1EC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB52*/
  0x1F08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB53*/
  0x1F09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB54*/
  0x1F0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB55*/
  0x1F04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB56*/
  0x1F05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB57*/
  0x1F06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB58*/
  0x1F07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB59*/
  0x1F1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB5A*/
  0x04F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB5B*/
  0x1D2B, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0001, /*AB5C*/
  0x1D91, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0001, /*AB5D*/
  0x1D87, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0001, /*AB5E*/
  0x1EC7, 0x0020, 0x0014, 0x0000, 0x0000, 0x0000, 0x0001, /*AB5F*/
  0x1F5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB60*/
  0x1F5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB61*/
  0x1DF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB62*/
  0x1F5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB63*/
  0x1C5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB64*/
  0x1FE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB65*/
  0xFBC1, 0x0020, 0x0002, 0xAB66, 0x0000, 0x0000, 0x0002, /*AB66*/
  0xFBC1, 0x0020, 0x0002, 0xAB67, 0x0000, 0x0000, 0x0002, /*AB67*/
  0xFBC1, 0x0020, 0x0002, 0xAB68, 0x0000, 0x0000, 0x0002, /*AB68*/
  0xFBC1, 0x0020, 0x0002, 0xAB69, 0x0000, 0x0000, 0x0002, /*AB69*/
  0xFBC1, 0x0020, 0x0002, 0xAB6A, 0x0000, 0x0000, 0x0002, /*AB6A*/
  0xFBC1, 0x0020, 0x0002, 0xAB6B, 0x0000, 0x0000, 0x0002, /*AB6B*/
  0xFBC1, 0x0020, 0x0002, 0xAB6C, 0x0000, 0x0000, 0x0002, /*AB6C*/
  0xFBC1, 0x0020, 0x0002, 0xAB6D, 0x0000, 0x0000, 0x0002, /*AB6D*/
  0xFBC1, 0x0020, 0x0002, 0xAB6E, 0x0000, 0x0000, 0x0002, /*AB6E*/
  0xFBC1, 0x0020, 0x0002, 0xAB6F, 0x0000, 0x0000, 0x0002, /*AB6F*/
  0x337F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB70*/
  0x3380, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB71*/
  0x3381, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB72*/
  0x3382, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB73*/
  0x3383, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB74*/
  0x3384, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB75*/
  0x3385, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB76*/
  0x3386, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB77*/
  0x3387, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB78*/
  0x3388, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB79*/
  0x3389, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB7A*/
  0x338A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB7B*/
  0x338B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB7C*/
  0x338C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB7D*/
  0x338D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB7E*/
  0x338E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB7F*/
  0x338F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB80*/
  0x3390, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB81*/
  0x3391, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB82*/
  0x3392, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB83*/
  0x3393, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB84*/
  0x3394, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB85*/
  0x3395, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB86*/
  0x3396, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB87*/
  0x3397, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB88*/
  0x3398, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB89*/
  0x3399, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB8A*/
  0x339A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB8B*/
  0x339B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB8C*/
  0x339C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB8D*/
  0x339D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB8E*/
  0x339E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB8F*/
  0x339F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB90*/
  0x33A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB91*/
  0x33A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB92*/
  0x33A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB93*/
  0x33A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB94*/
  0x33A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB95*/
  0x33A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB96*/
  0x33A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB97*/
  0x33A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB98*/
  0x33A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB99*/
  0x33A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB9A*/
  0x33AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB9B*/
  0x33AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB9C*/
  0x33AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB9D*/
  0x33AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB9E*/
  0x33AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*AB9F*/
  0x33AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABA0*/
  0x33B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABA1*/
  0x33B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABA2*/
  0x33B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABA3*/
  0x33B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABA4*/
  0x33B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABA5*/
  0x33B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABA6*/
  0x33B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABA7*/
  0x33B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABA8*/
  0x33B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABA9*/
  0x33B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABAA*/
  0x33BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABAB*/
  0x33BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABAC*/
  0x33BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABAD*/
  0x33BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABAE*/
  0x33BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABAF*/
  0x33BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABB0*/
  0x33C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABB1*/
  0x33C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABB2*/
  0x33C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABB3*/
  0x33C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABB4*/
  0x33C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABB5*/
  0x33C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABB6*/
  0x33C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABB7*/
  0x33C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABB8*/
  0x33C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABB9*/
  0x33C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABBA*/
  0x33CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABBB*/
  0x33CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABBC*/
  0x33CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABBD*/
  0x33CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABBE*/
  0x33CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABBF*/
  0x291E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABC0*/
  0x291F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABC1*/
  0x2920, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABC2*/
  0x2921, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABC3*/
  0x2922, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABC4*/
  0x2923, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABC5*/
  0x2924, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABC6*/
  0x2925, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABC7*/
  0x2926, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABC8*/
  0x2927, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABC9*/
  0x2928, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABCA*/
  0x2929, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABCB*/
  0x292A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABCC*/
  0x292B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABCD*/
  0x292C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABCE*/
  0x292D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABCF*/
  0x292E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABD0*/
  0x292F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABD1*/
  0x2930, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABD2*/
  0x2931, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABD3*/
  0x2932, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABD4*/
  0x2933, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABD5*/
  0x2934, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABD6*/
  0x2935, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABD7*/
  0x2936, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABD8*/
  0x2937, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABD9*/
  0x2938, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABDA*/
  0x2952, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABDB*/
  0x2953, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABDC*/
  0x2954, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABDD*/
  0x2955, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABDE*/
  0x2956, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABDF*/
  0x2957, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABE0*/
  0x2958, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABE1*/
  0x2959, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABE2*/
  0x2944, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABE3*/
  0x2945, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABE4*/
  0x2946, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABE5*/
  0x2947, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABE6*/
  0x2948, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABE7*/
  0x2949, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABE8*/
  0x294A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABE9*/
  0x294B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABEA*/
  0x02A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABEB*/
  0x0000, 0x00CB, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABEC*/
  0x295A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABED*/
  0xFBC1, 0x0020, 0x0002, 0xABEE, 0x0000, 0x0000, 0x0002, /*ABEE*/
  0xFBC1, 0x0020, 0x0002, 0xABEF, 0x0000, 0x0000, 0x0002, /*ABEF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABF0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABF1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABF2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABF3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABF4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABF5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABF6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABF7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABF8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*ABF9*/
  0xFBC1, 0x0020, 0x0002, 0xABFA, 0x0000, 0x0000, 0x0002, /*ABFA*/
  0xFBC1, 0x0020, 0x0002, 0xABFB, 0x0000, 0x0000, 0x0002, /*ABFB*/
  0xFBC1, 0x0020, 0x0002, 0xABFC, 0x0000, 0x0000, 0x0002, /*ABFC*/
  0xFBC1, 0x0020, 0x0002, 0xABFD, 0x0000, 0x0000, 0x0002, /*ABFD*/
  0xFBC1, 0x0020, 0x0002, 0xABFE, 0x0000, 0x0000, 0x0002, /*ABFE*/
  0xFBC1, 0x0020, 0x0002, 0xABFF, 0x0000, 0x0000, 0x0002  /*ABFF*/
};

uint16 uca900_p0D7[]= {
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D700*/
  0x0003, 0x3CD4, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D701*/
  0x0003, 0x3CD5, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D702*/
  0x0003, 0x3CD6, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D703*/
  0x0003, 0x3CD7, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D704*/
  0x0003, 0x3CD8, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D705*/
  0x0003, 0x3CD9, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D706*/
  0x0003, 0x3CDA, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D707*/
  0x0003, 0x3CDB, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D708*/
  0x0003, 0x3CDC, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D709*/
  0x0003, 0x3CDD, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D70A*/
  0x0003, 0x3CDE, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D70B*/
  0x0003, 0x3CDF, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D70C*/
  0x0003, 0x3CE0, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D70D*/
  0x0003, 0x3CE1, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D70E*/
  0x0003, 0x3CE2, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D70F*/
  0x0003, 0x3CE3, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D710*/
  0x0003, 0x3CE4, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D711*/
  0x0003, 0x3CE5, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D712*/
  0x0003, 0x3CE6, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D713*/
  0x0003, 0x3CE7, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D714*/
  0x0003, 0x3CE8, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D715*/
  0x0003, 0x3CE9, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D716*/
  0x0003, 0x3CEA, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C82, 0x0020, /*D717*/
  0x0003, 0x3CEB, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D718*/
  0x0003, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D719*/
  0x0003, 0x3CD1, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D71A*/
  0x0003, 0x3CD2, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D71B*/
  0x0003, 0x3CD3, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D71C*/
  0x0003, 0x3CD4, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D71D*/
  0x0003, 0x3CD5, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D71E*/
  0x0003, 0x3CD6, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D71F*/
  0x0003, 0x3CD7, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D720*/
  0x0003, 0x3CD8, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D721*/
  0x0003, 0x3CD9, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D722*/
  0x0003, 0x3CDA, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D723*/
  0x0003, 0x3CDB, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D724*/
  0x0003, 0x3CDC, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D725*/
  0x0003, 0x3CDD, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D726*/
  0x0003, 0x3CDE, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D727*/
  0x0003, 0x3CDF, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D728*/
  0x0003, 0x3CE0, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D729*/
  0x0003, 0x3CE1, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D72A*/
  0x0003, 0x3CE2, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D72B*/
  0x0003, 0x3CE3, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D72C*/
  0x0003, 0x3CE4, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D72D*/
  0x0003, 0x3CE5, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D72E*/
  0x0003, 0x3CE6, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D72F*/
  0x0003, 0x3CE7, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D730*/
  0x0003, 0x3CE8, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D731*/
  0x0003, 0x3CE9, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D732*/
  0x0003, 0x3CEA, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C83, 0x0020, /*D733*/
  0x0003, 0x3CEB, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D734*/
  0x0003, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D735*/
  0x0003, 0x3CD1, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D736*/
  0x0003, 0x3CD2, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D737*/
  0x0003, 0x3CD3, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D738*/
  0x0003, 0x3CD4, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D739*/
  0x0003, 0x3CD5, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D73A*/
  0x0003, 0x3CD6, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D73B*/
  0x0003, 0x3CD7, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D73C*/
  0x0003, 0x3CD8, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D73D*/
  0x0003, 0x3CD9, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D73E*/
  0x0003, 0x3CDA, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D73F*/
  0x0003, 0x3CDB, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D740*/
  0x0003, 0x3CDC, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D741*/
  0x0003, 0x3CDD, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D742*/
  0x0003, 0x3CDE, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D743*/
  0x0003, 0x3CDF, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D744*/
  0x0003, 0x3CE0, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D745*/
  0x0003, 0x3CE1, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D746*/
  0x0003, 0x3CE2, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D747*/
  0x0003, 0x3CE3, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D748*/
  0x0003, 0x3CE4, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D749*/
  0x0003, 0x3CE5, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D74A*/
  0x0003, 0x3CE6, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D74B*/
  0x0003, 0x3CE7, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D74C*/
  0x0003, 0x3CE8, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D74D*/
  0x0003, 0x3CE9, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D74E*/
  0x0003, 0x3CEA, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C84, 0x0020, /*D74F*/
  0x0003, 0x3CEB, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D750*/
  0x0003, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D751*/
  0x0003, 0x3CD1, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D752*/
  0x0003, 0x3CD2, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D753*/
  0x0003, 0x3CD3, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D754*/
  0x0003, 0x3CD4, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D755*/
  0x0003, 0x3CD5, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D756*/
  0x0003, 0x3CD6, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D757*/
  0x0003, 0x3CD7, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D758*/
  0x0003, 0x3CD8, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D759*/
  0x0003, 0x3CD9, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D75A*/
  0x0003, 0x3CDA, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D75B*/
  0x0003, 0x3CDB, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D75C*/
  0x0003, 0x3CDC, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D75D*/
  0x0003, 0x3CDD, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D75E*/
  0x0003, 0x3CDE, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D75F*/
  0x0003, 0x3CDF, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D760*/
  0x0003, 0x3CE0, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D761*/
  0x0003, 0x3CE1, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D762*/
  0x0003, 0x3CE2, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D763*/
  0x0003, 0x3CE3, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D764*/
  0x0003, 0x3CE4, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D765*/
  0x0003, 0x3CE5, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D766*/
  0x0003, 0x3CE6, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D767*/
  0x0003, 0x3CE7, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D768*/
  0x0003, 0x3CE8, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D769*/
  0x0003, 0x3CE9, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D76A*/
  0x0003, 0x3CEA, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C85, 0x0020, /*D76B*/
  0x0003, 0x3CEB, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D76C*/
  0x0003, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D76D*/
  0x0003, 0x3CD1, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D76E*/
  0x0003, 0x3CD2, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D76F*/
  0x0003, 0x3CD3, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D770*/
  0x0003, 0x3CD4, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D771*/
  0x0003, 0x3CD5, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D772*/
  0x0003, 0x3CD6, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D773*/
  0x0003, 0x3CD7, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D774*/
  0x0003, 0x3CD8, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D775*/
  0x0003, 0x3CD9, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D776*/
  0x0003, 0x3CDA, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D777*/
  0x0003, 0x3CDB, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D778*/
  0x0003, 0x3CDC, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D779*/
  0x0003, 0x3CDD, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D77A*/
  0x0003, 0x3CDE, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D77B*/
  0x0003, 0x3CDF, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D77C*/
  0x0003, 0x3CE0, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D77D*/
  0x0003, 0x3CE1, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D77E*/
  0x0003, 0x3CE2, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D77F*/
  0x0003, 0x3CE3, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D780*/
  0x0003, 0x3CE4, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D781*/
  0x0003, 0x3CE5, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D782*/
  0x0003, 0x3CE6, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D783*/
  0x0003, 0x3CE7, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D784*/
  0x0003, 0x3CE8, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D785*/
  0x0003, 0x3CE9, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D786*/
  0x0003, 0x3CEA, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C86, 0x0020, /*D787*/
  0x0003, 0x3CEB, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D788*/
  0x0003, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D789*/
  0x0003, 0x3CD1, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D78A*/
  0x0003, 0x3CD2, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D78B*/
  0x0003, 0x3CD3, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D78C*/
  0x0003, 0x3CD4, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D78D*/
  0x0003, 0x3CD5, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D78E*/
  0x0003, 0x3CD6, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D78F*/
  0x0003, 0x3CD7, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D790*/
  0x0003, 0x3CD8, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D791*/
  0x0003, 0x3CD9, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D792*/
  0x0003, 0x3CDA, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D793*/
  0x0003, 0x3CDB, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D794*/
  0x0003, 0x3CDC, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D795*/
  0x0003, 0x3CDD, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D796*/
  0x0003, 0x3CDE, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D797*/
  0x0003, 0x3CDF, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D798*/
  0x0003, 0x3CE0, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D799*/
  0x0003, 0x3CE1, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D79A*/
  0x0003, 0x3CE2, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D79B*/
  0x0003, 0x3CE3, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D79C*/
  0x0003, 0x3CE4, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D79D*/
  0x0003, 0x3CE5, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D79E*/
  0x0003, 0x3CE6, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D79F*/
  0x0003, 0x3CE7, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D7A0*/
  0x0003, 0x3CE8, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D7A1*/
  0x0003, 0x3CE9, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D7A2*/
  0x0003, 0x3CEA, 0x0020, 0x0003, 0x0003,
  0x3C07, 0x0020, 0x0003, 0x3C87, 0x0020, /*D7A3*/
  0x0003, 0x3CEB, 0x0020, 0x0003, 0x0003,
  0x3C08, 0x0020, 0x0003, 0x3C73, 0x0020, /*D7A4*/
  0x0003, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3C08, 0x0020, 0x0003, 0x3C73, 0x0020, /*D7A5*/
  0x0003, 0x3CD1, 0x0020, 0x0003, 0x0003,
  0x3C08, 0x0020, 0x0003, 0x3C73, 0x0020, /*D7A6*/
  0x0003, 0x3CD2, 0x0020, 0x0003, 0x0003,
  0x3C08, 0x0020, 0x0003, 0x3C73, 0x0020, /*D7A7*/
  0x0003, 0x3CD3, 0x0020, 0x0003, 0x0003,
  0x3C08, 0x0020, 0x0003, 0x3C73, 0x0020, /*D7A8*/
  0x0003, 0x3CD4, 0x0020, 0x0003, 0x0003,
  0x3C08, 0x0020, 0x0003, 0x3C73, 0x0020, /*D7A9*/
  0x0003, 0x3CD5, 0x0020, 0x0003, 0x0003,
  0x3C08, 0x0020, 0x0003, 0x3C73, 0x0020, /*D7AA*/
  0x0003, 0x3CD6, 0x0020, 0x0003, 0x0003,
  0x3C08, 0x0020, 0x0003, 0x3C73, 0x0020, /*D7AB*/
  0x0003, 0x3CD7, 0x0020, 0x0003, 0x0003,
  0x3C08, 0x0020, 0x0003, 0x3C73, 0x0020, /*D7AC*/
  0x0003, 0x3CD8, 0x0020, 0x0003, 0x0003,
  0x3C08, 0x0020, 0x0003, 0x3C73, 0x0020, /*D7AD*/
  0x0003, 0x3CD9, 0x0020, 0x0003, 0x0003,
  0x3C08, 0x0020, 0x0003, 0x3C73, 0x0020, /*D7AE*/
  0x0003, 0x3CDA, 0x0020, 0x0003, 0x0003,
  0x3C08, 0x0020, 0x0003, 0x3C73, 0x0020, /*D7AF*/
  0x0003, 0x3CDB, 0x0020, 0x0003, 0x0003,
  0x3CBA, 0x0020, 0x0002, 0x0000, 0x0000, /*D7B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CBB, 0x0020, 0x0002, 0x0000, 0x0000, /*D7B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CBC, 0x0020, 0x0002, 0x0000, 0x0000, /*D7B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CBD, 0x0020, 0x0002, 0x0000, 0x0000, /*D7B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CBE, 0x0020, 0x0002, 0x0000, 0x0000, /*D7B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CBF, 0x0020, 0x0002, 0x0000, 0x0000, /*D7B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CC0, 0x0020, 0x0002, 0x0000, 0x0000, /*D7B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CC1, 0x0020, 0x0002, 0x0000, 0x0000, /*D7B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CC2, 0x0020, 0x0002, 0x0000, 0x0000, /*D7B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CC3, 0x0020, 0x0002, 0x0000, 0x0000, /*D7B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CC4, 0x0020, 0x0002, 0x0000, 0x0000, /*D7BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CC5, 0x0020, 0x0002, 0x0000, 0x0000, /*D7BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CC6, 0x0020, 0x0002, 0x0000, 0x0000, /*D7BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CC7, 0x0020, 0x0002, 0x0000, 0x0000, /*D7BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CC8, 0x0020, 0x0002, 0x0000, 0x0000, /*D7BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CC9, 0x0020, 0x0002, 0x0000, 0x0000, /*D7BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CCA, 0x0020, 0x0002, 0x0000, 0x0000, /*D7C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CCB, 0x0020, 0x0002, 0x0000, 0x0000, /*D7C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CCC, 0x0020, 0x0002, 0x0000, 0x0000, /*D7C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CCD, 0x0020, 0x0002, 0x0000, 0x0000, /*D7C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CCE, 0x0020, 0x0002, 0x0000, 0x0000, /*D7C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CCF, 0x0020, 0x0002, 0x0000, 0x0000, /*D7C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3CD0, 0x0020, 0x0002, 0x0000, 0x0000, /*D7C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC1, 0x0020, 0x0002, 0xD7C7, 0x0000, /*D7C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xD7C8, 0x0000, /*D7C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xD7C9, 0x0000, /*D7C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xD7CA, 0x0000, /*D7CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3D29, 0x0020, 0x0002, 0x0000, 0x0000, /*D7CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D2A, 0x0020, 0x0002, 0x0000, 0x0000, /*D7CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D2B, 0x0020, 0x0002, 0x0000, 0x0000, /*D7CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D2C, 0x0020, 0x0002, 0x0000, 0x0000, /*D7CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D2D, 0x0020, 0x0002, 0x0000, 0x0000, /*D7CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D2E, 0x0020, 0x0002, 0x0000, 0x0000, /*D7D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D2F, 0x0020, 0x0002, 0x0000, 0x0000, /*D7D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D30, 0x0020, 0x0002, 0x0000, 0x0000, /*D7D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D31, 0x0020, 0x0002, 0x0000, 0x0000, /*D7D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D32, 0x0020, 0x0002, 0x0000, 0x0000, /*D7D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D33, 0x0020, 0x0002, 0x0000, 0x0000, /*D7D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D34, 0x0020, 0x0002, 0x0000, 0x0000, /*D7D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D35, 0x0020, 0x0002, 0x0000, 0x0000, /*D7D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D36, 0x0020, 0x0002, 0x0000, 0x0000, /*D7D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D37, 0x0020, 0x0002, 0x0000, 0x0000, /*D7D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D38, 0x0020, 0x0002, 0x0000, 0x0000, /*D7DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D39, 0x0020, 0x0002, 0x0000, 0x0000, /*D7DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D3A, 0x0020, 0x0002, 0x0000, 0x0000, /*D7DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D3B, 0x0020, 0x0002, 0x0000, 0x0000, /*D7DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D3C, 0x0020, 0x0002, 0x0000, 0x0000, /*D7DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D3D, 0x0020, 0x0002, 0x0000, 0x0000, /*D7DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D3E, 0x0020, 0x0002, 0x0000, 0x0000, /*D7E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D3F, 0x0020, 0x0002, 0x0000, 0x0000, /*D7E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D40, 0x0020, 0x0002, 0x0000, 0x0000, /*D7E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D41, 0x0020, 0x0002, 0x0000, 0x0000, /*D7E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D42, 0x0020, 0x0002, 0x0000, 0x0000, /*D7E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D43, 0x0020, 0x0002, 0x0000, 0x0000, /*D7E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D44, 0x0020, 0x0002, 0x0000, 0x0000, /*D7E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D45, 0x0020, 0x0002, 0x0000, 0x0000, /*D7E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D46, 0x0020, 0x0002, 0x0000, 0x0000, /*D7E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D47, 0x0020, 0x0002, 0x0000, 0x0000, /*D7E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D48, 0x0020, 0x0002, 0x0000, 0x0000, /*D7EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D49, 0x0020, 0x0002, 0x0000, 0x0000, /*D7EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D4A, 0x0020, 0x0002, 0x0000, 0x0000, /*D7EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D4B, 0x0020, 0x0002, 0x0000, 0x0000, /*D7ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D4C, 0x0020, 0x0002, 0x0000, 0x0000, /*D7EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D4D, 0x0020, 0x0002, 0x0000, 0x0000, /*D7EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D4E, 0x0020, 0x0002, 0x0000, 0x0000, /*D7F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D4F, 0x0020, 0x0002, 0x0000, 0x0000, /*D7F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D50, 0x0020, 0x0002, 0x0000, 0x0000, /*D7F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D51, 0x0020, 0x0002, 0x0000, 0x0000, /*D7F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D52, 0x0020, 0x0002, 0x0000, 0x0000, /*D7F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D53, 0x0020, 0x0002, 0x0000, 0x0000, /*D7F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D54, 0x0020, 0x0002, 0x0000, 0x0000, /*D7F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D55, 0x0020, 0x0002, 0x0000, 0x0000, /*D7F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D56, 0x0020, 0x0002, 0x0000, 0x0000, /*D7F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D57, 0x0020, 0x0002, 0x0000, 0x0000, /*D7F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D58, 0x0020, 0x0002, 0x0000, 0x0000, /*D7FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x3D59, 0x0020, 0x0002, 0x0000, 0x0000, /*D7FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC1, 0x0020, 0x0002, 0xD7FC, 0x0000, /*D7FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xD7FD, 0x0000, /*D7FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xD7FE, 0x0000, /*D7FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xD7FF, 0x0000, /*D7FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002
};

uint16 uca900_p0F9[]= {
  0xFB41, 0x0020, 0x0002, 0x8C48, 0x0000, 0x0000, 0x0002, /*F900*/
  0xFB40, 0x0020, 0x0002, 0xE6F4, 0x0000, 0x0000, 0x0002, /*F901*/
  0xFB41, 0x0020, 0x0002, 0x8ECA, 0x0000, 0x0000, 0x0002, /*F902*/
  0xFB41, 0x0020, 0x0002, 0x8CC8, 0x0000, 0x0000, 0x0002, /*F903*/
  0xFB40, 0x0020, 0x0002, 0xEED1, 0x0000, 0x0000, 0x0002, /*F904*/
  0xFB40, 0x0020, 0x0002, 0xCE32, 0x0000, 0x0000, 0x0002, /*F905*/
  0xFB40, 0x0020, 0x0002, 0xD3E5, 0x0000, 0x0000, 0x0002, /*F906*/
  0xFB41, 0x0020, 0x0002, 0x9F9C, 0x0000, 0x0000, 0x0002, /*F907*/
  0xFB41, 0x0020, 0x0002, 0x9F9C, 0x0000, 0x0000, 0x0002, /*F908*/
  0xFB40, 0x0020, 0x0002, 0xD951, 0x0000, 0x0000, 0x0002, /*F909*/
  0xFB41, 0x0020, 0x0002, 0x91D1, 0x0000, 0x0000, 0x0002, /*F90A*/
  0xFB40, 0x0020, 0x0002, 0xD587, 0x0000, 0x0000, 0x0002, /*F90B*/
  0xFB40, 0x0020, 0x0002, 0xD948, 0x0000, 0x0000, 0x0002, /*F90C*/
  0xFB40, 0x0020, 0x0002, 0xE1F6, 0x0000, 0x0000, 0x0002, /*F90D*/
  0xFB40, 0x0020, 0x0002, 0xF669, 0x0000, 0x0000, 0x0002, /*F90E*/
  0xFB40, 0x0020, 0x0002, 0xFF85, 0x0000, 0x0000, 0x0002, /*F90F*/
  0xFB41, 0x0020, 0x0002, 0x863F, 0x0000, 0x0000, 0x0002, /*F910*/
  0xFB41, 0x0020, 0x0002, 0x87BA, 0x0000, 0x0000, 0x0002, /*F911*/
  0xFB41, 0x0020, 0x0002, 0x88F8, 0x0000, 0x0000, 0x0002, /*F912*/
  0xFB41, 0x0020, 0x0002, 0x908F, 0x0000, 0x0000, 0x0002, /*F913*/
  0xFB40, 0x0020, 0x0002, 0xEA02, 0x0000, 0x0000, 0x0002, /*F914*/
  0xFB40, 0x0020, 0x0002, 0xED1B, 0x0000, 0x0000, 0x0002, /*F915*/
  0xFB40, 0x0020, 0x0002, 0xF0D9, 0x0000, 0x0000, 0x0002, /*F916*/
  0xFB40, 0x0020, 0x0002, 0xF3DE, 0x0000, 0x0000, 0x0002, /*F917*/
  0xFB41, 0x0020, 0x0002, 0x843D, 0x0000, 0x0000, 0x0002, /*F918*/
  0xFB41, 0x0020, 0x0002, 0x916A, 0x0000, 0x0000, 0x0002, /*F919*/
  0xFB41, 0x0020, 0x0002, 0x99F1, 0x0000, 0x0000, 0x0002, /*F91A*/
  0xFB40, 0x0020, 0x0002, 0xCE82, 0x0000, 0x0000, 0x0002, /*F91B*/
  0xFB40, 0x0020, 0x0002, 0xD375, 0x0000, 0x0000, 0x0002, /*F91C*/
  0xFB40, 0x0020, 0x0002, 0xEB04, 0x0000, 0x0000, 0x0002, /*F91D*/
  0xFB40, 0x0020, 0x0002, 0xF21B, 0x0000, 0x0000, 0x0002, /*F91E*/
  0xFB41, 0x0020, 0x0002, 0x862D, 0x0000, 0x0000, 0x0002, /*F91F*/
  0xFB41, 0x0020, 0x0002, 0x9E1E, 0x0000, 0x0000, 0x0002, /*F920*/
  0xFB40, 0x0020, 0x0002, 0xDD50, 0x0000, 0x0000, 0x0002, /*F921*/
  0xFB40, 0x0020, 0x0002, 0xEFEB, 0x0000, 0x0000, 0x0002, /*F922*/
  0xFB41, 0x0020, 0x0002, 0x85CD, 0x0000, 0x0000, 0x0002, /*F923*/
  0xFB41, 0x0020, 0x0002, 0x8964, 0x0000, 0x0000, 0x0002, /*F924*/
  0xFB40, 0x0020, 0x0002, 0xE2C9, 0x0000, 0x0000, 0x0002, /*F925*/
  0xFB41, 0x0020, 0x0002, 0x81D8, 0x0000, 0x0000, 0x0002, /*F926*/
  0xFB41, 0x0020, 0x0002, 0x881F, 0x0000, 0x0000, 0x0002, /*F927*/
  0xFB40, 0x0020, 0x0002, 0xDECA, 0x0000, 0x0000, 0x0002, /*F928*/
  0xFB40, 0x0020, 0x0002, 0xE717, 0x0000, 0x0000, 0x0002, /*F929*/
  0xFB40, 0x0020, 0x0002, 0xED6A, 0x0000, 0x0000, 0x0002, /*F92A*/
  0xFB40, 0x0020, 0x0002, 0xF2FC, 0x0000, 0x0000, 0x0002, /*F92B*/
  0xFB41, 0x0020, 0x0002, 0x90CE, 0x0000, 0x0000, 0x0002, /*F92C*/
  0xFB40, 0x0020, 0x0002, 0xCF86, 0x0000, 0x0000, 0x0002, /*F92D*/
  0xFB40, 0x0020, 0x0002, 0xD1B7, 0x0000, 0x0000, 0x0002, /*F92E*/
  0xFB40, 0x0020, 0x0002, 0xD2DE, 0x0000, 0x0000, 0x0002, /*F92F*/
  0xFB40, 0x0020, 0x0002, 0xE4C4, 0x0000, 0x0000, 0x0002, /*F930*/
  0xFB40, 0x0020, 0x0002, 0xEAD3, 0x0000, 0x0000, 0x0002, /*F931*/
  0xFB40, 0x0020, 0x0002, 0xF210, 0x0000, 0x0000, 0x0002, /*F932*/
  0xFB40, 0x0020, 0x0002, 0xF6E7, 0x0000, 0x0000, 0x0002, /*F933*/
  0xFB41, 0x0020, 0x0002, 0x8001, 0x0000, 0x0000, 0x0002, /*F934*/
  0xFB41, 0x0020, 0x0002, 0x8606, 0x0000, 0x0000, 0x0002, /*F935*/
  0xFB41, 0x0020, 0x0002, 0x865C, 0x0000, 0x0000, 0x0002, /*F936*/
  0xFB41, 0x0020, 0x0002, 0x8DEF, 0x0000, 0x0000, 0x0002, /*F937*/
  0xFB41, 0x0020, 0x0002, 0x9732, 0x0000, 0x0000, 0x0002, /*F938*/
  0xFB41, 0x0020, 0x0002, 0x9B6F, 0x0000, 0x0000, 0x0002, /*F939*/
  0xFB41, 0x0020, 0x0002, 0x9DFA, 0x0000, 0x0000, 0x0002, /*F93A*/
  0xFB40, 0x0020, 0x0002, 0xF88C, 0x0000, 0x0000, 0x0002, /*F93B*/
  0xFB40, 0x0020, 0x0002, 0xF97F, 0x0000, 0x0000, 0x0002, /*F93C*/
  0xFB40, 0x0020, 0x0002, 0xFDA0, 0x0000, 0x0000, 0x0002, /*F93D*/
  0xFB41, 0x0020, 0x0002, 0x83C9, 0x0000, 0x0000, 0x0002, /*F93E*/
  0xFB41, 0x0020, 0x0002, 0x9304, 0x0000, 0x0000, 0x0002, /*F93F*/
  0xFB41, 0x0020, 0x0002, 0x9E7F, 0x0000, 0x0000, 0x0002, /*F940*/
  0xFB41, 0x0020, 0x0002, 0x8AD6, 0x0000, 0x0000, 0x0002, /*F941*/
  0xFB40, 0x0020, 0x0002, 0xD8DF, 0x0000, 0x0000, 0x0002, /*F942*/
  0xFB40, 0x0020, 0x0002, 0xDF04, 0x0000, 0x0000, 0x0002, /*F943*/
  0xFB40, 0x0020, 0x0002, 0xFC60, 0x0000, 0x0000, 0x0002, /*F944*/
  0xFB41, 0x0020, 0x0002, 0x807E, 0x0000, 0x0000, 0x0002, /*F945*/
  0xFB40, 0x0020, 0x0002, 0xF262, 0x0000, 0x0000, 0x0002, /*F946*/
  0xFB40, 0x0020, 0x0002, 0xF8CA, 0x0000, 0x0000, 0x0002, /*F947*/
  0xFB41, 0x0020, 0x0002, 0x8CC2, 0x0000, 0x0000, 0x0002, /*F948*/
  0xFB41, 0x0020, 0x0002, 0x96F7, 0x0000, 0x0000, 0x0002, /*F949*/
  0xFB40, 0x0020, 0x0002, 0xD8D8, 0x0000, 0x0000, 0x0002, /*F94A*/
  0xFB40, 0x0020, 0x0002, 0xDC62, 0x0000, 0x0000, 0x0002, /*F94B*/
  0xFB40, 0x0020, 0x0002, 0xEA13, 0x0000, 0x0000, 0x0002, /*F94C*/
  0xFB40, 0x0020, 0x0002, 0xEDDA, 0x0000, 0x0000, 0x0002, /*F94D*/
  0xFB40, 0x0020, 0x0002, 0xEF0F, 0x0000, 0x0000, 0x0002, /*F94E*/
  0xFB40, 0x0020, 0x0002, 0xFD2F, 0x0000, 0x0000, 0x0002, /*F94F*/
  0xFB40, 0x0020, 0x0002, 0xFE37, 0x0000, 0x0000, 0x0002, /*F950*/
  0xFB41, 0x0020, 0x0002, 0x964B, 0x0000, 0x0000, 0x0002, /*F951*/
  0xFB40, 0x0020, 0x0002, 0xD2D2, 0x0000, 0x0000, 0x0002, /*F952*/
  0xFB41, 0x0020, 0x0002, 0x808B, 0x0000, 0x0000, 0x0002, /*F953*/
  0xFB40, 0x0020, 0x0002, 0xD1DC, 0x0000, 0x0000, 0x0002, /*F954*/
  0xFB40, 0x0020, 0x0002, 0xD1CC, 0x0000, 0x0000, 0x0002, /*F955*/
  0xFB40, 0x0020, 0x0002, 0xFA1C, 0x0000, 0x0000, 0x0002, /*F956*/
  0xFB40, 0x0020, 0x0002, 0xFDBE, 0x0000, 0x0000, 0x0002, /*F957*/
  0xFB41, 0x0020, 0x0002, 0x83F1, 0x0000, 0x0000, 0x0002, /*F958*/
  0xFB41, 0x0020, 0x0002, 0x9675, 0x0000, 0x0000, 0x0002, /*F959*/
  0xFB41, 0x0020, 0x0002, 0x8B80, 0x0000, 0x0000, 0x0002, /*F95A*/
  0xFB40, 0x0020, 0x0002, 0xE2CF, 0x0000, 0x0000, 0x0002, /*F95B*/
  0xFB40, 0x0020, 0x0002, 0xEA02, 0x0000, 0x0000, 0x0002, /*F95C*/
  0xFB41, 0x0020, 0x0002, 0x8AFE, 0x0000, 0x0000, 0x0002, /*F95D*/
  0xFB40, 0x0020, 0x0002, 0xCE39, 0x0000, 0x0000, 0x0002, /*F95E*/
  0xFB40, 0x0020, 0x0002, 0xDBE7, 0x0000, 0x0000, 0x0002, /*F95F*/
  0xFB40, 0x0020, 0x0002, 0xE012, 0x0000, 0x0000, 0x0002, /*F960*/
  0xFB40, 0x0020, 0x0002, 0xF387, 0x0000, 0x0000, 0x0002, /*F961*/
  0xFB40, 0x0020, 0x0002, 0xF570, 0x0000, 0x0000, 0x0002, /*F962*/
  0xFB40, 0x0020, 0x0002, 0xD317, 0x0000, 0x0000, 0x0002, /*F963*/
  0xFB40, 0x0020, 0x0002, 0xF8FB, 0x0000, 0x0000, 0x0002, /*F964*/
  0xFB40, 0x0020, 0x0002, 0xCFBF, 0x0000, 0x0000, 0x0002, /*F965*/
  0xFB40, 0x0020, 0x0002, 0xDFA9, 0x0000, 0x0000, 0x0002, /*F966*/
  0xFB40, 0x0020, 0x0002, 0xCE0D, 0x0000, 0x0000, 0x0002, /*F967*/
  0xFB40, 0x0020, 0x0002, 0xECCC, 0x0000, 0x0000, 0x0002, /*F968*/
  0xFB40, 0x0020, 0x0002, 0xE578, 0x0000, 0x0000, 0x0002, /*F969*/
  0xFB40, 0x0020, 0x0002, 0xFD22, 0x0000, 0x0000, 0x0002, /*F96A*/
  0xFB40, 0x0020, 0x0002, 0xD3C3, 0x0000, 0x0000, 0x0002, /*F96B*/
  0xFB40, 0x0020, 0x0002, 0xD85E, 0x0000, 0x0000, 0x0002, /*F96C*/
  0xFB40, 0x0020, 0x0002, 0xF701, 0x0000, 0x0000, 0x0002, /*F96D*/
  0xFB41, 0x0020, 0x0002, 0x8449, 0x0000, 0x0000, 0x0002, /*F96E*/
  0xFB41, 0x0020, 0x0002, 0x8AAA, 0x0000, 0x0000, 0x0002, /*F96F*/
  0xFB40, 0x0020, 0x0002, 0xEBBA, 0x0000, 0x0000, 0x0002, /*F970*/
  0xFB41, 0x0020, 0x0002, 0x8FB0, 0x0000, 0x0000, 0x0002, /*F971*/
  0xFB40, 0x0020, 0x0002, 0xEC88, 0x0000, 0x0000, 0x0002, /*F972*/
  0xFB40, 0x0020, 0x0002, 0xE2FE, 0x0000, 0x0000, 0x0002, /*F973*/
  0xFB41, 0x0020, 0x0002, 0x82E5, 0x0000, 0x0000, 0x0002, /*F974*/
  0xFB40, 0x0020, 0x0002, 0xE3A0, 0x0000, 0x0000, 0x0002, /*F975*/
  0xFB40, 0x0020, 0x0002, 0xF565, 0x0000, 0x0000, 0x0002, /*F976*/
  0xFB40, 0x0020, 0x0002, 0xCEAE, 0x0000, 0x0000, 0x0002, /*F977*/
  0xFB40, 0x0020, 0x0002, 0xD169, 0x0000, 0x0000, 0x0002, /*F978*/
  0xFB40, 0x0020, 0x0002, 0xD1C9, 0x0000, 0x0000, 0x0002, /*F979*/
  0xFB40, 0x0020, 0x0002, 0xE881, 0x0000, 0x0000, 0x0002, /*F97A*/
  0xFB40, 0x0020, 0x0002, 0xFCE7, 0x0000, 0x0000, 0x0002, /*F97B*/
  0xFB41, 0x0020, 0x0002, 0x826F, 0x0000, 0x0000, 0x0002, /*F97C*/
  0xFB41, 0x0020, 0x0002, 0x8AD2, 0x0000, 0x0000, 0x0002, /*F97D*/
  0xFB41, 0x0020, 0x0002, 0x91CF, 0x0000, 0x0000, 0x0002, /*F97E*/
  0xFB40, 0x0020, 0x0002, 0xD2F5, 0x0000, 0x0000, 0x0002, /*F97F*/
  0xFB40, 0x0020, 0x0002, 0xD442, 0x0000, 0x0000, 0x0002, /*F980*/
  0xFB40, 0x0020, 0x0002, 0xD973, 0x0000, 0x0000, 0x0002, /*F981*/
  0xFB40, 0x0020, 0x0002, 0xDEEC, 0x0000, 0x0000, 0x0002, /*F982*/
  0xFB40, 0x0020, 0x0002, 0xE5C5, 0x0000, 0x0000, 0x0002, /*F983*/
  0xFB40, 0x0020, 0x0002, 0xEFFE, 0x0000, 0x0000, 0x0002, /*F984*/
  0xFB40, 0x0020, 0x0002, 0xF92A, 0x0000, 0x0000, 0x0002, /*F985*/
  0xFB41, 0x0020, 0x0002, 0x95AD, 0x0000, 0x0000, 0x0002, /*F986*/
  0xFB41, 0x0020, 0x0002, 0x9A6A, 0x0000, 0x0000, 0x0002, /*F987*/
  0xFB41, 0x0020, 0x0002, 0x9E97, 0x0000, 0x0000, 0x0002, /*F988*/
  0xFB41, 0x0020, 0x0002, 0x9ECE, 0x0000, 0x0000, 0x0002, /*F989*/
  0xFB40, 0x0020, 0x0002, 0xD29B, 0x0000, 0x0000, 0x0002, /*F98A*/
  0xFB40, 0x0020, 0x0002, 0xE6C6, 0x0000, 0x0000, 0x0002, /*F98B*/
  0xFB40, 0x0020, 0x0002, 0xEB77, 0x0000, 0x0000, 0x0002, /*F98C*/
  0xFB41, 0x0020, 0x0002, 0x8F62, 0x0000, 0x0000, 0x0002, /*F98D*/
  0xFB40, 0x0020, 0x0002, 0xDE74, 0x0000, 0x0000, 0x0002, /*F98E*/
  0xFB40, 0x0020, 0x0002, 0xE190, 0x0000, 0x0000, 0x0002, /*F98F*/
  0xFB40, 0x0020, 0x0002, 0xE200, 0x0000, 0x0000, 0x0002, /*F990*/
  0xFB40, 0x0020, 0x0002, 0xE49A, 0x0000, 0x0000, 0x0002, /*F991*/
  0xFB40, 0x0020, 0x0002, 0xEF23, 0x0000, 0x0000, 0x0002, /*F992*/
  0xFB40, 0x0020, 0x0002, 0xF149, 0x0000, 0x0000, 0x0002, /*F993*/
  0xFB40, 0x0020, 0x0002, 0xF489, 0x0000, 0x0000, 0x0002, /*F994*/
  0xFB40, 0x0020, 0x0002, 0xF9CA, 0x0000, 0x0000, 0x0002, /*F995*/
  0xFB40, 0x0020, 0x0002, 0xFDF4, 0x0000, 0x0000, 0x0002, /*F996*/
  0xFB41, 0x0020, 0x0002, 0x806F, 0x0000, 0x0000, 0x0002, /*F997*/
  0xFB41, 0x0020, 0x0002, 0x8F26, 0x0000, 0x0000, 0x0002, /*F998*/
  0xFB41, 0x0020, 0x0002, 0x84EE, 0x0000, 0x0000, 0x0002, /*F999*/
  0xFB41, 0x0020, 0x0002, 0x9023, 0x0000, 0x0000, 0x0002, /*F99A*/
  0xFB41, 0x0020, 0x0002, 0x934A, 0x0000, 0x0000, 0x0002, /*F99B*/
  0xFB40, 0x0020, 0x0002, 0xD217, 0x0000, 0x0000, 0x0002, /*F99C*/
  0xFB40, 0x0020, 0x0002, 0xD2A3, 0x0000, 0x0000, 0x0002, /*F99D*/
  0xFB40, 0x0020, 0x0002, 0xD4BD, 0x0000, 0x0000, 0x0002, /*F99E*/
  0xFB40, 0x0020, 0x0002, 0xF0C8, 0x0000, 0x0000, 0x0002, /*F99F*/
  0xFB41, 0x0020, 0x0002, 0x88C2, 0x0000, 0x0000, 0x0002, /*F9A0*/
  0xFB41, 0x0020, 0x0002, 0x8AAA, 0x0000, 0x0000, 0x0002, /*F9A1*/
  0xFB40, 0x0020, 0x0002, 0xDEC9, 0x0000, 0x0000, 0x0002, /*F9A2*/
  0xFB40, 0x0020, 0x0002, 0xDFF5, 0x0000, 0x0000, 0x0002, /*F9A3*/
  0xFB40, 0x0020, 0x0002, 0xE37B, 0x0000, 0x0000, 0x0002, /*F9A4*/
  0xFB40, 0x0020, 0x0002, 0xEBAE, 0x0000, 0x0000, 0x0002, /*F9A5*/
  0xFB40, 0x0020, 0x0002, 0xFC3E, 0x0000, 0x0000, 0x0002, /*F9A6*/
  0xFB40, 0x0020, 0x0002, 0xF375, 0x0000, 0x0000, 0x0002, /*F9A7*/
  0xFB40, 0x0020, 0x0002, 0xCEE4, 0x0000, 0x0000, 0x0002, /*F9A8*/
  0xFB40, 0x0020, 0x0002, 0xD6F9, 0x0000, 0x0000, 0x0002, /*F9A9*/
  0xFB40, 0x0020, 0x0002, 0xDBE7, 0x0000, 0x0000, 0x0002, /*F9AA*/
  0xFB40, 0x0020, 0x0002, 0xDDBA, 0x0000, 0x0000, 0x0002, /*F9AB*/
  0xFB40, 0x0020, 0x0002, 0xE01C, 0x0000, 0x0000, 0x0002, /*F9AC*/
  0xFB40, 0x0020, 0x0002, 0xF3B2, 0x0000, 0x0000, 0x0002, /*F9AD*/
  0xFB40, 0x0020, 0x0002, 0xF469, 0x0000, 0x0000, 0x0002, /*F9AE*/
  0xFB40, 0x0020, 0x0002, 0xFF9A, 0x0000, 0x0000, 0x0002, /*F9AF*/
  0xFB41, 0x0020, 0x0002, 0x8046, 0x0000, 0x0000, 0x0002, /*F9B0*/
  0xFB41, 0x0020, 0x0002, 0x9234, 0x0000, 0x0000, 0x0002, /*F9B1*/
  0xFB41, 0x0020, 0x0002, 0x96F6, 0x0000, 0x0000, 0x0002, /*F9B2*/
  0xFB41, 0x0020, 0x0002, 0x9748, 0x0000, 0x0000, 0x0002, /*F9B3*/
  0xFB41, 0x0020, 0x0002, 0x9818, 0x0000, 0x0000, 0x0002, /*F9B4*/
  0xFB40, 0x0020, 0x0002, 0xCF8B, 0x0000, 0x0000, 0x0002, /*F9B5*/
  0xFB40, 0x0020, 0x0002, 0xF9AE, 0x0000, 0x0000, 0x0002, /*F9B6*/
  0xFB41, 0x0020, 0x0002, 0x91B4, 0x0000, 0x0000, 0x0002, /*F9B7*/
  0xFB41, 0x0020, 0x0002, 0x96B8, 0x0000, 0x0000, 0x0002, /*F9B8*/
  0xFB40, 0x0020, 0x0002, 0xE0E1, 0x0000, 0x0000, 0x0002, /*F9B9*/
  0xFB40, 0x0020, 0x0002, 0xCE86, 0x0000, 0x0000, 0x0002, /*F9BA*/
  0xFB40, 0x0020, 0x0002, 0xD0DA, 0x0000, 0x0000, 0x0002, /*F9BB*/
  0xFB40, 0x0020, 0x0002, 0xDBEE, 0x0000, 0x0000, 0x0002, /*F9BC*/
  0xFB40, 0x0020, 0x0002, 0xDC3F, 0x0000, 0x0000, 0x0002, /*F9BD*/
  0xFB40, 0x0020, 0x0002, 0xE599, 0x0000, 0x0000, 0x0002, /*F9BE*/
  0xFB40, 0x0020, 0x0002, 0xEA02, 0x0000, 0x0000, 0x0002, /*F9BF*/
  0xFB40, 0x0020, 0x0002, 0xF1CE, 0x0000, 0x0000, 0x0002, /*F9C0*/
  0xFB40, 0x0020, 0x0002, 0xF642, 0x0000, 0x0000, 0x0002, /*F9C1*/
  0xFB41, 0x0020, 0x0002, 0x84FC, 0x0000, 0x0000, 0x0002, /*F9C2*/
  0xFB41, 0x0020, 0x0002, 0x907C, 0x0000, 0x0000, 0x0002, /*F9C3*/
  0xFB41, 0x0020, 0x0002, 0x9F8D, 0x0000, 0x0000, 0x0002, /*F9C4*/
  0xFB40, 0x0020, 0x0002, 0xE688, 0x0000, 0x0000, 0x0002, /*F9C5*/
  0xFB41, 0x0020, 0x0002, 0x962E, 0x0000, 0x0000, 0x0002, /*F9C6*/
  0xFB40, 0x0020, 0x0002, 0xD289, 0x0000, 0x0000, 0x0002, /*F9C7*/
  0xFB40, 0x0020, 0x0002, 0xE77B, 0x0000, 0x0000, 0x0002, /*F9C8*/
  0xFB40, 0x0020, 0x0002, 0xE7F3, 0x0000, 0x0000, 0x0002, /*F9C9*/
  0xFB40, 0x0020, 0x0002, 0xED41, 0x0000, 0x0000, 0x0002, /*F9CA*/
  0xFB40, 0x0020, 0x0002, 0xEE9C, 0x0000, 0x0000, 0x0002, /*F9CB*/
  0xFB40, 0x0020, 0x0002, 0xF409, 0x0000, 0x0000, 0x0002, /*F9CC*/
  0xFB40, 0x0020, 0x0002, 0xF559, 0x0000, 0x0000, 0x0002, /*F9CD*/
  0xFB40, 0x0020, 0x0002, 0xF86B, 0x0000, 0x0000, 0x0002, /*F9CE*/
  0xFB40, 0x0020, 0x0002, 0xFD10, 0x0000, 0x0000, 0x0002, /*F9CF*/
  0xFB41, 0x0020, 0x0002, 0x985E, 0x0000, 0x0000, 0x0002, /*F9D0*/
  0xFB40, 0x0020, 0x0002, 0xD16D, 0x0000, 0x0000, 0x0002, /*F9D1*/
  0xFB40, 0x0020, 0x0002, 0xE22E, 0x0000, 0x0000, 0x0002, /*F9D2*/
  0xFB41, 0x0020, 0x0002, 0x9678, 0x0000, 0x0000, 0x0002, /*F9D3*/
  0xFB40, 0x0020, 0x0002, 0xD02B, 0x0000, 0x0000, 0x0002, /*F9D4*/
  0xFB40, 0x0020, 0x0002, 0xDD19, 0x0000, 0x0000, 0x0002, /*F9D5*/
  0xFB40, 0x0020, 0x0002, 0xEDEA, 0x0000, 0x0000, 0x0002, /*F9D6*/
  0xFB41, 0x0020, 0x0002, 0x8F2A, 0x0000, 0x0000, 0x0002, /*F9D7*/
  0xFB40, 0x0020, 0x0002, 0xDF8B, 0x0000, 0x0000, 0x0002, /*F9D8*/
  0xFB40, 0x0020, 0x0002, 0xE144, 0x0000, 0x0000, 0x0002, /*F9D9*/
  0xFB40, 0x0020, 0x0002, 0xE817, 0x0000, 0x0000, 0x0002, /*F9DA*/
  0xFB40, 0x0020, 0x0002, 0xF387, 0x0000, 0x0000, 0x0002, /*F9DB*/
  0xFB41, 0x0020, 0x0002, 0x9686, 0x0000, 0x0000, 0x0002, /*F9DC*/
  0xFB40, 0x0020, 0x0002, 0xD229, 0x0000, 0x0000, 0x0002, /*F9DD*/
  0xFB40, 0x0020, 0x0002, 0xD40F, 0x0000, 0x0000, 0x0002, /*F9DE*/
  0xFB40, 0x0020, 0x0002, 0xDC65, 0x0000, 0x0000, 0x0002, /*F9DF*/
  0xFB40, 0x0020, 0x0002, 0xE613, 0x0000, 0x0000, 0x0002, /*F9E0*/
  0xFB40, 0x0020, 0x0002, 0xE74E, 0x0000, 0x0000, 0x0002, /*F9E1*/
  0xFB40, 0x0020, 0x0002, 0xE8A8, 0x0000, 0x0000, 0x0002, /*F9E2*/
  0xFB40, 0x0020, 0x0002, 0xECE5, 0x0000, 0x0000, 0x0002, /*F9E3*/
  0xFB40, 0x0020, 0x0002, 0xF406, 0x0000, 0x0000, 0x0002, /*F9E4*/
  0xFB40, 0x0020, 0x0002, 0xF5E2, 0x0000, 0x0000, 0x0002, /*F9E5*/
  0xFB40, 0x0020, 0x0002, 0xFF79, 0x0000, 0x0000, 0x0002, /*F9E6*/
  0xFB41, 0x0020, 0x0002, 0x88CF, 0x0000, 0x0000, 0x0002, /*F9E7*/
  0xFB41, 0x0020, 0x0002, 0x88E1, 0x0000, 0x0000, 0x0002, /*F9E8*/
  0xFB41, 0x0020, 0x0002, 0x91CC, 0x0000, 0x0000, 0x0002, /*F9E9*/
  0xFB41, 0x0020, 0x0002, 0x96E2, 0x0000, 0x0000, 0x0002, /*F9EA*/
  0xFB40, 0x0020, 0x0002, 0xD33F, 0x0000, 0x0000, 0x0002, /*F9EB*/
  0xFB40, 0x0020, 0x0002, 0xEEBA, 0x0000, 0x0000, 0x0002, /*F9EC*/
  0xFB40, 0x0020, 0x0002, 0xD41D, 0x0000, 0x0000, 0x0002, /*F9ED*/
  0xFB40, 0x0020, 0x0002, 0xF1D0, 0x0000, 0x0000, 0x0002, /*F9EE*/
  0xFB40, 0x0020, 0x0002, 0xF498, 0x0000, 0x0000, 0x0002, /*F9EF*/
  0xFB41, 0x0020, 0x0002, 0x85FA, 0x0000, 0x0000, 0x0002, /*F9F0*/
  0xFB41, 0x0020, 0x0002, 0x96A3, 0x0000, 0x0000, 0x0002, /*F9F1*/
  0xFB41, 0x0020, 0x0002, 0x9C57, 0x0000, 0x0000, 0x0002, /*F9F2*/
  0xFB41, 0x0020, 0x0002, 0x9E9F, 0x0000, 0x0000, 0x0002, /*F9F3*/
  0xFB40, 0x0020, 0x0002, 0xE797, 0x0000, 0x0000, 0x0002, /*F9F4*/
  0xFB40, 0x0020, 0x0002, 0xEDCB, 0x0000, 0x0000, 0x0002, /*F9F5*/
  0xFB41, 0x0020, 0x0002, 0x81E8, 0x0000, 0x0000, 0x0002, /*F9F6*/
  0xFB40, 0x0020, 0x0002, 0xFACB, 0x0000, 0x0000, 0x0002, /*F9F7*/
  0xFB40, 0x0020, 0x0002, 0xFB20, 0x0000, 0x0000, 0x0002, /*F9F8*/
  0xFB40, 0x0020, 0x0002, 0xFC92, 0x0000, 0x0000, 0x0002, /*F9F9*/
  0xFB40, 0x0020, 0x0002, 0xF2C0, 0x0000, 0x0000, 0x0002, /*F9FA*/
  0xFB40, 0x0020, 0x0002, 0xF099, 0x0000, 0x0000, 0x0002, /*F9FB*/
  0xFB41, 0x0020, 0x0002, 0x8B58, 0x0000, 0x0000, 0x0002, /*F9FC*/
  0xFB40, 0x0020, 0x0002, 0xCEC0, 0x0000, 0x0000, 0x0002, /*F9FD*/
  0xFB41, 0x0020, 0x0002, 0x8336, 0x0000, 0x0000, 0x0002, /*F9FE*/
  0xFB40, 0x0020, 0x0002, 0xD23A, 0x0000, 0x0000, 0x0002  /*F9FF*/
};

uint16 uca900_p0FA[]= {
  0xFB40, 0x0020, 0x0002, 0xD207, 0x0000, 0x0000, 0x0002, /*FA00*/
  0xFB40, 0x0020, 0x0002, 0xDEA6, 0x0000, 0x0000, 0x0002, /*FA01*/
  0xFB40, 0x0020, 0x0002, 0xE2D3, 0x0000, 0x0000, 0x0002, /*FA02*/
  0xFB40, 0x0020, 0x0002, 0xFCD6, 0x0000, 0x0000, 0x0002, /*FA03*/
  0xFB40, 0x0020, 0x0002, 0xDB85, 0x0000, 0x0000, 0x0002, /*FA04*/
  0xFB40, 0x0020, 0x0002, 0xED1E, 0x0000, 0x0000, 0x0002, /*FA05*/
  0xFB40, 0x0020, 0x0002, 0xE6B4, 0x0000, 0x0000, 0x0002, /*FA06*/
  0xFB41, 0x0020, 0x0002, 0x8F3B, 0x0000, 0x0000, 0x0002, /*FA07*/
  0xFB41, 0x0020, 0x0002, 0x884C, 0x0000, 0x0000, 0x0002, /*FA08*/
  0xFB41, 0x0020, 0x0002, 0x964D, 0x0000, 0x0000, 0x0002, /*FA09*/
  0xFB41, 0x0020, 0x0002, 0x898B, 0x0000, 0x0000, 0x0002, /*FA0A*/
  0xFB40, 0x0020, 0x0002, 0xDED3, 0x0000, 0x0000, 0x0002, /*FA0B*/
  0xFB40, 0x0020, 0x0002, 0xD140, 0x0000, 0x0000, 0x0002, /*FA0C*/
  0xFB40, 0x0020, 0x0002, 0xD5C0, 0x0000, 0x0000, 0x0002, /*FA0D*/
  0xFB41, 0x0020, 0x0002, 0xFA0E, 0x0000, 0x0000, 0x0002, /*FA0E*/
  0xFB41, 0x0020, 0x0002, 0xFA0F, 0x0000, 0x0000, 0x0002, /*FA0F*/
  0xFB40, 0x0020, 0x0002, 0xD85A, 0x0000, 0x0000, 0x0002, /*FA10*/
  0xFB41, 0x0020, 0x0002, 0xFA11, 0x0000, 0x0000, 0x0002, /*FA11*/
  0xFB40, 0x0020, 0x0002, 0xE674, 0x0000, 0x0000, 0x0002, /*FA12*/
  0xFB41, 0x0020, 0x0002, 0xFA13, 0x0000, 0x0000, 0x0002, /*FA13*/
  0xFB41, 0x0020, 0x0002, 0xFA14, 0x0000, 0x0000, 0x0002, /*FA14*/
  0xFB40, 0x0020, 0x0002, 0xD1DE, 0x0000, 0x0000, 0x0002, /*FA15*/
  0xFB40, 0x0020, 0x0002, 0xF32A, 0x0000, 0x0000, 0x0002, /*FA16*/
  0xFB40, 0x0020, 0x0002, 0xF6CA, 0x0000, 0x0000, 0x0002, /*FA17*/
  0xFB40, 0x0020, 0x0002, 0xF93C, 0x0000, 0x0000, 0x0002, /*FA18*/
  0xFB40, 0x0020, 0x0002, 0xF95E, 0x0000, 0x0000, 0x0002, /*FA19*/
  0xFB40, 0x0020, 0x0002, 0xF965, 0x0000, 0x0000, 0x0002, /*FA1A*/
  0xFB40, 0x0020, 0x0002, 0xF98F, 0x0000, 0x0000, 0x0002, /*FA1B*/
  0xFB41, 0x0020, 0x0002, 0x9756, 0x0000, 0x0000, 0x0002, /*FA1C*/
  0xFB40, 0x0020, 0x0002, 0xFCBE, 0x0000, 0x0000, 0x0002, /*FA1D*/
  0xFB40, 0x0020, 0x0002, 0xFFBD, 0x0000, 0x0000, 0x0002, /*FA1E*/
  0xFB41, 0x0020, 0x0002, 0xFA1F, 0x0000, 0x0000, 0x0002, /*FA1F*/
  0xFB41, 0x0020, 0x0002, 0x8612, 0x0000, 0x0000, 0x0002, /*FA20*/
  0xFB41, 0x0020, 0x0002, 0xFA21, 0x0000, 0x0000, 0x0002, /*FA21*/
  0xFB41, 0x0020, 0x0002, 0x8AF8, 0x0000, 0x0000, 0x0002, /*FA22*/
  0xFB41, 0x0020, 0x0002, 0xFA23, 0x0000, 0x0000, 0x0002, /*FA23*/
  0xFB41, 0x0020, 0x0002, 0xFA24, 0x0000, 0x0000, 0x0002, /*FA24*/
  0xFB41, 0x0020, 0x0002, 0x9038, 0x0000, 0x0000, 0x0002, /*FA25*/
  0xFB41, 0x0020, 0x0002, 0x90FD, 0x0000, 0x0000, 0x0002, /*FA26*/
  0xFB41, 0x0020, 0x0002, 0xFA27, 0x0000, 0x0000, 0x0002, /*FA27*/
  0xFB41, 0x0020, 0x0002, 0xFA28, 0x0000, 0x0000, 0x0002, /*FA28*/
  0xFB41, 0x0020, 0x0002, 0xFA29, 0x0000, 0x0000, 0x0002, /*FA29*/
  0xFB41, 0x0020, 0x0002, 0x98EF, 0x0000, 0x0000, 0x0002, /*FA2A*/
  0xFB41, 0x0020, 0x0002, 0x98FC, 0x0000, 0x0000, 0x0002, /*FA2B*/
  0xFB41, 0x0020, 0x0002, 0x9928, 0x0000, 0x0000, 0x0002, /*FA2C*/
  0xFB41, 0x0020, 0x0002, 0x9DB4, 0x0000, 0x0000, 0x0002, /*FA2D*/
  0xFB41, 0x0020, 0x0002, 0x90DE, 0x0000, 0x0000, 0x0002, /*FA2E*/
  0xFB41, 0x0020, 0x0002, 0x96B7, 0x0000, 0x0000, 0x0002, /*FA2F*/
  0xFB40, 0x0020, 0x0002, 0xCFAE, 0x0000, 0x0000, 0x0002, /*FA30*/
  0xFB40, 0x0020, 0x0002, 0xD0E7, 0x0000, 0x0000, 0x0002, /*FA31*/
  0xFB40, 0x0020, 0x0002, 0xD14D, 0x0000, 0x0000, 0x0002, /*FA32*/
  0xFB40, 0x0020, 0x0002, 0xD2C9, 0x0000, 0x0000, 0x0002, /*FA33*/
  0xFB40, 0x0020, 0x0002, 0xD2E4, 0x0000, 0x0000, 0x0002, /*FA34*/
  0xFB40, 0x0020, 0x0002, 0xD351, 0x0000, 0x0000, 0x0002, /*FA35*/
  0xFB40, 0x0020, 0x0002, 0xD59D, 0x0000, 0x0000, 0x0002, /*FA36*/
  0xFB40, 0x0020, 0x0002, 0xD606, 0x0000, 0x0000, 0x0002, /*FA37*/
  0xFB40, 0x0020, 0x0002, 0xD668, 0x0000, 0x0000, 0x0002, /*FA38*/
  0xFB40, 0x0020, 0x0002, 0xD840, 0x0000, 0x0000, 0x0002, /*FA39*/
  0xFB40, 0x0020, 0x0002, 0xD8A8, 0x0000, 0x0000, 0x0002, /*FA3A*/
  0xFB40, 0x0020, 0x0002, 0xDC64, 0x0000, 0x0000, 0x0002, /*FA3B*/
  0xFB40, 0x0020, 0x0002, 0xDC6E, 0x0000, 0x0000, 0x0002, /*FA3C*/
  0xFB40, 0x0020, 0x0002, 0xE094, 0x0000, 0x0000, 0x0002, /*FA3D*/
  0xFB40, 0x0020, 0x0002, 0xE168, 0x0000, 0x0000, 0x0002, /*FA3E*/
  0xFB40, 0x0020, 0x0002, 0xE18E, 0x0000, 0x0000, 0x0002, /*FA3F*/
  0xFB40, 0x0020, 0x0002, 0xE1F2, 0x0000, 0x0000, 0x0002, /*FA40*/
  0xFB40, 0x0020, 0x0002, 0xE54F, 0x0000, 0x0000, 0x0002, /*FA41*/
  0xFB40, 0x0020, 0x0002, 0xE5E2, 0x0000, 0x0000, 0x0002, /*FA42*/
  0xFB40, 0x0020, 0x0002, 0xE691, 0x0000, 0x0000, 0x0002, /*FA43*/
  0xFB40, 0x0020, 0x0002, 0xE885, 0x0000, 0x0000, 0x0002, /*FA44*/
  0xFB40, 0x0020, 0x0002, 0xED77, 0x0000, 0x0000, 0x0002, /*FA45*/
  0xFB40, 0x0020, 0x0002, 0xEE1A, 0x0000, 0x0000, 0x0002, /*FA46*/
  0xFB40, 0x0020, 0x0002, 0xEF22, 0x0000, 0x0000, 0x0002, /*FA47*/
  0xFB40, 0x0020, 0x0002, 0xF16E, 0x0000, 0x0000, 0x0002, /*FA48*/
  0xFB40, 0x0020, 0x0002, 0xF22B, 0x0000, 0x0000, 0x0002, /*FA49*/
  0xFB40, 0x0020, 0x0002, 0xF422, 0x0000, 0x0000, 0x0002, /*FA4A*/
  0xFB40, 0x0020, 0x0002, 0xF891, 0x0000, 0x0000, 0x0002, /*FA4B*/
  0xFB40, 0x0020, 0x0002, 0xF93E, 0x0000, 0x0000, 0x0002, /*FA4C*/
  0xFB40, 0x0020, 0x0002, 0xF949, 0x0000, 0x0000, 0x0002, /*FA4D*/
  0xFB40, 0x0020, 0x0002, 0xF948, 0x0000, 0x0000, 0x0002, /*FA4E*/
  0xFB40, 0x0020, 0x0002, 0xF950, 0x0000, 0x0000, 0x0002, /*FA4F*/
  0xFB40, 0x0020, 0x0002, 0xF956, 0x0000, 0x0000, 0x0002, /*FA50*/
  0xFB40, 0x0020, 0x0002, 0xF95D, 0x0000, 0x0000, 0x0002, /*FA51*/
  0xFB40, 0x0020, 0x0002, 0xF98D, 0x0000, 0x0000, 0x0002, /*FA52*/
  0xFB40, 0x0020, 0x0002, 0xF98E, 0x0000, 0x0000, 0x0002, /*FA53*/
  0xFB40, 0x0020, 0x0002, 0xFA40, 0x0000, 0x0000, 0x0002, /*FA54*/
  0xFB40, 0x0020, 0x0002, 0xFA81, 0x0000, 0x0000, 0x0002, /*FA55*/
  0xFB40, 0x0020, 0x0002, 0xFBC0, 0x0000, 0x0000, 0x0002, /*FA56*/
  0xFB40, 0x0020, 0x0002, 0xFDF4, 0x0000, 0x0000, 0x0002, /*FA57*/
  0xFB40, 0x0020, 0x0002, 0xFE09, 0x0000, 0x0000, 0x0002, /*FA58*/
  0xFB40, 0x0020, 0x0002, 0xFE41, 0x0000, 0x0000, 0x0002, /*FA59*/
  0xFB40, 0x0020, 0x0002, 0xFF72, 0x0000, 0x0000, 0x0002, /*FA5A*/
  0xFB41, 0x0020, 0x0002, 0x8005, 0x0000, 0x0000, 0x0002, /*FA5B*/
  0xFB41, 0x0020, 0x0002, 0x81ED, 0x0000, 0x0000, 0x0002, /*FA5C*/
  0xFB41, 0x0020, 0x0002, 0x8279, 0x0000, 0x0000, 0x0002, /*FA5D*/
  0xFB41, 0x0020, 0x0002, 0x8279, 0x0000, 0x0000, 0x0002, /*FA5E*/
  0xFB41, 0x0020, 0x0002, 0x8457, 0x0000, 0x0000, 0x0002, /*FA5F*/
  0xFB41, 0x0020, 0x0002, 0x8910, 0x0000, 0x0000, 0x0002, /*FA60*/
  0xFB41, 0x0020, 0x0002, 0x8996, 0x0000, 0x0000, 0x0002, /*FA61*/
  0xFB41, 0x0020, 0x0002, 0x8B01, 0x0000, 0x0000, 0x0002, /*FA62*/
  0xFB41, 0x0020, 0x0002, 0x8B39, 0x0000, 0x0000, 0x0002, /*FA63*/
  0xFB41, 0x0020, 0x0002, 0x8CD3, 0x0000, 0x0000, 0x0002, /*FA64*/
  0xFB41, 0x0020, 0x0002, 0x8D08, 0x0000, 0x0000, 0x0002, /*FA65*/
  0xFB41, 0x0020, 0x0002, 0x8FB6, 0x0000, 0x0000, 0x0002, /*FA66*/
  0xFB41, 0x0020, 0x0002, 0x9038, 0x0000, 0x0000, 0x0002, /*FA67*/
  0xFB41, 0x0020, 0x0002, 0x96E3, 0x0000, 0x0000, 0x0002, /*FA68*/
  0xFB41, 0x0020, 0x0002, 0x97FF, 0x0000, 0x0000, 0x0002, /*FA69*/
  0xFB41, 0x0020, 0x0002, 0x983B, 0x0000, 0x0000, 0x0002, /*FA6A*/
  0xFB40, 0x0020, 0x0002, 0xE075, 0x0000, 0x0000, 0x0002, /*FA6B*/
  0xFB84, 0x0020, 0x0002, 0xC2EE, 0x0000, 0x0000, 0x0002, /*FA6C*/
  0xFB41, 0x0020, 0x0002, 0x8218, 0x0000, 0x0000, 0x0002, /*FA6D*/
  0xFBC1, 0x0020, 0x0002, 0xFA6E, 0x0000, 0x0000, 0x0002, /*FA6E*/
  0xFBC1, 0x0020, 0x0002, 0xFA6F, 0x0000, 0x0000, 0x0002, /*FA6F*/
  0xFB40, 0x0020, 0x0002, 0xCE26, 0x0000, 0x0000, 0x0002, /*FA70*/
  0xFB40, 0x0020, 0x0002, 0xD1B5, 0x0000, 0x0000, 0x0002, /*FA71*/
  0xFB40, 0x0020, 0x0002, 0xD168, 0x0000, 0x0000, 0x0002, /*FA72*/
  0xFB40, 0x0020, 0x0002, 0xCF80, 0x0000, 0x0000, 0x0002, /*FA73*/
  0xFB40, 0x0020, 0x0002, 0xD145, 0x0000, 0x0000, 0x0002, /*FA74*/
  0xFB40, 0x0020, 0x0002, 0xD180, 0x0000, 0x0000, 0x0002, /*FA75*/
  0xFB40, 0x0020, 0x0002, 0xD2C7, 0x0000, 0x0000, 0x0002, /*FA76*/
  0xFB40, 0x0020, 0x0002, 0xD2FA, 0x0000, 0x0000, 0x0002, /*FA77*/
  0xFB40, 0x0020, 0x0002, 0xD59D, 0x0000, 0x0000, 0x0002, /*FA78*/
  0xFB40, 0x0020, 0x0002, 0xD555, 0x0000, 0x0000, 0x0002, /*FA79*/
  0xFB40, 0x0020, 0x0002, 0xD599, 0x0000, 0x0000, 0x0002, /*FA7A*/
  0xFB40, 0x0020, 0x0002, 0xD5E2, 0x0000, 0x0000, 0x0002, /*FA7B*/
  0xFB40, 0x0020, 0x0002, 0xD85A, 0x0000, 0x0000, 0x0002, /*FA7C*/
  0xFB40, 0x0020, 0x0002, 0xD8B3, 0x0000, 0x0000, 0x0002, /*FA7D*/
  0xFB40, 0x0020, 0x0002, 0xD944, 0x0000, 0x0000, 0x0002, /*FA7E*/
  0xFB40, 0x0020, 0x0002, 0xD954, 0x0000, 0x0000, 0x0002, /*FA7F*/
  0xFB40, 0x0020, 0x0002, 0xDA62, 0x0000, 0x0000, 0x0002, /*FA80*/
  0xFB40, 0x0020, 0x0002, 0xDB28, 0x0000, 0x0000, 0x0002, /*FA81*/
  0xFB40, 0x0020, 0x0002, 0xDED2, 0x0000, 0x0000, 0x0002, /*FA82*/
  0xFB40, 0x0020, 0x0002, 0xDED9, 0x0000, 0x0000, 0x0002, /*FA83*/
  0xFB40, 0x0020, 0x0002, 0xDF69, 0x0000, 0x0000, 0x0002, /*FA84*/
  0xFB40, 0x0020, 0x0002, 0xDFAD, 0x0000, 0x0000, 0x0002, /*FA85*/
  0xFB40, 0x0020, 0x0002, 0xE0D8, 0x0000, 0x0000, 0x0002, /*FA86*/
  0xFB40, 0x0020, 0x0002, 0xE14E, 0x0000, 0x0000, 0x0002, /*FA87*/
  0xFB40, 0x0020, 0x0002, 0xE108, 0x0000, 0x0000, 0x0002, /*FA88*/
  0xFB40, 0x0020, 0x0002, 0xE18E, 0x0000, 0x0000, 0x0002, /*FA89*/
  0xFB40, 0x0020, 0x0002, 0xE160, 0x0000, 0x0000, 0x0002, /*FA8A*/
  0xFB40, 0x0020, 0x0002, 0xE1F2, 0x0000, 0x0000, 0x0002, /*FA8B*/
  0xFB40, 0x0020, 0x0002, 0xE234, 0x0000, 0x0000, 0x0002, /*FA8C*/
  0xFB40, 0x0020, 0x0002, 0xE3C4, 0x0000, 0x0000, 0x0002, /*FA8D*/
  0xFB40, 0x0020, 0x0002, 0xE41C, 0x0000, 0x0000, 0x0002, /*FA8E*/
  0xFB40, 0x0020, 0x0002, 0xE452, 0x0000, 0x0000, 0x0002, /*FA8F*/
  0xFB40, 0x0020, 0x0002, 0xE556, 0x0000, 0x0000, 0x0002, /*FA90*/
  0xFB40, 0x0020, 0x0002, 0xE674, 0x0000, 0x0000, 0x0002, /*FA91*/
  0xFB40, 0x0020, 0x0002, 0xE717, 0x0000, 0x0000, 0x0002, /*FA92*/
  0xFB40, 0x0020, 0x0002, 0xE71B, 0x0000, 0x0000, 0x0002, /*FA93*/
  0xFB40, 0x0020, 0x0002, 0xE756, 0x0000, 0x0000, 0x0002, /*FA94*/
  0xFB40, 0x0020, 0x0002, 0xEB79, 0x0000, 0x0000, 0x0002, /*FA95*/
  0xFB40, 0x0020, 0x0002, 0xEBBA, 0x0000, 0x0000, 0x0002, /*FA96*/
  0xFB40, 0x0020, 0x0002, 0xED41, 0x0000, 0x0000, 0x0002, /*FA97*/
  0xFB40, 0x0020, 0x0002, 0xEEDB, 0x0000, 0x0000, 0x0002, /*FA98*/
  0xFB40, 0x0020, 0x0002, 0xEECB, 0x0000, 0x0000, 0x0002, /*FA99*/
  0xFB40, 0x0020, 0x0002, 0xEF22, 0x0000, 0x0000, 0x0002, /*FA9A*/
  0xFB40, 0x0020, 0x0002, 0xF01E, 0x0000, 0x0000, 0x0002, /*FA9B*/
  0xFB40, 0x0020, 0x0002, 0xF16E, 0x0000, 0x0000, 0x0002, /*FA9C*/
  0xFB40, 0x0020, 0x0002, 0xF7A7, 0x0000, 0x0000, 0x0002, /*FA9D*/
  0xFB40, 0x0020, 0x0002, 0xF235, 0x0000, 0x0000, 0x0002, /*FA9E*/
  0xFB40, 0x0020, 0x0002, 0xF2AF, 0x0000, 0x0000, 0x0002, /*FA9F*/
  0xFB40, 0x0020, 0x0002, 0xF32A, 0x0000, 0x0000, 0x0002, /*FAA0*/
  0xFB40, 0x0020, 0x0002, 0xF471, 0x0000, 0x0000, 0x0002, /*FAA1*/
  0xFB40, 0x0020, 0x0002, 0xF506, 0x0000, 0x0000, 0x0002, /*FAA2*/
  0xFB40, 0x0020, 0x0002, 0xF53B, 0x0000, 0x0000, 0x0002, /*FAA3*/
  0xFB40, 0x0020, 0x0002, 0xF61D, 0x0000, 0x0000, 0x0002, /*FAA4*/
  0xFB40, 0x0020, 0x0002, 0xF61F, 0x0000, 0x0000, 0x0002, /*FAA5*/
  0xFB40, 0x0020, 0x0002, 0xF6CA, 0x0000, 0x0000, 0x0002, /*FAA6*/
  0xFB40, 0x0020, 0x0002, 0xF6DB, 0x0000, 0x0000, 0x0002, /*FAA7*/
  0xFB40, 0x0020, 0x0002, 0xF6F4, 0x0000, 0x0000, 0x0002, /*FAA8*/
  0xFB40, 0x0020, 0x0002, 0xF74A, 0x0000, 0x0000, 0x0002, /*FAA9*/
  0xFB40, 0x0020, 0x0002, 0xF740, 0x0000, 0x0000, 0x0002, /*FAAA*/
  0xFB40, 0x0020, 0x0002, 0xF8CC, 0x0000, 0x0000, 0x0002, /*FAAB*/
  0xFB40, 0x0020, 0x0002, 0xFAB1, 0x0000, 0x0000, 0x0002, /*FAAC*/
  0xFB40, 0x0020, 0x0002, 0xFBC0, 0x0000, 0x0000, 0x0002, /*FAAD*/
  0xFB40, 0x0020, 0x0002, 0xFC7B, 0x0000, 0x0000, 0x0002, /*FAAE*/
  0xFB40, 0x0020, 0x0002, 0xFD5B, 0x0000, 0x0000, 0x0002, /*FAAF*/
  0xFB40, 0x0020, 0x0002, 0xFDF4, 0x0000, 0x0000, 0x0002, /*FAB0*/
  0xFB40, 0x0020, 0x0002, 0xFF3E, 0x0000, 0x0000, 0x0002, /*FAB1*/
  0xFB41, 0x0020, 0x0002, 0x8005, 0x0000, 0x0000, 0x0002, /*FAB2*/
  0xFB41, 0x0020, 0x0002, 0x8352, 0x0000, 0x0000, 0x0002, /*FAB3*/
  0xFB41, 0x0020, 0x0002, 0x83EF, 0x0000, 0x0000, 0x0002, /*FAB4*/
  0xFB41, 0x0020, 0x0002, 0x8779, 0x0000, 0x0000, 0x0002, /*FAB5*/
  0xFB41, 0x0020, 0x0002, 0x8941, 0x0000, 0x0000, 0x0002, /*FAB6*/
  0xFB41, 0x0020, 0x0002, 0x8986, 0x0000, 0x0000, 0x0002, /*FAB7*/
  0xFB41, 0x0020, 0x0002, 0x8996, 0x0000, 0x0000, 0x0002, /*FAB8*/
  0xFB41, 0x0020, 0x0002, 0x8ABF, 0x0000, 0x0000, 0x0002, /*FAB9*/
  0xFB41, 0x0020, 0x0002, 0x8AF8, 0x0000, 0x0000, 0x0002, /*FABA*/
  0xFB41, 0x0020, 0x0002, 0x8ACB, 0x0000, 0x0000, 0x0002, /*FABB*/
  0xFB41, 0x0020, 0x0002, 0x8B01, 0x0000, 0x0000, 0x0002, /*FABC*/
  0xFB41, 0x0020, 0x0002, 0x8AFE, 0x0000, 0x0000, 0x0002, /*FABD*/
  0xFB41, 0x0020, 0x0002, 0x8AED, 0x0000, 0x0000, 0x0002, /*FABE*/
  0xFB41, 0x0020, 0x0002, 0x8B39, 0x0000, 0x0000, 0x0002, /*FABF*/
  0xFB41, 0x0020, 0x0002, 0x8B8A, 0x0000, 0x0000, 0x0002, /*FAC0*/
  0xFB41, 0x0020, 0x0002, 0x8D08, 0x0000, 0x0000, 0x0002, /*FAC1*/
  0xFB41, 0x0020, 0x0002, 0x8F38, 0x0000, 0x0000, 0x0002, /*FAC2*/
  0xFB41, 0x0020, 0x0002, 0x9072, 0x0000, 0x0000, 0x0002, /*FAC3*/
  0xFB41, 0x0020, 0x0002, 0x9199, 0x0000, 0x0000, 0x0002, /*FAC4*/
  0xFB41, 0x0020, 0x0002, 0x9276, 0x0000, 0x0000, 0x0002, /*FAC5*/
  0xFB41, 0x0020, 0x0002, 0x967C, 0x0000, 0x0000, 0x0002, /*FAC6*/
  0xFB41, 0x0020, 0x0002, 0x96E3, 0x0000, 0x0000, 0x0002, /*FAC7*/
  0xFB41, 0x0020, 0x0002, 0x9756, 0x0000, 0x0000, 0x0002, /*FAC8*/
  0xFB41, 0x0020, 0x0002, 0x97DB, 0x0000, 0x0000, 0x0002, /*FAC9*/
  0xFB41, 0x0020, 0x0002, 0x97FF, 0x0000, 0x0000, 0x0002, /*FACA*/
  0xFB41, 0x0020, 0x0002, 0x980B, 0x0000, 0x0000, 0x0002, /*FACB*/
  0xFB41, 0x0020, 0x0002, 0x983B, 0x0000, 0x0000, 0x0002, /*FACC*/
  0xFB41, 0x0020, 0x0002, 0x9B12, 0x0000, 0x0000, 0x0002, /*FACD*/
  0xFB41, 0x0020, 0x0002, 0x9F9C, 0x0000, 0x0000, 0x0002, /*FACE*/
  0xFB84, 0x0020, 0x0002, 0xA84A, 0x0000, 0x0000, 0x0002, /*FACF*/
  0xFB84, 0x0020, 0x0002, 0xA844, 0x0000, 0x0000, 0x0002, /*FAD0*/
  0xFB84, 0x0020, 0x0002, 0xB3D5, 0x0000, 0x0000, 0x0002, /*FAD1*/
  0xFB80, 0x0020, 0x0002, 0xBB9D, 0x0000, 0x0000, 0x0002, /*FAD2*/
  0xFB80, 0x0020, 0x0002, 0xC018, 0x0000, 0x0000, 0x0002, /*FAD3*/
  0xFB80, 0x0020, 0x0002, 0xC039, 0x0000, 0x0000, 0x0002, /*FAD4*/
  0xFB84, 0x0020, 0x0002, 0xD249, 0x0000, 0x0000, 0x0002, /*FAD5*/
  0xFB84, 0x0020, 0x0002, 0xDCD0, 0x0000, 0x0000, 0x0002, /*FAD6*/
  0xFB84, 0x0020, 0x0002, 0xFED3, 0x0000, 0x0000, 0x0002, /*FAD7*/
  0xFB41, 0x0020, 0x0002, 0x9F43, 0x0000, 0x0000, 0x0002, /*FAD8*/
  0xFB41, 0x0020, 0x0002, 0x9F8E, 0x0000, 0x0000, 0x0002, /*FAD9*/
  0xFBC1, 0x0020, 0x0002, 0xFADA, 0x0000, 0x0000, 0x0002, /*FADA*/
  0xFBC1, 0x0020, 0x0002, 0xFADB, 0x0000, 0x0000, 0x0002, /*FADB*/
  0xFBC1, 0x0020, 0x0002, 0xFADC, 0x0000, 0x0000, 0x0002, /*FADC*/
  0xFBC1, 0x0020, 0x0002, 0xFADD, 0x0000, 0x0000, 0x0002, /*FADD*/
  0xFBC1, 0x0020, 0x0002, 0xFADE, 0x0000, 0x0000, 0x0002, /*FADE*/
  0xFBC1, 0x0020, 0x0002, 0xFADF, 0x0000, 0x0000, 0x0002, /*FADF*/
  0xFBC1, 0x0020, 0x0002, 0xFAE0, 0x0000, 0x0000, 0x0002, /*FAE0*/
  0xFBC1, 0x0020, 0x0002, 0xFAE1, 0x0000, 0x0000, 0x0002, /*FAE1*/
  0xFBC1, 0x0020, 0x0002, 0xFAE2, 0x0000, 0x0000, 0x0002, /*FAE2*/
  0xFBC1, 0x0020, 0x0002, 0xFAE3, 0x0000, 0x0000, 0x0002, /*FAE3*/
  0xFBC1, 0x0020, 0x0002, 0xFAE4, 0x0000, 0x0000, 0x0002, /*FAE4*/
  0xFBC1, 0x0020, 0x0002, 0xFAE5, 0x0000, 0x0000, 0x0002, /*FAE5*/
  0xFBC1, 0x0020, 0x0002, 0xFAE6, 0x0000, 0x0000, 0x0002, /*FAE6*/
  0xFBC1, 0x0020, 0x0002, 0xFAE7, 0x0000, 0x0000, 0x0002, /*FAE7*/
  0xFBC1, 0x0020, 0x0002, 0xFAE8, 0x0000, 0x0000, 0x0002, /*FAE8*/
  0xFBC1, 0x0020, 0x0002, 0xFAE9, 0x0000, 0x0000, 0x0002, /*FAE9*/
  0xFBC1, 0x0020, 0x0002, 0xFAEA, 0x0000, 0x0000, 0x0002, /*FAEA*/
  0xFBC1, 0x0020, 0x0002, 0xFAEB, 0x0000, 0x0000, 0x0002, /*FAEB*/
  0xFBC1, 0x0020, 0x0002, 0xFAEC, 0x0000, 0x0000, 0x0002, /*FAEC*/
  0xFBC1, 0x0020, 0x0002, 0xFAED, 0x0000, 0x0000, 0x0002, /*FAED*/
  0xFBC1, 0x0020, 0x0002, 0xFAEE, 0x0000, 0x0000, 0x0002, /*FAEE*/
  0xFBC1, 0x0020, 0x0002, 0xFAEF, 0x0000, 0x0000, 0x0002, /*FAEF*/
  0xFBC1, 0x0020, 0x0002, 0xFAF0, 0x0000, 0x0000, 0x0002, /*FAF0*/
  0xFBC1, 0x0020, 0x0002, 0xFAF1, 0x0000, 0x0000, 0x0002, /*FAF1*/
  0xFBC1, 0x0020, 0x0002, 0xFAF2, 0x0000, 0x0000, 0x0002, /*FAF2*/
  0xFBC1, 0x0020, 0x0002, 0xFAF3, 0x0000, 0x0000, 0x0002, /*FAF3*/
  0xFBC1, 0x0020, 0x0002, 0xFAF4, 0x0000, 0x0000, 0x0002, /*FAF4*/
  0xFBC1, 0x0020, 0x0002, 0xFAF5, 0x0000, 0x0000, 0x0002, /*FAF5*/
  0xFBC1, 0x0020, 0x0002, 0xFAF6, 0x0000, 0x0000, 0x0002, /*FAF6*/
  0xFBC1, 0x0020, 0x0002, 0xFAF7, 0x0000, 0x0000, 0x0002, /*FAF7*/
  0xFBC1, 0x0020, 0x0002, 0xFAF8, 0x0000, 0x0000, 0x0002, /*FAF8*/
  0xFBC1, 0x0020, 0x0002, 0xFAF9, 0x0000, 0x0000, 0x0002, /*FAF9*/
  0xFBC1, 0x0020, 0x0002, 0xFAFA, 0x0000, 0x0000, 0x0002, /*FAFA*/
  0xFBC1, 0x0020, 0x0002, 0xFAFB, 0x0000, 0x0000, 0x0002, /*FAFB*/
  0xFBC1, 0x0020, 0x0002, 0xFAFC, 0x0000, 0x0000, 0x0002, /*FAFC*/
  0xFBC1, 0x0020, 0x0002, 0xFAFD, 0x0000, 0x0000, 0x0002, /*FAFD*/
  0xFBC1, 0x0020, 0x0002, 0xFAFE, 0x0000, 0x0000, 0x0002, /*FAFE*/
  0xFBC1, 0x0020, 0x0002, 0xFAFF, 0x0000, 0x0000, 0x0002  /*FAFF*/
};

uint16 uca900_p0FB[]= {
  0x1CE5, 0x0020, 0x0004, 0x1CE5, 0x0020, /*FB00*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CE5, 0x0020, 0x0004, 0x1D32, 0x0020, /*FB01*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CE5, 0x0020, 0x0004, 0x1D77, 0x0020, /*FB02*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1CE5, 0x0020, 0x0004, 0x1CE5, 0x0020, /*FB03*/
  0x0004, 0x1D32, 0x0020, 0x0004, 0x0003,
  0x1CE5, 0x0020, 0x0004, 0x1CE5, 0x0020, /*FB04*/
  0x0004, 0x1D77, 0x0020, 0x0004, 0x0003,
  0x1E71, 0x0020, 0x0004, 0x0000, 0x0111, /*FB05*/
  0x0004, 0x1E95, 0x0020, 0x0004, 0x0003,
  0x1E71, 0x0020, 0x0004, 0x1E95, 0x0020, /*FB06*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB07, 0x0000, /*FB07*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB08, 0x0000, /*FB08*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB09, 0x0000, /*FB09*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB0A, 0x0000, /*FB0A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB0B, 0x0000, /*FB0B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB0C, 0x0000, /*FB0C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB0D, 0x0000, /*FB0D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB0E, 0x0000, /*FB0E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB0F, 0x0000, /*FB0F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB10, 0x0000, /*FB10*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB11, 0x0000, /*FB11*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB12, 0x0000, /*FB12*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22A3, 0x0020, 0x0004, 0x22A5, 0x0020, /*FB13*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22A3, 0x0020, 0x0004, 0x2294, 0x0020, /*FB14*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22A3, 0x0020, 0x0004, 0x229A, 0x0020, /*FB15*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22AD, 0x0020, 0x0004, 0x22A5, 0x0020, /*FB16*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22A3, 0x0020, 0x0004, 0x229C, 0x0020, /*FB17*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB18, 0x0000, /*FB18*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB19, 0x0000, /*FB19*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB1A, 0x0000, /*FB1A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB1B, 0x0000, /*FB1B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB1C, 0x0000, /*FB1C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C0, 0x0020, 0x0002, 0x0000, 0x0056, /*FB1D*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0000, 0x0061, 0x0002, 0x0000, 0x0000, /*FB1E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22C0, 0x0020, 0x0004, 0x22C0, 0x0020, /*FB1F*/
  0x0004, 0x0000, 0x0059, 0x0002, 0x0003,
  0x22C6, 0x0020, 0x0005, 0x0000, 0x0000, /*FB20*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22B7, 0x0020, 0x0005, 0x0000, 0x0000, /*FB21*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22BA, 0x0020, 0x0005, 0x0000, 0x0000, /*FB22*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22BB, 0x0020, 0x0005, 0x0000, 0x0000, /*FB23*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22C1, 0x0020, 0x0005, 0x0000, 0x0000, /*FB24*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22C2, 0x0020, 0x0005, 0x0000, 0x0000, /*FB25*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22C3, 0x0020, 0x0005, 0x0000, 0x0000, /*FB26*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22CA, 0x0020, 0x0005, 0x0000, 0x0000, /*FB27*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22CC, 0x0020, 0x0005, 0x0000, 0x0000, /*FB28*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0616, 0x0020, 0x0005, 0x0000, 0x0000, /*FB29*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22CB, 0x0020, 0x0002, 0x0000, 0x005E, /*FB2A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22CB, 0x0020, 0x0002, 0x0000, 0x005D, /*FB2B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22CB, 0x0020, 0x0002, 0x0000, 0x005F, /*FB2C*/
  0x0002, 0x0000, 0x005E, 0x0002, 0x0003,
  0x22CB, 0x0020, 0x0002, 0x0000, 0x005F, /*FB2D*/
  0x0002, 0x0000, 0x005D, 0x0002, 0x0003,
  0x22B7, 0x0020, 0x0002, 0x0000, 0x0059, /*FB2E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22B7, 0x0020, 0x0002, 0x0000, 0x005A, /*FB2F*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22B7, 0x0020, 0x0002, 0x0000, 0x005F, /*FB30*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22B8, 0x0020, 0x0002, 0x0000, 0x005F, /*FB31*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22B9, 0x0020, 0x0002, 0x0000, 0x005F, /*FB32*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22BA, 0x0020, 0x0002, 0x0000, 0x005F, /*FB33*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22BB, 0x0020, 0x0002, 0x0000, 0x005F, /*FB34*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22BC, 0x0020, 0x0002, 0x0000, 0x005F, /*FB35*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22BD, 0x0020, 0x0002, 0x0000, 0x005F, /*FB36*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB37, 0x0000, /*FB37*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22BF, 0x0020, 0x0002, 0x0000, 0x005F, /*FB38*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C0, 0x0020, 0x0002, 0x0000, 0x005F, /*FB39*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C1, 0x0020, 0x0019, 0x0000, 0x005F, /*FB3A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C1, 0x0020, 0x0002, 0x0000, 0x005F, /*FB3B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C2, 0x0020, 0x0002, 0x0000, 0x005F, /*FB3C*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB3D, 0x0000, /*FB3D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C3, 0x0020, 0x0002, 0x0000, 0x005F, /*FB3E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB3F, 0x0000, /*FB3F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C4, 0x0020, 0x0002, 0x0000, 0x005F, /*FB40*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C5, 0x0020, 0x0002, 0x0000, 0x005F, /*FB41*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB42, 0x0000, /*FB42*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C7, 0x0020, 0x0019, 0x0000, 0x005F, /*FB43*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C7, 0x0020, 0x0002, 0x0000, 0x005F, /*FB44*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFB45, 0x0000, /*FB45*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C8, 0x0020, 0x0002, 0x0000, 0x005F, /*FB46*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C9, 0x0020, 0x0002, 0x0000, 0x005F, /*FB47*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22CA, 0x0020, 0x0002, 0x0000, 0x005F, /*FB48*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22CB, 0x0020, 0x0002, 0x0000, 0x005F, /*FB49*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22CC, 0x0020, 0x0002, 0x0000, 0x005F, /*FB4A*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22BC, 0x0020, 0x0002, 0x0000, 0x005B, /*FB4B*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22B8, 0x0020, 0x0002, 0x0000, 0x0060, /*FB4C*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C1, 0x0020, 0x0002, 0x0000, 0x0060, /*FB4D*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22C7, 0x0020, 0x0002, 0x0000, 0x0060, /*FB4E*/
  0x0002, 0x0000, 0x0000, 0x0000, 0x0002,
  0x22B7, 0x0020, 0x0004, 0x22C2, 0x0020, /*FB4F*/
  0x0004, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2301, 0x0020, 0x001A, 0x0000, 0x0000, /*FB50*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2301, 0x0020, 0x0019, 0x0000, 0x0000, /*FB51*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230E, 0x0020, 0x001A, 0x0000, 0x0000, /*FB52*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230E, 0x0020, 0x0019, 0x0000, 0x0000, /*FB53*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230E, 0x0020, 0x0017, 0x0000, 0x0000, /*FB54*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230E, 0x0020, 0x0018, 0x0000, 0x0000, /*FB55*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230F, 0x0020, 0x001A, 0x0000, 0x0000, /*FB56*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230F, 0x0020, 0x0019, 0x0000, 0x0000, /*FB57*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230F, 0x0020, 0x0017, 0x0000, 0x0000, /*FB58*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230F, 0x0020, 0x0018, 0x0000, 0x0000, /*FB59*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2310, 0x0020, 0x001A, 0x0000, 0x0000, /*FB5A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2310, 0x0020, 0x0019, 0x0000, 0x0000, /*FB5B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2310, 0x0020, 0x0017, 0x0000, 0x0000, /*FB5C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2310, 0x0020, 0x0018, 0x0000, 0x0000, /*FB5D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2320, 0x0020, 0x001A, 0x0000, 0x0000, /*FB5E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2320, 0x0020, 0x0019, 0x0000, 0x0000, /*FB5F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2320, 0x0020, 0x0017, 0x0000, 0x0000, /*FB60*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2320, 0x0020, 0x0018, 0x0000, 0x0000, /*FB61*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2323, 0x0020, 0x001A, 0x0000, 0x0000, /*FB62*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2323, 0x0020, 0x0019, 0x0000, 0x0000, /*FB63*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2323, 0x0020, 0x0017, 0x0000, 0x0000, /*FB64*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2323, 0x0020, 0x0018, 0x0000, 0x0000, /*FB65*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231F, 0x0020, 0x001A, 0x0000, 0x0000, /*FB66*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231F, 0x0020, 0x0019, 0x0000, 0x0000, /*FB67*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231F, 0x0020, 0x0017, 0x0000, 0x0000, /*FB68*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231F, 0x0020, 0x0018, 0x0000, 0x0000, /*FB69*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x237B, 0x0020, 0x001A, 0x0000, 0x0000, /*FB6A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x237B, 0x0020, 0x0019, 0x0000, 0x0000, /*FB6B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x237B, 0x0020, 0x0017, 0x0000, 0x0000, /*FB6C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x237B, 0x0020, 0x0018, 0x0000, 0x0000, /*FB6D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x237E, 0x0020, 0x001A, 0x0000, 0x0000, /*FB6E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x237E, 0x0020, 0x0019, 0x0000, 0x0000, /*FB6F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x237E, 0x0020, 0x0017, 0x0000, 0x0000, /*FB70*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x237E, 0x0020, 0x0018, 0x0000, 0x0000, /*FB71*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2327, 0x0020, 0x001A, 0x0000, 0x0000, /*FB72*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2327, 0x0020, 0x0019, 0x0000, 0x0000, /*FB73*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2327, 0x0020, 0x0017, 0x0000, 0x0000, /*FB74*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2327, 0x0020, 0x0018, 0x0000, 0x0000, /*FB75*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2326, 0x0020, 0x001A, 0x0000, 0x0000, /*FB76*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2326, 0x0020, 0x0019, 0x0000, 0x0000, /*FB77*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2326, 0x0020, 0x0017, 0x0000, 0x0000, /*FB78*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2326, 0x0020, 0x0018, 0x0000, 0x0000, /*FB79*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2328, 0x0020, 0x001A, 0x0000, 0x0000, /*FB7A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2328, 0x0020, 0x0019, 0x0000, 0x0000, /*FB7B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2328, 0x0020, 0x0017, 0x0000, 0x0000, /*FB7C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2328, 0x0020, 0x0018, 0x0000, 0x0000, /*FB7D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x232A, 0x0020, 0x001A, 0x0000, 0x0000, /*FB7E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x232A, 0x0020, 0x0019, 0x0000, 0x0000, /*FB7F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x232A, 0x0020, 0x0017, 0x0000, 0x0000, /*FB80*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x232A, 0x0020, 0x0018, 0x0000, 0x0000, /*FB81*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x233E, 0x0020, 0x001A, 0x0000, 0x0000, /*FB82*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x233E, 0x0020, 0x0019, 0x0000, 0x0000, /*FB83*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x233D, 0x0020, 0x001A, 0x0000, 0x0000, /*FB84*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x233D, 0x0020, 0x0019, 0x0000, 0x0000, /*FB85*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2340, 0x0020, 0x001A, 0x0000, 0x0000, /*FB86*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2340, 0x0020, 0x0019, 0x0000, 0x0000, /*FB87*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2339, 0x0020, 0x001A, 0x0000, 0x0000, /*FB88*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2339, 0x0020, 0x0019, 0x0000, 0x0000, /*FB89*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x234F, 0x0020, 0x001A, 0x0000, 0x0000, /*FB8A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x234F, 0x0020, 0x0019, 0x0000, 0x0000, /*FB8B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2348, 0x0020, 0x001A, 0x0000, 0x0000, /*FB8C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2348, 0x0020, 0x0019, 0x0000, 0x0000, /*FB8D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2388, 0x0020, 0x001A, 0x0000, 0x0000, /*FB8E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2388, 0x0020, 0x0019, 0x0000, 0x0000, /*FB8F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2388, 0x0020, 0x0017, 0x0000, 0x0000, /*FB90*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2388, 0x0020, 0x0018, 0x0000, 0x0000, /*FB91*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2390, 0x0020, 0x001A, 0x0000, 0x0000, /*FB92*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2390, 0x0020, 0x0019, 0x0000, 0x0000, /*FB93*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2390, 0x0020, 0x0017, 0x0000, 0x0000, /*FB94*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2390, 0x0020, 0x0018, 0x0000, 0x0000, /*FB95*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2395, 0x0020, 0x001A, 0x0000, 0x0000, /*FB96*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2395, 0x0020, 0x0019, 0x0000, 0x0000, /*FB97*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2395, 0x0020, 0x0017, 0x0000, 0x0000, /*FB98*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2395, 0x0020, 0x0018, 0x0000, 0x0000, /*FB99*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2393, 0x0020, 0x001A, 0x0000, 0x0000, /*FB9A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2393, 0x0020, 0x0019, 0x0000, 0x0000, /*FB9B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2393, 0x0020, 0x0017, 0x0000, 0x0000, /*FB9C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2393, 0x0020, 0x0018, 0x0000, 0x0000, /*FB9D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23A8, 0x0020, 0x001A, 0x0000, 0x0000, /*FB9E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23A8, 0x0020, 0x0019, 0x0000, 0x0000, /*FB9F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23AA, 0x0020, 0x001A, 0x0000, 0x0000, /*FBA0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23AA, 0x0020, 0x0019, 0x0000, 0x0000, /*FBA1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23AA, 0x0020, 0x0017, 0x0000, 0x0000, /*FBA2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23AA, 0x0020, 0x0018, 0x0000, 0x0000, /*FBA3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B6, 0x0020, 0x001A, 0x0000, 0x0083, /*FBA4*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x23B6, 0x0020, 0x0019, 0x0000, 0x0083, /*FBA5*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x23B3, 0x0020, 0x001A, 0x0000, 0x0000, /*FBA6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B3, 0x0020, 0x0019, 0x0000, 0x0000, /*FBA7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B3, 0x0020, 0x0017, 0x0000, 0x0000, /*FBA8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B3, 0x0020, 0x0018, 0x0000, 0x0000, /*FBA9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B2, 0x0020, 0x001A, 0x0000, 0x0000, /*FBAA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B2, 0x0020, 0x0019, 0x0000, 0x0000, /*FBAB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B2, 0x0020, 0x0017, 0x0000, 0x0000, /*FBAC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B2, 0x0020, 0x0018, 0x0000, 0x0000, /*FBAD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23D4, 0x0020, 0x001A, 0x0000, 0x0000, /*FBAE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23D4, 0x0020, 0x0019, 0x0000, 0x0000, /*FBAF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23D4, 0x0020, 0x001A, 0x0000, 0x0083, /*FBB0*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x23D4, 0x0020, 0x0019, 0x0000, 0x0083, /*FBB1*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0502, 0x0020, 0x0002, 0x0000, 0x0000, /*FBB2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0503, 0x0020, 0x0002, 0x0000, 0x0000, /*FBB3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0504, 0x0020, 0x0002, 0x0000, 0x0000, /*FBB4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0505, 0x0020, 0x0002, 0x0000, 0x0000, /*FBB5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0506, 0x0020, 0x0002, 0x0000, 0x0000, /*FBB6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0507, 0x0020, 0x0002, 0x0000, 0x0000, /*FBB7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0508, 0x0020, 0x0002, 0x0000, 0x0000, /*FBB8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0509, 0x0020, 0x0002, 0x0000, 0x0000, /*FBB9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x050A, 0x0020, 0x0002, 0x0000, 0x0000, /*FBBA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x050B, 0x0020, 0x0002, 0x0000, 0x0000, /*FBBB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x050C, 0x0020, 0x0002, 0x0000, 0x0000, /*FBBC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x050D, 0x0020, 0x0002, 0x0000, 0x0000, /*FBBD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x050E, 0x0020, 0x0002, 0x0000, 0x0000, /*FBBE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x050F, 0x0020, 0x0002, 0x0000, 0x0000, /*FBBF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0510, 0x0020, 0x0002, 0x0000, 0x0000, /*FBC0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0511, 0x0020, 0x0002, 0x0000, 0x0000, /*FBC1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC1, 0x0020, 0x0002, 0xFBC2, 0x0000, /*FBC2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBC3, 0x0000, /*FBC3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBC4, 0x0000, /*FBC4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBC5, 0x0000, /*FBC5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBC6, 0x0000, /*FBC6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBC7, 0x0000, /*FBC7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBC8, 0x0000, /*FBC8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBC9, 0x0000, /*FBC9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBCA, 0x0000, /*FBCA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBCB, 0x0000, /*FBCB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBCC, 0x0000, /*FBCC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBCD, 0x0000, /*FBCD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBCE, 0x0000, /*FBCE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBCF, 0x0000, /*FBCF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBD0, 0x0000, /*FBD0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBD1, 0x0000, /*FBD1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFBD2, 0x0000, /*FBD2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x238D, 0x0020, 0x001A, 0x0000, 0x0000, /*FBD3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x238D, 0x0020, 0x0019, 0x0000, 0x0000, /*FBD4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x238D, 0x0020, 0x0017, 0x0000, 0x0000, /*FBD5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x238D, 0x0020, 0x0018, 0x0000, 0x0000, /*FBD6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23BB, 0x0020, 0x001A, 0x0000, 0x0000, /*FBD7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23BB, 0x0020, 0x0019, 0x0000, 0x0000, /*FBD8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23BA, 0x0020, 0x001A, 0x0000, 0x0000, /*FBD9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23BA, 0x0020, 0x0019, 0x0000, 0x0000, /*FBDA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23BC, 0x0020, 0x001A, 0x0000, 0x0000, /*FBDB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23BC, 0x0020, 0x0019, 0x0000, 0x0000, /*FBDC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23BB, 0x0020, 0x001A, 0x22FD, 0x0020, /*FBDD*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x23BF, 0x0020, 0x001A, 0x0000, 0x0000, /*FBDE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23BF, 0x0020, 0x0019, 0x0000, 0x0000, /*FBDF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B9, 0x0020, 0x001A, 0x0000, 0x0000, /*FBE0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B9, 0x0020, 0x0019, 0x0000, 0x0000, /*FBE1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23BD, 0x0020, 0x001A, 0x0000, 0x0000, /*FBE2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23BD, 0x0020, 0x0019, 0x0000, 0x0000, /*FBE3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23CA, 0x0020, 0x001A, 0x0000, 0x0000, /*FBE4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23CA, 0x0020, 0x0019, 0x0000, 0x0000, /*FBE5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23CA, 0x0020, 0x0017, 0x0000, 0x0000, /*FBE6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23CA, 0x0020, 0x0018, 0x0000, 0x0000, /*FBE7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23C5, 0x0020, 0x0017, 0x0000, 0x0000, /*FBE8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23C5, 0x0020, 0x0018, 0x0000, 0x0000, /*FBE9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2307, 0x0020, 0x001A, 0x230B, 0x0020, /*FBEA*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x0019, 0x230B, 0x0020, /*FBEB*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x001A, 0x23B6, 0x0020, /*FBEC*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x0019, 0x23B6, 0x0020, /*FBED*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x001A, 0x23B7, 0x0020, /*FBEE*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x0019, 0x23B7, 0x0020, /*FBEF*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x001A, 0x23BB, 0x0020, /*FBF0*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x0019, 0x23BB, 0x0020, /*FBF1*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x001A, 0x23BA, 0x0020, /*FBF2*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x0019, 0x23BA, 0x0020, /*FBF3*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x001A, 0x23BC, 0x0020, /*FBF4*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x0019, 0x23BC, 0x0020, /*FBF5*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x001A, 0x23CA, 0x0020, /*FBF6*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x0019, 0x23CA, 0x0020, /*FBF7*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x0017, 0x23CA, 0x0020, /*FBF8*/
  0x0017, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x001A, 0x23C5, 0x0020, /*FBF9*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x0019, 0x23C5, 0x0020, /*FBFA*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2307, 0x0020, 0x0017, 0x23C5, 0x0020, /*FBFB*/
  0x0017, 0x0000, 0x0000, 0x0000, 0x0002,
  0x23C7, 0x0020, 0x001A, 0x0000, 0x0000, /*FBFC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23C7, 0x0020, 0x0019, 0x0000, 0x0000, /*FBFD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23C7, 0x0020, 0x0017, 0x0000, 0x0000, /*FBFE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23C7, 0x0020, 0x0018, 0x0000, 0x0000, /*FBFF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p0FC[]= {
  0x2307, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC00*/
  0x2307, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC01*/
  0x2307, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC02*/
  0x2307, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FC03*/
  0x2307, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FC04*/
  0x230D, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC05*/
  0x230D, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC06*/
  0x230D, 0x0020, 0x001A, 0x232D, 0x0020, 0x001A, 0x0002, /*FC07*/
  0x230D, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC08*/
  0x230D, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FC09*/
  0x230D, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FC0A*/
  0x231D, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC0B*/
  0x231D, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC0C*/
  0x231D, 0x0020, 0x001A, 0x232D, 0x0020, 0x001A, 0x0002, /*FC0D*/
  0x231D, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC0E*/
  0x231D, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FC0F*/
  0x231D, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FC10*/
  0x231E, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC11*/
  0x231E, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC12*/
  0x231E, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FC13*/
  0x231E, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FC14*/
  0x2325, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC15*/
  0x2325, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC16*/
  0x232C, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC17*/
  0x232C, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC18*/
  0x232D, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC19*/
  0x232D, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC1A*/
  0x232D, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC1B*/
  0x2359, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC1C*/
  0x2359, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC1D*/
  0x2359, 0x0020, 0x001A, 0x232D, 0x0020, 0x001A, 0x0002, /*FC1E*/
  0x2359, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC1F*/
  0x2364, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC20*/
  0x2364, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC21*/
  0x2365, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC22*/
  0x2365, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC23*/
  0x2365, 0x0020, 0x001A, 0x232D, 0x0020, 0x001A, 0x0002, /*FC24*/
  0x2365, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC25*/
  0x236A, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC26*/
  0x236A, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC27*/
  0x236B, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC28*/
  0x236E, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC29*/
  0x236E, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC2A*/
  0x236F, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC2B*/
  0x236F, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC2C*/
  0x2376, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC2D*/
  0x2376, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC2E*/
  0x2376, 0x0020, 0x001A, 0x232D, 0x0020, 0x001A, 0x0002, /*FC2F*/
  0x2376, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC30*/
  0x2376, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FC31*/
  0x2376, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FC32*/
  0x2382, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC33*/
  0x2382, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC34*/
  0x2382, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FC35*/
  0x2382, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FC36*/
  0x2387, 0x0020, 0x001A, 0x230B, 0x0020, 0x001A, 0x0002, /*FC37*/
  0x2387, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC38*/
  0x2387, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC39*/
  0x2387, 0x0020, 0x001A, 0x232D, 0x0020, 0x001A, 0x0002, /*FC3A*/
  0x2387, 0x0020, 0x001A, 0x239C, 0x0020, 0x001A, 0x0002, /*FC3B*/
  0x2387, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC3C*/
  0x2387, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FC3D*/
  0x2387, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FC3E*/
  0x239C, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC3F*/
  0x239C, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC40*/
  0x239C, 0x0020, 0x001A, 0x232D, 0x0020, 0x001A, 0x0002, /*FC41*/
  0x239C, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC42*/
  0x239C, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FC43*/
  0x239C, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FC44*/
  0x23A3, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC45*/
  0x23A3, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC46*/
  0x23A3, 0x0020, 0x001A, 0x232D, 0x0020, 0x001A, 0x0002, /*FC47*/
  0x23A3, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC48*/
  0x23A3, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FC49*/
  0x23A3, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FC4A*/
  0x23A7, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC4B*/
  0x23A7, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC4C*/
  0x23A7, 0x0020, 0x001A, 0x232D, 0x0020, 0x001A, 0x0002, /*FC4D*/
  0x23A7, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC4E*/
  0x23A7, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FC4F*/
  0x23A7, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FC50*/
  0x23B1, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC51*/
  0x23B1, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC52*/
  0x23B1, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FC53*/
  0x23B1, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FC54*/
  0x23C6, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0002, /*FC55*/
  0x23C6, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0002, /*FC56*/
  0x23C6, 0x0020, 0x001A, 0x232D, 0x0020, 0x001A, 0x0002, /*FC57*/
  0x23C6, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0002, /*FC58*/
  0x23C6, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FC59*/
  0x23C6, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FC5A*/
  0x2338, 0x0020, 0x001A, 0x0000, 0x0098, 0x001A, 0x0002, /*FC5B*/
  0x2346, 0x0020, 0x001A, 0x0000, 0x0098, 0x001A, 0x0002, /*FC5C*/
  0x23C5, 0x0020, 0x001A, 0x0000, 0x0098, 0x001A, 0x0002, /*FC5D*/
  0x0000, 0x0070, 0x001A, 0x0000, 0x0080, 0x001A, 0x0002, /*FC5E*/
  0x0000, 0x0073, 0x001A, 0x0000, 0x0080, 0x001A, 0x0002, /*FC5F*/
  0x0000, 0x0076, 0x001A, 0x0000, 0x0080, 0x001A, 0x0002, /*FC60*/
  0x0000, 0x007A, 0x001A, 0x0000, 0x0080, 0x001A, 0x0002, /*FC61*/
  0x0000, 0x007D, 0x001A, 0x0000, 0x0080, 0x001A, 0x0002, /*FC62*/
  0x0000, 0x0080, 0x001A, 0x0000, 0x0098, 0x001A, 0x0002, /*FC63*/
  0x2307, 0x0020, 0x0019, 0x2346, 0x0020, 0x0019, 0x0002, /*FC64*/
  0x2307, 0x0020, 0x0019, 0x2347, 0x0020, 0x0019, 0x0002, /*FC65*/
  0x2307, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x0002, /*FC66*/
  0x2307, 0x0020, 0x0019, 0x23A7, 0x0020, 0x0019, 0x0002, /*FC67*/
  0x2307, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0002, /*FC68*/
  0x2307, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0002, /*FC69*/
  0x230D, 0x0020, 0x0019, 0x2346, 0x0020, 0x0019, 0x0002, /*FC6A*/
  0x230D, 0x0020, 0x0019, 0x2347, 0x0020, 0x0019, 0x0002, /*FC6B*/
  0x230D, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x0002, /*FC6C*/
  0x230D, 0x0020, 0x0019, 0x23A7, 0x0020, 0x0019, 0x0002, /*FC6D*/
  0x230D, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0002, /*FC6E*/
  0x230D, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0002, /*FC6F*/
  0x231D, 0x0020, 0x0019, 0x2346, 0x0020, 0x0019, 0x0002, /*FC70*/
  0x231D, 0x0020, 0x0019, 0x2347, 0x0020, 0x0019, 0x0002, /*FC71*/
  0x231D, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x0002, /*FC72*/
  0x231D, 0x0020, 0x0019, 0x23A7, 0x0020, 0x0019, 0x0002, /*FC73*/
  0x231D, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0002, /*FC74*/
  0x231D, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0002, /*FC75*/
  0x231E, 0x0020, 0x0019, 0x2346, 0x0020, 0x0019, 0x0002, /*FC76*/
  0x231E, 0x0020, 0x0019, 0x2347, 0x0020, 0x0019, 0x0002, /*FC77*/
  0x231E, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x0002, /*FC78*/
  0x231E, 0x0020, 0x0019, 0x23A7, 0x0020, 0x0019, 0x0002, /*FC79*/
  0x231E, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0002, /*FC7A*/
  0x231E, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0002, /*FC7B*/
  0x2376, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0002, /*FC7C*/
  0x2376, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0002, /*FC7D*/
  0x2382, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0002, /*FC7E*/
  0x2382, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0002, /*FC7F*/
  0x2387, 0x0020, 0x0019, 0x230B, 0x0020, 0x0019, 0x0002, /*FC80*/
  0x2387, 0x0020, 0x0019, 0x239C, 0x0020, 0x0019, 0x0002, /*FC81*/
  0x2387, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x0002, /*FC82*/
  0x2387, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0002, /*FC83*/
  0x2387, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0002, /*FC84*/
  0x239C, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x0002, /*FC85*/
  0x239C, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0002, /*FC86*/
  0x239C, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0002, /*FC87*/
  0x23A3, 0x0020, 0x0019, 0x230B, 0x0020, 0x0019, 0x0002, /*FC88*/
  0x23A3, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x0002, /*FC89*/
  0x23A7, 0x0020, 0x0019, 0x2346, 0x0020, 0x0019, 0x0002, /*FC8A*/
  0x23A7, 0x0020, 0x0019, 0x2347, 0x0020, 0x0019, 0x0002, /*FC8B*/
  0x23A7, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x0002, /*FC8C*/
  0x23A7, 0x0020, 0x0019, 0x23A7, 0x0020, 0x0019, 0x0002, /*FC8D*/
  0x23A7, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0002, /*FC8E*/
  0x23A7, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0002, /*FC8F*/
  0x23C5, 0x0020, 0x0019, 0x0000, 0x0098, 0x0019, 0x0002, /*FC90*/
  0x23C6, 0x0020, 0x0019, 0x2346, 0x0020, 0x0019, 0x0002, /*FC91*/
  0x23C6, 0x0020, 0x0019, 0x2347, 0x0020, 0x0019, 0x0002, /*FC92*/
  0x23C6, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x0002, /*FC93*/
  0x23C6, 0x0020, 0x0019, 0x23A7, 0x0020, 0x0019, 0x0002, /*FC94*/
  0x23C6, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0002, /*FC95*/
  0x23C6, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0002, /*FC96*/
  0x2307, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FC97*/
  0x2307, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FC98*/
  0x2307, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0002, /*FC99*/
  0x2307, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FC9A*/
  0x2307, 0x0020, 0x0017, 0x23B1, 0x0020, 0x0017, 0x0002, /*FC9B*/
  0x230D, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FC9C*/
  0x230D, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FC9D*/
  0x230D, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0002, /*FC9E*/
  0x230D, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FC9F*/
  0x230D, 0x0020, 0x0017, 0x23B1, 0x0020, 0x0017, 0x0002, /*FCA0*/
  0x231D, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCA1*/
  0x231D, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCA2*/
  0x231D, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0002, /*FCA3*/
  0x231D, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCA4*/
  0x231D, 0x0020, 0x0017, 0x23B1, 0x0020, 0x0017, 0x0002, /*FCA5*/
  0x231E, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCA6*/
  0x2325, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCA7*/
  0x2325, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCA8*/
  0x232C, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCA9*/
  0x232C, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCAA*/
  0x232D, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCAB*/
  0x232D, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCAC*/
  0x2359, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCAD*/
  0x2359, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCAE*/
  0x2359, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0002, /*FCAF*/
  0x2359, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCB0*/
  0x2364, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCB1*/
  0x2364, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0002, /*FCB2*/
  0x2364, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCB3*/
  0x2365, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCB4*/
  0x2365, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCB5*/
  0x2365, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0002, /*FCB6*/
  0x2365, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCB7*/
  0x236A, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCB8*/
  0x236B, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCB9*/
  0x236E, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCBA*/
  0x236E, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCBB*/
  0x236F, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCBC*/
  0x236F, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCBD*/
  0x2376, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCBE*/
  0x2376, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCBF*/
  0x2376, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0002, /*FCC0*/
  0x2376, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCC1*/
  0x2382, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCC2*/
  0x2382, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCC3*/
  0x2387, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCC4*/
  0x2387, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCC5*/
  0x2387, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0002, /*FCC6*/
  0x2387, 0x0020, 0x0017, 0x239C, 0x0020, 0x0017, 0x0002, /*FCC7*/
  0x2387, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCC8*/
  0x239C, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCC9*/
  0x239C, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCCA*/
  0x239C, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0002, /*FCCB*/
  0x239C, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCCC*/
  0x239C, 0x0020, 0x0017, 0x23B1, 0x0020, 0x0017, 0x0002, /*FCCD*/
  0x23A3, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCCE*/
  0x23A3, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCCF*/
  0x23A3, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0002, /*FCD0*/
  0x23A3, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCD1*/
  0x23A7, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCD2*/
  0x23A7, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCD3*/
  0x23A7, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0002, /*FCD4*/
  0x23A7, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCD5*/
  0x23A7, 0x0020, 0x0017, 0x23B1, 0x0020, 0x0017, 0x0002, /*FCD6*/
  0x23B1, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCD7*/
  0x23B1, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCD8*/
  0x23B1, 0x0020, 0x0017, 0x0000, 0x0098, 0x0017, 0x0002, /*FCD9*/
  0x23C6, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0002, /*FCDA*/
  0x23C6, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0002, /*FCDB*/
  0x23C6, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0002, /*FCDC*/
  0x23C6, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0002, /*FCDD*/
  0x23C6, 0x0020, 0x0017, 0x23B1, 0x0020, 0x0017, 0x0002, /*FCDE*/
  0x2307, 0x0020, 0x0018, 0x23A3, 0x0020, 0x0018, 0x0002, /*FCDF*/
  0x2307, 0x0020, 0x0018, 0x23B1, 0x0020, 0x0018, 0x0002, /*FCE0*/
  0x230D, 0x0020, 0x0018, 0x23A3, 0x0020, 0x0018, 0x0002, /*FCE1*/
  0x230D, 0x0020, 0x0018, 0x23B1, 0x0020, 0x0018, 0x0002, /*FCE2*/
  0x231D, 0x0020, 0x0018, 0x23A3, 0x0020, 0x0018, 0x0002, /*FCE3*/
  0x231D, 0x0020, 0x0018, 0x23B1, 0x0020, 0x0018, 0x0002, /*FCE4*/
  0x231E, 0x0020, 0x0018, 0x23A3, 0x0020, 0x0018, 0x0002, /*FCE5*/
  0x231E, 0x0020, 0x0018, 0x23B1, 0x0020, 0x0018, 0x0002, /*FCE6*/
  0x2359, 0x0020, 0x0018, 0x23A3, 0x0020, 0x0018, 0x0002, /*FCE7*/
  0x2359, 0x0020, 0x0018, 0x23B1, 0x0020, 0x0018, 0x0002, /*FCE8*/
  0x235A, 0x0020, 0x0018, 0x23A3, 0x0020, 0x0018, 0x0002, /*FCE9*/
  0x235A, 0x0020, 0x0018, 0x23B1, 0x0020, 0x0018, 0x0002, /*FCEA*/
  0x2387, 0x0020, 0x0018, 0x239C, 0x0020, 0x0018, 0x0002, /*FCEB*/
  0x2387, 0x0020, 0x0018, 0x23A3, 0x0020, 0x0018, 0x0002, /*FCEC*/
  0x239C, 0x0020, 0x0018, 0x23A3, 0x0020, 0x0018, 0x0002, /*FCED*/
  0x23A7, 0x0020, 0x0018, 0x23A3, 0x0020, 0x0018, 0x0002, /*FCEE*/
  0x23A7, 0x0020, 0x0018, 0x23B1, 0x0020, 0x0018, 0x0002, /*FCEF*/
  0x23C6, 0x0020, 0x0018, 0x23A3, 0x0020, 0x0018, 0x0002, /*FCF0*/
  0x23C6, 0x0020, 0x0018, 0x23B1, 0x0020, 0x0018, 0x0002, /*FCF1*/
  0x0000, 0x0076, 0x0018, 0x0000, 0x0080, 0x0018, 0x0002, /*FCF2*/
  0x0000, 0x007A, 0x0018, 0x0000, 0x0080, 0x0018, 0x0002, /*FCF3*/
  0x0000, 0x007D, 0x0018, 0x0000, 0x0080, 0x0018, 0x0002, /*FCF4*/
  0x236A, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FCF5*/
  0x236A, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FCF6*/
  0x236E, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FCF7*/
  0x236E, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FCF8*/
  0x236F, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FCF9*/
  0x236F, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FCFA*/
  0x2359, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FCFB*/
  0x2359, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FCFC*/
  0x235A, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002, /*FCFD*/
  0x235A, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0002, /*FCFE*/
  0x232C, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0002  /*FCFF*/
};

uint16 uca900_p0FD[]= {
  0x232C, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0000, /*FD00*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2325, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0000, /*FD01*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2325, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0000, /*FD02*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x232D, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0000, /*FD03*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x232D, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0000, /*FD04*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2364, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0000, /*FD05*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2364, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0000, /*FD06*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2365, 0x0020, 0x001A, 0x23C5, 0x0020, 0x001A, 0x0000, /*FD07*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2365, 0x0020, 0x001A, 0x23C6, 0x0020, 0x001A, 0x0000, /*FD08*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x0000, /*FD09*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x0000, /*FD0A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x001A, 0x232D, 0x0020, 0x001A, 0x0000, /*FD0B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0000, /*FD0C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x001A, 0x2346, 0x0020, 0x001A, 0x0000, /*FD0D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2359, 0x0020, 0x001A, 0x2346, 0x0020, 0x001A, 0x0000, /*FD0E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2364, 0x0020, 0x001A, 0x2346, 0x0020, 0x001A, 0x0000, /*FD0F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2365, 0x0020, 0x001A, 0x2346, 0x0020, 0x001A, 0x0000, /*FD10*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x236A, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0000, /*FD11*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x236A, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0000, /*FD12*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x236E, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0000, /*FD13*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x236E, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0000, /*FD14*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x236F, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0000, /*FD15*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x236F, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0000, /*FD16*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2359, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0000, /*FD17*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2359, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0000, /*FD18*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0000, /*FD19*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0000, /*FD1A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x232C, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0000, /*FD1B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x232C, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0000, /*FD1C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2325, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0000, /*FD1D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2325, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0000, /*FD1E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x232D, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0000, /*FD1F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x232D, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0000, /*FD20*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2364, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0000, /*FD21*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2364, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0000, /*FD22*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2365, 0x0020, 0x0019, 0x23C5, 0x0020, 0x0019, 0x0000, /*FD23*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2365, 0x0020, 0x0019, 0x23C6, 0x0020, 0x0019, 0x0000, /*FD24*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x0000, /*FD25*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x0000, /*FD26*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0019, 0x232D, 0x0020, 0x0019, 0x0000, /*FD27*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x0000, /*FD28*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0019, 0x2346, 0x0020, 0x0019, 0x0000, /*FD29*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2359, 0x0020, 0x0019, 0x2346, 0x0020, 0x0019, 0x0000, /*FD2A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2364, 0x0020, 0x0019, 0x2346, 0x0020, 0x0019, 0x0000, /*FD2B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2365, 0x0020, 0x0019, 0x2346, 0x0020, 0x0019, 0x0000, /*FD2C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x0000, /*FD2D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x0000, /*FD2E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x0000, /*FD2F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0000, /*FD30*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2359, 0x0020, 0x0017, 0x23B1, 0x0020, 0x0017, 0x0000, /*FD31*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0017, 0x23B1, 0x0020, 0x0017, 0x0000, /*FD32*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x236A, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x0000, /*FD33*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2359, 0x0020, 0x0018, 0x2325, 0x0020, 0x0018, 0x0000, /*FD34*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2359, 0x0020, 0x0018, 0x232C, 0x0020, 0x0018, 0x0000, /*FD35*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2359, 0x0020, 0x0018, 0x232D, 0x0020, 0x0018, 0x0000, /*FD36*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0018, 0x2325, 0x0020, 0x0018, 0x0000, /*FD37*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0018, 0x232C, 0x0020, 0x0018, 0x0000, /*FD38*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x235A, 0x0020, 0x0018, 0x232D, 0x0020, 0x0018, 0x0000, /*FD39*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x236A, 0x0020, 0x0018, 0x23A3, 0x0020, 0x0018, 0x0000, /*FD3A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x236B, 0x0020, 0x0018, 0x23A3, 0x0020, 0x0018, 0x0000, /*FD3B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x230B, 0x0020, 0x0019, 0x0000, 0x006D, 0x0019, 0x0000, /*FD3C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x230B, 0x0020, 0x001A, 0x0000, 0x006D, 0x001A, 0x0000, /*FD3D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0381, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*FD3E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0382, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*FD3F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC1, 0x0020, 0x0002, 0xFD40, 0x0000, 0x0000, 0x0000, /*FD40*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD41, 0x0000, 0x0000, 0x0000, /*FD41*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD42, 0x0000, 0x0000, 0x0000, /*FD42*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD43, 0x0000, 0x0000, 0x0000, /*FD43*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD44, 0x0000, 0x0000, 0x0000, /*FD44*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD45, 0x0000, 0x0000, 0x0000, /*FD45*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD46, 0x0000, 0x0000, 0x0000, /*FD46*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD47, 0x0000, 0x0000, 0x0000, /*FD47*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD48, 0x0000, 0x0000, 0x0000, /*FD48*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD49, 0x0000, 0x0000, 0x0000, /*FD49*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD4A, 0x0000, 0x0000, 0x0000, /*FD4A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD4B, 0x0000, 0x0000, 0x0000, /*FD4B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD4C, 0x0000, 0x0000, 0x0000, /*FD4C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD4D, 0x0000, 0x0000, 0x0000, /*FD4D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD4E, 0x0000, 0x0000, 0x0000, /*FD4E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD4F, 0x0000, 0x0000, 0x0000, /*FD4F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x231D, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x23A3, /*FD50*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x2325, /*FD51*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x2325, /*FD52*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x23A3, /*FD53*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x23A3, /*FD54*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x2325, /*FD55*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x232C, /*FD56*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x232D, /*FD57*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2325, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x232C, /*FD58*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2325, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x232C, /*FD59*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x232C, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FD5A*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x232C, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C5, /*FD5B*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2359, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x2325, /*FD5C*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2359, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x232C, /*FD5D*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2359, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23C5, /*FD5E*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2359, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x232C, /*FD5F*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2359, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x232C, /*FD60*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2359, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x2325, /*FD61*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2359, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23A3, /*FD62*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2359, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x23A3, /*FD63*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2364, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x232C, /*FD64*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2364, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x232C, /*FD65*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2364, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23A3, /*FD66*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x235A, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23A3, /*FD67*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x235A, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x23A3, /*FD68*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x235A, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23C6, /*FD69*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x235A, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x232D, /*FD6A*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x235A, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x232D, /*FD6B*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x235A, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23A3, /*FD6C*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x235A, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x23A3, /*FD6D*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2365, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C5, /*FD6E*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2365, 0x0020, 0x0019, 0x232D, 0x0020, 0x0019, 0x23A3, /*FD6F*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2365, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x23A3, /*FD70*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236A, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x232C, /*FD71*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236A, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x232C, /*FD72*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236A, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x23A3, /*FD73*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236A, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FD74*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236E, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23A3, /*FD75*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236E, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23A3, /*FD76*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236E, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x23A3, /*FD77*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236E, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C5, /*FD78*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236F, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23A3, /*FD79*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236F, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FD7A*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236F, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C5, /*FD7B*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2376, 0x0020, 0x0019, 0x232D, 0x0020, 0x0019, 0x23A3, /*FD7C*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2376, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x23A3, /*FD7D*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2382, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x232C, /*FD7E*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2382, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23A3, /*FD7F*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23A3, /*FD80*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C6, /*FD81*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C5, /*FD82*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x2325, /*FD83*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x2325, /*FD84*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0019, 0x232D, 0x0020, 0x0019, 0x23A3, /*FD85*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x23A3, /*FD86*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x232C, /*FD87*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x232C, /*FD88*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A3, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x2325, /*FD89*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A3, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x23A3, /*FD8A*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A3, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C6, /*FD8B*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A3, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x232C, /*FD8C*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A3, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x23A3, /*FD8D*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A3, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x2325, /*FD8E*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A3, 0x0020, 0x0017, 0x232D, 0x0020, 0x0017, 0x23A3, /*FD8F*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC1, 0x0020, 0x0002, 0xFD90, 0x0000, 0x0000, 0x0000, /*FD90*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFD91, 0x0000, 0x0000, 0x0000, /*FD91*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x23A3, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x232D, /*FD92*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23B1, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x2325, /*FD93*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23B1, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x23A3, /*FD94*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A7, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x23A3, /*FD95*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A7, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C5, /*FD96*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A7, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23A3, /*FD97*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A7, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x23A3, /*FD98*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A7, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23C5, /*FD99*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A7, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FD9A*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A7, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C5, /*FD9B*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23C6, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23A3, /*FD9C*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23C6, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x23A3, /*FD9D*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x230D, 0x0020, 0x0019, 0x232D, 0x0020, 0x0019, 0x23C6, /*FD9E*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23C6, /*FD9F*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23C5, /*FDA0*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0019, 0x232D, 0x0020, 0x0019, 0x23C6, /*FDA1*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0019, 0x232D, 0x0020, 0x0019, 0x23C5, /*FDA2*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FDA3*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x231D, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C5, /*FDA4*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2325, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FDA5*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2325, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C5, /*FDA6*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2325, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C5, /*FDA7*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2359, 0x0020, 0x0019, 0x232D, 0x0020, 0x0019, 0x23C5, /*FDA8*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2364, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C6, /*FDA9*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x235A, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C6, /*FDAA*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2365, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C6, /*FDAB*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23C6, /*FDAC*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FDAD*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23C6, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C6, /*FDAE*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23C6, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23C6, /*FDAF*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23C6, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FDB0*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A3, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FDB1*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2382, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FDB2*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A7, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C6, /*FDB3*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2382, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x232C, /*FDB4*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0017, 0x232C, 0x0020, 0x0017, 0x23A3, /*FDB5*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236E, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FDB6*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2387, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FDB7*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A7, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x232C, /*FDB8*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A3, 0x0020, 0x0019, 0x232D, 0x0020, 0x0019, 0x23C6, /*FDB9*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x23A3, /*FDBA*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2387, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23A3, /*FDBB*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x239C, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23A3, /*FDBC*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A7, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x232C, /*FDBD*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2325, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C6, /*FDBE*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x232C, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23C6, /*FDBF*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A3, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23C6, /*FDC0*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2376, 0x0020, 0x0019, 0x23A3, 0x0020, 0x0019, 0x23C6, /*FDC1*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x230D, 0x0020, 0x0019, 0x232C, 0x0020, 0x0019, 0x23C6, /*FDC2*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2387, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x23A3, /*FDC3*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x236E, 0x0020, 0x0017, 0x2325, 0x0020, 0x0017, 0x23A3, /*FDC4*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2364, 0x0020, 0x0017, 0x23A3, 0x0020, 0x0017, 0x23A3, /*FDC5*/
  0x0020, 0x0017, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2359, 0x0020, 0x0019, 0x232D, 0x0020, 0x0019, 0x23C6, /*FDC6*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x23A7, 0x0020, 0x0019, 0x2325, 0x0020, 0x0019, 0x23C6, /*FDC7*/
  0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC1, 0x0020, 0x0002, 0xFDC8, 0x0000, 0x0000, 0x0000, /*FDC8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDC9, 0x0000, 0x0000, 0x0000, /*FDC9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDCA, 0x0000, 0x0000, 0x0000, /*FDCA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDCB, 0x0000, 0x0000, 0x0000, /*FDCB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDCC, 0x0000, 0x0000, 0x0000, /*FDCC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDCD, 0x0000, 0x0000, 0x0000, /*FDCD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDCE, 0x0000, 0x0000, 0x0000, /*FDCE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDCF, 0x0000, 0x0000, 0x0000, /*FDCF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDD0, 0x0000, 0x0000, 0x0000, /*FDD0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDD1, 0x0000, 0x0000, 0x0000, /*FDD1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDD2, 0x0000, 0x0000, 0x0000, /*FDD2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDD3, 0x0000, 0x0000, 0x0000, /*FDD3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDD4, 0x0000, 0x0000, 0x0000, /*FDD4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDD5, 0x0000, 0x0000, 0x0000, /*FDD5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDD6, 0x0000, 0x0000, 0x0000, /*FDD6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDD7, 0x0000, 0x0000, 0x0000, /*FDD7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDD8, 0x0000, 0x0000, 0x0000, /*FDD8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDD9, 0x0000, 0x0000, 0x0000, /*FDD9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDDA, 0x0000, 0x0000, 0x0000, /*FDDA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDDB, 0x0000, 0x0000, 0x0000, /*FDDB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDDC, 0x0000, 0x0000, 0x0000, /*FDDC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDDD, 0x0000, 0x0000, 0x0000, /*FDDD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDDE, 0x0000, 0x0000, 0x0000, /*FDDE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDDF, 0x0000, 0x0000, 0x0000, /*FDDF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDE0, 0x0000, 0x0000, 0x0000, /*FDE0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDE1, 0x0000, 0x0000, 0x0000, /*FDE1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDE2, 0x0000, 0x0000, 0x0000, /*FDE2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDE3, 0x0000, 0x0000, 0x0000, /*FDE3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDE4, 0x0000, 0x0000, 0x0000, /*FDE4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDE5, 0x0000, 0x0000, 0x0000, /*FDE5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDE6, 0x0000, 0x0000, 0x0000, /*FDE6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDE7, 0x0000, 0x0000, 0x0000, /*FDE7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDE8, 0x0000, 0x0000, 0x0000, /*FDE8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDE9, 0x0000, 0x0000, 0x0000, /*FDE9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDEA, 0x0000, 0x0000, 0x0000, /*FDEA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDEB, 0x0000, 0x0000, 0x0000, /*FDEB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDEC, 0x0000, 0x0000, 0x0000, /*FDEC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDED, 0x0000, 0x0000, 0x0000, /*FDED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDEE, 0x0000, 0x0000, 0x0000, /*FDEE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDEF, 0x0000, 0x0000, 0x0000, /*FDEF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x2364, 0x0020, 0x001A, 0x239C, 0x0020, 0x001A, 0x23D4, /*FDF0*/
  0x0020, 0x001A, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2382, 0x0020, 0x001A, 0x239C, 0x0020, 0x001A, 0x23D4, /*FDF1*/
  0x0020, 0x001A, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x230B, 0x0020, 0x001A, 0x239C, 0x0020, 0x001A, 0x239C, /*FDF2*/
  0x0020, 0x001A, 0x23B1, 0x0020, 0x001A, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x230B, 0x0020, 0x001A, 0x2387, 0x0020, 0x001A, 0x230D, /*FDF3*/
  0x0020, 0x001A, 0x2346, 0x0020, 0x001A, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x23A3, 0x0020, 0x001A, 0x232C, 0x0020, 0x001A, 0x23A3, /*FDF4*/
  0x0020, 0x001A, 0x2337, 0x0020, 0x001A, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x2364, 0x0020, 0x001A, 0x239C, 0x0020, 0x001A, 0x236E, /*FDF5*/
  0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x2346, 0x0020, 0x001A, 0x2359, 0x0020, 0x001A, 0x23B7, /*FDF6*/
  0x0020, 0x001A, 0x239C, 0x0020, 0x001A, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x236E, 0x0020, 0x001A, 0x239C, 0x0020, 0x001A, 0x23C6, /*FDF7*/
  0x0020, 0x001A, 0x23B1, 0x0020, 0x001A, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x23B7, 0x0020, 0x001A, 0x2359, 0x0020, 0x001A, 0x239C, /*FDF8*/
  0x0020, 0x001A, 0x23A3, 0x0020, 0x001A, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x2364, 0x0020, 0x001A, 0x239C, 0x0020, 0x001A, 0x23C5, /*FDF9*/
  0x0020, 0x001A, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x2364, 0x0020, 0x001A, 0x239C, 0x0020, 0x001A, 0x23C5, /*FDFA*/
  0x0020, 0x001A, 0x0209, 0x0020, 0x001A, 0x230B,
  0x0020, 0x001A, 0x239C, 0x0020, 0x001A, 0x239C,
  0x0020, 0x001A, 0x23B1, 0x0020, 0x001A, 0x0008,
  0x2325, 0x0020, 0x001A, 0x239C, 0x0020, 0x001A, 0x0209, /*FDFB*/
  0x0020, 0x001A, 0x2325, 0x0020, 0x001A, 0x239C,
  0x0020, 0x001A, 0x230B, 0x0020, 0x001A, 0x239C,
  0x0020, 0x001A, 0x23B1, 0x0020, 0x001A, 0x0008,
  0x2346, 0x0020, 0x001A, 0x23C7, 0x0020, 0x001A, 0x230B, /*FDFC*/
  0x0020, 0x001A, 0x239C, 0x0020, 0x001A, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x0501, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*FDFD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC1, 0x0020, 0x0002, 0xFDFE, 0x0000, 0x0000, 0x0000, /*FDFE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFDFF, 0x0000, 0x0000, 0x0000, /*FDFF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002
};

uint16 uca900_p0FE[]= {
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE00*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE01*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE02*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE03*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE04*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE05*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE06*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE07*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE08*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE09*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE0A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE0B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE0C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE0D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE0E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE0F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0222, 0x0020, 0x0016, 0x0000, 0x0000, /*FE10*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0231, 0x0020, 0x0016, 0x0000, 0x0000, /*FE11*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x028A, 0x0020, 0x0016, 0x0000, 0x0000, /*FE12*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0239, 0x0020, 0x0016, 0x0000, 0x0000, /*FE13*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0234, 0x0020, 0x0016, 0x0000, 0x0000, /*FE14*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0260, 0x0020, 0x0016, 0x0000, 0x0000, /*FE15*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0266, 0x0020, 0x0016, 0x0000, 0x0000, /*FE16*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x037B, 0x0020, 0x0016, 0x0000, 0x0000, /*FE17*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x037C, 0x0020, 0x0016, 0x0000, 0x0000, /*FE18*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0277, 0x0020, 0x0016, 0x0277, 0x0020, /*FE19*/
  0x0016, 0x0277, 0x0020, 0x0016, 0x0003,
  0xFBC1, 0x0020, 0x0002, 0xFE1A, 0x0000, /*FE1A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFE1B, 0x0000, /*FE1B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFE1C, 0x0000, /*FE1C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFE1D, 0x0000, /*FE1D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFE1E, 0x0000, /*FE1E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFE1F, 0x0000, /*FE1F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0000, 0x004F, 0x0002, 0x0000, 0x0000, /*FE20*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE21*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x004E, 0x0002, 0x0000, 0x0000, /*FE22*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE23*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE24*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE25*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE26*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, /*FE27*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE28*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x004E, 0x0002, 0x0000, 0x0000, /*FE29*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE2A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE2B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE2C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE2D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0050, 0x0002, 0x0000, 0x0000, /*FE2E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE2F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0277, 0x0020, 0x0016, 0x0277, 0x0020, /*FE30*/
  0x0016, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0216, 0x0020, 0x0016, 0x0000, 0x0000, /*FE31*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0215, 0x0020, 0x0016, 0x0000, 0x0000, /*FE32*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020B, 0x0020, 0x0016, 0x0000, 0x0000, /*FE33*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020B, 0x0020, 0x0016, 0x0000, 0x0000, /*FE34*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0317, 0x0020, 0x0016, 0x0000, 0x0000, /*FE35*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0318, 0x0020, 0x0016, 0x0000, 0x0000, /*FE36*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x031B, 0x0020, 0x0016, 0x0000, 0x0000, /*FE37*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x031C, 0x0020, 0x0016, 0x0000, 0x0000, /*FE38*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0379, 0x0020, 0x0016, 0x0000, 0x0000, /*FE39*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x037A, 0x0020, 0x0016, 0x0000, 0x0000, /*FE3A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0377, 0x0020, 0x0016, 0x0000, 0x0000, /*FE3B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0378, 0x0020, 0x0016, 0x0000, 0x0000, /*FE3C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0371, 0x0020, 0x0016, 0x0000, 0x0000, /*FE3D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0372, 0x0020, 0x0016, 0x0000, 0x0000, /*FE3E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x036F, 0x0020, 0x0016, 0x0000, 0x0000, /*FE3F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0370, 0x0020, 0x0016, 0x0000, 0x0000, /*FE40*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0373, 0x0020, 0x0016, 0x0000, 0x0000, /*FE41*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0374, 0x0020, 0x0016, 0x0000, 0x0000, /*FE42*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0375, 0x0020, 0x0016, 0x0000, 0x0000, /*FE43*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0376, 0x0020, 0x0016, 0x0000, 0x0000, /*FE44*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0232, 0x0020, 0x0002, 0x0000, 0x0000, /*FE45*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0233, 0x0020, 0x0002, 0x0000, 0x0000, /*FE46*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0319, 0x0020, 0x0016, 0x0000, 0x0000, /*FE47*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x031A, 0x0020, 0x0016, 0x0000, 0x0000, /*FE48*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020A, 0x0020, 0x0004, 0x0000, 0x0000, /*FE49*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020A, 0x0020, 0x0004, 0x0000, 0x0000, /*FE4A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020A, 0x0020, 0x0004, 0x0000, 0x0000, /*FE4B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020A, 0x0020, 0x0004, 0x0000, 0x0000, /*FE4C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020B, 0x0020, 0x0004, 0x0000, 0x0000, /*FE4D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020B, 0x0020, 0x0004, 0x0000, 0x0000, /*FE4E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020B, 0x0020, 0x0004, 0x0000, 0x0000, /*FE4F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0222, 0x0020, 0x000F, 0x0000, 0x0000, /*FE50*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0231, 0x0020, 0x000F, 0x0000, 0x0000, /*FE51*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0277, 0x0020, 0x000F, 0x0000, 0x0000, /*FE52*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC1, 0x0020, 0x0002, 0xFE53, 0x0000, /*FE53*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0234, 0x0020, 0x000F, 0x0000, 0x0000, /*FE54*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0239, 0x0020, 0x000F, 0x0000, 0x0000, /*FE55*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0266, 0x0020, 0x000F, 0x0000, 0x0000, /*FE56*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0260, 0x0020, 0x000F, 0x0000, 0x0000, /*FE57*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0216, 0x0020, 0x000F, 0x0000, 0x0000, /*FE58*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0317, 0x0020, 0x000F, 0x0000, 0x0000, /*FE59*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0318, 0x0020, 0x000F, 0x0000, 0x0000, /*FE5A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x031B, 0x0020, 0x000F, 0x0000, 0x0000, /*FE5B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x031C, 0x0020, 0x000F, 0x0000, 0x0000, /*FE5C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0379, 0x0020, 0x000F, 0x0000, 0x0000, /*FE5D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x037A, 0x0020, 0x000F, 0x0000, 0x0000, /*FE5E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0398, 0x0020, 0x000F, 0x0000, 0x0000, /*FE5F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0396, 0x0020, 0x000F, 0x0000, 0x0000, /*FE60*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x038F, 0x0020, 0x000F, 0x0000, 0x0000, /*FE61*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0616, 0x0020, 0x000F, 0x0000, 0x0000, /*FE62*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x020D, 0x0020, 0x000F, 0x0000, 0x0000, /*FE63*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x061A, 0x0020, 0x000F, 0x0000, 0x0000, /*FE64*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x061C, 0x0020, 0x000F, 0x0000, 0x0000, /*FE65*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x061B, 0x0020, 0x000F, 0x0000, 0x0000, /*FE66*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC1, 0x0020, 0x0002, 0xFE67, 0x0000, /*FE67*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0395, 0x0020, 0x000F, 0x0000, 0x0000, /*FE68*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C12, 0x0020, 0x000F, 0x0000, 0x0000, /*FE69*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0399, 0x0020, 0x000F, 0x0000, 0x0000, /*FE6A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x038E, 0x0020, 0x000F, 0x0000, 0x0000, /*FE6B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC1, 0x0020, 0x0002, 0xFE6C, 0x0000, /*FE6C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFE6D, 0x0000, /*FE6D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFE6E, 0x0000, /*FE6E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFE6F, 0x0000, /*FE6F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0000, 0x006D, 0x001A, 0x0000, 0x0000, /*FE70*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x006D, 0x0018, 0x0000, 0x0000, /*FE71*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0070, 0x001A, 0x0000, 0x0000, /*FE72*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FE73*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0073, 0x001A, 0x0000, 0x0000, /*FE74*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC1, 0x0020, 0x0002, 0xFE75, 0x0000, /*FE75*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0000, 0x0076, 0x001A, 0x0000, 0x0000, /*FE76*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0076, 0x0018, 0x0000, 0x0000, /*FE77*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x007A, 0x001A, 0x0000, 0x0000, /*FE78*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x007A, 0x0018, 0x0000, 0x0000, /*FE79*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x007D, 0x001A, 0x0000, 0x0000, /*FE7A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x007D, 0x0018, 0x0000, 0x0000, /*FE7B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0080, 0x001A, 0x0000, 0x0000, /*FE7C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0080, 0x0018, 0x0000, 0x0000, /*FE7D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0081, 0x001A, 0x0000, 0x0000, /*FE7E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0081, 0x0018, 0x0000, 0x0000, /*FE7F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22FD, 0x0020, 0x001A, 0x0000, 0x0000, /*FE80*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22FE, 0x0020, 0x001A, 0x0000, 0x0000, /*FE81*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22FE, 0x0020, 0x0019, 0x0000, 0x0000, /*FE82*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22FF, 0x0020, 0x001A, 0x0000, 0x0000, /*FE83*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x22FF, 0x0020, 0x0019, 0x0000, 0x0000, /*FE84*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2302, 0x0020, 0x001A, 0x0000, 0x0000, /*FE85*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2302, 0x0020, 0x0019, 0x0000, 0x0000, /*FE86*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2303, 0x0020, 0x001A, 0x0000, 0x0000, /*FE87*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2303, 0x0020, 0x0019, 0x0000, 0x0000, /*FE88*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2307, 0x0020, 0x001A, 0x0000, 0x0000, /*FE89*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2307, 0x0020, 0x0019, 0x0000, 0x0000, /*FE8A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2307, 0x0020, 0x0017, 0x0000, 0x0000, /*FE8B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2307, 0x0020, 0x0018, 0x0000, 0x0000, /*FE8C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230B, 0x0020, 0x001A, 0x0000, 0x0000, /*FE8D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230B, 0x0020, 0x0019, 0x0000, 0x0000, /*FE8E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230D, 0x0020, 0x001A, 0x0000, 0x0000, /*FE8F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230D, 0x0020, 0x0019, 0x0000, 0x0000, /*FE90*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230D, 0x0020, 0x0017, 0x0000, 0x0000, /*FE91*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x230D, 0x0020, 0x0018, 0x0000, 0x0000, /*FE92*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231C, 0x0020, 0x001A, 0x0000, 0x0000, /*FE93*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231C, 0x0020, 0x0019, 0x0000, 0x0000, /*FE94*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231D, 0x0020, 0x001A, 0x0000, 0x0000, /*FE95*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231D, 0x0020, 0x0019, 0x0000, 0x0000, /*FE96*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231D, 0x0020, 0x0017, 0x0000, 0x0000, /*FE97*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231D, 0x0020, 0x0018, 0x0000, 0x0000, /*FE98*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231E, 0x0020, 0x001A, 0x0000, 0x0000, /*FE99*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231E, 0x0020, 0x0019, 0x0000, 0x0000, /*FE9A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231E, 0x0020, 0x0017, 0x0000, 0x0000, /*FE9B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x231E, 0x0020, 0x0018, 0x0000, 0x0000, /*FE9C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2325, 0x0020, 0x001A, 0x0000, 0x0000, /*FE9D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2325, 0x0020, 0x0019, 0x0000, 0x0000, /*FE9E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2325, 0x0020, 0x0017, 0x0000, 0x0000, /*FE9F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2325, 0x0020, 0x0018, 0x0000, 0x0000, /*FEA0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x232C, 0x0020, 0x001A, 0x0000, 0x0000, /*FEA1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x232C, 0x0020, 0x0019, 0x0000, 0x0000, /*FEA2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x232C, 0x0020, 0x0017, 0x0000, 0x0000, /*FEA3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x232C, 0x0020, 0x0018, 0x0000, 0x0000, /*FEA4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x232D, 0x0020, 0x001A, 0x0000, 0x0000, /*FEA5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x232D, 0x0020, 0x0019, 0x0000, 0x0000, /*FEA6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x232D, 0x0020, 0x0017, 0x0000, 0x0000, /*FEA7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x232D, 0x0020, 0x0018, 0x0000, 0x0000, /*FEA8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2337, 0x0020, 0x001A, 0x0000, 0x0000, /*FEA9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2337, 0x0020, 0x0019, 0x0000, 0x0000, /*FEAA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2338, 0x0020, 0x001A, 0x0000, 0x0000, /*FEAB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2338, 0x0020, 0x0019, 0x0000, 0x0000, /*FEAC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2346, 0x0020, 0x001A, 0x0000, 0x0000, /*FEAD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2346, 0x0020, 0x0019, 0x0000, 0x0000, /*FEAE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2347, 0x0020, 0x001A, 0x0000, 0x0000, /*FEAF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2347, 0x0020, 0x0019, 0x0000, 0x0000, /*FEB0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2359, 0x0020, 0x001A, 0x0000, 0x0000, /*FEB1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2359, 0x0020, 0x0019, 0x0000, 0x0000, /*FEB2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2359, 0x0020, 0x0017, 0x0000, 0x0000, /*FEB3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2359, 0x0020, 0x0018, 0x0000, 0x0000, /*FEB4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x235A, 0x0020, 0x001A, 0x0000, 0x0000, /*FEB5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x235A, 0x0020, 0x0019, 0x0000, 0x0000, /*FEB6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x235A, 0x0020, 0x0017, 0x0000, 0x0000, /*FEB7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x235A, 0x0020, 0x0018, 0x0000, 0x0000, /*FEB8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2364, 0x0020, 0x001A, 0x0000, 0x0000, /*FEB9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2364, 0x0020, 0x0019, 0x0000, 0x0000, /*FEBA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2364, 0x0020, 0x0017, 0x0000, 0x0000, /*FEBB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2364, 0x0020, 0x0018, 0x0000, 0x0000, /*FEBC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2365, 0x0020, 0x001A, 0x0000, 0x0000, /*FEBD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2365, 0x0020, 0x0019, 0x0000, 0x0000, /*FEBE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2365, 0x0020, 0x0017, 0x0000, 0x0000, /*FEBF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2365, 0x0020, 0x0018, 0x0000, 0x0000, /*FEC0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236A, 0x0020, 0x001A, 0x0000, 0x0000, /*FEC1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236A, 0x0020, 0x0019, 0x0000, 0x0000, /*FEC2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236A, 0x0020, 0x0017, 0x0000, 0x0000, /*FEC3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236A, 0x0020, 0x0018, 0x0000, 0x0000, /*FEC4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236B, 0x0020, 0x001A, 0x0000, 0x0000, /*FEC5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236B, 0x0020, 0x0019, 0x0000, 0x0000, /*FEC6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236B, 0x0020, 0x0017, 0x0000, 0x0000, /*FEC7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236B, 0x0020, 0x0018, 0x0000, 0x0000, /*FEC8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236E, 0x0020, 0x001A, 0x0000, 0x0000, /*FEC9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236E, 0x0020, 0x0019, 0x0000, 0x0000, /*FECA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236E, 0x0020, 0x0017, 0x0000, 0x0000, /*FECB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236E, 0x0020, 0x0018, 0x0000, 0x0000, /*FECC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236F, 0x0020, 0x001A, 0x0000, 0x0000, /*FECD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236F, 0x0020, 0x0019, 0x0000, 0x0000, /*FECE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236F, 0x0020, 0x0017, 0x0000, 0x0000, /*FECF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x236F, 0x0020, 0x0018, 0x0000, 0x0000, /*FED0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2376, 0x0020, 0x001A, 0x0000, 0x0000, /*FED1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2376, 0x0020, 0x0019, 0x0000, 0x0000, /*FED2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2376, 0x0020, 0x0017, 0x0000, 0x0000, /*FED3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2376, 0x0020, 0x0018, 0x0000, 0x0000, /*FED4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2382, 0x0020, 0x001A, 0x0000, 0x0000, /*FED5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2382, 0x0020, 0x0019, 0x0000, 0x0000, /*FED6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2382, 0x0020, 0x0017, 0x0000, 0x0000, /*FED7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2382, 0x0020, 0x0018, 0x0000, 0x0000, /*FED8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2387, 0x0020, 0x001A, 0x0000, 0x0000, /*FED9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2387, 0x0020, 0x0019, 0x0000, 0x0000, /*FEDA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2387, 0x0020, 0x0017, 0x0000, 0x0000, /*FEDB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x2387, 0x0020, 0x0018, 0x0000, 0x0000, /*FEDC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x239C, 0x0020, 0x001A, 0x0000, 0x0000, /*FEDD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x239C, 0x0020, 0x0019, 0x0000, 0x0000, /*FEDE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x239C, 0x0020, 0x0017, 0x0000, 0x0000, /*FEDF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x239C, 0x0020, 0x0018, 0x0000, 0x0000, /*FEE0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23A3, 0x0020, 0x001A, 0x0000, 0x0000, /*FEE1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23A3, 0x0020, 0x0019, 0x0000, 0x0000, /*FEE2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23A3, 0x0020, 0x0017, 0x0000, 0x0000, /*FEE3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23A3, 0x0020, 0x0018, 0x0000, 0x0000, /*FEE4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23A7, 0x0020, 0x001A, 0x0000, 0x0000, /*FEE5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23A7, 0x0020, 0x0019, 0x0000, 0x0000, /*FEE6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23A7, 0x0020, 0x0017, 0x0000, 0x0000, /*FEE7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23A7, 0x0020, 0x0018, 0x0000, 0x0000, /*FEE8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B1, 0x0020, 0x001A, 0x0000, 0x0000, /*FEE9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B1, 0x0020, 0x0019, 0x0000, 0x0000, /*FEEA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B1, 0x0020, 0x0017, 0x0000, 0x0000, /*FEEB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B1, 0x0020, 0x0018, 0x0000, 0x0000, /*FEEC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B7, 0x0020, 0x001A, 0x0000, 0x0000, /*FEED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23B7, 0x0020, 0x0019, 0x0000, 0x0000, /*FEEE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23C5, 0x0020, 0x001A, 0x0000, 0x0000, /*FEEF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23C5, 0x0020, 0x0019, 0x0000, 0x0000, /*FEF0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23C6, 0x0020, 0x001A, 0x0000, 0x0000, /*FEF1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23C6, 0x0020, 0x0019, 0x0000, 0x0000, /*FEF2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23C6, 0x0020, 0x0017, 0x0000, 0x0000, /*FEF3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x23C6, 0x0020, 0x0018, 0x0000, 0x0000, /*FEF4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x239C, 0x0020, 0x001A, 0x22FE, 0x0020, /*FEF5*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x239C, 0x0020, 0x0019, 0x22FE, 0x0020, /*FEF6*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x239C, 0x0020, 0x001A, 0x22FF, 0x0020, /*FEF7*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x239C, 0x0020, 0x0019, 0x22FF, 0x0020, /*FEF8*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x239C, 0x0020, 0x001A, 0x2303, 0x0020, /*FEF9*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x239C, 0x0020, 0x0019, 0x2303, 0x0020, /*FEFA*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0x239C, 0x0020, 0x001A, 0x230B, 0x0020, /*FEFB*/
  0x001A, 0x0000, 0x0000, 0x0000, 0x0002,
  0x239C, 0x0020, 0x0019, 0x230B, 0x0020, /*FEFC*/
  0x0019, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFEFD, 0x0000, /*FEFD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC1, 0x0020, 0x0002, 0xFEFE, 0x0000, /*FEFE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*FEFF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p0FF[]= {
  0xFBC1, 0x0020, 0x0002, 0xFF00, 0x0000, 0x0000, 0x0002, /*FF00*/
  0x0260, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF01*/
  0x030C, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF02*/
  0x0398, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF03*/
  0x1C12, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF04*/
  0x0399, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF05*/
  0x0396, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF06*/
  0x0305, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF07*/
  0x0317, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF08*/
  0x0318, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF09*/
  0x038F, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF0A*/
  0x0616, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF0B*/
  0x0222, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF0C*/
  0x020D, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF0D*/
  0x0277, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF0E*/
  0x0394, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF0F*/
  0x1C3D, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF10*/
  0x1C3E, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF11*/
  0x1C3F, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF12*/
  0x1C40, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF13*/
  0x1C41, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF14*/
  0x1C42, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF15*/
  0x1C43, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF16*/
  0x1C44, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF17*/
  0x1C45, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF18*/
  0x1C46, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF19*/
  0x0239, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF1A*/
  0x0234, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF1B*/
  0x061A, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF1C*/
  0x061B, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF1D*/
  0x061C, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF1E*/
  0x0266, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF1F*/
  0x038E, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF20*/
  0x1C47, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF21*/
  0x1C60, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF22*/
  0x1C7A, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF23*/
  0x1C8F, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF24*/
  0x1CAA, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF25*/
  0x1CE5, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF26*/
  0x1CF4, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF27*/
  0x1D18, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF28*/
  0x1D32, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF29*/
  0x1D4C, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF2A*/
  0x1D65, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF2B*/
  0x1D77, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF2C*/
  0x1DAA, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF2D*/
  0x1DB9, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF2E*/
  0x1DDD, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF2F*/
  0x1E0C, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF30*/
  0x1E21, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF31*/
  0x1E33, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF32*/
  0x1E71, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF33*/
  0x1E95, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF34*/
  0x1EB5, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF35*/
  0x1EE3, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF36*/
  0x1EF5, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF37*/
  0x1EFF, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF38*/
  0x1F0B, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF39*/
  0x1F21, 0x0020, 0x0009, 0x0000, 0x0000, 0x0000, 0x0001, /*FF3A*/
  0x0319, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF3B*/
  0x0395, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF3C*/
  0x031A, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF3D*/
  0x0485, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF3E*/
  0x020B, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF3F*/
  0x0482, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF40*/
  0x1C47, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF41*/
  0x1C60, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF42*/
  0x1C7A, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF43*/
  0x1C8F, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF44*/
  0x1CAA, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF45*/
  0x1CE5, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF46*/
  0x1CF4, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF47*/
  0x1D18, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF48*/
  0x1D32, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF49*/
  0x1D4C, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF4A*/
  0x1D65, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF4B*/
  0x1D77, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF4C*/
  0x1DAA, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF4D*/
  0x1DB9, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF4E*/
  0x1DDD, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF4F*/
  0x1E0C, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF50*/
  0x1E21, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF51*/
  0x1E33, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF52*/
  0x1E71, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF53*/
  0x1E95, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF54*/
  0x1EB5, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF55*/
  0x1EE3, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF56*/
  0x1EF5, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF57*/
  0x1EFF, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF58*/
  0x1F0B, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF59*/
  0x1F21, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF5A*/
  0x031B, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF5B*/
  0x061E, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF5C*/
  0x031C, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF5D*/
  0x0620, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF5E*/
  0x032D, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF5F*/
  0x032E, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FF60*/
  0x028A, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF61*/
  0x0373, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF62*/
  0x0374, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF63*/
  0x0231, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF64*/
  0x0221, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF65*/
  0x3D8A, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF66*/
  0x3D5A, 0x0020, 0x0010, 0x0000, 0x0000, 0x0000, 0x0001, /*FF67*/
  0x3D5B, 0x0020, 0x0010, 0x0000, 0x0000, 0x0000, 0x0001, /*FF68*/
  0x3D5C, 0x0020, 0x0010, 0x0000, 0x0000, 0x0000, 0x0001, /*FF69*/
  0x3D5E, 0x0020, 0x0010, 0x0000, 0x0000, 0x0000, 0x0001, /*FF6A*/
  0x3D5F, 0x0020, 0x0010, 0x0000, 0x0000, 0x0000, 0x0001, /*FF6B*/
  0x3D7E, 0x0020, 0x0010, 0x0000, 0x0000, 0x0000, 0x0001, /*FF6C*/
  0x3D7F, 0x0020, 0x0010, 0x0000, 0x0000, 0x0000, 0x0001, /*FF6D*/
  0x3D81, 0x0020, 0x0010, 0x0000, 0x0000, 0x0000, 0x0001, /*FF6E*/
  0x3D6C, 0x0020, 0x0010, 0x0000, 0x0000, 0x0000, 0x0001, /*FF6F*/
  0x1C0E, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF70*/
  0x3D5A, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF71*/
  0x3D5B, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF72*/
  0x3D5C, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF73*/
  0x3D5E, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF74*/
  0x3D5F, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF75*/
  0x3D60, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF76*/
  0x3D61, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF77*/
  0x3D62, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF78*/
  0x3D63, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF79*/
  0x3D64, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF7A*/
  0x3D65, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF7B*/
  0x3D66, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF7C*/
  0x3D67, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF7D*/
  0x3D68, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF7E*/
  0x3D69, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF7F*/
  0x3D6A, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF80*/
  0x3D6B, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF81*/
  0x3D6C, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF82*/
  0x3D6D, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF83*/
  0x3D6E, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF84*/
  0x3D6F, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF85*/
  0x3D70, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF86*/
  0x3D71, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF87*/
  0x3D72, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF88*/
  0x3D73, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF89*/
  0x3D74, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF8A*/
  0x3D75, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF8B*/
  0x3D76, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF8C*/
  0x3D77, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF8D*/
  0x3D78, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF8E*/
  0x3D79, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF8F*/
  0x3D7A, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF90*/
  0x3D7B, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF91*/
  0x3D7C, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF92*/
  0x3D7D, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF93*/
  0x3D7E, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF94*/
  0x3D7F, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF95*/
  0x3D81, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF96*/
  0x3D82, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF97*/
  0x3D83, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF98*/
  0x3D84, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF99*/
  0x3D85, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF9A*/
  0x3D86, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF9B*/
  0x3D87, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF9C*/
  0x3D8B, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF9D*/
  0x0000, 0x0037, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF9E*/
  0x0000, 0x0038, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FF9F*/
  0x3C72, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFA0*/
  0x3BF5, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFA1*/
  0x3BF6, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFA2*/
  0x3CD3, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFA3*/
  0x3BF7, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFA4*/
  0x3CD5, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFA5*/
  0x3CD6, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFA6*/
  0x3BF8, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFA7*/
  0x3BF9, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFA8*/
  0x3BFA, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFA9*/
  0x3CD9, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFAA*/
  0x3CDA, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFAB*/
  0x3CDB, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFAC*/
  0x3CDC, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFAD*/
  0x3CDD, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFAE*/
  0x3CDE, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFAF*/
  0x3C0F, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFB0*/
  0x3BFB, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFB1*/
  0x3BFC, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFB2*/
  0x3BFD, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFB3*/
  0x3C16, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFB4*/
  0x3BFE, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFB5*/
  0x3BFF, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFB6*/
  0x3C00, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFB7*/
  0x3C01, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFB8*/
  0x3C02, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFB9*/
  0x3C03, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFBA*/
  0x3C04, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFBB*/
  0x3C05, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFBC*/
  0x3C06, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFBD*/
  0x3C07, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFBE*/
  0xFBC1, 0x0020, 0x0002, 0xFFBF, 0x0000, 0x0000, 0x0002, /*FFBF*/
  0xFBC1, 0x0020, 0x0002, 0xFFC0, 0x0000, 0x0000, 0x0002, /*FFC0*/
  0xFBC1, 0x0020, 0x0002, 0xFFC1, 0x0000, 0x0000, 0x0002, /*FFC1*/
  0x3C73, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFC2*/
  0x3C74, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFC3*/
  0x3C75, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFC4*/
  0x3C76, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFC5*/
  0x3C77, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFC6*/
  0x3C78, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFC7*/
  0xFBC1, 0x0020, 0x0002, 0xFFC8, 0x0000, 0x0000, 0x0002, /*FFC8*/
  0xFBC1, 0x0020, 0x0002, 0xFFC9, 0x0000, 0x0000, 0x0002, /*FFC9*/
  0x3C79, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFCA*/
  0x3C7A, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFCB*/
  0x3C7B, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFCC*/
  0x3C7C, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFCD*/
  0x3C7D, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFCE*/
  0x3C7E, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFCF*/
  0xFBC1, 0x0020, 0x0002, 0xFFD0, 0x0000, 0x0000, 0x0002, /*FFD0*/
  0xFBC1, 0x0020, 0x0002, 0xFFD1, 0x0000, 0x0000, 0x0002, /*FFD1*/
  0x3C7F, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFD2*/
  0x3C80, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFD3*/
  0x3C81, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFD4*/
  0x3C82, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFD5*/
  0x3C83, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFD6*/
  0x3C84, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFD7*/
  0xFBC1, 0x0020, 0x0002, 0xFFD8, 0x0000, 0x0000, 0x0002, /*FFD8*/
  0xFBC1, 0x0020, 0x0002, 0xFFD9, 0x0000, 0x0000, 0x0002, /*FFD9*/
  0x3C85, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFDA*/
  0x3C86, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFDB*/
  0x3C87, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFDC*/
  0xFBC1, 0x0020, 0x0002, 0xFFDD, 0x0000, 0x0000, 0x0002, /*FFDD*/
  0xFBC1, 0x0020, 0x0002, 0xFFDE, 0x0000, 0x0000, 0x0002, /*FFDE*/
  0xFBC1, 0x0020, 0x0002, 0xFFDF, 0x0000, 0x0000, 0x0002, /*FFDF*/
  0x1C11, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FFE0*/
  0x1C13, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FFE1*/
  0x061D, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FFE2*/
  0x0486, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FFE3*/
  0x061F, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FFE4*/
  0x1C14, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FFE5*/
  0x1C27, 0x0020, 0x0003, 0x0000, 0x0000, 0x0000, 0x0001, /*FFE6*/
  0xFBC1, 0x0020, 0x0002, 0xFFE7, 0x0000, 0x0000, 0x0002, /*FFE7*/
  0x081A, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFE8*/
  0x059C, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFE9*/
  0x059E, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFEA*/
  0x059D, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFEB*/
  0x059F, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFEC*/
  0x08B8, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFED*/
  0x08E3, 0x0020, 0x0012, 0x0000, 0x0000, 0x0000, 0x0001, /*FFEE*/
  0xFBC1, 0x0020, 0x0002, 0xFFEF, 0x0000, 0x0000, 0x0002, /*FFEF*/
  0xFBC1, 0x0020, 0x0002, 0xFFF0, 0x0000, 0x0000, 0x0002, /*FFF0*/
  0xFBC1, 0x0020, 0x0002, 0xFFF1, 0x0000, 0x0000, 0x0002, /*FFF1*/
  0xFBC1, 0x0020, 0x0002, 0xFFF2, 0x0000, 0x0000, 0x0002, /*FFF2*/
  0xFBC1, 0x0020, 0x0002, 0xFFF3, 0x0000, 0x0000, 0x0002, /*FFF3*/
  0xFBC1, 0x0020, 0x0002, 0xFFF4, 0x0000, 0x0000, 0x0002, /*FFF4*/
  0xFBC1, 0x0020, 0x0002, 0xFFF5, 0x0000, 0x0000, 0x0002, /*FFF5*/
  0xFBC1, 0x0020, 0x0002, 0xFFF6, 0x0000, 0x0000, 0x0002, /*FFF6*/
  0xFBC1, 0x0020, 0x0002, 0xFFF7, 0x0000, 0x0000, 0x0002, /*FFF7*/
  0xFBC1, 0x0020, 0x0002, 0xFFF8, 0x0000, 0x0000, 0x0002, /*FFF8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*FFF9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*FFFA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*FFFB*/
  0x1A95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*FFFC*/
  0xFFFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*FFFD*/
  0xFBC1, 0x0020, 0x0002, 0xFFFE, 0x0000, 0x0000, 0x0002, /*FFFE*/
  0xFBC1, 0x0020, 0x0002, 0xFFFF, 0x0000, 0x0000, 0x0002  /*FFFF*/
};

uint16 uca900_p100[]= {
  0x45E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10000*/
  0x45E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10001*/
  0x45E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10002*/
  0x45EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10003*/
  0x45EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10004*/
  0x45EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10005*/
  0x45ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10006*/
  0x45EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10007*/
  0x45EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10008*/
  0x45F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10009*/
  0x45F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1000A*/
  0x45F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1000B*/
  0xFBC2, 0x0020, 0x0002, 0x800C, 0x0000, 0x0000, 0x0002, /*1000C*/
  0x45F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1000D*/
  0x45F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1000E*/
  0x45F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1000F*/
  0x45F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10010*/
  0x45F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10011*/
  0x45F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10012*/
  0x45F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10013*/
  0x45FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10014*/
  0x45FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10015*/
  0x45FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10016*/
  0x45FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10017*/
  0x45FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10018*/
  0x45FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10019*/
  0x4600, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1001A*/
  0x4601, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1001B*/
  0x4602, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1001C*/
  0x4603, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1001D*/
  0x4604, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1001E*/
  0x4605, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1001F*/
  0x4606, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10020*/
  0x4607, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10021*/
  0x4608, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10022*/
  0x4609, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10023*/
  0x460A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10024*/
  0x460B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10025*/
  0x460C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10026*/
  0xFBC2, 0x0020, 0x0002, 0x8027, 0x0000, 0x0000, 0x0002, /*10027*/
  0x460D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10028*/
  0x460E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10029*/
  0x460F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1002A*/
  0x4610, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1002B*/
  0x4611, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1002C*/
  0x4612, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1002D*/
  0x4613, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1002E*/
  0x4614, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1002F*/
  0x4615, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10030*/
  0x4616, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10031*/
  0x4617, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10032*/
  0x4618, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10033*/
  0x4619, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10034*/
  0x461A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10035*/
  0x461B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10036*/
  0x461C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10037*/
  0x461D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10038*/
  0x461E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10039*/
  0x461F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1003A*/
  0xFBC2, 0x0020, 0x0002, 0x803B, 0x0000, 0x0000, 0x0002, /*1003B*/
  0x4620, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1003C*/
  0x4621, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1003D*/
  0xFBC2, 0x0020, 0x0002, 0x803E, 0x0000, 0x0000, 0x0002, /*1003E*/
  0x4622, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1003F*/
  0x4623, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10040*/
  0x4624, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10041*/
  0x4625, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10042*/
  0x4626, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10043*/
  0x4627, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10044*/
  0x4628, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10045*/
  0x4629, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10046*/
  0x462A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10047*/
  0x462B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10048*/
  0x462C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10049*/
  0x462D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1004A*/
  0x462E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1004B*/
  0x462F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1004C*/
  0x4630, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1004D*/
  0xFBC2, 0x0020, 0x0002, 0x804E, 0x0000, 0x0000, 0x0002, /*1004E*/
  0xFBC2, 0x0020, 0x0002, 0x804F, 0x0000, 0x0000, 0x0002, /*1004F*/
  0x4631, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10050*/
  0x4632, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10051*/
  0x4633, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10052*/
  0x4634, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10053*/
  0x4635, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10054*/
  0x4636, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10055*/
  0x4637, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10056*/
  0x4638, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10057*/
  0x4639, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10058*/
  0x463A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10059*/
  0x463B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1005A*/
  0x463C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1005B*/
  0x463D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1005C*/
  0x463E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1005D*/
  0xFBC2, 0x0020, 0x0002, 0x805E, 0x0000, 0x0000, 0x0002, /*1005E*/
  0xFBC2, 0x0020, 0x0002, 0x805F, 0x0000, 0x0000, 0x0002, /*1005F*/
  0xFBC2, 0x0020, 0x0002, 0x8060, 0x0000, 0x0000, 0x0002, /*10060*/
  0xFBC2, 0x0020, 0x0002, 0x8061, 0x0000, 0x0000, 0x0002, /*10061*/
  0xFBC2, 0x0020, 0x0002, 0x8062, 0x0000, 0x0000, 0x0002, /*10062*/
  0xFBC2, 0x0020, 0x0002, 0x8063, 0x0000, 0x0000, 0x0002, /*10063*/
  0xFBC2, 0x0020, 0x0002, 0x8064, 0x0000, 0x0000, 0x0002, /*10064*/
  0xFBC2, 0x0020, 0x0002, 0x8065, 0x0000, 0x0000, 0x0002, /*10065*/
  0xFBC2, 0x0020, 0x0002, 0x8066, 0x0000, 0x0000, 0x0002, /*10066*/
  0xFBC2, 0x0020, 0x0002, 0x8067, 0x0000, 0x0000, 0x0002, /*10067*/
  0xFBC2, 0x0020, 0x0002, 0x8068, 0x0000, 0x0000, 0x0002, /*10068*/
  0xFBC2, 0x0020, 0x0002, 0x8069, 0x0000, 0x0000, 0x0002, /*10069*/
  0xFBC2, 0x0020, 0x0002, 0x806A, 0x0000, 0x0000, 0x0002, /*1006A*/
  0xFBC2, 0x0020, 0x0002, 0x806B, 0x0000, 0x0000, 0x0002, /*1006B*/
  0xFBC2, 0x0020, 0x0002, 0x806C, 0x0000, 0x0000, 0x0002, /*1006C*/
  0xFBC2, 0x0020, 0x0002, 0x806D, 0x0000, 0x0000, 0x0002, /*1006D*/
  0xFBC2, 0x0020, 0x0002, 0x806E, 0x0000, 0x0000, 0x0002, /*1006E*/
  0xFBC2, 0x0020, 0x0002, 0x806F, 0x0000, 0x0000, 0x0002, /*1006F*/
  0xFBC2, 0x0020, 0x0002, 0x8070, 0x0000, 0x0000, 0x0002, /*10070*/
  0xFBC2, 0x0020, 0x0002, 0x8071, 0x0000, 0x0000, 0x0002, /*10071*/
  0xFBC2, 0x0020, 0x0002, 0x8072, 0x0000, 0x0000, 0x0002, /*10072*/
  0xFBC2, 0x0020, 0x0002, 0x8073, 0x0000, 0x0000, 0x0002, /*10073*/
  0xFBC2, 0x0020, 0x0002, 0x8074, 0x0000, 0x0000, 0x0002, /*10074*/
  0xFBC2, 0x0020, 0x0002, 0x8075, 0x0000, 0x0000, 0x0002, /*10075*/
  0xFBC2, 0x0020, 0x0002, 0x8076, 0x0000, 0x0000, 0x0002, /*10076*/
  0xFBC2, 0x0020, 0x0002, 0x8077, 0x0000, 0x0000, 0x0002, /*10077*/
  0xFBC2, 0x0020, 0x0002, 0x8078, 0x0000, 0x0000, 0x0002, /*10078*/
  0xFBC2, 0x0020, 0x0002, 0x8079, 0x0000, 0x0000, 0x0002, /*10079*/
  0xFBC2, 0x0020, 0x0002, 0x807A, 0x0000, 0x0000, 0x0002, /*1007A*/
  0xFBC2, 0x0020, 0x0002, 0x807B, 0x0000, 0x0000, 0x0002, /*1007B*/
  0xFBC2, 0x0020, 0x0002, 0x807C, 0x0000, 0x0000, 0x0002, /*1007C*/
  0xFBC2, 0x0020, 0x0002, 0x807D, 0x0000, 0x0000, 0x0002, /*1007D*/
  0xFBC2, 0x0020, 0x0002, 0x807E, 0x0000, 0x0000, 0x0002, /*1007E*/
  0xFBC2, 0x0020, 0x0002, 0x807F, 0x0000, 0x0000, 0x0002, /*1007F*/
  0x463F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10080*/
  0x4640, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10081*/
  0x4641, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10082*/
  0x4642, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10083*/
  0x4643, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10084*/
  0x4644, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10085*/
  0x4645, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10086*/
  0x4646, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10087*/
  0x4647, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10088*/
  0x4648, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10089*/
  0x4649, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1008A*/
  0x464A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1008B*/
  0x464B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1008C*/
  0x464C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1008D*/
  0x464D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1008E*/
  0x464E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1008F*/
  0x464F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10090*/
  0x4650, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10091*/
  0x4651, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10092*/
  0x4652, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10093*/
  0x4653, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10094*/
  0x4654, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10095*/
  0x4655, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10096*/
  0x4656, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10097*/
  0x4657, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10098*/
  0x4658, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10099*/
  0x4659, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1009A*/
  0x465A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1009B*/
  0x465B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1009C*/
  0x465C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1009D*/
  0x465D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1009E*/
  0x465E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1009F*/
  0x465F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100A0*/
  0x4660, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100A1*/
  0x4661, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100A2*/
  0x4662, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100A3*/
  0x4663, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100A4*/
  0x4664, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100A5*/
  0x4665, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100A6*/
  0x4666, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100A7*/
  0x4667, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100A8*/
  0x4668, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100A9*/
  0x4669, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100AA*/
  0x466A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100AB*/
  0x466B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100AC*/
  0x466C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100AD*/
  0x466D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100AE*/
  0x466E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100AF*/
  0x466F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100B0*/
  0x4670, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100B1*/
  0x4671, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100B2*/
  0x4672, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100B3*/
  0x4673, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100B4*/
  0x4674, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100B5*/
  0x4675, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100B6*/
  0x4676, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100B7*/
  0x4677, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100B8*/
  0x4678, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100B9*/
  0x4679, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100BA*/
  0x467A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100BB*/
  0x467B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100BC*/
  0x467C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100BD*/
  0x467D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100BE*/
  0x467E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100BF*/
  0x467F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100C0*/
  0x4680, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100C1*/
  0x4681, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100C2*/
  0x4682, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100C3*/
  0x4683, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100C4*/
  0x4684, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100C5*/
  0x4685, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100C6*/
  0x4686, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100C7*/
  0x4687, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100C8*/
  0x4688, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100C9*/
  0x4689, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100CA*/
  0x468A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100CB*/
  0x468B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100CC*/
  0x468C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100CD*/
  0x468D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100CE*/
  0x468E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100CF*/
  0x468F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100D0*/
  0x4690, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100D1*/
  0x4691, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100D2*/
  0x4692, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100D3*/
  0x4693, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100D4*/
  0x4694, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100D5*/
  0x4695, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100D6*/
  0x4696, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100D7*/
  0x4697, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100D8*/
  0x4698, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100D9*/
  0x4699, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100DA*/
  0x469A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100DB*/
  0x469B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100DC*/
  0x469C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100DD*/
  0x469D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100DE*/
  0x469E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100DF*/
  0x469F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100E0*/
  0x46A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100E1*/
  0x46A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100E2*/
  0x46A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100E3*/
  0x46A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100E4*/
  0x46A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100E5*/
  0x46A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100E6*/
  0x46A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100E7*/
  0x46A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100E8*/
  0x46A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100E9*/
  0x46A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100EA*/
  0x46AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100EB*/
  0x46AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100EC*/
  0x46AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100ED*/
  0x46AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100EE*/
  0x46AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100EF*/
  0x46AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100F0*/
  0x46B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100F1*/
  0x46B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100F2*/
  0x46B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100F3*/
  0x46B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100F4*/
  0x46B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100F5*/
  0x46B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100F6*/
  0x46B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100F7*/
  0x46B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100F8*/
  0x46B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100F9*/
  0x46B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*100FA*/
  0xFBC2, 0x0020, 0x0002, 0x80FB, 0x0000, 0x0000, 0x0002, /*100FB*/
  0xFBC2, 0x0020, 0x0002, 0x80FC, 0x0000, 0x0000, 0x0002, /*100FC*/
  0xFBC2, 0x0020, 0x0002, 0x80FD, 0x0000, 0x0000, 0x0002, /*100FD*/
  0xFBC2, 0x0020, 0x0002, 0x80FE, 0x0000, 0x0000, 0x0002, /*100FE*/
  0xFBC2, 0x0020, 0x0002, 0x80FF, 0x0000, 0x0000, 0x0002  /*100FF*/
};

uint16 uca900_p101[]= {
  0x02FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10100*/
  0x02FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10101*/
  0x02FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10102*/
  0xFBC2, 0x0020, 0x0002, 0x8103, 0x0000, 0x0000, 0x0002, /*10103*/
  0xFBC2, 0x0020, 0x0002, 0x8104, 0x0000, 0x0000, 0x0002, /*10104*/
  0xFBC2, 0x0020, 0x0002, 0x8105, 0x0000, 0x0000, 0x0002, /*10105*/
  0xFBC2, 0x0020, 0x0002, 0x8106, 0x0000, 0x0000, 0x0002, /*10106*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10107*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10108*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10109*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1010A*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1010B*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1010C*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1010D*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1010E*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1010F*/
  0x1AE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10110*/
  0x1AEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10111*/
  0x1AEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10112*/
  0x1AEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10113*/
  0x1AED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10114*/
  0x1AEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10115*/
  0x1AEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10116*/
  0x1AF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10117*/
  0x1AF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10118*/
  0x1AF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10119*/
  0x1AF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1011A*/
  0x1AF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1011B*/
  0x1AF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1011C*/
  0x1AF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1011D*/
  0x1AF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1011E*/
  0x1AF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1011F*/
  0x1AF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10120*/
  0x1AFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10121*/
  0x1AFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10122*/
  0x1AFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10123*/
  0x1AFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10124*/
  0x1AFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10125*/
  0x1AFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10126*/
  0x1B00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10127*/
  0x1B01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10128*/
  0x1B02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10129*/
  0x1B03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1012A*/
  0x1B04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1012B*/
  0x1B05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1012C*/
  0x1B06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1012D*/
  0x1B07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1012E*/
  0x1B08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1012F*/
  0x1B09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10130*/
  0x1B0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10131*/
  0x1B0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10132*/
  0x1B0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10133*/
  0xFBC2, 0x0020, 0x0002, 0x8134, 0x0000, 0x0000, 0x0002, /*10134*/
  0xFBC2, 0x0020, 0x0002, 0x8135, 0x0000, 0x0000, 0x0002, /*10135*/
  0xFBC2, 0x0020, 0x0002, 0x8136, 0x0000, 0x0000, 0x0002, /*10136*/
  0x0F78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10137*/
  0x0F79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10138*/
  0x0F7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10139*/
  0x0F7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1013A*/
  0x0F7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1013B*/
  0x0F7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1013C*/
  0x0F7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1013D*/
  0x0F7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1013E*/
  0x0F80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1013F*/
  0x1B0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10140*/
  0x1B0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10141*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10142*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10143*/
  0x1B0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10144*/
  0x1B10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10145*/
  0x1B11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10146*/
  0x1B12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10147*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10148*/
  0x1B13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10149*/
  0x1B14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1014A*/
  0x1B15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1014B*/
  0x1B16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1014C*/
  0x1B17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1014D*/
  0x1B18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1014E*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1014F*/
  0x1B19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10150*/
  0x1B1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10151*/
  0x1B1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10152*/
  0x1B1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10153*/
  0x1B1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10154*/
  0x1B1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10155*/
  0x1B1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10156*/
  0x1B20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10157*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10158*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10159*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1015A*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1015B*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1015C*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1015D*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1015E*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1015F*/
  0x1B21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10160*/
  0x1B22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10161*/
  0x1B23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10162*/
  0x1B24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10163*/
  0x1B25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10164*/
  0x1B26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10165*/
  0x1B27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10166*/
  0x1B28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10167*/
  0x1B29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10168*/
  0x1B2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10169*/
  0x1B2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1016A*/
  0x1B2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1016B*/
  0x1B2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1016C*/
  0x1B2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1016D*/
  0x1B2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1016E*/
  0x1B30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1016F*/
  0x1B31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10170*/
  0x1B32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10171*/
  0x1B33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10172*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10173*/
  0x1B34, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10174*/
  0x1B35, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10175*/
  0x1B36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10176*/
  0x1B37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10177*/
  0x1B38, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10178*/
  0x0F81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10179*/
  0x0F82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1017A*/
  0x0F83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1017B*/
  0x0F84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1017C*/
  0x0F85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1017D*/
  0x0F86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1017E*/
  0x0F87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1017F*/
  0x0F88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10180*/
  0x0F89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10181*/
  0x0F8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10182*/
  0x0F8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10183*/
  0x0F8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10184*/
  0x0F8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10185*/
  0x0F8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10186*/
  0x0F8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10187*/
  0x0F90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10188*/
  0x0F91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10189*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1018A*/
  0x1B39, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1018B*/
  0x0F92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1018C*/
  0x0F93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1018D*/
  0x0F94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1018E*/
  0xFBC2, 0x0020, 0x0002, 0x818F, 0x0000, 0x0000, 0x0002, /*1018F*/
  0x0F95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10190*/
  0x0F96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10191*/
  0x0F97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10192*/
  0x0F98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10193*/
  0x0F99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10194*/
  0x0F9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10195*/
  0x0F9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10196*/
  0x0F9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10197*/
  0x0F9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10198*/
  0x0F9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10199*/
  0x0F9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1019A*/
  0x0FA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1019B*/
  0xFBC2, 0x0020, 0x0002, 0x819C, 0x0000, 0x0000, 0x0002, /*1019C*/
  0xFBC2, 0x0020, 0x0002, 0x819D, 0x0000, 0x0000, 0x0002, /*1019D*/
  0xFBC2, 0x0020, 0x0002, 0x819E, 0x0000, 0x0000, 0x0002, /*1019E*/
  0xFBC2, 0x0020, 0x0002, 0x819F, 0x0000, 0x0000, 0x0002, /*1019F*/
  0x0FA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101A0*/
  0xFBC2, 0x0020, 0x0002, 0x81A1, 0x0000, 0x0000, 0x0002, /*101A1*/
  0xFBC2, 0x0020, 0x0002, 0x81A2, 0x0000, 0x0000, 0x0002, /*101A2*/
  0xFBC2, 0x0020, 0x0002, 0x81A3, 0x0000, 0x0000, 0x0002, /*101A3*/
  0xFBC2, 0x0020, 0x0002, 0x81A4, 0x0000, 0x0000, 0x0002, /*101A4*/
  0xFBC2, 0x0020, 0x0002, 0x81A5, 0x0000, 0x0000, 0x0002, /*101A5*/
  0xFBC2, 0x0020, 0x0002, 0x81A6, 0x0000, 0x0000, 0x0002, /*101A6*/
  0xFBC2, 0x0020, 0x0002, 0x81A7, 0x0000, 0x0000, 0x0002, /*101A7*/
  0xFBC2, 0x0020, 0x0002, 0x81A8, 0x0000, 0x0000, 0x0002, /*101A8*/
  0xFBC2, 0x0020, 0x0002, 0x81A9, 0x0000, 0x0000, 0x0002, /*101A9*/
  0xFBC2, 0x0020, 0x0002, 0x81AA, 0x0000, 0x0000, 0x0002, /*101AA*/
  0xFBC2, 0x0020, 0x0002, 0x81AB, 0x0000, 0x0000, 0x0002, /*101AB*/
  0xFBC2, 0x0020, 0x0002, 0x81AC, 0x0000, 0x0000, 0x0002, /*101AC*/
  0xFBC2, 0x0020, 0x0002, 0x81AD, 0x0000, 0x0000, 0x0002, /*101AD*/
  0xFBC2, 0x0020, 0x0002, 0x81AE, 0x0000, 0x0000, 0x0002, /*101AE*/
  0xFBC2, 0x0020, 0x0002, 0x81AF, 0x0000, 0x0000, 0x0002, /*101AF*/
  0xFBC2, 0x0020, 0x0002, 0x81B0, 0x0000, 0x0000, 0x0002, /*101B0*/
  0xFBC2, 0x0020, 0x0002, 0x81B1, 0x0000, 0x0000, 0x0002, /*101B1*/
  0xFBC2, 0x0020, 0x0002, 0x81B2, 0x0000, 0x0000, 0x0002, /*101B2*/
  0xFBC2, 0x0020, 0x0002, 0x81B3, 0x0000, 0x0000, 0x0002, /*101B3*/
  0xFBC2, 0x0020, 0x0002, 0x81B4, 0x0000, 0x0000, 0x0002, /*101B4*/
  0xFBC2, 0x0020, 0x0002, 0x81B5, 0x0000, 0x0000, 0x0002, /*101B5*/
  0xFBC2, 0x0020, 0x0002, 0x81B6, 0x0000, 0x0000, 0x0002, /*101B6*/
  0xFBC2, 0x0020, 0x0002, 0x81B7, 0x0000, 0x0000, 0x0002, /*101B7*/
  0xFBC2, 0x0020, 0x0002, 0x81B8, 0x0000, 0x0000, 0x0002, /*101B8*/
  0xFBC2, 0x0020, 0x0002, 0x81B9, 0x0000, 0x0000, 0x0002, /*101B9*/
  0xFBC2, 0x0020, 0x0002, 0x81BA, 0x0000, 0x0000, 0x0002, /*101BA*/
  0xFBC2, 0x0020, 0x0002, 0x81BB, 0x0000, 0x0000, 0x0002, /*101BB*/
  0xFBC2, 0x0020, 0x0002, 0x81BC, 0x0000, 0x0000, 0x0002, /*101BC*/
  0xFBC2, 0x0020, 0x0002, 0x81BD, 0x0000, 0x0000, 0x0002, /*101BD*/
  0xFBC2, 0x0020, 0x0002, 0x81BE, 0x0000, 0x0000, 0x0002, /*101BE*/
  0xFBC2, 0x0020, 0x0002, 0x81BF, 0x0000, 0x0000, 0x0002, /*101BF*/
  0xFBC2, 0x0020, 0x0002, 0x81C0, 0x0000, 0x0000, 0x0002, /*101C0*/
  0xFBC2, 0x0020, 0x0002, 0x81C1, 0x0000, 0x0000, 0x0002, /*101C1*/
  0xFBC2, 0x0020, 0x0002, 0x81C2, 0x0000, 0x0000, 0x0002, /*101C2*/
  0xFBC2, 0x0020, 0x0002, 0x81C3, 0x0000, 0x0000, 0x0002, /*101C3*/
  0xFBC2, 0x0020, 0x0002, 0x81C4, 0x0000, 0x0000, 0x0002, /*101C4*/
  0xFBC2, 0x0020, 0x0002, 0x81C5, 0x0000, 0x0000, 0x0002, /*101C5*/
  0xFBC2, 0x0020, 0x0002, 0x81C6, 0x0000, 0x0000, 0x0002, /*101C6*/
  0xFBC2, 0x0020, 0x0002, 0x81C7, 0x0000, 0x0000, 0x0002, /*101C7*/
  0xFBC2, 0x0020, 0x0002, 0x81C8, 0x0000, 0x0000, 0x0002, /*101C8*/
  0xFBC2, 0x0020, 0x0002, 0x81C9, 0x0000, 0x0000, 0x0002, /*101C9*/
  0xFBC2, 0x0020, 0x0002, 0x81CA, 0x0000, 0x0000, 0x0002, /*101CA*/
  0xFBC2, 0x0020, 0x0002, 0x81CB, 0x0000, 0x0000, 0x0002, /*101CB*/
  0xFBC2, 0x0020, 0x0002, 0x81CC, 0x0000, 0x0000, 0x0002, /*101CC*/
  0xFBC2, 0x0020, 0x0002, 0x81CD, 0x0000, 0x0000, 0x0002, /*101CD*/
  0xFBC2, 0x0020, 0x0002, 0x81CE, 0x0000, 0x0000, 0x0002, /*101CE*/
  0xFBC2, 0x0020, 0x0002, 0x81CF, 0x0000, 0x0000, 0x0002, /*101CF*/
  0x0FA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101D0*/
  0x0FA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101D1*/
  0x0FA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101D2*/
  0x0FA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101D3*/
  0x0FA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101D4*/
  0x0FA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101D5*/
  0x0FA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101D6*/
  0x0FA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101D7*/
  0x0FAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101D8*/
  0x0FAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101D9*/
  0x0FAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101DA*/
  0x0FAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101DB*/
  0x0FAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101DC*/
  0x0FAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101DD*/
  0x0FB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101DE*/
  0x0FB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101DF*/
  0x0FB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101E0*/
  0x0FB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101E1*/
  0x0FB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101E2*/
  0x0FB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101E3*/
  0x0FB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101E4*/
  0x0FB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101E5*/
  0x0FB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101E6*/
  0x0FB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101E7*/
  0x0FBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101E8*/
  0x0FBB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101E9*/
  0x0FBC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101EA*/
  0x0FBD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101EB*/
  0x0FBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101EC*/
  0x0FBF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101ED*/
  0x0FC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101EE*/
  0x0FC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101EF*/
  0x0FC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101F0*/
  0x0FC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101F1*/
  0x0FC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101F2*/
  0x0FC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101F3*/
  0x0FC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101F4*/
  0x0FC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101F5*/
  0x0FC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101F6*/
  0x0FC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101F7*/
  0x0FCA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101F8*/
  0x0FCB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101F9*/
  0x0FCC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101FA*/
  0x0FCD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101FB*/
  0x0FCE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101FC*/
  0x0000, 0x010F, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*101FD*/
  0xFBC2, 0x0020, 0x0002, 0x81FE, 0x0000, 0x0000, 0x0002, /*101FE*/
  0xFBC2, 0x0020, 0x0002, 0x81FF, 0x0000, 0x0000, 0x0002  /*101FF*/
};

uint16 uca900_p102[]= {
  0xFBC2, 0x0020, 0x0002, 0x8200, 0x0000, 0x0000, 0x0002, /*10200*/
  0xFBC2, 0x0020, 0x0002, 0x8201, 0x0000, 0x0000, 0x0002, /*10201*/
  0xFBC2, 0x0020, 0x0002, 0x8202, 0x0000, 0x0000, 0x0002, /*10202*/
  0xFBC2, 0x0020, 0x0002, 0x8203, 0x0000, 0x0000, 0x0002, /*10203*/
  0xFBC2, 0x0020, 0x0002, 0x8204, 0x0000, 0x0000, 0x0002, /*10204*/
  0xFBC2, 0x0020, 0x0002, 0x8205, 0x0000, 0x0000, 0x0002, /*10205*/
  0xFBC2, 0x0020, 0x0002, 0x8206, 0x0000, 0x0000, 0x0002, /*10206*/
  0xFBC2, 0x0020, 0x0002, 0x8207, 0x0000, 0x0000, 0x0002, /*10207*/
  0xFBC2, 0x0020, 0x0002, 0x8208, 0x0000, 0x0000, 0x0002, /*10208*/
  0xFBC2, 0x0020, 0x0002, 0x8209, 0x0000, 0x0000, 0x0002, /*10209*/
  0xFBC2, 0x0020, 0x0002, 0x820A, 0x0000, 0x0000, 0x0002, /*1020A*/
  0xFBC2, 0x0020, 0x0002, 0x820B, 0x0000, 0x0000, 0x0002, /*1020B*/
  0xFBC2, 0x0020, 0x0002, 0x820C, 0x0000, 0x0000, 0x0002, /*1020C*/
  0xFBC2, 0x0020, 0x0002, 0x820D, 0x0000, 0x0000, 0x0002, /*1020D*/
  0xFBC2, 0x0020, 0x0002, 0x820E, 0x0000, 0x0000, 0x0002, /*1020E*/
  0xFBC2, 0x0020, 0x0002, 0x820F, 0x0000, 0x0000, 0x0002, /*1020F*/
  0xFBC2, 0x0020, 0x0002, 0x8210, 0x0000, 0x0000, 0x0002, /*10210*/
  0xFBC2, 0x0020, 0x0002, 0x8211, 0x0000, 0x0000, 0x0002, /*10211*/
  0xFBC2, 0x0020, 0x0002, 0x8212, 0x0000, 0x0000, 0x0002, /*10212*/
  0xFBC2, 0x0020, 0x0002, 0x8213, 0x0000, 0x0000, 0x0002, /*10213*/
  0xFBC2, 0x0020, 0x0002, 0x8214, 0x0000, 0x0000, 0x0002, /*10214*/
  0xFBC2, 0x0020, 0x0002, 0x8215, 0x0000, 0x0000, 0x0002, /*10215*/
  0xFBC2, 0x0020, 0x0002, 0x8216, 0x0000, 0x0000, 0x0002, /*10216*/
  0xFBC2, 0x0020, 0x0002, 0x8217, 0x0000, 0x0000, 0x0002, /*10217*/
  0xFBC2, 0x0020, 0x0002, 0x8218, 0x0000, 0x0000, 0x0002, /*10218*/
  0xFBC2, 0x0020, 0x0002, 0x8219, 0x0000, 0x0000, 0x0002, /*10219*/
  0xFBC2, 0x0020, 0x0002, 0x821A, 0x0000, 0x0000, 0x0002, /*1021A*/
  0xFBC2, 0x0020, 0x0002, 0x821B, 0x0000, 0x0000, 0x0002, /*1021B*/
  0xFBC2, 0x0020, 0x0002, 0x821C, 0x0000, 0x0000, 0x0002, /*1021C*/
  0xFBC2, 0x0020, 0x0002, 0x821D, 0x0000, 0x0000, 0x0002, /*1021D*/
  0xFBC2, 0x0020, 0x0002, 0x821E, 0x0000, 0x0000, 0x0002, /*1021E*/
  0xFBC2, 0x0020, 0x0002, 0x821F, 0x0000, 0x0000, 0x0002, /*1021F*/
  0xFBC2, 0x0020, 0x0002, 0x8220, 0x0000, 0x0000, 0x0002, /*10220*/
  0xFBC2, 0x0020, 0x0002, 0x8221, 0x0000, 0x0000, 0x0002, /*10221*/
  0xFBC2, 0x0020, 0x0002, 0x8222, 0x0000, 0x0000, 0x0002, /*10222*/
  0xFBC2, 0x0020, 0x0002, 0x8223, 0x0000, 0x0000, 0x0002, /*10223*/
  0xFBC2, 0x0020, 0x0002, 0x8224, 0x0000, 0x0000, 0x0002, /*10224*/
  0xFBC2, 0x0020, 0x0002, 0x8225, 0x0000, 0x0000, 0x0002, /*10225*/
  0xFBC2, 0x0020, 0x0002, 0x8226, 0x0000, 0x0000, 0x0002, /*10226*/
  0xFBC2, 0x0020, 0x0002, 0x8227, 0x0000, 0x0000, 0x0002, /*10227*/
  0xFBC2, 0x0020, 0x0002, 0x8228, 0x0000, 0x0000, 0x0002, /*10228*/
  0xFBC2, 0x0020, 0x0002, 0x8229, 0x0000, 0x0000, 0x0002, /*10229*/
  0xFBC2, 0x0020, 0x0002, 0x822A, 0x0000, 0x0000, 0x0002, /*1022A*/
  0xFBC2, 0x0020, 0x0002, 0x822B, 0x0000, 0x0000, 0x0002, /*1022B*/
  0xFBC2, 0x0020, 0x0002, 0x822C, 0x0000, 0x0000, 0x0002, /*1022C*/
  0xFBC2, 0x0020, 0x0002, 0x822D, 0x0000, 0x0000, 0x0002, /*1022D*/
  0xFBC2, 0x0020, 0x0002, 0x822E, 0x0000, 0x0000, 0x0002, /*1022E*/
  0xFBC2, 0x0020, 0x0002, 0x822F, 0x0000, 0x0000, 0x0002, /*1022F*/
  0xFBC2, 0x0020, 0x0002, 0x8230, 0x0000, 0x0000, 0x0002, /*10230*/
  0xFBC2, 0x0020, 0x0002, 0x8231, 0x0000, 0x0000, 0x0002, /*10231*/
  0xFBC2, 0x0020, 0x0002, 0x8232, 0x0000, 0x0000, 0x0002, /*10232*/
  0xFBC2, 0x0020, 0x0002, 0x8233, 0x0000, 0x0000, 0x0002, /*10233*/
  0xFBC2, 0x0020, 0x0002, 0x8234, 0x0000, 0x0000, 0x0002, /*10234*/
  0xFBC2, 0x0020, 0x0002, 0x8235, 0x0000, 0x0000, 0x0002, /*10235*/
  0xFBC2, 0x0020, 0x0002, 0x8236, 0x0000, 0x0000, 0x0002, /*10236*/
  0xFBC2, 0x0020, 0x0002, 0x8237, 0x0000, 0x0000, 0x0002, /*10237*/
  0xFBC2, 0x0020, 0x0002, 0x8238, 0x0000, 0x0000, 0x0002, /*10238*/
  0xFBC2, 0x0020, 0x0002, 0x8239, 0x0000, 0x0000, 0x0002, /*10239*/
  0xFBC2, 0x0020, 0x0002, 0x823A, 0x0000, 0x0000, 0x0002, /*1023A*/
  0xFBC2, 0x0020, 0x0002, 0x823B, 0x0000, 0x0000, 0x0002, /*1023B*/
  0xFBC2, 0x0020, 0x0002, 0x823C, 0x0000, 0x0000, 0x0002, /*1023C*/
  0xFBC2, 0x0020, 0x0002, 0x823D, 0x0000, 0x0000, 0x0002, /*1023D*/
  0xFBC2, 0x0020, 0x0002, 0x823E, 0x0000, 0x0000, 0x0002, /*1023E*/
  0xFBC2, 0x0020, 0x0002, 0x823F, 0x0000, 0x0000, 0x0002, /*1023F*/
  0xFBC2, 0x0020, 0x0002, 0x8240, 0x0000, 0x0000, 0x0002, /*10240*/
  0xFBC2, 0x0020, 0x0002, 0x8241, 0x0000, 0x0000, 0x0002, /*10241*/
  0xFBC2, 0x0020, 0x0002, 0x8242, 0x0000, 0x0000, 0x0002, /*10242*/
  0xFBC2, 0x0020, 0x0002, 0x8243, 0x0000, 0x0000, 0x0002, /*10243*/
  0xFBC2, 0x0020, 0x0002, 0x8244, 0x0000, 0x0000, 0x0002, /*10244*/
  0xFBC2, 0x0020, 0x0002, 0x8245, 0x0000, 0x0000, 0x0002, /*10245*/
  0xFBC2, 0x0020, 0x0002, 0x8246, 0x0000, 0x0000, 0x0002, /*10246*/
  0xFBC2, 0x0020, 0x0002, 0x8247, 0x0000, 0x0000, 0x0002, /*10247*/
  0xFBC2, 0x0020, 0x0002, 0x8248, 0x0000, 0x0000, 0x0002, /*10248*/
  0xFBC2, 0x0020, 0x0002, 0x8249, 0x0000, 0x0000, 0x0002, /*10249*/
  0xFBC2, 0x0020, 0x0002, 0x824A, 0x0000, 0x0000, 0x0002, /*1024A*/
  0xFBC2, 0x0020, 0x0002, 0x824B, 0x0000, 0x0000, 0x0002, /*1024B*/
  0xFBC2, 0x0020, 0x0002, 0x824C, 0x0000, 0x0000, 0x0002, /*1024C*/
  0xFBC2, 0x0020, 0x0002, 0x824D, 0x0000, 0x0000, 0x0002, /*1024D*/
  0xFBC2, 0x0020, 0x0002, 0x824E, 0x0000, 0x0000, 0x0002, /*1024E*/
  0xFBC2, 0x0020, 0x0002, 0x824F, 0x0000, 0x0000, 0x0002, /*1024F*/
  0xFBC2, 0x0020, 0x0002, 0x8250, 0x0000, 0x0000, 0x0002, /*10250*/
  0xFBC2, 0x0020, 0x0002, 0x8251, 0x0000, 0x0000, 0x0002, /*10251*/
  0xFBC2, 0x0020, 0x0002, 0x8252, 0x0000, 0x0000, 0x0002, /*10252*/
  0xFBC2, 0x0020, 0x0002, 0x8253, 0x0000, 0x0000, 0x0002, /*10253*/
  0xFBC2, 0x0020, 0x0002, 0x8254, 0x0000, 0x0000, 0x0002, /*10254*/
  0xFBC2, 0x0020, 0x0002, 0x8255, 0x0000, 0x0000, 0x0002, /*10255*/
  0xFBC2, 0x0020, 0x0002, 0x8256, 0x0000, 0x0000, 0x0002, /*10256*/
  0xFBC2, 0x0020, 0x0002, 0x8257, 0x0000, 0x0000, 0x0002, /*10257*/
  0xFBC2, 0x0020, 0x0002, 0x8258, 0x0000, 0x0000, 0x0002, /*10258*/
  0xFBC2, 0x0020, 0x0002, 0x8259, 0x0000, 0x0000, 0x0002, /*10259*/
  0xFBC2, 0x0020, 0x0002, 0x825A, 0x0000, 0x0000, 0x0002, /*1025A*/
  0xFBC2, 0x0020, 0x0002, 0x825B, 0x0000, 0x0000, 0x0002, /*1025B*/
  0xFBC2, 0x0020, 0x0002, 0x825C, 0x0000, 0x0000, 0x0002, /*1025C*/
  0xFBC2, 0x0020, 0x0002, 0x825D, 0x0000, 0x0000, 0x0002, /*1025D*/
  0xFBC2, 0x0020, 0x0002, 0x825E, 0x0000, 0x0000, 0x0002, /*1025E*/
  0xFBC2, 0x0020, 0x0002, 0x825F, 0x0000, 0x0000, 0x0002, /*1025F*/
  0xFBC2, 0x0020, 0x0002, 0x8260, 0x0000, 0x0000, 0x0002, /*10260*/
  0xFBC2, 0x0020, 0x0002, 0x8261, 0x0000, 0x0000, 0x0002, /*10261*/
  0xFBC2, 0x0020, 0x0002, 0x8262, 0x0000, 0x0000, 0x0002, /*10262*/
  0xFBC2, 0x0020, 0x0002, 0x8263, 0x0000, 0x0000, 0x0002, /*10263*/
  0xFBC2, 0x0020, 0x0002, 0x8264, 0x0000, 0x0000, 0x0002, /*10264*/
  0xFBC2, 0x0020, 0x0002, 0x8265, 0x0000, 0x0000, 0x0002, /*10265*/
  0xFBC2, 0x0020, 0x0002, 0x8266, 0x0000, 0x0000, 0x0002, /*10266*/
  0xFBC2, 0x0020, 0x0002, 0x8267, 0x0000, 0x0000, 0x0002, /*10267*/
  0xFBC2, 0x0020, 0x0002, 0x8268, 0x0000, 0x0000, 0x0002, /*10268*/
  0xFBC2, 0x0020, 0x0002, 0x8269, 0x0000, 0x0000, 0x0002, /*10269*/
  0xFBC2, 0x0020, 0x0002, 0x826A, 0x0000, 0x0000, 0x0002, /*1026A*/
  0xFBC2, 0x0020, 0x0002, 0x826B, 0x0000, 0x0000, 0x0002, /*1026B*/
  0xFBC2, 0x0020, 0x0002, 0x826C, 0x0000, 0x0000, 0x0002, /*1026C*/
  0xFBC2, 0x0020, 0x0002, 0x826D, 0x0000, 0x0000, 0x0002, /*1026D*/
  0xFBC2, 0x0020, 0x0002, 0x826E, 0x0000, 0x0000, 0x0002, /*1026E*/
  0xFBC2, 0x0020, 0x0002, 0x826F, 0x0000, 0x0000, 0x0002, /*1026F*/
  0xFBC2, 0x0020, 0x0002, 0x8270, 0x0000, 0x0000, 0x0002, /*10270*/
  0xFBC2, 0x0020, 0x0002, 0x8271, 0x0000, 0x0000, 0x0002, /*10271*/
  0xFBC2, 0x0020, 0x0002, 0x8272, 0x0000, 0x0000, 0x0002, /*10272*/
  0xFBC2, 0x0020, 0x0002, 0x8273, 0x0000, 0x0000, 0x0002, /*10273*/
  0xFBC2, 0x0020, 0x0002, 0x8274, 0x0000, 0x0000, 0x0002, /*10274*/
  0xFBC2, 0x0020, 0x0002, 0x8275, 0x0000, 0x0000, 0x0002, /*10275*/
  0xFBC2, 0x0020, 0x0002, 0x8276, 0x0000, 0x0000, 0x0002, /*10276*/
  0xFBC2, 0x0020, 0x0002, 0x8277, 0x0000, 0x0000, 0x0002, /*10277*/
  0xFBC2, 0x0020, 0x0002, 0x8278, 0x0000, 0x0000, 0x0002, /*10278*/
  0xFBC2, 0x0020, 0x0002, 0x8279, 0x0000, 0x0000, 0x0002, /*10279*/
  0xFBC2, 0x0020, 0x0002, 0x827A, 0x0000, 0x0000, 0x0002, /*1027A*/
  0xFBC2, 0x0020, 0x0002, 0x827B, 0x0000, 0x0000, 0x0002, /*1027B*/
  0xFBC2, 0x0020, 0x0002, 0x827C, 0x0000, 0x0000, 0x0002, /*1027C*/
  0xFBC2, 0x0020, 0x0002, 0x827D, 0x0000, 0x0000, 0x0002, /*1027D*/
  0xFBC2, 0x0020, 0x0002, 0x827E, 0x0000, 0x0000, 0x0002, /*1027E*/
  0xFBC2, 0x0020, 0x0002, 0x827F, 0x0000, 0x0000, 0x0002, /*1027F*/
  0x43AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10280*/
  0x43B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10281*/
  0x43B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10282*/
  0x43B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10283*/
  0x43B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10284*/
  0x43B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10285*/
  0x43B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10286*/
  0x43B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10287*/
  0x43B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10288*/
  0x43B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10289*/
  0x43B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1028A*/
  0x43BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1028B*/
  0x43BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1028C*/
  0x43BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1028D*/
  0x43BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1028E*/
  0x43BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1028F*/
  0x43BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10290*/
  0x43C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10291*/
  0x43C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10292*/
  0x43C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10293*/
  0x43C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10294*/
  0x43C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10295*/
  0x43C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10296*/
  0x43C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10297*/
  0x43C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10298*/
  0x43C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10299*/
  0x43C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1029A*/
  0x43CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1029B*/
  0x43CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1029C*/
  0xFBC2, 0x0020, 0x0002, 0x829D, 0x0000, 0x0000, 0x0002, /*1029D*/
  0xFBC2, 0x0020, 0x0002, 0x829E, 0x0000, 0x0000, 0x0002, /*1029E*/
  0xFBC2, 0x0020, 0x0002, 0x829F, 0x0000, 0x0000, 0x0002, /*1029F*/
  0x43CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102A0*/
  0x43CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102A1*/
  0x43CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102A2*/
  0x43CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102A3*/
  0x43D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102A4*/
  0x43D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102A5*/
  0x43D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102A6*/
  0x43D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102A7*/
  0x43D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102A8*/
  0x43D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102A9*/
  0x43D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102AA*/
  0x43D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102AB*/
  0x43D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102AC*/
  0x43D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102AD*/
  0x43DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102AE*/
  0x43DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102AF*/
  0x43DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102B0*/
  0x43DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102B1*/
  0x43DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102B2*/
  0x43DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102B3*/
  0x43E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102B4*/
  0x43E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102B5*/
  0x43E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102B6*/
  0x43E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102B7*/
  0x43E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102B8*/
  0x43E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102B9*/
  0x43E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102BA*/
  0x43E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102BB*/
  0x43E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102BC*/
  0x43E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102BD*/
  0x43EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102BE*/
  0x43EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102BF*/
  0x43EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102C0*/
  0x43ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102C1*/
  0x43EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102C2*/
  0x43EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102C3*/
  0x43F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102C4*/
  0x43F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102C5*/
  0x43F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102C6*/
  0x43F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102C7*/
  0x43F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102C8*/
  0x43F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102C9*/
  0x43F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102CA*/
  0x43F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102CB*/
  0x43F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102CC*/
  0x43F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102CD*/
  0x43FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102CE*/
  0x43FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102CF*/
  0x43FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102D0*/
  0xFBC2, 0x0020, 0x0002, 0x82D1, 0x0000, 0x0000, 0x0002, /*102D1*/
  0xFBC2, 0x0020, 0x0002, 0x82D2, 0x0000, 0x0000, 0x0002, /*102D2*/
  0xFBC2, 0x0020, 0x0002, 0x82D3, 0x0000, 0x0000, 0x0002, /*102D3*/
  0xFBC2, 0x0020, 0x0002, 0x82D4, 0x0000, 0x0000, 0x0002, /*102D4*/
  0xFBC2, 0x0020, 0x0002, 0x82D5, 0x0000, 0x0000, 0x0002, /*102D5*/
  0xFBC2, 0x0020, 0x0002, 0x82D6, 0x0000, 0x0000, 0x0002, /*102D6*/
  0xFBC2, 0x0020, 0x0002, 0x82D7, 0x0000, 0x0000, 0x0002, /*102D7*/
  0xFBC2, 0x0020, 0x0002, 0x82D8, 0x0000, 0x0000, 0x0002, /*102D8*/
  0xFBC2, 0x0020, 0x0002, 0x82D9, 0x0000, 0x0000, 0x0002, /*102D9*/
  0xFBC2, 0x0020, 0x0002, 0x82DA, 0x0000, 0x0000, 0x0002, /*102DA*/
  0xFBC2, 0x0020, 0x0002, 0x82DB, 0x0000, 0x0000, 0x0002, /*102DB*/
  0xFBC2, 0x0020, 0x0002, 0x82DC, 0x0000, 0x0000, 0x0002, /*102DC*/
  0xFBC2, 0x0020, 0x0002, 0x82DD, 0x0000, 0x0000, 0x0002, /*102DD*/
  0xFBC2, 0x0020, 0x0002, 0x82DE, 0x0000, 0x0000, 0x0002, /*102DE*/
  0xFBC2, 0x0020, 0x0002, 0x82DF, 0x0000, 0x0000, 0x0002, /*102DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*102E0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102E1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102E2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102E3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102E4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102E5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102E6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102E7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102E8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102E9*/
  0x1B3A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102EA*/
  0x1B3B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102EB*/
  0x1B3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102EC*/
  0x1B3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102ED*/
  0x1B3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102EE*/
  0x1B3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102EF*/
  0x1B40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102F0*/
  0x1B41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102F1*/
  0x1B42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102F2*/
  0x1B43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102F3*/
  0x1B44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102F4*/
  0x1B45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102F5*/
  0x1B46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102F6*/
  0x1B47, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102F7*/
  0x1B48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102F8*/
  0x1B49, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102F9*/
  0x1B4A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102FA*/
  0x1B4B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*102FB*/
  0xFBC2, 0x0020, 0x0002, 0x82FC, 0x0000, 0x0000, 0x0002, /*102FC*/
  0xFBC2, 0x0020, 0x0002, 0x82FD, 0x0000, 0x0000, 0x0002, /*102FD*/
  0xFBC2, 0x0020, 0x0002, 0x82FE, 0x0000, 0x0000, 0x0002, /*102FE*/
  0xFBC2, 0x0020, 0x0002, 0x82FF, 0x0000, 0x0000, 0x0002  /*102FF*/
};

uint16 uca900_p103[]= {
  0x4417, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10300*/
  0x4418, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10301*/
  0x4419, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10302*/
  0x441A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10303*/
  0x441B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10304*/
  0x441C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10305*/
  0x441D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10306*/
  0x441E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10307*/
  0x441F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10308*/
  0x4420, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10309*/
  0x4421, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1030A*/
  0x4422, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1030B*/
  0x4423, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1030C*/
  0x4424, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1030D*/
  0x4425, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1030E*/
  0x4427, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1030F*/
  0x4428, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10310*/
  0x4429, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10311*/
  0x442A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10312*/
  0x442B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10313*/
  0x442C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10314*/
  0x442D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10315*/
  0x442E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10316*/
  0x442F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10317*/
  0x4430, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10318*/
  0x4431, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10319*/
  0x4432, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1031A*/
  0x4433, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1031B*/
  0x4434, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1031C*/
  0x4435, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1031D*/
  0x4436, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1031E*/
  0x4426, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1031F*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10320*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10321*/
  0x1AE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10322*/
  0x1AE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10323*/
  0xFBC2, 0x0020, 0x0002, 0x8324, 0x0000, 0x0000, 0x0002, /*10324*/
  0xFBC2, 0x0020, 0x0002, 0x8325, 0x0000, 0x0000, 0x0002, /*10325*/
  0xFBC2, 0x0020, 0x0002, 0x8326, 0x0000, 0x0000, 0x0002, /*10326*/
  0xFBC2, 0x0020, 0x0002, 0x8327, 0x0000, 0x0000, 0x0002, /*10327*/
  0xFBC2, 0x0020, 0x0002, 0x8328, 0x0000, 0x0000, 0x0002, /*10328*/
  0xFBC2, 0x0020, 0x0002, 0x8329, 0x0000, 0x0000, 0x0002, /*10329*/
  0xFBC2, 0x0020, 0x0002, 0x832A, 0x0000, 0x0000, 0x0002, /*1032A*/
  0xFBC2, 0x0020, 0x0002, 0x832B, 0x0000, 0x0000, 0x0002, /*1032B*/
  0xFBC2, 0x0020, 0x0002, 0x832C, 0x0000, 0x0000, 0x0002, /*1032C*/
  0xFBC2, 0x0020, 0x0002, 0x832D, 0x0000, 0x0000, 0x0002, /*1032D*/
  0xFBC2, 0x0020, 0x0002, 0x832E, 0x0000, 0x0000, 0x0002, /*1032E*/
  0xFBC2, 0x0020, 0x0002, 0x832F, 0x0000, 0x0000, 0x0002, /*1032F*/
  0x4437, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10330*/
  0x4438, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10331*/
  0x4439, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10332*/
  0x443A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10333*/
  0x443B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10334*/
  0x443C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10335*/
  0x443D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10336*/
  0x443E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10337*/
  0x443F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10338*/
  0x4440, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10339*/
  0x4441, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1033A*/
  0x4442, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1033B*/
  0x4443, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1033C*/
  0x4444, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1033D*/
  0x4445, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1033E*/
  0x4446, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1033F*/
  0x4447, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10340*/
  0x4448, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10341*/
  0x4449, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10342*/
  0x444A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10343*/
  0x444B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10344*/
  0x444C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10345*/
  0x444D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10346*/
  0x444E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10347*/
  0x444F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10348*/
  0x4450, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10349*/
  0x4451, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1034A*/
  0xFBC2, 0x0020, 0x0002, 0x834B, 0x0000, 0x0000, 0x0002, /*1034B*/
  0xFBC2, 0x0020, 0x0002, 0x834C, 0x0000, 0x0000, 0x0002, /*1034C*/
  0xFBC2, 0x0020, 0x0002, 0x834D, 0x0000, 0x0000, 0x0002, /*1034D*/
  0xFBC2, 0x0020, 0x0002, 0x834E, 0x0000, 0x0000, 0x0002, /*1034E*/
  0xFBC2, 0x0020, 0x0002, 0x834F, 0x0000, 0x0000, 0x0002, /*1034F*/
  0x2214, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10350*/
  0x2215, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10351*/
  0x2216, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10352*/
  0x2217, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10353*/
  0x2218, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10354*/
  0x2219, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10355*/
  0x221A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10356*/
  0x221B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10357*/
  0x221C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10358*/
  0x221D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10359*/
  0x221E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1035A*/
  0x221F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1035B*/
  0x2220, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1035C*/
  0x2221, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1035D*/
  0x2222, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1035E*/
  0x2223, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1035F*/
  0x2224, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10360*/
  0x2225, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10361*/
  0x2226, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10362*/
  0x2227, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10363*/
  0x2228, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10364*/
  0x2229, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10365*/
  0x222A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10366*/
  0x222B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10367*/
  0x222C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10368*/
  0x222D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10369*/
  0x222E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1036A*/
  0x222F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1036B*/
  0x2230, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1036C*/
  0x2231, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1036D*/
  0x2232, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1036E*/
  0x2233, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1036F*/
  0x2234, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10370*/
  0x2235, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10371*/
  0x2236, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10372*/
  0x2237, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10373*/
  0x2238, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10374*/
  0x2239, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10375*/
  0x2214, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*10376*/
  0x2217, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*10377*/
  0x221B, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*10378*/
  0x2221, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*10379*/
  0x2225, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1037A*/
  0xFBC2, 0x0020, 0x0002, 0x837B, 0x0000, 0x0000, 0x0002, /*1037B*/
  0xFBC2, 0x0020, 0x0002, 0x837C, 0x0000, 0x0000, 0x0002, /*1037C*/
  0xFBC2, 0x0020, 0x0002, 0x837D, 0x0000, 0x0000, 0x0002, /*1037D*/
  0xFBC2, 0x0020, 0x0002, 0x837E, 0x0000, 0x0000, 0x0002, /*1037E*/
  0xFBC2, 0x0020, 0x0002, 0x837F, 0x0000, 0x0000, 0x0002, /*1037F*/
  0x496B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10380*/
  0x496C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10381*/
  0x496D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10382*/
  0x496E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10383*/
  0x496F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10384*/
  0x4970, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10385*/
  0x4971, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10386*/
  0x4972, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10387*/
  0x4973, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10388*/
  0x4974, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10389*/
  0x4975, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1038A*/
  0x4976, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1038B*/
  0x4977, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1038C*/
  0x4978, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1038D*/
  0x4979, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1038E*/
  0x497A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1038F*/
  0x497B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10390*/
  0x497C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10391*/
  0x497D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10392*/
  0x497E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10393*/
  0x497F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10394*/
  0x4980, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10395*/
  0x4981, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10396*/
  0x4982, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10397*/
  0x4983, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10398*/
  0x4984, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10399*/
  0x4985, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1039A*/
  0x4986, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1039B*/
  0x4987, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1039C*/
  0x4988, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1039D*/
  0xFBC2, 0x0020, 0x0002, 0x839E, 0x0000, 0x0000, 0x0002, /*1039E*/
  0x02FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1039F*/
  0x4989, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103A0*/
  0x498A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103A1*/
  0x498B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103A2*/
  0x498C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103A3*/
  0x498D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103A4*/
  0x498E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103A5*/
  0x498F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103A6*/
  0x4990, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103A7*/
  0x4991, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103A8*/
  0x4992, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103A9*/
  0x4993, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103AA*/
  0x4994, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103AB*/
  0x4995, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103AC*/
  0x4996, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103AD*/
  0x4997, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103AE*/
  0x4998, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103AF*/
  0x4999, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103B0*/
  0x499A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103B1*/
  0x499B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103B2*/
  0x499C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103B3*/
  0x499D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103B4*/
  0x499E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103B5*/
  0x499F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103B6*/
  0x49A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103B7*/
  0x49A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103B8*/
  0x49A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103B9*/
  0x49A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103BA*/
  0x49A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103BB*/
  0x49A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103BC*/
  0x49A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103BD*/
  0x49A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103BE*/
  0x49A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103BF*/
  0x49A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103C0*/
  0x49AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103C1*/
  0x49AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103C2*/
  0x49AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103C3*/
  0xFBC2, 0x0020, 0x0002, 0x83C4, 0x0000, 0x0000, 0x0002, /*103C4*/
  0xFBC2, 0x0020, 0x0002, 0x83C5, 0x0000, 0x0000, 0x0002, /*103C5*/
  0xFBC2, 0x0020, 0x0002, 0x83C6, 0x0000, 0x0000, 0x0002, /*103C6*/
  0xFBC2, 0x0020, 0x0002, 0x83C7, 0x0000, 0x0000, 0x0002, /*103C7*/
  0x49AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103C8*/
  0x49AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103C9*/
  0x49AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103CA*/
  0x49B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103CB*/
  0x49B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103CC*/
  0x49B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103CD*/
  0x49B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103CE*/
  0x49B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103CF*/
  0x02FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103D0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103D1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103D2*/
  0x1B4C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103D3*/
  0x1B4D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103D4*/
  0x1B4E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*103D5*/
  0xFBC2, 0x0020, 0x0002, 0x83D6, 0x0000, 0x0000, 0x0002, /*103D6*/
  0xFBC2, 0x0020, 0x0002, 0x83D7, 0x0000, 0x0000, 0x0002, /*103D7*/
  0xFBC2, 0x0020, 0x0002, 0x83D8, 0x0000, 0x0000, 0x0002, /*103D8*/
  0xFBC2, 0x0020, 0x0002, 0x83D9, 0x0000, 0x0000, 0x0002, /*103D9*/
  0xFBC2, 0x0020, 0x0002, 0x83DA, 0x0000, 0x0000, 0x0002, /*103DA*/
  0xFBC2, 0x0020, 0x0002, 0x83DB, 0x0000, 0x0000, 0x0002, /*103DB*/
  0xFBC2, 0x0020, 0x0002, 0x83DC, 0x0000, 0x0000, 0x0002, /*103DC*/
  0xFBC2, 0x0020, 0x0002, 0x83DD, 0x0000, 0x0000, 0x0002, /*103DD*/
  0xFBC2, 0x0020, 0x0002, 0x83DE, 0x0000, 0x0000, 0x0002, /*103DE*/
  0xFBC2, 0x0020, 0x0002, 0x83DF, 0x0000, 0x0000, 0x0002, /*103DF*/
  0xFBC2, 0x0020, 0x0002, 0x83E0, 0x0000, 0x0000, 0x0002, /*103E0*/
  0xFBC2, 0x0020, 0x0002, 0x83E1, 0x0000, 0x0000, 0x0002, /*103E1*/
  0xFBC2, 0x0020, 0x0002, 0x83E2, 0x0000, 0x0000, 0x0002, /*103E2*/
  0xFBC2, 0x0020, 0x0002, 0x83E3, 0x0000, 0x0000, 0x0002, /*103E3*/
  0xFBC2, 0x0020, 0x0002, 0x83E4, 0x0000, 0x0000, 0x0002, /*103E4*/
  0xFBC2, 0x0020, 0x0002, 0x83E5, 0x0000, 0x0000, 0x0002, /*103E5*/
  0xFBC2, 0x0020, 0x0002, 0x83E6, 0x0000, 0x0000, 0x0002, /*103E6*/
  0xFBC2, 0x0020, 0x0002, 0x83E7, 0x0000, 0x0000, 0x0002, /*103E7*/
  0xFBC2, 0x0020, 0x0002, 0x83E8, 0x0000, 0x0000, 0x0002, /*103E8*/
  0xFBC2, 0x0020, 0x0002, 0x83E9, 0x0000, 0x0000, 0x0002, /*103E9*/
  0xFBC2, 0x0020, 0x0002, 0x83EA, 0x0000, 0x0000, 0x0002, /*103EA*/
  0xFBC2, 0x0020, 0x0002, 0x83EB, 0x0000, 0x0000, 0x0002, /*103EB*/
  0xFBC2, 0x0020, 0x0002, 0x83EC, 0x0000, 0x0000, 0x0002, /*103EC*/
  0xFBC2, 0x0020, 0x0002, 0x83ED, 0x0000, 0x0000, 0x0002, /*103ED*/
  0xFBC2, 0x0020, 0x0002, 0x83EE, 0x0000, 0x0000, 0x0002, /*103EE*/
  0xFBC2, 0x0020, 0x0002, 0x83EF, 0x0000, 0x0000, 0x0002, /*103EF*/
  0xFBC2, 0x0020, 0x0002, 0x83F0, 0x0000, 0x0000, 0x0002, /*103F0*/
  0xFBC2, 0x0020, 0x0002, 0x83F1, 0x0000, 0x0000, 0x0002, /*103F1*/
  0xFBC2, 0x0020, 0x0002, 0x83F2, 0x0000, 0x0000, 0x0002, /*103F2*/
  0xFBC2, 0x0020, 0x0002, 0x83F3, 0x0000, 0x0000, 0x0002, /*103F3*/
  0xFBC2, 0x0020, 0x0002, 0x83F4, 0x0000, 0x0000, 0x0002, /*103F4*/
  0xFBC2, 0x0020, 0x0002, 0x83F5, 0x0000, 0x0000, 0x0002, /*103F5*/
  0xFBC2, 0x0020, 0x0002, 0x83F6, 0x0000, 0x0000, 0x0002, /*103F6*/
  0xFBC2, 0x0020, 0x0002, 0x83F7, 0x0000, 0x0000, 0x0002, /*103F7*/
  0xFBC2, 0x0020, 0x0002, 0x83F8, 0x0000, 0x0000, 0x0002, /*103F8*/
  0xFBC2, 0x0020, 0x0002, 0x83F9, 0x0000, 0x0000, 0x0002, /*103F9*/
  0xFBC2, 0x0020, 0x0002, 0x83FA, 0x0000, 0x0000, 0x0002, /*103FA*/
  0xFBC2, 0x0020, 0x0002, 0x83FB, 0x0000, 0x0000, 0x0002, /*103FB*/
  0xFBC2, 0x0020, 0x0002, 0x83FC, 0x0000, 0x0000, 0x0002, /*103FC*/
  0xFBC2, 0x0020, 0x0002, 0x83FD, 0x0000, 0x0000, 0x0002, /*103FD*/
  0xFBC2, 0x0020, 0x0002, 0x83FE, 0x0000, 0x0000, 0x0002, /*103FE*/
  0xFBC2, 0x0020, 0x0002, 0x83FF, 0x0000, 0x0000, 0x0002  /*103FF*/
};

uint16 uca900_p104[]= {
  0x4452, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10400*/
  0x4453, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10401*/
  0x4454, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10402*/
  0x4455, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10403*/
  0x4456, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10404*/
  0x4457, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10405*/
  0x4458, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10406*/
  0x4459, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10407*/
  0x445A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10408*/
  0x445B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10409*/
  0x445C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1040A*/
  0x445D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1040B*/
  0x445E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1040C*/
  0x445F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1040D*/
  0x4460, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1040E*/
  0x4461, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1040F*/
  0x4462, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10410*/
  0x4463, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10411*/
  0x4464, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10412*/
  0x4465, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10413*/
  0x4466, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10414*/
  0x4467, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10415*/
  0x4468, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10416*/
  0x4469, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10417*/
  0x446A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10418*/
  0x446B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10419*/
  0x446C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1041A*/
  0x446D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1041B*/
  0x446E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1041C*/
  0x446F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1041D*/
  0x4470, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1041E*/
  0x4471, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1041F*/
  0x4472, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10420*/
  0x4473, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10421*/
  0x4474, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10422*/
  0x4475, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10423*/
  0x4476, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10424*/
  0x4477, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10425*/
  0x4478, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10426*/
  0x4479, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10427*/
  0x4452, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10428*/
  0x4453, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10429*/
  0x4454, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1042A*/
  0x4455, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1042B*/
  0x4456, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1042C*/
  0x4457, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1042D*/
  0x4458, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1042E*/
  0x4459, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1042F*/
  0x445A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10430*/
  0x445B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10431*/
  0x445C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10432*/
  0x445D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10433*/
  0x445E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10434*/
  0x445F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10435*/
  0x4460, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10436*/
  0x4461, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10437*/
  0x4462, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10438*/
  0x4463, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10439*/
  0x4464, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1043A*/
  0x4465, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1043B*/
  0x4466, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1043C*/
  0x4467, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1043D*/
  0x4468, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1043E*/
  0x4469, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1043F*/
  0x446A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10440*/
  0x446B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10441*/
  0x446C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10442*/
  0x446D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10443*/
  0x446E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10444*/
  0x446F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10445*/
  0x4470, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10446*/
  0x4471, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10447*/
  0x4472, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10448*/
  0x4473, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10449*/
  0x4474, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1044A*/
  0x4475, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1044B*/
  0x4476, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1044C*/
  0x4477, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1044D*/
  0x4478, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1044E*/
  0x4479, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1044F*/
  0x447A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10450*/
  0x447B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10451*/
  0x447C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10452*/
  0x447D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10453*/
  0x447E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10454*/
  0x447F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10455*/
  0x4480, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10456*/
  0x4481, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10457*/
  0x4482, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10458*/
  0x4483, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10459*/
  0x4484, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1045A*/
  0x4485, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1045B*/
  0x4486, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1045C*/
  0x4487, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1045D*/
  0x4488, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1045E*/
  0x4489, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1045F*/
  0x448A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10460*/
  0x448B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10461*/
  0x448C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10462*/
  0x448D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10463*/
  0x448E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10464*/
  0x448F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10465*/
  0x4490, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10466*/
  0x4491, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10467*/
  0x4492, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10468*/
  0x4493, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10469*/
  0x4494, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1046A*/
  0x4495, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1046B*/
  0x4496, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1046C*/
  0x4497, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1046D*/
  0x4498, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1046E*/
  0x4499, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1046F*/
  0x449A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10470*/
  0x449B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10471*/
  0x449C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10472*/
  0x449D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10473*/
  0x449E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10474*/
  0x449F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10475*/
  0x44A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10476*/
  0x44A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10477*/
  0x44A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10478*/
  0x44A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10479*/
  0x44A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1047A*/
  0x44A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1047B*/
  0x44A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1047C*/
  0x44A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1047D*/
  0x44A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1047E*/
  0x44A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1047F*/
  0x4535, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10480*/
  0x4536, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10481*/
  0x4537, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10482*/
  0x4538, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10483*/
  0x4539, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10484*/
  0x453A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10485*/
  0x453B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10486*/
  0x453C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10487*/
  0x453D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10488*/
  0x453E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10489*/
  0x453F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1048A*/
  0x4540, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1048B*/
  0x4541, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1048C*/
  0x4542, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1048D*/
  0x4543, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1048E*/
  0x4544, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1048F*/
  0x4545, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10490*/
  0x4546, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10491*/
  0x4547, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10492*/
  0x4548, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10493*/
  0x4549, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10494*/
  0x454A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10495*/
  0x454B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10496*/
  0x454C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10497*/
  0x454D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10498*/
  0x454E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10499*/
  0x454F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1049A*/
  0x4550, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1049B*/
  0x4551, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1049C*/
  0x4552, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1049D*/
  0xFBC2, 0x0020, 0x0002, 0x849E, 0x0000, 0x0000, 0x0002, /*1049E*/
  0xFBC2, 0x0020, 0x0002, 0x849F, 0x0000, 0x0000, 0x0002, /*1049F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104A0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104A1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104A2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104A3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104A4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104A5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104A6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104A7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104A8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104A9*/
  0xFBC2, 0x0020, 0x0002, 0x84AA, 0x0000, 0x0000, 0x0002, /*104AA*/
  0xFBC2, 0x0020, 0x0002, 0x84AB, 0x0000, 0x0000, 0x0002, /*104AB*/
  0xFBC2, 0x0020, 0x0002, 0x84AC, 0x0000, 0x0000, 0x0002, /*104AC*/
  0xFBC2, 0x0020, 0x0002, 0x84AD, 0x0000, 0x0000, 0x0002, /*104AD*/
  0xFBC2, 0x0020, 0x0002, 0x84AE, 0x0000, 0x0000, 0x0002, /*104AE*/
  0xFBC2, 0x0020, 0x0002, 0x84AF, 0x0000, 0x0000, 0x0002, /*104AF*/
  0x33D5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104B0*/
  0x33D6, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104B1*/
  0x33D7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104B2*/
  0x33D8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104B3*/
  0x33D9, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104B4*/
  0x33DA, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104B5*/
  0x33DB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104B6*/
  0x33DC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104B7*/
  0x33DD, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104B8*/
  0x33DE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104B9*/
  0x33DF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104BA*/
  0x33E0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104BB*/
  0x33E1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104BC*/
  0x33E2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104BD*/
  0x33E3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104BE*/
  0x33E4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104BF*/
  0x33E5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104C0*/
  0x33E6, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104C1*/
  0x33E7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104C2*/
  0x33E8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104C3*/
  0x33E9, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104C4*/
  0x33EA, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104C5*/
  0x33EB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104C6*/
  0x33EC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104C7*/
  0x33ED, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104C8*/
  0x33EE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104C9*/
  0x33EF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104CA*/
  0x33F0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104CB*/
  0x33F1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104CC*/
  0x33F2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104CD*/
  0x33F3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104CE*/
  0x33F4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104CF*/
  0x33F5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104D0*/
  0x33F6, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104D1*/
  0x33F7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104D2*/
  0x33F8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*104D3*/
  0xFBC2, 0x0020, 0x0002, 0x84D4, 0x0000, 0x0000, 0x0002, /*104D4*/
  0xFBC2, 0x0020, 0x0002, 0x84D5, 0x0000, 0x0000, 0x0002, /*104D5*/
  0xFBC2, 0x0020, 0x0002, 0x84D6, 0x0000, 0x0000, 0x0002, /*104D6*/
  0xFBC2, 0x0020, 0x0002, 0x84D7, 0x0000, 0x0000, 0x0002, /*104D7*/
  0x33D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104D8*/
  0x33D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104D9*/
  0x33D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104DA*/
  0x33D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104DB*/
  0x33D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104DC*/
  0x33DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104DD*/
  0x33DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104DE*/
  0x33DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104DF*/
  0x33DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104E0*/
  0x33DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104E1*/
  0x33DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104E2*/
  0x33E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104E3*/
  0x33E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104E4*/
  0x33E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104E5*/
  0x33E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104E6*/
  0x33E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104E7*/
  0x33E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104E8*/
  0x33E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104E9*/
  0x33E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104EA*/
  0x33E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104EB*/
  0x33E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104EC*/
  0x33EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104ED*/
  0x33EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104EE*/
  0x33EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104EF*/
  0x33ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104F0*/
  0x33EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104F1*/
  0x33EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104F2*/
  0x33F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104F3*/
  0x33F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104F4*/
  0x33F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104F5*/
  0x33F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104F6*/
  0x33F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104F7*/
  0x33F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104F8*/
  0x33F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104F9*/
  0x33F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104FA*/
  0x33F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*104FB*/
  0xFBC2, 0x0020, 0x0002, 0x84FC, 0x0000, 0x0000, 0x0002, /*104FC*/
  0xFBC2, 0x0020, 0x0002, 0x84FD, 0x0000, 0x0000, 0x0002, /*104FD*/
  0xFBC2, 0x0020, 0x0002, 0x84FE, 0x0000, 0x0000, 0x0002, /*104FE*/
  0xFBC2, 0x0020, 0x0002, 0x84FF, 0x0000, 0x0000, 0x0002  /*104FF*/
};

uint16 uca900_p105[]= {
  0x4553, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10500*/
  0x4554, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10501*/
  0x4555, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10502*/
  0x4556, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10503*/
  0x4557, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10504*/
  0x4558, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10505*/
  0x4559, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10506*/
  0x455A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10507*/
  0x455B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10508*/
  0x455C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10509*/
  0x455D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1050A*/
  0x455E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1050B*/
  0x455F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1050C*/
  0x4560, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1050D*/
  0x4561, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1050E*/
  0x4562, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1050F*/
  0x4563, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10510*/
  0x4564, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10511*/
  0x4565, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10512*/
  0x4566, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10513*/
  0x4567, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10514*/
  0x4568, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10515*/
  0x4569, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10516*/
  0x456A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10517*/
  0x456B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10518*/
  0x456C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10519*/
  0x456D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1051A*/
  0x456E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1051B*/
  0x456F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1051C*/
  0x4570, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1051D*/
  0x4571, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1051E*/
  0x4572, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1051F*/
  0x4573, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10520*/
  0x4574, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10521*/
  0x4575, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10522*/
  0x4576, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10523*/
  0x4577, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10524*/
  0x4578, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10525*/
  0x4579, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10526*/
  0x457A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10527*/
  0xFBC2, 0x0020, 0x0002, 0x8528, 0x0000, 0x0000, 0x0002, /*10528*/
  0xFBC2, 0x0020, 0x0002, 0x8529, 0x0000, 0x0000, 0x0002, /*10529*/
  0xFBC2, 0x0020, 0x0002, 0x852A, 0x0000, 0x0000, 0x0002, /*1052A*/
  0xFBC2, 0x0020, 0x0002, 0x852B, 0x0000, 0x0000, 0x0002, /*1052B*/
  0xFBC2, 0x0020, 0x0002, 0x852C, 0x0000, 0x0000, 0x0002, /*1052C*/
  0xFBC2, 0x0020, 0x0002, 0x852D, 0x0000, 0x0000, 0x0002, /*1052D*/
  0xFBC2, 0x0020, 0x0002, 0x852E, 0x0000, 0x0000, 0x0002, /*1052E*/
  0xFBC2, 0x0020, 0x0002, 0x852F, 0x0000, 0x0000, 0x0002, /*1052F*/
  0x457B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10530*/
  0x457C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10531*/
  0x457D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10532*/
  0x457E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10533*/
  0x457F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10534*/
  0x4580, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10535*/
  0x4581, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10536*/
  0x4582, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10537*/
  0x4583, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10538*/
  0x4584, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10539*/
  0x4585, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1053A*/
  0x4586, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1053B*/
  0x4587, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1053C*/
  0x4588, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1053D*/
  0x4589, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1053E*/
  0x458A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1053F*/
  0x458B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10540*/
  0x458C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10541*/
  0x458D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10542*/
  0x458E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10543*/
  0x458F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10544*/
  0x4590, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10545*/
  0x4591, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10546*/
  0x4592, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10547*/
  0x4593, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10548*/
  0x4594, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10549*/
  0x4595, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1054A*/
  0x4596, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1054B*/
  0x4597, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1054C*/
  0x4598, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1054D*/
  0x4599, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1054E*/
  0x459A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1054F*/
  0x459B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10550*/
  0x459C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10551*/
  0x459D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10552*/
  0x459E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10553*/
  0x459F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10554*/
  0x45A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10555*/
  0x45A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10556*/
  0x45A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10557*/
  0x45A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10558*/
  0x45A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10559*/
  0x45A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1055A*/
  0x45A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1055B*/
  0x45A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1055C*/
  0x45A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1055D*/
  0x45A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1055E*/
  0x45AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1055F*/
  0x45AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10560*/
  0x45AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10561*/
  0x45AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10562*/
  0x45AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10563*/
  0xFBC2, 0x0020, 0x0002, 0x8564, 0x0000, 0x0000, 0x0002, /*10564*/
  0xFBC2, 0x0020, 0x0002, 0x8565, 0x0000, 0x0000, 0x0002, /*10565*/
  0xFBC2, 0x0020, 0x0002, 0x8566, 0x0000, 0x0000, 0x0002, /*10566*/
  0xFBC2, 0x0020, 0x0002, 0x8567, 0x0000, 0x0000, 0x0002, /*10567*/
  0xFBC2, 0x0020, 0x0002, 0x8568, 0x0000, 0x0000, 0x0002, /*10568*/
  0xFBC2, 0x0020, 0x0002, 0x8569, 0x0000, 0x0000, 0x0002, /*10569*/
  0xFBC2, 0x0020, 0x0002, 0x856A, 0x0000, 0x0000, 0x0002, /*1056A*/
  0xFBC2, 0x0020, 0x0002, 0x856B, 0x0000, 0x0000, 0x0002, /*1056B*/
  0xFBC2, 0x0020, 0x0002, 0x856C, 0x0000, 0x0000, 0x0002, /*1056C*/
  0xFBC2, 0x0020, 0x0002, 0x856D, 0x0000, 0x0000, 0x0002, /*1056D*/
  0xFBC2, 0x0020, 0x0002, 0x856E, 0x0000, 0x0000, 0x0002, /*1056E*/
  0x0432, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1056F*/
  0xFBC2, 0x0020, 0x0002, 0x8570, 0x0000, 0x0000, 0x0002, /*10570*/
  0xFBC2, 0x0020, 0x0002, 0x8571, 0x0000, 0x0000, 0x0002, /*10571*/
  0xFBC2, 0x0020, 0x0002, 0x8572, 0x0000, 0x0000, 0x0002, /*10572*/
  0xFBC2, 0x0020, 0x0002, 0x8573, 0x0000, 0x0000, 0x0002, /*10573*/
  0xFBC2, 0x0020, 0x0002, 0x8574, 0x0000, 0x0000, 0x0002, /*10574*/
  0xFBC2, 0x0020, 0x0002, 0x8575, 0x0000, 0x0000, 0x0002, /*10575*/
  0xFBC2, 0x0020, 0x0002, 0x8576, 0x0000, 0x0000, 0x0002, /*10576*/
  0xFBC2, 0x0020, 0x0002, 0x8577, 0x0000, 0x0000, 0x0002, /*10577*/
  0xFBC2, 0x0020, 0x0002, 0x8578, 0x0000, 0x0000, 0x0002, /*10578*/
  0xFBC2, 0x0020, 0x0002, 0x8579, 0x0000, 0x0000, 0x0002, /*10579*/
  0xFBC2, 0x0020, 0x0002, 0x857A, 0x0000, 0x0000, 0x0002, /*1057A*/
  0xFBC2, 0x0020, 0x0002, 0x857B, 0x0000, 0x0000, 0x0002, /*1057B*/
  0xFBC2, 0x0020, 0x0002, 0x857C, 0x0000, 0x0000, 0x0002, /*1057C*/
  0xFBC2, 0x0020, 0x0002, 0x857D, 0x0000, 0x0000, 0x0002, /*1057D*/
  0xFBC2, 0x0020, 0x0002, 0x857E, 0x0000, 0x0000, 0x0002, /*1057E*/
  0xFBC2, 0x0020, 0x0002, 0x857F, 0x0000, 0x0000, 0x0002, /*1057F*/
  0xFBC2, 0x0020, 0x0002, 0x8580, 0x0000, 0x0000, 0x0002, /*10580*/
  0xFBC2, 0x0020, 0x0002, 0x8581, 0x0000, 0x0000, 0x0002, /*10581*/
  0xFBC2, 0x0020, 0x0002, 0x8582, 0x0000, 0x0000, 0x0002, /*10582*/
  0xFBC2, 0x0020, 0x0002, 0x8583, 0x0000, 0x0000, 0x0002, /*10583*/
  0xFBC2, 0x0020, 0x0002, 0x8584, 0x0000, 0x0000, 0x0002, /*10584*/
  0xFBC2, 0x0020, 0x0002, 0x8585, 0x0000, 0x0000, 0x0002, /*10585*/
  0xFBC2, 0x0020, 0x0002, 0x8586, 0x0000, 0x0000, 0x0002, /*10586*/
  0xFBC2, 0x0020, 0x0002, 0x8587, 0x0000, 0x0000, 0x0002, /*10587*/
  0xFBC2, 0x0020, 0x0002, 0x8588, 0x0000, 0x0000, 0x0002, /*10588*/
  0xFBC2, 0x0020, 0x0002, 0x8589, 0x0000, 0x0000, 0x0002, /*10589*/
  0xFBC2, 0x0020, 0x0002, 0x858A, 0x0000, 0x0000, 0x0002, /*1058A*/
  0xFBC2, 0x0020, 0x0002, 0x858B, 0x0000, 0x0000, 0x0002, /*1058B*/
  0xFBC2, 0x0020, 0x0002, 0x858C, 0x0000, 0x0000, 0x0002, /*1058C*/
  0xFBC2, 0x0020, 0x0002, 0x858D, 0x0000, 0x0000, 0x0002, /*1058D*/
  0xFBC2, 0x0020, 0x0002, 0x858E, 0x0000, 0x0000, 0x0002, /*1058E*/
  0xFBC2, 0x0020, 0x0002, 0x858F, 0x0000, 0x0000, 0x0002, /*1058F*/
  0xFBC2, 0x0020, 0x0002, 0x8590, 0x0000, 0x0000, 0x0002, /*10590*/
  0xFBC2, 0x0020, 0x0002, 0x8591, 0x0000, 0x0000, 0x0002, /*10591*/
  0xFBC2, 0x0020, 0x0002, 0x8592, 0x0000, 0x0000, 0x0002, /*10592*/
  0xFBC2, 0x0020, 0x0002, 0x8593, 0x0000, 0x0000, 0x0002, /*10593*/
  0xFBC2, 0x0020, 0x0002, 0x8594, 0x0000, 0x0000, 0x0002, /*10594*/
  0xFBC2, 0x0020, 0x0002, 0x8595, 0x0000, 0x0000, 0x0002, /*10595*/
  0xFBC2, 0x0020, 0x0002, 0x8596, 0x0000, 0x0000, 0x0002, /*10596*/
  0xFBC2, 0x0020, 0x0002, 0x8597, 0x0000, 0x0000, 0x0002, /*10597*/
  0xFBC2, 0x0020, 0x0002, 0x8598, 0x0000, 0x0000, 0x0002, /*10598*/
  0xFBC2, 0x0020, 0x0002, 0x8599, 0x0000, 0x0000, 0x0002, /*10599*/
  0xFBC2, 0x0020, 0x0002, 0x859A, 0x0000, 0x0000, 0x0002, /*1059A*/
  0xFBC2, 0x0020, 0x0002, 0x859B, 0x0000, 0x0000, 0x0002, /*1059B*/
  0xFBC2, 0x0020, 0x0002, 0x859C, 0x0000, 0x0000, 0x0002, /*1059C*/
  0xFBC2, 0x0020, 0x0002, 0x859D, 0x0000, 0x0000, 0x0002, /*1059D*/
  0xFBC2, 0x0020, 0x0002, 0x859E, 0x0000, 0x0000, 0x0002, /*1059E*/
  0xFBC2, 0x0020, 0x0002, 0x859F, 0x0000, 0x0000, 0x0002, /*1059F*/
  0xFBC2, 0x0020, 0x0002, 0x85A0, 0x0000, 0x0000, 0x0002, /*105A0*/
  0xFBC2, 0x0020, 0x0002, 0x85A1, 0x0000, 0x0000, 0x0002, /*105A1*/
  0xFBC2, 0x0020, 0x0002, 0x85A2, 0x0000, 0x0000, 0x0002, /*105A2*/
  0xFBC2, 0x0020, 0x0002, 0x85A3, 0x0000, 0x0000, 0x0002, /*105A3*/
  0xFBC2, 0x0020, 0x0002, 0x85A4, 0x0000, 0x0000, 0x0002, /*105A4*/
  0xFBC2, 0x0020, 0x0002, 0x85A5, 0x0000, 0x0000, 0x0002, /*105A5*/
  0xFBC2, 0x0020, 0x0002, 0x85A6, 0x0000, 0x0000, 0x0002, /*105A6*/
  0xFBC2, 0x0020, 0x0002, 0x85A7, 0x0000, 0x0000, 0x0002, /*105A7*/
  0xFBC2, 0x0020, 0x0002, 0x85A8, 0x0000, 0x0000, 0x0002, /*105A8*/
  0xFBC2, 0x0020, 0x0002, 0x85A9, 0x0000, 0x0000, 0x0002, /*105A9*/
  0xFBC2, 0x0020, 0x0002, 0x85AA, 0x0000, 0x0000, 0x0002, /*105AA*/
  0xFBC2, 0x0020, 0x0002, 0x85AB, 0x0000, 0x0000, 0x0002, /*105AB*/
  0xFBC2, 0x0020, 0x0002, 0x85AC, 0x0000, 0x0000, 0x0002, /*105AC*/
  0xFBC2, 0x0020, 0x0002, 0x85AD, 0x0000, 0x0000, 0x0002, /*105AD*/
  0xFBC2, 0x0020, 0x0002, 0x85AE, 0x0000, 0x0000, 0x0002, /*105AE*/
  0xFBC2, 0x0020, 0x0002, 0x85AF, 0x0000, 0x0000, 0x0002, /*105AF*/
  0xFBC2, 0x0020, 0x0002, 0x85B0, 0x0000, 0x0000, 0x0002, /*105B0*/
  0xFBC2, 0x0020, 0x0002, 0x85B1, 0x0000, 0x0000, 0x0002, /*105B1*/
  0xFBC2, 0x0020, 0x0002, 0x85B2, 0x0000, 0x0000, 0x0002, /*105B2*/
  0xFBC2, 0x0020, 0x0002, 0x85B3, 0x0000, 0x0000, 0x0002, /*105B3*/
  0xFBC2, 0x0020, 0x0002, 0x85B4, 0x0000, 0x0000, 0x0002, /*105B4*/
  0xFBC2, 0x0020, 0x0002, 0x85B5, 0x0000, 0x0000, 0x0002, /*105B5*/
  0xFBC2, 0x0020, 0x0002, 0x85B6, 0x0000, 0x0000, 0x0002, /*105B6*/
  0xFBC2, 0x0020, 0x0002, 0x85B7, 0x0000, 0x0000, 0x0002, /*105B7*/
  0xFBC2, 0x0020, 0x0002, 0x85B8, 0x0000, 0x0000, 0x0002, /*105B8*/
  0xFBC2, 0x0020, 0x0002, 0x85B9, 0x0000, 0x0000, 0x0002, /*105B9*/
  0xFBC2, 0x0020, 0x0002, 0x85BA, 0x0000, 0x0000, 0x0002, /*105BA*/
  0xFBC2, 0x0020, 0x0002, 0x85BB, 0x0000, 0x0000, 0x0002, /*105BB*/
  0xFBC2, 0x0020, 0x0002, 0x85BC, 0x0000, 0x0000, 0x0002, /*105BC*/
  0xFBC2, 0x0020, 0x0002, 0x85BD, 0x0000, 0x0000, 0x0002, /*105BD*/
  0xFBC2, 0x0020, 0x0002, 0x85BE, 0x0000, 0x0000, 0x0002, /*105BE*/
  0xFBC2, 0x0020, 0x0002, 0x85BF, 0x0000, 0x0000, 0x0002, /*105BF*/
  0xFBC2, 0x0020, 0x0002, 0x85C0, 0x0000, 0x0000, 0x0002, /*105C0*/
  0xFBC2, 0x0020, 0x0002, 0x85C1, 0x0000, 0x0000, 0x0002, /*105C1*/
  0xFBC2, 0x0020, 0x0002, 0x85C2, 0x0000, 0x0000, 0x0002, /*105C2*/
  0xFBC2, 0x0020, 0x0002, 0x85C3, 0x0000, 0x0000, 0x0002, /*105C3*/
  0xFBC2, 0x0020, 0x0002, 0x85C4, 0x0000, 0x0000, 0x0002, /*105C4*/
  0xFBC2, 0x0020, 0x0002, 0x85C5, 0x0000, 0x0000, 0x0002, /*105C5*/
  0xFBC2, 0x0020, 0x0002, 0x85C6, 0x0000, 0x0000, 0x0002, /*105C6*/
  0xFBC2, 0x0020, 0x0002, 0x85C7, 0x0000, 0x0000, 0x0002, /*105C7*/
  0xFBC2, 0x0020, 0x0002, 0x85C8, 0x0000, 0x0000, 0x0002, /*105C8*/
  0xFBC2, 0x0020, 0x0002, 0x85C9, 0x0000, 0x0000, 0x0002, /*105C9*/
  0xFBC2, 0x0020, 0x0002, 0x85CA, 0x0000, 0x0000, 0x0002, /*105CA*/
  0xFBC2, 0x0020, 0x0002, 0x85CB, 0x0000, 0x0000, 0x0002, /*105CB*/
  0xFBC2, 0x0020, 0x0002, 0x85CC, 0x0000, 0x0000, 0x0002, /*105CC*/
  0xFBC2, 0x0020, 0x0002, 0x85CD, 0x0000, 0x0000, 0x0002, /*105CD*/
  0xFBC2, 0x0020, 0x0002, 0x85CE, 0x0000, 0x0000, 0x0002, /*105CE*/
  0xFBC2, 0x0020, 0x0002, 0x85CF, 0x0000, 0x0000, 0x0002, /*105CF*/
  0xFBC2, 0x0020, 0x0002, 0x85D0, 0x0000, 0x0000, 0x0002, /*105D0*/
  0xFBC2, 0x0020, 0x0002, 0x85D1, 0x0000, 0x0000, 0x0002, /*105D1*/
  0xFBC2, 0x0020, 0x0002, 0x85D2, 0x0000, 0x0000, 0x0002, /*105D2*/
  0xFBC2, 0x0020, 0x0002, 0x85D3, 0x0000, 0x0000, 0x0002, /*105D3*/
  0xFBC2, 0x0020, 0x0002, 0x85D4, 0x0000, 0x0000, 0x0002, /*105D4*/
  0xFBC2, 0x0020, 0x0002, 0x85D5, 0x0000, 0x0000, 0x0002, /*105D5*/
  0xFBC2, 0x0020, 0x0002, 0x85D6, 0x0000, 0x0000, 0x0002, /*105D6*/
  0xFBC2, 0x0020, 0x0002, 0x85D7, 0x0000, 0x0000, 0x0002, /*105D7*/
  0xFBC2, 0x0020, 0x0002, 0x85D8, 0x0000, 0x0000, 0x0002, /*105D8*/
  0xFBC2, 0x0020, 0x0002, 0x85D9, 0x0000, 0x0000, 0x0002, /*105D9*/
  0xFBC2, 0x0020, 0x0002, 0x85DA, 0x0000, 0x0000, 0x0002, /*105DA*/
  0xFBC2, 0x0020, 0x0002, 0x85DB, 0x0000, 0x0000, 0x0002, /*105DB*/
  0xFBC2, 0x0020, 0x0002, 0x85DC, 0x0000, 0x0000, 0x0002, /*105DC*/
  0xFBC2, 0x0020, 0x0002, 0x85DD, 0x0000, 0x0000, 0x0002, /*105DD*/
  0xFBC2, 0x0020, 0x0002, 0x85DE, 0x0000, 0x0000, 0x0002, /*105DE*/
  0xFBC2, 0x0020, 0x0002, 0x85DF, 0x0000, 0x0000, 0x0002, /*105DF*/
  0xFBC2, 0x0020, 0x0002, 0x85E0, 0x0000, 0x0000, 0x0002, /*105E0*/
  0xFBC2, 0x0020, 0x0002, 0x85E1, 0x0000, 0x0000, 0x0002, /*105E1*/
  0xFBC2, 0x0020, 0x0002, 0x85E2, 0x0000, 0x0000, 0x0002, /*105E2*/
  0xFBC2, 0x0020, 0x0002, 0x85E3, 0x0000, 0x0000, 0x0002, /*105E3*/
  0xFBC2, 0x0020, 0x0002, 0x85E4, 0x0000, 0x0000, 0x0002, /*105E4*/
  0xFBC2, 0x0020, 0x0002, 0x85E5, 0x0000, 0x0000, 0x0002, /*105E5*/
  0xFBC2, 0x0020, 0x0002, 0x85E6, 0x0000, 0x0000, 0x0002, /*105E6*/
  0xFBC2, 0x0020, 0x0002, 0x85E7, 0x0000, 0x0000, 0x0002, /*105E7*/
  0xFBC2, 0x0020, 0x0002, 0x85E8, 0x0000, 0x0000, 0x0002, /*105E8*/
  0xFBC2, 0x0020, 0x0002, 0x85E9, 0x0000, 0x0000, 0x0002, /*105E9*/
  0xFBC2, 0x0020, 0x0002, 0x85EA, 0x0000, 0x0000, 0x0002, /*105EA*/
  0xFBC2, 0x0020, 0x0002, 0x85EB, 0x0000, 0x0000, 0x0002, /*105EB*/
  0xFBC2, 0x0020, 0x0002, 0x85EC, 0x0000, 0x0000, 0x0002, /*105EC*/
  0xFBC2, 0x0020, 0x0002, 0x85ED, 0x0000, 0x0000, 0x0002, /*105ED*/
  0xFBC2, 0x0020, 0x0002, 0x85EE, 0x0000, 0x0000, 0x0002, /*105EE*/
  0xFBC2, 0x0020, 0x0002, 0x85EF, 0x0000, 0x0000, 0x0002, /*105EF*/
  0xFBC2, 0x0020, 0x0002, 0x85F0, 0x0000, 0x0000, 0x0002, /*105F0*/
  0xFBC2, 0x0020, 0x0002, 0x85F1, 0x0000, 0x0000, 0x0002, /*105F1*/
  0xFBC2, 0x0020, 0x0002, 0x85F2, 0x0000, 0x0000, 0x0002, /*105F2*/
  0xFBC2, 0x0020, 0x0002, 0x85F3, 0x0000, 0x0000, 0x0002, /*105F3*/
  0xFBC2, 0x0020, 0x0002, 0x85F4, 0x0000, 0x0000, 0x0002, /*105F4*/
  0xFBC2, 0x0020, 0x0002, 0x85F5, 0x0000, 0x0000, 0x0002, /*105F5*/
  0xFBC2, 0x0020, 0x0002, 0x85F6, 0x0000, 0x0000, 0x0002, /*105F6*/
  0xFBC2, 0x0020, 0x0002, 0x85F7, 0x0000, 0x0000, 0x0002, /*105F7*/
  0xFBC2, 0x0020, 0x0002, 0x85F8, 0x0000, 0x0000, 0x0002, /*105F8*/
  0xFBC2, 0x0020, 0x0002, 0x85F9, 0x0000, 0x0000, 0x0002, /*105F9*/
  0xFBC2, 0x0020, 0x0002, 0x85FA, 0x0000, 0x0000, 0x0002, /*105FA*/
  0xFBC2, 0x0020, 0x0002, 0x85FB, 0x0000, 0x0000, 0x0002, /*105FB*/
  0xFBC2, 0x0020, 0x0002, 0x85FC, 0x0000, 0x0000, 0x0002, /*105FC*/
  0xFBC2, 0x0020, 0x0002, 0x85FD, 0x0000, 0x0000, 0x0002, /*105FD*/
  0xFBC2, 0x0020, 0x0002, 0x85FE, 0x0000, 0x0000, 0x0002, /*105FE*/
  0xFBC2, 0x0020, 0x0002, 0x85FF, 0x0000, 0x0000, 0x0002  /*105FF*/
};

uint16 uca900_p106[]= {
  0x46BA, 0x0020, 0x0002, 0x0001, /*10600*/
  0x46BB, 0x0020, 0x0002, 0x0001, /*10601*/
  0x46BC, 0x0020, 0x0002, 0x0001, /*10602*/
  0x46BD, 0x0020, 0x0002, 0x0001, /*10603*/
  0x46BE, 0x0020, 0x0002, 0x0001, /*10604*/
  0x46BF, 0x0020, 0x0002, 0x0001, /*10605*/
  0x46C0, 0x0020, 0x0002, 0x0001, /*10606*/
  0x46C1, 0x0020, 0x0002, 0x0001, /*10607*/
  0x46C2, 0x0020, 0x0002, 0x0001, /*10608*/
  0x46C3, 0x0020, 0x0002, 0x0001, /*10609*/
  0x46C4, 0x0020, 0x0002, 0x0001, /*1060A*/
  0x46C5, 0x0020, 0x0002, 0x0001, /*1060B*/
  0x46C6, 0x0020, 0x0002, 0x0001, /*1060C*/
  0x46C7, 0x0020, 0x0002, 0x0001, /*1060D*/
  0x46C8, 0x0020, 0x0002, 0x0001, /*1060E*/
  0x46C9, 0x0020, 0x0002, 0x0001, /*1060F*/
  0x46CA, 0x0020, 0x0002, 0x0001, /*10610*/
  0x46CB, 0x0020, 0x0002, 0x0001, /*10611*/
  0x46CC, 0x0020, 0x0002, 0x0001, /*10612*/
  0x46CD, 0x0020, 0x0002, 0x0001, /*10613*/
  0x46CE, 0x0020, 0x0002, 0x0001, /*10614*/
  0x46CF, 0x0020, 0x0002, 0x0001, /*10615*/
  0x46D0, 0x0020, 0x0002, 0x0001, /*10616*/
  0x46D1, 0x0020, 0x0002, 0x0001, /*10617*/
  0x46D2, 0x0020, 0x0002, 0x0001, /*10618*/
  0x46D3, 0x0020, 0x0002, 0x0001, /*10619*/
  0x46D4, 0x0020, 0x0002, 0x0001, /*1061A*/
  0x46D5, 0x0020, 0x0002, 0x0001, /*1061B*/
  0x46D6, 0x0020, 0x0002, 0x0001, /*1061C*/
  0x46D7, 0x0020, 0x0002, 0x0001, /*1061D*/
  0x46D8, 0x0020, 0x0002, 0x0001, /*1061E*/
  0x46D9, 0x0020, 0x0002, 0x0001, /*1061F*/
  0x46DA, 0x0020, 0x0002, 0x0001, /*10620*/
  0x46DB, 0x0020, 0x0002, 0x0001, /*10621*/
  0x46DC, 0x0020, 0x0002, 0x0001, /*10622*/
  0x46DD, 0x0020, 0x0002, 0x0001, /*10623*/
  0x46DE, 0x0020, 0x0002, 0x0001, /*10624*/
  0x46DF, 0x0020, 0x0002, 0x0001, /*10625*/
  0x46E0, 0x0020, 0x0002, 0x0001, /*10626*/
  0x46E1, 0x0020, 0x0002, 0x0001, /*10627*/
  0x46E2, 0x0020, 0x0002, 0x0001, /*10628*/
  0x46E3, 0x0020, 0x0002, 0x0001, /*10629*/
  0x46E4, 0x0020, 0x0002, 0x0001, /*1062A*/
  0x46E5, 0x0020, 0x0002, 0x0001, /*1062B*/
  0x46E6, 0x0020, 0x0002, 0x0001, /*1062C*/
  0x46E7, 0x0020, 0x0002, 0x0001, /*1062D*/
  0x46E8, 0x0020, 0x0002, 0x0001, /*1062E*/
  0x46E9, 0x0020, 0x0002, 0x0001, /*1062F*/
  0x46EA, 0x0020, 0x0002, 0x0001, /*10630*/
  0x46EB, 0x0020, 0x0002, 0x0001, /*10631*/
  0x46EC, 0x0020, 0x0002, 0x0001, /*10632*/
  0x46ED, 0x0020, 0x0002, 0x0001, /*10633*/
  0x46EE, 0x0020, 0x0002, 0x0001, /*10634*/
  0x46EF, 0x0020, 0x0002, 0x0001, /*10635*/
  0x46F0, 0x0020, 0x0002, 0x0001, /*10636*/
  0x46F1, 0x0020, 0x0002, 0x0001, /*10637*/
  0x46F2, 0x0020, 0x0002, 0x0001, /*10638*/
  0x46F3, 0x0020, 0x0002, 0x0001, /*10639*/
  0x46F4, 0x0020, 0x0002, 0x0001, /*1063A*/
  0x46F5, 0x0020, 0x0002, 0x0001, /*1063B*/
  0x46F6, 0x0020, 0x0002, 0x0001, /*1063C*/
  0x46F7, 0x0020, 0x0002, 0x0001, /*1063D*/
  0x46F8, 0x0020, 0x0002, 0x0001, /*1063E*/
  0x46F9, 0x0020, 0x0002, 0x0001, /*1063F*/
  0x46FA, 0x0020, 0x0002, 0x0001, /*10640*/
  0x46FB, 0x0020, 0x0002, 0x0001, /*10641*/
  0x46FC, 0x0020, 0x0002, 0x0001, /*10642*/
  0x46FD, 0x0020, 0x0002, 0x0001, /*10643*/
  0x46FE, 0x0020, 0x0002, 0x0001, /*10644*/
  0x46FF, 0x0020, 0x0002, 0x0001, /*10645*/
  0x4700, 0x0020, 0x0002, 0x0001, /*10646*/
  0x4701, 0x0020, 0x0002, 0x0001, /*10647*/
  0x4702, 0x0020, 0x0002, 0x0001, /*10648*/
  0x4703, 0x0020, 0x0002, 0x0001, /*10649*/
  0x4704, 0x0020, 0x0002, 0x0001, /*1064A*/
  0x4705, 0x0020, 0x0002, 0x0001, /*1064B*/
  0x4706, 0x0020, 0x0002, 0x0001, /*1064C*/
  0x4707, 0x0020, 0x0002, 0x0001, /*1064D*/
  0x4708, 0x0020, 0x0002, 0x0001, /*1064E*/
  0x4709, 0x0020, 0x0002, 0x0001, /*1064F*/
  0x470A, 0x0020, 0x0002, 0x0001, /*10650*/
  0x470B, 0x0020, 0x0002, 0x0001, /*10651*/
  0x470C, 0x0020, 0x0002, 0x0001, /*10652*/
  0x470D, 0x0020, 0x0002, 0x0001, /*10653*/
  0x470E, 0x0020, 0x0002, 0x0001, /*10654*/
  0x470F, 0x0020, 0x0002, 0x0001, /*10655*/
  0x4710, 0x0020, 0x0002, 0x0001, /*10656*/
  0x4711, 0x0020, 0x0002, 0x0001, /*10657*/
  0x4712, 0x0020, 0x0002, 0x0001, /*10658*/
  0x4713, 0x0020, 0x0002, 0x0001, /*10659*/
  0x4714, 0x0020, 0x0002, 0x0001, /*1065A*/
  0x4715, 0x0020, 0x0002, 0x0001, /*1065B*/
  0x4716, 0x0020, 0x0002, 0x0001, /*1065C*/
  0x4717, 0x0020, 0x0002, 0x0001, /*1065D*/
  0x4718, 0x0020, 0x0002, 0x0001, /*1065E*/
  0x4719, 0x0020, 0x0002, 0x0001, /*1065F*/
  0x471A, 0x0020, 0x0002, 0x0001, /*10660*/
  0x471B, 0x0020, 0x0002, 0x0001, /*10661*/
  0x471C, 0x0020, 0x0002, 0x0001, /*10662*/
  0x471D, 0x0020, 0x0002, 0x0001, /*10663*/
  0x471E, 0x0020, 0x0002, 0x0001, /*10664*/
  0x471F, 0x0020, 0x0002, 0x0001, /*10665*/
  0x4720, 0x0020, 0x0002, 0x0001, /*10666*/
  0x4721, 0x0020, 0x0002, 0x0001, /*10667*/
  0x4722, 0x0020, 0x0002, 0x0001, /*10668*/
  0x4723, 0x0020, 0x0002, 0x0001, /*10669*/
  0x4724, 0x0020, 0x0002, 0x0001, /*1066A*/
  0x4725, 0x0020, 0x0002, 0x0001, /*1066B*/
  0x4726, 0x0020, 0x0002, 0x0001, /*1066C*/
  0x4727, 0x0020, 0x0002, 0x0001, /*1066D*/
  0x4728, 0x0020, 0x0002, 0x0001, /*1066E*/
  0x4729, 0x0020, 0x0002, 0x0001, /*1066F*/
  0x472A, 0x0020, 0x0002, 0x0001, /*10670*/
  0x472B, 0x0020, 0x0002, 0x0001, /*10671*/
  0x472C, 0x0020, 0x0002, 0x0001, /*10672*/
  0x472D, 0x0020, 0x0002, 0x0001, /*10673*/
  0x472E, 0x0020, 0x0002, 0x0001, /*10674*/
  0x472F, 0x0020, 0x0002, 0x0001, /*10675*/
  0x4730, 0x0020, 0x0002, 0x0001, /*10676*/
  0x4731, 0x0020, 0x0002, 0x0001, /*10677*/
  0x4732, 0x0020, 0x0002, 0x0001, /*10678*/
  0x4733, 0x0020, 0x0002, 0x0001, /*10679*/
  0x4734, 0x0020, 0x0002, 0x0001, /*1067A*/
  0x4735, 0x0020, 0x0002, 0x0001, /*1067B*/
  0x4736, 0x0020, 0x0002, 0x0001, /*1067C*/
  0x4737, 0x0020, 0x0002, 0x0001, /*1067D*/
  0x4738, 0x0020, 0x0002, 0x0001, /*1067E*/
  0x4739, 0x0020, 0x0002, 0x0001, /*1067F*/
  0x473A, 0x0020, 0x0002, 0x0001, /*10680*/
  0x473B, 0x0020, 0x0002, 0x0001, /*10681*/
  0x473C, 0x0020, 0x0002, 0x0001, /*10682*/
  0x473D, 0x0020, 0x0002, 0x0001, /*10683*/
  0x473E, 0x0020, 0x0002, 0x0001, /*10684*/
  0x473F, 0x0020, 0x0002, 0x0001, /*10685*/
  0x4740, 0x0020, 0x0002, 0x0001, /*10686*/
  0x4741, 0x0020, 0x0002, 0x0001, /*10687*/
  0x4742, 0x0020, 0x0002, 0x0001, /*10688*/
  0x4743, 0x0020, 0x0002, 0x0001, /*10689*/
  0x4744, 0x0020, 0x0002, 0x0001, /*1068A*/
  0x4745, 0x0020, 0x0002, 0x0001, /*1068B*/
  0x4746, 0x0020, 0x0002, 0x0001, /*1068C*/
  0x4747, 0x0020, 0x0002, 0x0001, /*1068D*/
  0x4748, 0x0020, 0x0002, 0x0001, /*1068E*/
  0x4749, 0x0020, 0x0002, 0x0001, /*1068F*/
  0x474A, 0x0020, 0x0002, 0x0001, /*10690*/
  0x474B, 0x0020, 0x0002, 0x0001, /*10691*/
  0x474C, 0x0020, 0x0002, 0x0001, /*10692*/
  0x474D, 0x0020, 0x0002, 0x0001, /*10693*/
  0x474E, 0x0020, 0x0002, 0x0001, /*10694*/
  0x474F, 0x0020, 0x0002, 0x0001, /*10695*/
  0x4750, 0x0020, 0x0002, 0x0001, /*10696*/
  0x4751, 0x0020, 0x0002, 0x0001, /*10697*/
  0x4752, 0x0020, 0x0002, 0x0001, /*10698*/
  0x4753, 0x0020, 0x0002, 0x0001, /*10699*/
  0x4754, 0x0020, 0x0002, 0x0001, /*1069A*/
  0x4755, 0x0020, 0x0002, 0x0001, /*1069B*/
  0x4756, 0x0020, 0x0002, 0x0001, /*1069C*/
  0x4757, 0x0020, 0x0002, 0x0001, /*1069D*/
  0x4758, 0x0020, 0x0002, 0x0001, /*1069E*/
  0x4759, 0x0020, 0x0002, 0x0001, /*1069F*/
  0x475A, 0x0020, 0x0002, 0x0001, /*106A0*/
  0x475B, 0x0020, 0x0002, 0x0001, /*106A1*/
  0x475C, 0x0020, 0x0002, 0x0001, /*106A2*/
  0x475D, 0x0020, 0x0002, 0x0001, /*106A3*/
  0x475E, 0x0020, 0x0002, 0x0001, /*106A4*/
  0x475F, 0x0020, 0x0002, 0x0001, /*106A5*/
  0x4760, 0x0020, 0x0002, 0x0001, /*106A6*/
  0x4761, 0x0020, 0x0002, 0x0001, /*106A7*/
  0x4762, 0x0020, 0x0002, 0x0001, /*106A8*/
  0x4763, 0x0020, 0x0002, 0x0001, /*106A9*/
  0x4764, 0x0020, 0x0002, 0x0001, /*106AA*/
  0x4765, 0x0020, 0x0002, 0x0001, /*106AB*/
  0x4766, 0x0020, 0x0002, 0x0001, /*106AC*/
  0x4767, 0x0020, 0x0002, 0x0001, /*106AD*/
  0x4768, 0x0020, 0x0002, 0x0001, /*106AE*/
  0x4769, 0x0020, 0x0002, 0x0001, /*106AF*/
  0x476A, 0x0020, 0x0002, 0x0001, /*106B0*/
  0x476B, 0x0020, 0x0002, 0x0001, /*106B1*/
  0x476C, 0x0020, 0x0002, 0x0001, /*106B2*/
  0x476D, 0x0020, 0x0002, 0x0001, /*106B3*/
  0x476E, 0x0020, 0x0002, 0x0001, /*106B4*/
  0x476F, 0x0020, 0x0002, 0x0001, /*106B5*/
  0x4770, 0x0020, 0x0002, 0x0001, /*106B6*/
  0x4771, 0x0020, 0x0002, 0x0001, /*106B7*/
  0x4772, 0x0020, 0x0002, 0x0001, /*106B8*/
  0x4773, 0x0020, 0x0002, 0x0001, /*106B9*/
  0x4774, 0x0020, 0x0002, 0x0001, /*106BA*/
  0x4775, 0x0020, 0x0002, 0x0001, /*106BB*/
  0x4776, 0x0020, 0x0002, 0x0001, /*106BC*/
  0x4777, 0x0020, 0x0002, 0x0001, /*106BD*/
  0x4778, 0x0020, 0x0002, 0x0001, /*106BE*/
  0x4779, 0x0020, 0x0002, 0x0001, /*106BF*/
  0x477A, 0x0020, 0x0002, 0x0001, /*106C0*/
  0x477B, 0x0020, 0x0002, 0x0001, /*106C1*/
  0x477C, 0x0020, 0x0002, 0x0001, /*106C2*/
  0x477D, 0x0020, 0x0002, 0x0001, /*106C3*/
  0x477E, 0x0020, 0x0002, 0x0001, /*106C4*/
  0x477F, 0x0020, 0x0002, 0x0001, /*106C5*/
  0x4780, 0x0020, 0x0002, 0x0001, /*106C6*/
  0x4781, 0x0020, 0x0002, 0x0001, /*106C7*/
  0x4782, 0x0020, 0x0002, 0x0001, /*106C8*/
  0x4783, 0x0020, 0x0002, 0x0001, /*106C9*/
  0x4784, 0x0020, 0x0002, 0x0001, /*106CA*/
  0x4785, 0x0020, 0x0002, 0x0001, /*106CB*/
  0x4786, 0x0020, 0x0002, 0x0001, /*106CC*/
  0x4787, 0x0020, 0x0002, 0x0001, /*106CD*/
  0x4788, 0x0020, 0x0002, 0x0001, /*106CE*/
  0x4789, 0x0020, 0x0002, 0x0001, /*106CF*/
  0x478A, 0x0020, 0x0002, 0x0001, /*106D0*/
  0x478B, 0x0020, 0x0002, 0x0001, /*106D1*/
  0x478C, 0x0020, 0x0002, 0x0001, /*106D2*/
  0x478D, 0x0020, 0x0002, 0x0001, /*106D3*/
  0x478E, 0x0020, 0x0002, 0x0001, /*106D4*/
  0x478F, 0x0020, 0x0002, 0x0001, /*106D5*/
  0x4790, 0x0020, 0x0002, 0x0001, /*106D6*/
  0x4791, 0x0020, 0x0002, 0x0001, /*106D7*/
  0x4792, 0x0020, 0x0002, 0x0001, /*106D8*/
  0x4793, 0x0020, 0x0002, 0x0001, /*106D9*/
  0x4794, 0x0020, 0x0002, 0x0001, /*106DA*/
  0x4795, 0x0020, 0x0002, 0x0001, /*106DB*/
  0x4796, 0x0020, 0x0002, 0x0001, /*106DC*/
  0x4797, 0x0020, 0x0002, 0x0001, /*106DD*/
  0x4798, 0x0020, 0x0002, 0x0001, /*106DE*/
  0x4799, 0x0020, 0x0002, 0x0001, /*106DF*/
  0x479A, 0x0020, 0x0002, 0x0001, /*106E0*/
  0x479B, 0x0020, 0x0002, 0x0001, /*106E1*/
  0x479C, 0x0020, 0x0002, 0x0001, /*106E2*/
  0x479D, 0x0020, 0x0002, 0x0001, /*106E3*/
  0x479E, 0x0020, 0x0002, 0x0001, /*106E4*/
  0x479F, 0x0020, 0x0002, 0x0001, /*106E5*/
  0x47A0, 0x0020, 0x0002, 0x0001, /*106E6*/
  0x47A1, 0x0020, 0x0002, 0x0001, /*106E7*/
  0x47A2, 0x0020, 0x0002, 0x0001, /*106E8*/
  0x47A3, 0x0020, 0x0002, 0x0001, /*106E9*/
  0x47A4, 0x0020, 0x0002, 0x0001, /*106EA*/
  0x47A5, 0x0020, 0x0002, 0x0001, /*106EB*/
  0x47A6, 0x0020, 0x0002, 0x0001, /*106EC*/
  0x47A7, 0x0020, 0x0002, 0x0001, /*106ED*/
  0x47A8, 0x0020, 0x0002, 0x0001, /*106EE*/
  0x47A9, 0x0020, 0x0002, 0x0001, /*106EF*/
  0x47AA, 0x0020, 0x0002, 0x0001, /*106F0*/
  0x47AB, 0x0020, 0x0002, 0x0001, /*106F1*/
  0x47AC, 0x0020, 0x0002, 0x0001, /*106F2*/
  0x47AD, 0x0020, 0x0002, 0x0001, /*106F3*/
  0x47AE, 0x0020, 0x0002, 0x0001, /*106F4*/
  0x47AF, 0x0020, 0x0002, 0x0001, /*106F5*/
  0x47B0, 0x0020, 0x0002, 0x0001, /*106F6*/
  0x47B1, 0x0020, 0x0002, 0x0001, /*106F7*/
  0x47B2, 0x0020, 0x0002, 0x0001, /*106F8*/
  0x47B3, 0x0020, 0x0002, 0x0001, /*106F9*/
  0x47B4, 0x0020, 0x0002, 0x0001, /*106FA*/
  0x47B5, 0x0020, 0x0002, 0x0001, /*106FB*/
  0x47B6, 0x0020, 0x0002, 0x0001, /*106FC*/
  0x47B7, 0x0020, 0x0002, 0x0001, /*106FD*/
  0x47B8, 0x0020, 0x0002, 0x0001, /*106FE*/
  0x47B9, 0x0020, 0x0002, 0x0001  /*106FF*/
};

uint16 uca900_p107[]= {
  0x47BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10700*/
  0x47BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10701*/
  0x47BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10702*/
  0x47BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10703*/
  0x47BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10704*/
  0x47BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10705*/
  0x47C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10706*/
  0x47C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10707*/
  0x47C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10708*/
  0x47C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10709*/
  0x47C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1070A*/
  0x47C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1070B*/
  0x47C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1070C*/
  0x47C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1070D*/
  0x47C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1070E*/
  0x47C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1070F*/
  0x47CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10710*/
  0x47CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10711*/
  0x47CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10712*/
  0x47CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10713*/
  0x47CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10714*/
  0x47CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10715*/
  0x47D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10716*/
  0x47D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10717*/
  0x47D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10718*/
  0x47D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10719*/
  0x47D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1071A*/
  0x47D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1071B*/
  0x47D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1071C*/
  0x47D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1071D*/
  0x47D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1071E*/
  0x47D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1071F*/
  0x47DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10720*/
  0x47DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10721*/
  0x47DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10722*/
  0x47DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10723*/
  0x47DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10724*/
  0x47DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10725*/
  0x47E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10726*/
  0x47E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10727*/
  0x47E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10728*/
  0x47E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10729*/
  0x47E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1072A*/
  0x47E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1072B*/
  0x47E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1072C*/
  0x47E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1072D*/
  0x47E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1072E*/
  0x47E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1072F*/
  0x47EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10730*/
  0x47EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10731*/
  0x47EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10732*/
  0x47ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10733*/
  0x47EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10734*/
  0x47EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10735*/
  0x47F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10736*/
  0xFBC2, 0x0020, 0x0002, 0x8737, 0x0000, 0x0000, 0x0002, /*10737*/
  0xFBC2, 0x0020, 0x0002, 0x8738, 0x0000, 0x0000, 0x0002, /*10738*/
  0xFBC2, 0x0020, 0x0002, 0x8739, 0x0000, 0x0000, 0x0002, /*10739*/
  0xFBC2, 0x0020, 0x0002, 0x873A, 0x0000, 0x0000, 0x0002, /*1073A*/
  0xFBC2, 0x0020, 0x0002, 0x873B, 0x0000, 0x0000, 0x0002, /*1073B*/
  0xFBC2, 0x0020, 0x0002, 0x873C, 0x0000, 0x0000, 0x0002, /*1073C*/
  0xFBC2, 0x0020, 0x0002, 0x873D, 0x0000, 0x0000, 0x0002, /*1073D*/
  0xFBC2, 0x0020, 0x0002, 0x873E, 0x0000, 0x0000, 0x0002, /*1073E*/
  0xFBC2, 0x0020, 0x0002, 0x873F, 0x0000, 0x0000, 0x0002, /*1073F*/
  0x47F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10740*/
  0x47F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10741*/
  0x47F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10742*/
  0x47F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10743*/
  0x47F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10744*/
  0x47F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10745*/
  0x47F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10746*/
  0x47F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10747*/
  0x47F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10748*/
  0x47FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10749*/
  0x47FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1074A*/
  0x47FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1074B*/
  0x47FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1074C*/
  0x47FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1074D*/
  0x47FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1074E*/
  0x4800, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1074F*/
  0x4801, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10750*/
  0x4802, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10751*/
  0x4803, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10752*/
  0x4804, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10753*/
  0x4805, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10754*/
  0x4806, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10755*/
  0xFBC2, 0x0020, 0x0002, 0x8756, 0x0000, 0x0000, 0x0002, /*10756*/
  0xFBC2, 0x0020, 0x0002, 0x8757, 0x0000, 0x0000, 0x0002, /*10757*/
  0xFBC2, 0x0020, 0x0002, 0x8758, 0x0000, 0x0000, 0x0002, /*10758*/
  0xFBC2, 0x0020, 0x0002, 0x8759, 0x0000, 0x0000, 0x0002, /*10759*/
  0xFBC2, 0x0020, 0x0002, 0x875A, 0x0000, 0x0000, 0x0002, /*1075A*/
  0xFBC2, 0x0020, 0x0002, 0x875B, 0x0000, 0x0000, 0x0002, /*1075B*/
  0xFBC2, 0x0020, 0x0002, 0x875C, 0x0000, 0x0000, 0x0002, /*1075C*/
  0xFBC2, 0x0020, 0x0002, 0x875D, 0x0000, 0x0000, 0x0002, /*1075D*/
  0xFBC2, 0x0020, 0x0002, 0x875E, 0x0000, 0x0000, 0x0002, /*1075E*/
  0xFBC2, 0x0020, 0x0002, 0x875F, 0x0000, 0x0000, 0x0002, /*1075F*/
  0x4807, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10760*/
  0x4808, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10761*/
  0x4809, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10762*/
  0x480A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10763*/
  0x480B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10764*/
  0x480C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10765*/
  0x480D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10766*/
  0x480E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10767*/
  0xFBC2, 0x0020, 0x0002, 0x8768, 0x0000, 0x0000, 0x0002, /*10768*/
  0xFBC2, 0x0020, 0x0002, 0x8769, 0x0000, 0x0000, 0x0002, /*10769*/
  0xFBC2, 0x0020, 0x0002, 0x876A, 0x0000, 0x0000, 0x0002, /*1076A*/
  0xFBC2, 0x0020, 0x0002, 0x876B, 0x0000, 0x0000, 0x0002, /*1076B*/
  0xFBC2, 0x0020, 0x0002, 0x876C, 0x0000, 0x0000, 0x0002, /*1076C*/
  0xFBC2, 0x0020, 0x0002, 0x876D, 0x0000, 0x0000, 0x0002, /*1076D*/
  0xFBC2, 0x0020, 0x0002, 0x876E, 0x0000, 0x0000, 0x0002, /*1076E*/
  0xFBC2, 0x0020, 0x0002, 0x876F, 0x0000, 0x0000, 0x0002, /*1076F*/
  0xFBC2, 0x0020, 0x0002, 0x8770, 0x0000, 0x0000, 0x0002, /*10770*/
  0xFBC2, 0x0020, 0x0002, 0x8771, 0x0000, 0x0000, 0x0002, /*10771*/
  0xFBC2, 0x0020, 0x0002, 0x8772, 0x0000, 0x0000, 0x0002, /*10772*/
  0xFBC2, 0x0020, 0x0002, 0x8773, 0x0000, 0x0000, 0x0002, /*10773*/
  0xFBC2, 0x0020, 0x0002, 0x8774, 0x0000, 0x0000, 0x0002, /*10774*/
  0xFBC2, 0x0020, 0x0002, 0x8775, 0x0000, 0x0000, 0x0002, /*10775*/
  0xFBC2, 0x0020, 0x0002, 0x8776, 0x0000, 0x0000, 0x0002, /*10776*/
  0xFBC2, 0x0020, 0x0002, 0x8777, 0x0000, 0x0000, 0x0002, /*10777*/
  0xFBC2, 0x0020, 0x0002, 0x8778, 0x0000, 0x0000, 0x0002, /*10778*/
  0xFBC2, 0x0020, 0x0002, 0x8779, 0x0000, 0x0000, 0x0002, /*10779*/
  0xFBC2, 0x0020, 0x0002, 0x877A, 0x0000, 0x0000, 0x0002, /*1077A*/
  0xFBC2, 0x0020, 0x0002, 0x877B, 0x0000, 0x0000, 0x0002, /*1077B*/
  0xFBC2, 0x0020, 0x0002, 0x877C, 0x0000, 0x0000, 0x0002, /*1077C*/
  0xFBC2, 0x0020, 0x0002, 0x877D, 0x0000, 0x0000, 0x0002, /*1077D*/
  0xFBC2, 0x0020, 0x0002, 0x877E, 0x0000, 0x0000, 0x0002, /*1077E*/
  0xFBC2, 0x0020, 0x0002, 0x877F, 0x0000, 0x0000, 0x0002, /*1077F*/
  0xFBC2, 0x0020, 0x0002, 0x8780, 0x0000, 0x0000, 0x0002, /*10780*/
  0xFBC2, 0x0020, 0x0002, 0x8781, 0x0000, 0x0000, 0x0002, /*10781*/
  0xFBC2, 0x0020, 0x0002, 0x8782, 0x0000, 0x0000, 0x0002, /*10782*/
  0xFBC2, 0x0020, 0x0002, 0x8783, 0x0000, 0x0000, 0x0002, /*10783*/
  0xFBC2, 0x0020, 0x0002, 0x8784, 0x0000, 0x0000, 0x0002, /*10784*/
  0xFBC2, 0x0020, 0x0002, 0x8785, 0x0000, 0x0000, 0x0002, /*10785*/
  0xFBC2, 0x0020, 0x0002, 0x8786, 0x0000, 0x0000, 0x0002, /*10786*/
  0xFBC2, 0x0020, 0x0002, 0x8787, 0x0000, 0x0000, 0x0002, /*10787*/
  0xFBC2, 0x0020, 0x0002, 0x8788, 0x0000, 0x0000, 0x0002, /*10788*/
  0xFBC2, 0x0020, 0x0002, 0x8789, 0x0000, 0x0000, 0x0002, /*10789*/
  0xFBC2, 0x0020, 0x0002, 0x878A, 0x0000, 0x0000, 0x0002, /*1078A*/
  0xFBC2, 0x0020, 0x0002, 0x878B, 0x0000, 0x0000, 0x0002, /*1078B*/
  0xFBC2, 0x0020, 0x0002, 0x878C, 0x0000, 0x0000, 0x0002, /*1078C*/
  0xFBC2, 0x0020, 0x0002, 0x878D, 0x0000, 0x0000, 0x0002, /*1078D*/
  0xFBC2, 0x0020, 0x0002, 0x878E, 0x0000, 0x0000, 0x0002, /*1078E*/
  0xFBC2, 0x0020, 0x0002, 0x878F, 0x0000, 0x0000, 0x0002, /*1078F*/
  0xFBC2, 0x0020, 0x0002, 0x8790, 0x0000, 0x0000, 0x0002, /*10790*/
  0xFBC2, 0x0020, 0x0002, 0x8791, 0x0000, 0x0000, 0x0002, /*10791*/
  0xFBC2, 0x0020, 0x0002, 0x8792, 0x0000, 0x0000, 0x0002, /*10792*/
  0xFBC2, 0x0020, 0x0002, 0x8793, 0x0000, 0x0000, 0x0002, /*10793*/
  0xFBC2, 0x0020, 0x0002, 0x8794, 0x0000, 0x0000, 0x0002, /*10794*/
  0xFBC2, 0x0020, 0x0002, 0x8795, 0x0000, 0x0000, 0x0002, /*10795*/
  0xFBC2, 0x0020, 0x0002, 0x8796, 0x0000, 0x0000, 0x0002, /*10796*/
  0xFBC2, 0x0020, 0x0002, 0x8797, 0x0000, 0x0000, 0x0002, /*10797*/
  0xFBC2, 0x0020, 0x0002, 0x8798, 0x0000, 0x0000, 0x0002, /*10798*/
  0xFBC2, 0x0020, 0x0002, 0x8799, 0x0000, 0x0000, 0x0002, /*10799*/
  0xFBC2, 0x0020, 0x0002, 0x879A, 0x0000, 0x0000, 0x0002, /*1079A*/
  0xFBC2, 0x0020, 0x0002, 0x879B, 0x0000, 0x0000, 0x0002, /*1079B*/
  0xFBC2, 0x0020, 0x0002, 0x879C, 0x0000, 0x0000, 0x0002, /*1079C*/
  0xFBC2, 0x0020, 0x0002, 0x879D, 0x0000, 0x0000, 0x0002, /*1079D*/
  0xFBC2, 0x0020, 0x0002, 0x879E, 0x0000, 0x0000, 0x0002, /*1079E*/
  0xFBC2, 0x0020, 0x0002, 0x879F, 0x0000, 0x0000, 0x0002, /*1079F*/
  0xFBC2, 0x0020, 0x0002, 0x87A0, 0x0000, 0x0000, 0x0002, /*107A0*/
  0xFBC2, 0x0020, 0x0002, 0x87A1, 0x0000, 0x0000, 0x0002, /*107A1*/
  0xFBC2, 0x0020, 0x0002, 0x87A2, 0x0000, 0x0000, 0x0002, /*107A2*/
  0xFBC2, 0x0020, 0x0002, 0x87A3, 0x0000, 0x0000, 0x0002, /*107A3*/
  0xFBC2, 0x0020, 0x0002, 0x87A4, 0x0000, 0x0000, 0x0002, /*107A4*/
  0xFBC2, 0x0020, 0x0002, 0x87A5, 0x0000, 0x0000, 0x0002, /*107A5*/
  0xFBC2, 0x0020, 0x0002, 0x87A6, 0x0000, 0x0000, 0x0002, /*107A6*/
  0xFBC2, 0x0020, 0x0002, 0x87A7, 0x0000, 0x0000, 0x0002, /*107A7*/
  0xFBC2, 0x0020, 0x0002, 0x87A8, 0x0000, 0x0000, 0x0002, /*107A8*/
  0xFBC2, 0x0020, 0x0002, 0x87A9, 0x0000, 0x0000, 0x0002, /*107A9*/
  0xFBC2, 0x0020, 0x0002, 0x87AA, 0x0000, 0x0000, 0x0002, /*107AA*/
  0xFBC2, 0x0020, 0x0002, 0x87AB, 0x0000, 0x0000, 0x0002, /*107AB*/
  0xFBC2, 0x0020, 0x0002, 0x87AC, 0x0000, 0x0000, 0x0002, /*107AC*/
  0xFBC2, 0x0020, 0x0002, 0x87AD, 0x0000, 0x0000, 0x0002, /*107AD*/
  0xFBC2, 0x0020, 0x0002, 0x87AE, 0x0000, 0x0000, 0x0002, /*107AE*/
  0xFBC2, 0x0020, 0x0002, 0x87AF, 0x0000, 0x0000, 0x0002, /*107AF*/
  0xFBC2, 0x0020, 0x0002, 0x87B0, 0x0000, 0x0000, 0x0002, /*107B0*/
  0xFBC2, 0x0020, 0x0002, 0x87B1, 0x0000, 0x0000, 0x0002, /*107B1*/
  0xFBC2, 0x0020, 0x0002, 0x87B2, 0x0000, 0x0000, 0x0002, /*107B2*/
  0xFBC2, 0x0020, 0x0002, 0x87B3, 0x0000, 0x0000, 0x0002, /*107B3*/
  0xFBC2, 0x0020, 0x0002, 0x87B4, 0x0000, 0x0000, 0x0002, /*107B4*/
  0xFBC2, 0x0020, 0x0002, 0x87B5, 0x0000, 0x0000, 0x0002, /*107B5*/
  0xFBC2, 0x0020, 0x0002, 0x87B6, 0x0000, 0x0000, 0x0002, /*107B6*/
  0xFBC2, 0x0020, 0x0002, 0x87B7, 0x0000, 0x0000, 0x0002, /*107B7*/
  0xFBC2, 0x0020, 0x0002, 0x87B8, 0x0000, 0x0000, 0x0002, /*107B8*/
  0xFBC2, 0x0020, 0x0002, 0x87B9, 0x0000, 0x0000, 0x0002, /*107B9*/
  0xFBC2, 0x0020, 0x0002, 0x87BA, 0x0000, 0x0000, 0x0002, /*107BA*/
  0xFBC2, 0x0020, 0x0002, 0x87BB, 0x0000, 0x0000, 0x0002, /*107BB*/
  0xFBC2, 0x0020, 0x0002, 0x87BC, 0x0000, 0x0000, 0x0002, /*107BC*/
  0xFBC2, 0x0020, 0x0002, 0x87BD, 0x0000, 0x0000, 0x0002, /*107BD*/
  0xFBC2, 0x0020, 0x0002, 0x87BE, 0x0000, 0x0000, 0x0002, /*107BE*/
  0xFBC2, 0x0020, 0x0002, 0x87BF, 0x0000, 0x0000, 0x0002, /*107BF*/
  0xFBC2, 0x0020, 0x0002, 0x87C0, 0x0000, 0x0000, 0x0002, /*107C0*/
  0xFBC2, 0x0020, 0x0002, 0x87C1, 0x0000, 0x0000, 0x0002, /*107C1*/
  0xFBC2, 0x0020, 0x0002, 0x87C2, 0x0000, 0x0000, 0x0002, /*107C2*/
  0xFBC2, 0x0020, 0x0002, 0x87C3, 0x0000, 0x0000, 0x0002, /*107C3*/
  0xFBC2, 0x0020, 0x0002, 0x87C4, 0x0000, 0x0000, 0x0002, /*107C4*/
  0xFBC2, 0x0020, 0x0002, 0x87C5, 0x0000, 0x0000, 0x0002, /*107C5*/
  0xFBC2, 0x0020, 0x0002, 0x87C6, 0x0000, 0x0000, 0x0002, /*107C6*/
  0xFBC2, 0x0020, 0x0002, 0x87C7, 0x0000, 0x0000, 0x0002, /*107C7*/
  0xFBC2, 0x0020, 0x0002, 0x87C8, 0x0000, 0x0000, 0x0002, /*107C8*/
  0xFBC2, 0x0020, 0x0002, 0x87C9, 0x0000, 0x0000, 0x0002, /*107C9*/
  0xFBC2, 0x0020, 0x0002, 0x87CA, 0x0000, 0x0000, 0x0002, /*107CA*/
  0xFBC2, 0x0020, 0x0002, 0x87CB, 0x0000, 0x0000, 0x0002, /*107CB*/
  0xFBC2, 0x0020, 0x0002, 0x87CC, 0x0000, 0x0000, 0x0002, /*107CC*/
  0xFBC2, 0x0020, 0x0002, 0x87CD, 0x0000, 0x0000, 0x0002, /*107CD*/
  0xFBC2, 0x0020, 0x0002, 0x87CE, 0x0000, 0x0000, 0x0002, /*107CE*/
  0xFBC2, 0x0020, 0x0002, 0x87CF, 0x0000, 0x0000, 0x0002, /*107CF*/
  0xFBC2, 0x0020, 0x0002, 0x87D0, 0x0000, 0x0000, 0x0002, /*107D0*/
  0xFBC2, 0x0020, 0x0002, 0x87D1, 0x0000, 0x0000, 0x0002, /*107D1*/
  0xFBC2, 0x0020, 0x0002, 0x87D2, 0x0000, 0x0000, 0x0002, /*107D2*/
  0xFBC2, 0x0020, 0x0002, 0x87D3, 0x0000, 0x0000, 0x0002, /*107D3*/
  0xFBC2, 0x0020, 0x0002, 0x87D4, 0x0000, 0x0000, 0x0002, /*107D4*/
  0xFBC2, 0x0020, 0x0002, 0x87D5, 0x0000, 0x0000, 0x0002, /*107D5*/
  0xFBC2, 0x0020, 0x0002, 0x87D6, 0x0000, 0x0000, 0x0002, /*107D6*/
  0xFBC2, 0x0020, 0x0002, 0x87D7, 0x0000, 0x0000, 0x0002, /*107D7*/
  0xFBC2, 0x0020, 0x0002, 0x87D8, 0x0000, 0x0000, 0x0002, /*107D8*/
  0xFBC2, 0x0020, 0x0002, 0x87D9, 0x0000, 0x0000, 0x0002, /*107D9*/
  0xFBC2, 0x0020, 0x0002, 0x87DA, 0x0000, 0x0000, 0x0002, /*107DA*/
  0xFBC2, 0x0020, 0x0002, 0x87DB, 0x0000, 0x0000, 0x0002, /*107DB*/
  0xFBC2, 0x0020, 0x0002, 0x87DC, 0x0000, 0x0000, 0x0002, /*107DC*/
  0xFBC2, 0x0020, 0x0002, 0x87DD, 0x0000, 0x0000, 0x0002, /*107DD*/
  0xFBC2, 0x0020, 0x0002, 0x87DE, 0x0000, 0x0000, 0x0002, /*107DE*/
  0xFBC2, 0x0020, 0x0002, 0x87DF, 0x0000, 0x0000, 0x0002, /*107DF*/
  0xFBC2, 0x0020, 0x0002, 0x87E0, 0x0000, 0x0000, 0x0002, /*107E0*/
  0xFBC2, 0x0020, 0x0002, 0x87E1, 0x0000, 0x0000, 0x0002, /*107E1*/
  0xFBC2, 0x0020, 0x0002, 0x87E2, 0x0000, 0x0000, 0x0002, /*107E2*/
  0xFBC2, 0x0020, 0x0002, 0x87E3, 0x0000, 0x0000, 0x0002, /*107E3*/
  0xFBC2, 0x0020, 0x0002, 0x87E4, 0x0000, 0x0000, 0x0002, /*107E4*/
  0xFBC2, 0x0020, 0x0002, 0x87E5, 0x0000, 0x0000, 0x0002, /*107E5*/
  0xFBC2, 0x0020, 0x0002, 0x87E6, 0x0000, 0x0000, 0x0002, /*107E6*/
  0xFBC2, 0x0020, 0x0002, 0x87E7, 0x0000, 0x0000, 0x0002, /*107E7*/
  0xFBC2, 0x0020, 0x0002, 0x87E8, 0x0000, 0x0000, 0x0002, /*107E8*/
  0xFBC2, 0x0020, 0x0002, 0x87E9, 0x0000, 0x0000, 0x0002, /*107E9*/
  0xFBC2, 0x0020, 0x0002, 0x87EA, 0x0000, 0x0000, 0x0002, /*107EA*/
  0xFBC2, 0x0020, 0x0002, 0x87EB, 0x0000, 0x0000, 0x0002, /*107EB*/
  0xFBC2, 0x0020, 0x0002, 0x87EC, 0x0000, 0x0000, 0x0002, /*107EC*/
  0xFBC2, 0x0020, 0x0002, 0x87ED, 0x0000, 0x0000, 0x0002, /*107ED*/
  0xFBC2, 0x0020, 0x0002, 0x87EE, 0x0000, 0x0000, 0x0002, /*107EE*/
  0xFBC2, 0x0020, 0x0002, 0x87EF, 0x0000, 0x0000, 0x0002, /*107EF*/
  0xFBC2, 0x0020, 0x0002, 0x87F0, 0x0000, 0x0000, 0x0002, /*107F0*/
  0xFBC2, 0x0020, 0x0002, 0x87F1, 0x0000, 0x0000, 0x0002, /*107F1*/
  0xFBC2, 0x0020, 0x0002, 0x87F2, 0x0000, 0x0000, 0x0002, /*107F2*/
  0xFBC2, 0x0020, 0x0002, 0x87F3, 0x0000, 0x0000, 0x0002, /*107F3*/
  0xFBC2, 0x0020, 0x0002, 0x87F4, 0x0000, 0x0000, 0x0002, /*107F4*/
  0xFBC2, 0x0020, 0x0002, 0x87F5, 0x0000, 0x0000, 0x0002, /*107F5*/
  0xFBC2, 0x0020, 0x0002, 0x87F6, 0x0000, 0x0000, 0x0002, /*107F6*/
  0xFBC2, 0x0020, 0x0002, 0x87F7, 0x0000, 0x0000, 0x0002, /*107F7*/
  0xFBC2, 0x0020, 0x0002, 0x87F8, 0x0000, 0x0000, 0x0002, /*107F8*/
  0xFBC2, 0x0020, 0x0002, 0x87F9, 0x0000, 0x0000, 0x0002, /*107F9*/
  0xFBC2, 0x0020, 0x0002, 0x87FA, 0x0000, 0x0000, 0x0002, /*107FA*/
  0xFBC2, 0x0020, 0x0002, 0x87FB, 0x0000, 0x0000, 0x0002, /*107FB*/
  0xFBC2, 0x0020, 0x0002, 0x87FC, 0x0000, 0x0000, 0x0002, /*107FC*/
  0xFBC2, 0x0020, 0x0002, 0x87FD, 0x0000, 0x0000, 0x0002, /*107FD*/
  0xFBC2, 0x0020, 0x0002, 0x87FE, 0x0000, 0x0000, 0x0002, /*107FE*/
  0xFBC2, 0x0020, 0x0002, 0x87FF, 0x0000, 0x0000, 0x0002  /*107FF*/
};

uint16 uca900_p108[]= {
  0x480F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10800*/
  0x4810, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10801*/
  0x4811, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10802*/
  0x4812, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10803*/
  0x4813, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10804*/
  0x4814, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10805*/
  0xFBC2, 0x0020, 0x0002, 0x8806, 0x0000, 0x0000, 0x0002, /*10806*/
  0xFBC2, 0x0020, 0x0002, 0x8807, 0x0000, 0x0000, 0x0002, /*10807*/
  0x4815, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10808*/
  0xFBC2, 0x0020, 0x0002, 0x8809, 0x0000, 0x0000, 0x0002, /*10809*/
  0x4816, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1080A*/
  0x4817, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1080B*/
  0x4818, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1080C*/
  0x4819, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1080D*/
  0x481A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1080E*/
  0x481B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1080F*/
  0x481C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10810*/
  0x481D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10811*/
  0x481E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10812*/
  0x481F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10813*/
  0x4820, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10814*/
  0x4821, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10815*/
  0x4822, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10816*/
  0x4823, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10817*/
  0x4824, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10818*/
  0x4825, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10819*/
  0x4826, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1081A*/
  0x4827, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1081B*/
  0x4828, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1081C*/
  0x4829, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1081D*/
  0x482A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1081E*/
  0x482B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1081F*/
  0x482C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10820*/
  0x482D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10821*/
  0x482E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10822*/
  0x482F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10823*/
  0x4830, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10824*/
  0x4831, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10825*/
  0x4832, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10826*/
  0x4833, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10827*/
  0x4834, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10828*/
  0x4835, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10829*/
  0x4836, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1082A*/
  0x4837, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1082B*/
  0x4838, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1082C*/
  0x4839, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1082D*/
  0x483A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1082E*/
  0x483B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1082F*/
  0x483C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10830*/
  0x483D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10831*/
  0x483E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10832*/
  0x483F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10833*/
  0x4840, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10834*/
  0x4841, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10835*/
  0xFBC2, 0x0020, 0x0002, 0x8836, 0x0000, 0x0000, 0x0002, /*10836*/
  0x4842, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10837*/
  0x4843, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10838*/
  0xFBC2, 0x0020, 0x0002, 0x8839, 0x0000, 0x0000, 0x0002, /*10839*/
  0xFBC2, 0x0020, 0x0002, 0x883A, 0x0000, 0x0000, 0x0002, /*1083A*/
  0xFBC2, 0x0020, 0x0002, 0x883B, 0x0000, 0x0000, 0x0002, /*1083B*/
  0x4844, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1083C*/
  0xFBC2, 0x0020, 0x0002, 0x883D, 0x0000, 0x0000, 0x0002, /*1083D*/
  0xFBC2, 0x0020, 0x0002, 0x883E, 0x0000, 0x0000, 0x0002, /*1083E*/
  0x4845, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1083F*/
  0x48F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10840*/
  0x48F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10841*/
  0x48F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10842*/
  0x48F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10843*/
  0x48FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10844*/
  0x48FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10845*/
  0x48FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10846*/
  0x48FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10847*/
  0x48FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10848*/
  0x48FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10849*/
  0x4900, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1084A*/
  0x4901, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1084B*/
  0x4902, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1084C*/
  0x4903, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1084D*/
  0x4904, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1084E*/
  0x4905, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1084F*/
  0x4906, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10850*/
  0x4907, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10851*/
  0x4908, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10852*/
  0x4909, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10853*/
  0x490A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10854*/
  0x490B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10855*/
  0xFBC2, 0x0020, 0x0002, 0x8856, 0x0000, 0x0000, 0x0002, /*10856*/
  0x02D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10857*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10858*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10859*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1085A*/
  0x1B5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1085B*/
  0x1B5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1085C*/
  0x1B60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1085D*/
  0x1B61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1085E*/
  0x1B62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1085F*/
  0x48B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10860*/
  0x48B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10861*/
  0x48B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10862*/
  0x48B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10863*/
  0x48B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10864*/
  0x48BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10865*/
  0x48BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10866*/
  0x48BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10867*/
  0x48BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10868*/
  0x48BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10869*/
  0x48BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1086A*/
  0x48C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1086B*/
  0x48C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1086C*/
  0x48C2, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*1086D*/
  0x48C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1086E*/
  0x48C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1086F*/
  0x48C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10870*/
  0x48C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10871*/
  0x48C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10872*/
  0x48C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10873*/
  0x48C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10874*/
  0x48C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10875*/
  0x48CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10876*/
  0x0FCF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10877*/
  0x0FD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10878*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10879*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1087A*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1087B*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1087C*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1087D*/
  0x1B4F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1087E*/
  0x1B50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1087F*/
  0x48CB, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*10880*/
  0x48CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10881*/
  0x48CC, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*10882*/
  0x48CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10883*/
  0x48CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10884*/
  0x48CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10885*/
  0x48CF, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*10886*/
  0x48CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10887*/
  0x48D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10888*/
  0x48D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10889*/
  0x48D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1088A*/
  0x48D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1088B*/
  0x48D4, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*1088C*/
  0x48D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1088D*/
  0x48D5, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*1088E*/
  0x48D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1088F*/
  0x48D6, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*10890*/
  0x48D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10891*/
  0x48D7, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*10892*/
  0x48D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10893*/
  0x48D8, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*10894*/
  0x48D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10895*/
  0x48D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10896*/
  0x48DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10897*/
  0x48DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10898*/
  0x48DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10899*/
  0x48DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1089A*/
  0x48DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1089B*/
  0x48DF, 0x0020, 0x0019, 0x0000, 0x0000, 0x0000, 0x0001, /*1089C*/
  0x48DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1089D*/
  0x48E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1089E*/
  0xFBC2, 0x0020, 0x0002, 0x889F, 0x0000, 0x0000, 0x0002, /*1089F*/
  0xFBC2, 0x0020, 0x0002, 0x88A0, 0x0000, 0x0000, 0x0002, /*108A0*/
  0xFBC2, 0x0020, 0x0002, 0x88A1, 0x0000, 0x0000, 0x0002, /*108A1*/
  0xFBC2, 0x0020, 0x0002, 0x88A2, 0x0000, 0x0000, 0x0002, /*108A2*/
  0xFBC2, 0x0020, 0x0002, 0x88A3, 0x0000, 0x0000, 0x0002, /*108A3*/
  0xFBC2, 0x0020, 0x0002, 0x88A4, 0x0000, 0x0000, 0x0002, /*108A4*/
  0xFBC2, 0x0020, 0x0002, 0x88A5, 0x0000, 0x0000, 0x0002, /*108A5*/
  0xFBC2, 0x0020, 0x0002, 0x88A6, 0x0000, 0x0000, 0x0002, /*108A6*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108A7*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108A8*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108A9*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108AA*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108AB*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108AC*/
  0x1B51, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108AD*/
  0x1B52, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108AE*/
  0x1B53, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108AF*/
  0xFBC2, 0x0020, 0x0002, 0x88B0, 0x0000, 0x0000, 0x0002, /*108B0*/
  0xFBC2, 0x0020, 0x0002, 0x88B1, 0x0000, 0x0000, 0x0002, /*108B1*/
  0xFBC2, 0x0020, 0x0002, 0x88B2, 0x0000, 0x0000, 0x0002, /*108B2*/
  0xFBC2, 0x0020, 0x0002, 0x88B3, 0x0000, 0x0000, 0x0002, /*108B3*/
  0xFBC2, 0x0020, 0x0002, 0x88B4, 0x0000, 0x0000, 0x0002, /*108B4*/
  0xFBC2, 0x0020, 0x0002, 0x88B5, 0x0000, 0x0000, 0x0002, /*108B5*/
  0xFBC2, 0x0020, 0x0002, 0x88B6, 0x0000, 0x0000, 0x0002, /*108B6*/
  0xFBC2, 0x0020, 0x0002, 0x88B7, 0x0000, 0x0000, 0x0002, /*108B7*/
  0xFBC2, 0x0020, 0x0002, 0x88B8, 0x0000, 0x0000, 0x0002, /*108B8*/
  0xFBC2, 0x0020, 0x0002, 0x88B9, 0x0000, 0x0000, 0x0002, /*108B9*/
  0xFBC2, 0x0020, 0x0002, 0x88BA, 0x0000, 0x0000, 0x0002, /*108BA*/
  0xFBC2, 0x0020, 0x0002, 0x88BB, 0x0000, 0x0000, 0x0002, /*108BB*/
  0xFBC2, 0x0020, 0x0002, 0x88BC, 0x0000, 0x0000, 0x0002, /*108BC*/
  0xFBC2, 0x0020, 0x0002, 0x88BD, 0x0000, 0x0000, 0x0002, /*108BD*/
  0xFBC2, 0x0020, 0x0002, 0x88BE, 0x0000, 0x0000, 0x0002, /*108BE*/
  0xFBC2, 0x0020, 0x0002, 0x88BF, 0x0000, 0x0000, 0x0002, /*108BF*/
  0xFBC2, 0x0020, 0x0002, 0x88C0, 0x0000, 0x0000, 0x0002, /*108C0*/
  0xFBC2, 0x0020, 0x0002, 0x88C1, 0x0000, 0x0000, 0x0002, /*108C1*/
  0xFBC2, 0x0020, 0x0002, 0x88C2, 0x0000, 0x0000, 0x0002, /*108C2*/
  0xFBC2, 0x0020, 0x0002, 0x88C3, 0x0000, 0x0000, 0x0002, /*108C3*/
  0xFBC2, 0x0020, 0x0002, 0x88C4, 0x0000, 0x0000, 0x0002, /*108C4*/
  0xFBC2, 0x0020, 0x0002, 0x88C5, 0x0000, 0x0000, 0x0002, /*108C5*/
  0xFBC2, 0x0020, 0x0002, 0x88C6, 0x0000, 0x0000, 0x0002, /*108C6*/
  0xFBC2, 0x0020, 0x0002, 0x88C7, 0x0000, 0x0000, 0x0002, /*108C7*/
  0xFBC2, 0x0020, 0x0002, 0x88C8, 0x0000, 0x0000, 0x0002, /*108C8*/
  0xFBC2, 0x0020, 0x0002, 0x88C9, 0x0000, 0x0000, 0x0002, /*108C9*/
  0xFBC2, 0x0020, 0x0002, 0x88CA, 0x0000, 0x0000, 0x0002, /*108CA*/
  0xFBC2, 0x0020, 0x0002, 0x88CB, 0x0000, 0x0000, 0x0002, /*108CB*/
  0xFBC2, 0x0020, 0x0002, 0x88CC, 0x0000, 0x0000, 0x0002, /*108CC*/
  0xFBC2, 0x0020, 0x0002, 0x88CD, 0x0000, 0x0000, 0x0002, /*108CD*/
  0xFBC2, 0x0020, 0x0002, 0x88CE, 0x0000, 0x0000, 0x0002, /*108CE*/
  0xFBC2, 0x0020, 0x0002, 0x88CF, 0x0000, 0x0000, 0x0002, /*108CF*/
  0xFBC2, 0x0020, 0x0002, 0x88D0, 0x0000, 0x0000, 0x0002, /*108D0*/
  0xFBC2, 0x0020, 0x0002, 0x88D1, 0x0000, 0x0000, 0x0002, /*108D1*/
  0xFBC2, 0x0020, 0x0002, 0x88D2, 0x0000, 0x0000, 0x0002, /*108D2*/
  0xFBC2, 0x0020, 0x0002, 0x88D3, 0x0000, 0x0000, 0x0002, /*108D3*/
  0xFBC2, 0x0020, 0x0002, 0x88D4, 0x0000, 0x0000, 0x0002, /*108D4*/
  0xFBC2, 0x0020, 0x0002, 0x88D5, 0x0000, 0x0000, 0x0002, /*108D5*/
  0xFBC2, 0x0020, 0x0002, 0x88D6, 0x0000, 0x0000, 0x0002, /*108D6*/
  0xFBC2, 0x0020, 0x0002, 0x88D7, 0x0000, 0x0000, 0x0002, /*108D7*/
  0xFBC2, 0x0020, 0x0002, 0x88D8, 0x0000, 0x0000, 0x0002, /*108D8*/
  0xFBC2, 0x0020, 0x0002, 0x88D9, 0x0000, 0x0000, 0x0002, /*108D9*/
  0xFBC2, 0x0020, 0x0002, 0x88DA, 0x0000, 0x0000, 0x0002, /*108DA*/
  0xFBC2, 0x0020, 0x0002, 0x88DB, 0x0000, 0x0000, 0x0002, /*108DB*/
  0xFBC2, 0x0020, 0x0002, 0x88DC, 0x0000, 0x0000, 0x0002, /*108DC*/
  0xFBC2, 0x0020, 0x0002, 0x88DD, 0x0000, 0x0000, 0x0002, /*108DD*/
  0xFBC2, 0x0020, 0x0002, 0x88DE, 0x0000, 0x0000, 0x0002, /*108DE*/
  0xFBC2, 0x0020, 0x0002, 0x88DF, 0x0000, 0x0000, 0x0002, /*108DF*/
  0x48E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108E0*/
  0x48E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108E1*/
  0x48E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108E2*/
  0x48E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108E3*/
  0x48E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108E4*/
  0x48E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108E5*/
  0x48E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108E6*/
  0x48E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108E7*/
  0x48E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108E8*/
  0x48EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108E9*/
  0x48EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108EA*/
  0x48EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108EB*/
  0x48ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108EC*/
  0x48EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108ED*/
  0x48EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108EE*/
  0x48F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108EF*/
  0x48F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108F0*/
  0x48F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108F1*/
  0x48F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108F2*/
  0xFBC2, 0x0020, 0x0002, 0x88F3, 0x0000, 0x0000, 0x0002, /*108F3*/
  0x48F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108F4*/
  0x48F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108F5*/
  0xFBC2, 0x0020, 0x0002, 0x88F6, 0x0000, 0x0000, 0x0002, /*108F6*/
  0xFBC2, 0x0020, 0x0002, 0x88F7, 0x0000, 0x0000, 0x0002, /*108F7*/
  0xFBC2, 0x0020, 0x0002, 0x88F8, 0x0000, 0x0000, 0x0002, /*108F8*/
  0xFBC2, 0x0020, 0x0002, 0x88F9, 0x0000, 0x0000, 0x0002, /*108F9*/
  0xFBC2, 0x0020, 0x0002, 0x88FA, 0x0000, 0x0000, 0x0002, /*108FA*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108FB*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108FC*/
  0x1B54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108FD*/
  0x1B55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*108FE*/
  0x1B56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*108FF*/
};

uint16 uca900_p109[]= {
  0x22CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10900*/
  0x22CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10901*/
  0x22CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10902*/
  0x22D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10903*/
  0x22D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10904*/
  0x22D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10905*/
  0x22D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10906*/
  0x22D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10907*/
  0x22D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10908*/
  0x22D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10909*/
  0x22D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1090A*/
  0x22D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1090B*/
  0x22D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1090C*/
  0x22DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1090D*/
  0x22DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1090E*/
  0x22DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1090F*/
  0x22DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10910*/
  0x22DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10911*/
  0x22DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10912*/
  0x22E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10913*/
  0x22E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10914*/
  0x22E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10915*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10916*/
  0x1B5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10917*/
  0x1B5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10918*/
  0x1B5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10919*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1091A*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1091B*/
  0xFBC2, 0x0020, 0x0002, 0x891C, 0x0000, 0x0000, 0x0002, /*1091C*/
  0xFBC2, 0x0020, 0x0002, 0x891D, 0x0000, 0x0000, 0x0002, /*1091D*/
  0xFBC2, 0x0020, 0x0002, 0x891E, 0x0000, 0x0000, 0x0002, /*1091E*/
  0x02FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1091F*/
  0x43FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10920*/
  0x43FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10921*/
  0x43FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10922*/
  0x4400, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10923*/
  0x4401, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10924*/
  0x4402, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10925*/
  0x4403, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10926*/
  0x4404, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10927*/
  0x4405, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10928*/
  0x4406, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10929*/
  0x4407, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1092A*/
  0x4408, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1092B*/
  0x4409, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1092C*/
  0x440A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1092D*/
  0x440B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1092E*/
  0x440C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1092F*/
  0x440D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10930*/
  0x440E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10931*/
  0x440F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10932*/
  0x4410, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10933*/
  0x4411, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10934*/
  0x4412, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10935*/
  0x4413, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10936*/
  0x4414, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10937*/
  0x4415, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10938*/
  0x4416, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10939*/
  0xFBC2, 0x0020, 0x0002, 0x893A, 0x0000, 0x0000, 0x0002, /*1093A*/
  0xFBC2, 0x0020, 0x0002, 0x893B, 0x0000, 0x0000, 0x0002, /*1093B*/
  0xFBC2, 0x0020, 0x0002, 0x893C, 0x0000, 0x0000, 0x0002, /*1093C*/
  0xFBC2, 0x0020, 0x0002, 0x893D, 0x0000, 0x0000, 0x0002, /*1093D*/
  0xFBC2, 0x0020, 0x0002, 0x893E, 0x0000, 0x0000, 0x0002, /*1093E*/
  0x02F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1093F*/
  0xFBC2, 0x0020, 0x0002, 0x8940, 0x0000, 0x0000, 0x0002, /*10940*/
  0xFBC2, 0x0020, 0x0002, 0x8941, 0x0000, 0x0000, 0x0002, /*10941*/
  0xFBC2, 0x0020, 0x0002, 0x8942, 0x0000, 0x0000, 0x0002, /*10942*/
  0xFBC2, 0x0020, 0x0002, 0x8943, 0x0000, 0x0000, 0x0002, /*10943*/
  0xFBC2, 0x0020, 0x0002, 0x8944, 0x0000, 0x0000, 0x0002, /*10944*/
  0xFBC2, 0x0020, 0x0002, 0x8945, 0x0000, 0x0000, 0x0002, /*10945*/
  0xFBC2, 0x0020, 0x0002, 0x8946, 0x0000, 0x0000, 0x0002, /*10946*/
  0xFBC2, 0x0020, 0x0002, 0x8947, 0x0000, 0x0000, 0x0002, /*10947*/
  0xFBC2, 0x0020, 0x0002, 0x8948, 0x0000, 0x0000, 0x0002, /*10948*/
  0xFBC2, 0x0020, 0x0002, 0x8949, 0x0000, 0x0000, 0x0002, /*10949*/
  0xFBC2, 0x0020, 0x0002, 0x894A, 0x0000, 0x0000, 0x0002, /*1094A*/
  0xFBC2, 0x0020, 0x0002, 0x894B, 0x0000, 0x0000, 0x0002, /*1094B*/
  0xFBC2, 0x0020, 0x0002, 0x894C, 0x0000, 0x0000, 0x0002, /*1094C*/
  0xFBC2, 0x0020, 0x0002, 0x894D, 0x0000, 0x0000, 0x0002, /*1094D*/
  0xFBC2, 0x0020, 0x0002, 0x894E, 0x0000, 0x0000, 0x0002, /*1094E*/
  0xFBC2, 0x0020, 0x0002, 0x894F, 0x0000, 0x0000, 0x0002, /*1094F*/
  0xFBC2, 0x0020, 0x0002, 0x8950, 0x0000, 0x0000, 0x0002, /*10950*/
  0xFBC2, 0x0020, 0x0002, 0x8951, 0x0000, 0x0000, 0x0002, /*10951*/
  0xFBC2, 0x0020, 0x0002, 0x8952, 0x0000, 0x0000, 0x0002, /*10952*/
  0xFBC2, 0x0020, 0x0002, 0x8953, 0x0000, 0x0000, 0x0002, /*10953*/
  0xFBC2, 0x0020, 0x0002, 0x8954, 0x0000, 0x0000, 0x0002, /*10954*/
  0xFBC2, 0x0020, 0x0002, 0x8955, 0x0000, 0x0000, 0x0002, /*10955*/
  0xFBC2, 0x0020, 0x0002, 0x8956, 0x0000, 0x0000, 0x0002, /*10956*/
  0xFBC2, 0x0020, 0x0002, 0x8957, 0x0000, 0x0000, 0x0002, /*10957*/
  0xFBC2, 0x0020, 0x0002, 0x8958, 0x0000, 0x0000, 0x0002, /*10958*/
  0xFBC2, 0x0020, 0x0002, 0x8959, 0x0000, 0x0000, 0x0002, /*10959*/
  0xFBC2, 0x0020, 0x0002, 0x895A, 0x0000, 0x0000, 0x0002, /*1095A*/
  0xFBC2, 0x0020, 0x0002, 0x895B, 0x0000, 0x0000, 0x0002, /*1095B*/
  0xFBC2, 0x0020, 0x0002, 0x895C, 0x0000, 0x0000, 0x0002, /*1095C*/
  0xFBC2, 0x0020, 0x0002, 0x895D, 0x0000, 0x0000, 0x0002, /*1095D*/
  0xFBC2, 0x0020, 0x0002, 0x895E, 0x0000, 0x0000, 0x0002, /*1095E*/
  0xFBC2, 0x0020, 0x0002, 0x895F, 0x0000, 0x0000, 0x0002, /*1095F*/
  0xFBC2, 0x0020, 0x0002, 0x8960, 0x0000, 0x0000, 0x0002, /*10960*/
  0xFBC2, 0x0020, 0x0002, 0x8961, 0x0000, 0x0000, 0x0002, /*10961*/
  0xFBC2, 0x0020, 0x0002, 0x8962, 0x0000, 0x0000, 0x0002, /*10962*/
  0xFBC2, 0x0020, 0x0002, 0x8963, 0x0000, 0x0000, 0x0002, /*10963*/
  0xFBC2, 0x0020, 0x0002, 0x8964, 0x0000, 0x0000, 0x0002, /*10964*/
  0xFBC2, 0x0020, 0x0002, 0x8965, 0x0000, 0x0000, 0x0002, /*10965*/
  0xFBC2, 0x0020, 0x0002, 0x8966, 0x0000, 0x0000, 0x0002, /*10966*/
  0xFBC2, 0x0020, 0x0002, 0x8967, 0x0000, 0x0000, 0x0002, /*10967*/
  0xFBC2, 0x0020, 0x0002, 0x8968, 0x0000, 0x0000, 0x0002, /*10968*/
  0xFBC2, 0x0020, 0x0002, 0x8969, 0x0000, 0x0000, 0x0002, /*10969*/
  0xFBC2, 0x0020, 0x0002, 0x896A, 0x0000, 0x0000, 0x0002, /*1096A*/
  0xFBC2, 0x0020, 0x0002, 0x896B, 0x0000, 0x0000, 0x0002, /*1096B*/
  0xFBC2, 0x0020, 0x0002, 0x896C, 0x0000, 0x0000, 0x0002, /*1096C*/
  0xFBC2, 0x0020, 0x0002, 0x896D, 0x0000, 0x0000, 0x0002, /*1096D*/
  0xFBC2, 0x0020, 0x0002, 0x896E, 0x0000, 0x0000, 0x0002, /*1096E*/
  0xFBC2, 0x0020, 0x0002, 0x896F, 0x0000, 0x0000, 0x0002, /*1096F*/
  0xFBC2, 0x0020, 0x0002, 0x8970, 0x0000, 0x0000, 0x0002, /*10970*/
  0xFBC2, 0x0020, 0x0002, 0x8971, 0x0000, 0x0000, 0x0002, /*10971*/
  0xFBC2, 0x0020, 0x0002, 0x8972, 0x0000, 0x0000, 0x0002, /*10972*/
  0xFBC2, 0x0020, 0x0002, 0x8973, 0x0000, 0x0000, 0x0002, /*10973*/
  0xFBC2, 0x0020, 0x0002, 0x8974, 0x0000, 0x0000, 0x0002, /*10974*/
  0xFBC2, 0x0020, 0x0002, 0x8975, 0x0000, 0x0000, 0x0002, /*10975*/
  0xFBC2, 0x0020, 0x0002, 0x8976, 0x0000, 0x0000, 0x0002, /*10976*/
  0xFBC2, 0x0020, 0x0002, 0x8977, 0x0000, 0x0000, 0x0002, /*10977*/
  0xFBC2, 0x0020, 0x0002, 0x8978, 0x0000, 0x0000, 0x0002, /*10978*/
  0xFBC2, 0x0020, 0x0002, 0x8979, 0x0000, 0x0000, 0x0002, /*10979*/
  0xFBC2, 0x0020, 0x0002, 0x897A, 0x0000, 0x0000, 0x0002, /*1097A*/
  0xFBC2, 0x0020, 0x0002, 0x897B, 0x0000, 0x0000, 0x0002, /*1097B*/
  0xFBC2, 0x0020, 0x0002, 0x897C, 0x0000, 0x0000, 0x0002, /*1097C*/
  0xFBC2, 0x0020, 0x0002, 0x897D, 0x0000, 0x0000, 0x0002, /*1097D*/
  0xFBC2, 0x0020, 0x0002, 0x897E, 0x0000, 0x0000, 0x0002, /*1097E*/
  0xFBC2, 0x0020, 0x0002, 0x897F, 0x0000, 0x0000, 0x0002, /*1097F*/
  0x5242, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10980*/
  0x5243, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10981*/
  0x5244, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10982*/
  0x5245, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10983*/
  0x5246, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10984*/
  0x5247, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10985*/
  0x5248, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10986*/
  0x5248, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*10987*/
  0x5249, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10988*/
  0x524A, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10989*/
  0x524B, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*1098A*/
  0x524B, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*1098B*/
  0x524C, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*1098C*/
  0x524C, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*1098D*/
  0x524D, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*1098E*/
  0x524D, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*1098F*/
  0x524E, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10990*/
  0x524F, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10991*/
  0x5250, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10992*/
  0x5251, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*10993*/
  0x5251, 0x0020, 0x0004, 0x0000, 0x0112, 0x0004, 0x0002, /*10994*/
  0x5252, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10995*/
  0x5253, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10996*/
  0x5254, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10997*/
  0x5255, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10998*/
  0x5255, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*10999*/
  0x5256, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*1099A*/
  0x5256, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*1099B*/
  0x5257, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*1099C*/
  0x5258, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*1099D*/
  0x525B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1099E*/
  0x525C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1099F*/
  0x5242, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109A0*/
  0x5243, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109A1*/
  0x5244, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109A2*/
  0x5245, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109A3*/
  0x5246, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109A4*/
  0x5247, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109A5*/
  0x5248, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109A6*/
  0x5249, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109A7*/
  0x524A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109A8*/
  0x524B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109A9*/
  0x524C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109AA*/
  0x524D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109AB*/
  0x524E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109AC*/
  0x524F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109AD*/
  0x5250, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109AE*/
  0x5251, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109AF*/
  0x5251, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*109B0*/
  0x5252, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109B1*/
  0x5253, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109B2*/
  0x5254, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109B3*/
  0x5255, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109B4*/
  0x5256, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109B5*/
  0x5257, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109B6*/
  0x5258, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109B7*/
  0xFBC2, 0x0020, 0x0002, 0x89B8, 0x0000, 0x0000, 0x0002, /*109B8*/
  0xFBC2, 0x0020, 0x0002, 0x89B9, 0x0000, 0x0000, 0x0002, /*109B9*/
  0xFBC2, 0x0020, 0x0002, 0x89BA, 0x0000, 0x0000, 0x0002, /*109BA*/
  0xFBC2, 0x0020, 0x0002, 0x89BB, 0x0000, 0x0000, 0x0002, /*109BB*/
  0x1BDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109BC*/
  0x1BD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109BD*/
  0x5259, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109BE*/
  0x525A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109BF*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109C0*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109C1*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109C2*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109C3*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109C4*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109C5*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109C6*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109C7*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109C8*/
  0x1BA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109C9*/
  0x1BA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109CA*/
  0x1BA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109CB*/
  0x1BAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109CC*/
  0x1BAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109CD*/
  0x1BAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109CE*/
  0x1BAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109CF*/
  0xFBC2, 0x0020, 0x0002, 0x89D0, 0x0000, 0x0000, 0x0002, /*109D0*/
  0xFBC2, 0x0020, 0x0002, 0x89D1, 0x0000, 0x0000, 0x0002, /*109D1*/
  0x1BAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109D2*/
  0x1BAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109D3*/
  0x1BB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109D4*/
  0x1BB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109D5*/
  0x1BB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109D6*/
  0x1BB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109D7*/
  0x1BB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109D8*/
  0x1BB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109D9*/
  0x1BB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109DA*/
  0x1BB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109DB*/
  0x1BB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109DC*/
  0x1BB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109DD*/
  0x1BBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109DE*/
  0x1BBB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109DF*/
  0x1BBC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109E0*/
  0x1BBD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109E1*/
  0x1BBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109E2*/
  0x1BBF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109E3*/
  0x1BC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109E4*/
  0x1BC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109E5*/
  0x1BC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109E6*/
  0x1BC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109E7*/
  0x1BC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109E8*/
  0x1BC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109E9*/
  0x1BC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109EA*/
  0x1BC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109EB*/
  0x1BC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109EC*/
  0x1BC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109ED*/
  0x1BCA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109EE*/
  0x1BCB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109EF*/
  0x1BCC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109F0*/
  0x1BCD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109F1*/
  0x1BCE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109F2*/
  0x1BCF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109F3*/
  0x1BD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109F4*/
  0x1BD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109F5*/
  0x1BD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109F6*/
  0x1BD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109F7*/
  0x1BD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109F8*/
  0x1BD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109F9*/
  0x1BD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109FA*/
  0x1BD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109FB*/
  0x1BD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109FC*/
  0x1BDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109FD*/
  0x1BDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*109FE*/
  0x1BDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*109FF*/
};

uint16 uca900_p10A[]= {
  0x2D0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A00*/
  0x2D0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A01*/
  0x2D0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A02*/
  0x2D10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A03*/
  0xFBC2, 0x0020, 0x0002, 0x8A04, 0x0000, 0x0000, 0x0002, /*10A04*/
  0x2D11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A05*/
  0x2D12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A06*/
  0xFBC2, 0x0020, 0x0002, 0x8A07, 0x0000, 0x0000, 0x0002, /*10A07*/
  0xFBC2, 0x0020, 0x0002, 0x8A08, 0x0000, 0x0000, 0x0002, /*10A08*/
  0xFBC2, 0x0020, 0x0002, 0x8A09, 0x0000, 0x0000, 0x0002, /*10A09*/
  0xFBC2, 0x0020, 0x0002, 0x8A0A, 0x0000, 0x0000, 0x0002, /*10A0A*/
  0xFBC2, 0x0020, 0x0002, 0x8A0B, 0x0000, 0x0000, 0x0002, /*10A0B*/
  0x2D13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A0C*/
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A0D*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A0E*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A0F*/
  0x2D14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A10*/
  0x2D15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A11*/
  0x2D16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A12*/
  0x2D17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A13*/
  0xFBC2, 0x0020, 0x0002, 0x8A14, 0x0000, 0x0000, 0x0002, /*10A14*/
  0x2D18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A15*/
  0x2D19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A16*/
  0x2D1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A17*/
  0xFBC2, 0x0020, 0x0002, 0x8A18, 0x0000, 0x0000, 0x0002, /*10A18*/
  0x2D1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A19*/
  0x2D1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A1A*/
  0x2D1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A1B*/
  0x2D1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A1C*/
  0x2D1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A1D*/
  0x2D20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A1E*/
  0x2D21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A1F*/
  0x2D22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A20*/
  0x2D23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A21*/
  0x2D24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A22*/
  0x2D25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A23*/
  0x2D26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A24*/
  0x2D27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A25*/
  0x2D28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A26*/
  0x2D29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A27*/
  0x2D2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A28*/
  0x2D2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A29*/
  0x2D2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A2A*/
  0x2D2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A2B*/
  0x2D2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A2C*/
  0x2D2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A2D*/
  0x2D30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A2E*/
  0x2D31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A2F*/
  0x2D32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A30*/
  0x2D33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A31*/
  0x2D34, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A32*/
  0x2D35, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A33*/
  0xFBC2, 0x0020, 0x0002, 0x8A34, 0x0000, 0x0000, 0x0002, /*10A34*/
  0xFBC2, 0x0020, 0x0002, 0x8A35, 0x0000, 0x0000, 0x0002, /*10A35*/
  0xFBC2, 0x0020, 0x0002, 0x8A36, 0x0000, 0x0000, 0x0002, /*10A36*/
  0xFBC2, 0x0020, 0x0002, 0x8A37, 0x0000, 0x0000, 0x0002, /*10A37*/
  0x0000, 0x00CC, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A38*/
  0x0000, 0x00CD, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A39*/
  0x0000, 0x00CE, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A3A*/
  0xFBC2, 0x0020, 0x0002, 0x8A3B, 0x0000, 0x0000, 0x0002, /*10A3B*/
  0xFBC2, 0x0020, 0x0002, 0x8A3C, 0x0000, 0x0000, 0x0002, /*10A3C*/
  0xFBC2, 0x0020, 0x0002, 0x8A3D, 0x0000, 0x0000, 0x0002, /*10A3D*/
  0xFBC2, 0x0020, 0x0002, 0x8A3E, 0x0000, 0x0000, 0x0002, /*10A3E*/
  0x2D36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A3F*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A40*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A41*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A42*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A43*/
  0x1B7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A44*/
  0x1B7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A45*/
  0x1B80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A46*/
  0x1B81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A47*/
  0xFBC2, 0x0020, 0x0002, 0x8A48, 0x0000, 0x0000, 0x0002, /*10A48*/
  0xFBC2, 0x0020, 0x0002, 0x8A49, 0x0000, 0x0000, 0x0002, /*10A49*/
  0xFBC2, 0x0020, 0x0002, 0x8A4A, 0x0000, 0x0000, 0x0002, /*10A4A*/
  0xFBC2, 0x0020, 0x0002, 0x8A4B, 0x0000, 0x0000, 0x0002, /*10A4B*/
  0xFBC2, 0x0020, 0x0002, 0x8A4C, 0x0000, 0x0000, 0x0002, /*10A4C*/
  0xFBC2, 0x0020, 0x0002, 0x8A4D, 0x0000, 0x0000, 0x0002, /*10A4D*/
  0xFBC2, 0x0020, 0x0002, 0x8A4E, 0x0000, 0x0000, 0x0002, /*10A4E*/
  0xFBC2, 0x0020, 0x0002, 0x8A4F, 0x0000, 0x0000, 0x0002, /*10A4F*/
  0x0438, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A50*/
  0x0439, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A51*/
  0x043A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A52*/
  0x043B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A53*/
  0x043C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A54*/
  0x043D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A55*/
  0x02AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A56*/
  0x02AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A57*/
  0x043E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A58*/
  0xFBC2, 0x0020, 0x0002, 0x8A59, 0x0000, 0x0000, 0x0002, /*10A59*/
  0xFBC2, 0x0020, 0x0002, 0x8A5A, 0x0000, 0x0000, 0x0002, /*10A5A*/
  0xFBC2, 0x0020, 0x0002, 0x8A5B, 0x0000, 0x0000, 0x0002, /*10A5B*/
  0xFBC2, 0x0020, 0x0002, 0x8A5C, 0x0000, 0x0000, 0x0002, /*10A5C*/
  0xFBC2, 0x0020, 0x0002, 0x8A5D, 0x0000, 0x0000, 0x0002, /*10A5D*/
  0xFBC2, 0x0020, 0x0002, 0x8A5E, 0x0000, 0x0000, 0x0002, /*10A5E*/
  0xFBC2, 0x0020, 0x0002, 0x8A5F, 0x0000, 0x0000, 0x0002, /*10A5F*/
  0x4846, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A60*/
  0x4847, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A61*/
  0x4848, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A62*/
  0x4849, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A63*/
  0x484A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A64*/
  0x484B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A65*/
  0x484C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A66*/
  0x484D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A67*/
  0x484E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A68*/
  0x484F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A69*/
  0x4850, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A6A*/
  0x4851, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A6B*/
  0x4852, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A6C*/
  0x4853, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A6D*/
  0x4854, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A6E*/
  0x4855, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A6F*/
  0x4856, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A70*/
  0x4857, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A71*/
  0x4858, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A72*/
  0x4859, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A73*/
  0x485A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A74*/
  0x485B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A75*/
  0x485C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A76*/
  0x485D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A77*/
  0x485E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A78*/
  0x485F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A79*/
  0x4860, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A7A*/
  0x4861, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A7B*/
  0x4862, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A7C*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A7D*/
  0x1B57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A7E*/
  0x1B58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A7F*/
  0x4863, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A80*/
  0x4864, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A81*/
  0x4865, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A82*/
  0x4866, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A83*/
  0x4867, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A84*/
  0x4868, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A85*/
  0x4869, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A86*/
  0x486A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A87*/
  0x486B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A88*/
  0x486C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A89*/
  0x486D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A8A*/
  0x486E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A8B*/
  0x486F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A8C*/
  0x4870, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A8D*/
  0x4871, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A8E*/
  0x4872, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A8F*/
  0x4873, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A90*/
  0x4874, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A91*/
  0x4875, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A92*/
  0x4876, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A93*/
  0x4877, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A94*/
  0x4878, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A95*/
  0x4879, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A96*/
  0x487A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A97*/
  0x487B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A98*/
  0x487C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A99*/
  0x487D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A9A*/
  0x487E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A9B*/
  0x487F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A9C*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A9D*/
  0x1B59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A9E*/
  0x1B5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10A9F*/
  0xFBC2, 0x0020, 0x0002, 0x8AA0, 0x0000, 0x0000, 0x0002, /*10AA0*/
  0xFBC2, 0x0020, 0x0002, 0x8AA1, 0x0000, 0x0000, 0x0002, /*10AA1*/
  0xFBC2, 0x0020, 0x0002, 0x8AA2, 0x0000, 0x0000, 0x0002, /*10AA2*/
  0xFBC2, 0x0020, 0x0002, 0x8AA3, 0x0000, 0x0000, 0x0002, /*10AA3*/
  0xFBC2, 0x0020, 0x0002, 0x8AA4, 0x0000, 0x0000, 0x0002, /*10AA4*/
  0xFBC2, 0x0020, 0x0002, 0x8AA5, 0x0000, 0x0000, 0x0002, /*10AA5*/
  0xFBC2, 0x0020, 0x0002, 0x8AA6, 0x0000, 0x0000, 0x0002, /*10AA6*/
  0xFBC2, 0x0020, 0x0002, 0x8AA7, 0x0000, 0x0000, 0x0002, /*10AA7*/
  0xFBC2, 0x0020, 0x0002, 0x8AA8, 0x0000, 0x0000, 0x0002, /*10AA8*/
  0xFBC2, 0x0020, 0x0002, 0x8AA9, 0x0000, 0x0000, 0x0002, /*10AA9*/
  0xFBC2, 0x0020, 0x0002, 0x8AAA, 0x0000, 0x0000, 0x0002, /*10AAA*/
  0xFBC2, 0x0020, 0x0002, 0x8AAB, 0x0000, 0x0000, 0x0002, /*10AAB*/
  0xFBC2, 0x0020, 0x0002, 0x8AAC, 0x0000, 0x0000, 0x0002, /*10AAC*/
  0xFBC2, 0x0020, 0x0002, 0x8AAD, 0x0000, 0x0000, 0x0002, /*10AAD*/
  0xFBC2, 0x0020, 0x0002, 0x8AAE, 0x0000, 0x0000, 0x0002, /*10AAE*/
  0xFBC2, 0x0020, 0x0002, 0x8AAF, 0x0000, 0x0000, 0x0002, /*10AAF*/
  0xFBC2, 0x0020, 0x0002, 0x8AB0, 0x0000, 0x0000, 0x0002, /*10AB0*/
  0xFBC2, 0x0020, 0x0002, 0x8AB1, 0x0000, 0x0000, 0x0002, /*10AB1*/
  0xFBC2, 0x0020, 0x0002, 0x8AB2, 0x0000, 0x0000, 0x0002, /*10AB2*/
  0xFBC2, 0x0020, 0x0002, 0x8AB3, 0x0000, 0x0000, 0x0002, /*10AB3*/
  0xFBC2, 0x0020, 0x0002, 0x8AB4, 0x0000, 0x0000, 0x0002, /*10AB4*/
  0xFBC2, 0x0020, 0x0002, 0x8AB5, 0x0000, 0x0000, 0x0002, /*10AB5*/
  0xFBC2, 0x0020, 0x0002, 0x8AB6, 0x0000, 0x0000, 0x0002, /*10AB6*/
  0xFBC2, 0x0020, 0x0002, 0x8AB7, 0x0000, 0x0000, 0x0002, /*10AB7*/
  0xFBC2, 0x0020, 0x0002, 0x8AB8, 0x0000, 0x0000, 0x0002, /*10AB8*/
  0xFBC2, 0x0020, 0x0002, 0x8AB9, 0x0000, 0x0000, 0x0002, /*10AB9*/
  0xFBC2, 0x0020, 0x0002, 0x8ABA, 0x0000, 0x0000, 0x0002, /*10ABA*/
  0xFBC2, 0x0020, 0x0002, 0x8ABB, 0x0000, 0x0000, 0x0002, /*10ABB*/
  0xFBC2, 0x0020, 0x0002, 0x8ABC, 0x0000, 0x0000, 0x0002, /*10ABC*/
  0xFBC2, 0x0020, 0x0002, 0x8ABD, 0x0000, 0x0000, 0x0002, /*10ABD*/
  0xFBC2, 0x0020, 0x0002, 0x8ABE, 0x0000, 0x0000, 0x0002, /*10ABE*/
  0xFBC2, 0x0020, 0x0002, 0x8ABF, 0x0000, 0x0000, 0x0002, /*10ABF*/
  0x4947, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AC0*/
  0x4948, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AC1*/
  0x4949, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AC2*/
  0x494A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AC3*/
  0x494B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AC4*/
  0x494C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AC5*/
  0x494D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AC6*/
  0x494E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AC7*/
  0x494E, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*10AC8*/
  0x494F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AC9*/
  0x4950, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10ACA*/
  0x4951, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10ACB*/
  0x4952, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10ACC*/
  0x4953, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10ACD*/
  0x4954, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10ACE*/
  0x4955, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10ACF*/
  0x4956, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AD0*/
  0x4957, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AD1*/
  0x4958, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AD2*/
  0x4959, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AD3*/
  0x495A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AD4*/
  0x495B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AD5*/
  0x495C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AD6*/
  0x495D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AD7*/
  0x495E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AD8*/
  0x495F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AD9*/
  0x4960, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10ADA*/
  0x4961, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10ADB*/
  0x4962, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10ADC*/
  0x4963, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10ADD*/
  0x4964, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10ADE*/
  0x4965, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10ADF*/
  0x4966, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AE0*/
  0x4967, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AE1*/
  0x4968, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AE2*/
  0x4969, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AE3*/
  0x496A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AE4*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AE5*/
  0x0000, 0x0034, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AE6*/
  0xFBC2, 0x0020, 0x0002, 0x8AE7, 0x0000, 0x0000, 0x0002, /*10AE7*/
  0xFBC2, 0x0020, 0x0002, 0x8AE8, 0x0000, 0x0000, 0x0002, /*10AE8*/
  0xFBC2, 0x0020, 0x0002, 0x8AE9, 0x0000, 0x0000, 0x0002, /*10AE9*/
  0xFBC2, 0x0020, 0x0002, 0x8AEA, 0x0000, 0x0000, 0x0002, /*10AEA*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AEB*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AEC*/
  0x1B63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AED*/
  0x1B64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AEE*/
  0x1B65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AEF*/
  0x0443, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AF0*/
  0x0444, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AF1*/
  0x0445, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AF2*/
  0x0446, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AF3*/
  0x0447, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AF4*/
  0x0448, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AF5*/
  0x0449, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10AF6*/
  0xFBC2, 0x0020, 0x0002, 0x8AF7, 0x0000, 0x0000, 0x0002, /*10AF7*/
  0xFBC2, 0x0020, 0x0002, 0x8AF8, 0x0000, 0x0000, 0x0002, /*10AF8*/
  0xFBC2, 0x0020, 0x0002, 0x8AF9, 0x0000, 0x0000, 0x0002, /*10AF9*/
  0xFBC2, 0x0020, 0x0002, 0x8AFA, 0x0000, 0x0000, 0x0002, /*10AFA*/
  0xFBC2, 0x0020, 0x0002, 0x8AFB, 0x0000, 0x0000, 0x0002, /*10AFB*/
  0xFBC2, 0x0020, 0x0002, 0x8AFC, 0x0000, 0x0000, 0x0002, /*10AFC*/
  0xFBC2, 0x0020, 0x0002, 0x8AFD, 0x0000, 0x0000, 0x0002, /*10AFD*/
  0xFBC2, 0x0020, 0x0002, 0x8AFE, 0x0000, 0x0000, 0x0002, /*10AFE*/
  0xFBC2, 0x0020, 0x0002, 0x8AFF, 0x0000, 0x0000, 0x0002  /*10AFF*/
};

uint16 uca900_p10B[]= {
  0x4880, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B00*/
  0x4881, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B01*/
  0x4882, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B02*/
  0x4883, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B03*/
  0x4884, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B04*/
  0x4885, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B05*/
  0x4886, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B06*/
  0x4887, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B07*/
  0x4888, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B08*/
  0x4889, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B09*/
  0x488A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B0A*/
  0x488B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B0B*/
  0x488C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B0C*/
  0x488D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B0D*/
  0x488E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B0E*/
  0x488F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B0F*/
  0x4890, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B10*/
  0x4891, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B11*/
  0x4892, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B12*/
  0x4893, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B13*/
  0x4894, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B14*/
  0x4895, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B15*/
  0x4896, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B16*/
  0x4897, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B17*/
  0x4898, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B18*/
  0x4899, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B19*/
  0x489A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B1A*/
  0x489B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B1B*/
  0x489C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B1C*/
  0x489D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B1D*/
  0x489E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B1E*/
  0x489F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B1F*/
  0x48A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B20*/
  0x48A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B21*/
  0x48A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B22*/
  0x48A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B23*/
  0x48A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B24*/
  0x48A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B25*/
  0x48A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B26*/
  0x48A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B27*/
  0x48A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B28*/
  0x48A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B29*/
  0x48AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B2A*/
  0x48AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B2B*/
  0x48AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B2C*/
  0x48AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B2D*/
  0x48AD, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10B2E*/
  0x48AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B2F*/
  0x48AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B30*/
  0x48B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B31*/
  0x48B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B32*/
  0x48B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B33*/
  0x48B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B34*/
  0x48B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B35*/
  0xFBC2, 0x0020, 0x0002, 0x8B36, 0x0000, 0x0000, 0x0002, /*10B36*/
  0xFBC2, 0x0020, 0x0002, 0x8B37, 0x0000, 0x0000, 0x0002, /*10B37*/
  0xFBC2, 0x0020, 0x0002, 0x8B38, 0x0000, 0x0000, 0x0002, /*10B38*/
  0x0442, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B39*/
  0x02DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B3A*/
  0x02DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B3B*/
  0x02DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B3C*/
  0x02DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B3D*/
  0x02DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B3E*/
  0x02DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B3F*/
  0x490C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B40*/
  0x490D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B41*/
  0x490E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B42*/
  0x490F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B43*/
  0x4910, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B44*/
  0x4911, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B45*/
  0x4912, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B46*/
  0x4913, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B47*/
  0x4914, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B48*/
  0x4915, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B49*/
  0x4916, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B4A*/
  0x4917, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B4B*/
  0x4918, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B4C*/
  0x4919, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B4D*/
  0x491A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B4E*/
  0x491B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B4F*/
  0x491C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B50*/
  0x491D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B51*/
  0x491E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B52*/
  0x491F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B53*/
  0x4920, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B54*/
  0x4921, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B55*/
  0xFBC2, 0x0020, 0x0002, 0x8B56, 0x0000, 0x0000, 0x0002, /*10B56*/
  0xFBC2, 0x0020, 0x0002, 0x8B57, 0x0000, 0x0000, 0x0002, /*10B57*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B58*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B59*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B5A*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B5B*/
  0x1B66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B5C*/
  0x1B67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B5D*/
  0x1B68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B5E*/
  0x1B69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B5F*/
  0x4922, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B60*/
  0x4923, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B61*/
  0x4924, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B62*/
  0x4925, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B63*/
  0x4926, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B64*/
  0x4927, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B65*/
  0x4928, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B66*/
  0x4929, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B67*/
  0x492A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B68*/
  0x492B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B69*/
  0x492C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B6A*/
  0x492D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B6B*/
  0x492E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B6C*/
  0x492F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B6D*/
  0x4930, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B6E*/
  0x4931, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B6F*/
  0x4932, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B70*/
  0x4933, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B71*/
  0x4934, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B72*/
  0xFBC2, 0x0020, 0x0002, 0x8B73, 0x0000, 0x0000, 0x0002, /*10B73*/
  0xFBC2, 0x0020, 0x0002, 0x8B74, 0x0000, 0x0000, 0x0002, /*10B74*/
  0xFBC2, 0x0020, 0x0002, 0x8B75, 0x0000, 0x0000, 0x0002, /*10B75*/
  0xFBC2, 0x0020, 0x0002, 0x8B76, 0x0000, 0x0000, 0x0002, /*10B76*/
  0xFBC2, 0x0020, 0x0002, 0x8B77, 0x0000, 0x0000, 0x0002, /*10B77*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B78*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B79*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B7A*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B7B*/
  0x1B6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B7C*/
  0x1B6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B7D*/
  0x1B6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B7E*/
  0x1B6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B7F*/
  0x4935, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B80*/
  0x4936, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B81*/
  0x4937, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B82*/
  0x4938, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B83*/
  0x4939, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B84*/
  0x493A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B85*/
  0x493B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B86*/
  0x493C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B87*/
  0x493D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B88*/
  0x493E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B89*/
  0x493F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B8A*/
  0x4940, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B8B*/
  0x4941, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B8C*/
  0x4942, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B8D*/
  0x4943, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B8E*/
  0x4944, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B8F*/
  0x4945, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B90*/
  0x4946, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B91*/
  0xFBC2, 0x0020, 0x0002, 0x8B92, 0x0000, 0x0000, 0x0002, /*10B92*/
  0xFBC2, 0x0020, 0x0002, 0x8B93, 0x0000, 0x0000, 0x0002, /*10B93*/
  0xFBC2, 0x0020, 0x0002, 0x8B94, 0x0000, 0x0000, 0x0002, /*10B94*/
  0xFBC2, 0x0020, 0x0002, 0x8B95, 0x0000, 0x0000, 0x0002, /*10B95*/
  0xFBC2, 0x0020, 0x0002, 0x8B96, 0x0000, 0x0000, 0x0002, /*10B96*/
  0xFBC2, 0x0020, 0x0002, 0x8B97, 0x0000, 0x0000, 0x0002, /*10B97*/
  0xFBC2, 0x0020, 0x0002, 0x8B98, 0x0000, 0x0000, 0x0002, /*10B98*/
  0x044A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B99*/
  0x044B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B9A*/
  0x044C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B9B*/
  0x044D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10B9C*/
  0xFBC2, 0x0020, 0x0002, 0x8B9D, 0x0000, 0x0000, 0x0002, /*10B9D*/
  0xFBC2, 0x0020, 0x0002, 0x8B9E, 0x0000, 0x0000, 0x0002, /*10B9E*/
  0xFBC2, 0x0020, 0x0002, 0x8B9F, 0x0000, 0x0000, 0x0002, /*10B9F*/
  0xFBC2, 0x0020, 0x0002, 0x8BA0, 0x0000, 0x0000, 0x0002, /*10BA0*/
  0xFBC2, 0x0020, 0x0002, 0x8BA1, 0x0000, 0x0000, 0x0002, /*10BA1*/
  0xFBC2, 0x0020, 0x0002, 0x8BA2, 0x0000, 0x0000, 0x0002, /*10BA2*/
  0xFBC2, 0x0020, 0x0002, 0x8BA3, 0x0000, 0x0000, 0x0002, /*10BA3*/
  0xFBC2, 0x0020, 0x0002, 0x8BA4, 0x0000, 0x0000, 0x0002, /*10BA4*/
  0xFBC2, 0x0020, 0x0002, 0x8BA5, 0x0000, 0x0000, 0x0002, /*10BA5*/
  0xFBC2, 0x0020, 0x0002, 0x8BA6, 0x0000, 0x0000, 0x0002, /*10BA6*/
  0xFBC2, 0x0020, 0x0002, 0x8BA7, 0x0000, 0x0000, 0x0002, /*10BA7*/
  0xFBC2, 0x0020, 0x0002, 0x8BA8, 0x0000, 0x0000, 0x0002, /*10BA8*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10BA9*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10BAA*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10BAB*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10BAC*/
  0x1B6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10BAD*/
  0x1B6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10BAE*/
  0x1B70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10BAF*/
  0xFBC2, 0x0020, 0x0002, 0x8BB0, 0x0000, 0x0000, 0x0002, /*10BB0*/
  0xFBC2, 0x0020, 0x0002, 0x8BB1, 0x0000, 0x0000, 0x0002, /*10BB1*/
  0xFBC2, 0x0020, 0x0002, 0x8BB2, 0x0000, 0x0000, 0x0002, /*10BB2*/
  0xFBC2, 0x0020, 0x0002, 0x8BB3, 0x0000, 0x0000, 0x0002, /*10BB3*/
  0xFBC2, 0x0020, 0x0002, 0x8BB4, 0x0000, 0x0000, 0x0002, /*10BB4*/
  0xFBC2, 0x0020, 0x0002, 0x8BB5, 0x0000, 0x0000, 0x0002, /*10BB5*/
  0xFBC2, 0x0020, 0x0002, 0x8BB6, 0x0000, 0x0000, 0x0002, /*10BB6*/
  0xFBC2, 0x0020, 0x0002, 0x8BB7, 0x0000, 0x0000, 0x0002, /*10BB7*/
  0xFBC2, 0x0020, 0x0002, 0x8BB8, 0x0000, 0x0000, 0x0002, /*10BB8*/
  0xFBC2, 0x0020, 0x0002, 0x8BB9, 0x0000, 0x0000, 0x0002, /*10BB9*/
  0xFBC2, 0x0020, 0x0002, 0x8BBA, 0x0000, 0x0000, 0x0002, /*10BBA*/
  0xFBC2, 0x0020, 0x0002, 0x8BBB, 0x0000, 0x0000, 0x0002, /*10BBB*/
  0xFBC2, 0x0020, 0x0002, 0x8BBC, 0x0000, 0x0000, 0x0002, /*10BBC*/
  0xFBC2, 0x0020, 0x0002, 0x8BBD, 0x0000, 0x0000, 0x0002, /*10BBD*/
  0xFBC2, 0x0020, 0x0002, 0x8BBE, 0x0000, 0x0000, 0x0002, /*10BBE*/
  0xFBC2, 0x0020, 0x0002, 0x8BBF, 0x0000, 0x0000, 0x0002, /*10BBF*/
  0xFBC2, 0x0020, 0x0002, 0x8BC0, 0x0000, 0x0000, 0x0002, /*10BC0*/
  0xFBC2, 0x0020, 0x0002, 0x8BC1, 0x0000, 0x0000, 0x0002, /*10BC1*/
  0xFBC2, 0x0020, 0x0002, 0x8BC2, 0x0000, 0x0000, 0x0002, /*10BC2*/
  0xFBC2, 0x0020, 0x0002, 0x8BC3, 0x0000, 0x0000, 0x0002, /*10BC3*/
  0xFBC2, 0x0020, 0x0002, 0x8BC4, 0x0000, 0x0000, 0x0002, /*10BC4*/
  0xFBC2, 0x0020, 0x0002, 0x8BC5, 0x0000, 0x0000, 0x0002, /*10BC5*/
  0xFBC2, 0x0020, 0x0002, 0x8BC6, 0x0000, 0x0000, 0x0002, /*10BC6*/
  0xFBC2, 0x0020, 0x0002, 0x8BC7, 0x0000, 0x0000, 0x0002, /*10BC7*/
  0xFBC2, 0x0020, 0x0002, 0x8BC8, 0x0000, 0x0000, 0x0002, /*10BC8*/
  0xFBC2, 0x0020, 0x0002, 0x8BC9, 0x0000, 0x0000, 0x0002, /*10BC9*/
  0xFBC2, 0x0020, 0x0002, 0x8BCA, 0x0000, 0x0000, 0x0002, /*10BCA*/
  0xFBC2, 0x0020, 0x0002, 0x8BCB, 0x0000, 0x0000, 0x0002, /*10BCB*/
  0xFBC2, 0x0020, 0x0002, 0x8BCC, 0x0000, 0x0000, 0x0002, /*10BCC*/
  0xFBC2, 0x0020, 0x0002, 0x8BCD, 0x0000, 0x0000, 0x0002, /*10BCD*/
  0xFBC2, 0x0020, 0x0002, 0x8BCE, 0x0000, 0x0000, 0x0002, /*10BCE*/
  0xFBC2, 0x0020, 0x0002, 0x8BCF, 0x0000, 0x0000, 0x0002, /*10BCF*/
  0xFBC2, 0x0020, 0x0002, 0x8BD0, 0x0000, 0x0000, 0x0002, /*10BD0*/
  0xFBC2, 0x0020, 0x0002, 0x8BD1, 0x0000, 0x0000, 0x0002, /*10BD1*/
  0xFBC2, 0x0020, 0x0002, 0x8BD2, 0x0000, 0x0000, 0x0002, /*10BD2*/
  0xFBC2, 0x0020, 0x0002, 0x8BD3, 0x0000, 0x0000, 0x0002, /*10BD3*/
  0xFBC2, 0x0020, 0x0002, 0x8BD4, 0x0000, 0x0000, 0x0002, /*10BD4*/
  0xFBC2, 0x0020, 0x0002, 0x8BD5, 0x0000, 0x0000, 0x0002, /*10BD5*/
  0xFBC2, 0x0020, 0x0002, 0x8BD6, 0x0000, 0x0000, 0x0002, /*10BD6*/
  0xFBC2, 0x0020, 0x0002, 0x8BD7, 0x0000, 0x0000, 0x0002, /*10BD7*/
  0xFBC2, 0x0020, 0x0002, 0x8BD8, 0x0000, 0x0000, 0x0002, /*10BD8*/
  0xFBC2, 0x0020, 0x0002, 0x8BD9, 0x0000, 0x0000, 0x0002, /*10BD9*/
  0xFBC2, 0x0020, 0x0002, 0x8BDA, 0x0000, 0x0000, 0x0002, /*10BDA*/
  0xFBC2, 0x0020, 0x0002, 0x8BDB, 0x0000, 0x0000, 0x0002, /*10BDB*/
  0xFBC2, 0x0020, 0x0002, 0x8BDC, 0x0000, 0x0000, 0x0002, /*10BDC*/
  0xFBC2, 0x0020, 0x0002, 0x8BDD, 0x0000, 0x0000, 0x0002, /*10BDD*/
  0xFBC2, 0x0020, 0x0002, 0x8BDE, 0x0000, 0x0000, 0x0002, /*10BDE*/
  0xFBC2, 0x0020, 0x0002, 0x8BDF, 0x0000, 0x0000, 0x0002, /*10BDF*/
  0xFBC2, 0x0020, 0x0002, 0x8BE0, 0x0000, 0x0000, 0x0002, /*10BE0*/
  0xFBC2, 0x0020, 0x0002, 0x8BE1, 0x0000, 0x0000, 0x0002, /*10BE1*/
  0xFBC2, 0x0020, 0x0002, 0x8BE2, 0x0000, 0x0000, 0x0002, /*10BE2*/
  0xFBC2, 0x0020, 0x0002, 0x8BE3, 0x0000, 0x0000, 0x0002, /*10BE3*/
  0xFBC2, 0x0020, 0x0002, 0x8BE4, 0x0000, 0x0000, 0x0002, /*10BE4*/
  0xFBC2, 0x0020, 0x0002, 0x8BE5, 0x0000, 0x0000, 0x0002, /*10BE5*/
  0xFBC2, 0x0020, 0x0002, 0x8BE6, 0x0000, 0x0000, 0x0002, /*10BE6*/
  0xFBC2, 0x0020, 0x0002, 0x8BE7, 0x0000, 0x0000, 0x0002, /*10BE7*/
  0xFBC2, 0x0020, 0x0002, 0x8BE8, 0x0000, 0x0000, 0x0002, /*10BE8*/
  0xFBC2, 0x0020, 0x0002, 0x8BE9, 0x0000, 0x0000, 0x0002, /*10BE9*/
  0xFBC2, 0x0020, 0x0002, 0x8BEA, 0x0000, 0x0000, 0x0002, /*10BEA*/
  0xFBC2, 0x0020, 0x0002, 0x8BEB, 0x0000, 0x0000, 0x0002, /*10BEB*/
  0xFBC2, 0x0020, 0x0002, 0x8BEC, 0x0000, 0x0000, 0x0002, /*10BEC*/
  0xFBC2, 0x0020, 0x0002, 0x8BED, 0x0000, 0x0000, 0x0002, /*10BED*/
  0xFBC2, 0x0020, 0x0002, 0x8BEE, 0x0000, 0x0000, 0x0002, /*10BEE*/
  0xFBC2, 0x0020, 0x0002, 0x8BEF, 0x0000, 0x0000, 0x0002, /*10BEF*/
  0xFBC2, 0x0020, 0x0002, 0x8BF0, 0x0000, 0x0000, 0x0002, /*10BF0*/
  0xFBC2, 0x0020, 0x0002, 0x8BF1, 0x0000, 0x0000, 0x0002, /*10BF1*/
  0xFBC2, 0x0020, 0x0002, 0x8BF2, 0x0000, 0x0000, 0x0002, /*10BF2*/
  0xFBC2, 0x0020, 0x0002, 0x8BF3, 0x0000, 0x0000, 0x0002, /*10BF3*/
  0xFBC2, 0x0020, 0x0002, 0x8BF4, 0x0000, 0x0000, 0x0002, /*10BF4*/
  0xFBC2, 0x0020, 0x0002, 0x8BF5, 0x0000, 0x0000, 0x0002, /*10BF5*/
  0xFBC2, 0x0020, 0x0002, 0x8BF6, 0x0000, 0x0000, 0x0002, /*10BF6*/
  0xFBC2, 0x0020, 0x0002, 0x8BF7, 0x0000, 0x0000, 0x0002, /*10BF7*/
  0xFBC2, 0x0020, 0x0002, 0x8BF8, 0x0000, 0x0000, 0x0002, /*10BF8*/
  0xFBC2, 0x0020, 0x0002, 0x8BF9, 0x0000, 0x0000, 0x0002, /*10BF9*/
  0xFBC2, 0x0020, 0x0002, 0x8BFA, 0x0000, 0x0000, 0x0002, /*10BFA*/
  0xFBC2, 0x0020, 0x0002, 0x8BFB, 0x0000, 0x0000, 0x0002, /*10BFB*/
  0xFBC2, 0x0020, 0x0002, 0x8BFC, 0x0000, 0x0000, 0x0002, /*10BFC*/
  0xFBC2, 0x0020, 0x0002, 0x8BFD, 0x0000, 0x0000, 0x0002, /*10BFD*/
  0xFBC2, 0x0020, 0x0002, 0x8BFE, 0x0000, 0x0000, 0x0002, /*10BFE*/
  0xFBC2, 0x0020, 0x0002, 0x8BFF, 0x0000, 0x0000, 0x0002  /*10BFF*/
};

uint16 uca900_p10C[]= {
  0x372D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C00*/
  0x372D, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C01*/
  0x372E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C02*/
  0x372F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C03*/
  0x372F, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C04*/
  0x3730, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C05*/
  0x3731, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C06*/
  0x3732, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C07*/
  0x3732, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C08*/
  0x3733, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C09*/
  0x3733, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C0A*/
  0x3734, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C0B*/
  0x3734, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C0C*/
  0x3735, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C0D*/
  0x3735, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C0E*/
  0x3736, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C0F*/
  0x3736, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C10*/
  0x3737, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C11*/
  0x3737, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C12*/
  0x3738, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C13*/
  0x3739, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C14*/
  0x3739, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C15*/
  0x373A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C16*/
  0x373A, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C17*/
  0x373B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C18*/
  0x373B, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C19*/
  0x373C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C1A*/
  0x373C, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C1B*/
  0x373D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C1C*/
  0x373D, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C1D*/
  0x373E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C1E*/
  0x373E, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C1F*/
  0x373F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C20*/
  0x3740, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C21*/
  0x3741, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C22*/
  0x3742, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C23*/
  0x3743, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C24*/
  0x3743, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C25*/
  0x3744, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C26*/
  0x3744, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C27*/
  0x3745, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C28*/
  0x3745, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C29*/
  0x3746, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C2A*/
  0x3746, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C2B*/
  0x3747, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C2C*/
  0x3748, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C2D*/
  0x3748, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C2E*/
  0x3749, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C2F*/
  0x374A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C30*/
  0x374B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C31*/
  0x374C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C32*/
  0x374C, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C33*/
  0x374D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C34*/
  0x374D, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C35*/
  0x374E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C36*/
  0x374E, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C37*/
  0x374F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C38*/
  0x374F, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C39*/
  0x3750, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C3A*/
  0x3750, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C3B*/
  0x3751, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C3C*/
  0x3752, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C3D*/
  0x3753, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C3E*/
  0x3754, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C3F*/
  0x3754, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C40*/
  0x3755, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C41*/
  0x3755, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C42*/
  0x3756, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C43*/
  0x3756, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C44*/
  0x3757, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C45*/
  0x3757, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10C46*/
  0x3758, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C47*/
  0x3759, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10C48*/
  0xFBC2, 0x0020, 0x0002, 0x8C49, 0x0000, 0x0000, 0x0002, /*10C49*/
  0xFBC2, 0x0020, 0x0002, 0x8C4A, 0x0000, 0x0000, 0x0002, /*10C4A*/
  0xFBC2, 0x0020, 0x0002, 0x8C4B, 0x0000, 0x0000, 0x0002, /*10C4B*/
  0xFBC2, 0x0020, 0x0002, 0x8C4C, 0x0000, 0x0000, 0x0002, /*10C4C*/
  0xFBC2, 0x0020, 0x0002, 0x8C4D, 0x0000, 0x0000, 0x0002, /*10C4D*/
  0xFBC2, 0x0020, 0x0002, 0x8C4E, 0x0000, 0x0000, 0x0002, /*10C4E*/
  0xFBC2, 0x0020, 0x0002, 0x8C4F, 0x0000, 0x0000, 0x0002, /*10C4F*/
  0xFBC2, 0x0020, 0x0002, 0x8C50, 0x0000, 0x0000, 0x0002, /*10C50*/
  0xFBC2, 0x0020, 0x0002, 0x8C51, 0x0000, 0x0000, 0x0002, /*10C51*/
  0xFBC2, 0x0020, 0x0002, 0x8C52, 0x0000, 0x0000, 0x0002, /*10C52*/
  0xFBC2, 0x0020, 0x0002, 0x8C53, 0x0000, 0x0000, 0x0002, /*10C53*/
  0xFBC2, 0x0020, 0x0002, 0x8C54, 0x0000, 0x0000, 0x0002, /*10C54*/
  0xFBC2, 0x0020, 0x0002, 0x8C55, 0x0000, 0x0000, 0x0002, /*10C55*/
  0xFBC2, 0x0020, 0x0002, 0x8C56, 0x0000, 0x0000, 0x0002, /*10C56*/
  0xFBC2, 0x0020, 0x0002, 0x8C57, 0x0000, 0x0000, 0x0002, /*10C57*/
  0xFBC2, 0x0020, 0x0002, 0x8C58, 0x0000, 0x0000, 0x0002, /*10C58*/
  0xFBC2, 0x0020, 0x0002, 0x8C59, 0x0000, 0x0000, 0x0002, /*10C59*/
  0xFBC2, 0x0020, 0x0002, 0x8C5A, 0x0000, 0x0000, 0x0002, /*10C5A*/
  0xFBC2, 0x0020, 0x0002, 0x8C5B, 0x0000, 0x0000, 0x0002, /*10C5B*/
  0xFBC2, 0x0020, 0x0002, 0x8C5C, 0x0000, 0x0000, 0x0002, /*10C5C*/
  0xFBC2, 0x0020, 0x0002, 0x8C5D, 0x0000, 0x0000, 0x0002, /*10C5D*/
  0xFBC2, 0x0020, 0x0002, 0x8C5E, 0x0000, 0x0000, 0x0002, /*10C5E*/
  0xFBC2, 0x0020, 0x0002, 0x8C5F, 0x0000, 0x0000, 0x0002, /*10C5F*/
  0xFBC2, 0x0020, 0x0002, 0x8C60, 0x0000, 0x0000, 0x0002, /*10C60*/
  0xFBC2, 0x0020, 0x0002, 0x8C61, 0x0000, 0x0000, 0x0002, /*10C61*/
  0xFBC2, 0x0020, 0x0002, 0x8C62, 0x0000, 0x0000, 0x0002, /*10C62*/
  0xFBC2, 0x0020, 0x0002, 0x8C63, 0x0000, 0x0000, 0x0002, /*10C63*/
  0xFBC2, 0x0020, 0x0002, 0x8C64, 0x0000, 0x0000, 0x0002, /*10C64*/
  0xFBC2, 0x0020, 0x0002, 0x8C65, 0x0000, 0x0000, 0x0002, /*10C65*/
  0xFBC2, 0x0020, 0x0002, 0x8C66, 0x0000, 0x0000, 0x0002, /*10C66*/
  0xFBC2, 0x0020, 0x0002, 0x8C67, 0x0000, 0x0000, 0x0002, /*10C67*/
  0xFBC2, 0x0020, 0x0002, 0x8C68, 0x0000, 0x0000, 0x0002, /*10C68*/
  0xFBC2, 0x0020, 0x0002, 0x8C69, 0x0000, 0x0000, 0x0002, /*10C69*/
  0xFBC2, 0x0020, 0x0002, 0x8C6A, 0x0000, 0x0000, 0x0002, /*10C6A*/
  0xFBC2, 0x0020, 0x0002, 0x8C6B, 0x0000, 0x0000, 0x0002, /*10C6B*/
  0xFBC2, 0x0020, 0x0002, 0x8C6C, 0x0000, 0x0000, 0x0002, /*10C6C*/
  0xFBC2, 0x0020, 0x0002, 0x8C6D, 0x0000, 0x0000, 0x0002, /*10C6D*/
  0xFBC2, 0x0020, 0x0002, 0x8C6E, 0x0000, 0x0000, 0x0002, /*10C6E*/
  0xFBC2, 0x0020, 0x0002, 0x8C6F, 0x0000, 0x0000, 0x0002, /*10C6F*/
  0xFBC2, 0x0020, 0x0002, 0x8C70, 0x0000, 0x0000, 0x0002, /*10C70*/
  0xFBC2, 0x0020, 0x0002, 0x8C71, 0x0000, 0x0000, 0x0002, /*10C71*/
  0xFBC2, 0x0020, 0x0002, 0x8C72, 0x0000, 0x0000, 0x0002, /*10C72*/
  0xFBC2, 0x0020, 0x0002, 0x8C73, 0x0000, 0x0000, 0x0002, /*10C73*/
  0xFBC2, 0x0020, 0x0002, 0x8C74, 0x0000, 0x0000, 0x0002, /*10C74*/
  0xFBC2, 0x0020, 0x0002, 0x8C75, 0x0000, 0x0000, 0x0002, /*10C75*/
  0xFBC2, 0x0020, 0x0002, 0x8C76, 0x0000, 0x0000, 0x0002, /*10C76*/
  0xFBC2, 0x0020, 0x0002, 0x8C77, 0x0000, 0x0000, 0x0002, /*10C77*/
  0xFBC2, 0x0020, 0x0002, 0x8C78, 0x0000, 0x0000, 0x0002, /*10C78*/
  0xFBC2, 0x0020, 0x0002, 0x8C79, 0x0000, 0x0000, 0x0002, /*10C79*/
  0xFBC2, 0x0020, 0x0002, 0x8C7A, 0x0000, 0x0000, 0x0002, /*10C7A*/
  0xFBC2, 0x0020, 0x0002, 0x8C7B, 0x0000, 0x0000, 0x0002, /*10C7B*/
  0xFBC2, 0x0020, 0x0002, 0x8C7C, 0x0000, 0x0000, 0x0002, /*10C7C*/
  0xFBC2, 0x0020, 0x0002, 0x8C7D, 0x0000, 0x0000, 0x0002, /*10C7D*/
  0xFBC2, 0x0020, 0x0002, 0x8C7E, 0x0000, 0x0000, 0x0002, /*10C7E*/
  0xFBC2, 0x0020, 0x0002, 0x8C7F, 0x0000, 0x0000, 0x0002, /*10C7F*/
  0x3704, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C80*/
  0x3704, 0x0020, 0x000A, 0x0000, 0x0110, 0x0004, 0x0002, /*10C81*/
  0x3705, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C82*/
  0x3706, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C83*/
  0x3707, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C84*/
  0x3708, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C85*/
  0x3709, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C86*/
  0x370A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C87*/
  0x370B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C88*/
  0x370C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C89*/
  0x370C, 0x0020, 0x000A, 0x0000, 0x0110, 0x0004, 0x0002, /*10C8A*/
  0x370C, 0x0020, 0x000A, 0x0000, 0x0111, 0x0004, 0x0002, /*10C8B*/
  0x370D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C8C*/
  0x370E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C8D*/
  0x370F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C8E*/
  0x3710, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C8F*/
  0x3711, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C90*/
  0x3711, 0x0020, 0x000A, 0x0000, 0x0110, 0x0004, 0x0002, /*10C91*/
  0x3712, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C92*/
  0x3713, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C93*/
  0x3714, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C94*/
  0x3715, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C95*/
  0x3716, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C96*/
  0x3717, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C97*/
  0x3718, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C98*/
  0x3719, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C99*/
  0x371A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C9A*/
  0x371B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C9B*/
  0x371B, 0x0020, 0x000A, 0x0000, 0x0110, 0x0004, 0x0002, /*10C9C*/
  0x371C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10C9D*/
  0x371C, 0x0020, 0x000A, 0x0000, 0x0110, 0x0004, 0x0002, /*10C9E*/
  0x371C, 0x0020, 0x000A, 0x0000, 0x0111, 0x0004, 0x0002, /*10C9F*/
  0x371D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CA0*/
  0x371E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CA1*/
  0x371F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CA2*/
  0x371F, 0x0020, 0x000A, 0x0000, 0x0110, 0x0004, 0x0002, /*10CA3*/
  0x3720, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CA4*/
  0x3721, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CA5*/
  0x3722, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CA6*/
  0x3723, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CA7*/
  0x3724, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CA8*/
  0x3725, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CA9*/
  0x3726, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CAA*/
  0x3726, 0x0020, 0x000A, 0x0000, 0x0110, 0x0004, 0x0002, /*10CAB*/
  0x3727, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CAC*/
  0x3727, 0x0020, 0x000A, 0x0000, 0x0110, 0x0004, 0x0002, /*10CAD*/
  0x3728, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CAE*/
  0x3729, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CAF*/
  0x372A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CB0*/
  0x372B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CB1*/
  0x372C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*10CB2*/
  0xFBC2, 0x0020, 0x0002, 0x8CB3, 0x0000, 0x0000, 0x0002, /*10CB3*/
  0xFBC2, 0x0020, 0x0002, 0x8CB4, 0x0000, 0x0000, 0x0002, /*10CB4*/
  0xFBC2, 0x0020, 0x0002, 0x8CB5, 0x0000, 0x0000, 0x0002, /*10CB5*/
  0xFBC2, 0x0020, 0x0002, 0x8CB6, 0x0000, 0x0000, 0x0002, /*10CB6*/
  0xFBC2, 0x0020, 0x0002, 0x8CB7, 0x0000, 0x0000, 0x0002, /*10CB7*/
  0xFBC2, 0x0020, 0x0002, 0x8CB8, 0x0000, 0x0000, 0x0002, /*10CB8*/
  0xFBC2, 0x0020, 0x0002, 0x8CB9, 0x0000, 0x0000, 0x0002, /*10CB9*/
  0xFBC2, 0x0020, 0x0002, 0x8CBA, 0x0000, 0x0000, 0x0002, /*10CBA*/
  0xFBC2, 0x0020, 0x0002, 0x8CBB, 0x0000, 0x0000, 0x0002, /*10CBB*/
  0xFBC2, 0x0020, 0x0002, 0x8CBC, 0x0000, 0x0000, 0x0002, /*10CBC*/
  0xFBC2, 0x0020, 0x0002, 0x8CBD, 0x0000, 0x0000, 0x0002, /*10CBD*/
  0xFBC2, 0x0020, 0x0002, 0x8CBE, 0x0000, 0x0000, 0x0002, /*10CBE*/
  0xFBC2, 0x0020, 0x0002, 0x8CBF, 0x0000, 0x0000, 0x0002, /*10CBF*/
  0x3704, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CC0*/
  0x3704, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10CC1*/
  0x3705, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CC2*/
  0x3706, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CC3*/
  0x3707, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CC4*/
  0x3708, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CC5*/
  0x3709, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CC6*/
  0x370A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CC7*/
  0x370B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CC8*/
  0x370C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CC9*/
  0x370C, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10CCA*/
  0x370C, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*10CCB*/
  0x370D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CCC*/
  0x370E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CCD*/
  0x370F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CCE*/
  0x3710, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CCF*/
  0x3711, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CD0*/
  0x3711, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10CD1*/
  0x3712, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CD2*/
  0x3713, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CD3*/
  0x3714, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CD4*/
  0x3715, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CD5*/
  0x3716, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CD6*/
  0x3717, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CD7*/
  0x3718, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CD8*/
  0x3719, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CD9*/
  0x371A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CDA*/
  0x371B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CDB*/
  0x371B, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10CDC*/
  0x371C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CDD*/
  0x371C, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10CDE*/
  0x371C, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*10CDF*/
  0x371D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CE0*/
  0x371E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CE1*/
  0x371F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CE2*/
  0x371F, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10CE3*/
  0x3720, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CE4*/
  0x3721, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CE5*/
  0x3722, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CE6*/
  0x3723, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CE7*/
  0x3724, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CE8*/
  0x3725, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CE9*/
  0x3726, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CEA*/
  0x3726, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10CEB*/
  0x3727, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CEC*/
  0x3727, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*10CED*/
  0x3728, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CEE*/
  0x3729, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CEF*/
  0x372A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CF0*/
  0x372B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CF1*/
  0x372C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CF2*/
  0xFBC2, 0x0020, 0x0002, 0x8CF3, 0x0000, 0x0000, 0x0002, /*10CF3*/
  0xFBC2, 0x0020, 0x0002, 0x8CF4, 0x0000, 0x0000, 0x0002, /*10CF4*/
  0xFBC2, 0x0020, 0x0002, 0x8CF5, 0x0000, 0x0000, 0x0002, /*10CF5*/
  0xFBC2, 0x0020, 0x0002, 0x8CF6, 0x0000, 0x0000, 0x0002, /*10CF6*/
  0xFBC2, 0x0020, 0x0002, 0x8CF7, 0x0000, 0x0000, 0x0002, /*10CF7*/
  0xFBC2, 0x0020, 0x0002, 0x8CF8, 0x0000, 0x0000, 0x0002, /*10CF8*/
  0xFBC2, 0x0020, 0x0002, 0x8CF9, 0x0000, 0x0000, 0x0002, /*10CF9*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CFA*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CFB*/
  0x1AE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CFC*/
  0x1AE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CFD*/
  0x1AE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10CFE*/
  0x1AE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*10CFF*/
};

uint16 uca900_p10E[]= {
  0xFBC2, 0x0020, 0x0002, 0x8E00, 0x0000, 0x0000, 0x0002, /*10E00*/
  0xFBC2, 0x0020, 0x0002, 0x8E01, 0x0000, 0x0000, 0x0002, /*10E01*/
  0xFBC2, 0x0020, 0x0002, 0x8E02, 0x0000, 0x0000, 0x0002, /*10E02*/
  0xFBC2, 0x0020, 0x0002, 0x8E03, 0x0000, 0x0000, 0x0002, /*10E03*/
  0xFBC2, 0x0020, 0x0002, 0x8E04, 0x0000, 0x0000, 0x0002, /*10E04*/
  0xFBC2, 0x0020, 0x0002, 0x8E05, 0x0000, 0x0000, 0x0002, /*10E05*/
  0xFBC2, 0x0020, 0x0002, 0x8E06, 0x0000, 0x0000, 0x0002, /*10E06*/
  0xFBC2, 0x0020, 0x0002, 0x8E07, 0x0000, 0x0000, 0x0002, /*10E07*/
  0xFBC2, 0x0020, 0x0002, 0x8E08, 0x0000, 0x0000, 0x0002, /*10E08*/
  0xFBC2, 0x0020, 0x0002, 0x8E09, 0x0000, 0x0000, 0x0002, /*10E09*/
  0xFBC2, 0x0020, 0x0002, 0x8E0A, 0x0000, 0x0000, 0x0002, /*10E0A*/
  0xFBC2, 0x0020, 0x0002, 0x8E0B, 0x0000, 0x0000, 0x0002, /*10E0B*/
  0xFBC2, 0x0020, 0x0002, 0x8E0C, 0x0000, 0x0000, 0x0002, /*10E0C*/
  0xFBC2, 0x0020, 0x0002, 0x8E0D, 0x0000, 0x0000, 0x0002, /*10E0D*/
  0xFBC2, 0x0020, 0x0002, 0x8E0E, 0x0000, 0x0000, 0x0002, /*10E0E*/
  0xFBC2, 0x0020, 0x0002, 0x8E0F, 0x0000, 0x0000, 0x0002, /*10E0F*/
  0xFBC2, 0x0020, 0x0002, 0x8E10, 0x0000, 0x0000, 0x0002, /*10E10*/
  0xFBC2, 0x0020, 0x0002, 0x8E11, 0x0000, 0x0000, 0x0002, /*10E11*/
  0xFBC2, 0x0020, 0x0002, 0x8E12, 0x0000, 0x0000, 0x0002, /*10E12*/
  0xFBC2, 0x0020, 0x0002, 0x8E13, 0x0000, 0x0000, 0x0002, /*10E13*/
  0xFBC2, 0x0020, 0x0002, 0x8E14, 0x0000, 0x0000, 0x0002, /*10E14*/
  0xFBC2, 0x0020, 0x0002, 0x8E15, 0x0000, 0x0000, 0x0002, /*10E15*/
  0xFBC2, 0x0020, 0x0002, 0x8E16, 0x0000, 0x0000, 0x0002, /*10E16*/
  0xFBC2, 0x0020, 0x0002, 0x8E17, 0x0000, 0x0000, 0x0002, /*10E17*/
  0xFBC2, 0x0020, 0x0002, 0x8E18, 0x0000, 0x0000, 0x0002, /*10E18*/
  0xFBC2, 0x0020, 0x0002, 0x8E19, 0x0000, 0x0000, 0x0002, /*10E19*/
  0xFBC2, 0x0020, 0x0002, 0x8E1A, 0x0000, 0x0000, 0x0002, /*10E1A*/
  0xFBC2, 0x0020, 0x0002, 0x8E1B, 0x0000, 0x0000, 0x0002, /*10E1B*/
  0xFBC2, 0x0020, 0x0002, 0x8E1C, 0x0000, 0x0000, 0x0002, /*10E1C*/
  0xFBC2, 0x0020, 0x0002, 0x8E1D, 0x0000, 0x0000, 0x0002, /*10E1D*/
  0xFBC2, 0x0020, 0x0002, 0x8E1E, 0x0000, 0x0000, 0x0002, /*10E1E*/
  0xFBC2, 0x0020, 0x0002, 0x8E1F, 0x0000, 0x0000, 0x0002, /*10E1F*/
  0xFBC2, 0x0020, 0x0002, 0x8E20, 0x0000, 0x0000, 0x0002, /*10E20*/
  0xFBC2, 0x0020, 0x0002, 0x8E21, 0x0000, 0x0000, 0x0002, /*10E21*/
  0xFBC2, 0x0020, 0x0002, 0x8E22, 0x0000, 0x0000, 0x0002, /*10E22*/
  0xFBC2, 0x0020, 0x0002, 0x8E23, 0x0000, 0x0000, 0x0002, /*10E23*/
  0xFBC2, 0x0020, 0x0002, 0x8E24, 0x0000, 0x0000, 0x0002, /*10E24*/
  0xFBC2, 0x0020, 0x0002, 0x8E25, 0x0000, 0x0000, 0x0002, /*10E25*/
  0xFBC2, 0x0020, 0x0002, 0x8E26, 0x0000, 0x0000, 0x0002, /*10E26*/
  0xFBC2, 0x0020, 0x0002, 0x8E27, 0x0000, 0x0000, 0x0002, /*10E27*/
  0xFBC2, 0x0020, 0x0002, 0x8E28, 0x0000, 0x0000, 0x0002, /*10E28*/
  0xFBC2, 0x0020, 0x0002, 0x8E29, 0x0000, 0x0000, 0x0002, /*10E29*/
  0xFBC2, 0x0020, 0x0002, 0x8E2A, 0x0000, 0x0000, 0x0002, /*10E2A*/
  0xFBC2, 0x0020, 0x0002, 0x8E2B, 0x0000, 0x0000, 0x0002, /*10E2B*/
  0xFBC2, 0x0020, 0x0002, 0x8E2C, 0x0000, 0x0000, 0x0002, /*10E2C*/
  0xFBC2, 0x0020, 0x0002, 0x8E2D, 0x0000, 0x0000, 0x0002, /*10E2D*/
  0xFBC2, 0x0020, 0x0002, 0x8E2E, 0x0000, 0x0000, 0x0002, /*10E2E*/
  0xFBC2, 0x0020, 0x0002, 0x8E2F, 0x0000, 0x0000, 0x0002, /*10E2F*/
  0xFBC2, 0x0020, 0x0002, 0x8E30, 0x0000, 0x0000, 0x0002, /*10E30*/
  0xFBC2, 0x0020, 0x0002, 0x8E31, 0x0000, 0x0000, 0x0002, /*10E31*/
  0xFBC2, 0x0020, 0x0002, 0x8E32, 0x0000, 0x0000, 0x0002, /*10E32*/
  0xFBC2, 0x0020, 0x0002, 0x8E33, 0x0000, 0x0000, 0x0002, /*10E33*/
  0xFBC2, 0x0020, 0x0002, 0x8E34, 0x0000, 0x0000, 0x0002, /*10E34*/
  0xFBC2, 0x0020, 0x0002, 0x8E35, 0x0000, 0x0000, 0x0002, /*10E35*/
  0xFBC2, 0x0020, 0x0002, 0x8E36, 0x0000, 0x0000, 0x0002, /*10E36*/
  0xFBC2, 0x0020, 0x0002, 0x8E37, 0x0000, 0x0000, 0x0002, /*10E37*/
  0xFBC2, 0x0020, 0x0002, 0x8E38, 0x0000, 0x0000, 0x0002, /*10E38*/
  0xFBC2, 0x0020, 0x0002, 0x8E39, 0x0000, 0x0000, 0x0002, /*10E39*/
  0xFBC2, 0x0020, 0x0002, 0x8E3A, 0x0000, 0x0000, 0x0002, /*10E3A*/
  0xFBC2, 0x0020, 0x0002, 0x8E3B, 0x0000, 0x0000, 0x0002, /*10E3B*/
  0xFBC2, 0x0020, 0x0002, 0x8E3C, 0x0000, 0x0000, 0x0002, /*10E3C*/
  0xFBC2, 0x0020, 0x0002, 0x8E3D, 0x0000, 0x0000, 0x0002, /*10E3D*/
  0xFBC2, 0x0020, 0x0002, 0x8E3E, 0x0000, 0x0000, 0x0002, /*10E3E*/
  0xFBC2, 0x0020, 0x0002, 0x8E3F, 0x0000, 0x0000, 0x0002, /*10E3F*/
  0xFBC2, 0x0020, 0x0002, 0x8E40, 0x0000, 0x0000, 0x0002, /*10E40*/
  0xFBC2, 0x0020, 0x0002, 0x8E41, 0x0000, 0x0000, 0x0002, /*10E41*/
  0xFBC2, 0x0020, 0x0002, 0x8E42, 0x0000, 0x0000, 0x0002, /*10E42*/
  0xFBC2, 0x0020, 0x0002, 0x8E43, 0x0000, 0x0000, 0x0002, /*10E43*/
  0xFBC2, 0x0020, 0x0002, 0x8E44, 0x0000, 0x0000, 0x0002, /*10E44*/
  0xFBC2, 0x0020, 0x0002, 0x8E45, 0x0000, 0x0000, 0x0002, /*10E45*/
  0xFBC2, 0x0020, 0x0002, 0x8E46, 0x0000, 0x0000, 0x0002, /*10E46*/
  0xFBC2, 0x0020, 0x0002, 0x8E47, 0x0000, 0x0000, 0x0002, /*10E47*/
  0xFBC2, 0x0020, 0x0002, 0x8E48, 0x0000, 0x0000, 0x0002, /*10E48*/
  0xFBC2, 0x0020, 0x0002, 0x8E49, 0x0000, 0x0000, 0x0002, /*10E49*/
  0xFBC2, 0x0020, 0x0002, 0x8E4A, 0x0000, 0x0000, 0x0002, /*10E4A*/
  0xFBC2, 0x0020, 0x0002, 0x8E4B, 0x0000, 0x0000, 0x0002, /*10E4B*/
  0xFBC2, 0x0020, 0x0002, 0x8E4C, 0x0000, 0x0000, 0x0002, /*10E4C*/
  0xFBC2, 0x0020, 0x0002, 0x8E4D, 0x0000, 0x0000, 0x0002, /*10E4D*/
  0xFBC2, 0x0020, 0x0002, 0x8E4E, 0x0000, 0x0000, 0x0002, /*10E4E*/
  0xFBC2, 0x0020, 0x0002, 0x8E4F, 0x0000, 0x0000, 0x0002, /*10E4F*/
  0xFBC2, 0x0020, 0x0002, 0x8E50, 0x0000, 0x0000, 0x0002, /*10E50*/
  0xFBC2, 0x0020, 0x0002, 0x8E51, 0x0000, 0x0000, 0x0002, /*10E51*/
  0xFBC2, 0x0020, 0x0002, 0x8E52, 0x0000, 0x0000, 0x0002, /*10E52*/
  0xFBC2, 0x0020, 0x0002, 0x8E53, 0x0000, 0x0000, 0x0002, /*10E53*/
  0xFBC2, 0x0020, 0x0002, 0x8E54, 0x0000, 0x0000, 0x0002, /*10E54*/
  0xFBC2, 0x0020, 0x0002, 0x8E55, 0x0000, 0x0000, 0x0002, /*10E55*/
  0xFBC2, 0x0020, 0x0002, 0x8E56, 0x0000, 0x0000, 0x0002, /*10E56*/
  0xFBC2, 0x0020, 0x0002, 0x8E57, 0x0000, 0x0000, 0x0002, /*10E57*/
  0xFBC2, 0x0020, 0x0002, 0x8E58, 0x0000, 0x0000, 0x0002, /*10E58*/
  0xFBC2, 0x0020, 0x0002, 0x8E59, 0x0000, 0x0000, 0x0002, /*10E59*/
  0xFBC2, 0x0020, 0x0002, 0x8E5A, 0x0000, 0x0000, 0x0002, /*10E5A*/
  0xFBC2, 0x0020, 0x0002, 0x8E5B, 0x0000, 0x0000, 0x0002, /*10E5B*/
  0xFBC2, 0x0020, 0x0002, 0x8E5C, 0x0000, 0x0000, 0x0002, /*10E5C*/
  0xFBC2, 0x0020, 0x0002, 0x8E5D, 0x0000, 0x0000, 0x0002, /*10E5D*/
  0xFBC2, 0x0020, 0x0002, 0x8E5E, 0x0000, 0x0000, 0x0002, /*10E5E*/
  0xFBC2, 0x0020, 0x0002, 0x8E5F, 0x0000, 0x0000, 0x0002, /*10E5F*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E60*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E61*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E62*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E63*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E64*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E65*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E66*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E67*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E68*/
  0x1ACC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E69*/
  0x1ACD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E6A*/
  0x1ACE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E6B*/
  0x1ACF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E6C*/
  0x1AD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E6D*/
  0x1AD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E6E*/
  0x1AD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E6F*/
  0x1AD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E70*/
  0x1AD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E71*/
  0x1AD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E72*/
  0x1AD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E73*/
  0x1AD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E74*/
  0x1AD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E75*/
  0x1AD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E76*/
  0x1ADA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E77*/
  0x1ADB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E78*/
  0x1ADC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E79*/
  0x1ADD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E7A*/
  0x1ADE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E7B*/
  0x1ADF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E7C*/
  0x1AE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E7D*/
  0x1AE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*10E7E*/
  0xFBC2, 0x0020, 0x0002, 0x8E7F, 0x0000, 0x0000, 0x0002, /*10E7F*/
  0xFBC2, 0x0020, 0x0002, 0x8E80, 0x0000, 0x0000, 0x0002, /*10E80*/
  0xFBC2, 0x0020, 0x0002, 0x8E81, 0x0000, 0x0000, 0x0002, /*10E81*/
  0xFBC2, 0x0020, 0x0002, 0x8E82, 0x0000, 0x0000, 0x0002, /*10E82*/
  0xFBC2, 0x0020, 0x0002, 0x8E83, 0x0000, 0x0000, 0x0002, /*10E83*/
  0xFBC2, 0x0020, 0x0002, 0x8E84, 0x0000, 0x0000, 0x0002, /*10E84*/
  0xFBC2, 0x0020, 0x0002, 0x8E85, 0x0000, 0x0000, 0x0002, /*10E85*/
  0xFBC2, 0x0020, 0x0002, 0x8E86, 0x0000, 0x0000, 0x0002, /*10E86*/
  0xFBC2, 0x0020, 0x0002, 0x8E87, 0x0000, 0x0000, 0x0002, /*10E87*/
  0xFBC2, 0x0020, 0x0002, 0x8E88, 0x0000, 0x0000, 0x0002, /*10E88*/
  0xFBC2, 0x0020, 0x0002, 0x8E89, 0x0000, 0x0000, 0x0002, /*10E89*/
  0xFBC2, 0x0020, 0x0002, 0x8E8A, 0x0000, 0x0000, 0x0002, /*10E8A*/
  0xFBC2, 0x0020, 0x0002, 0x8E8B, 0x0000, 0x0000, 0x0002, /*10E8B*/
  0xFBC2, 0x0020, 0x0002, 0x8E8C, 0x0000, 0x0000, 0x0002, /*10E8C*/
  0xFBC2, 0x0020, 0x0002, 0x8E8D, 0x0000, 0x0000, 0x0002, /*10E8D*/
  0xFBC2, 0x0020, 0x0002, 0x8E8E, 0x0000, 0x0000, 0x0002, /*10E8E*/
  0xFBC2, 0x0020, 0x0002, 0x8E8F, 0x0000, 0x0000, 0x0002, /*10E8F*/
  0xFBC2, 0x0020, 0x0002, 0x8E90, 0x0000, 0x0000, 0x0002, /*10E90*/
  0xFBC2, 0x0020, 0x0002, 0x8E91, 0x0000, 0x0000, 0x0002, /*10E91*/
  0xFBC2, 0x0020, 0x0002, 0x8E92, 0x0000, 0x0000, 0x0002, /*10E92*/
  0xFBC2, 0x0020, 0x0002, 0x8E93, 0x0000, 0x0000, 0x0002, /*10E93*/
  0xFBC2, 0x0020, 0x0002, 0x8E94, 0x0000, 0x0000, 0x0002, /*10E94*/
  0xFBC2, 0x0020, 0x0002, 0x8E95, 0x0000, 0x0000, 0x0002, /*10E95*/
  0xFBC2, 0x0020, 0x0002, 0x8E96, 0x0000, 0x0000, 0x0002, /*10E96*/
  0xFBC2, 0x0020, 0x0002, 0x8E97, 0x0000, 0x0000, 0x0002, /*10E97*/
  0xFBC2, 0x0020, 0x0002, 0x8E98, 0x0000, 0x0000, 0x0002, /*10E98*/
  0xFBC2, 0x0020, 0x0002, 0x8E99, 0x0000, 0x0000, 0x0002, /*10E99*/
  0xFBC2, 0x0020, 0x0002, 0x8E9A, 0x0000, 0x0000, 0x0002, /*10E9A*/
  0xFBC2, 0x0020, 0x0002, 0x8E9B, 0x0000, 0x0000, 0x0002, /*10E9B*/
  0xFBC2, 0x0020, 0x0002, 0x8E9C, 0x0000, 0x0000, 0x0002, /*10E9C*/
  0xFBC2, 0x0020, 0x0002, 0x8E9D, 0x0000, 0x0000, 0x0002, /*10E9D*/
  0xFBC2, 0x0020, 0x0002, 0x8E9E, 0x0000, 0x0000, 0x0002, /*10E9E*/
  0xFBC2, 0x0020, 0x0002, 0x8E9F, 0x0000, 0x0000, 0x0002, /*10E9F*/
  0xFBC2, 0x0020, 0x0002, 0x8EA0, 0x0000, 0x0000, 0x0002, /*10EA0*/
  0xFBC2, 0x0020, 0x0002, 0x8EA1, 0x0000, 0x0000, 0x0002, /*10EA1*/
  0xFBC2, 0x0020, 0x0002, 0x8EA2, 0x0000, 0x0000, 0x0002, /*10EA2*/
  0xFBC2, 0x0020, 0x0002, 0x8EA3, 0x0000, 0x0000, 0x0002, /*10EA3*/
  0xFBC2, 0x0020, 0x0002, 0x8EA4, 0x0000, 0x0000, 0x0002, /*10EA4*/
  0xFBC2, 0x0020, 0x0002, 0x8EA5, 0x0000, 0x0000, 0x0002, /*10EA5*/
  0xFBC2, 0x0020, 0x0002, 0x8EA6, 0x0000, 0x0000, 0x0002, /*10EA6*/
  0xFBC2, 0x0020, 0x0002, 0x8EA7, 0x0000, 0x0000, 0x0002, /*10EA7*/
  0xFBC2, 0x0020, 0x0002, 0x8EA8, 0x0000, 0x0000, 0x0002, /*10EA8*/
  0xFBC2, 0x0020, 0x0002, 0x8EA9, 0x0000, 0x0000, 0x0002, /*10EA9*/
  0xFBC2, 0x0020, 0x0002, 0x8EAA, 0x0000, 0x0000, 0x0002, /*10EAA*/
  0xFBC2, 0x0020, 0x0002, 0x8EAB, 0x0000, 0x0000, 0x0002, /*10EAB*/
  0xFBC2, 0x0020, 0x0002, 0x8EAC, 0x0000, 0x0000, 0x0002, /*10EAC*/
  0xFBC2, 0x0020, 0x0002, 0x8EAD, 0x0000, 0x0000, 0x0002, /*10EAD*/
  0xFBC2, 0x0020, 0x0002, 0x8EAE, 0x0000, 0x0000, 0x0002, /*10EAE*/
  0xFBC2, 0x0020, 0x0002, 0x8EAF, 0x0000, 0x0000, 0x0002, /*10EAF*/
  0xFBC2, 0x0020, 0x0002, 0x8EB0, 0x0000, 0x0000, 0x0002, /*10EB0*/
  0xFBC2, 0x0020, 0x0002, 0x8EB1, 0x0000, 0x0000, 0x0002, /*10EB1*/
  0xFBC2, 0x0020, 0x0002, 0x8EB2, 0x0000, 0x0000, 0x0002, /*10EB2*/
  0xFBC2, 0x0020, 0x0002, 0x8EB3, 0x0000, 0x0000, 0x0002, /*10EB3*/
  0xFBC2, 0x0020, 0x0002, 0x8EB4, 0x0000, 0x0000, 0x0002, /*10EB4*/
  0xFBC2, 0x0020, 0x0002, 0x8EB5, 0x0000, 0x0000, 0x0002, /*10EB5*/
  0xFBC2, 0x0020, 0x0002, 0x8EB6, 0x0000, 0x0000, 0x0002, /*10EB6*/
  0xFBC2, 0x0020, 0x0002, 0x8EB7, 0x0000, 0x0000, 0x0002, /*10EB7*/
  0xFBC2, 0x0020, 0x0002, 0x8EB8, 0x0000, 0x0000, 0x0002, /*10EB8*/
  0xFBC2, 0x0020, 0x0002, 0x8EB9, 0x0000, 0x0000, 0x0002, /*10EB9*/
  0xFBC2, 0x0020, 0x0002, 0x8EBA, 0x0000, 0x0000, 0x0002, /*10EBA*/
  0xFBC2, 0x0020, 0x0002, 0x8EBB, 0x0000, 0x0000, 0x0002, /*10EBB*/
  0xFBC2, 0x0020, 0x0002, 0x8EBC, 0x0000, 0x0000, 0x0002, /*10EBC*/
  0xFBC2, 0x0020, 0x0002, 0x8EBD, 0x0000, 0x0000, 0x0002, /*10EBD*/
  0xFBC2, 0x0020, 0x0002, 0x8EBE, 0x0000, 0x0000, 0x0002, /*10EBE*/
  0xFBC2, 0x0020, 0x0002, 0x8EBF, 0x0000, 0x0000, 0x0002, /*10EBF*/
  0xFBC2, 0x0020, 0x0002, 0x8EC0, 0x0000, 0x0000, 0x0002, /*10EC0*/
  0xFBC2, 0x0020, 0x0002, 0x8EC1, 0x0000, 0x0000, 0x0002, /*10EC1*/
  0xFBC2, 0x0020, 0x0002, 0x8EC2, 0x0000, 0x0000, 0x0002, /*10EC2*/
  0xFBC2, 0x0020, 0x0002, 0x8EC3, 0x0000, 0x0000, 0x0002, /*10EC3*/
  0xFBC2, 0x0020, 0x0002, 0x8EC4, 0x0000, 0x0000, 0x0002, /*10EC4*/
  0xFBC2, 0x0020, 0x0002, 0x8EC5, 0x0000, 0x0000, 0x0002, /*10EC5*/
  0xFBC2, 0x0020, 0x0002, 0x8EC6, 0x0000, 0x0000, 0x0002, /*10EC6*/
  0xFBC2, 0x0020, 0x0002, 0x8EC7, 0x0000, 0x0000, 0x0002, /*10EC7*/
  0xFBC2, 0x0020, 0x0002, 0x8EC8, 0x0000, 0x0000, 0x0002, /*10EC8*/
  0xFBC2, 0x0020, 0x0002, 0x8EC9, 0x0000, 0x0000, 0x0002, /*10EC9*/
  0xFBC2, 0x0020, 0x0002, 0x8ECA, 0x0000, 0x0000, 0x0002, /*10ECA*/
  0xFBC2, 0x0020, 0x0002, 0x8ECB, 0x0000, 0x0000, 0x0002, /*10ECB*/
  0xFBC2, 0x0020, 0x0002, 0x8ECC, 0x0000, 0x0000, 0x0002, /*10ECC*/
  0xFBC2, 0x0020, 0x0002, 0x8ECD, 0x0000, 0x0000, 0x0002, /*10ECD*/
  0xFBC2, 0x0020, 0x0002, 0x8ECE, 0x0000, 0x0000, 0x0002, /*10ECE*/
  0xFBC2, 0x0020, 0x0002, 0x8ECF, 0x0000, 0x0000, 0x0002, /*10ECF*/
  0xFBC2, 0x0020, 0x0002, 0x8ED0, 0x0000, 0x0000, 0x0002, /*10ED0*/
  0xFBC2, 0x0020, 0x0002, 0x8ED1, 0x0000, 0x0000, 0x0002, /*10ED1*/
  0xFBC2, 0x0020, 0x0002, 0x8ED2, 0x0000, 0x0000, 0x0002, /*10ED2*/
  0xFBC2, 0x0020, 0x0002, 0x8ED3, 0x0000, 0x0000, 0x0002, /*10ED3*/
  0xFBC2, 0x0020, 0x0002, 0x8ED4, 0x0000, 0x0000, 0x0002, /*10ED4*/
  0xFBC2, 0x0020, 0x0002, 0x8ED5, 0x0000, 0x0000, 0x0002, /*10ED5*/
  0xFBC2, 0x0020, 0x0002, 0x8ED6, 0x0000, 0x0000, 0x0002, /*10ED6*/
  0xFBC2, 0x0020, 0x0002, 0x8ED7, 0x0000, 0x0000, 0x0002, /*10ED7*/
  0xFBC2, 0x0020, 0x0002, 0x8ED8, 0x0000, 0x0000, 0x0002, /*10ED8*/
  0xFBC2, 0x0020, 0x0002, 0x8ED9, 0x0000, 0x0000, 0x0002, /*10ED9*/
  0xFBC2, 0x0020, 0x0002, 0x8EDA, 0x0000, 0x0000, 0x0002, /*10EDA*/
  0xFBC2, 0x0020, 0x0002, 0x8EDB, 0x0000, 0x0000, 0x0002, /*10EDB*/
  0xFBC2, 0x0020, 0x0002, 0x8EDC, 0x0000, 0x0000, 0x0002, /*10EDC*/
  0xFBC2, 0x0020, 0x0002, 0x8EDD, 0x0000, 0x0000, 0x0002, /*10EDD*/
  0xFBC2, 0x0020, 0x0002, 0x8EDE, 0x0000, 0x0000, 0x0002, /*10EDE*/
  0xFBC2, 0x0020, 0x0002, 0x8EDF, 0x0000, 0x0000, 0x0002, /*10EDF*/
  0xFBC2, 0x0020, 0x0002, 0x8EE0, 0x0000, 0x0000, 0x0002, /*10EE0*/
  0xFBC2, 0x0020, 0x0002, 0x8EE1, 0x0000, 0x0000, 0x0002, /*10EE1*/
  0xFBC2, 0x0020, 0x0002, 0x8EE2, 0x0000, 0x0000, 0x0002, /*10EE2*/
  0xFBC2, 0x0020, 0x0002, 0x8EE3, 0x0000, 0x0000, 0x0002, /*10EE3*/
  0xFBC2, 0x0020, 0x0002, 0x8EE4, 0x0000, 0x0000, 0x0002, /*10EE4*/
  0xFBC2, 0x0020, 0x0002, 0x8EE5, 0x0000, 0x0000, 0x0002, /*10EE5*/
  0xFBC2, 0x0020, 0x0002, 0x8EE6, 0x0000, 0x0000, 0x0002, /*10EE6*/
  0xFBC2, 0x0020, 0x0002, 0x8EE7, 0x0000, 0x0000, 0x0002, /*10EE7*/
  0xFBC2, 0x0020, 0x0002, 0x8EE8, 0x0000, 0x0000, 0x0002, /*10EE8*/
  0xFBC2, 0x0020, 0x0002, 0x8EE9, 0x0000, 0x0000, 0x0002, /*10EE9*/
  0xFBC2, 0x0020, 0x0002, 0x8EEA, 0x0000, 0x0000, 0x0002, /*10EEA*/
  0xFBC2, 0x0020, 0x0002, 0x8EEB, 0x0000, 0x0000, 0x0002, /*10EEB*/
  0xFBC2, 0x0020, 0x0002, 0x8EEC, 0x0000, 0x0000, 0x0002, /*10EEC*/
  0xFBC2, 0x0020, 0x0002, 0x8EED, 0x0000, 0x0000, 0x0002, /*10EED*/
  0xFBC2, 0x0020, 0x0002, 0x8EEE, 0x0000, 0x0000, 0x0002, /*10EEE*/
  0xFBC2, 0x0020, 0x0002, 0x8EEF, 0x0000, 0x0000, 0x0002, /*10EEF*/
  0xFBC2, 0x0020, 0x0002, 0x8EF0, 0x0000, 0x0000, 0x0002, /*10EF0*/
  0xFBC2, 0x0020, 0x0002, 0x8EF1, 0x0000, 0x0000, 0x0002, /*10EF1*/
  0xFBC2, 0x0020, 0x0002, 0x8EF2, 0x0000, 0x0000, 0x0002, /*10EF2*/
  0xFBC2, 0x0020, 0x0002, 0x8EF3, 0x0000, 0x0000, 0x0002, /*10EF3*/
  0xFBC2, 0x0020, 0x0002, 0x8EF4, 0x0000, 0x0000, 0x0002, /*10EF4*/
  0xFBC2, 0x0020, 0x0002, 0x8EF5, 0x0000, 0x0000, 0x0002, /*10EF5*/
  0xFBC2, 0x0020, 0x0002, 0x8EF6, 0x0000, 0x0000, 0x0002, /*10EF6*/
  0xFBC2, 0x0020, 0x0002, 0x8EF7, 0x0000, 0x0000, 0x0002, /*10EF7*/
  0xFBC2, 0x0020, 0x0002, 0x8EF8, 0x0000, 0x0000, 0x0002, /*10EF8*/
  0xFBC2, 0x0020, 0x0002, 0x8EF9, 0x0000, 0x0000, 0x0002, /*10EF9*/
  0xFBC2, 0x0020, 0x0002, 0x8EFA, 0x0000, 0x0000, 0x0002, /*10EFA*/
  0xFBC2, 0x0020, 0x0002, 0x8EFB, 0x0000, 0x0000, 0x0002, /*10EFB*/
  0xFBC2, 0x0020, 0x0002, 0x8EFC, 0x0000, 0x0000, 0x0002, /*10EFC*/
  0xFBC2, 0x0020, 0x0002, 0x8EFD, 0x0000, 0x0000, 0x0002, /*10EFD*/
  0xFBC2, 0x0020, 0x0002, 0x8EFE, 0x0000, 0x0000, 0x0002, /*10EFE*/
  0xFBC2, 0x0020, 0x0002, 0x8EFF, 0x0000, 0x0000, 0x0002  /*10EFF*/
};

uint16 uca900_p110[]= {
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11000*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11001*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11002*/
  0x2CF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11003*/
  0x2CF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11004*/
  0x2CC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11005*/
  0x2CC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11006*/
  0x2CCA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11007*/
  0x2CCB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11008*/
  0x2CCC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11009*/
  0x2CCD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1100A*/
  0x2CCE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1100B*/
  0x2CCF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1100C*/
  0x2CD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1100D*/
  0x2CD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1100E*/
  0x2CD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1100F*/
  0x2CD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11010*/
  0x2CD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11011*/
  0x2CD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11012*/
  0x2CD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11013*/
  0x2CD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11014*/
  0x2CD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11015*/
  0x2CD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11016*/
  0x2CDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11017*/
  0x2CDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11018*/
  0x2CDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11019*/
  0x2CDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1101A*/
  0x2CDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1101B*/
  0x2CDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1101C*/
  0x2CE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1101D*/
  0x2CE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1101E*/
  0x2CE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1101F*/
  0x2CE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11020*/
  0x2CE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11021*/
  0x2CE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11022*/
  0x2CE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11023*/
  0x2CE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11024*/
  0x2CE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11025*/
  0x2CE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11026*/
  0x2CEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11027*/
  0x2CEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11028*/
  0x2CEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11029*/
  0x2CED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1102A*/
  0x2CEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1102B*/
  0x2CEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1102C*/
  0x2CF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1102D*/
  0x2CF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1102E*/
  0x2CF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1102F*/
  0x2CF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11030*/
  0x2CF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11031*/
  0x2CF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11032*/
  0x2CF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11033*/
  0x2CF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11034*/
  0x2CFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11035*/
  0x2CFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11036*/
  0x2CFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11037*/
  0x2CFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11038*/
  0x2CFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11039*/
  0x2CFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1103A*/
  0x2D00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1103B*/
  0x2D01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1103C*/
  0x2D02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1103D*/
  0x2D03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1103E*/
  0x2D04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1103F*/
  0x2D05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11040*/
  0x2D06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11041*/
  0x2D07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11042*/
  0x2D08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11043*/
  0x2D09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11044*/
  0x2D0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11045*/
  0x2D0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11046*/
  0x02AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11047*/
  0x02AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11048*/
  0x0433, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11049*/
  0x0434, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1104A*/
  0x0435, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1104B*/
  0x0436, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1104C*/
  0x0437, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1104D*/
  0xFBC2, 0x0020, 0x0002, 0x904E, 0x0000, 0x0000, 0x0002, /*1104E*/
  0xFBC2, 0x0020, 0x0002, 0x904F, 0x0000, 0x0000, 0x0002, /*1104F*/
  0xFBC2, 0x0020, 0x0002, 0x9050, 0x0000, 0x0000, 0x0002, /*11050*/
  0xFBC2, 0x0020, 0x0002, 0x9051, 0x0000, 0x0000, 0x0002, /*11051*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11052*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11053*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11054*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11055*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11056*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11057*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11058*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11059*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1105A*/
  0x1B73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1105B*/
  0x1B74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1105C*/
  0x1B75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1105D*/
  0x1B76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1105E*/
  0x1B77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1105F*/
  0x1B78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11060*/
  0x1B79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11061*/
  0x1B7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11062*/
  0x1B7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11063*/
  0x1B7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11064*/
  0x1B7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11065*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11066*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11067*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11068*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11069*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1106A*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1106B*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1106C*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1106D*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1106E*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1106F*/
  0xFBC2, 0x0020, 0x0002, 0x9070, 0x0000, 0x0000, 0x0002, /*11070*/
  0xFBC2, 0x0020, 0x0002, 0x9071, 0x0000, 0x0000, 0x0002, /*11071*/
  0xFBC2, 0x0020, 0x0002, 0x9072, 0x0000, 0x0000, 0x0002, /*11072*/
  0xFBC2, 0x0020, 0x0002, 0x9073, 0x0000, 0x0000, 0x0002, /*11073*/
  0xFBC2, 0x0020, 0x0002, 0x9074, 0x0000, 0x0000, 0x0002, /*11074*/
  0xFBC2, 0x0020, 0x0002, 0x9075, 0x0000, 0x0000, 0x0002, /*11075*/
  0xFBC2, 0x0020, 0x0002, 0x9076, 0x0000, 0x0000, 0x0002, /*11076*/
  0xFBC2, 0x0020, 0x0002, 0x9077, 0x0000, 0x0000, 0x0002, /*11077*/
  0xFBC2, 0x0020, 0x0002, 0x9078, 0x0000, 0x0000, 0x0002, /*11078*/
  0xFBC2, 0x0020, 0x0002, 0x9079, 0x0000, 0x0000, 0x0002, /*11079*/
  0xFBC2, 0x0020, 0x0002, 0x907A, 0x0000, 0x0000, 0x0002, /*1107A*/
  0xFBC2, 0x0020, 0x0002, 0x907B, 0x0000, 0x0000, 0x0002, /*1107B*/
  0xFBC2, 0x0020, 0x0002, 0x907C, 0x0000, 0x0000, 0x0002, /*1107C*/
  0xFBC2, 0x0020, 0x0002, 0x907D, 0x0000, 0x0000, 0x0002, /*1107D*/
  0xFBC2, 0x0020, 0x0002, 0x907E, 0x0000, 0x0000, 0x0002, /*1107E*/
  0x2D0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1107F*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11080*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11081*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11082*/
  0x29C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11083*/
  0x29C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11084*/
  0x29C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11085*/
  0x29C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11086*/
  0x29CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11087*/
  0x29CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11088*/
  0x29CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11089*/
  0x29CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1108A*/
  0x29CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1108B*/
  0x29CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1108C*/
  0x29D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1108D*/
  0x29D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1108E*/
  0x29D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1108F*/
  0x29D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11090*/
  0x29D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11091*/
  0x29D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11092*/
  0x29D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11093*/
  0x29D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11094*/
  0x29D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11095*/
  0x29D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11096*/
  0x29DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11097*/
  0x29DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11098*/
  0x29DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11099*/
  0x29DC, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*1109A*/
  0x29DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1109B*/
  0x29DD, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*1109C*/
  0x29DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1109D*/
  0x29DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1109E*/
  0x29E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1109F*/
  0x29E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110A0*/
  0x29E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110A1*/
  0x29E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110A2*/
  0x29E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110A3*/
  0x29E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110A4*/
  0x29E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110A5*/
  0x29E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110A6*/
  0x29E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110A7*/
  0x29E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110A8*/
  0x29EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110A9*/
  0x29EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110AA*/
  0x29E6, 0x0020, 0x0002, 0x0000, 0x00C2, 0x0002, 0x0002, /*110AB*/
  0x29EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110AC*/
  0x29ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110AD*/
  0x29EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110AE*/
  0x29EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110AF*/
  0x29F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110B0*/
  0x29F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110B1*/
  0x29F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110B2*/
  0x29F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110B3*/
  0x29F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110B4*/
  0x29F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110B5*/
  0x29F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110B6*/
  0x29F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110B7*/
  0x29F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110B8*/
  0x29F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110B9*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110BA*/
  0x044E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110BB*/
  0x044F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*110BD*/
  0x02E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110BE*/
  0x02E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110BF*/
  0x02AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110C0*/
  0x02AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110C1*/
  0xFBC2, 0x0020, 0x0002, 0x90C2, 0x0000, 0x0000, 0x0002, /*110C2*/
  0xFBC2, 0x0020, 0x0002, 0x90C3, 0x0000, 0x0000, 0x0002, /*110C3*/
  0xFBC2, 0x0020, 0x0002, 0x90C4, 0x0000, 0x0000, 0x0002, /*110C4*/
  0xFBC2, 0x0020, 0x0002, 0x90C5, 0x0000, 0x0000, 0x0002, /*110C5*/
  0xFBC2, 0x0020, 0x0002, 0x90C6, 0x0000, 0x0000, 0x0002, /*110C6*/
  0xFBC2, 0x0020, 0x0002, 0x90C7, 0x0000, 0x0000, 0x0002, /*110C7*/
  0xFBC2, 0x0020, 0x0002, 0x90C8, 0x0000, 0x0000, 0x0002, /*110C8*/
  0xFBC2, 0x0020, 0x0002, 0x90C9, 0x0000, 0x0000, 0x0002, /*110C9*/
  0xFBC2, 0x0020, 0x0002, 0x90CA, 0x0000, 0x0000, 0x0002, /*110CA*/
  0xFBC2, 0x0020, 0x0002, 0x90CB, 0x0000, 0x0000, 0x0002, /*110CB*/
  0xFBC2, 0x0020, 0x0002, 0x90CC, 0x0000, 0x0000, 0x0002, /*110CC*/
  0xFBC2, 0x0020, 0x0002, 0x90CD, 0x0000, 0x0000, 0x0002, /*110CD*/
  0xFBC2, 0x0020, 0x0002, 0x90CE, 0x0000, 0x0000, 0x0002, /*110CE*/
  0xFBC2, 0x0020, 0x0002, 0x90CF, 0x0000, 0x0000, 0x0002, /*110CF*/
  0x45AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110D0*/
  0x45B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110D1*/
  0x45B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110D2*/
  0x45B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110D3*/
  0x45B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110D4*/
  0x45B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110D5*/
  0x45B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110D6*/
  0x45B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110D7*/
  0x45B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110D8*/
  0x45B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110D9*/
  0x45B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110DA*/
  0x45BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110DB*/
  0x45BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110DC*/
  0x45BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110DD*/
  0x45BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110DE*/
  0x45BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110DF*/
  0x45BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110E0*/
  0x45C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110E1*/
  0x45C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110E2*/
  0x45C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110E3*/
  0x45C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110E4*/
  0x45C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110E5*/
  0x45C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110E6*/
  0x45C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110E7*/
  0x45C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110E8*/
  0xFBC2, 0x0020, 0x0002, 0x90E9, 0x0000, 0x0000, 0x0002, /*110E9*/
  0xFBC2, 0x0020, 0x0002, 0x90EA, 0x0000, 0x0000, 0x0002, /*110EA*/
  0xFBC2, 0x0020, 0x0002, 0x90EB, 0x0000, 0x0000, 0x0002, /*110EB*/
  0xFBC2, 0x0020, 0x0002, 0x90EC, 0x0000, 0x0000, 0x0002, /*110EC*/
  0xFBC2, 0x0020, 0x0002, 0x90ED, 0x0000, 0x0000, 0x0002, /*110ED*/
  0xFBC2, 0x0020, 0x0002, 0x90EE, 0x0000, 0x0000, 0x0002, /*110EE*/
  0xFBC2, 0x0020, 0x0002, 0x90EF, 0x0000, 0x0000, 0x0002, /*110EF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110F0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110F1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110F2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110F3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110F4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110F5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110F6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110F7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110F8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*110F9*/
  0xFBC2, 0x0020, 0x0002, 0x90FA, 0x0000, 0x0000, 0x0002, /*110FA*/
  0xFBC2, 0x0020, 0x0002, 0x90FB, 0x0000, 0x0000, 0x0002, /*110FB*/
  0xFBC2, 0x0020, 0x0002, 0x90FC, 0x0000, 0x0000, 0x0002, /*110FC*/
  0xFBC2, 0x0020, 0x0002, 0x90FD, 0x0000, 0x0000, 0x0002, /*110FD*/
  0xFBC2, 0x0020, 0x0002, 0x90FE, 0x0000, 0x0000, 0x0002, /*110FE*/
  0xFBC2, 0x0020, 0x0002, 0x90FF, 0x0000, 0x0000, 0x0002  /*110FF*/
};

uint16 uca900_p111[]= {
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11100*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11101*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11102*/
  0x30E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11103*/
  0x30E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11104*/
  0x30E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11105*/
  0x30E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11106*/
  0x30E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11107*/
  0x30E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11108*/
  0x30EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11109*/
  0x30EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1110A*/
  0x30EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1110B*/
  0x30ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1110C*/
  0x30EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1110D*/
  0x30EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1110E*/
  0x30F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1110F*/
  0x30F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11110*/
  0x30F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11111*/
  0x30F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11112*/
  0x30F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11113*/
  0x30F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11114*/
  0x30F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11115*/
  0x30F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11116*/
  0x30F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11117*/
  0x30F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11118*/
  0x30FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11119*/
  0x30FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1111A*/
  0x30FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1111B*/
  0x30FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1111C*/
  0x30FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1111D*/
  0x30FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1111E*/
  0x3100, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1111F*/
  0x3101, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11120*/
  0x3102, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11121*/
  0x3103, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11122*/
  0x3104, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11123*/
  0x3105, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11124*/
  0x3106, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11125*/
  0x3107, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11126*/
  0x3108, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11127*/
  0x3109, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11128*/
  0x310A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11129*/
  0x310B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1112A*/
  0x310C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1112B*/
  0x310D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1112C*/
  0x310E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1112D*/
  0x310F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1112E*/
  0x3110, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1112F*/
  0x3111, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11130*/
  0x3112, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11131*/
  0x3113, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11132*/
  0x3114, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11133*/
  0x3115, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11134*/
  0xFBC2, 0x0020, 0x0002, 0x9135, 0x0000, 0x0000, 0x0002, /*11135*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11136*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11137*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11138*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11139*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1113A*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1113B*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1113C*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1113D*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1113E*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1113F*/
  0x02E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11140*/
  0x02B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11141*/
  0x02B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11142*/
  0x0273, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11143*/
  0xFBC2, 0x0020, 0x0002, 0x9144, 0x0000, 0x0000, 0x0002, /*11144*/
  0xFBC2, 0x0020, 0x0002, 0x9145, 0x0000, 0x0000, 0x0002, /*11145*/
  0xFBC2, 0x0020, 0x0002, 0x9146, 0x0000, 0x0000, 0x0002, /*11146*/
  0xFBC2, 0x0020, 0x0002, 0x9147, 0x0000, 0x0000, 0x0002, /*11147*/
  0xFBC2, 0x0020, 0x0002, 0x9148, 0x0000, 0x0000, 0x0002, /*11148*/
  0xFBC2, 0x0020, 0x0002, 0x9149, 0x0000, 0x0000, 0x0002, /*11149*/
  0xFBC2, 0x0020, 0x0002, 0x914A, 0x0000, 0x0000, 0x0002, /*1114A*/
  0xFBC2, 0x0020, 0x0002, 0x914B, 0x0000, 0x0000, 0x0002, /*1114B*/
  0xFBC2, 0x0020, 0x0002, 0x914C, 0x0000, 0x0000, 0x0002, /*1114C*/
  0xFBC2, 0x0020, 0x0002, 0x914D, 0x0000, 0x0000, 0x0002, /*1114D*/
  0xFBC2, 0x0020, 0x0002, 0x914E, 0x0000, 0x0000, 0x0002, /*1114E*/
  0xFBC2, 0x0020, 0x0002, 0x914F, 0x0000, 0x0000, 0x0002, /*1114F*/
  0x29FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11150*/
  0x29FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11151*/
  0x29FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11152*/
  0x29FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11153*/
  0x29FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11154*/
  0x29FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11155*/
  0x2A00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11156*/
  0x2A01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11157*/
  0x2A02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11158*/
  0x2A03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11159*/
  0x2A04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1115A*/
  0x2A05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1115B*/
  0x2A06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1115C*/
  0x2A07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1115D*/
  0x2A08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1115E*/
  0x2A09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1115F*/
  0x2A0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11160*/
  0x2A0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11161*/
  0x2A0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11162*/
  0x2A0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11163*/
  0x2A0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11164*/
  0x2A0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11165*/
  0x2A10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11166*/
  0x2A11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11167*/
  0x2A12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11168*/
  0x2A13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11169*/
  0x2A14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1116A*/
  0x2A15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1116B*/
  0x2A16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1116C*/
  0x2A17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1116D*/
  0x2A18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1116E*/
  0x2A19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1116F*/
  0x2A1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11170*/
  0x2A1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11171*/
  0x2A1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11172*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11173*/
  0x0450, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11174*/
  0x0451, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11175*/
  0x2A1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11176*/
  0xFBC2, 0x0020, 0x0002, 0x9177, 0x0000, 0x0000, 0x0002, /*11177*/
  0xFBC2, 0x0020, 0x0002, 0x9178, 0x0000, 0x0000, 0x0002, /*11178*/
  0xFBC2, 0x0020, 0x0002, 0x9179, 0x0000, 0x0000, 0x0002, /*11179*/
  0xFBC2, 0x0020, 0x0002, 0x917A, 0x0000, 0x0000, 0x0002, /*1117A*/
  0xFBC2, 0x0020, 0x0002, 0x917B, 0x0000, 0x0000, 0x0002, /*1117B*/
  0xFBC2, 0x0020, 0x0002, 0x917C, 0x0000, 0x0000, 0x0002, /*1117C*/
  0xFBC2, 0x0020, 0x0002, 0x917D, 0x0000, 0x0000, 0x0002, /*1117D*/
  0xFBC2, 0x0020, 0x0002, 0x917E, 0x0000, 0x0000, 0x0002, /*1117E*/
  0xFBC2, 0x0020, 0x0002, 0x917F, 0x0000, 0x0000, 0x0002, /*1117F*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11180*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11181*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11182*/
  0x2A20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11183*/
  0x2A21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11184*/
  0x2A22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11185*/
  0x2A23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11186*/
  0x2A24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11187*/
  0x2A25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11188*/
  0x2A26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11189*/
  0x2A27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1118A*/
  0x2A28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1118B*/
  0x2A29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1118C*/
  0x2A2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1118D*/
  0x2A2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1118E*/
  0x2A2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1118F*/
  0x2A2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11190*/
  0x2A2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11191*/
  0x2A2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11192*/
  0x2A30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11193*/
  0x2A31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11194*/
  0x2A32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11195*/
  0x2A33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11196*/
  0x2A34, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11197*/
  0x2A35, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11198*/
  0x2A36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11199*/
  0x2A37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1119A*/
  0x2A38, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1119B*/
  0x2A39, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1119C*/
  0x2A3A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1119D*/
  0x2A3B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1119E*/
  0x2A3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1119F*/
  0x2A3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111A0*/
  0x2A3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111A1*/
  0x2A3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111A2*/
  0x2A40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111A3*/
  0x2A41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111A4*/
  0x2A42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111A5*/
  0x2A43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111A6*/
  0x2A44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111A7*/
  0x2A45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111A8*/
  0x2A46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111A9*/
  0x2A47, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111AA*/
  0x2A48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111AB*/
  0x2A49, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111AC*/
  0x2A4A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111AD*/
  0x2A4B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111AE*/
  0x2A4C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111AF*/
  0x2A4D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111B0*/
  0x2A4E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111B1*/
  0x2A4F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111B2*/
  0x2A54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111B3*/
  0x2A55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111B4*/
  0x2A56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111B5*/
  0x2A57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111B6*/
  0x2A58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111B7*/
  0x2A59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111B8*/
  0x2A5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111B9*/
  0x2A5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111BA*/
  0x2A5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111BB*/
  0x2A5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111BC*/
  0x2A5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111BD*/
  0x2A5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111BE*/
  0x2A60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111BF*/
  0x2A61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111C0*/
  0x2A50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111C1*/
  0x2A51, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111C2*/
  0x2A52, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111C3*/
  0x2A1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111C4*/
  0x02B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111C5*/
  0x02B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111C6*/
  0x0453, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111C7*/
  0x02E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111C8*/
  0x0454, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111C9*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111CA*/
  0x0000, 0x00CF, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111CB*/
  0x0000, 0x00D0, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111CC*/
  0x0452, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111CD*/
  0xFBC2, 0x0020, 0x0002, 0x91CE, 0x0000, 0x0000, 0x0002, /*111CE*/
  0xFBC2, 0x0020, 0x0002, 0x91CF, 0x0000, 0x0000, 0x0002, /*111CF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111D0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111D1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111D2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111D3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111D4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111D5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111D6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111D7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111D8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111D9*/
  0x2A1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111DA*/
  0x0455, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111DB*/
  0x2A53, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111DC*/
  0x0456, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111DD*/
  0x02E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111DE*/
  0x02E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111DF*/
  0xFBC2, 0x0020, 0x0002, 0x91E0, 0x0000, 0x0000, 0x0002, /*111E0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111E1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111E2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111E3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111E4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111E5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111E6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111E7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111E8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111E9*/
  0x1B8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111EA*/
  0x1B8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111EB*/
  0x1B8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111EC*/
  0x1B8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111ED*/
  0x1B90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111EE*/
  0x1B91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111EF*/
  0x1B92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111F0*/
  0x1B93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111F1*/
  0x1B94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111F2*/
  0x1B95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111F3*/
  0x1B96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*111F4*/
  0xFBC2, 0x0020, 0x0002, 0x91F5, 0x0000, 0x0000, 0x0002, /*111F5*/
  0xFBC2, 0x0020, 0x0002, 0x91F6, 0x0000, 0x0000, 0x0002, /*111F6*/
  0xFBC2, 0x0020, 0x0002, 0x91F7, 0x0000, 0x0000, 0x0002, /*111F7*/
  0xFBC2, 0x0020, 0x0002, 0x91F8, 0x0000, 0x0000, 0x0002, /*111F8*/
  0xFBC2, 0x0020, 0x0002, 0x91F9, 0x0000, 0x0000, 0x0002, /*111F9*/
  0xFBC2, 0x0020, 0x0002, 0x91FA, 0x0000, 0x0000, 0x0002, /*111FA*/
  0xFBC2, 0x0020, 0x0002, 0x91FB, 0x0000, 0x0000, 0x0002, /*111FB*/
  0xFBC2, 0x0020, 0x0002, 0x91FC, 0x0000, 0x0000, 0x0002, /*111FC*/
  0xFBC2, 0x0020, 0x0002, 0x91FD, 0x0000, 0x0000, 0x0002, /*111FD*/
  0xFBC2, 0x0020, 0x0002, 0x91FE, 0x0000, 0x0000, 0x0002, /*111FE*/
  0xFBC2, 0x0020, 0x0002, 0x91FF, 0x0000, 0x0000, 0x0002  /*111FF*/
};

uint16 uca900_p112[]= {
  0x2A62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11200*/
  0x2A63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11201*/
  0x2A64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11202*/
  0x2A65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11203*/
  0x2A66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11204*/
  0x2A67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11205*/
  0x2A68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11206*/
  0x2A69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11207*/
  0x2A6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11208*/
  0x2A6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11209*/
  0x2A6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1120A*/
  0x2A6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1120B*/
  0x2A6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1120C*/
  0x2A6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1120D*/
  0x2A70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1120E*/
  0x2A71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1120F*/
  0x2A72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11210*/
  0x2A73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11211*/
  0xFBC2, 0x0020, 0x0002, 0x9212, 0x0000, 0x0000, 0x0002, /*11212*/
  0x2A74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11213*/
  0x2A75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11214*/
  0x2A76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11215*/
  0x2A77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11216*/
  0x2A78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11217*/
  0x2A79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11218*/
  0x2A7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11219*/
  0x2A7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1121A*/
  0x2A7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1121B*/
  0x2A7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1121C*/
  0x2A7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1121D*/
  0x2A7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1121E*/
  0x2A80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1121F*/
  0x2A81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11220*/
  0x2A82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11221*/
  0x2A83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11222*/
  0x2A84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11223*/
  0x2A85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11224*/
  0x2A86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11225*/
  0x2A87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11226*/
  0x2A88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11227*/
  0x2A89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11228*/
  0x2A8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11229*/
  0x2A8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1122A*/
  0x2A8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1122B*/
  0x2A8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1122C*/
  0x2A8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1122D*/
  0x2A8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1122E*/
  0x2A90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1122F*/
  0x2A91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11230*/
  0x2A92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11231*/
  0x2A93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11232*/
  0x2A94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11233*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11234*/
  0x2A95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11235*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11236*/
  0x0000, 0x0080, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11237*/
  0x02B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11238*/
  0x02B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11239*/
  0x0457, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1123A*/
  0x0458, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1123B*/
  0x0459, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1123C*/
  0x045A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1123D*/
  0x0000, 0x0081, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1123E*/
  0xFBC2, 0x0020, 0x0002, 0x923F, 0x0000, 0x0000, 0x0002, /*1123F*/
  0xFBC2, 0x0020, 0x0002, 0x9240, 0x0000, 0x0000, 0x0002, /*11240*/
  0xFBC2, 0x0020, 0x0002, 0x9241, 0x0000, 0x0000, 0x0002, /*11241*/
  0xFBC2, 0x0020, 0x0002, 0x9242, 0x0000, 0x0000, 0x0002, /*11242*/
  0xFBC2, 0x0020, 0x0002, 0x9243, 0x0000, 0x0000, 0x0002, /*11243*/
  0xFBC2, 0x0020, 0x0002, 0x9244, 0x0000, 0x0000, 0x0002, /*11244*/
  0xFBC2, 0x0020, 0x0002, 0x9245, 0x0000, 0x0000, 0x0002, /*11245*/
  0xFBC2, 0x0020, 0x0002, 0x9246, 0x0000, 0x0000, 0x0002, /*11246*/
  0xFBC2, 0x0020, 0x0002, 0x9247, 0x0000, 0x0000, 0x0002, /*11247*/
  0xFBC2, 0x0020, 0x0002, 0x9248, 0x0000, 0x0000, 0x0002, /*11248*/
  0xFBC2, 0x0020, 0x0002, 0x9249, 0x0000, 0x0000, 0x0002, /*11249*/
  0xFBC2, 0x0020, 0x0002, 0x924A, 0x0000, 0x0000, 0x0002, /*1124A*/
  0xFBC2, 0x0020, 0x0002, 0x924B, 0x0000, 0x0000, 0x0002, /*1124B*/
  0xFBC2, 0x0020, 0x0002, 0x924C, 0x0000, 0x0000, 0x0002, /*1124C*/
  0xFBC2, 0x0020, 0x0002, 0x924D, 0x0000, 0x0000, 0x0002, /*1124D*/
  0xFBC2, 0x0020, 0x0002, 0x924E, 0x0000, 0x0000, 0x0002, /*1124E*/
  0xFBC2, 0x0020, 0x0002, 0x924F, 0x0000, 0x0000, 0x0002, /*1124F*/
  0xFBC2, 0x0020, 0x0002, 0x9250, 0x0000, 0x0000, 0x0002, /*11250*/
  0xFBC2, 0x0020, 0x0002, 0x9251, 0x0000, 0x0000, 0x0002, /*11251*/
  0xFBC2, 0x0020, 0x0002, 0x9252, 0x0000, 0x0000, 0x0002, /*11252*/
  0xFBC2, 0x0020, 0x0002, 0x9253, 0x0000, 0x0000, 0x0002, /*11253*/
  0xFBC2, 0x0020, 0x0002, 0x9254, 0x0000, 0x0000, 0x0002, /*11254*/
  0xFBC2, 0x0020, 0x0002, 0x9255, 0x0000, 0x0000, 0x0002, /*11255*/
  0xFBC2, 0x0020, 0x0002, 0x9256, 0x0000, 0x0000, 0x0002, /*11256*/
  0xFBC2, 0x0020, 0x0002, 0x9257, 0x0000, 0x0000, 0x0002, /*11257*/
  0xFBC2, 0x0020, 0x0002, 0x9258, 0x0000, 0x0000, 0x0002, /*11258*/
  0xFBC2, 0x0020, 0x0002, 0x9259, 0x0000, 0x0000, 0x0002, /*11259*/
  0xFBC2, 0x0020, 0x0002, 0x925A, 0x0000, 0x0000, 0x0002, /*1125A*/
  0xFBC2, 0x0020, 0x0002, 0x925B, 0x0000, 0x0000, 0x0002, /*1125B*/
  0xFBC2, 0x0020, 0x0002, 0x925C, 0x0000, 0x0000, 0x0002, /*1125C*/
  0xFBC2, 0x0020, 0x0002, 0x925D, 0x0000, 0x0000, 0x0002, /*1125D*/
  0xFBC2, 0x0020, 0x0002, 0x925E, 0x0000, 0x0000, 0x0002, /*1125E*/
  0xFBC2, 0x0020, 0x0002, 0x925F, 0x0000, 0x0000, 0x0002, /*1125F*/
  0xFBC2, 0x0020, 0x0002, 0x9260, 0x0000, 0x0000, 0x0002, /*11260*/
  0xFBC2, 0x0020, 0x0002, 0x9261, 0x0000, 0x0000, 0x0002, /*11261*/
  0xFBC2, 0x0020, 0x0002, 0x9262, 0x0000, 0x0000, 0x0002, /*11262*/
  0xFBC2, 0x0020, 0x0002, 0x9263, 0x0000, 0x0000, 0x0002, /*11263*/
  0xFBC2, 0x0020, 0x0002, 0x9264, 0x0000, 0x0000, 0x0002, /*11264*/
  0xFBC2, 0x0020, 0x0002, 0x9265, 0x0000, 0x0000, 0x0002, /*11265*/
  0xFBC2, 0x0020, 0x0002, 0x9266, 0x0000, 0x0000, 0x0002, /*11266*/
  0xFBC2, 0x0020, 0x0002, 0x9267, 0x0000, 0x0000, 0x0002, /*11267*/
  0xFBC2, 0x0020, 0x0002, 0x9268, 0x0000, 0x0000, 0x0002, /*11268*/
  0xFBC2, 0x0020, 0x0002, 0x9269, 0x0000, 0x0000, 0x0002, /*11269*/
  0xFBC2, 0x0020, 0x0002, 0x926A, 0x0000, 0x0000, 0x0002, /*1126A*/
  0xFBC2, 0x0020, 0x0002, 0x926B, 0x0000, 0x0000, 0x0002, /*1126B*/
  0xFBC2, 0x0020, 0x0002, 0x926C, 0x0000, 0x0000, 0x0002, /*1126C*/
  0xFBC2, 0x0020, 0x0002, 0x926D, 0x0000, 0x0000, 0x0002, /*1126D*/
  0xFBC2, 0x0020, 0x0002, 0x926E, 0x0000, 0x0000, 0x0002, /*1126E*/
  0xFBC2, 0x0020, 0x0002, 0x926F, 0x0000, 0x0000, 0x0002, /*1126F*/
  0xFBC2, 0x0020, 0x0002, 0x9270, 0x0000, 0x0000, 0x0002, /*11270*/
  0xFBC2, 0x0020, 0x0002, 0x9271, 0x0000, 0x0000, 0x0002, /*11271*/
  0xFBC2, 0x0020, 0x0002, 0x9272, 0x0000, 0x0000, 0x0002, /*11272*/
  0xFBC2, 0x0020, 0x0002, 0x9273, 0x0000, 0x0000, 0x0002, /*11273*/
  0xFBC2, 0x0020, 0x0002, 0x9274, 0x0000, 0x0000, 0x0002, /*11274*/
  0xFBC2, 0x0020, 0x0002, 0x9275, 0x0000, 0x0000, 0x0002, /*11275*/
  0xFBC2, 0x0020, 0x0002, 0x9276, 0x0000, 0x0000, 0x0002, /*11276*/
  0xFBC2, 0x0020, 0x0002, 0x9277, 0x0000, 0x0000, 0x0002, /*11277*/
  0xFBC2, 0x0020, 0x0002, 0x9278, 0x0000, 0x0000, 0x0002, /*11278*/
  0xFBC2, 0x0020, 0x0002, 0x9279, 0x0000, 0x0000, 0x0002, /*11279*/
  0xFBC2, 0x0020, 0x0002, 0x927A, 0x0000, 0x0000, 0x0002, /*1127A*/
  0xFBC2, 0x0020, 0x0002, 0x927B, 0x0000, 0x0000, 0x0002, /*1127B*/
  0xFBC2, 0x0020, 0x0002, 0x927C, 0x0000, 0x0000, 0x0002, /*1127C*/
  0xFBC2, 0x0020, 0x0002, 0x927D, 0x0000, 0x0000, 0x0002, /*1127D*/
  0xFBC2, 0x0020, 0x0002, 0x927E, 0x0000, 0x0000, 0x0002, /*1127E*/
  0xFBC2, 0x0020, 0x0002, 0x927F, 0x0000, 0x0000, 0x0002, /*1127F*/
  0x2ACF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11280*/
  0x2AD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11281*/
  0x2AD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11282*/
  0x2AD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11283*/
  0x2AD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11284*/
  0x2AD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11285*/
  0x2AD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11286*/
  0xFBC2, 0x0020, 0x0002, 0x9287, 0x0000, 0x0000, 0x0002, /*11287*/
  0x2AD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11288*/
  0xFBC2, 0x0020, 0x0002, 0x9289, 0x0000, 0x0000, 0x0002, /*11289*/
  0x2AD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1128A*/
  0x2ADA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1128B*/
  0x2ADB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1128C*/
  0x2ADC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1128D*/
  0xFBC2, 0x0020, 0x0002, 0x928E, 0x0000, 0x0000, 0x0002, /*1128E*/
  0x2ADD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1128F*/
  0x2ADE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11290*/
  0x2ADF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11291*/
  0x2AE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11292*/
  0x2AE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11293*/
  0x2AE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11294*/
  0x2AE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11295*/
  0x2AE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11296*/
  0x2AE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11297*/
  0x2AE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11298*/
  0x2AE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11299*/
  0x2AE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1129A*/
  0x2AE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1129B*/
  0x2AEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1129C*/
  0x2AEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1129D*/
  0xFBC2, 0x0020, 0x0002, 0x929E, 0x0000, 0x0000, 0x0002, /*1129E*/
  0x2AEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1129F*/
  0x2AED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112A0*/
  0x2AEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112A1*/
  0x2AEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112A2*/
  0x2AF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112A3*/
  0x2AF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112A4*/
  0x2AD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112A5*/
  0x2AD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112A6*/
  0x2AF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112A7*/
  0x2AF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112A8*/
  0x02E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112A9*/
  0xFBC2, 0x0020, 0x0002, 0x92AA, 0x0000, 0x0000, 0x0002, /*112AA*/
  0xFBC2, 0x0020, 0x0002, 0x92AB, 0x0000, 0x0000, 0x0002, /*112AB*/
  0xFBC2, 0x0020, 0x0002, 0x92AC, 0x0000, 0x0000, 0x0002, /*112AC*/
  0xFBC2, 0x0020, 0x0002, 0x92AD, 0x0000, 0x0000, 0x0002, /*112AD*/
  0xFBC2, 0x0020, 0x0002, 0x92AE, 0x0000, 0x0000, 0x0002, /*112AE*/
  0xFBC2, 0x0020, 0x0002, 0x92AF, 0x0000, 0x0000, 0x0002, /*112AF*/
  0x2A96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112B0*/
  0x2A97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112B1*/
  0x2A98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112B2*/
  0x2A99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112B3*/
  0x2A9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112B4*/
  0x2A9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112B5*/
  0x2A9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112B6*/
  0x2A9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112B7*/
  0x2A9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112B8*/
  0x2A9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112B9*/
  0x2AA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112BA*/
  0x2AA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112BB*/
  0x2AA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112BC*/
  0x2AA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112BD*/
  0x2AA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112BE*/
  0x2AA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112BF*/
  0x2AA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112C0*/
  0x2AA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112C1*/
  0x2AA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112C2*/
  0x2AA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112C3*/
  0x2AAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112C4*/
  0x2AAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112C5*/
  0x2AAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112C6*/
  0x2AAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112C7*/
  0x2AAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112C8*/
  0x2AAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112C9*/
  0x2AB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112CA*/
  0x2AB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112CB*/
  0x2AB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112CC*/
  0x2AB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112CD*/
  0x2AB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112CE*/
  0x2AB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112CF*/
  0x2AB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112D0*/
  0x2AB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112D1*/
  0x2AB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112D2*/
  0x2AB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112D3*/
  0x2ABA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112D4*/
  0x2ABB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112D5*/
  0x2ABC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112D6*/
  0x2ABD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112D7*/
  0x2ABE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112D8*/
  0x2ABF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112D9*/
  0x2AC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112DA*/
  0x2AC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112DB*/
  0x2AC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112DC*/
  0x2AC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112DD*/
  0x2AC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112DE*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112DF*/
  0x2AC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112E0*/
  0x2AC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112E1*/
  0x2AC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112E2*/
  0x2AC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112E3*/
  0x2AC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112E4*/
  0x2ACA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112E5*/
  0x2ACB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112E6*/
  0x2ACC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112E7*/
  0x2ACD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112E8*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112E9*/
  0x2ACE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112EA*/
  0xFBC2, 0x0020, 0x0002, 0x92EB, 0x0000, 0x0000, 0x0002, /*112EB*/
  0xFBC2, 0x0020, 0x0002, 0x92EC, 0x0000, 0x0000, 0x0002, /*112EC*/
  0xFBC2, 0x0020, 0x0002, 0x92ED, 0x0000, 0x0000, 0x0002, /*112ED*/
  0xFBC2, 0x0020, 0x0002, 0x92EE, 0x0000, 0x0000, 0x0002, /*112EE*/
  0xFBC2, 0x0020, 0x0002, 0x92EF, 0x0000, 0x0000, 0x0002, /*112EF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112F0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112F1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112F2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112F3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112F4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112F5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112F6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112F7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112F8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*112F9*/
  0xFBC2, 0x0020, 0x0002, 0x92FA, 0x0000, 0x0000, 0x0002, /*112FA*/
  0xFBC2, 0x0020, 0x0002, 0x92FB, 0x0000, 0x0000, 0x0002, /*112FB*/
  0xFBC2, 0x0020, 0x0002, 0x92FC, 0x0000, 0x0000, 0x0002, /*112FC*/
  0xFBC2, 0x0020, 0x0002, 0x92FD, 0x0000, 0x0000, 0x0002, /*112FD*/
  0xFBC2, 0x0020, 0x0002, 0x92FE, 0x0000, 0x0000, 0x0002, /*112FE*/
  0xFBC2, 0x0020, 0x0002, 0x92FF, 0x0000, 0x0000, 0x0002  /*112FF*/
};

uint16 uca900_p113[]= {
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11300*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11301*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11302*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11303*/
  0xFBC2, 0x0020, 0x0002, 0x9304, 0x0000, 0x0000, 0x0002, /*11304*/
  0x2AF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11305*/
  0x2AF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11306*/
  0x2AF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11307*/
  0x2AF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11308*/
  0x2AF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11309*/
  0x2AFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1130A*/
  0x2AFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1130B*/
  0x2AFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1130C*/
  0xFBC2, 0x0020, 0x0002, 0x930D, 0x0000, 0x0000, 0x0002, /*1130D*/
  0xFBC2, 0x0020, 0x0002, 0x930E, 0x0000, 0x0000, 0x0002, /*1130E*/
  0x2AFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1130F*/
  0x2B00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11310*/
  0xFBC2, 0x0020, 0x0002, 0x9311, 0x0000, 0x0000, 0x0002, /*11311*/
  0xFBC2, 0x0020, 0x0002, 0x9312, 0x0000, 0x0000, 0x0002, /*11312*/
  0x2B01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11313*/
  0x2B02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11314*/
  0x2B03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11315*/
  0x2B04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11316*/
  0x2B05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11317*/
  0x2B06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11318*/
  0x2B07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11319*/
  0x2B08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1131A*/
  0x2B09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1131B*/
  0x2B0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1131C*/
  0x2B0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1131D*/
  0x2B0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1131E*/
  0x2B0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1131F*/
  0x2B0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11320*/
  0x2B0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11321*/
  0x2B10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11322*/
  0x2B11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11323*/
  0x2B12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11324*/
  0x2B13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11325*/
  0x2B14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11326*/
  0x2B15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11327*/
  0x2B16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11328*/
  0xFBC2, 0x0020, 0x0002, 0x9329, 0x0000, 0x0000, 0x0002, /*11329*/
  0x2B17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1132A*/
  0x2B18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1132B*/
  0x2B19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1132C*/
  0x2B1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1132D*/
  0x2B1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1132E*/
  0x2B1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1132F*/
  0x2B1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11330*/
  0xFBC2, 0x0020, 0x0002, 0x9331, 0x0000, 0x0000, 0x0002, /*11331*/
  0x2B1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11332*/
  0x2B1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11333*/
  0xFBC2, 0x0020, 0x0002, 0x9334, 0x0000, 0x0000, 0x0002, /*11334*/
  0x2B20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11335*/
  0x2B21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11336*/
  0x2B22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11337*/
  0x2B23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11338*/
  0x2B24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11339*/
  0xFBC2, 0x0020, 0x0002, 0x933A, 0x0000, 0x0000, 0x0002, /*1133A*/
  0xFBC2, 0x0020, 0x0002, 0x933B, 0x0000, 0x0000, 0x0002, /*1133B*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1133C*/
  0x2B25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1133D*/
  0x2B28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1133E*/
  0x2B29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1133F*/
  0x2B2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11340*/
  0x2B2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11341*/
  0x2B2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11342*/
  0x2B2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11343*/
  0x2B2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11344*/
  0xFBC2, 0x0020, 0x0002, 0x9345, 0x0000, 0x0000, 0x0002, /*11345*/
  0xFBC2, 0x0020, 0x0002, 0x9346, 0x0000, 0x0000, 0x0002, /*11346*/
  0x2B31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11347*/
  0x2B32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11348*/
  0xFBC2, 0x0020, 0x0002, 0x9349, 0x0000, 0x0000, 0x0002, /*11349*/
  0xFBC2, 0x0020, 0x0002, 0x934A, 0x0000, 0x0000, 0x0002, /*1134A*/
  0x2B33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1134B*/
  0x2B34, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1134C*/
  0x2B35, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1134D*/
  0xFBC2, 0x0020, 0x0002, 0x934E, 0x0000, 0x0000, 0x0002, /*1134E*/
  0xFBC2, 0x0020, 0x0002, 0x934F, 0x0000, 0x0000, 0x0002, /*1134F*/
  0x2AF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11350*/
  0xFBC2, 0x0020, 0x0002, 0x9351, 0x0000, 0x0000, 0x0002, /*11351*/
  0xFBC2, 0x0020, 0x0002, 0x9352, 0x0000, 0x0000, 0x0002, /*11352*/
  0xFBC2, 0x0020, 0x0002, 0x9353, 0x0000, 0x0000, 0x0002, /*11353*/
  0xFBC2, 0x0020, 0x0002, 0x9354, 0x0000, 0x0000, 0x0002, /*11354*/
  0xFBC2, 0x0020, 0x0002, 0x9355, 0x0000, 0x0000, 0x0002, /*11355*/
  0xFBC2, 0x0020, 0x0002, 0x9356, 0x0000, 0x0000, 0x0002, /*11356*/
  0x2B36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11357*/
  0xFBC2, 0x0020, 0x0002, 0x9358, 0x0000, 0x0000, 0x0002, /*11358*/
  0xFBC2, 0x0020, 0x0002, 0x9359, 0x0000, 0x0000, 0x0002, /*11359*/
  0xFBC2, 0x0020, 0x0002, 0x935A, 0x0000, 0x0000, 0x0002, /*1135A*/
  0xFBC2, 0x0020, 0x0002, 0x935B, 0x0000, 0x0000, 0x0002, /*1135B*/
  0xFBC2, 0x0020, 0x0002, 0x935C, 0x0000, 0x0000, 0x0002, /*1135C*/
  0x2B37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1135D*/
  0x2B26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1135E*/
  0x2B27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1135F*/
  0x2AFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11360*/
  0x2AFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11361*/
  0x2B2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11362*/
  0x2B30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11363*/
  0xFBC2, 0x0020, 0x0002, 0x9364, 0x0000, 0x0000, 0x0002, /*11364*/
  0xFBC2, 0x0020, 0x0002, 0x9365, 0x0000, 0x0000, 0x0002, /*11365*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*11366*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*11367*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*11368*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*11369*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1136A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1136B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1136C*/
  0xFBC2, 0x0020, 0x0002, 0x936D, 0x0000, 0x0000, 0x0002, /*1136D*/
  0xFBC2, 0x0020, 0x0002, 0x936E, 0x0000, 0x0000, 0x0002, /*1136E*/
  0xFBC2, 0x0020, 0x0002, 0x936F, 0x0000, 0x0000, 0x0002, /*1136F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*11370*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*11371*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*11372*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*11373*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*11374*/
  0xFBC2, 0x0020, 0x0002, 0x9375, 0x0000, 0x0000, 0x0002, /*11375*/
  0xFBC2, 0x0020, 0x0002, 0x9376, 0x0000, 0x0000, 0x0002, /*11376*/
  0xFBC2, 0x0020, 0x0002, 0x9377, 0x0000, 0x0000, 0x0002, /*11377*/
  0xFBC2, 0x0020, 0x0002, 0x9378, 0x0000, 0x0000, 0x0002, /*11378*/
  0xFBC2, 0x0020, 0x0002, 0x9379, 0x0000, 0x0000, 0x0002, /*11379*/
  0xFBC2, 0x0020, 0x0002, 0x937A, 0x0000, 0x0000, 0x0002, /*1137A*/
  0xFBC2, 0x0020, 0x0002, 0x937B, 0x0000, 0x0000, 0x0002, /*1137B*/
  0xFBC2, 0x0020, 0x0002, 0x937C, 0x0000, 0x0000, 0x0002, /*1137C*/
  0xFBC2, 0x0020, 0x0002, 0x937D, 0x0000, 0x0000, 0x0002, /*1137D*/
  0xFBC2, 0x0020, 0x0002, 0x937E, 0x0000, 0x0000, 0x0002, /*1137E*/
  0xFBC2, 0x0020, 0x0002, 0x937F, 0x0000, 0x0000, 0x0002, /*1137F*/
  0xFBC2, 0x0020, 0x0002, 0x9380, 0x0000, 0x0000, 0x0002, /*11380*/
  0xFBC2, 0x0020, 0x0002, 0x9381, 0x0000, 0x0000, 0x0002, /*11381*/
  0xFBC2, 0x0020, 0x0002, 0x9382, 0x0000, 0x0000, 0x0002, /*11382*/
  0xFBC2, 0x0020, 0x0002, 0x9383, 0x0000, 0x0000, 0x0002, /*11383*/
  0xFBC2, 0x0020, 0x0002, 0x9384, 0x0000, 0x0000, 0x0002, /*11384*/
  0xFBC2, 0x0020, 0x0002, 0x9385, 0x0000, 0x0000, 0x0002, /*11385*/
  0xFBC2, 0x0020, 0x0002, 0x9386, 0x0000, 0x0000, 0x0002, /*11386*/
  0xFBC2, 0x0020, 0x0002, 0x9387, 0x0000, 0x0000, 0x0002, /*11387*/
  0xFBC2, 0x0020, 0x0002, 0x9388, 0x0000, 0x0000, 0x0002, /*11388*/
  0xFBC2, 0x0020, 0x0002, 0x9389, 0x0000, 0x0000, 0x0002, /*11389*/
  0xFBC2, 0x0020, 0x0002, 0x938A, 0x0000, 0x0000, 0x0002, /*1138A*/
  0xFBC2, 0x0020, 0x0002, 0x938B, 0x0000, 0x0000, 0x0002, /*1138B*/
  0xFBC2, 0x0020, 0x0002, 0x938C, 0x0000, 0x0000, 0x0002, /*1138C*/
  0xFBC2, 0x0020, 0x0002, 0x938D, 0x0000, 0x0000, 0x0002, /*1138D*/
  0xFBC2, 0x0020, 0x0002, 0x938E, 0x0000, 0x0000, 0x0002, /*1138E*/
  0xFBC2, 0x0020, 0x0002, 0x938F, 0x0000, 0x0000, 0x0002, /*1138F*/
  0xFBC2, 0x0020, 0x0002, 0x9390, 0x0000, 0x0000, 0x0002, /*11390*/
  0xFBC2, 0x0020, 0x0002, 0x9391, 0x0000, 0x0000, 0x0002, /*11391*/
  0xFBC2, 0x0020, 0x0002, 0x9392, 0x0000, 0x0000, 0x0002, /*11392*/
  0xFBC2, 0x0020, 0x0002, 0x9393, 0x0000, 0x0000, 0x0002, /*11393*/
  0xFBC2, 0x0020, 0x0002, 0x9394, 0x0000, 0x0000, 0x0002, /*11394*/
  0xFBC2, 0x0020, 0x0002, 0x9395, 0x0000, 0x0000, 0x0002, /*11395*/
  0xFBC2, 0x0020, 0x0002, 0x9396, 0x0000, 0x0000, 0x0002, /*11396*/
  0xFBC2, 0x0020, 0x0002, 0x9397, 0x0000, 0x0000, 0x0002, /*11397*/
  0xFBC2, 0x0020, 0x0002, 0x9398, 0x0000, 0x0000, 0x0002, /*11398*/
  0xFBC2, 0x0020, 0x0002, 0x9399, 0x0000, 0x0000, 0x0002, /*11399*/
  0xFBC2, 0x0020, 0x0002, 0x939A, 0x0000, 0x0000, 0x0002, /*1139A*/
  0xFBC2, 0x0020, 0x0002, 0x939B, 0x0000, 0x0000, 0x0002, /*1139B*/
  0xFBC2, 0x0020, 0x0002, 0x939C, 0x0000, 0x0000, 0x0002, /*1139C*/
  0xFBC2, 0x0020, 0x0002, 0x939D, 0x0000, 0x0000, 0x0002, /*1139D*/
  0xFBC2, 0x0020, 0x0002, 0x939E, 0x0000, 0x0000, 0x0002, /*1139E*/
  0xFBC2, 0x0020, 0x0002, 0x939F, 0x0000, 0x0000, 0x0002, /*1139F*/
  0xFBC2, 0x0020, 0x0002, 0x93A0, 0x0000, 0x0000, 0x0002, /*113A0*/
  0xFBC2, 0x0020, 0x0002, 0x93A1, 0x0000, 0x0000, 0x0002, /*113A1*/
  0xFBC2, 0x0020, 0x0002, 0x93A2, 0x0000, 0x0000, 0x0002, /*113A2*/
  0xFBC2, 0x0020, 0x0002, 0x93A3, 0x0000, 0x0000, 0x0002, /*113A3*/
  0xFBC2, 0x0020, 0x0002, 0x93A4, 0x0000, 0x0000, 0x0002, /*113A4*/
  0xFBC2, 0x0020, 0x0002, 0x93A5, 0x0000, 0x0000, 0x0002, /*113A5*/
  0xFBC2, 0x0020, 0x0002, 0x93A6, 0x0000, 0x0000, 0x0002, /*113A6*/
  0xFBC2, 0x0020, 0x0002, 0x93A7, 0x0000, 0x0000, 0x0002, /*113A7*/
  0xFBC2, 0x0020, 0x0002, 0x93A8, 0x0000, 0x0000, 0x0002, /*113A8*/
  0xFBC2, 0x0020, 0x0002, 0x93A9, 0x0000, 0x0000, 0x0002, /*113A9*/
  0xFBC2, 0x0020, 0x0002, 0x93AA, 0x0000, 0x0000, 0x0002, /*113AA*/
  0xFBC2, 0x0020, 0x0002, 0x93AB, 0x0000, 0x0000, 0x0002, /*113AB*/
  0xFBC2, 0x0020, 0x0002, 0x93AC, 0x0000, 0x0000, 0x0002, /*113AC*/
  0xFBC2, 0x0020, 0x0002, 0x93AD, 0x0000, 0x0000, 0x0002, /*113AD*/
  0xFBC2, 0x0020, 0x0002, 0x93AE, 0x0000, 0x0000, 0x0002, /*113AE*/
  0xFBC2, 0x0020, 0x0002, 0x93AF, 0x0000, 0x0000, 0x0002, /*113AF*/
  0xFBC2, 0x0020, 0x0002, 0x93B0, 0x0000, 0x0000, 0x0002, /*113B0*/
  0xFBC2, 0x0020, 0x0002, 0x93B1, 0x0000, 0x0000, 0x0002, /*113B1*/
  0xFBC2, 0x0020, 0x0002, 0x93B2, 0x0000, 0x0000, 0x0002, /*113B2*/
  0xFBC2, 0x0020, 0x0002, 0x93B3, 0x0000, 0x0000, 0x0002, /*113B3*/
  0xFBC2, 0x0020, 0x0002, 0x93B4, 0x0000, 0x0000, 0x0002, /*113B4*/
  0xFBC2, 0x0020, 0x0002, 0x93B5, 0x0000, 0x0000, 0x0002, /*113B5*/
  0xFBC2, 0x0020, 0x0002, 0x93B6, 0x0000, 0x0000, 0x0002, /*113B6*/
  0xFBC2, 0x0020, 0x0002, 0x93B7, 0x0000, 0x0000, 0x0002, /*113B7*/
  0xFBC2, 0x0020, 0x0002, 0x93B8, 0x0000, 0x0000, 0x0002, /*113B8*/
  0xFBC2, 0x0020, 0x0002, 0x93B9, 0x0000, 0x0000, 0x0002, /*113B9*/
  0xFBC2, 0x0020, 0x0002, 0x93BA, 0x0000, 0x0000, 0x0002, /*113BA*/
  0xFBC2, 0x0020, 0x0002, 0x93BB, 0x0000, 0x0000, 0x0002, /*113BB*/
  0xFBC2, 0x0020, 0x0002, 0x93BC, 0x0000, 0x0000, 0x0002, /*113BC*/
  0xFBC2, 0x0020, 0x0002, 0x93BD, 0x0000, 0x0000, 0x0002, /*113BD*/
  0xFBC2, 0x0020, 0x0002, 0x93BE, 0x0000, 0x0000, 0x0002, /*113BE*/
  0xFBC2, 0x0020, 0x0002, 0x93BF, 0x0000, 0x0000, 0x0002, /*113BF*/
  0xFBC2, 0x0020, 0x0002, 0x93C0, 0x0000, 0x0000, 0x0002, /*113C0*/
  0xFBC2, 0x0020, 0x0002, 0x93C1, 0x0000, 0x0000, 0x0002, /*113C1*/
  0xFBC2, 0x0020, 0x0002, 0x93C2, 0x0000, 0x0000, 0x0002, /*113C2*/
  0xFBC2, 0x0020, 0x0002, 0x93C3, 0x0000, 0x0000, 0x0002, /*113C3*/
  0xFBC2, 0x0020, 0x0002, 0x93C4, 0x0000, 0x0000, 0x0002, /*113C4*/
  0xFBC2, 0x0020, 0x0002, 0x93C5, 0x0000, 0x0000, 0x0002, /*113C5*/
  0xFBC2, 0x0020, 0x0002, 0x93C6, 0x0000, 0x0000, 0x0002, /*113C6*/
  0xFBC2, 0x0020, 0x0002, 0x93C7, 0x0000, 0x0000, 0x0002, /*113C7*/
  0xFBC2, 0x0020, 0x0002, 0x93C8, 0x0000, 0x0000, 0x0002, /*113C8*/
  0xFBC2, 0x0020, 0x0002, 0x93C9, 0x0000, 0x0000, 0x0002, /*113C9*/
  0xFBC2, 0x0020, 0x0002, 0x93CA, 0x0000, 0x0000, 0x0002, /*113CA*/
  0xFBC2, 0x0020, 0x0002, 0x93CB, 0x0000, 0x0000, 0x0002, /*113CB*/
  0xFBC2, 0x0020, 0x0002, 0x93CC, 0x0000, 0x0000, 0x0002, /*113CC*/
  0xFBC2, 0x0020, 0x0002, 0x93CD, 0x0000, 0x0000, 0x0002, /*113CD*/
  0xFBC2, 0x0020, 0x0002, 0x93CE, 0x0000, 0x0000, 0x0002, /*113CE*/
  0xFBC2, 0x0020, 0x0002, 0x93CF, 0x0000, 0x0000, 0x0002, /*113CF*/
  0xFBC2, 0x0020, 0x0002, 0x93D0, 0x0000, 0x0000, 0x0002, /*113D0*/
  0xFBC2, 0x0020, 0x0002, 0x93D1, 0x0000, 0x0000, 0x0002, /*113D1*/
  0xFBC2, 0x0020, 0x0002, 0x93D2, 0x0000, 0x0000, 0x0002, /*113D2*/
  0xFBC2, 0x0020, 0x0002, 0x93D3, 0x0000, 0x0000, 0x0002, /*113D3*/
  0xFBC2, 0x0020, 0x0002, 0x93D4, 0x0000, 0x0000, 0x0002, /*113D4*/
  0xFBC2, 0x0020, 0x0002, 0x93D5, 0x0000, 0x0000, 0x0002, /*113D5*/
  0xFBC2, 0x0020, 0x0002, 0x93D6, 0x0000, 0x0000, 0x0002, /*113D6*/
  0xFBC2, 0x0020, 0x0002, 0x93D7, 0x0000, 0x0000, 0x0002, /*113D7*/
  0xFBC2, 0x0020, 0x0002, 0x93D8, 0x0000, 0x0000, 0x0002, /*113D8*/
  0xFBC2, 0x0020, 0x0002, 0x93D9, 0x0000, 0x0000, 0x0002, /*113D9*/
  0xFBC2, 0x0020, 0x0002, 0x93DA, 0x0000, 0x0000, 0x0002, /*113DA*/
  0xFBC2, 0x0020, 0x0002, 0x93DB, 0x0000, 0x0000, 0x0002, /*113DB*/
  0xFBC2, 0x0020, 0x0002, 0x93DC, 0x0000, 0x0000, 0x0002, /*113DC*/
  0xFBC2, 0x0020, 0x0002, 0x93DD, 0x0000, 0x0000, 0x0002, /*113DD*/
  0xFBC2, 0x0020, 0x0002, 0x93DE, 0x0000, 0x0000, 0x0002, /*113DE*/
  0xFBC2, 0x0020, 0x0002, 0x93DF, 0x0000, 0x0000, 0x0002, /*113DF*/
  0xFBC2, 0x0020, 0x0002, 0x93E0, 0x0000, 0x0000, 0x0002, /*113E0*/
  0xFBC2, 0x0020, 0x0002, 0x93E1, 0x0000, 0x0000, 0x0002, /*113E1*/
  0xFBC2, 0x0020, 0x0002, 0x93E2, 0x0000, 0x0000, 0x0002, /*113E2*/
  0xFBC2, 0x0020, 0x0002, 0x93E3, 0x0000, 0x0000, 0x0002, /*113E3*/
  0xFBC2, 0x0020, 0x0002, 0x93E4, 0x0000, 0x0000, 0x0002, /*113E4*/
  0xFBC2, 0x0020, 0x0002, 0x93E5, 0x0000, 0x0000, 0x0002, /*113E5*/
  0xFBC2, 0x0020, 0x0002, 0x93E6, 0x0000, 0x0000, 0x0002, /*113E6*/
  0xFBC2, 0x0020, 0x0002, 0x93E7, 0x0000, 0x0000, 0x0002, /*113E7*/
  0xFBC2, 0x0020, 0x0002, 0x93E8, 0x0000, 0x0000, 0x0002, /*113E8*/
  0xFBC2, 0x0020, 0x0002, 0x93E9, 0x0000, 0x0000, 0x0002, /*113E9*/
  0xFBC2, 0x0020, 0x0002, 0x93EA, 0x0000, 0x0000, 0x0002, /*113EA*/
  0xFBC2, 0x0020, 0x0002, 0x93EB, 0x0000, 0x0000, 0x0002, /*113EB*/
  0xFBC2, 0x0020, 0x0002, 0x93EC, 0x0000, 0x0000, 0x0002, /*113EC*/
  0xFBC2, 0x0020, 0x0002, 0x93ED, 0x0000, 0x0000, 0x0002, /*113ED*/
  0xFBC2, 0x0020, 0x0002, 0x93EE, 0x0000, 0x0000, 0x0002, /*113EE*/
  0xFBC2, 0x0020, 0x0002, 0x93EF, 0x0000, 0x0000, 0x0002, /*113EF*/
  0xFBC2, 0x0020, 0x0002, 0x93F0, 0x0000, 0x0000, 0x0002, /*113F0*/
  0xFBC2, 0x0020, 0x0002, 0x93F1, 0x0000, 0x0000, 0x0002, /*113F1*/
  0xFBC2, 0x0020, 0x0002, 0x93F2, 0x0000, 0x0000, 0x0002, /*113F2*/
  0xFBC2, 0x0020, 0x0002, 0x93F3, 0x0000, 0x0000, 0x0002, /*113F3*/
  0xFBC2, 0x0020, 0x0002, 0x93F4, 0x0000, 0x0000, 0x0002, /*113F4*/
  0xFBC2, 0x0020, 0x0002, 0x93F5, 0x0000, 0x0000, 0x0002, /*113F5*/
  0xFBC2, 0x0020, 0x0002, 0x93F6, 0x0000, 0x0000, 0x0002, /*113F6*/
  0xFBC2, 0x0020, 0x0002, 0x93F7, 0x0000, 0x0000, 0x0002, /*113F7*/
  0xFBC2, 0x0020, 0x0002, 0x93F8, 0x0000, 0x0000, 0x0002, /*113F8*/
  0xFBC2, 0x0020, 0x0002, 0x93F9, 0x0000, 0x0000, 0x0002, /*113F9*/
  0xFBC2, 0x0020, 0x0002, 0x93FA, 0x0000, 0x0000, 0x0002, /*113FA*/
  0xFBC2, 0x0020, 0x0002, 0x93FB, 0x0000, 0x0000, 0x0002, /*113FB*/
  0xFBC2, 0x0020, 0x0002, 0x93FC, 0x0000, 0x0000, 0x0002, /*113FC*/
  0xFBC2, 0x0020, 0x0002, 0x93FD, 0x0000, 0x0000, 0x0002, /*113FD*/
  0xFBC2, 0x0020, 0x0002, 0x93FE, 0x0000, 0x0000, 0x0002, /*113FE*/
  0xFBC2, 0x0020, 0x0002, 0x93FF, 0x0000, 0x0000, 0x0002  /*113FF*/
};

uint16 uca900_p114[]= {
  0x2B3A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11400*/
  0x2B3B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11401*/
  0x2B3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11402*/
  0x2B3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11403*/
  0x2B3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11404*/
  0x2B3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11405*/
  0x2B40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11406*/
  0x2B41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11407*/
  0x2B42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11408*/
  0x2B43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11409*/
  0x2B44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1140A*/
  0x2B45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1140B*/
  0x2B46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1140C*/
  0x2B47, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1140D*/
  0x2B48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1140E*/
  0x2B49, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1140F*/
  0x2B4A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11410*/
  0x2B4B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11411*/
  0x2B4C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11412*/
  0x2B4D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11413*/
  0x2B4E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11414*/
  0x2B4F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11415*/
  0x2B50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11416*/
  0x2B51, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11417*/
  0x2B52, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11418*/
  0x2B53, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11419*/
  0x2B54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1141A*/
  0x2B55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1141B*/
  0x2B56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1141C*/
  0x2B57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1141D*/
  0x2B58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1141E*/
  0x2B59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1141F*/
  0x2B5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11420*/
  0x2B5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11421*/
  0x2B5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11422*/
  0x2B5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11423*/
  0x2B5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11424*/
  0x2B5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11425*/
  0x2B60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11426*/
  0x2B61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11427*/
  0x2B62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11428*/
  0x2B63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11429*/
  0x2B64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1142A*/
  0x2B65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1142B*/
  0x2B66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1142C*/
  0x2B67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1142D*/
  0x2B68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1142E*/
  0x2B69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1142F*/
  0x2B6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11430*/
  0x2B6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11431*/
  0x2B6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11432*/
  0x2B6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11433*/
  0x2B6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11434*/
  0x2B71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11435*/
  0x2B72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11436*/
  0x2B73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11437*/
  0x2B74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11438*/
  0x2B75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11439*/
  0x2B76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1143A*/
  0x2B77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1143B*/
  0x2B78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1143C*/
  0x2B79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1143D*/
  0x2B7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1143E*/
  0x2B7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1143F*/
  0x2B7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11440*/
  0x2B7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11441*/
  0x2B7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11442*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11443*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11444*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11445*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11446*/
  0x2B6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11447*/
  0x2B70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11448*/
  0x2B38, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11449*/
  0x2B39, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1144A*/
  0x02B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1144B*/
  0x02B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1144C*/
  0x045B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1144D*/
  0x045C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1144E*/
  0x045D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1144F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11450*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11451*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11452*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11453*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11454*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11455*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11456*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11457*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11458*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11459*/
  0xFBC2, 0x0020, 0x0002, 0x945A, 0x0000, 0x0000, 0x0002, /*1145A*/
  0x045E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1145B*/
  0xFBC2, 0x0020, 0x0002, 0x945C, 0x0000, 0x0000, 0x0002, /*1145C*/
  0x045F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1145D*/
  0xFBC2, 0x0020, 0x0002, 0x945E, 0x0000, 0x0000, 0x0002, /*1145E*/
  0xFBC2, 0x0020, 0x0002, 0x945F, 0x0000, 0x0000, 0x0002, /*1145F*/
  0xFBC2, 0x0020, 0x0002, 0x9460, 0x0000, 0x0000, 0x0002, /*11460*/
  0xFBC2, 0x0020, 0x0002, 0x9461, 0x0000, 0x0000, 0x0002, /*11461*/
  0xFBC2, 0x0020, 0x0002, 0x9462, 0x0000, 0x0000, 0x0002, /*11462*/
  0xFBC2, 0x0020, 0x0002, 0x9463, 0x0000, 0x0000, 0x0002, /*11463*/
  0xFBC2, 0x0020, 0x0002, 0x9464, 0x0000, 0x0000, 0x0002, /*11464*/
  0xFBC2, 0x0020, 0x0002, 0x9465, 0x0000, 0x0000, 0x0002, /*11465*/
  0xFBC2, 0x0020, 0x0002, 0x9466, 0x0000, 0x0000, 0x0002, /*11466*/
  0xFBC2, 0x0020, 0x0002, 0x9467, 0x0000, 0x0000, 0x0002, /*11467*/
  0xFBC2, 0x0020, 0x0002, 0x9468, 0x0000, 0x0000, 0x0002, /*11468*/
  0xFBC2, 0x0020, 0x0002, 0x9469, 0x0000, 0x0000, 0x0002, /*11469*/
  0xFBC2, 0x0020, 0x0002, 0x946A, 0x0000, 0x0000, 0x0002, /*1146A*/
  0xFBC2, 0x0020, 0x0002, 0x946B, 0x0000, 0x0000, 0x0002, /*1146B*/
  0xFBC2, 0x0020, 0x0002, 0x946C, 0x0000, 0x0000, 0x0002, /*1146C*/
  0xFBC2, 0x0020, 0x0002, 0x946D, 0x0000, 0x0000, 0x0002, /*1146D*/
  0xFBC2, 0x0020, 0x0002, 0x946E, 0x0000, 0x0000, 0x0002, /*1146E*/
  0xFBC2, 0x0020, 0x0002, 0x946F, 0x0000, 0x0000, 0x0002, /*1146F*/
  0xFBC2, 0x0020, 0x0002, 0x9470, 0x0000, 0x0000, 0x0002, /*11470*/
  0xFBC2, 0x0020, 0x0002, 0x9471, 0x0000, 0x0000, 0x0002, /*11471*/
  0xFBC2, 0x0020, 0x0002, 0x9472, 0x0000, 0x0000, 0x0002, /*11472*/
  0xFBC2, 0x0020, 0x0002, 0x9473, 0x0000, 0x0000, 0x0002, /*11473*/
  0xFBC2, 0x0020, 0x0002, 0x9474, 0x0000, 0x0000, 0x0002, /*11474*/
  0xFBC2, 0x0020, 0x0002, 0x9475, 0x0000, 0x0000, 0x0002, /*11475*/
  0xFBC2, 0x0020, 0x0002, 0x9476, 0x0000, 0x0000, 0x0002, /*11476*/
  0xFBC2, 0x0020, 0x0002, 0x9477, 0x0000, 0x0000, 0x0002, /*11477*/
  0xFBC2, 0x0020, 0x0002, 0x9478, 0x0000, 0x0000, 0x0002, /*11478*/
  0xFBC2, 0x0020, 0x0002, 0x9479, 0x0000, 0x0000, 0x0002, /*11479*/
  0xFBC2, 0x0020, 0x0002, 0x947A, 0x0000, 0x0000, 0x0002, /*1147A*/
  0xFBC2, 0x0020, 0x0002, 0x947B, 0x0000, 0x0000, 0x0002, /*1147B*/
  0xFBC2, 0x0020, 0x0002, 0x947C, 0x0000, 0x0000, 0x0002, /*1147C*/
  0xFBC2, 0x0020, 0x0002, 0x947D, 0x0000, 0x0000, 0x0002, /*1147D*/
  0xFBC2, 0x0020, 0x0002, 0x947E, 0x0000, 0x0000, 0x0002, /*1147E*/
  0xFBC2, 0x0020, 0x0002, 0x947F, 0x0000, 0x0000, 0x0002, /*1147F*/
  0x2B80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11480*/
  0x2B81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11481*/
  0x2B82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11482*/
  0x2B83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11483*/
  0x2B84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11484*/
  0x2B85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11485*/
  0x2B86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11486*/
  0x2B87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11487*/
  0x2B88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11488*/
  0x2B89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11489*/
  0x2B8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1148A*/
  0x2B8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1148B*/
  0x2B8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1148C*/
  0x2B8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1148D*/
  0x2B8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1148E*/
  0x2B8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1148F*/
  0x2B90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11490*/
  0x2B91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11491*/
  0x2B92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11492*/
  0x2B93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11493*/
  0x2B94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11494*/
  0x2B95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11495*/
  0x2B96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11496*/
  0x2B97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11497*/
  0x2B98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11498*/
  0x2B99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11499*/
  0x2B9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1149A*/
  0x2B9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1149B*/
  0x2B9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1149C*/
  0x2B9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1149D*/
  0x2B9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1149E*/
  0x2B9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1149F*/
  0x2BA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114A0*/
  0x2BA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114A1*/
  0x2BA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114A2*/
  0x2BA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114A3*/
  0x2BA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114A4*/
  0x2BA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114A5*/
  0x2BA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114A6*/
  0x2BA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114A7*/
  0x2BA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114A8*/
  0x2BA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114A9*/
  0x2BAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114AA*/
  0x2BAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114AB*/
  0x2BAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114AC*/
  0x2BAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114AD*/
  0x2BAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114AE*/
  0x2BAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114AF*/
  0x2BB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114B0*/
  0x2BB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114B1*/
  0x2BB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114B2*/
  0x2BB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114B3*/
  0x2BB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114B4*/
  0x2BB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114B5*/
  0x2BB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114B6*/
  0x2BB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114B7*/
  0x2BBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114B8*/
  0x2BBB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114B9*/
  0x2BBC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114BA*/
  0x2BBD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114BB*/
  0x2BBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114BC*/
  0x2BBF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114BD*/
  0x2BC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114BE*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114BF*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114C0*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114C1*/
  0x2BC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114C2*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114C3*/
  0x2BB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114C4*/
  0x2BB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114C5*/
  0x0460, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114C6*/
  0x2B7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114C7*/
  0xFBC2, 0x0020, 0x0002, 0x94C8, 0x0000, 0x0000, 0x0002, /*114C8*/
  0xFBC2, 0x0020, 0x0002, 0x94C9, 0x0000, 0x0000, 0x0002, /*114C9*/
  0xFBC2, 0x0020, 0x0002, 0x94CA, 0x0000, 0x0000, 0x0002, /*114CA*/
  0xFBC2, 0x0020, 0x0002, 0x94CB, 0x0000, 0x0000, 0x0002, /*114CB*/
  0xFBC2, 0x0020, 0x0002, 0x94CC, 0x0000, 0x0000, 0x0002, /*114CC*/
  0xFBC2, 0x0020, 0x0002, 0x94CD, 0x0000, 0x0000, 0x0002, /*114CD*/
  0xFBC2, 0x0020, 0x0002, 0x94CE, 0x0000, 0x0000, 0x0002, /*114CE*/
  0xFBC2, 0x0020, 0x0002, 0x94CF, 0x0000, 0x0000, 0x0002, /*114CF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114D0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114D1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114D2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114D3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114D4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114D5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114D6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114D7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114D8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*114D9*/
  0xFBC2, 0x0020, 0x0002, 0x94DA, 0x0000, 0x0000, 0x0002, /*114DA*/
  0xFBC2, 0x0020, 0x0002, 0x94DB, 0x0000, 0x0000, 0x0002, /*114DB*/
  0xFBC2, 0x0020, 0x0002, 0x94DC, 0x0000, 0x0000, 0x0002, /*114DC*/
  0xFBC2, 0x0020, 0x0002, 0x94DD, 0x0000, 0x0000, 0x0002, /*114DD*/
  0xFBC2, 0x0020, 0x0002, 0x94DE, 0x0000, 0x0000, 0x0002, /*114DE*/
  0xFBC2, 0x0020, 0x0002, 0x94DF, 0x0000, 0x0000, 0x0002, /*114DF*/
  0xFBC2, 0x0020, 0x0002, 0x94E0, 0x0000, 0x0000, 0x0002, /*114E0*/
  0xFBC2, 0x0020, 0x0002, 0x94E1, 0x0000, 0x0000, 0x0002, /*114E1*/
  0xFBC2, 0x0020, 0x0002, 0x94E2, 0x0000, 0x0000, 0x0002, /*114E2*/
  0xFBC2, 0x0020, 0x0002, 0x94E3, 0x0000, 0x0000, 0x0002, /*114E3*/
  0xFBC2, 0x0020, 0x0002, 0x94E4, 0x0000, 0x0000, 0x0002, /*114E4*/
  0xFBC2, 0x0020, 0x0002, 0x94E5, 0x0000, 0x0000, 0x0002, /*114E5*/
  0xFBC2, 0x0020, 0x0002, 0x94E6, 0x0000, 0x0000, 0x0002, /*114E6*/
  0xFBC2, 0x0020, 0x0002, 0x94E7, 0x0000, 0x0000, 0x0002, /*114E7*/
  0xFBC2, 0x0020, 0x0002, 0x94E8, 0x0000, 0x0000, 0x0002, /*114E8*/
  0xFBC2, 0x0020, 0x0002, 0x94E9, 0x0000, 0x0000, 0x0002, /*114E9*/
  0xFBC2, 0x0020, 0x0002, 0x94EA, 0x0000, 0x0000, 0x0002, /*114EA*/
  0xFBC2, 0x0020, 0x0002, 0x94EB, 0x0000, 0x0000, 0x0002, /*114EB*/
  0xFBC2, 0x0020, 0x0002, 0x94EC, 0x0000, 0x0000, 0x0002, /*114EC*/
  0xFBC2, 0x0020, 0x0002, 0x94ED, 0x0000, 0x0000, 0x0002, /*114ED*/
  0xFBC2, 0x0020, 0x0002, 0x94EE, 0x0000, 0x0000, 0x0002, /*114EE*/
  0xFBC2, 0x0020, 0x0002, 0x94EF, 0x0000, 0x0000, 0x0002, /*114EF*/
  0xFBC2, 0x0020, 0x0002, 0x94F0, 0x0000, 0x0000, 0x0002, /*114F0*/
  0xFBC2, 0x0020, 0x0002, 0x94F1, 0x0000, 0x0000, 0x0002, /*114F1*/
  0xFBC2, 0x0020, 0x0002, 0x94F2, 0x0000, 0x0000, 0x0002, /*114F2*/
  0xFBC2, 0x0020, 0x0002, 0x94F3, 0x0000, 0x0000, 0x0002, /*114F3*/
  0xFBC2, 0x0020, 0x0002, 0x94F4, 0x0000, 0x0000, 0x0002, /*114F4*/
  0xFBC2, 0x0020, 0x0002, 0x94F5, 0x0000, 0x0000, 0x0002, /*114F5*/
  0xFBC2, 0x0020, 0x0002, 0x94F6, 0x0000, 0x0000, 0x0002, /*114F6*/
  0xFBC2, 0x0020, 0x0002, 0x94F7, 0x0000, 0x0000, 0x0002, /*114F7*/
  0xFBC2, 0x0020, 0x0002, 0x94F8, 0x0000, 0x0000, 0x0002, /*114F8*/
  0xFBC2, 0x0020, 0x0002, 0x94F9, 0x0000, 0x0000, 0x0002, /*114F9*/
  0xFBC2, 0x0020, 0x0002, 0x94FA, 0x0000, 0x0000, 0x0002, /*114FA*/
  0xFBC2, 0x0020, 0x0002, 0x94FB, 0x0000, 0x0000, 0x0002, /*114FB*/
  0xFBC2, 0x0020, 0x0002, 0x94FC, 0x0000, 0x0000, 0x0002, /*114FC*/
  0xFBC2, 0x0020, 0x0002, 0x94FD, 0x0000, 0x0000, 0x0002, /*114FD*/
  0xFBC2, 0x0020, 0x0002, 0x94FE, 0x0000, 0x0000, 0x0002, /*114FE*/
  0xFBC2, 0x0020, 0x0002, 0x94FF, 0x0000, 0x0000, 0x0002  /*114FF*/
};

uint16 uca900_p115[]= {
  0xFBC2, 0x0020, 0x0002, 0x9500, 0x0000, 0x0000, 0x0002, /*11500*/
  0xFBC2, 0x0020, 0x0002, 0x9501, 0x0000, 0x0000, 0x0002, /*11501*/
  0xFBC2, 0x0020, 0x0002, 0x9502, 0x0000, 0x0000, 0x0002, /*11502*/
  0xFBC2, 0x0020, 0x0002, 0x9503, 0x0000, 0x0000, 0x0002, /*11503*/
  0xFBC2, 0x0020, 0x0002, 0x9504, 0x0000, 0x0000, 0x0002, /*11504*/
  0xFBC2, 0x0020, 0x0002, 0x9505, 0x0000, 0x0000, 0x0002, /*11505*/
  0xFBC2, 0x0020, 0x0002, 0x9506, 0x0000, 0x0000, 0x0002, /*11506*/
  0xFBC2, 0x0020, 0x0002, 0x9507, 0x0000, 0x0000, 0x0002, /*11507*/
  0xFBC2, 0x0020, 0x0002, 0x9508, 0x0000, 0x0000, 0x0002, /*11508*/
  0xFBC2, 0x0020, 0x0002, 0x9509, 0x0000, 0x0000, 0x0002, /*11509*/
  0xFBC2, 0x0020, 0x0002, 0x950A, 0x0000, 0x0000, 0x0002, /*1150A*/
  0xFBC2, 0x0020, 0x0002, 0x950B, 0x0000, 0x0000, 0x0002, /*1150B*/
  0xFBC2, 0x0020, 0x0002, 0x950C, 0x0000, 0x0000, 0x0002, /*1150C*/
  0xFBC2, 0x0020, 0x0002, 0x950D, 0x0000, 0x0000, 0x0002, /*1150D*/
  0xFBC2, 0x0020, 0x0002, 0x950E, 0x0000, 0x0000, 0x0002, /*1150E*/
  0xFBC2, 0x0020, 0x0002, 0x950F, 0x0000, 0x0000, 0x0002, /*1150F*/
  0xFBC2, 0x0020, 0x0002, 0x9510, 0x0000, 0x0000, 0x0002, /*11510*/
  0xFBC2, 0x0020, 0x0002, 0x9511, 0x0000, 0x0000, 0x0002, /*11511*/
  0xFBC2, 0x0020, 0x0002, 0x9512, 0x0000, 0x0000, 0x0002, /*11512*/
  0xFBC2, 0x0020, 0x0002, 0x9513, 0x0000, 0x0000, 0x0002, /*11513*/
  0xFBC2, 0x0020, 0x0002, 0x9514, 0x0000, 0x0000, 0x0002, /*11514*/
  0xFBC2, 0x0020, 0x0002, 0x9515, 0x0000, 0x0000, 0x0002, /*11515*/
  0xFBC2, 0x0020, 0x0002, 0x9516, 0x0000, 0x0000, 0x0002, /*11516*/
  0xFBC2, 0x0020, 0x0002, 0x9517, 0x0000, 0x0000, 0x0002, /*11517*/
  0xFBC2, 0x0020, 0x0002, 0x9518, 0x0000, 0x0000, 0x0002, /*11518*/
  0xFBC2, 0x0020, 0x0002, 0x9519, 0x0000, 0x0000, 0x0002, /*11519*/
  0xFBC2, 0x0020, 0x0002, 0x951A, 0x0000, 0x0000, 0x0002, /*1151A*/
  0xFBC2, 0x0020, 0x0002, 0x951B, 0x0000, 0x0000, 0x0002, /*1151B*/
  0xFBC2, 0x0020, 0x0002, 0x951C, 0x0000, 0x0000, 0x0002, /*1151C*/
  0xFBC2, 0x0020, 0x0002, 0x951D, 0x0000, 0x0000, 0x0002, /*1151D*/
  0xFBC2, 0x0020, 0x0002, 0x951E, 0x0000, 0x0000, 0x0002, /*1151E*/
  0xFBC2, 0x0020, 0x0002, 0x951F, 0x0000, 0x0000, 0x0002, /*1151F*/
  0xFBC2, 0x0020, 0x0002, 0x9520, 0x0000, 0x0000, 0x0002, /*11520*/
  0xFBC2, 0x0020, 0x0002, 0x9521, 0x0000, 0x0000, 0x0002, /*11521*/
  0xFBC2, 0x0020, 0x0002, 0x9522, 0x0000, 0x0000, 0x0002, /*11522*/
  0xFBC2, 0x0020, 0x0002, 0x9523, 0x0000, 0x0000, 0x0002, /*11523*/
  0xFBC2, 0x0020, 0x0002, 0x9524, 0x0000, 0x0000, 0x0002, /*11524*/
  0xFBC2, 0x0020, 0x0002, 0x9525, 0x0000, 0x0000, 0x0002, /*11525*/
  0xFBC2, 0x0020, 0x0002, 0x9526, 0x0000, 0x0000, 0x0002, /*11526*/
  0xFBC2, 0x0020, 0x0002, 0x9527, 0x0000, 0x0000, 0x0002, /*11527*/
  0xFBC2, 0x0020, 0x0002, 0x9528, 0x0000, 0x0000, 0x0002, /*11528*/
  0xFBC2, 0x0020, 0x0002, 0x9529, 0x0000, 0x0000, 0x0002, /*11529*/
  0xFBC2, 0x0020, 0x0002, 0x952A, 0x0000, 0x0000, 0x0002, /*1152A*/
  0xFBC2, 0x0020, 0x0002, 0x952B, 0x0000, 0x0000, 0x0002, /*1152B*/
  0xFBC2, 0x0020, 0x0002, 0x952C, 0x0000, 0x0000, 0x0002, /*1152C*/
  0xFBC2, 0x0020, 0x0002, 0x952D, 0x0000, 0x0000, 0x0002, /*1152D*/
  0xFBC2, 0x0020, 0x0002, 0x952E, 0x0000, 0x0000, 0x0002, /*1152E*/
  0xFBC2, 0x0020, 0x0002, 0x952F, 0x0000, 0x0000, 0x0002, /*1152F*/
  0xFBC2, 0x0020, 0x0002, 0x9530, 0x0000, 0x0000, 0x0002, /*11530*/
  0xFBC2, 0x0020, 0x0002, 0x9531, 0x0000, 0x0000, 0x0002, /*11531*/
  0xFBC2, 0x0020, 0x0002, 0x9532, 0x0000, 0x0000, 0x0002, /*11532*/
  0xFBC2, 0x0020, 0x0002, 0x9533, 0x0000, 0x0000, 0x0002, /*11533*/
  0xFBC2, 0x0020, 0x0002, 0x9534, 0x0000, 0x0000, 0x0002, /*11534*/
  0xFBC2, 0x0020, 0x0002, 0x9535, 0x0000, 0x0000, 0x0002, /*11535*/
  0xFBC2, 0x0020, 0x0002, 0x9536, 0x0000, 0x0000, 0x0002, /*11536*/
  0xFBC2, 0x0020, 0x0002, 0x9537, 0x0000, 0x0000, 0x0002, /*11537*/
  0xFBC2, 0x0020, 0x0002, 0x9538, 0x0000, 0x0000, 0x0002, /*11538*/
  0xFBC2, 0x0020, 0x0002, 0x9539, 0x0000, 0x0000, 0x0002, /*11539*/
  0xFBC2, 0x0020, 0x0002, 0x953A, 0x0000, 0x0000, 0x0002, /*1153A*/
  0xFBC2, 0x0020, 0x0002, 0x953B, 0x0000, 0x0000, 0x0002, /*1153B*/
  0xFBC2, 0x0020, 0x0002, 0x953C, 0x0000, 0x0000, 0x0002, /*1153C*/
  0xFBC2, 0x0020, 0x0002, 0x953D, 0x0000, 0x0000, 0x0002, /*1153D*/
  0xFBC2, 0x0020, 0x0002, 0x953E, 0x0000, 0x0000, 0x0002, /*1153E*/
  0xFBC2, 0x0020, 0x0002, 0x953F, 0x0000, 0x0000, 0x0002, /*1153F*/
  0xFBC2, 0x0020, 0x0002, 0x9540, 0x0000, 0x0000, 0x0002, /*11540*/
  0xFBC2, 0x0020, 0x0002, 0x9541, 0x0000, 0x0000, 0x0002, /*11541*/
  0xFBC2, 0x0020, 0x0002, 0x9542, 0x0000, 0x0000, 0x0002, /*11542*/
  0xFBC2, 0x0020, 0x0002, 0x9543, 0x0000, 0x0000, 0x0002, /*11543*/
  0xFBC2, 0x0020, 0x0002, 0x9544, 0x0000, 0x0000, 0x0002, /*11544*/
  0xFBC2, 0x0020, 0x0002, 0x9545, 0x0000, 0x0000, 0x0002, /*11545*/
  0xFBC2, 0x0020, 0x0002, 0x9546, 0x0000, 0x0000, 0x0002, /*11546*/
  0xFBC2, 0x0020, 0x0002, 0x9547, 0x0000, 0x0000, 0x0002, /*11547*/
  0xFBC2, 0x0020, 0x0002, 0x9548, 0x0000, 0x0000, 0x0002, /*11548*/
  0xFBC2, 0x0020, 0x0002, 0x9549, 0x0000, 0x0000, 0x0002, /*11549*/
  0xFBC2, 0x0020, 0x0002, 0x954A, 0x0000, 0x0000, 0x0002, /*1154A*/
  0xFBC2, 0x0020, 0x0002, 0x954B, 0x0000, 0x0000, 0x0002, /*1154B*/
  0xFBC2, 0x0020, 0x0002, 0x954C, 0x0000, 0x0000, 0x0002, /*1154C*/
  0xFBC2, 0x0020, 0x0002, 0x954D, 0x0000, 0x0000, 0x0002, /*1154D*/
  0xFBC2, 0x0020, 0x0002, 0x954E, 0x0000, 0x0000, 0x0002, /*1154E*/
  0xFBC2, 0x0020, 0x0002, 0x954F, 0x0000, 0x0000, 0x0002, /*1154F*/
  0xFBC2, 0x0020, 0x0002, 0x9550, 0x0000, 0x0000, 0x0002, /*11550*/
  0xFBC2, 0x0020, 0x0002, 0x9551, 0x0000, 0x0000, 0x0002, /*11551*/
  0xFBC2, 0x0020, 0x0002, 0x9552, 0x0000, 0x0000, 0x0002, /*11552*/
  0xFBC2, 0x0020, 0x0002, 0x9553, 0x0000, 0x0000, 0x0002, /*11553*/
  0xFBC2, 0x0020, 0x0002, 0x9554, 0x0000, 0x0000, 0x0002, /*11554*/
  0xFBC2, 0x0020, 0x0002, 0x9555, 0x0000, 0x0000, 0x0002, /*11555*/
  0xFBC2, 0x0020, 0x0002, 0x9556, 0x0000, 0x0000, 0x0002, /*11556*/
  0xFBC2, 0x0020, 0x0002, 0x9557, 0x0000, 0x0000, 0x0002, /*11557*/
  0xFBC2, 0x0020, 0x0002, 0x9558, 0x0000, 0x0000, 0x0002, /*11558*/
  0xFBC2, 0x0020, 0x0002, 0x9559, 0x0000, 0x0000, 0x0002, /*11559*/
  0xFBC2, 0x0020, 0x0002, 0x955A, 0x0000, 0x0000, 0x0002, /*1155A*/
  0xFBC2, 0x0020, 0x0002, 0x955B, 0x0000, 0x0000, 0x0002, /*1155B*/
  0xFBC2, 0x0020, 0x0002, 0x955C, 0x0000, 0x0000, 0x0002, /*1155C*/
  0xFBC2, 0x0020, 0x0002, 0x955D, 0x0000, 0x0000, 0x0002, /*1155D*/
  0xFBC2, 0x0020, 0x0002, 0x955E, 0x0000, 0x0000, 0x0002, /*1155E*/
  0xFBC2, 0x0020, 0x0002, 0x955F, 0x0000, 0x0000, 0x0002, /*1155F*/
  0xFBC2, 0x0020, 0x0002, 0x9560, 0x0000, 0x0000, 0x0002, /*11560*/
  0xFBC2, 0x0020, 0x0002, 0x9561, 0x0000, 0x0000, 0x0002, /*11561*/
  0xFBC2, 0x0020, 0x0002, 0x9562, 0x0000, 0x0000, 0x0002, /*11562*/
  0xFBC2, 0x0020, 0x0002, 0x9563, 0x0000, 0x0000, 0x0002, /*11563*/
  0xFBC2, 0x0020, 0x0002, 0x9564, 0x0000, 0x0000, 0x0002, /*11564*/
  0xFBC2, 0x0020, 0x0002, 0x9565, 0x0000, 0x0000, 0x0002, /*11565*/
  0xFBC2, 0x0020, 0x0002, 0x9566, 0x0000, 0x0000, 0x0002, /*11566*/
  0xFBC2, 0x0020, 0x0002, 0x9567, 0x0000, 0x0000, 0x0002, /*11567*/
  0xFBC2, 0x0020, 0x0002, 0x9568, 0x0000, 0x0000, 0x0002, /*11568*/
  0xFBC2, 0x0020, 0x0002, 0x9569, 0x0000, 0x0000, 0x0002, /*11569*/
  0xFBC2, 0x0020, 0x0002, 0x956A, 0x0000, 0x0000, 0x0002, /*1156A*/
  0xFBC2, 0x0020, 0x0002, 0x956B, 0x0000, 0x0000, 0x0002, /*1156B*/
  0xFBC2, 0x0020, 0x0002, 0x956C, 0x0000, 0x0000, 0x0002, /*1156C*/
  0xFBC2, 0x0020, 0x0002, 0x956D, 0x0000, 0x0000, 0x0002, /*1156D*/
  0xFBC2, 0x0020, 0x0002, 0x956E, 0x0000, 0x0000, 0x0002, /*1156E*/
  0xFBC2, 0x0020, 0x0002, 0x956F, 0x0000, 0x0000, 0x0002, /*1156F*/
  0xFBC2, 0x0020, 0x0002, 0x9570, 0x0000, 0x0000, 0x0002, /*11570*/
  0xFBC2, 0x0020, 0x0002, 0x9571, 0x0000, 0x0000, 0x0002, /*11571*/
  0xFBC2, 0x0020, 0x0002, 0x9572, 0x0000, 0x0000, 0x0002, /*11572*/
  0xFBC2, 0x0020, 0x0002, 0x9573, 0x0000, 0x0000, 0x0002, /*11573*/
  0xFBC2, 0x0020, 0x0002, 0x9574, 0x0000, 0x0000, 0x0002, /*11574*/
  0xFBC2, 0x0020, 0x0002, 0x9575, 0x0000, 0x0000, 0x0002, /*11575*/
  0xFBC2, 0x0020, 0x0002, 0x9576, 0x0000, 0x0000, 0x0002, /*11576*/
  0xFBC2, 0x0020, 0x0002, 0x9577, 0x0000, 0x0000, 0x0002, /*11577*/
  0xFBC2, 0x0020, 0x0002, 0x9578, 0x0000, 0x0000, 0x0002, /*11578*/
  0xFBC2, 0x0020, 0x0002, 0x9579, 0x0000, 0x0000, 0x0002, /*11579*/
  0xFBC2, 0x0020, 0x0002, 0x957A, 0x0000, 0x0000, 0x0002, /*1157A*/
  0xFBC2, 0x0020, 0x0002, 0x957B, 0x0000, 0x0000, 0x0002, /*1157B*/
  0xFBC2, 0x0020, 0x0002, 0x957C, 0x0000, 0x0000, 0x0002, /*1157C*/
  0xFBC2, 0x0020, 0x0002, 0x957D, 0x0000, 0x0000, 0x0002, /*1157D*/
  0xFBC2, 0x0020, 0x0002, 0x957E, 0x0000, 0x0000, 0x0002, /*1157E*/
  0xFBC2, 0x0020, 0x0002, 0x957F, 0x0000, 0x0000, 0x0002, /*1157F*/
  0x2BC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11580*/
  0x2BC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11581*/
  0x2BC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11582*/
  0x2BC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11583*/
  0x2BC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11584*/
  0x2BC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11585*/
  0x2BC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11586*/
  0x2BC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11587*/
  0x2BCA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11588*/
  0x2BCB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11589*/
  0x2BCC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1158A*/
  0x2BCD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1158B*/
  0x2BCE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1158C*/
  0x2BCF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1158D*/
  0x2BD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1158E*/
  0x2BD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1158F*/
  0x2BD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11590*/
  0x2BD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11591*/
  0x2BD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11592*/
  0x2BD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11593*/
  0x2BD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11594*/
  0x2BD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11595*/
  0x2BD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11596*/
  0x2BD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11597*/
  0x2BDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11598*/
  0x2BDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11599*/
  0x2BDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1159A*/
  0x2BDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1159B*/
  0x2BDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1159C*/
  0x2BDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1159D*/
  0x2BE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1159E*/
  0x2BE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1159F*/
  0x2BE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115A0*/
  0x2BE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115A1*/
  0x2BE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115A2*/
  0x2BE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115A3*/
  0x2BE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115A4*/
  0x2BE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115A5*/
  0x2BE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115A6*/
  0x2BE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115A7*/
  0x2BEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115A8*/
  0x2BEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115A9*/
  0x2BEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115AA*/
  0x2BED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115AB*/
  0x2BEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115AC*/
  0x2BEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115AD*/
  0x2BF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115AE*/
  0x2BF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115AF*/
  0x2BF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115B0*/
  0x2BF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115B1*/
  0x2BF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115B2*/
  0x2BF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115B3*/
  0x2BF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115B4*/
  0x2BF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115B5*/
  0xFBC2, 0x0020, 0x0002, 0x95B6, 0x0000, 0x0000, 0x0002, /*115B6*/
  0xFBC2, 0x0020, 0x0002, 0x95B7, 0x0000, 0x0000, 0x0002, /*115B7*/
  0x2BF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115B8*/
  0x2BF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115B9*/
  0x2BFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115BA*/
  0x2BFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115BB*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115BC*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115BD*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115BE*/
  0x2BFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115BF*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115C0*/
  0x0461, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115C1*/
  0x02B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115C2*/
  0x02B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115C3*/
  0x0462, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115C4*/
  0x0463, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115C5*/
  0x0464, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115C6*/
  0x0465, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115C7*/
  0x0466, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115C8*/
  0x0467, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115C9*/
  0x0468, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115CA*/
  0x0469, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115CB*/
  0x046A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115CC*/
  0x046B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115CD*/
  0x046C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115CE*/
  0x046D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115CF*/
  0x046E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115D0*/
  0x046F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115D1*/
  0x0470, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115D2*/
  0x0471, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115D3*/
  0x0472, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115D4*/
  0x0473, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115D5*/
  0x0474, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115D6*/
  0x0475, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*115D7*/
  0x2BC4, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*115D8*/
  0x2BC4, 0x0020, 0x0004, 0x0000, 0x0111, 0x0004, 0x0002, /*115D9*/
  0x2BC5, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*115DA*/
  0x2BC6, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*115DB*/
  0x2BF4, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*115DC*/
  0x2BF5, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*115DD*/
  0xFBC2, 0x0020, 0x0002, 0x95DE, 0x0000, 0x0000, 0x0002, /*115DE*/
  0xFBC2, 0x0020, 0x0002, 0x95DF, 0x0000, 0x0000, 0x0002, /*115DF*/
  0xFBC2, 0x0020, 0x0002, 0x95E0, 0x0000, 0x0000, 0x0002, /*115E0*/
  0xFBC2, 0x0020, 0x0002, 0x95E1, 0x0000, 0x0000, 0x0002, /*115E1*/
  0xFBC2, 0x0020, 0x0002, 0x95E2, 0x0000, 0x0000, 0x0002, /*115E2*/
  0xFBC2, 0x0020, 0x0002, 0x95E3, 0x0000, 0x0000, 0x0002, /*115E3*/
  0xFBC2, 0x0020, 0x0002, 0x95E4, 0x0000, 0x0000, 0x0002, /*115E4*/
  0xFBC2, 0x0020, 0x0002, 0x95E5, 0x0000, 0x0000, 0x0002, /*115E5*/
  0xFBC2, 0x0020, 0x0002, 0x95E6, 0x0000, 0x0000, 0x0002, /*115E6*/
  0xFBC2, 0x0020, 0x0002, 0x95E7, 0x0000, 0x0000, 0x0002, /*115E7*/
  0xFBC2, 0x0020, 0x0002, 0x95E8, 0x0000, 0x0000, 0x0002, /*115E8*/
  0xFBC2, 0x0020, 0x0002, 0x95E9, 0x0000, 0x0000, 0x0002, /*115E9*/
  0xFBC2, 0x0020, 0x0002, 0x95EA, 0x0000, 0x0000, 0x0002, /*115EA*/
  0xFBC2, 0x0020, 0x0002, 0x95EB, 0x0000, 0x0000, 0x0002, /*115EB*/
  0xFBC2, 0x0020, 0x0002, 0x95EC, 0x0000, 0x0000, 0x0002, /*115EC*/
  0xFBC2, 0x0020, 0x0002, 0x95ED, 0x0000, 0x0000, 0x0002, /*115ED*/
  0xFBC2, 0x0020, 0x0002, 0x95EE, 0x0000, 0x0000, 0x0002, /*115EE*/
  0xFBC2, 0x0020, 0x0002, 0x95EF, 0x0000, 0x0000, 0x0002, /*115EF*/
  0xFBC2, 0x0020, 0x0002, 0x95F0, 0x0000, 0x0000, 0x0002, /*115F0*/
  0xFBC2, 0x0020, 0x0002, 0x95F1, 0x0000, 0x0000, 0x0002, /*115F1*/
  0xFBC2, 0x0020, 0x0002, 0x95F2, 0x0000, 0x0000, 0x0002, /*115F2*/
  0xFBC2, 0x0020, 0x0002, 0x95F3, 0x0000, 0x0000, 0x0002, /*115F3*/
  0xFBC2, 0x0020, 0x0002, 0x95F4, 0x0000, 0x0000, 0x0002, /*115F4*/
  0xFBC2, 0x0020, 0x0002, 0x95F5, 0x0000, 0x0000, 0x0002, /*115F5*/
  0xFBC2, 0x0020, 0x0002, 0x95F6, 0x0000, 0x0000, 0x0002, /*115F6*/
  0xFBC2, 0x0020, 0x0002, 0x95F7, 0x0000, 0x0000, 0x0002, /*115F7*/
  0xFBC2, 0x0020, 0x0002, 0x95F8, 0x0000, 0x0000, 0x0002, /*115F8*/
  0xFBC2, 0x0020, 0x0002, 0x95F9, 0x0000, 0x0000, 0x0002, /*115F9*/
  0xFBC2, 0x0020, 0x0002, 0x95FA, 0x0000, 0x0000, 0x0002, /*115FA*/
  0xFBC2, 0x0020, 0x0002, 0x95FB, 0x0000, 0x0000, 0x0002, /*115FB*/
  0xFBC2, 0x0020, 0x0002, 0x95FC, 0x0000, 0x0000, 0x0002, /*115FC*/
  0xFBC2, 0x0020, 0x0002, 0x95FD, 0x0000, 0x0000, 0x0002, /*115FD*/
  0xFBC2, 0x0020, 0x0002, 0x95FE, 0x0000, 0x0000, 0x0002, /*115FE*/
  0xFBC2, 0x0020, 0x0002, 0x95FF, 0x0000, 0x0000, 0x0002  /*115FF*/
};

uint16 uca900_p116[]= {
  0x2BFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11600*/
  0x2BFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11601*/
  0x2BFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11602*/
  0x2C00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11603*/
  0x2C01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11604*/
  0x2C02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11605*/
  0x2C03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11606*/
  0x2C04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11607*/
  0x2C05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11608*/
  0x2C06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11609*/
  0x2C07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1160A*/
  0x2C08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1160B*/
  0x2C09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1160C*/
  0x2C0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1160D*/
  0x2C0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1160E*/
  0x2C0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1160F*/
  0x2C0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11610*/
  0x2C0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11611*/
  0x2C0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11612*/
  0x2C10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11613*/
  0x2C11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11614*/
  0x2C12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11615*/
  0x2C13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11616*/
  0x2C14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11617*/
  0x2C15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11618*/
  0x2C16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11619*/
  0x2C17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1161A*/
  0x2C18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1161B*/
  0x2C19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1161C*/
  0x2C1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1161D*/
  0x2C1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1161E*/
  0x2C1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1161F*/
  0x2C1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11620*/
  0x2C1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11621*/
  0x2C1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11622*/
  0x2C20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11623*/
  0x2C21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11624*/
  0x2C22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11625*/
  0x2C23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11626*/
  0x2C24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11627*/
  0x2C25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11628*/
  0x2C26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11629*/
  0x2C27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1162A*/
  0x2C28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1162B*/
  0x2C29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1162C*/
  0x2C2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1162D*/
  0x2C2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1162E*/
  0x2C2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1162F*/
  0x2C2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11630*/
  0x2C2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11631*/
  0x2C2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11632*/
  0x2C30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11633*/
  0x2C31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11634*/
  0x2C32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11635*/
  0x2C33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11636*/
  0x2C34, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11637*/
  0x2C35, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11638*/
  0x2C36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11639*/
  0x2C37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1163A*/
  0x2C38, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1163B*/
  0x2C39, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1163C*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1163D*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1163E*/
  0x2C3A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1163F*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11640*/
  0x02BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11641*/
  0x02BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11642*/
  0x0476, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11643*/
  0x2C3B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11644*/
  0xFBC2, 0x0020, 0x0002, 0x9645, 0x0000, 0x0000, 0x0002, /*11645*/
  0xFBC2, 0x0020, 0x0002, 0x9646, 0x0000, 0x0000, 0x0002, /*11646*/
  0xFBC2, 0x0020, 0x0002, 0x9647, 0x0000, 0x0000, 0x0002, /*11647*/
  0xFBC2, 0x0020, 0x0002, 0x9648, 0x0000, 0x0000, 0x0002, /*11648*/
  0xFBC2, 0x0020, 0x0002, 0x9649, 0x0000, 0x0000, 0x0002, /*11649*/
  0xFBC2, 0x0020, 0x0002, 0x964A, 0x0000, 0x0000, 0x0002, /*1164A*/
  0xFBC2, 0x0020, 0x0002, 0x964B, 0x0000, 0x0000, 0x0002, /*1164B*/
  0xFBC2, 0x0020, 0x0002, 0x964C, 0x0000, 0x0000, 0x0002, /*1164C*/
  0xFBC2, 0x0020, 0x0002, 0x964D, 0x0000, 0x0000, 0x0002, /*1164D*/
  0xFBC2, 0x0020, 0x0002, 0x964E, 0x0000, 0x0000, 0x0002, /*1164E*/
  0xFBC2, 0x0020, 0x0002, 0x964F, 0x0000, 0x0000, 0x0002, /*1164F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11650*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11651*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11652*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11653*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11654*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11655*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11656*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11657*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11658*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11659*/
  0xFBC2, 0x0020, 0x0002, 0x965A, 0x0000, 0x0000, 0x0002, /*1165A*/
  0xFBC2, 0x0020, 0x0002, 0x965B, 0x0000, 0x0000, 0x0002, /*1165B*/
  0xFBC2, 0x0020, 0x0002, 0x965C, 0x0000, 0x0000, 0x0002, /*1165C*/
  0xFBC2, 0x0020, 0x0002, 0x965D, 0x0000, 0x0000, 0x0002, /*1165D*/
  0xFBC2, 0x0020, 0x0002, 0x965E, 0x0000, 0x0000, 0x0002, /*1165E*/
  0xFBC2, 0x0020, 0x0002, 0x965F, 0x0000, 0x0000, 0x0002, /*1165F*/
  0x03DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11660*/
  0x03DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11661*/
  0x03DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11662*/
  0x03DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11663*/
  0x03DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11664*/
  0x03DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11665*/
  0x03E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11666*/
  0x03E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11667*/
  0x03E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11668*/
  0x03E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11669*/
  0x03E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1166A*/
  0x03E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1166B*/
  0x03E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1166C*/
  0xFBC2, 0x0020, 0x0002, 0x966D, 0x0000, 0x0000, 0x0002, /*1166D*/
  0xFBC2, 0x0020, 0x0002, 0x966E, 0x0000, 0x0000, 0x0002, /*1166E*/
  0xFBC2, 0x0020, 0x0002, 0x966F, 0x0000, 0x0000, 0x0002, /*1166F*/
  0xFBC2, 0x0020, 0x0002, 0x9670, 0x0000, 0x0000, 0x0002, /*11670*/
  0xFBC2, 0x0020, 0x0002, 0x9671, 0x0000, 0x0000, 0x0002, /*11671*/
  0xFBC2, 0x0020, 0x0002, 0x9672, 0x0000, 0x0000, 0x0002, /*11672*/
  0xFBC2, 0x0020, 0x0002, 0x9673, 0x0000, 0x0000, 0x0002, /*11673*/
  0xFBC2, 0x0020, 0x0002, 0x9674, 0x0000, 0x0000, 0x0002, /*11674*/
  0xFBC2, 0x0020, 0x0002, 0x9675, 0x0000, 0x0000, 0x0002, /*11675*/
  0xFBC2, 0x0020, 0x0002, 0x9676, 0x0000, 0x0000, 0x0002, /*11676*/
  0xFBC2, 0x0020, 0x0002, 0x9677, 0x0000, 0x0000, 0x0002, /*11677*/
  0xFBC2, 0x0020, 0x0002, 0x9678, 0x0000, 0x0000, 0x0002, /*11678*/
  0xFBC2, 0x0020, 0x0002, 0x9679, 0x0000, 0x0000, 0x0002, /*11679*/
  0xFBC2, 0x0020, 0x0002, 0x967A, 0x0000, 0x0000, 0x0002, /*1167A*/
  0xFBC2, 0x0020, 0x0002, 0x967B, 0x0000, 0x0000, 0x0002, /*1167B*/
  0xFBC2, 0x0020, 0x0002, 0x967C, 0x0000, 0x0000, 0x0002, /*1167C*/
  0xFBC2, 0x0020, 0x0002, 0x967D, 0x0000, 0x0000, 0x0002, /*1167D*/
  0xFBC2, 0x0020, 0x0002, 0x967E, 0x0000, 0x0000, 0x0002, /*1167E*/
  0xFBC2, 0x0020, 0x0002, 0x967F, 0x0000, 0x0000, 0x0002, /*1167F*/
  0x2C3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11680*/
  0x2C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11681*/
  0x2C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11682*/
  0x2C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11683*/
  0x2C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11684*/
  0x2C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11685*/
  0x2C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11686*/
  0x2C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11687*/
  0x2C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11688*/
  0x2C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11689*/
  0x2C49, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1168A*/
  0x2C4A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1168B*/
  0x2C4B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1168C*/
  0x2C4C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1168D*/
  0x2C4D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1168E*/
  0x2C4E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1168F*/
  0x2C4F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11690*/
  0x2C50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11691*/
  0x2C51, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11692*/
  0x2C52, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11693*/
  0x2C53, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11694*/
  0x2C54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11695*/
  0x2C55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11696*/
  0x2C56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11697*/
  0x2C57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11698*/
  0x2C58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11699*/
  0x2C59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1169A*/
  0x2C5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1169B*/
  0x2C5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1169C*/
  0x2C5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1169D*/
  0x2C5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1169E*/
  0x2C5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1169F*/
  0x2C5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116A0*/
  0x2C60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116A1*/
  0x2C61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116A2*/
  0x2C62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116A3*/
  0x2C63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116A4*/
  0x2C64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116A5*/
  0x2C65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116A6*/
  0x2C47, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116A7*/
  0x2C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116A8*/
  0x2C48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116A9*/
  0x2C66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116AA*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116AB*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116AC*/
  0x2C67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116AD*/
  0x2C68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116AE*/
  0x2C69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116AF*/
  0x2C6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116B0*/
  0x2C6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116B1*/
  0x2C6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116B2*/
  0x2C6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116B3*/
  0x2C6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116B4*/
  0x2C6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116B5*/
  0x2C70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116B6*/
  0x0000, 0x00C2, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116B7*/
  0xFBC2, 0x0020, 0x0002, 0x96B8, 0x0000, 0x0000, 0x0002, /*116B8*/
  0xFBC2, 0x0020, 0x0002, 0x96B9, 0x0000, 0x0000, 0x0002, /*116B9*/
  0xFBC2, 0x0020, 0x0002, 0x96BA, 0x0000, 0x0000, 0x0002, /*116BA*/
  0xFBC2, 0x0020, 0x0002, 0x96BB, 0x0000, 0x0000, 0x0002, /*116BB*/
  0xFBC2, 0x0020, 0x0002, 0x96BC, 0x0000, 0x0000, 0x0002, /*116BC*/
  0xFBC2, 0x0020, 0x0002, 0x96BD, 0x0000, 0x0000, 0x0002, /*116BD*/
  0xFBC2, 0x0020, 0x0002, 0x96BE, 0x0000, 0x0000, 0x0002, /*116BE*/
  0xFBC2, 0x0020, 0x0002, 0x96BF, 0x0000, 0x0000, 0x0002, /*116BF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116C0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116C1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116C2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116C3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116C4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116C5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116C6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116C7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116C8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*116C9*/
  0xFBC2, 0x0020, 0x0002, 0x96CA, 0x0000, 0x0000, 0x0002, /*116CA*/
  0xFBC2, 0x0020, 0x0002, 0x96CB, 0x0000, 0x0000, 0x0002, /*116CB*/
  0xFBC2, 0x0020, 0x0002, 0x96CC, 0x0000, 0x0000, 0x0002, /*116CC*/
  0xFBC2, 0x0020, 0x0002, 0x96CD, 0x0000, 0x0000, 0x0002, /*116CD*/
  0xFBC2, 0x0020, 0x0002, 0x96CE, 0x0000, 0x0000, 0x0002, /*116CE*/
  0xFBC2, 0x0020, 0x0002, 0x96CF, 0x0000, 0x0000, 0x0002, /*116CF*/
  0xFBC2, 0x0020, 0x0002, 0x96D0, 0x0000, 0x0000, 0x0002, /*116D0*/
  0xFBC2, 0x0020, 0x0002, 0x96D1, 0x0000, 0x0000, 0x0002, /*116D1*/
  0xFBC2, 0x0020, 0x0002, 0x96D2, 0x0000, 0x0000, 0x0002, /*116D2*/
  0xFBC2, 0x0020, 0x0002, 0x96D3, 0x0000, 0x0000, 0x0002, /*116D3*/
  0xFBC2, 0x0020, 0x0002, 0x96D4, 0x0000, 0x0000, 0x0002, /*116D4*/
  0xFBC2, 0x0020, 0x0002, 0x96D5, 0x0000, 0x0000, 0x0002, /*116D5*/
  0xFBC2, 0x0020, 0x0002, 0x96D6, 0x0000, 0x0000, 0x0002, /*116D6*/
  0xFBC2, 0x0020, 0x0002, 0x96D7, 0x0000, 0x0000, 0x0002, /*116D7*/
  0xFBC2, 0x0020, 0x0002, 0x96D8, 0x0000, 0x0000, 0x0002, /*116D8*/
  0xFBC2, 0x0020, 0x0002, 0x96D9, 0x0000, 0x0000, 0x0002, /*116D9*/
  0xFBC2, 0x0020, 0x0002, 0x96DA, 0x0000, 0x0000, 0x0002, /*116DA*/
  0xFBC2, 0x0020, 0x0002, 0x96DB, 0x0000, 0x0000, 0x0002, /*116DB*/
  0xFBC2, 0x0020, 0x0002, 0x96DC, 0x0000, 0x0000, 0x0002, /*116DC*/
  0xFBC2, 0x0020, 0x0002, 0x96DD, 0x0000, 0x0000, 0x0002, /*116DD*/
  0xFBC2, 0x0020, 0x0002, 0x96DE, 0x0000, 0x0000, 0x0002, /*116DE*/
  0xFBC2, 0x0020, 0x0002, 0x96DF, 0x0000, 0x0000, 0x0002, /*116DF*/
  0xFBC2, 0x0020, 0x0002, 0x96E0, 0x0000, 0x0000, 0x0002, /*116E0*/
  0xFBC2, 0x0020, 0x0002, 0x96E1, 0x0000, 0x0000, 0x0002, /*116E1*/
  0xFBC2, 0x0020, 0x0002, 0x96E2, 0x0000, 0x0000, 0x0002, /*116E2*/
  0xFBC2, 0x0020, 0x0002, 0x96E3, 0x0000, 0x0000, 0x0002, /*116E3*/
  0xFBC2, 0x0020, 0x0002, 0x96E4, 0x0000, 0x0000, 0x0002, /*116E4*/
  0xFBC2, 0x0020, 0x0002, 0x96E5, 0x0000, 0x0000, 0x0002, /*116E5*/
  0xFBC2, 0x0020, 0x0002, 0x96E6, 0x0000, 0x0000, 0x0002, /*116E6*/
  0xFBC2, 0x0020, 0x0002, 0x96E7, 0x0000, 0x0000, 0x0002, /*116E7*/
  0xFBC2, 0x0020, 0x0002, 0x96E8, 0x0000, 0x0000, 0x0002, /*116E8*/
  0xFBC2, 0x0020, 0x0002, 0x96E9, 0x0000, 0x0000, 0x0002, /*116E9*/
  0xFBC2, 0x0020, 0x0002, 0x96EA, 0x0000, 0x0000, 0x0002, /*116EA*/
  0xFBC2, 0x0020, 0x0002, 0x96EB, 0x0000, 0x0000, 0x0002, /*116EB*/
  0xFBC2, 0x0020, 0x0002, 0x96EC, 0x0000, 0x0000, 0x0002, /*116EC*/
  0xFBC2, 0x0020, 0x0002, 0x96ED, 0x0000, 0x0000, 0x0002, /*116ED*/
  0xFBC2, 0x0020, 0x0002, 0x96EE, 0x0000, 0x0000, 0x0002, /*116EE*/
  0xFBC2, 0x0020, 0x0002, 0x96EF, 0x0000, 0x0000, 0x0002, /*116EF*/
  0xFBC2, 0x0020, 0x0002, 0x96F0, 0x0000, 0x0000, 0x0002, /*116F0*/
  0xFBC2, 0x0020, 0x0002, 0x96F1, 0x0000, 0x0000, 0x0002, /*116F1*/
  0xFBC2, 0x0020, 0x0002, 0x96F2, 0x0000, 0x0000, 0x0002, /*116F2*/
  0xFBC2, 0x0020, 0x0002, 0x96F3, 0x0000, 0x0000, 0x0002, /*116F3*/
  0xFBC2, 0x0020, 0x0002, 0x96F4, 0x0000, 0x0000, 0x0002, /*116F4*/
  0xFBC2, 0x0020, 0x0002, 0x96F5, 0x0000, 0x0000, 0x0002, /*116F5*/
  0xFBC2, 0x0020, 0x0002, 0x96F6, 0x0000, 0x0000, 0x0002, /*116F6*/
  0xFBC2, 0x0020, 0x0002, 0x96F7, 0x0000, 0x0000, 0x0002, /*116F7*/
  0xFBC2, 0x0020, 0x0002, 0x96F8, 0x0000, 0x0000, 0x0002, /*116F8*/
  0xFBC2, 0x0020, 0x0002, 0x96F9, 0x0000, 0x0000, 0x0002, /*116F9*/
  0xFBC2, 0x0020, 0x0002, 0x96FA, 0x0000, 0x0000, 0x0002, /*116FA*/
  0xFBC2, 0x0020, 0x0002, 0x96FB, 0x0000, 0x0000, 0x0002, /*116FB*/
  0xFBC2, 0x0020, 0x0002, 0x96FC, 0x0000, 0x0000, 0x0002, /*116FC*/
  0xFBC2, 0x0020, 0x0002, 0x96FD, 0x0000, 0x0000, 0x0002, /*116FD*/
  0xFBC2, 0x0020, 0x0002, 0x96FE, 0x0000, 0x0000, 0x0002, /*116FE*/
  0xFBC2, 0x0020, 0x0002, 0x96FF, 0x0000, 0x0000, 0x0002  /*116FF*/
};

uint16 uca900_p117[]= {
  0x2C71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11700*/
  0x2C72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11701*/
  0x2C73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11702*/
  0x2C74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11703*/
  0x2C75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11704*/
  0x2C75, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*11705*/
  0x2C76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11706*/
  0x2C77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11707*/
  0x2C78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11708*/
  0x2C79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11709*/
  0x2C7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1170A*/
  0x2C7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1170B*/
  0x2C7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1170C*/
  0x2C7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1170D*/
  0x2C7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1170E*/
  0x2C7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1170F*/
  0x2C80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11710*/
  0x2C81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11711*/
  0x2C82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11712*/
  0x2C83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11713*/
  0x2C84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11714*/
  0x2C85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11715*/
  0x2C85, 0x0020, 0x0004, 0x0000, 0x0110, 0x0004, 0x0002, /*11716*/
  0x2C86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11717*/
  0x2C87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11718*/
  0x2C88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11719*/
  0xFBC2, 0x0020, 0x0002, 0x971A, 0x0000, 0x0000, 0x0002, /*1171A*/
  0xFBC2, 0x0020, 0x0002, 0x971B, 0x0000, 0x0000, 0x0002, /*1171B*/
  0xFBC2, 0x0020, 0x0002, 0x971C, 0x0000, 0x0000, 0x0002, /*1171C*/
  0x2C95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1171D*/
  0x2C96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1171E*/
  0x2C97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1171F*/
  0x2C89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11720*/
  0x2C8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11721*/
  0x2C8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11722*/
  0x2C8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11723*/
  0x2C8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11724*/
  0x2C8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11725*/
  0x2C8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11726*/
  0x2C90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11727*/
  0x2C91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11728*/
  0x2C92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11729*/
  0x2C93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1172A*/
  0x2C94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1172B*/
  0xFBC2, 0x0020, 0x0002, 0x972C, 0x0000, 0x0000, 0x0002, /*1172C*/
  0xFBC2, 0x0020, 0x0002, 0x972D, 0x0000, 0x0000, 0x0002, /*1172D*/
  0xFBC2, 0x0020, 0x0002, 0x972E, 0x0000, 0x0000, 0x0002, /*1172E*/
  0xFBC2, 0x0020, 0x0002, 0x972F, 0x0000, 0x0000, 0x0002, /*1172F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11730*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11731*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11732*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11733*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11734*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11735*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11736*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11737*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11738*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11739*/
  0x1B71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1173A*/
  0x1B72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1173B*/
  0x02BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1173C*/
  0x02BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1173D*/
  0x02E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1173E*/
  0x0550, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1173F*/
  0xFBC2, 0x0020, 0x0002, 0x9740, 0x0000, 0x0000, 0x0002, /*11740*/
  0xFBC2, 0x0020, 0x0002, 0x9741, 0x0000, 0x0000, 0x0002, /*11741*/
  0xFBC2, 0x0020, 0x0002, 0x9742, 0x0000, 0x0000, 0x0002, /*11742*/
  0xFBC2, 0x0020, 0x0002, 0x9743, 0x0000, 0x0000, 0x0002, /*11743*/
  0xFBC2, 0x0020, 0x0002, 0x9744, 0x0000, 0x0000, 0x0002, /*11744*/
  0xFBC2, 0x0020, 0x0002, 0x9745, 0x0000, 0x0000, 0x0002, /*11745*/
  0xFBC2, 0x0020, 0x0002, 0x9746, 0x0000, 0x0000, 0x0002, /*11746*/
  0xFBC2, 0x0020, 0x0002, 0x9747, 0x0000, 0x0000, 0x0002, /*11747*/
  0xFBC2, 0x0020, 0x0002, 0x9748, 0x0000, 0x0000, 0x0002, /*11748*/
  0xFBC2, 0x0020, 0x0002, 0x9749, 0x0000, 0x0000, 0x0002, /*11749*/
  0xFBC2, 0x0020, 0x0002, 0x974A, 0x0000, 0x0000, 0x0002, /*1174A*/
  0xFBC2, 0x0020, 0x0002, 0x974B, 0x0000, 0x0000, 0x0002, /*1174B*/
  0xFBC2, 0x0020, 0x0002, 0x974C, 0x0000, 0x0000, 0x0002, /*1174C*/
  0xFBC2, 0x0020, 0x0002, 0x974D, 0x0000, 0x0000, 0x0002, /*1174D*/
  0xFBC2, 0x0020, 0x0002, 0x974E, 0x0000, 0x0000, 0x0002, /*1174E*/
  0xFBC2, 0x0020, 0x0002, 0x974F, 0x0000, 0x0000, 0x0002, /*1174F*/
  0xFBC2, 0x0020, 0x0002, 0x9750, 0x0000, 0x0000, 0x0002, /*11750*/
  0xFBC2, 0x0020, 0x0002, 0x9751, 0x0000, 0x0000, 0x0002, /*11751*/
  0xFBC2, 0x0020, 0x0002, 0x9752, 0x0000, 0x0000, 0x0002, /*11752*/
  0xFBC2, 0x0020, 0x0002, 0x9753, 0x0000, 0x0000, 0x0002, /*11753*/
  0xFBC2, 0x0020, 0x0002, 0x9754, 0x0000, 0x0000, 0x0002, /*11754*/
  0xFBC2, 0x0020, 0x0002, 0x9755, 0x0000, 0x0000, 0x0002, /*11755*/
  0xFBC2, 0x0020, 0x0002, 0x9756, 0x0000, 0x0000, 0x0002, /*11756*/
  0xFBC2, 0x0020, 0x0002, 0x9757, 0x0000, 0x0000, 0x0002, /*11757*/
  0xFBC2, 0x0020, 0x0002, 0x9758, 0x0000, 0x0000, 0x0002, /*11758*/
  0xFBC2, 0x0020, 0x0002, 0x9759, 0x0000, 0x0000, 0x0002, /*11759*/
  0xFBC2, 0x0020, 0x0002, 0x975A, 0x0000, 0x0000, 0x0002, /*1175A*/
  0xFBC2, 0x0020, 0x0002, 0x975B, 0x0000, 0x0000, 0x0002, /*1175B*/
  0xFBC2, 0x0020, 0x0002, 0x975C, 0x0000, 0x0000, 0x0002, /*1175C*/
  0xFBC2, 0x0020, 0x0002, 0x975D, 0x0000, 0x0000, 0x0002, /*1175D*/
  0xFBC2, 0x0020, 0x0002, 0x975E, 0x0000, 0x0000, 0x0002, /*1175E*/
  0xFBC2, 0x0020, 0x0002, 0x975F, 0x0000, 0x0000, 0x0002, /*1175F*/
  0xFBC2, 0x0020, 0x0002, 0x9760, 0x0000, 0x0000, 0x0002, /*11760*/
  0xFBC2, 0x0020, 0x0002, 0x9761, 0x0000, 0x0000, 0x0002, /*11761*/
  0xFBC2, 0x0020, 0x0002, 0x9762, 0x0000, 0x0000, 0x0002, /*11762*/
  0xFBC2, 0x0020, 0x0002, 0x9763, 0x0000, 0x0000, 0x0002, /*11763*/
  0xFBC2, 0x0020, 0x0002, 0x9764, 0x0000, 0x0000, 0x0002, /*11764*/
  0xFBC2, 0x0020, 0x0002, 0x9765, 0x0000, 0x0000, 0x0002, /*11765*/
  0xFBC2, 0x0020, 0x0002, 0x9766, 0x0000, 0x0000, 0x0002, /*11766*/
  0xFBC2, 0x0020, 0x0002, 0x9767, 0x0000, 0x0000, 0x0002, /*11767*/
  0xFBC2, 0x0020, 0x0002, 0x9768, 0x0000, 0x0000, 0x0002, /*11768*/
  0xFBC2, 0x0020, 0x0002, 0x9769, 0x0000, 0x0000, 0x0002, /*11769*/
  0xFBC2, 0x0020, 0x0002, 0x976A, 0x0000, 0x0000, 0x0002, /*1176A*/
  0xFBC2, 0x0020, 0x0002, 0x976B, 0x0000, 0x0000, 0x0002, /*1176B*/
  0xFBC2, 0x0020, 0x0002, 0x976C, 0x0000, 0x0000, 0x0002, /*1176C*/
  0xFBC2, 0x0020, 0x0002, 0x976D, 0x0000, 0x0000, 0x0002, /*1176D*/
  0xFBC2, 0x0020, 0x0002, 0x976E, 0x0000, 0x0000, 0x0002, /*1176E*/
  0xFBC2, 0x0020, 0x0002, 0x976F, 0x0000, 0x0000, 0x0002, /*1176F*/
  0xFBC2, 0x0020, 0x0002, 0x9770, 0x0000, 0x0000, 0x0002, /*11770*/
  0xFBC2, 0x0020, 0x0002, 0x9771, 0x0000, 0x0000, 0x0002, /*11771*/
  0xFBC2, 0x0020, 0x0002, 0x9772, 0x0000, 0x0000, 0x0002, /*11772*/
  0xFBC2, 0x0020, 0x0002, 0x9773, 0x0000, 0x0000, 0x0002, /*11773*/
  0xFBC2, 0x0020, 0x0002, 0x9774, 0x0000, 0x0000, 0x0002, /*11774*/
  0xFBC2, 0x0020, 0x0002, 0x9775, 0x0000, 0x0000, 0x0002, /*11775*/
  0xFBC2, 0x0020, 0x0002, 0x9776, 0x0000, 0x0000, 0x0002, /*11776*/
  0xFBC2, 0x0020, 0x0002, 0x9777, 0x0000, 0x0000, 0x0002, /*11777*/
  0xFBC2, 0x0020, 0x0002, 0x9778, 0x0000, 0x0000, 0x0002, /*11778*/
  0xFBC2, 0x0020, 0x0002, 0x9779, 0x0000, 0x0000, 0x0002, /*11779*/
  0xFBC2, 0x0020, 0x0002, 0x977A, 0x0000, 0x0000, 0x0002, /*1177A*/
  0xFBC2, 0x0020, 0x0002, 0x977B, 0x0000, 0x0000, 0x0002, /*1177B*/
  0xFBC2, 0x0020, 0x0002, 0x977C, 0x0000, 0x0000, 0x0002, /*1177C*/
  0xFBC2, 0x0020, 0x0002, 0x977D, 0x0000, 0x0000, 0x0002, /*1177D*/
  0xFBC2, 0x0020, 0x0002, 0x977E, 0x0000, 0x0000, 0x0002, /*1177E*/
  0xFBC2, 0x0020, 0x0002, 0x977F, 0x0000, 0x0000, 0x0002, /*1177F*/
  0xFBC2, 0x0020, 0x0002, 0x9780, 0x0000, 0x0000, 0x0002, /*11780*/
  0xFBC2, 0x0020, 0x0002, 0x9781, 0x0000, 0x0000, 0x0002, /*11781*/
  0xFBC2, 0x0020, 0x0002, 0x9782, 0x0000, 0x0000, 0x0002, /*11782*/
  0xFBC2, 0x0020, 0x0002, 0x9783, 0x0000, 0x0000, 0x0002, /*11783*/
  0xFBC2, 0x0020, 0x0002, 0x9784, 0x0000, 0x0000, 0x0002, /*11784*/
  0xFBC2, 0x0020, 0x0002, 0x9785, 0x0000, 0x0000, 0x0002, /*11785*/
  0xFBC2, 0x0020, 0x0002, 0x9786, 0x0000, 0x0000, 0x0002, /*11786*/
  0xFBC2, 0x0020, 0x0002, 0x9787, 0x0000, 0x0000, 0x0002, /*11787*/
  0xFBC2, 0x0020, 0x0002, 0x9788, 0x0000, 0x0000, 0x0002, /*11788*/
  0xFBC2, 0x0020, 0x0002, 0x9789, 0x0000, 0x0000, 0x0002, /*11789*/
  0xFBC2, 0x0020, 0x0002, 0x978A, 0x0000, 0x0000, 0x0002, /*1178A*/
  0xFBC2, 0x0020, 0x0002, 0x978B, 0x0000, 0x0000, 0x0002, /*1178B*/
  0xFBC2, 0x0020, 0x0002, 0x978C, 0x0000, 0x0000, 0x0002, /*1178C*/
  0xFBC2, 0x0020, 0x0002, 0x978D, 0x0000, 0x0000, 0x0002, /*1178D*/
  0xFBC2, 0x0020, 0x0002, 0x978E, 0x0000, 0x0000, 0x0002, /*1178E*/
  0xFBC2, 0x0020, 0x0002, 0x978F, 0x0000, 0x0000, 0x0002, /*1178F*/
  0xFBC2, 0x0020, 0x0002, 0x9790, 0x0000, 0x0000, 0x0002, /*11790*/
  0xFBC2, 0x0020, 0x0002, 0x9791, 0x0000, 0x0000, 0x0002, /*11791*/
  0xFBC2, 0x0020, 0x0002, 0x9792, 0x0000, 0x0000, 0x0002, /*11792*/
  0xFBC2, 0x0020, 0x0002, 0x9793, 0x0000, 0x0000, 0x0002, /*11793*/
  0xFBC2, 0x0020, 0x0002, 0x9794, 0x0000, 0x0000, 0x0002, /*11794*/
  0xFBC2, 0x0020, 0x0002, 0x9795, 0x0000, 0x0000, 0x0002, /*11795*/
  0xFBC2, 0x0020, 0x0002, 0x9796, 0x0000, 0x0000, 0x0002, /*11796*/
  0xFBC2, 0x0020, 0x0002, 0x9797, 0x0000, 0x0000, 0x0002, /*11797*/
  0xFBC2, 0x0020, 0x0002, 0x9798, 0x0000, 0x0000, 0x0002, /*11798*/
  0xFBC2, 0x0020, 0x0002, 0x9799, 0x0000, 0x0000, 0x0002, /*11799*/
  0xFBC2, 0x0020, 0x0002, 0x979A, 0x0000, 0x0000, 0x0002, /*1179A*/
  0xFBC2, 0x0020, 0x0002, 0x979B, 0x0000, 0x0000, 0x0002, /*1179B*/
  0xFBC2, 0x0020, 0x0002, 0x979C, 0x0000, 0x0000, 0x0002, /*1179C*/
  0xFBC2, 0x0020, 0x0002, 0x979D, 0x0000, 0x0000, 0x0002, /*1179D*/
  0xFBC2, 0x0020, 0x0002, 0x979E, 0x0000, 0x0000, 0x0002, /*1179E*/
  0xFBC2, 0x0020, 0x0002, 0x979F, 0x0000, 0x0000, 0x0002, /*1179F*/
  0xFBC2, 0x0020, 0x0002, 0x97A0, 0x0000, 0x0000, 0x0002, /*117A0*/
  0xFBC2, 0x0020, 0x0002, 0x97A1, 0x0000, 0x0000, 0x0002, /*117A1*/
  0xFBC2, 0x0020, 0x0002, 0x97A2, 0x0000, 0x0000, 0x0002, /*117A2*/
  0xFBC2, 0x0020, 0x0002, 0x97A3, 0x0000, 0x0000, 0x0002, /*117A3*/
  0xFBC2, 0x0020, 0x0002, 0x97A4, 0x0000, 0x0000, 0x0002, /*117A4*/
  0xFBC2, 0x0020, 0x0002, 0x97A5, 0x0000, 0x0000, 0x0002, /*117A5*/
  0xFBC2, 0x0020, 0x0002, 0x97A6, 0x0000, 0x0000, 0x0002, /*117A6*/
  0xFBC2, 0x0020, 0x0002, 0x97A7, 0x0000, 0x0000, 0x0002, /*117A7*/
  0xFBC2, 0x0020, 0x0002, 0x97A8, 0x0000, 0x0000, 0x0002, /*117A8*/
  0xFBC2, 0x0020, 0x0002, 0x97A9, 0x0000, 0x0000, 0x0002, /*117A9*/
  0xFBC2, 0x0020, 0x0002, 0x97AA, 0x0000, 0x0000, 0x0002, /*117AA*/
  0xFBC2, 0x0020, 0x0002, 0x97AB, 0x0000, 0x0000, 0x0002, /*117AB*/
  0xFBC2, 0x0020, 0x0002, 0x97AC, 0x0000, 0x0000, 0x0002, /*117AC*/
  0xFBC2, 0x0020, 0x0002, 0x97AD, 0x0000, 0x0000, 0x0002, /*117AD*/
  0xFBC2, 0x0020, 0x0002, 0x97AE, 0x0000, 0x0000, 0x0002, /*117AE*/
  0xFBC2, 0x0020, 0x0002, 0x97AF, 0x0000, 0x0000, 0x0002, /*117AF*/
  0xFBC2, 0x0020, 0x0002, 0x97B0, 0x0000, 0x0000, 0x0002, /*117B0*/
  0xFBC2, 0x0020, 0x0002, 0x97B1, 0x0000, 0x0000, 0x0002, /*117B1*/
  0xFBC2, 0x0020, 0x0002, 0x97B2, 0x0000, 0x0000, 0x0002, /*117B2*/
  0xFBC2, 0x0020, 0x0002, 0x97B3, 0x0000, 0x0000, 0x0002, /*117B3*/
  0xFBC2, 0x0020, 0x0002, 0x97B4, 0x0000, 0x0000, 0x0002, /*117B4*/
  0xFBC2, 0x0020, 0x0002, 0x97B5, 0x0000, 0x0000, 0x0002, /*117B5*/
  0xFBC2, 0x0020, 0x0002, 0x97B6, 0x0000, 0x0000, 0x0002, /*117B6*/
  0xFBC2, 0x0020, 0x0002, 0x97B7, 0x0000, 0x0000, 0x0002, /*117B7*/
  0xFBC2, 0x0020, 0x0002, 0x97B8, 0x0000, 0x0000, 0x0002, /*117B8*/
  0xFBC2, 0x0020, 0x0002, 0x97B9, 0x0000, 0x0000, 0x0002, /*117B9*/
  0xFBC2, 0x0020, 0x0002, 0x97BA, 0x0000, 0x0000, 0x0002, /*117BA*/
  0xFBC2, 0x0020, 0x0002, 0x97BB, 0x0000, 0x0000, 0x0002, /*117BB*/
  0xFBC2, 0x0020, 0x0002, 0x97BC, 0x0000, 0x0000, 0x0002, /*117BC*/
  0xFBC2, 0x0020, 0x0002, 0x97BD, 0x0000, 0x0000, 0x0002, /*117BD*/
  0xFBC2, 0x0020, 0x0002, 0x97BE, 0x0000, 0x0000, 0x0002, /*117BE*/
  0xFBC2, 0x0020, 0x0002, 0x97BF, 0x0000, 0x0000, 0x0002, /*117BF*/
  0xFBC2, 0x0020, 0x0002, 0x97C0, 0x0000, 0x0000, 0x0002, /*117C0*/
  0xFBC2, 0x0020, 0x0002, 0x97C1, 0x0000, 0x0000, 0x0002, /*117C1*/
  0xFBC2, 0x0020, 0x0002, 0x97C2, 0x0000, 0x0000, 0x0002, /*117C2*/
  0xFBC2, 0x0020, 0x0002, 0x97C3, 0x0000, 0x0000, 0x0002, /*117C3*/
  0xFBC2, 0x0020, 0x0002, 0x97C4, 0x0000, 0x0000, 0x0002, /*117C4*/
  0xFBC2, 0x0020, 0x0002, 0x97C5, 0x0000, 0x0000, 0x0002, /*117C5*/
  0xFBC2, 0x0020, 0x0002, 0x97C6, 0x0000, 0x0000, 0x0002, /*117C6*/
  0xFBC2, 0x0020, 0x0002, 0x97C7, 0x0000, 0x0000, 0x0002, /*117C7*/
  0xFBC2, 0x0020, 0x0002, 0x97C8, 0x0000, 0x0000, 0x0002, /*117C8*/
  0xFBC2, 0x0020, 0x0002, 0x97C9, 0x0000, 0x0000, 0x0002, /*117C9*/
  0xFBC2, 0x0020, 0x0002, 0x97CA, 0x0000, 0x0000, 0x0002, /*117CA*/
  0xFBC2, 0x0020, 0x0002, 0x97CB, 0x0000, 0x0000, 0x0002, /*117CB*/
  0xFBC2, 0x0020, 0x0002, 0x97CC, 0x0000, 0x0000, 0x0002, /*117CC*/
  0xFBC2, 0x0020, 0x0002, 0x97CD, 0x0000, 0x0000, 0x0002, /*117CD*/
  0xFBC2, 0x0020, 0x0002, 0x97CE, 0x0000, 0x0000, 0x0002, /*117CE*/
  0xFBC2, 0x0020, 0x0002, 0x97CF, 0x0000, 0x0000, 0x0002, /*117CF*/
  0xFBC2, 0x0020, 0x0002, 0x97D0, 0x0000, 0x0000, 0x0002, /*117D0*/
  0xFBC2, 0x0020, 0x0002, 0x97D1, 0x0000, 0x0000, 0x0002, /*117D1*/
  0xFBC2, 0x0020, 0x0002, 0x97D2, 0x0000, 0x0000, 0x0002, /*117D2*/
  0xFBC2, 0x0020, 0x0002, 0x97D3, 0x0000, 0x0000, 0x0002, /*117D3*/
  0xFBC2, 0x0020, 0x0002, 0x97D4, 0x0000, 0x0000, 0x0002, /*117D4*/
  0xFBC2, 0x0020, 0x0002, 0x97D5, 0x0000, 0x0000, 0x0002, /*117D5*/
  0xFBC2, 0x0020, 0x0002, 0x97D6, 0x0000, 0x0000, 0x0002, /*117D6*/
  0xFBC2, 0x0020, 0x0002, 0x97D7, 0x0000, 0x0000, 0x0002, /*117D7*/
  0xFBC2, 0x0020, 0x0002, 0x97D8, 0x0000, 0x0000, 0x0002, /*117D8*/
  0xFBC2, 0x0020, 0x0002, 0x97D9, 0x0000, 0x0000, 0x0002, /*117D9*/
  0xFBC2, 0x0020, 0x0002, 0x97DA, 0x0000, 0x0000, 0x0002, /*117DA*/
  0xFBC2, 0x0020, 0x0002, 0x97DB, 0x0000, 0x0000, 0x0002, /*117DB*/
  0xFBC2, 0x0020, 0x0002, 0x97DC, 0x0000, 0x0000, 0x0002, /*117DC*/
  0xFBC2, 0x0020, 0x0002, 0x97DD, 0x0000, 0x0000, 0x0002, /*117DD*/
  0xFBC2, 0x0020, 0x0002, 0x97DE, 0x0000, 0x0000, 0x0002, /*117DE*/
  0xFBC2, 0x0020, 0x0002, 0x97DF, 0x0000, 0x0000, 0x0002, /*117DF*/
  0xFBC2, 0x0020, 0x0002, 0x97E0, 0x0000, 0x0000, 0x0002, /*117E0*/
  0xFBC2, 0x0020, 0x0002, 0x97E1, 0x0000, 0x0000, 0x0002, /*117E1*/
  0xFBC2, 0x0020, 0x0002, 0x97E2, 0x0000, 0x0000, 0x0002, /*117E2*/
  0xFBC2, 0x0020, 0x0002, 0x97E3, 0x0000, 0x0000, 0x0002, /*117E3*/
  0xFBC2, 0x0020, 0x0002, 0x97E4, 0x0000, 0x0000, 0x0002, /*117E4*/
  0xFBC2, 0x0020, 0x0002, 0x97E5, 0x0000, 0x0000, 0x0002, /*117E5*/
  0xFBC2, 0x0020, 0x0002, 0x97E6, 0x0000, 0x0000, 0x0002, /*117E6*/
  0xFBC2, 0x0020, 0x0002, 0x97E7, 0x0000, 0x0000, 0x0002, /*117E7*/
  0xFBC2, 0x0020, 0x0002, 0x97E8, 0x0000, 0x0000, 0x0002, /*117E8*/
  0xFBC2, 0x0020, 0x0002, 0x97E9, 0x0000, 0x0000, 0x0002, /*117E9*/
  0xFBC2, 0x0020, 0x0002, 0x97EA, 0x0000, 0x0000, 0x0002, /*117EA*/
  0xFBC2, 0x0020, 0x0002, 0x97EB, 0x0000, 0x0000, 0x0002, /*117EB*/
  0xFBC2, 0x0020, 0x0002, 0x97EC, 0x0000, 0x0000, 0x0002, /*117EC*/
  0xFBC2, 0x0020, 0x0002, 0x97ED, 0x0000, 0x0000, 0x0002, /*117ED*/
  0xFBC2, 0x0020, 0x0002, 0x97EE, 0x0000, 0x0000, 0x0002, /*117EE*/
  0xFBC2, 0x0020, 0x0002, 0x97EF, 0x0000, 0x0000, 0x0002, /*117EF*/
  0xFBC2, 0x0020, 0x0002, 0x97F0, 0x0000, 0x0000, 0x0002, /*117F0*/
  0xFBC2, 0x0020, 0x0002, 0x97F1, 0x0000, 0x0000, 0x0002, /*117F1*/
  0xFBC2, 0x0020, 0x0002, 0x97F2, 0x0000, 0x0000, 0x0002, /*117F2*/
  0xFBC2, 0x0020, 0x0002, 0x97F3, 0x0000, 0x0000, 0x0002, /*117F3*/
  0xFBC2, 0x0020, 0x0002, 0x97F4, 0x0000, 0x0000, 0x0002, /*117F4*/
  0xFBC2, 0x0020, 0x0002, 0x97F5, 0x0000, 0x0000, 0x0002, /*117F5*/
  0xFBC2, 0x0020, 0x0002, 0x97F6, 0x0000, 0x0000, 0x0002, /*117F6*/
  0xFBC2, 0x0020, 0x0002, 0x97F7, 0x0000, 0x0000, 0x0002, /*117F7*/
  0xFBC2, 0x0020, 0x0002, 0x97F8, 0x0000, 0x0000, 0x0002, /*117F8*/
  0xFBC2, 0x0020, 0x0002, 0x97F9, 0x0000, 0x0000, 0x0002, /*117F9*/
  0xFBC2, 0x0020, 0x0002, 0x97FA, 0x0000, 0x0000, 0x0002, /*117FA*/
  0xFBC2, 0x0020, 0x0002, 0x97FB, 0x0000, 0x0000, 0x0002, /*117FB*/
  0xFBC2, 0x0020, 0x0002, 0x97FC, 0x0000, 0x0000, 0x0002, /*117FC*/
  0xFBC2, 0x0020, 0x0002, 0x97FD, 0x0000, 0x0000, 0x0002, /*117FD*/
  0xFBC2, 0x0020, 0x0002, 0x97FE, 0x0000, 0x0000, 0x0002, /*117FE*/
  0xFBC2, 0x0020, 0x0002, 0x97FF, 0x0000, 0x0000, 0x0002  /*117FF*/
};

uint16 uca900_p118[]= {
  0xFBC2, 0x0020, 0x0002, 0x9800, 0x0000, 0x0000, 0x0002, /*11800*/
  0xFBC2, 0x0020, 0x0002, 0x9801, 0x0000, 0x0000, 0x0002, /*11801*/
  0xFBC2, 0x0020, 0x0002, 0x9802, 0x0000, 0x0000, 0x0002, /*11802*/
  0xFBC2, 0x0020, 0x0002, 0x9803, 0x0000, 0x0000, 0x0002, /*11803*/
  0xFBC2, 0x0020, 0x0002, 0x9804, 0x0000, 0x0000, 0x0002, /*11804*/
  0xFBC2, 0x0020, 0x0002, 0x9805, 0x0000, 0x0000, 0x0002, /*11805*/
  0xFBC2, 0x0020, 0x0002, 0x9806, 0x0000, 0x0000, 0x0002, /*11806*/
  0xFBC2, 0x0020, 0x0002, 0x9807, 0x0000, 0x0000, 0x0002, /*11807*/
  0xFBC2, 0x0020, 0x0002, 0x9808, 0x0000, 0x0000, 0x0002, /*11808*/
  0xFBC2, 0x0020, 0x0002, 0x9809, 0x0000, 0x0000, 0x0002, /*11809*/
  0xFBC2, 0x0020, 0x0002, 0x980A, 0x0000, 0x0000, 0x0002, /*1180A*/
  0xFBC2, 0x0020, 0x0002, 0x980B, 0x0000, 0x0000, 0x0002, /*1180B*/
  0xFBC2, 0x0020, 0x0002, 0x980C, 0x0000, 0x0000, 0x0002, /*1180C*/
  0xFBC2, 0x0020, 0x0002, 0x980D, 0x0000, 0x0000, 0x0002, /*1180D*/
  0xFBC2, 0x0020, 0x0002, 0x980E, 0x0000, 0x0000, 0x0002, /*1180E*/
  0xFBC2, 0x0020, 0x0002, 0x980F, 0x0000, 0x0000, 0x0002, /*1180F*/
  0xFBC2, 0x0020, 0x0002, 0x9810, 0x0000, 0x0000, 0x0002, /*11810*/
  0xFBC2, 0x0020, 0x0002, 0x9811, 0x0000, 0x0000, 0x0002, /*11811*/
  0xFBC2, 0x0020, 0x0002, 0x9812, 0x0000, 0x0000, 0x0002, /*11812*/
  0xFBC2, 0x0020, 0x0002, 0x9813, 0x0000, 0x0000, 0x0002, /*11813*/
  0xFBC2, 0x0020, 0x0002, 0x9814, 0x0000, 0x0000, 0x0002, /*11814*/
  0xFBC2, 0x0020, 0x0002, 0x9815, 0x0000, 0x0000, 0x0002, /*11815*/
  0xFBC2, 0x0020, 0x0002, 0x9816, 0x0000, 0x0000, 0x0002, /*11816*/
  0xFBC2, 0x0020, 0x0002, 0x9817, 0x0000, 0x0000, 0x0002, /*11817*/
  0xFBC2, 0x0020, 0x0002, 0x9818, 0x0000, 0x0000, 0x0002, /*11818*/
  0xFBC2, 0x0020, 0x0002, 0x9819, 0x0000, 0x0000, 0x0002, /*11819*/
  0xFBC2, 0x0020, 0x0002, 0x981A, 0x0000, 0x0000, 0x0002, /*1181A*/
  0xFBC2, 0x0020, 0x0002, 0x981B, 0x0000, 0x0000, 0x0002, /*1181B*/
  0xFBC2, 0x0020, 0x0002, 0x981C, 0x0000, 0x0000, 0x0002, /*1181C*/
  0xFBC2, 0x0020, 0x0002, 0x981D, 0x0000, 0x0000, 0x0002, /*1181D*/
  0xFBC2, 0x0020, 0x0002, 0x981E, 0x0000, 0x0000, 0x0002, /*1181E*/
  0xFBC2, 0x0020, 0x0002, 0x981F, 0x0000, 0x0000, 0x0002, /*1181F*/
  0xFBC2, 0x0020, 0x0002, 0x9820, 0x0000, 0x0000, 0x0002, /*11820*/
  0xFBC2, 0x0020, 0x0002, 0x9821, 0x0000, 0x0000, 0x0002, /*11821*/
  0xFBC2, 0x0020, 0x0002, 0x9822, 0x0000, 0x0000, 0x0002, /*11822*/
  0xFBC2, 0x0020, 0x0002, 0x9823, 0x0000, 0x0000, 0x0002, /*11823*/
  0xFBC2, 0x0020, 0x0002, 0x9824, 0x0000, 0x0000, 0x0002, /*11824*/
  0xFBC2, 0x0020, 0x0002, 0x9825, 0x0000, 0x0000, 0x0002, /*11825*/
  0xFBC2, 0x0020, 0x0002, 0x9826, 0x0000, 0x0000, 0x0002, /*11826*/
  0xFBC2, 0x0020, 0x0002, 0x9827, 0x0000, 0x0000, 0x0002, /*11827*/
  0xFBC2, 0x0020, 0x0002, 0x9828, 0x0000, 0x0000, 0x0002, /*11828*/
  0xFBC2, 0x0020, 0x0002, 0x9829, 0x0000, 0x0000, 0x0002, /*11829*/
  0xFBC2, 0x0020, 0x0002, 0x982A, 0x0000, 0x0000, 0x0002, /*1182A*/
  0xFBC2, 0x0020, 0x0002, 0x982B, 0x0000, 0x0000, 0x0002, /*1182B*/
  0xFBC2, 0x0020, 0x0002, 0x982C, 0x0000, 0x0000, 0x0002, /*1182C*/
  0xFBC2, 0x0020, 0x0002, 0x982D, 0x0000, 0x0000, 0x0002, /*1182D*/
  0xFBC2, 0x0020, 0x0002, 0x982E, 0x0000, 0x0000, 0x0002, /*1182E*/
  0xFBC2, 0x0020, 0x0002, 0x982F, 0x0000, 0x0000, 0x0002, /*1182F*/
  0xFBC2, 0x0020, 0x0002, 0x9830, 0x0000, 0x0000, 0x0002, /*11830*/
  0xFBC2, 0x0020, 0x0002, 0x9831, 0x0000, 0x0000, 0x0002, /*11831*/
  0xFBC2, 0x0020, 0x0002, 0x9832, 0x0000, 0x0000, 0x0002, /*11832*/
  0xFBC2, 0x0020, 0x0002, 0x9833, 0x0000, 0x0000, 0x0002, /*11833*/
  0xFBC2, 0x0020, 0x0002, 0x9834, 0x0000, 0x0000, 0x0002, /*11834*/
  0xFBC2, 0x0020, 0x0002, 0x9835, 0x0000, 0x0000, 0x0002, /*11835*/
  0xFBC2, 0x0020, 0x0002, 0x9836, 0x0000, 0x0000, 0x0002, /*11836*/
  0xFBC2, 0x0020, 0x0002, 0x9837, 0x0000, 0x0000, 0x0002, /*11837*/
  0xFBC2, 0x0020, 0x0002, 0x9838, 0x0000, 0x0000, 0x0002, /*11838*/
  0xFBC2, 0x0020, 0x0002, 0x9839, 0x0000, 0x0000, 0x0002, /*11839*/
  0xFBC2, 0x0020, 0x0002, 0x983A, 0x0000, 0x0000, 0x0002, /*1183A*/
  0xFBC2, 0x0020, 0x0002, 0x983B, 0x0000, 0x0000, 0x0002, /*1183B*/
  0xFBC2, 0x0020, 0x0002, 0x983C, 0x0000, 0x0000, 0x0002, /*1183C*/
  0xFBC2, 0x0020, 0x0002, 0x983D, 0x0000, 0x0000, 0x0002, /*1183D*/
  0xFBC2, 0x0020, 0x0002, 0x983E, 0x0000, 0x0000, 0x0002, /*1183E*/
  0xFBC2, 0x0020, 0x0002, 0x983F, 0x0000, 0x0000, 0x0002, /*1183F*/
  0xFBC2, 0x0020, 0x0002, 0x9840, 0x0000, 0x0000, 0x0002, /*11840*/
  0xFBC2, 0x0020, 0x0002, 0x9841, 0x0000, 0x0000, 0x0002, /*11841*/
  0xFBC2, 0x0020, 0x0002, 0x9842, 0x0000, 0x0000, 0x0002, /*11842*/
  0xFBC2, 0x0020, 0x0002, 0x9843, 0x0000, 0x0000, 0x0002, /*11843*/
  0xFBC2, 0x0020, 0x0002, 0x9844, 0x0000, 0x0000, 0x0002, /*11844*/
  0xFBC2, 0x0020, 0x0002, 0x9845, 0x0000, 0x0000, 0x0002, /*11845*/
  0xFBC2, 0x0020, 0x0002, 0x9846, 0x0000, 0x0000, 0x0002, /*11846*/
  0xFBC2, 0x0020, 0x0002, 0x9847, 0x0000, 0x0000, 0x0002, /*11847*/
  0xFBC2, 0x0020, 0x0002, 0x9848, 0x0000, 0x0000, 0x0002, /*11848*/
  0xFBC2, 0x0020, 0x0002, 0x9849, 0x0000, 0x0000, 0x0002, /*11849*/
  0xFBC2, 0x0020, 0x0002, 0x984A, 0x0000, 0x0000, 0x0002, /*1184A*/
  0xFBC2, 0x0020, 0x0002, 0x984B, 0x0000, 0x0000, 0x0002, /*1184B*/
  0xFBC2, 0x0020, 0x0002, 0x984C, 0x0000, 0x0000, 0x0002, /*1184C*/
  0xFBC2, 0x0020, 0x0002, 0x984D, 0x0000, 0x0000, 0x0002, /*1184D*/
  0xFBC2, 0x0020, 0x0002, 0x984E, 0x0000, 0x0000, 0x0002, /*1184E*/
  0xFBC2, 0x0020, 0x0002, 0x984F, 0x0000, 0x0000, 0x0002, /*1184F*/
  0xFBC2, 0x0020, 0x0002, 0x9850, 0x0000, 0x0000, 0x0002, /*11850*/
  0xFBC2, 0x0020, 0x0002, 0x9851, 0x0000, 0x0000, 0x0002, /*11851*/
  0xFBC2, 0x0020, 0x0002, 0x9852, 0x0000, 0x0000, 0x0002, /*11852*/
  0xFBC2, 0x0020, 0x0002, 0x9853, 0x0000, 0x0000, 0x0002, /*11853*/
  0xFBC2, 0x0020, 0x0002, 0x9854, 0x0000, 0x0000, 0x0002, /*11854*/
  0xFBC2, 0x0020, 0x0002, 0x9855, 0x0000, 0x0000, 0x0002, /*11855*/
  0xFBC2, 0x0020, 0x0002, 0x9856, 0x0000, 0x0000, 0x0002, /*11856*/
  0xFBC2, 0x0020, 0x0002, 0x9857, 0x0000, 0x0000, 0x0002, /*11857*/
  0xFBC2, 0x0020, 0x0002, 0x9858, 0x0000, 0x0000, 0x0002, /*11858*/
  0xFBC2, 0x0020, 0x0002, 0x9859, 0x0000, 0x0000, 0x0002, /*11859*/
  0xFBC2, 0x0020, 0x0002, 0x985A, 0x0000, 0x0000, 0x0002, /*1185A*/
  0xFBC2, 0x0020, 0x0002, 0x985B, 0x0000, 0x0000, 0x0002, /*1185B*/
  0xFBC2, 0x0020, 0x0002, 0x985C, 0x0000, 0x0000, 0x0002, /*1185C*/
  0xFBC2, 0x0020, 0x0002, 0x985D, 0x0000, 0x0000, 0x0002, /*1185D*/
  0xFBC2, 0x0020, 0x0002, 0x985E, 0x0000, 0x0000, 0x0002, /*1185E*/
  0xFBC2, 0x0020, 0x0002, 0x985F, 0x0000, 0x0000, 0x0002, /*1185F*/
  0xFBC2, 0x0020, 0x0002, 0x9860, 0x0000, 0x0000, 0x0002, /*11860*/
  0xFBC2, 0x0020, 0x0002, 0x9861, 0x0000, 0x0000, 0x0002, /*11861*/
  0xFBC2, 0x0020, 0x0002, 0x9862, 0x0000, 0x0000, 0x0002, /*11862*/
  0xFBC2, 0x0020, 0x0002, 0x9863, 0x0000, 0x0000, 0x0002, /*11863*/
  0xFBC2, 0x0020, 0x0002, 0x9864, 0x0000, 0x0000, 0x0002, /*11864*/
  0xFBC2, 0x0020, 0x0002, 0x9865, 0x0000, 0x0000, 0x0002, /*11865*/
  0xFBC2, 0x0020, 0x0002, 0x9866, 0x0000, 0x0000, 0x0002, /*11866*/
  0xFBC2, 0x0020, 0x0002, 0x9867, 0x0000, 0x0000, 0x0002, /*11867*/
  0xFBC2, 0x0020, 0x0002, 0x9868, 0x0000, 0x0000, 0x0002, /*11868*/
  0xFBC2, 0x0020, 0x0002, 0x9869, 0x0000, 0x0000, 0x0002, /*11869*/
  0xFBC2, 0x0020, 0x0002, 0x986A, 0x0000, 0x0000, 0x0002, /*1186A*/
  0xFBC2, 0x0020, 0x0002, 0x986B, 0x0000, 0x0000, 0x0002, /*1186B*/
  0xFBC2, 0x0020, 0x0002, 0x986C, 0x0000, 0x0000, 0x0002, /*1186C*/
  0xFBC2, 0x0020, 0x0002, 0x986D, 0x0000, 0x0000, 0x0002, /*1186D*/
  0xFBC2, 0x0020, 0x0002, 0x986E, 0x0000, 0x0000, 0x0002, /*1186E*/
  0xFBC2, 0x0020, 0x0002, 0x986F, 0x0000, 0x0000, 0x0002, /*1186F*/
  0xFBC2, 0x0020, 0x0002, 0x9870, 0x0000, 0x0000, 0x0002, /*11870*/
  0xFBC2, 0x0020, 0x0002, 0x9871, 0x0000, 0x0000, 0x0002, /*11871*/
  0xFBC2, 0x0020, 0x0002, 0x9872, 0x0000, 0x0000, 0x0002, /*11872*/
  0xFBC2, 0x0020, 0x0002, 0x9873, 0x0000, 0x0000, 0x0002, /*11873*/
  0xFBC2, 0x0020, 0x0002, 0x9874, 0x0000, 0x0000, 0x0002, /*11874*/
  0xFBC2, 0x0020, 0x0002, 0x9875, 0x0000, 0x0000, 0x0002, /*11875*/
  0xFBC2, 0x0020, 0x0002, 0x9876, 0x0000, 0x0000, 0x0002, /*11876*/
  0xFBC2, 0x0020, 0x0002, 0x9877, 0x0000, 0x0000, 0x0002, /*11877*/
  0xFBC2, 0x0020, 0x0002, 0x9878, 0x0000, 0x0000, 0x0002, /*11878*/
  0xFBC2, 0x0020, 0x0002, 0x9879, 0x0000, 0x0000, 0x0002, /*11879*/
  0xFBC2, 0x0020, 0x0002, 0x987A, 0x0000, 0x0000, 0x0002, /*1187A*/
  0xFBC2, 0x0020, 0x0002, 0x987B, 0x0000, 0x0000, 0x0002, /*1187B*/
  0xFBC2, 0x0020, 0x0002, 0x987C, 0x0000, 0x0000, 0x0002, /*1187C*/
  0xFBC2, 0x0020, 0x0002, 0x987D, 0x0000, 0x0000, 0x0002, /*1187D*/
  0xFBC2, 0x0020, 0x0002, 0x987E, 0x0000, 0x0000, 0x0002, /*1187E*/
  0xFBC2, 0x0020, 0x0002, 0x987F, 0x0000, 0x0000, 0x0002, /*1187F*/
  0xFBC2, 0x0020, 0x0002, 0x9880, 0x0000, 0x0000, 0x0002, /*11880*/
  0xFBC2, 0x0020, 0x0002, 0x9881, 0x0000, 0x0000, 0x0002, /*11881*/
  0xFBC2, 0x0020, 0x0002, 0x9882, 0x0000, 0x0000, 0x0002, /*11882*/
  0xFBC2, 0x0020, 0x0002, 0x9883, 0x0000, 0x0000, 0x0002, /*11883*/
  0xFBC2, 0x0020, 0x0002, 0x9884, 0x0000, 0x0000, 0x0002, /*11884*/
  0xFBC2, 0x0020, 0x0002, 0x9885, 0x0000, 0x0000, 0x0002, /*11885*/
  0xFBC2, 0x0020, 0x0002, 0x9886, 0x0000, 0x0000, 0x0002, /*11886*/
  0xFBC2, 0x0020, 0x0002, 0x9887, 0x0000, 0x0000, 0x0002, /*11887*/
  0xFBC2, 0x0020, 0x0002, 0x9888, 0x0000, 0x0000, 0x0002, /*11888*/
  0xFBC2, 0x0020, 0x0002, 0x9889, 0x0000, 0x0000, 0x0002, /*11889*/
  0xFBC2, 0x0020, 0x0002, 0x988A, 0x0000, 0x0000, 0x0002, /*1188A*/
  0xFBC2, 0x0020, 0x0002, 0x988B, 0x0000, 0x0000, 0x0002, /*1188B*/
  0xFBC2, 0x0020, 0x0002, 0x988C, 0x0000, 0x0000, 0x0002, /*1188C*/
  0xFBC2, 0x0020, 0x0002, 0x988D, 0x0000, 0x0000, 0x0002, /*1188D*/
  0xFBC2, 0x0020, 0x0002, 0x988E, 0x0000, 0x0000, 0x0002, /*1188E*/
  0xFBC2, 0x0020, 0x0002, 0x988F, 0x0000, 0x0000, 0x0002, /*1188F*/
  0xFBC2, 0x0020, 0x0002, 0x9890, 0x0000, 0x0000, 0x0002, /*11890*/
  0xFBC2, 0x0020, 0x0002, 0x9891, 0x0000, 0x0000, 0x0002, /*11891*/
  0xFBC2, 0x0020, 0x0002, 0x9892, 0x0000, 0x0000, 0x0002, /*11892*/
  0xFBC2, 0x0020, 0x0002, 0x9893, 0x0000, 0x0000, 0x0002, /*11893*/
  0xFBC2, 0x0020, 0x0002, 0x9894, 0x0000, 0x0000, 0x0002, /*11894*/
  0xFBC2, 0x0020, 0x0002, 0x9895, 0x0000, 0x0000, 0x0002, /*11895*/
  0xFBC2, 0x0020, 0x0002, 0x9896, 0x0000, 0x0000, 0x0002, /*11896*/
  0xFBC2, 0x0020, 0x0002, 0x9897, 0x0000, 0x0000, 0x0002, /*11897*/
  0xFBC2, 0x0020, 0x0002, 0x9898, 0x0000, 0x0000, 0x0002, /*11898*/
  0xFBC2, 0x0020, 0x0002, 0x9899, 0x0000, 0x0000, 0x0002, /*11899*/
  0xFBC2, 0x0020, 0x0002, 0x989A, 0x0000, 0x0000, 0x0002, /*1189A*/
  0xFBC2, 0x0020, 0x0002, 0x989B, 0x0000, 0x0000, 0x0002, /*1189B*/
  0xFBC2, 0x0020, 0x0002, 0x989C, 0x0000, 0x0000, 0x0002, /*1189C*/
  0xFBC2, 0x0020, 0x0002, 0x989D, 0x0000, 0x0000, 0x0002, /*1189D*/
  0xFBC2, 0x0020, 0x0002, 0x989E, 0x0000, 0x0000, 0x0002, /*1189E*/
  0xFBC2, 0x0020, 0x0002, 0x989F, 0x0000, 0x0000, 0x0002, /*1189F*/
  0x42FC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118A0*/
  0x42FD, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118A1*/
  0x42FE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118A2*/
  0x42FF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118A3*/
  0x4300, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118A4*/
  0x4301, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118A5*/
  0x4302, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118A6*/
  0x4303, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118A7*/
  0x4304, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118A8*/
  0x4305, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118A9*/
  0x4306, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118AA*/
  0x4307, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118AB*/
  0x4308, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118AC*/
  0x4309, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118AD*/
  0x430A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118AE*/
  0x430B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118AF*/
  0x430C, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118B0*/
  0x430D, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118B1*/
  0x430E, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118B2*/
  0x430F, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118B3*/
  0x4310, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118B4*/
  0x4311, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118B5*/
  0x4312, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118B6*/
  0x4313, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118B7*/
  0x4314, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118B8*/
  0x4315, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118B9*/
  0x4316, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118BA*/
  0x4317, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118BB*/
  0x4318, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118BC*/
  0x4319, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118BD*/
  0x431A, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118BE*/
  0x431B, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*118BF*/
  0x42FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118C0*/
  0x42FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118C1*/
  0x42FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118C2*/
  0x42FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118C3*/
  0x4300, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118C4*/
  0x4301, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118C5*/
  0x4302, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118C6*/
  0x4303, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118C7*/
  0x4304, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118C8*/
  0x4305, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118C9*/
  0x4306, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118CA*/
  0x4307, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118CB*/
  0x4308, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118CC*/
  0x4309, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118CD*/
  0x430A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118CE*/
  0x430B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118CF*/
  0x430C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118D0*/
  0x430D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118D1*/
  0x430E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118D2*/
  0x430F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118D3*/
  0x4310, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118D4*/
  0x4311, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118D5*/
  0x4312, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118D6*/
  0x4313, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118D7*/
  0x4314, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118D8*/
  0x4315, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118D9*/
  0x4316, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118DA*/
  0x4317, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118DB*/
  0x4318, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118DC*/
  0x4319, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118DD*/
  0x431A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118DE*/
  0x431B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118DF*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118E0*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118E1*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118E2*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118E3*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118E4*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118E5*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118E6*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118E7*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118E8*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118E9*/
  0x1B97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118EA*/
  0x1B98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118EB*/
  0x1B99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118EC*/
  0x1B9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118ED*/
  0x1B9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118EE*/
  0x1B9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118EF*/
  0x1B9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118F0*/
  0x1B9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118F1*/
  0x1B9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*118F2*/
  0xFBC2, 0x0020, 0x0002, 0x98F3, 0x0000, 0x0000, 0x0002, /*118F3*/
  0xFBC2, 0x0020, 0x0002, 0x98F4, 0x0000, 0x0000, 0x0002, /*118F4*/
  0xFBC2, 0x0020, 0x0002, 0x98F5, 0x0000, 0x0000, 0x0002, /*118F5*/
  0xFBC2, 0x0020, 0x0002, 0x98F6, 0x0000, 0x0000, 0x0002, /*118F6*/
  0xFBC2, 0x0020, 0x0002, 0x98F7, 0x0000, 0x0000, 0x0002, /*118F7*/
  0xFBC2, 0x0020, 0x0002, 0x98F8, 0x0000, 0x0000, 0x0002, /*118F8*/
  0xFBC2, 0x0020, 0x0002, 0x98F9, 0x0000, 0x0000, 0x0002, /*118F9*/
  0xFBC2, 0x0020, 0x0002, 0x98FA, 0x0000, 0x0000, 0x0002, /*118FA*/
  0xFBC2, 0x0020, 0x0002, 0x98FB, 0x0000, 0x0000, 0x0002, /*118FB*/
  0xFBC2, 0x0020, 0x0002, 0x98FC, 0x0000, 0x0000, 0x0002, /*118FC*/
  0xFBC2, 0x0020, 0x0002, 0x98FD, 0x0000, 0x0000, 0x0002, /*118FD*/
  0xFBC2, 0x0020, 0x0002, 0x98FE, 0x0000, 0x0000, 0x0002, /*118FE*/
  0x42FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*118FF*/
};

uint16 uca900_p11A[]= {
  0xFBC2, 0x0020, 0x0002, 0x9A00, 0x0000, 0x0000, 0x0002, /*11A00*/
  0xFBC2, 0x0020, 0x0002, 0x9A01, 0x0000, 0x0000, 0x0002, /*11A01*/
  0xFBC2, 0x0020, 0x0002, 0x9A02, 0x0000, 0x0000, 0x0002, /*11A02*/
  0xFBC2, 0x0020, 0x0002, 0x9A03, 0x0000, 0x0000, 0x0002, /*11A03*/
  0xFBC2, 0x0020, 0x0002, 0x9A04, 0x0000, 0x0000, 0x0002, /*11A04*/
  0xFBC2, 0x0020, 0x0002, 0x9A05, 0x0000, 0x0000, 0x0002, /*11A05*/
  0xFBC2, 0x0020, 0x0002, 0x9A06, 0x0000, 0x0000, 0x0002, /*11A06*/
  0xFBC2, 0x0020, 0x0002, 0x9A07, 0x0000, 0x0000, 0x0002, /*11A07*/
  0xFBC2, 0x0020, 0x0002, 0x9A08, 0x0000, 0x0000, 0x0002, /*11A08*/
  0xFBC2, 0x0020, 0x0002, 0x9A09, 0x0000, 0x0000, 0x0002, /*11A09*/
  0xFBC2, 0x0020, 0x0002, 0x9A0A, 0x0000, 0x0000, 0x0002, /*11A0A*/
  0xFBC2, 0x0020, 0x0002, 0x9A0B, 0x0000, 0x0000, 0x0002, /*11A0B*/
  0xFBC2, 0x0020, 0x0002, 0x9A0C, 0x0000, 0x0000, 0x0002, /*11A0C*/
  0xFBC2, 0x0020, 0x0002, 0x9A0D, 0x0000, 0x0000, 0x0002, /*11A0D*/
  0xFBC2, 0x0020, 0x0002, 0x9A0E, 0x0000, 0x0000, 0x0002, /*11A0E*/
  0xFBC2, 0x0020, 0x0002, 0x9A0F, 0x0000, 0x0000, 0x0002, /*11A0F*/
  0xFBC2, 0x0020, 0x0002, 0x9A10, 0x0000, 0x0000, 0x0002, /*11A10*/
  0xFBC2, 0x0020, 0x0002, 0x9A11, 0x0000, 0x0000, 0x0002, /*11A11*/
  0xFBC2, 0x0020, 0x0002, 0x9A12, 0x0000, 0x0000, 0x0002, /*11A12*/
  0xFBC2, 0x0020, 0x0002, 0x9A13, 0x0000, 0x0000, 0x0002, /*11A13*/
  0xFBC2, 0x0020, 0x0002, 0x9A14, 0x0000, 0x0000, 0x0002, /*11A14*/
  0xFBC2, 0x0020, 0x0002, 0x9A15, 0x0000, 0x0000, 0x0002, /*11A15*/
  0xFBC2, 0x0020, 0x0002, 0x9A16, 0x0000, 0x0000, 0x0002, /*11A16*/
  0xFBC2, 0x0020, 0x0002, 0x9A17, 0x0000, 0x0000, 0x0002, /*11A17*/
  0xFBC2, 0x0020, 0x0002, 0x9A18, 0x0000, 0x0000, 0x0002, /*11A18*/
  0xFBC2, 0x0020, 0x0002, 0x9A19, 0x0000, 0x0000, 0x0002, /*11A19*/
  0xFBC2, 0x0020, 0x0002, 0x9A1A, 0x0000, 0x0000, 0x0002, /*11A1A*/
  0xFBC2, 0x0020, 0x0002, 0x9A1B, 0x0000, 0x0000, 0x0002, /*11A1B*/
  0xFBC2, 0x0020, 0x0002, 0x9A1C, 0x0000, 0x0000, 0x0002, /*11A1C*/
  0xFBC2, 0x0020, 0x0002, 0x9A1D, 0x0000, 0x0000, 0x0002, /*11A1D*/
  0xFBC2, 0x0020, 0x0002, 0x9A1E, 0x0000, 0x0000, 0x0002, /*11A1E*/
  0xFBC2, 0x0020, 0x0002, 0x9A1F, 0x0000, 0x0000, 0x0002, /*11A1F*/
  0xFBC2, 0x0020, 0x0002, 0x9A20, 0x0000, 0x0000, 0x0002, /*11A20*/
  0xFBC2, 0x0020, 0x0002, 0x9A21, 0x0000, 0x0000, 0x0002, /*11A21*/
  0xFBC2, 0x0020, 0x0002, 0x9A22, 0x0000, 0x0000, 0x0002, /*11A22*/
  0xFBC2, 0x0020, 0x0002, 0x9A23, 0x0000, 0x0000, 0x0002, /*11A23*/
  0xFBC2, 0x0020, 0x0002, 0x9A24, 0x0000, 0x0000, 0x0002, /*11A24*/
  0xFBC2, 0x0020, 0x0002, 0x9A25, 0x0000, 0x0000, 0x0002, /*11A25*/
  0xFBC2, 0x0020, 0x0002, 0x9A26, 0x0000, 0x0000, 0x0002, /*11A26*/
  0xFBC2, 0x0020, 0x0002, 0x9A27, 0x0000, 0x0000, 0x0002, /*11A27*/
  0xFBC2, 0x0020, 0x0002, 0x9A28, 0x0000, 0x0000, 0x0002, /*11A28*/
  0xFBC2, 0x0020, 0x0002, 0x9A29, 0x0000, 0x0000, 0x0002, /*11A29*/
  0xFBC2, 0x0020, 0x0002, 0x9A2A, 0x0000, 0x0000, 0x0002, /*11A2A*/
  0xFBC2, 0x0020, 0x0002, 0x9A2B, 0x0000, 0x0000, 0x0002, /*11A2B*/
  0xFBC2, 0x0020, 0x0002, 0x9A2C, 0x0000, 0x0000, 0x0002, /*11A2C*/
  0xFBC2, 0x0020, 0x0002, 0x9A2D, 0x0000, 0x0000, 0x0002, /*11A2D*/
  0xFBC2, 0x0020, 0x0002, 0x9A2E, 0x0000, 0x0000, 0x0002, /*11A2E*/
  0xFBC2, 0x0020, 0x0002, 0x9A2F, 0x0000, 0x0000, 0x0002, /*11A2F*/
  0xFBC2, 0x0020, 0x0002, 0x9A30, 0x0000, 0x0000, 0x0002, /*11A30*/
  0xFBC2, 0x0020, 0x0002, 0x9A31, 0x0000, 0x0000, 0x0002, /*11A31*/
  0xFBC2, 0x0020, 0x0002, 0x9A32, 0x0000, 0x0000, 0x0002, /*11A32*/
  0xFBC2, 0x0020, 0x0002, 0x9A33, 0x0000, 0x0000, 0x0002, /*11A33*/
  0xFBC2, 0x0020, 0x0002, 0x9A34, 0x0000, 0x0000, 0x0002, /*11A34*/
  0xFBC2, 0x0020, 0x0002, 0x9A35, 0x0000, 0x0000, 0x0002, /*11A35*/
  0xFBC2, 0x0020, 0x0002, 0x9A36, 0x0000, 0x0000, 0x0002, /*11A36*/
  0xFBC2, 0x0020, 0x0002, 0x9A37, 0x0000, 0x0000, 0x0002, /*11A37*/
  0xFBC2, 0x0020, 0x0002, 0x9A38, 0x0000, 0x0000, 0x0002, /*11A38*/
  0xFBC2, 0x0020, 0x0002, 0x9A39, 0x0000, 0x0000, 0x0002, /*11A39*/
  0xFBC2, 0x0020, 0x0002, 0x9A3A, 0x0000, 0x0000, 0x0002, /*11A3A*/
  0xFBC2, 0x0020, 0x0002, 0x9A3B, 0x0000, 0x0000, 0x0002, /*11A3B*/
  0xFBC2, 0x0020, 0x0002, 0x9A3C, 0x0000, 0x0000, 0x0002, /*11A3C*/
  0xFBC2, 0x0020, 0x0002, 0x9A3D, 0x0000, 0x0000, 0x0002, /*11A3D*/
  0xFBC2, 0x0020, 0x0002, 0x9A3E, 0x0000, 0x0000, 0x0002, /*11A3E*/
  0xFBC2, 0x0020, 0x0002, 0x9A3F, 0x0000, 0x0000, 0x0002, /*11A3F*/
  0xFBC2, 0x0020, 0x0002, 0x9A40, 0x0000, 0x0000, 0x0002, /*11A40*/
  0xFBC2, 0x0020, 0x0002, 0x9A41, 0x0000, 0x0000, 0x0002, /*11A41*/
  0xFBC2, 0x0020, 0x0002, 0x9A42, 0x0000, 0x0000, 0x0002, /*11A42*/
  0xFBC2, 0x0020, 0x0002, 0x9A43, 0x0000, 0x0000, 0x0002, /*11A43*/
  0xFBC2, 0x0020, 0x0002, 0x9A44, 0x0000, 0x0000, 0x0002, /*11A44*/
  0xFBC2, 0x0020, 0x0002, 0x9A45, 0x0000, 0x0000, 0x0002, /*11A45*/
  0xFBC2, 0x0020, 0x0002, 0x9A46, 0x0000, 0x0000, 0x0002, /*11A46*/
  0xFBC2, 0x0020, 0x0002, 0x9A47, 0x0000, 0x0000, 0x0002, /*11A47*/
  0xFBC2, 0x0020, 0x0002, 0x9A48, 0x0000, 0x0000, 0x0002, /*11A48*/
  0xFBC2, 0x0020, 0x0002, 0x9A49, 0x0000, 0x0000, 0x0002, /*11A49*/
  0xFBC2, 0x0020, 0x0002, 0x9A4A, 0x0000, 0x0000, 0x0002, /*11A4A*/
  0xFBC2, 0x0020, 0x0002, 0x9A4B, 0x0000, 0x0000, 0x0002, /*11A4B*/
  0xFBC2, 0x0020, 0x0002, 0x9A4C, 0x0000, 0x0000, 0x0002, /*11A4C*/
  0xFBC2, 0x0020, 0x0002, 0x9A4D, 0x0000, 0x0000, 0x0002, /*11A4D*/
  0xFBC2, 0x0020, 0x0002, 0x9A4E, 0x0000, 0x0000, 0x0002, /*11A4E*/
  0xFBC2, 0x0020, 0x0002, 0x9A4F, 0x0000, 0x0000, 0x0002, /*11A4F*/
  0xFBC2, 0x0020, 0x0002, 0x9A50, 0x0000, 0x0000, 0x0002, /*11A50*/
  0xFBC2, 0x0020, 0x0002, 0x9A51, 0x0000, 0x0000, 0x0002, /*11A51*/
  0xFBC2, 0x0020, 0x0002, 0x9A52, 0x0000, 0x0000, 0x0002, /*11A52*/
  0xFBC2, 0x0020, 0x0002, 0x9A53, 0x0000, 0x0000, 0x0002, /*11A53*/
  0xFBC2, 0x0020, 0x0002, 0x9A54, 0x0000, 0x0000, 0x0002, /*11A54*/
  0xFBC2, 0x0020, 0x0002, 0x9A55, 0x0000, 0x0000, 0x0002, /*11A55*/
  0xFBC2, 0x0020, 0x0002, 0x9A56, 0x0000, 0x0000, 0x0002, /*11A56*/
  0xFBC2, 0x0020, 0x0002, 0x9A57, 0x0000, 0x0000, 0x0002, /*11A57*/
  0xFBC2, 0x0020, 0x0002, 0x9A58, 0x0000, 0x0000, 0x0002, /*11A58*/
  0xFBC2, 0x0020, 0x0002, 0x9A59, 0x0000, 0x0000, 0x0002, /*11A59*/
  0xFBC2, 0x0020, 0x0002, 0x9A5A, 0x0000, 0x0000, 0x0002, /*11A5A*/
  0xFBC2, 0x0020, 0x0002, 0x9A5B, 0x0000, 0x0000, 0x0002, /*11A5B*/
  0xFBC2, 0x0020, 0x0002, 0x9A5C, 0x0000, 0x0000, 0x0002, /*11A5C*/
  0xFBC2, 0x0020, 0x0002, 0x9A5D, 0x0000, 0x0000, 0x0002, /*11A5D*/
  0xFBC2, 0x0020, 0x0002, 0x9A5E, 0x0000, 0x0000, 0x0002, /*11A5E*/
  0xFBC2, 0x0020, 0x0002, 0x9A5F, 0x0000, 0x0000, 0x0002, /*11A5F*/
  0xFBC2, 0x0020, 0x0002, 0x9A60, 0x0000, 0x0000, 0x0002, /*11A60*/
  0xFBC2, 0x0020, 0x0002, 0x9A61, 0x0000, 0x0000, 0x0002, /*11A61*/
  0xFBC2, 0x0020, 0x0002, 0x9A62, 0x0000, 0x0000, 0x0002, /*11A62*/
  0xFBC2, 0x0020, 0x0002, 0x9A63, 0x0000, 0x0000, 0x0002, /*11A63*/
  0xFBC2, 0x0020, 0x0002, 0x9A64, 0x0000, 0x0000, 0x0002, /*11A64*/
  0xFBC2, 0x0020, 0x0002, 0x9A65, 0x0000, 0x0000, 0x0002, /*11A65*/
  0xFBC2, 0x0020, 0x0002, 0x9A66, 0x0000, 0x0000, 0x0002, /*11A66*/
  0xFBC2, 0x0020, 0x0002, 0x9A67, 0x0000, 0x0000, 0x0002, /*11A67*/
  0xFBC2, 0x0020, 0x0002, 0x9A68, 0x0000, 0x0000, 0x0002, /*11A68*/
  0xFBC2, 0x0020, 0x0002, 0x9A69, 0x0000, 0x0000, 0x0002, /*11A69*/
  0xFBC2, 0x0020, 0x0002, 0x9A6A, 0x0000, 0x0000, 0x0002, /*11A6A*/
  0xFBC2, 0x0020, 0x0002, 0x9A6B, 0x0000, 0x0000, 0x0002, /*11A6B*/
  0xFBC2, 0x0020, 0x0002, 0x9A6C, 0x0000, 0x0000, 0x0002, /*11A6C*/
  0xFBC2, 0x0020, 0x0002, 0x9A6D, 0x0000, 0x0000, 0x0002, /*11A6D*/
  0xFBC2, 0x0020, 0x0002, 0x9A6E, 0x0000, 0x0000, 0x0002, /*11A6E*/
  0xFBC2, 0x0020, 0x0002, 0x9A6F, 0x0000, 0x0000, 0x0002, /*11A6F*/
  0xFBC2, 0x0020, 0x0002, 0x9A70, 0x0000, 0x0000, 0x0002, /*11A70*/
  0xFBC2, 0x0020, 0x0002, 0x9A71, 0x0000, 0x0000, 0x0002, /*11A71*/
  0xFBC2, 0x0020, 0x0002, 0x9A72, 0x0000, 0x0000, 0x0002, /*11A72*/
  0xFBC2, 0x0020, 0x0002, 0x9A73, 0x0000, 0x0000, 0x0002, /*11A73*/
  0xFBC2, 0x0020, 0x0002, 0x9A74, 0x0000, 0x0000, 0x0002, /*11A74*/
  0xFBC2, 0x0020, 0x0002, 0x9A75, 0x0000, 0x0000, 0x0002, /*11A75*/
  0xFBC2, 0x0020, 0x0002, 0x9A76, 0x0000, 0x0000, 0x0002, /*11A76*/
  0xFBC2, 0x0020, 0x0002, 0x9A77, 0x0000, 0x0000, 0x0002, /*11A77*/
  0xFBC2, 0x0020, 0x0002, 0x9A78, 0x0000, 0x0000, 0x0002, /*11A78*/
  0xFBC2, 0x0020, 0x0002, 0x9A79, 0x0000, 0x0000, 0x0002, /*11A79*/
  0xFBC2, 0x0020, 0x0002, 0x9A7A, 0x0000, 0x0000, 0x0002, /*11A7A*/
  0xFBC2, 0x0020, 0x0002, 0x9A7B, 0x0000, 0x0000, 0x0002, /*11A7B*/
  0xFBC2, 0x0020, 0x0002, 0x9A7C, 0x0000, 0x0000, 0x0002, /*11A7C*/
  0xFBC2, 0x0020, 0x0002, 0x9A7D, 0x0000, 0x0000, 0x0002, /*11A7D*/
  0xFBC2, 0x0020, 0x0002, 0x9A7E, 0x0000, 0x0000, 0x0002, /*11A7E*/
  0xFBC2, 0x0020, 0x0002, 0x9A7F, 0x0000, 0x0000, 0x0002, /*11A7F*/
  0xFBC2, 0x0020, 0x0002, 0x9A80, 0x0000, 0x0000, 0x0002, /*11A80*/
  0xFBC2, 0x0020, 0x0002, 0x9A81, 0x0000, 0x0000, 0x0002, /*11A81*/
  0xFBC2, 0x0020, 0x0002, 0x9A82, 0x0000, 0x0000, 0x0002, /*11A82*/
  0xFBC2, 0x0020, 0x0002, 0x9A83, 0x0000, 0x0000, 0x0002, /*11A83*/
  0xFBC2, 0x0020, 0x0002, 0x9A84, 0x0000, 0x0000, 0x0002, /*11A84*/
  0xFBC2, 0x0020, 0x0002, 0x9A85, 0x0000, 0x0000, 0x0002, /*11A85*/
  0xFBC2, 0x0020, 0x0002, 0x9A86, 0x0000, 0x0000, 0x0002, /*11A86*/
  0xFBC2, 0x0020, 0x0002, 0x9A87, 0x0000, 0x0000, 0x0002, /*11A87*/
  0xFBC2, 0x0020, 0x0002, 0x9A88, 0x0000, 0x0000, 0x0002, /*11A88*/
  0xFBC2, 0x0020, 0x0002, 0x9A89, 0x0000, 0x0000, 0x0002, /*11A89*/
  0xFBC2, 0x0020, 0x0002, 0x9A8A, 0x0000, 0x0000, 0x0002, /*11A8A*/
  0xFBC2, 0x0020, 0x0002, 0x9A8B, 0x0000, 0x0000, 0x0002, /*11A8B*/
  0xFBC2, 0x0020, 0x0002, 0x9A8C, 0x0000, 0x0000, 0x0002, /*11A8C*/
  0xFBC2, 0x0020, 0x0002, 0x9A8D, 0x0000, 0x0000, 0x0002, /*11A8D*/
  0xFBC2, 0x0020, 0x0002, 0x9A8E, 0x0000, 0x0000, 0x0002, /*11A8E*/
  0xFBC2, 0x0020, 0x0002, 0x9A8F, 0x0000, 0x0000, 0x0002, /*11A8F*/
  0xFBC2, 0x0020, 0x0002, 0x9A90, 0x0000, 0x0000, 0x0002, /*11A90*/
  0xFBC2, 0x0020, 0x0002, 0x9A91, 0x0000, 0x0000, 0x0002, /*11A91*/
  0xFBC2, 0x0020, 0x0002, 0x9A92, 0x0000, 0x0000, 0x0002, /*11A92*/
  0xFBC2, 0x0020, 0x0002, 0x9A93, 0x0000, 0x0000, 0x0002, /*11A93*/
  0xFBC2, 0x0020, 0x0002, 0x9A94, 0x0000, 0x0000, 0x0002, /*11A94*/
  0xFBC2, 0x0020, 0x0002, 0x9A95, 0x0000, 0x0000, 0x0002, /*11A95*/
  0xFBC2, 0x0020, 0x0002, 0x9A96, 0x0000, 0x0000, 0x0002, /*11A96*/
  0xFBC2, 0x0020, 0x0002, 0x9A97, 0x0000, 0x0000, 0x0002, /*11A97*/
  0xFBC2, 0x0020, 0x0002, 0x9A98, 0x0000, 0x0000, 0x0002, /*11A98*/
  0xFBC2, 0x0020, 0x0002, 0x9A99, 0x0000, 0x0000, 0x0002, /*11A99*/
  0xFBC2, 0x0020, 0x0002, 0x9A9A, 0x0000, 0x0000, 0x0002, /*11A9A*/
  0xFBC2, 0x0020, 0x0002, 0x9A9B, 0x0000, 0x0000, 0x0002, /*11A9B*/
  0xFBC2, 0x0020, 0x0002, 0x9A9C, 0x0000, 0x0000, 0x0002, /*11A9C*/
  0xFBC2, 0x0020, 0x0002, 0x9A9D, 0x0000, 0x0000, 0x0002, /*11A9D*/
  0xFBC2, 0x0020, 0x0002, 0x9A9E, 0x0000, 0x0000, 0x0002, /*11A9E*/
  0xFBC2, 0x0020, 0x0002, 0x9A9F, 0x0000, 0x0000, 0x0002, /*11A9F*/
  0xFBC2, 0x0020, 0x0002, 0x9AA0, 0x0000, 0x0000, 0x0002, /*11AA0*/
  0xFBC2, 0x0020, 0x0002, 0x9AA1, 0x0000, 0x0000, 0x0002, /*11AA1*/
  0xFBC2, 0x0020, 0x0002, 0x9AA2, 0x0000, 0x0000, 0x0002, /*11AA2*/
  0xFBC2, 0x0020, 0x0002, 0x9AA3, 0x0000, 0x0000, 0x0002, /*11AA3*/
  0xFBC2, 0x0020, 0x0002, 0x9AA4, 0x0000, 0x0000, 0x0002, /*11AA4*/
  0xFBC2, 0x0020, 0x0002, 0x9AA5, 0x0000, 0x0000, 0x0002, /*11AA5*/
  0xFBC2, 0x0020, 0x0002, 0x9AA6, 0x0000, 0x0000, 0x0002, /*11AA6*/
  0xFBC2, 0x0020, 0x0002, 0x9AA7, 0x0000, 0x0000, 0x0002, /*11AA7*/
  0xFBC2, 0x0020, 0x0002, 0x9AA8, 0x0000, 0x0000, 0x0002, /*11AA8*/
  0xFBC2, 0x0020, 0x0002, 0x9AA9, 0x0000, 0x0000, 0x0002, /*11AA9*/
  0xFBC2, 0x0020, 0x0002, 0x9AAA, 0x0000, 0x0000, 0x0002, /*11AAA*/
  0xFBC2, 0x0020, 0x0002, 0x9AAB, 0x0000, 0x0000, 0x0002, /*11AAB*/
  0xFBC2, 0x0020, 0x0002, 0x9AAC, 0x0000, 0x0000, 0x0002, /*11AAC*/
  0xFBC2, 0x0020, 0x0002, 0x9AAD, 0x0000, 0x0000, 0x0002, /*11AAD*/
  0xFBC2, 0x0020, 0x0002, 0x9AAE, 0x0000, 0x0000, 0x0002, /*11AAE*/
  0xFBC2, 0x0020, 0x0002, 0x9AAF, 0x0000, 0x0000, 0x0002, /*11AAF*/
  0xFBC2, 0x0020, 0x0002, 0x9AB0, 0x0000, 0x0000, 0x0002, /*11AB0*/
  0xFBC2, 0x0020, 0x0002, 0x9AB1, 0x0000, 0x0000, 0x0002, /*11AB1*/
  0xFBC2, 0x0020, 0x0002, 0x9AB2, 0x0000, 0x0000, 0x0002, /*11AB2*/
  0xFBC2, 0x0020, 0x0002, 0x9AB3, 0x0000, 0x0000, 0x0002, /*11AB3*/
  0xFBC2, 0x0020, 0x0002, 0x9AB4, 0x0000, 0x0000, 0x0002, /*11AB4*/
  0xFBC2, 0x0020, 0x0002, 0x9AB5, 0x0000, 0x0000, 0x0002, /*11AB5*/
  0xFBC2, 0x0020, 0x0002, 0x9AB6, 0x0000, 0x0000, 0x0002, /*11AB6*/
  0xFBC2, 0x0020, 0x0002, 0x9AB7, 0x0000, 0x0000, 0x0002, /*11AB7*/
  0xFBC2, 0x0020, 0x0002, 0x9AB8, 0x0000, 0x0000, 0x0002, /*11AB8*/
  0xFBC2, 0x0020, 0x0002, 0x9AB9, 0x0000, 0x0000, 0x0002, /*11AB9*/
  0xFBC2, 0x0020, 0x0002, 0x9ABA, 0x0000, 0x0000, 0x0002, /*11ABA*/
  0xFBC2, 0x0020, 0x0002, 0x9ABB, 0x0000, 0x0000, 0x0002, /*11ABB*/
  0xFBC2, 0x0020, 0x0002, 0x9ABC, 0x0000, 0x0000, 0x0002, /*11ABC*/
  0xFBC2, 0x0020, 0x0002, 0x9ABD, 0x0000, 0x0000, 0x0002, /*11ABD*/
  0xFBC2, 0x0020, 0x0002, 0x9ABE, 0x0000, 0x0000, 0x0002, /*11ABE*/
  0xFBC2, 0x0020, 0x0002, 0x9ABF, 0x0000, 0x0000, 0x0002, /*11ABF*/
  0x4323, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AC0*/
  0x4324, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AC1*/
  0x4325, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AC2*/
  0x4326, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AC3*/
  0x4327, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AC4*/
  0x4328, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AC5*/
  0x4329, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AC6*/
  0x432A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AC7*/
  0x432B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AC8*/
  0x432C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AC9*/
  0x432D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11ACA*/
  0x432E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11ACB*/
  0x432F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11ACC*/
  0x4330, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11ACD*/
  0x4331, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11ACE*/
  0x4332, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11ACF*/
  0x4333, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AD0*/
  0x4334, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AD1*/
  0x4335, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AD2*/
  0x4336, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AD3*/
  0x4337, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AD4*/
  0x431C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AD5*/
  0x431D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AD6*/
  0x431E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AD7*/
  0x431F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AD8*/
  0x4320, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AD9*/
  0x4321, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11ADA*/
  0x4322, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11ADB*/
  0x4338, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11ADC*/
  0x4339, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11ADD*/
  0x433F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11ADE*/
  0x433A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11ADF*/
  0x433B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AE0*/
  0x433C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AE1*/
  0x433D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AE2*/
  0x433E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AE3*/
  0x4340, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AE4*/
  0x4346, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AE5*/
  0x4344, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AE6*/
  0x4352, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AE7*/
  0x4347, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AE8*/
  0x4345, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AE9*/
  0x4353, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AEA*/
  0x434E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AEB*/
  0x434C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AEC*/
  0x434F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AED*/
  0x434D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AEE*/
  0x4341, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AEF*/
  0x4354, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AF0*/
  0x4343, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AF1*/
  0x4342, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AF2*/
  0x434A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AF3*/
  0x4348, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AF4*/
  0x4350, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AF5*/
  0x434B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AF6*/
  0x4349, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AF7*/
  0x4351, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11AF8*/
  0xFBC2, 0x0020, 0x0002, 0x9AF9, 0x0000, 0x0000, 0x0002, /*11AF9*/
  0xFBC2, 0x0020, 0x0002, 0x9AFA, 0x0000, 0x0000, 0x0002, /*11AFA*/
  0xFBC2, 0x0020, 0x0002, 0x9AFB, 0x0000, 0x0000, 0x0002, /*11AFB*/
  0xFBC2, 0x0020, 0x0002, 0x9AFC, 0x0000, 0x0000, 0x0002, /*11AFC*/
  0xFBC2, 0x0020, 0x0002, 0x9AFD, 0x0000, 0x0000, 0x0002, /*11AFD*/
  0xFBC2, 0x0020, 0x0002, 0x9AFE, 0x0000, 0x0000, 0x0002, /*11AFE*/
  0xFBC2, 0x0020, 0x0002, 0x9AFF, 0x0000, 0x0000, 0x0002  /*11AFF*/
};

uint16 uca900_p11C[]= {
  0x2D37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C00*/
  0x2D38, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C01*/
  0x2D39, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C02*/
  0x2D3A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C03*/
  0x2D3B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C04*/
  0x2D3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C05*/
  0x2D3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C06*/
  0x2D3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C07*/
  0x2D3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C08*/
  0xFBC2, 0x0020, 0x0002, 0x9C09, 0x0000, 0x0000, 0x0002, /*11C09*/
  0x2D40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C0A*/
  0x2D41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C0B*/
  0x2D42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C0C*/
  0x2D43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C0D*/
  0x2D44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C0E*/
  0x2D45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C0F*/
  0x2D46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C10*/
  0x2D47, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C11*/
  0x2D48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C12*/
  0x2D49, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C13*/
  0x2D4A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C14*/
  0x2D4B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C15*/
  0x2D4C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C16*/
  0x2D4D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C17*/
  0x2D4E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C18*/
  0x2D4F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C19*/
  0x2D50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C1A*/
  0x2D51, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C1B*/
  0x2D52, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C1C*/
  0x2D53, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C1D*/
  0x2D54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C1E*/
  0x2D55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C1F*/
  0x2D56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C20*/
  0x2D57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C21*/
  0x2D58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C22*/
  0x2D59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C23*/
  0x2D5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C24*/
  0x2D5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C25*/
  0x2D5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C26*/
  0x2D5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C27*/
  0x2D5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C28*/
  0x2D5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C29*/
  0x2D60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C2A*/
  0x2D61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C2B*/
  0x2D62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C2C*/
  0x2D63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C2D*/
  0x2D64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C2E*/
  0x2D66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C2F*/
  0x2D67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C30*/
  0x2D68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C31*/
  0x2D69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C32*/
  0x2D6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C33*/
  0x2D6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C34*/
  0x2D6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C35*/
  0x2D6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C36*/
  0xFBC2, 0x0020, 0x0002, 0x9C37, 0x0000, 0x0000, 0x0002, /*11C37*/
  0x2D6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C38*/
  0x2D6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C39*/
  0x2D70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C3A*/
  0x2D71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C3B*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C3C*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C3D*/
  0x0000, 0x00C5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C3E*/
  0x2D72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C3F*/
  0x2D65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C40*/
  0x02BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C41*/
  0x02BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C42*/
  0x043F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C43*/
  0x0440, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C44*/
  0x0441, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C45*/
  0xFBC2, 0x0020, 0x0002, 0x9C46, 0x0000, 0x0000, 0x0002, /*11C46*/
  0xFBC2, 0x0020, 0x0002, 0x9C47, 0x0000, 0x0000, 0x0002, /*11C47*/
  0xFBC2, 0x0020, 0x0002, 0x9C48, 0x0000, 0x0000, 0x0002, /*11C48*/
  0xFBC2, 0x0020, 0x0002, 0x9C49, 0x0000, 0x0000, 0x0002, /*11C49*/
  0xFBC2, 0x0020, 0x0002, 0x9C4A, 0x0000, 0x0000, 0x0002, /*11C4A*/
  0xFBC2, 0x0020, 0x0002, 0x9C4B, 0x0000, 0x0000, 0x0002, /*11C4B*/
  0xFBC2, 0x0020, 0x0002, 0x9C4C, 0x0000, 0x0000, 0x0002, /*11C4C*/
  0xFBC2, 0x0020, 0x0002, 0x9C4D, 0x0000, 0x0000, 0x0002, /*11C4D*/
  0xFBC2, 0x0020, 0x0002, 0x9C4E, 0x0000, 0x0000, 0x0002, /*11C4E*/
  0xFBC2, 0x0020, 0x0002, 0x9C4F, 0x0000, 0x0000, 0x0002, /*11C4F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C50*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C51*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C52*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C53*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C54*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C55*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C56*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C57*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C58*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C59*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C5A*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C5B*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C5C*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C5D*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C5E*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C5F*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C60*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C61*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C62*/
  0x1B82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C63*/
  0x1B83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C64*/
  0x1B84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C65*/
  0x1B85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C66*/
  0x1B86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C67*/
  0x1B87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C68*/
  0x1B88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C69*/
  0x1B89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C6A*/
  0x1B8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C6B*/
  0x1B8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C6C*/
  0xFBC2, 0x0020, 0x0002, 0x9C6D, 0x0000, 0x0000, 0x0002, /*11C6D*/
  0xFBC2, 0x0020, 0x0002, 0x9C6E, 0x0000, 0x0000, 0x0002, /*11C6E*/
  0xFBC2, 0x0020, 0x0002, 0x9C6F, 0x0000, 0x0000, 0x0002, /*11C6F*/
  0x0409, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C70*/
  0x040A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C71*/
  0x2E86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C72*/
  0x2E88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C73*/
  0x2E8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C74*/
  0x2E8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C75*/
  0x2E8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C76*/
  0x2E90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C77*/
  0x2E92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C78*/
  0x2E94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C79*/
  0x2E96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C7A*/
  0x2E98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C7B*/
  0x2E9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C7C*/
  0x2E9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C7D*/
  0x2E9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C7E*/
  0x2EA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C7F*/
  0x2EA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C80*/
  0x2EA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C81*/
  0x2EA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C82*/
  0x2EA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C83*/
  0x2EAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C84*/
  0x2EAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C85*/
  0x2EAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C86*/
  0x2EB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C87*/
  0x2EB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C88*/
  0x2EB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C89*/
  0x2EB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C8A*/
  0x2EB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C8B*/
  0x2EB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C8C*/
  0x2EBB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C8D*/
  0x2EBD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C8E*/
  0x2EBF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C8F*/
  0xFBC2, 0x0020, 0x0002, 0x9C90, 0x0000, 0x0000, 0x0002, /*11C90*/
  0xFBC2, 0x0020, 0x0002, 0x9C91, 0x0000, 0x0000, 0x0002, /*11C91*/
  0x2E87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C92*/
  0x2E89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C93*/
  0x2E8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C94*/
  0x2E8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C95*/
  0x2E8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C96*/
  0x2E91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C97*/
  0x2E93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C98*/
  0x2E95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C99*/
  0x2E97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C9A*/
  0x2E99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C9B*/
  0x2E9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C9C*/
  0x2E9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C9D*/
  0x2E9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C9E*/
  0x2EA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11C9F*/
  0x2EA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CA0*/
  0x2EA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CA1*/
  0x2EA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CA2*/
  0x2EA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CA3*/
  0x2EAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CA4*/
  0x2EAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CA5*/
  0x2EAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CA6*/
  0x2EB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CA7*/
  0xFBC2, 0x0020, 0x0002, 0x9CA8, 0x0000, 0x0000, 0x0002, /*11CA8*/
  0x2EB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CA9*/
  0x2EB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CAA*/
  0x2EB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CAB*/
  0x2EBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CAC*/
  0x2EBC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CAD*/
  0x2EBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CAE*/
  0x2EC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CAF*/
  0x2EC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CB0*/
  0x2EC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CB1*/
  0x2EC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CB2*/
  0x2EC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CB3*/
  0x2EC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CB4*/
  0x0000, 0x00C4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CB5*/
  0x0000, 0x00C3, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*11CB6*/
  0xFBC2, 0x0020, 0x0002, 0x9CB7, 0x0000, 0x0000, 0x0002, /*11CB7*/
  0xFBC2, 0x0020, 0x0002, 0x9CB8, 0x0000, 0x0000, 0x0002, /*11CB8*/
  0xFBC2, 0x0020, 0x0002, 0x9CB9, 0x0000, 0x0000, 0x0002, /*11CB9*/
  0xFBC2, 0x0020, 0x0002, 0x9CBA, 0x0000, 0x0000, 0x0002, /*11CBA*/
  0xFBC2, 0x0020, 0x0002, 0x9CBB, 0x0000, 0x0000, 0x0002, /*11CBB*/
  0xFBC2, 0x0020, 0x0002, 0x9CBC, 0x0000, 0x0000, 0x0002, /*11CBC*/
  0xFBC2, 0x0020, 0x0002, 0x9CBD, 0x0000, 0x0000, 0x0002, /*11CBD*/
  0xFBC2, 0x0020, 0x0002, 0x9CBE, 0x0000, 0x0000, 0x0002, /*11CBE*/
  0xFBC2, 0x0020, 0x0002, 0x9CBF, 0x0000, 0x0000, 0x0002, /*11CBF*/
  0xFBC2, 0x0020, 0x0002, 0x9CC0, 0x0000, 0x0000, 0x0002, /*11CC0*/
  0xFBC2, 0x0020, 0x0002, 0x9CC1, 0x0000, 0x0000, 0x0002, /*11CC1*/
  0xFBC2, 0x0020, 0x0002, 0x9CC2, 0x0000, 0x0000, 0x0002, /*11CC2*/
  0xFBC2, 0x0020, 0x0002, 0x9CC3, 0x0000, 0x0000, 0x0002, /*11CC3*/
  0xFBC2, 0x0020, 0x0002, 0x9CC4, 0x0000, 0x0000, 0x0002, /*11CC4*/
  0xFBC2, 0x0020, 0x0002, 0x9CC5, 0x0000, 0x0000, 0x0002, /*11CC5*/
  0xFBC2, 0x0020, 0x0002, 0x9CC6, 0x0000, 0x0000, 0x0002, /*11CC6*/
  0xFBC2, 0x0020, 0x0002, 0x9CC7, 0x0000, 0x0000, 0x0002, /*11CC7*/
  0xFBC2, 0x0020, 0x0002, 0x9CC8, 0x0000, 0x0000, 0x0002, /*11CC8*/
  0xFBC2, 0x0020, 0x0002, 0x9CC9, 0x0000, 0x0000, 0x0002, /*11CC9*/
  0xFBC2, 0x0020, 0x0002, 0x9CCA, 0x0000, 0x0000, 0x0002, /*11CCA*/
  0xFBC2, 0x0020, 0x0002, 0x9CCB, 0x0000, 0x0000, 0x0002, /*11CCB*/
  0xFBC2, 0x0020, 0x0002, 0x9CCC, 0x0000, 0x0000, 0x0002, /*11CCC*/
  0xFBC2, 0x0020, 0x0002, 0x9CCD, 0x0000, 0x0000, 0x0002, /*11CCD*/
  0xFBC2, 0x0020, 0x0002, 0x9CCE, 0x0000, 0x0000, 0x0002, /*11CCE*/
  0xFBC2, 0x0020, 0x0002, 0x9CCF, 0x0000, 0x0000, 0x0002, /*11CCF*/
  0xFBC2, 0x0020, 0x0002, 0x9CD0, 0x0000, 0x0000, 0x0002, /*11CD0*/
  0xFBC2, 0x0020, 0x0002, 0x9CD1, 0x0000, 0x0000, 0x0002, /*11CD1*/
  0xFBC2, 0x0020, 0x0002, 0x9CD2, 0x0000, 0x0000, 0x0002, /*11CD2*/
  0xFBC2, 0x0020, 0x0002, 0x9CD3, 0x0000, 0x0000, 0x0002, /*11CD3*/
  0xFBC2, 0x0020, 0x0002, 0x9CD4, 0x0000, 0x0000, 0x0002, /*11CD4*/
  0xFBC2, 0x0020, 0x0002, 0x9CD5, 0x0000, 0x0000, 0x0002, /*11CD5*/
  0xFBC2, 0x0020, 0x0002, 0x9CD6, 0x0000, 0x0000, 0x0002, /*11CD6*/
  0xFBC2, 0x0020, 0x0002, 0x9CD7, 0x0000, 0x0000, 0x0002, /*11CD7*/
  0xFBC2, 0x0020, 0x0002, 0x9CD8, 0x0000, 0x0000, 0x0002, /*11CD8*/
  0xFBC2, 0x0020, 0x0002, 0x9CD9, 0x0000, 0x0000, 0x0002, /*11CD9*/
  0xFBC2, 0x0020, 0x0002, 0x9CDA, 0x0000, 0x0000, 0x0002, /*11CDA*/
  0xFBC2, 0x0020, 0x0002, 0x9CDB, 0x0000, 0x0000, 0x0002, /*11CDB*/
  0xFBC2, 0x0020, 0x0002, 0x9CDC, 0x0000, 0x0000, 0x0002, /*11CDC*/
  0xFBC2, 0x0020, 0x0002, 0x9CDD, 0x0000, 0x0000, 0x0002, /*11CDD*/
  0xFBC2, 0x0020, 0x0002, 0x9CDE, 0x0000, 0x0000, 0x0002, /*11CDE*/
  0xFBC2, 0x0020, 0x0002, 0x9CDF, 0x0000, 0x0000, 0x0002, /*11CDF*/
  0xFBC2, 0x0020, 0x0002, 0x9CE0, 0x0000, 0x0000, 0x0002, /*11CE0*/
  0xFBC2, 0x0020, 0x0002, 0x9CE1, 0x0000, 0x0000, 0x0002, /*11CE1*/
  0xFBC2, 0x0020, 0x0002, 0x9CE2, 0x0000, 0x0000, 0x0002, /*11CE2*/
  0xFBC2, 0x0020, 0x0002, 0x9CE3, 0x0000, 0x0000, 0x0002, /*11CE3*/
  0xFBC2, 0x0020, 0x0002, 0x9CE4, 0x0000, 0x0000, 0x0002, /*11CE4*/
  0xFBC2, 0x0020, 0x0002, 0x9CE5, 0x0000, 0x0000, 0x0002, /*11CE5*/
  0xFBC2, 0x0020, 0x0002, 0x9CE6, 0x0000, 0x0000, 0x0002, /*11CE6*/
  0xFBC2, 0x0020, 0x0002, 0x9CE7, 0x0000, 0x0000, 0x0002, /*11CE7*/
  0xFBC2, 0x0020, 0x0002, 0x9CE8, 0x0000, 0x0000, 0x0002, /*11CE8*/
  0xFBC2, 0x0020, 0x0002, 0x9CE9, 0x0000, 0x0000, 0x0002, /*11CE9*/
  0xFBC2, 0x0020, 0x0002, 0x9CEA, 0x0000, 0x0000, 0x0002, /*11CEA*/
  0xFBC2, 0x0020, 0x0002, 0x9CEB, 0x0000, 0x0000, 0x0002, /*11CEB*/
  0xFBC2, 0x0020, 0x0002, 0x9CEC, 0x0000, 0x0000, 0x0002, /*11CEC*/
  0xFBC2, 0x0020, 0x0002, 0x9CED, 0x0000, 0x0000, 0x0002, /*11CED*/
  0xFBC2, 0x0020, 0x0002, 0x9CEE, 0x0000, 0x0000, 0x0002, /*11CEE*/
  0xFBC2, 0x0020, 0x0002, 0x9CEF, 0x0000, 0x0000, 0x0002, /*11CEF*/
  0xFBC2, 0x0020, 0x0002, 0x9CF0, 0x0000, 0x0000, 0x0002, /*11CF0*/
  0xFBC2, 0x0020, 0x0002, 0x9CF1, 0x0000, 0x0000, 0x0002, /*11CF1*/
  0xFBC2, 0x0020, 0x0002, 0x9CF2, 0x0000, 0x0000, 0x0002, /*11CF2*/
  0xFBC2, 0x0020, 0x0002, 0x9CF3, 0x0000, 0x0000, 0x0002, /*11CF3*/
  0xFBC2, 0x0020, 0x0002, 0x9CF4, 0x0000, 0x0000, 0x0002, /*11CF4*/
  0xFBC2, 0x0020, 0x0002, 0x9CF5, 0x0000, 0x0000, 0x0002, /*11CF5*/
  0xFBC2, 0x0020, 0x0002, 0x9CF6, 0x0000, 0x0000, 0x0002, /*11CF6*/
  0xFBC2, 0x0020, 0x0002, 0x9CF7, 0x0000, 0x0000, 0x0002, /*11CF7*/
  0xFBC2, 0x0020, 0x0002, 0x9CF8, 0x0000, 0x0000, 0x0002, /*11CF8*/
  0xFBC2, 0x0020, 0x0002, 0x9CF9, 0x0000, 0x0000, 0x0002, /*11CF9*/
  0xFBC2, 0x0020, 0x0002, 0x9CFA, 0x0000, 0x0000, 0x0002, /*11CFA*/
  0xFBC2, 0x0020, 0x0002, 0x9CFB, 0x0000, 0x0000, 0x0002, /*11CFB*/
  0xFBC2, 0x0020, 0x0002, 0x9CFC, 0x0000, 0x0000, 0x0002, /*11CFC*/
  0xFBC2, 0x0020, 0x0002, 0x9CFD, 0x0000, 0x0000, 0x0002, /*11CFD*/
  0xFBC2, 0x0020, 0x0002, 0x9CFE, 0x0000, 0x0000, 0x0002, /*11CFE*/
  0xFBC2, 0x0020, 0x0002, 0x9CFF, 0x0000, 0x0000, 0x0002  /*11CFF*/
};

uint16 uca900_p120[]= {
  0x49B5, 0x0020, 0x0002, 0x0001, /*12000*/
  0x49B6, 0x0020, 0x0002, 0x0001, /*12001*/
  0x49B7, 0x0020, 0x0002, 0x0001, /*12002*/
  0x49B8, 0x0020, 0x0002, 0x0001, /*12003*/
  0x49B9, 0x0020, 0x0002, 0x0001, /*12004*/
  0x49BA, 0x0020, 0x0002, 0x0001, /*12005*/
  0x49BB, 0x0020, 0x0002, 0x0001, /*12006*/
  0x49BC, 0x0020, 0x0002, 0x0001, /*12007*/
  0x49BD, 0x0020, 0x0002, 0x0001, /*12008*/
  0x49BE, 0x0020, 0x0002, 0x0001, /*12009*/
  0x49BF, 0x0020, 0x0002, 0x0001, /*1200A*/
  0x49C0, 0x0020, 0x0002, 0x0001, /*1200B*/
  0x49C1, 0x0020, 0x0002, 0x0001, /*1200C*/
  0x49C2, 0x0020, 0x0002, 0x0001, /*1200D*/
  0x49C3, 0x0020, 0x0002, 0x0001, /*1200E*/
  0x49C4, 0x0020, 0x0002, 0x0001, /*1200F*/
  0x49C5, 0x0020, 0x0002, 0x0001, /*12010*/
  0x49C6, 0x0020, 0x0002, 0x0001, /*12011*/
  0x49C7, 0x0020, 0x0002, 0x0001, /*12012*/
  0x49C8, 0x0020, 0x0002, 0x0001, /*12013*/
  0x49C9, 0x0020, 0x0002, 0x0001, /*12014*/
  0x49CA, 0x0020, 0x0002, 0x0001, /*12015*/
  0x49CB, 0x0020, 0x0002, 0x0001, /*12016*/
  0x49CC, 0x0020, 0x0002, 0x0001, /*12017*/
  0x49CD, 0x0020, 0x0002, 0x0001, /*12018*/
  0x49CE, 0x0020, 0x0002, 0x0001, /*12019*/
  0x49CF, 0x0020, 0x0002, 0x0001, /*1201A*/
  0x49D0, 0x0020, 0x0002, 0x0001, /*1201B*/
  0x49D1, 0x0020, 0x0002, 0x0001, /*1201C*/
  0x49D2, 0x0020, 0x0002, 0x0001, /*1201D*/
  0x49D3, 0x0020, 0x0002, 0x0001, /*1201E*/
  0x49D4, 0x0020, 0x0002, 0x0001, /*1201F*/
  0x49D5, 0x0020, 0x0002, 0x0001, /*12020*/
  0x49D6, 0x0020, 0x0002, 0x0001, /*12021*/
  0x49D7, 0x0020, 0x0002, 0x0001, /*12022*/
  0x49D8, 0x0020, 0x0002, 0x0001, /*12023*/
  0x49D9, 0x0020, 0x0002, 0x0001, /*12024*/
  0x49DA, 0x0020, 0x0002, 0x0001, /*12025*/
  0x49DB, 0x0020, 0x0002, 0x0001, /*12026*/
  0x49DC, 0x0020, 0x0002, 0x0001, /*12027*/
  0x49DD, 0x0020, 0x0002, 0x0001, /*12028*/
  0x49DE, 0x0020, 0x0002, 0x0001, /*12029*/
  0x49DF, 0x0020, 0x0002, 0x0001, /*1202A*/
  0x49E0, 0x0020, 0x0002, 0x0001, /*1202B*/
  0x49E1, 0x0020, 0x0002, 0x0001, /*1202C*/
  0x49E2, 0x0020, 0x0002, 0x0001, /*1202D*/
  0x49E3, 0x0020, 0x0002, 0x0001, /*1202E*/
  0x49E4, 0x0020, 0x0002, 0x0001, /*1202F*/
  0x49E5, 0x0020, 0x0002, 0x0001, /*12030*/
  0x49E6, 0x0020, 0x0002, 0x0001, /*12031*/
  0x49E7, 0x0020, 0x0002, 0x0001, /*12032*/
  0x49E8, 0x0020, 0x0002, 0x0001, /*12033*/
  0x49E9, 0x0020, 0x0002, 0x0001, /*12034*/
  0x49EA, 0x0020, 0x0002, 0x0001, /*12035*/
  0x49EB, 0x0020, 0x0002, 0x0001, /*12036*/
  0x49EC, 0x0020, 0x0002, 0x0001, /*12037*/
  0x49ED, 0x0020, 0x0002, 0x0001, /*12038*/
  0x49EE, 0x0020, 0x0002, 0x0001, /*12039*/
  0x49EF, 0x0020, 0x0002, 0x0001, /*1203A*/
  0x49F0, 0x0020, 0x0002, 0x0001, /*1203B*/
  0x49F1, 0x0020, 0x0002, 0x0001, /*1203C*/
  0x49F2, 0x0020, 0x0002, 0x0001, /*1203D*/
  0x49F3, 0x0020, 0x0002, 0x0001, /*1203E*/
  0x49F4, 0x0020, 0x0002, 0x0001, /*1203F*/
  0x49F5, 0x0020, 0x0002, 0x0001, /*12040*/
  0x49F6, 0x0020, 0x0002, 0x0001, /*12041*/
  0x49F7, 0x0020, 0x0002, 0x0001, /*12042*/
  0x49F8, 0x0020, 0x0002, 0x0001, /*12043*/
  0x49F9, 0x0020, 0x0002, 0x0001, /*12044*/
  0x49FA, 0x0020, 0x0002, 0x0001, /*12045*/
  0x49FB, 0x0020, 0x0002, 0x0001, /*12046*/
  0x49FC, 0x0020, 0x0002, 0x0001, /*12047*/
  0x49FD, 0x0020, 0x0002, 0x0001, /*12048*/
  0x49FE, 0x0020, 0x0002, 0x0001, /*12049*/
  0x49FF, 0x0020, 0x0002, 0x0001, /*1204A*/
  0x4A00, 0x0020, 0x0002, 0x0001, /*1204B*/
  0x4A01, 0x0020, 0x0002, 0x0001, /*1204C*/
  0x4A02, 0x0020, 0x0002, 0x0001, /*1204D*/
  0x4A03, 0x0020, 0x0002, 0x0001, /*1204E*/
  0x4A04, 0x0020, 0x0002, 0x0001, /*1204F*/
  0x4A05, 0x0020, 0x0002, 0x0001, /*12050*/
  0x4A06, 0x0020, 0x0002, 0x0001, /*12051*/
  0x4A07, 0x0020, 0x0002, 0x0001, /*12052*/
  0x4A08, 0x0020, 0x0002, 0x0001, /*12053*/
  0x4A09, 0x0020, 0x0002, 0x0001, /*12054*/
  0x4A0A, 0x0020, 0x0002, 0x0001, /*12055*/
  0x4A0B, 0x0020, 0x0002, 0x0001, /*12056*/
  0x4A0C, 0x0020, 0x0002, 0x0001, /*12057*/
  0x4A0D, 0x0020, 0x0002, 0x0001, /*12058*/
  0x4A0E, 0x0020, 0x0002, 0x0001, /*12059*/
  0x4A0F, 0x0020, 0x0002, 0x0001, /*1205A*/
  0x4A10, 0x0020, 0x0002, 0x0001, /*1205B*/
  0x4A11, 0x0020, 0x0002, 0x0001, /*1205C*/
  0x4A12, 0x0020, 0x0002, 0x0001, /*1205D*/
  0x4A13, 0x0020, 0x0002, 0x0001, /*1205E*/
  0x4A14, 0x0020, 0x0002, 0x0001, /*1205F*/
  0x4A15, 0x0020, 0x0002, 0x0001, /*12060*/
  0x4A16, 0x0020, 0x0002, 0x0001, /*12061*/
  0x4A17, 0x0020, 0x0002, 0x0001, /*12062*/
  0x4A18, 0x0020, 0x0002, 0x0001, /*12063*/
  0x4A19, 0x0020, 0x0002, 0x0001, /*12064*/
  0x4A1A, 0x0020, 0x0002, 0x0001, /*12065*/
  0x4A1B, 0x0020, 0x0002, 0x0001, /*12066*/
  0x4A1C, 0x0020, 0x0002, 0x0001, /*12067*/
  0x4A1D, 0x0020, 0x0002, 0x0001, /*12068*/
  0x4A1E, 0x0020, 0x0002, 0x0001, /*12069*/
  0x4A1F, 0x0020, 0x0002, 0x0001, /*1206A*/
  0x4A20, 0x0020, 0x0002, 0x0001, /*1206B*/
  0x4A21, 0x0020, 0x0002, 0x0001, /*1206C*/
  0x4A22, 0x0020, 0x0002, 0x0001, /*1206D*/
  0x4A23, 0x0020, 0x0002, 0x0001, /*1206E*/
  0x4A24, 0x0020, 0x0002, 0x0001, /*1206F*/
  0x4A25, 0x0020, 0x0002, 0x0001, /*12070*/
  0x4A26, 0x0020, 0x0002, 0x0001, /*12071*/
  0x4A27, 0x0020, 0x0002, 0x0001, /*12072*/
  0x4A28, 0x0020, 0x0002, 0x0001, /*12073*/
  0x4A29, 0x0020, 0x0002, 0x0001, /*12074*/
  0x4A2A, 0x0020, 0x0002, 0x0001, /*12075*/
  0x4A2B, 0x0020, 0x0002, 0x0001, /*12076*/
  0x4A2C, 0x0020, 0x0002, 0x0001, /*12077*/
  0x4A2D, 0x0020, 0x0002, 0x0001, /*12078*/
  0x4A2E, 0x0020, 0x0002, 0x0001, /*12079*/
  0x4A2F, 0x0020, 0x0002, 0x0001, /*1207A*/
  0x4A30, 0x0020, 0x0002, 0x0001, /*1207B*/
  0x4A31, 0x0020, 0x0002, 0x0001, /*1207C*/
  0x4A32, 0x0020, 0x0002, 0x0001, /*1207D*/
  0x4A33, 0x0020, 0x0002, 0x0001, /*1207E*/
  0x4A34, 0x0020, 0x0002, 0x0001, /*1207F*/
  0x4A35, 0x0020, 0x0002, 0x0001, /*12080*/
  0x4A36, 0x0020, 0x0002, 0x0001, /*12081*/
  0x4A37, 0x0020, 0x0002, 0x0001, /*12082*/
  0x4A38, 0x0020, 0x0002, 0x0001, /*12083*/
  0x4A39, 0x0020, 0x0002, 0x0001, /*12084*/
  0x4A3A, 0x0020, 0x0002, 0x0001, /*12085*/
  0x4A3B, 0x0020, 0x0002, 0x0001, /*12086*/
  0x4A3C, 0x0020, 0x0002, 0x0001, /*12087*/
  0x4A3D, 0x0020, 0x0002, 0x0001, /*12088*/
  0x4A3E, 0x0020, 0x0002, 0x0001, /*12089*/
  0x4A3F, 0x0020, 0x0002, 0x0001, /*1208A*/
  0x4A40, 0x0020, 0x0002, 0x0001, /*1208B*/
  0x4A41, 0x0020, 0x0002, 0x0001, /*1208C*/
  0x4A42, 0x0020, 0x0002, 0x0001, /*1208D*/
  0x4A43, 0x0020, 0x0002, 0x0001, /*1208E*/
  0x4A44, 0x0020, 0x0002, 0x0001, /*1208F*/
  0x4A45, 0x0020, 0x0002, 0x0001, /*12090*/
  0x4A46, 0x0020, 0x0002, 0x0001, /*12091*/
  0x4A47, 0x0020, 0x0002, 0x0001, /*12092*/
  0x4A48, 0x0020, 0x0002, 0x0001, /*12093*/
  0x4A49, 0x0020, 0x0002, 0x0001, /*12094*/
  0x4A4A, 0x0020, 0x0002, 0x0001, /*12095*/
  0x4A4B, 0x0020, 0x0002, 0x0001, /*12096*/
  0x4A4C, 0x0020, 0x0002, 0x0001, /*12097*/
  0x4A4D, 0x0020, 0x0002, 0x0001, /*12098*/
  0x4A4E, 0x0020, 0x0002, 0x0001, /*12099*/
  0x4A4F, 0x0020, 0x0002, 0x0001, /*1209A*/
  0x4A50, 0x0020, 0x0002, 0x0001, /*1209B*/
  0x4A51, 0x0020, 0x0002, 0x0001, /*1209C*/
  0x4A52, 0x0020, 0x0002, 0x0001, /*1209D*/
  0x4A53, 0x0020, 0x0002, 0x0001, /*1209E*/
  0x4A54, 0x0020, 0x0002, 0x0001, /*1209F*/
  0x4A55, 0x0020, 0x0002, 0x0001, /*120A0*/
  0x4A56, 0x0020, 0x0002, 0x0001, /*120A1*/
  0x4A57, 0x0020, 0x0002, 0x0001, /*120A2*/
  0x4A58, 0x0020, 0x0002, 0x0001, /*120A3*/
  0x4A59, 0x0020, 0x0002, 0x0001, /*120A4*/
  0x4A5A, 0x0020, 0x0002, 0x0001, /*120A5*/
  0x4A5B, 0x0020, 0x0002, 0x0001, /*120A6*/
  0x4A5C, 0x0020, 0x0002, 0x0001, /*120A7*/
  0x4A5D, 0x0020, 0x0002, 0x0001, /*120A8*/
  0x4A5E, 0x0020, 0x0002, 0x0001, /*120A9*/
  0x4A5F, 0x0020, 0x0002, 0x0001, /*120AA*/
  0x4A60, 0x0020, 0x0002, 0x0001, /*120AB*/
  0x4A61, 0x0020, 0x0002, 0x0001, /*120AC*/
  0x4A62, 0x0020, 0x0002, 0x0001, /*120AD*/
  0x4A63, 0x0020, 0x0002, 0x0001, /*120AE*/
  0x4A64, 0x0020, 0x0002, 0x0001, /*120AF*/
  0x4A65, 0x0020, 0x0002, 0x0001, /*120B0*/
  0x4A66, 0x0020, 0x0002, 0x0001, /*120B1*/
  0x4A67, 0x0020, 0x0002, 0x0001, /*120B2*/
  0x4A68, 0x0020, 0x0002, 0x0001, /*120B3*/
  0x4A69, 0x0020, 0x0002, 0x0001, /*120B4*/
  0x4A6A, 0x0020, 0x0002, 0x0001, /*120B5*/
  0x4A6B, 0x0020, 0x0002, 0x0001, /*120B6*/
  0x4A6C, 0x0020, 0x0002, 0x0001, /*120B7*/
  0x4A6D, 0x0020, 0x0002, 0x0001, /*120B8*/
  0x4A6E, 0x0020, 0x0002, 0x0001, /*120B9*/
  0x4A6F, 0x0020, 0x0002, 0x0001, /*120BA*/
  0x4A70, 0x0020, 0x0002, 0x0001, /*120BB*/
  0x4A71, 0x0020, 0x0002, 0x0001, /*120BC*/
  0x4A72, 0x0020, 0x0002, 0x0001, /*120BD*/
  0x4A73, 0x0020, 0x0002, 0x0001, /*120BE*/
  0x4A74, 0x0020, 0x0002, 0x0001, /*120BF*/
  0x4A75, 0x0020, 0x0002, 0x0001, /*120C0*/
  0x4A76, 0x0020, 0x0002, 0x0001, /*120C1*/
  0x4A77, 0x0020, 0x0002, 0x0001, /*120C2*/
  0x4A78, 0x0020, 0x0002, 0x0001, /*120C3*/
  0x4A79, 0x0020, 0x0002, 0x0001, /*120C4*/
  0x4A7A, 0x0020, 0x0002, 0x0001, /*120C5*/
  0x4A7B, 0x0020, 0x0002, 0x0001, /*120C6*/
  0x4A7C, 0x0020, 0x0002, 0x0001, /*120C7*/
  0x4A7D, 0x0020, 0x0002, 0x0001, /*120C8*/
  0x4A7E, 0x0020, 0x0002, 0x0001, /*120C9*/
  0x4A7F, 0x0020, 0x0002, 0x0001, /*120CA*/
  0x4A80, 0x0020, 0x0002, 0x0001, /*120CB*/
  0x4A81, 0x0020, 0x0002, 0x0001, /*120CC*/
  0x4A82, 0x0020, 0x0002, 0x0001, /*120CD*/
  0x4A83, 0x0020, 0x0002, 0x0001, /*120CE*/
  0x4A84, 0x0020, 0x0002, 0x0001, /*120CF*/
  0x4A85, 0x0020, 0x0002, 0x0001, /*120D0*/
  0x4A86, 0x0020, 0x0002, 0x0001, /*120D1*/
  0x4A87, 0x0020, 0x0002, 0x0001, /*120D2*/
  0x4A88, 0x0020, 0x0002, 0x0001, /*120D3*/
  0x4A89, 0x0020, 0x0002, 0x0001, /*120D4*/
  0x4A8A, 0x0020, 0x0002, 0x0001, /*120D5*/
  0x4A8B, 0x0020, 0x0002, 0x0001, /*120D6*/
  0x4A8C, 0x0020, 0x0002, 0x0001, /*120D7*/
  0x4A8D, 0x0020, 0x0002, 0x0001, /*120D8*/
  0x4A8E, 0x0020, 0x0002, 0x0001, /*120D9*/
  0x4A8F, 0x0020, 0x0002, 0x0001, /*120DA*/
  0x4A90, 0x0020, 0x0002, 0x0001, /*120DB*/
  0x4A91, 0x0020, 0x0002, 0x0001, /*120DC*/
  0x4A92, 0x0020, 0x0002, 0x0001, /*120DD*/
  0x4A93, 0x0020, 0x0002, 0x0001, /*120DE*/
  0x4A94, 0x0020, 0x0002, 0x0001, /*120DF*/
  0x4A95, 0x0020, 0x0002, 0x0001, /*120E0*/
  0x4A96, 0x0020, 0x0002, 0x0001, /*120E1*/
  0x4A97, 0x0020, 0x0002, 0x0001, /*120E2*/
  0x4A98, 0x0020, 0x0002, 0x0001, /*120E3*/
  0x4A99, 0x0020, 0x0002, 0x0001, /*120E4*/
  0x4A9A, 0x0020, 0x0002, 0x0001, /*120E5*/
  0x4A9B, 0x0020, 0x0002, 0x0001, /*120E6*/
  0x4A9C, 0x0020, 0x0002, 0x0001, /*120E7*/
  0x4A9D, 0x0020, 0x0002, 0x0001, /*120E8*/
  0x4A9E, 0x0020, 0x0002, 0x0001, /*120E9*/
  0x4A9F, 0x0020, 0x0002, 0x0001, /*120EA*/
  0x4AA0, 0x0020, 0x0002, 0x0001, /*120EB*/
  0x4AA1, 0x0020, 0x0002, 0x0001, /*120EC*/
  0x4AA2, 0x0020, 0x0002, 0x0001, /*120ED*/
  0x4AA3, 0x0020, 0x0002, 0x0001, /*120EE*/
  0x4AA4, 0x0020, 0x0002, 0x0001, /*120EF*/
  0x4AA5, 0x0020, 0x0002, 0x0001, /*120F0*/
  0x4AA6, 0x0020, 0x0002, 0x0001, /*120F1*/
  0x4AA7, 0x0020, 0x0002, 0x0001, /*120F2*/
  0x4AA8, 0x0020, 0x0002, 0x0001, /*120F3*/
  0x4AA9, 0x0020, 0x0002, 0x0001, /*120F4*/
  0x4AAA, 0x0020, 0x0002, 0x0001, /*120F5*/
  0x4AAB, 0x0020, 0x0002, 0x0001, /*120F6*/
  0x4AAC, 0x0020, 0x0002, 0x0001, /*120F7*/
  0x4AAD, 0x0020, 0x0002, 0x0001, /*120F8*/
  0x4AAE, 0x0020, 0x0002, 0x0001, /*120F9*/
  0x4AAF, 0x0020, 0x0002, 0x0001, /*120FA*/
  0x4AB0, 0x0020, 0x0002, 0x0001, /*120FB*/
  0x4AB1, 0x0020, 0x0002, 0x0001, /*120FC*/
  0x4AB2, 0x0020, 0x0002, 0x0001, /*120FD*/
  0x4AB3, 0x0020, 0x0002, 0x0001, /*120FE*/
  0x4AB4, 0x0020, 0x0002, 0x0001  /*120FF*/
};

uint16 uca900_p121[]= {
  0x4AB5, 0x0020, 0x0002, 0x0001, /*12100*/
  0x4AB6, 0x0020, 0x0002, 0x0001, /*12101*/
  0x4AB7, 0x0020, 0x0002, 0x0001, /*12102*/
  0x4AB8, 0x0020, 0x0002, 0x0001, /*12103*/
  0x4AB9, 0x0020, 0x0002, 0x0001, /*12104*/
  0x4ABA, 0x0020, 0x0002, 0x0001, /*12105*/
  0x4ABB, 0x0020, 0x0002, 0x0001, /*12106*/
  0x4ABC, 0x0020, 0x0002, 0x0001, /*12107*/
  0x4ABD, 0x0020, 0x0002, 0x0001, /*12108*/
  0x4ABE, 0x0020, 0x0002, 0x0001, /*12109*/
  0x4ABF, 0x0020, 0x0002, 0x0001, /*1210A*/
  0x4AC0, 0x0020, 0x0002, 0x0001, /*1210B*/
  0x4AC1, 0x0020, 0x0002, 0x0001, /*1210C*/
  0x4AC2, 0x0020, 0x0002, 0x0001, /*1210D*/
  0x4AC3, 0x0020, 0x0002, 0x0001, /*1210E*/
  0x4AC4, 0x0020, 0x0002, 0x0001, /*1210F*/
  0x4AC5, 0x0020, 0x0002, 0x0001, /*12110*/
  0x4AC6, 0x0020, 0x0002, 0x0001, /*12111*/
  0x4AC7, 0x0020, 0x0002, 0x0001, /*12112*/
  0x4AC8, 0x0020, 0x0002, 0x0001, /*12113*/
  0x4AC9, 0x0020, 0x0002, 0x0001, /*12114*/
  0x4ACA, 0x0020, 0x0002, 0x0001, /*12115*/
  0x4ACB, 0x0020, 0x0002, 0x0001, /*12116*/
  0x4ACC, 0x0020, 0x0002, 0x0001, /*12117*/
  0x4ACD, 0x0020, 0x0002, 0x0001, /*12118*/
  0x4ACE, 0x0020, 0x0002, 0x0001, /*12119*/
  0x4ACF, 0x0020, 0x0002, 0x0001, /*1211A*/
  0x4AD0, 0x0020, 0x0002, 0x0001, /*1211B*/
  0x4AD1, 0x0020, 0x0002, 0x0001, /*1211C*/
  0x4AD2, 0x0020, 0x0002, 0x0001, /*1211D*/
  0x4AD3, 0x0020, 0x0002, 0x0001, /*1211E*/
  0x4AD4, 0x0020, 0x0002, 0x0001, /*1211F*/
  0x4AD5, 0x0020, 0x0002, 0x0001, /*12120*/
  0x4AD6, 0x0020, 0x0002, 0x0001, /*12121*/
  0x4AD7, 0x0020, 0x0002, 0x0001, /*12122*/
  0x4AD8, 0x0020, 0x0002, 0x0001, /*12123*/
  0x4AD9, 0x0020, 0x0002, 0x0001, /*12124*/
  0x4ADA, 0x0020, 0x0002, 0x0001, /*12125*/
  0x4ADB, 0x0020, 0x0002, 0x0001, /*12126*/
  0x4ADC, 0x0020, 0x0002, 0x0001, /*12127*/
  0x4ADD, 0x0020, 0x0002, 0x0001, /*12128*/
  0x4ADE, 0x0020, 0x0002, 0x0001, /*12129*/
  0x4ADF, 0x0020, 0x0002, 0x0001, /*1212A*/
  0x4AE0, 0x0020, 0x0002, 0x0001, /*1212B*/
  0x4AE1, 0x0020, 0x0002, 0x0001, /*1212C*/
  0x4AE2, 0x0020, 0x0002, 0x0001, /*1212D*/
  0x4AE3, 0x0020, 0x0002, 0x0001, /*1212E*/
  0x4AE4, 0x0020, 0x0002, 0x0001, /*1212F*/
  0x4AE5, 0x0020, 0x0002, 0x0001, /*12130*/
  0x4AE6, 0x0020, 0x0002, 0x0001, /*12131*/
  0x4AE7, 0x0020, 0x0002, 0x0001, /*12132*/
  0x4AE8, 0x0020, 0x0002, 0x0001, /*12133*/
  0x4AE9, 0x0020, 0x0002, 0x0001, /*12134*/
  0x4AEA, 0x0020, 0x0002, 0x0001, /*12135*/
  0x4AEB, 0x0020, 0x0002, 0x0001, /*12136*/
  0x4AEC, 0x0020, 0x0002, 0x0001, /*12137*/
  0x4AED, 0x0020, 0x0002, 0x0001, /*12138*/
  0x4AEE, 0x0020, 0x0002, 0x0001, /*12139*/
  0x4AEF, 0x0020, 0x0002, 0x0001, /*1213A*/
  0x4AF0, 0x0020, 0x0002, 0x0001, /*1213B*/
  0x4AF1, 0x0020, 0x0002, 0x0001, /*1213C*/
  0x4AF2, 0x0020, 0x0002, 0x0001, /*1213D*/
  0x4AF3, 0x0020, 0x0002, 0x0001, /*1213E*/
  0x4AF4, 0x0020, 0x0002, 0x0001, /*1213F*/
  0x4AF5, 0x0020, 0x0002, 0x0001, /*12140*/
  0x4AF6, 0x0020, 0x0002, 0x0001, /*12141*/
  0x4AF7, 0x0020, 0x0002, 0x0001, /*12142*/
  0x4AF8, 0x0020, 0x0002, 0x0001, /*12143*/
  0x4AF9, 0x0020, 0x0002, 0x0001, /*12144*/
  0x4AFA, 0x0020, 0x0002, 0x0001, /*12145*/
  0x4AFB, 0x0020, 0x0002, 0x0001, /*12146*/
  0x4AFC, 0x0020, 0x0002, 0x0001, /*12147*/
  0x4AFD, 0x0020, 0x0002, 0x0001, /*12148*/
  0x4AFE, 0x0020, 0x0002, 0x0001, /*12149*/
  0x4AFF, 0x0020, 0x0002, 0x0001, /*1214A*/
  0x4B00, 0x0020, 0x0002, 0x0001, /*1214B*/
  0x4B01, 0x0020, 0x0002, 0x0001, /*1214C*/
  0x4B02, 0x0020, 0x0002, 0x0001, /*1214D*/
  0x4B03, 0x0020, 0x0002, 0x0001, /*1214E*/
  0x4B04, 0x0020, 0x0002, 0x0001, /*1214F*/
  0x4B05, 0x0020, 0x0002, 0x0001, /*12150*/
  0x4B06, 0x0020, 0x0002, 0x0001, /*12151*/
  0x4B07, 0x0020, 0x0002, 0x0001, /*12152*/
  0x4B08, 0x0020, 0x0002, 0x0001, /*12153*/
  0x4B09, 0x0020, 0x0002, 0x0001, /*12154*/
  0x4B0A, 0x0020, 0x0002, 0x0001, /*12155*/
  0x4B0B, 0x0020, 0x0002, 0x0001, /*12156*/
  0x4B0C, 0x0020, 0x0002, 0x0001, /*12157*/
  0x4B0D, 0x0020, 0x0002, 0x0001, /*12158*/
  0x4B0E, 0x0020, 0x0002, 0x0001, /*12159*/
  0x4B0F, 0x0020, 0x0002, 0x0001, /*1215A*/
  0x4B10, 0x0020, 0x0002, 0x0001, /*1215B*/
  0x4B11, 0x0020, 0x0002, 0x0001, /*1215C*/
  0x4B12, 0x0020, 0x0002, 0x0001, /*1215D*/
  0x4B13, 0x0020, 0x0002, 0x0001, /*1215E*/
  0x4B14, 0x0020, 0x0002, 0x0001, /*1215F*/
  0x4B15, 0x0020, 0x0002, 0x0001, /*12160*/
  0x4B16, 0x0020, 0x0002, 0x0001, /*12161*/
  0x4B17, 0x0020, 0x0002, 0x0001, /*12162*/
  0x4B18, 0x0020, 0x0002, 0x0001, /*12163*/
  0x4B19, 0x0020, 0x0002, 0x0001, /*12164*/
  0x4B1A, 0x0020, 0x0002, 0x0001, /*12165*/
  0x4B1B, 0x0020, 0x0002, 0x0001, /*12166*/
  0x4B1C, 0x0020, 0x0002, 0x0001, /*12167*/
  0x4B1D, 0x0020, 0x0002, 0x0001, /*12168*/
  0x4B1E, 0x0020, 0x0002, 0x0001, /*12169*/
  0x4B1F, 0x0020, 0x0002, 0x0001, /*1216A*/
  0x4B20, 0x0020, 0x0002, 0x0001, /*1216B*/
  0x4B21, 0x0020, 0x0002, 0x0001, /*1216C*/
  0x4B22, 0x0020, 0x0002, 0x0001, /*1216D*/
  0x4B23, 0x0020, 0x0002, 0x0001, /*1216E*/
  0x4B24, 0x0020, 0x0002, 0x0001, /*1216F*/
  0x4B25, 0x0020, 0x0002, 0x0001, /*12170*/
  0x4B26, 0x0020, 0x0002, 0x0001, /*12171*/
  0x4B27, 0x0020, 0x0002, 0x0001, /*12172*/
  0x4B28, 0x0020, 0x0002, 0x0001, /*12173*/
  0x4B29, 0x0020, 0x0002, 0x0001, /*12174*/
  0x4B2A, 0x0020, 0x0002, 0x0001, /*12175*/
  0x4B2B, 0x0020, 0x0002, 0x0001, /*12176*/
  0x4B2C, 0x0020, 0x0002, 0x0001, /*12177*/
  0x4B2D, 0x0020, 0x0002, 0x0001, /*12178*/
  0x4B2E, 0x0020, 0x0002, 0x0001, /*12179*/
  0x4B2F, 0x0020, 0x0002, 0x0001, /*1217A*/
  0x4B30, 0x0020, 0x0002, 0x0001, /*1217B*/
  0x4B31, 0x0020, 0x0002, 0x0001, /*1217C*/
  0x4B32, 0x0020, 0x0002, 0x0001, /*1217D*/
  0x4B33, 0x0020, 0x0002, 0x0001, /*1217E*/
  0x4B34, 0x0020, 0x0002, 0x0001, /*1217F*/
  0x4B35, 0x0020, 0x0002, 0x0001, /*12180*/
  0x4B36, 0x0020, 0x0002, 0x0001, /*12181*/
  0x4B37, 0x0020, 0x0002, 0x0001, /*12182*/
  0x4B38, 0x0020, 0x0002, 0x0001, /*12183*/
  0x4B39, 0x0020, 0x0002, 0x0001, /*12184*/
  0x4B3A, 0x0020, 0x0002, 0x0001, /*12185*/
  0x4B3B, 0x0020, 0x0002, 0x0001, /*12186*/
  0x4B3C, 0x0020, 0x0002, 0x0001, /*12187*/
  0x4B3D, 0x0020, 0x0002, 0x0001, /*12188*/
  0x4B3E, 0x0020, 0x0002, 0x0001, /*12189*/
  0x4B3F, 0x0020, 0x0002, 0x0001, /*1218A*/
  0x4B40, 0x0020, 0x0002, 0x0001, /*1218B*/
  0x4B41, 0x0020, 0x0002, 0x0001, /*1218C*/
  0x4B42, 0x0020, 0x0002, 0x0001, /*1218D*/
  0x4B43, 0x0020, 0x0002, 0x0001, /*1218E*/
  0x4B44, 0x0020, 0x0002, 0x0001, /*1218F*/
  0x4B45, 0x0020, 0x0002, 0x0001, /*12190*/
  0x4B46, 0x0020, 0x0002, 0x0001, /*12191*/
  0x4B47, 0x0020, 0x0002, 0x0001, /*12192*/
  0x4B48, 0x0020, 0x0002, 0x0001, /*12193*/
  0x4B49, 0x0020, 0x0002, 0x0001, /*12194*/
  0x4B4A, 0x0020, 0x0002, 0x0001, /*12195*/
  0x4B4B, 0x0020, 0x0002, 0x0001, /*12196*/
  0x4B4C, 0x0020, 0x0002, 0x0001, /*12197*/
  0x4B4D, 0x0020, 0x0002, 0x0001, /*12198*/
  0x4B4E, 0x0020, 0x0002, 0x0001, /*12199*/
  0x4B4F, 0x0020, 0x0002, 0x0001, /*1219A*/
  0x4B50, 0x0020, 0x0002, 0x0001, /*1219B*/
  0x4B51, 0x0020, 0x0002, 0x0001, /*1219C*/
  0x4B52, 0x0020, 0x0002, 0x0001, /*1219D*/
  0x4B53, 0x0020, 0x0002, 0x0001, /*1219E*/
  0x4B54, 0x0020, 0x0002, 0x0001, /*1219F*/
  0x4B55, 0x0020, 0x0002, 0x0001, /*121A0*/
  0x4B56, 0x0020, 0x0002, 0x0001, /*121A1*/
  0x4B57, 0x0020, 0x0002, 0x0001, /*121A2*/
  0x4B58, 0x0020, 0x0002, 0x0001, /*121A3*/
  0x4B59, 0x0020, 0x0002, 0x0001, /*121A4*/
  0x4B5A, 0x0020, 0x0002, 0x0001, /*121A5*/
  0x4B5B, 0x0020, 0x0002, 0x0001, /*121A6*/
  0x4B5C, 0x0020, 0x0002, 0x0001, /*121A7*/
  0x4B5D, 0x0020, 0x0002, 0x0001, /*121A8*/
  0x4B5E, 0x0020, 0x0002, 0x0001, /*121A9*/
  0x4B5F, 0x0020, 0x0002, 0x0001, /*121AA*/
  0x4B60, 0x0020, 0x0002, 0x0001, /*121AB*/
  0x4B61, 0x0020, 0x0002, 0x0001, /*121AC*/
  0x4B62, 0x0020, 0x0002, 0x0001, /*121AD*/
  0x4B63, 0x0020, 0x0002, 0x0001, /*121AE*/
  0x4B64, 0x0020, 0x0002, 0x0001, /*121AF*/
  0x4B65, 0x0020, 0x0002, 0x0001, /*121B0*/
  0x4B66, 0x0020, 0x0002, 0x0001, /*121B1*/
  0x4B67, 0x0020, 0x0002, 0x0001, /*121B2*/
  0x4B68, 0x0020, 0x0002, 0x0001, /*121B3*/
  0x4B69, 0x0020, 0x0002, 0x0001, /*121B4*/
  0x4B6A, 0x0020, 0x0002, 0x0001, /*121B5*/
  0x4B6B, 0x0020, 0x0002, 0x0001, /*121B6*/
  0x4B6C, 0x0020, 0x0002, 0x0001, /*121B7*/
  0x4B6D, 0x0020, 0x0002, 0x0001, /*121B8*/
  0x4B6E, 0x0020, 0x0002, 0x0001, /*121B9*/
  0x4B6F, 0x0020, 0x0002, 0x0001, /*121BA*/
  0x4B70, 0x0020, 0x0002, 0x0001, /*121BB*/
  0x4B71, 0x0020, 0x0002, 0x0001, /*121BC*/
  0x4B72, 0x0020, 0x0002, 0x0001, /*121BD*/
  0x4B73, 0x0020, 0x0002, 0x0001, /*121BE*/
  0x4B74, 0x0020, 0x0002, 0x0001, /*121BF*/
  0x4B75, 0x0020, 0x0002, 0x0001, /*121C0*/
  0x4B76, 0x0020, 0x0002, 0x0001, /*121C1*/
  0x4B77, 0x0020, 0x0002, 0x0001, /*121C2*/
  0x4B78, 0x0020, 0x0002, 0x0001, /*121C3*/
  0x4B79, 0x0020, 0x0002, 0x0001, /*121C4*/
  0x4B7A, 0x0020, 0x0002, 0x0001, /*121C5*/
  0x4B7B, 0x0020, 0x0002, 0x0001, /*121C6*/
  0x4B7C, 0x0020, 0x0002, 0x0001, /*121C7*/
  0x4B7D, 0x0020, 0x0002, 0x0001, /*121C8*/
  0x4B7E, 0x0020, 0x0002, 0x0001, /*121C9*/
  0x4B7F, 0x0020, 0x0002, 0x0001, /*121CA*/
  0x4B80, 0x0020, 0x0002, 0x0001, /*121CB*/
  0x4B81, 0x0020, 0x0002, 0x0001, /*121CC*/
  0x4B82, 0x0020, 0x0002, 0x0001, /*121CD*/
  0x4B83, 0x0020, 0x0002, 0x0001, /*121CE*/
  0x4B84, 0x0020, 0x0002, 0x0001, /*121CF*/
  0x4B85, 0x0020, 0x0002, 0x0001, /*121D0*/
  0x4B86, 0x0020, 0x0002, 0x0001, /*121D1*/
  0x4B87, 0x0020, 0x0002, 0x0001, /*121D2*/
  0x4B88, 0x0020, 0x0002, 0x0001, /*121D3*/
  0x4B89, 0x0020, 0x0002, 0x0001, /*121D4*/
  0x4B8A, 0x0020, 0x0002, 0x0001, /*121D5*/
  0x4B8B, 0x0020, 0x0002, 0x0001, /*121D6*/
  0x4B8C, 0x0020, 0x0002, 0x0001, /*121D7*/
  0x4B8D, 0x0020, 0x0002, 0x0001, /*121D8*/
  0x4B8E, 0x0020, 0x0002, 0x0001, /*121D9*/
  0x4B8F, 0x0020, 0x0002, 0x0001, /*121DA*/
  0x4B90, 0x0020, 0x0002, 0x0001, /*121DB*/
  0x4B91, 0x0020, 0x0002, 0x0001, /*121DC*/
  0x4B92, 0x0020, 0x0002, 0x0001, /*121DD*/
  0x4B93, 0x0020, 0x0002, 0x0001, /*121DE*/
  0x4B94, 0x0020, 0x0002, 0x0001, /*121DF*/
  0x4B95, 0x0020, 0x0002, 0x0001, /*121E0*/
  0x4B96, 0x0020, 0x0002, 0x0001, /*121E1*/
  0x4B97, 0x0020, 0x0002, 0x0001, /*121E2*/
  0x4B98, 0x0020, 0x0002, 0x0001, /*121E3*/
  0x4B99, 0x0020, 0x0002, 0x0001, /*121E4*/
  0x4B9A, 0x0020, 0x0002, 0x0001, /*121E5*/
  0x4B9B, 0x0020, 0x0002, 0x0001, /*121E6*/
  0x4B9C, 0x0020, 0x0002, 0x0001, /*121E7*/
  0x4B9D, 0x0020, 0x0002, 0x0001, /*121E8*/
  0x4B9E, 0x0020, 0x0002, 0x0001, /*121E9*/
  0x4B9F, 0x0020, 0x0002, 0x0001, /*121EA*/
  0x4BA0, 0x0020, 0x0002, 0x0001, /*121EB*/
  0x4BA1, 0x0020, 0x0002, 0x0001, /*121EC*/
  0x4BA2, 0x0020, 0x0002, 0x0001, /*121ED*/
  0x4BA3, 0x0020, 0x0002, 0x0001, /*121EE*/
  0x4BA4, 0x0020, 0x0002, 0x0001, /*121EF*/
  0x4BA5, 0x0020, 0x0002, 0x0001, /*121F0*/
  0x4BA6, 0x0020, 0x0002, 0x0001, /*121F1*/
  0x4BA7, 0x0020, 0x0002, 0x0001, /*121F2*/
  0x4BA8, 0x0020, 0x0002, 0x0001, /*121F3*/
  0x4BA9, 0x0020, 0x0002, 0x0001, /*121F4*/
  0x4BAA, 0x0020, 0x0002, 0x0001, /*121F5*/
  0x4BAB, 0x0020, 0x0002, 0x0001, /*121F6*/
  0x4BAC, 0x0020, 0x0002, 0x0001, /*121F7*/
  0x4BAD, 0x0020, 0x0002, 0x0001, /*121F8*/
  0x4BAE, 0x0020, 0x0002, 0x0001, /*121F9*/
  0x4BAF, 0x0020, 0x0002, 0x0001, /*121FA*/
  0x4BB0, 0x0020, 0x0002, 0x0001, /*121FB*/
  0x4BB1, 0x0020, 0x0002, 0x0001, /*121FC*/
  0x4BB2, 0x0020, 0x0002, 0x0001, /*121FD*/
  0x4BB3, 0x0020, 0x0002, 0x0001, /*121FE*/
  0x4BB4, 0x0020, 0x0002, 0x0001  /*121FF*/
};

uint16 uca900_p122[]= {
  0x4BB5, 0x0020, 0x0002, 0x0001, /*12200*/
  0x4BB6, 0x0020, 0x0002, 0x0001, /*12201*/
  0x4BB7, 0x0020, 0x0002, 0x0001, /*12202*/
  0x4BB8, 0x0020, 0x0002, 0x0001, /*12203*/
  0x4BB9, 0x0020, 0x0002, 0x0001, /*12204*/
  0x4BBA, 0x0020, 0x0002, 0x0001, /*12205*/
  0x4BBB, 0x0020, 0x0002, 0x0001, /*12206*/
  0x4BBC, 0x0020, 0x0002, 0x0001, /*12207*/
  0x4BBD, 0x0020, 0x0002, 0x0001, /*12208*/
  0x4BBE, 0x0020, 0x0002, 0x0001, /*12209*/
  0x4BBF, 0x0020, 0x0002, 0x0001, /*1220A*/
  0x4BC0, 0x0020, 0x0002, 0x0001, /*1220B*/
  0x4BC1, 0x0020, 0x0002, 0x0001, /*1220C*/
  0x4BC2, 0x0020, 0x0002, 0x0001, /*1220D*/
  0x4BC3, 0x0020, 0x0002, 0x0001, /*1220E*/
  0x4BC4, 0x0020, 0x0002, 0x0001, /*1220F*/
  0x4BC5, 0x0020, 0x0002, 0x0001, /*12210*/
  0x4BC6, 0x0020, 0x0002, 0x0001, /*12211*/
  0x4BC7, 0x0020, 0x0002, 0x0001, /*12212*/
  0x4BC8, 0x0020, 0x0002, 0x0001, /*12213*/
  0x4BC9, 0x0020, 0x0002, 0x0001, /*12214*/
  0x4BCA, 0x0020, 0x0002, 0x0001, /*12215*/
  0x4BCB, 0x0020, 0x0002, 0x0001, /*12216*/
  0x4BCC, 0x0020, 0x0002, 0x0001, /*12217*/
  0x4BCD, 0x0020, 0x0002, 0x0001, /*12218*/
  0x4BCE, 0x0020, 0x0002, 0x0001, /*12219*/
  0x4BCF, 0x0020, 0x0002, 0x0001, /*1221A*/
  0x4BD0, 0x0020, 0x0002, 0x0001, /*1221B*/
  0x4BD1, 0x0020, 0x0002, 0x0001, /*1221C*/
  0x4BD2, 0x0020, 0x0002, 0x0001, /*1221D*/
  0x4BD3, 0x0020, 0x0002, 0x0001, /*1221E*/
  0x4BD4, 0x0020, 0x0002, 0x0001, /*1221F*/
  0x4BD5, 0x0020, 0x0002, 0x0001, /*12220*/
  0x4BD6, 0x0020, 0x0002, 0x0001, /*12221*/
  0x4BD7, 0x0020, 0x0002, 0x0001, /*12222*/
  0x4BD8, 0x0020, 0x0002, 0x0001, /*12223*/
  0x4BD9, 0x0020, 0x0002, 0x0001, /*12224*/
  0x4BDA, 0x0020, 0x0002, 0x0001, /*12225*/
  0x4BDB, 0x0020, 0x0002, 0x0001, /*12226*/
  0x4BDC, 0x0020, 0x0002, 0x0001, /*12227*/
  0x4BDD, 0x0020, 0x0002, 0x0001, /*12228*/
  0x4BDE, 0x0020, 0x0002, 0x0001, /*12229*/
  0x4BDF, 0x0020, 0x0002, 0x0001, /*1222A*/
  0x4BE0, 0x0020, 0x0002, 0x0001, /*1222B*/
  0x4BE1, 0x0020, 0x0002, 0x0001, /*1222C*/
  0x4BE2, 0x0020, 0x0002, 0x0001, /*1222D*/
  0x4BE3, 0x0020, 0x0002, 0x0001, /*1222E*/
  0x4BE4, 0x0020, 0x0002, 0x0001, /*1222F*/
  0x4BE5, 0x0020, 0x0002, 0x0001, /*12230*/
  0x4BE6, 0x0020, 0x0002, 0x0001, /*12231*/
  0x4BE7, 0x0020, 0x0002, 0x0001, /*12232*/
  0x4BE8, 0x0020, 0x0002, 0x0001, /*12233*/
  0x4BE9, 0x0020, 0x0002, 0x0001, /*12234*/
  0x4BEA, 0x0020, 0x0002, 0x0001, /*12235*/
  0x4BEB, 0x0020, 0x0002, 0x0001, /*12236*/
  0x4BEC, 0x0020, 0x0002, 0x0001, /*12237*/
  0x4BED, 0x0020, 0x0002, 0x0001, /*12238*/
  0x4BEE, 0x0020, 0x0002, 0x0001, /*12239*/
  0x4BEF, 0x0020, 0x0002, 0x0001, /*1223A*/
  0x4BF0, 0x0020, 0x0002, 0x0001, /*1223B*/
  0x4BF1, 0x0020, 0x0002, 0x0001, /*1223C*/
  0x4BF2, 0x0020, 0x0002, 0x0001, /*1223D*/
  0x4BF3, 0x0020, 0x0002, 0x0001, /*1223E*/
  0x4BF4, 0x0020, 0x0002, 0x0001, /*1223F*/
  0x4BF5, 0x0020, 0x0002, 0x0001, /*12240*/
  0x4BF6, 0x0020, 0x0002, 0x0001, /*12241*/
  0x4BF7, 0x0020, 0x0002, 0x0001, /*12242*/
  0x4BF8, 0x0020, 0x0002, 0x0001, /*12243*/
  0x4BF9, 0x0020, 0x0002, 0x0001, /*12244*/
  0x4BFA, 0x0020, 0x0002, 0x0001, /*12245*/
  0x4BFB, 0x0020, 0x0002, 0x0001, /*12246*/
  0x4BFC, 0x0020, 0x0002, 0x0001, /*12247*/
  0x4BFD, 0x0020, 0x0002, 0x0001, /*12248*/
  0x4BFE, 0x0020, 0x0002, 0x0001, /*12249*/
  0x4BFF, 0x0020, 0x0002, 0x0001, /*1224A*/
  0x4C00, 0x0020, 0x0002, 0x0001, /*1224B*/
  0x4C01, 0x0020, 0x0002, 0x0001, /*1224C*/
  0x4C02, 0x0020, 0x0002, 0x0001, /*1224D*/
  0x4C03, 0x0020, 0x0002, 0x0001, /*1224E*/
  0x4C04, 0x0020, 0x0002, 0x0001, /*1224F*/
  0x4C05, 0x0020, 0x0002, 0x0001, /*12250*/
  0x4C06, 0x0020, 0x0002, 0x0001, /*12251*/
  0x4C07, 0x0020, 0x0002, 0x0001, /*12252*/
  0x4C08, 0x0020, 0x0002, 0x0001, /*12253*/
  0x4C09, 0x0020, 0x0002, 0x0001, /*12254*/
  0x4C0A, 0x0020, 0x0002, 0x0001, /*12255*/
  0x4C0B, 0x0020, 0x0002, 0x0001, /*12256*/
  0x4C0C, 0x0020, 0x0002, 0x0001, /*12257*/
  0x4C0D, 0x0020, 0x0002, 0x0001, /*12258*/
  0x4C0E, 0x0020, 0x0002, 0x0001, /*12259*/
  0x4C0F, 0x0020, 0x0002, 0x0001, /*1225A*/
  0x4C10, 0x0020, 0x0002, 0x0001, /*1225B*/
  0x4C11, 0x0020, 0x0002, 0x0001, /*1225C*/
  0x4C12, 0x0020, 0x0002, 0x0001, /*1225D*/
  0x4C13, 0x0020, 0x0002, 0x0001, /*1225E*/
  0x4C14, 0x0020, 0x0002, 0x0001, /*1225F*/
  0x4C15, 0x0020, 0x0002, 0x0001, /*12260*/
  0x4C16, 0x0020, 0x0002, 0x0001, /*12261*/
  0x4C17, 0x0020, 0x0002, 0x0001, /*12262*/
  0x4C1A, 0x0020, 0x0002, 0x0001, /*12263*/
  0x4C1B, 0x0020, 0x0002, 0x0001, /*12264*/
  0x4C1C, 0x0020, 0x0002, 0x0001, /*12265*/
  0x4C1D, 0x0020, 0x0002, 0x0001, /*12266*/
  0x4C1E, 0x0020, 0x0002, 0x0001, /*12267*/
  0x4C1F, 0x0020, 0x0002, 0x0001, /*12268*/
  0x4C20, 0x0020, 0x0002, 0x0001, /*12269*/
  0x4C21, 0x0020, 0x0002, 0x0001, /*1226A*/
  0x4C22, 0x0020, 0x0002, 0x0001, /*1226B*/
  0x4C23, 0x0020, 0x0002, 0x0001, /*1226C*/
  0x4C24, 0x0020, 0x0002, 0x0001, /*1226D*/
  0x4C25, 0x0020, 0x0002, 0x0001, /*1226E*/
  0x4C26, 0x0020, 0x0002, 0x0001, /*1226F*/
  0x4C27, 0x0020, 0x0002, 0x0001, /*12270*/
  0x4C28, 0x0020, 0x0002, 0x0001, /*12271*/
  0x4C29, 0x0020, 0x0002, 0x0001, /*12272*/
  0x4C2A, 0x0020, 0x0002, 0x0001, /*12273*/
  0x4C2B, 0x0020, 0x0002, 0x0001, /*12274*/
  0x4C2C, 0x0020, 0x0002, 0x0001, /*12275*/
  0x4C2D, 0x0020, 0x0002, 0x0001, /*12276*/
  0x4C2E, 0x0020, 0x0002, 0x0001, /*12277*/
  0x4C2F, 0x0020, 0x0002, 0x0001, /*12278*/
  0x4C30, 0x0020, 0x0002, 0x0001, /*12279*/
  0x4C31, 0x0020, 0x0002, 0x0001, /*1227A*/
  0x4C32, 0x0020, 0x0002, 0x0001, /*1227B*/
  0x4C33, 0x0020, 0x0002, 0x0001, /*1227C*/
  0x4C34, 0x0020, 0x0002, 0x0001, /*1227D*/
  0x4C35, 0x0020, 0x0002, 0x0001, /*1227E*/
  0x4C36, 0x0020, 0x0002, 0x0001, /*1227F*/
  0x4C37, 0x0020, 0x0002, 0x0001, /*12280*/
  0x4C38, 0x0020, 0x0002, 0x0001, /*12281*/
  0x4C39, 0x0020, 0x0002, 0x0001, /*12282*/
  0x4C3A, 0x0020, 0x0002, 0x0001, /*12283*/
  0x4C3B, 0x0020, 0x0002, 0x0001, /*12284*/
  0x4C3C, 0x0020, 0x0002, 0x0001, /*12285*/
  0x4C3D, 0x0020, 0x0002, 0x0001, /*12286*/
  0x4C3E, 0x0020, 0x0002, 0x0001, /*12287*/
  0x4C3F, 0x0020, 0x0002, 0x0001, /*12288*/
  0x4C40, 0x0020, 0x0002, 0x0001, /*12289*/
  0x4C41, 0x0020, 0x0002, 0x0001, /*1228A*/
  0x4C42, 0x0020, 0x0002, 0x0001, /*1228B*/
  0x4C43, 0x0020, 0x0002, 0x0001, /*1228C*/
  0x4C44, 0x0020, 0x0002, 0x0001, /*1228D*/
  0x4C45, 0x0020, 0x0002, 0x0001, /*1228E*/
  0x4C46, 0x0020, 0x0002, 0x0001, /*1228F*/
  0x4C47, 0x0020, 0x0002, 0x0001, /*12290*/
  0x4C48, 0x0020, 0x0002, 0x0001, /*12291*/
  0x4C49, 0x0020, 0x0002, 0x0001, /*12292*/
  0x4C4A, 0x0020, 0x0002, 0x0001, /*12293*/
  0x4C4B, 0x0020, 0x0002, 0x0001, /*12294*/
  0x4C4C, 0x0020, 0x0002, 0x0001, /*12295*/
  0x4C4D, 0x0020, 0x0002, 0x0001, /*12296*/
  0x4C4E, 0x0020, 0x0002, 0x0001, /*12297*/
  0x4C4F, 0x0020, 0x0002, 0x0001, /*12298*/
  0x4C50, 0x0020, 0x0002, 0x0001, /*12299*/
  0x4C51, 0x0020, 0x0002, 0x0001, /*1229A*/
  0x4C52, 0x0020, 0x0002, 0x0001, /*1229B*/
  0x4C53, 0x0020, 0x0002, 0x0001, /*1229C*/
  0x4C54, 0x0020, 0x0002, 0x0001, /*1229D*/
  0x4C55, 0x0020, 0x0002, 0x0001, /*1229E*/
  0x4C56, 0x0020, 0x0002, 0x0001, /*1229F*/
  0x4C57, 0x0020, 0x0002, 0x0001, /*122A0*/
  0x4C58, 0x0020, 0x0002, 0x0001, /*122A1*/
  0x4C59, 0x0020, 0x0002, 0x0001, /*122A2*/
  0x4C5A, 0x0020, 0x0002, 0x0001, /*122A3*/
  0x4C5B, 0x0020, 0x0002, 0x0001, /*122A4*/
  0x4C5C, 0x0020, 0x0002, 0x0001, /*122A5*/
  0x4C5D, 0x0020, 0x0002, 0x0001, /*122A6*/
  0x4C5E, 0x0020, 0x0002, 0x0001, /*122A7*/
  0x4C5F, 0x0020, 0x0002, 0x0001, /*122A8*/
  0x4C60, 0x0020, 0x0002, 0x0001, /*122A9*/
  0x4C61, 0x0020, 0x0002, 0x0001, /*122AA*/
  0x4C62, 0x0020, 0x0002, 0x0001, /*122AB*/
  0x4C63, 0x0020, 0x0002, 0x0001, /*122AC*/
  0x4C64, 0x0020, 0x0002, 0x0001, /*122AD*/
  0x4C65, 0x0020, 0x0002, 0x0001, /*122AE*/
  0x4C66, 0x0020, 0x0002, 0x0001, /*122AF*/
  0x4C67, 0x0020, 0x0002, 0x0001, /*122B0*/
  0x4C68, 0x0020, 0x0002, 0x0001, /*122B1*/
  0x4C69, 0x0020, 0x0002, 0x0001, /*122B2*/
  0x4C6A, 0x0020, 0x0002, 0x0001, /*122B3*/
  0x4C6B, 0x0020, 0x0002, 0x0001, /*122B4*/
  0x4C6C, 0x0020, 0x0002, 0x0001, /*122B5*/
  0x4C6D, 0x0020, 0x0002, 0x0001, /*122B6*/
  0x4C6E, 0x0020, 0x0002, 0x0001, /*122B7*/
  0x4C6F, 0x0020, 0x0002, 0x0001, /*122B8*/
  0x4C70, 0x0020, 0x0002, 0x0001, /*122B9*/
  0x4C71, 0x0020, 0x0002, 0x0001, /*122BA*/
  0x4C72, 0x0020, 0x0002, 0x0001, /*122BB*/
  0x4C73, 0x0020, 0x0002, 0x0001, /*122BC*/
  0x4C74, 0x0020, 0x0002, 0x0001, /*122BD*/
  0x4C75, 0x0020, 0x0002, 0x0001, /*122BE*/
  0x4C76, 0x0020, 0x0002, 0x0001, /*122BF*/
  0x4C77, 0x0020, 0x0002, 0x0001, /*122C0*/
  0x4C78, 0x0020, 0x0002, 0x0001, /*122C1*/
  0x4C79, 0x0020, 0x0002, 0x0001, /*122C2*/
  0x4C7A, 0x0020, 0x0002, 0x0001, /*122C3*/
  0x4C7B, 0x0020, 0x0002, 0x0001, /*122C4*/
  0x4C7C, 0x0020, 0x0002, 0x0001, /*122C5*/
  0x4C7D, 0x0020, 0x0002, 0x0001, /*122C6*/
  0x4C7E, 0x0020, 0x0002, 0x0001, /*122C7*/
  0x4C7F, 0x0020, 0x0002, 0x0001, /*122C8*/
  0x4C80, 0x0020, 0x0002, 0x0001, /*122C9*/
  0x4C81, 0x0020, 0x0002, 0x0001, /*122CA*/
  0x4C82, 0x0020, 0x0002, 0x0001, /*122CB*/
  0x4C83, 0x0020, 0x0002, 0x0001, /*122CC*/
  0x4C84, 0x0020, 0x0002, 0x0001, /*122CD*/
  0x4C85, 0x0020, 0x0002, 0x0001, /*122CE*/
  0x4C86, 0x0020, 0x0002, 0x0001, /*122CF*/
  0x4C87, 0x0020, 0x0002, 0x0001, /*122D0*/
  0x4C88, 0x0020, 0x0002, 0x0001, /*122D1*/
  0x4C89, 0x0020, 0x0002, 0x0001, /*122D2*/
  0x4C8A, 0x0020, 0x0002, 0x0001, /*122D3*/
  0x4C18, 0x0020, 0x0002, 0x0001, /*122D4*/
  0x4C19, 0x0020, 0x0002, 0x0001, /*122D5*/
  0x4C8B, 0x0020, 0x0002, 0x0001, /*122D6*/
  0x4C8C, 0x0020, 0x0002, 0x0001, /*122D7*/
  0x4C8D, 0x0020, 0x0002, 0x0001, /*122D8*/
  0x4C8E, 0x0020, 0x0002, 0x0001, /*122D9*/
  0x4C8F, 0x0020, 0x0002, 0x0001, /*122DA*/
  0x4C90, 0x0020, 0x0002, 0x0001, /*122DB*/
  0x4C91, 0x0020, 0x0002, 0x0001, /*122DC*/
  0x4C92, 0x0020, 0x0002, 0x0001, /*122DD*/
  0x4C93, 0x0020, 0x0002, 0x0001, /*122DE*/
  0x4C94, 0x0020, 0x0002, 0x0001, /*122DF*/
  0x4C95, 0x0020, 0x0002, 0x0001, /*122E0*/
  0x4C96, 0x0020, 0x0002, 0x0001, /*122E1*/
  0x4C97, 0x0020, 0x0002, 0x0001, /*122E2*/
  0x4C98, 0x0020, 0x0002, 0x0001, /*122E3*/
  0x4C99, 0x0020, 0x0002, 0x0001, /*122E4*/
  0x4C9A, 0x0020, 0x0002, 0x0001, /*122E5*/
  0x4C9B, 0x0020, 0x0002, 0x0001, /*122E6*/
  0x4C9C, 0x0020, 0x0002, 0x0001, /*122E7*/
  0x4C9D, 0x0020, 0x0002, 0x0001, /*122E8*/
  0x4C9E, 0x0020, 0x0002, 0x0001, /*122E9*/
  0x4C9F, 0x0020, 0x0002, 0x0001, /*122EA*/
  0x4CA0, 0x0020, 0x0002, 0x0001, /*122EB*/
  0x4CA1, 0x0020, 0x0002, 0x0001, /*122EC*/
  0x4CA2, 0x0020, 0x0002, 0x0001, /*122ED*/
  0x4CA3, 0x0020, 0x0002, 0x0001, /*122EE*/
  0x4CA4, 0x0020, 0x0002, 0x0001, /*122EF*/
  0x4CA5, 0x0020, 0x0002, 0x0001, /*122F0*/
  0x4CA6, 0x0020, 0x0002, 0x0001, /*122F1*/
  0x4CA7, 0x0020, 0x0002, 0x0001, /*122F2*/
  0x4CA8, 0x0020, 0x0002, 0x0001, /*122F3*/
  0x4CA9, 0x0020, 0x0002, 0x0001, /*122F4*/
  0x4CAA, 0x0020, 0x0002, 0x0001, /*122F5*/
  0x4CAB, 0x0020, 0x0002, 0x0001, /*122F6*/
  0x4CAC, 0x0020, 0x0002, 0x0001, /*122F7*/
  0x4CAD, 0x0020, 0x0002, 0x0001, /*122F8*/
  0x4CAE, 0x0020, 0x0002, 0x0001, /*122F9*/
  0x4CAF, 0x0020, 0x0002, 0x0001, /*122FA*/
  0x4CB0, 0x0020, 0x0002, 0x0001, /*122FB*/
  0x4CB1, 0x0020, 0x0002, 0x0001, /*122FC*/
  0x4CB2, 0x0020, 0x0002, 0x0001, /*122FD*/
  0x4CB3, 0x0020, 0x0002, 0x0001, /*122FE*/
  0x4CB4, 0x0020, 0x0002, 0x0001  /*122FF*/
};

uint16 uca900_p123[]= {
  0x4CB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12300*/
  0x4CB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12301*/
  0x4CB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12302*/
  0x4CB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12303*/
  0x4CB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12304*/
  0x4CBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12305*/
  0x4CBB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12306*/
  0x4CBC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12307*/
  0x4CBD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12308*/
  0x4CBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12309*/
  0x4CBF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1230A*/
  0x4CC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1230B*/
  0x4CC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1230C*/
  0x4CC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1230D*/
  0x4CC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1230E*/
  0x4CC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1230F*/
  0x4CC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12310*/
  0x4CC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12311*/
  0x4CC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12312*/
  0x4CC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12313*/
  0x4CC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12314*/
  0x4CCA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12315*/
  0x4CCB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12316*/
  0x4CCC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12317*/
  0x4CCD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12318*/
  0x4CCE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12319*/
  0x4CCF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1231A*/
  0x4CD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1231B*/
  0x4CD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1231C*/
  0x4CD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1231D*/
  0x4CD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1231E*/
  0x4CD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1231F*/
  0x4CD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12320*/
  0x4CD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12321*/
  0x4CD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12322*/
  0x4CD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12323*/
  0x4CD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12324*/
  0x4CDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12325*/
  0x4CDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12326*/
  0x4CDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12327*/
  0x4CDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12328*/
  0x4CDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12329*/
  0x4CDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1232A*/
  0x4CE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1232B*/
  0x4CE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1232C*/
  0x4CE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1232D*/
  0x4CE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1232E*/
  0x4CE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1232F*/
  0x4CE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12330*/
  0x4CE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12331*/
  0x4CE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12332*/
  0x4CE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12333*/
  0x4CE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12334*/
  0x4CEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12335*/
  0x4CEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12336*/
  0x4CEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12337*/
  0x4CED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12338*/
  0x4CEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12339*/
  0x4CEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1233A*/
  0x4CF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1233B*/
  0x4CF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1233C*/
  0x4CF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1233D*/
  0x4CF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1233E*/
  0x4CF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1233F*/
  0x4CF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12340*/
  0x4CF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12341*/
  0x4CF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12342*/
  0x4CF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12343*/
  0x4CF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12344*/
  0x4CFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12345*/
  0x4CFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12346*/
  0x4CFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12347*/
  0x4CFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12348*/
  0x4CFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12349*/
  0x4CFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1234A*/
  0x4D00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1234B*/
  0x4D01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1234C*/
  0x4D02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1234D*/
  0x4D03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1234E*/
  0x4D04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1234F*/
  0x4D05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12350*/
  0x4D06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12351*/
  0x4D07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12352*/
  0x4D08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12353*/
  0x4D09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12354*/
  0x4D0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12355*/
  0x4D0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12356*/
  0x4D0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12357*/
  0x4D0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12358*/
  0x4D0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12359*/
  0x4D0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1235A*/
  0x4D10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1235B*/
  0x4D11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1235C*/
  0x4D12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1235D*/
  0x4D13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1235E*/
  0x4D14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1235F*/
  0x4D15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12360*/
  0x4D16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12361*/
  0x4D17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12362*/
  0x4D18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12363*/
  0x4D19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12364*/
  0x4D1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12365*/
  0x4D1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12366*/
  0x4D1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12367*/
  0x4D1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12368*/
  0x4D1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12369*/
  0x4D1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1236A*/
  0x4D20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1236B*/
  0x4D21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1236C*/
  0x4D22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1236D*/
  0x4D23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1236E*/
  0x4D24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1236F*/
  0x4D25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12370*/
  0x4D26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12371*/
  0x4D27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12372*/
  0x4D28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12373*/
  0x4D29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12374*/
  0x4D2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12375*/
  0x4D2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12376*/
  0x4D2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12377*/
  0x4D2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12378*/
  0x4D2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12379*/
  0x4D2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1237A*/
  0x4D30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1237B*/
  0x4D31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1237C*/
  0x4D32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1237D*/
  0x4D33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1237E*/
  0x4D34, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1237F*/
  0x4D35, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12380*/
  0x4D36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12381*/
  0x4D37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12382*/
  0x4D38, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12383*/
  0x4D39, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12384*/
  0x4D3A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12385*/
  0x4D3B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12386*/
  0x4D3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12387*/
  0x4D3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12388*/
  0x4D3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12389*/
  0x4D3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1238A*/
  0x4D40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1238B*/
  0x4D41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1238C*/
  0x4D42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1238D*/
  0x4D43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1238E*/
  0x4D44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1238F*/
  0x4D45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12390*/
  0x4D46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12391*/
  0x4D47, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12392*/
  0x4D48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12393*/
  0x4D49, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12394*/
  0x4D4A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12395*/
  0x4D4B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12396*/
  0x4D4C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12397*/
  0x4D4D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12398*/
  0x4D4E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12399*/
  0xFBC2, 0x0020, 0x0002, 0xA39A, 0x0000, 0x0000, 0x0002, /*1239A*/
  0xFBC2, 0x0020, 0x0002, 0xA39B, 0x0000, 0x0000, 0x0002, /*1239B*/
  0xFBC2, 0x0020, 0x0002, 0xA39C, 0x0000, 0x0000, 0x0002, /*1239C*/
  0xFBC2, 0x0020, 0x0002, 0xA39D, 0x0000, 0x0000, 0x0002, /*1239D*/
  0xFBC2, 0x0020, 0x0002, 0xA39E, 0x0000, 0x0000, 0x0002, /*1239E*/
  0xFBC2, 0x0020, 0x0002, 0xA39F, 0x0000, 0x0000, 0x0002, /*1239F*/
  0xFBC2, 0x0020, 0x0002, 0xA3A0, 0x0000, 0x0000, 0x0002, /*123A0*/
  0xFBC2, 0x0020, 0x0002, 0xA3A1, 0x0000, 0x0000, 0x0002, /*123A1*/
  0xFBC2, 0x0020, 0x0002, 0xA3A2, 0x0000, 0x0000, 0x0002, /*123A2*/
  0xFBC2, 0x0020, 0x0002, 0xA3A3, 0x0000, 0x0000, 0x0002, /*123A3*/
  0xFBC2, 0x0020, 0x0002, 0xA3A4, 0x0000, 0x0000, 0x0002, /*123A4*/
  0xFBC2, 0x0020, 0x0002, 0xA3A5, 0x0000, 0x0000, 0x0002, /*123A5*/
  0xFBC2, 0x0020, 0x0002, 0xA3A6, 0x0000, 0x0000, 0x0002, /*123A6*/
  0xFBC2, 0x0020, 0x0002, 0xA3A7, 0x0000, 0x0000, 0x0002, /*123A7*/
  0xFBC2, 0x0020, 0x0002, 0xA3A8, 0x0000, 0x0000, 0x0002, /*123A8*/
  0xFBC2, 0x0020, 0x0002, 0xA3A9, 0x0000, 0x0000, 0x0002, /*123A9*/
  0xFBC2, 0x0020, 0x0002, 0xA3AA, 0x0000, 0x0000, 0x0002, /*123AA*/
  0xFBC2, 0x0020, 0x0002, 0xA3AB, 0x0000, 0x0000, 0x0002, /*123AB*/
  0xFBC2, 0x0020, 0x0002, 0xA3AC, 0x0000, 0x0000, 0x0002, /*123AC*/
  0xFBC2, 0x0020, 0x0002, 0xA3AD, 0x0000, 0x0000, 0x0002, /*123AD*/
  0xFBC2, 0x0020, 0x0002, 0xA3AE, 0x0000, 0x0000, 0x0002, /*123AE*/
  0xFBC2, 0x0020, 0x0002, 0xA3AF, 0x0000, 0x0000, 0x0002, /*123AF*/
  0xFBC2, 0x0020, 0x0002, 0xA3B0, 0x0000, 0x0000, 0x0002, /*123B0*/
  0xFBC2, 0x0020, 0x0002, 0xA3B1, 0x0000, 0x0000, 0x0002, /*123B1*/
  0xFBC2, 0x0020, 0x0002, 0xA3B2, 0x0000, 0x0000, 0x0002, /*123B2*/
  0xFBC2, 0x0020, 0x0002, 0xA3B3, 0x0000, 0x0000, 0x0002, /*123B3*/
  0xFBC2, 0x0020, 0x0002, 0xA3B4, 0x0000, 0x0000, 0x0002, /*123B4*/
  0xFBC2, 0x0020, 0x0002, 0xA3B5, 0x0000, 0x0000, 0x0002, /*123B5*/
  0xFBC2, 0x0020, 0x0002, 0xA3B6, 0x0000, 0x0000, 0x0002, /*123B6*/
  0xFBC2, 0x0020, 0x0002, 0xA3B7, 0x0000, 0x0000, 0x0002, /*123B7*/
  0xFBC2, 0x0020, 0x0002, 0xA3B8, 0x0000, 0x0000, 0x0002, /*123B8*/
  0xFBC2, 0x0020, 0x0002, 0xA3B9, 0x0000, 0x0000, 0x0002, /*123B9*/
  0xFBC2, 0x0020, 0x0002, 0xA3BA, 0x0000, 0x0000, 0x0002, /*123BA*/
  0xFBC2, 0x0020, 0x0002, 0xA3BB, 0x0000, 0x0000, 0x0002, /*123BB*/
  0xFBC2, 0x0020, 0x0002, 0xA3BC, 0x0000, 0x0000, 0x0002, /*123BC*/
  0xFBC2, 0x0020, 0x0002, 0xA3BD, 0x0000, 0x0000, 0x0002, /*123BD*/
  0xFBC2, 0x0020, 0x0002, 0xA3BE, 0x0000, 0x0000, 0x0002, /*123BE*/
  0xFBC2, 0x0020, 0x0002, 0xA3BF, 0x0000, 0x0000, 0x0002, /*123BF*/
  0xFBC2, 0x0020, 0x0002, 0xA3C0, 0x0000, 0x0000, 0x0002, /*123C0*/
  0xFBC2, 0x0020, 0x0002, 0xA3C1, 0x0000, 0x0000, 0x0002, /*123C1*/
  0xFBC2, 0x0020, 0x0002, 0xA3C2, 0x0000, 0x0000, 0x0002, /*123C2*/
  0xFBC2, 0x0020, 0x0002, 0xA3C3, 0x0000, 0x0000, 0x0002, /*123C3*/
  0xFBC2, 0x0020, 0x0002, 0xA3C4, 0x0000, 0x0000, 0x0002, /*123C4*/
  0xFBC2, 0x0020, 0x0002, 0xA3C5, 0x0000, 0x0000, 0x0002, /*123C5*/
  0xFBC2, 0x0020, 0x0002, 0xA3C6, 0x0000, 0x0000, 0x0002, /*123C6*/
  0xFBC2, 0x0020, 0x0002, 0xA3C7, 0x0000, 0x0000, 0x0002, /*123C7*/
  0xFBC2, 0x0020, 0x0002, 0xA3C8, 0x0000, 0x0000, 0x0002, /*123C8*/
  0xFBC2, 0x0020, 0x0002, 0xA3C9, 0x0000, 0x0000, 0x0002, /*123C9*/
  0xFBC2, 0x0020, 0x0002, 0xA3CA, 0x0000, 0x0000, 0x0002, /*123CA*/
  0xFBC2, 0x0020, 0x0002, 0xA3CB, 0x0000, 0x0000, 0x0002, /*123CB*/
  0xFBC2, 0x0020, 0x0002, 0xA3CC, 0x0000, 0x0000, 0x0002, /*123CC*/
  0xFBC2, 0x0020, 0x0002, 0xA3CD, 0x0000, 0x0000, 0x0002, /*123CD*/
  0xFBC2, 0x0020, 0x0002, 0xA3CE, 0x0000, 0x0000, 0x0002, /*123CE*/
  0xFBC2, 0x0020, 0x0002, 0xA3CF, 0x0000, 0x0000, 0x0002, /*123CF*/
  0xFBC2, 0x0020, 0x0002, 0xA3D0, 0x0000, 0x0000, 0x0002, /*123D0*/
  0xFBC2, 0x0020, 0x0002, 0xA3D1, 0x0000, 0x0000, 0x0002, /*123D1*/
  0xFBC2, 0x0020, 0x0002, 0xA3D2, 0x0000, 0x0000, 0x0002, /*123D2*/
  0xFBC2, 0x0020, 0x0002, 0xA3D3, 0x0000, 0x0000, 0x0002, /*123D3*/
  0xFBC2, 0x0020, 0x0002, 0xA3D4, 0x0000, 0x0000, 0x0002, /*123D4*/
  0xFBC2, 0x0020, 0x0002, 0xA3D5, 0x0000, 0x0000, 0x0002, /*123D5*/
  0xFBC2, 0x0020, 0x0002, 0xA3D6, 0x0000, 0x0000, 0x0002, /*123D6*/
  0xFBC2, 0x0020, 0x0002, 0xA3D7, 0x0000, 0x0000, 0x0002, /*123D7*/
  0xFBC2, 0x0020, 0x0002, 0xA3D8, 0x0000, 0x0000, 0x0002, /*123D8*/
  0xFBC2, 0x0020, 0x0002, 0xA3D9, 0x0000, 0x0000, 0x0002, /*123D9*/
  0xFBC2, 0x0020, 0x0002, 0xA3DA, 0x0000, 0x0000, 0x0002, /*123DA*/
  0xFBC2, 0x0020, 0x0002, 0xA3DB, 0x0000, 0x0000, 0x0002, /*123DB*/
  0xFBC2, 0x0020, 0x0002, 0xA3DC, 0x0000, 0x0000, 0x0002, /*123DC*/
  0xFBC2, 0x0020, 0x0002, 0xA3DD, 0x0000, 0x0000, 0x0002, /*123DD*/
  0xFBC2, 0x0020, 0x0002, 0xA3DE, 0x0000, 0x0000, 0x0002, /*123DE*/
  0xFBC2, 0x0020, 0x0002, 0xA3DF, 0x0000, 0x0000, 0x0002, /*123DF*/
  0xFBC2, 0x0020, 0x0002, 0xA3E0, 0x0000, 0x0000, 0x0002, /*123E0*/
  0xFBC2, 0x0020, 0x0002, 0xA3E1, 0x0000, 0x0000, 0x0002, /*123E1*/
  0xFBC2, 0x0020, 0x0002, 0xA3E2, 0x0000, 0x0000, 0x0002, /*123E2*/
  0xFBC2, 0x0020, 0x0002, 0xA3E3, 0x0000, 0x0000, 0x0002, /*123E3*/
  0xFBC2, 0x0020, 0x0002, 0xA3E4, 0x0000, 0x0000, 0x0002, /*123E4*/
  0xFBC2, 0x0020, 0x0002, 0xA3E5, 0x0000, 0x0000, 0x0002, /*123E5*/
  0xFBC2, 0x0020, 0x0002, 0xA3E6, 0x0000, 0x0000, 0x0002, /*123E6*/
  0xFBC2, 0x0020, 0x0002, 0xA3E7, 0x0000, 0x0000, 0x0002, /*123E7*/
  0xFBC2, 0x0020, 0x0002, 0xA3E8, 0x0000, 0x0000, 0x0002, /*123E8*/
  0xFBC2, 0x0020, 0x0002, 0xA3E9, 0x0000, 0x0000, 0x0002, /*123E9*/
  0xFBC2, 0x0020, 0x0002, 0xA3EA, 0x0000, 0x0000, 0x0002, /*123EA*/
  0xFBC2, 0x0020, 0x0002, 0xA3EB, 0x0000, 0x0000, 0x0002, /*123EB*/
  0xFBC2, 0x0020, 0x0002, 0xA3EC, 0x0000, 0x0000, 0x0002, /*123EC*/
  0xFBC2, 0x0020, 0x0002, 0xA3ED, 0x0000, 0x0000, 0x0002, /*123ED*/
  0xFBC2, 0x0020, 0x0002, 0xA3EE, 0x0000, 0x0000, 0x0002, /*123EE*/
  0xFBC2, 0x0020, 0x0002, 0xA3EF, 0x0000, 0x0000, 0x0002, /*123EF*/
  0xFBC2, 0x0020, 0x0002, 0xA3F0, 0x0000, 0x0000, 0x0002, /*123F0*/
  0xFBC2, 0x0020, 0x0002, 0xA3F1, 0x0000, 0x0000, 0x0002, /*123F1*/
  0xFBC2, 0x0020, 0x0002, 0xA3F2, 0x0000, 0x0000, 0x0002, /*123F2*/
  0xFBC2, 0x0020, 0x0002, 0xA3F3, 0x0000, 0x0000, 0x0002, /*123F3*/
  0xFBC2, 0x0020, 0x0002, 0xA3F4, 0x0000, 0x0000, 0x0002, /*123F4*/
  0xFBC2, 0x0020, 0x0002, 0xA3F5, 0x0000, 0x0000, 0x0002, /*123F5*/
  0xFBC2, 0x0020, 0x0002, 0xA3F6, 0x0000, 0x0000, 0x0002, /*123F6*/
  0xFBC2, 0x0020, 0x0002, 0xA3F7, 0x0000, 0x0000, 0x0002, /*123F7*/
  0xFBC2, 0x0020, 0x0002, 0xA3F8, 0x0000, 0x0000, 0x0002, /*123F8*/
  0xFBC2, 0x0020, 0x0002, 0xA3F9, 0x0000, 0x0000, 0x0002, /*123F9*/
  0xFBC2, 0x0020, 0x0002, 0xA3FA, 0x0000, 0x0000, 0x0002, /*123FA*/
  0xFBC2, 0x0020, 0x0002, 0xA3FB, 0x0000, 0x0000, 0x0002, /*123FB*/
  0xFBC2, 0x0020, 0x0002, 0xA3FC, 0x0000, 0x0000, 0x0002, /*123FC*/
  0xFBC2, 0x0020, 0x0002, 0xA3FD, 0x0000, 0x0000, 0x0002, /*123FD*/
  0xFBC2, 0x0020, 0x0002, 0xA3FE, 0x0000, 0x0000, 0x0002, /*123FE*/
  0xFBC2, 0x0020, 0x0002, 0xA3FF, 0x0000, 0x0000, 0x0002  /*123FF*/
};

uint16 uca900_p124[]= {
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12400*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12401*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12402*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12403*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12404*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12405*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12406*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12407*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12408*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12409*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1240A*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1240B*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1240C*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1240D*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1240E*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1240F*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12410*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12411*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12412*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12413*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12414*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12415*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12416*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12417*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12418*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12419*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1241A*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1241B*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1241C*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1241D*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1241E*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1241F*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12420*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12421*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12422*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12423*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12424*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12425*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12426*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12427*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12428*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12429*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1242A*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1242B*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1242C*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1242D*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1242E*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1242F*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12430*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12431*/
  0x1BDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12432*/
  0x1BDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12433*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12434*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12435*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12436*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12437*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12438*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12439*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1243A*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1243B*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1243C*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1243D*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1243E*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1243F*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12440*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12441*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12442*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12443*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12444*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12445*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12446*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12447*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12448*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12449*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1244A*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1244B*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1244C*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1244D*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1244E*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1244F*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12450*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12451*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12452*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12453*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12454*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12455*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12456*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12457*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12458*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12459*/
  0x1BE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1245A*/
  0x1BE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1245B*/
  0x1BE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1245C*/
  0x1BE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1245D*/
  0x1BE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1245E*/
  0x1BE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1245F*/
  0x1BE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12460*/
  0x1BE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12461*/
  0x1BE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12462*/
  0x1BE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12463*/
  0x1BEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12464*/
  0x1BEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12465*/
  0x1BEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12466*/
  0x1BED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12467*/
  0x1BEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12468*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12469*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1246A*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1246B*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1246C*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1246D*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1246E*/
  0xFBC2, 0x0020, 0x0002, 0xA46F, 0x0000, 0x0000, 0x0002, /*1246F*/
  0x0300, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12470*/
  0x0301, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12471*/
  0x0302, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12472*/
  0x0303, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12473*/
  0x0304, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12474*/
  0xFBC2, 0x0020, 0x0002, 0xA475, 0x0000, 0x0000, 0x0002, /*12475*/
  0xFBC2, 0x0020, 0x0002, 0xA476, 0x0000, 0x0000, 0x0002, /*12476*/
  0xFBC2, 0x0020, 0x0002, 0xA477, 0x0000, 0x0000, 0x0002, /*12477*/
  0xFBC2, 0x0020, 0x0002, 0xA478, 0x0000, 0x0000, 0x0002, /*12478*/
  0xFBC2, 0x0020, 0x0002, 0xA479, 0x0000, 0x0000, 0x0002, /*12479*/
  0xFBC2, 0x0020, 0x0002, 0xA47A, 0x0000, 0x0000, 0x0002, /*1247A*/
  0xFBC2, 0x0020, 0x0002, 0xA47B, 0x0000, 0x0000, 0x0002, /*1247B*/
  0xFBC2, 0x0020, 0x0002, 0xA47C, 0x0000, 0x0000, 0x0002, /*1247C*/
  0xFBC2, 0x0020, 0x0002, 0xA47D, 0x0000, 0x0000, 0x0002, /*1247D*/
  0xFBC2, 0x0020, 0x0002, 0xA47E, 0x0000, 0x0000, 0x0002, /*1247E*/
  0xFBC2, 0x0020, 0x0002, 0xA47F, 0x0000, 0x0000, 0x0002, /*1247F*/
  0x4D4F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12480*/
  0x4D50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12481*/
  0x4D51, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12482*/
  0x4D52, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12483*/
  0x4D53, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12484*/
  0x4D54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12485*/
  0x4D55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12486*/
  0x4D56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12487*/
  0x4D57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12488*/
  0x4D58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12489*/
  0x4D59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1248A*/
  0x4D5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1248B*/
  0x4D5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1248C*/
  0x4D5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1248D*/
  0x4D5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1248E*/
  0x4D5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1248F*/
  0x4D5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12490*/
  0x4D60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12491*/
  0x4D61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12492*/
  0x4D62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12493*/
  0x4D63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12494*/
  0x4D64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12495*/
  0x4D65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12496*/
  0x4D66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12497*/
  0x4D67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12498*/
  0x4D68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12499*/
  0x4D69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1249A*/
  0x4D6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1249B*/
  0x4D6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1249C*/
  0x4D6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1249D*/
  0x4D6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1249E*/
  0x4D6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1249F*/
  0x4D6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124A0*/
  0x4D70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124A1*/
  0x4D71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124A2*/
  0x4D72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124A3*/
  0x4D73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124A4*/
  0x4D74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124A5*/
  0x4D75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124A6*/
  0x4D76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124A7*/
  0x4D77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124A8*/
  0x4D78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124A9*/
  0x4D79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124AA*/
  0x4D7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124AB*/
  0x4D7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124AC*/
  0x4D7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124AD*/
  0x4D7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124AE*/
  0x4D7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124AF*/
  0x4D7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124B0*/
  0x4D80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124B1*/
  0x4D81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124B2*/
  0x4D82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124B3*/
  0x4D83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124B4*/
  0x4D84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124B5*/
  0x4D85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124B6*/
  0x4D86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124B7*/
  0x4D87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124B8*/
  0x4D88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124B9*/
  0x4D89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124BA*/
  0x4D8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124BB*/
  0x4D8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124BC*/
  0x4D8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124BD*/
  0x4D8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124BE*/
  0x4D8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124BF*/
  0x4D8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124C0*/
  0x4D90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124C1*/
  0x4D91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124C2*/
  0x4D92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124C3*/
  0x4D93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124C4*/
  0x4D94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124C5*/
  0x4D95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124C6*/
  0x4D96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124C7*/
  0x4D97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124C8*/
  0x4D98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124C9*/
  0x4D99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124CA*/
  0x4D9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124CB*/
  0x4D9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124CC*/
  0x4D9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124CD*/
  0x4D9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124CE*/
  0x4D9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124CF*/
  0x4D9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124D0*/
  0x4DA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124D1*/
  0x4DA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124D2*/
  0x4DA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124D3*/
  0x4DA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124D4*/
  0x4DA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124D5*/
  0x4DA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124D6*/
  0x4DA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124D7*/
  0x4DA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124D8*/
  0x4DA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124D9*/
  0x4DA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124DA*/
  0x4DAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124DB*/
  0x4DAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124DC*/
  0x4DAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124DD*/
  0x4DAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124DE*/
  0x4DAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124DF*/
  0x4DAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124E0*/
  0x4DB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124E1*/
  0x4DB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124E2*/
  0x4DB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124E3*/
  0x4DB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124E4*/
  0x4DB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124E5*/
  0x4DB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124E6*/
  0x4DB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124E7*/
  0x4DB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124E8*/
  0x4DB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124E9*/
  0x4DB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124EA*/
  0x4DBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124EB*/
  0x4DBB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124EC*/
  0x4DBC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124ED*/
  0x4DBD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124EE*/
  0x4DBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124EF*/
  0x4DBF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124F0*/
  0x4DC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124F1*/
  0x4DC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124F2*/
  0x4DC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124F3*/
  0x4DC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124F4*/
  0x4DC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124F5*/
  0x4DC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124F6*/
  0x4DC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124F7*/
  0x4DC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124F8*/
  0x4DC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124F9*/
  0x4DC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124FA*/
  0x4DCA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124FB*/
  0x4DCB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124FC*/
  0x4DCC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124FD*/
  0x4DCD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*124FE*/
  0x4DCE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001  /*124FF*/
};

uint16 uca900_p125[]= {
  0x4DCF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12500*/
  0x4DD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12501*/
  0x4DD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12502*/
  0x4DD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12503*/
  0x4DD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12504*/
  0x4DD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12505*/
  0x4DD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12506*/
  0x4DD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12507*/
  0x4DD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12508*/
  0x4DD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12509*/
  0x4DD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1250A*/
  0x4DDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1250B*/
  0x4DDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1250C*/
  0x4DDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1250D*/
  0x4DDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1250E*/
  0x4DDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1250F*/
  0x4DDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12510*/
  0x4DE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12511*/
  0x4DE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12512*/
  0x4DE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12513*/
  0x4DE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12514*/
  0x4DE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12515*/
  0x4DE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12516*/
  0x4DE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12517*/
  0x4DE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12518*/
  0x4DE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12519*/
  0x4DE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1251A*/
  0x4DEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1251B*/
  0x4DEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1251C*/
  0x4DEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1251D*/
  0x4DED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1251E*/
  0x4DEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1251F*/
  0x4DEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12520*/
  0x4DF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12521*/
  0x4DF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12522*/
  0x4DF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12523*/
  0x4DF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12524*/
  0x4DF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12525*/
  0x4DF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12526*/
  0x4DF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12527*/
  0x4DF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12528*/
  0x4DF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12529*/
  0x4DF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1252A*/
  0x4DFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1252B*/
  0x4DFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1252C*/
  0x4DFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1252D*/
  0x4DFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1252E*/
  0x4DFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1252F*/
  0x4DFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12530*/
  0x4E00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12531*/
  0x4E01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12532*/
  0x4E02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12533*/
  0x4E03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12534*/
  0x4E04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12535*/
  0x4E05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12536*/
  0x4E06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12537*/
  0x4E07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12538*/
  0x4E08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12539*/
  0x4E09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1253A*/
  0x4E0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1253B*/
  0x4E0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1253C*/
  0x4E0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1253D*/
  0x4E0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1253E*/
  0x4E0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1253F*/
  0x4E0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12540*/
  0x4E10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12541*/
  0x4E11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12542*/
  0x4E12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*12543*/
  0xFBC2, 0x0020, 0x0002, 0xA544, 0x0000, 0x0000, 0x0002, /*12544*/
  0xFBC2, 0x0020, 0x0002, 0xA545, 0x0000, 0x0000, 0x0002, /*12545*/
  0xFBC2, 0x0020, 0x0002, 0xA546, 0x0000, 0x0000, 0x0002, /*12546*/
  0xFBC2, 0x0020, 0x0002, 0xA547, 0x0000, 0x0000, 0x0002, /*12547*/
  0xFBC2, 0x0020, 0x0002, 0xA548, 0x0000, 0x0000, 0x0002, /*12548*/
  0xFBC2, 0x0020, 0x0002, 0xA549, 0x0000, 0x0000, 0x0002, /*12549*/
  0xFBC2, 0x0020, 0x0002, 0xA54A, 0x0000, 0x0000, 0x0002, /*1254A*/
  0xFBC2, 0x0020, 0x0002, 0xA54B, 0x0000, 0x0000, 0x0002, /*1254B*/
  0xFBC2, 0x0020, 0x0002, 0xA54C, 0x0000, 0x0000, 0x0002, /*1254C*/
  0xFBC2, 0x0020, 0x0002, 0xA54D, 0x0000, 0x0000, 0x0002, /*1254D*/
  0xFBC2, 0x0020, 0x0002, 0xA54E, 0x0000, 0x0000, 0x0002, /*1254E*/
  0xFBC2, 0x0020, 0x0002, 0xA54F, 0x0000, 0x0000, 0x0002, /*1254F*/
  0xFBC2, 0x0020, 0x0002, 0xA550, 0x0000, 0x0000, 0x0002, /*12550*/
  0xFBC2, 0x0020, 0x0002, 0xA551, 0x0000, 0x0000, 0x0002, /*12551*/
  0xFBC2, 0x0020, 0x0002, 0xA552, 0x0000, 0x0000, 0x0002, /*12552*/
  0xFBC2, 0x0020, 0x0002, 0xA553, 0x0000, 0x0000, 0x0002, /*12553*/
  0xFBC2, 0x0020, 0x0002, 0xA554, 0x0000, 0x0000, 0x0002, /*12554*/
  0xFBC2, 0x0020, 0x0002, 0xA555, 0x0000, 0x0000, 0x0002, /*12555*/
  0xFBC2, 0x0020, 0x0002, 0xA556, 0x0000, 0x0000, 0x0002, /*12556*/
  0xFBC2, 0x0020, 0x0002, 0xA557, 0x0000, 0x0000, 0x0002, /*12557*/
  0xFBC2, 0x0020, 0x0002, 0xA558, 0x0000, 0x0000, 0x0002, /*12558*/
  0xFBC2, 0x0020, 0x0002, 0xA559, 0x0000, 0x0000, 0x0002, /*12559*/
  0xFBC2, 0x0020, 0x0002, 0xA55A, 0x0000, 0x0000, 0x0002, /*1255A*/
  0xFBC2, 0x0020, 0x0002, 0xA55B, 0x0000, 0x0000, 0x0002, /*1255B*/
  0xFBC2, 0x0020, 0x0002, 0xA55C, 0x0000, 0x0000, 0x0002, /*1255C*/
  0xFBC2, 0x0020, 0x0002, 0xA55D, 0x0000, 0x0000, 0x0002, /*1255D*/
  0xFBC2, 0x0020, 0x0002, 0xA55E, 0x0000, 0x0000, 0x0002, /*1255E*/
  0xFBC2, 0x0020, 0x0002, 0xA55F, 0x0000, 0x0000, 0x0002, /*1255F*/
  0xFBC2, 0x0020, 0x0002, 0xA560, 0x0000, 0x0000, 0x0002, /*12560*/
  0xFBC2, 0x0020, 0x0002, 0xA561, 0x0000, 0x0000, 0x0002, /*12561*/
  0xFBC2, 0x0020, 0x0002, 0xA562, 0x0000, 0x0000, 0x0002, /*12562*/
  0xFBC2, 0x0020, 0x0002, 0xA563, 0x0000, 0x0000, 0x0002, /*12563*/
  0xFBC2, 0x0020, 0x0002, 0xA564, 0x0000, 0x0000, 0x0002, /*12564*/
  0xFBC2, 0x0020, 0x0002, 0xA565, 0x0000, 0x0000, 0x0002, /*12565*/
  0xFBC2, 0x0020, 0x0002, 0xA566, 0x0000, 0x0000, 0x0002, /*12566*/
  0xFBC2, 0x0020, 0x0002, 0xA567, 0x0000, 0x0000, 0x0002, /*12567*/
  0xFBC2, 0x0020, 0x0002, 0xA568, 0x0000, 0x0000, 0x0002, /*12568*/
  0xFBC2, 0x0020, 0x0002, 0xA569, 0x0000, 0x0000, 0x0002, /*12569*/
  0xFBC2, 0x0020, 0x0002, 0xA56A, 0x0000, 0x0000, 0x0002, /*1256A*/
  0xFBC2, 0x0020, 0x0002, 0xA56B, 0x0000, 0x0000, 0x0002, /*1256B*/
  0xFBC2, 0x0020, 0x0002, 0xA56C, 0x0000, 0x0000, 0x0002, /*1256C*/
  0xFBC2, 0x0020, 0x0002, 0xA56D, 0x0000, 0x0000, 0x0002, /*1256D*/
  0xFBC2, 0x0020, 0x0002, 0xA56E, 0x0000, 0x0000, 0x0002, /*1256E*/
  0xFBC2, 0x0020, 0x0002, 0xA56F, 0x0000, 0x0000, 0x0002, /*1256F*/
  0xFBC2, 0x0020, 0x0002, 0xA570, 0x0000, 0x0000, 0x0002, /*12570*/
  0xFBC2, 0x0020, 0x0002, 0xA571, 0x0000, 0x0000, 0x0002, /*12571*/
  0xFBC2, 0x0020, 0x0002, 0xA572, 0x0000, 0x0000, 0x0002, /*12572*/
  0xFBC2, 0x0020, 0x0002, 0xA573, 0x0000, 0x0000, 0x0002, /*12573*/
  0xFBC2, 0x0020, 0x0002, 0xA574, 0x0000, 0x0000, 0x0002, /*12574*/
  0xFBC2, 0x0020, 0x0002, 0xA575, 0x0000, 0x0000, 0x0002, /*12575*/
  0xFBC2, 0x0020, 0x0002, 0xA576, 0x0000, 0x0000, 0x0002, /*12576*/
  0xFBC2, 0x0020, 0x0002, 0xA577, 0x0000, 0x0000, 0x0002, /*12577*/
  0xFBC2, 0x0020, 0x0002, 0xA578, 0x0000, 0x0000, 0x0002, /*12578*/
  0xFBC2, 0x0020, 0x0002, 0xA579, 0x0000, 0x0000, 0x0002, /*12579*/
  0xFBC2, 0x0020, 0x0002, 0xA57A, 0x0000, 0x0000, 0x0002, /*1257A*/
  0xFBC2, 0x0020, 0x0002, 0xA57B, 0x0000, 0x0000, 0x0002, /*1257B*/
  0xFBC2, 0x0020, 0x0002, 0xA57C, 0x0000, 0x0000, 0x0002, /*1257C*/
  0xFBC2, 0x0020, 0x0002, 0xA57D, 0x0000, 0x0000, 0x0002, /*1257D*/
  0xFBC2, 0x0020, 0x0002, 0xA57E, 0x0000, 0x0000, 0x0002, /*1257E*/
  0xFBC2, 0x0020, 0x0002, 0xA57F, 0x0000, 0x0000, 0x0002, /*1257F*/
  0xFBC2, 0x0020, 0x0002, 0xA580, 0x0000, 0x0000, 0x0002, /*12580*/
  0xFBC2, 0x0020, 0x0002, 0xA581, 0x0000, 0x0000, 0x0002, /*12581*/
  0xFBC2, 0x0020, 0x0002, 0xA582, 0x0000, 0x0000, 0x0002, /*12582*/
  0xFBC2, 0x0020, 0x0002, 0xA583, 0x0000, 0x0000, 0x0002, /*12583*/
  0xFBC2, 0x0020, 0x0002, 0xA584, 0x0000, 0x0000, 0x0002, /*12584*/
  0xFBC2, 0x0020, 0x0002, 0xA585, 0x0000, 0x0000, 0x0002, /*12585*/
  0xFBC2, 0x0020, 0x0002, 0xA586, 0x0000, 0x0000, 0x0002, /*12586*/
  0xFBC2, 0x0020, 0x0002, 0xA587, 0x0000, 0x0000, 0x0002, /*12587*/
  0xFBC2, 0x0020, 0x0002, 0xA588, 0x0000, 0x0000, 0x0002, /*12588*/
  0xFBC2, 0x0020, 0x0002, 0xA589, 0x0000, 0x0000, 0x0002, /*12589*/
  0xFBC2, 0x0020, 0x0002, 0xA58A, 0x0000, 0x0000, 0x0002, /*1258A*/
  0xFBC2, 0x0020, 0x0002, 0xA58B, 0x0000, 0x0000, 0x0002, /*1258B*/
  0xFBC2, 0x0020, 0x0002, 0xA58C, 0x0000, 0x0000, 0x0002, /*1258C*/
  0xFBC2, 0x0020, 0x0002, 0xA58D, 0x0000, 0x0000, 0x0002, /*1258D*/
  0xFBC2, 0x0020, 0x0002, 0xA58E, 0x0000, 0x0000, 0x0002, /*1258E*/
  0xFBC2, 0x0020, 0x0002, 0xA58F, 0x0000, 0x0000, 0x0002, /*1258F*/
  0xFBC2, 0x0020, 0x0002, 0xA590, 0x0000, 0x0000, 0x0002, /*12590*/
  0xFBC2, 0x0020, 0x0002, 0xA591, 0x0000, 0x0000, 0x0002, /*12591*/
  0xFBC2, 0x0020, 0x0002, 0xA592, 0x0000, 0x0000, 0x0002, /*12592*/
  0xFBC2, 0x0020, 0x0002, 0xA593, 0x0000, 0x0000, 0x0002, /*12593*/
  0xFBC2, 0x0020, 0x0002, 0xA594, 0x0000, 0x0000, 0x0002, /*12594*/
  0xFBC2, 0x0020, 0x0002, 0xA595, 0x0000, 0x0000, 0x0002, /*12595*/
  0xFBC2, 0x0020, 0x0002, 0xA596, 0x0000, 0x0000, 0x0002, /*12596*/
  0xFBC2, 0x0020, 0x0002, 0xA597, 0x0000, 0x0000, 0x0002, /*12597*/
  0xFBC2, 0x0020, 0x0002, 0xA598, 0x0000, 0x0000, 0x0002, /*12598*/
  0xFBC2, 0x0020, 0x0002, 0xA599, 0x0000, 0x0000, 0x0002, /*12599*/
  0xFBC2, 0x0020, 0x0002, 0xA59A, 0x0000, 0x0000, 0x0002, /*1259A*/
  0xFBC2, 0x0020, 0x0002, 0xA59B, 0x0000, 0x0000, 0x0002, /*1259B*/
  0xFBC2, 0x0020, 0x0002, 0xA59C, 0x0000, 0x0000, 0x0002, /*1259C*/
  0xFBC2, 0x0020, 0x0002, 0xA59D, 0x0000, 0x0000, 0x0002, /*1259D*/
  0xFBC2, 0x0020, 0x0002, 0xA59E, 0x0000, 0x0000, 0x0002, /*1259E*/
  0xFBC2, 0x0020, 0x0002, 0xA59F, 0x0000, 0x0000, 0x0002, /*1259F*/
  0xFBC2, 0x0020, 0x0002, 0xA5A0, 0x0000, 0x0000, 0x0002, /*125A0*/
  0xFBC2, 0x0020, 0x0002, 0xA5A1, 0x0000, 0x0000, 0x0002, /*125A1*/
  0xFBC2, 0x0020, 0x0002, 0xA5A2, 0x0000, 0x0000, 0x0002, /*125A2*/
  0xFBC2, 0x0020, 0x0002, 0xA5A3, 0x0000, 0x0000, 0x0002, /*125A3*/
  0xFBC2, 0x0020, 0x0002, 0xA5A4, 0x0000, 0x0000, 0x0002, /*125A4*/
  0xFBC2, 0x0020, 0x0002, 0xA5A5, 0x0000, 0x0000, 0x0002, /*125A5*/
  0xFBC2, 0x0020, 0x0002, 0xA5A6, 0x0000, 0x0000, 0x0002, /*125A6*/
  0xFBC2, 0x0020, 0x0002, 0xA5A7, 0x0000, 0x0000, 0x0002, /*125A7*/
  0xFBC2, 0x0020, 0x0002, 0xA5A8, 0x0000, 0x0000, 0x0002, /*125A8*/
  0xFBC2, 0x0020, 0x0002, 0xA5A9, 0x0000, 0x0000, 0x0002, /*125A9*/
  0xFBC2, 0x0020, 0x0002, 0xA5AA, 0x0000, 0x0000, 0x0002, /*125AA*/
  0xFBC2, 0x0020, 0x0002, 0xA5AB, 0x0000, 0x0000, 0x0002, /*125AB*/
  0xFBC2, 0x0020, 0x0002, 0xA5AC, 0x0000, 0x0000, 0x0002, /*125AC*/
  0xFBC2, 0x0020, 0x0002, 0xA5AD, 0x0000, 0x0000, 0x0002, /*125AD*/
  0xFBC2, 0x0020, 0x0002, 0xA5AE, 0x0000, 0x0000, 0x0002, /*125AE*/
  0xFBC2, 0x0020, 0x0002, 0xA5AF, 0x0000, 0x0000, 0x0002, /*125AF*/
  0xFBC2, 0x0020, 0x0002, 0xA5B0, 0x0000, 0x0000, 0x0002, /*125B0*/
  0xFBC2, 0x0020, 0x0002, 0xA5B1, 0x0000, 0x0000, 0x0002, /*125B1*/
  0xFBC2, 0x0020, 0x0002, 0xA5B2, 0x0000, 0x0000, 0x0002, /*125B2*/
  0xFBC2, 0x0020, 0x0002, 0xA5B3, 0x0000, 0x0000, 0x0002, /*125B3*/
  0xFBC2, 0x0020, 0x0002, 0xA5B4, 0x0000, 0x0000, 0x0002, /*125B4*/
  0xFBC2, 0x0020, 0x0002, 0xA5B5, 0x0000, 0x0000, 0x0002, /*125B5*/
  0xFBC2, 0x0020, 0x0002, 0xA5B6, 0x0000, 0x0000, 0x0002, /*125B6*/
  0xFBC2, 0x0020, 0x0002, 0xA5B7, 0x0000, 0x0000, 0x0002, /*125B7*/
  0xFBC2, 0x0020, 0x0002, 0xA5B8, 0x0000, 0x0000, 0x0002, /*125B8*/
  0xFBC2, 0x0020, 0x0002, 0xA5B9, 0x0000, 0x0000, 0x0002, /*125B9*/
  0xFBC2, 0x0020, 0x0002, 0xA5BA, 0x0000, 0x0000, 0x0002, /*125BA*/
  0xFBC2, 0x0020, 0x0002, 0xA5BB, 0x0000, 0x0000, 0x0002, /*125BB*/
  0xFBC2, 0x0020, 0x0002, 0xA5BC, 0x0000, 0x0000, 0x0002, /*125BC*/
  0xFBC2, 0x0020, 0x0002, 0xA5BD, 0x0000, 0x0000, 0x0002, /*125BD*/
  0xFBC2, 0x0020, 0x0002, 0xA5BE, 0x0000, 0x0000, 0x0002, /*125BE*/
  0xFBC2, 0x0020, 0x0002, 0xA5BF, 0x0000, 0x0000, 0x0002, /*125BF*/
  0xFBC2, 0x0020, 0x0002, 0xA5C0, 0x0000, 0x0000, 0x0002, /*125C0*/
  0xFBC2, 0x0020, 0x0002, 0xA5C1, 0x0000, 0x0000, 0x0002, /*125C1*/
  0xFBC2, 0x0020, 0x0002, 0xA5C2, 0x0000, 0x0000, 0x0002, /*125C2*/
  0xFBC2, 0x0020, 0x0002, 0xA5C3, 0x0000, 0x0000, 0x0002, /*125C3*/
  0xFBC2, 0x0020, 0x0002, 0xA5C4, 0x0000, 0x0000, 0x0002, /*125C4*/
  0xFBC2, 0x0020, 0x0002, 0xA5C5, 0x0000, 0x0000, 0x0002, /*125C5*/
  0xFBC2, 0x0020, 0x0002, 0xA5C6, 0x0000, 0x0000, 0x0002, /*125C6*/
  0xFBC2, 0x0020, 0x0002, 0xA5C7, 0x0000, 0x0000, 0x0002, /*125C7*/
  0xFBC2, 0x0020, 0x0002, 0xA5C8, 0x0000, 0x0000, 0x0002, /*125C8*/
  0xFBC2, 0x0020, 0x0002, 0xA5C9, 0x0000, 0x0000, 0x0002, /*125C9*/
  0xFBC2, 0x0020, 0x0002, 0xA5CA, 0x0000, 0x0000, 0x0002, /*125CA*/
  0xFBC2, 0x0020, 0x0002, 0xA5CB, 0x0000, 0x0000, 0x0002, /*125CB*/
  0xFBC2, 0x0020, 0x0002, 0xA5CC, 0x0000, 0x0000, 0x0002, /*125CC*/
  0xFBC2, 0x0020, 0x0002, 0xA5CD, 0x0000, 0x0000, 0x0002, /*125CD*/
  0xFBC2, 0x0020, 0x0002, 0xA5CE, 0x0000, 0x0000, 0x0002, /*125CE*/
  0xFBC2, 0x0020, 0x0002, 0xA5CF, 0x0000, 0x0000, 0x0002, /*125CF*/
  0xFBC2, 0x0020, 0x0002, 0xA5D0, 0x0000, 0x0000, 0x0002, /*125D0*/
  0xFBC2, 0x0020, 0x0002, 0xA5D1, 0x0000, 0x0000, 0x0002, /*125D1*/
  0xFBC2, 0x0020, 0x0002, 0xA5D2, 0x0000, 0x0000, 0x0002, /*125D2*/
  0xFBC2, 0x0020, 0x0002, 0xA5D3, 0x0000, 0x0000, 0x0002, /*125D3*/
  0xFBC2, 0x0020, 0x0002, 0xA5D4, 0x0000, 0x0000, 0x0002, /*125D4*/
  0xFBC2, 0x0020, 0x0002, 0xA5D5, 0x0000, 0x0000, 0x0002, /*125D5*/
  0xFBC2, 0x0020, 0x0002, 0xA5D6, 0x0000, 0x0000, 0x0002, /*125D6*/
  0xFBC2, 0x0020, 0x0002, 0xA5D7, 0x0000, 0x0000, 0x0002, /*125D7*/
  0xFBC2, 0x0020, 0x0002, 0xA5D8, 0x0000, 0x0000, 0x0002, /*125D8*/
  0xFBC2, 0x0020, 0x0002, 0xA5D9, 0x0000, 0x0000, 0x0002, /*125D9*/
  0xFBC2, 0x0020, 0x0002, 0xA5DA, 0x0000, 0x0000, 0x0002, /*125DA*/
  0xFBC2, 0x0020, 0x0002, 0xA5DB, 0x0000, 0x0000, 0x0002, /*125DB*/
  0xFBC2, 0x0020, 0x0002, 0xA5DC, 0x0000, 0x0000, 0x0002, /*125DC*/
  0xFBC2, 0x0020, 0x0002, 0xA5DD, 0x0000, 0x0000, 0x0002, /*125DD*/
  0xFBC2, 0x0020, 0x0002, 0xA5DE, 0x0000, 0x0000, 0x0002, /*125DE*/
  0xFBC2, 0x0020, 0x0002, 0xA5DF, 0x0000, 0x0000, 0x0002, /*125DF*/
  0xFBC2, 0x0020, 0x0002, 0xA5E0, 0x0000, 0x0000, 0x0002, /*125E0*/
  0xFBC2, 0x0020, 0x0002, 0xA5E1, 0x0000, 0x0000, 0x0002, /*125E1*/
  0xFBC2, 0x0020, 0x0002, 0xA5E2, 0x0000, 0x0000, 0x0002, /*125E2*/
  0xFBC2, 0x0020, 0x0002, 0xA5E3, 0x0000, 0x0000, 0x0002, /*125E3*/
  0xFBC2, 0x0020, 0x0002, 0xA5E4, 0x0000, 0x0000, 0x0002, /*125E4*/
  0xFBC2, 0x0020, 0x0002, 0xA5E5, 0x0000, 0x0000, 0x0002, /*125E5*/
  0xFBC2, 0x0020, 0x0002, 0xA5E6, 0x0000, 0x0000, 0x0002, /*125E6*/
  0xFBC2, 0x0020, 0x0002, 0xA5E7, 0x0000, 0x0000, 0x0002, /*125E7*/
  0xFBC2, 0x0020, 0x0002, 0xA5E8, 0x0000, 0x0000, 0x0002, /*125E8*/
  0xFBC2, 0x0020, 0x0002, 0xA5E9, 0x0000, 0x0000, 0x0002, /*125E9*/
  0xFBC2, 0x0020, 0x0002, 0xA5EA, 0x0000, 0x0000, 0x0002, /*125EA*/
  0xFBC2, 0x0020, 0x0002, 0xA5EB, 0x0000, 0x0000, 0x0002, /*125EB*/
  0xFBC2, 0x0020, 0x0002, 0xA5EC, 0x0000, 0x0000, 0x0002, /*125EC*/
  0xFBC2, 0x0020, 0x0002, 0xA5ED, 0x0000, 0x0000, 0x0002, /*125ED*/
  0xFBC2, 0x0020, 0x0002, 0xA5EE, 0x0000, 0x0000, 0x0002, /*125EE*/
  0xFBC2, 0x0020, 0x0002, 0xA5EF, 0x0000, 0x0000, 0x0002, /*125EF*/
  0xFBC2, 0x0020, 0x0002, 0xA5F0, 0x0000, 0x0000, 0x0002, /*125F0*/
  0xFBC2, 0x0020, 0x0002, 0xA5F1, 0x0000, 0x0000, 0x0002, /*125F1*/
  0xFBC2, 0x0020, 0x0002, 0xA5F2, 0x0000, 0x0000, 0x0002, /*125F2*/
  0xFBC2, 0x0020, 0x0002, 0xA5F3, 0x0000, 0x0000, 0x0002, /*125F3*/
  0xFBC2, 0x0020, 0x0002, 0xA5F4, 0x0000, 0x0000, 0x0002, /*125F4*/
  0xFBC2, 0x0020, 0x0002, 0xA5F5, 0x0000, 0x0000, 0x0002, /*125F5*/
  0xFBC2, 0x0020, 0x0002, 0xA5F6, 0x0000, 0x0000, 0x0002, /*125F6*/
  0xFBC2, 0x0020, 0x0002, 0xA5F7, 0x0000, 0x0000, 0x0002, /*125F7*/
  0xFBC2, 0x0020, 0x0002, 0xA5F8, 0x0000, 0x0000, 0x0002, /*125F8*/
  0xFBC2, 0x0020, 0x0002, 0xA5F9, 0x0000, 0x0000, 0x0002, /*125F9*/
  0xFBC2, 0x0020, 0x0002, 0xA5FA, 0x0000, 0x0000, 0x0002, /*125FA*/
  0xFBC2, 0x0020, 0x0002, 0xA5FB, 0x0000, 0x0000, 0x0002, /*125FB*/
  0xFBC2, 0x0020, 0x0002, 0xA5FC, 0x0000, 0x0000, 0x0002, /*125FC*/
  0xFBC2, 0x0020, 0x0002, 0xA5FD, 0x0000, 0x0000, 0x0002, /*125FD*/
  0xFBC2, 0x0020, 0x0002, 0xA5FE, 0x0000, 0x0000, 0x0002, /*125FE*/
  0xFBC2, 0x0020, 0x0002, 0xA5FF, 0x0000, 0x0000, 0x0002  /*125FF*/
};

uint16 uca900_p130[]= {
  0x4E13, 0x0020, 0x0002, 0x0001, /*13000*/
  0x4E14, 0x0020, 0x0002, 0x0001, /*13001*/
  0x4E15, 0x0020, 0x0002, 0x0001, /*13002*/
  0x4E16, 0x0020, 0x0002, 0x0001, /*13003*/
  0x4E17, 0x0020, 0x0002, 0x0001, /*13004*/
  0x4E18, 0x0020, 0x0002, 0x0001, /*13005*/
  0x4E19, 0x0020, 0x0002, 0x0001, /*13006*/
  0x4E1A, 0x0020, 0x0002, 0x0001, /*13007*/
  0x4E1B, 0x0020, 0x0002, 0x0001, /*13008*/
  0x4E1C, 0x0020, 0x0002, 0x0001, /*13009*/
  0x4E1D, 0x0020, 0x0002, 0x0001, /*1300A*/
  0x4E1E, 0x0020, 0x0002, 0x0001, /*1300B*/
  0x4E1F, 0x0020, 0x0002, 0x0001, /*1300C*/
  0x4E20, 0x0020, 0x0002, 0x0001, /*1300D*/
  0x4E21, 0x0020, 0x0002, 0x0001, /*1300E*/
  0x4E22, 0x0020, 0x0002, 0x0001, /*1300F*/
  0x4E23, 0x0020, 0x0002, 0x0001, /*13010*/
  0x4E24, 0x0020, 0x0002, 0x0001, /*13011*/
  0x4E25, 0x0020, 0x0002, 0x0001, /*13012*/
  0x4E26, 0x0020, 0x0002, 0x0001, /*13013*/
  0x4E27, 0x0020, 0x0002, 0x0001, /*13014*/
  0x4E28, 0x0020, 0x0002, 0x0001, /*13015*/
  0x4E29, 0x0020, 0x0002, 0x0001, /*13016*/
  0x4E2A, 0x0020, 0x0002, 0x0001, /*13017*/
  0x4E2B, 0x0020, 0x0002, 0x0001, /*13018*/
  0x4E2C, 0x0020, 0x0002, 0x0001, /*13019*/
  0x4E2D, 0x0020, 0x0002, 0x0001, /*1301A*/
  0x4E2E, 0x0020, 0x0002, 0x0001, /*1301B*/
  0x4E2F, 0x0020, 0x0002, 0x0001, /*1301C*/
  0x4E30, 0x0020, 0x0002, 0x0001, /*1301D*/
  0x4E31, 0x0020, 0x0002, 0x0001, /*1301E*/
  0x4E32, 0x0020, 0x0002, 0x0001, /*1301F*/
  0x4E33, 0x0020, 0x0002, 0x0001, /*13020*/
  0x4E34, 0x0020, 0x0002, 0x0001, /*13021*/
  0x4E35, 0x0020, 0x0002, 0x0001, /*13022*/
  0x4E36, 0x0020, 0x0002, 0x0001, /*13023*/
  0x4E37, 0x0020, 0x0002, 0x0001, /*13024*/
  0x4E38, 0x0020, 0x0002, 0x0001, /*13025*/
  0x4E39, 0x0020, 0x0002, 0x0001, /*13026*/
  0x4E3A, 0x0020, 0x0002, 0x0001, /*13027*/
  0x4E3B, 0x0020, 0x0002, 0x0001, /*13028*/
  0x4E3C, 0x0020, 0x0002, 0x0001, /*13029*/
  0x4E3D, 0x0020, 0x0002, 0x0001, /*1302A*/
  0x4E3E, 0x0020, 0x0002, 0x0001, /*1302B*/
  0x4E3F, 0x0020, 0x0002, 0x0001, /*1302C*/
  0x4E40, 0x0020, 0x0002, 0x0001, /*1302D*/
  0x4E41, 0x0020, 0x0002, 0x0001, /*1302E*/
  0x4E42, 0x0020, 0x0002, 0x0001, /*1302F*/
  0x4E43, 0x0020, 0x0002, 0x0001, /*13030*/
  0x4E44, 0x0020, 0x0002, 0x0001, /*13031*/
  0x4E45, 0x0020, 0x0002, 0x0001, /*13032*/
  0x4E46, 0x0020, 0x0002, 0x0001, /*13033*/
  0x4E47, 0x0020, 0x0002, 0x0001, /*13034*/
  0x4E48, 0x0020, 0x0002, 0x0001, /*13035*/
  0x4E49, 0x0020, 0x0002, 0x0001, /*13036*/
  0x4E4A, 0x0020, 0x0002, 0x0001, /*13037*/
  0x4E4B, 0x0020, 0x0002, 0x0001, /*13038*/
  0x4E4C, 0x0020, 0x0002, 0x0001, /*13039*/
  0x4E4D, 0x0020, 0x0002, 0x0001, /*1303A*/
  0x4E4E, 0x0020, 0x0002, 0x0001, /*1303B*/
  0x4E4F, 0x0020, 0x0002, 0x0001, /*1303C*/
  0x4E50, 0x0020, 0x0002, 0x0001, /*1303D*/
  0x4E51, 0x0020, 0x0002, 0x0001, /*1303E*/
  0x4E52, 0x0020, 0x0002, 0x0001, /*1303F*/
  0x4E53, 0x0020, 0x0002, 0x0001, /*13040*/
  0x4E54, 0x0020, 0x0002, 0x0001, /*13041*/
  0x4E55, 0x0020, 0x0002, 0x0001, /*13042*/
  0x4E56, 0x0020, 0x0002, 0x0001, /*13043*/
  0x4E57, 0x0020, 0x0002, 0x0001, /*13044*/
  0x4E58, 0x0020, 0x0002, 0x0001, /*13045*/
  0x4E59, 0x0020, 0x0002, 0x0001, /*13046*/
  0x4E5A, 0x0020, 0x0002, 0x0001, /*13047*/
  0x4E5B, 0x0020, 0x0002, 0x0001, /*13048*/
  0x4E5C, 0x0020, 0x0002, 0x0001, /*13049*/
  0x4E5D, 0x0020, 0x0002, 0x0001, /*1304A*/
  0x4E5E, 0x0020, 0x0002, 0x0001, /*1304B*/
  0x4E5F, 0x0020, 0x0002, 0x0001, /*1304C*/
  0x4E60, 0x0020, 0x0002, 0x0001, /*1304D*/
  0x4E61, 0x0020, 0x0002, 0x0001, /*1304E*/
  0x4E62, 0x0020, 0x0002, 0x0001, /*1304F*/
  0x4E63, 0x0020, 0x0002, 0x0001, /*13050*/
  0x4E64, 0x0020, 0x0002, 0x0001, /*13051*/
  0x4E65, 0x0020, 0x0002, 0x0001, /*13052*/
  0x4E66, 0x0020, 0x0002, 0x0001, /*13053*/
  0x4E67, 0x0020, 0x0002, 0x0001, /*13054*/
  0x4E68, 0x0020, 0x0002, 0x0001, /*13055*/
  0x4E69, 0x0020, 0x0002, 0x0001, /*13056*/
  0x4E6A, 0x0020, 0x0002, 0x0001, /*13057*/
  0x4E6B, 0x0020, 0x0002, 0x0001, /*13058*/
  0x4E6C, 0x0020, 0x0002, 0x0001, /*13059*/
  0x4E6D, 0x0020, 0x0002, 0x0001, /*1305A*/
  0x4E6E, 0x0020, 0x0002, 0x0001, /*1305B*/
  0x4E6F, 0x0020, 0x0002, 0x0001, /*1305C*/
  0x4E70, 0x0020, 0x0002, 0x0001, /*1305D*/
  0x4E71, 0x0020, 0x0002, 0x0001, /*1305E*/
  0x4E72, 0x0020, 0x0002, 0x0001, /*1305F*/
  0x4E73, 0x0020, 0x0002, 0x0001, /*13060*/
  0x4E74, 0x0020, 0x0002, 0x0001, /*13061*/
  0x4E75, 0x0020, 0x0002, 0x0001, /*13062*/
  0x4E76, 0x0020, 0x0002, 0x0001, /*13063*/
  0x4E77, 0x0020, 0x0002, 0x0001, /*13064*/
  0x4E78, 0x0020, 0x0002, 0x0001, /*13065*/
  0x4E79, 0x0020, 0x0002, 0x0001, /*13066*/
  0x4E7A, 0x0020, 0x0002, 0x0001, /*13067*/
  0x4E7B, 0x0020, 0x0002, 0x0001, /*13068*/
  0x4E7C, 0x0020, 0x0002, 0x0001, /*13069*/
  0x4E7D, 0x0020, 0x0002, 0x0001, /*1306A*/
  0x4E7E, 0x0020, 0x0002, 0x0001, /*1306B*/
  0x4E7F, 0x0020, 0x0002, 0x0001, /*1306C*/
  0x4E80, 0x0020, 0x0002, 0x0001, /*1306D*/
  0x4E81, 0x0020, 0x0002, 0x0001, /*1306E*/
  0x4E82, 0x0020, 0x0002, 0x0001, /*1306F*/
  0x4E83, 0x0020, 0x0002, 0x0001, /*13070*/
  0x4E84, 0x0020, 0x0002, 0x0001, /*13071*/
  0x4E85, 0x0020, 0x0002, 0x0001, /*13072*/
  0x4E86, 0x0020, 0x0002, 0x0001, /*13073*/
  0x4E87, 0x0020, 0x0002, 0x0001, /*13074*/
  0x4E88, 0x0020, 0x0002, 0x0001, /*13075*/
  0x4E89, 0x0020, 0x0002, 0x0001, /*13076*/
  0x4E8A, 0x0020, 0x0002, 0x0001, /*13077*/
  0x4E8B, 0x0020, 0x0002, 0x0001, /*13078*/
  0x4E8C, 0x0020, 0x0002, 0x0001, /*13079*/
  0x4E8D, 0x0020, 0x0002, 0x0001, /*1307A*/
  0x4E8E, 0x0020, 0x0002, 0x0001, /*1307B*/
  0x4E8F, 0x0020, 0x0002, 0x0001, /*1307C*/
  0x4E90, 0x0020, 0x0002, 0x0001, /*1307D*/
  0x4E91, 0x0020, 0x0002, 0x0001, /*1307E*/
  0x4E92, 0x0020, 0x0002, 0x0001, /*1307F*/
  0x4E93, 0x0020, 0x0002, 0x0001, /*13080*/
  0x4E94, 0x0020, 0x0002, 0x0001, /*13081*/
  0x4E95, 0x0020, 0x0002, 0x0001, /*13082*/
  0x4E96, 0x0020, 0x0002, 0x0001, /*13083*/
  0x4E97, 0x0020, 0x0002, 0x0001, /*13084*/
  0x4E98, 0x0020, 0x0002, 0x0001, /*13085*/
  0x4E99, 0x0020, 0x0002, 0x0001, /*13086*/
  0x4E9A, 0x0020, 0x0002, 0x0001, /*13087*/
  0x4E9B, 0x0020, 0x0002, 0x0001, /*13088*/
  0x4E9C, 0x0020, 0x0002, 0x0001, /*13089*/
  0x4E9D, 0x0020, 0x0002, 0x0001, /*1308A*/
  0x4E9E, 0x0020, 0x0002, 0x0001, /*1308B*/
  0x4E9F, 0x0020, 0x0002, 0x0001, /*1308C*/
  0x4EA0, 0x0020, 0x0002, 0x0001, /*1308D*/
  0x4EA1, 0x0020, 0x0002, 0x0001, /*1308E*/
  0x4EA2, 0x0020, 0x0002, 0x0001, /*1308F*/
  0x4EA3, 0x0020, 0x0002, 0x0001, /*13090*/
  0x4EA4, 0x0020, 0x0002, 0x0001, /*13091*/
  0x4EA5, 0x0020, 0x0002, 0x0001, /*13092*/
  0x4EA6, 0x0020, 0x0002, 0x0001, /*13093*/
  0x4EA7, 0x0020, 0x0002, 0x0001, /*13094*/
  0x4EA8, 0x0020, 0x0002, 0x0001, /*13095*/
  0x4EA9, 0x0020, 0x0002, 0x0001, /*13096*/
  0x4EAA, 0x0020, 0x0002, 0x0001, /*13097*/
  0x4EAB, 0x0020, 0x0002, 0x0001, /*13098*/
  0x4EAC, 0x0020, 0x0002, 0x0001, /*13099*/
  0x4EAD, 0x0020, 0x0002, 0x0001, /*1309A*/
  0x4EAE, 0x0020, 0x0002, 0x0001, /*1309B*/
  0x4EAF, 0x0020, 0x0002, 0x0001, /*1309C*/
  0x4EB0, 0x0020, 0x0002, 0x0001, /*1309D*/
  0x4EB1, 0x0020, 0x0002, 0x0001, /*1309E*/
  0x4EB2, 0x0020, 0x0002, 0x0001, /*1309F*/
  0x4EB3, 0x0020, 0x0002, 0x0001, /*130A0*/
  0x4EB4, 0x0020, 0x0002, 0x0001, /*130A1*/
  0x4EB5, 0x0020, 0x0002, 0x0001, /*130A2*/
  0x4EB6, 0x0020, 0x0002, 0x0001, /*130A3*/
  0x4EB7, 0x0020, 0x0002, 0x0001, /*130A4*/
  0x4EB8, 0x0020, 0x0002, 0x0001, /*130A5*/
  0x4EB9, 0x0020, 0x0002, 0x0001, /*130A6*/
  0x4EBA, 0x0020, 0x0002, 0x0001, /*130A7*/
  0x4EBB, 0x0020, 0x0002, 0x0001, /*130A8*/
  0x4EBC, 0x0020, 0x0002, 0x0001, /*130A9*/
  0x4EBD, 0x0020, 0x0002, 0x0001, /*130AA*/
  0x4EBE, 0x0020, 0x0002, 0x0001, /*130AB*/
  0x4EBF, 0x0020, 0x0002, 0x0001, /*130AC*/
  0x4EC0, 0x0020, 0x0002, 0x0001, /*130AD*/
  0x4EC1, 0x0020, 0x0002, 0x0001, /*130AE*/
  0x4EC2, 0x0020, 0x0002, 0x0001, /*130AF*/
  0x4EC3, 0x0020, 0x0002, 0x0001, /*130B0*/
  0x4EC4, 0x0020, 0x0002, 0x0001, /*130B1*/
  0x4EC5, 0x0020, 0x0002, 0x0001, /*130B2*/
  0x4EC6, 0x0020, 0x0002, 0x0001, /*130B3*/
  0x4EC7, 0x0020, 0x0002, 0x0001, /*130B4*/
  0x4EC8, 0x0020, 0x0002, 0x0001, /*130B5*/
  0x4EC9, 0x0020, 0x0002, 0x0001, /*130B6*/
  0x4ECA, 0x0020, 0x0002, 0x0001, /*130B7*/
  0x4ECB, 0x0020, 0x0002, 0x0001, /*130B8*/
  0x4ECC, 0x0020, 0x0002, 0x0001, /*130B9*/
  0x4ECD, 0x0020, 0x0002, 0x0001, /*130BA*/
  0x4ECE, 0x0020, 0x0002, 0x0001, /*130BB*/
  0x4ECF, 0x0020, 0x0002, 0x0001, /*130BC*/
  0x4ED0, 0x0020, 0x0002, 0x0001, /*130BD*/
  0x4ED1, 0x0020, 0x0002, 0x0001, /*130BE*/
  0x4ED2, 0x0020, 0x0002, 0x0001, /*130BF*/
  0x4ED3, 0x0020, 0x0002, 0x0001, /*130C0*/
  0x4ED4, 0x0020, 0x0002, 0x0001, /*130C1*/
  0x4ED5, 0x0020, 0x0002, 0x0001, /*130C2*/
  0x4ED6, 0x0020, 0x0002, 0x0001, /*130C3*/
  0x4ED7, 0x0020, 0x0002, 0x0001, /*130C4*/
  0x4ED8, 0x0020, 0x0002, 0x0001, /*130C5*/
  0x4ED9, 0x0020, 0x0002, 0x0001, /*130C6*/
  0x4EDA, 0x0020, 0x0002, 0x0001, /*130C7*/
  0x4EDB, 0x0020, 0x0002, 0x0001, /*130C8*/
  0x4EDC, 0x0020, 0x0002, 0x0001, /*130C9*/
  0x4EDD, 0x0020, 0x0002, 0x0001, /*130CA*/
  0x4EDE, 0x0020, 0x0002, 0x0001, /*130CB*/
  0x4EDF, 0x0020, 0x0002, 0x0001, /*130CC*/
  0x4EE0, 0x0020, 0x0002, 0x0001, /*130CD*/
  0x4EE1, 0x0020, 0x0002, 0x0001, /*130CE*/
  0x4EE2, 0x0020, 0x0002, 0x0001, /*130CF*/
  0x4EE3, 0x0020, 0x0002, 0x0001, /*130D0*/
  0x4EE4, 0x0020, 0x0002, 0x0001, /*130D1*/
  0x4EE5, 0x0020, 0x0002, 0x0001, /*130D2*/
  0x4EE6, 0x0020, 0x0002, 0x0001, /*130D3*/
  0x4EE7, 0x0020, 0x0002, 0x0001, /*130D4*/
  0x4EE8, 0x0020, 0x0002, 0x0001, /*130D5*/
  0x4EE9, 0x0020, 0x0002, 0x0001, /*130D6*/
  0x4EEA, 0x0020, 0x0002, 0x0001, /*130D7*/
  0x4EEB, 0x0020, 0x0002, 0x0001, /*130D8*/
  0x4EEC, 0x0020, 0x0002, 0x0001, /*130D9*/
  0x4EED, 0x0020, 0x0002, 0x0001, /*130DA*/
  0x4EEE, 0x0020, 0x0002, 0x0001, /*130DB*/
  0x4EEF, 0x0020, 0x0002, 0x0001, /*130DC*/
  0x4EF0, 0x0020, 0x0002, 0x0001, /*130DD*/
  0x4EF1, 0x0020, 0x0002, 0x0001, /*130DE*/
  0x4EF2, 0x0020, 0x0002, 0x0001, /*130DF*/
  0x4EF3, 0x0020, 0x0002, 0x0001, /*130E0*/
  0x4EF4, 0x0020, 0x0002, 0x0001, /*130E1*/
  0x4EF5, 0x0020, 0x0002, 0x0001, /*130E2*/
  0x4EF6, 0x0020, 0x0002, 0x0001, /*130E3*/
  0x4EF7, 0x0020, 0x0002, 0x0001, /*130E4*/
  0x4EF8, 0x0020, 0x0002, 0x0001, /*130E5*/
  0x4EF9, 0x0020, 0x0002, 0x0001, /*130E6*/
  0x4EFA, 0x0020, 0x0002, 0x0001, /*130E7*/
  0x4EFB, 0x0020, 0x0002, 0x0001, /*130E8*/
  0x4EFC, 0x0020, 0x0002, 0x0001, /*130E9*/
  0x4EFD, 0x0020, 0x0002, 0x0001, /*130EA*/
  0x4EFE, 0x0020, 0x0002, 0x0001, /*130EB*/
  0x4EFF, 0x0020, 0x0002, 0x0001, /*130EC*/
  0x4F00, 0x0020, 0x0002, 0x0001, /*130ED*/
  0x4F01, 0x0020, 0x0002, 0x0001, /*130EE*/
  0x4F02, 0x0020, 0x0002, 0x0001, /*130EF*/
  0x4F03, 0x0020, 0x0002, 0x0001, /*130F0*/
  0x4F04, 0x0020, 0x0002, 0x0001, /*130F1*/
  0x4F05, 0x0020, 0x0002, 0x0001, /*130F2*/
  0x4F06, 0x0020, 0x0002, 0x0001, /*130F3*/
  0x4F07, 0x0020, 0x0002, 0x0001, /*130F4*/
  0x4F08, 0x0020, 0x0002, 0x0001, /*130F5*/
  0x4F09, 0x0020, 0x0002, 0x0001, /*130F6*/
  0x4F0A, 0x0020, 0x0002, 0x0001, /*130F7*/
  0x4F0B, 0x0020, 0x0002, 0x0001, /*130F8*/
  0x4F0C, 0x0020, 0x0002, 0x0001, /*130F9*/
  0x4F0D, 0x0020, 0x0002, 0x0001, /*130FA*/
  0x4F0E, 0x0020, 0x0002, 0x0001, /*130FB*/
  0x4F0F, 0x0020, 0x0002, 0x0001, /*130FC*/
  0x4F10, 0x0020, 0x0002, 0x0001, /*130FD*/
  0x4F11, 0x0020, 0x0002, 0x0001, /*130FE*/
  0x4F12, 0x0020, 0x0002, 0x0001  /*130FF*/
};

uint16 uca900_p131[]= {
  0x4F13, 0x0020, 0x0002, 0x0001, /*13100*/
  0x4F14, 0x0020, 0x0002, 0x0001, /*13101*/
  0x4F15, 0x0020, 0x0002, 0x0001, /*13102*/
  0x4F16, 0x0020, 0x0002, 0x0001, /*13103*/
  0x4F17, 0x0020, 0x0002, 0x0001, /*13104*/
  0x4F18, 0x0020, 0x0002, 0x0001, /*13105*/
  0x4F19, 0x0020, 0x0002, 0x0001, /*13106*/
  0x4F1A, 0x0020, 0x0002, 0x0001, /*13107*/
  0x4F1B, 0x0020, 0x0002, 0x0001, /*13108*/
  0x4F1C, 0x0020, 0x0002, 0x0001, /*13109*/
  0x4F1D, 0x0020, 0x0002, 0x0001, /*1310A*/
  0x4F1E, 0x0020, 0x0002, 0x0001, /*1310B*/
  0x4F1F, 0x0020, 0x0002, 0x0001, /*1310C*/
  0x4F20, 0x0020, 0x0002, 0x0001, /*1310D*/
  0x4F21, 0x0020, 0x0002, 0x0001, /*1310E*/
  0x4F22, 0x0020, 0x0002, 0x0001, /*1310F*/
  0x4F23, 0x0020, 0x0002, 0x0001, /*13110*/
  0x4F24, 0x0020, 0x0002, 0x0001, /*13111*/
  0x4F25, 0x0020, 0x0002, 0x0001, /*13112*/
  0x4F26, 0x0020, 0x0002, 0x0001, /*13113*/
  0x4F27, 0x0020, 0x0002, 0x0001, /*13114*/
  0x4F28, 0x0020, 0x0002, 0x0001, /*13115*/
  0x4F29, 0x0020, 0x0002, 0x0001, /*13116*/
  0x4F2A, 0x0020, 0x0002, 0x0001, /*13117*/
  0x4F2B, 0x0020, 0x0002, 0x0001, /*13118*/
  0x4F2C, 0x0020, 0x0002, 0x0001, /*13119*/
  0x4F2D, 0x0020, 0x0002, 0x0001, /*1311A*/
  0x4F2E, 0x0020, 0x0002, 0x0001, /*1311B*/
  0x4F2F, 0x0020, 0x0002, 0x0001, /*1311C*/
  0x4F30, 0x0020, 0x0002, 0x0001, /*1311D*/
  0x4F31, 0x0020, 0x0002, 0x0001, /*1311E*/
  0x4F32, 0x0020, 0x0002, 0x0001, /*1311F*/
  0x4F33, 0x0020, 0x0002, 0x0001, /*13120*/
  0x4F34, 0x0020, 0x0002, 0x0001, /*13121*/
  0x4F35, 0x0020, 0x0002, 0x0001, /*13122*/
  0x4F36, 0x0020, 0x0002, 0x0001, /*13123*/
  0x4F37, 0x0020, 0x0002, 0x0001, /*13124*/
  0x4F38, 0x0020, 0x0002, 0x0001, /*13125*/
  0x4F39, 0x0020, 0x0002, 0x0001, /*13126*/
  0x4F3A, 0x0020, 0x0002, 0x0001, /*13127*/
  0x4F3B, 0x0020, 0x0002, 0x0001, /*13128*/
  0x4F3C, 0x0020, 0x0002, 0x0001, /*13129*/
  0x4F3D, 0x0020, 0x0002, 0x0001, /*1312A*/
  0x4F3E, 0x0020, 0x0002, 0x0001, /*1312B*/
  0x4F3F, 0x0020, 0x0002, 0x0001, /*1312C*/
  0x4F40, 0x0020, 0x0002, 0x0001, /*1312D*/
  0x4F41, 0x0020, 0x0002, 0x0001, /*1312E*/
  0x4F42, 0x0020, 0x0002, 0x0001, /*1312F*/
  0x4F43, 0x0020, 0x0002, 0x0001, /*13130*/
  0x4F44, 0x0020, 0x0002, 0x0001, /*13131*/
  0x4F45, 0x0020, 0x0002, 0x0001, /*13132*/
  0x4F46, 0x0020, 0x0002, 0x0001, /*13133*/
  0x4F47, 0x0020, 0x0002, 0x0001, /*13134*/
  0x4F48, 0x0020, 0x0002, 0x0001, /*13135*/
  0x4F49, 0x0020, 0x0002, 0x0001, /*13136*/
  0x4F4A, 0x0020, 0x0002, 0x0001, /*13137*/
  0x4F4B, 0x0020, 0x0002, 0x0001, /*13138*/
  0x4F4C, 0x0020, 0x0002, 0x0001, /*13139*/
  0x4F4D, 0x0020, 0x0002, 0x0001, /*1313A*/
  0x4F4E, 0x0020, 0x0002, 0x0001, /*1313B*/
  0x4F4F, 0x0020, 0x0002, 0x0001, /*1313C*/
  0x4F50, 0x0020, 0x0002, 0x0001, /*1313D*/
  0x4F51, 0x0020, 0x0002, 0x0001, /*1313E*/
  0x4F52, 0x0020, 0x0002, 0x0001, /*1313F*/
  0x4F53, 0x0020, 0x0002, 0x0001, /*13140*/
  0x4F54, 0x0020, 0x0002, 0x0001, /*13141*/
  0x4F55, 0x0020, 0x0002, 0x0001, /*13142*/
  0x4F56, 0x0020, 0x0002, 0x0001, /*13143*/
  0x4F57, 0x0020, 0x0002, 0x0001, /*13144*/
  0x4F58, 0x0020, 0x0002, 0x0001, /*13145*/
  0x4F59, 0x0020, 0x0002, 0x0001, /*13146*/
  0x4F5A, 0x0020, 0x0002, 0x0001, /*13147*/
  0x4F5B, 0x0020, 0x0002, 0x0001, /*13148*/
  0x4F5C, 0x0020, 0x0002, 0x0001, /*13149*/
  0x4F5D, 0x0020, 0x0002, 0x0001, /*1314A*/
  0x4F5E, 0x0020, 0x0002, 0x0001, /*1314B*/
  0x4F5F, 0x0020, 0x0002, 0x0001, /*1314C*/
  0x4F60, 0x0020, 0x0002, 0x0001, /*1314D*/
  0x4F61, 0x0020, 0x0002, 0x0001, /*1314E*/
  0x4F62, 0x0020, 0x0002, 0x0001, /*1314F*/
  0x4F63, 0x0020, 0x0002, 0x0001, /*13150*/
  0x4F64, 0x0020, 0x0002, 0x0001, /*13151*/
  0x4F65, 0x0020, 0x0002, 0x0001, /*13152*/
  0x4F66, 0x0020, 0x0002, 0x0001, /*13153*/
  0x4F67, 0x0020, 0x0002, 0x0001, /*13154*/
  0x4F68, 0x0020, 0x0002, 0x0001, /*13155*/
  0x4F69, 0x0020, 0x0002, 0x0001, /*13156*/
  0x4F6A, 0x0020, 0x0002, 0x0001, /*13157*/
  0x4F6B, 0x0020, 0x0002, 0x0001, /*13158*/
  0x4F6C, 0x0020, 0x0002, 0x0001, /*13159*/
  0x4F6D, 0x0020, 0x0002, 0x0001, /*1315A*/
  0x4F6E, 0x0020, 0x0002, 0x0001, /*1315B*/
  0x4F6F, 0x0020, 0x0002, 0x0001, /*1315C*/
  0x4F70, 0x0020, 0x0002, 0x0001, /*1315D*/
  0x4F71, 0x0020, 0x0002, 0x0001, /*1315E*/
  0x4F72, 0x0020, 0x0002, 0x0001, /*1315F*/
  0x4F73, 0x0020, 0x0002, 0x0001, /*13160*/
  0x4F74, 0x0020, 0x0002, 0x0001, /*13161*/
  0x4F75, 0x0020, 0x0002, 0x0001, /*13162*/
  0x4F76, 0x0020, 0x0002, 0x0001, /*13163*/
  0x4F77, 0x0020, 0x0002, 0x0001, /*13164*/
  0x4F78, 0x0020, 0x0002, 0x0001, /*13165*/
  0x4F79, 0x0020, 0x0002, 0x0001, /*13166*/
  0x4F7A, 0x0020, 0x0002, 0x0001, /*13167*/
  0x4F7B, 0x0020, 0x0002, 0x0001, /*13168*/
  0x4F7C, 0x0020, 0x0002, 0x0001, /*13169*/
  0x4F7D, 0x0020, 0x0002, 0x0001, /*1316A*/
  0x4F7E, 0x0020, 0x0002, 0x0001, /*1316B*/
  0x4F7F, 0x0020, 0x0002, 0x0001, /*1316C*/
  0x4F80, 0x0020, 0x0002, 0x0001, /*1316D*/
  0x4F81, 0x0020, 0x0002, 0x0001, /*1316E*/
  0x4F82, 0x0020, 0x0002, 0x0001, /*1316F*/
  0x4F83, 0x0020, 0x0002, 0x0001, /*13170*/
  0x4F84, 0x0020, 0x0002, 0x0001, /*13171*/
  0x4F85, 0x0020, 0x0002, 0x0001, /*13172*/
  0x4F86, 0x0020, 0x0002, 0x0001, /*13173*/
  0x4F87, 0x0020, 0x0002, 0x0001, /*13174*/
  0x4F88, 0x0020, 0x0002, 0x0001, /*13175*/
  0x4F89, 0x0020, 0x0002, 0x0001, /*13176*/
  0x4F8A, 0x0020, 0x0002, 0x0001, /*13177*/
  0x4F8B, 0x0020, 0x0002, 0x0001, /*13178*/
  0x4F8C, 0x0020, 0x0002, 0x0001, /*13179*/
  0x4F8D, 0x0020, 0x0002, 0x0001, /*1317A*/
  0x4F8E, 0x0020, 0x0002, 0x0001, /*1317B*/
  0x4F8F, 0x0020, 0x0002, 0x0001, /*1317C*/
  0x4F90, 0x0020, 0x0002, 0x0001, /*1317D*/
  0x4F91, 0x0020, 0x0002, 0x0001, /*1317E*/
  0x4F92, 0x0020, 0x0002, 0x0001, /*1317F*/
  0x4F93, 0x0020, 0x0002, 0x0001, /*13180*/
  0x4F94, 0x0020, 0x0002, 0x0001, /*13181*/
  0x4F95, 0x0020, 0x0002, 0x0001, /*13182*/
  0x4F96, 0x0020, 0x0002, 0x0001, /*13183*/
  0x4F97, 0x0020, 0x0002, 0x0001, /*13184*/
  0x4F98, 0x0020, 0x0002, 0x0001, /*13185*/
  0x4F99, 0x0020, 0x0002, 0x0001, /*13186*/
  0x4F9A, 0x0020, 0x0002, 0x0001, /*13187*/
  0x4F9B, 0x0020, 0x0002, 0x0001, /*13188*/
  0x4F9C, 0x0020, 0x0002, 0x0001, /*13189*/
  0x4F9D, 0x0020, 0x0002, 0x0001, /*1318A*/
  0x4F9E, 0x0020, 0x0002, 0x0001, /*1318B*/
  0x4F9F, 0x0020, 0x0002, 0x0001, /*1318C*/
  0x4FA0, 0x0020, 0x0002, 0x0001, /*1318D*/
  0x4FA1, 0x0020, 0x0002, 0x0001, /*1318E*/
  0x4FA2, 0x0020, 0x0002, 0x0001, /*1318F*/
  0x4FA3, 0x0020, 0x0002, 0x0001, /*13190*/
  0x4FA4, 0x0020, 0x0002, 0x0001, /*13191*/
  0x4FA5, 0x0020, 0x0002, 0x0001, /*13192*/
  0x4FA6, 0x0020, 0x0002, 0x0001, /*13193*/
  0x4FA7, 0x0020, 0x0002, 0x0001, /*13194*/
  0x4FA8, 0x0020, 0x0002, 0x0001, /*13195*/
  0x4FA9, 0x0020, 0x0002, 0x0001, /*13196*/
  0x4FAA, 0x0020, 0x0002, 0x0001, /*13197*/
  0x4FAB, 0x0020, 0x0002, 0x0001, /*13198*/
  0x4FAC, 0x0020, 0x0002, 0x0001, /*13199*/
  0x4FAD, 0x0020, 0x0002, 0x0001, /*1319A*/
  0x4FAE, 0x0020, 0x0002, 0x0001, /*1319B*/
  0x4FAF, 0x0020, 0x0002, 0x0001, /*1319C*/
  0x4FB0, 0x0020, 0x0002, 0x0001, /*1319D*/
  0x4FB1, 0x0020, 0x0002, 0x0001, /*1319E*/
  0x4FB2, 0x0020, 0x0002, 0x0001, /*1319F*/
  0x4FB3, 0x0020, 0x0002, 0x0001, /*131A0*/
  0x4FB4, 0x0020, 0x0002, 0x0001, /*131A1*/
  0x4FB5, 0x0020, 0x0002, 0x0001, /*131A2*/
  0x4FB6, 0x0020, 0x0002, 0x0001, /*131A3*/
  0x4FB7, 0x0020, 0x0002, 0x0001, /*131A4*/
  0x4FB8, 0x0020, 0x0002, 0x0001, /*131A5*/
  0x4FB9, 0x0020, 0x0002, 0x0001, /*131A6*/
  0x4FBA, 0x0020, 0x0002, 0x0001, /*131A7*/
  0x4FBB, 0x0020, 0x0002, 0x0001, /*131A8*/
  0x4FBC, 0x0020, 0x0002, 0x0001, /*131A9*/
  0x4FBD, 0x0020, 0x0002, 0x0001, /*131AA*/
  0x4FBE, 0x0020, 0x0002, 0x0001, /*131AB*/
  0x4FBF, 0x0020, 0x0002, 0x0001, /*131AC*/
  0x4FC0, 0x0020, 0x0002, 0x0001, /*131AD*/
  0x4FC1, 0x0020, 0x0002, 0x0001, /*131AE*/
  0x4FC2, 0x0020, 0x0002, 0x0001, /*131AF*/
  0x4FC3, 0x0020, 0x0002, 0x0001, /*131B0*/
  0x4FC4, 0x0020, 0x0002, 0x0001, /*131B1*/
  0x4FC5, 0x0020, 0x0002, 0x0001, /*131B2*/
  0x4FC6, 0x0020, 0x0002, 0x0001, /*131B3*/
  0x4FC7, 0x0020, 0x0002, 0x0001, /*131B4*/
  0x4FC8, 0x0020, 0x0002, 0x0001, /*131B5*/
  0x4FC9, 0x0020, 0x0002, 0x0001, /*131B6*/
  0x4FCA, 0x0020, 0x0002, 0x0001, /*131B7*/
  0x4FCB, 0x0020, 0x0002, 0x0001, /*131B8*/
  0x4FCC, 0x0020, 0x0002, 0x0001, /*131B9*/
  0x4FCD, 0x0020, 0x0002, 0x0001, /*131BA*/
  0x4FCE, 0x0020, 0x0002, 0x0001, /*131BB*/
  0x4FCF, 0x0020, 0x0002, 0x0001, /*131BC*/
  0x4FD0, 0x0020, 0x0002, 0x0001, /*131BD*/
  0x4FD1, 0x0020, 0x0002, 0x0001, /*131BE*/
  0x4FD2, 0x0020, 0x0002, 0x0001, /*131BF*/
  0x4FD3, 0x0020, 0x0002, 0x0001, /*131C0*/
  0x4FD4, 0x0020, 0x0002, 0x0001, /*131C1*/
  0x4FD5, 0x0020, 0x0002, 0x0001, /*131C2*/
  0x4FD6, 0x0020, 0x0002, 0x0001, /*131C3*/
  0x4FD7, 0x0020, 0x0002, 0x0001, /*131C4*/
  0x4FD8, 0x0020, 0x0002, 0x0001, /*131C5*/
  0x4FD9, 0x0020, 0x0002, 0x0001, /*131C6*/
  0x4FDA, 0x0020, 0x0002, 0x0001, /*131C7*/
  0x4FDB, 0x0020, 0x0002, 0x0001, /*131C8*/
  0x4FDC, 0x0020, 0x0002, 0x0001, /*131C9*/
  0x4FDD, 0x0020, 0x0002, 0x0001, /*131CA*/
  0x4FDE, 0x0020, 0x0002, 0x0001, /*131CB*/
  0x4FDF, 0x0020, 0x0002, 0x0001, /*131CC*/
  0x4FE0, 0x0020, 0x0002, 0x0001, /*131CD*/
  0x4FE1, 0x0020, 0x0002, 0x0001, /*131CE*/
  0x4FE2, 0x0020, 0x0002, 0x0001, /*131CF*/
  0x4FE3, 0x0020, 0x0002, 0x0001, /*131D0*/
  0x4FE4, 0x0020, 0x0002, 0x0001, /*131D1*/
  0x4FE5, 0x0020, 0x0002, 0x0001, /*131D2*/
  0x4FE6, 0x0020, 0x0002, 0x0001, /*131D3*/
  0x4FE7, 0x0020, 0x0002, 0x0001, /*131D4*/
  0x4FE8, 0x0020, 0x0002, 0x0001, /*131D5*/
  0x4FE9, 0x0020, 0x0002, 0x0001, /*131D6*/
  0x4FEA, 0x0020, 0x0002, 0x0001, /*131D7*/
  0x4FEB, 0x0020, 0x0002, 0x0001, /*131D8*/
  0x4FEC, 0x0020, 0x0002, 0x0001, /*131D9*/
  0x4FED, 0x0020, 0x0002, 0x0001, /*131DA*/
  0x4FEE, 0x0020, 0x0002, 0x0001, /*131DB*/
  0x4FEF, 0x0020, 0x0002, 0x0001, /*131DC*/
  0x4FF0, 0x0020, 0x0002, 0x0001, /*131DD*/
  0x4FF1, 0x0020, 0x0002, 0x0001, /*131DE*/
  0x4FF2, 0x0020, 0x0002, 0x0001, /*131DF*/
  0x4FF3, 0x0020, 0x0002, 0x0001, /*131E0*/
  0x4FF4, 0x0020, 0x0002, 0x0001, /*131E1*/
  0x4FF5, 0x0020, 0x0002, 0x0001, /*131E2*/
  0x4FF6, 0x0020, 0x0002, 0x0001, /*131E3*/
  0x4FF7, 0x0020, 0x0002, 0x0001, /*131E4*/
  0x4FF8, 0x0020, 0x0002, 0x0001, /*131E5*/
  0x4FF9, 0x0020, 0x0002, 0x0001, /*131E6*/
  0x4FFA, 0x0020, 0x0002, 0x0001, /*131E7*/
  0x4FFB, 0x0020, 0x0002, 0x0001, /*131E8*/
  0x4FFC, 0x0020, 0x0002, 0x0001, /*131E9*/
  0x4FFD, 0x0020, 0x0002, 0x0001, /*131EA*/
  0x4FFE, 0x0020, 0x0002, 0x0001, /*131EB*/
  0x4FFF, 0x0020, 0x0002, 0x0001, /*131EC*/
  0x5000, 0x0020, 0x0002, 0x0001, /*131ED*/
  0x5001, 0x0020, 0x0002, 0x0001, /*131EE*/
  0x5002, 0x0020, 0x0002, 0x0001, /*131EF*/
  0x5003, 0x0020, 0x0002, 0x0001, /*131F0*/
  0x5004, 0x0020, 0x0002, 0x0001, /*131F1*/
  0x5005, 0x0020, 0x0002, 0x0001, /*131F2*/
  0x5006, 0x0020, 0x0002, 0x0001, /*131F3*/
  0x5007, 0x0020, 0x0002, 0x0001, /*131F4*/
  0x5008, 0x0020, 0x0002, 0x0001, /*131F5*/
  0x5009, 0x0020, 0x0002, 0x0001, /*131F6*/
  0x500A, 0x0020, 0x0002, 0x0001, /*131F7*/
  0x500B, 0x0020, 0x0002, 0x0001, /*131F8*/
  0x500C, 0x0020, 0x0002, 0x0001, /*131F9*/
  0x500D, 0x0020, 0x0002, 0x0001, /*131FA*/
  0x500E, 0x0020, 0x0002, 0x0001, /*131FB*/
  0x500F, 0x0020, 0x0002, 0x0001, /*131FC*/
  0x5010, 0x0020, 0x0002, 0x0001, /*131FD*/
  0x5011, 0x0020, 0x0002, 0x0001, /*131FE*/
  0x5012, 0x0020, 0x0002, 0x0001  /*131FF*/
};

uint16 uca900_p132[]= {
  0x5013, 0x0020, 0x0002, 0x0001, /*13200*/
  0x5014, 0x0020, 0x0002, 0x0001, /*13201*/
  0x5015, 0x0020, 0x0002, 0x0001, /*13202*/
  0x5016, 0x0020, 0x0002, 0x0001, /*13203*/
  0x5017, 0x0020, 0x0002, 0x0001, /*13204*/
  0x5018, 0x0020, 0x0002, 0x0001, /*13205*/
  0x5019, 0x0020, 0x0002, 0x0001, /*13206*/
  0x501A, 0x0020, 0x0002, 0x0001, /*13207*/
  0x501B, 0x0020, 0x0002, 0x0001, /*13208*/
  0x501C, 0x0020, 0x0002, 0x0001, /*13209*/
  0x501D, 0x0020, 0x0002, 0x0001, /*1320A*/
  0x501E, 0x0020, 0x0002, 0x0001, /*1320B*/
  0x501F, 0x0020, 0x0002, 0x0001, /*1320C*/
  0x5020, 0x0020, 0x0002, 0x0001, /*1320D*/
  0x5021, 0x0020, 0x0002, 0x0001, /*1320E*/
  0x5022, 0x0020, 0x0002, 0x0001, /*1320F*/
  0x5023, 0x0020, 0x0002, 0x0001, /*13210*/
  0x5024, 0x0020, 0x0002, 0x0001, /*13211*/
  0x5025, 0x0020, 0x0002, 0x0001, /*13212*/
  0x5026, 0x0020, 0x0002, 0x0001, /*13213*/
  0x5027, 0x0020, 0x0002, 0x0001, /*13214*/
  0x5028, 0x0020, 0x0002, 0x0001, /*13215*/
  0x5029, 0x0020, 0x0002, 0x0001, /*13216*/
  0x502A, 0x0020, 0x0002, 0x0001, /*13217*/
  0x502B, 0x0020, 0x0002, 0x0001, /*13218*/
  0x502C, 0x0020, 0x0002, 0x0001, /*13219*/
  0x502D, 0x0020, 0x0002, 0x0001, /*1321A*/
  0x502E, 0x0020, 0x0002, 0x0001, /*1321B*/
  0x502F, 0x0020, 0x0002, 0x0001, /*1321C*/
  0x5030, 0x0020, 0x0002, 0x0001, /*1321D*/
  0x5031, 0x0020, 0x0002, 0x0001, /*1321E*/
  0x5032, 0x0020, 0x0002, 0x0001, /*1321F*/
  0x5033, 0x0020, 0x0002, 0x0001, /*13220*/
  0x5034, 0x0020, 0x0002, 0x0001, /*13221*/
  0x5035, 0x0020, 0x0002, 0x0001, /*13222*/
  0x5036, 0x0020, 0x0002, 0x0001, /*13223*/
  0x5037, 0x0020, 0x0002, 0x0001, /*13224*/
  0x5038, 0x0020, 0x0002, 0x0001, /*13225*/
  0x5039, 0x0020, 0x0002, 0x0001, /*13226*/
  0x503A, 0x0020, 0x0002, 0x0001, /*13227*/
  0x503B, 0x0020, 0x0002, 0x0001, /*13228*/
  0x503C, 0x0020, 0x0002, 0x0001, /*13229*/
  0x503D, 0x0020, 0x0002, 0x0001, /*1322A*/
  0x503E, 0x0020, 0x0002, 0x0001, /*1322B*/
  0x503F, 0x0020, 0x0002, 0x0001, /*1322C*/
  0x5040, 0x0020, 0x0002, 0x0001, /*1322D*/
  0x5041, 0x0020, 0x0002, 0x0001, /*1322E*/
  0x5042, 0x0020, 0x0002, 0x0001, /*1322F*/
  0x5043, 0x0020, 0x0002, 0x0001, /*13230*/
  0x5044, 0x0020, 0x0002, 0x0001, /*13231*/
  0x5045, 0x0020, 0x0002, 0x0001, /*13232*/
  0x5046, 0x0020, 0x0002, 0x0001, /*13233*/
  0x5047, 0x0020, 0x0002, 0x0001, /*13234*/
  0x5048, 0x0020, 0x0002, 0x0001, /*13235*/
  0x5049, 0x0020, 0x0002, 0x0001, /*13236*/
  0x504A, 0x0020, 0x0002, 0x0001, /*13237*/
  0x504B, 0x0020, 0x0002, 0x0001, /*13238*/
  0x504C, 0x0020, 0x0002, 0x0001, /*13239*/
  0x504D, 0x0020, 0x0002, 0x0001, /*1323A*/
  0x504E, 0x0020, 0x0002, 0x0001, /*1323B*/
  0x504F, 0x0020, 0x0002, 0x0001, /*1323C*/
  0x5050, 0x0020, 0x0002, 0x0001, /*1323D*/
  0x5051, 0x0020, 0x0002, 0x0001, /*1323E*/
  0x5052, 0x0020, 0x0002, 0x0001, /*1323F*/
  0x5053, 0x0020, 0x0002, 0x0001, /*13240*/
  0x5054, 0x0020, 0x0002, 0x0001, /*13241*/
  0x5055, 0x0020, 0x0002, 0x0001, /*13242*/
  0x5056, 0x0020, 0x0002, 0x0001, /*13243*/
  0x5057, 0x0020, 0x0002, 0x0001, /*13244*/
  0x5058, 0x0020, 0x0002, 0x0001, /*13245*/
  0x5059, 0x0020, 0x0002, 0x0001, /*13246*/
  0x505A, 0x0020, 0x0002, 0x0001, /*13247*/
  0x505B, 0x0020, 0x0002, 0x0001, /*13248*/
  0x505C, 0x0020, 0x0002, 0x0001, /*13249*/
  0x505D, 0x0020, 0x0002, 0x0001, /*1324A*/
  0x505E, 0x0020, 0x0002, 0x0001, /*1324B*/
  0x505F, 0x0020, 0x0002, 0x0001, /*1324C*/
  0x5060, 0x0020, 0x0002, 0x0001, /*1324D*/
  0x5061, 0x0020, 0x0002, 0x0001, /*1324E*/
  0x5062, 0x0020, 0x0002, 0x0001, /*1324F*/
  0x5063, 0x0020, 0x0002, 0x0001, /*13250*/
  0x5064, 0x0020, 0x0002, 0x0001, /*13251*/
  0x5065, 0x0020, 0x0002, 0x0001, /*13252*/
  0x5066, 0x0020, 0x0002, 0x0001, /*13253*/
  0x5067, 0x0020, 0x0002, 0x0001, /*13254*/
  0x5068, 0x0020, 0x0002, 0x0001, /*13255*/
  0x5069, 0x0020, 0x0002, 0x0001, /*13256*/
  0x506A, 0x0020, 0x0002, 0x0001, /*13257*/
  0x506B, 0x0020, 0x0002, 0x0001, /*13258*/
  0x506C, 0x0020, 0x0002, 0x0001, /*13259*/
  0x506D, 0x0020, 0x0002, 0x0001, /*1325A*/
  0x506E, 0x0020, 0x0002, 0x0001, /*1325B*/
  0x506F, 0x0020, 0x0002, 0x0001, /*1325C*/
  0x5070, 0x0020, 0x0002, 0x0001, /*1325D*/
  0x5071, 0x0020, 0x0002, 0x0001, /*1325E*/
  0x5072, 0x0020, 0x0002, 0x0001, /*1325F*/
  0x5073, 0x0020, 0x0002, 0x0001, /*13260*/
  0x5074, 0x0020, 0x0002, 0x0001, /*13261*/
  0x5075, 0x0020, 0x0002, 0x0001, /*13262*/
  0x5076, 0x0020, 0x0002, 0x0001, /*13263*/
  0x5077, 0x0020, 0x0002, 0x0001, /*13264*/
  0x5078, 0x0020, 0x0002, 0x0001, /*13265*/
  0x5079, 0x0020, 0x0002, 0x0001, /*13266*/
  0x507A, 0x0020, 0x0002, 0x0001, /*13267*/
  0x507B, 0x0020, 0x0002, 0x0001, /*13268*/
  0x507C, 0x0020, 0x0002, 0x0001, /*13269*/
  0x507D, 0x0020, 0x0002, 0x0001, /*1326A*/
  0x507E, 0x0020, 0x0002, 0x0001, /*1326B*/
  0x507F, 0x0020, 0x0002, 0x0001, /*1326C*/
  0x5080, 0x0020, 0x0002, 0x0001, /*1326D*/
  0x5081, 0x0020, 0x0002, 0x0001, /*1326E*/
  0x5082, 0x0020, 0x0002, 0x0001, /*1326F*/
  0x5083, 0x0020, 0x0002, 0x0001, /*13270*/
  0x5084, 0x0020, 0x0002, 0x0001, /*13271*/
  0x5085, 0x0020, 0x0002, 0x0001, /*13272*/
  0x5086, 0x0020, 0x0002, 0x0001, /*13273*/
  0x5087, 0x0020, 0x0002, 0x0001, /*13274*/
  0x5088, 0x0020, 0x0002, 0x0001, /*13275*/
  0x5089, 0x0020, 0x0002, 0x0001, /*13276*/
  0x508A, 0x0020, 0x0002, 0x0001, /*13277*/
  0x508B, 0x0020, 0x0002, 0x0001, /*13278*/
  0x508C, 0x0020, 0x0002, 0x0001, /*13279*/
  0x508D, 0x0020, 0x0002, 0x0001, /*1327A*/
  0x508E, 0x0020, 0x0002, 0x0001, /*1327B*/
  0x508F, 0x0020, 0x0002, 0x0001, /*1327C*/
  0x5090, 0x0020, 0x0002, 0x0001, /*1327D*/
  0x5091, 0x0020, 0x0002, 0x0001, /*1327E*/
  0x5092, 0x0020, 0x0002, 0x0001, /*1327F*/
  0x5093, 0x0020, 0x0002, 0x0001, /*13280*/
  0x5094, 0x0020, 0x0002, 0x0001, /*13281*/
  0x5095, 0x0020, 0x0002, 0x0001, /*13282*/
  0x5096, 0x0020, 0x0002, 0x0001, /*13283*/
  0x5097, 0x0020, 0x0002, 0x0001, /*13284*/
  0x5098, 0x0020, 0x0002, 0x0001, /*13285*/
  0x5099, 0x0020, 0x0002, 0x0001, /*13286*/
  0x509A, 0x0020, 0x0002, 0x0001, /*13287*/
  0x509B, 0x0020, 0x0002, 0x0001, /*13288*/
  0x509C, 0x0020, 0x0002, 0x0001, /*13289*/
  0x509D, 0x0020, 0x0002, 0x0001, /*1328A*/
  0x509E, 0x0020, 0x0002, 0x0001, /*1328B*/
  0x509F, 0x0020, 0x0002, 0x0001, /*1328C*/
  0x50A0, 0x0020, 0x0002, 0x0001, /*1328D*/
  0x50A1, 0x0020, 0x0002, 0x0001, /*1328E*/
  0x50A2, 0x0020, 0x0002, 0x0001, /*1328F*/
  0x50A3, 0x0020, 0x0002, 0x0001, /*13290*/
  0x50A4, 0x0020, 0x0002, 0x0001, /*13291*/
  0x50A5, 0x0020, 0x0002, 0x0001, /*13292*/
  0x50A6, 0x0020, 0x0002, 0x0001, /*13293*/
  0x50A7, 0x0020, 0x0002, 0x0001, /*13294*/
  0x50A8, 0x0020, 0x0002, 0x0001, /*13295*/
  0x50A9, 0x0020, 0x0002, 0x0001, /*13296*/
  0x50AA, 0x0020, 0x0002, 0x0001, /*13297*/
  0x50AB, 0x0020, 0x0002, 0x0001, /*13298*/
  0x50AC, 0x0020, 0x0002, 0x0001, /*13299*/
  0x50AD, 0x0020, 0x0002, 0x0001, /*1329A*/
  0x50AE, 0x0020, 0x0002, 0x0001, /*1329B*/
  0x50AF, 0x0020, 0x0002, 0x0001, /*1329C*/
  0x50B0, 0x0020, 0x0002, 0x0001, /*1329D*/
  0x50B1, 0x0020, 0x0002, 0x0001, /*1329E*/
  0x50B2, 0x0020, 0x0002, 0x0001, /*1329F*/
  0x50B3, 0x0020, 0x0002, 0x0001, /*132A0*/
  0x50B4, 0x0020, 0x0002, 0x0001, /*132A1*/
  0x50B5, 0x0020, 0x0002, 0x0001, /*132A2*/
  0x50B6, 0x0020, 0x0002, 0x0001, /*132A3*/
  0x50B7, 0x0020, 0x0002, 0x0001, /*132A4*/
  0x50B8, 0x0020, 0x0002, 0x0001, /*132A5*/
  0x50B9, 0x0020, 0x0002, 0x0001, /*132A6*/
  0x50BA, 0x0020, 0x0002, 0x0001, /*132A7*/
  0x50BB, 0x0020, 0x0002, 0x0001, /*132A8*/
  0x50BC, 0x0020, 0x0002, 0x0001, /*132A9*/
  0x50BD, 0x0020, 0x0002, 0x0001, /*132AA*/
  0x50BE, 0x0020, 0x0002, 0x0001, /*132AB*/
  0x50BF, 0x0020, 0x0002, 0x0001, /*132AC*/
  0x50C0, 0x0020, 0x0002, 0x0001, /*132AD*/
  0x50C1, 0x0020, 0x0002, 0x0001, /*132AE*/
  0x50C2, 0x0020, 0x0002, 0x0001, /*132AF*/
  0x50C3, 0x0020, 0x0002, 0x0001, /*132B0*/
  0x50C4, 0x0020, 0x0002, 0x0001, /*132B1*/
  0x50C5, 0x0020, 0x0002, 0x0001, /*132B2*/
  0x50C6, 0x0020, 0x0002, 0x0001, /*132B3*/
  0x50C7, 0x0020, 0x0002, 0x0001, /*132B4*/
  0x50C8, 0x0020, 0x0002, 0x0001, /*132B5*/
  0x50C9, 0x0020, 0x0002, 0x0001, /*132B6*/
  0x50CA, 0x0020, 0x0002, 0x0001, /*132B7*/
  0x50CB, 0x0020, 0x0002, 0x0001, /*132B8*/
  0x50CC, 0x0020, 0x0002, 0x0001, /*132B9*/
  0x50CD, 0x0020, 0x0002, 0x0001, /*132BA*/
  0x50CE, 0x0020, 0x0002, 0x0001, /*132BB*/
  0x50CF, 0x0020, 0x0002, 0x0001, /*132BC*/
  0x50D0, 0x0020, 0x0002, 0x0001, /*132BD*/
  0x50D1, 0x0020, 0x0002, 0x0001, /*132BE*/
  0x50D2, 0x0020, 0x0002, 0x0001, /*132BF*/
  0x50D3, 0x0020, 0x0002, 0x0001, /*132C0*/
  0x50D4, 0x0020, 0x0002, 0x0001, /*132C1*/
  0x50D5, 0x0020, 0x0002, 0x0001, /*132C2*/
  0x50D6, 0x0020, 0x0002, 0x0001, /*132C3*/
  0x50D7, 0x0020, 0x0002, 0x0001, /*132C4*/
  0x50D8, 0x0020, 0x0002, 0x0001, /*132C5*/
  0x50D9, 0x0020, 0x0002, 0x0001, /*132C6*/
  0x50DA, 0x0020, 0x0002, 0x0001, /*132C7*/
  0x50DB, 0x0020, 0x0002, 0x0001, /*132C8*/
  0x50DC, 0x0020, 0x0002, 0x0001, /*132C9*/
  0x50DD, 0x0020, 0x0002, 0x0001, /*132CA*/
  0x50DE, 0x0020, 0x0002, 0x0001, /*132CB*/
  0x50DF, 0x0020, 0x0002, 0x0001, /*132CC*/
  0x50E0, 0x0020, 0x0002, 0x0001, /*132CD*/
  0x50E1, 0x0020, 0x0002, 0x0001, /*132CE*/
  0x50E2, 0x0020, 0x0002, 0x0001, /*132CF*/
  0x50E3, 0x0020, 0x0002, 0x0001, /*132D0*/
  0x50E4, 0x0020, 0x0002, 0x0001, /*132D1*/
  0x50E5, 0x0020, 0x0002, 0x0001, /*132D2*/
  0x50E6, 0x0020, 0x0002, 0x0001, /*132D3*/
  0x50E7, 0x0020, 0x0002, 0x0001, /*132D4*/
  0x50E8, 0x0020, 0x0002, 0x0001, /*132D5*/
  0x50E9, 0x0020, 0x0002, 0x0001, /*132D6*/
  0x50EA, 0x0020, 0x0002, 0x0001, /*132D7*/
  0x50EB, 0x0020, 0x0002, 0x0001, /*132D8*/
  0x50EC, 0x0020, 0x0002, 0x0001, /*132D9*/
  0x50ED, 0x0020, 0x0002, 0x0001, /*132DA*/
  0x50EE, 0x0020, 0x0002, 0x0001, /*132DB*/
  0x50EF, 0x0020, 0x0002, 0x0001, /*132DC*/
  0x50F0, 0x0020, 0x0002, 0x0001, /*132DD*/
  0x50F1, 0x0020, 0x0002, 0x0001, /*132DE*/
  0x50F2, 0x0020, 0x0002, 0x0001, /*132DF*/
  0x50F3, 0x0020, 0x0002, 0x0001, /*132E0*/
  0x50F4, 0x0020, 0x0002, 0x0001, /*132E1*/
  0x50F5, 0x0020, 0x0002, 0x0001, /*132E2*/
  0x50F6, 0x0020, 0x0002, 0x0001, /*132E3*/
  0x50F7, 0x0020, 0x0002, 0x0001, /*132E4*/
  0x50F8, 0x0020, 0x0002, 0x0001, /*132E5*/
  0x50F9, 0x0020, 0x0002, 0x0001, /*132E6*/
  0x50FA, 0x0020, 0x0002, 0x0001, /*132E7*/
  0x50FB, 0x0020, 0x0002, 0x0001, /*132E8*/
  0x50FC, 0x0020, 0x0002, 0x0001, /*132E9*/
  0x50FD, 0x0020, 0x0002, 0x0001, /*132EA*/
  0x50FE, 0x0020, 0x0002, 0x0001, /*132EB*/
  0x50FF, 0x0020, 0x0002, 0x0001, /*132EC*/
  0x5100, 0x0020, 0x0002, 0x0001, /*132ED*/
  0x5101, 0x0020, 0x0002, 0x0001, /*132EE*/
  0x5102, 0x0020, 0x0002, 0x0001, /*132EF*/
  0x5103, 0x0020, 0x0002, 0x0001, /*132F0*/
  0x5104, 0x0020, 0x0002, 0x0001, /*132F1*/
  0x5105, 0x0020, 0x0002, 0x0001, /*132F2*/
  0x5106, 0x0020, 0x0002, 0x0001, /*132F3*/
  0x5107, 0x0020, 0x0002, 0x0001, /*132F4*/
  0x5108, 0x0020, 0x0002, 0x0001, /*132F5*/
  0x5109, 0x0020, 0x0002, 0x0001, /*132F6*/
  0x510A, 0x0020, 0x0002, 0x0001, /*132F7*/
  0x510B, 0x0020, 0x0002, 0x0001, /*132F8*/
  0x510C, 0x0020, 0x0002, 0x0001, /*132F9*/
  0x510D, 0x0020, 0x0002, 0x0001, /*132FA*/
  0x510E, 0x0020, 0x0002, 0x0001, /*132FB*/
  0x510F, 0x0020, 0x0002, 0x0001, /*132FC*/
  0x5110, 0x0020, 0x0002, 0x0001, /*132FD*/
  0x5111, 0x0020, 0x0002, 0x0001, /*132FE*/
  0x5112, 0x0020, 0x0002, 0x0001  /*132FF*/
};

uint16 uca900_p133[]= {
  0x5113, 0x0020, 0x0002, 0x0001, /*13300*/
  0x5114, 0x0020, 0x0002, 0x0001, /*13301*/
  0x5115, 0x0020, 0x0002, 0x0001, /*13302*/
  0x5116, 0x0020, 0x0002, 0x0001, /*13303*/
  0x5117, 0x0020, 0x0002, 0x0001, /*13304*/
  0x5118, 0x0020, 0x0002, 0x0001, /*13305*/
  0x5119, 0x0020, 0x0002, 0x0001, /*13306*/
  0x511A, 0x0020, 0x0002, 0x0001, /*13307*/
  0x511B, 0x0020, 0x0002, 0x0001, /*13308*/
  0x511C, 0x0020, 0x0002, 0x0001, /*13309*/
  0x511D, 0x0020, 0x0002, 0x0001, /*1330A*/
  0x511E, 0x0020, 0x0002, 0x0001, /*1330B*/
  0x511F, 0x0020, 0x0002, 0x0001, /*1330C*/
  0x5120, 0x0020, 0x0002, 0x0001, /*1330D*/
  0x5121, 0x0020, 0x0002, 0x0001, /*1330E*/
  0x5122, 0x0020, 0x0002, 0x0001, /*1330F*/
  0x5123, 0x0020, 0x0002, 0x0001, /*13310*/
  0x5124, 0x0020, 0x0002, 0x0001, /*13311*/
  0x5125, 0x0020, 0x0002, 0x0001, /*13312*/
  0x5126, 0x0020, 0x0002, 0x0001, /*13313*/
  0x5127, 0x0020, 0x0002, 0x0001, /*13314*/
  0x5128, 0x0020, 0x0002, 0x0001, /*13315*/
  0x5129, 0x0020, 0x0002, 0x0001, /*13316*/
  0x512A, 0x0020, 0x0002, 0x0001, /*13317*/
  0x512B, 0x0020, 0x0002, 0x0001, /*13318*/
  0x512C, 0x0020, 0x0002, 0x0001, /*13319*/
  0x512D, 0x0020, 0x0002, 0x0001, /*1331A*/
  0x512E, 0x0020, 0x0002, 0x0001, /*1331B*/
  0x512F, 0x0020, 0x0002, 0x0001, /*1331C*/
  0x5130, 0x0020, 0x0002, 0x0001, /*1331D*/
  0x5131, 0x0020, 0x0002, 0x0001, /*1331E*/
  0x5132, 0x0020, 0x0002, 0x0001, /*1331F*/
  0x5133, 0x0020, 0x0002, 0x0001, /*13320*/
  0x5134, 0x0020, 0x0002, 0x0001, /*13321*/
  0x5135, 0x0020, 0x0002, 0x0001, /*13322*/
  0x5136, 0x0020, 0x0002, 0x0001, /*13323*/
  0x5137, 0x0020, 0x0002, 0x0001, /*13324*/
  0x5138, 0x0020, 0x0002, 0x0001, /*13325*/
  0x5139, 0x0020, 0x0002, 0x0001, /*13326*/
  0x513A, 0x0020, 0x0002, 0x0001, /*13327*/
  0x513B, 0x0020, 0x0002, 0x0001, /*13328*/
  0x513C, 0x0020, 0x0002, 0x0001, /*13329*/
  0x513D, 0x0020, 0x0002, 0x0001, /*1332A*/
  0x513E, 0x0020, 0x0002, 0x0001, /*1332B*/
  0x513F, 0x0020, 0x0002, 0x0001, /*1332C*/
  0x5140, 0x0020, 0x0002, 0x0001, /*1332D*/
  0x5141, 0x0020, 0x0002, 0x0001, /*1332E*/
  0x5142, 0x0020, 0x0002, 0x0001, /*1332F*/
  0x5143, 0x0020, 0x0002, 0x0001, /*13330*/
  0x5144, 0x0020, 0x0002, 0x0001, /*13331*/
  0x5145, 0x0020, 0x0002, 0x0001, /*13332*/
  0x5146, 0x0020, 0x0002, 0x0001, /*13333*/
  0x5147, 0x0020, 0x0002, 0x0001, /*13334*/
  0x5148, 0x0020, 0x0002, 0x0001, /*13335*/
  0x5149, 0x0020, 0x0002, 0x0001, /*13336*/
  0x514A, 0x0020, 0x0002, 0x0001, /*13337*/
  0x514B, 0x0020, 0x0002, 0x0001, /*13338*/
  0x514C, 0x0020, 0x0002, 0x0001, /*13339*/
  0x514D, 0x0020, 0x0002, 0x0001, /*1333A*/
  0x514E, 0x0020, 0x0002, 0x0001, /*1333B*/
  0x514F, 0x0020, 0x0002, 0x0001, /*1333C*/
  0x5150, 0x0020, 0x0002, 0x0001, /*1333D*/
  0x5151, 0x0020, 0x0002, 0x0001, /*1333E*/
  0x5152, 0x0020, 0x0002, 0x0001, /*1333F*/
  0x5153, 0x0020, 0x0002, 0x0001, /*13340*/
  0x5154, 0x0020, 0x0002, 0x0001, /*13341*/
  0x5155, 0x0020, 0x0002, 0x0001, /*13342*/
  0x5156, 0x0020, 0x0002, 0x0001, /*13343*/
  0x5157, 0x0020, 0x0002, 0x0001, /*13344*/
  0x5158, 0x0020, 0x0002, 0x0001, /*13345*/
  0x5159, 0x0020, 0x0002, 0x0001, /*13346*/
  0x515A, 0x0020, 0x0002, 0x0001, /*13347*/
  0x515B, 0x0020, 0x0002, 0x0001, /*13348*/
  0x515C, 0x0020, 0x0002, 0x0001, /*13349*/
  0x515D, 0x0020, 0x0002, 0x0001, /*1334A*/
  0x515E, 0x0020, 0x0002, 0x0001, /*1334B*/
  0x515F, 0x0020, 0x0002, 0x0001, /*1334C*/
  0x5160, 0x0020, 0x0002, 0x0001, /*1334D*/
  0x5161, 0x0020, 0x0002, 0x0001, /*1334E*/
  0x5162, 0x0020, 0x0002, 0x0001, /*1334F*/
  0x5163, 0x0020, 0x0002, 0x0001, /*13350*/
  0x5164, 0x0020, 0x0002, 0x0001, /*13351*/
  0x5165, 0x0020, 0x0002, 0x0001, /*13352*/
  0x5166, 0x0020, 0x0002, 0x0001, /*13353*/
  0x5167, 0x0020, 0x0002, 0x0001, /*13354*/
  0x5168, 0x0020, 0x0002, 0x0001, /*13355*/
  0x5169, 0x0020, 0x0002, 0x0001, /*13356*/
  0x516A, 0x0020, 0x0002, 0x0001, /*13357*/
  0x516B, 0x0020, 0x0002, 0x0001, /*13358*/
  0x516C, 0x0020, 0x0002, 0x0001, /*13359*/
  0x516D, 0x0020, 0x0002, 0x0001, /*1335A*/
  0x516E, 0x0020, 0x0002, 0x0001, /*1335B*/
  0x516F, 0x0020, 0x0002, 0x0001, /*1335C*/
  0x5170, 0x0020, 0x0002, 0x0001, /*1335D*/
  0x5171, 0x0020, 0x0002, 0x0001, /*1335E*/
  0x5172, 0x0020, 0x0002, 0x0001, /*1335F*/
  0x5173, 0x0020, 0x0002, 0x0001, /*13360*/
  0x5174, 0x0020, 0x0002, 0x0001, /*13361*/
  0x5175, 0x0020, 0x0002, 0x0001, /*13362*/
  0x5176, 0x0020, 0x0002, 0x0001, /*13363*/
  0x5177, 0x0020, 0x0002, 0x0001, /*13364*/
  0x5178, 0x0020, 0x0002, 0x0001, /*13365*/
  0x5179, 0x0020, 0x0002, 0x0001, /*13366*/
  0x517A, 0x0020, 0x0002, 0x0001, /*13367*/
  0x517B, 0x0020, 0x0002, 0x0001, /*13368*/
  0x517C, 0x0020, 0x0002, 0x0001, /*13369*/
  0x517D, 0x0020, 0x0002, 0x0001, /*1336A*/
  0x517E, 0x0020, 0x0002, 0x0001, /*1336B*/
  0x517F, 0x0020, 0x0002, 0x0001, /*1336C*/
  0x5180, 0x0020, 0x0002, 0x0001, /*1336D*/
  0x5181, 0x0020, 0x0002, 0x0001, /*1336E*/
  0x5182, 0x0020, 0x0002, 0x0001, /*1336F*/
  0x5183, 0x0020, 0x0002, 0x0001, /*13370*/
  0x5184, 0x0020, 0x0002, 0x0001, /*13371*/
  0x5185, 0x0020, 0x0002, 0x0001, /*13372*/
  0x5186, 0x0020, 0x0002, 0x0001, /*13373*/
  0x5187, 0x0020, 0x0002, 0x0001, /*13374*/
  0x5188, 0x0020, 0x0002, 0x0001, /*13375*/
  0x5189, 0x0020, 0x0002, 0x0001, /*13376*/
  0x518A, 0x0020, 0x0002, 0x0001, /*13377*/
  0x518B, 0x0020, 0x0002, 0x0001, /*13378*/
  0x518C, 0x0020, 0x0002, 0x0001, /*13379*/
  0x518D, 0x0020, 0x0002, 0x0001, /*1337A*/
  0x518E, 0x0020, 0x0002, 0x0001, /*1337B*/
  0x518F, 0x0020, 0x0002, 0x0001, /*1337C*/
  0x5190, 0x0020, 0x0002, 0x0001, /*1337D*/
  0x5191, 0x0020, 0x0002, 0x0001, /*1337E*/
  0x5192, 0x0020, 0x0002, 0x0001, /*1337F*/
  0x5193, 0x0020, 0x0002, 0x0001, /*13380*/
  0x5194, 0x0020, 0x0002, 0x0001, /*13381*/
  0x5195, 0x0020, 0x0002, 0x0001, /*13382*/
  0x5196, 0x0020, 0x0002, 0x0001, /*13383*/
  0x5197, 0x0020, 0x0002, 0x0001, /*13384*/
  0x5198, 0x0020, 0x0002, 0x0001, /*13385*/
  0x5199, 0x0020, 0x0002, 0x0001, /*13386*/
  0x519A, 0x0020, 0x0002, 0x0001, /*13387*/
  0x519B, 0x0020, 0x0002, 0x0001, /*13388*/
  0x519C, 0x0020, 0x0002, 0x0001, /*13389*/
  0x519D, 0x0020, 0x0002, 0x0001, /*1338A*/
  0x519E, 0x0020, 0x0002, 0x0001, /*1338B*/
  0x519F, 0x0020, 0x0002, 0x0001, /*1338C*/
  0x51A0, 0x0020, 0x0002, 0x0001, /*1338D*/
  0x51A1, 0x0020, 0x0002, 0x0001, /*1338E*/
  0x51A2, 0x0020, 0x0002, 0x0001, /*1338F*/
  0x51A3, 0x0020, 0x0002, 0x0001, /*13390*/
  0x51A4, 0x0020, 0x0002, 0x0001, /*13391*/
  0x51A5, 0x0020, 0x0002, 0x0001, /*13392*/
  0x51A6, 0x0020, 0x0002, 0x0001, /*13393*/
  0x51A7, 0x0020, 0x0002, 0x0001, /*13394*/
  0x51A8, 0x0020, 0x0002, 0x0001, /*13395*/
  0x51A9, 0x0020, 0x0002, 0x0001, /*13396*/
  0x51AA, 0x0020, 0x0002, 0x0001, /*13397*/
  0x51AB, 0x0020, 0x0002, 0x0001, /*13398*/
  0x51AC, 0x0020, 0x0002, 0x0001, /*13399*/
  0x51AD, 0x0020, 0x0002, 0x0001, /*1339A*/
  0x51AE, 0x0020, 0x0002, 0x0001, /*1339B*/
  0x51AF, 0x0020, 0x0002, 0x0001, /*1339C*/
  0x51B0, 0x0020, 0x0002, 0x0001, /*1339D*/
  0x51B1, 0x0020, 0x0002, 0x0001, /*1339E*/
  0x51B2, 0x0020, 0x0002, 0x0001, /*1339F*/
  0x51B3, 0x0020, 0x0002, 0x0001, /*133A0*/
  0x51B4, 0x0020, 0x0002, 0x0001, /*133A1*/
  0x51B5, 0x0020, 0x0002, 0x0001, /*133A2*/
  0x51B6, 0x0020, 0x0002, 0x0001, /*133A3*/
  0x51B7, 0x0020, 0x0002, 0x0001, /*133A4*/
  0x51B8, 0x0020, 0x0002, 0x0001, /*133A5*/
  0x51B9, 0x0020, 0x0002, 0x0001, /*133A6*/
  0x51BA, 0x0020, 0x0002, 0x0001, /*133A7*/
  0x51BB, 0x0020, 0x0002, 0x0001, /*133A8*/
  0x51BC, 0x0020, 0x0002, 0x0001, /*133A9*/
  0x51BD, 0x0020, 0x0002, 0x0001, /*133AA*/
  0x51BE, 0x0020, 0x0002, 0x0001, /*133AB*/
  0x51BF, 0x0020, 0x0002, 0x0001, /*133AC*/
  0x51C0, 0x0020, 0x0002, 0x0001, /*133AD*/
  0x51C1, 0x0020, 0x0002, 0x0001, /*133AE*/
  0x51C2, 0x0020, 0x0002, 0x0001, /*133AF*/
  0x51C3, 0x0020, 0x0002, 0x0001, /*133B0*/
  0x51C4, 0x0020, 0x0002, 0x0001, /*133B1*/
  0x51C5, 0x0020, 0x0002, 0x0001, /*133B2*/
  0x51C6, 0x0020, 0x0002, 0x0001, /*133B3*/
  0x51C7, 0x0020, 0x0002, 0x0001, /*133B4*/
  0x51C8, 0x0020, 0x0002, 0x0001, /*133B5*/
  0x51C9, 0x0020, 0x0002, 0x0001, /*133B6*/
  0x51CA, 0x0020, 0x0002, 0x0001, /*133B7*/
  0x51CB, 0x0020, 0x0002, 0x0001, /*133B8*/
  0x51CC, 0x0020, 0x0002, 0x0001, /*133B9*/
  0x51CD, 0x0020, 0x0002, 0x0001, /*133BA*/
  0x51CE, 0x0020, 0x0002, 0x0001, /*133BB*/
  0x51CF, 0x0020, 0x0002, 0x0001, /*133BC*/
  0x51D0, 0x0020, 0x0002, 0x0001, /*133BD*/
  0x51D1, 0x0020, 0x0002, 0x0001, /*133BE*/
  0x51D2, 0x0020, 0x0002, 0x0001, /*133BF*/
  0x51D3, 0x0020, 0x0002, 0x0001, /*133C0*/
  0x51D4, 0x0020, 0x0002, 0x0001, /*133C1*/
  0x51D5, 0x0020, 0x0002, 0x0001, /*133C2*/
  0x51D6, 0x0020, 0x0002, 0x0001, /*133C3*/
  0x51D7, 0x0020, 0x0002, 0x0001, /*133C4*/
  0x51D8, 0x0020, 0x0002, 0x0001, /*133C5*/
  0x51D9, 0x0020, 0x0002, 0x0001, /*133C6*/
  0x51DA, 0x0020, 0x0002, 0x0001, /*133C7*/
  0x51DB, 0x0020, 0x0002, 0x0001, /*133C8*/
  0x51DC, 0x0020, 0x0002, 0x0001, /*133C9*/
  0x51DD, 0x0020, 0x0002, 0x0001, /*133CA*/
  0x51DE, 0x0020, 0x0002, 0x0001, /*133CB*/
  0x51DF, 0x0020, 0x0002, 0x0001, /*133CC*/
  0x51E0, 0x0020, 0x0002, 0x0001, /*133CD*/
  0x51E1, 0x0020, 0x0002, 0x0001, /*133CE*/
  0x51E2, 0x0020, 0x0002, 0x0001, /*133CF*/
  0x51E3, 0x0020, 0x0002, 0x0001, /*133D0*/
  0x51E4, 0x0020, 0x0002, 0x0001, /*133D1*/
  0x51E5, 0x0020, 0x0002, 0x0001, /*133D2*/
  0x51E6, 0x0020, 0x0002, 0x0001, /*133D3*/
  0x51E7, 0x0020, 0x0002, 0x0001, /*133D4*/
  0x51E8, 0x0020, 0x0002, 0x0001, /*133D5*/
  0x51E9, 0x0020, 0x0002, 0x0001, /*133D6*/
  0x51EA, 0x0020, 0x0002, 0x0001, /*133D7*/
  0x51EB, 0x0020, 0x0002, 0x0001, /*133D8*/
  0x51EC, 0x0020, 0x0002, 0x0001, /*133D9*/
  0x51ED, 0x0020, 0x0002, 0x0001, /*133DA*/
  0x51EE, 0x0020, 0x0002, 0x0001, /*133DB*/
  0x51EF, 0x0020, 0x0002, 0x0001, /*133DC*/
  0x51F0, 0x0020, 0x0002, 0x0001, /*133DD*/
  0x51F1, 0x0020, 0x0002, 0x0001, /*133DE*/
  0x51F2, 0x0020, 0x0002, 0x0001, /*133DF*/
  0x51F3, 0x0020, 0x0002, 0x0001, /*133E0*/
  0x51F4, 0x0020, 0x0002, 0x0001, /*133E1*/
  0x51F5, 0x0020, 0x0002, 0x0001, /*133E2*/
  0x51F6, 0x0020, 0x0002, 0x0001, /*133E3*/
  0x51F7, 0x0020, 0x0002, 0x0001, /*133E4*/
  0x51F8, 0x0020, 0x0002, 0x0001, /*133E5*/
  0x51F9, 0x0020, 0x0002, 0x0001, /*133E6*/
  0x51FA, 0x0020, 0x0002, 0x0001, /*133E7*/
  0x51FB, 0x0020, 0x0002, 0x0001, /*133E8*/
  0x51FC, 0x0020, 0x0002, 0x0001, /*133E9*/
  0x51FD, 0x0020, 0x0002, 0x0001, /*133EA*/
  0x51FE, 0x0020, 0x0002, 0x0001, /*133EB*/
  0x51FF, 0x0020, 0x0002, 0x0001, /*133EC*/
  0x5200, 0x0020, 0x0002, 0x0001, /*133ED*/
  0x5201, 0x0020, 0x0002, 0x0001, /*133EE*/
  0x5202, 0x0020, 0x0002, 0x0001, /*133EF*/
  0x5203, 0x0020, 0x0002, 0x0001, /*133F0*/
  0x5204, 0x0020, 0x0002, 0x0001, /*133F1*/
  0x5205, 0x0020, 0x0002, 0x0001, /*133F2*/
  0x5206, 0x0020, 0x0002, 0x0001, /*133F3*/
  0x5207, 0x0020, 0x0002, 0x0001, /*133F4*/
  0x5208, 0x0020, 0x0002, 0x0001, /*133F5*/
  0x5209, 0x0020, 0x0002, 0x0001, /*133F6*/
  0x520A, 0x0020, 0x0002, 0x0001, /*133F7*/
  0x520B, 0x0020, 0x0002, 0x0001, /*133F8*/
  0x520C, 0x0020, 0x0002, 0x0001, /*133F9*/
  0x520D, 0x0020, 0x0002, 0x0001, /*133FA*/
  0x520E, 0x0020, 0x0002, 0x0001, /*133FB*/
  0x520F, 0x0020, 0x0002, 0x0001, /*133FC*/
  0x5210, 0x0020, 0x0002, 0x0001, /*133FD*/
  0x5211, 0x0020, 0x0002, 0x0001, /*133FE*/
  0x5212, 0x0020, 0x0002, 0x0001  /*133FF*/
};

uint16 uca900_p134[]= {
  0x5213, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13400*/
  0x5214, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13401*/
  0x5215, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13402*/
  0x5216, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13403*/
  0x5217, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13404*/
  0x5218, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13405*/
  0x5219, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13406*/
  0x521A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13407*/
  0x521B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13408*/
  0x521C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13409*/
  0x521D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1340A*/
  0x521E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1340B*/
  0x521F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1340C*/
  0x5220, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1340D*/
  0x5221, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1340E*/
  0x5222, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1340F*/
  0x5223, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13410*/
  0x5224, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13411*/
  0x5225, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13412*/
  0x5226, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13413*/
  0x5227, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13414*/
  0x5228, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13415*/
  0x5229, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13416*/
  0x522A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13417*/
  0x522B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13418*/
  0x522C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13419*/
  0x522D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1341A*/
  0x522E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1341B*/
  0x522F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1341C*/
  0x5230, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1341D*/
  0x5231, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1341E*/
  0x5232, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1341F*/
  0x5233, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13420*/
  0x5234, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13421*/
  0x5235, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13422*/
  0x5236, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13423*/
  0x5237, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13424*/
  0x5238, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13425*/
  0x5239, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13426*/
  0x523A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13427*/
  0x523B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13428*/
  0x523C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*13429*/
  0x523D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1342A*/
  0x523E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1342B*/
  0x523F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1342C*/
  0x5240, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1342D*/
  0x5241, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1342E*/
  0xFBC2, 0x0020, 0x0002, 0xB42F, 0x0000, 0x0000, 0x0002, /*1342F*/
  0xFBC2, 0x0020, 0x0002, 0xB430, 0x0000, 0x0000, 0x0002, /*13430*/
  0xFBC2, 0x0020, 0x0002, 0xB431, 0x0000, 0x0000, 0x0002, /*13431*/
  0xFBC2, 0x0020, 0x0002, 0xB432, 0x0000, 0x0000, 0x0002, /*13432*/
  0xFBC2, 0x0020, 0x0002, 0xB433, 0x0000, 0x0000, 0x0002, /*13433*/
  0xFBC2, 0x0020, 0x0002, 0xB434, 0x0000, 0x0000, 0x0002, /*13434*/
  0xFBC2, 0x0020, 0x0002, 0xB435, 0x0000, 0x0000, 0x0002, /*13435*/
  0xFBC2, 0x0020, 0x0002, 0xB436, 0x0000, 0x0000, 0x0002, /*13436*/
  0xFBC2, 0x0020, 0x0002, 0xB437, 0x0000, 0x0000, 0x0002, /*13437*/
  0xFBC2, 0x0020, 0x0002, 0xB438, 0x0000, 0x0000, 0x0002, /*13438*/
  0xFBC2, 0x0020, 0x0002, 0xB439, 0x0000, 0x0000, 0x0002, /*13439*/
  0xFBC2, 0x0020, 0x0002, 0xB43A, 0x0000, 0x0000, 0x0002, /*1343A*/
  0xFBC2, 0x0020, 0x0002, 0xB43B, 0x0000, 0x0000, 0x0002, /*1343B*/
  0xFBC2, 0x0020, 0x0002, 0xB43C, 0x0000, 0x0000, 0x0002, /*1343C*/
  0xFBC2, 0x0020, 0x0002, 0xB43D, 0x0000, 0x0000, 0x0002, /*1343D*/
  0xFBC2, 0x0020, 0x0002, 0xB43E, 0x0000, 0x0000, 0x0002, /*1343E*/
  0xFBC2, 0x0020, 0x0002, 0xB43F, 0x0000, 0x0000, 0x0002, /*1343F*/
  0xFBC2, 0x0020, 0x0002, 0xB440, 0x0000, 0x0000, 0x0002, /*13440*/
  0xFBC2, 0x0020, 0x0002, 0xB441, 0x0000, 0x0000, 0x0002, /*13441*/
  0xFBC2, 0x0020, 0x0002, 0xB442, 0x0000, 0x0000, 0x0002, /*13442*/
  0xFBC2, 0x0020, 0x0002, 0xB443, 0x0000, 0x0000, 0x0002, /*13443*/
  0xFBC2, 0x0020, 0x0002, 0xB444, 0x0000, 0x0000, 0x0002, /*13444*/
  0xFBC2, 0x0020, 0x0002, 0xB445, 0x0000, 0x0000, 0x0002, /*13445*/
  0xFBC2, 0x0020, 0x0002, 0xB446, 0x0000, 0x0000, 0x0002, /*13446*/
  0xFBC2, 0x0020, 0x0002, 0xB447, 0x0000, 0x0000, 0x0002, /*13447*/
  0xFBC2, 0x0020, 0x0002, 0xB448, 0x0000, 0x0000, 0x0002, /*13448*/
  0xFBC2, 0x0020, 0x0002, 0xB449, 0x0000, 0x0000, 0x0002, /*13449*/
  0xFBC2, 0x0020, 0x0002, 0xB44A, 0x0000, 0x0000, 0x0002, /*1344A*/
  0xFBC2, 0x0020, 0x0002, 0xB44B, 0x0000, 0x0000, 0x0002, /*1344B*/
  0xFBC2, 0x0020, 0x0002, 0xB44C, 0x0000, 0x0000, 0x0002, /*1344C*/
  0xFBC2, 0x0020, 0x0002, 0xB44D, 0x0000, 0x0000, 0x0002, /*1344D*/
  0xFBC2, 0x0020, 0x0002, 0xB44E, 0x0000, 0x0000, 0x0002, /*1344E*/
  0xFBC2, 0x0020, 0x0002, 0xB44F, 0x0000, 0x0000, 0x0002, /*1344F*/
  0xFBC2, 0x0020, 0x0002, 0xB450, 0x0000, 0x0000, 0x0002, /*13450*/
  0xFBC2, 0x0020, 0x0002, 0xB451, 0x0000, 0x0000, 0x0002, /*13451*/
  0xFBC2, 0x0020, 0x0002, 0xB452, 0x0000, 0x0000, 0x0002, /*13452*/
  0xFBC2, 0x0020, 0x0002, 0xB453, 0x0000, 0x0000, 0x0002, /*13453*/
  0xFBC2, 0x0020, 0x0002, 0xB454, 0x0000, 0x0000, 0x0002, /*13454*/
  0xFBC2, 0x0020, 0x0002, 0xB455, 0x0000, 0x0000, 0x0002, /*13455*/
  0xFBC2, 0x0020, 0x0002, 0xB456, 0x0000, 0x0000, 0x0002, /*13456*/
  0xFBC2, 0x0020, 0x0002, 0xB457, 0x0000, 0x0000, 0x0002, /*13457*/
  0xFBC2, 0x0020, 0x0002, 0xB458, 0x0000, 0x0000, 0x0002, /*13458*/
  0xFBC2, 0x0020, 0x0002, 0xB459, 0x0000, 0x0000, 0x0002, /*13459*/
  0xFBC2, 0x0020, 0x0002, 0xB45A, 0x0000, 0x0000, 0x0002, /*1345A*/
  0xFBC2, 0x0020, 0x0002, 0xB45B, 0x0000, 0x0000, 0x0002, /*1345B*/
  0xFBC2, 0x0020, 0x0002, 0xB45C, 0x0000, 0x0000, 0x0002, /*1345C*/
  0xFBC2, 0x0020, 0x0002, 0xB45D, 0x0000, 0x0000, 0x0002, /*1345D*/
  0xFBC2, 0x0020, 0x0002, 0xB45E, 0x0000, 0x0000, 0x0002, /*1345E*/
  0xFBC2, 0x0020, 0x0002, 0xB45F, 0x0000, 0x0000, 0x0002, /*1345F*/
  0xFBC2, 0x0020, 0x0002, 0xB460, 0x0000, 0x0000, 0x0002, /*13460*/
  0xFBC2, 0x0020, 0x0002, 0xB461, 0x0000, 0x0000, 0x0002, /*13461*/
  0xFBC2, 0x0020, 0x0002, 0xB462, 0x0000, 0x0000, 0x0002, /*13462*/
  0xFBC2, 0x0020, 0x0002, 0xB463, 0x0000, 0x0000, 0x0002, /*13463*/
  0xFBC2, 0x0020, 0x0002, 0xB464, 0x0000, 0x0000, 0x0002, /*13464*/
  0xFBC2, 0x0020, 0x0002, 0xB465, 0x0000, 0x0000, 0x0002, /*13465*/
  0xFBC2, 0x0020, 0x0002, 0xB466, 0x0000, 0x0000, 0x0002, /*13466*/
  0xFBC2, 0x0020, 0x0002, 0xB467, 0x0000, 0x0000, 0x0002, /*13467*/
  0xFBC2, 0x0020, 0x0002, 0xB468, 0x0000, 0x0000, 0x0002, /*13468*/
  0xFBC2, 0x0020, 0x0002, 0xB469, 0x0000, 0x0000, 0x0002, /*13469*/
  0xFBC2, 0x0020, 0x0002, 0xB46A, 0x0000, 0x0000, 0x0002, /*1346A*/
  0xFBC2, 0x0020, 0x0002, 0xB46B, 0x0000, 0x0000, 0x0002, /*1346B*/
  0xFBC2, 0x0020, 0x0002, 0xB46C, 0x0000, 0x0000, 0x0002, /*1346C*/
  0xFBC2, 0x0020, 0x0002, 0xB46D, 0x0000, 0x0000, 0x0002, /*1346D*/
  0xFBC2, 0x0020, 0x0002, 0xB46E, 0x0000, 0x0000, 0x0002, /*1346E*/
  0xFBC2, 0x0020, 0x0002, 0xB46F, 0x0000, 0x0000, 0x0002, /*1346F*/
  0xFBC2, 0x0020, 0x0002, 0xB470, 0x0000, 0x0000, 0x0002, /*13470*/
  0xFBC2, 0x0020, 0x0002, 0xB471, 0x0000, 0x0000, 0x0002, /*13471*/
  0xFBC2, 0x0020, 0x0002, 0xB472, 0x0000, 0x0000, 0x0002, /*13472*/
  0xFBC2, 0x0020, 0x0002, 0xB473, 0x0000, 0x0000, 0x0002, /*13473*/
  0xFBC2, 0x0020, 0x0002, 0xB474, 0x0000, 0x0000, 0x0002, /*13474*/
  0xFBC2, 0x0020, 0x0002, 0xB475, 0x0000, 0x0000, 0x0002, /*13475*/
  0xFBC2, 0x0020, 0x0002, 0xB476, 0x0000, 0x0000, 0x0002, /*13476*/
  0xFBC2, 0x0020, 0x0002, 0xB477, 0x0000, 0x0000, 0x0002, /*13477*/
  0xFBC2, 0x0020, 0x0002, 0xB478, 0x0000, 0x0000, 0x0002, /*13478*/
  0xFBC2, 0x0020, 0x0002, 0xB479, 0x0000, 0x0000, 0x0002, /*13479*/
  0xFBC2, 0x0020, 0x0002, 0xB47A, 0x0000, 0x0000, 0x0002, /*1347A*/
  0xFBC2, 0x0020, 0x0002, 0xB47B, 0x0000, 0x0000, 0x0002, /*1347B*/
  0xFBC2, 0x0020, 0x0002, 0xB47C, 0x0000, 0x0000, 0x0002, /*1347C*/
  0xFBC2, 0x0020, 0x0002, 0xB47D, 0x0000, 0x0000, 0x0002, /*1347D*/
  0xFBC2, 0x0020, 0x0002, 0xB47E, 0x0000, 0x0000, 0x0002, /*1347E*/
  0xFBC2, 0x0020, 0x0002, 0xB47F, 0x0000, 0x0000, 0x0002, /*1347F*/
  0xFBC2, 0x0020, 0x0002, 0xB480, 0x0000, 0x0000, 0x0002, /*13480*/
  0xFBC2, 0x0020, 0x0002, 0xB481, 0x0000, 0x0000, 0x0002, /*13481*/
  0xFBC2, 0x0020, 0x0002, 0xB482, 0x0000, 0x0000, 0x0002, /*13482*/
  0xFBC2, 0x0020, 0x0002, 0xB483, 0x0000, 0x0000, 0x0002, /*13483*/
  0xFBC2, 0x0020, 0x0002, 0xB484, 0x0000, 0x0000, 0x0002, /*13484*/
  0xFBC2, 0x0020, 0x0002, 0xB485, 0x0000, 0x0000, 0x0002, /*13485*/
  0xFBC2, 0x0020, 0x0002, 0xB486, 0x0000, 0x0000, 0x0002, /*13486*/
  0xFBC2, 0x0020, 0x0002, 0xB487, 0x0000, 0x0000, 0x0002, /*13487*/
  0xFBC2, 0x0020, 0x0002, 0xB488, 0x0000, 0x0000, 0x0002, /*13488*/
  0xFBC2, 0x0020, 0x0002, 0xB489, 0x0000, 0x0000, 0x0002, /*13489*/
  0xFBC2, 0x0020, 0x0002, 0xB48A, 0x0000, 0x0000, 0x0002, /*1348A*/
  0xFBC2, 0x0020, 0x0002, 0xB48B, 0x0000, 0x0000, 0x0002, /*1348B*/
  0xFBC2, 0x0020, 0x0002, 0xB48C, 0x0000, 0x0000, 0x0002, /*1348C*/
  0xFBC2, 0x0020, 0x0002, 0xB48D, 0x0000, 0x0000, 0x0002, /*1348D*/
  0xFBC2, 0x0020, 0x0002, 0xB48E, 0x0000, 0x0000, 0x0002, /*1348E*/
  0xFBC2, 0x0020, 0x0002, 0xB48F, 0x0000, 0x0000, 0x0002, /*1348F*/
  0xFBC2, 0x0020, 0x0002, 0xB490, 0x0000, 0x0000, 0x0002, /*13490*/
  0xFBC2, 0x0020, 0x0002, 0xB491, 0x0000, 0x0000, 0x0002, /*13491*/
  0xFBC2, 0x0020, 0x0002, 0xB492, 0x0000, 0x0000, 0x0002, /*13492*/
  0xFBC2, 0x0020, 0x0002, 0xB493, 0x0000, 0x0000, 0x0002, /*13493*/
  0xFBC2, 0x0020, 0x0002, 0xB494, 0x0000, 0x0000, 0x0002, /*13494*/
  0xFBC2, 0x0020, 0x0002, 0xB495, 0x0000, 0x0000, 0x0002, /*13495*/
  0xFBC2, 0x0020, 0x0002, 0xB496, 0x0000, 0x0000, 0x0002, /*13496*/
  0xFBC2, 0x0020, 0x0002, 0xB497, 0x0000, 0x0000, 0x0002, /*13497*/
  0xFBC2, 0x0020, 0x0002, 0xB498, 0x0000, 0x0000, 0x0002, /*13498*/
  0xFBC2, 0x0020, 0x0002, 0xB499, 0x0000, 0x0000, 0x0002, /*13499*/
  0xFBC2, 0x0020, 0x0002, 0xB49A, 0x0000, 0x0000, 0x0002, /*1349A*/
  0xFBC2, 0x0020, 0x0002, 0xB49B, 0x0000, 0x0000, 0x0002, /*1349B*/
  0xFBC2, 0x0020, 0x0002, 0xB49C, 0x0000, 0x0000, 0x0002, /*1349C*/
  0xFBC2, 0x0020, 0x0002, 0xB49D, 0x0000, 0x0000, 0x0002, /*1349D*/
  0xFBC2, 0x0020, 0x0002, 0xB49E, 0x0000, 0x0000, 0x0002, /*1349E*/
  0xFBC2, 0x0020, 0x0002, 0xB49F, 0x0000, 0x0000, 0x0002, /*1349F*/
  0xFBC2, 0x0020, 0x0002, 0xB4A0, 0x0000, 0x0000, 0x0002, /*134A0*/
  0xFBC2, 0x0020, 0x0002, 0xB4A1, 0x0000, 0x0000, 0x0002, /*134A1*/
  0xFBC2, 0x0020, 0x0002, 0xB4A2, 0x0000, 0x0000, 0x0002, /*134A2*/
  0xFBC2, 0x0020, 0x0002, 0xB4A3, 0x0000, 0x0000, 0x0002, /*134A3*/
  0xFBC2, 0x0020, 0x0002, 0xB4A4, 0x0000, 0x0000, 0x0002, /*134A4*/
  0xFBC2, 0x0020, 0x0002, 0xB4A5, 0x0000, 0x0000, 0x0002, /*134A5*/
  0xFBC2, 0x0020, 0x0002, 0xB4A6, 0x0000, 0x0000, 0x0002, /*134A6*/
  0xFBC2, 0x0020, 0x0002, 0xB4A7, 0x0000, 0x0000, 0x0002, /*134A7*/
  0xFBC2, 0x0020, 0x0002, 0xB4A8, 0x0000, 0x0000, 0x0002, /*134A8*/
  0xFBC2, 0x0020, 0x0002, 0xB4A9, 0x0000, 0x0000, 0x0002, /*134A9*/
  0xFBC2, 0x0020, 0x0002, 0xB4AA, 0x0000, 0x0000, 0x0002, /*134AA*/
  0xFBC2, 0x0020, 0x0002, 0xB4AB, 0x0000, 0x0000, 0x0002, /*134AB*/
  0xFBC2, 0x0020, 0x0002, 0xB4AC, 0x0000, 0x0000, 0x0002, /*134AC*/
  0xFBC2, 0x0020, 0x0002, 0xB4AD, 0x0000, 0x0000, 0x0002, /*134AD*/
  0xFBC2, 0x0020, 0x0002, 0xB4AE, 0x0000, 0x0000, 0x0002, /*134AE*/
  0xFBC2, 0x0020, 0x0002, 0xB4AF, 0x0000, 0x0000, 0x0002, /*134AF*/
  0xFBC2, 0x0020, 0x0002, 0xB4B0, 0x0000, 0x0000, 0x0002, /*134B0*/
  0xFBC2, 0x0020, 0x0002, 0xB4B1, 0x0000, 0x0000, 0x0002, /*134B1*/
  0xFBC2, 0x0020, 0x0002, 0xB4B2, 0x0000, 0x0000, 0x0002, /*134B2*/
  0xFBC2, 0x0020, 0x0002, 0xB4B3, 0x0000, 0x0000, 0x0002, /*134B3*/
  0xFBC2, 0x0020, 0x0002, 0xB4B4, 0x0000, 0x0000, 0x0002, /*134B4*/
  0xFBC2, 0x0020, 0x0002, 0xB4B5, 0x0000, 0x0000, 0x0002, /*134B5*/
  0xFBC2, 0x0020, 0x0002, 0xB4B6, 0x0000, 0x0000, 0x0002, /*134B6*/
  0xFBC2, 0x0020, 0x0002, 0xB4B7, 0x0000, 0x0000, 0x0002, /*134B7*/
  0xFBC2, 0x0020, 0x0002, 0xB4B8, 0x0000, 0x0000, 0x0002, /*134B8*/
  0xFBC2, 0x0020, 0x0002, 0xB4B9, 0x0000, 0x0000, 0x0002, /*134B9*/
  0xFBC2, 0x0020, 0x0002, 0xB4BA, 0x0000, 0x0000, 0x0002, /*134BA*/
  0xFBC2, 0x0020, 0x0002, 0xB4BB, 0x0000, 0x0000, 0x0002, /*134BB*/
  0xFBC2, 0x0020, 0x0002, 0xB4BC, 0x0000, 0x0000, 0x0002, /*134BC*/
  0xFBC2, 0x0020, 0x0002, 0xB4BD, 0x0000, 0x0000, 0x0002, /*134BD*/
  0xFBC2, 0x0020, 0x0002, 0xB4BE, 0x0000, 0x0000, 0x0002, /*134BE*/
  0xFBC2, 0x0020, 0x0002, 0xB4BF, 0x0000, 0x0000, 0x0002, /*134BF*/
  0xFBC2, 0x0020, 0x0002, 0xB4C0, 0x0000, 0x0000, 0x0002, /*134C0*/
  0xFBC2, 0x0020, 0x0002, 0xB4C1, 0x0000, 0x0000, 0x0002, /*134C1*/
  0xFBC2, 0x0020, 0x0002, 0xB4C2, 0x0000, 0x0000, 0x0002, /*134C2*/
  0xFBC2, 0x0020, 0x0002, 0xB4C3, 0x0000, 0x0000, 0x0002, /*134C3*/
  0xFBC2, 0x0020, 0x0002, 0xB4C4, 0x0000, 0x0000, 0x0002, /*134C4*/
  0xFBC2, 0x0020, 0x0002, 0xB4C5, 0x0000, 0x0000, 0x0002, /*134C5*/
  0xFBC2, 0x0020, 0x0002, 0xB4C6, 0x0000, 0x0000, 0x0002, /*134C6*/
  0xFBC2, 0x0020, 0x0002, 0xB4C7, 0x0000, 0x0000, 0x0002, /*134C7*/
  0xFBC2, 0x0020, 0x0002, 0xB4C8, 0x0000, 0x0000, 0x0002, /*134C8*/
  0xFBC2, 0x0020, 0x0002, 0xB4C9, 0x0000, 0x0000, 0x0002, /*134C9*/
  0xFBC2, 0x0020, 0x0002, 0xB4CA, 0x0000, 0x0000, 0x0002, /*134CA*/
  0xFBC2, 0x0020, 0x0002, 0xB4CB, 0x0000, 0x0000, 0x0002, /*134CB*/
  0xFBC2, 0x0020, 0x0002, 0xB4CC, 0x0000, 0x0000, 0x0002, /*134CC*/
  0xFBC2, 0x0020, 0x0002, 0xB4CD, 0x0000, 0x0000, 0x0002, /*134CD*/
  0xFBC2, 0x0020, 0x0002, 0xB4CE, 0x0000, 0x0000, 0x0002, /*134CE*/
  0xFBC2, 0x0020, 0x0002, 0xB4CF, 0x0000, 0x0000, 0x0002, /*134CF*/
  0xFBC2, 0x0020, 0x0002, 0xB4D0, 0x0000, 0x0000, 0x0002, /*134D0*/
  0xFBC2, 0x0020, 0x0002, 0xB4D1, 0x0000, 0x0000, 0x0002, /*134D1*/
  0xFBC2, 0x0020, 0x0002, 0xB4D2, 0x0000, 0x0000, 0x0002, /*134D2*/
  0xFBC2, 0x0020, 0x0002, 0xB4D3, 0x0000, 0x0000, 0x0002, /*134D3*/
  0xFBC2, 0x0020, 0x0002, 0xB4D4, 0x0000, 0x0000, 0x0002, /*134D4*/
  0xFBC2, 0x0020, 0x0002, 0xB4D5, 0x0000, 0x0000, 0x0002, /*134D5*/
  0xFBC2, 0x0020, 0x0002, 0xB4D6, 0x0000, 0x0000, 0x0002, /*134D6*/
  0xFBC2, 0x0020, 0x0002, 0xB4D7, 0x0000, 0x0000, 0x0002, /*134D7*/
  0xFBC2, 0x0020, 0x0002, 0xB4D8, 0x0000, 0x0000, 0x0002, /*134D8*/
  0xFBC2, 0x0020, 0x0002, 0xB4D9, 0x0000, 0x0000, 0x0002, /*134D9*/
  0xFBC2, 0x0020, 0x0002, 0xB4DA, 0x0000, 0x0000, 0x0002, /*134DA*/
  0xFBC2, 0x0020, 0x0002, 0xB4DB, 0x0000, 0x0000, 0x0002, /*134DB*/
  0xFBC2, 0x0020, 0x0002, 0xB4DC, 0x0000, 0x0000, 0x0002, /*134DC*/
  0xFBC2, 0x0020, 0x0002, 0xB4DD, 0x0000, 0x0000, 0x0002, /*134DD*/
  0xFBC2, 0x0020, 0x0002, 0xB4DE, 0x0000, 0x0000, 0x0002, /*134DE*/
  0xFBC2, 0x0020, 0x0002, 0xB4DF, 0x0000, 0x0000, 0x0002, /*134DF*/
  0xFBC2, 0x0020, 0x0002, 0xB4E0, 0x0000, 0x0000, 0x0002, /*134E0*/
  0xFBC2, 0x0020, 0x0002, 0xB4E1, 0x0000, 0x0000, 0x0002, /*134E1*/
  0xFBC2, 0x0020, 0x0002, 0xB4E2, 0x0000, 0x0000, 0x0002, /*134E2*/
  0xFBC2, 0x0020, 0x0002, 0xB4E3, 0x0000, 0x0000, 0x0002, /*134E3*/
  0xFBC2, 0x0020, 0x0002, 0xB4E4, 0x0000, 0x0000, 0x0002, /*134E4*/
  0xFBC2, 0x0020, 0x0002, 0xB4E5, 0x0000, 0x0000, 0x0002, /*134E5*/
  0xFBC2, 0x0020, 0x0002, 0xB4E6, 0x0000, 0x0000, 0x0002, /*134E6*/
  0xFBC2, 0x0020, 0x0002, 0xB4E7, 0x0000, 0x0000, 0x0002, /*134E7*/
  0xFBC2, 0x0020, 0x0002, 0xB4E8, 0x0000, 0x0000, 0x0002, /*134E8*/
  0xFBC2, 0x0020, 0x0002, 0xB4E9, 0x0000, 0x0000, 0x0002, /*134E9*/
  0xFBC2, 0x0020, 0x0002, 0xB4EA, 0x0000, 0x0000, 0x0002, /*134EA*/
  0xFBC2, 0x0020, 0x0002, 0xB4EB, 0x0000, 0x0000, 0x0002, /*134EB*/
  0xFBC2, 0x0020, 0x0002, 0xB4EC, 0x0000, 0x0000, 0x0002, /*134EC*/
  0xFBC2, 0x0020, 0x0002, 0xB4ED, 0x0000, 0x0000, 0x0002, /*134ED*/
  0xFBC2, 0x0020, 0x0002, 0xB4EE, 0x0000, 0x0000, 0x0002, /*134EE*/
  0xFBC2, 0x0020, 0x0002, 0xB4EF, 0x0000, 0x0000, 0x0002, /*134EF*/
  0xFBC2, 0x0020, 0x0002, 0xB4F0, 0x0000, 0x0000, 0x0002, /*134F0*/
  0xFBC2, 0x0020, 0x0002, 0xB4F1, 0x0000, 0x0000, 0x0002, /*134F1*/
  0xFBC2, 0x0020, 0x0002, 0xB4F2, 0x0000, 0x0000, 0x0002, /*134F2*/
  0xFBC2, 0x0020, 0x0002, 0xB4F3, 0x0000, 0x0000, 0x0002, /*134F3*/
  0xFBC2, 0x0020, 0x0002, 0xB4F4, 0x0000, 0x0000, 0x0002, /*134F4*/
  0xFBC2, 0x0020, 0x0002, 0xB4F5, 0x0000, 0x0000, 0x0002, /*134F5*/
  0xFBC2, 0x0020, 0x0002, 0xB4F6, 0x0000, 0x0000, 0x0002, /*134F6*/
  0xFBC2, 0x0020, 0x0002, 0xB4F7, 0x0000, 0x0000, 0x0002, /*134F7*/
  0xFBC2, 0x0020, 0x0002, 0xB4F8, 0x0000, 0x0000, 0x0002, /*134F8*/
  0xFBC2, 0x0020, 0x0002, 0xB4F9, 0x0000, 0x0000, 0x0002, /*134F9*/
  0xFBC2, 0x0020, 0x0002, 0xB4FA, 0x0000, 0x0000, 0x0002, /*134FA*/
  0xFBC2, 0x0020, 0x0002, 0xB4FB, 0x0000, 0x0000, 0x0002, /*134FB*/
  0xFBC2, 0x0020, 0x0002, 0xB4FC, 0x0000, 0x0000, 0x0002, /*134FC*/
  0xFBC2, 0x0020, 0x0002, 0xB4FD, 0x0000, 0x0000, 0x0002, /*134FD*/
  0xFBC2, 0x0020, 0x0002, 0xB4FE, 0x0000, 0x0000, 0x0002, /*134FE*/
  0xFBC2, 0x0020, 0x0002, 0xB4FF, 0x0000, 0x0000, 0x0002  /*134FF*/
};

uint16 uca900_p144[]= {
  0x525D, 0x0020, 0x0002, 0x0001, /*14400*/
  0x525E, 0x0020, 0x0002, 0x0001, /*14401*/
  0x525F, 0x0020, 0x0002, 0x0001, /*14402*/
  0x5260, 0x0020, 0x0002, 0x0001, /*14403*/
  0x5261, 0x0020, 0x0002, 0x0001, /*14404*/
  0x5262, 0x0020, 0x0002, 0x0001, /*14405*/
  0x5263, 0x0020, 0x0002, 0x0001, /*14406*/
  0x5264, 0x0020, 0x0002, 0x0001, /*14407*/
  0x5265, 0x0020, 0x0002, 0x0001, /*14408*/
  0x5266, 0x0020, 0x0002, 0x0001, /*14409*/
  0x5267, 0x0020, 0x0002, 0x0001, /*1440A*/
  0x5268, 0x0020, 0x0002, 0x0001, /*1440B*/
  0x5269, 0x0020, 0x0002, 0x0001, /*1440C*/
  0x526A, 0x0020, 0x0002, 0x0001, /*1440D*/
  0x526B, 0x0020, 0x0002, 0x0001, /*1440E*/
  0x526C, 0x0020, 0x0002, 0x0001, /*1440F*/
  0x526D, 0x0020, 0x0002, 0x0001, /*14410*/
  0x526E, 0x0020, 0x0002, 0x0001, /*14411*/
  0x526F, 0x0020, 0x0002, 0x0001, /*14412*/
  0x5270, 0x0020, 0x0002, 0x0001, /*14413*/
  0x5271, 0x0020, 0x0002, 0x0001, /*14414*/
  0x5272, 0x0020, 0x0002, 0x0001, /*14415*/
  0x5273, 0x0020, 0x0002, 0x0001, /*14416*/
  0x5274, 0x0020, 0x0002, 0x0001, /*14417*/
  0x5275, 0x0020, 0x0002, 0x0001, /*14418*/
  0x5276, 0x0020, 0x0002, 0x0001, /*14419*/
  0x5277, 0x0020, 0x0002, 0x0001, /*1441A*/
  0x5278, 0x0020, 0x0002, 0x0001, /*1441B*/
  0x5279, 0x0020, 0x0002, 0x0001, /*1441C*/
  0x527A, 0x0020, 0x0002, 0x0001, /*1441D*/
  0x527B, 0x0020, 0x0002, 0x0001, /*1441E*/
  0x527C, 0x0020, 0x0002, 0x0001, /*1441F*/
  0x527D, 0x0020, 0x0002, 0x0001, /*14420*/
  0x527E, 0x0020, 0x0002, 0x0001, /*14421*/
  0x527F, 0x0020, 0x0002, 0x0001, /*14422*/
  0x5280, 0x0020, 0x0002, 0x0001, /*14423*/
  0x5281, 0x0020, 0x0002, 0x0001, /*14424*/
  0x5282, 0x0020, 0x0002, 0x0001, /*14425*/
  0x5283, 0x0020, 0x0002, 0x0001, /*14426*/
  0x5284, 0x0020, 0x0002, 0x0001, /*14427*/
  0x5285, 0x0020, 0x0002, 0x0001, /*14428*/
  0x5286, 0x0020, 0x0002, 0x0001, /*14429*/
  0x5287, 0x0020, 0x0002, 0x0001, /*1442A*/
  0x5288, 0x0020, 0x0002, 0x0001, /*1442B*/
  0x5289, 0x0020, 0x0002, 0x0001, /*1442C*/
  0x528A, 0x0020, 0x0002, 0x0001, /*1442D*/
  0x528B, 0x0020, 0x0002, 0x0001, /*1442E*/
  0x528C, 0x0020, 0x0002, 0x0001, /*1442F*/
  0x528D, 0x0020, 0x0002, 0x0001, /*14430*/
  0x528E, 0x0020, 0x0002, 0x0001, /*14431*/
  0x528F, 0x0020, 0x0002, 0x0001, /*14432*/
  0x5290, 0x0020, 0x0002, 0x0001, /*14433*/
  0x5291, 0x0020, 0x0002, 0x0001, /*14434*/
  0x5292, 0x0020, 0x0002, 0x0001, /*14435*/
  0x5293, 0x0020, 0x0002, 0x0001, /*14436*/
  0x5294, 0x0020, 0x0002, 0x0001, /*14437*/
  0x5295, 0x0020, 0x0002, 0x0001, /*14438*/
  0x5296, 0x0020, 0x0002, 0x0001, /*14439*/
  0x5297, 0x0020, 0x0002, 0x0001, /*1443A*/
  0x5298, 0x0020, 0x0002, 0x0001, /*1443B*/
  0x5299, 0x0020, 0x0002, 0x0001, /*1443C*/
  0x529A, 0x0020, 0x0002, 0x0001, /*1443D*/
  0x529B, 0x0020, 0x0002, 0x0001, /*1443E*/
  0x529C, 0x0020, 0x0002, 0x0001, /*1443F*/
  0x529D, 0x0020, 0x0002, 0x0001, /*14440*/
  0x529E, 0x0020, 0x0002, 0x0001, /*14441*/
  0x529F, 0x0020, 0x0002, 0x0001, /*14442*/
  0x52A0, 0x0020, 0x0002, 0x0001, /*14443*/
  0x52A1, 0x0020, 0x0002, 0x0001, /*14444*/
  0x52A2, 0x0020, 0x0002, 0x0001, /*14445*/
  0x52A3, 0x0020, 0x0002, 0x0001, /*14446*/
  0x52A4, 0x0020, 0x0002, 0x0001, /*14447*/
  0x52A5, 0x0020, 0x0002, 0x0001, /*14448*/
  0x52A6, 0x0020, 0x0002, 0x0001, /*14449*/
  0x52A7, 0x0020, 0x0002, 0x0001, /*1444A*/
  0x52A8, 0x0020, 0x0002, 0x0001, /*1444B*/
  0x52A9, 0x0020, 0x0002, 0x0001, /*1444C*/
  0x52AA, 0x0020, 0x0002, 0x0001, /*1444D*/
  0x52AB, 0x0020, 0x0002, 0x0001, /*1444E*/
  0x52AC, 0x0020, 0x0002, 0x0001, /*1444F*/
  0x52AD, 0x0020, 0x0002, 0x0001, /*14450*/
  0x52AE, 0x0020, 0x0002, 0x0001, /*14451*/
  0x52AF, 0x0020, 0x0002, 0x0001, /*14452*/
  0x52B0, 0x0020, 0x0002, 0x0001, /*14453*/
  0x52B1, 0x0020, 0x0002, 0x0001, /*14454*/
  0x52B2, 0x0020, 0x0002, 0x0001, /*14455*/
  0x52B3, 0x0020, 0x0002, 0x0001, /*14456*/
  0x52B4, 0x0020, 0x0002, 0x0001, /*14457*/
  0x52B5, 0x0020, 0x0002, 0x0001, /*14458*/
  0x52B6, 0x0020, 0x0002, 0x0001, /*14459*/
  0x52B7, 0x0020, 0x0002, 0x0001, /*1445A*/
  0x52B8, 0x0020, 0x0002, 0x0001, /*1445B*/
  0x52B9, 0x0020, 0x0002, 0x0001, /*1445C*/
  0x52BA, 0x0020, 0x0002, 0x0001, /*1445D*/
  0x52BB, 0x0020, 0x0002, 0x0001, /*1445E*/
  0x52BC, 0x0020, 0x0002, 0x0001, /*1445F*/
  0x52BD, 0x0020, 0x0002, 0x0001, /*14460*/
  0x52BE, 0x0020, 0x0002, 0x0001, /*14461*/
  0x52BF, 0x0020, 0x0002, 0x0001, /*14462*/
  0x52C0, 0x0020, 0x0002, 0x0001, /*14463*/
  0x52C1, 0x0020, 0x0002, 0x0001, /*14464*/
  0x52C2, 0x0020, 0x0002, 0x0001, /*14465*/
  0x52C3, 0x0020, 0x0002, 0x0001, /*14466*/
  0x52C4, 0x0020, 0x0002, 0x0001, /*14467*/
  0x52C5, 0x0020, 0x0002, 0x0001, /*14468*/
  0x52C6, 0x0020, 0x0002, 0x0001, /*14469*/
  0x52C7, 0x0020, 0x0002, 0x0001, /*1446A*/
  0x52C8, 0x0020, 0x0002, 0x0001, /*1446B*/
  0x52C9, 0x0020, 0x0002, 0x0001, /*1446C*/
  0x52CA, 0x0020, 0x0002, 0x0001, /*1446D*/
  0x52CB, 0x0020, 0x0002, 0x0001, /*1446E*/
  0x52CC, 0x0020, 0x0002, 0x0001, /*1446F*/
  0x52CD, 0x0020, 0x0002, 0x0001, /*14470*/
  0x52CE, 0x0020, 0x0002, 0x0001, /*14471*/
  0x52CF, 0x0020, 0x0002, 0x0001, /*14472*/
  0x52D0, 0x0020, 0x0002, 0x0001, /*14473*/
  0x52D1, 0x0020, 0x0002, 0x0001, /*14474*/
  0x52D2, 0x0020, 0x0002, 0x0001, /*14475*/
  0x52D3, 0x0020, 0x0002, 0x0001, /*14476*/
  0x52D4, 0x0020, 0x0002, 0x0001, /*14477*/
  0x52D5, 0x0020, 0x0002, 0x0001, /*14478*/
  0x52D6, 0x0020, 0x0002, 0x0001, /*14479*/
  0x52D7, 0x0020, 0x0002, 0x0001, /*1447A*/
  0x52D8, 0x0020, 0x0002, 0x0001, /*1447B*/
  0x52D9, 0x0020, 0x0002, 0x0001, /*1447C*/
  0x52DA, 0x0020, 0x0002, 0x0001, /*1447D*/
  0x52DB, 0x0020, 0x0002, 0x0001, /*1447E*/
  0x52DC, 0x0020, 0x0002, 0x0001, /*1447F*/
  0x52DD, 0x0020, 0x0002, 0x0001, /*14480*/
  0x52DE, 0x0020, 0x0002, 0x0001, /*14481*/
  0x52DF, 0x0020, 0x0002, 0x0001, /*14482*/
  0x52E0, 0x0020, 0x0002, 0x0001, /*14483*/
  0x52E1, 0x0020, 0x0002, 0x0001, /*14484*/
  0x52E2, 0x0020, 0x0002, 0x0001, /*14485*/
  0x52E3, 0x0020, 0x0002, 0x0001, /*14486*/
  0x52E4, 0x0020, 0x0002, 0x0001, /*14487*/
  0x52E5, 0x0020, 0x0002, 0x0001, /*14488*/
  0x52E6, 0x0020, 0x0002, 0x0001, /*14489*/
  0x52E7, 0x0020, 0x0002, 0x0001, /*1448A*/
  0x52E8, 0x0020, 0x0002, 0x0001, /*1448B*/
  0x52E9, 0x0020, 0x0002, 0x0001, /*1448C*/
  0x52EA, 0x0020, 0x0002, 0x0001, /*1448D*/
  0x52EB, 0x0020, 0x0002, 0x0001, /*1448E*/
  0x52EC, 0x0020, 0x0002, 0x0001, /*1448F*/
  0x52ED, 0x0020, 0x0002, 0x0001, /*14490*/
  0x52EE, 0x0020, 0x0002, 0x0001, /*14491*/
  0x52EF, 0x0020, 0x0002, 0x0001, /*14492*/
  0x52F0, 0x0020, 0x0002, 0x0001, /*14493*/
  0x52F1, 0x0020, 0x0002, 0x0001, /*14494*/
  0x52F2, 0x0020, 0x0002, 0x0001, /*14495*/
  0x52F3, 0x0020, 0x0002, 0x0001, /*14496*/
  0x52F4, 0x0020, 0x0002, 0x0001, /*14497*/
  0x52F5, 0x0020, 0x0002, 0x0001, /*14498*/
  0x52F6, 0x0020, 0x0002, 0x0001, /*14499*/
  0x52F7, 0x0020, 0x0002, 0x0001, /*1449A*/
  0x52F8, 0x0020, 0x0002, 0x0001, /*1449B*/
  0x52F9, 0x0020, 0x0002, 0x0001, /*1449C*/
  0x52FA, 0x0020, 0x0002, 0x0001, /*1449D*/
  0x52FB, 0x0020, 0x0002, 0x0001, /*1449E*/
  0x52FC, 0x0020, 0x0002, 0x0001, /*1449F*/
  0x52FD, 0x0020, 0x0002, 0x0001, /*144A0*/
  0x52FE, 0x0020, 0x0002, 0x0001, /*144A1*/
  0x52FF, 0x0020, 0x0002, 0x0001, /*144A2*/
  0x5300, 0x0020, 0x0002, 0x0001, /*144A3*/
  0x5301, 0x0020, 0x0002, 0x0001, /*144A4*/
  0x5302, 0x0020, 0x0002, 0x0001, /*144A5*/
  0x5303, 0x0020, 0x0002, 0x0001, /*144A6*/
  0x5304, 0x0020, 0x0002, 0x0001, /*144A7*/
  0x5305, 0x0020, 0x0002, 0x0001, /*144A8*/
  0x5306, 0x0020, 0x0002, 0x0001, /*144A9*/
  0x5307, 0x0020, 0x0002, 0x0001, /*144AA*/
  0x5308, 0x0020, 0x0002, 0x0001, /*144AB*/
  0x5309, 0x0020, 0x0002, 0x0001, /*144AC*/
  0x530A, 0x0020, 0x0002, 0x0001, /*144AD*/
  0x530B, 0x0020, 0x0002, 0x0001, /*144AE*/
  0x530C, 0x0020, 0x0002, 0x0001, /*144AF*/
  0x530D, 0x0020, 0x0002, 0x0001, /*144B0*/
  0x530E, 0x0020, 0x0002, 0x0001, /*144B1*/
  0x530F, 0x0020, 0x0002, 0x0001, /*144B2*/
  0x5310, 0x0020, 0x0002, 0x0001, /*144B3*/
  0x5311, 0x0020, 0x0002, 0x0001, /*144B4*/
  0x5312, 0x0020, 0x0002, 0x0001, /*144B5*/
  0x5313, 0x0020, 0x0002, 0x0001, /*144B6*/
  0x5314, 0x0020, 0x0002, 0x0001, /*144B7*/
  0x5315, 0x0020, 0x0002, 0x0001, /*144B8*/
  0x5316, 0x0020, 0x0002, 0x0001, /*144B9*/
  0x5317, 0x0020, 0x0002, 0x0001, /*144BA*/
  0x5318, 0x0020, 0x0002, 0x0001, /*144BB*/
  0x5319, 0x0020, 0x0002, 0x0001, /*144BC*/
  0x531A, 0x0020, 0x0002, 0x0001, /*144BD*/
  0x531B, 0x0020, 0x0002, 0x0001, /*144BE*/
  0x531C, 0x0020, 0x0002, 0x0001, /*144BF*/
  0x531D, 0x0020, 0x0002, 0x0001, /*144C0*/
  0x531E, 0x0020, 0x0002, 0x0001, /*144C1*/
  0x531F, 0x0020, 0x0002, 0x0001, /*144C2*/
  0x5320, 0x0020, 0x0002, 0x0001, /*144C3*/
  0x5321, 0x0020, 0x0002, 0x0001, /*144C4*/
  0x5322, 0x0020, 0x0002, 0x0001, /*144C5*/
  0x5323, 0x0020, 0x0002, 0x0001, /*144C6*/
  0x5324, 0x0020, 0x0002, 0x0001, /*144C7*/
  0x5325, 0x0020, 0x0002, 0x0001, /*144C8*/
  0x5326, 0x0020, 0x0002, 0x0001, /*144C9*/
  0x5327, 0x0020, 0x0002, 0x0001, /*144CA*/
  0x5328, 0x0020, 0x0002, 0x0001, /*144CB*/
  0x5329, 0x0020, 0x0002, 0x0001, /*144CC*/
  0x532A, 0x0020, 0x0002, 0x0001, /*144CD*/
  0x532B, 0x0020, 0x0002, 0x0001, /*144CE*/
  0x532C, 0x0020, 0x0002, 0x0001, /*144CF*/
  0x532D, 0x0020, 0x0002, 0x0001, /*144D0*/
  0x532E, 0x0020, 0x0002, 0x0001, /*144D1*/
  0x532F, 0x0020, 0x0002, 0x0001, /*144D2*/
  0x5330, 0x0020, 0x0002, 0x0001, /*144D3*/
  0x5331, 0x0020, 0x0002, 0x0001, /*144D4*/
  0x5332, 0x0020, 0x0002, 0x0001, /*144D5*/
  0x5333, 0x0020, 0x0002, 0x0001, /*144D6*/
  0x5334, 0x0020, 0x0002, 0x0001, /*144D7*/
  0x5335, 0x0020, 0x0002, 0x0001, /*144D8*/
  0x5336, 0x0020, 0x0002, 0x0001, /*144D9*/
  0x5337, 0x0020, 0x0002, 0x0001, /*144DA*/
  0x5338, 0x0020, 0x0002, 0x0001, /*144DB*/
  0x5339, 0x0020, 0x0002, 0x0001, /*144DC*/
  0x533A, 0x0020, 0x0002, 0x0001, /*144DD*/
  0x533B, 0x0020, 0x0002, 0x0001, /*144DE*/
  0x533C, 0x0020, 0x0002, 0x0001, /*144DF*/
  0x533D, 0x0020, 0x0002, 0x0001, /*144E0*/
  0x533E, 0x0020, 0x0002, 0x0001, /*144E1*/
  0x533F, 0x0020, 0x0002, 0x0001, /*144E2*/
  0x5340, 0x0020, 0x0002, 0x0001, /*144E3*/
  0x5341, 0x0020, 0x0002, 0x0001, /*144E4*/
  0x5342, 0x0020, 0x0002, 0x0001, /*144E5*/
  0x5343, 0x0020, 0x0002, 0x0001, /*144E6*/
  0x5344, 0x0020, 0x0002, 0x0001, /*144E7*/
  0x5345, 0x0020, 0x0002, 0x0001, /*144E8*/
  0x5346, 0x0020, 0x0002, 0x0001, /*144E9*/
  0x5347, 0x0020, 0x0002, 0x0001, /*144EA*/
  0x5348, 0x0020, 0x0002, 0x0001, /*144EB*/
  0x5349, 0x0020, 0x0002, 0x0001, /*144EC*/
  0x534A, 0x0020, 0x0002, 0x0001, /*144ED*/
  0x534B, 0x0020, 0x0002, 0x0001, /*144EE*/
  0x534C, 0x0020, 0x0002, 0x0001, /*144EF*/
  0x534D, 0x0020, 0x0002, 0x0001, /*144F0*/
  0x534E, 0x0020, 0x0002, 0x0001, /*144F1*/
  0x534F, 0x0020, 0x0002, 0x0001, /*144F2*/
  0x5350, 0x0020, 0x0002, 0x0001, /*144F3*/
  0x5351, 0x0020, 0x0002, 0x0001, /*144F4*/
  0x5352, 0x0020, 0x0002, 0x0001, /*144F5*/
  0x5353, 0x0020, 0x0002, 0x0001, /*144F6*/
  0x5354, 0x0020, 0x0002, 0x0001, /*144F7*/
  0x5355, 0x0020, 0x0002, 0x0001, /*144F8*/
  0x5356, 0x0020, 0x0002, 0x0001, /*144F9*/
  0x5357, 0x0020, 0x0002, 0x0001, /*144FA*/
  0x5358, 0x0020, 0x0002, 0x0001, /*144FB*/
  0x5359, 0x0020, 0x0002, 0x0001, /*144FC*/
  0x535A, 0x0020, 0x0002, 0x0001, /*144FD*/
  0x535B, 0x0020, 0x0002, 0x0001, /*144FE*/
  0x535C, 0x0020, 0x0002, 0x0001  /*144FF*/
};

uint16 uca900_p145[]= {
  0x535D, 0x0020, 0x0002, 0x0001, /*14500*/
  0x535E, 0x0020, 0x0002, 0x0001, /*14501*/
  0x535F, 0x0020, 0x0002, 0x0001, /*14502*/
  0x5360, 0x0020, 0x0002, 0x0001, /*14503*/
  0x5361, 0x0020, 0x0002, 0x0001, /*14504*/
  0x5362, 0x0020, 0x0002, 0x0001, /*14505*/
  0x5363, 0x0020, 0x0002, 0x0001, /*14506*/
  0x5364, 0x0020, 0x0002, 0x0001, /*14507*/
  0x5365, 0x0020, 0x0002, 0x0001, /*14508*/
  0x5366, 0x0020, 0x0002, 0x0001, /*14509*/
  0x5367, 0x0020, 0x0002, 0x0001, /*1450A*/
  0x5368, 0x0020, 0x0002, 0x0001, /*1450B*/
  0x5369, 0x0020, 0x0002, 0x0001, /*1450C*/
  0x536A, 0x0020, 0x0002, 0x0001, /*1450D*/
  0x536B, 0x0020, 0x0002, 0x0001, /*1450E*/
  0x536C, 0x0020, 0x0002, 0x0001, /*1450F*/
  0x536D, 0x0020, 0x0002, 0x0001, /*14510*/
  0x536E, 0x0020, 0x0002, 0x0001, /*14511*/
  0x536F, 0x0020, 0x0002, 0x0001, /*14512*/
  0x5370, 0x0020, 0x0002, 0x0001, /*14513*/
  0x5371, 0x0020, 0x0002, 0x0001, /*14514*/
  0x5372, 0x0020, 0x0002, 0x0001, /*14515*/
  0x5373, 0x0020, 0x0002, 0x0001, /*14516*/
  0x5374, 0x0020, 0x0002, 0x0001, /*14517*/
  0x5375, 0x0020, 0x0002, 0x0001, /*14518*/
  0x5376, 0x0020, 0x0002, 0x0001, /*14519*/
  0x5377, 0x0020, 0x0002, 0x0001, /*1451A*/
  0x5378, 0x0020, 0x0002, 0x0001, /*1451B*/
  0x5379, 0x0020, 0x0002, 0x0001, /*1451C*/
  0x537A, 0x0020, 0x0002, 0x0001, /*1451D*/
  0x537B, 0x0020, 0x0002, 0x0001, /*1451E*/
  0x537C, 0x0020, 0x0002, 0x0001, /*1451F*/
  0x537D, 0x0020, 0x0002, 0x0001, /*14520*/
  0x537E, 0x0020, 0x0002, 0x0001, /*14521*/
  0x537F, 0x0020, 0x0002, 0x0001, /*14522*/
  0x5380, 0x0020, 0x0002, 0x0001, /*14523*/
  0x5381, 0x0020, 0x0002, 0x0001, /*14524*/
  0x5382, 0x0020, 0x0002, 0x0001, /*14525*/
  0x5383, 0x0020, 0x0002, 0x0001, /*14526*/
  0x5384, 0x0020, 0x0002, 0x0001, /*14527*/
  0x5385, 0x0020, 0x0002, 0x0001, /*14528*/
  0x5386, 0x0020, 0x0002, 0x0001, /*14529*/
  0x5387, 0x0020, 0x0002, 0x0001, /*1452A*/
  0x5388, 0x0020, 0x0002, 0x0001, /*1452B*/
  0x5389, 0x0020, 0x0002, 0x0001, /*1452C*/
  0x538A, 0x0020, 0x0002, 0x0001, /*1452D*/
  0x538B, 0x0020, 0x0002, 0x0001, /*1452E*/
  0x538C, 0x0020, 0x0002, 0x0001, /*1452F*/
  0x538D, 0x0020, 0x0002, 0x0001, /*14530*/
  0x538E, 0x0020, 0x0002, 0x0001, /*14531*/
  0x538F, 0x0020, 0x0002, 0x0001, /*14532*/
  0x5390, 0x0020, 0x0002, 0x0001, /*14533*/
  0x5391, 0x0020, 0x0002, 0x0001, /*14534*/
  0x5392, 0x0020, 0x0002, 0x0001, /*14535*/
  0x5393, 0x0020, 0x0002, 0x0001, /*14536*/
  0x5394, 0x0020, 0x0002, 0x0001, /*14537*/
  0x5395, 0x0020, 0x0002, 0x0001, /*14538*/
  0x5396, 0x0020, 0x0002, 0x0001, /*14539*/
  0x5397, 0x0020, 0x0002, 0x0001, /*1453A*/
  0x5398, 0x0020, 0x0002, 0x0001, /*1453B*/
  0x5399, 0x0020, 0x0002, 0x0001, /*1453C*/
  0x539A, 0x0020, 0x0002, 0x0001, /*1453D*/
  0x539B, 0x0020, 0x0002, 0x0001, /*1453E*/
  0x539C, 0x0020, 0x0002, 0x0001, /*1453F*/
  0x539D, 0x0020, 0x0002, 0x0001, /*14540*/
  0x539E, 0x0020, 0x0002, 0x0001, /*14541*/
  0x539F, 0x0020, 0x0002, 0x0001, /*14542*/
  0x53A0, 0x0020, 0x0002, 0x0001, /*14543*/
  0x53A1, 0x0020, 0x0002, 0x0001, /*14544*/
  0x53A2, 0x0020, 0x0002, 0x0001, /*14545*/
  0x53A3, 0x0020, 0x0002, 0x0001, /*14546*/
  0x53A4, 0x0020, 0x0002, 0x0001, /*14547*/
  0x53A5, 0x0020, 0x0002, 0x0001, /*14548*/
  0x53A6, 0x0020, 0x0002, 0x0001, /*14549*/
  0x53A7, 0x0020, 0x0002, 0x0001, /*1454A*/
  0x53A8, 0x0020, 0x0002, 0x0001, /*1454B*/
  0x53A9, 0x0020, 0x0002, 0x0001, /*1454C*/
  0x53AA, 0x0020, 0x0002, 0x0001, /*1454D*/
  0x53AB, 0x0020, 0x0002, 0x0001, /*1454E*/
  0x53AC, 0x0020, 0x0002, 0x0001, /*1454F*/
  0x53AD, 0x0020, 0x0002, 0x0001, /*14550*/
  0x53AE, 0x0020, 0x0002, 0x0001, /*14551*/
  0x53AF, 0x0020, 0x0002, 0x0001, /*14552*/
  0x53B0, 0x0020, 0x0002, 0x0001, /*14553*/
  0x53B1, 0x0020, 0x0002, 0x0001, /*14554*/
  0x53B2, 0x0020, 0x0002, 0x0001, /*14555*/
  0x53B3, 0x0020, 0x0002, 0x0001, /*14556*/
  0x53B4, 0x0020, 0x0002, 0x0001, /*14557*/
  0x53B5, 0x0020, 0x0002, 0x0001, /*14558*/
  0x53B6, 0x0020, 0x0002, 0x0001, /*14559*/
  0x53B7, 0x0020, 0x0002, 0x0001, /*1455A*/
  0x53B8, 0x0020, 0x0002, 0x0001, /*1455B*/
  0x53B9, 0x0020, 0x0002, 0x0001, /*1455C*/
  0x53BA, 0x0020, 0x0002, 0x0001, /*1455D*/
  0x53BB, 0x0020, 0x0002, 0x0001, /*1455E*/
  0x53BC, 0x0020, 0x0002, 0x0001, /*1455F*/
  0x53BD, 0x0020, 0x0002, 0x0001, /*14560*/
  0x53BE, 0x0020, 0x0002, 0x0001, /*14561*/
  0x53BF, 0x0020, 0x0002, 0x0001, /*14562*/
  0x53C0, 0x0020, 0x0002, 0x0001, /*14563*/
  0x53C1, 0x0020, 0x0002, 0x0001, /*14564*/
  0x53C2, 0x0020, 0x0002, 0x0001, /*14565*/
  0x53C3, 0x0020, 0x0002, 0x0001, /*14566*/
  0x53C4, 0x0020, 0x0002, 0x0001, /*14567*/
  0x53C5, 0x0020, 0x0002, 0x0001, /*14568*/
  0x53C6, 0x0020, 0x0002, 0x0001, /*14569*/
  0x53C7, 0x0020, 0x0002, 0x0001, /*1456A*/
  0x53C8, 0x0020, 0x0002, 0x0001, /*1456B*/
  0x53C9, 0x0020, 0x0002, 0x0001, /*1456C*/
  0x53CA, 0x0020, 0x0002, 0x0001, /*1456D*/
  0x53CB, 0x0020, 0x0002, 0x0001, /*1456E*/
  0x53CC, 0x0020, 0x0002, 0x0001, /*1456F*/
  0x53CD, 0x0020, 0x0002, 0x0001, /*14570*/
  0x53CE, 0x0020, 0x0002, 0x0001, /*14571*/
  0x53CF, 0x0020, 0x0002, 0x0001, /*14572*/
  0x53D0, 0x0020, 0x0002, 0x0001, /*14573*/
  0x53D1, 0x0020, 0x0002, 0x0001, /*14574*/
  0x53D2, 0x0020, 0x0002, 0x0001, /*14575*/
  0x53D3, 0x0020, 0x0002, 0x0001, /*14576*/
  0x53D4, 0x0020, 0x0002, 0x0001, /*14577*/
  0x53D5, 0x0020, 0x0002, 0x0001, /*14578*/
  0x53D6, 0x0020, 0x0002, 0x0001, /*14579*/
  0x53D7, 0x0020, 0x0002, 0x0001, /*1457A*/
  0x53D8, 0x0020, 0x0002, 0x0001, /*1457B*/
  0x53D9, 0x0020, 0x0002, 0x0001, /*1457C*/
  0x53DA, 0x0020, 0x0002, 0x0001, /*1457D*/
  0x53DB, 0x0020, 0x0002, 0x0001, /*1457E*/
  0x53DC, 0x0020, 0x0002, 0x0001, /*1457F*/
  0x53DD, 0x0020, 0x0002, 0x0001, /*14580*/
  0x53DE, 0x0020, 0x0002, 0x0001, /*14581*/
  0x53DF, 0x0020, 0x0002, 0x0001, /*14582*/
  0x53E0, 0x0020, 0x0002, 0x0001, /*14583*/
  0x53E1, 0x0020, 0x0002, 0x0001, /*14584*/
  0x53E2, 0x0020, 0x0002, 0x0001, /*14585*/
  0x53E3, 0x0020, 0x0002, 0x0001, /*14586*/
  0x53E4, 0x0020, 0x0002, 0x0001, /*14587*/
  0x53E5, 0x0020, 0x0002, 0x0001, /*14588*/
  0x53E6, 0x0020, 0x0002, 0x0001, /*14589*/
  0x53E7, 0x0020, 0x0002, 0x0001, /*1458A*/
  0x53E8, 0x0020, 0x0002, 0x0001, /*1458B*/
  0x53E9, 0x0020, 0x0002, 0x0001, /*1458C*/
  0x53EA, 0x0020, 0x0002, 0x0001, /*1458D*/
  0x53EB, 0x0020, 0x0002, 0x0001, /*1458E*/
  0x53EC, 0x0020, 0x0002, 0x0001, /*1458F*/
  0x53ED, 0x0020, 0x0002, 0x0001, /*14590*/
  0x53EE, 0x0020, 0x0002, 0x0001, /*14591*/
  0x53EF, 0x0020, 0x0002, 0x0001, /*14592*/
  0x53F0, 0x0020, 0x0002, 0x0001, /*14593*/
  0x53F1, 0x0020, 0x0002, 0x0001, /*14594*/
  0x53F2, 0x0020, 0x0002, 0x0001, /*14595*/
  0x53F3, 0x0020, 0x0002, 0x0001, /*14596*/
  0x53F4, 0x0020, 0x0002, 0x0001, /*14597*/
  0x53F5, 0x0020, 0x0002, 0x0001, /*14598*/
  0x53F6, 0x0020, 0x0002, 0x0001, /*14599*/
  0x53F7, 0x0020, 0x0002, 0x0001, /*1459A*/
  0x53F8, 0x0020, 0x0002, 0x0001, /*1459B*/
  0x53F9, 0x0020, 0x0002, 0x0001, /*1459C*/
  0x53FA, 0x0020, 0x0002, 0x0001, /*1459D*/
  0x53FB, 0x0020, 0x0002, 0x0001, /*1459E*/
  0x53FC, 0x0020, 0x0002, 0x0001, /*1459F*/
  0x53FD, 0x0020, 0x0002, 0x0001, /*145A0*/
  0x53FE, 0x0020, 0x0002, 0x0001, /*145A1*/
  0x53FF, 0x0020, 0x0002, 0x0001, /*145A2*/
  0x5400, 0x0020, 0x0002, 0x0001, /*145A3*/
  0x5401, 0x0020, 0x0002, 0x0001, /*145A4*/
  0x5402, 0x0020, 0x0002, 0x0001, /*145A5*/
  0x5403, 0x0020, 0x0002, 0x0001, /*145A6*/
  0x5404, 0x0020, 0x0002, 0x0001, /*145A7*/
  0x5405, 0x0020, 0x0002, 0x0001, /*145A8*/
  0x5406, 0x0020, 0x0002, 0x0001, /*145A9*/
  0x5407, 0x0020, 0x0002, 0x0001, /*145AA*/
  0x5408, 0x0020, 0x0002, 0x0001, /*145AB*/
  0x5409, 0x0020, 0x0002, 0x0001, /*145AC*/
  0x540A, 0x0020, 0x0002, 0x0001, /*145AD*/
  0x540B, 0x0020, 0x0002, 0x0001, /*145AE*/
  0x540C, 0x0020, 0x0002, 0x0001, /*145AF*/
  0x540D, 0x0020, 0x0002, 0x0001, /*145B0*/
  0x540E, 0x0020, 0x0002, 0x0001, /*145B1*/
  0x540F, 0x0020, 0x0002, 0x0001, /*145B2*/
  0x5410, 0x0020, 0x0002, 0x0001, /*145B3*/
  0x5411, 0x0020, 0x0002, 0x0001, /*145B4*/
  0x5412, 0x0020, 0x0002, 0x0001, /*145B5*/
  0x5413, 0x0020, 0x0002, 0x0001, /*145B6*/
  0x5414, 0x0020, 0x0002, 0x0001, /*145B7*/
  0x5415, 0x0020, 0x0002, 0x0001, /*145B8*/
  0x5416, 0x0020, 0x0002, 0x0001, /*145B9*/
  0x5417, 0x0020, 0x0002, 0x0001, /*145BA*/
  0x5418, 0x0020, 0x0002, 0x0001, /*145BB*/
  0x5419, 0x0020, 0x0002, 0x0001, /*145BC*/
  0x541A, 0x0020, 0x0002, 0x0001, /*145BD*/
  0x541B, 0x0020, 0x0002, 0x0001, /*145BE*/
  0x541C, 0x0020, 0x0002, 0x0001, /*145BF*/
  0x541D, 0x0020, 0x0002, 0x0001, /*145C0*/
  0x541E, 0x0020, 0x0002, 0x0001, /*145C1*/
  0x541F, 0x0020, 0x0002, 0x0001, /*145C2*/
  0x5420, 0x0020, 0x0002, 0x0001, /*145C3*/
  0x5421, 0x0020, 0x0002, 0x0001, /*145C4*/
  0x5422, 0x0020, 0x0002, 0x0001, /*145C5*/
  0x5423, 0x0020, 0x0002, 0x0001, /*145C6*/
  0x5424, 0x0020, 0x0002, 0x0001, /*145C7*/
  0x5425, 0x0020, 0x0002, 0x0001, /*145C8*/
  0x5426, 0x0020, 0x0002, 0x0001, /*145C9*/
  0x5427, 0x0020, 0x0002, 0x0001, /*145CA*/
  0x5428, 0x0020, 0x0002, 0x0001, /*145CB*/
  0x5429, 0x0020, 0x0002, 0x0001, /*145CC*/
  0x542A, 0x0020, 0x0002, 0x0001, /*145CD*/
  0x542B, 0x0020, 0x0002, 0x0001, /*145CE*/
  0x542C, 0x0020, 0x0002, 0x0001, /*145CF*/
  0x542D, 0x0020, 0x0002, 0x0001, /*145D0*/
  0x542E, 0x0020, 0x0002, 0x0001, /*145D1*/
  0x542F, 0x0020, 0x0002, 0x0001, /*145D2*/
  0x5430, 0x0020, 0x0002, 0x0001, /*145D3*/
  0x5431, 0x0020, 0x0002, 0x0001, /*145D4*/
  0x5432, 0x0020, 0x0002, 0x0001, /*145D5*/
  0x5433, 0x0020, 0x0002, 0x0001, /*145D6*/
  0x5434, 0x0020, 0x0002, 0x0001, /*145D7*/
  0x5435, 0x0020, 0x0002, 0x0001, /*145D8*/
  0x5436, 0x0020, 0x0002, 0x0001, /*145D9*/
  0x5437, 0x0020, 0x0002, 0x0001, /*145DA*/
  0x5438, 0x0020, 0x0002, 0x0001, /*145DB*/
  0x5439, 0x0020, 0x0002, 0x0001, /*145DC*/
  0x543A, 0x0020, 0x0002, 0x0001, /*145DD*/
  0x543B, 0x0020, 0x0002, 0x0001, /*145DE*/
  0x543C, 0x0020, 0x0002, 0x0001, /*145DF*/
  0x543D, 0x0020, 0x0002, 0x0001, /*145E0*/
  0x543E, 0x0020, 0x0002, 0x0001, /*145E1*/
  0x543F, 0x0020, 0x0002, 0x0001, /*145E2*/
  0x5440, 0x0020, 0x0002, 0x0001, /*145E3*/
  0x5441, 0x0020, 0x0002, 0x0001, /*145E4*/
  0x5442, 0x0020, 0x0002, 0x0001, /*145E5*/
  0x5443, 0x0020, 0x0002, 0x0001, /*145E6*/
  0x5444, 0x0020, 0x0002, 0x0001, /*145E7*/
  0x5445, 0x0020, 0x0002, 0x0001, /*145E8*/
  0x5446, 0x0020, 0x0002, 0x0001, /*145E9*/
  0x5447, 0x0020, 0x0002, 0x0001, /*145EA*/
  0x5448, 0x0020, 0x0002, 0x0001, /*145EB*/
  0x5449, 0x0020, 0x0002, 0x0001, /*145EC*/
  0x544A, 0x0020, 0x0002, 0x0001, /*145ED*/
  0x544B, 0x0020, 0x0002, 0x0001, /*145EE*/
  0x544C, 0x0020, 0x0002, 0x0001, /*145EF*/
  0x544D, 0x0020, 0x0002, 0x0001, /*145F0*/
  0x544E, 0x0020, 0x0002, 0x0001, /*145F1*/
  0x544F, 0x0020, 0x0002, 0x0001, /*145F2*/
  0x5450, 0x0020, 0x0002, 0x0001, /*145F3*/
  0x5451, 0x0020, 0x0002, 0x0001, /*145F4*/
  0x5452, 0x0020, 0x0002, 0x0001, /*145F5*/
  0x5453, 0x0020, 0x0002, 0x0001, /*145F6*/
  0x5454, 0x0020, 0x0002, 0x0001, /*145F7*/
  0x5455, 0x0020, 0x0002, 0x0001, /*145F8*/
  0x5456, 0x0020, 0x0002, 0x0001, /*145F9*/
  0x5457, 0x0020, 0x0002, 0x0001, /*145FA*/
  0x5458, 0x0020, 0x0002, 0x0001, /*145FB*/
  0x5459, 0x0020, 0x0002, 0x0001, /*145FC*/
  0x545A, 0x0020, 0x0002, 0x0001, /*145FD*/
  0x545B, 0x0020, 0x0002, 0x0001, /*145FE*/
  0x545C, 0x0020, 0x0002, 0x0001  /*145FF*/
};

uint16 uca900_p146[]= {
  0x545D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14600*/
  0x545E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14601*/
  0x545F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14602*/
  0x5460, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14603*/
  0x5461, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14604*/
  0x5462, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14605*/
  0x5463, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14606*/
  0x5464, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14607*/
  0x5465, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14608*/
  0x5466, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14609*/
  0x5467, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1460A*/
  0x5468, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1460B*/
  0x5469, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1460C*/
  0x546A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1460D*/
  0x546B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1460E*/
  0x546C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1460F*/
  0x546D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14610*/
  0x546E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14611*/
  0x546F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14612*/
  0x5470, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14613*/
  0x5471, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14614*/
  0x5472, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14615*/
  0x5473, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14616*/
  0x5474, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14617*/
  0x5475, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14618*/
  0x5476, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14619*/
  0x5477, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1461A*/
  0x5478, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1461B*/
  0x5479, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1461C*/
  0x547A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1461D*/
  0x547B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1461E*/
  0x547C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1461F*/
  0x547D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14620*/
  0x547E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14621*/
  0x547F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14622*/
  0x5480, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14623*/
  0x5481, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14624*/
  0x5482, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14625*/
  0x5483, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14626*/
  0x5484, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14627*/
  0x5485, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14628*/
  0x5486, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14629*/
  0x5487, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1462A*/
  0x5488, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1462B*/
  0x5489, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1462C*/
  0x548A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1462D*/
  0x548B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1462E*/
  0x548C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1462F*/
  0x548D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14630*/
  0x548E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14631*/
  0x548F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14632*/
  0x5490, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14633*/
  0x5491, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14634*/
  0x5492, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14635*/
  0x5493, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14636*/
  0x5494, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14637*/
  0x5495, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14638*/
  0x5496, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14639*/
  0x5497, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1463A*/
  0x5498, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1463B*/
  0x5499, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1463C*/
  0x549A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1463D*/
  0x549B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1463E*/
  0x549C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1463F*/
  0x549D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14640*/
  0x549E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14641*/
  0x549F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14642*/
  0x54A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14643*/
  0x54A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14644*/
  0x54A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14645*/
  0x54A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*14646*/
  0xFBC2, 0x0020, 0x0002, 0xC647, 0x0000, 0x0000, 0x0002, /*14647*/
  0xFBC2, 0x0020, 0x0002, 0xC648, 0x0000, 0x0000, 0x0002, /*14648*/
  0xFBC2, 0x0020, 0x0002, 0xC649, 0x0000, 0x0000, 0x0002, /*14649*/
  0xFBC2, 0x0020, 0x0002, 0xC64A, 0x0000, 0x0000, 0x0002, /*1464A*/
  0xFBC2, 0x0020, 0x0002, 0xC64B, 0x0000, 0x0000, 0x0002, /*1464B*/
  0xFBC2, 0x0020, 0x0002, 0xC64C, 0x0000, 0x0000, 0x0002, /*1464C*/
  0xFBC2, 0x0020, 0x0002, 0xC64D, 0x0000, 0x0000, 0x0002, /*1464D*/
  0xFBC2, 0x0020, 0x0002, 0xC64E, 0x0000, 0x0000, 0x0002, /*1464E*/
  0xFBC2, 0x0020, 0x0002, 0xC64F, 0x0000, 0x0000, 0x0002, /*1464F*/
  0xFBC2, 0x0020, 0x0002, 0xC650, 0x0000, 0x0000, 0x0002, /*14650*/
  0xFBC2, 0x0020, 0x0002, 0xC651, 0x0000, 0x0000, 0x0002, /*14651*/
  0xFBC2, 0x0020, 0x0002, 0xC652, 0x0000, 0x0000, 0x0002, /*14652*/
  0xFBC2, 0x0020, 0x0002, 0xC653, 0x0000, 0x0000, 0x0002, /*14653*/
  0xFBC2, 0x0020, 0x0002, 0xC654, 0x0000, 0x0000, 0x0002, /*14654*/
  0xFBC2, 0x0020, 0x0002, 0xC655, 0x0000, 0x0000, 0x0002, /*14655*/
  0xFBC2, 0x0020, 0x0002, 0xC656, 0x0000, 0x0000, 0x0002, /*14656*/
  0xFBC2, 0x0020, 0x0002, 0xC657, 0x0000, 0x0000, 0x0002, /*14657*/
  0xFBC2, 0x0020, 0x0002, 0xC658, 0x0000, 0x0000, 0x0002, /*14658*/
  0xFBC2, 0x0020, 0x0002, 0xC659, 0x0000, 0x0000, 0x0002, /*14659*/
  0xFBC2, 0x0020, 0x0002, 0xC65A, 0x0000, 0x0000, 0x0002, /*1465A*/
  0xFBC2, 0x0020, 0x0002, 0xC65B, 0x0000, 0x0000, 0x0002, /*1465B*/
  0xFBC2, 0x0020, 0x0002, 0xC65C, 0x0000, 0x0000, 0x0002, /*1465C*/
  0xFBC2, 0x0020, 0x0002, 0xC65D, 0x0000, 0x0000, 0x0002, /*1465D*/
  0xFBC2, 0x0020, 0x0002, 0xC65E, 0x0000, 0x0000, 0x0002, /*1465E*/
  0xFBC2, 0x0020, 0x0002, 0xC65F, 0x0000, 0x0000, 0x0002, /*1465F*/
  0xFBC2, 0x0020, 0x0002, 0xC660, 0x0000, 0x0000, 0x0002, /*14660*/
  0xFBC2, 0x0020, 0x0002, 0xC661, 0x0000, 0x0000, 0x0002, /*14661*/
  0xFBC2, 0x0020, 0x0002, 0xC662, 0x0000, 0x0000, 0x0002, /*14662*/
  0xFBC2, 0x0020, 0x0002, 0xC663, 0x0000, 0x0000, 0x0002, /*14663*/
  0xFBC2, 0x0020, 0x0002, 0xC664, 0x0000, 0x0000, 0x0002, /*14664*/
  0xFBC2, 0x0020, 0x0002, 0xC665, 0x0000, 0x0000, 0x0002, /*14665*/
  0xFBC2, 0x0020, 0x0002, 0xC666, 0x0000, 0x0000, 0x0002, /*14666*/
  0xFBC2, 0x0020, 0x0002, 0xC667, 0x0000, 0x0000, 0x0002, /*14667*/
  0xFBC2, 0x0020, 0x0002, 0xC668, 0x0000, 0x0000, 0x0002, /*14668*/
  0xFBC2, 0x0020, 0x0002, 0xC669, 0x0000, 0x0000, 0x0002, /*14669*/
  0xFBC2, 0x0020, 0x0002, 0xC66A, 0x0000, 0x0000, 0x0002, /*1466A*/
  0xFBC2, 0x0020, 0x0002, 0xC66B, 0x0000, 0x0000, 0x0002, /*1466B*/
  0xFBC2, 0x0020, 0x0002, 0xC66C, 0x0000, 0x0000, 0x0002, /*1466C*/
  0xFBC2, 0x0020, 0x0002, 0xC66D, 0x0000, 0x0000, 0x0002, /*1466D*/
  0xFBC2, 0x0020, 0x0002, 0xC66E, 0x0000, 0x0000, 0x0002, /*1466E*/
  0xFBC2, 0x0020, 0x0002, 0xC66F, 0x0000, 0x0000, 0x0002, /*1466F*/
  0xFBC2, 0x0020, 0x0002, 0xC670, 0x0000, 0x0000, 0x0002, /*14670*/
  0xFBC2, 0x0020, 0x0002, 0xC671, 0x0000, 0x0000, 0x0002, /*14671*/
  0xFBC2, 0x0020, 0x0002, 0xC672, 0x0000, 0x0000, 0x0002, /*14672*/
  0xFBC2, 0x0020, 0x0002, 0xC673, 0x0000, 0x0000, 0x0002, /*14673*/
  0xFBC2, 0x0020, 0x0002, 0xC674, 0x0000, 0x0000, 0x0002, /*14674*/
  0xFBC2, 0x0020, 0x0002, 0xC675, 0x0000, 0x0000, 0x0002, /*14675*/
  0xFBC2, 0x0020, 0x0002, 0xC676, 0x0000, 0x0000, 0x0002, /*14676*/
  0xFBC2, 0x0020, 0x0002, 0xC677, 0x0000, 0x0000, 0x0002, /*14677*/
  0xFBC2, 0x0020, 0x0002, 0xC678, 0x0000, 0x0000, 0x0002, /*14678*/
  0xFBC2, 0x0020, 0x0002, 0xC679, 0x0000, 0x0000, 0x0002, /*14679*/
  0xFBC2, 0x0020, 0x0002, 0xC67A, 0x0000, 0x0000, 0x0002, /*1467A*/
  0xFBC2, 0x0020, 0x0002, 0xC67B, 0x0000, 0x0000, 0x0002, /*1467B*/
  0xFBC2, 0x0020, 0x0002, 0xC67C, 0x0000, 0x0000, 0x0002, /*1467C*/
  0xFBC2, 0x0020, 0x0002, 0xC67D, 0x0000, 0x0000, 0x0002, /*1467D*/
  0xFBC2, 0x0020, 0x0002, 0xC67E, 0x0000, 0x0000, 0x0002, /*1467E*/
  0xFBC2, 0x0020, 0x0002, 0xC67F, 0x0000, 0x0000, 0x0002, /*1467F*/
  0xFBC2, 0x0020, 0x0002, 0xC680, 0x0000, 0x0000, 0x0002, /*14680*/
  0xFBC2, 0x0020, 0x0002, 0xC681, 0x0000, 0x0000, 0x0002, /*14681*/
  0xFBC2, 0x0020, 0x0002, 0xC682, 0x0000, 0x0000, 0x0002, /*14682*/
  0xFBC2, 0x0020, 0x0002, 0xC683, 0x0000, 0x0000, 0x0002, /*14683*/
  0xFBC2, 0x0020, 0x0002, 0xC684, 0x0000, 0x0000, 0x0002, /*14684*/
  0xFBC2, 0x0020, 0x0002, 0xC685, 0x0000, 0x0000, 0x0002, /*14685*/
  0xFBC2, 0x0020, 0x0002, 0xC686, 0x0000, 0x0000, 0x0002, /*14686*/
  0xFBC2, 0x0020, 0x0002, 0xC687, 0x0000, 0x0000, 0x0002, /*14687*/
  0xFBC2, 0x0020, 0x0002, 0xC688, 0x0000, 0x0000, 0x0002, /*14688*/
  0xFBC2, 0x0020, 0x0002, 0xC689, 0x0000, 0x0000, 0x0002, /*14689*/
  0xFBC2, 0x0020, 0x0002, 0xC68A, 0x0000, 0x0000, 0x0002, /*1468A*/
  0xFBC2, 0x0020, 0x0002, 0xC68B, 0x0000, 0x0000, 0x0002, /*1468B*/
  0xFBC2, 0x0020, 0x0002, 0xC68C, 0x0000, 0x0000, 0x0002, /*1468C*/
  0xFBC2, 0x0020, 0x0002, 0xC68D, 0x0000, 0x0000, 0x0002, /*1468D*/
  0xFBC2, 0x0020, 0x0002, 0xC68E, 0x0000, 0x0000, 0x0002, /*1468E*/
  0xFBC2, 0x0020, 0x0002, 0xC68F, 0x0000, 0x0000, 0x0002, /*1468F*/
  0xFBC2, 0x0020, 0x0002, 0xC690, 0x0000, 0x0000, 0x0002, /*14690*/
  0xFBC2, 0x0020, 0x0002, 0xC691, 0x0000, 0x0000, 0x0002, /*14691*/
  0xFBC2, 0x0020, 0x0002, 0xC692, 0x0000, 0x0000, 0x0002, /*14692*/
  0xFBC2, 0x0020, 0x0002, 0xC693, 0x0000, 0x0000, 0x0002, /*14693*/
  0xFBC2, 0x0020, 0x0002, 0xC694, 0x0000, 0x0000, 0x0002, /*14694*/
  0xFBC2, 0x0020, 0x0002, 0xC695, 0x0000, 0x0000, 0x0002, /*14695*/
  0xFBC2, 0x0020, 0x0002, 0xC696, 0x0000, 0x0000, 0x0002, /*14696*/
  0xFBC2, 0x0020, 0x0002, 0xC697, 0x0000, 0x0000, 0x0002, /*14697*/
  0xFBC2, 0x0020, 0x0002, 0xC698, 0x0000, 0x0000, 0x0002, /*14698*/
  0xFBC2, 0x0020, 0x0002, 0xC699, 0x0000, 0x0000, 0x0002, /*14699*/
  0xFBC2, 0x0020, 0x0002, 0xC69A, 0x0000, 0x0000, 0x0002, /*1469A*/
  0xFBC2, 0x0020, 0x0002, 0xC69B, 0x0000, 0x0000, 0x0002, /*1469B*/
  0xFBC2, 0x0020, 0x0002, 0xC69C, 0x0000, 0x0000, 0x0002, /*1469C*/
  0xFBC2, 0x0020, 0x0002, 0xC69D, 0x0000, 0x0000, 0x0002, /*1469D*/
  0xFBC2, 0x0020, 0x0002, 0xC69E, 0x0000, 0x0000, 0x0002, /*1469E*/
  0xFBC2, 0x0020, 0x0002, 0xC69F, 0x0000, 0x0000, 0x0002, /*1469F*/
  0xFBC2, 0x0020, 0x0002, 0xC6A0, 0x0000, 0x0000, 0x0002, /*146A0*/
  0xFBC2, 0x0020, 0x0002, 0xC6A1, 0x0000, 0x0000, 0x0002, /*146A1*/
  0xFBC2, 0x0020, 0x0002, 0xC6A2, 0x0000, 0x0000, 0x0002, /*146A2*/
  0xFBC2, 0x0020, 0x0002, 0xC6A3, 0x0000, 0x0000, 0x0002, /*146A3*/
  0xFBC2, 0x0020, 0x0002, 0xC6A4, 0x0000, 0x0000, 0x0002, /*146A4*/
  0xFBC2, 0x0020, 0x0002, 0xC6A5, 0x0000, 0x0000, 0x0002, /*146A5*/
  0xFBC2, 0x0020, 0x0002, 0xC6A6, 0x0000, 0x0000, 0x0002, /*146A6*/
  0xFBC2, 0x0020, 0x0002, 0xC6A7, 0x0000, 0x0000, 0x0002, /*146A7*/
  0xFBC2, 0x0020, 0x0002, 0xC6A8, 0x0000, 0x0000, 0x0002, /*146A8*/
  0xFBC2, 0x0020, 0x0002, 0xC6A9, 0x0000, 0x0000, 0x0002, /*146A9*/
  0xFBC2, 0x0020, 0x0002, 0xC6AA, 0x0000, 0x0000, 0x0002, /*146AA*/
  0xFBC2, 0x0020, 0x0002, 0xC6AB, 0x0000, 0x0000, 0x0002, /*146AB*/
  0xFBC2, 0x0020, 0x0002, 0xC6AC, 0x0000, 0x0000, 0x0002, /*146AC*/
  0xFBC2, 0x0020, 0x0002, 0xC6AD, 0x0000, 0x0000, 0x0002, /*146AD*/
  0xFBC2, 0x0020, 0x0002, 0xC6AE, 0x0000, 0x0000, 0x0002, /*146AE*/
  0xFBC2, 0x0020, 0x0002, 0xC6AF, 0x0000, 0x0000, 0x0002, /*146AF*/
  0xFBC2, 0x0020, 0x0002, 0xC6B0, 0x0000, 0x0000, 0x0002, /*146B0*/
  0xFBC2, 0x0020, 0x0002, 0xC6B1, 0x0000, 0x0000, 0x0002, /*146B1*/
  0xFBC2, 0x0020, 0x0002, 0xC6B2, 0x0000, 0x0000, 0x0002, /*146B2*/
  0xFBC2, 0x0020, 0x0002, 0xC6B3, 0x0000, 0x0000, 0x0002, /*146B3*/
  0xFBC2, 0x0020, 0x0002, 0xC6B4, 0x0000, 0x0000, 0x0002, /*146B4*/
  0xFBC2, 0x0020, 0x0002, 0xC6B5, 0x0000, 0x0000, 0x0002, /*146B5*/
  0xFBC2, 0x0020, 0x0002, 0xC6B6, 0x0000, 0x0000, 0x0002, /*146B6*/
  0xFBC2, 0x0020, 0x0002, 0xC6B7, 0x0000, 0x0000, 0x0002, /*146B7*/
  0xFBC2, 0x0020, 0x0002, 0xC6B8, 0x0000, 0x0000, 0x0002, /*146B8*/
  0xFBC2, 0x0020, 0x0002, 0xC6B9, 0x0000, 0x0000, 0x0002, /*146B9*/
  0xFBC2, 0x0020, 0x0002, 0xC6BA, 0x0000, 0x0000, 0x0002, /*146BA*/
  0xFBC2, 0x0020, 0x0002, 0xC6BB, 0x0000, 0x0000, 0x0002, /*146BB*/
  0xFBC2, 0x0020, 0x0002, 0xC6BC, 0x0000, 0x0000, 0x0002, /*146BC*/
  0xFBC2, 0x0020, 0x0002, 0xC6BD, 0x0000, 0x0000, 0x0002, /*146BD*/
  0xFBC2, 0x0020, 0x0002, 0xC6BE, 0x0000, 0x0000, 0x0002, /*146BE*/
  0xFBC2, 0x0020, 0x0002, 0xC6BF, 0x0000, 0x0000, 0x0002, /*146BF*/
  0xFBC2, 0x0020, 0x0002, 0xC6C0, 0x0000, 0x0000, 0x0002, /*146C0*/
  0xFBC2, 0x0020, 0x0002, 0xC6C1, 0x0000, 0x0000, 0x0002, /*146C1*/
  0xFBC2, 0x0020, 0x0002, 0xC6C2, 0x0000, 0x0000, 0x0002, /*146C2*/
  0xFBC2, 0x0020, 0x0002, 0xC6C3, 0x0000, 0x0000, 0x0002, /*146C3*/
  0xFBC2, 0x0020, 0x0002, 0xC6C4, 0x0000, 0x0000, 0x0002, /*146C4*/
  0xFBC2, 0x0020, 0x0002, 0xC6C5, 0x0000, 0x0000, 0x0002, /*146C5*/
  0xFBC2, 0x0020, 0x0002, 0xC6C6, 0x0000, 0x0000, 0x0002, /*146C6*/
  0xFBC2, 0x0020, 0x0002, 0xC6C7, 0x0000, 0x0000, 0x0002, /*146C7*/
  0xFBC2, 0x0020, 0x0002, 0xC6C8, 0x0000, 0x0000, 0x0002, /*146C8*/
  0xFBC2, 0x0020, 0x0002, 0xC6C9, 0x0000, 0x0000, 0x0002, /*146C9*/
  0xFBC2, 0x0020, 0x0002, 0xC6CA, 0x0000, 0x0000, 0x0002, /*146CA*/
  0xFBC2, 0x0020, 0x0002, 0xC6CB, 0x0000, 0x0000, 0x0002, /*146CB*/
  0xFBC2, 0x0020, 0x0002, 0xC6CC, 0x0000, 0x0000, 0x0002, /*146CC*/
  0xFBC2, 0x0020, 0x0002, 0xC6CD, 0x0000, 0x0000, 0x0002, /*146CD*/
  0xFBC2, 0x0020, 0x0002, 0xC6CE, 0x0000, 0x0000, 0x0002, /*146CE*/
  0xFBC2, 0x0020, 0x0002, 0xC6CF, 0x0000, 0x0000, 0x0002, /*146CF*/
  0xFBC2, 0x0020, 0x0002, 0xC6D0, 0x0000, 0x0000, 0x0002, /*146D0*/
  0xFBC2, 0x0020, 0x0002, 0xC6D1, 0x0000, 0x0000, 0x0002, /*146D1*/
  0xFBC2, 0x0020, 0x0002, 0xC6D2, 0x0000, 0x0000, 0x0002, /*146D2*/
  0xFBC2, 0x0020, 0x0002, 0xC6D3, 0x0000, 0x0000, 0x0002, /*146D3*/
  0xFBC2, 0x0020, 0x0002, 0xC6D4, 0x0000, 0x0000, 0x0002, /*146D4*/
  0xFBC2, 0x0020, 0x0002, 0xC6D5, 0x0000, 0x0000, 0x0002, /*146D5*/
  0xFBC2, 0x0020, 0x0002, 0xC6D6, 0x0000, 0x0000, 0x0002, /*146D6*/
  0xFBC2, 0x0020, 0x0002, 0xC6D7, 0x0000, 0x0000, 0x0002, /*146D7*/
  0xFBC2, 0x0020, 0x0002, 0xC6D8, 0x0000, 0x0000, 0x0002, /*146D8*/
  0xFBC2, 0x0020, 0x0002, 0xC6D9, 0x0000, 0x0000, 0x0002, /*146D9*/
  0xFBC2, 0x0020, 0x0002, 0xC6DA, 0x0000, 0x0000, 0x0002, /*146DA*/
  0xFBC2, 0x0020, 0x0002, 0xC6DB, 0x0000, 0x0000, 0x0002, /*146DB*/
  0xFBC2, 0x0020, 0x0002, 0xC6DC, 0x0000, 0x0000, 0x0002, /*146DC*/
  0xFBC2, 0x0020, 0x0002, 0xC6DD, 0x0000, 0x0000, 0x0002, /*146DD*/
  0xFBC2, 0x0020, 0x0002, 0xC6DE, 0x0000, 0x0000, 0x0002, /*146DE*/
  0xFBC2, 0x0020, 0x0002, 0xC6DF, 0x0000, 0x0000, 0x0002, /*146DF*/
  0xFBC2, 0x0020, 0x0002, 0xC6E0, 0x0000, 0x0000, 0x0002, /*146E0*/
  0xFBC2, 0x0020, 0x0002, 0xC6E1, 0x0000, 0x0000, 0x0002, /*146E1*/
  0xFBC2, 0x0020, 0x0002, 0xC6E2, 0x0000, 0x0000, 0x0002, /*146E2*/
  0xFBC2, 0x0020, 0x0002, 0xC6E3, 0x0000, 0x0000, 0x0002, /*146E3*/
  0xFBC2, 0x0020, 0x0002, 0xC6E4, 0x0000, 0x0000, 0x0002, /*146E4*/
  0xFBC2, 0x0020, 0x0002, 0xC6E5, 0x0000, 0x0000, 0x0002, /*146E5*/
  0xFBC2, 0x0020, 0x0002, 0xC6E6, 0x0000, 0x0000, 0x0002, /*146E6*/
  0xFBC2, 0x0020, 0x0002, 0xC6E7, 0x0000, 0x0000, 0x0002, /*146E7*/
  0xFBC2, 0x0020, 0x0002, 0xC6E8, 0x0000, 0x0000, 0x0002, /*146E8*/
  0xFBC2, 0x0020, 0x0002, 0xC6E9, 0x0000, 0x0000, 0x0002, /*146E9*/
  0xFBC2, 0x0020, 0x0002, 0xC6EA, 0x0000, 0x0000, 0x0002, /*146EA*/
  0xFBC2, 0x0020, 0x0002, 0xC6EB, 0x0000, 0x0000, 0x0002, /*146EB*/
  0xFBC2, 0x0020, 0x0002, 0xC6EC, 0x0000, 0x0000, 0x0002, /*146EC*/
  0xFBC2, 0x0020, 0x0002, 0xC6ED, 0x0000, 0x0000, 0x0002, /*146ED*/
  0xFBC2, 0x0020, 0x0002, 0xC6EE, 0x0000, 0x0000, 0x0002, /*146EE*/
  0xFBC2, 0x0020, 0x0002, 0xC6EF, 0x0000, 0x0000, 0x0002, /*146EF*/
  0xFBC2, 0x0020, 0x0002, 0xC6F0, 0x0000, 0x0000, 0x0002, /*146F0*/
  0xFBC2, 0x0020, 0x0002, 0xC6F1, 0x0000, 0x0000, 0x0002, /*146F1*/
  0xFBC2, 0x0020, 0x0002, 0xC6F2, 0x0000, 0x0000, 0x0002, /*146F2*/
  0xFBC2, 0x0020, 0x0002, 0xC6F3, 0x0000, 0x0000, 0x0002, /*146F3*/
  0xFBC2, 0x0020, 0x0002, 0xC6F4, 0x0000, 0x0000, 0x0002, /*146F4*/
  0xFBC2, 0x0020, 0x0002, 0xC6F5, 0x0000, 0x0000, 0x0002, /*146F5*/
  0xFBC2, 0x0020, 0x0002, 0xC6F6, 0x0000, 0x0000, 0x0002, /*146F6*/
  0xFBC2, 0x0020, 0x0002, 0xC6F7, 0x0000, 0x0000, 0x0002, /*146F7*/
  0xFBC2, 0x0020, 0x0002, 0xC6F8, 0x0000, 0x0000, 0x0002, /*146F8*/
  0xFBC2, 0x0020, 0x0002, 0xC6F9, 0x0000, 0x0000, 0x0002, /*146F9*/
  0xFBC2, 0x0020, 0x0002, 0xC6FA, 0x0000, 0x0000, 0x0002, /*146FA*/
  0xFBC2, 0x0020, 0x0002, 0xC6FB, 0x0000, 0x0000, 0x0002, /*146FB*/
  0xFBC2, 0x0020, 0x0002, 0xC6FC, 0x0000, 0x0000, 0x0002, /*146FC*/
  0xFBC2, 0x0020, 0x0002, 0xC6FD, 0x0000, 0x0000, 0x0002, /*146FD*/
  0xFBC2, 0x0020, 0x0002, 0xC6FE, 0x0000, 0x0000, 0x0002, /*146FE*/
  0xFBC2, 0x0020, 0x0002, 0xC6FF, 0x0000, 0x0000, 0x0002  /*146FF*/
};

uint16 uca900_p168[]= {
  0x38B7, 0x0020, 0x0002, 0x0001, /*16800*/
  0x38B8, 0x0020, 0x0002, 0x0001, /*16801*/
  0x38B9, 0x0020, 0x0002, 0x0001, /*16802*/
  0x38BA, 0x0020, 0x0002, 0x0001, /*16803*/
  0x38BB, 0x0020, 0x0002, 0x0001, /*16804*/
  0x38BC, 0x0020, 0x0002, 0x0001, /*16805*/
  0x38BD, 0x0020, 0x0002, 0x0001, /*16806*/
  0x38BE, 0x0020, 0x0002, 0x0001, /*16807*/
  0x38BF, 0x0020, 0x0002, 0x0001, /*16808*/
  0x38C0, 0x0020, 0x0002, 0x0001, /*16809*/
  0x38C1, 0x0020, 0x0002, 0x0001, /*1680A*/
  0x38C2, 0x0020, 0x0002, 0x0001, /*1680B*/
  0x38C3, 0x0020, 0x0002, 0x0001, /*1680C*/
  0x38C4, 0x0020, 0x0002, 0x0001, /*1680D*/
  0x38C5, 0x0020, 0x0002, 0x0001, /*1680E*/
  0x38C6, 0x0020, 0x0002, 0x0001, /*1680F*/
  0x38C7, 0x0020, 0x0002, 0x0001, /*16810*/
  0x38C8, 0x0020, 0x0002, 0x0001, /*16811*/
  0x38C9, 0x0020, 0x0002, 0x0001, /*16812*/
  0x38CA, 0x0020, 0x0002, 0x0001, /*16813*/
  0x38CB, 0x0020, 0x0002, 0x0001, /*16814*/
  0x38CC, 0x0020, 0x0002, 0x0001, /*16815*/
  0x38CD, 0x0020, 0x0002, 0x0001, /*16816*/
  0x38CE, 0x0020, 0x0002, 0x0001, /*16817*/
  0x38CF, 0x0020, 0x0002, 0x0001, /*16818*/
  0x38D0, 0x0020, 0x0002, 0x0001, /*16819*/
  0x38D1, 0x0020, 0x0002, 0x0001, /*1681A*/
  0x38D2, 0x0020, 0x0002, 0x0001, /*1681B*/
  0x38D3, 0x0020, 0x0002, 0x0001, /*1681C*/
  0x38D4, 0x0020, 0x0002, 0x0001, /*1681D*/
  0x38D5, 0x0020, 0x0002, 0x0001, /*1681E*/
  0x38D6, 0x0020, 0x0002, 0x0001, /*1681F*/
  0x38D7, 0x0020, 0x0002, 0x0001, /*16820*/
  0x38D8, 0x0020, 0x0002, 0x0001, /*16821*/
  0x38D9, 0x0020, 0x0002, 0x0001, /*16822*/
  0x38DA, 0x0020, 0x0002, 0x0001, /*16823*/
  0x38DB, 0x0020, 0x0002, 0x0001, /*16824*/
  0x38DC, 0x0020, 0x0002, 0x0001, /*16825*/
  0x38DD, 0x0020, 0x0002, 0x0001, /*16826*/
  0x38DE, 0x0020, 0x0002, 0x0001, /*16827*/
  0x38DF, 0x0020, 0x0002, 0x0001, /*16828*/
  0x38E0, 0x0020, 0x0002, 0x0001, /*16829*/
  0x38E1, 0x0020, 0x0002, 0x0001, /*1682A*/
  0x38E2, 0x0020, 0x0002, 0x0001, /*1682B*/
  0x38E3, 0x0020, 0x0002, 0x0001, /*1682C*/
  0x38E4, 0x0020, 0x0002, 0x0001, /*1682D*/
  0x38E5, 0x0020, 0x0002, 0x0001, /*1682E*/
  0x38E6, 0x0020, 0x0002, 0x0001, /*1682F*/
  0x38E7, 0x0020, 0x0002, 0x0001, /*16830*/
  0x38E8, 0x0020, 0x0002, 0x0001, /*16831*/
  0x38E9, 0x0020, 0x0002, 0x0001, /*16832*/
  0x38EA, 0x0020, 0x0002, 0x0001, /*16833*/
  0x38EB, 0x0020, 0x0002, 0x0001, /*16834*/
  0x38EC, 0x0020, 0x0002, 0x0001, /*16835*/
  0x38ED, 0x0020, 0x0002, 0x0001, /*16836*/
  0x38EE, 0x0020, 0x0002, 0x0001, /*16837*/
  0x38EF, 0x0020, 0x0002, 0x0001, /*16838*/
  0x38F0, 0x0020, 0x0002, 0x0001, /*16839*/
  0x38F1, 0x0020, 0x0002, 0x0001, /*1683A*/
  0x38F2, 0x0020, 0x0002, 0x0001, /*1683B*/
  0x38F3, 0x0020, 0x0002, 0x0001, /*1683C*/
  0x38F4, 0x0020, 0x0002, 0x0001, /*1683D*/
  0x38F5, 0x0020, 0x0002, 0x0001, /*1683E*/
  0x38F6, 0x0020, 0x0002, 0x0001, /*1683F*/
  0x38F7, 0x0020, 0x0002, 0x0001, /*16840*/
  0x38F8, 0x0020, 0x0002, 0x0001, /*16841*/
  0x38F9, 0x0020, 0x0002, 0x0001, /*16842*/
  0x38FA, 0x0020, 0x0002, 0x0001, /*16843*/
  0x38FB, 0x0020, 0x0002, 0x0001, /*16844*/
  0x38FC, 0x0020, 0x0002, 0x0001, /*16845*/
  0x38FD, 0x0020, 0x0002, 0x0001, /*16846*/
  0x38FE, 0x0020, 0x0002, 0x0001, /*16847*/
  0x38FF, 0x0020, 0x0002, 0x0001, /*16848*/
  0x3900, 0x0020, 0x0002, 0x0001, /*16849*/
  0x3901, 0x0020, 0x0002, 0x0001, /*1684A*/
  0x3902, 0x0020, 0x0002, 0x0001, /*1684B*/
  0x3903, 0x0020, 0x0002, 0x0001, /*1684C*/
  0x3904, 0x0020, 0x0002, 0x0001, /*1684D*/
  0x3905, 0x0020, 0x0002, 0x0001, /*1684E*/
  0x3906, 0x0020, 0x0002, 0x0001, /*1684F*/
  0x3907, 0x0020, 0x0002, 0x0001, /*16850*/
  0x3908, 0x0020, 0x0002, 0x0001, /*16851*/
  0x3909, 0x0020, 0x0002, 0x0001, /*16852*/
  0x390A, 0x0020, 0x0002, 0x0001, /*16853*/
  0x390B, 0x0020, 0x0002, 0x0001, /*16854*/
  0x390C, 0x0020, 0x0002, 0x0001, /*16855*/
  0x390D, 0x0020, 0x0002, 0x0001, /*16856*/
  0x390E, 0x0020, 0x0002, 0x0001, /*16857*/
  0x390F, 0x0020, 0x0002, 0x0001, /*16858*/
  0x3910, 0x0020, 0x0002, 0x0001, /*16859*/
  0x3911, 0x0020, 0x0002, 0x0001, /*1685A*/
  0x3912, 0x0020, 0x0002, 0x0001, /*1685B*/
  0x3913, 0x0020, 0x0002, 0x0001, /*1685C*/
  0x3914, 0x0020, 0x0002, 0x0001, /*1685D*/
  0x3915, 0x0020, 0x0002, 0x0001, /*1685E*/
  0x3916, 0x0020, 0x0002, 0x0001, /*1685F*/
  0x3917, 0x0020, 0x0002, 0x0001, /*16860*/
  0x3918, 0x0020, 0x0002, 0x0001, /*16861*/
  0x3919, 0x0020, 0x0002, 0x0001, /*16862*/
  0x391A, 0x0020, 0x0002, 0x0001, /*16863*/
  0x391B, 0x0020, 0x0002, 0x0001, /*16864*/
  0x391C, 0x0020, 0x0002, 0x0001, /*16865*/
  0x391D, 0x0020, 0x0002, 0x0001, /*16866*/
  0x391E, 0x0020, 0x0002, 0x0001, /*16867*/
  0x391F, 0x0020, 0x0002, 0x0001, /*16868*/
  0x3920, 0x0020, 0x0002, 0x0001, /*16869*/
  0x3921, 0x0020, 0x0002, 0x0001, /*1686A*/
  0x3922, 0x0020, 0x0002, 0x0001, /*1686B*/
  0x3923, 0x0020, 0x0002, 0x0001, /*1686C*/
  0x3924, 0x0020, 0x0002, 0x0001, /*1686D*/
  0x3925, 0x0020, 0x0002, 0x0001, /*1686E*/
  0x3926, 0x0020, 0x0002, 0x0001, /*1686F*/
  0x3927, 0x0020, 0x0002, 0x0001, /*16870*/
  0x3928, 0x0020, 0x0002, 0x0001, /*16871*/
  0x3929, 0x0020, 0x0002, 0x0001, /*16872*/
  0x392A, 0x0020, 0x0002, 0x0001, /*16873*/
  0x392B, 0x0020, 0x0002, 0x0001, /*16874*/
  0x392C, 0x0020, 0x0002, 0x0001, /*16875*/
  0x392D, 0x0020, 0x0002, 0x0001, /*16876*/
  0x392E, 0x0020, 0x0002, 0x0001, /*16877*/
  0x392F, 0x0020, 0x0002, 0x0001, /*16878*/
  0x3930, 0x0020, 0x0002, 0x0001, /*16879*/
  0x3931, 0x0020, 0x0002, 0x0001, /*1687A*/
  0x3932, 0x0020, 0x0002, 0x0001, /*1687B*/
  0x3933, 0x0020, 0x0002, 0x0001, /*1687C*/
  0x3934, 0x0020, 0x0002, 0x0001, /*1687D*/
  0x3935, 0x0020, 0x0002, 0x0001, /*1687E*/
  0x3936, 0x0020, 0x0002, 0x0001, /*1687F*/
  0x3937, 0x0020, 0x0002, 0x0001, /*16880*/
  0x3938, 0x0020, 0x0002, 0x0001, /*16881*/
  0x3939, 0x0020, 0x0002, 0x0001, /*16882*/
  0x393A, 0x0020, 0x0002, 0x0001, /*16883*/
  0x393B, 0x0020, 0x0002, 0x0001, /*16884*/
  0x393C, 0x0020, 0x0002, 0x0001, /*16885*/
  0x393D, 0x0020, 0x0002, 0x0001, /*16886*/
  0x393E, 0x0020, 0x0002, 0x0001, /*16887*/
  0x393F, 0x0020, 0x0002, 0x0001, /*16888*/
  0x3940, 0x0020, 0x0002, 0x0001, /*16889*/
  0x3941, 0x0020, 0x0002, 0x0001, /*1688A*/
  0x3942, 0x0020, 0x0002, 0x0001, /*1688B*/
  0x3943, 0x0020, 0x0002, 0x0001, /*1688C*/
  0x3944, 0x0020, 0x0002, 0x0001, /*1688D*/
  0x3945, 0x0020, 0x0002, 0x0001, /*1688E*/
  0x3946, 0x0020, 0x0002, 0x0001, /*1688F*/
  0x3947, 0x0020, 0x0002, 0x0001, /*16890*/
  0x3948, 0x0020, 0x0002, 0x0001, /*16891*/
  0x3949, 0x0020, 0x0002, 0x0001, /*16892*/
  0x394A, 0x0020, 0x0002, 0x0001, /*16893*/
  0x394B, 0x0020, 0x0002, 0x0001, /*16894*/
  0x394C, 0x0020, 0x0002, 0x0001, /*16895*/
  0x394D, 0x0020, 0x0002, 0x0001, /*16896*/
  0x394E, 0x0020, 0x0002, 0x0001, /*16897*/
  0x394F, 0x0020, 0x0002, 0x0001, /*16898*/
  0x3950, 0x0020, 0x0002, 0x0001, /*16899*/
  0x3951, 0x0020, 0x0002, 0x0001, /*1689A*/
  0x3952, 0x0020, 0x0002, 0x0001, /*1689B*/
  0x3953, 0x0020, 0x0002, 0x0001, /*1689C*/
  0x3954, 0x0020, 0x0002, 0x0001, /*1689D*/
  0x3955, 0x0020, 0x0002, 0x0001, /*1689E*/
  0x3956, 0x0020, 0x0002, 0x0001, /*1689F*/
  0x3957, 0x0020, 0x0002, 0x0001, /*168A0*/
  0x3958, 0x0020, 0x0002, 0x0001, /*168A1*/
  0x3959, 0x0020, 0x0002, 0x0001, /*168A2*/
  0x395A, 0x0020, 0x0002, 0x0001, /*168A3*/
  0x395B, 0x0020, 0x0002, 0x0001, /*168A4*/
  0x395C, 0x0020, 0x0002, 0x0001, /*168A5*/
  0x395D, 0x0020, 0x0002, 0x0001, /*168A6*/
  0x395E, 0x0020, 0x0002, 0x0001, /*168A7*/
  0x395F, 0x0020, 0x0002, 0x0001, /*168A8*/
  0x3960, 0x0020, 0x0002, 0x0001, /*168A9*/
  0x3961, 0x0020, 0x0002, 0x0001, /*168AA*/
  0x3962, 0x0020, 0x0002, 0x0001, /*168AB*/
  0x3963, 0x0020, 0x0002, 0x0001, /*168AC*/
  0x3964, 0x0020, 0x0002, 0x0001, /*168AD*/
  0x3965, 0x0020, 0x0002, 0x0001, /*168AE*/
  0x3966, 0x0020, 0x0002, 0x0001, /*168AF*/
  0x3967, 0x0020, 0x0002, 0x0001, /*168B0*/
  0x3968, 0x0020, 0x0002, 0x0001, /*168B1*/
  0x3969, 0x0020, 0x0002, 0x0001, /*168B2*/
  0x396A, 0x0020, 0x0002, 0x0001, /*168B3*/
  0x396B, 0x0020, 0x0002, 0x0001, /*168B4*/
  0x396C, 0x0020, 0x0002, 0x0001, /*168B5*/
  0x396D, 0x0020, 0x0002, 0x0001, /*168B6*/
  0x396E, 0x0020, 0x0002, 0x0001, /*168B7*/
  0x396F, 0x0020, 0x0002, 0x0001, /*168B8*/
  0x3970, 0x0020, 0x0002, 0x0001, /*168B9*/
  0x3971, 0x0020, 0x0002, 0x0001, /*168BA*/
  0x3972, 0x0020, 0x0002, 0x0001, /*168BB*/
  0x3973, 0x0020, 0x0002, 0x0001, /*168BC*/
  0x3974, 0x0020, 0x0002, 0x0001, /*168BD*/
  0x3975, 0x0020, 0x0002, 0x0001, /*168BE*/
  0x3976, 0x0020, 0x0002, 0x0001, /*168BF*/
  0x3977, 0x0020, 0x0002, 0x0001, /*168C0*/
  0x3978, 0x0020, 0x0002, 0x0001, /*168C1*/
  0x3979, 0x0020, 0x0002, 0x0001, /*168C2*/
  0x397A, 0x0020, 0x0002, 0x0001, /*168C3*/
  0x397B, 0x0020, 0x0002, 0x0001, /*168C4*/
  0x397C, 0x0020, 0x0002, 0x0001, /*168C5*/
  0x397D, 0x0020, 0x0002, 0x0001, /*168C6*/
  0x397E, 0x0020, 0x0002, 0x0001, /*168C7*/
  0x397F, 0x0020, 0x0002, 0x0001, /*168C8*/
  0x3980, 0x0020, 0x0002, 0x0001, /*168C9*/
  0x3981, 0x0020, 0x0002, 0x0001, /*168CA*/
  0x3982, 0x0020, 0x0002, 0x0001, /*168CB*/
  0x3983, 0x0020, 0x0002, 0x0001, /*168CC*/
  0x3984, 0x0020, 0x0002, 0x0001, /*168CD*/
  0x3985, 0x0020, 0x0002, 0x0001, /*168CE*/
  0x3986, 0x0020, 0x0002, 0x0001, /*168CF*/
  0x3987, 0x0020, 0x0002, 0x0001, /*168D0*/
  0x3988, 0x0020, 0x0002, 0x0001, /*168D1*/
  0x3989, 0x0020, 0x0002, 0x0001, /*168D2*/
  0x398A, 0x0020, 0x0002, 0x0001, /*168D3*/
  0x398B, 0x0020, 0x0002, 0x0001, /*168D4*/
  0x398C, 0x0020, 0x0002, 0x0001, /*168D5*/
  0x398D, 0x0020, 0x0002, 0x0001, /*168D6*/
  0x398E, 0x0020, 0x0002, 0x0001, /*168D7*/
  0x398F, 0x0020, 0x0002, 0x0001, /*168D8*/
  0x3990, 0x0020, 0x0002, 0x0001, /*168D9*/
  0x3991, 0x0020, 0x0002, 0x0001, /*168DA*/
  0x3992, 0x0020, 0x0002, 0x0001, /*168DB*/
  0x3993, 0x0020, 0x0002, 0x0001, /*168DC*/
  0x3994, 0x0020, 0x0002, 0x0001, /*168DD*/
  0x3995, 0x0020, 0x0002, 0x0001, /*168DE*/
  0x3996, 0x0020, 0x0002, 0x0001, /*168DF*/
  0x3997, 0x0020, 0x0002, 0x0001, /*168E0*/
  0x3998, 0x0020, 0x0002, 0x0001, /*168E1*/
  0x3999, 0x0020, 0x0002, 0x0001, /*168E2*/
  0x399A, 0x0020, 0x0002, 0x0001, /*168E3*/
  0x399B, 0x0020, 0x0002, 0x0001, /*168E4*/
  0x399C, 0x0020, 0x0002, 0x0001, /*168E5*/
  0x399D, 0x0020, 0x0002, 0x0001, /*168E6*/
  0x399E, 0x0020, 0x0002, 0x0001, /*168E7*/
  0x399F, 0x0020, 0x0002, 0x0001, /*168E8*/
  0x39A0, 0x0020, 0x0002, 0x0001, /*168E9*/
  0x39A1, 0x0020, 0x0002, 0x0001, /*168EA*/
  0x39A2, 0x0020, 0x0002, 0x0001, /*168EB*/
  0x39A3, 0x0020, 0x0002, 0x0001, /*168EC*/
  0x39A4, 0x0020, 0x0002, 0x0001, /*168ED*/
  0x39A5, 0x0020, 0x0002, 0x0001, /*168EE*/
  0x39A6, 0x0020, 0x0002, 0x0001, /*168EF*/
  0x39A7, 0x0020, 0x0002, 0x0001, /*168F0*/
  0x39A8, 0x0020, 0x0002, 0x0001, /*168F1*/
  0x39A9, 0x0020, 0x0002, 0x0001, /*168F2*/
  0x39AA, 0x0020, 0x0002, 0x0001, /*168F3*/
  0x39AB, 0x0020, 0x0002, 0x0001, /*168F4*/
  0x39AC, 0x0020, 0x0002, 0x0001, /*168F5*/
  0x39AD, 0x0020, 0x0002, 0x0001, /*168F6*/
  0x39AE, 0x0020, 0x0002, 0x0001, /*168F7*/
  0x39AF, 0x0020, 0x0002, 0x0001, /*168F8*/
  0x39B0, 0x0020, 0x0002, 0x0001, /*168F9*/
  0x39B1, 0x0020, 0x0002, 0x0001, /*168FA*/
  0x39B2, 0x0020, 0x0002, 0x0001, /*168FB*/
  0x39B3, 0x0020, 0x0002, 0x0001, /*168FC*/
  0x39B4, 0x0020, 0x0002, 0x0001, /*168FD*/
  0x39B5, 0x0020, 0x0002, 0x0001, /*168FE*/
  0x39B6, 0x0020, 0x0002, 0x0001  /*168FF*/
};

uint16 uca900_p169[]= {
  0x39B7, 0x0020, 0x0002, 0x0001, /*16900*/
  0x39B8, 0x0020, 0x0002, 0x0001, /*16901*/
  0x39B9, 0x0020, 0x0002, 0x0001, /*16902*/
  0x39BA, 0x0020, 0x0002, 0x0001, /*16903*/
  0x39BB, 0x0020, 0x0002, 0x0001, /*16904*/
  0x39BC, 0x0020, 0x0002, 0x0001, /*16905*/
  0x39BD, 0x0020, 0x0002, 0x0001, /*16906*/
  0x39BE, 0x0020, 0x0002, 0x0001, /*16907*/
  0x39BF, 0x0020, 0x0002, 0x0001, /*16908*/
  0x39C0, 0x0020, 0x0002, 0x0001, /*16909*/
  0x39C1, 0x0020, 0x0002, 0x0001, /*1690A*/
  0x39C2, 0x0020, 0x0002, 0x0001, /*1690B*/
  0x39C3, 0x0020, 0x0002, 0x0001, /*1690C*/
  0x39C4, 0x0020, 0x0002, 0x0001, /*1690D*/
  0x39C5, 0x0020, 0x0002, 0x0001, /*1690E*/
  0x39C6, 0x0020, 0x0002, 0x0001, /*1690F*/
  0x39C7, 0x0020, 0x0002, 0x0001, /*16910*/
  0x39C8, 0x0020, 0x0002, 0x0001, /*16911*/
  0x39C9, 0x0020, 0x0002, 0x0001, /*16912*/
  0x39CA, 0x0020, 0x0002, 0x0001, /*16913*/
  0x39CB, 0x0020, 0x0002, 0x0001, /*16914*/
  0x39CC, 0x0020, 0x0002, 0x0001, /*16915*/
  0x39CD, 0x0020, 0x0002, 0x0001, /*16916*/
  0x39CE, 0x0020, 0x0002, 0x0001, /*16917*/
  0x39CF, 0x0020, 0x0002, 0x0001, /*16918*/
  0x39D0, 0x0020, 0x0002, 0x0001, /*16919*/
  0x39D1, 0x0020, 0x0002, 0x0001, /*1691A*/
  0x39D2, 0x0020, 0x0002, 0x0001, /*1691B*/
  0x39D3, 0x0020, 0x0002, 0x0001, /*1691C*/
  0x39D4, 0x0020, 0x0002, 0x0001, /*1691D*/
  0x39D5, 0x0020, 0x0002, 0x0001, /*1691E*/
  0x39D6, 0x0020, 0x0002, 0x0001, /*1691F*/
  0x39D7, 0x0020, 0x0002, 0x0001, /*16920*/
  0x39D8, 0x0020, 0x0002, 0x0001, /*16921*/
  0x39D9, 0x0020, 0x0002, 0x0001, /*16922*/
  0x39DA, 0x0020, 0x0002, 0x0001, /*16923*/
  0x39DB, 0x0020, 0x0002, 0x0001, /*16924*/
  0x39DC, 0x0020, 0x0002, 0x0001, /*16925*/
  0x39DD, 0x0020, 0x0002, 0x0001, /*16926*/
  0x39DE, 0x0020, 0x0002, 0x0001, /*16927*/
  0x39DF, 0x0020, 0x0002, 0x0001, /*16928*/
  0x39E0, 0x0020, 0x0002, 0x0001, /*16929*/
  0x39E1, 0x0020, 0x0002, 0x0001, /*1692A*/
  0x39E2, 0x0020, 0x0002, 0x0001, /*1692B*/
  0x39E3, 0x0020, 0x0002, 0x0001, /*1692C*/
  0x39E4, 0x0020, 0x0002, 0x0001, /*1692D*/
  0x39E5, 0x0020, 0x0002, 0x0001, /*1692E*/
  0x39E6, 0x0020, 0x0002, 0x0001, /*1692F*/
  0x39E7, 0x0020, 0x0002, 0x0001, /*16930*/
  0x39E8, 0x0020, 0x0002, 0x0001, /*16931*/
  0x39E9, 0x0020, 0x0002, 0x0001, /*16932*/
  0x39EA, 0x0020, 0x0002, 0x0001, /*16933*/
  0x39EB, 0x0020, 0x0002, 0x0001, /*16934*/
  0x39EC, 0x0020, 0x0002, 0x0001, /*16935*/
  0x39ED, 0x0020, 0x0002, 0x0001, /*16936*/
  0x39EE, 0x0020, 0x0002, 0x0001, /*16937*/
  0x39EF, 0x0020, 0x0002, 0x0001, /*16938*/
  0x39F0, 0x0020, 0x0002, 0x0001, /*16939*/
  0x39F1, 0x0020, 0x0002, 0x0001, /*1693A*/
  0x39F2, 0x0020, 0x0002, 0x0001, /*1693B*/
  0x39F3, 0x0020, 0x0002, 0x0001, /*1693C*/
  0x39F4, 0x0020, 0x0002, 0x0001, /*1693D*/
  0x39F5, 0x0020, 0x0002, 0x0001, /*1693E*/
  0x39F6, 0x0020, 0x0002, 0x0001, /*1693F*/
  0x39F7, 0x0020, 0x0002, 0x0001, /*16940*/
  0x39F8, 0x0020, 0x0002, 0x0001, /*16941*/
  0x39F9, 0x0020, 0x0002, 0x0001, /*16942*/
  0x39FA, 0x0020, 0x0002, 0x0001, /*16943*/
  0x39FB, 0x0020, 0x0002, 0x0001, /*16944*/
  0x39FC, 0x0020, 0x0002, 0x0001, /*16945*/
  0x39FD, 0x0020, 0x0002, 0x0001, /*16946*/
  0x39FE, 0x0020, 0x0002, 0x0001, /*16947*/
  0x39FF, 0x0020, 0x0002, 0x0001, /*16948*/
  0x3A00, 0x0020, 0x0002, 0x0001, /*16949*/
  0x3A01, 0x0020, 0x0002, 0x0001, /*1694A*/
  0x3A02, 0x0020, 0x0002, 0x0001, /*1694B*/
  0x3A03, 0x0020, 0x0002, 0x0001, /*1694C*/
  0x3A04, 0x0020, 0x0002, 0x0001, /*1694D*/
  0x3A05, 0x0020, 0x0002, 0x0001, /*1694E*/
  0x3A06, 0x0020, 0x0002, 0x0001, /*1694F*/
  0x3A07, 0x0020, 0x0002, 0x0001, /*16950*/
  0x3A08, 0x0020, 0x0002, 0x0001, /*16951*/
  0x3A09, 0x0020, 0x0002, 0x0001, /*16952*/
  0x3A0A, 0x0020, 0x0002, 0x0001, /*16953*/
  0x3A0B, 0x0020, 0x0002, 0x0001, /*16954*/
  0x3A0C, 0x0020, 0x0002, 0x0001, /*16955*/
  0x3A0D, 0x0020, 0x0002, 0x0001, /*16956*/
  0x3A0E, 0x0020, 0x0002, 0x0001, /*16957*/
  0x3A0F, 0x0020, 0x0002, 0x0001, /*16958*/
  0x3A10, 0x0020, 0x0002, 0x0001, /*16959*/
  0x3A11, 0x0020, 0x0002, 0x0001, /*1695A*/
  0x3A12, 0x0020, 0x0002, 0x0001, /*1695B*/
  0x3A13, 0x0020, 0x0002, 0x0001, /*1695C*/
  0x3A14, 0x0020, 0x0002, 0x0001, /*1695D*/
  0x3A15, 0x0020, 0x0002, 0x0001, /*1695E*/
  0x3A16, 0x0020, 0x0002, 0x0001, /*1695F*/
  0x3A17, 0x0020, 0x0002, 0x0001, /*16960*/
  0x3A18, 0x0020, 0x0002, 0x0001, /*16961*/
  0x3A19, 0x0020, 0x0002, 0x0001, /*16962*/
  0x3A1A, 0x0020, 0x0002, 0x0001, /*16963*/
  0x3A1B, 0x0020, 0x0002, 0x0001, /*16964*/
  0x3A1C, 0x0020, 0x0002, 0x0001, /*16965*/
  0x3A1D, 0x0020, 0x0002, 0x0001, /*16966*/
  0x3A1E, 0x0020, 0x0002, 0x0001, /*16967*/
  0x3A1F, 0x0020, 0x0002, 0x0001, /*16968*/
  0x3A20, 0x0020, 0x0002, 0x0001, /*16969*/
  0x3A21, 0x0020, 0x0002, 0x0001, /*1696A*/
  0x3A22, 0x0020, 0x0002, 0x0001, /*1696B*/
  0x3A23, 0x0020, 0x0002, 0x0001, /*1696C*/
  0x3A24, 0x0020, 0x0002, 0x0001, /*1696D*/
  0x3A25, 0x0020, 0x0002, 0x0001, /*1696E*/
  0x3A26, 0x0020, 0x0002, 0x0001, /*1696F*/
  0x3A27, 0x0020, 0x0002, 0x0001, /*16970*/
  0x3A28, 0x0020, 0x0002, 0x0001, /*16971*/
  0x3A29, 0x0020, 0x0002, 0x0001, /*16972*/
  0x3A2A, 0x0020, 0x0002, 0x0001, /*16973*/
  0x3A2B, 0x0020, 0x0002, 0x0001, /*16974*/
  0x3A2C, 0x0020, 0x0002, 0x0001, /*16975*/
  0x3A2D, 0x0020, 0x0002, 0x0001, /*16976*/
  0x3A2E, 0x0020, 0x0002, 0x0001, /*16977*/
  0x3A2F, 0x0020, 0x0002, 0x0001, /*16978*/
  0x3A30, 0x0020, 0x0002, 0x0001, /*16979*/
  0x3A31, 0x0020, 0x0002, 0x0001, /*1697A*/
  0x3A32, 0x0020, 0x0002, 0x0001, /*1697B*/
  0x3A33, 0x0020, 0x0002, 0x0001, /*1697C*/
  0x3A34, 0x0020, 0x0002, 0x0001, /*1697D*/
  0x3A35, 0x0020, 0x0002, 0x0001, /*1697E*/
  0x3A36, 0x0020, 0x0002, 0x0001, /*1697F*/
  0x3A37, 0x0020, 0x0002, 0x0001, /*16980*/
  0x3A38, 0x0020, 0x0002, 0x0001, /*16981*/
  0x3A39, 0x0020, 0x0002, 0x0001, /*16982*/
  0x3A3A, 0x0020, 0x0002, 0x0001, /*16983*/
  0x3A3B, 0x0020, 0x0002, 0x0001, /*16984*/
  0x3A3C, 0x0020, 0x0002, 0x0001, /*16985*/
  0x3A3D, 0x0020, 0x0002, 0x0001, /*16986*/
  0x3A3E, 0x0020, 0x0002, 0x0001, /*16987*/
  0x3A3F, 0x0020, 0x0002, 0x0001, /*16988*/
  0x3A40, 0x0020, 0x0002, 0x0001, /*16989*/
  0x3A41, 0x0020, 0x0002, 0x0001, /*1698A*/
  0x3A42, 0x0020, 0x0002, 0x0001, /*1698B*/
  0x3A43, 0x0020, 0x0002, 0x0001, /*1698C*/
  0x3A44, 0x0020, 0x0002, 0x0001, /*1698D*/
  0x3A45, 0x0020, 0x0002, 0x0001, /*1698E*/
  0x3A46, 0x0020, 0x0002, 0x0001, /*1698F*/
  0x3A47, 0x0020, 0x0002, 0x0001, /*16990*/
  0x3A48, 0x0020, 0x0002, 0x0001, /*16991*/
  0x3A49, 0x0020, 0x0002, 0x0001, /*16992*/
  0x3A4A, 0x0020, 0x0002, 0x0001, /*16993*/
  0x3A4B, 0x0020, 0x0002, 0x0001, /*16994*/
  0x3A4C, 0x0020, 0x0002, 0x0001, /*16995*/
  0x3A4D, 0x0020, 0x0002, 0x0001, /*16996*/
  0x3A4E, 0x0020, 0x0002, 0x0001, /*16997*/
  0x3A4F, 0x0020, 0x0002, 0x0001, /*16998*/
  0x3A50, 0x0020, 0x0002, 0x0001, /*16999*/
  0x3A51, 0x0020, 0x0002, 0x0001, /*1699A*/
  0x3A52, 0x0020, 0x0002, 0x0001, /*1699B*/
  0x3A53, 0x0020, 0x0002, 0x0001, /*1699C*/
  0x3A54, 0x0020, 0x0002, 0x0001, /*1699D*/
  0x3A55, 0x0020, 0x0002, 0x0001, /*1699E*/
  0x3A56, 0x0020, 0x0002, 0x0001, /*1699F*/
  0x3A57, 0x0020, 0x0002, 0x0001, /*169A0*/
  0x3A58, 0x0020, 0x0002, 0x0001, /*169A1*/
  0x3A59, 0x0020, 0x0002, 0x0001, /*169A2*/
  0x3A5A, 0x0020, 0x0002, 0x0001, /*169A3*/
  0x3A5B, 0x0020, 0x0002, 0x0001, /*169A4*/
  0x3A5C, 0x0020, 0x0002, 0x0001, /*169A5*/
  0x3A5D, 0x0020, 0x0002, 0x0001, /*169A6*/
  0x3A5E, 0x0020, 0x0002, 0x0001, /*169A7*/
  0x3A5F, 0x0020, 0x0002, 0x0001, /*169A8*/
  0x3A60, 0x0020, 0x0002, 0x0001, /*169A9*/
  0x3A61, 0x0020, 0x0002, 0x0001, /*169AA*/
  0x3A62, 0x0020, 0x0002, 0x0001, /*169AB*/
  0x3A63, 0x0020, 0x0002, 0x0001, /*169AC*/
  0x3A64, 0x0020, 0x0002, 0x0001, /*169AD*/
  0x3A65, 0x0020, 0x0002, 0x0001, /*169AE*/
  0x3A66, 0x0020, 0x0002, 0x0001, /*169AF*/
  0x3A67, 0x0020, 0x0002, 0x0001, /*169B0*/
  0x3A68, 0x0020, 0x0002, 0x0001, /*169B1*/
  0x3A69, 0x0020, 0x0002, 0x0001, /*169B2*/
  0x3A6A, 0x0020, 0x0002, 0x0001, /*169B3*/
  0x3A6B, 0x0020, 0x0002, 0x0001, /*169B4*/
  0x3A6C, 0x0020, 0x0002, 0x0001, /*169B5*/
  0x3A6D, 0x0020, 0x0002, 0x0001, /*169B6*/
  0x3A6E, 0x0020, 0x0002, 0x0001, /*169B7*/
  0x3A6F, 0x0020, 0x0002, 0x0001, /*169B8*/
  0x3A70, 0x0020, 0x0002, 0x0001, /*169B9*/
  0x3A71, 0x0020, 0x0002, 0x0001, /*169BA*/
  0x3A72, 0x0020, 0x0002, 0x0001, /*169BB*/
  0x3A73, 0x0020, 0x0002, 0x0001, /*169BC*/
  0x3A74, 0x0020, 0x0002, 0x0001, /*169BD*/
  0x3A75, 0x0020, 0x0002, 0x0001, /*169BE*/
  0x3A76, 0x0020, 0x0002, 0x0001, /*169BF*/
  0x3A77, 0x0020, 0x0002, 0x0001, /*169C0*/
  0x3A78, 0x0020, 0x0002, 0x0001, /*169C1*/
  0x3A79, 0x0020, 0x0002, 0x0001, /*169C2*/
  0x3A7A, 0x0020, 0x0002, 0x0001, /*169C3*/
  0x3A7B, 0x0020, 0x0002, 0x0001, /*169C4*/
  0x3A7C, 0x0020, 0x0002, 0x0001, /*169C5*/
  0x3A7D, 0x0020, 0x0002, 0x0001, /*169C6*/
  0x3A7E, 0x0020, 0x0002, 0x0001, /*169C7*/
  0x3A7F, 0x0020, 0x0002, 0x0001, /*169C8*/
  0x3A80, 0x0020, 0x0002, 0x0001, /*169C9*/
  0x3A81, 0x0020, 0x0002, 0x0001, /*169CA*/
  0x3A82, 0x0020, 0x0002, 0x0001, /*169CB*/
  0x3A83, 0x0020, 0x0002, 0x0001, /*169CC*/
  0x3A84, 0x0020, 0x0002, 0x0001, /*169CD*/
  0x3A85, 0x0020, 0x0002, 0x0001, /*169CE*/
  0x3A86, 0x0020, 0x0002, 0x0001, /*169CF*/
  0x3A87, 0x0020, 0x0002, 0x0001, /*169D0*/
  0x3A88, 0x0020, 0x0002, 0x0001, /*169D1*/
  0x3A89, 0x0020, 0x0002, 0x0001, /*169D2*/
  0x3A8A, 0x0020, 0x0002, 0x0001, /*169D3*/
  0x3A8B, 0x0020, 0x0002, 0x0001, /*169D4*/
  0x3A8C, 0x0020, 0x0002, 0x0001, /*169D5*/
  0x3A8D, 0x0020, 0x0002, 0x0001, /*169D6*/
  0x3A8E, 0x0020, 0x0002, 0x0001, /*169D7*/
  0x3A8F, 0x0020, 0x0002, 0x0001, /*169D8*/
  0x3A90, 0x0020, 0x0002, 0x0001, /*169D9*/
  0x3A91, 0x0020, 0x0002, 0x0001, /*169DA*/
  0x3A92, 0x0020, 0x0002, 0x0001, /*169DB*/
  0x3A93, 0x0020, 0x0002, 0x0001, /*169DC*/
  0x3A94, 0x0020, 0x0002, 0x0001, /*169DD*/
  0x3A95, 0x0020, 0x0002, 0x0001, /*169DE*/
  0x3A96, 0x0020, 0x0002, 0x0001, /*169DF*/
  0x3A97, 0x0020, 0x0002, 0x0001, /*169E0*/
  0x3A98, 0x0020, 0x0002, 0x0001, /*169E1*/
  0x3A99, 0x0020, 0x0002, 0x0001, /*169E2*/
  0x3A9A, 0x0020, 0x0002, 0x0001, /*169E3*/
  0x3A9B, 0x0020, 0x0002, 0x0001, /*169E4*/
  0x3A9C, 0x0020, 0x0002, 0x0001, /*169E5*/
  0x3A9D, 0x0020, 0x0002, 0x0001, /*169E6*/
  0x3A9E, 0x0020, 0x0002, 0x0001, /*169E7*/
  0x3A9F, 0x0020, 0x0002, 0x0001, /*169E8*/
  0x3AA0, 0x0020, 0x0002, 0x0001, /*169E9*/
  0x3AA1, 0x0020, 0x0002, 0x0001, /*169EA*/
  0x3AA2, 0x0020, 0x0002, 0x0001, /*169EB*/
  0x3AA3, 0x0020, 0x0002, 0x0001, /*169EC*/
  0x3AA4, 0x0020, 0x0002, 0x0001, /*169ED*/
  0x3AA5, 0x0020, 0x0002, 0x0001, /*169EE*/
  0x3AA6, 0x0020, 0x0002, 0x0001, /*169EF*/
  0x3AA7, 0x0020, 0x0002, 0x0001, /*169F0*/
  0x3AA8, 0x0020, 0x0002, 0x0001, /*169F1*/
  0x3AA9, 0x0020, 0x0002, 0x0001, /*169F2*/
  0x3AAA, 0x0020, 0x0002, 0x0001, /*169F3*/
  0x3AAB, 0x0020, 0x0002, 0x0001, /*169F4*/
  0x3AAC, 0x0020, 0x0002, 0x0001, /*169F5*/
  0x3AAD, 0x0020, 0x0002, 0x0001, /*169F6*/
  0x3AAE, 0x0020, 0x0002, 0x0001, /*169F7*/
  0x3AAF, 0x0020, 0x0002, 0x0001, /*169F8*/
  0x3AB0, 0x0020, 0x0002, 0x0001, /*169F9*/
  0x3AB1, 0x0020, 0x0002, 0x0001, /*169FA*/
  0x3AB2, 0x0020, 0x0002, 0x0001, /*169FB*/
  0x3AB3, 0x0020, 0x0002, 0x0001, /*169FC*/
  0x3AB4, 0x0020, 0x0002, 0x0001, /*169FD*/
  0x3AB5, 0x0020, 0x0002, 0x0001, /*169FE*/
  0x3AB6, 0x0020, 0x0002, 0x0001  /*169FF*/
};

uint16 uca900_p16A[]= {
  0x3AB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A00*/
  0x3AB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A01*/
  0x3AB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A02*/
  0x3ABA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A03*/
  0x3ABB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A04*/
  0x3ABC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A05*/
  0x3ABD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A06*/
  0x3ABE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A07*/
  0x3ABF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A08*/
  0x3AC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A09*/
  0x3AC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A0A*/
  0x3AC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A0B*/
  0x3AC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A0C*/
  0x3AC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A0D*/
  0x3AC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A0E*/
  0x3AC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A0F*/
  0x3AC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A10*/
  0x3AC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A11*/
  0x3AC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A12*/
  0x3ACA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A13*/
  0x3ACB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A14*/
  0x3ACC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A15*/
  0x3ACD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A16*/
  0x3ACE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A17*/
  0x3ACF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A18*/
  0x3AD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A19*/
  0x3AD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A1A*/
  0x3AD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A1B*/
  0x3AD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A1C*/
  0x3AD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A1D*/
  0x3AD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A1E*/
  0x3AD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A1F*/
  0x3AD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A20*/
  0x3AD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A21*/
  0x3AD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A22*/
  0x3ADA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A23*/
  0x3ADB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A24*/
  0x3ADC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A25*/
  0x3ADD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A26*/
  0x3ADE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A27*/
  0x3ADF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A28*/
  0x3AE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A29*/
  0x3AE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A2A*/
  0x3AE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A2B*/
  0x3AE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A2C*/
  0x3AE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A2D*/
  0x3AE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A2E*/
  0x3AE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A2F*/
  0x3AE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A30*/
  0x3AE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A31*/
  0x3AE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A32*/
  0x3AEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A33*/
  0x3AEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A34*/
  0x3AEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A35*/
  0x3AED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A36*/
  0x3AEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A37*/
  0x3AEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A38*/
  0xFBC2, 0x0020, 0x0002, 0xEA39, 0x0000, 0x0000, 0x0002, /*16A39*/
  0xFBC2, 0x0020, 0x0002, 0xEA3A, 0x0000, 0x0000, 0x0002, /*16A3A*/
  0xFBC2, 0x0020, 0x0002, 0xEA3B, 0x0000, 0x0000, 0x0002, /*16A3B*/
  0xFBC2, 0x0020, 0x0002, 0xEA3C, 0x0000, 0x0000, 0x0002, /*16A3C*/
  0xFBC2, 0x0020, 0x0002, 0xEA3D, 0x0000, 0x0000, 0x0002, /*16A3D*/
  0xFBC2, 0x0020, 0x0002, 0xEA3E, 0x0000, 0x0000, 0x0002, /*16A3E*/
  0xFBC2, 0x0020, 0x0002, 0xEA3F, 0x0000, 0x0000, 0x0002, /*16A3F*/
  0x45C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A40*/
  0x45C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A41*/
  0x45CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A42*/
  0x45CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A43*/
  0x45CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A44*/
  0x45CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A45*/
  0x45CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A46*/
  0x45CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A47*/
  0x45D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A48*/
  0x45D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A49*/
  0x45D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A4A*/
  0x45D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A4B*/
  0x45D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A4C*/
  0x45D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A4D*/
  0x45D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A4E*/
  0x45D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A4F*/
  0x45D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A50*/
  0x45D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A51*/
  0x45DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A52*/
  0x45DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A53*/
  0x45DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A54*/
  0x45DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A55*/
  0x45DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A56*/
  0x45DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A57*/
  0x45E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A58*/
  0x45E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A59*/
  0x45E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A5A*/
  0x45E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A5B*/
  0x45E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A5C*/
  0x45E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A5D*/
  0x45E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A5E*/
  0xFBC2, 0x0020, 0x0002, 0xEA5F, 0x0000, 0x0000, 0x0002, /*16A5F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A60*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A61*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A62*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A63*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A64*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A65*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A66*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A67*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A68*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A69*/
  0xFBC2, 0x0020, 0x0002, 0xEA6A, 0x0000, 0x0000, 0x0002, /*16A6A*/
  0xFBC2, 0x0020, 0x0002, 0xEA6B, 0x0000, 0x0000, 0x0002, /*16A6B*/
  0xFBC2, 0x0020, 0x0002, 0xEA6C, 0x0000, 0x0000, 0x0002, /*16A6C*/
  0xFBC2, 0x0020, 0x0002, 0xEA6D, 0x0000, 0x0000, 0x0002, /*16A6D*/
  0x02C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A6E*/
  0x02C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16A6F*/
  0xFBC2, 0x0020, 0x0002, 0xEA70, 0x0000, 0x0000, 0x0002, /*16A70*/
  0xFBC2, 0x0020, 0x0002, 0xEA71, 0x0000, 0x0000, 0x0002, /*16A71*/
  0xFBC2, 0x0020, 0x0002, 0xEA72, 0x0000, 0x0000, 0x0002, /*16A72*/
  0xFBC2, 0x0020, 0x0002, 0xEA73, 0x0000, 0x0000, 0x0002, /*16A73*/
  0xFBC2, 0x0020, 0x0002, 0xEA74, 0x0000, 0x0000, 0x0002, /*16A74*/
  0xFBC2, 0x0020, 0x0002, 0xEA75, 0x0000, 0x0000, 0x0002, /*16A75*/
  0xFBC2, 0x0020, 0x0002, 0xEA76, 0x0000, 0x0000, 0x0002, /*16A76*/
  0xFBC2, 0x0020, 0x0002, 0xEA77, 0x0000, 0x0000, 0x0002, /*16A77*/
  0xFBC2, 0x0020, 0x0002, 0xEA78, 0x0000, 0x0000, 0x0002, /*16A78*/
  0xFBC2, 0x0020, 0x0002, 0xEA79, 0x0000, 0x0000, 0x0002, /*16A79*/
  0xFBC2, 0x0020, 0x0002, 0xEA7A, 0x0000, 0x0000, 0x0002, /*16A7A*/
  0xFBC2, 0x0020, 0x0002, 0xEA7B, 0x0000, 0x0000, 0x0002, /*16A7B*/
  0xFBC2, 0x0020, 0x0002, 0xEA7C, 0x0000, 0x0000, 0x0002, /*16A7C*/
  0xFBC2, 0x0020, 0x0002, 0xEA7D, 0x0000, 0x0000, 0x0002, /*16A7D*/
  0xFBC2, 0x0020, 0x0002, 0xEA7E, 0x0000, 0x0000, 0x0002, /*16A7E*/
  0xFBC2, 0x0020, 0x0002, 0xEA7F, 0x0000, 0x0000, 0x0002, /*16A7F*/
  0xFBC2, 0x0020, 0x0002, 0xEA80, 0x0000, 0x0000, 0x0002, /*16A80*/
  0xFBC2, 0x0020, 0x0002, 0xEA81, 0x0000, 0x0000, 0x0002, /*16A81*/
  0xFBC2, 0x0020, 0x0002, 0xEA82, 0x0000, 0x0000, 0x0002, /*16A82*/
  0xFBC2, 0x0020, 0x0002, 0xEA83, 0x0000, 0x0000, 0x0002, /*16A83*/
  0xFBC2, 0x0020, 0x0002, 0xEA84, 0x0000, 0x0000, 0x0002, /*16A84*/
  0xFBC2, 0x0020, 0x0002, 0xEA85, 0x0000, 0x0000, 0x0002, /*16A85*/
  0xFBC2, 0x0020, 0x0002, 0xEA86, 0x0000, 0x0000, 0x0002, /*16A86*/
  0xFBC2, 0x0020, 0x0002, 0xEA87, 0x0000, 0x0000, 0x0002, /*16A87*/
  0xFBC2, 0x0020, 0x0002, 0xEA88, 0x0000, 0x0000, 0x0002, /*16A88*/
  0xFBC2, 0x0020, 0x0002, 0xEA89, 0x0000, 0x0000, 0x0002, /*16A89*/
  0xFBC2, 0x0020, 0x0002, 0xEA8A, 0x0000, 0x0000, 0x0002, /*16A8A*/
  0xFBC2, 0x0020, 0x0002, 0xEA8B, 0x0000, 0x0000, 0x0002, /*16A8B*/
  0xFBC2, 0x0020, 0x0002, 0xEA8C, 0x0000, 0x0000, 0x0002, /*16A8C*/
  0xFBC2, 0x0020, 0x0002, 0xEA8D, 0x0000, 0x0000, 0x0002, /*16A8D*/
  0xFBC2, 0x0020, 0x0002, 0xEA8E, 0x0000, 0x0000, 0x0002, /*16A8E*/
  0xFBC2, 0x0020, 0x0002, 0xEA8F, 0x0000, 0x0000, 0x0002, /*16A8F*/
  0xFBC2, 0x0020, 0x0002, 0xEA90, 0x0000, 0x0000, 0x0002, /*16A90*/
  0xFBC2, 0x0020, 0x0002, 0xEA91, 0x0000, 0x0000, 0x0002, /*16A91*/
  0xFBC2, 0x0020, 0x0002, 0xEA92, 0x0000, 0x0000, 0x0002, /*16A92*/
  0xFBC2, 0x0020, 0x0002, 0xEA93, 0x0000, 0x0000, 0x0002, /*16A93*/
  0xFBC2, 0x0020, 0x0002, 0xEA94, 0x0000, 0x0000, 0x0002, /*16A94*/
  0xFBC2, 0x0020, 0x0002, 0xEA95, 0x0000, 0x0000, 0x0002, /*16A95*/
  0xFBC2, 0x0020, 0x0002, 0xEA96, 0x0000, 0x0000, 0x0002, /*16A96*/
  0xFBC2, 0x0020, 0x0002, 0xEA97, 0x0000, 0x0000, 0x0002, /*16A97*/
  0xFBC2, 0x0020, 0x0002, 0xEA98, 0x0000, 0x0000, 0x0002, /*16A98*/
  0xFBC2, 0x0020, 0x0002, 0xEA99, 0x0000, 0x0000, 0x0002, /*16A99*/
  0xFBC2, 0x0020, 0x0002, 0xEA9A, 0x0000, 0x0000, 0x0002, /*16A9A*/
  0xFBC2, 0x0020, 0x0002, 0xEA9B, 0x0000, 0x0000, 0x0002, /*16A9B*/
  0xFBC2, 0x0020, 0x0002, 0xEA9C, 0x0000, 0x0000, 0x0002, /*16A9C*/
  0xFBC2, 0x0020, 0x0002, 0xEA9D, 0x0000, 0x0000, 0x0002, /*16A9D*/
  0xFBC2, 0x0020, 0x0002, 0xEA9E, 0x0000, 0x0000, 0x0002, /*16A9E*/
  0xFBC2, 0x0020, 0x0002, 0xEA9F, 0x0000, 0x0000, 0x0002, /*16A9F*/
  0xFBC2, 0x0020, 0x0002, 0xEAA0, 0x0000, 0x0000, 0x0002, /*16AA0*/
  0xFBC2, 0x0020, 0x0002, 0xEAA1, 0x0000, 0x0000, 0x0002, /*16AA1*/
  0xFBC2, 0x0020, 0x0002, 0xEAA2, 0x0000, 0x0000, 0x0002, /*16AA2*/
  0xFBC2, 0x0020, 0x0002, 0xEAA3, 0x0000, 0x0000, 0x0002, /*16AA3*/
  0xFBC2, 0x0020, 0x0002, 0xEAA4, 0x0000, 0x0000, 0x0002, /*16AA4*/
  0xFBC2, 0x0020, 0x0002, 0xEAA5, 0x0000, 0x0000, 0x0002, /*16AA5*/
  0xFBC2, 0x0020, 0x0002, 0xEAA6, 0x0000, 0x0000, 0x0002, /*16AA6*/
  0xFBC2, 0x0020, 0x0002, 0xEAA7, 0x0000, 0x0000, 0x0002, /*16AA7*/
  0xFBC2, 0x0020, 0x0002, 0xEAA8, 0x0000, 0x0000, 0x0002, /*16AA8*/
  0xFBC2, 0x0020, 0x0002, 0xEAA9, 0x0000, 0x0000, 0x0002, /*16AA9*/
  0xFBC2, 0x0020, 0x0002, 0xEAAA, 0x0000, 0x0000, 0x0002, /*16AAA*/
  0xFBC2, 0x0020, 0x0002, 0xEAAB, 0x0000, 0x0000, 0x0002, /*16AAB*/
  0xFBC2, 0x0020, 0x0002, 0xEAAC, 0x0000, 0x0000, 0x0002, /*16AAC*/
  0xFBC2, 0x0020, 0x0002, 0xEAAD, 0x0000, 0x0000, 0x0002, /*16AAD*/
  0xFBC2, 0x0020, 0x0002, 0xEAAE, 0x0000, 0x0000, 0x0002, /*16AAE*/
  0xFBC2, 0x0020, 0x0002, 0xEAAF, 0x0000, 0x0000, 0x0002, /*16AAF*/
  0xFBC2, 0x0020, 0x0002, 0xEAB0, 0x0000, 0x0000, 0x0002, /*16AB0*/
  0xFBC2, 0x0020, 0x0002, 0xEAB1, 0x0000, 0x0000, 0x0002, /*16AB1*/
  0xFBC2, 0x0020, 0x0002, 0xEAB2, 0x0000, 0x0000, 0x0002, /*16AB2*/
  0xFBC2, 0x0020, 0x0002, 0xEAB3, 0x0000, 0x0000, 0x0002, /*16AB3*/
  0xFBC2, 0x0020, 0x0002, 0xEAB4, 0x0000, 0x0000, 0x0002, /*16AB4*/
  0xFBC2, 0x0020, 0x0002, 0xEAB5, 0x0000, 0x0000, 0x0002, /*16AB5*/
  0xFBC2, 0x0020, 0x0002, 0xEAB6, 0x0000, 0x0000, 0x0002, /*16AB6*/
  0xFBC2, 0x0020, 0x0002, 0xEAB7, 0x0000, 0x0000, 0x0002, /*16AB7*/
  0xFBC2, 0x0020, 0x0002, 0xEAB8, 0x0000, 0x0000, 0x0002, /*16AB8*/
  0xFBC2, 0x0020, 0x0002, 0xEAB9, 0x0000, 0x0000, 0x0002, /*16AB9*/
  0xFBC2, 0x0020, 0x0002, 0xEABA, 0x0000, 0x0000, 0x0002, /*16ABA*/
  0xFBC2, 0x0020, 0x0002, 0xEABB, 0x0000, 0x0000, 0x0002, /*16ABB*/
  0xFBC2, 0x0020, 0x0002, 0xEABC, 0x0000, 0x0000, 0x0002, /*16ABC*/
  0xFBC2, 0x0020, 0x0002, 0xEABD, 0x0000, 0x0000, 0x0002, /*16ABD*/
  0xFBC2, 0x0020, 0x0002, 0xEABE, 0x0000, 0x0000, 0x0002, /*16ABE*/
  0xFBC2, 0x0020, 0x0002, 0xEABF, 0x0000, 0x0000, 0x0002, /*16ABF*/
  0xFBC2, 0x0020, 0x0002, 0xEAC0, 0x0000, 0x0000, 0x0002, /*16AC0*/
  0xFBC2, 0x0020, 0x0002, 0xEAC1, 0x0000, 0x0000, 0x0002, /*16AC1*/
  0xFBC2, 0x0020, 0x0002, 0xEAC2, 0x0000, 0x0000, 0x0002, /*16AC2*/
  0xFBC2, 0x0020, 0x0002, 0xEAC3, 0x0000, 0x0000, 0x0002, /*16AC3*/
  0xFBC2, 0x0020, 0x0002, 0xEAC4, 0x0000, 0x0000, 0x0002, /*16AC4*/
  0xFBC2, 0x0020, 0x0002, 0xEAC5, 0x0000, 0x0000, 0x0002, /*16AC5*/
  0xFBC2, 0x0020, 0x0002, 0xEAC6, 0x0000, 0x0000, 0x0002, /*16AC6*/
  0xFBC2, 0x0020, 0x0002, 0xEAC7, 0x0000, 0x0000, 0x0002, /*16AC7*/
  0xFBC2, 0x0020, 0x0002, 0xEAC8, 0x0000, 0x0000, 0x0002, /*16AC8*/
  0xFBC2, 0x0020, 0x0002, 0xEAC9, 0x0000, 0x0000, 0x0002, /*16AC9*/
  0xFBC2, 0x0020, 0x0002, 0xEACA, 0x0000, 0x0000, 0x0002, /*16ACA*/
  0xFBC2, 0x0020, 0x0002, 0xEACB, 0x0000, 0x0000, 0x0002, /*16ACB*/
  0xFBC2, 0x0020, 0x0002, 0xEACC, 0x0000, 0x0000, 0x0002, /*16ACC*/
  0xFBC2, 0x0020, 0x0002, 0xEACD, 0x0000, 0x0000, 0x0002, /*16ACD*/
  0xFBC2, 0x0020, 0x0002, 0xEACE, 0x0000, 0x0000, 0x0002, /*16ACE*/
  0xFBC2, 0x0020, 0x0002, 0xEACF, 0x0000, 0x0000, 0x0002, /*16ACF*/
  0x3AF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AD0*/
  0x3AF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AD1*/
  0x3AF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AD2*/
  0x3AF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AD3*/
  0x3AF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AD4*/
  0x3AF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AD5*/
  0x3AF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AD6*/
  0x3AF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AD7*/
  0x3AF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AD8*/
  0x3AF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AD9*/
  0x3AFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16ADA*/
  0x3AFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16ADB*/
  0x3AFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16ADC*/
  0x3AFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16ADD*/
  0x3AFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16ADE*/
  0x3AFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16ADF*/
  0x3B00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AE0*/
  0x3B01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AE1*/
  0x3B02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AE2*/
  0x3B03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AE3*/
  0x3B04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AE4*/
  0x3B05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AE5*/
  0x3B06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AE6*/
  0x3B07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AE7*/
  0x3B08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AE8*/
  0x3B09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AE9*/
  0x3B0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AEA*/
  0x3B0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AEB*/
  0x3B0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AEC*/
  0x3B0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AED*/
  0xFBC2, 0x0020, 0x0002, 0xEAEE, 0x0000, 0x0000, 0x0002, /*16AEE*/
  0xFBC2, 0x0020, 0x0002, 0xEAEF, 0x0000, 0x0000, 0x0002, /*16AEF*/
  0x0000, 0x00B8, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AF0*/
  0x0000, 0x00B9, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AF1*/
  0x0000, 0x00BA, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AF2*/
  0x0000, 0x00BB, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AF3*/
  0x0000, 0x00BC, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AF4*/
  0x0288, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16AF5*/
  0xFBC2, 0x0020, 0x0002, 0xEAF6, 0x0000, 0x0000, 0x0002, /*16AF6*/
  0xFBC2, 0x0020, 0x0002, 0xEAF7, 0x0000, 0x0000, 0x0002, /*16AF7*/
  0xFBC2, 0x0020, 0x0002, 0xEAF8, 0x0000, 0x0000, 0x0002, /*16AF8*/
  0xFBC2, 0x0020, 0x0002, 0xEAF9, 0x0000, 0x0000, 0x0002, /*16AF9*/
  0xFBC2, 0x0020, 0x0002, 0xEAFA, 0x0000, 0x0000, 0x0002, /*16AFA*/
  0xFBC2, 0x0020, 0x0002, 0xEAFB, 0x0000, 0x0000, 0x0002, /*16AFB*/
  0xFBC2, 0x0020, 0x0002, 0xEAFC, 0x0000, 0x0000, 0x0002, /*16AFC*/
  0xFBC2, 0x0020, 0x0002, 0xEAFD, 0x0000, 0x0000, 0x0002, /*16AFD*/
  0xFBC2, 0x0020, 0x0002, 0xEAFE, 0x0000, 0x0000, 0x0002, /*16AFE*/
  0xFBC2, 0x0020, 0x0002, 0xEAFF, 0x0000, 0x0000, 0x0002  /*16AFF*/
};

uint16 uca900_p16B[]= {
  0x4355, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B00*/
  0x4356, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B01*/
  0x4357, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B02*/
  0x4358, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B03*/
  0x4359, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B04*/
  0x435A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B05*/
  0x435B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B06*/
  0x435C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B07*/
  0x435D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B08*/
  0x435E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B09*/
  0x435F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B0A*/
  0x4360, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B0B*/
  0x4361, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B0C*/
  0x4362, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B0D*/
  0x4363, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B0E*/
  0x4364, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B0F*/
  0x4365, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B10*/
  0x4366, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B11*/
  0x4367, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B12*/
  0x4368, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B13*/
  0x4369, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B14*/
  0x436A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B15*/
  0x436B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B16*/
  0x436C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B17*/
  0x436D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B18*/
  0x436E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B19*/
  0x436F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B1A*/
  0x4370, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B1B*/
  0x4371, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B1C*/
  0x4372, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B1D*/
  0x4373, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B1E*/
  0x4374, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B1F*/
  0x4375, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B20*/
  0x4376, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B21*/
  0x4377, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B22*/
  0x4378, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B23*/
  0x4379, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B24*/
  0x437A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B25*/
  0x437B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B26*/
  0x437C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B27*/
  0x437D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B28*/
  0x437E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B29*/
  0x437F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B2A*/
  0x4380, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B2B*/
  0x4381, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B2C*/
  0x4382, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B2D*/
  0x4383, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B2E*/
  0x4384, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B2F*/
  0x0000, 0x00F4, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B30*/
  0x0000, 0x00F5, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B31*/
  0x0000, 0x00F6, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B32*/
  0x0000, 0x00F7, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B33*/
  0x0000, 0x00F8, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B34*/
  0x0000, 0x00F9, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B35*/
  0x0000, 0x00FA, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B36*/
  0x0477, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B37*/
  0x0478, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B38*/
  0x0479, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B39*/
  0x047A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B3A*/
  0x047B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B3B*/
  0x0FD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B3C*/
  0x0FD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B3D*/
  0x0FD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B3E*/
  0x0FD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B3F*/
  0x4385, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B40*/
  0x4386, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B41*/
  0x1C05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B42*/
  0x1C06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B43*/
  0x047C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B44*/
  0x0FD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B45*/
  0xFBC2, 0x0020, 0x0002, 0xEB46, 0x0000, 0x0000, 0x0002, /*16B46*/
  0xFBC2, 0x0020, 0x0002, 0xEB47, 0x0000, 0x0000, 0x0002, /*16B47*/
  0xFBC2, 0x0020, 0x0002, 0xEB48, 0x0000, 0x0000, 0x0002, /*16B48*/
  0xFBC2, 0x0020, 0x0002, 0xEB49, 0x0000, 0x0000, 0x0002, /*16B49*/
  0xFBC2, 0x0020, 0x0002, 0xEB4A, 0x0000, 0x0000, 0x0002, /*16B4A*/
  0xFBC2, 0x0020, 0x0002, 0xEB4B, 0x0000, 0x0000, 0x0002, /*16B4B*/
  0xFBC2, 0x0020, 0x0002, 0xEB4C, 0x0000, 0x0000, 0x0002, /*16B4C*/
  0xFBC2, 0x0020, 0x0002, 0xEB4D, 0x0000, 0x0000, 0x0002, /*16B4D*/
  0xFBC2, 0x0020, 0x0002, 0xEB4E, 0x0000, 0x0000, 0x0002, /*16B4E*/
  0xFBC2, 0x0020, 0x0002, 0xEB4F, 0x0000, 0x0000, 0x0002, /*16B4F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B50*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B51*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B52*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B53*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B54*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B55*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B56*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B57*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B58*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B59*/
  0xFBC2, 0x0020, 0x0002, 0xEB5A, 0x0000, 0x0000, 0x0002, /*16B5A*/
  0x1BA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B5B*/
  0x1BA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B5C*/
  0x1BA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B5D*/
  0x1BA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B5E*/
  0x1BA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B5F*/
  0x1BA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B60*/
  0x1BA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B61*/
  0xFBC2, 0x0020, 0x0002, 0xEB62, 0x0000, 0x0000, 0x0002, /*16B62*/
  0x4387, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B63*/
  0x4388, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B64*/
  0x4389, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B65*/
  0x438A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B66*/
  0x438B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B67*/
  0x438C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B68*/
  0x438D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B69*/
  0x438E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B6A*/
  0x438F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B6B*/
  0x4390, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B6C*/
  0x4391, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B6D*/
  0x4392, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B6E*/
  0x4393, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B6F*/
  0x4394, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B70*/
  0x4395, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B71*/
  0x4396, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B72*/
  0x4397, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B73*/
  0x4398, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B74*/
  0x4399, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B75*/
  0x439A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B76*/
  0x439B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B77*/
  0xFBC2, 0x0020, 0x0002, 0xEB78, 0x0000, 0x0000, 0x0002, /*16B78*/
  0xFBC2, 0x0020, 0x0002, 0xEB79, 0x0000, 0x0000, 0x0002, /*16B79*/
  0xFBC2, 0x0020, 0x0002, 0xEB7A, 0x0000, 0x0000, 0x0002, /*16B7A*/
  0xFBC2, 0x0020, 0x0002, 0xEB7B, 0x0000, 0x0000, 0x0002, /*16B7B*/
  0xFBC2, 0x0020, 0x0002, 0xEB7C, 0x0000, 0x0000, 0x0002, /*16B7C*/
  0x439C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B7D*/
  0x439D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B7E*/
  0x439E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B7F*/
  0x439F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B80*/
  0x43A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B81*/
  0x43A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B82*/
  0x43A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B83*/
  0x43A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B84*/
  0x43A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B85*/
  0x43A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B86*/
  0x43A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B87*/
  0x43A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B88*/
  0x43A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B89*/
  0x43A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B8A*/
  0x43AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B8B*/
  0x43AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B8C*/
  0x43AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B8D*/
  0x43AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B8E*/
  0x43AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16B8F*/
  0xFBC2, 0x0020, 0x0002, 0xEB90, 0x0000, 0x0000, 0x0002, /*16B90*/
  0xFBC2, 0x0020, 0x0002, 0xEB91, 0x0000, 0x0000, 0x0002, /*16B91*/
  0xFBC2, 0x0020, 0x0002, 0xEB92, 0x0000, 0x0000, 0x0002, /*16B92*/
  0xFBC2, 0x0020, 0x0002, 0xEB93, 0x0000, 0x0000, 0x0002, /*16B93*/
  0xFBC2, 0x0020, 0x0002, 0xEB94, 0x0000, 0x0000, 0x0002, /*16B94*/
  0xFBC2, 0x0020, 0x0002, 0xEB95, 0x0000, 0x0000, 0x0002, /*16B95*/
  0xFBC2, 0x0020, 0x0002, 0xEB96, 0x0000, 0x0000, 0x0002, /*16B96*/
  0xFBC2, 0x0020, 0x0002, 0xEB97, 0x0000, 0x0000, 0x0002, /*16B97*/
  0xFBC2, 0x0020, 0x0002, 0xEB98, 0x0000, 0x0000, 0x0002, /*16B98*/
  0xFBC2, 0x0020, 0x0002, 0xEB99, 0x0000, 0x0000, 0x0002, /*16B99*/
  0xFBC2, 0x0020, 0x0002, 0xEB9A, 0x0000, 0x0000, 0x0002, /*16B9A*/
  0xFBC2, 0x0020, 0x0002, 0xEB9B, 0x0000, 0x0000, 0x0002, /*16B9B*/
  0xFBC2, 0x0020, 0x0002, 0xEB9C, 0x0000, 0x0000, 0x0002, /*16B9C*/
  0xFBC2, 0x0020, 0x0002, 0xEB9D, 0x0000, 0x0000, 0x0002, /*16B9D*/
  0xFBC2, 0x0020, 0x0002, 0xEB9E, 0x0000, 0x0000, 0x0002, /*16B9E*/
  0xFBC2, 0x0020, 0x0002, 0xEB9F, 0x0000, 0x0000, 0x0002, /*16B9F*/
  0xFBC2, 0x0020, 0x0002, 0xEBA0, 0x0000, 0x0000, 0x0002, /*16BA0*/
  0xFBC2, 0x0020, 0x0002, 0xEBA1, 0x0000, 0x0000, 0x0002, /*16BA1*/
  0xFBC2, 0x0020, 0x0002, 0xEBA2, 0x0000, 0x0000, 0x0002, /*16BA2*/
  0xFBC2, 0x0020, 0x0002, 0xEBA3, 0x0000, 0x0000, 0x0002, /*16BA3*/
  0xFBC2, 0x0020, 0x0002, 0xEBA4, 0x0000, 0x0000, 0x0002, /*16BA4*/
  0xFBC2, 0x0020, 0x0002, 0xEBA5, 0x0000, 0x0000, 0x0002, /*16BA5*/
  0xFBC2, 0x0020, 0x0002, 0xEBA6, 0x0000, 0x0000, 0x0002, /*16BA6*/
  0xFBC2, 0x0020, 0x0002, 0xEBA7, 0x0000, 0x0000, 0x0002, /*16BA7*/
  0xFBC2, 0x0020, 0x0002, 0xEBA8, 0x0000, 0x0000, 0x0002, /*16BA8*/
  0xFBC2, 0x0020, 0x0002, 0xEBA9, 0x0000, 0x0000, 0x0002, /*16BA9*/
  0xFBC2, 0x0020, 0x0002, 0xEBAA, 0x0000, 0x0000, 0x0002, /*16BAA*/
  0xFBC2, 0x0020, 0x0002, 0xEBAB, 0x0000, 0x0000, 0x0002, /*16BAB*/
  0xFBC2, 0x0020, 0x0002, 0xEBAC, 0x0000, 0x0000, 0x0002, /*16BAC*/
  0xFBC2, 0x0020, 0x0002, 0xEBAD, 0x0000, 0x0000, 0x0002, /*16BAD*/
  0xFBC2, 0x0020, 0x0002, 0xEBAE, 0x0000, 0x0000, 0x0002, /*16BAE*/
  0xFBC2, 0x0020, 0x0002, 0xEBAF, 0x0000, 0x0000, 0x0002, /*16BAF*/
  0xFBC2, 0x0020, 0x0002, 0xEBB0, 0x0000, 0x0000, 0x0002, /*16BB0*/
  0xFBC2, 0x0020, 0x0002, 0xEBB1, 0x0000, 0x0000, 0x0002, /*16BB1*/
  0xFBC2, 0x0020, 0x0002, 0xEBB2, 0x0000, 0x0000, 0x0002, /*16BB2*/
  0xFBC2, 0x0020, 0x0002, 0xEBB3, 0x0000, 0x0000, 0x0002, /*16BB3*/
  0xFBC2, 0x0020, 0x0002, 0xEBB4, 0x0000, 0x0000, 0x0002, /*16BB4*/
  0xFBC2, 0x0020, 0x0002, 0xEBB5, 0x0000, 0x0000, 0x0002, /*16BB5*/
  0xFBC2, 0x0020, 0x0002, 0xEBB6, 0x0000, 0x0000, 0x0002, /*16BB6*/
  0xFBC2, 0x0020, 0x0002, 0xEBB7, 0x0000, 0x0000, 0x0002, /*16BB7*/
  0xFBC2, 0x0020, 0x0002, 0xEBB8, 0x0000, 0x0000, 0x0002, /*16BB8*/
  0xFBC2, 0x0020, 0x0002, 0xEBB9, 0x0000, 0x0000, 0x0002, /*16BB9*/
  0xFBC2, 0x0020, 0x0002, 0xEBBA, 0x0000, 0x0000, 0x0002, /*16BBA*/
  0xFBC2, 0x0020, 0x0002, 0xEBBB, 0x0000, 0x0000, 0x0002, /*16BBB*/
  0xFBC2, 0x0020, 0x0002, 0xEBBC, 0x0000, 0x0000, 0x0002, /*16BBC*/
  0xFBC2, 0x0020, 0x0002, 0xEBBD, 0x0000, 0x0000, 0x0002, /*16BBD*/
  0xFBC2, 0x0020, 0x0002, 0xEBBE, 0x0000, 0x0000, 0x0002, /*16BBE*/
  0xFBC2, 0x0020, 0x0002, 0xEBBF, 0x0000, 0x0000, 0x0002, /*16BBF*/
  0xFBC2, 0x0020, 0x0002, 0xEBC0, 0x0000, 0x0000, 0x0002, /*16BC0*/
  0xFBC2, 0x0020, 0x0002, 0xEBC1, 0x0000, 0x0000, 0x0002, /*16BC1*/
  0xFBC2, 0x0020, 0x0002, 0xEBC2, 0x0000, 0x0000, 0x0002, /*16BC2*/
  0xFBC2, 0x0020, 0x0002, 0xEBC3, 0x0000, 0x0000, 0x0002, /*16BC3*/
  0xFBC2, 0x0020, 0x0002, 0xEBC4, 0x0000, 0x0000, 0x0002, /*16BC4*/
  0xFBC2, 0x0020, 0x0002, 0xEBC5, 0x0000, 0x0000, 0x0002, /*16BC5*/
  0xFBC2, 0x0020, 0x0002, 0xEBC6, 0x0000, 0x0000, 0x0002, /*16BC6*/
  0xFBC2, 0x0020, 0x0002, 0xEBC7, 0x0000, 0x0000, 0x0002, /*16BC7*/
  0xFBC2, 0x0020, 0x0002, 0xEBC8, 0x0000, 0x0000, 0x0002, /*16BC8*/
  0xFBC2, 0x0020, 0x0002, 0xEBC9, 0x0000, 0x0000, 0x0002, /*16BC9*/
  0xFBC2, 0x0020, 0x0002, 0xEBCA, 0x0000, 0x0000, 0x0002, /*16BCA*/
  0xFBC2, 0x0020, 0x0002, 0xEBCB, 0x0000, 0x0000, 0x0002, /*16BCB*/
  0xFBC2, 0x0020, 0x0002, 0xEBCC, 0x0000, 0x0000, 0x0002, /*16BCC*/
  0xFBC2, 0x0020, 0x0002, 0xEBCD, 0x0000, 0x0000, 0x0002, /*16BCD*/
  0xFBC2, 0x0020, 0x0002, 0xEBCE, 0x0000, 0x0000, 0x0002, /*16BCE*/
  0xFBC2, 0x0020, 0x0002, 0xEBCF, 0x0000, 0x0000, 0x0002, /*16BCF*/
  0xFBC2, 0x0020, 0x0002, 0xEBD0, 0x0000, 0x0000, 0x0002, /*16BD0*/
  0xFBC2, 0x0020, 0x0002, 0xEBD1, 0x0000, 0x0000, 0x0002, /*16BD1*/
  0xFBC2, 0x0020, 0x0002, 0xEBD2, 0x0000, 0x0000, 0x0002, /*16BD2*/
  0xFBC2, 0x0020, 0x0002, 0xEBD3, 0x0000, 0x0000, 0x0002, /*16BD3*/
  0xFBC2, 0x0020, 0x0002, 0xEBD4, 0x0000, 0x0000, 0x0002, /*16BD4*/
  0xFBC2, 0x0020, 0x0002, 0xEBD5, 0x0000, 0x0000, 0x0002, /*16BD5*/
  0xFBC2, 0x0020, 0x0002, 0xEBD6, 0x0000, 0x0000, 0x0002, /*16BD6*/
  0xFBC2, 0x0020, 0x0002, 0xEBD7, 0x0000, 0x0000, 0x0002, /*16BD7*/
  0xFBC2, 0x0020, 0x0002, 0xEBD8, 0x0000, 0x0000, 0x0002, /*16BD8*/
  0xFBC2, 0x0020, 0x0002, 0xEBD9, 0x0000, 0x0000, 0x0002, /*16BD9*/
  0xFBC2, 0x0020, 0x0002, 0xEBDA, 0x0000, 0x0000, 0x0002, /*16BDA*/
  0xFBC2, 0x0020, 0x0002, 0xEBDB, 0x0000, 0x0000, 0x0002, /*16BDB*/
  0xFBC2, 0x0020, 0x0002, 0xEBDC, 0x0000, 0x0000, 0x0002, /*16BDC*/
  0xFBC2, 0x0020, 0x0002, 0xEBDD, 0x0000, 0x0000, 0x0002, /*16BDD*/
  0xFBC2, 0x0020, 0x0002, 0xEBDE, 0x0000, 0x0000, 0x0002, /*16BDE*/
  0xFBC2, 0x0020, 0x0002, 0xEBDF, 0x0000, 0x0000, 0x0002, /*16BDF*/
  0xFBC2, 0x0020, 0x0002, 0xEBE0, 0x0000, 0x0000, 0x0002, /*16BE0*/
  0xFBC2, 0x0020, 0x0002, 0xEBE1, 0x0000, 0x0000, 0x0002, /*16BE1*/
  0xFBC2, 0x0020, 0x0002, 0xEBE2, 0x0000, 0x0000, 0x0002, /*16BE2*/
  0xFBC2, 0x0020, 0x0002, 0xEBE3, 0x0000, 0x0000, 0x0002, /*16BE3*/
  0xFBC2, 0x0020, 0x0002, 0xEBE4, 0x0000, 0x0000, 0x0002, /*16BE4*/
  0xFBC2, 0x0020, 0x0002, 0xEBE5, 0x0000, 0x0000, 0x0002, /*16BE5*/
  0xFBC2, 0x0020, 0x0002, 0xEBE6, 0x0000, 0x0000, 0x0002, /*16BE6*/
  0xFBC2, 0x0020, 0x0002, 0xEBE7, 0x0000, 0x0000, 0x0002, /*16BE7*/
  0xFBC2, 0x0020, 0x0002, 0xEBE8, 0x0000, 0x0000, 0x0002, /*16BE8*/
  0xFBC2, 0x0020, 0x0002, 0xEBE9, 0x0000, 0x0000, 0x0002, /*16BE9*/
  0xFBC2, 0x0020, 0x0002, 0xEBEA, 0x0000, 0x0000, 0x0002, /*16BEA*/
  0xFBC2, 0x0020, 0x0002, 0xEBEB, 0x0000, 0x0000, 0x0002, /*16BEB*/
  0xFBC2, 0x0020, 0x0002, 0xEBEC, 0x0000, 0x0000, 0x0002, /*16BEC*/
  0xFBC2, 0x0020, 0x0002, 0xEBED, 0x0000, 0x0000, 0x0002, /*16BED*/
  0xFBC2, 0x0020, 0x0002, 0xEBEE, 0x0000, 0x0000, 0x0002, /*16BEE*/
  0xFBC2, 0x0020, 0x0002, 0xEBEF, 0x0000, 0x0000, 0x0002, /*16BEF*/
  0xFBC2, 0x0020, 0x0002, 0xEBF0, 0x0000, 0x0000, 0x0002, /*16BF0*/
  0xFBC2, 0x0020, 0x0002, 0xEBF1, 0x0000, 0x0000, 0x0002, /*16BF1*/
  0xFBC2, 0x0020, 0x0002, 0xEBF2, 0x0000, 0x0000, 0x0002, /*16BF2*/
  0xFBC2, 0x0020, 0x0002, 0xEBF3, 0x0000, 0x0000, 0x0002, /*16BF3*/
  0xFBC2, 0x0020, 0x0002, 0xEBF4, 0x0000, 0x0000, 0x0002, /*16BF4*/
  0xFBC2, 0x0020, 0x0002, 0xEBF5, 0x0000, 0x0000, 0x0002, /*16BF5*/
  0xFBC2, 0x0020, 0x0002, 0xEBF6, 0x0000, 0x0000, 0x0002, /*16BF6*/
  0xFBC2, 0x0020, 0x0002, 0xEBF7, 0x0000, 0x0000, 0x0002, /*16BF7*/
  0xFBC2, 0x0020, 0x0002, 0xEBF8, 0x0000, 0x0000, 0x0002, /*16BF8*/
  0xFBC2, 0x0020, 0x0002, 0xEBF9, 0x0000, 0x0000, 0x0002, /*16BF9*/
  0xFBC2, 0x0020, 0x0002, 0xEBFA, 0x0000, 0x0000, 0x0002, /*16BFA*/
  0xFBC2, 0x0020, 0x0002, 0xEBFB, 0x0000, 0x0000, 0x0002, /*16BFB*/
  0xFBC2, 0x0020, 0x0002, 0xEBFC, 0x0000, 0x0000, 0x0002, /*16BFC*/
  0xFBC2, 0x0020, 0x0002, 0xEBFD, 0x0000, 0x0000, 0x0002, /*16BFD*/
  0xFBC2, 0x0020, 0x0002, 0xEBFE, 0x0000, 0x0000, 0x0002, /*16BFE*/
  0xFBC2, 0x0020, 0x0002, 0xEBFF, 0x0000, 0x0000, 0x0002  /*16BFF*/
};

uint16 uca900_p16F[]= {
  0x427A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F00*/
  0x427B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F01*/
  0x427C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F02*/
  0x427D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F03*/
  0x427E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F04*/
  0x427F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F05*/
  0x427E, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*16F06*/
  0x4280, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F07*/
  0x4281, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F08*/
  0x4282, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F09*/
  0x4283, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F0A*/
  0x4284, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F0B*/
  0x4285, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F0C*/
  0x4286, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F0D*/
  0x4287, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F0E*/
  0x4288, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F0F*/
  0x4289, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F10*/
  0x428A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F11*/
  0x428B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F12*/
  0x4289, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*16F13*/
  0x428C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F14*/
  0x428D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F15*/
  0x428E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F16*/
  0x428F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F17*/
  0x4290, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F18*/
  0x4291, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F19*/
  0x4292, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F1A*/
  0x4293, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F1B*/
  0x4294, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F1C*/
  0x4295, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F1D*/
  0x4296, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F1E*/
  0x4297, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F1F*/
  0x4298, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F20*/
  0x4299, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F21*/
  0x429A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F22*/
  0x429B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F23*/
  0x429C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F24*/
  0x429B, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*16F25*/
  0x429D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F26*/
  0x429E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F27*/
  0x429F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F28*/
  0x42A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F29*/
  0x42A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F2A*/
  0x42A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F2B*/
  0x42A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F2C*/
  0x42A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F2D*/
  0x42A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F2E*/
  0x42A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F2F*/
  0x42A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F30*/
  0x42A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F31*/
  0x42A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F32*/
  0x42AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F33*/
  0x42AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F34*/
  0x42AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F35*/
  0x42AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F36*/
  0x42AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F37*/
  0x42AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F38*/
  0x42B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F39*/
  0x42B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F3A*/
  0x42B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F3B*/
  0x42B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F3C*/
  0x42B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F3D*/
  0x42B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F3E*/
  0x42B4, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*16F3F*/
  0x42B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F40*/
  0x42B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F41*/
  0x42B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F42*/
  0x42B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F43*/
  0x42BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F44*/
  0xFBC2, 0x0020, 0x0002, 0xEF45, 0x0000, 0x0000, 0x0002, /*16F45*/
  0xFBC2, 0x0020, 0x0002, 0xEF46, 0x0000, 0x0000, 0x0002, /*16F46*/
  0xFBC2, 0x0020, 0x0002, 0xEF47, 0x0000, 0x0000, 0x0002, /*16F47*/
  0xFBC2, 0x0020, 0x0002, 0xEF48, 0x0000, 0x0000, 0x0002, /*16F48*/
  0xFBC2, 0x0020, 0x0002, 0xEF49, 0x0000, 0x0000, 0x0002, /*16F49*/
  0xFBC2, 0x0020, 0x0002, 0xEF4A, 0x0000, 0x0000, 0x0002, /*16F4A*/
  0xFBC2, 0x0020, 0x0002, 0xEF4B, 0x0000, 0x0000, 0x0002, /*16F4B*/
  0xFBC2, 0x0020, 0x0002, 0xEF4C, 0x0000, 0x0000, 0x0002, /*16F4C*/
  0xFBC2, 0x0020, 0x0002, 0xEF4D, 0x0000, 0x0000, 0x0002, /*16F4D*/
  0xFBC2, 0x0020, 0x0002, 0xEF4E, 0x0000, 0x0000, 0x0002, /*16F4E*/
  0xFBC2, 0x0020, 0x0002, 0xEF4F, 0x0000, 0x0000, 0x0002, /*16F4F*/
  0x42BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F50*/
  0x42BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F51*/
  0x42BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F52*/
  0x42BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F53*/
  0x42BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F54*/
  0x42C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F55*/
  0x42C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F56*/
  0x42C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F57*/
  0x42C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F58*/
  0x42C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F59*/
  0x42C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F5A*/
  0x42C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F5B*/
  0x42C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F5C*/
  0x42C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F5D*/
  0x42C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F5E*/
  0x42CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F5F*/
  0x42CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F60*/
  0x42CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F61*/
  0x42CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F62*/
  0x42CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F63*/
  0x42CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F64*/
  0x42D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F65*/
  0x42D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F66*/
  0x42D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F67*/
  0x42D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F68*/
  0x42D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F69*/
  0x42D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F6A*/
  0x42D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F6B*/
  0x42D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F6C*/
  0x42D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F6D*/
  0x42D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F6E*/
  0x42DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F6F*/
  0x42DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F70*/
  0x42DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F71*/
  0x42DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F72*/
  0x42DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F73*/
  0x42DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F74*/
  0x42E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F75*/
  0x42E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F76*/
  0x42E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F77*/
  0x42E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F78*/
  0x42E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F79*/
  0x42E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F7A*/
  0x42E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F7B*/
  0x42E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F7C*/
  0x42E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F7D*/
  0x42E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F7E*/
  0xFBC2, 0x0020, 0x0002, 0xEF7F, 0x0000, 0x0000, 0x0002, /*16F7F*/
  0xFBC2, 0x0020, 0x0002, 0xEF80, 0x0000, 0x0000, 0x0002, /*16F80*/
  0xFBC2, 0x0020, 0x0002, 0xEF81, 0x0000, 0x0000, 0x0002, /*16F81*/
  0xFBC2, 0x0020, 0x0002, 0xEF82, 0x0000, 0x0000, 0x0002, /*16F82*/
  0xFBC2, 0x0020, 0x0002, 0xEF83, 0x0000, 0x0000, 0x0002, /*16F83*/
  0xFBC2, 0x0020, 0x0002, 0xEF84, 0x0000, 0x0000, 0x0002, /*16F84*/
  0xFBC2, 0x0020, 0x0002, 0xEF85, 0x0000, 0x0000, 0x0002, /*16F85*/
  0xFBC2, 0x0020, 0x0002, 0xEF86, 0x0000, 0x0000, 0x0002, /*16F86*/
  0xFBC2, 0x0020, 0x0002, 0xEF87, 0x0000, 0x0000, 0x0002, /*16F87*/
  0xFBC2, 0x0020, 0x0002, 0xEF88, 0x0000, 0x0000, 0x0002, /*16F88*/
  0xFBC2, 0x0020, 0x0002, 0xEF89, 0x0000, 0x0000, 0x0002, /*16F89*/
  0xFBC2, 0x0020, 0x0002, 0xEF8A, 0x0000, 0x0000, 0x0002, /*16F8A*/
  0xFBC2, 0x0020, 0x0002, 0xEF8B, 0x0000, 0x0000, 0x0002, /*16F8B*/
  0xFBC2, 0x0020, 0x0002, 0xEF8C, 0x0000, 0x0000, 0x0002, /*16F8C*/
  0xFBC2, 0x0020, 0x0002, 0xEF8D, 0x0000, 0x0000, 0x0002, /*16F8D*/
  0xFBC2, 0x0020, 0x0002, 0xEF8E, 0x0000, 0x0000, 0x0002, /*16F8E*/
  0x42EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F8F*/
  0x42EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F90*/
  0x42EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F91*/
  0x42ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F92*/
  0x42EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F93*/
  0x42EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F94*/
  0x42F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F95*/
  0x42F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F96*/
  0x42F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F97*/
  0x42F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F98*/
  0x42F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F99*/
  0x42F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F9A*/
  0x42F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F9B*/
  0x42F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F9C*/
  0x42F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F9D*/
  0x42F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F9E*/
  0x42FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16F9F*/
  0xFBC2, 0x0020, 0x0002, 0xEFA0, 0x0000, 0x0000, 0x0002, /*16FA0*/
  0xFBC2, 0x0020, 0x0002, 0xEFA1, 0x0000, 0x0000, 0x0002, /*16FA1*/
  0xFBC2, 0x0020, 0x0002, 0xEFA2, 0x0000, 0x0000, 0x0002, /*16FA2*/
  0xFBC2, 0x0020, 0x0002, 0xEFA3, 0x0000, 0x0000, 0x0002, /*16FA3*/
  0xFBC2, 0x0020, 0x0002, 0xEFA4, 0x0000, 0x0000, 0x0002, /*16FA4*/
  0xFBC2, 0x0020, 0x0002, 0xEFA5, 0x0000, 0x0000, 0x0002, /*16FA5*/
  0xFBC2, 0x0020, 0x0002, 0xEFA6, 0x0000, 0x0000, 0x0002, /*16FA6*/
  0xFBC2, 0x0020, 0x0002, 0xEFA7, 0x0000, 0x0000, 0x0002, /*16FA7*/
  0xFBC2, 0x0020, 0x0002, 0xEFA8, 0x0000, 0x0000, 0x0002, /*16FA8*/
  0xFBC2, 0x0020, 0x0002, 0xEFA9, 0x0000, 0x0000, 0x0002, /*16FA9*/
  0xFBC2, 0x0020, 0x0002, 0xEFAA, 0x0000, 0x0000, 0x0002, /*16FAA*/
  0xFBC2, 0x0020, 0x0002, 0xEFAB, 0x0000, 0x0000, 0x0002, /*16FAB*/
  0xFBC2, 0x0020, 0x0002, 0xEFAC, 0x0000, 0x0000, 0x0002, /*16FAC*/
  0xFBC2, 0x0020, 0x0002, 0xEFAD, 0x0000, 0x0000, 0x0002, /*16FAD*/
  0xFBC2, 0x0020, 0x0002, 0xEFAE, 0x0000, 0x0000, 0x0002, /*16FAE*/
  0xFBC2, 0x0020, 0x0002, 0xEFAF, 0x0000, 0x0000, 0x0002, /*16FAF*/
  0xFBC2, 0x0020, 0x0002, 0xEFB0, 0x0000, 0x0000, 0x0002, /*16FB0*/
  0xFBC2, 0x0020, 0x0002, 0xEFB1, 0x0000, 0x0000, 0x0002, /*16FB1*/
  0xFBC2, 0x0020, 0x0002, 0xEFB2, 0x0000, 0x0000, 0x0002, /*16FB2*/
  0xFBC2, 0x0020, 0x0002, 0xEFB3, 0x0000, 0x0000, 0x0002, /*16FB3*/
  0xFBC2, 0x0020, 0x0002, 0xEFB4, 0x0000, 0x0000, 0x0002, /*16FB4*/
  0xFBC2, 0x0020, 0x0002, 0xEFB5, 0x0000, 0x0000, 0x0002, /*16FB5*/
  0xFBC2, 0x0020, 0x0002, 0xEFB6, 0x0000, 0x0000, 0x0002, /*16FB6*/
  0xFBC2, 0x0020, 0x0002, 0xEFB7, 0x0000, 0x0000, 0x0002, /*16FB7*/
  0xFBC2, 0x0020, 0x0002, 0xEFB8, 0x0000, 0x0000, 0x0002, /*16FB8*/
  0xFBC2, 0x0020, 0x0002, 0xEFB9, 0x0000, 0x0000, 0x0002, /*16FB9*/
  0xFBC2, 0x0020, 0x0002, 0xEFBA, 0x0000, 0x0000, 0x0002, /*16FBA*/
  0xFBC2, 0x0020, 0x0002, 0xEFBB, 0x0000, 0x0000, 0x0002, /*16FBB*/
  0xFBC2, 0x0020, 0x0002, 0xEFBC, 0x0000, 0x0000, 0x0002, /*16FBC*/
  0xFBC2, 0x0020, 0x0002, 0xEFBD, 0x0000, 0x0000, 0x0002, /*16FBD*/
  0xFBC2, 0x0020, 0x0002, 0xEFBE, 0x0000, 0x0000, 0x0002, /*16FBE*/
  0xFBC2, 0x0020, 0x0002, 0xEFBF, 0x0000, 0x0000, 0x0002, /*16FBF*/
  0xFBC2, 0x0020, 0x0002, 0xEFC0, 0x0000, 0x0000, 0x0002, /*16FC0*/
  0xFBC2, 0x0020, 0x0002, 0xEFC1, 0x0000, 0x0000, 0x0002, /*16FC1*/
  0xFBC2, 0x0020, 0x0002, 0xEFC2, 0x0000, 0x0000, 0x0002, /*16FC2*/
  0xFBC2, 0x0020, 0x0002, 0xEFC3, 0x0000, 0x0000, 0x0002, /*16FC3*/
  0xFBC2, 0x0020, 0x0002, 0xEFC4, 0x0000, 0x0000, 0x0002, /*16FC4*/
  0xFBC2, 0x0020, 0x0002, 0xEFC5, 0x0000, 0x0000, 0x0002, /*16FC5*/
  0xFBC2, 0x0020, 0x0002, 0xEFC6, 0x0000, 0x0000, 0x0002, /*16FC6*/
  0xFBC2, 0x0020, 0x0002, 0xEFC7, 0x0000, 0x0000, 0x0002, /*16FC7*/
  0xFBC2, 0x0020, 0x0002, 0xEFC8, 0x0000, 0x0000, 0x0002, /*16FC8*/
  0xFBC2, 0x0020, 0x0002, 0xEFC9, 0x0000, 0x0000, 0x0002, /*16FC9*/
  0xFBC2, 0x0020, 0x0002, 0xEFCA, 0x0000, 0x0000, 0x0002, /*16FCA*/
  0xFBC2, 0x0020, 0x0002, 0xEFCB, 0x0000, 0x0000, 0x0002, /*16FCB*/
  0xFBC2, 0x0020, 0x0002, 0xEFCC, 0x0000, 0x0000, 0x0002, /*16FCC*/
  0xFBC2, 0x0020, 0x0002, 0xEFCD, 0x0000, 0x0000, 0x0002, /*16FCD*/
  0xFBC2, 0x0020, 0x0002, 0xEFCE, 0x0000, 0x0000, 0x0002, /*16FCE*/
  0xFBC2, 0x0020, 0x0002, 0xEFCF, 0x0000, 0x0000, 0x0002, /*16FCF*/
  0xFBC2, 0x0020, 0x0002, 0xEFD0, 0x0000, 0x0000, 0x0002, /*16FD0*/
  0xFBC2, 0x0020, 0x0002, 0xEFD1, 0x0000, 0x0000, 0x0002, /*16FD1*/
  0xFBC2, 0x0020, 0x0002, 0xEFD2, 0x0000, 0x0000, 0x0002, /*16FD2*/
  0xFBC2, 0x0020, 0x0002, 0xEFD3, 0x0000, 0x0000, 0x0002, /*16FD3*/
  0xFBC2, 0x0020, 0x0002, 0xEFD4, 0x0000, 0x0000, 0x0002, /*16FD4*/
  0xFBC2, 0x0020, 0x0002, 0xEFD5, 0x0000, 0x0000, 0x0002, /*16FD5*/
  0xFBC2, 0x0020, 0x0002, 0xEFD6, 0x0000, 0x0000, 0x0002, /*16FD6*/
  0xFBC2, 0x0020, 0x0002, 0xEFD7, 0x0000, 0x0000, 0x0002, /*16FD7*/
  0xFBC2, 0x0020, 0x0002, 0xEFD8, 0x0000, 0x0000, 0x0002, /*16FD8*/
  0xFBC2, 0x0020, 0x0002, 0xEFD9, 0x0000, 0x0000, 0x0002, /*16FD9*/
  0xFBC2, 0x0020, 0x0002, 0xEFDA, 0x0000, 0x0000, 0x0002, /*16FDA*/
  0xFBC2, 0x0020, 0x0002, 0xEFDB, 0x0000, 0x0000, 0x0002, /*16FDB*/
  0xFBC2, 0x0020, 0x0002, 0xEFDC, 0x0000, 0x0000, 0x0002, /*16FDC*/
  0xFBC2, 0x0020, 0x0002, 0xEFDD, 0x0000, 0x0000, 0x0002, /*16FDD*/
  0xFBC2, 0x0020, 0x0002, 0xEFDE, 0x0000, 0x0000, 0x0002, /*16FDE*/
  0xFBC2, 0x0020, 0x0002, 0xEFDF, 0x0000, 0x0000, 0x0002, /*16FDF*/
  0x1C09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*16FE0*/
  0xFBC2, 0x0020, 0x0002, 0xEFE1, 0x0000, 0x0000, 0x0002, /*16FE1*/
  0xFBC2, 0x0020, 0x0002, 0xEFE2, 0x0000, 0x0000, 0x0002, /*16FE2*/
  0xFBC2, 0x0020, 0x0002, 0xEFE3, 0x0000, 0x0000, 0x0002, /*16FE3*/
  0xFBC2, 0x0020, 0x0002, 0xEFE4, 0x0000, 0x0000, 0x0002, /*16FE4*/
  0xFBC2, 0x0020, 0x0002, 0xEFE5, 0x0000, 0x0000, 0x0002, /*16FE5*/
  0xFBC2, 0x0020, 0x0002, 0xEFE6, 0x0000, 0x0000, 0x0002, /*16FE6*/
  0xFBC2, 0x0020, 0x0002, 0xEFE7, 0x0000, 0x0000, 0x0002, /*16FE7*/
  0xFBC2, 0x0020, 0x0002, 0xEFE8, 0x0000, 0x0000, 0x0002, /*16FE8*/
  0xFBC2, 0x0020, 0x0002, 0xEFE9, 0x0000, 0x0000, 0x0002, /*16FE9*/
  0xFBC2, 0x0020, 0x0002, 0xEFEA, 0x0000, 0x0000, 0x0002, /*16FEA*/
  0xFBC2, 0x0020, 0x0002, 0xEFEB, 0x0000, 0x0000, 0x0002, /*16FEB*/
  0xFBC2, 0x0020, 0x0002, 0xEFEC, 0x0000, 0x0000, 0x0002, /*16FEC*/
  0xFBC2, 0x0020, 0x0002, 0xEFED, 0x0000, 0x0000, 0x0002, /*16FED*/
  0xFBC2, 0x0020, 0x0002, 0xEFEE, 0x0000, 0x0000, 0x0002, /*16FEE*/
  0xFBC2, 0x0020, 0x0002, 0xEFEF, 0x0000, 0x0000, 0x0002, /*16FEF*/
  0xFBC2, 0x0020, 0x0002, 0xEFF0, 0x0000, 0x0000, 0x0002, /*16FF0*/
  0xFBC2, 0x0020, 0x0002, 0xEFF1, 0x0000, 0x0000, 0x0002, /*16FF1*/
  0xFBC2, 0x0020, 0x0002, 0xEFF2, 0x0000, 0x0000, 0x0002, /*16FF2*/
  0xFBC2, 0x0020, 0x0002, 0xEFF3, 0x0000, 0x0000, 0x0002, /*16FF3*/
  0xFBC2, 0x0020, 0x0002, 0xEFF4, 0x0000, 0x0000, 0x0002, /*16FF4*/
  0xFBC2, 0x0020, 0x0002, 0xEFF5, 0x0000, 0x0000, 0x0002, /*16FF5*/
  0xFBC2, 0x0020, 0x0002, 0xEFF6, 0x0000, 0x0000, 0x0002, /*16FF6*/
  0xFBC2, 0x0020, 0x0002, 0xEFF7, 0x0000, 0x0000, 0x0002, /*16FF7*/
  0xFBC2, 0x0020, 0x0002, 0xEFF8, 0x0000, 0x0000, 0x0002, /*16FF8*/
  0xFBC2, 0x0020, 0x0002, 0xEFF9, 0x0000, 0x0000, 0x0002, /*16FF9*/
  0xFBC2, 0x0020, 0x0002, 0xEFFA, 0x0000, 0x0000, 0x0002, /*16FFA*/
  0xFBC2, 0x0020, 0x0002, 0xEFFB, 0x0000, 0x0000, 0x0002, /*16FFB*/
  0xFBC2, 0x0020, 0x0002, 0xEFFC, 0x0000, 0x0000, 0x0002, /*16FFC*/
  0xFBC2, 0x0020, 0x0002, 0xEFFD, 0x0000, 0x0000, 0x0002, /*16FFD*/
  0xFBC2, 0x0020, 0x0002, 0xEFFE, 0x0000, 0x0000, 0x0002, /*16FFE*/
  0xFBC2, 0x0020, 0x0002, 0xEFFF, 0x0000, 0x0000, 0x0002  /*16FFF*/
};

uint16 uca900_p1B0[]= {
  0x3D5D, 0x0020, 0x0011, 0x0000, 0x0000, 0x0000, 0x0001, /*1B000*/
  0x3D80, 0x0020, 0x000E, 0x0000, 0x0000, 0x0000, 0x0001, /*1B001*/
  0xFBC3, 0x0020, 0x0002, 0xB002, 0x0000, 0x0000, 0x0002, /*1B002*/
  0xFBC3, 0x0020, 0x0002, 0xB003, 0x0000, 0x0000, 0x0002, /*1B003*/
  0xFBC3, 0x0020, 0x0002, 0xB004, 0x0000, 0x0000, 0x0002, /*1B004*/
  0xFBC3, 0x0020, 0x0002, 0xB005, 0x0000, 0x0000, 0x0002, /*1B005*/
  0xFBC3, 0x0020, 0x0002, 0xB006, 0x0000, 0x0000, 0x0002, /*1B006*/
  0xFBC3, 0x0020, 0x0002, 0xB007, 0x0000, 0x0000, 0x0002, /*1B007*/
  0xFBC3, 0x0020, 0x0002, 0xB008, 0x0000, 0x0000, 0x0002, /*1B008*/
  0xFBC3, 0x0020, 0x0002, 0xB009, 0x0000, 0x0000, 0x0002, /*1B009*/
  0xFBC3, 0x0020, 0x0002, 0xB00A, 0x0000, 0x0000, 0x0002, /*1B00A*/
  0xFBC3, 0x0020, 0x0002, 0xB00B, 0x0000, 0x0000, 0x0002, /*1B00B*/
  0xFBC3, 0x0020, 0x0002, 0xB00C, 0x0000, 0x0000, 0x0002, /*1B00C*/
  0xFBC3, 0x0020, 0x0002, 0xB00D, 0x0000, 0x0000, 0x0002, /*1B00D*/
  0xFBC3, 0x0020, 0x0002, 0xB00E, 0x0000, 0x0000, 0x0002, /*1B00E*/
  0xFBC3, 0x0020, 0x0002, 0xB00F, 0x0000, 0x0000, 0x0002, /*1B00F*/
  0xFBC3, 0x0020, 0x0002, 0xB010, 0x0000, 0x0000, 0x0002, /*1B010*/
  0xFBC3, 0x0020, 0x0002, 0xB011, 0x0000, 0x0000, 0x0002, /*1B011*/
  0xFBC3, 0x0020, 0x0002, 0xB012, 0x0000, 0x0000, 0x0002, /*1B012*/
  0xFBC3, 0x0020, 0x0002, 0xB013, 0x0000, 0x0000, 0x0002, /*1B013*/
  0xFBC3, 0x0020, 0x0002, 0xB014, 0x0000, 0x0000, 0x0002, /*1B014*/
  0xFBC3, 0x0020, 0x0002, 0xB015, 0x0000, 0x0000, 0x0002, /*1B015*/
  0xFBC3, 0x0020, 0x0002, 0xB016, 0x0000, 0x0000, 0x0002, /*1B016*/
  0xFBC3, 0x0020, 0x0002, 0xB017, 0x0000, 0x0000, 0x0002, /*1B017*/
  0xFBC3, 0x0020, 0x0002, 0xB018, 0x0000, 0x0000, 0x0002, /*1B018*/
  0xFBC3, 0x0020, 0x0002, 0xB019, 0x0000, 0x0000, 0x0002, /*1B019*/
  0xFBC3, 0x0020, 0x0002, 0xB01A, 0x0000, 0x0000, 0x0002, /*1B01A*/
  0xFBC3, 0x0020, 0x0002, 0xB01B, 0x0000, 0x0000, 0x0002, /*1B01B*/
  0xFBC3, 0x0020, 0x0002, 0xB01C, 0x0000, 0x0000, 0x0002, /*1B01C*/
  0xFBC3, 0x0020, 0x0002, 0xB01D, 0x0000, 0x0000, 0x0002, /*1B01D*/
  0xFBC3, 0x0020, 0x0002, 0xB01E, 0x0000, 0x0000, 0x0002, /*1B01E*/
  0xFBC3, 0x0020, 0x0002, 0xB01F, 0x0000, 0x0000, 0x0002, /*1B01F*/
  0xFBC3, 0x0020, 0x0002, 0xB020, 0x0000, 0x0000, 0x0002, /*1B020*/
  0xFBC3, 0x0020, 0x0002, 0xB021, 0x0000, 0x0000, 0x0002, /*1B021*/
  0xFBC3, 0x0020, 0x0002, 0xB022, 0x0000, 0x0000, 0x0002, /*1B022*/
  0xFBC3, 0x0020, 0x0002, 0xB023, 0x0000, 0x0000, 0x0002, /*1B023*/
  0xFBC3, 0x0020, 0x0002, 0xB024, 0x0000, 0x0000, 0x0002, /*1B024*/
  0xFBC3, 0x0020, 0x0002, 0xB025, 0x0000, 0x0000, 0x0002, /*1B025*/
  0xFBC3, 0x0020, 0x0002, 0xB026, 0x0000, 0x0000, 0x0002, /*1B026*/
  0xFBC3, 0x0020, 0x0002, 0xB027, 0x0000, 0x0000, 0x0002, /*1B027*/
  0xFBC3, 0x0020, 0x0002, 0xB028, 0x0000, 0x0000, 0x0002, /*1B028*/
  0xFBC3, 0x0020, 0x0002, 0xB029, 0x0000, 0x0000, 0x0002, /*1B029*/
  0xFBC3, 0x0020, 0x0002, 0xB02A, 0x0000, 0x0000, 0x0002, /*1B02A*/
  0xFBC3, 0x0020, 0x0002, 0xB02B, 0x0000, 0x0000, 0x0002, /*1B02B*/
  0xFBC3, 0x0020, 0x0002, 0xB02C, 0x0000, 0x0000, 0x0002, /*1B02C*/
  0xFBC3, 0x0020, 0x0002, 0xB02D, 0x0000, 0x0000, 0x0002, /*1B02D*/
  0xFBC3, 0x0020, 0x0002, 0xB02E, 0x0000, 0x0000, 0x0002, /*1B02E*/
  0xFBC3, 0x0020, 0x0002, 0xB02F, 0x0000, 0x0000, 0x0002, /*1B02F*/
  0xFBC3, 0x0020, 0x0002, 0xB030, 0x0000, 0x0000, 0x0002, /*1B030*/
  0xFBC3, 0x0020, 0x0002, 0xB031, 0x0000, 0x0000, 0x0002, /*1B031*/
  0xFBC3, 0x0020, 0x0002, 0xB032, 0x0000, 0x0000, 0x0002, /*1B032*/
  0xFBC3, 0x0020, 0x0002, 0xB033, 0x0000, 0x0000, 0x0002, /*1B033*/
  0xFBC3, 0x0020, 0x0002, 0xB034, 0x0000, 0x0000, 0x0002, /*1B034*/
  0xFBC3, 0x0020, 0x0002, 0xB035, 0x0000, 0x0000, 0x0002, /*1B035*/
  0xFBC3, 0x0020, 0x0002, 0xB036, 0x0000, 0x0000, 0x0002, /*1B036*/
  0xFBC3, 0x0020, 0x0002, 0xB037, 0x0000, 0x0000, 0x0002, /*1B037*/
  0xFBC3, 0x0020, 0x0002, 0xB038, 0x0000, 0x0000, 0x0002, /*1B038*/
  0xFBC3, 0x0020, 0x0002, 0xB039, 0x0000, 0x0000, 0x0002, /*1B039*/
  0xFBC3, 0x0020, 0x0002, 0xB03A, 0x0000, 0x0000, 0x0002, /*1B03A*/
  0xFBC3, 0x0020, 0x0002, 0xB03B, 0x0000, 0x0000, 0x0002, /*1B03B*/
  0xFBC3, 0x0020, 0x0002, 0xB03C, 0x0000, 0x0000, 0x0002, /*1B03C*/
  0xFBC3, 0x0020, 0x0002, 0xB03D, 0x0000, 0x0000, 0x0002, /*1B03D*/
  0xFBC3, 0x0020, 0x0002, 0xB03E, 0x0000, 0x0000, 0x0002, /*1B03E*/
  0xFBC3, 0x0020, 0x0002, 0xB03F, 0x0000, 0x0000, 0x0002, /*1B03F*/
  0xFBC3, 0x0020, 0x0002, 0xB040, 0x0000, 0x0000, 0x0002, /*1B040*/
  0xFBC3, 0x0020, 0x0002, 0xB041, 0x0000, 0x0000, 0x0002, /*1B041*/
  0xFBC3, 0x0020, 0x0002, 0xB042, 0x0000, 0x0000, 0x0002, /*1B042*/
  0xFBC3, 0x0020, 0x0002, 0xB043, 0x0000, 0x0000, 0x0002, /*1B043*/
  0xFBC3, 0x0020, 0x0002, 0xB044, 0x0000, 0x0000, 0x0002, /*1B044*/
  0xFBC3, 0x0020, 0x0002, 0xB045, 0x0000, 0x0000, 0x0002, /*1B045*/
  0xFBC3, 0x0020, 0x0002, 0xB046, 0x0000, 0x0000, 0x0002, /*1B046*/
  0xFBC3, 0x0020, 0x0002, 0xB047, 0x0000, 0x0000, 0x0002, /*1B047*/
  0xFBC3, 0x0020, 0x0002, 0xB048, 0x0000, 0x0000, 0x0002, /*1B048*/
  0xFBC3, 0x0020, 0x0002, 0xB049, 0x0000, 0x0000, 0x0002, /*1B049*/
  0xFBC3, 0x0020, 0x0002, 0xB04A, 0x0000, 0x0000, 0x0002, /*1B04A*/
  0xFBC3, 0x0020, 0x0002, 0xB04B, 0x0000, 0x0000, 0x0002, /*1B04B*/
  0xFBC3, 0x0020, 0x0002, 0xB04C, 0x0000, 0x0000, 0x0002, /*1B04C*/
  0xFBC3, 0x0020, 0x0002, 0xB04D, 0x0000, 0x0000, 0x0002, /*1B04D*/
  0xFBC3, 0x0020, 0x0002, 0xB04E, 0x0000, 0x0000, 0x0002, /*1B04E*/
  0xFBC3, 0x0020, 0x0002, 0xB04F, 0x0000, 0x0000, 0x0002, /*1B04F*/
  0xFBC3, 0x0020, 0x0002, 0xB050, 0x0000, 0x0000, 0x0002, /*1B050*/
  0xFBC3, 0x0020, 0x0002, 0xB051, 0x0000, 0x0000, 0x0002, /*1B051*/
  0xFBC3, 0x0020, 0x0002, 0xB052, 0x0000, 0x0000, 0x0002, /*1B052*/
  0xFBC3, 0x0020, 0x0002, 0xB053, 0x0000, 0x0000, 0x0002, /*1B053*/
  0xFBC3, 0x0020, 0x0002, 0xB054, 0x0000, 0x0000, 0x0002, /*1B054*/
  0xFBC3, 0x0020, 0x0002, 0xB055, 0x0000, 0x0000, 0x0002, /*1B055*/
  0xFBC3, 0x0020, 0x0002, 0xB056, 0x0000, 0x0000, 0x0002, /*1B056*/
  0xFBC3, 0x0020, 0x0002, 0xB057, 0x0000, 0x0000, 0x0002, /*1B057*/
  0xFBC3, 0x0020, 0x0002, 0xB058, 0x0000, 0x0000, 0x0002, /*1B058*/
  0xFBC3, 0x0020, 0x0002, 0xB059, 0x0000, 0x0000, 0x0002, /*1B059*/
  0xFBC3, 0x0020, 0x0002, 0xB05A, 0x0000, 0x0000, 0x0002, /*1B05A*/
  0xFBC3, 0x0020, 0x0002, 0xB05B, 0x0000, 0x0000, 0x0002, /*1B05B*/
  0xFBC3, 0x0020, 0x0002, 0xB05C, 0x0000, 0x0000, 0x0002, /*1B05C*/
  0xFBC3, 0x0020, 0x0002, 0xB05D, 0x0000, 0x0000, 0x0002, /*1B05D*/
  0xFBC3, 0x0020, 0x0002, 0xB05E, 0x0000, 0x0000, 0x0002, /*1B05E*/
  0xFBC3, 0x0020, 0x0002, 0xB05F, 0x0000, 0x0000, 0x0002, /*1B05F*/
  0xFBC3, 0x0020, 0x0002, 0xB060, 0x0000, 0x0000, 0x0002, /*1B060*/
  0xFBC3, 0x0020, 0x0002, 0xB061, 0x0000, 0x0000, 0x0002, /*1B061*/
  0xFBC3, 0x0020, 0x0002, 0xB062, 0x0000, 0x0000, 0x0002, /*1B062*/
  0xFBC3, 0x0020, 0x0002, 0xB063, 0x0000, 0x0000, 0x0002, /*1B063*/
  0xFBC3, 0x0020, 0x0002, 0xB064, 0x0000, 0x0000, 0x0002, /*1B064*/
  0xFBC3, 0x0020, 0x0002, 0xB065, 0x0000, 0x0000, 0x0002, /*1B065*/
  0xFBC3, 0x0020, 0x0002, 0xB066, 0x0000, 0x0000, 0x0002, /*1B066*/
  0xFBC3, 0x0020, 0x0002, 0xB067, 0x0000, 0x0000, 0x0002, /*1B067*/
  0xFBC3, 0x0020, 0x0002, 0xB068, 0x0000, 0x0000, 0x0002, /*1B068*/
  0xFBC3, 0x0020, 0x0002, 0xB069, 0x0000, 0x0000, 0x0002, /*1B069*/
  0xFBC3, 0x0020, 0x0002, 0xB06A, 0x0000, 0x0000, 0x0002, /*1B06A*/
  0xFBC3, 0x0020, 0x0002, 0xB06B, 0x0000, 0x0000, 0x0002, /*1B06B*/
  0xFBC3, 0x0020, 0x0002, 0xB06C, 0x0000, 0x0000, 0x0002, /*1B06C*/
  0xFBC3, 0x0020, 0x0002, 0xB06D, 0x0000, 0x0000, 0x0002, /*1B06D*/
  0xFBC3, 0x0020, 0x0002, 0xB06E, 0x0000, 0x0000, 0x0002, /*1B06E*/
  0xFBC3, 0x0020, 0x0002, 0xB06F, 0x0000, 0x0000, 0x0002, /*1B06F*/
  0xFBC3, 0x0020, 0x0002, 0xB070, 0x0000, 0x0000, 0x0002, /*1B070*/
  0xFBC3, 0x0020, 0x0002, 0xB071, 0x0000, 0x0000, 0x0002, /*1B071*/
  0xFBC3, 0x0020, 0x0002, 0xB072, 0x0000, 0x0000, 0x0002, /*1B072*/
  0xFBC3, 0x0020, 0x0002, 0xB073, 0x0000, 0x0000, 0x0002, /*1B073*/
  0xFBC3, 0x0020, 0x0002, 0xB074, 0x0000, 0x0000, 0x0002, /*1B074*/
  0xFBC3, 0x0020, 0x0002, 0xB075, 0x0000, 0x0000, 0x0002, /*1B075*/
  0xFBC3, 0x0020, 0x0002, 0xB076, 0x0000, 0x0000, 0x0002, /*1B076*/
  0xFBC3, 0x0020, 0x0002, 0xB077, 0x0000, 0x0000, 0x0002, /*1B077*/
  0xFBC3, 0x0020, 0x0002, 0xB078, 0x0000, 0x0000, 0x0002, /*1B078*/
  0xFBC3, 0x0020, 0x0002, 0xB079, 0x0000, 0x0000, 0x0002, /*1B079*/
  0xFBC3, 0x0020, 0x0002, 0xB07A, 0x0000, 0x0000, 0x0002, /*1B07A*/
  0xFBC3, 0x0020, 0x0002, 0xB07B, 0x0000, 0x0000, 0x0002, /*1B07B*/
  0xFBC3, 0x0020, 0x0002, 0xB07C, 0x0000, 0x0000, 0x0002, /*1B07C*/
  0xFBC3, 0x0020, 0x0002, 0xB07D, 0x0000, 0x0000, 0x0002, /*1B07D*/
  0xFBC3, 0x0020, 0x0002, 0xB07E, 0x0000, 0x0000, 0x0002, /*1B07E*/
  0xFBC3, 0x0020, 0x0002, 0xB07F, 0x0000, 0x0000, 0x0002, /*1B07F*/
  0xFBC3, 0x0020, 0x0002, 0xB080, 0x0000, 0x0000, 0x0002, /*1B080*/
  0xFBC3, 0x0020, 0x0002, 0xB081, 0x0000, 0x0000, 0x0002, /*1B081*/
  0xFBC3, 0x0020, 0x0002, 0xB082, 0x0000, 0x0000, 0x0002, /*1B082*/
  0xFBC3, 0x0020, 0x0002, 0xB083, 0x0000, 0x0000, 0x0002, /*1B083*/
  0xFBC3, 0x0020, 0x0002, 0xB084, 0x0000, 0x0000, 0x0002, /*1B084*/
  0xFBC3, 0x0020, 0x0002, 0xB085, 0x0000, 0x0000, 0x0002, /*1B085*/
  0xFBC3, 0x0020, 0x0002, 0xB086, 0x0000, 0x0000, 0x0002, /*1B086*/
  0xFBC3, 0x0020, 0x0002, 0xB087, 0x0000, 0x0000, 0x0002, /*1B087*/
  0xFBC3, 0x0020, 0x0002, 0xB088, 0x0000, 0x0000, 0x0002, /*1B088*/
  0xFBC3, 0x0020, 0x0002, 0xB089, 0x0000, 0x0000, 0x0002, /*1B089*/
  0xFBC3, 0x0020, 0x0002, 0xB08A, 0x0000, 0x0000, 0x0002, /*1B08A*/
  0xFBC3, 0x0020, 0x0002, 0xB08B, 0x0000, 0x0000, 0x0002, /*1B08B*/
  0xFBC3, 0x0020, 0x0002, 0xB08C, 0x0000, 0x0000, 0x0002, /*1B08C*/
  0xFBC3, 0x0020, 0x0002, 0xB08D, 0x0000, 0x0000, 0x0002, /*1B08D*/
  0xFBC3, 0x0020, 0x0002, 0xB08E, 0x0000, 0x0000, 0x0002, /*1B08E*/
  0xFBC3, 0x0020, 0x0002, 0xB08F, 0x0000, 0x0000, 0x0002, /*1B08F*/
  0xFBC3, 0x0020, 0x0002, 0xB090, 0x0000, 0x0000, 0x0002, /*1B090*/
  0xFBC3, 0x0020, 0x0002, 0xB091, 0x0000, 0x0000, 0x0002, /*1B091*/
  0xFBC3, 0x0020, 0x0002, 0xB092, 0x0000, 0x0000, 0x0002, /*1B092*/
  0xFBC3, 0x0020, 0x0002, 0xB093, 0x0000, 0x0000, 0x0002, /*1B093*/
  0xFBC3, 0x0020, 0x0002, 0xB094, 0x0000, 0x0000, 0x0002, /*1B094*/
  0xFBC3, 0x0020, 0x0002, 0xB095, 0x0000, 0x0000, 0x0002, /*1B095*/
  0xFBC3, 0x0020, 0x0002, 0xB096, 0x0000, 0x0000, 0x0002, /*1B096*/
  0xFBC3, 0x0020, 0x0002, 0xB097, 0x0000, 0x0000, 0x0002, /*1B097*/
  0xFBC3, 0x0020, 0x0002, 0xB098, 0x0000, 0x0000, 0x0002, /*1B098*/
  0xFBC3, 0x0020, 0x0002, 0xB099, 0x0000, 0x0000, 0x0002, /*1B099*/
  0xFBC3, 0x0020, 0x0002, 0xB09A, 0x0000, 0x0000, 0x0002, /*1B09A*/
  0xFBC3, 0x0020, 0x0002, 0xB09B, 0x0000, 0x0000, 0x0002, /*1B09B*/
  0xFBC3, 0x0020, 0x0002, 0xB09C, 0x0000, 0x0000, 0x0002, /*1B09C*/
  0xFBC3, 0x0020, 0x0002, 0xB09D, 0x0000, 0x0000, 0x0002, /*1B09D*/
  0xFBC3, 0x0020, 0x0002, 0xB09E, 0x0000, 0x0000, 0x0002, /*1B09E*/
  0xFBC3, 0x0020, 0x0002, 0xB09F, 0x0000, 0x0000, 0x0002, /*1B09F*/
  0xFBC3, 0x0020, 0x0002, 0xB0A0, 0x0000, 0x0000, 0x0002, /*1B0A0*/
  0xFBC3, 0x0020, 0x0002, 0xB0A1, 0x0000, 0x0000, 0x0002, /*1B0A1*/
  0xFBC3, 0x0020, 0x0002, 0xB0A2, 0x0000, 0x0000, 0x0002, /*1B0A2*/
  0xFBC3, 0x0020, 0x0002, 0xB0A3, 0x0000, 0x0000, 0x0002, /*1B0A3*/
  0xFBC3, 0x0020, 0x0002, 0xB0A4, 0x0000, 0x0000, 0x0002, /*1B0A4*/
  0xFBC3, 0x0020, 0x0002, 0xB0A5, 0x0000, 0x0000, 0x0002, /*1B0A5*/
  0xFBC3, 0x0020, 0x0002, 0xB0A6, 0x0000, 0x0000, 0x0002, /*1B0A6*/
  0xFBC3, 0x0020, 0x0002, 0xB0A7, 0x0000, 0x0000, 0x0002, /*1B0A7*/
  0xFBC3, 0x0020, 0x0002, 0xB0A8, 0x0000, 0x0000, 0x0002, /*1B0A8*/
  0xFBC3, 0x0020, 0x0002, 0xB0A9, 0x0000, 0x0000, 0x0002, /*1B0A9*/
  0xFBC3, 0x0020, 0x0002, 0xB0AA, 0x0000, 0x0000, 0x0002, /*1B0AA*/
  0xFBC3, 0x0020, 0x0002, 0xB0AB, 0x0000, 0x0000, 0x0002, /*1B0AB*/
  0xFBC3, 0x0020, 0x0002, 0xB0AC, 0x0000, 0x0000, 0x0002, /*1B0AC*/
  0xFBC3, 0x0020, 0x0002, 0xB0AD, 0x0000, 0x0000, 0x0002, /*1B0AD*/
  0xFBC3, 0x0020, 0x0002, 0xB0AE, 0x0000, 0x0000, 0x0002, /*1B0AE*/
  0xFBC3, 0x0020, 0x0002, 0xB0AF, 0x0000, 0x0000, 0x0002, /*1B0AF*/
  0xFBC3, 0x0020, 0x0002, 0xB0B0, 0x0000, 0x0000, 0x0002, /*1B0B0*/
  0xFBC3, 0x0020, 0x0002, 0xB0B1, 0x0000, 0x0000, 0x0002, /*1B0B1*/
  0xFBC3, 0x0020, 0x0002, 0xB0B2, 0x0000, 0x0000, 0x0002, /*1B0B2*/
  0xFBC3, 0x0020, 0x0002, 0xB0B3, 0x0000, 0x0000, 0x0002, /*1B0B3*/
  0xFBC3, 0x0020, 0x0002, 0xB0B4, 0x0000, 0x0000, 0x0002, /*1B0B4*/
  0xFBC3, 0x0020, 0x0002, 0xB0B5, 0x0000, 0x0000, 0x0002, /*1B0B5*/
  0xFBC3, 0x0020, 0x0002, 0xB0B6, 0x0000, 0x0000, 0x0002, /*1B0B6*/
  0xFBC3, 0x0020, 0x0002, 0xB0B7, 0x0000, 0x0000, 0x0002, /*1B0B7*/
  0xFBC3, 0x0020, 0x0002, 0xB0B8, 0x0000, 0x0000, 0x0002, /*1B0B8*/
  0xFBC3, 0x0020, 0x0002, 0xB0B9, 0x0000, 0x0000, 0x0002, /*1B0B9*/
  0xFBC3, 0x0020, 0x0002, 0xB0BA, 0x0000, 0x0000, 0x0002, /*1B0BA*/
  0xFBC3, 0x0020, 0x0002, 0xB0BB, 0x0000, 0x0000, 0x0002, /*1B0BB*/
  0xFBC3, 0x0020, 0x0002, 0xB0BC, 0x0000, 0x0000, 0x0002, /*1B0BC*/
  0xFBC3, 0x0020, 0x0002, 0xB0BD, 0x0000, 0x0000, 0x0002, /*1B0BD*/
  0xFBC3, 0x0020, 0x0002, 0xB0BE, 0x0000, 0x0000, 0x0002, /*1B0BE*/
  0xFBC3, 0x0020, 0x0002, 0xB0BF, 0x0000, 0x0000, 0x0002, /*1B0BF*/
  0xFBC3, 0x0020, 0x0002, 0xB0C0, 0x0000, 0x0000, 0x0002, /*1B0C0*/
  0xFBC3, 0x0020, 0x0002, 0xB0C1, 0x0000, 0x0000, 0x0002, /*1B0C1*/
  0xFBC3, 0x0020, 0x0002, 0xB0C2, 0x0000, 0x0000, 0x0002, /*1B0C2*/
  0xFBC3, 0x0020, 0x0002, 0xB0C3, 0x0000, 0x0000, 0x0002, /*1B0C3*/
  0xFBC3, 0x0020, 0x0002, 0xB0C4, 0x0000, 0x0000, 0x0002, /*1B0C4*/
  0xFBC3, 0x0020, 0x0002, 0xB0C5, 0x0000, 0x0000, 0x0002, /*1B0C5*/
  0xFBC3, 0x0020, 0x0002, 0xB0C6, 0x0000, 0x0000, 0x0002, /*1B0C6*/
  0xFBC3, 0x0020, 0x0002, 0xB0C7, 0x0000, 0x0000, 0x0002, /*1B0C7*/
  0xFBC3, 0x0020, 0x0002, 0xB0C8, 0x0000, 0x0000, 0x0002, /*1B0C8*/
  0xFBC3, 0x0020, 0x0002, 0xB0C9, 0x0000, 0x0000, 0x0002, /*1B0C9*/
  0xFBC3, 0x0020, 0x0002, 0xB0CA, 0x0000, 0x0000, 0x0002, /*1B0CA*/
  0xFBC3, 0x0020, 0x0002, 0xB0CB, 0x0000, 0x0000, 0x0002, /*1B0CB*/
  0xFBC3, 0x0020, 0x0002, 0xB0CC, 0x0000, 0x0000, 0x0002, /*1B0CC*/
  0xFBC3, 0x0020, 0x0002, 0xB0CD, 0x0000, 0x0000, 0x0002, /*1B0CD*/
  0xFBC3, 0x0020, 0x0002, 0xB0CE, 0x0000, 0x0000, 0x0002, /*1B0CE*/
  0xFBC3, 0x0020, 0x0002, 0xB0CF, 0x0000, 0x0000, 0x0002, /*1B0CF*/
  0xFBC3, 0x0020, 0x0002, 0xB0D0, 0x0000, 0x0000, 0x0002, /*1B0D0*/
  0xFBC3, 0x0020, 0x0002, 0xB0D1, 0x0000, 0x0000, 0x0002, /*1B0D1*/
  0xFBC3, 0x0020, 0x0002, 0xB0D2, 0x0000, 0x0000, 0x0002, /*1B0D2*/
  0xFBC3, 0x0020, 0x0002, 0xB0D3, 0x0000, 0x0000, 0x0002, /*1B0D3*/
  0xFBC3, 0x0020, 0x0002, 0xB0D4, 0x0000, 0x0000, 0x0002, /*1B0D4*/
  0xFBC3, 0x0020, 0x0002, 0xB0D5, 0x0000, 0x0000, 0x0002, /*1B0D5*/
  0xFBC3, 0x0020, 0x0002, 0xB0D6, 0x0000, 0x0000, 0x0002, /*1B0D6*/
  0xFBC3, 0x0020, 0x0002, 0xB0D7, 0x0000, 0x0000, 0x0002, /*1B0D7*/
  0xFBC3, 0x0020, 0x0002, 0xB0D8, 0x0000, 0x0000, 0x0002, /*1B0D8*/
  0xFBC3, 0x0020, 0x0002, 0xB0D9, 0x0000, 0x0000, 0x0002, /*1B0D9*/
  0xFBC3, 0x0020, 0x0002, 0xB0DA, 0x0000, 0x0000, 0x0002, /*1B0DA*/
  0xFBC3, 0x0020, 0x0002, 0xB0DB, 0x0000, 0x0000, 0x0002, /*1B0DB*/
  0xFBC3, 0x0020, 0x0002, 0xB0DC, 0x0000, 0x0000, 0x0002, /*1B0DC*/
  0xFBC3, 0x0020, 0x0002, 0xB0DD, 0x0000, 0x0000, 0x0002, /*1B0DD*/
  0xFBC3, 0x0020, 0x0002, 0xB0DE, 0x0000, 0x0000, 0x0002, /*1B0DE*/
  0xFBC3, 0x0020, 0x0002, 0xB0DF, 0x0000, 0x0000, 0x0002, /*1B0DF*/
  0xFBC3, 0x0020, 0x0002, 0xB0E0, 0x0000, 0x0000, 0x0002, /*1B0E0*/
  0xFBC3, 0x0020, 0x0002, 0xB0E1, 0x0000, 0x0000, 0x0002, /*1B0E1*/
  0xFBC3, 0x0020, 0x0002, 0xB0E2, 0x0000, 0x0000, 0x0002, /*1B0E2*/
  0xFBC3, 0x0020, 0x0002, 0xB0E3, 0x0000, 0x0000, 0x0002, /*1B0E3*/
  0xFBC3, 0x0020, 0x0002, 0xB0E4, 0x0000, 0x0000, 0x0002, /*1B0E4*/
  0xFBC3, 0x0020, 0x0002, 0xB0E5, 0x0000, 0x0000, 0x0002, /*1B0E5*/
  0xFBC3, 0x0020, 0x0002, 0xB0E6, 0x0000, 0x0000, 0x0002, /*1B0E6*/
  0xFBC3, 0x0020, 0x0002, 0xB0E7, 0x0000, 0x0000, 0x0002, /*1B0E7*/
  0xFBC3, 0x0020, 0x0002, 0xB0E8, 0x0000, 0x0000, 0x0002, /*1B0E8*/
  0xFBC3, 0x0020, 0x0002, 0xB0E9, 0x0000, 0x0000, 0x0002, /*1B0E9*/
  0xFBC3, 0x0020, 0x0002, 0xB0EA, 0x0000, 0x0000, 0x0002, /*1B0EA*/
  0xFBC3, 0x0020, 0x0002, 0xB0EB, 0x0000, 0x0000, 0x0002, /*1B0EB*/
  0xFBC3, 0x0020, 0x0002, 0xB0EC, 0x0000, 0x0000, 0x0002, /*1B0EC*/
  0xFBC3, 0x0020, 0x0002, 0xB0ED, 0x0000, 0x0000, 0x0002, /*1B0ED*/
  0xFBC3, 0x0020, 0x0002, 0xB0EE, 0x0000, 0x0000, 0x0002, /*1B0EE*/
  0xFBC3, 0x0020, 0x0002, 0xB0EF, 0x0000, 0x0000, 0x0002, /*1B0EF*/
  0xFBC3, 0x0020, 0x0002, 0xB0F0, 0x0000, 0x0000, 0x0002, /*1B0F0*/
  0xFBC3, 0x0020, 0x0002, 0xB0F1, 0x0000, 0x0000, 0x0002, /*1B0F1*/
  0xFBC3, 0x0020, 0x0002, 0xB0F2, 0x0000, 0x0000, 0x0002, /*1B0F2*/
  0xFBC3, 0x0020, 0x0002, 0xB0F3, 0x0000, 0x0000, 0x0002, /*1B0F3*/
  0xFBC3, 0x0020, 0x0002, 0xB0F4, 0x0000, 0x0000, 0x0002, /*1B0F4*/
  0xFBC3, 0x0020, 0x0002, 0xB0F5, 0x0000, 0x0000, 0x0002, /*1B0F5*/
  0xFBC3, 0x0020, 0x0002, 0xB0F6, 0x0000, 0x0000, 0x0002, /*1B0F6*/
  0xFBC3, 0x0020, 0x0002, 0xB0F7, 0x0000, 0x0000, 0x0002, /*1B0F7*/
  0xFBC3, 0x0020, 0x0002, 0xB0F8, 0x0000, 0x0000, 0x0002, /*1B0F8*/
  0xFBC3, 0x0020, 0x0002, 0xB0F9, 0x0000, 0x0000, 0x0002, /*1B0F9*/
  0xFBC3, 0x0020, 0x0002, 0xB0FA, 0x0000, 0x0000, 0x0002, /*1B0FA*/
  0xFBC3, 0x0020, 0x0002, 0xB0FB, 0x0000, 0x0000, 0x0002, /*1B0FB*/
  0xFBC3, 0x0020, 0x0002, 0xB0FC, 0x0000, 0x0000, 0x0002, /*1B0FC*/
  0xFBC3, 0x0020, 0x0002, 0xB0FD, 0x0000, 0x0000, 0x0002, /*1B0FD*/
  0xFBC3, 0x0020, 0x0002, 0xB0FE, 0x0000, 0x0000, 0x0002, /*1B0FE*/
  0xFBC3, 0x0020, 0x0002, 0xB0FF, 0x0000, 0x0000, 0x0002  /*1B0FF*/
};

uint16 uca900_p1BC[]= {
  0x44AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC00*/
  0x44AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC01*/
  0x44AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC02*/
  0x44AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC03*/
  0x44AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC04*/
  0x44AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC05*/
  0x44B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC06*/
  0x44B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC07*/
  0x44B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC08*/
  0x44B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC09*/
  0x44B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC0A*/
  0x44B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC0B*/
  0x44B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC0C*/
  0x44B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC0D*/
  0x44B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC0E*/
  0x44B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC0F*/
  0x44BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC10*/
  0x44BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC11*/
  0x44BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC12*/
  0x44BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC13*/
  0x44BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC14*/
  0x44BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC15*/
  0x44C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC16*/
  0x44C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC17*/
  0x44C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC18*/
  0x44C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC19*/
  0x44C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC1A*/
  0x44C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC1B*/
  0x44C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC1C*/
  0x44C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC1D*/
  0x44C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC1E*/
  0x44C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC1F*/
  0x44CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC20*/
  0x44CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC21*/
  0x44CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC22*/
  0x44CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC23*/
  0x44CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC24*/
  0x44CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC25*/
  0x44D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC26*/
  0x44D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC27*/
  0x44D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC28*/
  0x44D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC29*/
  0x44D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC2A*/
  0x44D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC2B*/
  0x44D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC2C*/
  0x44D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC2D*/
  0x44D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC2E*/
  0x44D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC2F*/
  0x44DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC30*/
  0x44DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC31*/
  0x44DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC32*/
  0x44DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC33*/
  0x44DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC34*/
  0x44DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC35*/
  0x44E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC36*/
  0x44E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC37*/
  0x44E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC38*/
  0x44E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC39*/
  0x44E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC3A*/
  0x44E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC3B*/
  0x44E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC3C*/
  0x44E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC3D*/
  0x44E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC3E*/
  0x44E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC3F*/
  0x44EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC40*/
  0x44EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC41*/
  0x44EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC42*/
  0x44ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC43*/
  0x44EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC44*/
  0x44EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC45*/
  0x44F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC46*/
  0x44F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC47*/
  0x44F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC48*/
  0x44F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC49*/
  0x44F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC4A*/
  0x44F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC4B*/
  0x44F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC4C*/
  0x44F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC4D*/
  0x44F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC4E*/
  0x44F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC4F*/
  0x44FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC50*/
  0x44FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC51*/
  0x44FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC52*/
  0x44FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC53*/
  0x44FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC54*/
  0x44FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC55*/
  0x4500, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC56*/
  0x4501, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC57*/
  0x4502, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC58*/
  0x4503, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC59*/
  0x4504, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC5A*/
  0x4505, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC5B*/
  0x4506, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC5C*/
  0x4507, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC5D*/
  0x4508, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC5E*/
  0x4509, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC5F*/
  0x450A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC60*/
  0x450B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC61*/
  0x450C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC62*/
  0x450D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC63*/
  0x450E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC64*/
  0x450F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC65*/
  0x4510, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC66*/
  0x4511, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC67*/
  0x4512, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC68*/
  0x4513, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC69*/
  0x4514, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC6A*/
  0xFBC3, 0x0020, 0x0002, 0xBC6B, 0x0000, 0x0000, 0x0002, /*1BC6B*/
  0xFBC3, 0x0020, 0x0002, 0xBC6C, 0x0000, 0x0000, 0x0002, /*1BC6C*/
  0xFBC3, 0x0020, 0x0002, 0xBC6D, 0x0000, 0x0000, 0x0002, /*1BC6D*/
  0xFBC3, 0x0020, 0x0002, 0xBC6E, 0x0000, 0x0000, 0x0002, /*1BC6E*/
  0xFBC3, 0x0020, 0x0002, 0xBC6F, 0x0000, 0x0000, 0x0002, /*1BC6F*/
  0x4515, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC70*/
  0x4516, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC71*/
  0x4517, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC72*/
  0x4518, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC73*/
  0x4519, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC74*/
  0x451A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC75*/
  0x451B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC76*/
  0x451C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC77*/
  0x451D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC78*/
  0x451E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC79*/
  0x451F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC7A*/
  0x4520, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC7B*/
  0x4521, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC7C*/
  0xFBC3, 0x0020, 0x0002, 0xBC7D, 0x0000, 0x0000, 0x0002, /*1BC7D*/
  0xFBC3, 0x0020, 0x0002, 0xBC7E, 0x0000, 0x0000, 0x0002, /*1BC7E*/
  0xFBC3, 0x0020, 0x0002, 0xBC7F, 0x0000, 0x0000, 0x0002, /*1BC7F*/
  0x4522, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC80*/
  0x4523, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC81*/
  0x4524, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC82*/
  0x4525, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC83*/
  0x4526, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC84*/
  0x4527, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC85*/
  0x4528, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC86*/
  0x4529, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC87*/
  0x452A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC88*/
  0xFBC3, 0x0020, 0x0002, 0xBC89, 0x0000, 0x0000, 0x0002, /*1BC89*/
  0xFBC3, 0x0020, 0x0002, 0xBC8A, 0x0000, 0x0000, 0x0002, /*1BC8A*/
  0xFBC3, 0x0020, 0x0002, 0xBC8B, 0x0000, 0x0000, 0x0002, /*1BC8B*/
  0xFBC3, 0x0020, 0x0002, 0xBC8C, 0x0000, 0x0000, 0x0002, /*1BC8C*/
  0xFBC3, 0x0020, 0x0002, 0xBC8D, 0x0000, 0x0000, 0x0002, /*1BC8D*/
  0xFBC3, 0x0020, 0x0002, 0xBC8E, 0x0000, 0x0000, 0x0002, /*1BC8E*/
  0xFBC3, 0x0020, 0x0002, 0xBC8F, 0x0000, 0x0000, 0x0002, /*1BC8F*/
  0x452B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC90*/
  0x452C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC91*/
  0x452D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC92*/
  0x452E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC93*/
  0x452F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC94*/
  0x4530, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC95*/
  0x4531, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC96*/
  0x4532, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC97*/
  0x4533, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC98*/
  0x4534, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC99*/
  0xFBC3, 0x0020, 0x0002, 0xBC9A, 0x0000, 0x0000, 0x0002, /*1BC9A*/
  0xFBC3, 0x0020, 0x0002, 0xBC9B, 0x0000, 0x0000, 0x0002, /*1BC9B*/
  0x11C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC9C*/
  0x0000, 0x0033, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC9D*/
  0x0000, 0x0035, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC9E*/
  0x0289, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1BC9F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1BCA0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1BCA1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1BCA2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1BCA3*/
  0xFBC3, 0x0020, 0x0002, 0xBCA4, 0x0000, 0x0000, 0x0002, /*1BCA4*/
  0xFBC3, 0x0020, 0x0002, 0xBCA5, 0x0000, 0x0000, 0x0002, /*1BCA5*/
  0xFBC3, 0x0020, 0x0002, 0xBCA6, 0x0000, 0x0000, 0x0002, /*1BCA6*/
  0xFBC3, 0x0020, 0x0002, 0xBCA7, 0x0000, 0x0000, 0x0002, /*1BCA7*/
  0xFBC3, 0x0020, 0x0002, 0xBCA8, 0x0000, 0x0000, 0x0002, /*1BCA8*/
  0xFBC3, 0x0020, 0x0002, 0xBCA9, 0x0000, 0x0000, 0x0002, /*1BCA9*/
  0xFBC3, 0x0020, 0x0002, 0xBCAA, 0x0000, 0x0000, 0x0002, /*1BCAA*/
  0xFBC3, 0x0020, 0x0002, 0xBCAB, 0x0000, 0x0000, 0x0002, /*1BCAB*/
  0xFBC3, 0x0020, 0x0002, 0xBCAC, 0x0000, 0x0000, 0x0002, /*1BCAC*/
  0xFBC3, 0x0020, 0x0002, 0xBCAD, 0x0000, 0x0000, 0x0002, /*1BCAD*/
  0xFBC3, 0x0020, 0x0002, 0xBCAE, 0x0000, 0x0000, 0x0002, /*1BCAE*/
  0xFBC3, 0x0020, 0x0002, 0xBCAF, 0x0000, 0x0000, 0x0002, /*1BCAF*/
  0xFBC3, 0x0020, 0x0002, 0xBCB0, 0x0000, 0x0000, 0x0002, /*1BCB0*/
  0xFBC3, 0x0020, 0x0002, 0xBCB1, 0x0000, 0x0000, 0x0002, /*1BCB1*/
  0xFBC3, 0x0020, 0x0002, 0xBCB2, 0x0000, 0x0000, 0x0002, /*1BCB2*/
  0xFBC3, 0x0020, 0x0002, 0xBCB3, 0x0000, 0x0000, 0x0002, /*1BCB3*/
  0xFBC3, 0x0020, 0x0002, 0xBCB4, 0x0000, 0x0000, 0x0002, /*1BCB4*/
  0xFBC3, 0x0020, 0x0002, 0xBCB5, 0x0000, 0x0000, 0x0002, /*1BCB5*/
  0xFBC3, 0x0020, 0x0002, 0xBCB6, 0x0000, 0x0000, 0x0002, /*1BCB6*/
  0xFBC3, 0x0020, 0x0002, 0xBCB7, 0x0000, 0x0000, 0x0002, /*1BCB7*/
  0xFBC3, 0x0020, 0x0002, 0xBCB8, 0x0000, 0x0000, 0x0002, /*1BCB8*/
  0xFBC3, 0x0020, 0x0002, 0xBCB9, 0x0000, 0x0000, 0x0002, /*1BCB9*/
  0xFBC3, 0x0020, 0x0002, 0xBCBA, 0x0000, 0x0000, 0x0002, /*1BCBA*/
  0xFBC3, 0x0020, 0x0002, 0xBCBB, 0x0000, 0x0000, 0x0002, /*1BCBB*/
  0xFBC3, 0x0020, 0x0002, 0xBCBC, 0x0000, 0x0000, 0x0002, /*1BCBC*/
  0xFBC3, 0x0020, 0x0002, 0xBCBD, 0x0000, 0x0000, 0x0002, /*1BCBD*/
  0xFBC3, 0x0020, 0x0002, 0xBCBE, 0x0000, 0x0000, 0x0002, /*1BCBE*/
  0xFBC3, 0x0020, 0x0002, 0xBCBF, 0x0000, 0x0000, 0x0002, /*1BCBF*/
  0xFBC3, 0x0020, 0x0002, 0xBCC0, 0x0000, 0x0000, 0x0002, /*1BCC0*/
  0xFBC3, 0x0020, 0x0002, 0xBCC1, 0x0000, 0x0000, 0x0002, /*1BCC1*/
  0xFBC3, 0x0020, 0x0002, 0xBCC2, 0x0000, 0x0000, 0x0002, /*1BCC2*/
  0xFBC3, 0x0020, 0x0002, 0xBCC3, 0x0000, 0x0000, 0x0002, /*1BCC3*/
  0xFBC3, 0x0020, 0x0002, 0xBCC4, 0x0000, 0x0000, 0x0002, /*1BCC4*/
  0xFBC3, 0x0020, 0x0002, 0xBCC5, 0x0000, 0x0000, 0x0002, /*1BCC5*/
  0xFBC3, 0x0020, 0x0002, 0xBCC6, 0x0000, 0x0000, 0x0002, /*1BCC6*/
  0xFBC3, 0x0020, 0x0002, 0xBCC7, 0x0000, 0x0000, 0x0002, /*1BCC7*/
  0xFBC3, 0x0020, 0x0002, 0xBCC8, 0x0000, 0x0000, 0x0002, /*1BCC8*/
  0xFBC3, 0x0020, 0x0002, 0xBCC9, 0x0000, 0x0000, 0x0002, /*1BCC9*/
  0xFBC3, 0x0020, 0x0002, 0xBCCA, 0x0000, 0x0000, 0x0002, /*1BCCA*/
  0xFBC3, 0x0020, 0x0002, 0xBCCB, 0x0000, 0x0000, 0x0002, /*1BCCB*/
  0xFBC3, 0x0020, 0x0002, 0xBCCC, 0x0000, 0x0000, 0x0002, /*1BCCC*/
  0xFBC3, 0x0020, 0x0002, 0xBCCD, 0x0000, 0x0000, 0x0002, /*1BCCD*/
  0xFBC3, 0x0020, 0x0002, 0xBCCE, 0x0000, 0x0000, 0x0002, /*1BCCE*/
  0xFBC3, 0x0020, 0x0002, 0xBCCF, 0x0000, 0x0000, 0x0002, /*1BCCF*/
  0xFBC3, 0x0020, 0x0002, 0xBCD0, 0x0000, 0x0000, 0x0002, /*1BCD0*/
  0xFBC3, 0x0020, 0x0002, 0xBCD1, 0x0000, 0x0000, 0x0002, /*1BCD1*/
  0xFBC3, 0x0020, 0x0002, 0xBCD2, 0x0000, 0x0000, 0x0002, /*1BCD2*/
  0xFBC3, 0x0020, 0x0002, 0xBCD3, 0x0000, 0x0000, 0x0002, /*1BCD3*/
  0xFBC3, 0x0020, 0x0002, 0xBCD4, 0x0000, 0x0000, 0x0002, /*1BCD4*/
  0xFBC3, 0x0020, 0x0002, 0xBCD5, 0x0000, 0x0000, 0x0002, /*1BCD5*/
  0xFBC3, 0x0020, 0x0002, 0xBCD6, 0x0000, 0x0000, 0x0002, /*1BCD6*/
  0xFBC3, 0x0020, 0x0002, 0xBCD7, 0x0000, 0x0000, 0x0002, /*1BCD7*/
  0xFBC3, 0x0020, 0x0002, 0xBCD8, 0x0000, 0x0000, 0x0002, /*1BCD8*/
  0xFBC3, 0x0020, 0x0002, 0xBCD9, 0x0000, 0x0000, 0x0002, /*1BCD9*/
  0xFBC3, 0x0020, 0x0002, 0xBCDA, 0x0000, 0x0000, 0x0002, /*1BCDA*/
  0xFBC3, 0x0020, 0x0002, 0xBCDB, 0x0000, 0x0000, 0x0002, /*1BCDB*/
  0xFBC3, 0x0020, 0x0002, 0xBCDC, 0x0000, 0x0000, 0x0002, /*1BCDC*/
  0xFBC3, 0x0020, 0x0002, 0xBCDD, 0x0000, 0x0000, 0x0002, /*1BCDD*/
  0xFBC3, 0x0020, 0x0002, 0xBCDE, 0x0000, 0x0000, 0x0002, /*1BCDE*/
  0xFBC3, 0x0020, 0x0002, 0xBCDF, 0x0000, 0x0000, 0x0002, /*1BCDF*/
  0xFBC3, 0x0020, 0x0002, 0xBCE0, 0x0000, 0x0000, 0x0002, /*1BCE0*/
  0xFBC3, 0x0020, 0x0002, 0xBCE1, 0x0000, 0x0000, 0x0002, /*1BCE1*/
  0xFBC3, 0x0020, 0x0002, 0xBCE2, 0x0000, 0x0000, 0x0002, /*1BCE2*/
  0xFBC3, 0x0020, 0x0002, 0xBCE3, 0x0000, 0x0000, 0x0002, /*1BCE3*/
  0xFBC3, 0x0020, 0x0002, 0xBCE4, 0x0000, 0x0000, 0x0002, /*1BCE4*/
  0xFBC3, 0x0020, 0x0002, 0xBCE5, 0x0000, 0x0000, 0x0002, /*1BCE5*/
  0xFBC3, 0x0020, 0x0002, 0xBCE6, 0x0000, 0x0000, 0x0002, /*1BCE6*/
  0xFBC3, 0x0020, 0x0002, 0xBCE7, 0x0000, 0x0000, 0x0002, /*1BCE7*/
  0xFBC3, 0x0020, 0x0002, 0xBCE8, 0x0000, 0x0000, 0x0002, /*1BCE8*/
  0xFBC3, 0x0020, 0x0002, 0xBCE9, 0x0000, 0x0000, 0x0002, /*1BCE9*/
  0xFBC3, 0x0020, 0x0002, 0xBCEA, 0x0000, 0x0000, 0x0002, /*1BCEA*/
  0xFBC3, 0x0020, 0x0002, 0xBCEB, 0x0000, 0x0000, 0x0002, /*1BCEB*/
  0xFBC3, 0x0020, 0x0002, 0xBCEC, 0x0000, 0x0000, 0x0002, /*1BCEC*/
  0xFBC3, 0x0020, 0x0002, 0xBCED, 0x0000, 0x0000, 0x0002, /*1BCED*/
  0xFBC3, 0x0020, 0x0002, 0xBCEE, 0x0000, 0x0000, 0x0002, /*1BCEE*/
  0xFBC3, 0x0020, 0x0002, 0xBCEF, 0x0000, 0x0000, 0x0002, /*1BCEF*/
  0xFBC3, 0x0020, 0x0002, 0xBCF0, 0x0000, 0x0000, 0x0002, /*1BCF0*/
  0xFBC3, 0x0020, 0x0002, 0xBCF1, 0x0000, 0x0000, 0x0002, /*1BCF1*/
  0xFBC3, 0x0020, 0x0002, 0xBCF2, 0x0000, 0x0000, 0x0002, /*1BCF2*/
  0xFBC3, 0x0020, 0x0002, 0xBCF3, 0x0000, 0x0000, 0x0002, /*1BCF3*/
  0xFBC3, 0x0020, 0x0002, 0xBCF4, 0x0000, 0x0000, 0x0002, /*1BCF4*/
  0xFBC3, 0x0020, 0x0002, 0xBCF5, 0x0000, 0x0000, 0x0002, /*1BCF5*/
  0xFBC3, 0x0020, 0x0002, 0xBCF6, 0x0000, 0x0000, 0x0002, /*1BCF6*/
  0xFBC3, 0x0020, 0x0002, 0xBCF7, 0x0000, 0x0000, 0x0002, /*1BCF7*/
  0xFBC3, 0x0020, 0x0002, 0xBCF8, 0x0000, 0x0000, 0x0002, /*1BCF8*/
  0xFBC3, 0x0020, 0x0002, 0xBCF9, 0x0000, 0x0000, 0x0002, /*1BCF9*/
  0xFBC3, 0x0020, 0x0002, 0xBCFA, 0x0000, 0x0000, 0x0002, /*1BCFA*/
  0xFBC3, 0x0020, 0x0002, 0xBCFB, 0x0000, 0x0000, 0x0002, /*1BCFB*/
  0xFBC3, 0x0020, 0x0002, 0xBCFC, 0x0000, 0x0000, 0x0002, /*1BCFC*/
  0xFBC3, 0x0020, 0x0002, 0xBCFD, 0x0000, 0x0000, 0x0002, /*1BCFD*/
  0xFBC3, 0x0020, 0x0002, 0xBCFE, 0x0000, 0x0000, 0x0002, /*1BCFE*/
  0xFBC3, 0x0020, 0x0002, 0xBCFF, 0x0000, 0x0000, 0x0002  /*1BCFF*/
};

uint16 uca900_p1D0[]= {
  0x0FD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D000*/
  0x0FD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D001*/
  0x0FD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D002*/
  0x0FD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D003*/
  0x0FDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D004*/
  0x0FDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D005*/
  0x0FDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D006*/
  0x0FDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D007*/
  0x0FDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D008*/
  0x0FDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D009*/
  0x0FE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D00A*/
  0x0FE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D00B*/
  0x0FE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D00C*/
  0x0FE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D00D*/
  0x0FE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D00E*/
  0x0FE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D00F*/
  0x0FE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D010*/
  0x0FE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D011*/
  0x0FE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D012*/
  0x0FE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D013*/
  0x0FEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D014*/
  0x0FEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D015*/
  0x0FEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D016*/
  0x0FED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D017*/
  0x0FEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D018*/
  0x0FEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D019*/
  0x0FF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D01A*/
  0x0FF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D01B*/
  0x0FF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D01C*/
  0x0FF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D01D*/
  0x0FF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D01E*/
  0x0FF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D01F*/
  0x0FF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D020*/
  0x0FF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D021*/
  0x0FF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D022*/
  0x0FF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D023*/
  0x0FFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D024*/
  0x0FFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D025*/
  0x0FFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D026*/
  0x0FFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D027*/
  0x0FFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D028*/
  0x0FFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D029*/
  0x1000, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D02A*/
  0x1001, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D02B*/
  0x1002, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D02C*/
  0x1003, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D02D*/
  0x1004, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D02E*/
  0x1005, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D02F*/
  0x1006, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D030*/
  0x1007, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D031*/
  0x1008, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D032*/
  0x1009, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D033*/
  0x100A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D034*/
  0x100B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D035*/
  0x100C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D036*/
  0x100D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D037*/
  0x100E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D038*/
  0x100F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D039*/
  0x1010, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D03A*/
  0x1011, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D03B*/
  0x1012, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D03C*/
  0x1013, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D03D*/
  0x1014, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D03E*/
  0x1015, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D03F*/
  0x1016, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D040*/
  0x1017, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D041*/
  0x1018, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D042*/
  0x1019, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D043*/
  0x101A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D044*/
  0x101B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D045*/
  0x101C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D046*/
  0x101D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D047*/
  0x101E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D048*/
  0x101F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D049*/
  0x1020, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D04A*/
  0x1021, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D04B*/
  0x1022, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D04C*/
  0x1023, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D04D*/
  0x1024, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D04E*/
  0x1025, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D04F*/
  0x1026, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D050*/
  0x1027, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D051*/
  0x1028, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D052*/
  0x1029, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D053*/
  0x102A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D054*/
  0x102B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D055*/
  0x102C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D056*/
  0x102D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D057*/
  0x102E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D058*/
  0x102F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D059*/
  0x1030, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D05A*/
  0x1031, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D05B*/
  0x1032, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D05C*/
  0x1033, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D05D*/
  0x1034, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D05E*/
  0x1035, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D05F*/
  0x1036, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D060*/
  0x1037, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D061*/
  0x1038, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D062*/
  0x1039, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D063*/
  0x103A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D064*/
  0x103B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D065*/
  0x103C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D066*/
  0x103D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D067*/
  0x103E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D068*/
  0x103F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D069*/
  0x1040, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D06A*/
  0x1041, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D06B*/
  0x1042, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D06C*/
  0x1043, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D06D*/
  0x1044, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D06E*/
  0x1045, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D06F*/
  0x1046, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D070*/
  0x1047, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D071*/
  0x1048, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D072*/
  0x1049, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D073*/
  0x104A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D074*/
  0x104B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D075*/
  0x104C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D076*/
  0x104D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D077*/
  0x104E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D078*/
  0x104F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D079*/
  0x1050, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D07A*/
  0x1051, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D07B*/
  0x1052, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D07C*/
  0x1053, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D07D*/
  0x1054, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D07E*/
  0x1055, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D07F*/
  0x1056, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D080*/
  0x1057, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D081*/
  0x1058, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D082*/
  0x1059, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D083*/
  0x105A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D084*/
  0x105B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D085*/
  0x105C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D086*/
  0x105D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D087*/
  0x105E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D088*/
  0x105F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D089*/
  0x1060, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D08A*/
  0x1061, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D08B*/
  0x1062, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D08C*/
  0x1063, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D08D*/
  0x1064, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D08E*/
  0x1065, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D08F*/
  0x1066, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D090*/
  0x1067, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D091*/
  0x1068, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D092*/
  0x1069, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D093*/
  0x106A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D094*/
  0x106B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D095*/
  0x106C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D096*/
  0x106D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D097*/
  0x106E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D098*/
  0x106F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D099*/
  0x1070, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D09A*/
  0x1071, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D09B*/
  0x1072, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D09C*/
  0x1073, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D09D*/
  0x1074, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D09E*/
  0x1075, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D09F*/
  0x1076, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0A0*/
  0x1077, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0A1*/
  0x1078, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0A2*/
  0x1079, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0A3*/
  0x107A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0A4*/
  0x107B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0A5*/
  0x107C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0A6*/
  0x107D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0A7*/
  0x107E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0A8*/
  0x107F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0A9*/
  0x1080, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0AA*/
  0x1081, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0AB*/
  0x1082, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0AC*/
  0x1083, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0AD*/
  0x1084, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0AE*/
  0x1085, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0AF*/
  0x1086, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0B0*/
  0x1087, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0B1*/
  0x1088, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0B2*/
  0x1089, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0B3*/
  0x108A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0B4*/
  0x108B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0B5*/
  0x108C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0B6*/
  0x108D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0B7*/
  0x108E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0B8*/
  0x108F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0B9*/
  0x1090, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0BA*/
  0x1091, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0BB*/
  0x1092, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0BC*/
  0x1093, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0BD*/
  0x1094, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0BE*/
  0x1095, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0BF*/
  0x1096, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0C0*/
  0x1097, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0C1*/
  0x1098, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0C2*/
  0x1099, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0C3*/
  0x109A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0C4*/
  0x109B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0C5*/
  0x109C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0C6*/
  0x109D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0C7*/
  0x109E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0C8*/
  0x109F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0C9*/
  0x10A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0CA*/
  0x10A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0CB*/
  0x10A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0CC*/
  0x10A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0CD*/
  0x10A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0CE*/
  0x10A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0CF*/
  0x10A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0D0*/
  0x10A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0D1*/
  0x10A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0D2*/
  0x10A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0D3*/
  0x10AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0D4*/
  0x10AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0D5*/
  0x10AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0D6*/
  0x10AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0D7*/
  0x10AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0D8*/
  0x10AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0D9*/
  0x10B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0DA*/
  0x10B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0DB*/
  0x10B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0DC*/
  0x10B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0DD*/
  0x10B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0DE*/
  0x10B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0DF*/
  0x10B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0E0*/
  0x10B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0E1*/
  0x10B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0E2*/
  0x10B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0E3*/
  0x10BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0E4*/
  0x10BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0E5*/
  0x10BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0E6*/
  0x10BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0E7*/
  0x10BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0E8*/
  0x10BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0E9*/
  0x10C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0EA*/
  0x10C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0EB*/
  0x10C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0EC*/
  0x10C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0ED*/
  0x10C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0EE*/
  0x10C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0EF*/
  0x10C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0F0*/
  0x10C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0F1*/
  0x10C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0F2*/
  0x10C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0F3*/
  0x10CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0F4*/
  0x10CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D0F5*/
  0xFBC3, 0x0020, 0x0002, 0xD0F6, 0x0000, 0x0000, 0x0002, /*1D0F6*/
  0xFBC3, 0x0020, 0x0002, 0xD0F7, 0x0000, 0x0000, 0x0002, /*1D0F7*/
  0xFBC3, 0x0020, 0x0002, 0xD0F8, 0x0000, 0x0000, 0x0002, /*1D0F8*/
  0xFBC3, 0x0020, 0x0002, 0xD0F9, 0x0000, 0x0000, 0x0002, /*1D0F9*/
  0xFBC3, 0x0020, 0x0002, 0xD0FA, 0x0000, 0x0000, 0x0002, /*1D0FA*/
  0xFBC3, 0x0020, 0x0002, 0xD0FB, 0x0000, 0x0000, 0x0002, /*1D0FB*/
  0xFBC3, 0x0020, 0x0002, 0xD0FC, 0x0000, 0x0000, 0x0002, /*1D0FC*/
  0xFBC3, 0x0020, 0x0002, 0xD0FD, 0x0000, 0x0000, 0x0002, /*1D0FD*/
  0xFBC3, 0x0020, 0x0002, 0xD0FE, 0x0000, 0x0000, 0x0002, /*1D0FE*/
  0xFBC3, 0x0020, 0x0002, 0xD0FF, 0x0000, 0x0000, 0x0002  /*1D0FF*/
};

uint16 uca900_p1D1[]= {
  0x10CC, 0x0020, 0x0002, 0x0000, 0x0000, /*1D100*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10CD, 0x0020, 0x0002, 0x0000, 0x0000, /*1D101*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10CE, 0x0020, 0x0002, 0x0000, 0x0000, /*1D102*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10CF, 0x0020, 0x0002, 0x0000, 0x0000, /*1D103*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10D0, 0x0020, 0x0002, 0x0000, 0x0000, /*1D104*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10D1, 0x0020, 0x0002, 0x0000, 0x0000, /*1D105*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10D2, 0x0020, 0x0002, 0x0000, 0x0000, /*1D106*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10D3, 0x0020, 0x0002, 0x0000, 0x0000, /*1D107*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10D4, 0x0020, 0x0002, 0x0000, 0x0000, /*1D108*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10D5, 0x0020, 0x0002, 0x0000, 0x0000, /*1D109*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10D6, 0x0020, 0x0002, 0x0000, 0x0000, /*1D10A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10D7, 0x0020, 0x0002, 0x0000, 0x0000, /*1D10B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10D8, 0x0020, 0x0002, 0x0000, 0x0000, /*1D10C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10D9, 0x0020, 0x0002, 0x0000, 0x0000, /*1D10D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10DA, 0x0020, 0x0002, 0x0000, 0x0000, /*1D10E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10DB, 0x0020, 0x0002, 0x0000, 0x0000, /*1D10F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10DC, 0x0020, 0x0002, 0x0000, 0x0000, /*1D110*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10DD, 0x0020, 0x0002, 0x0000, 0x0000, /*1D111*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10DE, 0x0020, 0x0002, 0x0000, 0x0000, /*1D112*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10DF, 0x0020, 0x0002, 0x0000, 0x0000, /*1D113*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10E0, 0x0020, 0x0002, 0x0000, 0x0000, /*1D114*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10E1, 0x0020, 0x0002, 0x0000, 0x0000, /*1D115*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10E2, 0x0020, 0x0002, 0x0000, 0x0000, /*1D116*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10E3, 0x0020, 0x0002, 0x0000, 0x0000, /*1D117*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10E4, 0x0020, 0x0002, 0x0000, 0x0000, /*1D118*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10E5, 0x0020, 0x0002, 0x0000, 0x0000, /*1D119*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10E6, 0x0020, 0x0002, 0x0000, 0x0000, /*1D11A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10E7, 0x0020, 0x0002, 0x0000, 0x0000, /*1D11B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10E8, 0x0020, 0x0002, 0x0000, 0x0000, /*1D11C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10E9, 0x0020, 0x0002, 0x0000, 0x0000, /*1D11D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10EA, 0x0020, 0x0002, 0x0000, 0x0000, /*1D11E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10EB, 0x0020, 0x0002, 0x0000, 0x0000, /*1D11F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10EC, 0x0020, 0x0002, 0x0000, 0x0000, /*1D120*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10ED, 0x0020, 0x0002, 0x0000, 0x0000, /*1D121*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10EE, 0x0020, 0x0002, 0x0000, 0x0000, /*1D122*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10EF, 0x0020, 0x0002, 0x0000, 0x0000, /*1D123*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10F0, 0x0020, 0x0002, 0x0000, 0x0000, /*1D124*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10F1, 0x0020, 0x0002, 0x0000, 0x0000, /*1D125*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10F2, 0x0020, 0x0002, 0x0000, 0x0000, /*1D126*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC3, 0x0020, 0x0002, 0xD127, 0x0000, /*1D127*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD128, 0x0000, /*1D128*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1106, 0x0020, 0x0002, 0x0000, 0x0000, /*1D129*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10F6, 0x0020, 0x0002, 0x0000, 0x0000, /*1D12A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10F7, 0x0020, 0x0002, 0x0000, 0x0000, /*1D12B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10F8, 0x0020, 0x0002, 0x0000, 0x0000, /*1D12C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10F9, 0x0020, 0x0002, 0x0000, 0x0000, /*1D12D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10FA, 0x0020, 0x0002, 0x0000, 0x0000, /*1D12E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10FB, 0x0020, 0x0002, 0x0000, 0x0000, /*1D12F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10FC, 0x0020, 0x0002, 0x0000, 0x0000, /*1D130*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10FD, 0x0020, 0x0002, 0x0000, 0x0000, /*1D131*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10FE, 0x0020, 0x0002, 0x0000, 0x0000, /*1D132*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x10FF, 0x0020, 0x0002, 0x0000, 0x0000, /*1D133*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1100, 0x0020, 0x0002, 0x0000, 0x0000, /*1D134*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1101, 0x0020, 0x0002, 0x0000, 0x0000, /*1D135*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1102, 0x0020, 0x0002, 0x0000, 0x0000, /*1D136*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1103, 0x0020, 0x0002, 0x0000, 0x0000, /*1D137*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1104, 0x0020, 0x0002, 0x0000, 0x0000, /*1D138*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1105, 0x0020, 0x0002, 0x0000, 0x0000, /*1D139*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1107, 0x0020, 0x0002, 0x0000, 0x0000, /*1D13A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1108, 0x0020, 0x0002, 0x0000, 0x0000, /*1D13B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1109, 0x0020, 0x0002, 0x0000, 0x0000, /*1D13C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x110A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D13D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x110B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D13E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x110C, 0x0020, 0x0002, 0x0000, 0x0000, /*1D13F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x110D, 0x0020, 0x0002, 0x0000, 0x0000, /*1D140*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x110E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D141*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x110F, 0x0020, 0x0002, 0x0000, 0x0000, /*1D142*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1110, 0x0020, 0x0002, 0x0000, 0x0000, /*1D143*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1111, 0x0020, 0x0002, 0x0000, 0x0000, /*1D144*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1112, 0x0020, 0x0002, 0x0000, 0x0000, /*1D145*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1113, 0x0020, 0x0002, 0x0000, 0x0000, /*1D146*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1114, 0x0020, 0x0002, 0x0000, 0x0000, /*1D147*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1115, 0x0020, 0x0002, 0x0000, 0x0000, /*1D148*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1116, 0x0020, 0x0002, 0x0000, 0x0000, /*1D149*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1117, 0x0020, 0x0002, 0x0000, 0x0000, /*1D14A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1118, 0x0020, 0x0002, 0x0000, 0x0000, /*1D14B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1119, 0x0020, 0x0002, 0x0000, 0x0000, /*1D14C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x111A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D14D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x111B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D14E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x111C, 0x0020, 0x0002, 0x0000, 0x0000, /*1D14F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x111D, 0x0020, 0x0002, 0x0000, 0x0000, /*1D150*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x111E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D151*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x111F, 0x0020, 0x0002, 0x0000, 0x0000, /*1D152*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1120, 0x0020, 0x0002, 0x0000, 0x0000, /*1D153*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1121, 0x0020, 0x0002, 0x0000, 0x0000, /*1D154*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1122, 0x0020, 0x0002, 0x0000, 0x0000, /*1D155*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1123, 0x0020, 0x0002, 0x0000, 0x0000, /*1D156*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1124, 0x0020, 0x0002, 0x0000, 0x0000, /*1D157*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1125, 0x0020, 0x0002, 0x0000, 0x0000, /*1D158*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1126, 0x0020, 0x0002, 0x0000, 0x0000, /*1D159*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1127, 0x0020, 0x0002, 0x0000, 0x0000, /*1D15A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1128, 0x0020, 0x0002, 0x0000, 0x0000, /*1D15B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1129, 0x0020, 0x0002, 0x0000, 0x0000, /*1D15C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x112A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D15D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1124, 0x0020, 0x0002, 0x0000, 0x0000, /*1D15E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1125, 0x0020, 0x0002, 0x0000, 0x0000, /*1D15F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1125, 0x0020, 0x0002, 0x0000, 0x0000, /*1D160*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1125, 0x0020, 0x0002, 0x0000, 0x0000, /*1D161*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1125, 0x0020, 0x0002, 0x0000, 0x0000, /*1D162*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1125, 0x0020, 0x0002, 0x0000, 0x0000, /*1D163*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1125, 0x0020, 0x0002, 0x0000, 0x0000, /*1D164*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D165*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D166*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D167*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D168*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D169*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x112B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D16A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x112C, 0x0020, 0x0002, 0x0000, 0x0000, /*1D16B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x112D, 0x0020, 0x0002, 0x0000, 0x0000, /*1D16C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D16D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D16E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D16F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D170*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D171*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D172*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D173*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D174*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D175*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D176*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D177*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D178*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D179*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D17A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D17B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D17C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D17D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D17E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D17F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D180*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D181*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D182*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x112E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D183*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x112F, 0x0020, 0x0002, 0x0000, 0x0000, /*1D184*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D185*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D186*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D187*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D188*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D189*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D18A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D18B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1130, 0x0020, 0x0002, 0x0000, 0x0000, /*1D18C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1131, 0x0020, 0x0002, 0x0000, 0x0000, /*1D18D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1132, 0x0020, 0x0002, 0x0000, 0x0000, /*1D18E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1133, 0x0020, 0x0002, 0x0000, 0x0000, /*1D18F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1134, 0x0020, 0x0002, 0x0000, 0x0000, /*1D190*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1135, 0x0020, 0x0002, 0x0000, 0x0000, /*1D191*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1136, 0x0020, 0x0002, 0x0000, 0x0000, /*1D192*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1137, 0x0020, 0x0002, 0x0000, 0x0000, /*1D193*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1138, 0x0020, 0x0002, 0x0000, 0x0000, /*1D194*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1139, 0x0020, 0x0002, 0x0000, 0x0000, /*1D195*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x113A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D196*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x113B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D197*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x113C, 0x0020, 0x0002, 0x0000, 0x0000, /*1D198*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x113D, 0x0020, 0x0002, 0x0000, 0x0000, /*1D199*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x113E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D19A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x113F, 0x0020, 0x0002, 0x0000, 0x0000, /*1D19B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1140, 0x0020, 0x0002, 0x0000, 0x0000, /*1D19C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1141, 0x0020, 0x0002, 0x0000, 0x0000, /*1D19D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1142, 0x0020, 0x0002, 0x0000, 0x0000, /*1D19E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1143, 0x0020, 0x0002, 0x0000, 0x0000, /*1D19F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1144, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1145, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1146, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1147, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1148, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1149, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x114A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x114B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x114C, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x114D, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D1AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D1AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D1AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, /*1D1AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x114E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x114F, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1150, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1151, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1152, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1153, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1154, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1155, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1156, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1157, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1158, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1159, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x115A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1159, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x115A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1159, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x115A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1159, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x115A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x115B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x115C, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x115D, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x115E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x115F, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1160, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1161, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1162, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1163, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1164, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1165, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1166, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1167, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1168, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1169, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x116A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x116B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x116C, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x116D, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x116E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x116F, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1170, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1171, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1172, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1173, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1174, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1175, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1176, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1177, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1178, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1179, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x117A, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x117B, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x117C, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x117D, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x117E, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x117F, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1180, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1181, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1182, 0x0020, 0x0002, 0x0000, 0x0000, /*1D1E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC3, 0x0020, 0x0002, 0xD1E9, 0x0000, /*1D1E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1EA, 0x0000, /*1D1EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1EB, 0x0000, /*1D1EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1EC, 0x0000, /*1D1EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1ED, 0x0000, /*1D1ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1EE, 0x0000, /*1D1EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1EF, 0x0000, /*1D1EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1F0, 0x0000, /*1D1F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1F1, 0x0000, /*1D1F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1F2, 0x0000, /*1D1F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1F3, 0x0000, /*1D1F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1F4, 0x0000, /*1D1F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1F5, 0x0000, /*1D1F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1F6, 0x0000, /*1D1F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1F7, 0x0000, /*1D1F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1F8, 0x0000, /*1D1F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1F9, 0x0000, /*1D1F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1FA, 0x0000, /*1D1FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1FB, 0x0000, /*1D1FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1FC, 0x0000, /*1D1FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1FD, 0x0000, /*1D1FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1FE, 0x0000, /*1D1FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xD1FF, 0x0000, /*1D1FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0002
};

uint16 uca900_p1D2[]= {
  0x1183, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D200*/
  0x1184, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D201*/
  0x1185, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D202*/
  0x1186, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D203*/
  0x1187, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D204*/
  0x1188, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D205*/
  0x1189, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D206*/
  0x118A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D207*/
  0x118B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D208*/
  0x118C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D209*/
  0x118D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D20A*/
  0x118E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D20B*/
  0x118F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D20C*/
  0x1190, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D20D*/
  0x1191, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D20E*/
  0x1192, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D20F*/
  0x1193, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D210*/
  0x1194, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D211*/
  0x1195, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D212*/
  0x1196, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D213*/
  0x1197, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D214*/
  0x1198, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D215*/
  0x1199, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D216*/
  0x119A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D217*/
  0x119B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D218*/
  0x119C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D219*/
  0x119D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D21A*/
  0x119E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D21B*/
  0x119F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D21C*/
  0x11A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D21D*/
  0x11A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D21E*/
  0x11A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D21F*/
  0x11A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D220*/
  0x11A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D221*/
  0x11A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D222*/
  0x11A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D223*/
  0x11A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D224*/
  0x11A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D225*/
  0x11A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D226*/
  0x11AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D227*/
  0x11AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D228*/
  0x11AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D229*/
  0x11AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D22A*/
  0x11AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D22B*/
  0x11AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D22C*/
  0x11B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D22D*/
  0x11B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D22E*/
  0x11B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D22F*/
  0x11B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D230*/
  0x11B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D231*/
  0x11B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D232*/
  0x11B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D233*/
  0x11B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D234*/
  0x11B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D235*/
  0x11B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D236*/
  0x11BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D237*/
  0x11BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D238*/
  0x11BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D239*/
  0x11BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D23A*/
  0x11BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D23B*/
  0x11BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D23C*/
  0x11C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D23D*/
  0x11C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D23E*/
  0x11C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D23F*/
  0x11C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D240*/
  0x11C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D241*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1D242*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1D243*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1D244*/
  0x11C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D245*/
  0xFBC3, 0x0020, 0x0002, 0xD246, 0x0000, 0x0000, 0x0002, /*1D246*/
  0xFBC3, 0x0020, 0x0002, 0xD247, 0x0000, 0x0000, 0x0002, /*1D247*/
  0xFBC3, 0x0020, 0x0002, 0xD248, 0x0000, 0x0000, 0x0002, /*1D248*/
  0xFBC3, 0x0020, 0x0002, 0xD249, 0x0000, 0x0000, 0x0002, /*1D249*/
  0xFBC3, 0x0020, 0x0002, 0xD24A, 0x0000, 0x0000, 0x0002, /*1D24A*/
  0xFBC3, 0x0020, 0x0002, 0xD24B, 0x0000, 0x0000, 0x0002, /*1D24B*/
  0xFBC3, 0x0020, 0x0002, 0xD24C, 0x0000, 0x0000, 0x0002, /*1D24C*/
  0xFBC3, 0x0020, 0x0002, 0xD24D, 0x0000, 0x0000, 0x0002, /*1D24D*/
  0xFBC3, 0x0020, 0x0002, 0xD24E, 0x0000, 0x0000, 0x0002, /*1D24E*/
  0xFBC3, 0x0020, 0x0002, 0xD24F, 0x0000, 0x0000, 0x0002, /*1D24F*/
  0xFBC3, 0x0020, 0x0002, 0xD250, 0x0000, 0x0000, 0x0002, /*1D250*/
  0xFBC3, 0x0020, 0x0002, 0xD251, 0x0000, 0x0000, 0x0002, /*1D251*/
  0xFBC3, 0x0020, 0x0002, 0xD252, 0x0000, 0x0000, 0x0002, /*1D252*/
  0xFBC3, 0x0020, 0x0002, 0xD253, 0x0000, 0x0000, 0x0002, /*1D253*/
  0xFBC3, 0x0020, 0x0002, 0xD254, 0x0000, 0x0000, 0x0002, /*1D254*/
  0xFBC3, 0x0020, 0x0002, 0xD255, 0x0000, 0x0000, 0x0002, /*1D255*/
  0xFBC3, 0x0020, 0x0002, 0xD256, 0x0000, 0x0000, 0x0002, /*1D256*/
  0xFBC3, 0x0020, 0x0002, 0xD257, 0x0000, 0x0000, 0x0002, /*1D257*/
  0xFBC3, 0x0020, 0x0002, 0xD258, 0x0000, 0x0000, 0x0002, /*1D258*/
  0xFBC3, 0x0020, 0x0002, 0xD259, 0x0000, 0x0000, 0x0002, /*1D259*/
  0xFBC3, 0x0020, 0x0002, 0xD25A, 0x0000, 0x0000, 0x0002, /*1D25A*/
  0xFBC3, 0x0020, 0x0002, 0xD25B, 0x0000, 0x0000, 0x0002, /*1D25B*/
  0xFBC3, 0x0020, 0x0002, 0xD25C, 0x0000, 0x0000, 0x0002, /*1D25C*/
  0xFBC3, 0x0020, 0x0002, 0xD25D, 0x0000, 0x0000, 0x0002, /*1D25D*/
  0xFBC3, 0x0020, 0x0002, 0xD25E, 0x0000, 0x0000, 0x0002, /*1D25E*/
  0xFBC3, 0x0020, 0x0002, 0xD25F, 0x0000, 0x0000, 0x0002, /*1D25F*/
  0xFBC3, 0x0020, 0x0002, 0xD260, 0x0000, 0x0000, 0x0002, /*1D260*/
  0xFBC3, 0x0020, 0x0002, 0xD261, 0x0000, 0x0000, 0x0002, /*1D261*/
  0xFBC3, 0x0020, 0x0002, 0xD262, 0x0000, 0x0000, 0x0002, /*1D262*/
  0xFBC3, 0x0020, 0x0002, 0xD263, 0x0000, 0x0000, 0x0002, /*1D263*/
  0xFBC3, 0x0020, 0x0002, 0xD264, 0x0000, 0x0000, 0x0002, /*1D264*/
  0xFBC3, 0x0020, 0x0002, 0xD265, 0x0000, 0x0000, 0x0002, /*1D265*/
  0xFBC3, 0x0020, 0x0002, 0xD266, 0x0000, 0x0000, 0x0002, /*1D266*/
  0xFBC3, 0x0020, 0x0002, 0xD267, 0x0000, 0x0000, 0x0002, /*1D267*/
  0xFBC3, 0x0020, 0x0002, 0xD268, 0x0000, 0x0000, 0x0002, /*1D268*/
  0xFBC3, 0x0020, 0x0002, 0xD269, 0x0000, 0x0000, 0x0002, /*1D269*/
  0xFBC3, 0x0020, 0x0002, 0xD26A, 0x0000, 0x0000, 0x0002, /*1D26A*/
  0xFBC3, 0x0020, 0x0002, 0xD26B, 0x0000, 0x0000, 0x0002, /*1D26B*/
  0xFBC3, 0x0020, 0x0002, 0xD26C, 0x0000, 0x0000, 0x0002, /*1D26C*/
  0xFBC3, 0x0020, 0x0002, 0xD26D, 0x0000, 0x0000, 0x0002, /*1D26D*/
  0xFBC3, 0x0020, 0x0002, 0xD26E, 0x0000, 0x0000, 0x0002, /*1D26E*/
  0xFBC3, 0x0020, 0x0002, 0xD26F, 0x0000, 0x0000, 0x0002, /*1D26F*/
  0xFBC3, 0x0020, 0x0002, 0xD270, 0x0000, 0x0000, 0x0002, /*1D270*/
  0xFBC3, 0x0020, 0x0002, 0xD271, 0x0000, 0x0000, 0x0002, /*1D271*/
  0xFBC3, 0x0020, 0x0002, 0xD272, 0x0000, 0x0000, 0x0002, /*1D272*/
  0xFBC3, 0x0020, 0x0002, 0xD273, 0x0000, 0x0000, 0x0002, /*1D273*/
  0xFBC3, 0x0020, 0x0002, 0xD274, 0x0000, 0x0000, 0x0002, /*1D274*/
  0xFBC3, 0x0020, 0x0002, 0xD275, 0x0000, 0x0000, 0x0002, /*1D275*/
  0xFBC3, 0x0020, 0x0002, 0xD276, 0x0000, 0x0000, 0x0002, /*1D276*/
  0xFBC3, 0x0020, 0x0002, 0xD277, 0x0000, 0x0000, 0x0002, /*1D277*/
  0xFBC3, 0x0020, 0x0002, 0xD278, 0x0000, 0x0000, 0x0002, /*1D278*/
  0xFBC3, 0x0020, 0x0002, 0xD279, 0x0000, 0x0000, 0x0002, /*1D279*/
  0xFBC3, 0x0020, 0x0002, 0xD27A, 0x0000, 0x0000, 0x0002, /*1D27A*/
  0xFBC3, 0x0020, 0x0002, 0xD27B, 0x0000, 0x0000, 0x0002, /*1D27B*/
  0xFBC3, 0x0020, 0x0002, 0xD27C, 0x0000, 0x0000, 0x0002, /*1D27C*/
  0xFBC3, 0x0020, 0x0002, 0xD27D, 0x0000, 0x0000, 0x0002, /*1D27D*/
  0xFBC3, 0x0020, 0x0002, 0xD27E, 0x0000, 0x0000, 0x0002, /*1D27E*/
  0xFBC3, 0x0020, 0x0002, 0xD27F, 0x0000, 0x0000, 0x0002, /*1D27F*/
  0xFBC3, 0x0020, 0x0002, 0xD280, 0x0000, 0x0000, 0x0002, /*1D280*/
  0xFBC3, 0x0020, 0x0002, 0xD281, 0x0000, 0x0000, 0x0002, /*1D281*/
  0xFBC3, 0x0020, 0x0002, 0xD282, 0x0000, 0x0000, 0x0002, /*1D282*/
  0xFBC3, 0x0020, 0x0002, 0xD283, 0x0000, 0x0000, 0x0002, /*1D283*/
  0xFBC3, 0x0020, 0x0002, 0xD284, 0x0000, 0x0000, 0x0002, /*1D284*/
  0xFBC3, 0x0020, 0x0002, 0xD285, 0x0000, 0x0000, 0x0002, /*1D285*/
  0xFBC3, 0x0020, 0x0002, 0xD286, 0x0000, 0x0000, 0x0002, /*1D286*/
  0xFBC3, 0x0020, 0x0002, 0xD287, 0x0000, 0x0000, 0x0002, /*1D287*/
  0xFBC3, 0x0020, 0x0002, 0xD288, 0x0000, 0x0000, 0x0002, /*1D288*/
  0xFBC3, 0x0020, 0x0002, 0xD289, 0x0000, 0x0000, 0x0002, /*1D289*/
  0xFBC3, 0x0020, 0x0002, 0xD28A, 0x0000, 0x0000, 0x0002, /*1D28A*/
  0xFBC3, 0x0020, 0x0002, 0xD28B, 0x0000, 0x0000, 0x0002, /*1D28B*/
  0xFBC3, 0x0020, 0x0002, 0xD28C, 0x0000, 0x0000, 0x0002, /*1D28C*/
  0xFBC3, 0x0020, 0x0002, 0xD28D, 0x0000, 0x0000, 0x0002, /*1D28D*/
  0xFBC3, 0x0020, 0x0002, 0xD28E, 0x0000, 0x0000, 0x0002, /*1D28E*/
  0xFBC3, 0x0020, 0x0002, 0xD28F, 0x0000, 0x0000, 0x0002, /*1D28F*/
  0xFBC3, 0x0020, 0x0002, 0xD290, 0x0000, 0x0000, 0x0002, /*1D290*/
  0xFBC3, 0x0020, 0x0002, 0xD291, 0x0000, 0x0000, 0x0002, /*1D291*/
  0xFBC3, 0x0020, 0x0002, 0xD292, 0x0000, 0x0000, 0x0002, /*1D292*/
  0xFBC3, 0x0020, 0x0002, 0xD293, 0x0000, 0x0000, 0x0002, /*1D293*/
  0xFBC3, 0x0020, 0x0002, 0xD294, 0x0000, 0x0000, 0x0002, /*1D294*/
  0xFBC3, 0x0020, 0x0002, 0xD295, 0x0000, 0x0000, 0x0002, /*1D295*/
  0xFBC3, 0x0020, 0x0002, 0xD296, 0x0000, 0x0000, 0x0002, /*1D296*/
  0xFBC3, 0x0020, 0x0002, 0xD297, 0x0000, 0x0000, 0x0002, /*1D297*/
  0xFBC3, 0x0020, 0x0002, 0xD298, 0x0000, 0x0000, 0x0002, /*1D298*/
  0xFBC3, 0x0020, 0x0002, 0xD299, 0x0000, 0x0000, 0x0002, /*1D299*/
  0xFBC3, 0x0020, 0x0002, 0xD29A, 0x0000, 0x0000, 0x0002, /*1D29A*/
  0xFBC3, 0x0020, 0x0002, 0xD29B, 0x0000, 0x0000, 0x0002, /*1D29B*/
  0xFBC3, 0x0020, 0x0002, 0xD29C, 0x0000, 0x0000, 0x0002, /*1D29C*/
  0xFBC3, 0x0020, 0x0002, 0xD29D, 0x0000, 0x0000, 0x0002, /*1D29D*/
  0xFBC3, 0x0020, 0x0002, 0xD29E, 0x0000, 0x0000, 0x0002, /*1D29E*/
  0xFBC3, 0x0020, 0x0002, 0xD29F, 0x0000, 0x0000, 0x0002, /*1D29F*/
  0xFBC3, 0x0020, 0x0002, 0xD2A0, 0x0000, 0x0000, 0x0002, /*1D2A0*/
  0xFBC3, 0x0020, 0x0002, 0xD2A1, 0x0000, 0x0000, 0x0002, /*1D2A1*/
  0xFBC3, 0x0020, 0x0002, 0xD2A2, 0x0000, 0x0000, 0x0002, /*1D2A2*/
  0xFBC3, 0x0020, 0x0002, 0xD2A3, 0x0000, 0x0000, 0x0002, /*1D2A3*/
  0xFBC3, 0x0020, 0x0002, 0xD2A4, 0x0000, 0x0000, 0x0002, /*1D2A4*/
  0xFBC3, 0x0020, 0x0002, 0xD2A5, 0x0000, 0x0000, 0x0002, /*1D2A5*/
  0xFBC3, 0x0020, 0x0002, 0xD2A6, 0x0000, 0x0000, 0x0002, /*1D2A6*/
  0xFBC3, 0x0020, 0x0002, 0xD2A7, 0x0000, 0x0000, 0x0002, /*1D2A7*/
  0xFBC3, 0x0020, 0x0002, 0xD2A8, 0x0000, 0x0000, 0x0002, /*1D2A8*/
  0xFBC3, 0x0020, 0x0002, 0xD2A9, 0x0000, 0x0000, 0x0002, /*1D2A9*/
  0xFBC3, 0x0020, 0x0002, 0xD2AA, 0x0000, 0x0000, 0x0002, /*1D2AA*/
  0xFBC3, 0x0020, 0x0002, 0xD2AB, 0x0000, 0x0000, 0x0002, /*1D2AB*/
  0xFBC3, 0x0020, 0x0002, 0xD2AC, 0x0000, 0x0000, 0x0002, /*1D2AC*/
  0xFBC3, 0x0020, 0x0002, 0xD2AD, 0x0000, 0x0000, 0x0002, /*1D2AD*/
  0xFBC3, 0x0020, 0x0002, 0xD2AE, 0x0000, 0x0000, 0x0002, /*1D2AE*/
  0xFBC3, 0x0020, 0x0002, 0xD2AF, 0x0000, 0x0000, 0x0002, /*1D2AF*/
  0xFBC3, 0x0020, 0x0002, 0xD2B0, 0x0000, 0x0000, 0x0002, /*1D2B0*/
  0xFBC3, 0x0020, 0x0002, 0xD2B1, 0x0000, 0x0000, 0x0002, /*1D2B1*/
  0xFBC3, 0x0020, 0x0002, 0xD2B2, 0x0000, 0x0000, 0x0002, /*1D2B2*/
  0xFBC3, 0x0020, 0x0002, 0xD2B3, 0x0000, 0x0000, 0x0002, /*1D2B3*/
  0xFBC3, 0x0020, 0x0002, 0xD2B4, 0x0000, 0x0000, 0x0002, /*1D2B4*/
  0xFBC3, 0x0020, 0x0002, 0xD2B5, 0x0000, 0x0000, 0x0002, /*1D2B5*/
  0xFBC3, 0x0020, 0x0002, 0xD2B6, 0x0000, 0x0000, 0x0002, /*1D2B6*/
  0xFBC3, 0x0020, 0x0002, 0xD2B7, 0x0000, 0x0000, 0x0002, /*1D2B7*/
  0xFBC3, 0x0020, 0x0002, 0xD2B8, 0x0000, 0x0000, 0x0002, /*1D2B8*/
  0xFBC3, 0x0020, 0x0002, 0xD2B9, 0x0000, 0x0000, 0x0002, /*1D2B9*/
  0xFBC3, 0x0020, 0x0002, 0xD2BA, 0x0000, 0x0000, 0x0002, /*1D2BA*/
  0xFBC3, 0x0020, 0x0002, 0xD2BB, 0x0000, 0x0000, 0x0002, /*1D2BB*/
  0xFBC3, 0x0020, 0x0002, 0xD2BC, 0x0000, 0x0000, 0x0002, /*1D2BC*/
  0xFBC3, 0x0020, 0x0002, 0xD2BD, 0x0000, 0x0000, 0x0002, /*1D2BD*/
  0xFBC3, 0x0020, 0x0002, 0xD2BE, 0x0000, 0x0000, 0x0002, /*1D2BE*/
  0xFBC3, 0x0020, 0x0002, 0xD2BF, 0x0000, 0x0000, 0x0002, /*1D2BF*/
  0xFBC3, 0x0020, 0x0002, 0xD2C0, 0x0000, 0x0000, 0x0002, /*1D2C0*/
  0xFBC3, 0x0020, 0x0002, 0xD2C1, 0x0000, 0x0000, 0x0002, /*1D2C1*/
  0xFBC3, 0x0020, 0x0002, 0xD2C2, 0x0000, 0x0000, 0x0002, /*1D2C2*/
  0xFBC3, 0x0020, 0x0002, 0xD2C3, 0x0000, 0x0000, 0x0002, /*1D2C3*/
  0xFBC3, 0x0020, 0x0002, 0xD2C4, 0x0000, 0x0000, 0x0002, /*1D2C4*/
  0xFBC3, 0x0020, 0x0002, 0xD2C5, 0x0000, 0x0000, 0x0002, /*1D2C5*/
  0xFBC3, 0x0020, 0x0002, 0xD2C6, 0x0000, 0x0000, 0x0002, /*1D2C6*/
  0xFBC3, 0x0020, 0x0002, 0xD2C7, 0x0000, 0x0000, 0x0002, /*1D2C7*/
  0xFBC3, 0x0020, 0x0002, 0xD2C8, 0x0000, 0x0000, 0x0002, /*1D2C8*/
  0xFBC3, 0x0020, 0x0002, 0xD2C9, 0x0000, 0x0000, 0x0002, /*1D2C9*/
  0xFBC3, 0x0020, 0x0002, 0xD2CA, 0x0000, 0x0000, 0x0002, /*1D2CA*/
  0xFBC3, 0x0020, 0x0002, 0xD2CB, 0x0000, 0x0000, 0x0002, /*1D2CB*/
  0xFBC3, 0x0020, 0x0002, 0xD2CC, 0x0000, 0x0000, 0x0002, /*1D2CC*/
  0xFBC3, 0x0020, 0x0002, 0xD2CD, 0x0000, 0x0000, 0x0002, /*1D2CD*/
  0xFBC3, 0x0020, 0x0002, 0xD2CE, 0x0000, 0x0000, 0x0002, /*1D2CE*/
  0xFBC3, 0x0020, 0x0002, 0xD2CF, 0x0000, 0x0000, 0x0002, /*1D2CF*/
  0xFBC3, 0x0020, 0x0002, 0xD2D0, 0x0000, 0x0000, 0x0002, /*1D2D0*/
  0xFBC3, 0x0020, 0x0002, 0xD2D1, 0x0000, 0x0000, 0x0002, /*1D2D1*/
  0xFBC3, 0x0020, 0x0002, 0xD2D2, 0x0000, 0x0000, 0x0002, /*1D2D2*/
  0xFBC3, 0x0020, 0x0002, 0xD2D3, 0x0000, 0x0000, 0x0002, /*1D2D3*/
  0xFBC3, 0x0020, 0x0002, 0xD2D4, 0x0000, 0x0000, 0x0002, /*1D2D4*/
  0xFBC3, 0x0020, 0x0002, 0xD2D5, 0x0000, 0x0000, 0x0002, /*1D2D5*/
  0xFBC3, 0x0020, 0x0002, 0xD2D6, 0x0000, 0x0000, 0x0002, /*1D2D6*/
  0xFBC3, 0x0020, 0x0002, 0xD2D7, 0x0000, 0x0000, 0x0002, /*1D2D7*/
  0xFBC3, 0x0020, 0x0002, 0xD2D8, 0x0000, 0x0000, 0x0002, /*1D2D8*/
  0xFBC3, 0x0020, 0x0002, 0xD2D9, 0x0000, 0x0000, 0x0002, /*1D2D9*/
  0xFBC3, 0x0020, 0x0002, 0xD2DA, 0x0000, 0x0000, 0x0002, /*1D2DA*/
  0xFBC3, 0x0020, 0x0002, 0xD2DB, 0x0000, 0x0000, 0x0002, /*1D2DB*/
  0xFBC3, 0x0020, 0x0002, 0xD2DC, 0x0000, 0x0000, 0x0002, /*1D2DC*/
  0xFBC3, 0x0020, 0x0002, 0xD2DD, 0x0000, 0x0000, 0x0002, /*1D2DD*/
  0xFBC3, 0x0020, 0x0002, 0xD2DE, 0x0000, 0x0000, 0x0002, /*1D2DE*/
  0xFBC3, 0x0020, 0x0002, 0xD2DF, 0x0000, 0x0000, 0x0002, /*1D2DF*/
  0xFBC3, 0x0020, 0x0002, 0xD2E0, 0x0000, 0x0000, 0x0002, /*1D2E0*/
  0xFBC3, 0x0020, 0x0002, 0xD2E1, 0x0000, 0x0000, 0x0002, /*1D2E1*/
  0xFBC3, 0x0020, 0x0002, 0xD2E2, 0x0000, 0x0000, 0x0002, /*1D2E2*/
  0xFBC3, 0x0020, 0x0002, 0xD2E3, 0x0000, 0x0000, 0x0002, /*1D2E3*/
  0xFBC3, 0x0020, 0x0002, 0xD2E4, 0x0000, 0x0000, 0x0002, /*1D2E4*/
  0xFBC3, 0x0020, 0x0002, 0xD2E5, 0x0000, 0x0000, 0x0002, /*1D2E5*/
  0xFBC3, 0x0020, 0x0002, 0xD2E6, 0x0000, 0x0000, 0x0002, /*1D2E6*/
  0xFBC3, 0x0020, 0x0002, 0xD2E7, 0x0000, 0x0000, 0x0002, /*1D2E7*/
  0xFBC3, 0x0020, 0x0002, 0xD2E8, 0x0000, 0x0000, 0x0002, /*1D2E8*/
  0xFBC3, 0x0020, 0x0002, 0xD2E9, 0x0000, 0x0000, 0x0002, /*1D2E9*/
  0xFBC3, 0x0020, 0x0002, 0xD2EA, 0x0000, 0x0000, 0x0002, /*1D2EA*/
  0xFBC3, 0x0020, 0x0002, 0xD2EB, 0x0000, 0x0000, 0x0002, /*1D2EB*/
  0xFBC3, 0x0020, 0x0002, 0xD2EC, 0x0000, 0x0000, 0x0002, /*1D2EC*/
  0xFBC3, 0x0020, 0x0002, 0xD2ED, 0x0000, 0x0000, 0x0002, /*1D2ED*/
  0xFBC3, 0x0020, 0x0002, 0xD2EE, 0x0000, 0x0000, 0x0002, /*1D2EE*/
  0xFBC3, 0x0020, 0x0002, 0xD2EF, 0x0000, 0x0000, 0x0002, /*1D2EF*/
  0xFBC3, 0x0020, 0x0002, 0xD2F0, 0x0000, 0x0000, 0x0002, /*1D2F0*/
  0xFBC3, 0x0020, 0x0002, 0xD2F1, 0x0000, 0x0000, 0x0002, /*1D2F1*/
  0xFBC3, 0x0020, 0x0002, 0xD2F2, 0x0000, 0x0000, 0x0002, /*1D2F2*/
  0xFBC3, 0x0020, 0x0002, 0xD2F3, 0x0000, 0x0000, 0x0002, /*1D2F3*/
  0xFBC3, 0x0020, 0x0002, 0xD2F4, 0x0000, 0x0000, 0x0002, /*1D2F4*/
  0xFBC3, 0x0020, 0x0002, 0xD2F5, 0x0000, 0x0000, 0x0002, /*1D2F5*/
  0xFBC3, 0x0020, 0x0002, 0xD2F6, 0x0000, 0x0000, 0x0002, /*1D2F6*/
  0xFBC3, 0x0020, 0x0002, 0xD2F7, 0x0000, 0x0000, 0x0002, /*1D2F7*/
  0xFBC3, 0x0020, 0x0002, 0xD2F8, 0x0000, 0x0000, 0x0002, /*1D2F8*/
  0xFBC3, 0x0020, 0x0002, 0xD2F9, 0x0000, 0x0000, 0x0002, /*1D2F9*/
  0xFBC3, 0x0020, 0x0002, 0xD2FA, 0x0000, 0x0000, 0x0002, /*1D2FA*/
  0xFBC3, 0x0020, 0x0002, 0xD2FB, 0x0000, 0x0000, 0x0002, /*1D2FB*/
  0xFBC3, 0x0020, 0x0002, 0xD2FC, 0x0000, 0x0000, 0x0002, /*1D2FC*/
  0xFBC3, 0x0020, 0x0002, 0xD2FD, 0x0000, 0x0000, 0x0002, /*1D2FD*/
  0xFBC3, 0x0020, 0x0002, 0xD2FE, 0x0000, 0x0000, 0x0002, /*1D2FE*/
  0xFBC3, 0x0020, 0x0002, 0xD2FF, 0x0000, 0x0000, 0x0002  /*1D2FF*/
};

uint16 uca900_p1D3[]= {
  0x0EEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D300*/
  0x0EEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D301*/
  0x0EEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D302*/
  0x0EED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D303*/
  0x0EEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D304*/
  0x0EEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D305*/
  0x0EF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D306*/
  0x0EF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D307*/
  0x0EF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D308*/
  0x0EF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D309*/
  0x0EF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D30A*/
  0x0EF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D30B*/
  0x0EF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D30C*/
  0x0EF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D30D*/
  0x0EF8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D30E*/
  0x0EF9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D30F*/
  0x0EFA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D310*/
  0x0EFB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D311*/
  0x0EFC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D312*/
  0x0EFD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D313*/
  0x0EFE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D314*/
  0x0EFF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D315*/
  0x0F00, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D316*/
  0x0F01, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D317*/
  0x0F02, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D318*/
  0x0F03, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D319*/
  0x0F04, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D31A*/
  0x0F05, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D31B*/
  0x0F06, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D31C*/
  0x0F07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D31D*/
  0x0F08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D31E*/
  0x0F09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D31F*/
  0x0F0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D320*/
  0x0F0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D321*/
  0x0F0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D322*/
  0x0F0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D323*/
  0x0F0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D324*/
  0x0F0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D325*/
  0x0F10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D326*/
  0x0F11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D327*/
  0x0F12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D328*/
  0x0F13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D329*/
  0x0F14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D32A*/
  0x0F15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D32B*/
  0x0F16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D32C*/
  0x0F17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D32D*/
  0x0F18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D32E*/
  0x0F19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D32F*/
  0x0F1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D330*/
  0x0F1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D331*/
  0x0F1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D332*/
  0x0F1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D333*/
  0x0F1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D334*/
  0x0F1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D335*/
  0x0F20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D336*/
  0x0F21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D337*/
  0x0F22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D338*/
  0x0F23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D339*/
  0x0F24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D33A*/
  0x0F25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D33B*/
  0x0F26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D33C*/
  0x0F27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D33D*/
  0x0F28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D33E*/
  0x0F29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D33F*/
  0x0F2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D340*/
  0x0F2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D341*/
  0x0F2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D342*/
  0x0F2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D343*/
  0x0F2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D344*/
  0x0F2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D345*/
  0x0F30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D346*/
  0x0F31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D347*/
  0x0F32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D348*/
  0x0F33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D349*/
  0x0F34, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D34A*/
  0x0F35, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D34B*/
  0x0F36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D34C*/
  0x0F37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D34D*/
  0x0F38, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D34E*/
  0x0F39, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D34F*/
  0x0F3A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D350*/
  0x0F3B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D351*/
  0x0F3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D352*/
  0x0F3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D353*/
  0x0F3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D354*/
  0x0F3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D355*/
  0x0F40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D356*/
  0xFBC3, 0x0020, 0x0002, 0xD357, 0x0000, 0x0000, 0x0002, /*1D357*/
  0xFBC3, 0x0020, 0x0002, 0xD358, 0x0000, 0x0000, 0x0002, /*1D358*/
  0xFBC3, 0x0020, 0x0002, 0xD359, 0x0000, 0x0000, 0x0002, /*1D359*/
  0xFBC3, 0x0020, 0x0002, 0xD35A, 0x0000, 0x0000, 0x0002, /*1D35A*/
  0xFBC3, 0x0020, 0x0002, 0xD35B, 0x0000, 0x0000, 0x0002, /*1D35B*/
  0xFBC3, 0x0020, 0x0002, 0xD35C, 0x0000, 0x0000, 0x0002, /*1D35C*/
  0xFBC3, 0x0020, 0x0002, 0xD35D, 0x0000, 0x0000, 0x0002, /*1D35D*/
  0xFBC3, 0x0020, 0x0002, 0xD35E, 0x0000, 0x0000, 0x0002, /*1D35E*/
  0xFBC3, 0x0020, 0x0002, 0xD35F, 0x0000, 0x0000, 0x0002, /*1D35F*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D360*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D361*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D362*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D363*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D364*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D365*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D366*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D367*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D368*/
  0x1BEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D369*/
  0x1BF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D36A*/
  0x1BF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D36B*/
  0x1BF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D36C*/
  0x1BF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D36D*/
  0x1BF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D36E*/
  0x1BF5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D36F*/
  0x1BF6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D370*/
  0x1BF7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1D371*/
  0xFBC3, 0x0020, 0x0002, 0xD372, 0x0000, 0x0000, 0x0002, /*1D372*/
  0xFBC3, 0x0020, 0x0002, 0xD373, 0x0000, 0x0000, 0x0002, /*1D373*/
  0xFBC3, 0x0020, 0x0002, 0xD374, 0x0000, 0x0000, 0x0002, /*1D374*/
  0xFBC3, 0x0020, 0x0002, 0xD375, 0x0000, 0x0000, 0x0002, /*1D375*/
  0xFBC3, 0x0020, 0x0002, 0xD376, 0x0000, 0x0000, 0x0002, /*1D376*/
  0xFBC3, 0x0020, 0x0002, 0xD377, 0x0000, 0x0000, 0x0002, /*1D377*/
  0xFBC3, 0x0020, 0x0002, 0xD378, 0x0000, 0x0000, 0x0002, /*1D378*/
  0xFBC3, 0x0020, 0x0002, 0xD379, 0x0000, 0x0000, 0x0002, /*1D379*/
  0xFBC3, 0x0020, 0x0002, 0xD37A, 0x0000, 0x0000, 0x0002, /*1D37A*/
  0xFBC3, 0x0020, 0x0002, 0xD37B, 0x0000, 0x0000, 0x0002, /*1D37B*/
  0xFBC3, 0x0020, 0x0002, 0xD37C, 0x0000, 0x0000, 0x0002, /*1D37C*/
  0xFBC3, 0x0020, 0x0002, 0xD37D, 0x0000, 0x0000, 0x0002, /*1D37D*/
  0xFBC3, 0x0020, 0x0002, 0xD37E, 0x0000, 0x0000, 0x0002, /*1D37E*/
  0xFBC3, 0x0020, 0x0002, 0xD37F, 0x0000, 0x0000, 0x0002, /*1D37F*/
  0xFBC3, 0x0020, 0x0002, 0xD380, 0x0000, 0x0000, 0x0002, /*1D380*/
  0xFBC3, 0x0020, 0x0002, 0xD381, 0x0000, 0x0000, 0x0002, /*1D381*/
  0xFBC3, 0x0020, 0x0002, 0xD382, 0x0000, 0x0000, 0x0002, /*1D382*/
  0xFBC3, 0x0020, 0x0002, 0xD383, 0x0000, 0x0000, 0x0002, /*1D383*/
  0xFBC3, 0x0020, 0x0002, 0xD384, 0x0000, 0x0000, 0x0002, /*1D384*/
  0xFBC3, 0x0020, 0x0002, 0xD385, 0x0000, 0x0000, 0x0002, /*1D385*/
  0xFBC3, 0x0020, 0x0002, 0xD386, 0x0000, 0x0000, 0x0002, /*1D386*/
  0xFBC3, 0x0020, 0x0002, 0xD387, 0x0000, 0x0000, 0x0002, /*1D387*/
  0xFBC3, 0x0020, 0x0002, 0xD388, 0x0000, 0x0000, 0x0002, /*1D388*/
  0xFBC3, 0x0020, 0x0002, 0xD389, 0x0000, 0x0000, 0x0002, /*1D389*/
  0xFBC3, 0x0020, 0x0002, 0xD38A, 0x0000, 0x0000, 0x0002, /*1D38A*/
  0xFBC3, 0x0020, 0x0002, 0xD38B, 0x0000, 0x0000, 0x0002, /*1D38B*/
  0xFBC3, 0x0020, 0x0002, 0xD38C, 0x0000, 0x0000, 0x0002, /*1D38C*/
  0xFBC3, 0x0020, 0x0002, 0xD38D, 0x0000, 0x0000, 0x0002, /*1D38D*/
  0xFBC3, 0x0020, 0x0002, 0xD38E, 0x0000, 0x0000, 0x0002, /*1D38E*/
  0xFBC3, 0x0020, 0x0002, 0xD38F, 0x0000, 0x0000, 0x0002, /*1D38F*/
  0xFBC3, 0x0020, 0x0002, 0xD390, 0x0000, 0x0000, 0x0002, /*1D390*/
  0xFBC3, 0x0020, 0x0002, 0xD391, 0x0000, 0x0000, 0x0002, /*1D391*/
  0xFBC3, 0x0020, 0x0002, 0xD392, 0x0000, 0x0000, 0x0002, /*1D392*/
  0xFBC3, 0x0020, 0x0002, 0xD393, 0x0000, 0x0000, 0x0002, /*1D393*/
  0xFBC3, 0x0020, 0x0002, 0xD394, 0x0000, 0x0000, 0x0002, /*1D394*/
  0xFBC3, 0x0020, 0x0002, 0xD395, 0x0000, 0x0000, 0x0002, /*1D395*/
  0xFBC3, 0x0020, 0x0002, 0xD396, 0x0000, 0x0000, 0x0002, /*1D396*/
  0xFBC3, 0x0020, 0x0002, 0xD397, 0x0000, 0x0000, 0x0002, /*1D397*/
  0xFBC3, 0x0020, 0x0002, 0xD398, 0x0000, 0x0000, 0x0002, /*1D398*/
  0xFBC3, 0x0020, 0x0002, 0xD399, 0x0000, 0x0000, 0x0002, /*1D399*/
  0xFBC3, 0x0020, 0x0002, 0xD39A, 0x0000, 0x0000, 0x0002, /*1D39A*/
  0xFBC3, 0x0020, 0x0002, 0xD39B, 0x0000, 0x0000, 0x0002, /*1D39B*/
  0xFBC3, 0x0020, 0x0002, 0xD39C, 0x0000, 0x0000, 0x0002, /*1D39C*/
  0xFBC3, 0x0020, 0x0002, 0xD39D, 0x0000, 0x0000, 0x0002, /*1D39D*/
  0xFBC3, 0x0020, 0x0002, 0xD39E, 0x0000, 0x0000, 0x0002, /*1D39E*/
  0xFBC3, 0x0020, 0x0002, 0xD39F, 0x0000, 0x0000, 0x0002, /*1D39F*/
  0xFBC3, 0x0020, 0x0002, 0xD3A0, 0x0000, 0x0000, 0x0002, /*1D3A0*/
  0xFBC3, 0x0020, 0x0002, 0xD3A1, 0x0000, 0x0000, 0x0002, /*1D3A1*/
  0xFBC3, 0x0020, 0x0002, 0xD3A2, 0x0000, 0x0000, 0x0002, /*1D3A2*/
  0xFBC3, 0x0020, 0x0002, 0xD3A3, 0x0000, 0x0000, 0x0002, /*1D3A3*/
  0xFBC3, 0x0020, 0x0002, 0xD3A4, 0x0000, 0x0000, 0x0002, /*1D3A4*/
  0xFBC3, 0x0020, 0x0002, 0xD3A5, 0x0000, 0x0000, 0x0002, /*1D3A5*/
  0xFBC3, 0x0020, 0x0002, 0xD3A6, 0x0000, 0x0000, 0x0002, /*1D3A6*/
  0xFBC3, 0x0020, 0x0002, 0xD3A7, 0x0000, 0x0000, 0x0002, /*1D3A7*/
  0xFBC3, 0x0020, 0x0002, 0xD3A8, 0x0000, 0x0000, 0x0002, /*1D3A8*/
  0xFBC3, 0x0020, 0x0002, 0xD3A9, 0x0000, 0x0000, 0x0002, /*1D3A9*/
  0xFBC3, 0x0020, 0x0002, 0xD3AA, 0x0000, 0x0000, 0x0002, /*1D3AA*/
  0xFBC3, 0x0020, 0x0002, 0xD3AB, 0x0000, 0x0000, 0x0002, /*1D3AB*/
  0xFBC3, 0x0020, 0x0002, 0xD3AC, 0x0000, 0x0000, 0x0002, /*1D3AC*/
  0xFBC3, 0x0020, 0x0002, 0xD3AD, 0x0000, 0x0000, 0x0002, /*1D3AD*/
  0xFBC3, 0x0020, 0x0002, 0xD3AE, 0x0000, 0x0000, 0x0002, /*1D3AE*/
  0xFBC3, 0x0020, 0x0002, 0xD3AF, 0x0000, 0x0000, 0x0002, /*1D3AF*/
  0xFBC3, 0x0020, 0x0002, 0xD3B0, 0x0000, 0x0000, 0x0002, /*1D3B0*/
  0xFBC3, 0x0020, 0x0002, 0xD3B1, 0x0000, 0x0000, 0x0002, /*1D3B1*/
  0xFBC3, 0x0020, 0x0002, 0xD3B2, 0x0000, 0x0000, 0x0002, /*1D3B2*/
  0xFBC3, 0x0020, 0x0002, 0xD3B3, 0x0000, 0x0000, 0x0002, /*1D3B3*/
  0xFBC3, 0x0020, 0x0002, 0xD3B4, 0x0000, 0x0000, 0x0002, /*1D3B4*/
  0xFBC3, 0x0020, 0x0002, 0xD3B5, 0x0000, 0x0000, 0x0002, /*1D3B5*/
  0xFBC3, 0x0020, 0x0002, 0xD3B6, 0x0000, 0x0000, 0x0002, /*1D3B6*/
  0xFBC3, 0x0020, 0x0002, 0xD3B7, 0x0000, 0x0000, 0x0002, /*1D3B7*/
  0xFBC3, 0x0020, 0x0002, 0xD3B8, 0x0000, 0x0000, 0x0002, /*1D3B8*/
  0xFBC3, 0x0020, 0x0002, 0xD3B9, 0x0000, 0x0000, 0x0002, /*1D3B9*/
  0xFBC3, 0x0020, 0x0002, 0xD3BA, 0x0000, 0x0000, 0x0002, /*1D3BA*/
  0xFBC3, 0x0020, 0x0002, 0xD3BB, 0x0000, 0x0000, 0x0002, /*1D3BB*/
  0xFBC3, 0x0020, 0x0002, 0xD3BC, 0x0000, 0x0000, 0x0002, /*1D3BC*/
  0xFBC3, 0x0020, 0x0002, 0xD3BD, 0x0000, 0x0000, 0x0002, /*1D3BD*/
  0xFBC3, 0x0020, 0x0002, 0xD3BE, 0x0000, 0x0000, 0x0002, /*1D3BE*/
  0xFBC3, 0x0020, 0x0002, 0xD3BF, 0x0000, 0x0000, 0x0002, /*1D3BF*/
  0xFBC3, 0x0020, 0x0002, 0xD3C0, 0x0000, 0x0000, 0x0002, /*1D3C0*/
  0xFBC3, 0x0020, 0x0002, 0xD3C1, 0x0000, 0x0000, 0x0002, /*1D3C1*/
  0xFBC3, 0x0020, 0x0002, 0xD3C2, 0x0000, 0x0000, 0x0002, /*1D3C2*/
  0xFBC3, 0x0020, 0x0002, 0xD3C3, 0x0000, 0x0000, 0x0002, /*1D3C3*/
  0xFBC3, 0x0020, 0x0002, 0xD3C4, 0x0000, 0x0000, 0x0002, /*1D3C4*/
  0xFBC3, 0x0020, 0x0002, 0xD3C5, 0x0000, 0x0000, 0x0002, /*1D3C5*/
  0xFBC3, 0x0020, 0x0002, 0xD3C6, 0x0000, 0x0000, 0x0002, /*1D3C6*/
  0xFBC3, 0x0020, 0x0002, 0xD3C7, 0x0000, 0x0000, 0x0002, /*1D3C7*/
  0xFBC3, 0x0020, 0x0002, 0xD3C8, 0x0000, 0x0000, 0x0002, /*1D3C8*/
  0xFBC3, 0x0020, 0x0002, 0xD3C9, 0x0000, 0x0000, 0x0002, /*1D3C9*/
  0xFBC3, 0x0020, 0x0002, 0xD3CA, 0x0000, 0x0000, 0x0002, /*1D3CA*/
  0xFBC3, 0x0020, 0x0002, 0xD3CB, 0x0000, 0x0000, 0x0002, /*1D3CB*/
  0xFBC3, 0x0020, 0x0002, 0xD3CC, 0x0000, 0x0000, 0x0002, /*1D3CC*/
  0xFBC3, 0x0020, 0x0002, 0xD3CD, 0x0000, 0x0000, 0x0002, /*1D3CD*/
  0xFBC3, 0x0020, 0x0002, 0xD3CE, 0x0000, 0x0000, 0x0002, /*1D3CE*/
  0xFBC3, 0x0020, 0x0002, 0xD3CF, 0x0000, 0x0000, 0x0002, /*1D3CF*/
  0xFBC3, 0x0020, 0x0002, 0xD3D0, 0x0000, 0x0000, 0x0002, /*1D3D0*/
  0xFBC3, 0x0020, 0x0002, 0xD3D1, 0x0000, 0x0000, 0x0002, /*1D3D1*/
  0xFBC3, 0x0020, 0x0002, 0xD3D2, 0x0000, 0x0000, 0x0002, /*1D3D2*/
  0xFBC3, 0x0020, 0x0002, 0xD3D3, 0x0000, 0x0000, 0x0002, /*1D3D3*/
  0xFBC3, 0x0020, 0x0002, 0xD3D4, 0x0000, 0x0000, 0x0002, /*1D3D4*/
  0xFBC3, 0x0020, 0x0002, 0xD3D5, 0x0000, 0x0000, 0x0002, /*1D3D5*/
  0xFBC3, 0x0020, 0x0002, 0xD3D6, 0x0000, 0x0000, 0x0002, /*1D3D6*/
  0xFBC3, 0x0020, 0x0002, 0xD3D7, 0x0000, 0x0000, 0x0002, /*1D3D7*/
  0xFBC3, 0x0020, 0x0002, 0xD3D8, 0x0000, 0x0000, 0x0002, /*1D3D8*/
  0xFBC3, 0x0020, 0x0002, 0xD3D9, 0x0000, 0x0000, 0x0002, /*1D3D9*/
  0xFBC3, 0x0020, 0x0002, 0xD3DA, 0x0000, 0x0000, 0x0002, /*1D3DA*/
  0xFBC3, 0x0020, 0x0002, 0xD3DB, 0x0000, 0x0000, 0x0002, /*1D3DB*/
  0xFBC3, 0x0020, 0x0002, 0xD3DC, 0x0000, 0x0000, 0x0002, /*1D3DC*/
  0xFBC3, 0x0020, 0x0002, 0xD3DD, 0x0000, 0x0000, 0x0002, /*1D3DD*/
  0xFBC3, 0x0020, 0x0002, 0xD3DE, 0x0000, 0x0000, 0x0002, /*1D3DE*/
  0xFBC3, 0x0020, 0x0002, 0xD3DF, 0x0000, 0x0000, 0x0002, /*1D3DF*/
  0xFBC3, 0x0020, 0x0002, 0xD3E0, 0x0000, 0x0000, 0x0002, /*1D3E0*/
  0xFBC3, 0x0020, 0x0002, 0xD3E1, 0x0000, 0x0000, 0x0002, /*1D3E1*/
  0xFBC3, 0x0020, 0x0002, 0xD3E2, 0x0000, 0x0000, 0x0002, /*1D3E2*/
  0xFBC3, 0x0020, 0x0002, 0xD3E3, 0x0000, 0x0000, 0x0002, /*1D3E3*/
  0xFBC3, 0x0020, 0x0002, 0xD3E4, 0x0000, 0x0000, 0x0002, /*1D3E4*/
  0xFBC3, 0x0020, 0x0002, 0xD3E5, 0x0000, 0x0000, 0x0002, /*1D3E5*/
  0xFBC3, 0x0020, 0x0002, 0xD3E6, 0x0000, 0x0000, 0x0002, /*1D3E6*/
  0xFBC3, 0x0020, 0x0002, 0xD3E7, 0x0000, 0x0000, 0x0002, /*1D3E7*/
  0xFBC3, 0x0020, 0x0002, 0xD3E8, 0x0000, 0x0000, 0x0002, /*1D3E8*/
  0xFBC3, 0x0020, 0x0002, 0xD3E9, 0x0000, 0x0000, 0x0002, /*1D3E9*/
  0xFBC3, 0x0020, 0x0002, 0xD3EA, 0x0000, 0x0000, 0x0002, /*1D3EA*/
  0xFBC3, 0x0020, 0x0002, 0xD3EB, 0x0000, 0x0000, 0x0002, /*1D3EB*/
  0xFBC3, 0x0020, 0x0002, 0xD3EC, 0x0000, 0x0000, 0x0002, /*1D3EC*/
  0xFBC3, 0x0020, 0x0002, 0xD3ED, 0x0000, 0x0000, 0x0002, /*1D3ED*/
  0xFBC3, 0x0020, 0x0002, 0xD3EE, 0x0000, 0x0000, 0x0002, /*1D3EE*/
  0xFBC3, 0x0020, 0x0002, 0xD3EF, 0x0000, 0x0000, 0x0002, /*1D3EF*/
  0xFBC3, 0x0020, 0x0002, 0xD3F0, 0x0000, 0x0000, 0x0002, /*1D3F0*/
  0xFBC3, 0x0020, 0x0002, 0xD3F1, 0x0000, 0x0000, 0x0002, /*1D3F1*/
  0xFBC3, 0x0020, 0x0002, 0xD3F2, 0x0000, 0x0000, 0x0002, /*1D3F2*/
  0xFBC3, 0x0020, 0x0002, 0xD3F3, 0x0000, 0x0000, 0x0002, /*1D3F3*/
  0xFBC3, 0x0020, 0x0002, 0xD3F4, 0x0000, 0x0000, 0x0002, /*1D3F4*/
  0xFBC3, 0x0020, 0x0002, 0xD3F5, 0x0000, 0x0000, 0x0002, /*1D3F5*/
  0xFBC3, 0x0020, 0x0002, 0xD3F6, 0x0000, 0x0000, 0x0002, /*1D3F6*/
  0xFBC3, 0x0020, 0x0002, 0xD3F7, 0x0000, 0x0000, 0x0002, /*1D3F7*/
  0xFBC3, 0x0020, 0x0002, 0xD3F8, 0x0000, 0x0000, 0x0002, /*1D3F8*/
  0xFBC3, 0x0020, 0x0002, 0xD3F9, 0x0000, 0x0000, 0x0002, /*1D3F9*/
  0xFBC3, 0x0020, 0x0002, 0xD3FA, 0x0000, 0x0000, 0x0002, /*1D3FA*/
  0xFBC3, 0x0020, 0x0002, 0xD3FB, 0x0000, 0x0000, 0x0002, /*1D3FB*/
  0xFBC3, 0x0020, 0x0002, 0xD3FC, 0x0000, 0x0000, 0x0002, /*1D3FC*/
  0xFBC3, 0x0020, 0x0002, 0xD3FD, 0x0000, 0x0000, 0x0002, /*1D3FD*/
  0xFBC3, 0x0020, 0x0002, 0xD3FE, 0x0000, 0x0000, 0x0002, /*1D3FE*/
  0xFBC3, 0x0020, 0x0002, 0xD3FF, 0x0000, 0x0000, 0x0002  /*1D3FF*/
};

uint16 uca900_p1D4[]= {
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D400*/
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D401*/
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D402*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D403*/
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D404*/
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D405*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D406*/
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D407*/
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D408*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D409*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D40A*/
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D40B*/
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D40C*/
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D40D*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D40E*/
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D40F*/
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D410*/
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D411*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D412*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D413*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D414*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D415*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D416*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D417*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D418*/
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D419*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D41A*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D41B*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D41C*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D41D*/
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D41E*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D41F*/
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D420*/
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D421*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D422*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D423*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D424*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D425*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D426*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D427*/
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D428*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D429*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D42A*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D42B*/
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D42C*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D42D*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D42E*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D42F*/
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D430*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D431*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D432*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D433*/
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D434*/
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D435*/
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D436*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D437*/
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D438*/
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D439*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D43A*/
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D43B*/
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D43C*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D43D*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D43E*/
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D43F*/
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D440*/
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D441*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D442*/
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D443*/
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D444*/
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D445*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D446*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D447*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D448*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D449*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D44A*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D44B*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D44C*/
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D44D*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D44E*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D44F*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D450*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D451*/
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D452*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D453*/
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D454*/
  0xFBC3, 0x0020, 0x0002, 0xD455, 0x0000, 0x0000, 0x0002, /*1D455*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D456*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D457*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D458*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D459*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D45A*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D45B*/
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D45C*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D45D*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D45E*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D45F*/
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D460*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D461*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D462*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D463*/
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D464*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D465*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D466*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D467*/
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D468*/
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D469*/
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D46A*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D46B*/
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D46C*/
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D46D*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D46E*/
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D46F*/
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D470*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D471*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D472*/
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D473*/
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D474*/
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D475*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D476*/
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D477*/
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D478*/
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D479*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D47A*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D47B*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D47C*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D47D*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D47E*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D47F*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D480*/
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D481*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D482*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D483*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D484*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D485*/
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D486*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D487*/
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D488*/
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D489*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D48A*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D48B*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D48C*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D48D*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D48E*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D48F*/
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D490*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D491*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D492*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D493*/
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D494*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D495*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D496*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D497*/
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D498*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D499*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D49A*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D49B*/
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D49C*/
  0xFBC3, 0x0020, 0x0002, 0xD49D, 0x0000, 0x0000, 0x0002, /*1D49D*/
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D49E*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D49F*/
  0xFBC3, 0x0020, 0x0002, 0xD4A0, 0x0000, 0x0000, 0x0002, /*1D4A0*/
  0xFBC3, 0x0020, 0x0002, 0xD4A1, 0x0000, 0x0000, 0x0002, /*1D4A1*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4A2*/
  0xFBC3, 0x0020, 0x0002, 0xD4A3, 0x0000, 0x0000, 0x0002, /*1D4A3*/
  0xFBC3, 0x0020, 0x0002, 0xD4A4, 0x0000, 0x0000, 0x0002, /*1D4A4*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4A5*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4A6*/
  0xFBC3, 0x0020, 0x0002, 0xD4A7, 0x0000, 0x0000, 0x0002, /*1D4A7*/
  0xFBC3, 0x0020, 0x0002, 0xD4A8, 0x0000, 0x0000, 0x0002, /*1D4A8*/
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4A9*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4AA*/
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4AB*/
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4AC*/
  0xFBC3, 0x0020, 0x0002, 0xD4AD, 0x0000, 0x0000, 0x0002, /*1D4AD*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4AE*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4AF*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4B0*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4B1*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4B2*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4B3*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4B4*/
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4B5*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4B6*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4B7*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4B8*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4B9*/
  0xFBC3, 0x0020, 0x0002, 0xD4BA, 0x0000, 0x0000, 0x0002, /*1D4BA*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4BB*/
  0xFBC3, 0x0020, 0x0002, 0xD4BC, 0x0000, 0x0000, 0x0002, /*1D4BC*/
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4BD*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4BE*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4BF*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4C0*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4C1*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4C2*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4C3*/
  0xFBC3, 0x0020, 0x0002, 0xD4C4, 0x0000, 0x0000, 0x0002, /*1D4C4*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4C5*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4C6*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4C7*/
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4C8*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4C9*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4CA*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4CB*/
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4CC*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4CD*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4CE*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4CF*/
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4D0*/
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4D1*/
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4D2*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4D3*/
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4D4*/
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4D5*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4D6*/
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4D7*/
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4D8*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4D9*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4DA*/
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4DB*/
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4DC*/
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4DD*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4DE*/
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4DF*/
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4E0*/
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4E1*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4E2*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4E3*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4E4*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4E5*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4E6*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4E7*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4E8*/
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4E9*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4EA*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4EB*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4EC*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4ED*/
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4EE*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4EF*/
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4F0*/
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4F1*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4F2*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4F3*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4F4*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4F5*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4F6*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4F7*/
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4F8*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4F9*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4FA*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4FB*/
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4FC*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4FD*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D4FE*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001  /*1D4FF*/
};

uint16 uca900_p1D5[]= {
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D500*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D501*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D502*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D503*/
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D504*/
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D505*/
  0xFBC3, 0x0020, 0x0002, 0xD506, 0x0000, 0x0000, 0x0002, /*1D506*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D507*/
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D508*/
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D509*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D50A*/
  0xFBC3, 0x0020, 0x0002, 0xD50B, 0x0000, 0x0000, 0x0002, /*1D50B*/
  0xFBC3, 0x0020, 0x0002, 0xD50C, 0x0000, 0x0000, 0x0002, /*1D50C*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D50D*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D50E*/
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D50F*/
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D510*/
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D511*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D512*/
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D513*/
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D514*/
  0xFBC3, 0x0020, 0x0002, 0xD515, 0x0000, 0x0000, 0x0002, /*1D515*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D516*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D517*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D518*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D519*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D51A*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D51B*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D51C*/
  0xFBC3, 0x0020, 0x0002, 0xD51D, 0x0000, 0x0000, 0x0002, /*1D51D*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D51E*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D51F*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D520*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D521*/
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D522*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D523*/
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D524*/
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D525*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D526*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D527*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D528*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D529*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D52A*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D52B*/
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D52C*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D52D*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D52E*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D52F*/
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D530*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D531*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D532*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D533*/
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D534*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D535*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D536*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D537*/
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D538*/
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D539*/
  0xFBC3, 0x0020, 0x0002, 0xD53A, 0x0000, 0x0000, 0x0002, /*1D53A*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D53B*/
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D53C*/
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D53D*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D53E*/
  0xFBC3, 0x0020, 0x0002, 0xD53F, 0x0000, 0x0000, 0x0002, /*1D53F*/
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D540*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D541*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D542*/
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D543*/
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D544*/
  0xFBC3, 0x0020, 0x0002, 0xD545, 0x0000, 0x0000, 0x0002, /*1D545*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D546*/
  0xFBC3, 0x0020, 0x0002, 0xD547, 0x0000, 0x0000, 0x0002, /*1D547*/
  0xFBC3, 0x0020, 0x0002, 0xD548, 0x0000, 0x0000, 0x0002, /*1D548*/
  0xFBC3, 0x0020, 0x0002, 0xD549, 0x0000, 0x0000, 0x0002, /*1D549*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D54A*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D54B*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D54C*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D54D*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D54E*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D54F*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D550*/
  0xFBC3, 0x0020, 0x0002, 0xD551, 0x0000, 0x0000, 0x0002, /*1D551*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D552*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D553*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D554*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D555*/
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D556*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D557*/
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D558*/
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D559*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D55A*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D55B*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D55C*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D55D*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D55E*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D55F*/
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D560*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D561*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D562*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D563*/
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D564*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D565*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D566*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D567*/
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D568*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D569*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D56A*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D56B*/
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D56C*/
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D56D*/
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D56E*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D56F*/
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D570*/
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D571*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D572*/
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D573*/
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D574*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D575*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D576*/
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D577*/
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D578*/
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D579*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D57A*/
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D57B*/
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D57C*/
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D57D*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D57E*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D57F*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D580*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D581*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D582*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D583*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D584*/
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D585*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D586*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D587*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D588*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D589*/
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D58A*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D58B*/
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D58C*/
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D58D*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D58E*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D58F*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D590*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D591*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D592*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D593*/
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D594*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D595*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D596*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D597*/
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D598*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D599*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D59A*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D59B*/
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D59C*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D59D*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D59E*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D59F*/
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5A0*/
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5A1*/
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5A2*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5A3*/
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5A4*/
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5A5*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5A6*/
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5A7*/
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5A8*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5A9*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5AA*/
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5AB*/
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5AC*/
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5AD*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5AE*/
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5AF*/
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5B0*/
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5B1*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5B2*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5B3*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5B4*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5B5*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5B6*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5B7*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5B8*/
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5B9*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5BA*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5BB*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5BC*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5BD*/
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5BE*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5BF*/
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5C0*/
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5C1*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5C2*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5C3*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5C4*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5C5*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5C6*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5C7*/
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5C8*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5C9*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5CA*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5CB*/
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5CC*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5CD*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5CE*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5CF*/
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5D0*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5D1*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5D2*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5D3*/
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5D4*/
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5D5*/
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5D6*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5D7*/
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5D8*/
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5D9*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5DA*/
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5DB*/
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5DC*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5DD*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5DE*/
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5DF*/
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5E0*/
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5E1*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5E2*/
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5E3*/
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5E4*/
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5E5*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5E6*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5E7*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5E8*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5E9*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5EA*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5EB*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5EC*/
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5ED*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5EE*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5EF*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5F0*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5F1*/
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5F2*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5F3*/
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5F4*/
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5F5*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5F6*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5F7*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5F8*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5F9*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5FA*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5FB*/
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5FC*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5FD*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D5FE*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001  /*1D5FF*/
};

uint16 uca900_p1D6[]= {
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D600*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D601*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D602*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D603*/
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D604*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D605*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D606*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D607*/
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D608*/
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D609*/
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D60A*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D60B*/
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D60C*/
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D60D*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D60E*/
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D60F*/
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D610*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D611*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D612*/
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D613*/
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D614*/
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D615*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D616*/
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D617*/
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D618*/
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D619*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D61A*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D61B*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D61C*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D61D*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D61E*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D61F*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D620*/
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D621*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D622*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D623*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D624*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D625*/
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D626*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D627*/
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D628*/
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D629*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D62A*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D62B*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D62C*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D62D*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D62E*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D62F*/
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D630*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D631*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D632*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D633*/
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D634*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D635*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D636*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D637*/
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D638*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D639*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D63A*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D63B*/
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D63C*/
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D63D*/
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D63E*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D63F*/
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D640*/
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D641*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D642*/
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D643*/
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D644*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D645*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D646*/
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D647*/
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D648*/
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D649*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D64A*/
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D64B*/
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D64C*/
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D64D*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D64E*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D64F*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D650*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D651*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D652*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D653*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D654*/
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D655*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D656*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D657*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D658*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D659*/
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D65A*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D65B*/
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D65C*/
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D65D*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D65E*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D65F*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D660*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D661*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D662*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D663*/
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D664*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D665*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D666*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D667*/
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D668*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D669*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D66A*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D66B*/
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D66C*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D66D*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D66E*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D66F*/
  0x1C47, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D670*/
  0x1C60, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D671*/
  0x1C7A, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D672*/
  0x1C8F, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D673*/
  0x1CAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D674*/
  0x1CE5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D675*/
  0x1CF4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D676*/
  0x1D18, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D677*/
  0x1D32, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D678*/
  0x1D4C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D679*/
  0x1D65, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D67A*/
  0x1D77, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D67B*/
  0x1DAA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D67C*/
  0x1DB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D67D*/
  0x1DDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D67E*/
  0x1E0C, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D67F*/
  0x1E21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D680*/
  0x1E33, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D681*/
  0x1E71, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D682*/
  0x1E95, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D683*/
  0x1EB5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D684*/
  0x1EE3, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D685*/
  0x1EF5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D686*/
  0x1EFF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D687*/
  0x1F0B, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D688*/
  0x1F21, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D689*/
  0x1C47, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D68A*/
  0x1C60, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D68B*/
  0x1C7A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D68C*/
  0x1C8F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D68D*/
  0x1CAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D68E*/
  0x1CE5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D68F*/
  0x1CF4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D690*/
  0x1D18, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D691*/
  0x1D32, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D692*/
  0x1D4C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D693*/
  0x1D65, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D694*/
  0x1D77, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D695*/
  0x1DAA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D696*/
  0x1DB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D697*/
  0x1DDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D698*/
  0x1E0C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D699*/
  0x1E21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D69A*/
  0x1E33, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D69B*/
  0x1E71, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D69C*/
  0x1E95, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D69D*/
  0x1EB5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D69E*/
  0x1EE3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D69F*/
  0x1EF5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6A0*/
  0x1EFF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6A1*/
  0x1F0B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6A2*/
  0x1F21, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6A3*/
  0x1D36, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6A4*/
  0x1D50, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6A5*/
  0xFBC3, 0x0020, 0x0002, 0xD6A6, 0x0000, 0x0000, 0x0002, /*1D6A6*/
  0xFBC3, 0x0020, 0x0002, 0xD6A7, 0x0000, 0x0000, 0x0002, /*1D6A7*/
  0x1FB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6A8*/
  0x1FBA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6A9*/
  0x1FBB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6AA*/
  0x1FBD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6AB*/
  0x1FBE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6AC*/
  0x1FC2, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6AD*/
  0x1FC4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6AE*/
  0x1FC5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6AF*/
  0x1FC6, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6B0*/
  0x1FC8, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6B1*/
  0x1FC9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6B2*/
  0x1FCB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6B3*/
  0x1FCC, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6B4*/
  0x1FCD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6B5*/
  0x1FCE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6B6*/
  0x1FCF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6B7*/
  0x1FD4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6B8*/
  0x1FC5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6B9*/
  0x1FD7, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6BA*/
  0x1FDB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6BB*/
  0x1FDC, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6BC*/
  0x1FDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6BD*/
  0x1FDE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6BE*/
  0x1FDF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6BF*/
  0x1FE1, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6C0*/
  0x060C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6C1*/
  0x1FB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6C2*/
  0x1FBA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6C3*/
  0x1FBB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6C4*/
  0x1FBD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6C5*/
  0x1FBE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6C6*/
  0x1FC2, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6C7*/
  0x1FC4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6C8*/
  0x1FC5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6C9*/
  0x1FC6, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6CA*/
  0x1FC8, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6CB*/
  0x1FC9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6CC*/
  0x1FCB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6CD*/
  0x1FCC, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6CE*/
  0x1FCD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6CF*/
  0x1FCE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6D0*/
  0x1FCF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6D1*/
  0x1FD4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6D2*/
  0x1FD7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6D3*/
  0x1FD7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6D4*/
  0x1FDB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6D5*/
  0x1FDC, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6D6*/
  0x1FDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6D7*/
  0x1FDE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6D8*/
  0x1FDF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6D9*/
  0x1FE1, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6DA*/
  0x0608, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6DB*/
  0x1FBE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6DC*/
  0x1FC5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6DD*/
  0x1FC8, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6DE*/
  0x1FDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6DF*/
  0x1FD4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6E0*/
  0x1FCF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6E1*/
  0x1FB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6E2*/
  0x1FBA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6E3*/
  0x1FBB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6E4*/
  0x1FBD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6E5*/
  0x1FBE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6E6*/
  0x1FC2, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6E7*/
  0x1FC4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6E8*/
  0x1FC5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6E9*/
  0x1FC6, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6EA*/
  0x1FC8, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6EB*/
  0x1FC9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6EC*/
  0x1FCB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6ED*/
  0x1FCC, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6EE*/
  0x1FCD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6EF*/
  0x1FCE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6F0*/
  0x1FCF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6F1*/
  0x1FD4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6F2*/
  0x1FC5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6F3*/
  0x1FD7, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6F4*/
  0x1FDB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6F5*/
  0x1FDC, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6F6*/
  0x1FDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6F7*/
  0x1FDE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6F8*/
  0x1FDF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6F9*/
  0x1FE1, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6FA*/
  0x060C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6FB*/
  0x1FB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6FC*/
  0x1FBA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6FD*/
  0x1FBB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D6FE*/
  0x1FBD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001  /*1D6FF*/
};

uint16 uca900_p1D7[]= {
  0x1FBE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D700*/
  0x1FC2, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D701*/
  0x1FC4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D702*/
  0x1FC5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D703*/
  0x1FC6, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D704*/
  0x1FC8, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D705*/
  0x1FC9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D706*/
  0x1FCB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D707*/
  0x1FCC, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D708*/
  0x1FCD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D709*/
  0x1FCE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D70A*/
  0x1FCF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D70B*/
  0x1FD4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D70C*/
  0x1FD7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D70D*/
  0x1FD7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D70E*/
  0x1FDB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D70F*/
  0x1FDC, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D710*/
  0x1FDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D711*/
  0x1FDE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D712*/
  0x1FDF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D713*/
  0x1FE1, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D714*/
  0x0608, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D715*/
  0x1FBE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D716*/
  0x1FC5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D717*/
  0x1FC8, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D718*/
  0x1FDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D719*/
  0x1FD4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D71A*/
  0x1FCF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D71B*/
  0x1FB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D71C*/
  0x1FBA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D71D*/
  0x1FBB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D71E*/
  0x1FBD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D71F*/
  0x1FBE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D720*/
  0x1FC2, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D721*/
  0x1FC4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D722*/
  0x1FC5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D723*/
  0x1FC6, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D724*/
  0x1FC8, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D725*/
  0x1FC9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D726*/
  0x1FCB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D727*/
  0x1FCC, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D728*/
  0x1FCD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D729*/
  0x1FCE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D72A*/
  0x1FCF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D72B*/
  0x1FD4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D72C*/
  0x1FC5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D72D*/
  0x1FD7, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D72E*/
  0x1FDB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D72F*/
  0x1FDC, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D730*/
  0x1FDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D731*/
  0x1FDE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D732*/
  0x1FDF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D733*/
  0x1FE1, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D734*/
  0x060C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D735*/
  0x1FB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D736*/
  0x1FBA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D737*/
  0x1FBB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D738*/
  0x1FBD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D739*/
  0x1FBE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D73A*/
  0x1FC2, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D73B*/
  0x1FC4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D73C*/
  0x1FC5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D73D*/
  0x1FC6, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D73E*/
  0x1FC8, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D73F*/
  0x1FC9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D740*/
  0x1FCB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D741*/
  0x1FCC, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D742*/
  0x1FCD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D743*/
  0x1FCE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D744*/
  0x1FCF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D745*/
  0x1FD4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D746*/
  0x1FD7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D747*/
  0x1FD7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D748*/
  0x1FDB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D749*/
  0x1FDC, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D74A*/
  0x1FDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D74B*/
  0x1FDE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D74C*/
  0x1FDF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D74D*/
  0x1FE1, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D74E*/
  0x0608, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D74F*/
  0x1FBE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D750*/
  0x1FC5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D751*/
  0x1FC8, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D752*/
  0x1FDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D753*/
  0x1FD4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D754*/
  0x1FCF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D755*/
  0x1FB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D756*/
  0x1FBA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D757*/
  0x1FBB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D758*/
  0x1FBD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D759*/
  0x1FBE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D75A*/
  0x1FC2, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D75B*/
  0x1FC4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D75C*/
  0x1FC5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D75D*/
  0x1FC6, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D75E*/
  0x1FC8, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D75F*/
  0x1FC9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D760*/
  0x1FCB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D761*/
  0x1FCC, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D762*/
  0x1FCD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D763*/
  0x1FCE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D764*/
  0x1FCF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D765*/
  0x1FD4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D766*/
  0x1FC5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D767*/
  0x1FD7, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D768*/
  0x1FDB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D769*/
  0x1FDC, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D76A*/
  0x1FDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D76B*/
  0x1FDE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D76C*/
  0x1FDF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D76D*/
  0x1FE1, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D76E*/
  0x060C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D76F*/
  0x1FB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D770*/
  0x1FBA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D771*/
  0x1FBB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D772*/
  0x1FBD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D773*/
  0x1FBE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D774*/
  0x1FC2, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D775*/
  0x1FC4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D776*/
  0x1FC5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D777*/
  0x1FC6, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D778*/
  0x1FC8, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D779*/
  0x1FC9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D77A*/
  0x1FCB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D77B*/
  0x1FCC, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D77C*/
  0x1FCD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D77D*/
  0x1FCE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D77E*/
  0x1FCF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D77F*/
  0x1FD4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D780*/
  0x1FD7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D781*/
  0x1FD7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D782*/
  0x1FDB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D783*/
  0x1FDC, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D784*/
  0x1FDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D785*/
  0x1FDE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D786*/
  0x1FDF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D787*/
  0x1FE1, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D788*/
  0x0608, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D789*/
  0x1FBE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D78A*/
  0x1FC5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D78B*/
  0x1FC8, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D78C*/
  0x1FDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D78D*/
  0x1FD4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D78E*/
  0x1FCF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D78F*/
  0x1FB9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D790*/
  0x1FBA, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D791*/
  0x1FBB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D792*/
  0x1FBD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D793*/
  0x1FBE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D794*/
  0x1FC2, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D795*/
  0x1FC4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D796*/
  0x1FC5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D797*/
  0x1FC6, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D798*/
  0x1FC8, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D799*/
  0x1FC9, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D79A*/
  0x1FCB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D79B*/
  0x1FCC, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D79C*/
  0x1FCD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D79D*/
  0x1FCE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D79E*/
  0x1FCF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D79F*/
  0x1FD4, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7A0*/
  0x1FC5, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7A1*/
  0x1FD7, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7A2*/
  0x1FDB, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7A3*/
  0x1FDC, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7A4*/
  0x1FDD, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7A5*/
  0x1FDE, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7A6*/
  0x1FDF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7A7*/
  0x1FE1, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7A8*/
  0x060C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7A9*/
  0x1FB9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7AA*/
  0x1FBA, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7AB*/
  0x1FBB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7AC*/
  0x1FBD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7AD*/
  0x1FBE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7AE*/
  0x1FC2, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7AF*/
  0x1FC4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7B0*/
  0x1FC5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7B1*/
  0x1FC6, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7B2*/
  0x1FC8, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7B3*/
  0x1FC9, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7B4*/
  0x1FCB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7B5*/
  0x1FCC, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7B6*/
  0x1FCD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7B7*/
  0x1FCE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7B8*/
  0x1FCF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7B9*/
  0x1FD4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7BA*/
  0x1FD7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7BB*/
  0x1FD7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7BC*/
  0x1FDB, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7BD*/
  0x1FDC, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7BE*/
  0x1FDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7BF*/
  0x1FDE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7C0*/
  0x1FDF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7C1*/
  0x1FE1, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7C2*/
  0x0608, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7C3*/
  0x1FBE, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7C4*/
  0x1FC5, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7C5*/
  0x1FC8, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7C6*/
  0x1FDD, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7C7*/
  0x1FD4, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7C8*/
  0x1FCF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7C9*/
  0x1FBF, 0x0020, 0x000B, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7CA*/
  0x1FBF, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7CB*/
  0xFBC3, 0x0020, 0x0002, 0xD7CC, 0x0000, 0x0000, 0x0002, /*1D7CC*/
  0xFBC3, 0x0020, 0x0002, 0xD7CD, 0x0000, 0x0000, 0x0002, /*1D7CD*/
  0x1C3D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7CE*/
  0x1C3E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7CF*/
  0x1C3F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7D0*/
  0x1C40, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7D1*/
  0x1C41, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7D2*/
  0x1C42, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7D3*/
  0x1C43, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7D4*/
  0x1C44, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7D5*/
  0x1C45, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7D6*/
  0x1C46, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7D7*/
  0x1C3D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7D8*/
  0x1C3E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7D9*/
  0x1C3F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7DA*/
  0x1C40, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7DB*/
  0x1C41, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7DC*/
  0x1C42, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7DD*/
  0x1C43, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7DE*/
  0x1C44, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7DF*/
  0x1C45, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7E0*/
  0x1C46, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7E1*/
  0x1C3D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7E2*/
  0x1C3E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7E3*/
  0x1C3F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7E4*/
  0x1C40, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7E5*/
  0x1C41, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7E6*/
  0x1C42, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7E7*/
  0x1C43, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7E8*/
  0x1C44, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7E9*/
  0x1C45, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7EA*/
  0x1C46, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7EB*/
  0x1C3D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7EC*/
  0x1C3E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7ED*/
  0x1C3F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7EE*/
  0x1C40, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7EF*/
  0x1C41, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7F0*/
  0x1C42, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7F1*/
  0x1C43, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7F2*/
  0x1C44, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7F3*/
  0x1C45, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7F4*/
  0x1C46, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7F5*/
  0x1C3D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7F6*/
  0x1C3E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7F7*/
  0x1C3F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7F8*/
  0x1C40, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7F9*/
  0x1C41, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7FA*/
  0x1C42, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7FB*/
  0x1C43, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7FC*/
  0x1C44, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7FD*/
  0x1C45, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1D7FE*/
  0x1C46, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001  /*1D7FF*/
};

uint16 uca900_p1D8[]= {
  0x183F, 0x0020, 0x0002, 0x0001, /*1D800*/
  0x1840, 0x0020, 0x0002, 0x0001, /*1D801*/
  0x1841, 0x0020, 0x0002, 0x0001, /*1D802*/
  0x1842, 0x0020, 0x0002, 0x0001, /*1D803*/
  0x1843, 0x0020, 0x0002, 0x0001, /*1D804*/
  0x1844, 0x0020, 0x0002, 0x0001, /*1D805*/
  0x1845, 0x0020, 0x0002, 0x0001, /*1D806*/
  0x1846, 0x0020, 0x0002, 0x0001, /*1D807*/
  0x1847, 0x0020, 0x0002, 0x0001, /*1D808*/
  0x1848, 0x0020, 0x0002, 0x0001, /*1D809*/
  0x1849, 0x0020, 0x0002, 0x0001, /*1D80A*/
  0x184A, 0x0020, 0x0002, 0x0001, /*1D80B*/
  0x184B, 0x0020, 0x0002, 0x0001, /*1D80C*/
  0x184C, 0x0020, 0x0002, 0x0001, /*1D80D*/
  0x184D, 0x0020, 0x0002, 0x0001, /*1D80E*/
  0x184E, 0x0020, 0x0002, 0x0001, /*1D80F*/
  0x184F, 0x0020, 0x0002, 0x0001, /*1D810*/
  0x1850, 0x0020, 0x0002, 0x0001, /*1D811*/
  0x1851, 0x0020, 0x0002, 0x0001, /*1D812*/
  0x1852, 0x0020, 0x0002, 0x0001, /*1D813*/
  0x1853, 0x0020, 0x0002, 0x0001, /*1D814*/
  0x1854, 0x0020, 0x0002, 0x0001, /*1D815*/
  0x1855, 0x0020, 0x0002, 0x0001, /*1D816*/
  0x1856, 0x0020, 0x0002, 0x0001, /*1D817*/
  0x1857, 0x0020, 0x0002, 0x0001, /*1D818*/
  0x1858, 0x0020, 0x0002, 0x0001, /*1D819*/
  0x1859, 0x0020, 0x0002, 0x0001, /*1D81A*/
  0x185A, 0x0020, 0x0002, 0x0001, /*1D81B*/
  0x185B, 0x0020, 0x0002, 0x0001, /*1D81C*/
  0x185C, 0x0020, 0x0002, 0x0001, /*1D81D*/
  0x185D, 0x0020, 0x0002, 0x0001, /*1D81E*/
  0x185E, 0x0020, 0x0002, 0x0001, /*1D81F*/
  0x185F, 0x0020, 0x0002, 0x0001, /*1D820*/
  0x1860, 0x0020, 0x0002, 0x0001, /*1D821*/
  0x1861, 0x0020, 0x0002, 0x0001, /*1D822*/
  0x1862, 0x0020, 0x0002, 0x0001, /*1D823*/
  0x1863, 0x0020, 0x0002, 0x0001, /*1D824*/
  0x1864, 0x0020, 0x0002, 0x0001, /*1D825*/
  0x1865, 0x0020, 0x0002, 0x0001, /*1D826*/
  0x1866, 0x0020, 0x0002, 0x0001, /*1D827*/
  0x1867, 0x0020, 0x0002, 0x0001, /*1D828*/
  0x1868, 0x0020, 0x0002, 0x0001, /*1D829*/
  0x1869, 0x0020, 0x0002, 0x0001, /*1D82A*/
  0x186A, 0x0020, 0x0002, 0x0001, /*1D82B*/
  0x186B, 0x0020, 0x0002, 0x0001, /*1D82C*/
  0x186C, 0x0020, 0x0002, 0x0001, /*1D82D*/
  0x186D, 0x0020, 0x0002, 0x0001, /*1D82E*/
  0x186E, 0x0020, 0x0002, 0x0001, /*1D82F*/
  0x186F, 0x0020, 0x0002, 0x0001, /*1D830*/
  0x1870, 0x0020, 0x0002, 0x0001, /*1D831*/
  0x1871, 0x0020, 0x0002, 0x0001, /*1D832*/
  0x1872, 0x0020, 0x0002, 0x0001, /*1D833*/
  0x1873, 0x0020, 0x0002, 0x0001, /*1D834*/
  0x1874, 0x0020, 0x0002, 0x0001, /*1D835*/
  0x1875, 0x0020, 0x0002, 0x0001, /*1D836*/
  0x1876, 0x0020, 0x0002, 0x0001, /*1D837*/
  0x1877, 0x0020, 0x0002, 0x0001, /*1D838*/
  0x1878, 0x0020, 0x0002, 0x0001, /*1D839*/
  0x1879, 0x0020, 0x0002, 0x0001, /*1D83A*/
  0x187A, 0x0020, 0x0002, 0x0001, /*1D83B*/
  0x187B, 0x0020, 0x0002, 0x0001, /*1D83C*/
  0x187C, 0x0020, 0x0002, 0x0001, /*1D83D*/
  0x187D, 0x0020, 0x0002, 0x0001, /*1D83E*/
  0x187E, 0x0020, 0x0002, 0x0001, /*1D83F*/
  0x187F, 0x0020, 0x0002, 0x0001, /*1D840*/
  0x1880, 0x0020, 0x0002, 0x0001, /*1D841*/
  0x1881, 0x0020, 0x0002, 0x0001, /*1D842*/
  0x1882, 0x0020, 0x0002, 0x0001, /*1D843*/
  0x1883, 0x0020, 0x0002, 0x0001, /*1D844*/
  0x1884, 0x0020, 0x0002, 0x0001, /*1D845*/
  0x1885, 0x0020, 0x0002, 0x0001, /*1D846*/
  0x1886, 0x0020, 0x0002, 0x0001, /*1D847*/
  0x1887, 0x0020, 0x0002, 0x0001, /*1D848*/
  0x1888, 0x0020, 0x0002, 0x0001, /*1D849*/
  0x1889, 0x0020, 0x0002, 0x0001, /*1D84A*/
  0x188A, 0x0020, 0x0002, 0x0001, /*1D84B*/
  0x188B, 0x0020, 0x0002, 0x0001, /*1D84C*/
  0x188C, 0x0020, 0x0002, 0x0001, /*1D84D*/
  0x188D, 0x0020, 0x0002, 0x0001, /*1D84E*/
  0x188E, 0x0020, 0x0002, 0x0001, /*1D84F*/
  0x188F, 0x0020, 0x0002, 0x0001, /*1D850*/
  0x1890, 0x0020, 0x0002, 0x0001, /*1D851*/
  0x1891, 0x0020, 0x0002, 0x0001, /*1D852*/
  0x1892, 0x0020, 0x0002, 0x0001, /*1D853*/
  0x1893, 0x0020, 0x0002, 0x0001, /*1D854*/
  0x1894, 0x0020, 0x0002, 0x0001, /*1D855*/
  0x1895, 0x0020, 0x0002, 0x0001, /*1D856*/
  0x1896, 0x0020, 0x0002, 0x0001, /*1D857*/
  0x1897, 0x0020, 0x0002, 0x0001, /*1D858*/
  0x1898, 0x0020, 0x0002, 0x0001, /*1D859*/
  0x1899, 0x0020, 0x0002, 0x0001, /*1D85A*/
  0x189A, 0x0020, 0x0002, 0x0001, /*1D85B*/
  0x189B, 0x0020, 0x0002, 0x0001, /*1D85C*/
  0x189C, 0x0020, 0x0002, 0x0001, /*1D85D*/
  0x189D, 0x0020, 0x0002, 0x0001, /*1D85E*/
  0x189E, 0x0020, 0x0002, 0x0001, /*1D85F*/
  0x189F, 0x0020, 0x0002, 0x0001, /*1D860*/
  0x18A0, 0x0020, 0x0002, 0x0001, /*1D861*/
  0x18A1, 0x0020, 0x0002, 0x0001, /*1D862*/
  0x18A2, 0x0020, 0x0002, 0x0001, /*1D863*/
  0x18A3, 0x0020, 0x0002, 0x0001, /*1D864*/
  0x18A4, 0x0020, 0x0002, 0x0001, /*1D865*/
  0x18A5, 0x0020, 0x0002, 0x0001, /*1D866*/
  0x18A6, 0x0020, 0x0002, 0x0001, /*1D867*/
  0x18A7, 0x0020, 0x0002, 0x0001, /*1D868*/
  0x18A8, 0x0020, 0x0002, 0x0001, /*1D869*/
  0x18A9, 0x0020, 0x0002, 0x0001, /*1D86A*/
  0x18AA, 0x0020, 0x0002, 0x0001, /*1D86B*/
  0x18AB, 0x0020, 0x0002, 0x0001, /*1D86C*/
  0x18AC, 0x0020, 0x0002, 0x0001, /*1D86D*/
  0x18AD, 0x0020, 0x0002, 0x0001, /*1D86E*/
  0x18AE, 0x0020, 0x0002, 0x0001, /*1D86F*/
  0x18AF, 0x0020, 0x0002, 0x0001, /*1D870*/
  0x18B0, 0x0020, 0x0002, 0x0001, /*1D871*/
  0x18B1, 0x0020, 0x0002, 0x0001, /*1D872*/
  0x18B2, 0x0020, 0x0002, 0x0001, /*1D873*/
  0x18B3, 0x0020, 0x0002, 0x0001, /*1D874*/
  0x18B4, 0x0020, 0x0002, 0x0001, /*1D875*/
  0x18B5, 0x0020, 0x0002, 0x0001, /*1D876*/
  0x18B6, 0x0020, 0x0002, 0x0001, /*1D877*/
  0x18B7, 0x0020, 0x0002, 0x0001, /*1D878*/
  0x18B8, 0x0020, 0x0002, 0x0001, /*1D879*/
  0x18B9, 0x0020, 0x0002, 0x0001, /*1D87A*/
  0x18BA, 0x0020, 0x0002, 0x0001, /*1D87B*/
  0x18BB, 0x0020, 0x0002, 0x0001, /*1D87C*/
  0x18BC, 0x0020, 0x0002, 0x0001, /*1D87D*/
  0x18BD, 0x0020, 0x0002, 0x0001, /*1D87E*/
  0x18BE, 0x0020, 0x0002, 0x0001, /*1D87F*/
  0x18BF, 0x0020, 0x0002, 0x0001, /*1D880*/
  0x18C0, 0x0020, 0x0002, 0x0001, /*1D881*/
  0x18C1, 0x0020, 0x0002, 0x0001, /*1D882*/
  0x18C2, 0x0020, 0x0002, 0x0001, /*1D883*/
  0x18C3, 0x0020, 0x0002, 0x0001, /*1D884*/
  0x18C4, 0x0020, 0x0002, 0x0001, /*1D885*/
  0x18C5, 0x0020, 0x0002, 0x0001, /*1D886*/
  0x18C6, 0x0020, 0x0002, 0x0001, /*1D887*/
  0x18C7, 0x0020, 0x0002, 0x0001, /*1D888*/
  0x18C8, 0x0020, 0x0002, 0x0001, /*1D889*/
  0x18C9, 0x0020, 0x0002, 0x0001, /*1D88A*/
  0x18CA, 0x0020, 0x0002, 0x0001, /*1D88B*/
  0x18CB, 0x0020, 0x0002, 0x0001, /*1D88C*/
  0x18CC, 0x0020, 0x0002, 0x0001, /*1D88D*/
  0x18CD, 0x0020, 0x0002, 0x0001, /*1D88E*/
  0x18CE, 0x0020, 0x0002, 0x0001, /*1D88F*/
  0x18CF, 0x0020, 0x0002, 0x0001, /*1D890*/
  0x18D0, 0x0020, 0x0002, 0x0001, /*1D891*/
  0x18D1, 0x0020, 0x0002, 0x0001, /*1D892*/
  0x18D2, 0x0020, 0x0002, 0x0001, /*1D893*/
  0x18D3, 0x0020, 0x0002, 0x0001, /*1D894*/
  0x18D4, 0x0020, 0x0002, 0x0001, /*1D895*/
  0x18D5, 0x0020, 0x0002, 0x0001, /*1D896*/
  0x18D6, 0x0020, 0x0002, 0x0001, /*1D897*/
  0x18D7, 0x0020, 0x0002, 0x0001, /*1D898*/
  0x18D8, 0x0020, 0x0002, 0x0001, /*1D899*/
  0x18D9, 0x0020, 0x0002, 0x0001, /*1D89A*/
  0x18DA, 0x0020, 0x0002, 0x0001, /*1D89B*/
  0x18DB, 0x0020, 0x0002, 0x0001, /*1D89C*/
  0x18DC, 0x0020, 0x0002, 0x0001, /*1D89D*/
  0x18DD, 0x0020, 0x0002, 0x0001, /*1D89E*/
  0x18DE, 0x0020, 0x0002, 0x0001, /*1D89F*/
  0x18DF, 0x0020, 0x0002, 0x0001, /*1D8A0*/
  0x18E0, 0x0020, 0x0002, 0x0001, /*1D8A1*/
  0x18E1, 0x0020, 0x0002, 0x0001, /*1D8A2*/
  0x18E2, 0x0020, 0x0002, 0x0001, /*1D8A3*/
  0x18E3, 0x0020, 0x0002, 0x0001, /*1D8A4*/
  0x18E4, 0x0020, 0x0002, 0x0001, /*1D8A5*/
  0x18E5, 0x0020, 0x0002, 0x0001, /*1D8A6*/
  0x18E6, 0x0020, 0x0002, 0x0001, /*1D8A7*/
  0x18E7, 0x0020, 0x0002, 0x0001, /*1D8A8*/
  0x18E8, 0x0020, 0x0002, 0x0001, /*1D8A9*/
  0x18E9, 0x0020, 0x0002, 0x0001, /*1D8AA*/
  0x18EA, 0x0020, 0x0002, 0x0001, /*1D8AB*/
  0x18EB, 0x0020, 0x0002, 0x0001, /*1D8AC*/
  0x18EC, 0x0020, 0x0002, 0x0001, /*1D8AD*/
  0x18ED, 0x0020, 0x0002, 0x0001, /*1D8AE*/
  0x18EE, 0x0020, 0x0002, 0x0001, /*1D8AF*/
  0x18EF, 0x0020, 0x0002, 0x0001, /*1D8B0*/
  0x18F0, 0x0020, 0x0002, 0x0001, /*1D8B1*/
  0x18F1, 0x0020, 0x0002, 0x0001, /*1D8B2*/
  0x18F2, 0x0020, 0x0002, 0x0001, /*1D8B3*/
  0x18F3, 0x0020, 0x0002, 0x0001, /*1D8B4*/
  0x18F4, 0x0020, 0x0002, 0x0001, /*1D8B5*/
  0x18F5, 0x0020, 0x0002, 0x0001, /*1D8B6*/
  0x18F6, 0x0020, 0x0002, 0x0001, /*1D8B7*/
  0x18F7, 0x0020, 0x0002, 0x0001, /*1D8B8*/
  0x18F8, 0x0020, 0x0002, 0x0001, /*1D8B9*/
  0x18F9, 0x0020, 0x0002, 0x0001, /*1D8BA*/
  0x18FA, 0x0020, 0x0002, 0x0001, /*1D8BB*/
  0x18FB, 0x0020, 0x0002, 0x0001, /*1D8BC*/
  0x18FC, 0x0020, 0x0002, 0x0001, /*1D8BD*/
  0x18FD, 0x0020, 0x0002, 0x0001, /*1D8BE*/
  0x18FE, 0x0020, 0x0002, 0x0001, /*1D8BF*/
  0x18FF, 0x0020, 0x0002, 0x0001, /*1D8C0*/
  0x1900, 0x0020, 0x0002, 0x0001, /*1D8C1*/
  0x1901, 0x0020, 0x0002, 0x0001, /*1D8C2*/
  0x1902, 0x0020, 0x0002, 0x0001, /*1D8C3*/
  0x1903, 0x0020, 0x0002, 0x0001, /*1D8C4*/
  0x1904, 0x0020, 0x0002, 0x0001, /*1D8C5*/
  0x1905, 0x0020, 0x0002, 0x0001, /*1D8C6*/
  0x1906, 0x0020, 0x0002, 0x0001, /*1D8C7*/
  0x1907, 0x0020, 0x0002, 0x0001, /*1D8C8*/
  0x1908, 0x0020, 0x0002, 0x0001, /*1D8C9*/
  0x1909, 0x0020, 0x0002, 0x0001, /*1D8CA*/
  0x190A, 0x0020, 0x0002, 0x0001, /*1D8CB*/
  0x190B, 0x0020, 0x0002, 0x0001, /*1D8CC*/
  0x190C, 0x0020, 0x0002, 0x0001, /*1D8CD*/
  0x190D, 0x0020, 0x0002, 0x0001, /*1D8CE*/
  0x190E, 0x0020, 0x0002, 0x0001, /*1D8CF*/
  0x190F, 0x0020, 0x0002, 0x0001, /*1D8D0*/
  0x1910, 0x0020, 0x0002, 0x0001, /*1D8D1*/
  0x1911, 0x0020, 0x0002, 0x0001, /*1D8D2*/
  0x1912, 0x0020, 0x0002, 0x0001, /*1D8D3*/
  0x1913, 0x0020, 0x0002, 0x0001, /*1D8D4*/
  0x1914, 0x0020, 0x0002, 0x0001, /*1D8D5*/
  0x1915, 0x0020, 0x0002, 0x0001, /*1D8D6*/
  0x1916, 0x0020, 0x0002, 0x0001, /*1D8D7*/
  0x1917, 0x0020, 0x0002, 0x0001, /*1D8D8*/
  0x1918, 0x0020, 0x0002, 0x0001, /*1D8D9*/
  0x1919, 0x0020, 0x0002, 0x0001, /*1D8DA*/
  0x191A, 0x0020, 0x0002, 0x0001, /*1D8DB*/
  0x191B, 0x0020, 0x0002, 0x0001, /*1D8DC*/
  0x191C, 0x0020, 0x0002, 0x0001, /*1D8DD*/
  0x191D, 0x0020, 0x0002, 0x0001, /*1D8DE*/
  0x191E, 0x0020, 0x0002, 0x0001, /*1D8DF*/
  0x191F, 0x0020, 0x0002, 0x0001, /*1D8E0*/
  0x1920, 0x0020, 0x0002, 0x0001, /*1D8E1*/
  0x1921, 0x0020, 0x0002, 0x0001, /*1D8E2*/
  0x1922, 0x0020, 0x0002, 0x0001, /*1D8E3*/
  0x1923, 0x0020, 0x0002, 0x0001, /*1D8E4*/
  0x1924, 0x0020, 0x0002, 0x0001, /*1D8E5*/
  0x1925, 0x0020, 0x0002, 0x0001, /*1D8E6*/
  0x1926, 0x0020, 0x0002, 0x0001, /*1D8E7*/
  0x1927, 0x0020, 0x0002, 0x0001, /*1D8E8*/
  0x1928, 0x0020, 0x0002, 0x0001, /*1D8E9*/
  0x1929, 0x0020, 0x0002, 0x0001, /*1D8EA*/
  0x192A, 0x0020, 0x0002, 0x0001, /*1D8EB*/
  0x192B, 0x0020, 0x0002, 0x0001, /*1D8EC*/
  0x192C, 0x0020, 0x0002, 0x0001, /*1D8ED*/
  0x192D, 0x0020, 0x0002, 0x0001, /*1D8EE*/
  0x192E, 0x0020, 0x0002, 0x0001, /*1D8EF*/
  0x192F, 0x0020, 0x0002, 0x0001, /*1D8F0*/
  0x1930, 0x0020, 0x0002, 0x0001, /*1D8F1*/
  0x1931, 0x0020, 0x0002, 0x0001, /*1D8F2*/
  0x1932, 0x0020, 0x0002, 0x0001, /*1D8F3*/
  0x1933, 0x0020, 0x0002, 0x0001, /*1D8F4*/
  0x1934, 0x0020, 0x0002, 0x0001, /*1D8F5*/
  0x1935, 0x0020, 0x0002, 0x0001, /*1D8F6*/
  0x1936, 0x0020, 0x0002, 0x0001, /*1D8F7*/
  0x1937, 0x0020, 0x0002, 0x0001, /*1D8F8*/
  0x1938, 0x0020, 0x0002, 0x0001, /*1D8F9*/
  0x1939, 0x0020, 0x0002, 0x0001, /*1D8FA*/
  0x193A, 0x0020, 0x0002, 0x0001, /*1D8FB*/
  0x193B, 0x0020, 0x0002, 0x0001, /*1D8FC*/
  0x193C, 0x0020, 0x0002, 0x0001, /*1D8FD*/
  0x193D, 0x0020, 0x0002, 0x0001, /*1D8FE*/
  0x193E, 0x0020, 0x0002, 0x0001  /*1D8FF*/
};

uint16 uca900_p1D9[]= {
  0x193F, 0x0020, 0x0002, 0x0001, /*1D900*/
  0x1940, 0x0020, 0x0002, 0x0001, /*1D901*/
  0x1941, 0x0020, 0x0002, 0x0001, /*1D902*/
  0x1942, 0x0020, 0x0002, 0x0001, /*1D903*/
  0x1943, 0x0020, 0x0002, 0x0001, /*1D904*/
  0x1944, 0x0020, 0x0002, 0x0001, /*1D905*/
  0x1945, 0x0020, 0x0002, 0x0001, /*1D906*/
  0x1946, 0x0020, 0x0002, 0x0001, /*1D907*/
  0x1947, 0x0020, 0x0002, 0x0001, /*1D908*/
  0x1948, 0x0020, 0x0002, 0x0001, /*1D909*/
  0x1949, 0x0020, 0x0002, 0x0001, /*1D90A*/
  0x194A, 0x0020, 0x0002, 0x0001, /*1D90B*/
  0x194B, 0x0020, 0x0002, 0x0001, /*1D90C*/
  0x194C, 0x0020, 0x0002, 0x0001, /*1D90D*/
  0x194D, 0x0020, 0x0002, 0x0001, /*1D90E*/
  0x194E, 0x0020, 0x0002, 0x0001, /*1D90F*/
  0x194F, 0x0020, 0x0002, 0x0001, /*1D910*/
  0x1950, 0x0020, 0x0002, 0x0001, /*1D911*/
  0x1951, 0x0020, 0x0002, 0x0001, /*1D912*/
  0x1952, 0x0020, 0x0002, 0x0001, /*1D913*/
  0x1953, 0x0020, 0x0002, 0x0001, /*1D914*/
  0x1954, 0x0020, 0x0002, 0x0001, /*1D915*/
  0x1955, 0x0020, 0x0002, 0x0001, /*1D916*/
  0x1956, 0x0020, 0x0002, 0x0001, /*1D917*/
  0x1957, 0x0020, 0x0002, 0x0001, /*1D918*/
  0x1958, 0x0020, 0x0002, 0x0001, /*1D919*/
  0x1959, 0x0020, 0x0002, 0x0001, /*1D91A*/
  0x195A, 0x0020, 0x0002, 0x0001, /*1D91B*/
  0x195B, 0x0020, 0x0002, 0x0001, /*1D91C*/
  0x195C, 0x0020, 0x0002, 0x0001, /*1D91D*/
  0x195D, 0x0020, 0x0002, 0x0001, /*1D91E*/
  0x195E, 0x0020, 0x0002, 0x0001, /*1D91F*/
  0x195F, 0x0020, 0x0002, 0x0001, /*1D920*/
  0x1960, 0x0020, 0x0002, 0x0001, /*1D921*/
  0x1961, 0x0020, 0x0002, 0x0001, /*1D922*/
  0x1962, 0x0020, 0x0002, 0x0001, /*1D923*/
  0x1963, 0x0020, 0x0002, 0x0001, /*1D924*/
  0x1964, 0x0020, 0x0002, 0x0001, /*1D925*/
  0x1965, 0x0020, 0x0002, 0x0001, /*1D926*/
  0x1966, 0x0020, 0x0002, 0x0001, /*1D927*/
  0x1967, 0x0020, 0x0002, 0x0001, /*1D928*/
  0x1968, 0x0020, 0x0002, 0x0001, /*1D929*/
  0x1969, 0x0020, 0x0002, 0x0001, /*1D92A*/
  0x196A, 0x0020, 0x0002, 0x0001, /*1D92B*/
  0x196B, 0x0020, 0x0002, 0x0001, /*1D92C*/
  0x196C, 0x0020, 0x0002, 0x0001, /*1D92D*/
  0x196D, 0x0020, 0x0002, 0x0001, /*1D92E*/
  0x196E, 0x0020, 0x0002, 0x0001, /*1D92F*/
  0x196F, 0x0020, 0x0002, 0x0001, /*1D930*/
  0x1970, 0x0020, 0x0002, 0x0001, /*1D931*/
  0x1971, 0x0020, 0x0002, 0x0001, /*1D932*/
  0x1972, 0x0020, 0x0002, 0x0001, /*1D933*/
  0x1973, 0x0020, 0x0002, 0x0001, /*1D934*/
  0x1974, 0x0020, 0x0002, 0x0001, /*1D935*/
  0x1975, 0x0020, 0x0002, 0x0001, /*1D936*/
  0x1976, 0x0020, 0x0002, 0x0001, /*1D937*/
  0x1977, 0x0020, 0x0002, 0x0001, /*1D938*/
  0x1978, 0x0020, 0x0002, 0x0001, /*1D939*/
  0x1979, 0x0020, 0x0002, 0x0001, /*1D93A*/
  0x197A, 0x0020, 0x0002, 0x0001, /*1D93B*/
  0x197B, 0x0020, 0x0002, 0x0001, /*1D93C*/
  0x197C, 0x0020, 0x0002, 0x0001, /*1D93D*/
  0x197D, 0x0020, 0x0002, 0x0001, /*1D93E*/
  0x197E, 0x0020, 0x0002, 0x0001, /*1D93F*/
  0x197F, 0x0020, 0x0002, 0x0001, /*1D940*/
  0x1980, 0x0020, 0x0002, 0x0001, /*1D941*/
  0x1981, 0x0020, 0x0002, 0x0001, /*1D942*/
  0x1982, 0x0020, 0x0002, 0x0001, /*1D943*/
  0x1983, 0x0020, 0x0002, 0x0001, /*1D944*/
  0x1984, 0x0020, 0x0002, 0x0001, /*1D945*/
  0x1985, 0x0020, 0x0002, 0x0001, /*1D946*/
  0x1986, 0x0020, 0x0002, 0x0001, /*1D947*/
  0x1987, 0x0020, 0x0002, 0x0001, /*1D948*/
  0x1988, 0x0020, 0x0002, 0x0001, /*1D949*/
  0x1989, 0x0020, 0x0002, 0x0001, /*1D94A*/
  0x198A, 0x0020, 0x0002, 0x0001, /*1D94B*/
  0x198B, 0x0020, 0x0002, 0x0001, /*1D94C*/
  0x198C, 0x0020, 0x0002, 0x0001, /*1D94D*/
  0x198D, 0x0020, 0x0002, 0x0001, /*1D94E*/
  0x198E, 0x0020, 0x0002, 0x0001, /*1D94F*/
  0x198F, 0x0020, 0x0002, 0x0001, /*1D950*/
  0x1990, 0x0020, 0x0002, 0x0001, /*1D951*/
  0x1991, 0x0020, 0x0002, 0x0001, /*1D952*/
  0x1992, 0x0020, 0x0002, 0x0001, /*1D953*/
  0x1993, 0x0020, 0x0002, 0x0001, /*1D954*/
  0x1994, 0x0020, 0x0002, 0x0001, /*1D955*/
  0x1995, 0x0020, 0x0002, 0x0001, /*1D956*/
  0x1996, 0x0020, 0x0002, 0x0001, /*1D957*/
  0x1997, 0x0020, 0x0002, 0x0001, /*1D958*/
  0x1998, 0x0020, 0x0002, 0x0001, /*1D959*/
  0x1999, 0x0020, 0x0002, 0x0001, /*1D95A*/
  0x199A, 0x0020, 0x0002, 0x0001, /*1D95B*/
  0x199B, 0x0020, 0x0002, 0x0001, /*1D95C*/
  0x199C, 0x0020, 0x0002, 0x0001, /*1D95D*/
  0x199D, 0x0020, 0x0002, 0x0001, /*1D95E*/
  0x199E, 0x0020, 0x0002, 0x0001, /*1D95F*/
  0x199F, 0x0020, 0x0002, 0x0001, /*1D960*/
  0x19A0, 0x0020, 0x0002, 0x0001, /*1D961*/
  0x19A1, 0x0020, 0x0002, 0x0001, /*1D962*/
  0x19A2, 0x0020, 0x0002, 0x0001, /*1D963*/
  0x19A3, 0x0020, 0x0002, 0x0001, /*1D964*/
  0x19A4, 0x0020, 0x0002, 0x0001, /*1D965*/
  0x19A5, 0x0020, 0x0002, 0x0001, /*1D966*/
  0x19A6, 0x0020, 0x0002, 0x0001, /*1D967*/
  0x19A7, 0x0020, 0x0002, 0x0001, /*1D968*/
  0x19A8, 0x0020, 0x0002, 0x0001, /*1D969*/
  0x19A9, 0x0020, 0x0002, 0x0001, /*1D96A*/
  0x19AA, 0x0020, 0x0002, 0x0001, /*1D96B*/
  0x19AB, 0x0020, 0x0002, 0x0001, /*1D96C*/
  0x19AC, 0x0020, 0x0002, 0x0001, /*1D96D*/
  0x19AD, 0x0020, 0x0002, 0x0001, /*1D96E*/
  0x19AE, 0x0020, 0x0002, 0x0001, /*1D96F*/
  0x19AF, 0x0020, 0x0002, 0x0001, /*1D970*/
  0x19B0, 0x0020, 0x0002, 0x0001, /*1D971*/
  0x19B1, 0x0020, 0x0002, 0x0001, /*1D972*/
  0x19B2, 0x0020, 0x0002, 0x0001, /*1D973*/
  0x19B3, 0x0020, 0x0002, 0x0001, /*1D974*/
  0x19B4, 0x0020, 0x0002, 0x0001, /*1D975*/
  0x19B5, 0x0020, 0x0002, 0x0001, /*1D976*/
  0x19B6, 0x0020, 0x0002, 0x0001, /*1D977*/
  0x19B7, 0x0020, 0x0002, 0x0001, /*1D978*/
  0x19B8, 0x0020, 0x0002, 0x0001, /*1D979*/
  0x19B9, 0x0020, 0x0002, 0x0001, /*1D97A*/
  0x19BA, 0x0020, 0x0002, 0x0001, /*1D97B*/
  0x19BB, 0x0020, 0x0002, 0x0001, /*1D97C*/
  0x19BC, 0x0020, 0x0002, 0x0001, /*1D97D*/
  0x19BD, 0x0020, 0x0002, 0x0001, /*1D97E*/
  0x19BE, 0x0020, 0x0002, 0x0001, /*1D97F*/
  0x19BF, 0x0020, 0x0002, 0x0001, /*1D980*/
  0x19C0, 0x0020, 0x0002, 0x0001, /*1D981*/
  0x19C1, 0x0020, 0x0002, 0x0001, /*1D982*/
  0x19C2, 0x0020, 0x0002, 0x0001, /*1D983*/
  0x19C3, 0x0020, 0x0002, 0x0001, /*1D984*/
  0x19C4, 0x0020, 0x0002, 0x0001, /*1D985*/
  0x19C5, 0x0020, 0x0002, 0x0001, /*1D986*/
  0x19C6, 0x0020, 0x0002, 0x0001, /*1D987*/
  0x19C7, 0x0020, 0x0002, 0x0001, /*1D988*/
  0x19C8, 0x0020, 0x0002, 0x0001, /*1D989*/
  0x19C9, 0x0020, 0x0002, 0x0001, /*1D98A*/
  0x19CA, 0x0020, 0x0002, 0x0001, /*1D98B*/
  0x19CB, 0x0020, 0x0002, 0x0001, /*1D98C*/
  0x19CC, 0x0020, 0x0002, 0x0001, /*1D98D*/
  0x19CD, 0x0020, 0x0002, 0x0001, /*1D98E*/
  0x19CE, 0x0020, 0x0002, 0x0001, /*1D98F*/
  0x19CF, 0x0020, 0x0002, 0x0001, /*1D990*/
  0x19D0, 0x0020, 0x0002, 0x0001, /*1D991*/
  0x19D1, 0x0020, 0x0002, 0x0001, /*1D992*/
  0x19D2, 0x0020, 0x0002, 0x0001, /*1D993*/
  0x19D3, 0x0020, 0x0002, 0x0001, /*1D994*/
  0x19D4, 0x0020, 0x0002, 0x0001, /*1D995*/
  0x19D5, 0x0020, 0x0002, 0x0001, /*1D996*/
  0x19D6, 0x0020, 0x0002, 0x0001, /*1D997*/
  0x19D7, 0x0020, 0x0002, 0x0001, /*1D998*/
  0x19D8, 0x0020, 0x0002, 0x0001, /*1D999*/
  0x19D9, 0x0020, 0x0002, 0x0001, /*1D99A*/
  0x19DA, 0x0020, 0x0002, 0x0001, /*1D99B*/
  0x19DB, 0x0020, 0x0002, 0x0001, /*1D99C*/
  0x19DC, 0x0020, 0x0002, 0x0001, /*1D99D*/
  0x19DD, 0x0020, 0x0002, 0x0001, /*1D99E*/
  0x19DE, 0x0020, 0x0002, 0x0001, /*1D99F*/
  0x19DF, 0x0020, 0x0002, 0x0001, /*1D9A0*/
  0x19E0, 0x0020, 0x0002, 0x0001, /*1D9A1*/
  0x19E1, 0x0020, 0x0002, 0x0001, /*1D9A2*/
  0x19E2, 0x0020, 0x0002, 0x0001, /*1D9A3*/
  0x19E3, 0x0020, 0x0002, 0x0001, /*1D9A4*/
  0x19E4, 0x0020, 0x0002, 0x0001, /*1D9A5*/
  0x19E5, 0x0020, 0x0002, 0x0001, /*1D9A6*/
  0x19E6, 0x0020, 0x0002, 0x0001, /*1D9A7*/
  0x19E7, 0x0020, 0x0002, 0x0001, /*1D9A8*/
  0x19E8, 0x0020, 0x0002, 0x0001, /*1D9A9*/
  0x19E9, 0x0020, 0x0002, 0x0001, /*1D9AA*/
  0x19EA, 0x0020, 0x0002, 0x0001, /*1D9AB*/
  0x19EB, 0x0020, 0x0002, 0x0001, /*1D9AC*/
  0x19EC, 0x0020, 0x0002, 0x0001, /*1D9AD*/
  0x19ED, 0x0020, 0x0002, 0x0001, /*1D9AE*/
  0x19EE, 0x0020, 0x0002, 0x0001, /*1D9AF*/
  0x19EF, 0x0020, 0x0002, 0x0001, /*1D9B0*/
  0x19F0, 0x0020, 0x0002, 0x0001, /*1D9B1*/
  0x19F1, 0x0020, 0x0002, 0x0001, /*1D9B2*/
  0x19F2, 0x0020, 0x0002, 0x0001, /*1D9B3*/
  0x19F3, 0x0020, 0x0002, 0x0001, /*1D9B4*/
  0x19F4, 0x0020, 0x0002, 0x0001, /*1D9B5*/
  0x19F5, 0x0020, 0x0002, 0x0001, /*1D9B6*/
  0x19F6, 0x0020, 0x0002, 0x0001, /*1D9B7*/
  0x19F7, 0x0020, 0x0002, 0x0001, /*1D9B8*/
  0x19F8, 0x0020, 0x0002, 0x0001, /*1D9B9*/
  0x19F9, 0x0020, 0x0002, 0x0001, /*1D9BA*/
  0x19FA, 0x0020, 0x0002, 0x0001, /*1D9BB*/
  0x19FB, 0x0020, 0x0002, 0x0001, /*1D9BC*/
  0x19FC, 0x0020, 0x0002, 0x0001, /*1D9BD*/
  0x19FD, 0x0020, 0x0002, 0x0001, /*1D9BE*/
  0x19FE, 0x0020, 0x0002, 0x0001, /*1D9BF*/
  0x19FF, 0x0020, 0x0002, 0x0001, /*1D9C0*/
  0x1A00, 0x0020, 0x0002, 0x0001, /*1D9C1*/
  0x1A01, 0x0020, 0x0002, 0x0001, /*1D9C2*/
  0x1A02, 0x0020, 0x0002, 0x0001, /*1D9C3*/
  0x1A03, 0x0020, 0x0002, 0x0001, /*1D9C4*/
  0x1A04, 0x0020, 0x0002, 0x0001, /*1D9C5*/
  0x1A05, 0x0020, 0x0002, 0x0001, /*1D9C6*/
  0x1A06, 0x0020, 0x0002, 0x0001, /*1D9C7*/
  0x1A07, 0x0020, 0x0002, 0x0001, /*1D9C8*/
  0x1A08, 0x0020, 0x0002, 0x0001, /*1D9C9*/
  0x1A09, 0x0020, 0x0002, 0x0001, /*1D9CA*/
  0x1A0A, 0x0020, 0x0002, 0x0001, /*1D9CB*/
  0x1A0B, 0x0020, 0x0002, 0x0001, /*1D9CC*/
  0x1A0C, 0x0020, 0x0002, 0x0001, /*1D9CD*/
  0x1A0D, 0x0020, 0x0002, 0x0001, /*1D9CE*/
  0x1A0E, 0x0020, 0x0002, 0x0001, /*1D9CF*/
  0x1A0F, 0x0020, 0x0002, 0x0001, /*1D9D0*/
  0x1A10, 0x0020, 0x0002, 0x0001, /*1D9D1*/
  0x1A11, 0x0020, 0x0002, 0x0001, /*1D9D2*/
  0x1A12, 0x0020, 0x0002, 0x0001, /*1D9D3*/
  0x1A13, 0x0020, 0x0002, 0x0001, /*1D9D4*/
  0x1A14, 0x0020, 0x0002, 0x0001, /*1D9D5*/
  0x1A15, 0x0020, 0x0002, 0x0001, /*1D9D6*/
  0x1A16, 0x0020, 0x0002, 0x0001, /*1D9D7*/
  0x1A17, 0x0020, 0x0002, 0x0001, /*1D9D8*/
  0x1A18, 0x0020, 0x0002, 0x0001, /*1D9D9*/
  0x1A19, 0x0020, 0x0002, 0x0001, /*1D9DA*/
  0x1A1A, 0x0020, 0x0002, 0x0001, /*1D9DB*/
  0x1A1B, 0x0020, 0x0002, 0x0001, /*1D9DC*/
  0x1A1C, 0x0020, 0x0002, 0x0001, /*1D9DD*/
  0x1A1D, 0x0020, 0x0002, 0x0001, /*1D9DE*/
  0x1A1E, 0x0020, 0x0002, 0x0001, /*1D9DF*/
  0x1A1F, 0x0020, 0x0002, 0x0001, /*1D9E0*/
  0x1A20, 0x0020, 0x0002, 0x0001, /*1D9E1*/
  0x1A21, 0x0020, 0x0002, 0x0001, /*1D9E2*/
  0x1A22, 0x0020, 0x0002, 0x0001, /*1D9E3*/
  0x1A23, 0x0020, 0x0002, 0x0001, /*1D9E4*/
  0x1A24, 0x0020, 0x0002, 0x0001, /*1D9E5*/
  0x1A25, 0x0020, 0x0002, 0x0001, /*1D9E6*/
  0x1A26, 0x0020, 0x0002, 0x0001, /*1D9E7*/
  0x1A27, 0x0020, 0x0002, 0x0001, /*1D9E8*/
  0x1A28, 0x0020, 0x0002, 0x0001, /*1D9E9*/
  0x1A29, 0x0020, 0x0002, 0x0001, /*1D9EA*/
  0x1A2A, 0x0020, 0x0002, 0x0001, /*1D9EB*/
  0x1A2B, 0x0020, 0x0002, 0x0001, /*1D9EC*/
  0x1A2C, 0x0020, 0x0002, 0x0001, /*1D9ED*/
  0x1A2D, 0x0020, 0x0002, 0x0001, /*1D9EE*/
  0x1A2E, 0x0020, 0x0002, 0x0001, /*1D9EF*/
  0x1A2F, 0x0020, 0x0002, 0x0001, /*1D9F0*/
  0x1A30, 0x0020, 0x0002, 0x0001, /*1D9F1*/
  0x1A31, 0x0020, 0x0002, 0x0001, /*1D9F2*/
  0x1A32, 0x0020, 0x0002, 0x0001, /*1D9F3*/
  0x1A33, 0x0020, 0x0002, 0x0001, /*1D9F4*/
  0x1A34, 0x0020, 0x0002, 0x0001, /*1D9F5*/
  0x1A35, 0x0020, 0x0002, 0x0001, /*1D9F6*/
  0x1A36, 0x0020, 0x0002, 0x0001, /*1D9F7*/
  0x1A37, 0x0020, 0x0002, 0x0001, /*1D9F8*/
  0x1A38, 0x0020, 0x0002, 0x0001, /*1D9F9*/
  0x1A39, 0x0020, 0x0002, 0x0001, /*1D9FA*/
  0x1A3A, 0x0020, 0x0002, 0x0001, /*1D9FB*/
  0x1A3B, 0x0020, 0x0002, 0x0001, /*1D9FC*/
  0x1A3C, 0x0020, 0x0002, 0x0001, /*1D9FD*/
  0x1A3D, 0x0020, 0x0002, 0x0001, /*1D9FE*/
  0x1A3E, 0x0020, 0x0002, 0x0001  /*1D9FF*/
};

uint16 uca900_p1DA[]= {
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA00*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA01*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA02*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA03*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA04*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA05*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA06*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA07*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA08*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA09*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA0A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA0B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA0C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA0D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA0E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA0F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA10*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA11*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA12*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA13*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA14*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA15*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA16*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA17*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA18*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA19*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA1A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA1B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA1C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA1D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA1E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA1F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA20*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA21*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA22*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA23*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA24*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA25*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA26*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA27*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA28*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA29*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA2A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA2B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA2C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA2D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA2E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA2F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA30*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA31*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA32*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA33*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA34*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA35*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA36*/
  0x1A3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA37*/
  0x1A40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA38*/
  0x1A41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA39*/
  0x1A42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA3A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA3B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA3C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA3D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA3E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA3F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA40*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA41*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA42*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA43*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA44*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA45*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA46*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA47*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA48*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA49*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA4A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA4B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA4C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA4D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA4E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA4F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA50*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA51*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA52*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA53*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA54*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA55*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA56*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA57*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA58*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA59*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA5A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA5B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA5C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA5D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA5E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA5F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA60*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA61*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA62*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA63*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA64*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA65*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA66*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA67*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA68*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA69*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA6A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA6B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA6C*/
  0x1A43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA6D*/
  0x1A44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA6E*/
  0x1A45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA6F*/
  0x1A46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA70*/
  0x1A47, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA71*/
  0x1A48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA72*/
  0x1A49, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA73*/
  0x1A4A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA74*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA75*/
  0x1A4B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA76*/
  0x1A4C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA77*/
  0x1A4D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA78*/
  0x1A4E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA79*/
  0x1A4F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA7A*/
  0x1A50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA7B*/
  0x1A51, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA7C*/
  0x1A52, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA7D*/
  0x1A53, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA7E*/
  0x1A54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA7F*/
  0x1A55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA80*/
  0x1A56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA81*/
  0x1A57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA82*/
  0x1A58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA83*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA84*/
  0x1A59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA85*/
  0x1A5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA86*/
  0x047D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA87*/
  0x047E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA88*/
  0x047F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA89*/
  0x0480, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA8A*/
  0x0481, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA8B*/
  0xFBC3, 0x0020, 0x0002, 0xDA8C, 0x0000, 0x0000, 0x0002, /*1DA8C*/
  0xFBC3, 0x0020, 0x0002, 0xDA8D, 0x0000, 0x0000, 0x0002, /*1DA8D*/
  0xFBC3, 0x0020, 0x0002, 0xDA8E, 0x0000, 0x0000, 0x0002, /*1DA8E*/
  0xFBC3, 0x0020, 0x0002, 0xDA8F, 0x0000, 0x0000, 0x0002, /*1DA8F*/
  0xFBC3, 0x0020, 0x0002, 0xDA90, 0x0000, 0x0000, 0x0002, /*1DA90*/
  0xFBC3, 0x0020, 0x0002, 0xDA91, 0x0000, 0x0000, 0x0002, /*1DA91*/
  0xFBC3, 0x0020, 0x0002, 0xDA92, 0x0000, 0x0000, 0x0002, /*1DA92*/
  0xFBC3, 0x0020, 0x0002, 0xDA93, 0x0000, 0x0000, 0x0002, /*1DA93*/
  0xFBC3, 0x0020, 0x0002, 0xDA94, 0x0000, 0x0000, 0x0002, /*1DA94*/
  0xFBC3, 0x0020, 0x0002, 0xDA95, 0x0000, 0x0000, 0x0002, /*1DA95*/
  0xFBC3, 0x0020, 0x0002, 0xDA96, 0x0000, 0x0000, 0x0002, /*1DA96*/
  0xFBC3, 0x0020, 0x0002, 0xDA97, 0x0000, 0x0000, 0x0002, /*1DA97*/
  0xFBC3, 0x0020, 0x0002, 0xDA98, 0x0000, 0x0000, 0x0002, /*1DA98*/
  0xFBC3, 0x0020, 0x0002, 0xDA99, 0x0000, 0x0000, 0x0002, /*1DA99*/
  0xFBC3, 0x0020, 0x0002, 0xDA9A, 0x0000, 0x0000, 0x0002, /*1DA9A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA9B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA9C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA9D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA9E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DA9F*/
  0xFBC3, 0x0020, 0x0002, 0xDAA0, 0x0000, 0x0000, 0x0002, /*1DAA0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAA1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAA2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAA3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAA4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAA5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAA6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAA7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAA8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAA9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAAA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAAB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAAC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAAD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAAE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1DAAF*/
  0xFBC3, 0x0020, 0x0002, 0xDAB0, 0x0000, 0x0000, 0x0002, /*1DAB0*/
  0xFBC3, 0x0020, 0x0002, 0xDAB1, 0x0000, 0x0000, 0x0002, /*1DAB1*/
  0xFBC3, 0x0020, 0x0002, 0xDAB2, 0x0000, 0x0000, 0x0002, /*1DAB2*/
  0xFBC3, 0x0020, 0x0002, 0xDAB3, 0x0000, 0x0000, 0x0002, /*1DAB3*/
  0xFBC3, 0x0020, 0x0002, 0xDAB4, 0x0000, 0x0000, 0x0002, /*1DAB4*/
  0xFBC3, 0x0020, 0x0002, 0xDAB5, 0x0000, 0x0000, 0x0002, /*1DAB5*/
  0xFBC3, 0x0020, 0x0002, 0xDAB6, 0x0000, 0x0000, 0x0002, /*1DAB6*/
  0xFBC3, 0x0020, 0x0002, 0xDAB7, 0x0000, 0x0000, 0x0002, /*1DAB7*/
  0xFBC3, 0x0020, 0x0002, 0xDAB8, 0x0000, 0x0000, 0x0002, /*1DAB8*/
  0xFBC3, 0x0020, 0x0002, 0xDAB9, 0x0000, 0x0000, 0x0002, /*1DAB9*/
  0xFBC3, 0x0020, 0x0002, 0xDABA, 0x0000, 0x0000, 0x0002, /*1DABA*/
  0xFBC3, 0x0020, 0x0002, 0xDABB, 0x0000, 0x0000, 0x0002, /*1DABB*/
  0xFBC3, 0x0020, 0x0002, 0xDABC, 0x0000, 0x0000, 0x0002, /*1DABC*/
  0xFBC3, 0x0020, 0x0002, 0xDABD, 0x0000, 0x0000, 0x0002, /*1DABD*/
  0xFBC3, 0x0020, 0x0002, 0xDABE, 0x0000, 0x0000, 0x0002, /*1DABE*/
  0xFBC3, 0x0020, 0x0002, 0xDABF, 0x0000, 0x0000, 0x0002, /*1DABF*/
  0xFBC3, 0x0020, 0x0002, 0xDAC0, 0x0000, 0x0000, 0x0002, /*1DAC0*/
  0xFBC3, 0x0020, 0x0002, 0xDAC1, 0x0000, 0x0000, 0x0002, /*1DAC1*/
  0xFBC3, 0x0020, 0x0002, 0xDAC2, 0x0000, 0x0000, 0x0002, /*1DAC2*/
  0xFBC3, 0x0020, 0x0002, 0xDAC3, 0x0000, 0x0000, 0x0002, /*1DAC3*/
  0xFBC3, 0x0020, 0x0002, 0xDAC4, 0x0000, 0x0000, 0x0002, /*1DAC4*/
  0xFBC3, 0x0020, 0x0002, 0xDAC5, 0x0000, 0x0000, 0x0002, /*1DAC5*/
  0xFBC3, 0x0020, 0x0002, 0xDAC6, 0x0000, 0x0000, 0x0002, /*1DAC6*/
  0xFBC3, 0x0020, 0x0002, 0xDAC7, 0x0000, 0x0000, 0x0002, /*1DAC7*/
  0xFBC3, 0x0020, 0x0002, 0xDAC8, 0x0000, 0x0000, 0x0002, /*1DAC8*/
  0xFBC3, 0x0020, 0x0002, 0xDAC9, 0x0000, 0x0000, 0x0002, /*1DAC9*/
  0xFBC3, 0x0020, 0x0002, 0xDACA, 0x0000, 0x0000, 0x0002, /*1DACA*/
  0xFBC3, 0x0020, 0x0002, 0xDACB, 0x0000, 0x0000, 0x0002, /*1DACB*/
  0xFBC3, 0x0020, 0x0002, 0xDACC, 0x0000, 0x0000, 0x0002, /*1DACC*/
  0xFBC3, 0x0020, 0x0002, 0xDACD, 0x0000, 0x0000, 0x0002, /*1DACD*/
  0xFBC3, 0x0020, 0x0002, 0xDACE, 0x0000, 0x0000, 0x0002, /*1DACE*/
  0xFBC3, 0x0020, 0x0002, 0xDACF, 0x0000, 0x0000, 0x0002, /*1DACF*/
  0xFBC3, 0x0020, 0x0002, 0xDAD0, 0x0000, 0x0000, 0x0002, /*1DAD0*/
  0xFBC3, 0x0020, 0x0002, 0xDAD1, 0x0000, 0x0000, 0x0002, /*1DAD1*/
  0xFBC3, 0x0020, 0x0002, 0xDAD2, 0x0000, 0x0000, 0x0002, /*1DAD2*/
  0xFBC3, 0x0020, 0x0002, 0xDAD3, 0x0000, 0x0000, 0x0002, /*1DAD3*/
  0xFBC3, 0x0020, 0x0002, 0xDAD4, 0x0000, 0x0000, 0x0002, /*1DAD4*/
  0xFBC3, 0x0020, 0x0002, 0xDAD5, 0x0000, 0x0000, 0x0002, /*1DAD5*/
  0xFBC3, 0x0020, 0x0002, 0xDAD6, 0x0000, 0x0000, 0x0002, /*1DAD6*/
  0xFBC3, 0x0020, 0x0002, 0xDAD7, 0x0000, 0x0000, 0x0002, /*1DAD7*/
  0xFBC3, 0x0020, 0x0002, 0xDAD8, 0x0000, 0x0000, 0x0002, /*1DAD8*/
  0xFBC3, 0x0020, 0x0002, 0xDAD9, 0x0000, 0x0000, 0x0002, /*1DAD9*/
  0xFBC3, 0x0020, 0x0002, 0xDADA, 0x0000, 0x0000, 0x0002, /*1DADA*/
  0xFBC3, 0x0020, 0x0002, 0xDADB, 0x0000, 0x0000, 0x0002, /*1DADB*/
  0xFBC3, 0x0020, 0x0002, 0xDADC, 0x0000, 0x0000, 0x0002, /*1DADC*/
  0xFBC3, 0x0020, 0x0002, 0xDADD, 0x0000, 0x0000, 0x0002, /*1DADD*/
  0xFBC3, 0x0020, 0x0002, 0xDADE, 0x0000, 0x0000, 0x0002, /*1DADE*/
  0xFBC3, 0x0020, 0x0002, 0xDADF, 0x0000, 0x0000, 0x0002, /*1DADF*/
  0xFBC3, 0x0020, 0x0002, 0xDAE0, 0x0000, 0x0000, 0x0002, /*1DAE0*/
  0xFBC3, 0x0020, 0x0002, 0xDAE1, 0x0000, 0x0000, 0x0002, /*1DAE1*/
  0xFBC3, 0x0020, 0x0002, 0xDAE2, 0x0000, 0x0000, 0x0002, /*1DAE2*/
  0xFBC3, 0x0020, 0x0002, 0xDAE3, 0x0000, 0x0000, 0x0002, /*1DAE3*/
  0xFBC3, 0x0020, 0x0002, 0xDAE4, 0x0000, 0x0000, 0x0002, /*1DAE4*/
  0xFBC3, 0x0020, 0x0002, 0xDAE5, 0x0000, 0x0000, 0x0002, /*1DAE5*/
  0xFBC3, 0x0020, 0x0002, 0xDAE6, 0x0000, 0x0000, 0x0002, /*1DAE6*/
  0xFBC3, 0x0020, 0x0002, 0xDAE7, 0x0000, 0x0000, 0x0002, /*1DAE7*/
  0xFBC3, 0x0020, 0x0002, 0xDAE8, 0x0000, 0x0000, 0x0002, /*1DAE8*/
  0xFBC3, 0x0020, 0x0002, 0xDAE9, 0x0000, 0x0000, 0x0002, /*1DAE9*/
  0xFBC3, 0x0020, 0x0002, 0xDAEA, 0x0000, 0x0000, 0x0002, /*1DAEA*/
  0xFBC3, 0x0020, 0x0002, 0xDAEB, 0x0000, 0x0000, 0x0002, /*1DAEB*/
  0xFBC3, 0x0020, 0x0002, 0xDAEC, 0x0000, 0x0000, 0x0002, /*1DAEC*/
  0xFBC3, 0x0020, 0x0002, 0xDAED, 0x0000, 0x0000, 0x0002, /*1DAED*/
  0xFBC3, 0x0020, 0x0002, 0xDAEE, 0x0000, 0x0000, 0x0002, /*1DAEE*/
  0xFBC3, 0x0020, 0x0002, 0xDAEF, 0x0000, 0x0000, 0x0002, /*1DAEF*/
  0xFBC3, 0x0020, 0x0002, 0xDAF0, 0x0000, 0x0000, 0x0002, /*1DAF0*/
  0xFBC3, 0x0020, 0x0002, 0xDAF1, 0x0000, 0x0000, 0x0002, /*1DAF1*/
  0xFBC3, 0x0020, 0x0002, 0xDAF2, 0x0000, 0x0000, 0x0002, /*1DAF2*/
  0xFBC3, 0x0020, 0x0002, 0xDAF3, 0x0000, 0x0000, 0x0002, /*1DAF3*/
  0xFBC3, 0x0020, 0x0002, 0xDAF4, 0x0000, 0x0000, 0x0002, /*1DAF4*/
  0xFBC3, 0x0020, 0x0002, 0xDAF5, 0x0000, 0x0000, 0x0002, /*1DAF5*/
  0xFBC3, 0x0020, 0x0002, 0xDAF6, 0x0000, 0x0000, 0x0002, /*1DAF6*/
  0xFBC3, 0x0020, 0x0002, 0xDAF7, 0x0000, 0x0000, 0x0002, /*1DAF7*/
  0xFBC3, 0x0020, 0x0002, 0xDAF8, 0x0000, 0x0000, 0x0002, /*1DAF8*/
  0xFBC3, 0x0020, 0x0002, 0xDAF9, 0x0000, 0x0000, 0x0002, /*1DAF9*/
  0xFBC3, 0x0020, 0x0002, 0xDAFA, 0x0000, 0x0000, 0x0002, /*1DAFA*/
  0xFBC3, 0x0020, 0x0002, 0xDAFB, 0x0000, 0x0000, 0x0002, /*1DAFB*/
  0xFBC3, 0x0020, 0x0002, 0xDAFC, 0x0000, 0x0000, 0x0002, /*1DAFC*/
  0xFBC3, 0x0020, 0x0002, 0xDAFD, 0x0000, 0x0000, 0x0002, /*1DAFD*/
  0xFBC3, 0x0020, 0x0002, 0xDAFE, 0x0000, 0x0000, 0x0002, /*1DAFE*/
  0xFBC3, 0x0020, 0x0002, 0xDAFF, 0x0000, 0x0000, 0x0002  /*1DAFF*/
};

uint16 uca900_p1E0[]= {
  0x21E5, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E000*/
  0x21E6, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E001*/
  0x21E7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E002*/
  0x21E8, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E003*/
  0x21E9, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E004*/
  0x21EA, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E005*/
  0x21EB, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E006*/
  0xFBC3, 0x0020, 0x0002, 0xE007, 0x0000, 0x0000, 0x0002, /*1E007*/
  0x21ED, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E008*/
  0x21EE, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E009*/
  0x21EF, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E00A*/
  0x21F0, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E00B*/
  0x21F1, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E00C*/
  0x21F2, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E00D*/
  0x21F3, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E00E*/
  0x21F4, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E00F*/
  0x21F5, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E010*/
  0x21F6, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E011*/
  0x21F7, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E012*/
  0x21F8, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E013*/
  0x21F9, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E014*/
  0x21FA, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E015*/
  0x21FB, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E016*/
  0x21FC, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E017*/
  0x21FD, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E018*/
  0xFBC3, 0x0020, 0x0002, 0xE019, 0x0000, 0x0000, 0x0002, /*1E019*/
  0xFBC3, 0x0020, 0x0002, 0xE01A, 0x0000, 0x0000, 0x0002, /*1E01A*/
  0x2200, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E01B*/
  0x2201, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E01C*/
  0x2202, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E01D*/
  0x2203, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E01E*/
  0x2204, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E01F*/
  0x2205, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E020*/
  0x2206, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E021*/
  0xFBC3, 0x0020, 0x0002, 0xE022, 0x0000, 0x0000, 0x0002, /*1E022*/
  0x2208, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E023*/
  0x2209, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E024*/
  0xFBC3, 0x0020, 0x0002, 0xE025, 0x0000, 0x0000, 0x0002, /*1E025*/
  0x220B, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E026*/
  0x220C, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E027*/
  0x220D, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E028*/
  0x220E, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E029*/
  0x220F, 0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0001, /*1E02A*/
  0xFBC3, 0x0020, 0x0002, 0xE02B, 0x0000, 0x0000, 0x0002, /*1E02B*/
  0xFBC3, 0x0020, 0x0002, 0xE02C, 0x0000, 0x0000, 0x0002, /*1E02C*/
  0xFBC3, 0x0020, 0x0002, 0xE02D, 0x0000, 0x0000, 0x0002, /*1E02D*/
  0xFBC3, 0x0020, 0x0002, 0xE02E, 0x0000, 0x0000, 0x0002, /*1E02E*/
  0xFBC3, 0x0020, 0x0002, 0xE02F, 0x0000, 0x0000, 0x0002, /*1E02F*/
  0xFBC3, 0x0020, 0x0002, 0xE030, 0x0000, 0x0000, 0x0002, /*1E030*/
  0xFBC3, 0x0020, 0x0002, 0xE031, 0x0000, 0x0000, 0x0002, /*1E031*/
  0xFBC3, 0x0020, 0x0002, 0xE032, 0x0000, 0x0000, 0x0002, /*1E032*/
  0xFBC3, 0x0020, 0x0002, 0xE033, 0x0000, 0x0000, 0x0002, /*1E033*/
  0xFBC3, 0x0020, 0x0002, 0xE034, 0x0000, 0x0000, 0x0002, /*1E034*/
  0xFBC3, 0x0020, 0x0002, 0xE035, 0x0000, 0x0000, 0x0002, /*1E035*/
  0xFBC3, 0x0020, 0x0002, 0xE036, 0x0000, 0x0000, 0x0002, /*1E036*/
  0xFBC3, 0x0020, 0x0002, 0xE037, 0x0000, 0x0000, 0x0002, /*1E037*/
  0xFBC3, 0x0020, 0x0002, 0xE038, 0x0000, 0x0000, 0x0002, /*1E038*/
  0xFBC3, 0x0020, 0x0002, 0xE039, 0x0000, 0x0000, 0x0002, /*1E039*/
  0xFBC3, 0x0020, 0x0002, 0xE03A, 0x0000, 0x0000, 0x0002, /*1E03A*/
  0xFBC3, 0x0020, 0x0002, 0xE03B, 0x0000, 0x0000, 0x0002, /*1E03B*/
  0xFBC3, 0x0020, 0x0002, 0xE03C, 0x0000, 0x0000, 0x0002, /*1E03C*/
  0xFBC3, 0x0020, 0x0002, 0xE03D, 0x0000, 0x0000, 0x0002, /*1E03D*/
  0xFBC3, 0x0020, 0x0002, 0xE03E, 0x0000, 0x0000, 0x0002, /*1E03E*/
  0xFBC3, 0x0020, 0x0002, 0xE03F, 0x0000, 0x0000, 0x0002, /*1E03F*/
  0xFBC3, 0x0020, 0x0002, 0xE040, 0x0000, 0x0000, 0x0002, /*1E040*/
  0xFBC3, 0x0020, 0x0002, 0xE041, 0x0000, 0x0000, 0x0002, /*1E041*/
  0xFBC3, 0x0020, 0x0002, 0xE042, 0x0000, 0x0000, 0x0002, /*1E042*/
  0xFBC3, 0x0020, 0x0002, 0xE043, 0x0000, 0x0000, 0x0002, /*1E043*/
  0xFBC3, 0x0020, 0x0002, 0xE044, 0x0000, 0x0000, 0x0002, /*1E044*/
  0xFBC3, 0x0020, 0x0002, 0xE045, 0x0000, 0x0000, 0x0002, /*1E045*/
  0xFBC3, 0x0020, 0x0002, 0xE046, 0x0000, 0x0000, 0x0002, /*1E046*/
  0xFBC3, 0x0020, 0x0002, 0xE047, 0x0000, 0x0000, 0x0002, /*1E047*/
  0xFBC3, 0x0020, 0x0002, 0xE048, 0x0000, 0x0000, 0x0002, /*1E048*/
  0xFBC3, 0x0020, 0x0002, 0xE049, 0x0000, 0x0000, 0x0002, /*1E049*/
  0xFBC3, 0x0020, 0x0002, 0xE04A, 0x0000, 0x0000, 0x0002, /*1E04A*/
  0xFBC3, 0x0020, 0x0002, 0xE04B, 0x0000, 0x0000, 0x0002, /*1E04B*/
  0xFBC3, 0x0020, 0x0002, 0xE04C, 0x0000, 0x0000, 0x0002, /*1E04C*/
  0xFBC3, 0x0020, 0x0002, 0xE04D, 0x0000, 0x0000, 0x0002, /*1E04D*/
  0xFBC3, 0x0020, 0x0002, 0xE04E, 0x0000, 0x0000, 0x0002, /*1E04E*/
  0xFBC3, 0x0020, 0x0002, 0xE04F, 0x0000, 0x0000, 0x0002, /*1E04F*/
  0xFBC3, 0x0020, 0x0002, 0xE050, 0x0000, 0x0000, 0x0002, /*1E050*/
  0xFBC3, 0x0020, 0x0002, 0xE051, 0x0000, 0x0000, 0x0002, /*1E051*/
  0xFBC3, 0x0020, 0x0002, 0xE052, 0x0000, 0x0000, 0x0002, /*1E052*/
  0xFBC3, 0x0020, 0x0002, 0xE053, 0x0000, 0x0000, 0x0002, /*1E053*/
  0xFBC3, 0x0020, 0x0002, 0xE054, 0x0000, 0x0000, 0x0002, /*1E054*/
  0xFBC3, 0x0020, 0x0002, 0xE055, 0x0000, 0x0000, 0x0002, /*1E055*/
  0xFBC3, 0x0020, 0x0002, 0xE056, 0x0000, 0x0000, 0x0002, /*1E056*/
  0xFBC3, 0x0020, 0x0002, 0xE057, 0x0000, 0x0000, 0x0002, /*1E057*/
  0xFBC3, 0x0020, 0x0002, 0xE058, 0x0000, 0x0000, 0x0002, /*1E058*/
  0xFBC3, 0x0020, 0x0002, 0xE059, 0x0000, 0x0000, 0x0002, /*1E059*/
  0xFBC3, 0x0020, 0x0002, 0xE05A, 0x0000, 0x0000, 0x0002, /*1E05A*/
  0xFBC3, 0x0020, 0x0002, 0xE05B, 0x0000, 0x0000, 0x0002, /*1E05B*/
  0xFBC3, 0x0020, 0x0002, 0xE05C, 0x0000, 0x0000, 0x0002, /*1E05C*/
  0xFBC3, 0x0020, 0x0002, 0xE05D, 0x0000, 0x0000, 0x0002, /*1E05D*/
  0xFBC3, 0x0020, 0x0002, 0xE05E, 0x0000, 0x0000, 0x0002, /*1E05E*/
  0xFBC3, 0x0020, 0x0002, 0xE05F, 0x0000, 0x0000, 0x0002, /*1E05F*/
  0xFBC3, 0x0020, 0x0002, 0xE060, 0x0000, 0x0000, 0x0002, /*1E060*/
  0xFBC3, 0x0020, 0x0002, 0xE061, 0x0000, 0x0000, 0x0002, /*1E061*/
  0xFBC3, 0x0020, 0x0002, 0xE062, 0x0000, 0x0000, 0x0002, /*1E062*/
  0xFBC3, 0x0020, 0x0002, 0xE063, 0x0000, 0x0000, 0x0002, /*1E063*/
  0xFBC3, 0x0020, 0x0002, 0xE064, 0x0000, 0x0000, 0x0002, /*1E064*/
  0xFBC3, 0x0020, 0x0002, 0xE065, 0x0000, 0x0000, 0x0002, /*1E065*/
  0xFBC3, 0x0020, 0x0002, 0xE066, 0x0000, 0x0000, 0x0002, /*1E066*/
  0xFBC3, 0x0020, 0x0002, 0xE067, 0x0000, 0x0000, 0x0002, /*1E067*/
  0xFBC3, 0x0020, 0x0002, 0xE068, 0x0000, 0x0000, 0x0002, /*1E068*/
  0xFBC3, 0x0020, 0x0002, 0xE069, 0x0000, 0x0000, 0x0002, /*1E069*/
  0xFBC3, 0x0020, 0x0002, 0xE06A, 0x0000, 0x0000, 0x0002, /*1E06A*/
  0xFBC3, 0x0020, 0x0002, 0xE06B, 0x0000, 0x0000, 0x0002, /*1E06B*/
  0xFBC3, 0x0020, 0x0002, 0xE06C, 0x0000, 0x0000, 0x0002, /*1E06C*/
  0xFBC3, 0x0020, 0x0002, 0xE06D, 0x0000, 0x0000, 0x0002, /*1E06D*/
  0xFBC3, 0x0020, 0x0002, 0xE06E, 0x0000, 0x0000, 0x0002, /*1E06E*/
  0xFBC3, 0x0020, 0x0002, 0xE06F, 0x0000, 0x0000, 0x0002, /*1E06F*/
  0xFBC3, 0x0020, 0x0002, 0xE070, 0x0000, 0x0000, 0x0002, /*1E070*/
  0xFBC3, 0x0020, 0x0002, 0xE071, 0x0000, 0x0000, 0x0002, /*1E071*/
  0xFBC3, 0x0020, 0x0002, 0xE072, 0x0000, 0x0000, 0x0002, /*1E072*/
  0xFBC3, 0x0020, 0x0002, 0xE073, 0x0000, 0x0000, 0x0002, /*1E073*/
  0xFBC3, 0x0020, 0x0002, 0xE074, 0x0000, 0x0000, 0x0002, /*1E074*/
  0xFBC3, 0x0020, 0x0002, 0xE075, 0x0000, 0x0000, 0x0002, /*1E075*/
  0xFBC3, 0x0020, 0x0002, 0xE076, 0x0000, 0x0000, 0x0002, /*1E076*/
  0xFBC3, 0x0020, 0x0002, 0xE077, 0x0000, 0x0000, 0x0002, /*1E077*/
  0xFBC3, 0x0020, 0x0002, 0xE078, 0x0000, 0x0000, 0x0002, /*1E078*/
  0xFBC3, 0x0020, 0x0002, 0xE079, 0x0000, 0x0000, 0x0002, /*1E079*/
  0xFBC3, 0x0020, 0x0002, 0xE07A, 0x0000, 0x0000, 0x0002, /*1E07A*/
  0xFBC3, 0x0020, 0x0002, 0xE07B, 0x0000, 0x0000, 0x0002, /*1E07B*/
  0xFBC3, 0x0020, 0x0002, 0xE07C, 0x0000, 0x0000, 0x0002, /*1E07C*/
  0xFBC3, 0x0020, 0x0002, 0xE07D, 0x0000, 0x0000, 0x0002, /*1E07D*/
  0xFBC3, 0x0020, 0x0002, 0xE07E, 0x0000, 0x0000, 0x0002, /*1E07E*/
  0xFBC3, 0x0020, 0x0002, 0xE07F, 0x0000, 0x0000, 0x0002, /*1E07F*/
  0xFBC3, 0x0020, 0x0002, 0xE080, 0x0000, 0x0000, 0x0002, /*1E080*/
  0xFBC3, 0x0020, 0x0002, 0xE081, 0x0000, 0x0000, 0x0002, /*1E081*/
  0xFBC3, 0x0020, 0x0002, 0xE082, 0x0000, 0x0000, 0x0002, /*1E082*/
  0xFBC3, 0x0020, 0x0002, 0xE083, 0x0000, 0x0000, 0x0002, /*1E083*/
  0xFBC3, 0x0020, 0x0002, 0xE084, 0x0000, 0x0000, 0x0002, /*1E084*/
  0xFBC3, 0x0020, 0x0002, 0xE085, 0x0000, 0x0000, 0x0002, /*1E085*/
  0xFBC3, 0x0020, 0x0002, 0xE086, 0x0000, 0x0000, 0x0002, /*1E086*/
  0xFBC3, 0x0020, 0x0002, 0xE087, 0x0000, 0x0000, 0x0002, /*1E087*/
  0xFBC3, 0x0020, 0x0002, 0xE088, 0x0000, 0x0000, 0x0002, /*1E088*/
  0xFBC3, 0x0020, 0x0002, 0xE089, 0x0000, 0x0000, 0x0002, /*1E089*/
  0xFBC3, 0x0020, 0x0002, 0xE08A, 0x0000, 0x0000, 0x0002, /*1E08A*/
  0xFBC3, 0x0020, 0x0002, 0xE08B, 0x0000, 0x0000, 0x0002, /*1E08B*/
  0xFBC3, 0x0020, 0x0002, 0xE08C, 0x0000, 0x0000, 0x0002, /*1E08C*/
  0xFBC3, 0x0020, 0x0002, 0xE08D, 0x0000, 0x0000, 0x0002, /*1E08D*/
  0xFBC3, 0x0020, 0x0002, 0xE08E, 0x0000, 0x0000, 0x0002, /*1E08E*/
  0xFBC3, 0x0020, 0x0002, 0xE08F, 0x0000, 0x0000, 0x0002, /*1E08F*/
  0xFBC3, 0x0020, 0x0002, 0xE090, 0x0000, 0x0000, 0x0002, /*1E090*/
  0xFBC3, 0x0020, 0x0002, 0xE091, 0x0000, 0x0000, 0x0002, /*1E091*/
  0xFBC3, 0x0020, 0x0002, 0xE092, 0x0000, 0x0000, 0x0002, /*1E092*/
  0xFBC3, 0x0020, 0x0002, 0xE093, 0x0000, 0x0000, 0x0002, /*1E093*/
  0xFBC3, 0x0020, 0x0002, 0xE094, 0x0000, 0x0000, 0x0002, /*1E094*/
  0xFBC3, 0x0020, 0x0002, 0xE095, 0x0000, 0x0000, 0x0002, /*1E095*/
  0xFBC3, 0x0020, 0x0002, 0xE096, 0x0000, 0x0000, 0x0002, /*1E096*/
  0xFBC3, 0x0020, 0x0002, 0xE097, 0x0000, 0x0000, 0x0002, /*1E097*/
  0xFBC3, 0x0020, 0x0002, 0xE098, 0x0000, 0x0000, 0x0002, /*1E098*/
  0xFBC3, 0x0020, 0x0002, 0xE099, 0x0000, 0x0000, 0x0002, /*1E099*/
  0xFBC3, 0x0020, 0x0002, 0xE09A, 0x0000, 0x0000, 0x0002, /*1E09A*/
  0xFBC3, 0x0020, 0x0002, 0xE09B, 0x0000, 0x0000, 0x0002, /*1E09B*/
  0xFBC3, 0x0020, 0x0002, 0xE09C, 0x0000, 0x0000, 0x0002, /*1E09C*/
  0xFBC3, 0x0020, 0x0002, 0xE09D, 0x0000, 0x0000, 0x0002, /*1E09D*/
  0xFBC3, 0x0020, 0x0002, 0xE09E, 0x0000, 0x0000, 0x0002, /*1E09E*/
  0xFBC3, 0x0020, 0x0002, 0xE09F, 0x0000, 0x0000, 0x0002, /*1E09F*/
  0xFBC3, 0x0020, 0x0002, 0xE0A0, 0x0000, 0x0000, 0x0002, /*1E0A0*/
  0xFBC3, 0x0020, 0x0002, 0xE0A1, 0x0000, 0x0000, 0x0002, /*1E0A1*/
  0xFBC3, 0x0020, 0x0002, 0xE0A2, 0x0000, 0x0000, 0x0002, /*1E0A2*/
  0xFBC3, 0x0020, 0x0002, 0xE0A3, 0x0000, 0x0000, 0x0002, /*1E0A3*/
  0xFBC3, 0x0020, 0x0002, 0xE0A4, 0x0000, 0x0000, 0x0002, /*1E0A4*/
  0xFBC3, 0x0020, 0x0002, 0xE0A5, 0x0000, 0x0000, 0x0002, /*1E0A5*/
  0xFBC3, 0x0020, 0x0002, 0xE0A6, 0x0000, 0x0000, 0x0002, /*1E0A6*/
  0xFBC3, 0x0020, 0x0002, 0xE0A7, 0x0000, 0x0000, 0x0002, /*1E0A7*/
  0xFBC3, 0x0020, 0x0002, 0xE0A8, 0x0000, 0x0000, 0x0002, /*1E0A8*/
  0xFBC3, 0x0020, 0x0002, 0xE0A9, 0x0000, 0x0000, 0x0002, /*1E0A9*/
  0xFBC3, 0x0020, 0x0002, 0xE0AA, 0x0000, 0x0000, 0x0002, /*1E0AA*/
  0xFBC3, 0x0020, 0x0002, 0xE0AB, 0x0000, 0x0000, 0x0002, /*1E0AB*/
  0xFBC3, 0x0020, 0x0002, 0xE0AC, 0x0000, 0x0000, 0x0002, /*1E0AC*/
  0xFBC3, 0x0020, 0x0002, 0xE0AD, 0x0000, 0x0000, 0x0002, /*1E0AD*/
  0xFBC3, 0x0020, 0x0002, 0xE0AE, 0x0000, 0x0000, 0x0002, /*1E0AE*/
  0xFBC3, 0x0020, 0x0002, 0xE0AF, 0x0000, 0x0000, 0x0002, /*1E0AF*/
  0xFBC3, 0x0020, 0x0002, 0xE0B0, 0x0000, 0x0000, 0x0002, /*1E0B0*/
  0xFBC3, 0x0020, 0x0002, 0xE0B1, 0x0000, 0x0000, 0x0002, /*1E0B1*/
  0xFBC3, 0x0020, 0x0002, 0xE0B2, 0x0000, 0x0000, 0x0002, /*1E0B2*/
  0xFBC3, 0x0020, 0x0002, 0xE0B3, 0x0000, 0x0000, 0x0002, /*1E0B3*/
  0xFBC3, 0x0020, 0x0002, 0xE0B4, 0x0000, 0x0000, 0x0002, /*1E0B4*/
  0xFBC3, 0x0020, 0x0002, 0xE0B5, 0x0000, 0x0000, 0x0002, /*1E0B5*/
  0xFBC3, 0x0020, 0x0002, 0xE0B6, 0x0000, 0x0000, 0x0002, /*1E0B6*/
  0xFBC3, 0x0020, 0x0002, 0xE0B7, 0x0000, 0x0000, 0x0002, /*1E0B7*/
  0xFBC3, 0x0020, 0x0002, 0xE0B8, 0x0000, 0x0000, 0x0002, /*1E0B8*/
  0xFBC3, 0x0020, 0x0002, 0xE0B9, 0x0000, 0x0000, 0x0002, /*1E0B9*/
  0xFBC3, 0x0020, 0x0002, 0xE0BA, 0x0000, 0x0000, 0x0002, /*1E0BA*/
  0xFBC3, 0x0020, 0x0002, 0xE0BB, 0x0000, 0x0000, 0x0002, /*1E0BB*/
  0xFBC3, 0x0020, 0x0002, 0xE0BC, 0x0000, 0x0000, 0x0002, /*1E0BC*/
  0xFBC3, 0x0020, 0x0002, 0xE0BD, 0x0000, 0x0000, 0x0002, /*1E0BD*/
  0xFBC3, 0x0020, 0x0002, 0xE0BE, 0x0000, 0x0000, 0x0002, /*1E0BE*/
  0xFBC3, 0x0020, 0x0002, 0xE0BF, 0x0000, 0x0000, 0x0002, /*1E0BF*/
  0xFBC3, 0x0020, 0x0002, 0xE0C0, 0x0000, 0x0000, 0x0002, /*1E0C0*/
  0xFBC3, 0x0020, 0x0002, 0xE0C1, 0x0000, 0x0000, 0x0002, /*1E0C1*/
  0xFBC3, 0x0020, 0x0002, 0xE0C2, 0x0000, 0x0000, 0x0002, /*1E0C2*/
  0xFBC3, 0x0020, 0x0002, 0xE0C3, 0x0000, 0x0000, 0x0002, /*1E0C3*/
  0xFBC3, 0x0020, 0x0002, 0xE0C4, 0x0000, 0x0000, 0x0002, /*1E0C4*/
  0xFBC3, 0x0020, 0x0002, 0xE0C5, 0x0000, 0x0000, 0x0002, /*1E0C5*/
  0xFBC3, 0x0020, 0x0002, 0xE0C6, 0x0000, 0x0000, 0x0002, /*1E0C6*/
  0xFBC3, 0x0020, 0x0002, 0xE0C7, 0x0000, 0x0000, 0x0002, /*1E0C7*/
  0xFBC3, 0x0020, 0x0002, 0xE0C8, 0x0000, 0x0000, 0x0002, /*1E0C8*/
  0xFBC3, 0x0020, 0x0002, 0xE0C9, 0x0000, 0x0000, 0x0002, /*1E0C9*/
  0xFBC3, 0x0020, 0x0002, 0xE0CA, 0x0000, 0x0000, 0x0002, /*1E0CA*/
  0xFBC3, 0x0020, 0x0002, 0xE0CB, 0x0000, 0x0000, 0x0002, /*1E0CB*/
  0xFBC3, 0x0020, 0x0002, 0xE0CC, 0x0000, 0x0000, 0x0002, /*1E0CC*/
  0xFBC3, 0x0020, 0x0002, 0xE0CD, 0x0000, 0x0000, 0x0002, /*1E0CD*/
  0xFBC3, 0x0020, 0x0002, 0xE0CE, 0x0000, 0x0000, 0x0002, /*1E0CE*/
  0xFBC3, 0x0020, 0x0002, 0xE0CF, 0x0000, 0x0000, 0x0002, /*1E0CF*/
  0xFBC3, 0x0020, 0x0002, 0xE0D0, 0x0000, 0x0000, 0x0002, /*1E0D0*/
  0xFBC3, 0x0020, 0x0002, 0xE0D1, 0x0000, 0x0000, 0x0002, /*1E0D1*/
  0xFBC3, 0x0020, 0x0002, 0xE0D2, 0x0000, 0x0000, 0x0002, /*1E0D2*/
  0xFBC3, 0x0020, 0x0002, 0xE0D3, 0x0000, 0x0000, 0x0002, /*1E0D3*/
  0xFBC3, 0x0020, 0x0002, 0xE0D4, 0x0000, 0x0000, 0x0002, /*1E0D4*/
  0xFBC3, 0x0020, 0x0002, 0xE0D5, 0x0000, 0x0000, 0x0002, /*1E0D5*/
  0xFBC3, 0x0020, 0x0002, 0xE0D6, 0x0000, 0x0000, 0x0002, /*1E0D6*/
  0xFBC3, 0x0020, 0x0002, 0xE0D7, 0x0000, 0x0000, 0x0002, /*1E0D7*/
  0xFBC3, 0x0020, 0x0002, 0xE0D8, 0x0000, 0x0000, 0x0002, /*1E0D8*/
  0xFBC3, 0x0020, 0x0002, 0xE0D9, 0x0000, 0x0000, 0x0002, /*1E0D9*/
  0xFBC3, 0x0020, 0x0002, 0xE0DA, 0x0000, 0x0000, 0x0002, /*1E0DA*/
  0xFBC3, 0x0020, 0x0002, 0xE0DB, 0x0000, 0x0000, 0x0002, /*1E0DB*/
  0xFBC3, 0x0020, 0x0002, 0xE0DC, 0x0000, 0x0000, 0x0002, /*1E0DC*/
  0xFBC3, 0x0020, 0x0002, 0xE0DD, 0x0000, 0x0000, 0x0002, /*1E0DD*/
  0xFBC3, 0x0020, 0x0002, 0xE0DE, 0x0000, 0x0000, 0x0002, /*1E0DE*/
  0xFBC3, 0x0020, 0x0002, 0xE0DF, 0x0000, 0x0000, 0x0002, /*1E0DF*/
  0xFBC3, 0x0020, 0x0002, 0xE0E0, 0x0000, 0x0000, 0x0002, /*1E0E0*/
  0xFBC3, 0x0020, 0x0002, 0xE0E1, 0x0000, 0x0000, 0x0002, /*1E0E1*/
  0xFBC3, 0x0020, 0x0002, 0xE0E2, 0x0000, 0x0000, 0x0002, /*1E0E2*/
  0xFBC3, 0x0020, 0x0002, 0xE0E3, 0x0000, 0x0000, 0x0002, /*1E0E3*/
  0xFBC3, 0x0020, 0x0002, 0xE0E4, 0x0000, 0x0000, 0x0002, /*1E0E4*/
  0xFBC3, 0x0020, 0x0002, 0xE0E5, 0x0000, 0x0000, 0x0002, /*1E0E5*/
  0xFBC3, 0x0020, 0x0002, 0xE0E6, 0x0000, 0x0000, 0x0002, /*1E0E6*/
  0xFBC3, 0x0020, 0x0002, 0xE0E7, 0x0000, 0x0000, 0x0002, /*1E0E7*/
  0xFBC3, 0x0020, 0x0002, 0xE0E8, 0x0000, 0x0000, 0x0002, /*1E0E8*/
  0xFBC3, 0x0020, 0x0002, 0xE0E9, 0x0000, 0x0000, 0x0002, /*1E0E9*/
  0xFBC3, 0x0020, 0x0002, 0xE0EA, 0x0000, 0x0000, 0x0002, /*1E0EA*/
  0xFBC3, 0x0020, 0x0002, 0xE0EB, 0x0000, 0x0000, 0x0002, /*1E0EB*/
  0xFBC3, 0x0020, 0x0002, 0xE0EC, 0x0000, 0x0000, 0x0002, /*1E0EC*/
  0xFBC3, 0x0020, 0x0002, 0xE0ED, 0x0000, 0x0000, 0x0002, /*1E0ED*/
  0xFBC3, 0x0020, 0x0002, 0xE0EE, 0x0000, 0x0000, 0x0002, /*1E0EE*/
  0xFBC3, 0x0020, 0x0002, 0xE0EF, 0x0000, 0x0000, 0x0002, /*1E0EF*/
  0xFBC3, 0x0020, 0x0002, 0xE0F0, 0x0000, 0x0000, 0x0002, /*1E0F0*/
  0xFBC3, 0x0020, 0x0002, 0xE0F1, 0x0000, 0x0000, 0x0002, /*1E0F1*/
  0xFBC3, 0x0020, 0x0002, 0xE0F2, 0x0000, 0x0000, 0x0002, /*1E0F2*/
  0xFBC3, 0x0020, 0x0002, 0xE0F3, 0x0000, 0x0000, 0x0002, /*1E0F3*/
  0xFBC3, 0x0020, 0x0002, 0xE0F4, 0x0000, 0x0000, 0x0002, /*1E0F4*/
  0xFBC3, 0x0020, 0x0002, 0xE0F5, 0x0000, 0x0000, 0x0002, /*1E0F5*/
  0xFBC3, 0x0020, 0x0002, 0xE0F6, 0x0000, 0x0000, 0x0002, /*1E0F6*/
  0xFBC3, 0x0020, 0x0002, 0xE0F7, 0x0000, 0x0000, 0x0002, /*1E0F7*/
  0xFBC3, 0x0020, 0x0002, 0xE0F8, 0x0000, 0x0000, 0x0002, /*1E0F8*/
  0xFBC3, 0x0020, 0x0002, 0xE0F9, 0x0000, 0x0000, 0x0002, /*1E0F9*/
  0xFBC3, 0x0020, 0x0002, 0xE0FA, 0x0000, 0x0000, 0x0002, /*1E0FA*/
  0xFBC3, 0x0020, 0x0002, 0xE0FB, 0x0000, 0x0000, 0x0002, /*1E0FB*/
  0xFBC3, 0x0020, 0x0002, 0xE0FC, 0x0000, 0x0000, 0x0002, /*1E0FC*/
  0xFBC3, 0x0020, 0x0002, 0xE0FD, 0x0000, 0x0000, 0x0002, /*1E0FD*/
  0xFBC3, 0x0020, 0x0002, 0xE0FE, 0x0000, 0x0000, 0x0002, /*1E0FE*/
  0xFBC3, 0x0020, 0x0002, 0xE0FF, 0x0000, 0x0000, 0x0002  /*1E0FF*/
};

uint16 uca900_p1E8[]= {
  0x3B0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E800*/
  0x3B0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E801*/
  0x3B10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E802*/
  0x3B11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E803*/
  0x3B12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E804*/
  0x3B13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E805*/
  0x3B14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E806*/
  0x3B15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E807*/
  0x3B16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E808*/
  0x3B17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E809*/
  0x3B18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E80A*/
  0x3B19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E80B*/
  0x3B1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E80C*/
  0x3B1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E80D*/
  0x3B1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E80E*/
  0x3B1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E80F*/
  0x3B1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E810*/
  0x3B1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E811*/
  0x3B20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E812*/
  0x3B21, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E813*/
  0x3B22, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E814*/
  0x3B23, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E815*/
  0x3B24, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E816*/
  0x3B25, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E817*/
  0x3B26, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E818*/
  0x3B27, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E819*/
  0x3B28, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E81A*/
  0x3B29, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E81B*/
  0x3B2A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E81C*/
  0x3B2B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E81D*/
  0x3B2C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E81E*/
  0x3B2D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E81F*/
  0x3B2E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E820*/
  0x3B2F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E821*/
  0x3B30, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E822*/
  0x3B31, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E823*/
  0x3B32, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E824*/
  0x3B33, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E825*/
  0x3B34, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E826*/
  0x3B35, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E827*/
  0x3B36, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E828*/
  0x3B37, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E829*/
  0x3B38, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E82A*/
  0x3B39, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E82B*/
  0x3B3A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E82C*/
  0x3B3B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E82D*/
  0x3B3C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E82E*/
  0x3B3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E82F*/
  0x3B3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E830*/
  0x3B3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E831*/
  0x3B40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E832*/
  0x3B41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E833*/
  0x3B42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E834*/
  0x3B43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E835*/
  0x3B44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E836*/
  0x3B45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E837*/
  0x3B46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E838*/
  0x3B47, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E839*/
  0x3B48, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E83A*/
  0x3B49, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E83B*/
  0x3B4A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E83C*/
  0x3B4B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E83D*/
  0x3B4C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E83E*/
  0x3B4D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E83F*/
  0x3B4E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E840*/
  0x3B4F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E841*/
  0x3B50, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E842*/
  0x3B51, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E843*/
  0x3B52, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E844*/
  0x3B53, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E845*/
  0x3B54, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E846*/
  0x3B55, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E847*/
  0x3B56, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E848*/
  0x3B57, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E849*/
  0x3B58, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E84A*/
  0x3B59, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E84B*/
  0x3B5A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E84C*/
  0x3B5B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E84D*/
  0x3B5C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E84E*/
  0x3B5D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E84F*/
  0x3B5E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E850*/
  0x3B5F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E851*/
  0x3B60, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E852*/
  0x3B61, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E853*/
  0x3B62, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E854*/
  0x3B63, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E855*/
  0x3B64, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E856*/
  0x3B65, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E857*/
  0x3B66, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E858*/
  0x3B67, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E859*/
  0x3B68, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E85A*/
  0x3B69, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E85B*/
  0x3B6A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E85C*/
  0x3B6B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E85D*/
  0x3B6C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E85E*/
  0x3B6D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E85F*/
  0x3B6E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E860*/
  0x3B6F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E861*/
  0x3B70, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E862*/
  0x3B71, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E863*/
  0x3B72, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E864*/
  0x3B73, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E865*/
  0x3B74, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E866*/
  0x3B75, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E867*/
  0x3B76, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E868*/
  0x3B77, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E869*/
  0x3B78, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E86A*/
  0x3B79, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E86B*/
  0x3B7A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E86C*/
  0x3B7B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E86D*/
  0x3B7C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E86E*/
  0x3B7D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E86F*/
  0x3B7E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E870*/
  0x3B7F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E871*/
  0x3B80, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E872*/
  0x3B81, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E873*/
  0x3B82, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E874*/
  0x3B83, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E875*/
  0x3B84, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E876*/
  0x3B85, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E877*/
  0x3B86, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E878*/
  0x3B87, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E879*/
  0x3B88, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E87A*/
  0x3B89, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E87B*/
  0x3B8A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E87C*/
  0x3B8B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E87D*/
  0x3B8C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E87E*/
  0x3B8D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E87F*/
  0x3B8E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E880*/
  0x3B8F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E881*/
  0x3B90, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E882*/
  0x3B91, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E883*/
  0x3B92, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E884*/
  0x3B93, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E885*/
  0x3B94, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E886*/
  0x3B95, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E887*/
  0x3B96, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E888*/
  0x3B97, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E889*/
  0x3B98, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E88A*/
  0x3B99, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E88B*/
  0x3B9A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E88C*/
  0x3B9B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E88D*/
  0x3B9C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E88E*/
  0x3B9D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E88F*/
  0x3B9E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E890*/
  0x3B9F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E891*/
  0x3BA0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E892*/
  0x3BA1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E893*/
  0x3BA2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E894*/
  0x3BA3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E895*/
  0x3BA4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E896*/
  0x3BA5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E897*/
  0x3BA6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E898*/
  0x3BA7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E899*/
  0x3BA8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E89A*/
  0x3BA9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E89B*/
  0x3BAA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E89C*/
  0x3BAB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E89D*/
  0x3BAC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E89E*/
  0x3BAD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E89F*/
  0x3BAE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8A0*/
  0x3BAF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8A1*/
  0x3BB0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8A2*/
  0x3BB1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8A3*/
  0x3BB2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8A4*/
  0x3BB3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8A5*/
  0x3BB4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8A6*/
  0x3BB5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8A7*/
  0x3BB6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8A8*/
  0x3BB7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8A9*/
  0x3BB8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8AA*/
  0x3BB9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8AB*/
  0x3BBA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8AC*/
  0x3BBB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8AD*/
  0x3BBC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8AE*/
  0x3BBD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8AF*/
  0x3BBE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8B0*/
  0x3BBF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8B1*/
  0x3BC0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8B2*/
  0x3BC1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8B3*/
  0x3BC2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8B4*/
  0x3BC3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8B5*/
  0x3BC4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8B6*/
  0x3BC5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8B7*/
  0x3BC6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8B8*/
  0x3BC7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8B9*/
  0x3BC8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8BA*/
  0x3BC9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8BB*/
  0x3BCA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8BC*/
  0x3BCB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8BD*/
  0x3BCC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8BE*/
  0x3BCD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8BF*/
  0x3BCE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8C0*/
  0x3BCF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8C1*/
  0x3BD0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8C2*/
  0x3BD1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8C3*/
  0x3BD2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8C4*/
  0xFBC3, 0x0020, 0x0002, 0xE8C5, 0x0000, 0x0000, 0x0002, /*1E8C5*/
  0xFBC3, 0x0020, 0x0002, 0xE8C6, 0x0000, 0x0000, 0x0002, /*1E8C6*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8C7*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8C8*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8C9*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8CA*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8CB*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8CC*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8CD*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8CE*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*1E8D6*/
  0xFBC3, 0x0020, 0x0002, 0xE8D7, 0x0000, 0x0000, 0x0002, /*1E8D7*/
  0xFBC3, 0x0020, 0x0002, 0xE8D8, 0x0000, 0x0000, 0x0002, /*1E8D8*/
  0xFBC3, 0x0020, 0x0002, 0xE8D9, 0x0000, 0x0000, 0x0002, /*1E8D9*/
  0xFBC3, 0x0020, 0x0002, 0xE8DA, 0x0000, 0x0000, 0x0002, /*1E8DA*/
  0xFBC3, 0x0020, 0x0002, 0xE8DB, 0x0000, 0x0000, 0x0002, /*1E8DB*/
  0xFBC3, 0x0020, 0x0002, 0xE8DC, 0x0000, 0x0000, 0x0002, /*1E8DC*/
  0xFBC3, 0x0020, 0x0002, 0xE8DD, 0x0000, 0x0000, 0x0002, /*1E8DD*/
  0xFBC3, 0x0020, 0x0002, 0xE8DE, 0x0000, 0x0000, 0x0002, /*1E8DE*/
  0xFBC3, 0x0020, 0x0002, 0xE8DF, 0x0000, 0x0000, 0x0002, /*1E8DF*/
  0xFBC3, 0x0020, 0x0002, 0xE8E0, 0x0000, 0x0000, 0x0002, /*1E8E0*/
  0xFBC3, 0x0020, 0x0002, 0xE8E1, 0x0000, 0x0000, 0x0002, /*1E8E1*/
  0xFBC3, 0x0020, 0x0002, 0xE8E2, 0x0000, 0x0000, 0x0002, /*1E8E2*/
  0xFBC3, 0x0020, 0x0002, 0xE8E3, 0x0000, 0x0000, 0x0002, /*1E8E3*/
  0xFBC3, 0x0020, 0x0002, 0xE8E4, 0x0000, 0x0000, 0x0002, /*1E8E4*/
  0xFBC3, 0x0020, 0x0002, 0xE8E5, 0x0000, 0x0000, 0x0002, /*1E8E5*/
  0xFBC3, 0x0020, 0x0002, 0xE8E6, 0x0000, 0x0000, 0x0002, /*1E8E6*/
  0xFBC3, 0x0020, 0x0002, 0xE8E7, 0x0000, 0x0000, 0x0002, /*1E8E7*/
  0xFBC3, 0x0020, 0x0002, 0xE8E8, 0x0000, 0x0000, 0x0002, /*1E8E8*/
  0xFBC3, 0x0020, 0x0002, 0xE8E9, 0x0000, 0x0000, 0x0002, /*1E8E9*/
  0xFBC3, 0x0020, 0x0002, 0xE8EA, 0x0000, 0x0000, 0x0002, /*1E8EA*/
  0xFBC3, 0x0020, 0x0002, 0xE8EB, 0x0000, 0x0000, 0x0002, /*1E8EB*/
  0xFBC3, 0x0020, 0x0002, 0xE8EC, 0x0000, 0x0000, 0x0002, /*1E8EC*/
  0xFBC3, 0x0020, 0x0002, 0xE8ED, 0x0000, 0x0000, 0x0002, /*1E8ED*/
  0xFBC3, 0x0020, 0x0002, 0xE8EE, 0x0000, 0x0000, 0x0002, /*1E8EE*/
  0xFBC3, 0x0020, 0x0002, 0xE8EF, 0x0000, 0x0000, 0x0002, /*1E8EF*/
  0xFBC3, 0x0020, 0x0002, 0xE8F0, 0x0000, 0x0000, 0x0002, /*1E8F0*/
  0xFBC3, 0x0020, 0x0002, 0xE8F1, 0x0000, 0x0000, 0x0002, /*1E8F1*/
  0xFBC3, 0x0020, 0x0002, 0xE8F2, 0x0000, 0x0000, 0x0002, /*1E8F2*/
  0xFBC3, 0x0020, 0x0002, 0xE8F3, 0x0000, 0x0000, 0x0002, /*1E8F3*/
  0xFBC3, 0x0020, 0x0002, 0xE8F4, 0x0000, 0x0000, 0x0002, /*1E8F4*/
  0xFBC3, 0x0020, 0x0002, 0xE8F5, 0x0000, 0x0000, 0x0002, /*1E8F5*/
  0xFBC3, 0x0020, 0x0002, 0xE8F6, 0x0000, 0x0000, 0x0002, /*1E8F6*/
  0xFBC3, 0x0020, 0x0002, 0xE8F7, 0x0000, 0x0000, 0x0002, /*1E8F7*/
  0xFBC3, 0x0020, 0x0002, 0xE8F8, 0x0000, 0x0000, 0x0002, /*1E8F8*/
  0xFBC3, 0x0020, 0x0002, 0xE8F9, 0x0000, 0x0000, 0x0002, /*1E8F9*/
  0xFBC3, 0x0020, 0x0002, 0xE8FA, 0x0000, 0x0000, 0x0002, /*1E8FA*/
  0xFBC3, 0x0020, 0x0002, 0xE8FB, 0x0000, 0x0000, 0x0002, /*1E8FB*/
  0xFBC3, 0x0020, 0x0002, 0xE8FC, 0x0000, 0x0000, 0x0002, /*1E8FC*/
  0xFBC3, 0x0020, 0x0002, 0xE8FD, 0x0000, 0x0000, 0x0002, /*1E8FD*/
  0xFBC3, 0x0020, 0x0002, 0xE8FE, 0x0000, 0x0000, 0x0002, /*1E8FE*/
  0xFBC3, 0x0020, 0x0002, 0xE8FF, 0x0000, 0x0000, 0x0002  /*1E8FF*/
};

uint16 uca900_p1E9[]= {
  0x3BD3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E900*/
  0x3BD4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E901*/
  0x3BD5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E902*/
  0x3BD6, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E903*/
  0x3BD7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E904*/
  0x3BD8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E905*/
  0x3BD9, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E906*/
  0x3BDA, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E907*/
  0x3BDB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E908*/
  0x3BDC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E909*/
  0x3BDD, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E90A*/
  0x3BDE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E90B*/
  0x3BDF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E90C*/
  0x3BE0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E90D*/
  0x3BE1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E90E*/
  0x3BE2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E90F*/
  0x3BE3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E910*/
  0x3BE4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E911*/
  0x3BE5, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E912*/
  0x3BE6, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E913*/
  0x3BE7, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E914*/
  0x3BE8, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E915*/
  0x3BE9, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E916*/
  0x3BEA, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E917*/
  0x3BEB, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E918*/
  0x3BEC, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E919*/
  0x3BED, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E91A*/
  0x3BEE, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E91B*/
  0x3BEF, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E91C*/
  0x3BF0, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E91D*/
  0x3BF1, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E91E*/
  0x3BF2, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E91F*/
  0x3BF3, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E920*/
  0x3BF4, 0x0020, 0x0008, 0x0000, 0x0000, 0x0000, 0x0001, /*1E921*/
  0x3BD3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E922*/
  0x3BD4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E923*/
  0x3BD5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E924*/
  0x3BD6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E925*/
  0x3BD7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E926*/
  0x3BD8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E927*/
  0x3BD9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E928*/
  0x3BDA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E929*/
  0x3BDB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E92A*/
  0x3BDC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E92B*/
  0x3BDD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E92C*/
  0x3BDE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E92D*/
  0x3BDF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E92E*/
  0x3BE0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E92F*/
  0x3BE1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E930*/
  0x3BE2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E931*/
  0x3BE3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E932*/
  0x3BE4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E933*/
  0x3BE5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E934*/
  0x3BE6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E935*/
  0x3BE7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E936*/
  0x3BE8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E937*/
  0x3BE9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E938*/
  0x3BEA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E939*/
  0x3BEB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E93A*/
  0x3BEC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E93B*/
  0x3BED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E93C*/
  0x3BEE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E93D*/
  0x3BEF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E93E*/
  0x3BF0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E93F*/
  0x3BF1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E940*/
  0x3BF2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E941*/
  0x3BF3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E942*/
  0x3BF4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E943*/
  0x0000, 0x00BD, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E944*/
  0x0000, 0x00BD, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E945*/
  0x0000, 0x00BD, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E946*/
  0x0000, 0x00BF, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E947*/
  0x0000, 0x00C0, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E948*/
  0x0000, 0x00C1, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E949*/
  0x0000, 0x00BE, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E94A*/
  0xFBC3, 0x0020, 0x0002, 0xE94B, 0x0000, 0x0000, 0x0002, /*1E94B*/
  0xFBC3, 0x0020, 0x0002, 0xE94C, 0x0000, 0x0000, 0x0002, /*1E94C*/
  0xFBC3, 0x0020, 0x0002, 0xE94D, 0x0000, 0x0000, 0x0002, /*1E94D*/
  0xFBC3, 0x0020, 0x0002, 0xE94E, 0x0000, 0x0000, 0x0002, /*1E94E*/
  0xFBC3, 0x0020, 0x0002, 0xE94F, 0x0000, 0x0000, 0x0002, /*1E94F*/
  0x1C3D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E950*/
  0x1C3E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E951*/
  0x1C3F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E952*/
  0x1C40, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E953*/
  0x1C41, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E954*/
  0x1C42, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E955*/
  0x1C43, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E956*/
  0x1C44, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E957*/
  0x1C45, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E958*/
  0x1C46, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E959*/
  0xFBC3, 0x0020, 0x0002, 0xE95A, 0x0000, 0x0000, 0x0002, /*1E95A*/
  0xFBC3, 0x0020, 0x0002, 0xE95B, 0x0000, 0x0000, 0x0002, /*1E95B*/
  0xFBC3, 0x0020, 0x0002, 0xE95C, 0x0000, 0x0000, 0x0002, /*1E95C*/
  0xFBC3, 0x0020, 0x0002, 0xE95D, 0x0000, 0x0000, 0x0002, /*1E95D*/
  0x0265, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E95E*/
  0x0274, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1E95F*/
  0xFBC3, 0x0020, 0x0002, 0xE960, 0x0000, 0x0000, 0x0002, /*1E960*/
  0xFBC3, 0x0020, 0x0002, 0xE961, 0x0000, 0x0000, 0x0002, /*1E961*/
  0xFBC3, 0x0020, 0x0002, 0xE962, 0x0000, 0x0000, 0x0002, /*1E962*/
  0xFBC3, 0x0020, 0x0002, 0xE963, 0x0000, 0x0000, 0x0002, /*1E963*/
  0xFBC3, 0x0020, 0x0002, 0xE964, 0x0000, 0x0000, 0x0002, /*1E964*/
  0xFBC3, 0x0020, 0x0002, 0xE965, 0x0000, 0x0000, 0x0002, /*1E965*/
  0xFBC3, 0x0020, 0x0002, 0xE966, 0x0000, 0x0000, 0x0002, /*1E966*/
  0xFBC3, 0x0020, 0x0002, 0xE967, 0x0000, 0x0000, 0x0002, /*1E967*/
  0xFBC3, 0x0020, 0x0002, 0xE968, 0x0000, 0x0000, 0x0002, /*1E968*/
  0xFBC3, 0x0020, 0x0002, 0xE969, 0x0000, 0x0000, 0x0002, /*1E969*/
  0xFBC3, 0x0020, 0x0002, 0xE96A, 0x0000, 0x0000, 0x0002, /*1E96A*/
  0xFBC3, 0x0020, 0x0002, 0xE96B, 0x0000, 0x0000, 0x0002, /*1E96B*/
  0xFBC3, 0x0020, 0x0002, 0xE96C, 0x0000, 0x0000, 0x0002, /*1E96C*/
  0xFBC3, 0x0020, 0x0002, 0xE96D, 0x0000, 0x0000, 0x0002, /*1E96D*/
  0xFBC3, 0x0020, 0x0002, 0xE96E, 0x0000, 0x0000, 0x0002, /*1E96E*/
  0xFBC3, 0x0020, 0x0002, 0xE96F, 0x0000, 0x0000, 0x0002, /*1E96F*/
  0xFBC3, 0x0020, 0x0002, 0xE970, 0x0000, 0x0000, 0x0002, /*1E970*/
  0xFBC3, 0x0020, 0x0002, 0xE971, 0x0000, 0x0000, 0x0002, /*1E971*/
  0xFBC3, 0x0020, 0x0002, 0xE972, 0x0000, 0x0000, 0x0002, /*1E972*/
  0xFBC3, 0x0020, 0x0002, 0xE973, 0x0000, 0x0000, 0x0002, /*1E973*/
  0xFBC3, 0x0020, 0x0002, 0xE974, 0x0000, 0x0000, 0x0002, /*1E974*/
  0xFBC3, 0x0020, 0x0002, 0xE975, 0x0000, 0x0000, 0x0002, /*1E975*/
  0xFBC3, 0x0020, 0x0002, 0xE976, 0x0000, 0x0000, 0x0002, /*1E976*/
  0xFBC3, 0x0020, 0x0002, 0xE977, 0x0000, 0x0000, 0x0002, /*1E977*/
  0xFBC3, 0x0020, 0x0002, 0xE978, 0x0000, 0x0000, 0x0002, /*1E978*/
  0xFBC3, 0x0020, 0x0002, 0xE979, 0x0000, 0x0000, 0x0002, /*1E979*/
  0xFBC3, 0x0020, 0x0002, 0xE97A, 0x0000, 0x0000, 0x0002, /*1E97A*/
  0xFBC3, 0x0020, 0x0002, 0xE97B, 0x0000, 0x0000, 0x0002, /*1E97B*/
  0xFBC3, 0x0020, 0x0002, 0xE97C, 0x0000, 0x0000, 0x0002, /*1E97C*/
  0xFBC3, 0x0020, 0x0002, 0xE97D, 0x0000, 0x0000, 0x0002, /*1E97D*/
  0xFBC3, 0x0020, 0x0002, 0xE97E, 0x0000, 0x0000, 0x0002, /*1E97E*/
  0xFBC3, 0x0020, 0x0002, 0xE97F, 0x0000, 0x0000, 0x0002, /*1E97F*/
  0xFBC3, 0x0020, 0x0002, 0xE980, 0x0000, 0x0000, 0x0002, /*1E980*/
  0xFBC3, 0x0020, 0x0002, 0xE981, 0x0000, 0x0000, 0x0002, /*1E981*/
  0xFBC3, 0x0020, 0x0002, 0xE982, 0x0000, 0x0000, 0x0002, /*1E982*/
  0xFBC3, 0x0020, 0x0002, 0xE983, 0x0000, 0x0000, 0x0002, /*1E983*/
  0xFBC3, 0x0020, 0x0002, 0xE984, 0x0000, 0x0000, 0x0002, /*1E984*/
  0xFBC3, 0x0020, 0x0002, 0xE985, 0x0000, 0x0000, 0x0002, /*1E985*/
  0xFBC3, 0x0020, 0x0002, 0xE986, 0x0000, 0x0000, 0x0002, /*1E986*/
  0xFBC3, 0x0020, 0x0002, 0xE987, 0x0000, 0x0000, 0x0002, /*1E987*/
  0xFBC3, 0x0020, 0x0002, 0xE988, 0x0000, 0x0000, 0x0002, /*1E988*/
  0xFBC3, 0x0020, 0x0002, 0xE989, 0x0000, 0x0000, 0x0002, /*1E989*/
  0xFBC3, 0x0020, 0x0002, 0xE98A, 0x0000, 0x0000, 0x0002, /*1E98A*/
  0xFBC3, 0x0020, 0x0002, 0xE98B, 0x0000, 0x0000, 0x0002, /*1E98B*/
  0xFBC3, 0x0020, 0x0002, 0xE98C, 0x0000, 0x0000, 0x0002, /*1E98C*/
  0xFBC3, 0x0020, 0x0002, 0xE98D, 0x0000, 0x0000, 0x0002, /*1E98D*/
  0xFBC3, 0x0020, 0x0002, 0xE98E, 0x0000, 0x0000, 0x0002, /*1E98E*/
  0xFBC3, 0x0020, 0x0002, 0xE98F, 0x0000, 0x0000, 0x0002, /*1E98F*/
  0xFBC3, 0x0020, 0x0002, 0xE990, 0x0000, 0x0000, 0x0002, /*1E990*/
  0xFBC3, 0x0020, 0x0002, 0xE991, 0x0000, 0x0000, 0x0002, /*1E991*/
  0xFBC3, 0x0020, 0x0002, 0xE992, 0x0000, 0x0000, 0x0002, /*1E992*/
  0xFBC3, 0x0020, 0x0002, 0xE993, 0x0000, 0x0000, 0x0002, /*1E993*/
  0xFBC3, 0x0020, 0x0002, 0xE994, 0x0000, 0x0000, 0x0002, /*1E994*/
  0xFBC3, 0x0020, 0x0002, 0xE995, 0x0000, 0x0000, 0x0002, /*1E995*/
  0xFBC3, 0x0020, 0x0002, 0xE996, 0x0000, 0x0000, 0x0002, /*1E996*/
  0xFBC3, 0x0020, 0x0002, 0xE997, 0x0000, 0x0000, 0x0002, /*1E997*/
  0xFBC3, 0x0020, 0x0002, 0xE998, 0x0000, 0x0000, 0x0002, /*1E998*/
  0xFBC3, 0x0020, 0x0002, 0xE999, 0x0000, 0x0000, 0x0002, /*1E999*/
  0xFBC3, 0x0020, 0x0002, 0xE99A, 0x0000, 0x0000, 0x0002, /*1E99A*/
  0xFBC3, 0x0020, 0x0002, 0xE99B, 0x0000, 0x0000, 0x0002, /*1E99B*/
  0xFBC3, 0x0020, 0x0002, 0xE99C, 0x0000, 0x0000, 0x0002, /*1E99C*/
  0xFBC3, 0x0020, 0x0002, 0xE99D, 0x0000, 0x0000, 0x0002, /*1E99D*/
  0xFBC3, 0x0020, 0x0002, 0xE99E, 0x0000, 0x0000, 0x0002, /*1E99E*/
  0xFBC3, 0x0020, 0x0002, 0xE99F, 0x0000, 0x0000, 0x0002, /*1E99F*/
  0xFBC3, 0x0020, 0x0002, 0xE9A0, 0x0000, 0x0000, 0x0002, /*1E9A0*/
  0xFBC3, 0x0020, 0x0002, 0xE9A1, 0x0000, 0x0000, 0x0002, /*1E9A1*/
  0xFBC3, 0x0020, 0x0002, 0xE9A2, 0x0000, 0x0000, 0x0002, /*1E9A2*/
  0xFBC3, 0x0020, 0x0002, 0xE9A3, 0x0000, 0x0000, 0x0002, /*1E9A3*/
  0xFBC3, 0x0020, 0x0002, 0xE9A4, 0x0000, 0x0000, 0x0002, /*1E9A4*/
  0xFBC3, 0x0020, 0x0002, 0xE9A5, 0x0000, 0x0000, 0x0002, /*1E9A5*/
  0xFBC3, 0x0020, 0x0002, 0xE9A6, 0x0000, 0x0000, 0x0002, /*1E9A6*/
  0xFBC3, 0x0020, 0x0002, 0xE9A7, 0x0000, 0x0000, 0x0002, /*1E9A7*/
  0xFBC3, 0x0020, 0x0002, 0xE9A8, 0x0000, 0x0000, 0x0002, /*1E9A8*/
  0xFBC3, 0x0020, 0x0002, 0xE9A9, 0x0000, 0x0000, 0x0002, /*1E9A9*/
  0xFBC3, 0x0020, 0x0002, 0xE9AA, 0x0000, 0x0000, 0x0002, /*1E9AA*/
  0xFBC3, 0x0020, 0x0002, 0xE9AB, 0x0000, 0x0000, 0x0002, /*1E9AB*/
  0xFBC3, 0x0020, 0x0002, 0xE9AC, 0x0000, 0x0000, 0x0002, /*1E9AC*/
  0xFBC3, 0x0020, 0x0002, 0xE9AD, 0x0000, 0x0000, 0x0002, /*1E9AD*/
  0xFBC3, 0x0020, 0x0002, 0xE9AE, 0x0000, 0x0000, 0x0002, /*1E9AE*/
  0xFBC3, 0x0020, 0x0002, 0xE9AF, 0x0000, 0x0000, 0x0002, /*1E9AF*/
  0xFBC3, 0x0020, 0x0002, 0xE9B0, 0x0000, 0x0000, 0x0002, /*1E9B0*/
  0xFBC3, 0x0020, 0x0002, 0xE9B1, 0x0000, 0x0000, 0x0002, /*1E9B1*/
  0xFBC3, 0x0020, 0x0002, 0xE9B2, 0x0000, 0x0000, 0x0002, /*1E9B2*/
  0xFBC3, 0x0020, 0x0002, 0xE9B3, 0x0000, 0x0000, 0x0002, /*1E9B3*/
  0xFBC3, 0x0020, 0x0002, 0xE9B4, 0x0000, 0x0000, 0x0002, /*1E9B4*/
  0xFBC3, 0x0020, 0x0002, 0xE9B5, 0x0000, 0x0000, 0x0002, /*1E9B5*/
  0xFBC3, 0x0020, 0x0002, 0xE9B6, 0x0000, 0x0000, 0x0002, /*1E9B6*/
  0xFBC3, 0x0020, 0x0002, 0xE9B7, 0x0000, 0x0000, 0x0002, /*1E9B7*/
  0xFBC3, 0x0020, 0x0002, 0xE9B8, 0x0000, 0x0000, 0x0002, /*1E9B8*/
  0xFBC3, 0x0020, 0x0002, 0xE9B9, 0x0000, 0x0000, 0x0002, /*1E9B9*/
  0xFBC3, 0x0020, 0x0002, 0xE9BA, 0x0000, 0x0000, 0x0002, /*1E9BA*/
  0xFBC3, 0x0020, 0x0002, 0xE9BB, 0x0000, 0x0000, 0x0002, /*1E9BB*/
  0xFBC3, 0x0020, 0x0002, 0xE9BC, 0x0000, 0x0000, 0x0002, /*1E9BC*/
  0xFBC3, 0x0020, 0x0002, 0xE9BD, 0x0000, 0x0000, 0x0002, /*1E9BD*/
  0xFBC3, 0x0020, 0x0002, 0xE9BE, 0x0000, 0x0000, 0x0002, /*1E9BE*/
  0xFBC3, 0x0020, 0x0002, 0xE9BF, 0x0000, 0x0000, 0x0002, /*1E9BF*/
  0xFBC3, 0x0020, 0x0002, 0xE9C0, 0x0000, 0x0000, 0x0002, /*1E9C0*/
  0xFBC3, 0x0020, 0x0002, 0xE9C1, 0x0000, 0x0000, 0x0002, /*1E9C1*/
  0xFBC3, 0x0020, 0x0002, 0xE9C2, 0x0000, 0x0000, 0x0002, /*1E9C2*/
  0xFBC3, 0x0020, 0x0002, 0xE9C3, 0x0000, 0x0000, 0x0002, /*1E9C3*/
  0xFBC3, 0x0020, 0x0002, 0xE9C4, 0x0000, 0x0000, 0x0002, /*1E9C4*/
  0xFBC3, 0x0020, 0x0002, 0xE9C5, 0x0000, 0x0000, 0x0002, /*1E9C5*/
  0xFBC3, 0x0020, 0x0002, 0xE9C6, 0x0000, 0x0000, 0x0002, /*1E9C6*/
  0xFBC3, 0x0020, 0x0002, 0xE9C7, 0x0000, 0x0000, 0x0002, /*1E9C7*/
  0xFBC3, 0x0020, 0x0002, 0xE9C8, 0x0000, 0x0000, 0x0002, /*1E9C8*/
  0xFBC3, 0x0020, 0x0002, 0xE9C9, 0x0000, 0x0000, 0x0002, /*1E9C9*/
  0xFBC3, 0x0020, 0x0002, 0xE9CA, 0x0000, 0x0000, 0x0002, /*1E9CA*/
  0xFBC3, 0x0020, 0x0002, 0xE9CB, 0x0000, 0x0000, 0x0002, /*1E9CB*/
  0xFBC3, 0x0020, 0x0002, 0xE9CC, 0x0000, 0x0000, 0x0002, /*1E9CC*/
  0xFBC3, 0x0020, 0x0002, 0xE9CD, 0x0000, 0x0000, 0x0002, /*1E9CD*/
  0xFBC3, 0x0020, 0x0002, 0xE9CE, 0x0000, 0x0000, 0x0002, /*1E9CE*/
  0xFBC3, 0x0020, 0x0002, 0xE9CF, 0x0000, 0x0000, 0x0002, /*1E9CF*/
  0xFBC3, 0x0020, 0x0002, 0xE9D0, 0x0000, 0x0000, 0x0002, /*1E9D0*/
  0xFBC3, 0x0020, 0x0002, 0xE9D1, 0x0000, 0x0000, 0x0002, /*1E9D1*/
  0xFBC3, 0x0020, 0x0002, 0xE9D2, 0x0000, 0x0000, 0x0002, /*1E9D2*/
  0xFBC3, 0x0020, 0x0002, 0xE9D3, 0x0000, 0x0000, 0x0002, /*1E9D3*/
  0xFBC3, 0x0020, 0x0002, 0xE9D4, 0x0000, 0x0000, 0x0002, /*1E9D4*/
  0xFBC3, 0x0020, 0x0002, 0xE9D5, 0x0000, 0x0000, 0x0002, /*1E9D5*/
  0xFBC3, 0x0020, 0x0002, 0xE9D6, 0x0000, 0x0000, 0x0002, /*1E9D6*/
  0xFBC3, 0x0020, 0x0002, 0xE9D7, 0x0000, 0x0000, 0x0002, /*1E9D7*/
  0xFBC3, 0x0020, 0x0002, 0xE9D8, 0x0000, 0x0000, 0x0002, /*1E9D8*/
  0xFBC3, 0x0020, 0x0002, 0xE9D9, 0x0000, 0x0000, 0x0002, /*1E9D9*/
  0xFBC3, 0x0020, 0x0002, 0xE9DA, 0x0000, 0x0000, 0x0002, /*1E9DA*/
  0xFBC3, 0x0020, 0x0002, 0xE9DB, 0x0000, 0x0000, 0x0002, /*1E9DB*/
  0xFBC3, 0x0020, 0x0002, 0xE9DC, 0x0000, 0x0000, 0x0002, /*1E9DC*/
  0xFBC3, 0x0020, 0x0002, 0xE9DD, 0x0000, 0x0000, 0x0002, /*1E9DD*/
  0xFBC3, 0x0020, 0x0002, 0xE9DE, 0x0000, 0x0000, 0x0002, /*1E9DE*/
  0xFBC3, 0x0020, 0x0002, 0xE9DF, 0x0000, 0x0000, 0x0002, /*1E9DF*/
  0xFBC3, 0x0020, 0x0002, 0xE9E0, 0x0000, 0x0000, 0x0002, /*1E9E0*/
  0xFBC3, 0x0020, 0x0002, 0xE9E1, 0x0000, 0x0000, 0x0002, /*1E9E1*/
  0xFBC3, 0x0020, 0x0002, 0xE9E2, 0x0000, 0x0000, 0x0002, /*1E9E2*/
  0xFBC3, 0x0020, 0x0002, 0xE9E3, 0x0000, 0x0000, 0x0002, /*1E9E3*/
  0xFBC3, 0x0020, 0x0002, 0xE9E4, 0x0000, 0x0000, 0x0002, /*1E9E4*/
  0xFBC3, 0x0020, 0x0002, 0xE9E5, 0x0000, 0x0000, 0x0002, /*1E9E5*/
  0xFBC3, 0x0020, 0x0002, 0xE9E6, 0x0000, 0x0000, 0x0002, /*1E9E6*/
  0xFBC3, 0x0020, 0x0002, 0xE9E7, 0x0000, 0x0000, 0x0002, /*1E9E7*/
  0xFBC3, 0x0020, 0x0002, 0xE9E8, 0x0000, 0x0000, 0x0002, /*1E9E8*/
  0xFBC3, 0x0020, 0x0002, 0xE9E9, 0x0000, 0x0000, 0x0002, /*1E9E9*/
  0xFBC3, 0x0020, 0x0002, 0xE9EA, 0x0000, 0x0000, 0x0002, /*1E9EA*/
  0xFBC3, 0x0020, 0x0002, 0xE9EB, 0x0000, 0x0000, 0x0002, /*1E9EB*/
  0xFBC3, 0x0020, 0x0002, 0xE9EC, 0x0000, 0x0000, 0x0002, /*1E9EC*/
  0xFBC3, 0x0020, 0x0002, 0xE9ED, 0x0000, 0x0000, 0x0002, /*1E9ED*/
  0xFBC3, 0x0020, 0x0002, 0xE9EE, 0x0000, 0x0000, 0x0002, /*1E9EE*/
  0xFBC3, 0x0020, 0x0002, 0xE9EF, 0x0000, 0x0000, 0x0002, /*1E9EF*/
  0xFBC3, 0x0020, 0x0002, 0xE9F0, 0x0000, 0x0000, 0x0002, /*1E9F0*/
  0xFBC3, 0x0020, 0x0002, 0xE9F1, 0x0000, 0x0000, 0x0002, /*1E9F1*/
  0xFBC3, 0x0020, 0x0002, 0xE9F2, 0x0000, 0x0000, 0x0002, /*1E9F2*/
  0xFBC3, 0x0020, 0x0002, 0xE9F3, 0x0000, 0x0000, 0x0002, /*1E9F3*/
  0xFBC3, 0x0020, 0x0002, 0xE9F4, 0x0000, 0x0000, 0x0002, /*1E9F4*/
  0xFBC3, 0x0020, 0x0002, 0xE9F5, 0x0000, 0x0000, 0x0002, /*1E9F5*/
  0xFBC3, 0x0020, 0x0002, 0xE9F6, 0x0000, 0x0000, 0x0002, /*1E9F6*/
  0xFBC3, 0x0020, 0x0002, 0xE9F7, 0x0000, 0x0000, 0x0002, /*1E9F7*/
  0xFBC3, 0x0020, 0x0002, 0xE9F8, 0x0000, 0x0000, 0x0002, /*1E9F8*/
  0xFBC3, 0x0020, 0x0002, 0xE9F9, 0x0000, 0x0000, 0x0002, /*1E9F9*/
  0xFBC3, 0x0020, 0x0002, 0xE9FA, 0x0000, 0x0000, 0x0002, /*1E9FA*/
  0xFBC3, 0x0020, 0x0002, 0xE9FB, 0x0000, 0x0000, 0x0002, /*1E9FB*/
  0xFBC3, 0x0020, 0x0002, 0xE9FC, 0x0000, 0x0000, 0x0002, /*1E9FC*/
  0xFBC3, 0x0020, 0x0002, 0xE9FD, 0x0000, 0x0000, 0x0002, /*1E9FD*/
  0xFBC3, 0x0020, 0x0002, 0xE9FE, 0x0000, 0x0000, 0x0002, /*1E9FE*/
  0xFBC3, 0x0020, 0x0002, 0xE9FF, 0x0000, 0x0000, 0x0002  /*1E9FF*/
};

uint16 uca900_p1EE[]= {
  0x230B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE00*/
  0x230D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE01*/
  0x2325, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE02*/
  0x2337, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE03*/
  0xFBC3, 0x0020, 0x0002, 0xEE04, 0x0000, 0x0000, 0x0002, /*1EE04*/
  0x23B7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE05*/
  0x2347, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE06*/
  0x232C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE07*/
  0x236A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE08*/
  0x23C6, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE09*/
  0x2387, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE0A*/
  0x239C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE0B*/
  0x23A3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE0C*/
  0x23A7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE0D*/
  0x2359, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE0E*/
  0x236E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE0F*/
  0x2376, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE10*/
  0x2364, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE11*/
  0x2382, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE12*/
  0x2346, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE13*/
  0x235A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE14*/
  0x231D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE15*/
  0x231E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE16*/
  0x232D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE17*/
  0x2338, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE18*/
  0x2365, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE19*/
  0x236B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE1A*/
  0x236F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE1B*/
  0x230C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE1C*/
  0x23A8, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE1D*/
  0x2377, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE1E*/
  0x2381, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE1F*/
  0xFBC3, 0x0020, 0x0002, 0xEE20, 0x0000, 0x0000, 0x0002, /*1EE20*/
  0x230D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE21*/
  0x2325, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE22*/
  0xFBC3, 0x0020, 0x0002, 0xEE23, 0x0000, 0x0000, 0x0002, /*1EE23*/
  0x23B1, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE24*/
  0xFBC3, 0x0020, 0x0002, 0xEE25, 0x0000, 0x0000, 0x0002, /*1EE25*/
  0xFBC3, 0x0020, 0x0002, 0xEE26, 0x0000, 0x0000, 0x0002, /*1EE26*/
  0x232C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE27*/
  0xFBC3, 0x0020, 0x0002, 0xEE28, 0x0000, 0x0000, 0x0002, /*1EE28*/
  0x23C6, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE29*/
  0x2387, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE2A*/
  0x239C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE2B*/
  0x23A3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE2C*/
  0x23A7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE2D*/
  0x2359, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE2E*/
  0x236E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE2F*/
  0x2376, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE30*/
  0x2364, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE31*/
  0x2382, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE32*/
  0xFBC3, 0x0020, 0x0002, 0xEE33, 0x0000, 0x0000, 0x0002, /*1EE33*/
  0x235A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE34*/
  0x231D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE35*/
  0x231E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE36*/
  0x232D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE37*/
  0xFBC3, 0x0020, 0x0002, 0xEE38, 0x0000, 0x0000, 0x0002, /*1EE38*/
  0x2365, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE39*/
  0xFBC3, 0x0020, 0x0002, 0xEE3A, 0x0000, 0x0000, 0x0002, /*1EE3A*/
  0x236F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE3B*/
  0xFBC3, 0x0020, 0x0002, 0xEE3C, 0x0000, 0x0000, 0x0002, /*1EE3C*/
  0xFBC3, 0x0020, 0x0002, 0xEE3D, 0x0000, 0x0000, 0x0002, /*1EE3D*/
  0xFBC3, 0x0020, 0x0002, 0xEE3E, 0x0000, 0x0000, 0x0002, /*1EE3E*/
  0xFBC3, 0x0020, 0x0002, 0xEE3F, 0x0000, 0x0000, 0x0002, /*1EE3F*/
  0xFBC3, 0x0020, 0x0002, 0xEE40, 0x0000, 0x0000, 0x0002, /*1EE40*/
  0xFBC3, 0x0020, 0x0002, 0xEE41, 0x0000, 0x0000, 0x0002, /*1EE41*/
  0x2325, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE42*/
  0xFBC3, 0x0020, 0x0002, 0xEE43, 0x0000, 0x0000, 0x0002, /*1EE43*/
  0xFBC3, 0x0020, 0x0002, 0xEE44, 0x0000, 0x0000, 0x0002, /*1EE44*/
  0xFBC3, 0x0020, 0x0002, 0xEE45, 0x0000, 0x0000, 0x0002, /*1EE45*/
  0xFBC3, 0x0020, 0x0002, 0xEE46, 0x0000, 0x0000, 0x0002, /*1EE46*/
  0x232C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE47*/
  0xFBC3, 0x0020, 0x0002, 0xEE48, 0x0000, 0x0000, 0x0002, /*1EE48*/
  0x23C6, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE49*/
  0xFBC3, 0x0020, 0x0002, 0xEE4A, 0x0000, 0x0000, 0x0002, /*1EE4A*/
  0x239C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE4B*/
  0xFBC3, 0x0020, 0x0002, 0xEE4C, 0x0000, 0x0000, 0x0002, /*1EE4C*/
  0x23A7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE4D*/
  0x2359, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE4E*/
  0x236E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE4F*/
  0xFBC3, 0x0020, 0x0002, 0xEE50, 0x0000, 0x0000, 0x0002, /*1EE50*/
  0x2364, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE51*/
  0x2382, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE52*/
  0xFBC3, 0x0020, 0x0002, 0xEE53, 0x0000, 0x0000, 0x0002, /*1EE53*/
  0x235A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE54*/
  0xFBC3, 0x0020, 0x0002, 0xEE55, 0x0000, 0x0000, 0x0002, /*1EE55*/
  0xFBC3, 0x0020, 0x0002, 0xEE56, 0x0000, 0x0000, 0x0002, /*1EE56*/
  0x232D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE57*/
  0xFBC3, 0x0020, 0x0002, 0xEE58, 0x0000, 0x0000, 0x0002, /*1EE58*/
  0x2365, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE59*/
  0xFBC3, 0x0020, 0x0002, 0xEE5A, 0x0000, 0x0000, 0x0002, /*1EE5A*/
  0x236F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE5B*/
  0xFBC3, 0x0020, 0x0002, 0xEE5C, 0x0000, 0x0000, 0x0002, /*1EE5C*/
  0x23A8, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE5D*/
  0xFBC3, 0x0020, 0x0002, 0xEE5E, 0x0000, 0x0000, 0x0002, /*1EE5E*/
  0x2381, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE5F*/
  0xFBC3, 0x0020, 0x0002, 0xEE60, 0x0000, 0x0000, 0x0002, /*1EE60*/
  0x230D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE61*/
  0x2325, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE62*/
  0xFBC3, 0x0020, 0x0002, 0xEE63, 0x0000, 0x0000, 0x0002, /*1EE63*/
  0x23B1, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE64*/
  0xFBC3, 0x0020, 0x0002, 0xEE65, 0x0000, 0x0000, 0x0002, /*1EE65*/
  0xFBC3, 0x0020, 0x0002, 0xEE66, 0x0000, 0x0000, 0x0002, /*1EE66*/
  0x232C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE67*/
  0x236A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE68*/
  0x23C6, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE69*/
  0x2387, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE6A*/
  0xFBC3, 0x0020, 0x0002, 0xEE6B, 0x0000, 0x0000, 0x0002, /*1EE6B*/
  0x23A3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE6C*/
  0x23A7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE6D*/
  0x2359, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE6E*/
  0x236E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE6F*/
  0x2376, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE70*/
  0x2364, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE71*/
  0x2382, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE72*/
  0xFBC3, 0x0020, 0x0002, 0xEE73, 0x0000, 0x0000, 0x0002, /*1EE73*/
  0x235A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE74*/
  0x231D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE75*/
  0x231E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE76*/
  0x232D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE77*/
  0xFBC3, 0x0020, 0x0002, 0xEE78, 0x0000, 0x0000, 0x0002, /*1EE78*/
  0x2365, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE79*/
  0x236B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE7A*/
  0x236F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE7B*/
  0x230C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE7C*/
  0xFBC3, 0x0020, 0x0002, 0xEE7D, 0x0000, 0x0000, 0x0002, /*1EE7D*/
  0x2377, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE7E*/
  0xFBC3, 0x0020, 0x0002, 0xEE7F, 0x0000, 0x0000, 0x0002, /*1EE7F*/
  0x230B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE80*/
  0x230D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE81*/
  0x2325, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE82*/
  0x2337, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE83*/
  0x23B1, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE84*/
  0x23B7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE85*/
  0x2347, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE86*/
  0x232C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE87*/
  0x236A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE88*/
  0x23C6, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE89*/
  0xFBC3, 0x0020, 0x0002, 0xEE8A, 0x0000, 0x0000, 0x0002, /*1EE8A*/
  0x239C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE8B*/
  0x23A3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE8C*/
  0x23A7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE8D*/
  0x2359, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE8E*/
  0x236E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE8F*/
  0x2376, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE90*/
  0x2364, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE91*/
  0x2382, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE92*/
  0x2346, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE93*/
  0x235A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE94*/
  0x231D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE95*/
  0x231E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE96*/
  0x232D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE97*/
  0x2338, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE98*/
  0x2365, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE99*/
  0x236B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE9A*/
  0x236F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EE9B*/
  0xFBC3, 0x0020, 0x0002, 0xEE9C, 0x0000, 0x0000, 0x0002, /*1EE9C*/
  0xFBC3, 0x0020, 0x0002, 0xEE9D, 0x0000, 0x0000, 0x0002, /*1EE9D*/
  0xFBC3, 0x0020, 0x0002, 0xEE9E, 0x0000, 0x0000, 0x0002, /*1EE9E*/
  0xFBC3, 0x0020, 0x0002, 0xEE9F, 0x0000, 0x0000, 0x0002, /*1EE9F*/
  0xFBC3, 0x0020, 0x0002, 0xEEA0, 0x0000, 0x0000, 0x0002, /*1EEA0*/
  0x230D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEA1*/
  0x2325, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEA2*/
  0x2337, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEA3*/
  0xFBC3, 0x0020, 0x0002, 0xEEA4, 0x0000, 0x0000, 0x0002, /*1EEA4*/
  0x23B7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEA5*/
  0x2347, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEA6*/
  0x232C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEA7*/
  0x236A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEA8*/
  0x23C6, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEA9*/
  0xFBC3, 0x0020, 0x0002, 0xEEAA, 0x0000, 0x0000, 0x0002, /*1EEAA*/
  0x239C, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEAB*/
  0x23A3, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEAC*/
  0x23A7, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEAD*/
  0x2359, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEAE*/
  0x236E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEAF*/
  0x2376, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEB0*/
  0x2364, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEB1*/
  0x2382, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEB2*/
  0x2346, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEB3*/
  0x235A, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEB4*/
  0x231D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEB5*/
  0x231E, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEB6*/
  0x232D, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEB7*/
  0x2338, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEB8*/
  0x2365, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEB9*/
  0x236B, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEBA*/
  0x236F, 0x0020, 0x0005, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEBB*/
  0xFBC3, 0x0020, 0x0002, 0xEEBC, 0x0000, 0x0000, 0x0002, /*1EEBC*/
  0xFBC3, 0x0020, 0x0002, 0xEEBD, 0x0000, 0x0000, 0x0002, /*1EEBD*/
  0xFBC3, 0x0020, 0x0002, 0xEEBE, 0x0000, 0x0000, 0x0002, /*1EEBE*/
  0xFBC3, 0x0020, 0x0002, 0xEEBF, 0x0000, 0x0000, 0x0002, /*1EEBF*/
  0xFBC3, 0x0020, 0x0002, 0xEEC0, 0x0000, 0x0000, 0x0002, /*1EEC0*/
  0xFBC3, 0x0020, 0x0002, 0xEEC1, 0x0000, 0x0000, 0x0002, /*1EEC1*/
  0xFBC3, 0x0020, 0x0002, 0xEEC2, 0x0000, 0x0000, 0x0002, /*1EEC2*/
  0xFBC3, 0x0020, 0x0002, 0xEEC3, 0x0000, 0x0000, 0x0002, /*1EEC3*/
  0xFBC3, 0x0020, 0x0002, 0xEEC4, 0x0000, 0x0000, 0x0002, /*1EEC4*/
  0xFBC3, 0x0020, 0x0002, 0xEEC5, 0x0000, 0x0000, 0x0002, /*1EEC5*/
  0xFBC3, 0x0020, 0x0002, 0xEEC6, 0x0000, 0x0000, 0x0002, /*1EEC6*/
  0xFBC3, 0x0020, 0x0002, 0xEEC7, 0x0000, 0x0000, 0x0002, /*1EEC7*/
  0xFBC3, 0x0020, 0x0002, 0xEEC8, 0x0000, 0x0000, 0x0002, /*1EEC8*/
  0xFBC3, 0x0020, 0x0002, 0xEEC9, 0x0000, 0x0000, 0x0002, /*1EEC9*/
  0xFBC3, 0x0020, 0x0002, 0xEECA, 0x0000, 0x0000, 0x0002, /*1EECA*/
  0xFBC3, 0x0020, 0x0002, 0xEECB, 0x0000, 0x0000, 0x0002, /*1EECB*/
  0xFBC3, 0x0020, 0x0002, 0xEECC, 0x0000, 0x0000, 0x0002, /*1EECC*/
  0xFBC3, 0x0020, 0x0002, 0xEECD, 0x0000, 0x0000, 0x0002, /*1EECD*/
  0xFBC3, 0x0020, 0x0002, 0xEECE, 0x0000, 0x0000, 0x0002, /*1EECE*/
  0xFBC3, 0x0020, 0x0002, 0xEECF, 0x0000, 0x0000, 0x0002, /*1EECF*/
  0xFBC3, 0x0020, 0x0002, 0xEED0, 0x0000, 0x0000, 0x0002, /*1EED0*/
  0xFBC3, 0x0020, 0x0002, 0xEED1, 0x0000, 0x0000, 0x0002, /*1EED1*/
  0xFBC3, 0x0020, 0x0002, 0xEED2, 0x0000, 0x0000, 0x0002, /*1EED2*/
  0xFBC3, 0x0020, 0x0002, 0xEED3, 0x0000, 0x0000, 0x0002, /*1EED3*/
  0xFBC3, 0x0020, 0x0002, 0xEED4, 0x0000, 0x0000, 0x0002, /*1EED4*/
  0xFBC3, 0x0020, 0x0002, 0xEED5, 0x0000, 0x0000, 0x0002, /*1EED5*/
  0xFBC3, 0x0020, 0x0002, 0xEED6, 0x0000, 0x0000, 0x0002, /*1EED6*/
  0xFBC3, 0x0020, 0x0002, 0xEED7, 0x0000, 0x0000, 0x0002, /*1EED7*/
  0xFBC3, 0x0020, 0x0002, 0xEED8, 0x0000, 0x0000, 0x0002, /*1EED8*/
  0xFBC3, 0x0020, 0x0002, 0xEED9, 0x0000, 0x0000, 0x0002, /*1EED9*/
  0xFBC3, 0x0020, 0x0002, 0xEEDA, 0x0000, 0x0000, 0x0002, /*1EEDA*/
  0xFBC3, 0x0020, 0x0002, 0xEEDB, 0x0000, 0x0000, 0x0002, /*1EEDB*/
  0xFBC3, 0x0020, 0x0002, 0xEEDC, 0x0000, 0x0000, 0x0002, /*1EEDC*/
  0xFBC3, 0x0020, 0x0002, 0xEEDD, 0x0000, 0x0000, 0x0002, /*1EEDD*/
  0xFBC3, 0x0020, 0x0002, 0xEEDE, 0x0000, 0x0000, 0x0002, /*1EEDE*/
  0xFBC3, 0x0020, 0x0002, 0xEEDF, 0x0000, 0x0000, 0x0002, /*1EEDF*/
  0xFBC3, 0x0020, 0x0002, 0xEEE0, 0x0000, 0x0000, 0x0002, /*1EEE0*/
  0xFBC3, 0x0020, 0x0002, 0xEEE1, 0x0000, 0x0000, 0x0002, /*1EEE1*/
  0xFBC3, 0x0020, 0x0002, 0xEEE2, 0x0000, 0x0000, 0x0002, /*1EEE2*/
  0xFBC3, 0x0020, 0x0002, 0xEEE3, 0x0000, 0x0000, 0x0002, /*1EEE3*/
  0xFBC3, 0x0020, 0x0002, 0xEEE4, 0x0000, 0x0000, 0x0002, /*1EEE4*/
  0xFBC3, 0x0020, 0x0002, 0xEEE5, 0x0000, 0x0000, 0x0002, /*1EEE5*/
  0xFBC3, 0x0020, 0x0002, 0xEEE6, 0x0000, 0x0000, 0x0002, /*1EEE6*/
  0xFBC3, 0x0020, 0x0002, 0xEEE7, 0x0000, 0x0000, 0x0002, /*1EEE7*/
  0xFBC3, 0x0020, 0x0002, 0xEEE8, 0x0000, 0x0000, 0x0002, /*1EEE8*/
  0xFBC3, 0x0020, 0x0002, 0xEEE9, 0x0000, 0x0000, 0x0002, /*1EEE9*/
  0xFBC3, 0x0020, 0x0002, 0xEEEA, 0x0000, 0x0000, 0x0002, /*1EEEA*/
  0xFBC3, 0x0020, 0x0002, 0xEEEB, 0x0000, 0x0000, 0x0002, /*1EEEB*/
  0xFBC3, 0x0020, 0x0002, 0xEEEC, 0x0000, 0x0000, 0x0002, /*1EEEC*/
  0xFBC3, 0x0020, 0x0002, 0xEEED, 0x0000, 0x0000, 0x0002, /*1EEED*/
  0xFBC3, 0x0020, 0x0002, 0xEEEE, 0x0000, 0x0000, 0x0002, /*1EEEE*/
  0xFBC3, 0x0020, 0x0002, 0xEEEF, 0x0000, 0x0000, 0x0002, /*1EEEF*/
  0x04FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEF0*/
  0x04FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1EEF1*/
  0xFBC3, 0x0020, 0x0002, 0xEEF2, 0x0000, 0x0000, 0x0002, /*1EEF2*/
  0xFBC3, 0x0020, 0x0002, 0xEEF3, 0x0000, 0x0000, 0x0002, /*1EEF3*/
  0xFBC3, 0x0020, 0x0002, 0xEEF4, 0x0000, 0x0000, 0x0002, /*1EEF4*/
  0xFBC3, 0x0020, 0x0002, 0xEEF5, 0x0000, 0x0000, 0x0002, /*1EEF5*/
  0xFBC3, 0x0020, 0x0002, 0xEEF6, 0x0000, 0x0000, 0x0002, /*1EEF6*/
  0xFBC3, 0x0020, 0x0002, 0xEEF7, 0x0000, 0x0000, 0x0002, /*1EEF7*/
  0xFBC3, 0x0020, 0x0002, 0xEEF8, 0x0000, 0x0000, 0x0002, /*1EEF8*/
  0xFBC3, 0x0020, 0x0002, 0xEEF9, 0x0000, 0x0000, 0x0002, /*1EEF9*/
  0xFBC3, 0x0020, 0x0002, 0xEEFA, 0x0000, 0x0000, 0x0002, /*1EEFA*/
  0xFBC3, 0x0020, 0x0002, 0xEEFB, 0x0000, 0x0000, 0x0002, /*1EEFB*/
  0xFBC3, 0x0020, 0x0002, 0xEEFC, 0x0000, 0x0000, 0x0002, /*1EEFC*/
  0xFBC3, 0x0020, 0x0002, 0xEEFD, 0x0000, 0x0000, 0x0002, /*1EEFD*/
  0xFBC3, 0x0020, 0x0002, 0xEEFE, 0x0000, 0x0000, 0x0002, /*1EEFE*/
  0xFBC3, 0x0020, 0x0002, 0xEEFF, 0x0000, 0x0000, 0x0002  /*1EEFF*/
};

uint16 uca900_p1F0[]= {
  0x11C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F000*/
  0x11C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F001*/
  0x11C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F002*/
  0x11CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F003*/
  0x11CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F004*/
  0x11CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F005*/
  0x11CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F006*/
  0x11CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F007*/
  0x11CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F008*/
  0x11D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F009*/
  0x11D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F00A*/
  0x11D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F00B*/
  0x11D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F00C*/
  0x11D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F00D*/
  0x11D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F00E*/
  0x11D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F00F*/
  0x11D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F010*/
  0x11D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F011*/
  0x11D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F012*/
  0x11DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F013*/
  0x11DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F014*/
  0x11DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F015*/
  0x11DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F016*/
  0x11DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F017*/
  0x11DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F018*/
  0x11E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F019*/
  0x11E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F01A*/
  0x11E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F01B*/
  0x11E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F01C*/
  0x11E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F01D*/
  0x11E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F01E*/
  0x11E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F01F*/
  0x11E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F020*/
  0x11E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F021*/
  0x11E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F022*/
  0x11EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F023*/
  0x11EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F024*/
  0x11EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F025*/
  0x11ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F026*/
  0x11EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F027*/
  0x11EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F028*/
  0x11F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F029*/
  0x11F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F02A*/
  0x11F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F02B*/
  0xFBC3, 0x0020, 0x0002, 0xF02C, 0x0000, 0x0000, 0x0002, /*1F02C*/
  0xFBC3, 0x0020, 0x0002, 0xF02D, 0x0000, 0x0000, 0x0002, /*1F02D*/
  0xFBC3, 0x0020, 0x0002, 0xF02E, 0x0000, 0x0000, 0x0002, /*1F02E*/
  0xFBC3, 0x0020, 0x0002, 0xF02F, 0x0000, 0x0000, 0x0002, /*1F02F*/
  0x11F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F030*/
  0x11F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F031*/
  0x11F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F032*/
  0x11F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F033*/
  0x11F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F034*/
  0x11F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F035*/
  0x11F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F036*/
  0x11FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F037*/
  0x11FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F038*/
  0x11FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F039*/
  0x11FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F03A*/
  0x11FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F03B*/
  0x11FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F03C*/
  0x1200, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F03D*/
  0x1201, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F03E*/
  0x1202, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F03F*/
  0x1203, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F040*/
  0x1204, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F041*/
  0x1205, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F042*/
  0x1206, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F043*/
  0x1207, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F044*/
  0x1208, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F045*/
  0x1209, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F046*/
  0x120A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F047*/
  0x120B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F048*/
  0x120C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F049*/
  0x120D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F04A*/
  0x120E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F04B*/
  0x120F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F04C*/
  0x1210, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F04D*/
  0x1211, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F04E*/
  0x1212, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F04F*/
  0x1213, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F050*/
  0x1214, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F051*/
  0x1215, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F052*/
  0x1216, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F053*/
  0x1217, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F054*/
  0x1218, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F055*/
  0x1219, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F056*/
  0x121A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F057*/
  0x121B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F058*/
  0x121C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F059*/
  0x121D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F05A*/
  0x121E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F05B*/
  0x121F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F05C*/
  0x1220, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F05D*/
  0x1221, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F05E*/
  0x1222, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F05F*/
  0x1223, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F060*/
  0x1224, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F061*/
  0x1225, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F062*/
  0x1226, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F063*/
  0x1227, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F064*/
  0x1228, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F065*/
  0x1229, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F066*/
  0x122A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F067*/
  0x122B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F068*/
  0x122C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F069*/
  0x122D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F06A*/
  0x122E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F06B*/
  0x122F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F06C*/
  0x1230, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F06D*/
  0x1231, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F06E*/
  0x1232, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F06F*/
  0x1233, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F070*/
  0x1234, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F071*/
  0x1235, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F072*/
  0x1236, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F073*/
  0x1237, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F074*/
  0x1238, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F075*/
  0x1239, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F076*/
  0x123A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F077*/
  0x123B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F078*/
  0x123C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F079*/
  0x123D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F07A*/
  0x123E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F07B*/
  0x123F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F07C*/
  0x1240, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F07D*/
  0x1241, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F07E*/
  0x1242, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F07F*/
  0x1243, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F080*/
  0x1244, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F081*/
  0x1245, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F082*/
  0x1246, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F083*/
  0x1247, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F084*/
  0x1248, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F085*/
  0x1249, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F086*/
  0x124A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F087*/
  0x124B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F088*/
  0x124C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F089*/
  0x124D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F08A*/
  0x124E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F08B*/
  0x124F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F08C*/
  0x1250, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F08D*/
  0x1251, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F08E*/
  0x1252, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F08F*/
  0x1253, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F090*/
  0x1254, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F091*/
  0x1255, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F092*/
  0x1256, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F093*/
  0xFBC3, 0x0020, 0x0002, 0xF094, 0x0000, 0x0000, 0x0002, /*1F094*/
  0xFBC3, 0x0020, 0x0002, 0xF095, 0x0000, 0x0000, 0x0002, /*1F095*/
  0xFBC3, 0x0020, 0x0002, 0xF096, 0x0000, 0x0000, 0x0002, /*1F096*/
  0xFBC3, 0x0020, 0x0002, 0xF097, 0x0000, 0x0000, 0x0002, /*1F097*/
  0xFBC3, 0x0020, 0x0002, 0xF098, 0x0000, 0x0000, 0x0002, /*1F098*/
  0xFBC3, 0x0020, 0x0002, 0xF099, 0x0000, 0x0000, 0x0002, /*1F099*/
  0xFBC3, 0x0020, 0x0002, 0xF09A, 0x0000, 0x0000, 0x0002, /*1F09A*/
  0xFBC3, 0x0020, 0x0002, 0xF09B, 0x0000, 0x0000, 0x0002, /*1F09B*/
  0xFBC3, 0x0020, 0x0002, 0xF09C, 0x0000, 0x0000, 0x0002, /*1F09C*/
  0xFBC3, 0x0020, 0x0002, 0xF09D, 0x0000, 0x0000, 0x0002, /*1F09D*/
  0xFBC3, 0x0020, 0x0002, 0xF09E, 0x0000, 0x0000, 0x0002, /*1F09E*/
  0xFBC3, 0x0020, 0x0002, 0xF09F, 0x0000, 0x0000, 0x0002, /*1F09F*/
  0x1257, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0A0*/
  0x1258, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0A1*/
  0x1259, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0A2*/
  0x125A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0A3*/
  0x125B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0A4*/
  0x125C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0A5*/
  0x125D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0A6*/
  0x125E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0A7*/
  0x125F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0A8*/
  0x1260, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0A9*/
  0x1261, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0AA*/
  0x1262, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0AB*/
  0x1263, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0AC*/
  0x1264, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0AD*/
  0x1265, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0AE*/
  0xFBC3, 0x0020, 0x0002, 0xF0AF, 0x0000, 0x0000, 0x0002, /*1F0AF*/
  0xFBC3, 0x0020, 0x0002, 0xF0B0, 0x0000, 0x0000, 0x0002, /*1F0B0*/
  0x1266, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0B1*/
  0x1267, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0B2*/
  0x1268, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0B3*/
  0x1269, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0B4*/
  0x126A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0B5*/
  0x126B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0B6*/
  0x126C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0B7*/
  0x126D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0B8*/
  0x126E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0B9*/
  0x126F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0BA*/
  0x1270, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0BB*/
  0x1271, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0BC*/
  0x1272, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0BD*/
  0x1273, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0BE*/
  0x1274, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0BF*/
  0xFBC3, 0x0020, 0x0002, 0xF0C0, 0x0000, 0x0000, 0x0002, /*1F0C0*/
  0x1275, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0C1*/
  0x1276, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0C2*/
  0x1277, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0C3*/
  0x1278, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0C4*/
  0x1279, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0C5*/
  0x127A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0C6*/
  0x127B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0C7*/
  0x127C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0C8*/
  0x127D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0C9*/
  0x127E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0CA*/
  0x127F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0CB*/
  0x1280, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0CC*/
  0x1281, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0CD*/
  0x1282, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0CE*/
  0x1283, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0CF*/
  0xFBC3, 0x0020, 0x0002, 0xF0D0, 0x0000, 0x0000, 0x0002, /*1F0D0*/
  0x1284, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0D1*/
  0x1285, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0D2*/
  0x1286, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0D3*/
  0x1287, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0D4*/
  0x1288, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0D5*/
  0x1289, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0D6*/
  0x128A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0D7*/
  0x128B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0D8*/
  0x128C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0D9*/
  0x128D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0DA*/
  0x128E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0DB*/
  0x128F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0DC*/
  0x1290, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0DD*/
  0x1291, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0DE*/
  0x1292, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0DF*/
  0x1293, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0E0*/
  0x1294, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0E1*/
  0x1295, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0E2*/
  0x1296, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0E3*/
  0x1297, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0E4*/
  0x1298, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0E5*/
  0x1299, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0E6*/
  0x129A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0E7*/
  0x129B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0E8*/
  0x129C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0E9*/
  0x129D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0EA*/
  0x129E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0EB*/
  0x129F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0EC*/
  0x12A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0ED*/
  0x12A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0EE*/
  0x12A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0EF*/
  0x12A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0F0*/
  0x12A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0F1*/
  0x12A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0F2*/
  0x12A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0F3*/
  0x12A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0F4*/
  0x12A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F0F5*/
  0xFBC3, 0x0020, 0x0002, 0xF0F6, 0x0000, 0x0000, 0x0002, /*1F0F6*/
  0xFBC3, 0x0020, 0x0002, 0xF0F7, 0x0000, 0x0000, 0x0002, /*1F0F7*/
  0xFBC3, 0x0020, 0x0002, 0xF0F8, 0x0000, 0x0000, 0x0002, /*1F0F8*/
  0xFBC3, 0x0020, 0x0002, 0xF0F9, 0x0000, 0x0000, 0x0002, /*1F0F9*/
  0xFBC3, 0x0020, 0x0002, 0xF0FA, 0x0000, 0x0000, 0x0002, /*1F0FA*/
  0xFBC3, 0x0020, 0x0002, 0xF0FB, 0x0000, 0x0000, 0x0002, /*1F0FB*/
  0xFBC3, 0x0020, 0x0002, 0xF0FC, 0x0000, 0x0000, 0x0002, /*1F0FC*/
  0xFBC3, 0x0020, 0x0002, 0xF0FD, 0x0000, 0x0000, 0x0002, /*1F0FD*/
  0xFBC3, 0x0020, 0x0002, 0xF0FE, 0x0000, 0x0000, 0x0002, /*1F0FE*/
  0xFBC3, 0x0020, 0x0002, 0xF0FF, 0x0000, 0x0000, 0x0002  /*1F0FF*/
};

uint16 uca900_p1F1[]= {
  0x1C3D, 0x0020, 0x0004, 0x0277, 0x0020, 0x0004, 0x0000, /*1F100*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3D, 0x0020, 0x0004, 0x0222, 0x0020, 0x0004, 0x0000, /*1F101*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3E, 0x0020, 0x0004, 0x0222, 0x0020, 0x0004, 0x0000, /*1F102*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x0004, 0x0222, 0x0020, 0x0004, 0x0000, /*1F103*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x0004, 0x0222, 0x0020, 0x0004, 0x0000, /*1F104*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x0004, 0x0222, 0x0020, 0x0004, 0x0000, /*1F105*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C42, 0x0020, 0x0004, 0x0222, 0x0020, 0x0004, 0x0000, /*1F106*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C43, 0x0020, 0x0004, 0x0222, 0x0020, 0x0004, 0x0000, /*1F107*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C44, 0x0020, 0x0004, 0x0222, 0x0020, 0x0004, 0x0000, /*1F108*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C45, 0x0020, 0x0004, 0x0222, 0x0020, 0x0004, 0x0000, /*1F109*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C46, 0x0020, 0x0004, 0x0222, 0x0020, 0x0004, 0x0000, /*1F10A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*1F10B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C3D, 0x0020, 0x0006, 0x0000, 0x0000, 0x0000, 0x0000, /*1F10C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC3, 0x0020, 0x0002, 0xF10D, 0x0000, 0x0000, 0x0000, /*1F10D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF10E, 0x0000, 0x0000, 0x0000, /*1F10E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF10F, 0x0000, 0x0000, 0x0000, /*1F10F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0317, 0x0020, 0x0004, 0x1C47, 0x0020, 0x000A, 0x0318, /*1F110*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C60, 0x0020, 0x000A, 0x0318, /*1F111*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C7A, 0x0020, 0x000A, 0x0318, /*1F112*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1C8F, 0x0020, 0x000A, 0x0318, /*1F113*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1CAA, 0x0020, 0x000A, 0x0318, /*1F114*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1CE5, 0x0020, 0x000A, 0x0318, /*1F115*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1CF4, 0x0020, 0x000A, 0x0318, /*1F116*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1D18, 0x0020, 0x000A, 0x0318, /*1F117*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1D32, 0x0020, 0x000A, 0x0318, /*1F118*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1D4C, 0x0020, 0x000A, 0x0318, /*1F119*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1D65, 0x0020, 0x000A, 0x0318, /*1F11A*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1D77, 0x0020, 0x000A, 0x0318, /*1F11B*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1DAA, 0x0020, 0x000A, 0x0318, /*1F11C*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1DB9, 0x0020, 0x000A, 0x0318, /*1F11D*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1DDD, 0x0020, 0x000A, 0x0318, /*1F11E*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1E0C, 0x0020, 0x000A, 0x0318, /*1F11F*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1E21, 0x0020, 0x000A, 0x0318, /*1F120*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1E33, 0x0020, 0x000A, 0x0318, /*1F121*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1E71, 0x0020, 0x000A, 0x0318, /*1F122*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1E95, 0x0020, 0x000A, 0x0318, /*1F123*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1EB5, 0x0020, 0x000A, 0x0318, /*1F124*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1EE3, 0x0020, 0x000A, 0x0318, /*1F125*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1EF5, 0x0020, 0x000A, 0x0318, /*1F126*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1EFF, 0x0020, 0x000A, 0x0318, /*1F127*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1F0B, 0x0020, 0x000A, 0x0318, /*1F128*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0317, 0x0020, 0x0004, 0x1F21, 0x0020, 0x000A, 0x0318, /*1F129*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x0379, 0x0020, 0x0004, 0x1E71, 0x0020, 0x000A, 0x037A, /*1F12A*/
  0x0020, 0x0004, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C7A, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F12B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F12C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x000C, 0x1C8F, 0x0020, 0x000C, 0x0000, /*1F12D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x000C, 0x1F21, 0x0020, 0x000C, 0x0000, /*1F12E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF12F, 0x0000, 0x0000, 0x0000, /*1F12F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F130*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C60, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F131*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F132*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F133*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F134*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE5, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F135*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF4, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F136*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F137*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F138*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D4C, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F139*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D65, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F13A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F13B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F13C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F13D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F13E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0C, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F13F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E21, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F140*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F141*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F142*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F143*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F144*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F145*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF5, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F146*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFF, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F147*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F0B, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F148*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F21, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F149*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x001D, 0x1EE3, 0x0020, 0x001D, 0x0000, /*1F14A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x001D, 0x1EE3, 0x0020, 0x001D, 0x0000, /*1F14B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x001D, 0x1C8F, 0x0020, 0x001D, 0x0000, /*1F14C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x001D, 0x1E71, 0x0020, 0x001D, 0x0000, /*1F14D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E0C, 0x0020, 0x001D, 0x1E0C, 0x0020, 0x001D, 0x1EE3, /*1F14E*/
  0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EF5, 0x0020, 0x001D, 0x1C7A, 0x0020, 0x001D, 0x0000, /*1F14F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F150*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C60, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F151*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F152*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F153*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F154*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE5, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F155*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF4, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F156*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F157*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F158*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D4C, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F159*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D65, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F15A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F15B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F15C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F15D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F15E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0C, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F15F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E21, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F160*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F161*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F162*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F163*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F164*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F165*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF5, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F166*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFF, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F167*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F0B, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F168*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F21, 0x0020, 0x000C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F169*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x0014, 0x1C7A, 0x0020, 0x0014, 0x0000, /*1F16A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DAA, 0x0020, 0x0014, 0x1C8F, 0x0020, 0x0014, 0x0000, /*1F16B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF16C, 0x0000, 0x0000, 0x0000, /*1F16C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF16D, 0x0000, 0x0000, 0x0000, /*1F16D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF16E, 0x0000, 0x0000, 0x0000, /*1F16E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF16F, 0x0000, 0x0000, 0x0000, /*1F16F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F170*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C60, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F171*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C7A, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F172*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1C8F, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F173*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CAA, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F174*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CE5, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F175*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1CF4, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F176*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F177*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F178*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D4C, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F179*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D65, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F17A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D77, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F17B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DAA, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F17C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DB9, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F17D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1DDD, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F17E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0C, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F17F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E21, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F180*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E33, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F181*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E71, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F182*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E95, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F183*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EB5, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F184*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EE3, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F185*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EF5, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F186*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1EFF, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F187*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F0B, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F188*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1F21, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F189*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1E0C, 0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000, /*1F18A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D32, 0x0020, 0x001D, 0x1C7A, 0x0020, 0x001D, 0x0000, /*1F18B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E0C, 0x0020, 0x001D, 0x1C47, 0x0020, 0x001D, 0x0000, /*1F18C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x001D, 0x1C47, 0x0020, 0x001D, 0x0000, /*1F18D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C47, 0x0020, 0x001D, 0x1C60, 0x0020, 0x001D, 0x0000, /*1F18E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1EF5, 0x0020, 0x001D, 0x1C7A, 0x0020, 0x001D, 0x0000, /*1F18F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C8F, 0x0020, 0x001D, 0x1D4C, 0x0020, 0x001D, 0x0000, /*1F190*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x001D, 0x1D77, 0x0020, 0x001D, 0x0000, /*1F191*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C7A, 0x0020, 0x001D, 0x1DDD, 0x0020, 0x001D, 0x1DDD, /*1F192*/
  0x0020, 0x001D, 0x1D77, 0x0020, 0x001D, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1CE5, 0x0020, 0x001D, 0x1E33, 0x0020, 0x001D, 0x1CAA, /*1F193*/
  0x0020, 0x001D, 0x1CAA, 0x0020, 0x001D, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1D32, 0x0020, 0x001D, 0x1C8F, 0x0020, 0x001D, 0x0000, /*1F194*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DB9, 0x0020, 0x001D, 0x1CAA, 0x0020, 0x001D, 0x1EF5, /*1F195*/
  0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1DB9, 0x0020, 0x001D, 0x1CF4, 0x0020, 0x001D, 0x0000, /*1F196*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1DDD, 0x0020, 0x001D, 0x1D65, 0x0020, 0x001D, 0x0000, /*1F197*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1E71, 0x0020, 0x001D, 0x1DDD, 0x0020, 0x001D, 0x1E71, /*1F198*/
  0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EB5, 0x0020, 0x001D, 0x1E0C, 0x0020, 0x001D, 0x0260, /*1F199*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EE3, 0x0020, 0x001D, 0x1E71, 0x0020, 0x001D, 0x0000, /*1F19A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C40, 0x0020, 0x001C, 0x1C8F, 0x0020, 0x001D, 0x0000, /*1F19B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C3F, 0x0020, 0x001C, 0x1DB9, 0x0020, 0x001C, 0x1C8F, /*1F19C*/
  0x0020, 0x001C, 0x0209, 0x0020, 0x001C, 0x1E71,
  0x0020, 0x001D, 0x1C7A, 0x0020, 0x001C, 0x1E33,
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0007,
  0x1C3F, 0x0020, 0x001C, 0x1D65, 0x0020, 0x001D, 0x0000, /*1F19D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C41, 0x0020, 0x001C, 0x1D65, 0x0020, 0x001D, 0x0000, /*1F19E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C45, 0x0020, 0x001C, 0x1D65, 0x0020, 0x001D, 0x0000, /*1F19F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1C42, 0x0020, 0x001C, 0x0277, 0x0020, 0x001C, 0x1C3E, /*1F1A0*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C44, 0x0020, 0x001C, 0x0277, 0x0020, 0x001C, 0x1C3E, /*1F1A1*/
  0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3F, 0x0020, 0x001C, 0x1C3F, 0x0020, 0x001C, 0x0277, /*1F1A2*/
  0x0020, 0x001C, 0x1C3F, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C43, 0x0020, 0x001C, 0x1C3D, 0x0020, 0x001C, 0x1E0C, /*1F1A3*/
  0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1C3E, 0x0020, 0x001C, 0x1C3F, 0x0020, 0x001C, 0x1C3D, /*1F1A4*/
  0x0020, 0x001C, 0x1E0C, 0x0020, 0x001D, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0004,
  0x1C8F, 0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x1D18, 0x0020, 0x001D, 0x1C7A, 0x0020, 0x001C, 0x0000, /*1F1A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x1D18, 0x0020, 0x001D, 0x1C8F, 0x0020, 0x001C, 0x1E33, /*1F1A7*/
  0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1D18, 0x0020, 0x001D, 0x1D32, 0x0020, 0x001C, 0x0209, /*1F1A8*/
  0x0020, 0x001C, 0x1E33, 0x0020, 0x001D, 0x1CAA,
  0x0020, 0x001C, 0x1E71, 0x0020, 0x001C, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0006,
  0x1D77, 0x0020, 0x001D, 0x1DDD, 0x0020, 0x001C, 0x1E71, /*1F1A9*/
  0x0020, 0x001C, 0x1E71, 0x0020, 0x001C, 0x1D77,
  0x0020, 0x001C, 0x1CAA, 0x0020, 0x001C, 0x1E71,
  0x0020, 0x001C, 0x1E71, 0x0020, 0x001C, 0x0008,
  0x1E71, 0x0020, 0x001D, 0x1D18, 0x0020, 0x001D, 0x1EE3, /*1F1AA*/
  0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EB5, 0x0020, 0x001D, 0x1D18, 0x0020, 0x001D, 0x1C8F, /*1F1AB*/
  0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0x1EE3, 0x0020, 0x001D, 0x1DDD, 0x0020, 0x001D, 0x1C8F, /*1F1AC*/
  0x0020, 0x001D, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0003,
  0xFBC3, 0x0020, 0x0002, 0xF1AD, 0x0000, 0x0000, 0x0000, /*1F1AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1AE, 0x0000, 0x0000, 0x0000, /*1F1AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1AF, 0x0000, 0x0000, 0x0000, /*1F1AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1B0, 0x0000, 0x0000, 0x0000, /*1F1B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1B1, 0x0000, 0x0000, 0x0000, /*1F1B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1B2, 0x0000, 0x0000, 0x0000, /*1F1B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1B3, 0x0000, 0x0000, 0x0000, /*1F1B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1B4, 0x0000, 0x0000, 0x0000, /*1F1B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1B5, 0x0000, 0x0000, 0x0000, /*1F1B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1B6, 0x0000, 0x0000, 0x0000, /*1F1B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1B7, 0x0000, 0x0000, 0x0000, /*1F1B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1B8, 0x0000, 0x0000, 0x0000, /*1F1B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1B9, 0x0000, 0x0000, 0x0000, /*1F1B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1BA, 0x0000, 0x0000, 0x0000, /*1F1BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1BB, 0x0000, 0x0000, 0x0000, /*1F1BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1BC, 0x0000, 0x0000, 0x0000, /*1F1BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1BD, 0x0000, 0x0000, 0x0000, /*1F1BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1BE, 0x0000, 0x0000, 0x0000, /*1F1BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1BF, 0x0000, 0x0000, 0x0000, /*1F1BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1C0, 0x0000, 0x0000, 0x0000, /*1F1C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1C1, 0x0000, 0x0000, 0x0000, /*1F1C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1C2, 0x0000, 0x0000, 0x0000, /*1F1C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1C3, 0x0000, 0x0000, 0x0000, /*1F1C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1C4, 0x0000, 0x0000, 0x0000, /*1F1C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1C5, 0x0000, 0x0000, 0x0000, /*1F1C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1C6, 0x0000, 0x0000, 0x0000, /*1F1C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1C7, 0x0000, 0x0000, 0x0000, /*1F1C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1C8, 0x0000, 0x0000, 0x0000, /*1F1C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1C9, 0x0000, 0x0000, 0x0000, /*1F1C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1CA, 0x0000, 0x0000, 0x0000, /*1F1CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1CB, 0x0000, 0x0000, 0x0000, /*1F1CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1CC, 0x0000, 0x0000, 0x0000, /*1F1CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1CD, 0x0000, 0x0000, 0x0000, /*1F1CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1CE, 0x0000, 0x0000, 0x0000, /*1F1CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1CF, 0x0000, 0x0000, 0x0000, /*1F1CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1D0, 0x0000, 0x0000, 0x0000, /*1F1D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1D1, 0x0000, 0x0000, 0x0000, /*1F1D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1D2, 0x0000, 0x0000, 0x0000, /*1F1D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1D3, 0x0000, 0x0000, 0x0000, /*1F1D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1D4, 0x0000, 0x0000, 0x0000, /*1F1D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1D5, 0x0000, 0x0000, 0x0000, /*1F1D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1D6, 0x0000, 0x0000, 0x0000, /*1F1D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1D7, 0x0000, 0x0000, 0x0000, /*1F1D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1D8, 0x0000, 0x0000, 0x0000, /*1F1D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1D9, 0x0000, 0x0000, 0x0000, /*1F1D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1DA, 0x0000, 0x0000, 0x0000, /*1F1DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1DB, 0x0000, 0x0000, 0x0000, /*1F1DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1DC, 0x0000, 0x0000, 0x0000, /*1F1DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1DD, 0x0000, 0x0000, 0x0000, /*1F1DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1DE, 0x0000, 0x0000, 0x0000, /*1F1DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1DF, 0x0000, 0x0000, 0x0000, /*1F1DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1E0, 0x0000, 0x0000, 0x0000, /*1F1E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1E1, 0x0000, 0x0000, 0x0000, /*1F1E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1E2, 0x0000, 0x0000, 0x0000, /*1F1E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1E3, 0x0000, 0x0000, 0x0000, /*1F1E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1E4, 0x0000, 0x0000, 0x0000, /*1F1E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF1E5, 0x0000, 0x0000, 0x0000, /*1F1E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0A07, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A08, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A09, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A0A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A0B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A0C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A0D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A0E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A0F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A10, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A11, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A12, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A13, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A14, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A15, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A16, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A17, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A18, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A19, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A1A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A1B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A1C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A1D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A1E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A1F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0x0A20, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0000, /*1F1FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001
};

uint16 uca900_p1F2[]= {
  0x3D78, 0x0020, 0x001C, 0x3D60, 0x0020, 0x001C, 0x0000, /*1F200*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3D64, 0x0020, 0x001C, 0x3D64, 0x0020, 0x001C, 0x0000, /*1F201*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3D65, 0x0020, 0x001C, 0x0000, 0x0000, 0x0000, 0x0000, /*1F202*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001,
  0xFBC3, 0x0020, 0x0002, 0xF203, 0x0000, 0x0000, 0x0000, /*1F203*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF204, 0x0000, 0x0000, 0x0000, /*1F204*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF205, 0x0000, 0x0000, 0x0000, /*1F205*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF206, 0x0000, 0x0000, 0x0000, /*1F206*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF207, 0x0000, 0x0000, 0x0000, /*1F207*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF208, 0x0000, 0x0000, 0x0000, /*1F208*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF209, 0x0000, 0x0000, 0x0000, /*1F209*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF20A, 0x0000, 0x0000, 0x0000, /*1F20A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF20B, 0x0000, 0x0000, 0x0000, /*1F20B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF20C, 0x0000, 0x0000, 0x0000, /*1F20C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF20D, 0x0000, 0x0000, 0x0000, /*1F20D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF20E, 0x0000, 0x0000, 0x0000, /*1F20E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF20F, 0x0000, 0x0000, 0x0000, /*1F20F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xE24B, 0x0000, 0x0000, 0x0000, /*1F210*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xDB57, 0x0000, 0x0000, 0x0000, /*1F211*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xD3CC, 0x0000, 0x0000, 0x0000, /*1F212*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x3D6D, 0x0020, 0x001C, 0x0000, 0x0037, 0x001C, 0x0000, /*1F213*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xCE8C, 0x0000, 0x0000, 0x0000, /*1F214*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xD91A, 0x0000, 0x0000, 0x0000, /*1F215*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x001C, 0x89E3, 0x0000, 0x0000, 0x0000, /*1F216*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xD929, 0x0000, 0x0000, 0x0000, /*1F217*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xCEA4, 0x0000, 0x0000, 0x0000, /*1F218*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xE620, 0x0000, 0x0000, 0x0000, /*1F219*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xF121, 0x0000, 0x0000, 0x0000, /*1F21A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xE599, 0x0000, 0x0000, 0x0000, /*1F21B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xD24D, 0x0000, 0x0000, 0x0000, /*1F21C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xDF8C, 0x0000, 0x0000, 0x0000, /*1F21D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xD18D, 0x0000, 0x0000, 0x0000, /*1F21E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xE5B0, 0x0000, 0x0000, 0x0000, /*1F21F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xD21D, 0x0000, 0x0000, 0x0000, /*1F220*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xFD42, 0x0000, 0x0000, 0x0000, /*1F221*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xF51F, 0x0000, 0x0000, 0x0000, /*1F222*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x001C, 0x8CA9, 0x0000, 0x0000, 0x0000, /*1F223*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xD8F0, 0x0000, 0x0000, 0x0000, /*1F224*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xD439, 0x0000, 0x0000, 0x0000, /*1F225*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xEF14, 0x0000, 0x0000, 0x0000, /*1F226*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xE295, 0x0000, 0x0000, 0x0000, /*1F227*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xE355, 0x0000, 0x0000, 0x0000, /*1F228*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xCE00, 0x0000, 0x0000, 0x0000, /*1F229*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xCE09, 0x0000, 0x0000, 0x0000, /*1F22A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x001C, 0x904A, 0x0000, 0x0000, 0x0000, /*1F22B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xDDE6, 0x0000, 0x0000, 0x0000, /*1F22C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xCE2D, 0x0000, 0x0000, 0x0000, /*1F22D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xD3F3, 0x0000, 0x0000, 0x0000, /*1F22E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xE307, 0x0000, 0x0000, 0x0000, /*1F22F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x001C, 0x8D70, 0x0000, 0x0000, 0x0000, /*1F230*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xE253, 0x0000, 0x0000, 0x0000, /*1F231*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xF981, 0x0000, 0x0000, 0x0000, /*1F232*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xFA7A, 0x0000, 0x0000, 0x0000, /*1F233*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xD408, 0x0000, 0x0000, 0x0000, /*1F234*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xEE80, 0x0000, 0x0000, 0x0000, /*1F235*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xE709, 0x0000, 0x0000, 0x0000, /*1F236*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xE708, 0x0000, 0x0000, 0x0000, /*1F237*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xF533, 0x0000, 0x0000, 0x0000, /*1F238*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xD272, 0x0000, 0x0000, 0x0000, /*1F239*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x001C, 0xD5B6, 0x0000, 0x0000, 0x0000, /*1F23A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB41, 0x0020, 0x001C, 0x914D, 0x0000, 0x0000, 0x0000, /*1F23B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF23C, 0x0000, 0x0000, 0x0000, /*1F23C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF23D, 0x0000, 0x0000, 0x0000, /*1F23D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF23E, 0x0000, 0x0000, 0x0000, /*1F23E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF23F, 0x0000, 0x0000, 0x0000, /*1F23F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0x0379, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE72C, /*1F240*/
  0x0000, 0x0000, 0x037A, 0x0020, 0x0004, 0x0004,
  0x0379, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xCE09, /*1F241*/
  0x0000, 0x0000, 0x037A, 0x0020, 0x0004, 0x0004,
  0x0379, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xCE8C, /*1F242*/
  0x0000, 0x0000, 0x037A, 0x0020, 0x0004, 0x0004,
  0x0379, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xDB89, /*1F243*/
  0x0000, 0x0000, 0x037A, 0x0020, 0x0004, 0x0004,
  0x0379, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF0B9, /*1F244*/
  0x0000, 0x0000, 0x037A, 0x0020, 0x0004, 0x0004,
  0x0379, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE253, /*1F245*/
  0x0000, 0x0000, 0x037A, 0x0020, 0x0004, 0x0004,
  0x0379, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xF6D7, /*1F246*/
  0x0000, 0x0000, 0x037A, 0x0020, 0x0004, 0x0004,
  0x0379, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xD2DD, /*1F247*/
  0x0000, 0x0000, 0x037A, 0x0020, 0x0004, 0x0004,
  0x0379, 0x0020, 0x0004, 0xFB40, 0x0020, 0x0004, 0xE557, /*1F248*/
  0x0000, 0x0000, 0x037A, 0x0020, 0x0004, 0x0004,
  0xFBC3, 0x0020, 0x0002, 0xF249, 0x0000, 0x0000, 0x0000, /*1F249*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF24A, 0x0000, 0x0000, 0x0000, /*1F24A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF24B, 0x0000, 0x0000, 0x0000, /*1F24B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF24C, 0x0000, 0x0000, 0x0000, /*1F24C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF24D, 0x0000, 0x0000, 0x0000, /*1F24D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF24E, 0x0000, 0x0000, 0x0000, /*1F24E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF24F, 0x0000, 0x0000, 0x0000, /*1F24F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xDF97, 0x0000, 0x0000, 0x0000, /*1F250*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFB40, 0x0020, 0x0006, 0xD3EF, 0x0000, 0x0000, 0x0000, /*1F251*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF252, 0x0000, 0x0000, 0x0000, /*1F252*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF253, 0x0000, 0x0000, 0x0000, /*1F253*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF254, 0x0000, 0x0000, 0x0000, /*1F254*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF255, 0x0000, 0x0000, 0x0000, /*1F255*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF256, 0x0000, 0x0000, 0x0000, /*1F256*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF257, 0x0000, 0x0000, 0x0000, /*1F257*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF258, 0x0000, 0x0000, 0x0000, /*1F258*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF259, 0x0000, 0x0000, 0x0000, /*1F259*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF25A, 0x0000, 0x0000, 0x0000, /*1F25A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF25B, 0x0000, 0x0000, 0x0000, /*1F25B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF25C, 0x0000, 0x0000, 0x0000, /*1F25C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF25D, 0x0000, 0x0000, 0x0000, /*1F25D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF25E, 0x0000, 0x0000, 0x0000, /*1F25E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF25F, 0x0000, 0x0000, 0x0000, /*1F25F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF260, 0x0000, 0x0000, 0x0000, /*1F260*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF261, 0x0000, 0x0000, 0x0000, /*1F261*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF262, 0x0000, 0x0000, 0x0000, /*1F262*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF263, 0x0000, 0x0000, 0x0000, /*1F263*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF264, 0x0000, 0x0000, 0x0000, /*1F264*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF265, 0x0000, 0x0000, 0x0000, /*1F265*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF266, 0x0000, 0x0000, 0x0000, /*1F266*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF267, 0x0000, 0x0000, 0x0000, /*1F267*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF268, 0x0000, 0x0000, 0x0000, /*1F268*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF269, 0x0000, 0x0000, 0x0000, /*1F269*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF26A, 0x0000, 0x0000, 0x0000, /*1F26A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF26B, 0x0000, 0x0000, 0x0000, /*1F26B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF26C, 0x0000, 0x0000, 0x0000, /*1F26C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF26D, 0x0000, 0x0000, 0x0000, /*1F26D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF26E, 0x0000, 0x0000, 0x0000, /*1F26E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF26F, 0x0000, 0x0000, 0x0000, /*1F26F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF270, 0x0000, 0x0000, 0x0000, /*1F270*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF271, 0x0000, 0x0000, 0x0000, /*1F271*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF272, 0x0000, 0x0000, 0x0000, /*1F272*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF273, 0x0000, 0x0000, 0x0000, /*1F273*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF274, 0x0000, 0x0000, 0x0000, /*1F274*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF275, 0x0000, 0x0000, 0x0000, /*1F275*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF276, 0x0000, 0x0000, 0x0000, /*1F276*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF277, 0x0000, 0x0000, 0x0000, /*1F277*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF278, 0x0000, 0x0000, 0x0000, /*1F278*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF279, 0x0000, 0x0000, 0x0000, /*1F279*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF27A, 0x0000, 0x0000, 0x0000, /*1F27A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF27B, 0x0000, 0x0000, 0x0000, /*1F27B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF27C, 0x0000, 0x0000, 0x0000, /*1F27C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF27D, 0x0000, 0x0000, 0x0000, /*1F27D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF27E, 0x0000, 0x0000, 0x0000, /*1F27E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF27F, 0x0000, 0x0000, 0x0000, /*1F27F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF280, 0x0000, 0x0000, 0x0000, /*1F280*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF281, 0x0000, 0x0000, 0x0000, /*1F281*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF282, 0x0000, 0x0000, 0x0000, /*1F282*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF283, 0x0000, 0x0000, 0x0000, /*1F283*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF284, 0x0000, 0x0000, 0x0000, /*1F284*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF285, 0x0000, 0x0000, 0x0000, /*1F285*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF286, 0x0000, 0x0000, 0x0000, /*1F286*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF287, 0x0000, 0x0000, 0x0000, /*1F287*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF288, 0x0000, 0x0000, 0x0000, /*1F288*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF289, 0x0000, 0x0000, 0x0000, /*1F289*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF28A, 0x0000, 0x0000, 0x0000, /*1F28A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF28B, 0x0000, 0x0000, 0x0000, /*1F28B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF28C, 0x0000, 0x0000, 0x0000, /*1F28C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF28D, 0x0000, 0x0000, 0x0000, /*1F28D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF28E, 0x0000, 0x0000, 0x0000, /*1F28E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF28F, 0x0000, 0x0000, 0x0000, /*1F28F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF290, 0x0000, 0x0000, 0x0000, /*1F290*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF291, 0x0000, 0x0000, 0x0000, /*1F291*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF292, 0x0000, 0x0000, 0x0000, /*1F292*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF293, 0x0000, 0x0000, 0x0000, /*1F293*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF294, 0x0000, 0x0000, 0x0000, /*1F294*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF295, 0x0000, 0x0000, 0x0000, /*1F295*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF296, 0x0000, 0x0000, 0x0000, /*1F296*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF297, 0x0000, 0x0000, 0x0000, /*1F297*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF298, 0x0000, 0x0000, 0x0000, /*1F298*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF299, 0x0000, 0x0000, 0x0000, /*1F299*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF29A, 0x0000, 0x0000, 0x0000, /*1F29A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF29B, 0x0000, 0x0000, 0x0000, /*1F29B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF29C, 0x0000, 0x0000, 0x0000, /*1F29C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF29D, 0x0000, 0x0000, 0x0000, /*1F29D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF29E, 0x0000, 0x0000, 0x0000, /*1F29E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF29F, 0x0000, 0x0000, 0x0000, /*1F29F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2A0, 0x0000, 0x0000, 0x0000, /*1F2A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2A1, 0x0000, 0x0000, 0x0000, /*1F2A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2A2, 0x0000, 0x0000, 0x0000, /*1F2A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2A3, 0x0000, 0x0000, 0x0000, /*1F2A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2A4, 0x0000, 0x0000, 0x0000, /*1F2A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2A5, 0x0000, 0x0000, 0x0000, /*1F2A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2A6, 0x0000, 0x0000, 0x0000, /*1F2A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2A7, 0x0000, 0x0000, 0x0000, /*1F2A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2A8, 0x0000, 0x0000, 0x0000, /*1F2A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2A9, 0x0000, 0x0000, 0x0000, /*1F2A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2AA, 0x0000, 0x0000, 0x0000, /*1F2AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2AB, 0x0000, 0x0000, 0x0000, /*1F2AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2AC, 0x0000, 0x0000, 0x0000, /*1F2AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2AD, 0x0000, 0x0000, 0x0000, /*1F2AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2AE, 0x0000, 0x0000, 0x0000, /*1F2AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2AF, 0x0000, 0x0000, 0x0000, /*1F2AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2B0, 0x0000, 0x0000, 0x0000, /*1F2B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2B1, 0x0000, 0x0000, 0x0000, /*1F2B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2B2, 0x0000, 0x0000, 0x0000, /*1F2B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2B3, 0x0000, 0x0000, 0x0000, /*1F2B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2B4, 0x0000, 0x0000, 0x0000, /*1F2B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2B5, 0x0000, 0x0000, 0x0000, /*1F2B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2B6, 0x0000, 0x0000, 0x0000, /*1F2B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2B7, 0x0000, 0x0000, 0x0000, /*1F2B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2B8, 0x0000, 0x0000, 0x0000, /*1F2B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2B9, 0x0000, 0x0000, 0x0000, /*1F2B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2BA, 0x0000, 0x0000, 0x0000, /*1F2BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2BB, 0x0000, 0x0000, 0x0000, /*1F2BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2BC, 0x0000, 0x0000, 0x0000, /*1F2BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2BD, 0x0000, 0x0000, 0x0000, /*1F2BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2BE, 0x0000, 0x0000, 0x0000, /*1F2BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2BF, 0x0000, 0x0000, 0x0000, /*1F2BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2C0, 0x0000, 0x0000, 0x0000, /*1F2C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2C1, 0x0000, 0x0000, 0x0000, /*1F2C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2C2, 0x0000, 0x0000, 0x0000, /*1F2C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2C3, 0x0000, 0x0000, 0x0000, /*1F2C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2C4, 0x0000, 0x0000, 0x0000, /*1F2C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2C5, 0x0000, 0x0000, 0x0000, /*1F2C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2C6, 0x0000, 0x0000, 0x0000, /*1F2C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2C7, 0x0000, 0x0000, 0x0000, /*1F2C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2C8, 0x0000, 0x0000, 0x0000, /*1F2C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2C9, 0x0000, 0x0000, 0x0000, /*1F2C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2CA, 0x0000, 0x0000, 0x0000, /*1F2CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2CB, 0x0000, 0x0000, 0x0000, /*1F2CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2CC, 0x0000, 0x0000, 0x0000, /*1F2CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2CD, 0x0000, 0x0000, 0x0000, /*1F2CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2CE, 0x0000, 0x0000, 0x0000, /*1F2CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2CF, 0x0000, 0x0000, 0x0000, /*1F2CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2D0, 0x0000, 0x0000, 0x0000, /*1F2D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2D1, 0x0000, 0x0000, 0x0000, /*1F2D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2D2, 0x0000, 0x0000, 0x0000, /*1F2D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2D3, 0x0000, 0x0000, 0x0000, /*1F2D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2D4, 0x0000, 0x0000, 0x0000, /*1F2D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2D5, 0x0000, 0x0000, 0x0000, /*1F2D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2D6, 0x0000, 0x0000, 0x0000, /*1F2D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2D7, 0x0000, 0x0000, 0x0000, /*1F2D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2D8, 0x0000, 0x0000, 0x0000, /*1F2D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2D9, 0x0000, 0x0000, 0x0000, /*1F2D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2DA, 0x0000, 0x0000, 0x0000, /*1F2DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2DB, 0x0000, 0x0000, 0x0000, /*1F2DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2DC, 0x0000, 0x0000, 0x0000, /*1F2DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2DD, 0x0000, 0x0000, 0x0000, /*1F2DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2DE, 0x0000, 0x0000, 0x0000, /*1F2DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2DF, 0x0000, 0x0000, 0x0000, /*1F2DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2E0, 0x0000, 0x0000, 0x0000, /*1F2E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2E1, 0x0000, 0x0000, 0x0000, /*1F2E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2E2, 0x0000, 0x0000, 0x0000, /*1F2E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2E3, 0x0000, 0x0000, 0x0000, /*1F2E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2E4, 0x0000, 0x0000, 0x0000, /*1F2E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2E5, 0x0000, 0x0000, 0x0000, /*1F2E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2E6, 0x0000, 0x0000, 0x0000, /*1F2E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2E7, 0x0000, 0x0000, 0x0000, /*1F2E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2E8, 0x0000, 0x0000, 0x0000, /*1F2E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2E9, 0x0000, 0x0000, 0x0000, /*1F2E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2EA, 0x0000, 0x0000, 0x0000, /*1F2EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2EB, 0x0000, 0x0000, 0x0000, /*1F2EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2EC, 0x0000, 0x0000, 0x0000, /*1F2EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2ED, 0x0000, 0x0000, 0x0000, /*1F2ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2EE, 0x0000, 0x0000, 0x0000, /*1F2EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2EF, 0x0000, 0x0000, 0x0000, /*1F2EF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2F0, 0x0000, 0x0000, 0x0000, /*1F2F0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2F1, 0x0000, 0x0000, 0x0000, /*1F2F1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2F2, 0x0000, 0x0000, 0x0000, /*1F2F2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2F3, 0x0000, 0x0000, 0x0000, /*1F2F3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2F4, 0x0000, 0x0000, 0x0000, /*1F2F4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2F5, 0x0000, 0x0000, 0x0000, /*1F2F5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2F6, 0x0000, 0x0000, 0x0000, /*1F2F6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2F7, 0x0000, 0x0000, 0x0000, /*1F2F7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2F8, 0x0000, 0x0000, 0x0000, /*1F2F8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2F9, 0x0000, 0x0000, 0x0000, /*1F2F9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2FA, 0x0000, 0x0000, 0x0000, /*1F2FA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2FB, 0x0000, 0x0000, 0x0000, /*1F2FB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2FC, 0x0000, 0x0000, 0x0000, /*1F2FC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2FD, 0x0000, 0x0000, 0x0000, /*1F2FD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2FE, 0x0000, 0x0000, 0x0000, /*1F2FE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002,
  0xFBC3, 0x0020, 0x0002, 0xF2FF, 0x0000, 0x0000, 0x0000, /*1F2FF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0002
};

uint16 uca900_p1F3[]= {
  0x12A9, 0x0020, 0x0002, 0x0001, /*1F300*/
  0x12AA, 0x0020, 0x0002, 0x0001, /*1F301*/
  0x12AB, 0x0020, 0x0002, 0x0001, /*1F302*/
  0x12AC, 0x0020, 0x0002, 0x0001, /*1F303*/
  0x12AD, 0x0020, 0x0002, 0x0001, /*1F304*/
  0x12AE, 0x0020, 0x0002, 0x0001, /*1F305*/
  0x12AF, 0x0020, 0x0002, 0x0001, /*1F306*/
  0x12B0, 0x0020, 0x0002, 0x0001, /*1F307*/
  0x12B1, 0x0020, 0x0002, 0x0001, /*1F308*/
  0x12B2, 0x0020, 0x0002, 0x0001, /*1F309*/
  0x12B3, 0x0020, 0x0002, 0x0001, /*1F30A*/
  0x12B4, 0x0020, 0x0002, 0x0001, /*1F30B*/
  0x12B5, 0x0020, 0x0002, 0x0001, /*1F30C*/
  0x12B6, 0x0020, 0x0002, 0x0001, /*1F30D*/
  0x12B7, 0x0020, 0x0002, 0x0001, /*1F30E*/
  0x12B8, 0x0020, 0x0002, 0x0001, /*1F30F*/
  0x12B9, 0x0020, 0x0002, 0x0001, /*1F310*/
  0x12BA, 0x0020, 0x0002, 0x0001, /*1F311*/
  0x12BB, 0x0020, 0x0002, 0x0001, /*1F312*/
  0x12BC, 0x0020, 0x0002, 0x0001, /*1F313*/
  0x12BD, 0x0020, 0x0002, 0x0001, /*1F314*/
  0x12BE, 0x0020, 0x0002, 0x0001, /*1F315*/
  0x12BF, 0x0020, 0x0002, 0x0001, /*1F316*/
  0x12C0, 0x0020, 0x0002, 0x0001, /*1F317*/
  0x12C1, 0x0020, 0x0002, 0x0001, /*1F318*/
  0x12C2, 0x0020, 0x0002, 0x0001, /*1F319*/
  0x12C3, 0x0020, 0x0002, 0x0001, /*1F31A*/
  0x12C4, 0x0020, 0x0002, 0x0001, /*1F31B*/
  0x12C5, 0x0020, 0x0002, 0x0001, /*1F31C*/
  0x12C6, 0x0020, 0x0002, 0x0001, /*1F31D*/
  0x12C7, 0x0020, 0x0002, 0x0001, /*1F31E*/
  0x12C8, 0x0020, 0x0002, 0x0001, /*1F31F*/
  0x12C9, 0x0020, 0x0002, 0x0001, /*1F320*/
  0x12CA, 0x0020, 0x0002, 0x0001, /*1F321*/
  0x12CB, 0x0020, 0x0002, 0x0001, /*1F322*/
  0x12CC, 0x0020, 0x0002, 0x0001, /*1F323*/
  0x12CD, 0x0020, 0x0002, 0x0001, /*1F324*/
  0x12CE, 0x0020, 0x0002, 0x0001, /*1F325*/
  0x12CF, 0x0020, 0x0002, 0x0001, /*1F326*/
  0x12D0, 0x0020, 0x0002, 0x0001, /*1F327*/
  0x12D1, 0x0020, 0x0002, 0x0001, /*1F328*/
  0x12D2, 0x0020, 0x0002, 0x0001, /*1F329*/
  0x12D3, 0x0020, 0x0002, 0x0001, /*1F32A*/
  0x12D4, 0x0020, 0x0002, 0x0001, /*1F32B*/
  0x12D5, 0x0020, 0x0002, 0x0001, /*1F32C*/
  0x12D6, 0x0020, 0x0002, 0x0001, /*1F32D*/
  0x12D7, 0x0020, 0x0002, 0x0001, /*1F32E*/
  0x12D8, 0x0020, 0x0002, 0x0001, /*1F32F*/
  0x12D9, 0x0020, 0x0002, 0x0001, /*1F330*/
  0x12DA, 0x0020, 0x0002, 0x0001, /*1F331*/
  0x12DB, 0x0020, 0x0002, 0x0001, /*1F332*/
  0x12DC, 0x0020, 0x0002, 0x0001, /*1F333*/
  0x12DD, 0x0020, 0x0002, 0x0001, /*1F334*/
  0x12DE, 0x0020, 0x0002, 0x0001, /*1F335*/
  0x12DF, 0x0020, 0x0002, 0x0001, /*1F336*/
  0x12E0, 0x0020, 0x0002, 0x0001, /*1F337*/
  0x12E1, 0x0020, 0x0002, 0x0001, /*1F338*/
  0x12E2, 0x0020, 0x0002, 0x0001, /*1F339*/
  0x12E3, 0x0020, 0x0002, 0x0001, /*1F33A*/
  0x12E4, 0x0020, 0x0002, 0x0001, /*1F33B*/
  0x12E5, 0x0020, 0x0002, 0x0001, /*1F33C*/
  0x12E6, 0x0020, 0x0002, 0x0001, /*1F33D*/
  0x12E7, 0x0020, 0x0002, 0x0001, /*1F33E*/
  0x12E8, 0x0020, 0x0002, 0x0001, /*1F33F*/
  0x12E9, 0x0020, 0x0002, 0x0001, /*1F340*/
  0x12EA, 0x0020, 0x0002, 0x0001, /*1F341*/
  0x12EB, 0x0020, 0x0002, 0x0001, /*1F342*/
  0x12EC, 0x0020, 0x0002, 0x0001, /*1F343*/
  0x12ED, 0x0020, 0x0002, 0x0001, /*1F344*/
  0x12EE, 0x0020, 0x0002, 0x0001, /*1F345*/
  0x12EF, 0x0020, 0x0002, 0x0001, /*1F346*/
  0x12F0, 0x0020, 0x0002, 0x0001, /*1F347*/
  0x12F1, 0x0020, 0x0002, 0x0001, /*1F348*/
  0x12F2, 0x0020, 0x0002, 0x0001, /*1F349*/
  0x12F3, 0x0020, 0x0002, 0x0001, /*1F34A*/
  0x12F4, 0x0020, 0x0002, 0x0001, /*1F34B*/
  0x12F5, 0x0020, 0x0002, 0x0001, /*1F34C*/
  0x12F6, 0x0020, 0x0002, 0x0001, /*1F34D*/
  0x12F7, 0x0020, 0x0002, 0x0001, /*1F34E*/
  0x12F8, 0x0020, 0x0002, 0x0001, /*1F34F*/
  0x12F9, 0x0020, 0x0002, 0x0001, /*1F350*/
  0x12FA, 0x0020, 0x0002, 0x0001, /*1F351*/
  0x12FB, 0x0020, 0x0002, 0x0001, /*1F352*/
  0x12FC, 0x0020, 0x0002, 0x0001, /*1F353*/
  0x12FD, 0x0020, 0x0002, 0x0001, /*1F354*/
  0x12FE, 0x0020, 0x0002, 0x0001, /*1F355*/
  0x12FF, 0x0020, 0x0002, 0x0001, /*1F356*/
  0x1300, 0x0020, 0x0002, 0x0001, /*1F357*/
  0x1301, 0x0020, 0x0002, 0x0001, /*1F358*/
  0x1302, 0x0020, 0x0002, 0x0001, /*1F359*/
  0x1303, 0x0020, 0x0002, 0x0001, /*1F35A*/
  0x1304, 0x0020, 0x0002, 0x0001, /*1F35B*/
  0x1305, 0x0020, 0x0002, 0x0001, /*1F35C*/
  0x1306, 0x0020, 0x0002, 0x0001, /*1F35D*/
  0x1307, 0x0020, 0x0002, 0x0001, /*1F35E*/
  0x1308, 0x0020, 0x0002, 0x0001, /*1F35F*/
  0x1309, 0x0020, 0x0002, 0x0001, /*1F360*/
  0x130A, 0x0020, 0x0002, 0x0001, /*1F361*/
  0x130B, 0x0020, 0x0002, 0x0001, /*1F362*/
  0x130C, 0x0020, 0x0002, 0x0001, /*1F363*/
  0x130D, 0x0020, 0x0002, 0x0001, /*1F364*/
  0x130E, 0x0020, 0x0002, 0x0001, /*1F365*/
  0x130F, 0x0020, 0x0002, 0x0001, /*1F366*/
  0x1310, 0x0020, 0x0002, 0x0001, /*1F367*/
  0x1311, 0x0020, 0x0002, 0x0001, /*1F368*/
  0x1312, 0x0020, 0x0002, 0x0001, /*1F369*/
  0x1313, 0x0020, 0x0002, 0x0001, /*1F36A*/
  0x1314, 0x0020, 0x0002, 0x0001, /*1F36B*/
  0x1315, 0x0020, 0x0002, 0x0001, /*1F36C*/
  0x1316, 0x0020, 0x0002, 0x0001, /*1F36D*/
  0x1317, 0x0020, 0x0002, 0x0001, /*1F36E*/
  0x1318, 0x0020, 0x0002, 0x0001, /*1F36F*/
  0x1319, 0x0020, 0x0002, 0x0001, /*1F370*/
  0x131A, 0x0020, 0x0002, 0x0001, /*1F371*/
  0x131B, 0x0020, 0x0002, 0x0001, /*1F372*/
  0x131C, 0x0020, 0x0002, 0x0001, /*1F373*/
  0x131D, 0x0020, 0x0002, 0x0001, /*1F374*/
  0x131E, 0x0020, 0x0002, 0x0001, /*1F375*/
  0x131F, 0x0020, 0x0002, 0x0001, /*1F376*/
  0x1320, 0x0020, 0x0002, 0x0001, /*1F377*/
  0x1321, 0x0020, 0x0002, 0x0001, /*1F378*/
  0x1322, 0x0020, 0x0002, 0x0001, /*1F379*/
  0x1323, 0x0020, 0x0002, 0x0001, /*1F37A*/
  0x1324, 0x0020, 0x0002, 0x0001, /*1F37B*/
  0x1325, 0x0020, 0x0002, 0x0001, /*1F37C*/
  0x1326, 0x0020, 0x0002, 0x0001, /*1F37D*/
  0x1327, 0x0020, 0x0002, 0x0001, /*1F37E*/
  0x1328, 0x0020, 0x0002, 0x0001, /*1F37F*/
  0x1329, 0x0020, 0x0002, 0x0001, /*1F380*/
  0x132A, 0x0020, 0x0002, 0x0001, /*1F381*/
  0x132B, 0x0020, 0x0002, 0x0001, /*1F382*/
  0x132C, 0x0020, 0x0002, 0x0001, /*1F383*/
  0x132D, 0x0020, 0x0002, 0x0001, /*1F384*/
  0x132E, 0x0020, 0x0002, 0x0001, /*1F385*/
  0x132F, 0x0020, 0x0002, 0x0001, /*1F386*/
  0x1330, 0x0020, 0x0002, 0x0001, /*1F387*/
  0x1331, 0x0020, 0x0002, 0x0001, /*1F388*/
  0x1332, 0x0020, 0x0002, 0x0001, /*1F389*/
  0x1333, 0x0020, 0x0002, 0x0001, /*1F38A*/
  0x1334, 0x0020, 0x0002, 0x0001, /*1F38B*/
  0x1335, 0x0020, 0x0002, 0x0001, /*1F38C*/
  0x1336, 0x0020, 0x0002, 0x0001, /*1F38D*/
  0x1337, 0x0020, 0x0002, 0x0001, /*1F38E*/
  0x1338, 0x0020, 0x0002, 0x0001, /*1F38F*/
  0x1339, 0x0020, 0x0002, 0x0001, /*1F390*/
  0x133A, 0x0020, 0x0002, 0x0001, /*1F391*/
  0x133B, 0x0020, 0x0002, 0x0001, /*1F392*/
  0x133C, 0x0020, 0x0002, 0x0001, /*1F393*/
  0x133D, 0x0020, 0x0002, 0x0001, /*1F394*/
  0x133E, 0x0020, 0x0002, 0x0001, /*1F395*/
  0x133F, 0x0020, 0x0002, 0x0001, /*1F396*/
  0x1340, 0x0020, 0x0002, 0x0001, /*1F397*/
  0x1341, 0x0020, 0x0002, 0x0001, /*1F398*/
  0x1342, 0x0020, 0x0002, 0x0001, /*1F399*/
  0x1343, 0x0020, 0x0002, 0x0001, /*1F39A*/
  0x1344, 0x0020, 0x0002, 0x0001, /*1F39B*/
  0x1345, 0x0020, 0x0002, 0x0001, /*1F39C*/
  0x1346, 0x0020, 0x0002, 0x0001, /*1F39D*/
  0x1347, 0x0020, 0x0002, 0x0001, /*1F39E*/
  0x1348, 0x0020, 0x0002, 0x0001, /*1F39F*/
  0x1349, 0x0020, 0x0002, 0x0001, /*1F3A0*/
  0x134A, 0x0020, 0x0002, 0x0001, /*1F3A1*/
  0x134B, 0x0020, 0x0002, 0x0001, /*1F3A2*/
  0x134C, 0x0020, 0x0002, 0x0001, /*1F3A3*/
  0x134D, 0x0020, 0x0002, 0x0001, /*1F3A4*/
  0x134E, 0x0020, 0x0002, 0x0001, /*1F3A5*/
  0x134F, 0x0020, 0x0002, 0x0001, /*1F3A6*/
  0x1350, 0x0020, 0x0002, 0x0001, /*1F3A7*/
  0x1351, 0x0020, 0x0002, 0x0001, /*1F3A8*/
  0x1352, 0x0020, 0x0002, 0x0001, /*1F3A9*/
  0x1353, 0x0020, 0x0002, 0x0001, /*1F3AA*/
  0x1354, 0x0020, 0x0002, 0x0001, /*1F3AB*/
  0x1355, 0x0020, 0x0002, 0x0001, /*1F3AC*/
  0x1356, 0x0020, 0x0002, 0x0001, /*1F3AD*/
  0x1357, 0x0020, 0x0002, 0x0001, /*1F3AE*/
  0x1358, 0x0020, 0x0002, 0x0001, /*1F3AF*/
  0x1359, 0x0020, 0x0002, 0x0001, /*1F3B0*/
  0x135A, 0x0020, 0x0002, 0x0001, /*1F3B1*/
  0x135B, 0x0020, 0x0002, 0x0001, /*1F3B2*/
  0x135C, 0x0020, 0x0002, 0x0001, /*1F3B3*/
  0x135D, 0x0020, 0x0002, 0x0001, /*1F3B4*/
  0x135E, 0x0020, 0x0002, 0x0001, /*1F3B5*/
  0x135F, 0x0020, 0x0002, 0x0001, /*1F3B6*/
  0x1360, 0x0020, 0x0002, 0x0001, /*1F3B7*/
  0x1361, 0x0020, 0x0002, 0x0001, /*1F3B8*/
  0x1362, 0x0020, 0x0002, 0x0001, /*1F3B9*/
  0x1363, 0x0020, 0x0002, 0x0001, /*1F3BA*/
  0x1364, 0x0020, 0x0002, 0x0001, /*1F3BB*/
  0x1365, 0x0020, 0x0002, 0x0001, /*1F3BC*/
  0x1366, 0x0020, 0x0002, 0x0001, /*1F3BD*/
  0x1367, 0x0020, 0x0002, 0x0001, /*1F3BE*/
  0x1368, 0x0020, 0x0002, 0x0001, /*1F3BF*/
  0x1369, 0x0020, 0x0002, 0x0001, /*1F3C0*/
  0x136A, 0x0020, 0x0002, 0x0001, /*1F3C1*/
  0x136B, 0x0020, 0x0002, 0x0001, /*1F3C2*/
  0x136C, 0x0020, 0x0002, 0x0001, /*1F3C3*/
  0x136D, 0x0020, 0x0002, 0x0001, /*1F3C4*/
  0x136E, 0x0020, 0x0002, 0x0001, /*1F3C5*/
  0x136F, 0x0020, 0x0002, 0x0001, /*1F3C6*/
  0x1370, 0x0020, 0x0002, 0x0001, /*1F3C7*/
  0x1371, 0x0020, 0x0002, 0x0001, /*1F3C8*/
  0x1372, 0x0020, 0x0002, 0x0001, /*1F3C9*/
  0x1373, 0x0020, 0x0002, 0x0001, /*1F3CA*/
  0x1374, 0x0020, 0x0002, 0x0001, /*1F3CB*/
  0x1375, 0x0020, 0x0002, 0x0001, /*1F3CC*/
  0x1376, 0x0020, 0x0002, 0x0001, /*1F3CD*/
  0x1377, 0x0020, 0x0002, 0x0001, /*1F3CE*/
  0x1378, 0x0020, 0x0002, 0x0001, /*1F3CF*/
  0x1379, 0x0020, 0x0002, 0x0001, /*1F3D0*/
  0x137A, 0x0020, 0x0002, 0x0001, /*1F3D1*/
  0x137B, 0x0020, 0x0002, 0x0001, /*1F3D2*/
  0x137C, 0x0020, 0x0002, 0x0001, /*1F3D3*/
  0x137D, 0x0020, 0x0002, 0x0001, /*1F3D4*/
  0x137E, 0x0020, 0x0002, 0x0001, /*1F3D5*/
  0x137F, 0x0020, 0x0002, 0x0001, /*1F3D6*/
  0x1380, 0x0020, 0x0002, 0x0001, /*1F3D7*/
  0x1381, 0x0020, 0x0002, 0x0001, /*1F3D8*/
  0x1382, 0x0020, 0x0002, 0x0001, /*1F3D9*/
  0x1383, 0x0020, 0x0002, 0x0001, /*1F3DA*/
  0x1384, 0x0020, 0x0002, 0x0001, /*1F3DB*/
  0x1385, 0x0020, 0x0002, 0x0001, /*1F3DC*/
  0x1386, 0x0020, 0x0002, 0x0001, /*1F3DD*/
  0x1387, 0x0020, 0x0002, 0x0001, /*1F3DE*/
  0x1388, 0x0020, 0x0002, 0x0001, /*1F3DF*/
  0x1389, 0x0020, 0x0002, 0x0001, /*1F3E0*/
  0x138A, 0x0020, 0x0002, 0x0001, /*1F3E1*/
  0x138B, 0x0020, 0x0002, 0x0001, /*1F3E2*/
  0x138C, 0x0020, 0x0002, 0x0001, /*1F3E3*/
  0x138D, 0x0020, 0x0002, 0x0001, /*1F3E4*/
  0x138E, 0x0020, 0x0002, 0x0001, /*1F3E5*/
  0x138F, 0x0020, 0x0002, 0x0001, /*1F3E6*/
  0x1390, 0x0020, 0x0002, 0x0001, /*1F3E7*/
  0x1391, 0x0020, 0x0002, 0x0001, /*1F3E8*/
  0x1392, 0x0020, 0x0002, 0x0001, /*1F3E9*/
  0x1393, 0x0020, 0x0002, 0x0001, /*1F3EA*/
  0x1394, 0x0020, 0x0002, 0x0001, /*1F3EB*/
  0x1395, 0x0020, 0x0002, 0x0001, /*1F3EC*/
  0x1396, 0x0020, 0x0002, 0x0001, /*1F3ED*/
  0x1397, 0x0020, 0x0002, 0x0001, /*1F3EE*/
  0x1398, 0x0020, 0x0002, 0x0001, /*1F3EF*/
  0x1399, 0x0020, 0x0002, 0x0001, /*1F3F0*/
  0x139A, 0x0020, 0x0002, 0x0001, /*1F3F1*/
  0x139B, 0x0020, 0x0002, 0x0001, /*1F3F2*/
  0x139C, 0x0020, 0x0002, 0x0001, /*1F3F3*/
  0x139D, 0x0020, 0x0002, 0x0001, /*1F3F4*/
  0x139E, 0x0020, 0x0002, 0x0001, /*1F3F5*/
  0x139F, 0x0020, 0x0002, 0x0001, /*1F3F6*/
  0x13A0, 0x0020, 0x0002, 0x0001, /*1F3F7*/
  0x13A1, 0x0020, 0x0002, 0x0001, /*1F3F8*/
  0x13A2, 0x0020, 0x0002, 0x0001, /*1F3F9*/
  0x13A3, 0x0020, 0x0002, 0x0001, /*1F3FA*/
  0x13A4, 0x0020, 0x0002, 0x0001, /*1F3FB*/
  0x13A5, 0x0020, 0x0002, 0x0001, /*1F3FC*/
  0x13A6, 0x0020, 0x0002, 0x0001, /*1F3FD*/
  0x13A7, 0x0020, 0x0002, 0x0001, /*1F3FE*/
  0x13A8, 0x0020, 0x0002, 0x0001  /*1F3FF*/
};

uint16 uca900_p1F4[]= {
  0x13A9, 0x0020, 0x0002, 0x0001, /*1F400*/
  0x13AA, 0x0020, 0x0002, 0x0001, /*1F401*/
  0x13AB, 0x0020, 0x0002, 0x0001, /*1F402*/
  0x13AC, 0x0020, 0x0002, 0x0001, /*1F403*/
  0x13AD, 0x0020, 0x0002, 0x0001, /*1F404*/
  0x13AE, 0x0020, 0x0002, 0x0001, /*1F405*/
  0x13AF, 0x0020, 0x0002, 0x0001, /*1F406*/
  0x13B0, 0x0020, 0x0002, 0x0001, /*1F407*/
  0x13B1, 0x0020, 0x0002, 0x0001, /*1F408*/
  0x13B2, 0x0020, 0x0002, 0x0001, /*1F409*/
  0x13B3, 0x0020, 0x0002, 0x0001, /*1F40A*/
  0x13B4, 0x0020, 0x0002, 0x0001, /*1F40B*/
  0x13B5, 0x0020, 0x0002, 0x0001, /*1F40C*/
  0x13B6, 0x0020, 0x0002, 0x0001, /*1F40D*/
  0x13B7, 0x0020, 0x0002, 0x0001, /*1F40E*/
  0x13B8, 0x0020, 0x0002, 0x0001, /*1F40F*/
  0x13B9, 0x0020, 0x0002, 0x0001, /*1F410*/
  0x13BA, 0x0020, 0x0002, 0x0001, /*1F411*/
  0x13BB, 0x0020, 0x0002, 0x0001, /*1F412*/
  0x13BC, 0x0020, 0x0002, 0x0001, /*1F413*/
  0x13BD, 0x0020, 0x0002, 0x0001, /*1F414*/
  0x13BE, 0x0020, 0x0002, 0x0001, /*1F415*/
  0x13BF, 0x0020, 0x0002, 0x0001, /*1F416*/
  0x13C0, 0x0020, 0x0002, 0x0001, /*1F417*/
  0x13C1, 0x0020, 0x0002, 0x0001, /*1F418*/
  0x13C2, 0x0020, 0x0002, 0x0001, /*1F419*/
  0x13C3, 0x0020, 0x0002, 0x0001, /*1F41A*/
  0x13C4, 0x0020, 0x0002, 0x0001, /*1F41B*/
  0x13C5, 0x0020, 0x0002, 0x0001, /*1F41C*/
  0x13C6, 0x0020, 0x0002, 0x0001, /*1F41D*/
  0x13C7, 0x0020, 0x0002, 0x0001, /*1F41E*/
  0x13C8, 0x0020, 0x0002, 0x0001, /*1F41F*/
  0x13C9, 0x0020, 0x0002, 0x0001, /*1F420*/
  0x13CA, 0x0020, 0x0002, 0x0001, /*1F421*/
  0x13CB, 0x0020, 0x0002, 0x0001, /*1F422*/
  0x13CC, 0x0020, 0x0002, 0x0001, /*1F423*/
  0x13CD, 0x0020, 0x0002, 0x0001, /*1F424*/
  0x13CE, 0x0020, 0x0002, 0x0001, /*1F425*/
  0x13CF, 0x0020, 0x0002, 0x0001, /*1F426*/
  0x13D0, 0x0020, 0x0002, 0x0001, /*1F427*/
  0x13D1, 0x0020, 0x0002, 0x0001, /*1F428*/
  0x13D2, 0x0020, 0x0002, 0x0001, /*1F429*/
  0x13D3, 0x0020, 0x0002, 0x0001, /*1F42A*/
  0x13D4, 0x0020, 0x0002, 0x0001, /*1F42B*/
  0x13D5, 0x0020, 0x0002, 0x0001, /*1F42C*/
  0x13D6, 0x0020, 0x0002, 0x0001, /*1F42D*/
  0x13D7, 0x0020, 0x0002, 0x0001, /*1F42E*/
  0x13D8, 0x0020, 0x0002, 0x0001, /*1F42F*/
  0x13D9, 0x0020, 0x0002, 0x0001, /*1F430*/
  0x13DA, 0x0020, 0x0002, 0x0001, /*1F431*/
  0x13DB, 0x0020, 0x0002, 0x0001, /*1F432*/
  0x13DC, 0x0020, 0x0002, 0x0001, /*1F433*/
  0x13DD, 0x0020, 0x0002, 0x0001, /*1F434*/
  0x13DE, 0x0020, 0x0002, 0x0001, /*1F435*/
  0x13DF, 0x0020, 0x0002, 0x0001, /*1F436*/
  0x13E0, 0x0020, 0x0002, 0x0001, /*1F437*/
  0x13E1, 0x0020, 0x0002, 0x0001, /*1F438*/
  0x13E2, 0x0020, 0x0002, 0x0001, /*1F439*/
  0x13E3, 0x0020, 0x0002, 0x0001, /*1F43A*/
  0x13E4, 0x0020, 0x0002, 0x0001, /*1F43B*/
  0x13E5, 0x0020, 0x0002, 0x0001, /*1F43C*/
  0x13E6, 0x0020, 0x0002, 0x0001, /*1F43D*/
  0x13E7, 0x0020, 0x0002, 0x0001, /*1F43E*/
  0x13E8, 0x0020, 0x0002, 0x0001, /*1F43F*/
  0x13E9, 0x0020, 0x0002, 0x0001, /*1F440*/
  0x13EA, 0x0020, 0x0002, 0x0001, /*1F441*/
  0x13EB, 0x0020, 0x0002, 0x0001, /*1F442*/
  0x13EC, 0x0020, 0x0002, 0x0001, /*1F443*/
  0x13ED, 0x0020, 0x0002, 0x0001, /*1F444*/
  0x13EE, 0x0020, 0x0002, 0x0001, /*1F445*/
  0x13EF, 0x0020, 0x0002, 0x0001, /*1F446*/
  0x13F0, 0x0020, 0x0002, 0x0001, /*1F447*/
  0x13F1, 0x0020, 0x0002, 0x0001, /*1F448*/
  0x13F2, 0x0020, 0x0002, 0x0001, /*1F449*/
  0x13F3, 0x0020, 0x0002, 0x0001, /*1F44A*/
  0x13F4, 0x0020, 0x0002, 0x0001, /*1F44B*/
  0x13F5, 0x0020, 0x0002, 0x0001, /*1F44C*/
  0x13F6, 0x0020, 0x0002, 0x0001, /*1F44D*/
  0x13F7, 0x0020, 0x0002, 0x0001, /*1F44E*/
  0x13F8, 0x0020, 0x0002, 0x0001, /*1F44F*/
  0x13F9, 0x0020, 0x0002, 0x0001, /*1F450*/
  0x13FA, 0x0020, 0x0002, 0x0001, /*1F451*/
  0x13FB, 0x0020, 0x0002, 0x0001, /*1F452*/
  0x13FC, 0x0020, 0x0002, 0x0001, /*1F453*/
  0x13FD, 0x0020, 0x0002, 0x0001, /*1F454*/
  0x13FE, 0x0020, 0x0002, 0x0001, /*1F455*/
  0x13FF, 0x0020, 0x0002, 0x0001, /*1F456*/
  0x1400, 0x0020, 0x0002, 0x0001, /*1F457*/
  0x1401, 0x0020, 0x0002, 0x0001, /*1F458*/
  0x1402, 0x0020, 0x0002, 0x0001, /*1F459*/
  0x1403, 0x0020, 0x0002, 0x0001, /*1F45A*/
  0x1404, 0x0020, 0x0002, 0x0001, /*1F45B*/
  0x1405, 0x0020, 0x0002, 0x0001, /*1F45C*/
  0x1406, 0x0020, 0x0002, 0x0001, /*1F45D*/
  0x1407, 0x0020, 0x0002, 0x0001, /*1F45E*/
  0x1408, 0x0020, 0x0002, 0x0001, /*1F45F*/
  0x1409, 0x0020, 0x0002, 0x0001, /*1F460*/
  0x140A, 0x0020, 0x0002, 0x0001, /*1F461*/
  0x140B, 0x0020, 0x0002, 0x0001, /*1F462*/
  0x140C, 0x0020, 0x0002, 0x0001, /*1F463*/
  0x140D, 0x0020, 0x0002, 0x0001, /*1F464*/
  0x140E, 0x0020, 0x0002, 0x0001, /*1F465*/
  0x140F, 0x0020, 0x0002, 0x0001, /*1F466*/
  0x1410, 0x0020, 0x0002, 0x0001, /*1F467*/
  0x1411, 0x0020, 0x0002, 0x0001, /*1F468*/
  0x1412, 0x0020, 0x0002, 0x0001, /*1F469*/
  0x1413, 0x0020, 0x0002, 0x0001, /*1F46A*/
  0x1414, 0x0020, 0x0002, 0x0001, /*1F46B*/
  0x1415, 0x0020, 0x0002, 0x0001, /*1F46C*/
  0x1416, 0x0020, 0x0002, 0x0001, /*1F46D*/
  0x1417, 0x0020, 0x0002, 0x0001, /*1F46E*/
  0x1418, 0x0020, 0x0002, 0x0001, /*1F46F*/
  0x1419, 0x0020, 0x0002, 0x0001, /*1F470*/
  0x141A, 0x0020, 0x0002, 0x0001, /*1F471*/
  0x141B, 0x0020, 0x0002, 0x0001, /*1F472*/
  0x141C, 0x0020, 0x0002, 0x0001, /*1F473*/
  0x141D, 0x0020, 0x0002, 0x0001, /*1F474*/
  0x141E, 0x0020, 0x0002, 0x0001, /*1F475*/
  0x141F, 0x0020, 0x0002, 0x0001, /*1F476*/
  0x1420, 0x0020, 0x0002, 0x0001, /*1F477*/
  0x1421, 0x0020, 0x0002, 0x0001, /*1F478*/
  0x1422, 0x0020, 0x0002, 0x0001, /*1F479*/
  0x1423, 0x0020, 0x0002, 0x0001, /*1F47A*/
  0x1424, 0x0020, 0x0002, 0x0001, /*1F47B*/
  0x1425, 0x0020, 0x0002, 0x0001, /*1F47C*/
  0x1426, 0x0020, 0x0002, 0x0001, /*1F47D*/
  0x1427, 0x0020, 0x0002, 0x0001, /*1F47E*/
  0x1428, 0x0020, 0x0002, 0x0001, /*1F47F*/
  0x1429, 0x0020, 0x0002, 0x0001, /*1F480*/
  0x142A, 0x0020, 0x0002, 0x0001, /*1F481*/
  0x142B, 0x0020, 0x0002, 0x0001, /*1F482*/
  0x142C, 0x0020, 0x0002, 0x0001, /*1F483*/
  0x142D, 0x0020, 0x0002, 0x0001, /*1F484*/
  0x142E, 0x0020, 0x0002, 0x0001, /*1F485*/
  0x142F, 0x0020, 0x0002, 0x0001, /*1F486*/
  0x1430, 0x0020, 0x0002, 0x0001, /*1F487*/
  0x1431, 0x0020, 0x0002, 0x0001, /*1F488*/
  0x1432, 0x0020, 0x0002, 0x0001, /*1F489*/
  0x1433, 0x0020, 0x0002, 0x0001, /*1F48A*/
  0x1434, 0x0020, 0x0002, 0x0001, /*1F48B*/
  0x1435, 0x0020, 0x0002, 0x0001, /*1F48C*/
  0x1436, 0x0020, 0x0002, 0x0001, /*1F48D*/
  0x1437, 0x0020, 0x0002, 0x0001, /*1F48E*/
  0x1438, 0x0020, 0x0002, 0x0001, /*1F48F*/
  0x1439, 0x0020, 0x0002, 0x0001, /*1F490*/
  0x143A, 0x0020, 0x0002, 0x0001, /*1F491*/
  0x143B, 0x0020, 0x0002, 0x0001, /*1F492*/
  0x143C, 0x0020, 0x0002, 0x0001, /*1F493*/
  0x143D, 0x0020, 0x0002, 0x0001, /*1F494*/
  0x143E, 0x0020, 0x0002, 0x0001, /*1F495*/
  0x143F, 0x0020, 0x0002, 0x0001, /*1F496*/
  0x1440, 0x0020, 0x0002, 0x0001, /*1F497*/
  0x1441, 0x0020, 0x0002, 0x0001, /*1F498*/
  0x1442, 0x0020, 0x0002, 0x0001, /*1F499*/
  0x1443, 0x0020, 0x0002, 0x0001, /*1F49A*/
  0x1444, 0x0020, 0x0002, 0x0001, /*1F49B*/
  0x1445, 0x0020, 0x0002, 0x0001, /*1F49C*/
  0x1446, 0x0020, 0x0002, 0x0001, /*1F49D*/
  0x1447, 0x0020, 0x0002, 0x0001, /*1F49E*/
  0x1448, 0x0020, 0x0002, 0x0001, /*1F49F*/
  0x1449, 0x0020, 0x0002, 0x0001, /*1F4A0*/
  0x144A, 0x0020, 0x0002, 0x0001, /*1F4A1*/
  0x144B, 0x0020, 0x0002, 0x0001, /*1F4A2*/
  0x144C, 0x0020, 0x0002, 0x0001, /*1F4A3*/
  0x144D, 0x0020, 0x0002, 0x0001, /*1F4A4*/
  0x144E, 0x0020, 0x0002, 0x0001, /*1F4A5*/
  0x144F, 0x0020, 0x0002, 0x0001, /*1F4A6*/
  0x1450, 0x0020, 0x0002, 0x0001, /*1F4A7*/
  0x1451, 0x0020, 0x0002, 0x0001, /*1F4A8*/
  0x1452, 0x0020, 0x0002, 0x0001, /*1F4A9*/
  0x1453, 0x0020, 0x0002, 0x0001, /*1F4AA*/
  0x1454, 0x0020, 0x0002, 0x0001, /*1F4AB*/
  0x1455, 0x0020, 0x0002, 0x0001, /*1F4AC*/
  0x1456, 0x0020, 0x0002, 0x0001, /*1F4AD*/
  0x1457, 0x0020, 0x0002, 0x0001, /*1F4AE*/
  0x1458, 0x0020, 0x0002, 0x0001, /*1F4AF*/
  0x1459, 0x0020, 0x0002, 0x0001, /*1F4B0*/
  0x145A, 0x0020, 0x0002, 0x0001, /*1F4B1*/
  0x145B, 0x0020, 0x0002, 0x0001, /*1F4B2*/
  0x145C, 0x0020, 0x0002, 0x0001, /*1F4B3*/
  0x145D, 0x0020, 0x0002, 0x0001, /*1F4B4*/
  0x145E, 0x0020, 0x0002, 0x0001, /*1F4B5*/
  0x145F, 0x0020, 0x0002, 0x0001, /*1F4B6*/
  0x1460, 0x0020, 0x0002, 0x0001, /*1F4B7*/
  0x1461, 0x0020, 0x0002, 0x0001, /*1F4B8*/
  0x1462, 0x0020, 0x0002, 0x0001, /*1F4B9*/
  0x1463, 0x0020, 0x0002, 0x0001, /*1F4BA*/
  0x1464, 0x0020, 0x0002, 0x0001, /*1F4BB*/
  0x1465, 0x0020, 0x0002, 0x0001, /*1F4BC*/
  0x1466, 0x0020, 0x0002, 0x0001, /*1F4BD*/
  0x1467, 0x0020, 0x0002, 0x0001, /*1F4BE*/
  0x1468, 0x0020, 0x0002, 0x0001, /*1F4BF*/
  0x1469, 0x0020, 0x0002, 0x0001, /*1F4C0*/
  0x146A, 0x0020, 0x0002, 0x0001, /*1F4C1*/
  0x146B, 0x0020, 0x0002, 0x0001, /*1F4C2*/
  0x146C, 0x0020, 0x0002, 0x0001, /*1F4C3*/
  0x146D, 0x0020, 0x0002, 0x0001, /*1F4C4*/
  0x146E, 0x0020, 0x0002, 0x0001, /*1F4C5*/
  0x146F, 0x0020, 0x0002, 0x0001, /*1F4C6*/
  0x1470, 0x0020, 0x0002, 0x0001, /*1F4C7*/
  0x1471, 0x0020, 0x0002, 0x0001, /*1F4C8*/
  0x1472, 0x0020, 0x0002, 0x0001, /*1F4C9*/
  0x1473, 0x0020, 0x0002, 0x0001, /*1F4CA*/
  0x1474, 0x0020, 0x0002, 0x0001, /*1F4CB*/
  0x1475, 0x0020, 0x0002, 0x0001, /*1F4CC*/
  0x1476, 0x0020, 0x0002, 0x0001, /*1F4CD*/
  0x1477, 0x0020, 0x0002, 0x0001, /*1F4CE*/
  0x1478, 0x0020, 0x0002, 0x0001, /*1F4CF*/
  0x1479, 0x0020, 0x0002, 0x0001, /*1F4D0*/
  0x147A, 0x0020, 0x0002, 0x0001, /*1F4D1*/
  0x147B, 0x0020, 0x0002, 0x0001, /*1F4D2*/
  0x147C, 0x0020, 0x0002, 0x0001, /*1F4D3*/
  0x147D, 0x0020, 0x0002, 0x0001, /*1F4D4*/
  0x147E, 0x0020, 0x0002, 0x0001, /*1F4D5*/
  0x147F, 0x0020, 0x0002, 0x0001, /*1F4D6*/
  0x1480, 0x0020, 0x0002, 0x0001, /*1F4D7*/
  0x1481, 0x0020, 0x0002, 0x0001, /*1F4D8*/
  0x1482, 0x0020, 0x0002, 0x0001, /*1F4D9*/
  0x1483, 0x0020, 0x0002, 0x0001, /*1F4DA*/
  0x1484, 0x0020, 0x0002, 0x0001, /*1F4DB*/
  0x1485, 0x0020, 0x0002, 0x0001, /*1F4DC*/
  0x1486, 0x0020, 0x0002, 0x0001, /*1F4DD*/
  0x1487, 0x0020, 0x0002, 0x0001, /*1F4DE*/
  0x1488, 0x0020, 0x0002, 0x0001, /*1F4DF*/
  0x1489, 0x0020, 0x0002, 0x0001, /*1F4E0*/
  0x148A, 0x0020, 0x0002, 0x0001, /*1F4E1*/
  0x148B, 0x0020, 0x0002, 0x0001, /*1F4E2*/
  0x148C, 0x0020, 0x0002, 0x0001, /*1F4E3*/
  0x148D, 0x0020, 0x0002, 0x0001, /*1F4E4*/
  0x148E, 0x0020, 0x0002, 0x0001, /*1F4E5*/
  0x148F, 0x0020, 0x0002, 0x0001, /*1F4E6*/
  0x1490, 0x0020, 0x0002, 0x0001, /*1F4E7*/
  0x1491, 0x0020, 0x0002, 0x0001, /*1F4E8*/
  0x1492, 0x0020, 0x0002, 0x0001, /*1F4E9*/
  0x1493, 0x0020, 0x0002, 0x0001, /*1F4EA*/
  0x1494, 0x0020, 0x0002, 0x0001, /*1F4EB*/
  0x1495, 0x0020, 0x0002, 0x0001, /*1F4EC*/
  0x1496, 0x0020, 0x0002, 0x0001, /*1F4ED*/
  0x1497, 0x0020, 0x0002, 0x0001, /*1F4EE*/
  0x1498, 0x0020, 0x0002, 0x0001, /*1F4EF*/
  0x1499, 0x0020, 0x0002, 0x0001, /*1F4F0*/
  0x149A, 0x0020, 0x0002, 0x0001, /*1F4F1*/
  0x149B, 0x0020, 0x0002, 0x0001, /*1F4F2*/
  0x149C, 0x0020, 0x0002, 0x0001, /*1F4F3*/
  0x149D, 0x0020, 0x0002, 0x0001, /*1F4F4*/
  0x149E, 0x0020, 0x0002, 0x0001, /*1F4F5*/
  0x149F, 0x0020, 0x0002, 0x0001, /*1F4F6*/
  0x14A0, 0x0020, 0x0002, 0x0001, /*1F4F7*/
  0x14A1, 0x0020, 0x0002, 0x0001, /*1F4F8*/
  0x14A2, 0x0020, 0x0002, 0x0001, /*1F4F9*/
  0x14A3, 0x0020, 0x0002, 0x0001, /*1F4FA*/
  0x14A4, 0x0020, 0x0002, 0x0001, /*1F4FB*/
  0x14A5, 0x0020, 0x0002, 0x0001, /*1F4FC*/
  0x14A6, 0x0020, 0x0002, 0x0001, /*1F4FD*/
  0x14A7, 0x0020, 0x0002, 0x0001, /*1F4FE*/
  0x14A8, 0x0020, 0x0002, 0x0001  /*1F4FF*/
};

uint16 uca900_p1F5[]= {
  0x14A9, 0x0020, 0x0002, 0x0001, /*1F500*/
  0x14AA, 0x0020, 0x0002, 0x0001, /*1F501*/
  0x14AB, 0x0020, 0x0002, 0x0001, /*1F502*/
  0x14AC, 0x0020, 0x0002, 0x0001, /*1F503*/
  0x14AD, 0x0020, 0x0002, 0x0001, /*1F504*/
  0x14AE, 0x0020, 0x0002, 0x0001, /*1F505*/
  0x14AF, 0x0020, 0x0002, 0x0001, /*1F506*/
  0x14B0, 0x0020, 0x0002, 0x0001, /*1F507*/
  0x14B1, 0x0020, 0x0002, 0x0001, /*1F508*/
  0x14B2, 0x0020, 0x0002, 0x0001, /*1F509*/
  0x14B3, 0x0020, 0x0002, 0x0001, /*1F50A*/
  0x14B4, 0x0020, 0x0002, 0x0001, /*1F50B*/
  0x14B5, 0x0020, 0x0002, 0x0001, /*1F50C*/
  0x14B6, 0x0020, 0x0002, 0x0001, /*1F50D*/
  0x14B7, 0x0020, 0x0002, 0x0001, /*1F50E*/
  0x14B8, 0x0020, 0x0002, 0x0001, /*1F50F*/
  0x14B9, 0x0020, 0x0002, 0x0001, /*1F510*/
  0x14BA, 0x0020, 0x0002, 0x0001, /*1F511*/
  0x14BB, 0x0020, 0x0002, 0x0001, /*1F512*/
  0x14BC, 0x0020, 0x0002, 0x0001, /*1F513*/
  0x14BD, 0x0020, 0x0002, 0x0001, /*1F514*/
  0x14BE, 0x0020, 0x0002, 0x0001, /*1F515*/
  0x14BF, 0x0020, 0x0002, 0x0001, /*1F516*/
  0x14C0, 0x0020, 0x0002, 0x0001, /*1F517*/
  0x14C1, 0x0020, 0x0002, 0x0001, /*1F518*/
  0x14C2, 0x0020, 0x0002, 0x0001, /*1F519*/
  0x14C3, 0x0020, 0x0002, 0x0001, /*1F51A*/
  0x14C4, 0x0020, 0x0002, 0x0001, /*1F51B*/
  0x14C5, 0x0020, 0x0002, 0x0001, /*1F51C*/
  0x14C6, 0x0020, 0x0002, 0x0001, /*1F51D*/
  0x14C7, 0x0020, 0x0002, 0x0001, /*1F51E*/
  0x14C8, 0x0020, 0x0002, 0x0001, /*1F51F*/
  0x14C9, 0x0020, 0x0002, 0x0001, /*1F520*/
  0x14CA, 0x0020, 0x0002, 0x0001, /*1F521*/
  0x14CB, 0x0020, 0x0002, 0x0001, /*1F522*/
  0x14CC, 0x0020, 0x0002, 0x0001, /*1F523*/
  0x14CD, 0x0020, 0x0002, 0x0001, /*1F524*/
  0x14CE, 0x0020, 0x0002, 0x0001, /*1F525*/
  0x14CF, 0x0020, 0x0002, 0x0001, /*1F526*/
  0x14D0, 0x0020, 0x0002, 0x0001, /*1F527*/
  0x14D1, 0x0020, 0x0002, 0x0001, /*1F528*/
  0x14D2, 0x0020, 0x0002, 0x0001, /*1F529*/
  0x14D3, 0x0020, 0x0002, 0x0001, /*1F52A*/
  0x14D4, 0x0020, 0x0002, 0x0001, /*1F52B*/
  0x14D5, 0x0020, 0x0002, 0x0001, /*1F52C*/
  0x14D6, 0x0020, 0x0002, 0x0001, /*1F52D*/
  0x14D7, 0x0020, 0x0002, 0x0001, /*1F52E*/
  0x14D8, 0x0020, 0x0002, 0x0001, /*1F52F*/
  0x14D9, 0x0020, 0x0002, 0x0001, /*1F530*/
  0x14DA, 0x0020, 0x0002, 0x0001, /*1F531*/
  0x14DB, 0x0020, 0x0002, 0x0001, /*1F532*/
  0x14DC, 0x0020, 0x0002, 0x0001, /*1F533*/
  0x14DD, 0x0020, 0x0002, 0x0001, /*1F534*/
  0x14DE, 0x0020, 0x0002, 0x0001, /*1F535*/
  0x14DF, 0x0020, 0x0002, 0x0001, /*1F536*/
  0x14E0, 0x0020, 0x0002, 0x0001, /*1F537*/
  0x14E1, 0x0020, 0x0002, 0x0001, /*1F538*/
  0x14E2, 0x0020, 0x0002, 0x0001, /*1F539*/
  0x14E3, 0x0020, 0x0002, 0x0001, /*1F53A*/
  0x14E4, 0x0020, 0x0002, 0x0001, /*1F53B*/
  0x14E5, 0x0020, 0x0002, 0x0001, /*1F53C*/
  0x14E6, 0x0020, 0x0002, 0x0001, /*1F53D*/
  0x14E7, 0x0020, 0x0002, 0x0001, /*1F53E*/
  0x14E8, 0x0020, 0x0002, 0x0001, /*1F53F*/
  0x14E9, 0x0020, 0x0002, 0x0001, /*1F540*/
  0x14EA, 0x0020, 0x0002, 0x0001, /*1F541*/
  0x14EB, 0x0020, 0x0002, 0x0001, /*1F542*/
  0x14EC, 0x0020, 0x0002, 0x0001, /*1F543*/
  0x14ED, 0x0020, 0x0002, 0x0001, /*1F544*/
  0x14EE, 0x0020, 0x0002, 0x0001, /*1F545*/
  0x14EF, 0x0020, 0x0002, 0x0001, /*1F546*/
  0x14F0, 0x0020, 0x0002, 0x0001, /*1F547*/
  0x14F1, 0x0020, 0x0002, 0x0001, /*1F548*/
  0x14F2, 0x0020, 0x0002, 0x0001, /*1F549*/
  0x14F3, 0x0020, 0x0002, 0x0001, /*1F54A*/
  0x14F4, 0x0020, 0x0002, 0x0001, /*1F54B*/
  0x14F5, 0x0020, 0x0002, 0x0001, /*1F54C*/
  0x14F6, 0x0020, 0x0002, 0x0001, /*1F54D*/
  0x14F7, 0x0020, 0x0002, 0x0001, /*1F54E*/
  0x14F8, 0x0020, 0x0002, 0x0001, /*1F54F*/
  0x14F9, 0x0020, 0x0002, 0x0001, /*1F550*/
  0x14FA, 0x0020, 0x0002, 0x0001, /*1F551*/
  0x14FB, 0x0020, 0x0002, 0x0001, /*1F552*/
  0x14FC, 0x0020, 0x0002, 0x0001, /*1F553*/
  0x14FD, 0x0020, 0x0002, 0x0001, /*1F554*/
  0x14FE, 0x0020, 0x0002, 0x0001, /*1F555*/
  0x14FF, 0x0020, 0x0002, 0x0001, /*1F556*/
  0x1500, 0x0020, 0x0002, 0x0001, /*1F557*/
  0x1501, 0x0020, 0x0002, 0x0001, /*1F558*/
  0x1502, 0x0020, 0x0002, 0x0001, /*1F559*/
  0x1503, 0x0020, 0x0002, 0x0001, /*1F55A*/
  0x1504, 0x0020, 0x0002, 0x0001, /*1F55B*/
  0x1505, 0x0020, 0x0002, 0x0001, /*1F55C*/
  0x1506, 0x0020, 0x0002, 0x0001, /*1F55D*/
  0x1507, 0x0020, 0x0002, 0x0001, /*1F55E*/
  0x1508, 0x0020, 0x0002, 0x0001, /*1F55F*/
  0x1509, 0x0020, 0x0002, 0x0001, /*1F560*/
  0x150A, 0x0020, 0x0002, 0x0001, /*1F561*/
  0x150B, 0x0020, 0x0002, 0x0001, /*1F562*/
  0x150C, 0x0020, 0x0002, 0x0001, /*1F563*/
  0x150D, 0x0020, 0x0002, 0x0001, /*1F564*/
  0x150E, 0x0020, 0x0002, 0x0001, /*1F565*/
  0x150F, 0x0020, 0x0002, 0x0001, /*1F566*/
  0x1510, 0x0020, 0x0002, 0x0001, /*1F567*/
  0x1511, 0x0020, 0x0002, 0x0001, /*1F568*/
  0x1512, 0x0020, 0x0002, 0x0001, /*1F569*/
  0x1513, 0x0020, 0x0002, 0x0001, /*1F56A*/
  0x1514, 0x0020, 0x0002, 0x0001, /*1F56B*/
  0x1515, 0x0020, 0x0002, 0x0001, /*1F56C*/
  0x1516, 0x0020, 0x0002, 0x0001, /*1F56D*/
  0x1517, 0x0020, 0x0002, 0x0001, /*1F56E*/
  0x1518, 0x0020, 0x0002, 0x0001, /*1F56F*/
  0x1519, 0x0020, 0x0002, 0x0001, /*1F570*/
  0x151A, 0x0020, 0x0002, 0x0001, /*1F571*/
  0x151B, 0x0020, 0x0002, 0x0001, /*1F572*/
  0x151C, 0x0020, 0x0002, 0x0001, /*1F573*/
  0x151D, 0x0020, 0x0002, 0x0001, /*1F574*/
  0x151E, 0x0020, 0x0002, 0x0001, /*1F575*/
  0x151F, 0x0020, 0x0002, 0x0001, /*1F576*/
  0x1520, 0x0020, 0x0002, 0x0001, /*1F577*/
  0x1521, 0x0020, 0x0002, 0x0001, /*1F578*/
  0x1522, 0x0020, 0x0002, 0x0001, /*1F579*/
  0x1523, 0x0020, 0x0002, 0x0001, /*1F57A*/
  0x1524, 0x0020, 0x0002, 0x0001, /*1F57B*/
  0x1525, 0x0020, 0x0002, 0x0001, /*1F57C*/
  0x1526, 0x0020, 0x0002, 0x0001, /*1F57D*/
  0x1527, 0x0020, 0x0002, 0x0001, /*1F57E*/
  0x1528, 0x0020, 0x0002, 0x0001, /*1F57F*/
  0x1529, 0x0020, 0x0002, 0x0001, /*1F580*/
  0x152A, 0x0020, 0x0002, 0x0001, /*1F581*/
  0x152B, 0x0020, 0x0002, 0x0001, /*1F582*/
  0x152C, 0x0020, 0x0002, 0x0001, /*1F583*/
  0x152D, 0x0020, 0x0002, 0x0001, /*1F584*/
  0x152E, 0x0020, 0x0002, 0x0001, /*1F585*/
  0x152F, 0x0020, 0x0002, 0x0001, /*1F586*/
  0x1530, 0x0020, 0x0002, 0x0001, /*1F587*/
  0x1531, 0x0020, 0x0002, 0x0001, /*1F588*/
  0x1532, 0x0020, 0x0002, 0x0001, /*1F589*/
  0x1533, 0x0020, 0x0002, 0x0001, /*1F58A*/
  0x1534, 0x0020, 0x0002, 0x0001, /*1F58B*/
  0x1535, 0x0020, 0x0002, 0x0001, /*1F58C*/
  0x1536, 0x0020, 0x0002, 0x0001, /*1F58D*/
  0x1537, 0x0020, 0x0002, 0x0001, /*1F58E*/
  0x1538, 0x0020, 0x0002, 0x0001, /*1F58F*/
  0x1539, 0x0020, 0x0002, 0x0001, /*1F590*/
  0x153A, 0x0020, 0x0002, 0x0001, /*1F591*/
  0x153B, 0x0020, 0x0002, 0x0001, /*1F592*/
  0x153C, 0x0020, 0x0002, 0x0001, /*1F593*/
  0x153D, 0x0020, 0x0002, 0x0001, /*1F594*/
  0x153E, 0x0020, 0x0002, 0x0001, /*1F595*/
  0x153F, 0x0020, 0x0002, 0x0001, /*1F596*/
  0x1540, 0x0020, 0x0002, 0x0001, /*1F597*/
  0x1541, 0x0020, 0x0002, 0x0001, /*1F598*/
  0x1542, 0x0020, 0x0002, 0x0001, /*1F599*/
  0x1543, 0x0020, 0x0002, 0x0001, /*1F59A*/
  0x1544, 0x0020, 0x0002, 0x0001, /*1F59B*/
  0x1545, 0x0020, 0x0002, 0x0001, /*1F59C*/
  0x1546, 0x0020, 0x0002, 0x0001, /*1F59D*/
  0x1547, 0x0020, 0x0002, 0x0001, /*1F59E*/
  0x1548, 0x0020, 0x0002, 0x0001, /*1F59F*/
  0x1549, 0x0020, 0x0002, 0x0001, /*1F5A0*/
  0x154A, 0x0020, 0x0002, 0x0001, /*1F5A1*/
  0x154B, 0x0020, 0x0002, 0x0001, /*1F5A2*/
  0x154C, 0x0020, 0x0002, 0x0001, /*1F5A3*/
  0x154D, 0x0020, 0x0002, 0x0001, /*1F5A4*/
  0x154E, 0x0020, 0x0002, 0x0001, /*1F5A5*/
  0x154F, 0x0020, 0x0002, 0x0001, /*1F5A6*/
  0x1550, 0x0020, 0x0002, 0x0001, /*1F5A7*/
  0x1551, 0x0020, 0x0002, 0x0001, /*1F5A8*/
  0x1552, 0x0020, 0x0002, 0x0001, /*1F5A9*/
  0x1553, 0x0020, 0x0002, 0x0001, /*1F5AA*/
  0x1554, 0x0020, 0x0002, 0x0001, /*1F5AB*/
  0x1555, 0x0020, 0x0002, 0x0001, /*1F5AC*/
  0x1556, 0x0020, 0x0002, 0x0001, /*1F5AD*/
  0x1557, 0x0020, 0x0002, 0x0001, /*1F5AE*/
  0x1558, 0x0020, 0x0002, 0x0001, /*1F5AF*/
  0x1559, 0x0020, 0x0002, 0x0001, /*1F5B0*/
  0x155A, 0x0020, 0x0002, 0x0001, /*1F5B1*/
  0x155B, 0x0020, 0x0002, 0x0001, /*1F5B2*/
  0x155C, 0x0020, 0x0002, 0x0001, /*1F5B3*/
  0x155D, 0x0020, 0x0002, 0x0001, /*1F5B4*/
  0x155E, 0x0020, 0x0002, 0x0001, /*1F5B5*/
  0x155F, 0x0020, 0x0002, 0x0001, /*1F5B6*/
  0x1560, 0x0020, 0x0002, 0x0001, /*1F5B7*/
  0x1561, 0x0020, 0x0002, 0x0001, /*1F5B8*/
  0x1562, 0x0020, 0x0002, 0x0001, /*1F5B9*/
  0x1563, 0x0020, 0x0002, 0x0001, /*1F5BA*/
  0x1564, 0x0020, 0x0002, 0x0001, /*1F5BB*/
  0x1565, 0x0020, 0x0002, 0x0001, /*1F5BC*/
  0x1566, 0x0020, 0x0002, 0x0001, /*1F5BD*/
  0x1567, 0x0020, 0x0002, 0x0001, /*1F5BE*/
  0x1568, 0x0020, 0x0002, 0x0001, /*1F5BF*/
  0x1569, 0x0020, 0x0002, 0x0001, /*1F5C0*/
  0x156A, 0x0020, 0x0002, 0x0001, /*1F5C1*/
  0x156B, 0x0020, 0x0002, 0x0001, /*1F5C2*/
  0x156C, 0x0020, 0x0002, 0x0001, /*1F5C3*/
  0x156D, 0x0020, 0x0002, 0x0001, /*1F5C4*/
  0x156E, 0x0020, 0x0002, 0x0001, /*1F5C5*/
  0x156F, 0x0020, 0x0002, 0x0001, /*1F5C6*/
  0x1570, 0x0020, 0x0002, 0x0001, /*1F5C7*/
  0x1571, 0x0020, 0x0002, 0x0001, /*1F5C8*/
  0x1572, 0x0020, 0x0002, 0x0001, /*1F5C9*/
  0x1573, 0x0020, 0x0002, 0x0001, /*1F5CA*/
  0x1574, 0x0020, 0x0002, 0x0001, /*1F5CB*/
  0x1575, 0x0020, 0x0002, 0x0001, /*1F5CC*/
  0x1576, 0x0020, 0x0002, 0x0001, /*1F5CD*/
  0x1577, 0x0020, 0x0002, 0x0001, /*1F5CE*/
  0x1578, 0x0020, 0x0002, 0x0001, /*1F5CF*/
  0x1579, 0x0020, 0x0002, 0x0001, /*1F5D0*/
  0x157A, 0x0020, 0x0002, 0x0001, /*1F5D1*/
  0x157B, 0x0020, 0x0002, 0x0001, /*1F5D2*/
  0x157C, 0x0020, 0x0002, 0x0001, /*1F5D3*/
  0x157D, 0x0020, 0x0002, 0x0001, /*1F5D4*/
  0x157E, 0x0020, 0x0002, 0x0001, /*1F5D5*/
  0x157F, 0x0020, 0x0002, 0x0001, /*1F5D6*/
  0x1580, 0x0020, 0x0002, 0x0001, /*1F5D7*/
  0x1581, 0x0020, 0x0002, 0x0001, /*1F5D8*/
  0x1582, 0x0020, 0x0002, 0x0001, /*1F5D9*/
  0x1583, 0x0020, 0x0002, 0x0001, /*1F5DA*/
  0x1584, 0x0020, 0x0002, 0x0001, /*1F5DB*/
  0x1585, 0x0020, 0x0002, 0x0001, /*1F5DC*/
  0x1586, 0x0020, 0x0002, 0x0001, /*1F5DD*/
  0x1587, 0x0020, 0x0002, 0x0001, /*1F5DE*/
  0x1588, 0x0020, 0x0002, 0x0001, /*1F5DF*/
  0x1589, 0x0020, 0x0002, 0x0001, /*1F5E0*/
  0x158A, 0x0020, 0x0002, 0x0001, /*1F5E1*/
  0x158B, 0x0020, 0x0002, 0x0001, /*1F5E2*/
  0x158C, 0x0020, 0x0002, 0x0001, /*1F5E3*/
  0x158D, 0x0020, 0x0002, 0x0001, /*1F5E4*/
  0x158E, 0x0020, 0x0002, 0x0001, /*1F5E5*/
  0x158F, 0x0020, 0x0002, 0x0001, /*1F5E6*/
  0x1590, 0x0020, 0x0002, 0x0001, /*1F5E7*/
  0x1591, 0x0020, 0x0002, 0x0001, /*1F5E8*/
  0x1592, 0x0020, 0x0002, 0x0001, /*1F5E9*/
  0x1593, 0x0020, 0x0002, 0x0001, /*1F5EA*/
  0x1594, 0x0020, 0x0002, 0x0001, /*1F5EB*/
  0x1595, 0x0020, 0x0002, 0x0001, /*1F5EC*/
  0x1596, 0x0020, 0x0002, 0x0001, /*1F5ED*/
  0x1597, 0x0020, 0x0002, 0x0001, /*1F5EE*/
  0x1598, 0x0020, 0x0002, 0x0001, /*1F5EF*/
  0x1599, 0x0020, 0x0002, 0x0001, /*1F5F0*/
  0x159A, 0x0020, 0x0002, 0x0001, /*1F5F1*/
  0x159B, 0x0020, 0x0002, 0x0001, /*1F5F2*/
  0x159C, 0x0020, 0x0002, 0x0001, /*1F5F3*/
  0x159D, 0x0020, 0x0002, 0x0001, /*1F5F4*/
  0x159E, 0x0020, 0x0002, 0x0001, /*1F5F5*/
  0x159F, 0x0020, 0x0002, 0x0001, /*1F5F6*/
  0x15A0, 0x0020, 0x0002, 0x0001, /*1F5F7*/
  0x15A1, 0x0020, 0x0002, 0x0001, /*1F5F8*/
  0x15A2, 0x0020, 0x0002, 0x0001, /*1F5F9*/
  0x15A3, 0x0020, 0x0002, 0x0001, /*1F5FA*/
  0x15A4, 0x0020, 0x0002, 0x0001, /*1F5FB*/
  0x15A5, 0x0020, 0x0002, 0x0001, /*1F5FC*/
  0x15A6, 0x0020, 0x0002, 0x0001, /*1F5FD*/
  0x15A7, 0x0020, 0x0002, 0x0001, /*1F5FE*/
  0x15A8, 0x0020, 0x0002, 0x0001  /*1F5FF*/
};

uint16 uca900_p1F6[]= {
  0x15FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F600*/
  0x15FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F601*/
  0x15FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F602*/
  0x15FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F603*/
  0x15FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F604*/
  0x1600, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F605*/
  0x1601, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F606*/
  0x1602, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F607*/
  0x1603, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F608*/
  0x1604, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F609*/
  0x1605, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F60A*/
  0x1606, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F60B*/
  0x1607, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F60C*/
  0x1608, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F60D*/
  0x1609, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F60E*/
  0x160A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F60F*/
  0x160B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F610*/
  0x160C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F611*/
  0x160D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F612*/
  0x160E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F613*/
  0x160F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F614*/
  0x1610, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F615*/
  0x1611, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F616*/
  0x1612, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F617*/
  0x1613, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F618*/
  0x1614, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F619*/
  0x1615, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F61A*/
  0x1616, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F61B*/
  0x1617, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F61C*/
  0x1618, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F61D*/
  0x1619, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F61E*/
  0x161A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F61F*/
  0x161B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F620*/
  0x161C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F621*/
  0x161D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F622*/
  0x161E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F623*/
  0x161F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F624*/
  0x1620, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F625*/
  0x1621, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F626*/
  0x1622, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F627*/
  0x1623, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F628*/
  0x1624, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F629*/
  0x1625, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F62A*/
  0x1626, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F62B*/
  0x1627, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F62C*/
  0x1628, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F62D*/
  0x1629, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F62E*/
  0x162A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F62F*/
  0x162B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F630*/
  0x162C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F631*/
  0x162D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F632*/
  0x162E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F633*/
  0x162F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F634*/
  0x1630, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F635*/
  0x1631, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F636*/
  0x1632, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F637*/
  0x1633, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F638*/
  0x1634, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F639*/
  0x1635, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F63A*/
  0x1636, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F63B*/
  0x1637, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F63C*/
  0x1638, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F63D*/
  0x1639, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F63E*/
  0x163A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F63F*/
  0x163B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F640*/
  0x163C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F641*/
  0x163D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F642*/
  0x163E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F643*/
  0x163F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F644*/
  0x1640, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F645*/
  0x1641, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F646*/
  0x1642, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F647*/
  0x1643, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F648*/
  0x1644, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F649*/
  0x1645, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F64A*/
  0x1646, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F64B*/
  0x1647, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F64C*/
  0x1648, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F64D*/
  0x1649, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F64E*/
  0x164A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F64F*/
  0x164B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F650*/
  0x164C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F651*/
  0x164D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F652*/
  0x164E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F653*/
  0x164F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F654*/
  0x1650, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F655*/
  0x1651, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F656*/
  0x1652, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F657*/
  0x1653, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F658*/
  0x1654, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F659*/
  0x1655, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F65A*/
  0x1656, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F65B*/
  0x1657, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F65C*/
  0x1658, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F65D*/
  0x1659, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F65E*/
  0x165A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F65F*/
  0x165B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F660*/
  0x165C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F661*/
  0x165D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F662*/
  0x165E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F663*/
  0x165F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F664*/
  0x1660, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F665*/
  0x1661, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F666*/
  0x1662, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F667*/
  0x1663, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F668*/
  0x1664, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F669*/
  0x1665, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F66A*/
  0x1666, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F66B*/
  0x1667, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F66C*/
  0x1668, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F66D*/
  0x1669, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F66E*/
  0x166A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F66F*/
  0x166B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F670*/
  0x166C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F671*/
  0x166D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F672*/
  0x166E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F673*/
  0x166F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F674*/
  0x1670, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F675*/
  0x1671, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F676*/
  0x1672, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F677*/
  0x1673, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F678*/
  0x1674, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F679*/
  0x1675, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F67A*/
  0x1676, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F67B*/
  0x1677, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F67C*/
  0x1678, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F67D*/
  0x1679, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F67E*/
  0x167A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F67F*/
  0x167B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F680*/
  0x167C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F681*/
  0x167D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F682*/
  0x167E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F683*/
  0x167F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F684*/
  0x1680, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F685*/
  0x1681, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F686*/
  0x1682, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F687*/
  0x1683, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F688*/
  0x1684, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F689*/
  0x1685, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F68A*/
  0x1686, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F68B*/
  0x1687, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F68C*/
  0x1688, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F68D*/
  0x1689, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F68E*/
  0x168A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F68F*/
  0x168B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F690*/
  0x168C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F691*/
  0x168D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F692*/
  0x168E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F693*/
  0x168F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F694*/
  0x1690, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F695*/
  0x1691, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F696*/
  0x1692, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F697*/
  0x1693, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F698*/
  0x1694, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F699*/
  0x1695, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F69A*/
  0x1696, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F69B*/
  0x1697, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F69C*/
  0x1698, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F69D*/
  0x1699, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F69E*/
  0x169A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F69F*/
  0x169B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6A0*/
  0x169C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6A1*/
  0x169D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6A2*/
  0x169E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6A3*/
  0x169F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6A4*/
  0x16A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6A5*/
  0x16A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6A6*/
  0x16A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6A7*/
  0x16A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6A8*/
  0x16A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6A9*/
  0x16A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6AA*/
  0x16A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6AB*/
  0x16A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6AC*/
  0x16A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6AD*/
  0x16A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6AE*/
  0x16AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6AF*/
  0x16AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6B0*/
  0x16AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6B1*/
  0x16AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6B2*/
  0x16AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6B3*/
  0x16AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6B4*/
  0x16B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6B5*/
  0x16B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6B6*/
  0x16B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6B7*/
  0x16B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6B8*/
  0x16B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6B9*/
  0x16B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6BA*/
  0x16B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6BB*/
  0x16B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6BC*/
  0x16B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6BD*/
  0x16B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6BE*/
  0x16BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6BF*/
  0x16BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6C0*/
  0x16BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6C1*/
  0x16BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6C2*/
  0x16BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6C3*/
  0x16BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6C4*/
  0x16C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6C5*/
  0x16C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6C6*/
  0x16C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6C7*/
  0x16C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6C8*/
  0x16C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6C9*/
  0x16C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6CA*/
  0x16C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6CB*/
  0x16C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6CC*/
  0x16C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6CD*/
  0x16C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6CE*/
  0x16CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6CF*/
  0x16CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6D0*/
  0x16CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6D1*/
  0x16CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6D2*/
  0xFBC3, 0x0020, 0x0002, 0xF6D3, 0x0000, 0x0000, 0x0002, /*1F6D3*/
  0xFBC3, 0x0020, 0x0002, 0xF6D4, 0x0000, 0x0000, 0x0002, /*1F6D4*/
  0xFBC3, 0x0020, 0x0002, 0xF6D5, 0x0000, 0x0000, 0x0002, /*1F6D5*/
  0xFBC3, 0x0020, 0x0002, 0xF6D6, 0x0000, 0x0000, 0x0002, /*1F6D6*/
  0xFBC3, 0x0020, 0x0002, 0xF6D7, 0x0000, 0x0000, 0x0002, /*1F6D7*/
  0xFBC3, 0x0020, 0x0002, 0xF6D8, 0x0000, 0x0000, 0x0002, /*1F6D8*/
  0xFBC3, 0x0020, 0x0002, 0xF6D9, 0x0000, 0x0000, 0x0002, /*1F6D9*/
  0xFBC3, 0x0020, 0x0002, 0xF6DA, 0x0000, 0x0000, 0x0002, /*1F6DA*/
  0xFBC3, 0x0020, 0x0002, 0xF6DB, 0x0000, 0x0000, 0x0002, /*1F6DB*/
  0xFBC3, 0x0020, 0x0002, 0xF6DC, 0x0000, 0x0000, 0x0002, /*1F6DC*/
  0xFBC3, 0x0020, 0x0002, 0xF6DD, 0x0000, 0x0000, 0x0002, /*1F6DD*/
  0xFBC3, 0x0020, 0x0002, 0xF6DE, 0x0000, 0x0000, 0x0002, /*1F6DE*/
  0xFBC3, 0x0020, 0x0002, 0xF6DF, 0x0000, 0x0000, 0x0002, /*1F6DF*/
  0x16CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6E0*/
  0x16CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6E1*/
  0x16D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6E2*/
  0x16D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6E3*/
  0x16D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6E4*/
  0x16D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6E5*/
  0x16D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6E6*/
  0x16D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6E7*/
  0x16D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6E8*/
  0x16D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6E9*/
  0x16D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6EA*/
  0x16D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6EB*/
  0x16DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6EC*/
  0xFBC3, 0x0020, 0x0002, 0xF6ED, 0x0000, 0x0000, 0x0002, /*1F6ED*/
  0xFBC3, 0x0020, 0x0002, 0xF6EE, 0x0000, 0x0000, 0x0002, /*1F6EE*/
  0xFBC3, 0x0020, 0x0002, 0xF6EF, 0x0000, 0x0000, 0x0002, /*1F6EF*/
  0x16DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6F0*/
  0x16DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6F1*/
  0x16DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6F2*/
  0x16DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6F3*/
  0x16DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6F4*/
  0x16E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6F5*/
  0x16E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F6F6*/
  0xFBC3, 0x0020, 0x0002, 0xF6F7, 0x0000, 0x0000, 0x0002, /*1F6F7*/
  0xFBC3, 0x0020, 0x0002, 0xF6F8, 0x0000, 0x0000, 0x0002, /*1F6F8*/
  0xFBC3, 0x0020, 0x0002, 0xF6F9, 0x0000, 0x0000, 0x0002, /*1F6F9*/
  0xFBC3, 0x0020, 0x0002, 0xF6FA, 0x0000, 0x0000, 0x0002, /*1F6FA*/
  0xFBC3, 0x0020, 0x0002, 0xF6FB, 0x0000, 0x0000, 0x0002, /*1F6FB*/
  0xFBC3, 0x0020, 0x0002, 0xF6FC, 0x0000, 0x0000, 0x0002, /*1F6FC*/
  0xFBC3, 0x0020, 0x0002, 0xF6FD, 0x0000, 0x0000, 0x0002, /*1F6FD*/
  0xFBC3, 0x0020, 0x0002, 0xF6FE, 0x0000, 0x0000, 0x0002, /*1F6FE*/
  0xFBC3, 0x0020, 0x0002, 0xF6FF, 0x0000, 0x0000, 0x0002  /*1F6FF*/
};

uint16 uca900_p1F7[]= {
  0x16E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F700*/
  0x16E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F701*/
  0x16E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F702*/
  0x16E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F703*/
  0x16E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F704*/
  0x16E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F705*/
  0x16E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F706*/
  0x16E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F707*/
  0x16EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F708*/
  0x16EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F709*/
  0x16EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F70A*/
  0x16ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F70B*/
  0x16EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F70C*/
  0x16EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F70D*/
  0x16F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F70E*/
  0x16F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F70F*/
  0x16F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F710*/
  0x16F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F711*/
  0x16F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F712*/
  0x16F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F713*/
  0x16F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F714*/
  0x16F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F715*/
  0x16F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F716*/
  0x16F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F717*/
  0x16FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F718*/
  0x16FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F719*/
  0x16FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F71A*/
  0x16FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F71B*/
  0x16FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F71C*/
  0x16FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F71D*/
  0x1700, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F71E*/
  0x1701, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F71F*/
  0x1702, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F720*/
  0x1703, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F721*/
  0x1704, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F722*/
  0x1705, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F723*/
  0x1706, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F724*/
  0x1707, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F725*/
  0x1708, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F726*/
  0x1709, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F727*/
  0x170A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F728*/
  0x170B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F729*/
  0x170C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F72A*/
  0x170D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F72B*/
  0x170E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F72C*/
  0x170F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F72D*/
  0x1710, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F72E*/
  0x1711, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F72F*/
  0x1712, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F730*/
  0x1713, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F731*/
  0x1714, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F732*/
  0x1715, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F733*/
  0x1716, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F734*/
  0x1717, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F735*/
  0x1718, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F736*/
  0x1719, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F737*/
  0x171A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F738*/
  0x171B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F739*/
  0x171C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F73A*/
  0x171D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F73B*/
  0x171E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F73C*/
  0x171F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F73D*/
  0x1720, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F73E*/
  0x1721, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F73F*/
  0x1722, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F740*/
  0x1723, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F741*/
  0x1724, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F742*/
  0x1725, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F743*/
  0x1726, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F744*/
  0x1727, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F745*/
  0x1728, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F746*/
  0x1729, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F747*/
  0x172A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F748*/
  0x172B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F749*/
  0x172C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F74A*/
  0x172D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F74B*/
  0x172E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F74C*/
  0x172F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F74D*/
  0x1730, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F74E*/
  0x1731, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F74F*/
  0x1732, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F750*/
  0x1733, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F751*/
  0x1734, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F752*/
  0x1735, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F753*/
  0x1736, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F754*/
  0x1737, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F755*/
  0x1738, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F756*/
  0x1739, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F757*/
  0x173A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F758*/
  0x173B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F759*/
  0x173C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F75A*/
  0x173D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F75B*/
  0x173E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F75C*/
  0x173F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F75D*/
  0x1740, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F75E*/
  0x1741, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F75F*/
  0x1742, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F760*/
  0x1743, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F761*/
  0x1744, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F762*/
  0x1745, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F763*/
  0x1746, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F764*/
  0x1747, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F765*/
  0x1748, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F766*/
  0x1749, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F767*/
  0x174A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F768*/
  0x174B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F769*/
  0x174C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F76A*/
  0x174D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F76B*/
  0x174E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F76C*/
  0x174F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F76D*/
  0x1750, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F76E*/
  0x1751, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F76F*/
  0x1752, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F770*/
  0x1753, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F771*/
  0x1754, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F772*/
  0x1755, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F773*/
  0xFBC3, 0x0020, 0x0002, 0xF774, 0x0000, 0x0000, 0x0002, /*1F774*/
  0xFBC3, 0x0020, 0x0002, 0xF775, 0x0000, 0x0000, 0x0002, /*1F775*/
  0xFBC3, 0x0020, 0x0002, 0xF776, 0x0000, 0x0000, 0x0002, /*1F776*/
  0xFBC3, 0x0020, 0x0002, 0xF777, 0x0000, 0x0000, 0x0002, /*1F777*/
  0xFBC3, 0x0020, 0x0002, 0xF778, 0x0000, 0x0000, 0x0002, /*1F778*/
  0xFBC3, 0x0020, 0x0002, 0xF779, 0x0000, 0x0000, 0x0002, /*1F779*/
  0xFBC3, 0x0020, 0x0002, 0xF77A, 0x0000, 0x0000, 0x0002, /*1F77A*/
  0xFBC3, 0x0020, 0x0002, 0xF77B, 0x0000, 0x0000, 0x0002, /*1F77B*/
  0xFBC3, 0x0020, 0x0002, 0xF77C, 0x0000, 0x0000, 0x0002, /*1F77C*/
  0xFBC3, 0x0020, 0x0002, 0xF77D, 0x0000, 0x0000, 0x0002, /*1F77D*/
  0xFBC3, 0x0020, 0x0002, 0xF77E, 0x0000, 0x0000, 0x0002, /*1F77E*/
  0xFBC3, 0x0020, 0x0002, 0xF77F, 0x0000, 0x0000, 0x0002, /*1F77F*/
  0x1756, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F780*/
  0x1757, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F781*/
  0x1758, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F782*/
  0x1759, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F783*/
  0x175A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F784*/
  0x175B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F785*/
  0x175C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F786*/
  0x175D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F787*/
  0x175E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F788*/
  0x175F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F789*/
  0x1760, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F78A*/
  0x1761, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F78B*/
  0x1762, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F78C*/
  0x1763, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F78D*/
  0x1764, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F78E*/
  0x1765, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F78F*/
  0x1766, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F790*/
  0x1767, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F791*/
  0x1768, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F792*/
  0x1769, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F793*/
  0x176A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F794*/
  0x176B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F795*/
  0x176C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F796*/
  0x176D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F797*/
  0x176E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F798*/
  0x176F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F799*/
  0x1770, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F79A*/
  0x1771, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F79B*/
  0x1772, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F79C*/
  0x1773, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F79D*/
  0x1774, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F79E*/
  0x1775, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F79F*/
  0x1776, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7A0*/
  0x1777, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7A1*/
  0x1778, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7A2*/
  0x1779, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7A3*/
  0x177A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7A4*/
  0x177B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7A5*/
  0x177C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7A6*/
  0x177D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7A7*/
  0x177E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7A8*/
  0x177F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7A9*/
  0x1780, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7AA*/
  0x1781, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7AB*/
  0x1782, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7AC*/
  0x1783, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7AD*/
  0x1784, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7AE*/
  0x1785, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7AF*/
  0x1786, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7B0*/
  0x1787, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7B1*/
  0x1788, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7B2*/
  0x1789, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7B3*/
  0x178A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7B4*/
  0x178B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7B5*/
  0x178C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7B6*/
  0x178D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7B7*/
  0x178E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7B8*/
  0x178F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7B9*/
  0x1790, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7BA*/
  0x1791, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7BB*/
  0x1792, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7BC*/
  0x1793, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7BD*/
  0x1794, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7BE*/
  0x1795, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7BF*/
  0x1796, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7C0*/
  0x1797, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7C1*/
  0x1798, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7C2*/
  0x1799, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7C3*/
  0x179A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7C4*/
  0x179B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7C5*/
  0x179C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7C6*/
  0x179D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7C7*/
  0x179E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7C8*/
  0x179F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7C9*/
  0x17A0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7CA*/
  0x17A1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7CB*/
  0x17A2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7CC*/
  0x17A3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7CD*/
  0x17A4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7CE*/
  0x17A5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7CF*/
  0x17A6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7D0*/
  0x17A7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7D1*/
  0x17A8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7D2*/
  0x17A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7D3*/
  0x17AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F7D4*/
  0xFBC3, 0x0020, 0x0002, 0xF7D5, 0x0000, 0x0000, 0x0002, /*1F7D5*/
  0xFBC3, 0x0020, 0x0002, 0xF7D6, 0x0000, 0x0000, 0x0002, /*1F7D6*/
  0xFBC3, 0x0020, 0x0002, 0xF7D7, 0x0000, 0x0000, 0x0002, /*1F7D7*/
  0xFBC3, 0x0020, 0x0002, 0xF7D8, 0x0000, 0x0000, 0x0002, /*1F7D8*/
  0xFBC3, 0x0020, 0x0002, 0xF7D9, 0x0000, 0x0000, 0x0002, /*1F7D9*/
  0xFBC3, 0x0020, 0x0002, 0xF7DA, 0x0000, 0x0000, 0x0002, /*1F7DA*/
  0xFBC3, 0x0020, 0x0002, 0xF7DB, 0x0000, 0x0000, 0x0002, /*1F7DB*/
  0xFBC3, 0x0020, 0x0002, 0xF7DC, 0x0000, 0x0000, 0x0002, /*1F7DC*/
  0xFBC3, 0x0020, 0x0002, 0xF7DD, 0x0000, 0x0000, 0x0002, /*1F7DD*/
  0xFBC3, 0x0020, 0x0002, 0xF7DE, 0x0000, 0x0000, 0x0002, /*1F7DE*/
  0xFBC3, 0x0020, 0x0002, 0xF7DF, 0x0000, 0x0000, 0x0002, /*1F7DF*/
  0xFBC3, 0x0020, 0x0002, 0xF7E0, 0x0000, 0x0000, 0x0002, /*1F7E0*/
  0xFBC3, 0x0020, 0x0002, 0xF7E1, 0x0000, 0x0000, 0x0002, /*1F7E1*/
  0xFBC3, 0x0020, 0x0002, 0xF7E2, 0x0000, 0x0000, 0x0002, /*1F7E2*/
  0xFBC3, 0x0020, 0x0002, 0xF7E3, 0x0000, 0x0000, 0x0002, /*1F7E3*/
  0xFBC3, 0x0020, 0x0002, 0xF7E4, 0x0000, 0x0000, 0x0002, /*1F7E4*/
  0xFBC3, 0x0020, 0x0002, 0xF7E5, 0x0000, 0x0000, 0x0002, /*1F7E5*/
  0xFBC3, 0x0020, 0x0002, 0xF7E6, 0x0000, 0x0000, 0x0002, /*1F7E6*/
  0xFBC3, 0x0020, 0x0002, 0xF7E7, 0x0000, 0x0000, 0x0002, /*1F7E7*/
  0xFBC3, 0x0020, 0x0002, 0xF7E8, 0x0000, 0x0000, 0x0002, /*1F7E8*/
  0xFBC3, 0x0020, 0x0002, 0xF7E9, 0x0000, 0x0000, 0x0002, /*1F7E9*/
  0xFBC3, 0x0020, 0x0002, 0xF7EA, 0x0000, 0x0000, 0x0002, /*1F7EA*/
  0xFBC3, 0x0020, 0x0002, 0xF7EB, 0x0000, 0x0000, 0x0002, /*1F7EB*/
  0xFBC3, 0x0020, 0x0002, 0xF7EC, 0x0000, 0x0000, 0x0002, /*1F7EC*/
  0xFBC3, 0x0020, 0x0002, 0xF7ED, 0x0000, 0x0000, 0x0002, /*1F7ED*/
  0xFBC3, 0x0020, 0x0002, 0xF7EE, 0x0000, 0x0000, 0x0002, /*1F7EE*/
  0xFBC3, 0x0020, 0x0002, 0xF7EF, 0x0000, 0x0000, 0x0002, /*1F7EF*/
  0xFBC3, 0x0020, 0x0002, 0xF7F0, 0x0000, 0x0000, 0x0002, /*1F7F0*/
  0xFBC3, 0x0020, 0x0002, 0xF7F1, 0x0000, 0x0000, 0x0002, /*1F7F1*/
  0xFBC3, 0x0020, 0x0002, 0xF7F2, 0x0000, 0x0000, 0x0002, /*1F7F2*/
  0xFBC3, 0x0020, 0x0002, 0xF7F3, 0x0000, 0x0000, 0x0002, /*1F7F3*/
  0xFBC3, 0x0020, 0x0002, 0xF7F4, 0x0000, 0x0000, 0x0002, /*1F7F4*/
  0xFBC3, 0x0020, 0x0002, 0xF7F5, 0x0000, 0x0000, 0x0002, /*1F7F5*/
  0xFBC3, 0x0020, 0x0002, 0xF7F6, 0x0000, 0x0000, 0x0002, /*1F7F6*/
  0xFBC3, 0x0020, 0x0002, 0xF7F7, 0x0000, 0x0000, 0x0002, /*1F7F7*/
  0xFBC3, 0x0020, 0x0002, 0xF7F8, 0x0000, 0x0000, 0x0002, /*1F7F8*/
  0xFBC3, 0x0020, 0x0002, 0xF7F9, 0x0000, 0x0000, 0x0002, /*1F7F9*/
  0xFBC3, 0x0020, 0x0002, 0xF7FA, 0x0000, 0x0000, 0x0002, /*1F7FA*/
  0xFBC3, 0x0020, 0x0002, 0xF7FB, 0x0000, 0x0000, 0x0002, /*1F7FB*/
  0xFBC3, 0x0020, 0x0002, 0xF7FC, 0x0000, 0x0000, 0x0002, /*1F7FC*/
  0xFBC3, 0x0020, 0x0002, 0xF7FD, 0x0000, 0x0000, 0x0002, /*1F7FD*/
  0xFBC3, 0x0020, 0x0002, 0xF7FE, 0x0000, 0x0000, 0x0002, /*1F7FE*/
  0xFBC3, 0x0020, 0x0002, 0xF7FF, 0x0000, 0x0000, 0x0002  /*1F7FF*/
};

uint16 uca900_p1F8[]= {
  0x17AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F800*/
  0x17AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F801*/
  0x17AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F802*/
  0x17AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F803*/
  0x17AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F804*/
  0x17B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F805*/
  0x17B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F806*/
  0x17B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F807*/
  0x17B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F808*/
  0x17B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F809*/
  0x17B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F80A*/
  0x17B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F80B*/
  0xFBC3, 0x0020, 0x0002, 0xF80C, 0x0000, 0x0000, 0x0002, /*1F80C*/
  0xFBC3, 0x0020, 0x0002, 0xF80D, 0x0000, 0x0000, 0x0002, /*1F80D*/
  0xFBC3, 0x0020, 0x0002, 0xF80E, 0x0000, 0x0000, 0x0002, /*1F80E*/
  0xFBC3, 0x0020, 0x0002, 0xF80F, 0x0000, 0x0000, 0x0002, /*1F80F*/
  0x17B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F810*/
  0x17B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F811*/
  0x17B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F812*/
  0x17BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F813*/
  0x17BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F814*/
  0x17BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F815*/
  0x17BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F816*/
  0x17BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F817*/
  0x17BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F818*/
  0x17C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F819*/
  0x17C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F81A*/
  0x17C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F81B*/
  0x17C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F81C*/
  0x17C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F81D*/
  0x17C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F81E*/
  0x17C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F81F*/
  0x17C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F820*/
  0x17C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F821*/
  0x17C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F822*/
  0x17CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F823*/
  0x17CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F824*/
  0x17CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F825*/
  0x17CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F826*/
  0x17CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F827*/
  0x17CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F828*/
  0x17D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F829*/
  0x17D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F82A*/
  0x17D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F82B*/
  0x17D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F82C*/
  0x17D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F82D*/
  0x17D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F82E*/
  0x17D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F82F*/
  0x17D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F830*/
  0x17D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F831*/
  0x17D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F832*/
  0x17DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F833*/
  0x17DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F834*/
  0x17DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F835*/
  0x17DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F836*/
  0x17DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F837*/
  0x17DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F838*/
  0x17E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F839*/
  0x17E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F83A*/
  0x17E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F83B*/
  0x17E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F83C*/
  0x17E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F83D*/
  0x17E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F83E*/
  0x17E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F83F*/
  0x17E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F840*/
  0x17E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F841*/
  0x17E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F842*/
  0x17EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F843*/
  0x17EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F844*/
  0x17EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F845*/
  0x17ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F846*/
  0x17EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F847*/
  0xFBC3, 0x0020, 0x0002, 0xF848, 0x0000, 0x0000, 0x0002, /*1F848*/
  0xFBC3, 0x0020, 0x0002, 0xF849, 0x0000, 0x0000, 0x0002, /*1F849*/
  0xFBC3, 0x0020, 0x0002, 0xF84A, 0x0000, 0x0000, 0x0002, /*1F84A*/
  0xFBC3, 0x0020, 0x0002, 0xF84B, 0x0000, 0x0000, 0x0002, /*1F84B*/
  0xFBC3, 0x0020, 0x0002, 0xF84C, 0x0000, 0x0000, 0x0002, /*1F84C*/
  0xFBC3, 0x0020, 0x0002, 0xF84D, 0x0000, 0x0000, 0x0002, /*1F84D*/
  0xFBC3, 0x0020, 0x0002, 0xF84E, 0x0000, 0x0000, 0x0002, /*1F84E*/
  0xFBC3, 0x0020, 0x0002, 0xF84F, 0x0000, 0x0000, 0x0002, /*1F84F*/
  0x17EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F850*/
  0x17F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F851*/
  0x17F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F852*/
  0x17F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F853*/
  0x17F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F854*/
  0x17F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F855*/
  0x17F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F856*/
  0x17F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F857*/
  0x17F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F858*/
  0x17F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F859*/
  0xFBC3, 0x0020, 0x0002, 0xF85A, 0x0000, 0x0000, 0x0002, /*1F85A*/
  0xFBC3, 0x0020, 0x0002, 0xF85B, 0x0000, 0x0000, 0x0002, /*1F85B*/
  0xFBC3, 0x0020, 0x0002, 0xF85C, 0x0000, 0x0000, 0x0002, /*1F85C*/
  0xFBC3, 0x0020, 0x0002, 0xF85D, 0x0000, 0x0000, 0x0002, /*1F85D*/
  0xFBC3, 0x0020, 0x0002, 0xF85E, 0x0000, 0x0000, 0x0002, /*1F85E*/
  0xFBC3, 0x0020, 0x0002, 0xF85F, 0x0000, 0x0000, 0x0002, /*1F85F*/
  0x17F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F860*/
  0x17FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F861*/
  0x17FB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F862*/
  0x17FC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F863*/
  0x17FD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F864*/
  0x17FE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F865*/
  0x17FF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F866*/
  0x1800, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F867*/
  0x1801, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F868*/
  0x1802, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F869*/
  0x1803, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F86A*/
  0x1804, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F86B*/
  0x1805, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F86C*/
  0x1806, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F86D*/
  0x1807, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F86E*/
  0x1808, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F86F*/
  0x1809, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F870*/
  0x180A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F871*/
  0x180B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F872*/
  0x180C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F873*/
  0x180D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F874*/
  0x180E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F875*/
  0x180F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F876*/
  0x1810, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F877*/
  0x1811, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F878*/
  0x1812, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F879*/
  0x1813, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F87A*/
  0x1814, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F87B*/
  0x1815, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F87C*/
  0x1816, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F87D*/
  0x1817, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F87E*/
  0x1818, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F87F*/
  0x1819, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F880*/
  0x181A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F881*/
  0x181B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F882*/
  0x181C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F883*/
  0x181D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F884*/
  0x181E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F885*/
  0x181F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F886*/
  0x1820, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F887*/
  0xFBC3, 0x0020, 0x0002, 0xF888, 0x0000, 0x0000, 0x0002, /*1F888*/
  0xFBC3, 0x0020, 0x0002, 0xF889, 0x0000, 0x0000, 0x0002, /*1F889*/
  0xFBC3, 0x0020, 0x0002, 0xF88A, 0x0000, 0x0000, 0x0002, /*1F88A*/
  0xFBC3, 0x0020, 0x0002, 0xF88B, 0x0000, 0x0000, 0x0002, /*1F88B*/
  0xFBC3, 0x0020, 0x0002, 0xF88C, 0x0000, 0x0000, 0x0002, /*1F88C*/
  0xFBC3, 0x0020, 0x0002, 0xF88D, 0x0000, 0x0000, 0x0002, /*1F88D*/
  0xFBC3, 0x0020, 0x0002, 0xF88E, 0x0000, 0x0000, 0x0002, /*1F88E*/
  0xFBC3, 0x0020, 0x0002, 0xF88F, 0x0000, 0x0000, 0x0002, /*1F88F*/
  0x1821, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F890*/
  0x1822, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F891*/
  0x1823, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F892*/
  0x1824, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F893*/
  0x1825, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F894*/
  0x1826, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F895*/
  0x1827, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F896*/
  0x1828, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F897*/
  0x1829, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F898*/
  0x182A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F899*/
  0x182B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F89A*/
  0x182C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F89B*/
  0x182D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F89C*/
  0x182E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F89D*/
  0x182F, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F89E*/
  0x1830, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F89F*/
  0x1831, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8A0*/
  0x1832, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8A1*/
  0x1833, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8A2*/
  0x1834, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8A3*/
  0x1835, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8A4*/
  0x1836, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8A5*/
  0x1837, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8A6*/
  0x1838, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8A7*/
  0x1839, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8A8*/
  0x183A, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8A9*/
  0x183B, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8AA*/
  0x183C, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8AB*/
  0x183D, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8AC*/
  0x183E, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F8AD*/
  0xFBC3, 0x0020, 0x0002, 0xF8AE, 0x0000, 0x0000, 0x0002, /*1F8AE*/
  0xFBC3, 0x0020, 0x0002, 0xF8AF, 0x0000, 0x0000, 0x0002, /*1F8AF*/
  0xFBC3, 0x0020, 0x0002, 0xF8B0, 0x0000, 0x0000, 0x0002, /*1F8B0*/
  0xFBC3, 0x0020, 0x0002, 0xF8B1, 0x0000, 0x0000, 0x0002, /*1F8B1*/
  0xFBC3, 0x0020, 0x0002, 0xF8B2, 0x0000, 0x0000, 0x0002, /*1F8B2*/
  0xFBC3, 0x0020, 0x0002, 0xF8B3, 0x0000, 0x0000, 0x0002, /*1F8B3*/
  0xFBC3, 0x0020, 0x0002, 0xF8B4, 0x0000, 0x0000, 0x0002, /*1F8B4*/
  0xFBC3, 0x0020, 0x0002, 0xF8B5, 0x0000, 0x0000, 0x0002, /*1F8B5*/
  0xFBC3, 0x0020, 0x0002, 0xF8B6, 0x0000, 0x0000, 0x0002, /*1F8B6*/
  0xFBC3, 0x0020, 0x0002, 0xF8B7, 0x0000, 0x0000, 0x0002, /*1F8B7*/
  0xFBC3, 0x0020, 0x0002, 0xF8B8, 0x0000, 0x0000, 0x0002, /*1F8B8*/
  0xFBC3, 0x0020, 0x0002, 0xF8B9, 0x0000, 0x0000, 0x0002, /*1F8B9*/
  0xFBC3, 0x0020, 0x0002, 0xF8BA, 0x0000, 0x0000, 0x0002, /*1F8BA*/
  0xFBC3, 0x0020, 0x0002, 0xF8BB, 0x0000, 0x0000, 0x0002, /*1F8BB*/
  0xFBC3, 0x0020, 0x0002, 0xF8BC, 0x0000, 0x0000, 0x0002, /*1F8BC*/
  0xFBC3, 0x0020, 0x0002, 0xF8BD, 0x0000, 0x0000, 0x0002, /*1F8BD*/
  0xFBC3, 0x0020, 0x0002, 0xF8BE, 0x0000, 0x0000, 0x0002, /*1F8BE*/
  0xFBC3, 0x0020, 0x0002, 0xF8BF, 0x0000, 0x0000, 0x0002, /*1F8BF*/
  0xFBC3, 0x0020, 0x0002, 0xF8C0, 0x0000, 0x0000, 0x0002, /*1F8C0*/
  0xFBC3, 0x0020, 0x0002, 0xF8C1, 0x0000, 0x0000, 0x0002, /*1F8C1*/
  0xFBC3, 0x0020, 0x0002, 0xF8C2, 0x0000, 0x0000, 0x0002, /*1F8C2*/
  0xFBC3, 0x0020, 0x0002, 0xF8C3, 0x0000, 0x0000, 0x0002, /*1F8C3*/
  0xFBC3, 0x0020, 0x0002, 0xF8C4, 0x0000, 0x0000, 0x0002, /*1F8C4*/
  0xFBC3, 0x0020, 0x0002, 0xF8C5, 0x0000, 0x0000, 0x0002, /*1F8C5*/
  0xFBC3, 0x0020, 0x0002, 0xF8C6, 0x0000, 0x0000, 0x0002, /*1F8C6*/
  0xFBC3, 0x0020, 0x0002, 0xF8C7, 0x0000, 0x0000, 0x0002, /*1F8C7*/
  0xFBC3, 0x0020, 0x0002, 0xF8C8, 0x0000, 0x0000, 0x0002, /*1F8C8*/
  0xFBC3, 0x0020, 0x0002, 0xF8C9, 0x0000, 0x0000, 0x0002, /*1F8C9*/
  0xFBC3, 0x0020, 0x0002, 0xF8CA, 0x0000, 0x0000, 0x0002, /*1F8CA*/
  0xFBC3, 0x0020, 0x0002, 0xF8CB, 0x0000, 0x0000, 0x0002, /*1F8CB*/
  0xFBC3, 0x0020, 0x0002, 0xF8CC, 0x0000, 0x0000, 0x0002, /*1F8CC*/
  0xFBC3, 0x0020, 0x0002, 0xF8CD, 0x0000, 0x0000, 0x0002, /*1F8CD*/
  0xFBC3, 0x0020, 0x0002, 0xF8CE, 0x0000, 0x0000, 0x0002, /*1F8CE*/
  0xFBC3, 0x0020, 0x0002, 0xF8CF, 0x0000, 0x0000, 0x0002, /*1F8CF*/
  0xFBC3, 0x0020, 0x0002, 0xF8D0, 0x0000, 0x0000, 0x0002, /*1F8D0*/
  0xFBC3, 0x0020, 0x0002, 0xF8D1, 0x0000, 0x0000, 0x0002, /*1F8D1*/
  0xFBC3, 0x0020, 0x0002, 0xF8D2, 0x0000, 0x0000, 0x0002, /*1F8D2*/
  0xFBC3, 0x0020, 0x0002, 0xF8D3, 0x0000, 0x0000, 0x0002, /*1F8D3*/
  0xFBC3, 0x0020, 0x0002, 0xF8D4, 0x0000, 0x0000, 0x0002, /*1F8D4*/
  0xFBC3, 0x0020, 0x0002, 0xF8D5, 0x0000, 0x0000, 0x0002, /*1F8D5*/
  0xFBC3, 0x0020, 0x0002, 0xF8D6, 0x0000, 0x0000, 0x0002, /*1F8D6*/
  0xFBC3, 0x0020, 0x0002, 0xF8D7, 0x0000, 0x0000, 0x0002, /*1F8D7*/
  0xFBC3, 0x0020, 0x0002, 0xF8D8, 0x0000, 0x0000, 0x0002, /*1F8D8*/
  0xFBC3, 0x0020, 0x0002, 0xF8D9, 0x0000, 0x0000, 0x0002, /*1F8D9*/
  0xFBC3, 0x0020, 0x0002, 0xF8DA, 0x0000, 0x0000, 0x0002, /*1F8DA*/
  0xFBC3, 0x0020, 0x0002, 0xF8DB, 0x0000, 0x0000, 0x0002, /*1F8DB*/
  0xFBC3, 0x0020, 0x0002, 0xF8DC, 0x0000, 0x0000, 0x0002, /*1F8DC*/
  0xFBC3, 0x0020, 0x0002, 0xF8DD, 0x0000, 0x0000, 0x0002, /*1F8DD*/
  0xFBC3, 0x0020, 0x0002, 0xF8DE, 0x0000, 0x0000, 0x0002, /*1F8DE*/
  0xFBC3, 0x0020, 0x0002, 0xF8DF, 0x0000, 0x0000, 0x0002, /*1F8DF*/
  0xFBC3, 0x0020, 0x0002, 0xF8E0, 0x0000, 0x0000, 0x0002, /*1F8E0*/
  0xFBC3, 0x0020, 0x0002, 0xF8E1, 0x0000, 0x0000, 0x0002, /*1F8E1*/
  0xFBC3, 0x0020, 0x0002, 0xF8E2, 0x0000, 0x0000, 0x0002, /*1F8E2*/
  0xFBC3, 0x0020, 0x0002, 0xF8E3, 0x0000, 0x0000, 0x0002, /*1F8E3*/
  0xFBC3, 0x0020, 0x0002, 0xF8E4, 0x0000, 0x0000, 0x0002, /*1F8E4*/
  0xFBC3, 0x0020, 0x0002, 0xF8E5, 0x0000, 0x0000, 0x0002, /*1F8E5*/
  0xFBC3, 0x0020, 0x0002, 0xF8E6, 0x0000, 0x0000, 0x0002, /*1F8E6*/
  0xFBC3, 0x0020, 0x0002, 0xF8E7, 0x0000, 0x0000, 0x0002, /*1F8E7*/
  0xFBC3, 0x0020, 0x0002, 0xF8E8, 0x0000, 0x0000, 0x0002, /*1F8E8*/
  0xFBC3, 0x0020, 0x0002, 0xF8E9, 0x0000, 0x0000, 0x0002, /*1F8E9*/
  0xFBC3, 0x0020, 0x0002, 0xF8EA, 0x0000, 0x0000, 0x0002, /*1F8EA*/
  0xFBC3, 0x0020, 0x0002, 0xF8EB, 0x0000, 0x0000, 0x0002, /*1F8EB*/
  0xFBC3, 0x0020, 0x0002, 0xF8EC, 0x0000, 0x0000, 0x0002, /*1F8EC*/
  0xFBC3, 0x0020, 0x0002, 0xF8ED, 0x0000, 0x0000, 0x0002, /*1F8ED*/
  0xFBC3, 0x0020, 0x0002, 0xF8EE, 0x0000, 0x0000, 0x0002, /*1F8EE*/
  0xFBC3, 0x0020, 0x0002, 0xF8EF, 0x0000, 0x0000, 0x0002, /*1F8EF*/
  0xFBC3, 0x0020, 0x0002, 0xF8F0, 0x0000, 0x0000, 0x0002, /*1F8F0*/
  0xFBC3, 0x0020, 0x0002, 0xF8F1, 0x0000, 0x0000, 0x0002, /*1F8F1*/
  0xFBC3, 0x0020, 0x0002, 0xF8F2, 0x0000, 0x0000, 0x0002, /*1F8F2*/
  0xFBC3, 0x0020, 0x0002, 0xF8F3, 0x0000, 0x0000, 0x0002, /*1F8F3*/
  0xFBC3, 0x0020, 0x0002, 0xF8F4, 0x0000, 0x0000, 0x0002, /*1F8F4*/
  0xFBC3, 0x0020, 0x0002, 0xF8F5, 0x0000, 0x0000, 0x0002, /*1F8F5*/
  0xFBC3, 0x0020, 0x0002, 0xF8F6, 0x0000, 0x0000, 0x0002, /*1F8F6*/
  0xFBC3, 0x0020, 0x0002, 0xF8F7, 0x0000, 0x0000, 0x0002, /*1F8F7*/
  0xFBC3, 0x0020, 0x0002, 0xF8F8, 0x0000, 0x0000, 0x0002, /*1F8F8*/
  0xFBC3, 0x0020, 0x0002, 0xF8F9, 0x0000, 0x0000, 0x0002, /*1F8F9*/
  0xFBC3, 0x0020, 0x0002, 0xF8FA, 0x0000, 0x0000, 0x0002, /*1F8FA*/
  0xFBC3, 0x0020, 0x0002, 0xF8FB, 0x0000, 0x0000, 0x0002, /*1F8FB*/
  0xFBC3, 0x0020, 0x0002, 0xF8FC, 0x0000, 0x0000, 0x0002, /*1F8FC*/
  0xFBC3, 0x0020, 0x0002, 0xF8FD, 0x0000, 0x0000, 0x0002, /*1F8FD*/
  0xFBC3, 0x0020, 0x0002, 0xF8FE, 0x0000, 0x0000, 0x0002, /*1F8FE*/
  0xFBC3, 0x0020, 0x0002, 0xF8FF, 0x0000, 0x0000, 0x0002  /*1F8FF*/
};

uint16 uca900_p1F9[]= {
  0xFBC3, 0x0020, 0x0002, 0xF900, 0x0000, 0x0000, 0x0002, /*1F900*/
  0xFBC3, 0x0020, 0x0002, 0xF901, 0x0000, 0x0000, 0x0002, /*1F901*/
  0xFBC3, 0x0020, 0x0002, 0xF902, 0x0000, 0x0000, 0x0002, /*1F902*/
  0xFBC3, 0x0020, 0x0002, 0xF903, 0x0000, 0x0000, 0x0002, /*1F903*/
  0xFBC3, 0x0020, 0x0002, 0xF904, 0x0000, 0x0000, 0x0002, /*1F904*/
  0xFBC3, 0x0020, 0x0002, 0xF905, 0x0000, 0x0000, 0x0002, /*1F905*/
  0xFBC3, 0x0020, 0x0002, 0xF906, 0x0000, 0x0000, 0x0002, /*1F906*/
  0xFBC3, 0x0020, 0x0002, 0xF907, 0x0000, 0x0000, 0x0002, /*1F907*/
  0xFBC3, 0x0020, 0x0002, 0xF908, 0x0000, 0x0000, 0x0002, /*1F908*/
  0xFBC3, 0x0020, 0x0002, 0xF909, 0x0000, 0x0000, 0x0002, /*1F909*/
  0xFBC3, 0x0020, 0x0002, 0xF90A, 0x0000, 0x0000, 0x0002, /*1F90A*/
  0xFBC3, 0x0020, 0x0002, 0xF90B, 0x0000, 0x0000, 0x0002, /*1F90B*/
  0xFBC3, 0x0020, 0x0002, 0xF90C, 0x0000, 0x0000, 0x0002, /*1F90C*/
  0xFBC3, 0x0020, 0x0002, 0xF90D, 0x0000, 0x0000, 0x0002, /*1F90D*/
  0xFBC3, 0x0020, 0x0002, 0xF90E, 0x0000, 0x0000, 0x0002, /*1F90E*/
  0xFBC3, 0x0020, 0x0002, 0xF90F, 0x0000, 0x0000, 0x0002, /*1F90F*/
  0x15A9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F910*/
  0x15AA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F911*/
  0x15AB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F912*/
  0x15AC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F913*/
  0x15AD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F914*/
  0x15AE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F915*/
  0x15AF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F916*/
  0x15B0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F917*/
  0x15B1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F918*/
  0x15B2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F919*/
  0x15B3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F91A*/
  0x15B4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F91B*/
  0x15B5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F91C*/
  0x15B6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F91D*/
  0x15B7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F91E*/
  0xFBC3, 0x0020, 0x0002, 0xF91F, 0x0000, 0x0000, 0x0002, /*1F91F*/
  0x15B8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F920*/
  0x15B9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F921*/
  0x15BA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F922*/
  0x15BB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F923*/
  0x15BC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F924*/
  0x15BD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F925*/
  0x15BE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F926*/
  0x15BF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F927*/
  0xFBC3, 0x0020, 0x0002, 0xF928, 0x0000, 0x0000, 0x0002, /*1F928*/
  0xFBC3, 0x0020, 0x0002, 0xF929, 0x0000, 0x0000, 0x0002, /*1F929*/
  0xFBC3, 0x0020, 0x0002, 0xF92A, 0x0000, 0x0000, 0x0002, /*1F92A*/
  0xFBC3, 0x0020, 0x0002, 0xF92B, 0x0000, 0x0000, 0x0002, /*1F92B*/
  0xFBC3, 0x0020, 0x0002, 0xF92C, 0x0000, 0x0000, 0x0002, /*1F92C*/
  0xFBC3, 0x0020, 0x0002, 0xF92D, 0x0000, 0x0000, 0x0002, /*1F92D*/
  0xFBC3, 0x0020, 0x0002, 0xF92E, 0x0000, 0x0000, 0x0002, /*1F92E*/
  0xFBC3, 0x0020, 0x0002, 0xF92F, 0x0000, 0x0000, 0x0002, /*1F92F*/
  0x15C0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F930*/
  0xFBC3, 0x0020, 0x0002, 0xF931, 0x0000, 0x0000, 0x0002, /*1F931*/
  0xFBC3, 0x0020, 0x0002, 0xF932, 0x0000, 0x0000, 0x0002, /*1F932*/
  0x15C1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F933*/
  0x15C2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F934*/
  0x15C3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F935*/
  0x15C4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F936*/
  0x15C5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F937*/
  0x15C6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F938*/
  0x15C7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F939*/
  0x15C8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F93A*/
  0x15C9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F93B*/
  0x15CA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F93C*/
  0x15CB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F93D*/
  0x15CC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F93E*/
  0xFBC3, 0x0020, 0x0002, 0xF93F, 0x0000, 0x0000, 0x0002, /*1F93F*/
  0x15CD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F940*/
  0x15CE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F941*/
  0x15CF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F942*/
  0x15D0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F943*/
  0x15D1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F944*/
  0x15D2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F945*/
  0x15D3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F946*/
  0x15D4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F947*/
  0x15D5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F948*/
  0x15D6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F949*/
  0x15D7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F94A*/
  0x15D8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F94B*/
  0xFBC3, 0x0020, 0x0002, 0xF94C, 0x0000, 0x0000, 0x0002, /*1F94C*/
  0xFBC3, 0x0020, 0x0002, 0xF94D, 0x0000, 0x0000, 0x0002, /*1F94D*/
  0xFBC3, 0x0020, 0x0002, 0xF94E, 0x0000, 0x0000, 0x0002, /*1F94E*/
  0xFBC3, 0x0020, 0x0002, 0xF94F, 0x0000, 0x0000, 0x0002, /*1F94F*/
  0x15D9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F950*/
  0x15DA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F951*/
  0x15DB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F952*/
  0x15DC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F953*/
  0x15DD, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F954*/
  0x15DE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F955*/
  0x15DF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F956*/
  0x15E0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F957*/
  0x15E1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F958*/
  0x15E2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F959*/
  0x15E3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F95A*/
  0x15E4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F95B*/
  0x15E5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F95C*/
  0x15E6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F95D*/
  0x15E7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F95E*/
  0xFBC3, 0x0020, 0x0002, 0xF95F, 0x0000, 0x0000, 0x0002, /*1F95F*/
  0xFBC3, 0x0020, 0x0002, 0xF960, 0x0000, 0x0000, 0x0002, /*1F960*/
  0xFBC3, 0x0020, 0x0002, 0xF961, 0x0000, 0x0000, 0x0002, /*1F961*/
  0xFBC3, 0x0020, 0x0002, 0xF962, 0x0000, 0x0000, 0x0002, /*1F962*/
  0xFBC3, 0x0020, 0x0002, 0xF963, 0x0000, 0x0000, 0x0002, /*1F963*/
  0xFBC3, 0x0020, 0x0002, 0xF964, 0x0000, 0x0000, 0x0002, /*1F964*/
  0xFBC3, 0x0020, 0x0002, 0xF965, 0x0000, 0x0000, 0x0002, /*1F965*/
  0xFBC3, 0x0020, 0x0002, 0xF966, 0x0000, 0x0000, 0x0002, /*1F966*/
  0xFBC3, 0x0020, 0x0002, 0xF967, 0x0000, 0x0000, 0x0002, /*1F967*/
  0xFBC3, 0x0020, 0x0002, 0xF968, 0x0000, 0x0000, 0x0002, /*1F968*/
  0xFBC3, 0x0020, 0x0002, 0xF969, 0x0000, 0x0000, 0x0002, /*1F969*/
  0xFBC3, 0x0020, 0x0002, 0xF96A, 0x0000, 0x0000, 0x0002, /*1F96A*/
  0xFBC3, 0x0020, 0x0002, 0xF96B, 0x0000, 0x0000, 0x0002, /*1F96B*/
  0xFBC3, 0x0020, 0x0002, 0xF96C, 0x0000, 0x0000, 0x0002, /*1F96C*/
  0xFBC3, 0x0020, 0x0002, 0xF96D, 0x0000, 0x0000, 0x0002, /*1F96D*/
  0xFBC3, 0x0020, 0x0002, 0xF96E, 0x0000, 0x0000, 0x0002, /*1F96E*/
  0xFBC3, 0x0020, 0x0002, 0xF96F, 0x0000, 0x0000, 0x0002, /*1F96F*/
  0xFBC3, 0x0020, 0x0002, 0xF970, 0x0000, 0x0000, 0x0002, /*1F970*/
  0xFBC3, 0x0020, 0x0002, 0xF971, 0x0000, 0x0000, 0x0002, /*1F971*/
  0xFBC3, 0x0020, 0x0002, 0xF972, 0x0000, 0x0000, 0x0002, /*1F972*/
  0xFBC3, 0x0020, 0x0002, 0xF973, 0x0000, 0x0000, 0x0002, /*1F973*/
  0xFBC3, 0x0020, 0x0002, 0xF974, 0x0000, 0x0000, 0x0002, /*1F974*/
  0xFBC3, 0x0020, 0x0002, 0xF975, 0x0000, 0x0000, 0x0002, /*1F975*/
  0xFBC3, 0x0020, 0x0002, 0xF976, 0x0000, 0x0000, 0x0002, /*1F976*/
  0xFBC3, 0x0020, 0x0002, 0xF977, 0x0000, 0x0000, 0x0002, /*1F977*/
  0xFBC3, 0x0020, 0x0002, 0xF978, 0x0000, 0x0000, 0x0002, /*1F978*/
  0xFBC3, 0x0020, 0x0002, 0xF979, 0x0000, 0x0000, 0x0002, /*1F979*/
  0xFBC3, 0x0020, 0x0002, 0xF97A, 0x0000, 0x0000, 0x0002, /*1F97A*/
  0xFBC3, 0x0020, 0x0002, 0xF97B, 0x0000, 0x0000, 0x0002, /*1F97B*/
  0xFBC3, 0x0020, 0x0002, 0xF97C, 0x0000, 0x0000, 0x0002, /*1F97C*/
  0xFBC3, 0x0020, 0x0002, 0xF97D, 0x0000, 0x0000, 0x0002, /*1F97D*/
  0xFBC3, 0x0020, 0x0002, 0xF97E, 0x0000, 0x0000, 0x0002, /*1F97E*/
  0xFBC3, 0x0020, 0x0002, 0xF97F, 0x0000, 0x0000, 0x0002, /*1F97F*/
  0x15E8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F980*/
  0x15E9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F981*/
  0x15EA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F982*/
  0x15EB, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F983*/
  0x15EC, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F984*/
  0x15ED, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F985*/
  0x15EE, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F986*/
  0x15EF, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F987*/
  0x15F0, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F988*/
  0x15F1, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F989*/
  0x15F2, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F98A*/
  0x15F3, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F98B*/
  0x15F4, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F98C*/
  0x15F5, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F98D*/
  0x15F6, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F98E*/
  0x15F7, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F98F*/
  0x15F8, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F990*/
  0x15F9, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F991*/
  0xFBC3, 0x0020, 0x0002, 0xF992, 0x0000, 0x0000, 0x0002, /*1F992*/
  0xFBC3, 0x0020, 0x0002, 0xF993, 0x0000, 0x0000, 0x0002, /*1F993*/
  0xFBC3, 0x0020, 0x0002, 0xF994, 0x0000, 0x0000, 0x0002, /*1F994*/
  0xFBC3, 0x0020, 0x0002, 0xF995, 0x0000, 0x0000, 0x0002, /*1F995*/
  0xFBC3, 0x0020, 0x0002, 0xF996, 0x0000, 0x0000, 0x0002, /*1F996*/
  0xFBC3, 0x0020, 0x0002, 0xF997, 0x0000, 0x0000, 0x0002, /*1F997*/
  0xFBC3, 0x0020, 0x0002, 0xF998, 0x0000, 0x0000, 0x0002, /*1F998*/
  0xFBC3, 0x0020, 0x0002, 0xF999, 0x0000, 0x0000, 0x0002, /*1F999*/
  0xFBC3, 0x0020, 0x0002, 0xF99A, 0x0000, 0x0000, 0x0002, /*1F99A*/
  0xFBC3, 0x0020, 0x0002, 0xF99B, 0x0000, 0x0000, 0x0002, /*1F99B*/
  0xFBC3, 0x0020, 0x0002, 0xF99C, 0x0000, 0x0000, 0x0002, /*1F99C*/
  0xFBC3, 0x0020, 0x0002, 0xF99D, 0x0000, 0x0000, 0x0002, /*1F99D*/
  0xFBC3, 0x0020, 0x0002, 0xF99E, 0x0000, 0x0000, 0x0002, /*1F99E*/
  0xFBC3, 0x0020, 0x0002, 0xF99F, 0x0000, 0x0000, 0x0002, /*1F99F*/
  0xFBC3, 0x0020, 0x0002, 0xF9A0, 0x0000, 0x0000, 0x0002, /*1F9A0*/
  0xFBC3, 0x0020, 0x0002, 0xF9A1, 0x0000, 0x0000, 0x0002, /*1F9A1*/
  0xFBC3, 0x0020, 0x0002, 0xF9A2, 0x0000, 0x0000, 0x0002, /*1F9A2*/
  0xFBC3, 0x0020, 0x0002, 0xF9A3, 0x0000, 0x0000, 0x0002, /*1F9A3*/
  0xFBC3, 0x0020, 0x0002, 0xF9A4, 0x0000, 0x0000, 0x0002, /*1F9A4*/
  0xFBC3, 0x0020, 0x0002, 0xF9A5, 0x0000, 0x0000, 0x0002, /*1F9A5*/
  0xFBC3, 0x0020, 0x0002, 0xF9A6, 0x0000, 0x0000, 0x0002, /*1F9A6*/
  0xFBC3, 0x0020, 0x0002, 0xF9A7, 0x0000, 0x0000, 0x0002, /*1F9A7*/
  0xFBC3, 0x0020, 0x0002, 0xF9A8, 0x0000, 0x0000, 0x0002, /*1F9A8*/
  0xFBC3, 0x0020, 0x0002, 0xF9A9, 0x0000, 0x0000, 0x0002, /*1F9A9*/
  0xFBC3, 0x0020, 0x0002, 0xF9AA, 0x0000, 0x0000, 0x0002, /*1F9AA*/
  0xFBC3, 0x0020, 0x0002, 0xF9AB, 0x0000, 0x0000, 0x0002, /*1F9AB*/
  0xFBC3, 0x0020, 0x0002, 0xF9AC, 0x0000, 0x0000, 0x0002, /*1F9AC*/
  0xFBC3, 0x0020, 0x0002, 0xF9AD, 0x0000, 0x0000, 0x0002, /*1F9AD*/
  0xFBC3, 0x0020, 0x0002, 0xF9AE, 0x0000, 0x0000, 0x0002, /*1F9AE*/
  0xFBC3, 0x0020, 0x0002, 0xF9AF, 0x0000, 0x0000, 0x0002, /*1F9AF*/
  0xFBC3, 0x0020, 0x0002, 0xF9B0, 0x0000, 0x0000, 0x0002, /*1F9B0*/
  0xFBC3, 0x0020, 0x0002, 0xF9B1, 0x0000, 0x0000, 0x0002, /*1F9B1*/
  0xFBC3, 0x0020, 0x0002, 0xF9B2, 0x0000, 0x0000, 0x0002, /*1F9B2*/
  0xFBC3, 0x0020, 0x0002, 0xF9B3, 0x0000, 0x0000, 0x0002, /*1F9B3*/
  0xFBC3, 0x0020, 0x0002, 0xF9B4, 0x0000, 0x0000, 0x0002, /*1F9B4*/
  0xFBC3, 0x0020, 0x0002, 0xF9B5, 0x0000, 0x0000, 0x0002, /*1F9B5*/
  0xFBC3, 0x0020, 0x0002, 0xF9B6, 0x0000, 0x0000, 0x0002, /*1F9B6*/
  0xFBC3, 0x0020, 0x0002, 0xF9B7, 0x0000, 0x0000, 0x0002, /*1F9B7*/
  0xFBC3, 0x0020, 0x0002, 0xF9B8, 0x0000, 0x0000, 0x0002, /*1F9B8*/
  0xFBC3, 0x0020, 0x0002, 0xF9B9, 0x0000, 0x0000, 0x0002, /*1F9B9*/
  0xFBC3, 0x0020, 0x0002, 0xF9BA, 0x0000, 0x0000, 0x0002, /*1F9BA*/
  0xFBC3, 0x0020, 0x0002, 0xF9BB, 0x0000, 0x0000, 0x0002, /*1F9BB*/
  0xFBC3, 0x0020, 0x0002, 0xF9BC, 0x0000, 0x0000, 0x0002, /*1F9BC*/
  0xFBC3, 0x0020, 0x0002, 0xF9BD, 0x0000, 0x0000, 0x0002, /*1F9BD*/
  0xFBC3, 0x0020, 0x0002, 0xF9BE, 0x0000, 0x0000, 0x0002, /*1F9BE*/
  0xFBC3, 0x0020, 0x0002, 0xF9BF, 0x0000, 0x0000, 0x0002, /*1F9BF*/
  0x15FA, 0x0020, 0x0002, 0x0000, 0x0000, 0x0000, 0x0001, /*1F9C0*/
  0xFBC3, 0x0020, 0x0002, 0xF9C1, 0x0000, 0x0000, 0x0002, /*1F9C1*/
  0xFBC3, 0x0020, 0x0002, 0xF9C2, 0x0000, 0x0000, 0x0002, /*1F9C2*/
  0xFBC3, 0x0020, 0x0002, 0xF9C3, 0x0000, 0x0000, 0x0002, /*1F9C3*/
  0xFBC3, 0x0020, 0x0002, 0xF9C4, 0x0000, 0x0000, 0x0002, /*1F9C4*/
  0xFBC3, 0x0020, 0x0002, 0xF9C5, 0x0000, 0x0000, 0x0002, /*1F9C5*/
  0xFBC3, 0x0020, 0x0002, 0xF9C6, 0x0000, 0x0000, 0x0002, /*1F9C6*/
  0xFBC3, 0x0020, 0x0002, 0xF9C7, 0x0000, 0x0000, 0x0002, /*1F9C7*/
  0xFBC3, 0x0020, 0x0002, 0xF9C8, 0x0000, 0x0000, 0x0002, /*1F9C8*/
  0xFBC3, 0x0020, 0x0002, 0xF9C9, 0x0000, 0x0000, 0x0002, /*1F9C9*/
  0xFBC3, 0x0020, 0x0002, 0xF9CA, 0x0000, 0x0000, 0x0002, /*1F9CA*/
  0xFBC3, 0x0020, 0x0002, 0xF9CB, 0x0000, 0x0000, 0x0002, /*1F9CB*/
  0xFBC3, 0x0020, 0x0002, 0xF9CC, 0x0000, 0x0000, 0x0002, /*1F9CC*/
  0xFBC3, 0x0020, 0x0002, 0xF9CD, 0x0000, 0x0000, 0x0002, /*1F9CD*/
  0xFBC3, 0x0020, 0x0002, 0xF9CE, 0x0000, 0x0000, 0x0002, /*1F9CE*/
  0xFBC3, 0x0020, 0x0002, 0xF9CF, 0x0000, 0x0000, 0x0002, /*1F9CF*/
  0xFBC3, 0x0020, 0x0002, 0xF9D0, 0x0000, 0x0000, 0x0002, /*1F9D0*/
  0xFBC3, 0x0020, 0x0002, 0xF9D1, 0x0000, 0x0000, 0x0002, /*1F9D1*/
  0xFBC3, 0x0020, 0x0002, 0xF9D2, 0x0000, 0x0000, 0x0002, /*1F9D2*/
  0xFBC3, 0x0020, 0x0002, 0xF9D3, 0x0000, 0x0000, 0x0002, /*1F9D3*/
  0xFBC3, 0x0020, 0x0002, 0xF9D4, 0x0000, 0x0000, 0x0002, /*1F9D4*/
  0xFBC3, 0x0020, 0x0002, 0xF9D5, 0x0000, 0x0000, 0x0002, /*1F9D5*/
  0xFBC3, 0x0020, 0x0002, 0xF9D6, 0x0000, 0x0000, 0x0002, /*1F9D6*/
  0xFBC3, 0x0020, 0x0002, 0xF9D7, 0x0000, 0x0000, 0x0002, /*1F9D7*/
  0xFBC3, 0x0020, 0x0002, 0xF9D8, 0x0000, 0x0000, 0x0002, /*1F9D8*/
  0xFBC3, 0x0020, 0x0002, 0xF9D9, 0x0000, 0x0000, 0x0002, /*1F9D9*/
  0xFBC3, 0x0020, 0x0002, 0xF9DA, 0x0000, 0x0000, 0x0002, /*1F9DA*/
  0xFBC3, 0x0020, 0x0002, 0xF9DB, 0x0000, 0x0000, 0x0002, /*1F9DB*/
  0xFBC3, 0x0020, 0x0002, 0xF9DC, 0x0000, 0x0000, 0x0002, /*1F9DC*/
  0xFBC3, 0x0020, 0x0002, 0xF9DD, 0x0000, 0x0000, 0x0002, /*1F9DD*/
  0xFBC3, 0x0020, 0x0002, 0xF9DE, 0x0000, 0x0000, 0x0002, /*1F9DE*/
  0xFBC3, 0x0020, 0x0002, 0xF9DF, 0x0000, 0x0000, 0x0002, /*1F9DF*/
  0xFBC3, 0x0020, 0x0002, 0xF9E0, 0x0000, 0x0000, 0x0002, /*1F9E0*/
  0xFBC3, 0x0020, 0x0002, 0xF9E1, 0x0000, 0x0000, 0x0002, /*1F9E1*/
  0xFBC3, 0x0020, 0x0002, 0xF9E2, 0x0000, 0x0000, 0x0002, /*1F9E2*/
  0xFBC3, 0x0020, 0x0002, 0xF9E3, 0x0000, 0x0000, 0x0002, /*1F9E3*/
  0xFBC3, 0x0020, 0x0002, 0xF9E4, 0x0000, 0x0000, 0x0002, /*1F9E4*/
  0xFBC3, 0x0020, 0x0002, 0xF9E5, 0x0000, 0x0000, 0x0002, /*1F9E5*/
  0xFBC3, 0x0020, 0x0002, 0xF9E6, 0x0000, 0x0000, 0x0002, /*1F9E6*/
  0xFBC3, 0x0020, 0x0002, 0xF9E7, 0x0000, 0x0000, 0x0002, /*1F9E7*/
  0xFBC3, 0x0020, 0x0002, 0xF9E8, 0x0000, 0x0000, 0x0002, /*1F9E8*/
  0xFBC3, 0x0020, 0x0002, 0xF9E9, 0x0000, 0x0000, 0x0002, /*1F9E9*/
  0xFBC3, 0x0020, 0x0002, 0xF9EA, 0x0000, 0x0000, 0x0002, /*1F9EA*/
  0xFBC3, 0x0020, 0x0002, 0xF9EB, 0x0000, 0x0000, 0x0002, /*1F9EB*/
  0xFBC3, 0x0020, 0x0002, 0xF9EC, 0x0000, 0x0000, 0x0002, /*1F9EC*/
  0xFBC3, 0x0020, 0x0002, 0xF9ED, 0x0000, 0x0000, 0x0002, /*1F9ED*/
  0xFBC3, 0x0020, 0x0002, 0xF9EE, 0x0000, 0x0000, 0x0002, /*1F9EE*/
  0xFBC3, 0x0020, 0x0002, 0xF9EF, 0x0000, 0x0000, 0x0002, /*1F9EF*/
  0xFBC3, 0x0020, 0x0002, 0xF9F0, 0x0000, 0x0000, 0x0002, /*1F9F0*/
  0xFBC3, 0x0020, 0x0002, 0xF9F1, 0x0000, 0x0000, 0x0002, /*1F9F1*/
  0xFBC3, 0x0020, 0x0002, 0xF9F2, 0x0000, 0x0000, 0x0002, /*1F9F2*/
  0xFBC3, 0x0020, 0x0002, 0xF9F3, 0x0000, 0x0000, 0x0002, /*1F9F3*/
  0xFBC3, 0x0020, 0x0002, 0xF9F4, 0x0000, 0x0000, 0x0002, /*1F9F4*/
  0xFBC3, 0x0020, 0x0002, 0xF9F5, 0x0000, 0x0000, 0x0002, /*1F9F5*/
  0xFBC3, 0x0020, 0x0002, 0xF9F6, 0x0000, 0x0000, 0x0002, /*1F9F6*/
  0xFBC3, 0x0020, 0x0002, 0xF9F7, 0x0000, 0x0000, 0x0002, /*1F9F7*/
  0xFBC3, 0x0020, 0x0002, 0xF9F8, 0x0000, 0x0000, 0x0002, /*1F9F8*/
  0xFBC3, 0x0020, 0x0002, 0xF9F9, 0x0000, 0x0000, 0x0002, /*1F9F9*/
  0xFBC3, 0x0020, 0x0002, 0xF9FA, 0x0000, 0x0000, 0x0002, /*1F9FA*/
  0xFBC3, 0x0020, 0x0002, 0xF9FB, 0x0000, 0x0000, 0x0002, /*1F9FB*/
  0xFBC3, 0x0020, 0x0002, 0xF9FC, 0x0000, 0x0000, 0x0002, /*1F9FC*/
  0xFBC3, 0x0020, 0x0002, 0xF9FD, 0x0000, 0x0000, 0x0002, /*1F9FD*/
  0xFBC3, 0x0020, 0x0002, 0xF9FE, 0x0000, 0x0000, 0x0002, /*1F9FE*/
  0xFBC3, 0x0020, 0x0002, 0xF9FF, 0x0000, 0x0000, 0x0002  /*1F9FF*/
};

uint16 uca900_p2F8[]= {
  0xFB40, 0x0020, 0x0002, 0xCE3D, 0x0000, 0x0000, 0x0002, /*2F800*/
  0xFB40, 0x0020, 0x0002, 0xCE38, 0x0000, 0x0000, 0x0002, /*2F801*/
  0xFB40, 0x0020, 0x0002, 0xCE41, 0x0000, 0x0000, 0x0002, /*2F802*/
  0xFB84, 0x0020, 0x0002, 0x8122, 0x0000, 0x0000, 0x0002, /*2F803*/
  0xFB40, 0x0020, 0x0002, 0xCF60, 0x0000, 0x0000, 0x0002, /*2F804*/
  0xFB40, 0x0020, 0x0002, 0xCFAE, 0x0000, 0x0000, 0x0002, /*2F805*/
  0xFB40, 0x0020, 0x0002, 0xCFBB, 0x0000, 0x0000, 0x0002, /*2F806*/
  0xFB40, 0x0020, 0x0002, 0xD002, 0x0000, 0x0000, 0x0002, /*2F807*/
  0xFB40, 0x0020, 0x0002, 0xD07A, 0x0000, 0x0000, 0x0002, /*2F808*/
  0xFB40, 0x0020, 0x0002, 0xD099, 0x0000, 0x0000, 0x0002, /*2F809*/
  0xFB40, 0x0020, 0x0002, 0xD0E7, 0x0000, 0x0000, 0x0002, /*2F80A*/
  0xFB40, 0x0020, 0x0002, 0xD0CF, 0x0000, 0x0000, 0x0002, /*2F80B*/
  0xFB80, 0x0020, 0x0002, 0xB49E, 0x0000, 0x0000, 0x0002, /*2F80C*/
  0xFB84, 0x0020, 0x0002, 0x863A, 0x0000, 0x0000, 0x0002, /*2F80D*/
  0xFB40, 0x0020, 0x0002, 0xD14D, 0x0000, 0x0000, 0x0002, /*2F80E*/
  0xFB40, 0x0020, 0x0002, 0xD154, 0x0000, 0x0000, 0x0002, /*2F80F*/
  0xFB40, 0x0020, 0x0002, 0xD164, 0x0000, 0x0000, 0x0002, /*2F810*/
  0xFB40, 0x0020, 0x0002, 0xD177, 0x0000, 0x0000, 0x0002, /*2F811*/
  0xFB84, 0x0020, 0x0002, 0x851C, 0x0000, 0x0000, 0x0002, /*2F812*/
  0xFB80, 0x0020, 0x0002, 0xB4B9, 0x0000, 0x0000, 0x0002, /*2F813*/
  0xFB40, 0x0020, 0x0002, 0xD167, 0x0000, 0x0000, 0x0002, /*2F814*/
  0xFB40, 0x0020, 0x0002, 0xD18D, 0x0000, 0x0000, 0x0002, /*2F815*/
  0xFB84, 0x0020, 0x0002, 0x854B, 0x0000, 0x0000, 0x0002, /*2F816*/
  0xFB40, 0x0020, 0x0002, 0xD197, 0x0000, 0x0000, 0x0002, /*2F817*/
  0xFB40, 0x0020, 0x0002, 0xD1A4, 0x0000, 0x0000, 0x0002, /*2F818*/
  0xFB40, 0x0020, 0x0002, 0xCECC, 0x0000, 0x0000, 0x0002, /*2F819*/
  0xFB40, 0x0020, 0x0002, 0xD1AC, 0x0000, 0x0000, 0x0002, /*2F81A*/
  0xFB40, 0x0020, 0x0002, 0xD1B5, 0x0000, 0x0000, 0x0002, /*2F81B*/
  0xFB85, 0x0020, 0x0002, 0x91DF, 0x0000, 0x0000, 0x0002, /*2F81C*/
  0xFB40, 0x0020, 0x0002, 0xD1F5, 0x0000, 0x0000, 0x0002, /*2F81D*/
  0xFB40, 0x0020, 0x0002, 0xD203, 0x0000, 0x0000, 0x0002, /*2F81E*/
  0xFB80, 0x0020, 0x0002, 0xB4DF, 0x0000, 0x0000, 0x0002, /*2F81F*/
  0xFB40, 0x0020, 0x0002, 0xD23B, 0x0000, 0x0000, 0x0002, /*2F820*/
  0xFB40, 0x0020, 0x0002, 0xD246, 0x0000, 0x0000, 0x0002, /*2F821*/
  0xFB40, 0x0020, 0x0002, 0xD272, 0x0000, 0x0000, 0x0002, /*2F822*/
  0xFB40, 0x0020, 0x0002, 0xD277, 0x0000, 0x0000, 0x0002, /*2F823*/
  0xFB80, 0x0020, 0x0002, 0xB515, 0x0000, 0x0000, 0x0002, /*2F824*/
  0xFB40, 0x0020, 0x0002, 0xD2C7, 0x0000, 0x0000, 0x0002, /*2F825*/
  0xFB40, 0x0020, 0x0002, 0xD2C9, 0x0000, 0x0000, 0x0002, /*2F826*/
  0xFB40, 0x0020, 0x0002, 0xD2E4, 0x0000, 0x0000, 0x0002, /*2F827*/
  0xFB40, 0x0020, 0x0002, 0xD2FA, 0x0000, 0x0000, 0x0002, /*2F828*/
  0xFB40, 0x0020, 0x0002, 0xD305, 0x0000, 0x0000, 0x0002, /*2F829*/
  0xFB40, 0x0020, 0x0002, 0xD306, 0x0000, 0x0000, 0x0002, /*2F82A*/
  0xFB40, 0x0020, 0x0002, 0xD317, 0x0000, 0x0000, 0x0002, /*2F82B*/
  0xFB40, 0x0020, 0x0002, 0xD349, 0x0000, 0x0000, 0x0002, /*2F82C*/
  0xFB40, 0x0020, 0x0002, 0xD351, 0x0000, 0x0000, 0x0002, /*2F82D*/
  0xFB40, 0x0020, 0x0002, 0xD35A, 0x0000, 0x0000, 0x0002, /*2F82E*/
  0xFB40, 0x0020, 0x0002, 0xD373, 0x0000, 0x0000, 0x0002, /*2F82F*/
  0xFB40, 0x0020, 0x0002, 0xD37D, 0x0000, 0x0000, 0x0002, /*2F830*/
  0xFB40, 0x0020, 0x0002, 0xD37F, 0x0000, 0x0000, 0x0002, /*2F831*/
  0xFB40, 0x0020, 0x0002, 0xD37F, 0x0000, 0x0000, 0x0002, /*2F832*/
  0xFB40, 0x0020, 0x0002, 0xD37F, 0x0000, 0x0000, 0x0002, /*2F833*/
  0xFB84, 0x0020, 0x0002, 0x8A2C, 0x0000, 0x0000, 0x0002, /*2F834*/
  0xFB40, 0x0020, 0x0002, 0xF070, 0x0000, 0x0000, 0x0002, /*2F835*/
  0xFB40, 0x0020, 0x0002, 0xD3CA, 0x0000, 0x0000, 0x0002, /*2F836*/
  0xFB40, 0x0020, 0x0002, 0xD3DF, 0x0000, 0x0000, 0x0002, /*2F837*/
  0xFB84, 0x0020, 0x0002, 0x8B63, 0x0000, 0x0000, 0x0002, /*2F838*/
  0xFB40, 0x0020, 0x0002, 0xD3EB, 0x0000, 0x0000, 0x0002, /*2F839*/
  0xFB40, 0x0020, 0x0002, 0xD3F1, 0x0000, 0x0000, 0x0002, /*2F83A*/
  0xFB40, 0x0020, 0x0002, 0xD406, 0x0000, 0x0000, 0x0002, /*2F83B*/
  0xFB40, 0x0020, 0x0002, 0xD49E, 0x0000, 0x0000, 0x0002, /*2F83C*/
  0xFB40, 0x0020, 0x0002, 0xD438, 0x0000, 0x0000, 0x0002, /*2F83D*/
  0xFB40, 0x0020, 0x0002, 0xD448, 0x0000, 0x0000, 0x0002, /*2F83E*/
  0xFB40, 0x0020, 0x0002, 0xD468, 0x0000, 0x0000, 0x0002, /*2F83F*/
  0xFB40, 0x0020, 0x0002, 0xD4A2, 0x0000, 0x0000, 0x0002, /*2F840*/
  0xFB40, 0x0020, 0x0002, 0xD4F6, 0x0000, 0x0000, 0x0002, /*2F841*/
  0xFB40, 0x0020, 0x0002, 0xD510, 0x0000, 0x0000, 0x0002, /*2F842*/
  0xFB40, 0x0020, 0x0002, 0xD553, 0x0000, 0x0000, 0x0002, /*2F843*/
  0xFB40, 0x0020, 0x0002, 0xD563, 0x0000, 0x0000, 0x0002, /*2F844*/
  0xFB40, 0x0020, 0x0002, 0xD584, 0x0000, 0x0000, 0x0002, /*2F845*/
  0xFB40, 0x0020, 0x0002, 0xD584, 0x0000, 0x0000, 0x0002, /*2F846*/
  0xFB40, 0x0020, 0x0002, 0xD599, 0x0000, 0x0000, 0x0002, /*2F847*/
  0xFB40, 0x0020, 0x0002, 0xD5AB, 0x0000, 0x0000, 0x0002, /*2F848*/
  0xFB40, 0x0020, 0x0002, 0xD5B3, 0x0000, 0x0000, 0x0002, /*2F849*/
  0xFB40, 0x0020, 0x0002, 0xD5C2, 0x0000, 0x0000, 0x0002, /*2F84A*/
  0xFB40, 0x0020, 0x0002, 0xD716, 0x0000, 0x0000, 0x0002, /*2F84B*/
  0xFB40, 0x0020, 0x0002, 0xD606, 0x0000, 0x0000, 0x0002, /*2F84C*/
  0xFB40, 0x0020, 0x0002, 0xD717, 0x0000, 0x0000, 0x0002, /*2F84D*/
  0xFB40, 0x0020, 0x0002, 0xD651, 0x0000, 0x0000, 0x0002, /*2F84E*/
  0xFB40, 0x0020, 0x0002, 0xD674, 0x0000, 0x0000, 0x0002, /*2F84F*/
  0xFB40, 0x0020, 0x0002, 0xD207, 0x0000, 0x0000, 0x0002, /*2F850*/
  0xFB40, 0x0020, 0x0002, 0xD8EE, 0x0000, 0x0000, 0x0002, /*2F851*/
  0xFB40, 0x0020, 0x0002, 0xD7CE, 0x0000, 0x0000, 0x0002, /*2F852*/
  0xFB40, 0x0020, 0x0002, 0xD7F4, 0x0000, 0x0000, 0x0002, /*2F853*/
  0xFB40, 0x0020, 0x0002, 0xD80D, 0x0000, 0x0000, 0x0002, /*2F854*/
  0xFB40, 0x0020, 0x0002, 0xD78B, 0x0000, 0x0000, 0x0002, /*2F855*/
  0xFB40, 0x0020, 0x0002, 0xD832, 0x0000, 0x0000, 0x0002, /*2F856*/
  0xFB40, 0x0020, 0x0002, 0xD831, 0x0000, 0x0000, 0x0002, /*2F857*/
  0xFB40, 0x0020, 0x0002, 0xD8AC, 0x0000, 0x0000, 0x0002, /*2F858*/
  0xFB84, 0x0020, 0x0002, 0x94E4, 0x0000, 0x0000, 0x0002, /*2F859*/
  0xFB40, 0x0020, 0x0002, 0xD8F2, 0x0000, 0x0000, 0x0002, /*2F85A*/
  0xFB40, 0x0020, 0x0002, 0xD8F7, 0x0000, 0x0000, 0x0002, /*2F85B*/
  0xFB40, 0x0020, 0x0002, 0xD906, 0x0000, 0x0000, 0x0002, /*2F85C*/
  0xFB40, 0x0020, 0x0002, 0xD91A, 0x0000, 0x0000, 0x0002, /*2F85D*/
  0xFB40, 0x0020, 0x0002, 0xD922, 0x0000, 0x0000, 0x0002, /*2F85E*/
  0xFB40, 0x0020, 0x0002, 0xD962, 0x0000, 0x0000, 0x0002, /*2F85F*/
  0xFB84, 0x0020, 0x0002, 0x96A8, 0x0000, 0x0000, 0x0002, /*2F860*/
  0xFB84, 0x0020, 0x0002, 0x96EA, 0x0000, 0x0000, 0x0002, /*2F861*/
  0xFB40, 0x0020, 0x0002, 0xD9EC, 0x0000, 0x0000, 0x0002, /*2F862*/
  0xFB40, 0x0020, 0x0002, 0xDA1B, 0x0000, 0x0000, 0x0002, /*2F863*/
  0xFB40, 0x0020, 0x0002, 0xDA27, 0x0000, 0x0000, 0x0002, /*2F864*/
  0xFB40, 0x0020, 0x0002, 0xD9D8, 0x0000, 0x0000, 0x0002, /*2F865*/
  0xFB40, 0x0020, 0x0002, 0xDA66, 0x0000, 0x0000, 0x0002, /*2F866*/
  0xFB80, 0x0020, 0x0002, 0xB6EE, 0x0000, 0x0000, 0x0002, /*2F867*/
  0xFB80, 0x0020, 0x0002, 0xB6FC, 0x0000, 0x0000, 0x0002, /*2F868*/
  0xFB40, 0x0020, 0x0002, 0xDB08, 0x0000, 0x0000, 0x0002, /*2F869*/
  0xFB40, 0x0020, 0x0002, 0xDB3E, 0x0000, 0x0000, 0x0002, /*2F86A*/
  0xFB40, 0x0020, 0x0002, 0xDB3E, 0x0000, 0x0000, 0x0002, /*2F86B*/
  0xFB84, 0x0020, 0x0002, 0x99C8, 0x0000, 0x0000, 0x0002, /*2F86C*/
  0xFB40, 0x0020, 0x0002, 0xDBC3, 0x0000, 0x0000, 0x0002, /*2F86D*/
  0xFB40, 0x0020, 0x0002, 0xDBD8, 0x0000, 0x0000, 0x0002, /*2F86E*/
  0xFB40, 0x0020, 0x0002, 0xDBE7, 0x0000, 0x0000, 0x0002, /*2F86F*/
  0xFB40, 0x0020, 0x0002, 0xDBF3, 0x0000, 0x0000, 0x0002, /*2F870*/
  0xFB84, 0x0020, 0x0002, 0x9B18, 0x0000, 0x0000, 0x0002, /*2F871*/
  0xFB40, 0x0020, 0x0002, 0xDBFF, 0x0000, 0x0000, 0x0002, /*2F872*/
  0xFB40, 0x0020, 0x0002, 0xDC06, 0x0000, 0x0000, 0x0002, /*2F873*/
  0xFB40, 0x0020, 0x0002, 0xDF53, 0x0000, 0x0000, 0x0002, /*2F874*/
  0xFB40, 0x0020, 0x0002, 0xDC22, 0x0000, 0x0000, 0x0002, /*2F875*/
  0xFB80, 0x0020, 0x0002, 0xB781, 0x0000, 0x0000, 0x0002, /*2F876*/
  0xFB40, 0x0020, 0x0002, 0xDC60, 0x0000, 0x0000, 0x0002, /*2F877*/
  0xFB40, 0x0020, 0x0002, 0xDC6E, 0x0000, 0x0000, 0x0002, /*2F878*/
  0xFB40, 0x0020, 0x0002, 0xDCC0, 0x0000, 0x0000, 0x0002, /*2F879*/
  0xFB40, 0x0020, 0x0002, 0xDC8D, 0x0000, 0x0000, 0x0002, /*2F87A*/
  0xFB84, 0x0020, 0x0002, 0x9DE4, 0x0000, 0x0000, 0x0002, /*2F87B*/
  0xFB40, 0x0020, 0x0002, 0xDD43, 0x0000, 0x0000, 0x0002, /*2F87C*/
  0xFB84, 0x0020, 0x0002, 0x9DE6, 0x0000, 0x0000, 0x0002, /*2F87D*/
  0xFB40, 0x0020, 0x0002, 0xDD6E, 0x0000, 0x0000, 0x0002, /*2F87E*/
  0xFB40, 0x0020, 0x0002, 0xDD6B, 0x0000, 0x0000, 0x0002, /*2F87F*/
  0xFB40, 0x0020, 0x0002, 0xDD7C, 0x0000, 0x0000, 0x0002, /*2F880*/
  0xFB40, 0x0020, 0x0002, 0xDDE1, 0x0000, 0x0000, 0x0002, /*2F881*/
  0xFB40, 0x0020, 0x0002, 0xDDE2, 0x0000, 0x0000, 0x0002, /*2F882*/
  0xFB80, 0x0020, 0x0002, 0xB82F, 0x0000, 0x0000, 0x0002, /*2F883*/
  0xFB40, 0x0020, 0x0002, 0xDDFD, 0x0000, 0x0000, 0x0002, /*2F884*/
  0xFB40, 0x0020, 0x0002, 0xDE28, 0x0000, 0x0000, 0x0002, /*2F885*/
  0xFB40, 0x0020, 0x0002, 0xDE3D, 0x0000, 0x0000, 0x0002, /*2F886*/
  0xFB40, 0x0020, 0x0002, 0xDE69, 0x0000, 0x0000, 0x0002, /*2F887*/
  0xFB80, 0x0020, 0x0002, 0xB862, 0x0000, 0x0000, 0x0002, /*2F888*/
  0xFB84, 0x0020, 0x0002, 0xA183, 0x0000, 0x0000, 0x0002, /*2F889*/
  0xFB80, 0x0020, 0x0002, 0xB87C, 0x0000, 0x0000, 0x0002, /*2F88A*/
  0xFB40, 0x0020, 0x0002, 0xDEB0, 0x0000, 0x0000, 0x0002, /*2F88B*/
  0xFB40, 0x0020, 0x0002, 0xDEB3, 0x0000, 0x0000, 0x0002, /*2F88C*/
  0xFB40, 0x0020, 0x0002, 0xDEB6, 0x0000, 0x0000, 0x0002, /*2F88D*/
  0xFB40, 0x0020, 0x0002, 0xDECA, 0x0000, 0x0000, 0x0002, /*2F88E*/
  0xFB85, 0x0020, 0x0002, 0xA392, 0x0000, 0x0000, 0x0002, /*2F88F*/
  0xFB40, 0x0020, 0x0002, 0xDEFE, 0x0000, 0x0000, 0x0002, /*2F890*/
  0xFB84, 0x0020, 0x0002, 0xA331, 0x0000, 0x0000, 0x0002, /*2F891*/
  0xFB84, 0x0020, 0x0002, 0xA331, 0x0000, 0x0000, 0x0002, /*2F892*/
  0xFB41, 0x0020, 0x0002, 0x8201, 0x0000, 0x0000, 0x0002, /*2F893*/
  0xFB40, 0x0020, 0x0002, 0xDF22, 0x0000, 0x0000, 0x0002, /*2F894*/
  0xFB40, 0x0020, 0x0002, 0xDF22, 0x0000, 0x0000, 0x0002, /*2F895*/
  0xFB80, 0x0020, 0x0002, 0xB8C7, 0x0000, 0x0000, 0x0002, /*2F896*/
  0xFB84, 0x0020, 0x0002, 0xB2B8, 0x0000, 0x0000, 0x0002, /*2F897*/
  0xFB84, 0x0020, 0x0002, 0xE1DA, 0x0000, 0x0000, 0x0002, /*2F898*/
  0xFB40, 0x0020, 0x0002, 0xDF62, 0x0000, 0x0000, 0x0002, /*2F899*/
  0xFB40, 0x0020, 0x0002, 0xDF6B, 0x0000, 0x0000, 0x0002, /*2F89A*/
  0xFB80, 0x0020, 0x0002, 0xB8E3, 0x0000, 0x0000, 0x0002, /*2F89B*/
  0xFB40, 0x0020, 0x0002, 0xDF9A, 0x0000, 0x0000, 0x0002, /*2F89C*/
  0xFB40, 0x0020, 0x0002, 0xDFCD, 0x0000, 0x0000, 0x0002, /*2F89D*/
  0xFB40, 0x0020, 0x0002, 0xDFD7, 0x0000, 0x0000, 0x0002, /*2F89E*/
  0xFB40, 0x0020, 0x0002, 0xDFF9, 0x0000, 0x0000, 0x0002, /*2F89F*/
  0xFB40, 0x0020, 0x0002, 0xE081, 0x0000, 0x0000, 0x0002, /*2F8A0*/
  0xFB80, 0x0020, 0x0002, 0xB93A, 0x0000, 0x0000, 0x0002, /*2F8A1*/
  0xFB80, 0x0020, 0x0002, 0xB91C, 0x0000, 0x0000, 0x0002, /*2F8A2*/
  0xFB40, 0x0020, 0x0002, 0xE094, 0x0000, 0x0000, 0x0002, /*2F8A3*/
  0xFB84, 0x0020, 0x0002, 0xA6D4, 0x0000, 0x0000, 0x0002, /*2F8A4*/
  0xFB40, 0x0020, 0x0002, 0xE0C7, 0x0000, 0x0000, 0x0002, /*2F8A5*/
  0xFB40, 0x0020, 0x0002, 0xE148, 0x0000, 0x0000, 0x0002, /*2F8A6*/
  0xFB40, 0x0020, 0x0002, 0xE14C, 0x0000, 0x0000, 0x0002, /*2F8A7*/
  0xFB40, 0x0020, 0x0002, 0xE14E, 0x0000, 0x0000, 0x0002, /*2F8A8*/
  0xFB40, 0x0020, 0x0002, 0xE14C, 0x0000, 0x0000, 0x0002, /*2F8A9*/
  0xFB40, 0x0020, 0x0002, 0xE17A, 0x0000, 0x0000, 0x0002, /*2F8AA*/
  0xFB40, 0x0020, 0x0002, 0xE18E, 0x0000, 0x0000, 0x0002, /*2F8AB*/
  0xFB40, 0x0020, 0x0002, 0xE1B2, 0x0000, 0x0000, 0x0002, /*2F8AC*/
  0xFB40, 0x0020, 0x0002, 0xE1A4, 0x0000, 0x0000, 0x0002, /*2F8AD*/
  0xFB40, 0x0020, 0x0002, 0xE1AF, 0x0000, 0x0000, 0x0002, /*2F8AE*/
  0xFB40, 0x0020, 0x0002, 0xE1DE, 0x0000, 0x0000, 0x0002, /*2F8AF*/
  0xFB40, 0x0020, 0x0002, 0xE1F2, 0x0000, 0x0000, 0x0002, /*2F8B0*/
  0xFB40, 0x0020, 0x0002, 0xE1F6, 0x0000, 0x0000, 0x0002, /*2F8B1*/
  0xFB40, 0x0020, 0x0002, 0xE210, 0x0000, 0x0000, 0x0002, /*2F8B2*/
  0xFB40, 0x0020, 0x0002, 0xE21B, 0x0000, 0x0000, 0x0002, /*2F8B3*/
  0xFB40, 0x0020, 0x0002, 0xE25D, 0x0000, 0x0000, 0x0002, /*2F8B4*/
  0xFB40, 0x0020, 0x0002, 0xE2B1, 0x0000, 0x0000, 0x0002, /*2F8B5*/
  0xFB40, 0x0020, 0x0002, 0xE2D4, 0x0000, 0x0000, 0x0002, /*2F8B6*/
  0xFB40, 0x0020, 0x0002, 0xE350, 0x0000, 0x0000, 0x0002, /*2F8B7*/
  0xFB84, 0x0020, 0x0002, 0xAB0C, 0x0000, 0x0000, 0x0002, /*2F8B8*/
  0xFB40, 0x0020, 0x0002, 0xE33D, 0x0000, 0x0000, 0x0002, /*2F8B9*/
  0xFB40, 0x0020, 0x0002, 0xE2FC, 0x0000, 0x0000, 0x0002, /*2F8BA*/
  0xFB40, 0x0020, 0x0002, 0xE368, 0x0000, 0x0000, 0x0002, /*2F8BB*/
  0xFB40, 0x0020, 0x0002, 0xE383, 0x0000, 0x0000, 0x0002, /*2F8BC*/
  0xFB40, 0x0020, 0x0002, 0xE3E4, 0x0000, 0x0000, 0x0002, /*2F8BD*/
  0xFB84, 0x0020, 0x0002, 0xABF1, 0x0000, 0x0000, 0x0002, /*2F8BE*/
  0xFB40, 0x0020, 0x0002, 0xE422, 0x0000, 0x0000, 0x0002, /*2F8BF*/
  0xFB40, 0x0020, 0x0002, 0xE3C5, 0x0000, 0x0000, 0x0002, /*2F8C0*/
  0xFB40, 0x0020, 0x0002, 0xE3A9, 0x0000, 0x0000, 0x0002, /*2F8C1*/
  0xFB80, 0x0020, 0x0002, 0xBA2E, 0x0000, 0x0000, 0x0002, /*2F8C2*/
  0xFB40, 0x0020, 0x0002, 0xE469, 0x0000, 0x0000, 0x0002, /*2F8C3*/
  0xFB40, 0x0020, 0x0002, 0xE47E, 0x0000, 0x0000, 0x0002, /*2F8C4*/
  0xFB40, 0x0020, 0x0002, 0xE49D, 0x0000, 0x0000, 0x0002, /*2F8C5*/
  0xFB40, 0x0020, 0x0002, 0xE477, 0x0000, 0x0000, 0x0002, /*2F8C6*/
  0xFB80, 0x0020, 0x0002, 0xBA6C, 0x0000, 0x0000, 0x0002, /*2F8C7*/
  0xFB40, 0x0020, 0x0002, 0xE54F, 0x0000, 0x0000, 0x0002, /*2F8C8*/
  0xFB40, 0x0020, 0x0002, 0xE56C, 0x0000, 0x0000, 0x0002, /*2F8C9*/
  0xFB84, 0x0020, 0x0002, 0xB00A, 0x0000, 0x0000, 0x0002, /*2F8CA*/
  0xFB40, 0x0020, 0x0002, 0xE5E3, 0x0000, 0x0000, 0x0002, /*2F8CB*/
  0xFB40, 0x0020, 0x0002, 0xE6F8, 0x0000, 0x0000, 0x0002, /*2F8CC*/
  0xFB40, 0x0020, 0x0002, 0xE649, 0x0000, 0x0000, 0x0002, /*2F8CD*/
  0xFB80, 0x0020, 0x0002, 0xBB19, 0x0000, 0x0000, 0x0002, /*2F8CE*/
  0xFB40, 0x0020, 0x0002, 0xE691, 0x0000, 0x0000, 0x0002, /*2F8CF*/
  0xFB80, 0x0020, 0x0002, 0xBB08, 0x0000, 0x0000, 0x0002, /*2F8D0*/
  0xFB80, 0x0020, 0x0002, 0xBAE4, 0x0000, 0x0000, 0x0002, /*2F8D1*/
  0xFB40, 0x0020, 0x0002, 0xD192, 0x0000, 0x0000, 0x0002, /*2F8D2*/
  0xFB40, 0x0020, 0x0002, 0xD195, 0x0000, 0x0000, 0x0002, /*2F8D3*/
  0xFB40, 0x0020, 0x0002, 0xE700, 0x0000, 0x0000, 0x0002, /*2F8D4*/
  0xFB40, 0x0020, 0x0002, 0xE69C, 0x0000, 0x0000, 0x0002, /*2F8D5*/
  0xFB41, 0x0020, 0x0002, 0x80AD, 0x0000, 0x0000, 0x0002, /*2F8D6*/
  0xFB80, 0x0020, 0x0002, 0xC3D9, 0x0000, 0x0000, 0x0002, /*2F8D7*/
  0xFB40, 0x0020, 0x0002, 0xE717, 0x0000, 0x0000, 0x0002, /*2F8D8*/
  0xFB40, 0x0020, 0x0002, 0xE71B, 0x0000, 0x0000, 0x0002, /*2F8D9*/
  0xFB40, 0x0020, 0x0002, 0xE721, 0x0000, 0x0000, 0x0002, /*2F8DA*/
  0xFB40, 0x0020, 0x0002, 0xE75E, 0x0000, 0x0000, 0x0002, /*2F8DB*/
  0xFB40, 0x0020, 0x0002, 0xE753, 0x0000, 0x0000, 0x0002, /*2F8DC*/
  0xFB84, 0x0020, 0x0002, 0xB3C3, 0x0000, 0x0000, 0x0002, /*2F8DD*/
  0xFB80, 0x0020, 0x0002, 0xBB49, 0x0000, 0x0000, 0x0002, /*2F8DE*/
  0xFB40, 0x0020, 0x0002, 0xE7FA, 0x0000, 0x0000, 0x0002, /*2F8DF*/
  0xFB40, 0x0020, 0x0002, 0xE785, 0x0000, 0x0000, 0x0002, /*2F8E0*/
  0xFB40, 0x0020, 0x0002, 0xE852, 0x0000, 0x0000, 0x0002, /*2F8E1*/
  0xFB40, 0x0020, 0x0002, 0xE885, 0x0000, 0x0000, 0x0002, /*2F8E2*/
  0xFB84, 0x0020, 0x0002, 0xB46D, 0x0000, 0x0000, 0x0002, /*2F8E3*/
  0xFB40, 0x0020, 0x0002, 0xE88E, 0x0000, 0x0000, 0x0002, /*2F8E4*/
  0xFB40, 0x0020, 0x0002, 0xE81F, 0x0000, 0x0000, 0x0002, /*2F8E5*/
  0xFB40, 0x0020, 0x0002, 0xE914, 0x0000, 0x0000, 0x0002, /*2F8E6*/
  0xFB80, 0x0020, 0x0002, 0xBB9D, 0x0000, 0x0000, 0x0002, /*2F8E7*/
  0xFB40, 0x0020, 0x0002, 0xE942, 0x0000, 0x0000, 0x0002, /*2F8E8*/
  0xFB40, 0x0020, 0x0002, 0xE9A3, 0x0000, 0x0000, 0x0002, /*2F8E9*/
  0xFB40, 0x0020, 0x0002, 0xE9EA, 0x0000, 0x0000, 0x0002, /*2F8EA*/
  0xFB40, 0x0020, 0x0002, 0xEAA8, 0x0000, 0x0000, 0x0002, /*2F8EB*/
  0xFB84, 0x0020, 0x0002, 0xB6A3, 0x0000, 0x0000, 0x0002, /*2F8EC*/
  0xFB40, 0x0020, 0x0002, 0xEADB, 0x0000, 0x0000, 0x0002, /*2F8ED*/
  0xFB80, 0x0020, 0x0002, 0xBC18, 0x0000, 0x0000, 0x0002, /*2F8EE*/
  0xFB40, 0x0020, 0x0002, 0xEB21, 0x0000, 0x0000, 0x0002, /*2F8EF*/
  0xFB84, 0x0020, 0x0002, 0xB8A7, 0x0000, 0x0000, 0x0002, /*2F8F0*/
  0xFB40, 0x0020, 0x0002, 0xEB54, 0x0000, 0x0000, 0x0002, /*2F8F1*/
  0xFB80, 0x0020, 0x0002, 0xBC4E, 0x0000, 0x0000, 0x0002, /*2F8F2*/
  0xFB40, 0x0020, 0x0002, 0xEB72, 0x0000, 0x0000, 0x0002, /*2F8F3*/
  0xFB40, 0x0020, 0x0002, 0xEB9F, 0x0000, 0x0000, 0x0002, /*2F8F4*/
  0xFB40, 0x0020, 0x0002, 0xEBBA, 0x0000, 0x0000, 0x0002, /*2F8F5*/
  0xFB40, 0x0020, 0x0002, 0xEBBB, 0x0000, 0x0000, 0x0002, /*2F8F6*/
  0xFB84, 0x0020, 0x0002, 0xBA8D, 0x0000, 0x0000, 0x0002, /*2F8F7*/
  0xFB84, 0x0020, 0x0002, 0x9D0B, 0x0000, 0x0000, 0x0002, /*2F8F8*/
  0xFB84, 0x0020, 0x0002, 0xBAFA, 0x0000, 0x0000, 0x0002, /*2F8F9*/
  0xFB40, 0x0020, 0x0002, 0xEC4E, 0x0000, 0x0000, 0x0002, /*2F8FA*/
  0xFB84, 0x0020, 0x0002, 0xBCBC, 0x0000, 0x0000, 0x0002, /*2F8FB*/
  0xFB40, 0x0020, 0x0002, 0xECBF, 0x0000, 0x0000, 0x0002, /*2F8FC*/
  0xFB40, 0x0020, 0x0002, 0xECCD, 0x0000, 0x0000, 0x0002, /*2F8FD*/
  0xFB40, 0x0020, 0x0002, 0xEC67, 0x0000, 0x0000, 0x0002, /*2F8FE*/
  0xFB40, 0x0020, 0x0002, 0xED16, 0x0000, 0x0000, 0x0002  /*2F8FF*/
};

uint16 uca900_p2F9[]= {
  0xFB40, 0x0020, 0x0002, 0xED3E, 0x0000, 0x0000, 0x0002, /*2F900*/
  0xFB40, 0x0020, 0x0002, 0xED77, 0x0000, 0x0000, 0x0002, /*2F901*/
  0xFB40, 0x0020, 0x0002, 0xED41, 0x0000, 0x0000, 0x0002, /*2F902*/
  0xFB40, 0x0020, 0x0002, 0xED69, 0x0000, 0x0000, 0x0002, /*2F903*/
  0xFB40, 0x0020, 0x0002, 0xED78, 0x0000, 0x0000, 0x0002, /*2F904*/
  0xFB40, 0x0020, 0x0002, 0xED85, 0x0000, 0x0000, 0x0002, /*2F905*/
  0xFB84, 0x0020, 0x0002, 0xBD1E, 0x0000, 0x0000, 0x0002, /*2F906*/
  0xFB40, 0x0020, 0x0002, 0xED34, 0x0000, 0x0000, 0x0002, /*2F907*/
  0xFB40, 0x0020, 0x0002, 0xEE2F, 0x0000, 0x0000, 0x0002, /*2F908*/
  0xFB40, 0x0020, 0x0002, 0xEE6E, 0x0000, 0x0000, 0x0002, /*2F909*/
  0xFB80, 0x0020, 0x0002, 0xBD33, 0x0000, 0x0000, 0x0002, /*2F90A*/
  0xFB40, 0x0020, 0x0002, 0xEECB, 0x0000, 0x0000, 0x0002, /*2F90B*/
  0xFB40, 0x0020, 0x0002, 0xEEC7, 0x0000, 0x0000, 0x0002, /*2F90C*/
  0xFB84, 0x0020, 0x0002, 0xBED1, 0x0000, 0x0000, 0x0002, /*2F90D*/
  0xFB40, 0x0020, 0x0002, 0xEDF9, 0x0000, 0x0000, 0x0002, /*2F90E*/
  0xFB40, 0x0020, 0x0002, 0xEF6E, 0x0000, 0x0000, 0x0002, /*2F90F*/
  0xFB84, 0x0020, 0x0002, 0xBF5E, 0x0000, 0x0000, 0x0002, /*2F910*/
  0xFB84, 0x0020, 0x0002, 0xBF8E, 0x0000, 0x0000, 0x0002, /*2F911*/
  0xFB40, 0x0020, 0x0002, 0xEFC6, 0x0000, 0x0000, 0x0002, /*2F912*/
  0xFB40, 0x0020, 0x0002, 0xF039, 0x0000, 0x0000, 0x0002, /*2F913*/
  0xFB40, 0x0020, 0x0002, 0xF01E, 0x0000, 0x0000, 0x0002, /*2F914*/
  0xFB40, 0x0020, 0x0002, 0xF01B, 0x0000, 0x0000, 0x0002, /*2F915*/
  0xFB80, 0x0020, 0x0002, 0xBD96, 0x0000, 0x0000, 0x0002, /*2F916*/
  0xFB40, 0x0020, 0x0002, 0xF04A, 0x0000, 0x0000, 0x0002, /*2F917*/
  0xFB40, 0x0020, 0x0002, 0xF07D, 0x0000, 0x0000, 0x0002, /*2F918*/
  0xFB40, 0x0020, 0x0002, 0xF077, 0x0000, 0x0000, 0x0002, /*2F919*/
  0xFB40, 0x0020, 0x0002, 0xF0AD, 0x0000, 0x0000, 0x0002, /*2F91A*/
  0xFB84, 0x0020, 0x0002, 0x8525, 0x0000, 0x0000, 0x0002, /*2F91B*/
  0xFB40, 0x0020, 0x0002, 0xF145, 0x0000, 0x0000, 0x0002, /*2F91C*/
  0xFB84, 0x0020, 0x0002, 0xC263, 0x0000, 0x0000, 0x0002, /*2F91D*/
  0xFB40, 0x0020, 0x0002, 0xF19C, 0x0000, 0x0000, 0x0002, /*2F91E*/
  0xFB84, 0x0020, 0x0002, 0xC3AB, 0x0000, 0x0000, 0x0002, /*2F91F*/
  0xFB40, 0x0020, 0x0002, 0xF228, 0x0000, 0x0000, 0x0002, /*2F920*/
  0xFB40, 0x0020, 0x0002, 0xF235, 0x0000, 0x0000, 0x0002, /*2F921*/
  0xFB40, 0x0020, 0x0002, 0xF250, 0x0000, 0x0000, 0x0002, /*2F922*/
  0xFB84, 0x0020, 0x0002, 0xC608, 0x0000, 0x0000, 0x0002, /*2F923*/
  0xFB40, 0x0020, 0x0002, 0xF280, 0x0000, 0x0000, 0x0002, /*2F924*/
  0xFB40, 0x0020, 0x0002, 0xF295, 0x0000, 0x0000, 0x0002, /*2F925*/
  0xFB84, 0x0020, 0x0002, 0xC735, 0x0000, 0x0000, 0x0002, /*2F926*/
  0xFB84, 0x0020, 0x0002, 0xC814, 0x0000, 0x0000, 0x0002, /*2F927*/
  0xFB40, 0x0020, 0x0002, 0xF37A, 0x0000, 0x0000, 0x0002, /*2F928*/
  0xFB40, 0x0020, 0x0002, 0xF38B, 0x0000, 0x0000, 0x0002, /*2F929*/
  0xFB80, 0x0020, 0x0002, 0xBEAC, 0x0000, 0x0000, 0x0002, /*2F92A*/
  0xFB40, 0x0020, 0x0002, 0xF3A5, 0x0000, 0x0000, 0x0002, /*2F92B*/
  0xFB80, 0x0020, 0x0002, 0xBEB8, 0x0000, 0x0000, 0x0002, /*2F92C*/
  0xFB80, 0x0020, 0x0002, 0xBEB8, 0x0000, 0x0000, 0x0002, /*2F92D*/
  0xFB40, 0x0020, 0x0002, 0xF447, 0x0000, 0x0000, 0x0002, /*2F92E*/
  0xFB40, 0x0020, 0x0002, 0xF45C, 0x0000, 0x0000, 0x0002, /*2F92F*/
  0xFB40, 0x0020, 0x0002, 0xF471, 0x0000, 0x0000, 0x0002, /*2F930*/
  0xFB40, 0x0020, 0x0002, 0xF485, 0x0000, 0x0000, 0x0002, /*2F931*/
  0xFB40, 0x0020, 0x0002, 0xF4CA, 0x0000, 0x0000, 0x0002, /*2F932*/
  0xFB80, 0x0020, 0x0002, 0xBF1B, 0x0000, 0x0000, 0x0002, /*2F933*/
  0xFB40, 0x0020, 0x0002, 0xF524, 0x0000, 0x0000, 0x0002, /*2F934*/
  0xFB84, 0x0020, 0x0002, 0xCC36, 0x0000, 0x0000, 0x0002, /*2F935*/
  0xFB40, 0x0020, 0x0002, 0xF53E, 0x0000, 0x0000, 0x0002, /*2F936*/
  0xFB84, 0x0020, 0x0002, 0xCC92, 0x0000, 0x0000, 0x0002, /*2F937*/
  0xFB40, 0x0020, 0x0002, 0xF570, 0x0000, 0x0000, 0x0002, /*2F938*/
  0xFB84, 0x0020, 0x0002, 0xA19F, 0x0000, 0x0000, 0x0002, /*2F939*/
  0xFB40, 0x0020, 0x0002, 0xF610, 0x0000, 0x0000, 0x0002, /*2F93A*/
  0xFB84, 0x0020, 0x0002, 0xCFA1, 0x0000, 0x0000, 0x0002, /*2F93B*/
  0xFB84, 0x0020, 0x0002, 0xCFB8, 0x0000, 0x0000, 0x0002, /*2F93C*/
  0xFB84, 0x0020, 0x0002, 0xD044, 0x0000, 0x0000, 0x0002, /*2F93D*/
  0xFB80, 0x0020, 0x0002, 0xBFFC, 0x0000, 0x0000, 0x0002, /*2F93E*/
  0xFB80, 0x0020, 0x0002, 0xC008, 0x0000, 0x0000, 0x0002, /*2F93F*/
  0xFB40, 0x0020, 0x0002, 0xF6F4, 0x0000, 0x0000, 0x0002, /*2F940*/
  0xFB84, 0x0020, 0x0002, 0xD0F3, 0x0000, 0x0000, 0x0002, /*2F941*/
  0xFB84, 0x0020, 0x0002, 0xD0F2, 0x0000, 0x0000, 0x0002, /*2F942*/
  0xFB84, 0x0020, 0x0002, 0xD119, 0x0000, 0x0000, 0x0002, /*2F943*/
  0xFB84, 0x0020, 0x0002, 0xD133, 0x0000, 0x0000, 0x0002, /*2F944*/
  0xFB40, 0x0020, 0x0002, 0xF71E, 0x0000, 0x0000, 0x0002, /*2F945*/
  0xFB40, 0x0020, 0x0002, 0xF71F, 0x0000, 0x0000, 0x0002, /*2F946*/
  0xFB40, 0x0020, 0x0002, 0xF71F, 0x0000, 0x0000, 0x0002, /*2F947*/
  0xFB40, 0x0020, 0x0002, 0xF74A, 0x0000, 0x0000, 0x0002, /*2F948*/
  0xFB80, 0x0020, 0x0002, 0xC039, 0x0000, 0x0000, 0x0002, /*2F949*/
  0xFB40, 0x0020, 0x0002, 0xF78B, 0x0000, 0x0000, 0x0002, /*2F94A*/
  0xFB80, 0x0020, 0x0002, 0xC046, 0x0000, 0x0000, 0x0002, /*2F94B*/
  0xFB80, 0x0020, 0x0002, 0xC096, 0x0000, 0x0000, 0x0002, /*2F94C*/
  0xFB84, 0x0020, 0x0002, 0xD41D, 0x0000, 0x0000, 0x0002, /*2F94D*/
  0xFB40, 0x0020, 0x0002, 0xF84E, 0x0000, 0x0000, 0x0002, /*2F94E*/
  0xFB40, 0x0020, 0x0002, 0xF88C, 0x0000, 0x0000, 0x0002, /*2F94F*/
  0xFB40, 0x0020, 0x0002, 0xF8CC, 0x0000, 0x0000, 0x0002, /*2F950*/
  0xFB80, 0x0020, 0x0002, 0xC0E3, 0x0000, 0x0000, 0x0002, /*2F951*/
  0xFB84, 0x0020, 0x0002, 0xD626, 0x0000, 0x0000, 0x0002, /*2F952*/
  0xFB40, 0x0020, 0x0002, 0xF956, 0x0000, 0x0000, 0x0002, /*2F953*/
  0xFB84, 0x0020, 0x0002, 0xD69A, 0x0000, 0x0000, 0x0002, /*2F954*/
  0xFB84, 0x0020, 0x0002, 0xD6C5, 0x0000, 0x0000, 0x0002, /*2F955*/
  0xFB40, 0x0020, 0x0002, 0xF98F, 0x0000, 0x0000, 0x0002, /*2F956*/
  0xFB40, 0x0020, 0x0002, 0xF9EB, 0x0000, 0x0000, 0x0002, /*2F957*/
  0xFB80, 0x0020, 0x0002, 0xC12F, 0x0000, 0x0000, 0x0002, /*2F958*/
  0xFB40, 0x0020, 0x0002, 0xFA40, 0x0000, 0x0000, 0x0002, /*2F959*/
  0xFB40, 0x0020, 0x0002, 0xFA4A, 0x0000, 0x0000, 0x0002, /*2F95A*/
  0xFB40, 0x0020, 0x0002, 0xFA4F, 0x0000, 0x0000, 0x0002, /*2F95B*/
  0xFB84, 0x0020, 0x0002, 0xD97C, 0x0000, 0x0000, 0x0002, /*2F95C*/
  0xFB84, 0x0020, 0x0002, 0xDAA7, 0x0000, 0x0000, 0x0002, /*2F95D*/
  0xFB84, 0x0020, 0x0002, 0xDAA7, 0x0000, 0x0000, 0x0002, /*2F95E*/
  0xFB40, 0x0020, 0x0002, 0xFAEE, 0x0000, 0x0000, 0x0002, /*2F95F*/
  0xFB80, 0x0020, 0x0002, 0xC202, 0x0000, 0x0000, 0x0002, /*2F960*/
  0xFB84, 0x0020, 0x0002, 0xDBAB, 0x0000, 0x0000, 0x0002, /*2F961*/
  0xFB40, 0x0020, 0x0002, 0xFBC6, 0x0000, 0x0000, 0x0002, /*2F962*/
  0xFB40, 0x0020, 0x0002, 0xFBC9, 0x0000, 0x0000, 0x0002, /*2F963*/
  0xFB80, 0x0020, 0x0002, 0xC227, 0x0000, 0x0000, 0x0002, /*2F964*/
  0xFB84, 0x0020, 0x0002, 0xDC80, 0x0000, 0x0000, 0x0002, /*2F965*/
  0xFB40, 0x0020, 0x0002, 0xFCD2, 0x0000, 0x0000, 0x0002, /*2F966*/
  0xFB80, 0x0020, 0x0002, 0xC2A0, 0x0000, 0x0000, 0x0002, /*2F967*/
  0xFB40, 0x0020, 0x0002, 0xFCE8, 0x0000, 0x0000, 0x0002, /*2F968*/
  0xFB40, 0x0020, 0x0002, 0xFCE3, 0x0000, 0x0000, 0x0002, /*2F969*/
  0xFB40, 0x0020, 0x0002, 0xFD00, 0x0000, 0x0000, 0x0002, /*2F96A*/
  0xFB84, 0x0020, 0x0002, 0xDF86, 0x0000, 0x0000, 0x0002, /*2F96B*/
  0xFB40, 0x0020, 0x0002, 0xFD63, 0x0000, 0x0000, 0x0002, /*2F96C*/
  0xFB80, 0x0020, 0x0002, 0xC301, 0x0000, 0x0000, 0x0002, /*2F96D*/
  0xFB40, 0x0020, 0x0002, 0xFDC7, 0x0000, 0x0000, 0x0002, /*2F96E*/
  0xFB40, 0x0020, 0x0002, 0xFE02, 0x0000, 0x0000, 0x0002, /*2F96F*/
  0xFB40, 0x0020, 0x0002, 0xFE45, 0x0000, 0x0000, 0x0002, /*2F970*/
  0xFB80, 0x0020, 0x0002, 0xC334, 0x0000, 0x0000, 0x0002, /*2F971*/
  0xFB84, 0x0020, 0x0002, 0xE228, 0x0000, 0x0000, 0x0002, /*2F972*/
  0xFB84, 0x0020, 0x0002, 0xE247, 0x0000, 0x0000, 0x0002, /*2F973*/
  0xFB80, 0x0020, 0x0002, 0xC359, 0x0000, 0x0000, 0x0002, /*2F974*/
  0xFB84, 0x0020, 0x0002, 0xE2D9, 0x0000, 0x0000, 0x0002, /*2F975*/
  0xFB40, 0x0020, 0x0002, 0xFF7A, 0x0000, 0x0000, 0x0002, /*2F976*/
  0xFB84, 0x0020, 0x0002, 0xE33E, 0x0000, 0x0000, 0x0002, /*2F977*/
  0xFB40, 0x0020, 0x0002, 0xFF95, 0x0000, 0x0000, 0x0002, /*2F978*/
  0xFB40, 0x0020, 0x0002, 0xFFFA, 0x0000, 0x0000, 0x0002, /*2F979*/
  0xFB41, 0x0020, 0x0002, 0x8005, 0x0000, 0x0000, 0x0002, /*2F97A*/
  0xFB84, 0x0020, 0x0002, 0xE4DA, 0x0000, 0x0000, 0x0002, /*2F97B*/
  0xFB84, 0x0020, 0x0002, 0xE523, 0x0000, 0x0000, 0x0002, /*2F97C*/
  0xFB41, 0x0020, 0x0002, 0x8060, 0x0000, 0x0000, 0x0002, /*2F97D*/
  0xFB84, 0x0020, 0x0002, 0xE5A8, 0x0000, 0x0000, 0x0002, /*2F97E*/
  0xFB41, 0x0020, 0x0002, 0x8070, 0x0000, 0x0000, 0x0002, /*2F97F*/
  0xFB84, 0x0020, 0x0002, 0xB35F, 0x0000, 0x0000, 0x0002, /*2F980*/
  0xFB80, 0x0020, 0x0002, 0xC3D5, 0x0000, 0x0000, 0x0002, /*2F981*/
  0xFB41, 0x0020, 0x0002, 0x80B2, 0x0000, 0x0000, 0x0002, /*2F982*/
  0xFB41, 0x0020, 0x0002, 0x8103, 0x0000, 0x0000, 0x0002, /*2F983*/
  0xFB80, 0x0020, 0x0002, 0xC40B, 0x0000, 0x0000, 0x0002, /*2F984*/
  0xFB41, 0x0020, 0x0002, 0x813E, 0x0000, 0x0000, 0x0002, /*2F985*/
  0xFB40, 0x0020, 0x0002, 0xDAB5, 0x0000, 0x0000, 0x0002, /*2F986*/
  0xFB84, 0x0020, 0x0002, 0xE7A7, 0x0000, 0x0000, 0x0002, /*2F987*/
  0xFB84, 0x0020, 0x0002, 0xE7B5, 0x0000, 0x0000, 0x0002, /*2F988*/
  0xFB84, 0x0020, 0x0002, 0xB393, 0x0000, 0x0000, 0x0002, /*2F989*/
  0xFB84, 0x0020, 0x0002, 0xB39C, 0x0000, 0x0000, 0x0002, /*2F98A*/
  0xFB41, 0x0020, 0x0002, 0x8201, 0x0000, 0x0000, 0x0002, /*2F98B*/
  0xFB41, 0x0020, 0x0002, 0x8204, 0x0000, 0x0000, 0x0002, /*2F98C*/
  0xFB41, 0x0020, 0x0002, 0x8F9E, 0x0000, 0x0000, 0x0002, /*2F98D*/
  0xFB80, 0x0020, 0x0002, 0xC46B, 0x0000, 0x0000, 0x0002, /*2F98E*/
  0xFB41, 0x0020, 0x0002, 0x8291, 0x0000, 0x0000, 0x0002, /*2F98F*/
  0xFB41, 0x0020, 0x0002, 0x828B, 0x0000, 0x0000, 0x0002, /*2F990*/
  0xFB41, 0x0020, 0x0002, 0x829D, 0x0000, 0x0000, 0x0002, /*2F991*/
  0xFB40, 0x0020, 0x0002, 0xD2B3, 0x0000, 0x0000, 0x0002, /*2F992*/
  0xFB41, 0x0020, 0x0002, 0x82B1, 0x0000, 0x0000, 0x0002, /*2F993*/
  0xFB41, 0x0020, 0x0002, 0x82B3, 0x0000, 0x0000, 0x0002, /*2F994*/
  0xFB41, 0x0020, 0x0002, 0x82BD, 0x0000, 0x0000, 0x0002, /*2F995*/
  0xFB41, 0x0020, 0x0002, 0x82E6, 0x0000, 0x0000, 0x0002, /*2F996*/
  0xFB84, 0x0020, 0x0002, 0xEB3C, 0x0000, 0x0000, 0x0002, /*2F997*/
  0xFB41, 0x0020, 0x0002, 0x82E5, 0x0000, 0x0000, 0x0002, /*2F998*/
  0xFB41, 0x0020, 0x0002, 0x831D, 0x0000, 0x0000, 0x0002, /*2F999*/
  0xFB41, 0x0020, 0x0002, 0x8363, 0x0000, 0x0000, 0x0002, /*2F99A*/
  0xFB41, 0x0020, 0x0002, 0x83AD, 0x0000, 0x0000, 0x0002, /*2F99B*/
  0xFB41, 0x0020, 0x0002, 0x8323, 0x0000, 0x0000, 0x0002, /*2F99C*/
  0xFB41, 0x0020, 0x0002, 0x83BD, 0x0000, 0x0000, 0x0002, /*2F99D*/
  0xFB41, 0x0020, 0x0002, 0x83E7, 0x0000, 0x0000, 0x0002, /*2F99E*/
  0xFB41, 0x0020, 0x0002, 0x8457, 0x0000, 0x0000, 0x0002, /*2F99F*/
  0xFB41, 0x0020, 0x0002, 0x8353, 0x0000, 0x0000, 0x0002, /*2F9A0*/
  0xFB41, 0x0020, 0x0002, 0x83CA, 0x0000, 0x0000, 0x0002, /*2F9A1*/
  0xFB41, 0x0020, 0x0002, 0x83CC, 0x0000, 0x0000, 0x0002, /*2F9A2*/
  0xFB41, 0x0020, 0x0002, 0x83DC, 0x0000, 0x0000, 0x0002, /*2F9A3*/
  0xFB84, 0x0020, 0x0002, 0xEC36, 0x0000, 0x0000, 0x0002, /*2F9A4*/
  0xFB84, 0x0020, 0x0002, 0xED6B, 0x0000, 0x0000, 0x0002, /*2F9A5*/
  0xFB84, 0x0020, 0x0002, 0xECD5, 0x0000, 0x0000, 0x0002, /*2F9A6*/
  0xFB80, 0x0020, 0x0002, 0xC52B, 0x0000, 0x0000, 0x0002, /*2F9A7*/
  0xFB41, 0x0020, 0x0002, 0x84F1, 0x0000, 0x0000, 0x0002, /*2F9A8*/
  0xFB41, 0x0020, 0x0002, 0x84F3, 0x0000, 0x0000, 0x0002, /*2F9A9*/
  0xFB41, 0x0020, 0x0002, 0x8516, 0x0000, 0x0000, 0x0002, /*2F9AA*/
  0xFB84, 0x0020, 0x0002, 0xF3CA, 0x0000, 0x0000, 0x0002, /*2F9AB*/
  0xFB41, 0x0020, 0x0002, 0x8564, 0x0000, 0x0000, 0x0002, /*2F9AC*/
  0xFB84, 0x0020, 0x0002, 0xEF2C, 0x0000, 0x0000, 0x0002, /*2F9AD*/
  0xFB80, 0x0020, 0x0002, 0xC55D, 0x0000, 0x0000, 0x0002, /*2F9AE*/
  0xFB80, 0x0020, 0x0002, 0xC561, 0x0000, 0x0000, 0x0002, /*2F9AF*/
  0xFB84, 0x0020, 0x0002, 0xEFB1, 0x0000, 0x0000, 0x0002, /*2F9B0*/
  0xFB84, 0x0020, 0x0002, 0xF0D2, 0x0000, 0x0000, 0x0002, /*2F9B1*/
  0xFB80, 0x0020, 0x0002, 0xC56B, 0x0000, 0x0000, 0x0002, /*2F9B2*/
  0xFB41, 0x0020, 0x0002, 0x8650, 0x0000, 0x0000, 0x0002, /*2F9B3*/
  0xFB41, 0x0020, 0x0002, 0x865C, 0x0000, 0x0000, 0x0002, /*2F9B4*/
  0xFB41, 0x0020, 0x0002, 0x8667, 0x0000, 0x0000, 0x0002, /*2F9B5*/
  0xFB41, 0x0020, 0x0002, 0x8669, 0x0000, 0x0000, 0x0002, /*2F9B6*/
  0xFB41, 0x0020, 0x0002, 0x86A9, 0x0000, 0x0000, 0x0002, /*2F9B7*/
  0xFB41, 0x0020, 0x0002, 0x8688, 0x0000, 0x0000, 0x0002, /*2F9B8*/
  0xFB41, 0x0020, 0x0002, 0x870E, 0x0000, 0x0000, 0x0002, /*2F9B9*/
  0xFB41, 0x0020, 0x0002, 0x86E2, 0x0000, 0x0000, 0x0002, /*2F9BA*/
  0xFB41, 0x0020, 0x0002, 0x8779, 0x0000, 0x0000, 0x0002, /*2F9BB*/
  0xFB41, 0x0020, 0x0002, 0x8728, 0x0000, 0x0000, 0x0002, /*2F9BC*/
  0xFB41, 0x0020, 0x0002, 0x876B, 0x0000, 0x0000, 0x0002, /*2F9BD*/
  0xFB41, 0x0020, 0x0002, 0x8786, 0x0000, 0x0000, 0x0002, /*2F9BE*/
  0xFB80, 0x0020, 0x0002, 0xC5D7, 0x0000, 0x0000, 0x0002, /*2F9BF*/
  0xFB41, 0x0020, 0x0002, 0x87E1, 0x0000, 0x0000, 0x0002, /*2F9C0*/
  0xFB41, 0x0020, 0x0002, 0x8801, 0x0000, 0x0000, 0x0002, /*2F9C1*/
  0xFB80, 0x0020, 0x0002, 0xC5F9, 0x0000, 0x0000, 0x0002, /*2F9C2*/
  0xFB41, 0x0020, 0x0002, 0x8860, 0x0000, 0x0000, 0x0002, /*2F9C3*/
  0xFB41, 0x0020, 0x0002, 0x8863, 0x0000, 0x0000, 0x0002, /*2F9C4*/
  0xFB84, 0x0020, 0x0002, 0xF667, 0x0000, 0x0000, 0x0002, /*2F9C5*/
  0xFB41, 0x0020, 0x0002, 0x88D7, 0x0000, 0x0000, 0x0002, /*2F9C6*/
  0xFB41, 0x0020, 0x0002, 0x88DE, 0x0000, 0x0000, 0x0002, /*2F9C7*/
  0xFB80, 0x0020, 0x0002, 0xC635, 0x0000, 0x0000, 0x0002, /*2F9C8*/
  0xFB41, 0x0020, 0x0002, 0x88FA, 0x0000, 0x0000, 0x0002, /*2F9C9*/
  0xFB80, 0x0020, 0x0002, 0xB4BB, 0x0000, 0x0000, 0x0002, /*2F9CA*/
  0xFB84, 0x0020, 0x0002, 0xF8AE, 0x0000, 0x0000, 0x0002, /*2F9CB*/
  0xFB84, 0x0020, 0x0002, 0xF966, 0x0000, 0x0000, 0x0002, /*2F9CC*/
  0xFB80, 0x0020, 0x0002, 0xC6BE, 0x0000, 0x0000, 0x0002, /*2F9CD*/
  0xFB80, 0x0020, 0x0002, 0xC6C7, 0x0000, 0x0000, 0x0002, /*2F9CE*/
  0xFB41, 0x0020, 0x0002, 0x8AA0, 0x0000, 0x0000, 0x0002, /*2F9CF*/
  0xFB41, 0x0020, 0x0002, 0x8AED, 0x0000, 0x0000, 0x0002, /*2F9D0*/
  0xFB41, 0x0020, 0x0002, 0x8B8A, 0x0000, 0x0000, 0x0002, /*2F9D1*/
  0xFB41, 0x0020, 0x0002, 0x8C55, 0x0000, 0x0000, 0x0002, /*2F9D2*/
  0xFB84, 0x0020, 0x0002, 0xFCA8, 0x0000, 0x0000, 0x0002, /*2F9D3*/
  0xFB41, 0x0020, 0x0002, 0x8CAB, 0x0000, 0x0000, 0x0002, /*2F9D4*/
  0xFB41, 0x0020, 0x0002, 0x8CC1, 0x0000, 0x0000, 0x0002, /*2F9D5*/
  0xFB41, 0x0020, 0x0002, 0x8D1B, 0x0000, 0x0000, 0x0002, /*2F9D6*/
  0xFB41, 0x0020, 0x0002, 0x8D77, 0x0000, 0x0000, 0x0002, /*2F9D7*/
  0xFB84, 0x0020, 0x0002, 0xFF2F, 0x0000, 0x0000, 0x0002, /*2F9D8*/
  0xFB84, 0x0020, 0x0002, 0x8804, 0x0000, 0x0000, 0x0002, /*2F9D9*/
  0xFB41, 0x0020, 0x0002, 0x8DCB, 0x0000, 0x0000, 0x0002, /*2F9DA*/
  0xFB41, 0x0020, 0x0002, 0x8DBC, 0x0000, 0x0000, 0x0002, /*2F9DB*/
  0xFB41, 0x0020, 0x0002, 0x8DF0, 0x0000, 0x0000, 0x0002, /*2F9DC*/
  0xFB84, 0x0020, 0x0002, 0x88DE, 0x0000, 0x0000, 0x0002, /*2F9DD*/
  0xFB41, 0x0020, 0x0002, 0x8ED4, 0x0000, 0x0000, 0x0002, /*2F9DE*/
  0xFB41, 0x0020, 0x0002, 0x8F38, 0x0000, 0x0000, 0x0002, /*2F9DF*/
  0xFB85, 0x0020, 0x0002, 0x85D2, 0x0000, 0x0000, 0x0002, /*2F9E0*/
  0xFB85, 0x0020, 0x0002, 0x85ED, 0x0000, 0x0000, 0x0002, /*2F9E1*/
  0xFB41, 0x0020, 0x0002, 0x9094, 0x0000, 0x0000, 0x0002, /*2F9E2*/
  0xFB41, 0x0020, 0x0002, 0x90F1, 0x0000, 0x0000, 0x0002, /*2F9E3*/
  0xFB41, 0x0020, 0x0002, 0x9111, 0x0000, 0x0000, 0x0002, /*2F9E4*/
  0xFB85, 0x0020, 0x0002, 0x872E, 0x0000, 0x0000, 0x0002, /*2F9E5*/
  0xFB41, 0x0020, 0x0002, 0x911B, 0x0000, 0x0000, 0x0002, /*2F9E6*/
  0xFB41, 0x0020, 0x0002, 0x9238, 0x0000, 0x0000, 0x0002, /*2F9E7*/
  0xFB41, 0x0020, 0x0002, 0x92D7, 0x0000, 0x0000, 0x0002, /*2F9E8*/
  0xFB41, 0x0020, 0x0002, 0x92D8, 0x0000, 0x0000, 0x0002, /*2F9E9*/
  0xFB41, 0x0020, 0x0002, 0x927C, 0x0000, 0x0000, 0x0002, /*2F9EA*/
  0xFB41, 0x0020, 0x0002, 0x93F9, 0x0000, 0x0000, 0x0002, /*2F9EB*/
  0xFB41, 0x0020, 0x0002, 0x9415, 0x0000, 0x0000, 0x0002, /*2F9EC*/
  0xFB85, 0x0020, 0x0002, 0x8BFA, 0x0000, 0x0000, 0x0002, /*2F9ED*/
  0xFB41, 0x0020, 0x0002, 0x958B, 0x0000, 0x0000, 0x0002, /*2F9EE*/
  0xFB80, 0x0020, 0x0002, 0xC995, 0x0000, 0x0000, 0x0002, /*2F9EF*/
  0xFB41, 0x0020, 0x0002, 0x95B7, 0x0000, 0x0000, 0x0002, /*2F9F0*/
  0xFB85, 0x0020, 0x0002, 0x8D77, 0x0000, 0x0000, 0x0002, /*2F9F1*/
  0xFB80, 0x0020, 0x0002, 0xC9E6, 0x0000, 0x0000, 0x0002, /*2F9F2*/
  0xFB41, 0x0020, 0x0002, 0x96C3, 0x0000, 0x0000, 0x0002, /*2F9F3*/
  0xFB40, 0x0020, 0x0002, 0xDDB2, 0x0000, 0x0000, 0x0002, /*2F9F4*/
  0xFB41, 0x0020, 0x0002, 0x9723, 0x0000, 0x0000, 0x0002, /*2F9F5*/
  0xFB85, 0x0020, 0x0002, 0x9145, 0x0000, 0x0000, 0x0002, /*2F9F6*/
  0xFB85, 0x0020, 0x0002, 0x921A, 0x0000, 0x0000, 0x0002, /*2F9F7*/
  0xFB80, 0x0020, 0x0002, 0xCA6E, 0x0000, 0x0000, 0x0002, /*2F9F8*/
  0xFB80, 0x0020, 0x0002, 0xCA76, 0x0000, 0x0000, 0x0002, /*2F9F9*/
  0xFB41, 0x0020, 0x0002, 0x97E0, 0x0000, 0x0000, 0x0002, /*2F9FA*/
  0xFB85, 0x0020, 0x0002, 0x940A, 0x0000, 0x0000, 0x0002, /*2F9FB*/
  0xFB80, 0x0020, 0x0002, 0xCAB2, 0x0000, 0x0000, 0x0002, /*2F9FC*/
  0xFB85, 0x0020, 0x0002, 0x9496, 0x0000, 0x0000, 0x0002, /*2F9FD*/
  0xFB41, 0x0020, 0x0002, 0x980B, 0x0000, 0x0000, 0x0002, /*2F9FE*/
  0xFB41, 0x0020, 0x0002, 0x980B, 0x0000, 0x0000, 0x0002  /*2F9FF*/
};

uint16 uca900_p2FA[]= {
  0xFB41, 0x0020, 0x0002, 0x9829, 0x0000, 0x0000, 0x0002, /*2FA00*/
  0xFB85, 0x0020, 0x0002, 0x95B6, 0x0000, 0x0000, 0x0002, /*2FA01*/
  0xFB41, 0x0020, 0x0002, 0x98E2, 0x0000, 0x0000, 0x0002, /*2FA02*/
  0xFB80, 0x0020, 0x0002, 0xCB33, 0x0000, 0x0000, 0x0002, /*2FA03*/
  0xFB41, 0x0020, 0x0002, 0x9929, 0x0000, 0x0000, 0x0002, /*2FA04*/
  0xFB41, 0x0020, 0x0002, 0x99A7, 0x0000, 0x0000, 0x0002, /*2FA05*/
  0xFB41, 0x0020, 0x0002, 0x99C2, 0x0000, 0x0000, 0x0002, /*2FA06*/
  0xFB41, 0x0020, 0x0002, 0x99FE, 0x0000, 0x0000, 0x0002, /*2FA07*/
  0xFB80, 0x0020, 0x0002, 0xCBCE, 0x0000, 0x0000, 0x0002, /*2FA08*/
  0xFB85, 0x0020, 0x0002, 0x9B30, 0x0000, 0x0000, 0x0002, /*2FA09*/
  0xFB41, 0x0020, 0x0002, 0x9B12, 0x0000, 0x0000, 0x0002, /*2FA0A*/
  0xFB41, 0x0020, 0x0002, 0x9C40, 0x0000, 0x0000, 0x0002, /*2FA0B*/
  0xFB41, 0x0020, 0x0002, 0x9CFD, 0x0000, 0x0000, 0x0002, /*2FA0C*/
  0xFB80, 0x0020, 0x0002, 0xCCCE, 0x0000, 0x0000, 0x0002, /*2FA0D*/
  0xFB80, 0x0020, 0x0002, 0xCCED, 0x0000, 0x0000, 0x0002, /*2FA0E*/
  0xFB41, 0x0020, 0x0002, 0x9D67, 0x0000, 0x0000, 0x0002, /*2FA0F*/
  0xFB85, 0x0020, 0x0002, 0xA0CE, 0x0000, 0x0000, 0x0002, /*2FA10*/
  0xFB80, 0x0020, 0x0002, 0xCCF8, 0x0000, 0x0000, 0x0002, /*2FA11*/
  0xFB85, 0x0020, 0x0002, 0xA105, 0x0000, 0x0000, 0x0002, /*2FA12*/
  0xFB85, 0x0020, 0x0002, 0xA20E, 0x0000, 0x0000, 0x0002, /*2FA13*/
  0xFB85, 0x0020, 0x0002, 0xA291, 0x0000, 0x0000, 0x0002, /*2FA14*/
  0xFB41, 0x0020, 0x0002, 0x9EBB, 0x0000, 0x0000, 0x0002, /*2FA15*/
  0xFB80, 0x0020, 0x0002, 0xCD56, 0x0000, 0x0000, 0x0002, /*2FA16*/
  0xFB41, 0x0020, 0x0002, 0x9EF9, 0x0000, 0x0000, 0x0002, /*2FA17*/
  0xFB41, 0x0020, 0x0002, 0x9EFE, 0x0000, 0x0000, 0x0002, /*2FA18*/
  0xFB41, 0x0020, 0x0002, 0x9F05, 0x0000, 0x0000, 0x0002, /*2FA19*/
  0xFB41, 0x0020, 0x0002, 0x9F0F, 0x0000, 0x0000, 0x0002, /*2FA1A*/
  0xFB41, 0x0020, 0x0002, 0x9F16, 0x0000, 0x0000, 0x0002, /*2FA1B*/
  0xFB41, 0x0020, 0x0002, 0x9F3B, 0x0000, 0x0000, 0x0002, /*2FA1C*/
  0xFB85, 0x0020, 0x0002, 0xA600, 0x0000, 0x0000, 0x0002, /*2FA1D*/
  0xFBC5, 0x0020, 0x0002, 0xFA1E, 0x0000, 0x0000, 0x0002, /*2FA1E*/
  0xFBC5, 0x0020, 0x0002, 0xFA1F, 0x0000, 0x0000, 0x0002, /*2FA1F*/
  0xFBC5, 0x0020, 0x0002, 0xFA20, 0x0000, 0x0000, 0x0002, /*2FA20*/
  0xFBC5, 0x0020, 0x0002, 0xFA21, 0x0000, 0x0000, 0x0002, /*2FA21*/
  0xFBC5, 0x0020, 0x0002, 0xFA22, 0x0000, 0x0000, 0x0002, /*2FA22*/
  0xFBC5, 0x0020, 0x0002, 0xFA23, 0x0000, 0x0000, 0x0002, /*2FA23*/
  0xFBC5, 0x0020, 0x0002, 0xFA24, 0x0000, 0x0000, 0x0002, /*2FA24*/
  0xFBC5, 0x0020, 0x0002, 0xFA25, 0x0000, 0x0000, 0x0002, /*2FA25*/
  0xFBC5, 0x0020, 0x0002, 0xFA26, 0x0000, 0x0000, 0x0002, /*2FA26*/
  0xFBC5, 0x0020, 0x0002, 0xFA27, 0x0000, 0x0000, 0x0002, /*2FA27*/
  0xFBC5, 0x0020, 0x0002, 0xFA28, 0x0000, 0x0000, 0x0002, /*2FA28*/
  0xFBC5, 0x0020, 0x0002, 0xFA29, 0x0000, 0x0000, 0x0002, /*2FA29*/
  0xFBC5, 0x0020, 0x0002, 0xFA2A, 0x0000, 0x0000, 0x0002, /*2FA2A*/
  0xFBC5, 0x0020, 0x0002, 0xFA2B, 0x0000, 0x0000, 0x0002, /*2FA2B*/
  0xFBC5, 0x0020, 0x0002, 0xFA2C, 0x0000, 0x0000, 0x0002, /*2FA2C*/
  0xFBC5, 0x0020, 0x0002, 0xFA2D, 0x0000, 0x0000, 0x0002, /*2FA2D*/
  0xFBC5, 0x0020, 0x0002, 0xFA2E, 0x0000, 0x0000, 0x0002, /*2FA2E*/
  0xFBC5, 0x0020, 0x0002, 0xFA2F, 0x0000, 0x0000, 0x0002, /*2FA2F*/
  0xFBC5, 0x0020, 0x0002, 0xFA30, 0x0000, 0x0000, 0x0002, /*2FA30*/
  0xFBC5, 0x0020, 0x0002, 0xFA31, 0x0000, 0x0000, 0x0002, /*2FA31*/
  0xFBC5, 0x0020, 0x0002, 0xFA32, 0x0000, 0x0000, 0x0002, /*2FA32*/
  0xFBC5, 0x0020, 0x0002, 0xFA33, 0x0000, 0x0000, 0x0002, /*2FA33*/
  0xFBC5, 0x0020, 0x0002, 0xFA34, 0x0000, 0x0000, 0x0002, /*2FA34*/
  0xFBC5, 0x0020, 0x0002, 0xFA35, 0x0000, 0x0000, 0x0002, /*2FA35*/
  0xFBC5, 0x0020, 0x0002, 0xFA36, 0x0000, 0x0000, 0x0002, /*2FA36*/
  0xFBC5, 0x0020, 0x0002, 0xFA37, 0x0000, 0x0000, 0x0002, /*2FA37*/
  0xFBC5, 0x0020, 0x0002, 0xFA38, 0x0000, 0x0000, 0x0002, /*2FA38*/
  0xFBC5, 0x0020, 0x0002, 0xFA39, 0x0000, 0x0000, 0x0002, /*2FA39*/
  0xFBC5, 0x0020, 0x0002, 0xFA3A, 0x0000, 0x0000, 0x0002, /*2FA3A*/
  0xFBC5, 0x0020, 0x0002, 0xFA3B, 0x0000, 0x0000, 0x0002, /*2FA3B*/
  0xFBC5, 0x0020, 0x0002, 0xFA3C, 0x0000, 0x0000, 0x0002, /*2FA3C*/
  0xFBC5, 0x0020, 0x0002, 0xFA3D, 0x0000, 0x0000, 0x0002, /*2FA3D*/
  0xFBC5, 0x0020, 0x0002, 0xFA3E, 0x0000, 0x0000, 0x0002, /*2FA3E*/
  0xFBC5, 0x0020, 0x0002, 0xFA3F, 0x0000, 0x0000, 0x0002, /*2FA3F*/
  0xFBC5, 0x0020, 0x0002, 0xFA40, 0x0000, 0x0000, 0x0002, /*2FA40*/
  0xFBC5, 0x0020, 0x0002, 0xFA41, 0x0000, 0x0000, 0x0002, /*2FA41*/
  0xFBC5, 0x0020, 0x0002, 0xFA42, 0x0000, 0x0000, 0x0002, /*2FA42*/
  0xFBC5, 0x0020, 0x0002, 0xFA43, 0x0000, 0x0000, 0x0002, /*2FA43*/
  0xFBC5, 0x0020, 0x0002, 0xFA44, 0x0000, 0x0000, 0x0002, /*2FA44*/
  0xFBC5, 0x0020, 0x0002, 0xFA45, 0x0000, 0x0000, 0x0002, /*2FA45*/
  0xFBC5, 0x0020, 0x0002, 0xFA46, 0x0000, 0x0000, 0x0002, /*2FA46*/
  0xFBC5, 0x0020, 0x0002, 0xFA47, 0x0000, 0x0000, 0x0002, /*2FA47*/
  0xFBC5, 0x0020, 0x0002, 0xFA48, 0x0000, 0x0000, 0x0002, /*2FA48*/
  0xFBC5, 0x0020, 0x0002, 0xFA49, 0x0000, 0x0000, 0x0002, /*2FA49*/
  0xFBC5, 0x0020, 0x0002, 0xFA4A, 0x0000, 0x0000, 0x0002, /*2FA4A*/
  0xFBC5, 0x0020, 0x0002, 0xFA4B, 0x0000, 0x0000, 0x0002, /*2FA4B*/
  0xFBC5, 0x0020, 0x0002, 0xFA4C, 0x0000, 0x0000, 0x0002, /*2FA4C*/
  0xFBC5, 0x0020, 0x0002, 0xFA4D, 0x0000, 0x0000, 0x0002, /*2FA4D*/
  0xFBC5, 0x0020, 0x0002, 0xFA4E, 0x0000, 0x0000, 0x0002, /*2FA4E*/
  0xFBC5, 0x0020, 0x0002, 0xFA4F, 0x0000, 0x0000, 0x0002, /*2FA4F*/
  0xFBC5, 0x0020, 0x0002, 0xFA50, 0x0000, 0x0000, 0x0002, /*2FA50*/
  0xFBC5, 0x0020, 0x0002, 0xFA51, 0x0000, 0x0000, 0x0002, /*2FA51*/
  0xFBC5, 0x0020, 0x0002, 0xFA52, 0x0000, 0x0000, 0x0002, /*2FA52*/
  0xFBC5, 0x0020, 0x0002, 0xFA53, 0x0000, 0x0000, 0x0002, /*2FA53*/
  0xFBC5, 0x0020, 0x0002, 0xFA54, 0x0000, 0x0000, 0x0002, /*2FA54*/
  0xFBC5, 0x0020, 0x0002, 0xFA55, 0x0000, 0x0000, 0x0002, /*2FA55*/
  0xFBC5, 0x0020, 0x0002, 0xFA56, 0x0000, 0x0000, 0x0002, /*2FA56*/
  0xFBC5, 0x0020, 0x0002, 0xFA57, 0x0000, 0x0000, 0x0002, /*2FA57*/
  0xFBC5, 0x0020, 0x0002, 0xFA58, 0x0000, 0x0000, 0x0002, /*2FA58*/
  0xFBC5, 0x0020, 0x0002, 0xFA59, 0x0000, 0x0000, 0x0002, /*2FA59*/
  0xFBC5, 0x0020, 0x0002, 0xFA5A, 0x0000, 0x0000, 0x0002, /*2FA5A*/
  0xFBC5, 0x0020, 0x0002, 0xFA5B, 0x0000, 0x0000, 0x0002, /*2FA5B*/
  0xFBC5, 0x0020, 0x0002, 0xFA5C, 0x0000, 0x0000, 0x0002, /*2FA5C*/
  0xFBC5, 0x0020, 0x0002, 0xFA5D, 0x0000, 0x0000, 0x0002, /*2FA5D*/
  0xFBC5, 0x0020, 0x0002, 0xFA5E, 0x0000, 0x0000, 0x0002, /*2FA5E*/
  0xFBC5, 0x0020, 0x0002, 0xFA5F, 0x0000, 0x0000, 0x0002, /*2FA5F*/
  0xFBC5, 0x0020, 0x0002, 0xFA60, 0x0000, 0x0000, 0x0002, /*2FA60*/
  0xFBC5, 0x0020, 0x0002, 0xFA61, 0x0000, 0x0000, 0x0002, /*2FA61*/
  0xFBC5, 0x0020, 0x0002, 0xFA62, 0x0000, 0x0000, 0x0002, /*2FA62*/
  0xFBC5, 0x0020, 0x0002, 0xFA63, 0x0000, 0x0000, 0x0002, /*2FA63*/
  0xFBC5, 0x0020, 0x0002, 0xFA64, 0x0000, 0x0000, 0x0002, /*2FA64*/
  0xFBC5, 0x0020, 0x0002, 0xFA65, 0x0000, 0x0000, 0x0002, /*2FA65*/
  0xFBC5, 0x0020, 0x0002, 0xFA66, 0x0000, 0x0000, 0x0002, /*2FA66*/
  0xFBC5, 0x0020, 0x0002, 0xFA67, 0x0000, 0x0000, 0x0002, /*2FA67*/
  0xFBC5, 0x0020, 0x0002, 0xFA68, 0x0000, 0x0000, 0x0002, /*2FA68*/
  0xFBC5, 0x0020, 0x0002, 0xFA69, 0x0000, 0x0000, 0x0002, /*2FA69*/
  0xFBC5, 0x0020, 0x0002, 0xFA6A, 0x0000, 0x0000, 0x0002, /*2FA6A*/
  0xFBC5, 0x0020, 0x0002, 0xFA6B, 0x0000, 0x0000, 0x0002, /*2FA6B*/
  0xFBC5, 0x0020, 0x0002, 0xFA6C, 0x0000, 0x0000, 0x0002, /*2FA6C*/
  0xFBC5, 0x0020, 0x0002, 0xFA6D, 0x0000, 0x0000, 0x0002, /*2FA6D*/
  0xFBC5, 0x0020, 0x0002, 0xFA6E, 0x0000, 0x0000, 0x0002, /*2FA6E*/
  0xFBC5, 0x0020, 0x0002, 0xFA6F, 0x0000, 0x0000, 0x0002, /*2FA6F*/
  0xFBC5, 0x0020, 0x0002, 0xFA70, 0x0000, 0x0000, 0x0002, /*2FA70*/
  0xFBC5, 0x0020, 0x0002, 0xFA71, 0x0000, 0x0000, 0x0002, /*2FA71*/
  0xFBC5, 0x0020, 0x0002, 0xFA72, 0x0000, 0x0000, 0x0002, /*2FA72*/
  0xFBC5, 0x0020, 0x0002, 0xFA73, 0x0000, 0x0000, 0x0002, /*2FA73*/
  0xFBC5, 0x0020, 0x0002, 0xFA74, 0x0000, 0x0000, 0x0002, /*2FA74*/
  0xFBC5, 0x0020, 0x0002, 0xFA75, 0x0000, 0x0000, 0x0002, /*2FA75*/
  0xFBC5, 0x0020, 0x0002, 0xFA76, 0x0000, 0x0000, 0x0002, /*2FA76*/
  0xFBC5, 0x0020, 0x0002, 0xFA77, 0x0000, 0x0000, 0x0002, /*2FA77*/
  0xFBC5, 0x0020, 0x0002, 0xFA78, 0x0000, 0x0000, 0x0002, /*2FA78*/
  0xFBC5, 0x0020, 0x0002, 0xFA79, 0x0000, 0x0000, 0x0002, /*2FA79*/
  0xFBC5, 0x0020, 0x0002, 0xFA7A, 0x0000, 0x0000, 0x0002, /*2FA7A*/
  0xFBC5, 0x0020, 0x0002, 0xFA7B, 0x0000, 0x0000, 0x0002, /*2FA7B*/
  0xFBC5, 0x0020, 0x0002, 0xFA7C, 0x0000, 0x0000, 0x0002, /*2FA7C*/
  0xFBC5, 0x0020, 0x0002, 0xFA7D, 0x0000, 0x0000, 0x0002, /*2FA7D*/
  0xFBC5, 0x0020, 0x0002, 0xFA7E, 0x0000, 0x0000, 0x0002, /*2FA7E*/
  0xFBC5, 0x0020, 0x0002, 0xFA7F, 0x0000, 0x0000, 0x0002, /*2FA7F*/
  0xFBC5, 0x0020, 0x0002, 0xFA80, 0x0000, 0x0000, 0x0002, /*2FA80*/
  0xFBC5, 0x0020, 0x0002, 0xFA81, 0x0000, 0x0000, 0x0002, /*2FA81*/
  0xFBC5, 0x0020, 0x0002, 0xFA82, 0x0000, 0x0000, 0x0002, /*2FA82*/
  0xFBC5, 0x0020, 0x0002, 0xFA83, 0x0000, 0x0000, 0x0002, /*2FA83*/
  0xFBC5, 0x0020, 0x0002, 0xFA84, 0x0000, 0x0000, 0x0002, /*2FA84*/
  0xFBC5, 0x0020, 0x0002, 0xFA85, 0x0000, 0x0000, 0x0002, /*2FA85*/
  0xFBC5, 0x0020, 0x0002, 0xFA86, 0x0000, 0x0000, 0x0002, /*2FA86*/
  0xFBC5, 0x0020, 0x0002, 0xFA87, 0x0000, 0x0000, 0x0002, /*2FA87*/
  0xFBC5, 0x0020, 0x0002, 0xFA88, 0x0000, 0x0000, 0x0002, /*2FA88*/
  0xFBC5, 0x0020, 0x0002, 0xFA89, 0x0000, 0x0000, 0x0002, /*2FA89*/
  0xFBC5, 0x0020, 0x0002, 0xFA8A, 0x0000, 0x0000, 0x0002, /*2FA8A*/
  0xFBC5, 0x0020, 0x0002, 0xFA8B, 0x0000, 0x0000, 0x0002, /*2FA8B*/
  0xFBC5, 0x0020, 0x0002, 0xFA8C, 0x0000, 0x0000, 0x0002, /*2FA8C*/
  0xFBC5, 0x0020, 0x0002, 0xFA8D, 0x0000, 0x0000, 0x0002, /*2FA8D*/
  0xFBC5, 0x0020, 0x0002, 0xFA8E, 0x0000, 0x0000, 0x0002, /*2FA8E*/
  0xFBC5, 0x0020, 0x0002, 0xFA8F, 0x0000, 0x0000, 0x0002, /*2FA8F*/
  0xFBC5, 0x0020, 0x0002, 0xFA90, 0x0000, 0x0000, 0x0002, /*2FA90*/
  0xFBC5, 0x0020, 0x0002, 0xFA91, 0x0000, 0x0000, 0x0002, /*2FA91*/
  0xFBC5, 0x0020, 0x0002, 0xFA92, 0x0000, 0x0000, 0x0002, /*2FA92*/
  0xFBC5, 0x0020, 0x0002, 0xFA93, 0x0000, 0x0000, 0x0002, /*2FA93*/
  0xFBC5, 0x0020, 0x0002, 0xFA94, 0x0000, 0x0000, 0x0002, /*2FA94*/
  0xFBC5, 0x0020, 0x0002, 0xFA95, 0x0000, 0x0000, 0x0002, /*2FA95*/
  0xFBC5, 0x0020, 0x0002, 0xFA96, 0x0000, 0x0000, 0x0002, /*2FA96*/
  0xFBC5, 0x0020, 0x0002, 0xFA97, 0x0000, 0x0000, 0x0002, /*2FA97*/
  0xFBC5, 0x0020, 0x0002, 0xFA98, 0x0000, 0x0000, 0x0002, /*2FA98*/
  0xFBC5, 0x0020, 0x0002, 0xFA99, 0x0000, 0x0000, 0x0002, /*2FA99*/
  0xFBC5, 0x0020, 0x0002, 0xFA9A, 0x0000, 0x0000, 0x0002, /*2FA9A*/
  0xFBC5, 0x0020, 0x0002, 0xFA9B, 0x0000, 0x0000, 0x0002, /*2FA9B*/
  0xFBC5, 0x0020, 0x0002, 0xFA9C, 0x0000, 0x0000, 0x0002, /*2FA9C*/
  0xFBC5, 0x0020, 0x0002, 0xFA9D, 0x0000, 0x0000, 0x0002, /*2FA9D*/
  0xFBC5, 0x0020, 0x0002, 0xFA9E, 0x0000, 0x0000, 0x0002, /*2FA9E*/
  0xFBC5, 0x0020, 0x0002, 0xFA9F, 0x0000, 0x0000, 0x0002, /*2FA9F*/
  0xFBC5, 0x0020, 0x0002, 0xFAA0, 0x0000, 0x0000, 0x0002, /*2FAA0*/
  0xFBC5, 0x0020, 0x0002, 0xFAA1, 0x0000, 0x0000, 0x0002, /*2FAA1*/
  0xFBC5, 0x0020, 0x0002, 0xFAA2, 0x0000, 0x0000, 0x0002, /*2FAA2*/
  0xFBC5, 0x0020, 0x0002, 0xFAA3, 0x0000, 0x0000, 0x0002, /*2FAA3*/
  0xFBC5, 0x0020, 0x0002, 0xFAA4, 0x0000, 0x0000, 0x0002, /*2FAA4*/
  0xFBC5, 0x0020, 0x0002, 0xFAA5, 0x0000, 0x0000, 0x0002, /*2FAA5*/
  0xFBC5, 0x0020, 0x0002, 0xFAA6, 0x0000, 0x0000, 0x0002, /*2FAA6*/
  0xFBC5, 0x0020, 0x0002, 0xFAA7, 0x0000, 0x0000, 0x0002, /*2FAA7*/
  0xFBC5, 0x0020, 0x0002, 0xFAA8, 0x0000, 0x0000, 0x0002, /*2FAA8*/
  0xFBC5, 0x0020, 0x0002, 0xFAA9, 0x0000, 0x0000, 0x0002, /*2FAA9*/
  0xFBC5, 0x0020, 0x0002, 0xFAAA, 0x0000, 0x0000, 0x0002, /*2FAAA*/
  0xFBC5, 0x0020, 0x0002, 0xFAAB, 0x0000, 0x0000, 0x0002, /*2FAAB*/
  0xFBC5, 0x0020, 0x0002, 0xFAAC, 0x0000, 0x0000, 0x0002, /*2FAAC*/
  0xFBC5, 0x0020, 0x0002, 0xFAAD, 0x0000, 0x0000, 0x0002, /*2FAAD*/
  0xFBC5, 0x0020, 0x0002, 0xFAAE, 0x0000, 0x0000, 0x0002, /*2FAAE*/
  0xFBC5, 0x0020, 0x0002, 0xFAAF, 0x0000, 0x0000, 0x0002, /*2FAAF*/
  0xFBC5, 0x0020, 0x0002, 0xFAB0, 0x0000, 0x0000, 0x0002, /*2FAB0*/
  0xFBC5, 0x0020, 0x0002, 0xFAB1, 0x0000, 0x0000, 0x0002, /*2FAB1*/
  0xFBC5, 0x0020, 0x0002, 0xFAB2, 0x0000, 0x0000, 0x0002, /*2FAB2*/
  0xFBC5, 0x0020, 0x0002, 0xFAB3, 0x0000, 0x0000, 0x0002, /*2FAB3*/
  0xFBC5, 0x0020, 0x0002, 0xFAB4, 0x0000, 0x0000, 0x0002, /*2FAB4*/
  0xFBC5, 0x0020, 0x0002, 0xFAB5, 0x0000, 0x0000, 0x0002, /*2FAB5*/
  0xFBC5, 0x0020, 0x0002, 0xFAB6, 0x0000, 0x0000, 0x0002, /*2FAB6*/
  0xFBC5, 0x0020, 0x0002, 0xFAB7, 0x0000, 0x0000, 0x0002, /*2FAB7*/
  0xFBC5, 0x0020, 0x0002, 0xFAB8, 0x0000, 0x0000, 0x0002, /*2FAB8*/
  0xFBC5, 0x0020, 0x0002, 0xFAB9, 0x0000, 0x0000, 0x0002, /*2FAB9*/
  0xFBC5, 0x0020, 0x0002, 0xFABA, 0x0000, 0x0000, 0x0002, /*2FABA*/
  0xFBC5, 0x0020, 0x0002, 0xFABB, 0x0000, 0x0000, 0x0002, /*2FABB*/
  0xFBC5, 0x0020, 0x0002, 0xFABC, 0x0000, 0x0000, 0x0002, /*2FABC*/
  0xFBC5, 0x0020, 0x0002, 0xFABD, 0x0000, 0x0000, 0x0002, /*2FABD*/
  0xFBC5, 0x0020, 0x0002, 0xFABE, 0x0000, 0x0000, 0x0002, /*2FABE*/
  0xFBC5, 0x0020, 0x0002, 0xFABF, 0x0000, 0x0000, 0x0002, /*2FABF*/
  0xFBC5, 0x0020, 0x0002, 0xFAC0, 0x0000, 0x0000, 0x0002, /*2FAC0*/
  0xFBC5, 0x0020, 0x0002, 0xFAC1, 0x0000, 0x0000, 0x0002, /*2FAC1*/
  0xFBC5, 0x0020, 0x0002, 0xFAC2, 0x0000, 0x0000, 0x0002, /*2FAC2*/
  0xFBC5, 0x0020, 0x0002, 0xFAC3, 0x0000, 0x0000, 0x0002, /*2FAC3*/
  0xFBC5, 0x0020, 0x0002, 0xFAC4, 0x0000, 0x0000, 0x0002, /*2FAC4*/
  0xFBC5, 0x0020, 0x0002, 0xFAC5, 0x0000, 0x0000, 0x0002, /*2FAC5*/
  0xFBC5, 0x0020, 0x0002, 0xFAC6, 0x0000, 0x0000, 0x0002, /*2FAC6*/
  0xFBC5, 0x0020, 0x0002, 0xFAC7, 0x0000, 0x0000, 0x0002, /*2FAC7*/
  0xFBC5, 0x0020, 0x0002, 0xFAC8, 0x0000, 0x0000, 0x0002, /*2FAC8*/
  0xFBC5, 0x0020, 0x0002, 0xFAC9, 0x0000, 0x0000, 0x0002, /*2FAC9*/
  0xFBC5, 0x0020, 0x0002, 0xFACA, 0x0000, 0x0000, 0x0002, /*2FACA*/
  0xFBC5, 0x0020, 0x0002, 0xFACB, 0x0000, 0x0000, 0x0002, /*2FACB*/
  0xFBC5, 0x0020, 0x0002, 0xFACC, 0x0000, 0x0000, 0x0002, /*2FACC*/
  0xFBC5, 0x0020, 0x0002, 0xFACD, 0x0000, 0x0000, 0x0002, /*2FACD*/
  0xFBC5, 0x0020, 0x0002, 0xFACE, 0x0000, 0x0000, 0x0002, /*2FACE*/
  0xFBC5, 0x0020, 0x0002, 0xFACF, 0x0000, 0x0000, 0x0002, /*2FACF*/
  0xFBC5, 0x0020, 0x0002, 0xFAD0, 0x0000, 0x0000, 0x0002, /*2FAD0*/
  0xFBC5, 0x0020, 0x0002, 0xFAD1, 0x0000, 0x0000, 0x0002, /*2FAD1*/
  0xFBC5, 0x0020, 0x0002, 0xFAD2, 0x0000, 0x0000, 0x0002, /*2FAD2*/
  0xFBC5, 0x0020, 0x0002, 0xFAD3, 0x0000, 0x0000, 0x0002, /*2FAD3*/
  0xFBC5, 0x0020, 0x0002, 0xFAD4, 0x0000, 0x0000, 0x0002, /*2FAD4*/
  0xFBC5, 0x0020, 0x0002, 0xFAD5, 0x0000, 0x0000, 0x0002, /*2FAD5*/
  0xFBC5, 0x0020, 0x0002, 0xFAD6, 0x0000, 0x0000, 0x0002, /*2FAD6*/
  0xFBC5, 0x0020, 0x0002, 0xFAD7, 0x0000, 0x0000, 0x0002, /*2FAD7*/
  0xFBC5, 0x0020, 0x0002, 0xFAD8, 0x0000, 0x0000, 0x0002, /*2FAD8*/
  0xFBC5, 0x0020, 0x0002, 0xFAD9, 0x0000, 0x0000, 0x0002, /*2FAD9*/
  0xFBC5, 0x0020, 0x0002, 0xFADA, 0x0000, 0x0000, 0x0002, /*2FADA*/
  0xFBC5, 0x0020, 0x0002, 0xFADB, 0x0000, 0x0000, 0x0002, /*2FADB*/
  0xFBC5, 0x0020, 0x0002, 0xFADC, 0x0000, 0x0000, 0x0002, /*2FADC*/
  0xFBC5, 0x0020, 0x0002, 0xFADD, 0x0000, 0x0000, 0x0002, /*2FADD*/
  0xFBC5, 0x0020, 0x0002, 0xFADE, 0x0000, 0x0000, 0x0002, /*2FADE*/
  0xFBC5, 0x0020, 0x0002, 0xFADF, 0x0000, 0x0000, 0x0002, /*2FADF*/
  0xFBC5, 0x0020, 0x0002, 0xFAE0, 0x0000, 0x0000, 0x0002, /*2FAE0*/
  0xFBC5, 0x0020, 0x0002, 0xFAE1, 0x0000, 0x0000, 0x0002, /*2FAE1*/
  0xFBC5, 0x0020, 0x0002, 0xFAE2, 0x0000, 0x0000, 0x0002, /*2FAE2*/
  0xFBC5, 0x0020, 0x0002, 0xFAE3, 0x0000, 0x0000, 0x0002, /*2FAE3*/
  0xFBC5, 0x0020, 0x0002, 0xFAE4, 0x0000, 0x0000, 0x0002, /*2FAE4*/
  0xFBC5, 0x0020, 0x0002, 0xFAE5, 0x0000, 0x0000, 0x0002, /*2FAE5*/
  0xFBC5, 0x0020, 0x0002, 0xFAE6, 0x0000, 0x0000, 0x0002, /*2FAE6*/
  0xFBC5, 0x0020, 0x0002, 0xFAE7, 0x0000, 0x0000, 0x0002, /*2FAE7*/
  0xFBC5, 0x0020, 0x0002, 0xFAE8, 0x0000, 0x0000, 0x0002, /*2FAE8*/
  0xFBC5, 0x0020, 0x0002, 0xFAE9, 0x0000, 0x0000, 0x0002, /*2FAE9*/
  0xFBC5, 0x0020, 0x0002, 0xFAEA, 0x0000, 0x0000, 0x0002, /*2FAEA*/
  0xFBC5, 0x0020, 0x0002, 0xFAEB, 0x0000, 0x0000, 0x0002, /*2FAEB*/
  0xFBC5, 0x0020, 0x0002, 0xFAEC, 0x0000, 0x0000, 0x0002, /*2FAEC*/
  0xFBC5, 0x0020, 0x0002, 0xFAED, 0x0000, 0x0000, 0x0002, /*2FAED*/
  0xFBC5, 0x0020, 0x0002, 0xFAEE, 0x0000, 0x0000, 0x0002, /*2FAEE*/
  0xFBC5, 0x0020, 0x0002, 0xFAEF, 0x0000, 0x0000, 0x0002, /*2FAEF*/
  0xFBC5, 0x0020, 0x0002, 0xFAF0, 0x0000, 0x0000, 0x0002, /*2FAF0*/
  0xFBC5, 0x0020, 0x0002, 0xFAF1, 0x0000, 0x0000, 0x0002, /*2FAF1*/
  0xFBC5, 0x0020, 0x0002, 0xFAF2, 0x0000, 0x0000, 0x0002, /*2FAF2*/
  0xFBC5, 0x0020, 0x0002, 0xFAF3, 0x0000, 0x0000, 0x0002, /*2FAF3*/
  0xFBC5, 0x0020, 0x0002, 0xFAF4, 0x0000, 0x0000, 0x0002, /*2FAF4*/
  0xFBC5, 0x0020, 0x0002, 0xFAF5, 0x0000, 0x0000, 0x0002, /*2FAF5*/
  0xFBC5, 0x0020, 0x0002, 0xFAF6, 0x0000, 0x0000, 0x0002, /*2FAF6*/
  0xFBC5, 0x0020, 0x0002, 0xFAF7, 0x0000, 0x0000, 0x0002, /*2FAF7*/
  0xFBC5, 0x0020, 0x0002, 0xFAF8, 0x0000, 0x0000, 0x0002, /*2FAF8*/
  0xFBC5, 0x0020, 0x0002, 0xFAF9, 0x0000, 0x0000, 0x0002, /*2FAF9*/
  0xFBC5, 0x0020, 0x0002, 0xFAFA, 0x0000, 0x0000, 0x0002, /*2FAFA*/
  0xFBC5, 0x0020, 0x0002, 0xFAFB, 0x0000, 0x0000, 0x0002, /*2FAFB*/
  0xFBC5, 0x0020, 0x0002, 0xFAFC, 0x0000, 0x0000, 0x0002, /*2FAFC*/
  0xFBC5, 0x0020, 0x0002, 0xFAFD, 0x0000, 0x0000, 0x0002, /*2FAFD*/
  0xFBC5, 0x0020, 0x0002, 0xFAFE, 0x0000, 0x0000, 0x0002, /*2FAFE*/
  0xFBC5, 0x0020, 0x0002, 0xFAFF, 0x0000, 0x0000, 0x0002  /*2FAFF*/
};

uint16 uca900_pE00[]= {
  0xFBDC, 0x0020, 0x0002, 0x8000, 0x0000, 0x0000, 0x0002, /*E0000*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0001*/
  0xFBDC, 0x0020, 0x0002, 0x8002, 0x0000, 0x0000, 0x0002, /*E0002*/
  0xFBDC, 0x0020, 0x0002, 0x8003, 0x0000, 0x0000, 0x0002, /*E0003*/
  0xFBDC, 0x0020, 0x0002, 0x8004, 0x0000, 0x0000, 0x0002, /*E0004*/
  0xFBDC, 0x0020, 0x0002, 0x8005, 0x0000, 0x0000, 0x0002, /*E0005*/
  0xFBDC, 0x0020, 0x0002, 0x8006, 0x0000, 0x0000, 0x0002, /*E0006*/
  0xFBDC, 0x0020, 0x0002, 0x8007, 0x0000, 0x0000, 0x0002, /*E0007*/
  0xFBDC, 0x0020, 0x0002, 0x8008, 0x0000, 0x0000, 0x0002, /*E0008*/
  0xFBDC, 0x0020, 0x0002, 0x8009, 0x0000, 0x0000, 0x0002, /*E0009*/
  0xFBDC, 0x0020, 0x0002, 0x800A, 0x0000, 0x0000, 0x0002, /*E000A*/
  0xFBDC, 0x0020, 0x0002, 0x800B, 0x0000, 0x0000, 0x0002, /*E000B*/
  0xFBDC, 0x0020, 0x0002, 0x800C, 0x0000, 0x0000, 0x0002, /*E000C*/
  0xFBDC, 0x0020, 0x0002, 0x800D, 0x0000, 0x0000, 0x0002, /*E000D*/
  0xFBDC, 0x0020, 0x0002, 0x800E, 0x0000, 0x0000, 0x0002, /*E000E*/
  0xFBDC, 0x0020, 0x0002, 0x800F, 0x0000, 0x0000, 0x0002, /*E000F*/
  0xFBDC, 0x0020, 0x0002, 0x8010, 0x0000, 0x0000, 0x0002, /*E0010*/
  0xFBDC, 0x0020, 0x0002, 0x8011, 0x0000, 0x0000, 0x0002, /*E0011*/
  0xFBDC, 0x0020, 0x0002, 0x8012, 0x0000, 0x0000, 0x0002, /*E0012*/
  0xFBDC, 0x0020, 0x0002, 0x8013, 0x0000, 0x0000, 0x0002, /*E0013*/
  0xFBDC, 0x0020, 0x0002, 0x8014, 0x0000, 0x0000, 0x0002, /*E0014*/
  0xFBDC, 0x0020, 0x0002, 0x8015, 0x0000, 0x0000, 0x0002, /*E0015*/
  0xFBDC, 0x0020, 0x0002, 0x8016, 0x0000, 0x0000, 0x0002, /*E0016*/
  0xFBDC, 0x0020, 0x0002, 0x8017, 0x0000, 0x0000, 0x0002, /*E0017*/
  0xFBDC, 0x0020, 0x0002, 0x8018, 0x0000, 0x0000, 0x0002, /*E0018*/
  0xFBDC, 0x0020, 0x0002, 0x8019, 0x0000, 0x0000, 0x0002, /*E0019*/
  0xFBDC, 0x0020, 0x0002, 0x801A, 0x0000, 0x0000, 0x0002, /*E001A*/
  0xFBDC, 0x0020, 0x0002, 0x801B, 0x0000, 0x0000, 0x0002, /*E001B*/
  0xFBDC, 0x0020, 0x0002, 0x801C, 0x0000, 0x0000, 0x0002, /*E001C*/
  0xFBDC, 0x0020, 0x0002, 0x801D, 0x0000, 0x0000, 0x0002, /*E001D*/
  0xFBDC, 0x0020, 0x0002, 0x801E, 0x0000, 0x0000, 0x0002, /*E001E*/
  0xFBDC, 0x0020, 0x0002, 0x801F, 0x0000, 0x0000, 0x0002, /*E001F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0020*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0021*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0022*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0023*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0024*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0025*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0026*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0027*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0028*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0029*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E002A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E002B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E002C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E002D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E002E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E002F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0030*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0031*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0032*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0033*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0034*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0035*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0036*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0037*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0038*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0039*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E003A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E003B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E003C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E003D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E003E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E003F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0040*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0041*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0042*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0043*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0044*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0045*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0046*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0047*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0048*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0049*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E004A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E004B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E004C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E004D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E004E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E004F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0050*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0051*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0052*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0053*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0054*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0055*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0056*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0057*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0058*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0059*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E005A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E005B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E005C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E005D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E005E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E005F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0060*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0061*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0062*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0063*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0064*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0065*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0066*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0067*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0068*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0069*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E006A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E006B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E006C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E006D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E006E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E006F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0070*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0071*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0072*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0073*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0074*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0075*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0076*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0077*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0078*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0079*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E007A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E007B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E007C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E007D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E007E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E007F*/
  0xFBDC, 0x0020, 0x0002, 0x8080, 0x0000, 0x0000, 0x0002, /*E0080*/
  0xFBDC, 0x0020, 0x0002, 0x8081, 0x0000, 0x0000, 0x0002, /*E0081*/
  0xFBDC, 0x0020, 0x0002, 0x8082, 0x0000, 0x0000, 0x0002, /*E0082*/
  0xFBDC, 0x0020, 0x0002, 0x8083, 0x0000, 0x0000, 0x0002, /*E0083*/
  0xFBDC, 0x0020, 0x0002, 0x8084, 0x0000, 0x0000, 0x0002, /*E0084*/
  0xFBDC, 0x0020, 0x0002, 0x8085, 0x0000, 0x0000, 0x0002, /*E0085*/
  0xFBDC, 0x0020, 0x0002, 0x8086, 0x0000, 0x0000, 0x0002, /*E0086*/
  0xFBDC, 0x0020, 0x0002, 0x8087, 0x0000, 0x0000, 0x0002, /*E0087*/
  0xFBDC, 0x0020, 0x0002, 0x8088, 0x0000, 0x0000, 0x0002, /*E0088*/
  0xFBDC, 0x0020, 0x0002, 0x8089, 0x0000, 0x0000, 0x0002, /*E0089*/
  0xFBDC, 0x0020, 0x0002, 0x808A, 0x0000, 0x0000, 0x0002, /*E008A*/
  0xFBDC, 0x0020, 0x0002, 0x808B, 0x0000, 0x0000, 0x0002, /*E008B*/
  0xFBDC, 0x0020, 0x0002, 0x808C, 0x0000, 0x0000, 0x0002, /*E008C*/
  0xFBDC, 0x0020, 0x0002, 0x808D, 0x0000, 0x0000, 0x0002, /*E008D*/
  0xFBDC, 0x0020, 0x0002, 0x808E, 0x0000, 0x0000, 0x0002, /*E008E*/
  0xFBDC, 0x0020, 0x0002, 0x808F, 0x0000, 0x0000, 0x0002, /*E008F*/
  0xFBDC, 0x0020, 0x0002, 0x8090, 0x0000, 0x0000, 0x0002, /*E0090*/
  0xFBDC, 0x0020, 0x0002, 0x8091, 0x0000, 0x0000, 0x0002, /*E0091*/
  0xFBDC, 0x0020, 0x0002, 0x8092, 0x0000, 0x0000, 0x0002, /*E0092*/
  0xFBDC, 0x0020, 0x0002, 0x8093, 0x0000, 0x0000, 0x0002, /*E0093*/
  0xFBDC, 0x0020, 0x0002, 0x8094, 0x0000, 0x0000, 0x0002, /*E0094*/
  0xFBDC, 0x0020, 0x0002, 0x8095, 0x0000, 0x0000, 0x0002, /*E0095*/
  0xFBDC, 0x0020, 0x0002, 0x8096, 0x0000, 0x0000, 0x0002, /*E0096*/
  0xFBDC, 0x0020, 0x0002, 0x8097, 0x0000, 0x0000, 0x0002, /*E0097*/
  0xFBDC, 0x0020, 0x0002, 0x8098, 0x0000, 0x0000, 0x0002, /*E0098*/
  0xFBDC, 0x0020, 0x0002, 0x8099, 0x0000, 0x0000, 0x0002, /*E0099*/
  0xFBDC, 0x0020, 0x0002, 0x809A, 0x0000, 0x0000, 0x0002, /*E009A*/
  0xFBDC, 0x0020, 0x0002, 0x809B, 0x0000, 0x0000, 0x0002, /*E009B*/
  0xFBDC, 0x0020, 0x0002, 0x809C, 0x0000, 0x0000, 0x0002, /*E009C*/
  0xFBDC, 0x0020, 0x0002, 0x809D, 0x0000, 0x0000, 0x0002, /*E009D*/
  0xFBDC, 0x0020, 0x0002, 0x809E, 0x0000, 0x0000, 0x0002, /*E009E*/
  0xFBDC, 0x0020, 0x0002, 0x809F, 0x0000, 0x0000, 0x0002, /*E009F*/
  0xFBDC, 0x0020, 0x0002, 0x80A0, 0x0000, 0x0000, 0x0002, /*E00A0*/
  0xFBDC, 0x0020, 0x0002, 0x80A1, 0x0000, 0x0000, 0x0002, /*E00A1*/
  0xFBDC, 0x0020, 0x0002, 0x80A2, 0x0000, 0x0000, 0x0002, /*E00A2*/
  0xFBDC, 0x0020, 0x0002, 0x80A3, 0x0000, 0x0000, 0x0002, /*E00A3*/
  0xFBDC, 0x0020, 0x0002, 0x80A4, 0x0000, 0x0000, 0x0002, /*E00A4*/
  0xFBDC, 0x0020, 0x0002, 0x80A5, 0x0000, 0x0000, 0x0002, /*E00A5*/
  0xFBDC, 0x0020, 0x0002, 0x80A6, 0x0000, 0x0000, 0x0002, /*E00A6*/
  0xFBDC, 0x0020, 0x0002, 0x80A7, 0x0000, 0x0000, 0x0002, /*E00A7*/
  0xFBDC, 0x0020, 0x0002, 0x80A8, 0x0000, 0x0000, 0x0002, /*E00A8*/
  0xFBDC, 0x0020, 0x0002, 0x80A9, 0x0000, 0x0000, 0x0002, /*E00A9*/
  0xFBDC, 0x0020, 0x0002, 0x80AA, 0x0000, 0x0000, 0x0002, /*E00AA*/
  0xFBDC, 0x0020, 0x0002, 0x80AB, 0x0000, 0x0000, 0x0002, /*E00AB*/
  0xFBDC, 0x0020, 0x0002, 0x80AC, 0x0000, 0x0000, 0x0002, /*E00AC*/
  0xFBDC, 0x0020, 0x0002, 0x80AD, 0x0000, 0x0000, 0x0002, /*E00AD*/
  0xFBDC, 0x0020, 0x0002, 0x80AE, 0x0000, 0x0000, 0x0002, /*E00AE*/
  0xFBDC, 0x0020, 0x0002, 0x80AF, 0x0000, 0x0000, 0x0002, /*E00AF*/
  0xFBDC, 0x0020, 0x0002, 0x80B0, 0x0000, 0x0000, 0x0002, /*E00B0*/
  0xFBDC, 0x0020, 0x0002, 0x80B1, 0x0000, 0x0000, 0x0002, /*E00B1*/
  0xFBDC, 0x0020, 0x0002, 0x80B2, 0x0000, 0x0000, 0x0002, /*E00B2*/
  0xFBDC, 0x0020, 0x0002, 0x80B3, 0x0000, 0x0000, 0x0002, /*E00B3*/
  0xFBDC, 0x0020, 0x0002, 0x80B4, 0x0000, 0x0000, 0x0002, /*E00B4*/
  0xFBDC, 0x0020, 0x0002, 0x80B5, 0x0000, 0x0000, 0x0002, /*E00B5*/
  0xFBDC, 0x0020, 0x0002, 0x80B6, 0x0000, 0x0000, 0x0002, /*E00B6*/
  0xFBDC, 0x0020, 0x0002, 0x80B7, 0x0000, 0x0000, 0x0002, /*E00B7*/
  0xFBDC, 0x0020, 0x0002, 0x80B8, 0x0000, 0x0000, 0x0002, /*E00B8*/
  0xFBDC, 0x0020, 0x0002, 0x80B9, 0x0000, 0x0000, 0x0002, /*E00B9*/
  0xFBDC, 0x0020, 0x0002, 0x80BA, 0x0000, 0x0000, 0x0002, /*E00BA*/
  0xFBDC, 0x0020, 0x0002, 0x80BB, 0x0000, 0x0000, 0x0002, /*E00BB*/
  0xFBDC, 0x0020, 0x0002, 0x80BC, 0x0000, 0x0000, 0x0002, /*E00BC*/
  0xFBDC, 0x0020, 0x0002, 0x80BD, 0x0000, 0x0000, 0x0002, /*E00BD*/
  0xFBDC, 0x0020, 0x0002, 0x80BE, 0x0000, 0x0000, 0x0002, /*E00BE*/
  0xFBDC, 0x0020, 0x0002, 0x80BF, 0x0000, 0x0000, 0x0002, /*E00BF*/
  0xFBDC, 0x0020, 0x0002, 0x80C0, 0x0000, 0x0000, 0x0002, /*E00C0*/
  0xFBDC, 0x0020, 0x0002, 0x80C1, 0x0000, 0x0000, 0x0002, /*E00C1*/
  0xFBDC, 0x0020, 0x0002, 0x80C2, 0x0000, 0x0000, 0x0002, /*E00C2*/
  0xFBDC, 0x0020, 0x0002, 0x80C3, 0x0000, 0x0000, 0x0002, /*E00C3*/
  0xFBDC, 0x0020, 0x0002, 0x80C4, 0x0000, 0x0000, 0x0002, /*E00C4*/
  0xFBDC, 0x0020, 0x0002, 0x80C5, 0x0000, 0x0000, 0x0002, /*E00C5*/
  0xFBDC, 0x0020, 0x0002, 0x80C6, 0x0000, 0x0000, 0x0002, /*E00C6*/
  0xFBDC, 0x0020, 0x0002, 0x80C7, 0x0000, 0x0000, 0x0002, /*E00C7*/
  0xFBDC, 0x0020, 0x0002, 0x80C8, 0x0000, 0x0000, 0x0002, /*E00C8*/
  0xFBDC, 0x0020, 0x0002, 0x80C9, 0x0000, 0x0000, 0x0002, /*E00C9*/
  0xFBDC, 0x0020, 0x0002, 0x80CA, 0x0000, 0x0000, 0x0002, /*E00CA*/
  0xFBDC, 0x0020, 0x0002, 0x80CB, 0x0000, 0x0000, 0x0002, /*E00CB*/
  0xFBDC, 0x0020, 0x0002, 0x80CC, 0x0000, 0x0000, 0x0002, /*E00CC*/
  0xFBDC, 0x0020, 0x0002, 0x80CD, 0x0000, 0x0000, 0x0002, /*E00CD*/
  0xFBDC, 0x0020, 0x0002, 0x80CE, 0x0000, 0x0000, 0x0002, /*E00CE*/
  0xFBDC, 0x0020, 0x0002, 0x80CF, 0x0000, 0x0000, 0x0002, /*E00CF*/
  0xFBDC, 0x0020, 0x0002, 0x80D0, 0x0000, 0x0000, 0x0002, /*E00D0*/
  0xFBDC, 0x0020, 0x0002, 0x80D1, 0x0000, 0x0000, 0x0002, /*E00D1*/
  0xFBDC, 0x0020, 0x0002, 0x80D2, 0x0000, 0x0000, 0x0002, /*E00D2*/
  0xFBDC, 0x0020, 0x0002, 0x80D3, 0x0000, 0x0000, 0x0002, /*E00D3*/
  0xFBDC, 0x0020, 0x0002, 0x80D4, 0x0000, 0x0000, 0x0002, /*E00D4*/
  0xFBDC, 0x0020, 0x0002, 0x80D5, 0x0000, 0x0000, 0x0002, /*E00D5*/
  0xFBDC, 0x0020, 0x0002, 0x80D6, 0x0000, 0x0000, 0x0002, /*E00D6*/
  0xFBDC, 0x0020, 0x0002, 0x80D7, 0x0000, 0x0000, 0x0002, /*E00D7*/
  0xFBDC, 0x0020, 0x0002, 0x80D8, 0x0000, 0x0000, 0x0002, /*E00D8*/
  0xFBDC, 0x0020, 0x0002, 0x80D9, 0x0000, 0x0000, 0x0002, /*E00D9*/
  0xFBDC, 0x0020, 0x0002, 0x80DA, 0x0000, 0x0000, 0x0002, /*E00DA*/
  0xFBDC, 0x0020, 0x0002, 0x80DB, 0x0000, 0x0000, 0x0002, /*E00DB*/
  0xFBDC, 0x0020, 0x0002, 0x80DC, 0x0000, 0x0000, 0x0002, /*E00DC*/
  0xFBDC, 0x0020, 0x0002, 0x80DD, 0x0000, 0x0000, 0x0002, /*E00DD*/
  0xFBDC, 0x0020, 0x0002, 0x80DE, 0x0000, 0x0000, 0x0002, /*E00DE*/
  0xFBDC, 0x0020, 0x0002, 0x80DF, 0x0000, 0x0000, 0x0002, /*E00DF*/
  0xFBDC, 0x0020, 0x0002, 0x80E0, 0x0000, 0x0000, 0x0002, /*E00E0*/
  0xFBDC, 0x0020, 0x0002, 0x80E1, 0x0000, 0x0000, 0x0002, /*E00E1*/
  0xFBDC, 0x0020, 0x0002, 0x80E2, 0x0000, 0x0000, 0x0002, /*E00E2*/
  0xFBDC, 0x0020, 0x0002, 0x80E3, 0x0000, 0x0000, 0x0002, /*E00E3*/
  0xFBDC, 0x0020, 0x0002, 0x80E4, 0x0000, 0x0000, 0x0002, /*E00E4*/
  0xFBDC, 0x0020, 0x0002, 0x80E5, 0x0000, 0x0000, 0x0002, /*E00E5*/
  0xFBDC, 0x0020, 0x0002, 0x80E6, 0x0000, 0x0000, 0x0002, /*E00E6*/
  0xFBDC, 0x0020, 0x0002, 0x80E7, 0x0000, 0x0000, 0x0002, /*E00E7*/
  0xFBDC, 0x0020, 0x0002, 0x80E8, 0x0000, 0x0000, 0x0002, /*E00E8*/
  0xFBDC, 0x0020, 0x0002, 0x80E9, 0x0000, 0x0000, 0x0002, /*E00E9*/
  0xFBDC, 0x0020, 0x0002, 0x80EA, 0x0000, 0x0000, 0x0002, /*E00EA*/
  0xFBDC, 0x0020, 0x0002, 0x80EB, 0x0000, 0x0000, 0x0002, /*E00EB*/
  0xFBDC, 0x0020, 0x0002, 0x80EC, 0x0000, 0x0000, 0x0002, /*E00EC*/
  0xFBDC, 0x0020, 0x0002, 0x80ED, 0x0000, 0x0000, 0x0002, /*E00ED*/
  0xFBDC, 0x0020, 0x0002, 0x80EE, 0x0000, 0x0000, 0x0002, /*E00EE*/
  0xFBDC, 0x0020, 0x0002, 0x80EF, 0x0000, 0x0000, 0x0002, /*E00EF*/
  0xFBDC, 0x0020, 0x0002, 0x80F0, 0x0000, 0x0000, 0x0002, /*E00F0*/
  0xFBDC, 0x0020, 0x0002, 0x80F1, 0x0000, 0x0000, 0x0002, /*E00F1*/
  0xFBDC, 0x0020, 0x0002, 0x80F2, 0x0000, 0x0000, 0x0002, /*E00F2*/
  0xFBDC, 0x0020, 0x0002, 0x80F3, 0x0000, 0x0000, 0x0002, /*E00F3*/
  0xFBDC, 0x0020, 0x0002, 0x80F4, 0x0000, 0x0000, 0x0002, /*E00F4*/
  0xFBDC, 0x0020, 0x0002, 0x80F5, 0x0000, 0x0000, 0x0002, /*E00F5*/
  0xFBDC, 0x0020, 0x0002, 0x80F6, 0x0000, 0x0000, 0x0002, /*E00F6*/
  0xFBDC, 0x0020, 0x0002, 0x80F7, 0x0000, 0x0000, 0x0002, /*E00F7*/
  0xFBDC, 0x0020, 0x0002, 0x80F8, 0x0000, 0x0000, 0x0002, /*E00F8*/
  0xFBDC, 0x0020, 0x0002, 0x80F9, 0x0000, 0x0000, 0x0002, /*E00F9*/
  0xFBDC, 0x0020, 0x0002, 0x80FA, 0x0000, 0x0000, 0x0002, /*E00FA*/
  0xFBDC, 0x0020, 0x0002, 0x80FB, 0x0000, 0x0000, 0x0002, /*E00FB*/
  0xFBDC, 0x0020, 0x0002, 0x80FC, 0x0000, 0x0000, 0x0002, /*E00FC*/
  0xFBDC, 0x0020, 0x0002, 0x80FD, 0x0000, 0x0000, 0x0002, /*E00FD*/
  0xFBDC, 0x0020, 0x0002, 0x80FE, 0x0000, 0x0000, 0x0002, /*E00FE*/
  0xFBDC, 0x0020, 0x0002, 0x80FF, 0x0000, 0x0000, 0x0002  /*E00FF*/
};

uint16 uca900_pE01[]= {
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0100*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0101*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0102*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0103*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0104*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0105*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0106*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0107*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0108*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0109*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E010A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E010B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E010C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E010D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E010E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E010F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0110*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0111*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0112*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0113*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0114*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0115*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0116*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0117*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0118*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0119*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E011A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E011B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E011C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E011D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E011E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E011F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0120*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0121*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0122*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0123*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0124*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0125*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0126*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0127*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0128*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0129*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E012A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E012B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E012C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E012D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E012E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E012F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0130*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0131*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0132*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0133*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0134*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0135*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0136*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0137*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0138*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0139*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E013A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E013B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E013C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E013D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E013E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E013F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0140*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0141*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0142*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0143*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0144*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0145*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0146*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0147*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0148*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0149*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E014A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E014B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E014C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E014D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E014E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E014F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0150*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0151*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0152*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0153*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0154*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0155*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0156*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0157*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0158*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0159*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E015A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E015B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E015C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E015D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E015E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E015F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0160*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0161*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0162*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0163*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0164*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0165*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0166*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0167*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0168*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0169*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E016A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E016B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E016C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E016D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E016E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E016F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0170*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0171*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0172*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0173*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0174*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0175*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0176*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0177*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0178*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0179*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E017A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E017B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E017C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E017D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E017E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E017F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0180*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0181*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0182*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0183*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0184*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0185*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0186*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0187*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0188*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0189*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E018A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E018B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E018C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E018D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E018E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E018F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0190*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0191*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0192*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0193*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0194*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0195*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0196*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0197*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0198*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E0199*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E019A*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E019B*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E019C*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E019D*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E019E*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E019F*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01A0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01A1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01A2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01A3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01A4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01A5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01A6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01A7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01A8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01A9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01AA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01AB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01AC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01AD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01AE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01AF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01B0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01B1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01B2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01B3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01B4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01B5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01B6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01B7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01B8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01B9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01BA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01BB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01BC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01BD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01BE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01BF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01C0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01C1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01C2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01C3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01C4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01C5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01C6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01C7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01C8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01C9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01CA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01CB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01CC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01CD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01CE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01CF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01D0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01D1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01D2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01D3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01D4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01D5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01D6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01D7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01D8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01D9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01DA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01DB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01DC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01DD*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01DE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01DF*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01E0*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01E1*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01E2*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01E3*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01E4*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01E5*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01E6*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01E7*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01E8*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01E9*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01EA*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01EB*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01EC*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01ED*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01EE*/
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, /*E01EF*/
  0xFBDC, 0x0020, 0x0002, 0x81F0, 0x0000, 0x0000, 0x0002, /*E01F0*/
  0xFBDC, 0x0020, 0x0002, 0x81F1, 0x0000, 0x0000, 0x0002, /*E01F1*/
  0xFBDC, 0x0020, 0x0002, 0x81F2, 0x0000, 0x0000, 0x0002, /*E01F2*/
  0xFBDC, 0x0020, 0x0002, 0x81F3, 0x0000, 0x0000, 0x0002, /*E01F3*/
  0xFBDC, 0x0020, 0x0002, 0x81F4, 0x0000, 0x0000, 0x0002, /*E01F4*/
  0xFBDC, 0x0020, 0x0002, 0x81F5, 0x0000, 0x0000, 0x0002, /*E01F5*/
  0xFBDC, 0x0020, 0x0002, 0x81F6, 0x0000, 0x0000, 0x0002, /*E01F6*/
  0xFBDC, 0x0020, 0x0002, 0x81F7, 0x0000, 0x0000, 0x0002, /*E01F7*/
  0xFBDC, 0x0020, 0x0002, 0x81F8, 0x0000, 0x0000, 0x0002, /*E01F8*/
  0xFBDC, 0x0020, 0x0002, 0x81F9, 0x0000, 0x0000, 0x0002, /*E01F9*/
  0xFBDC, 0x0020, 0x0002, 0x81FA, 0x0000, 0x0000, 0x0002, /*E01FA*/
  0xFBDC, 0x0020, 0x0002, 0x81FB, 0x0000, 0x0000, 0x0002, /*E01FB*/
  0xFBDC, 0x0020, 0x0002, 0x81FC, 0x0000, 0x0000, 0x0002, /*E01FC*/
  0xFBDC, 0x0020, 0x0002, 0x81FD, 0x0000, 0x0000, 0x0002, /*E01FD*/
  0xFBDC, 0x0020, 0x0002, 0x81FE, 0x0000, 0x0000, 0x0002, /*E01FE*/
  0xFBDC, 0x0020, 0x0002, 0x81FF, 0x0000, 0x0000, 0x0002  /*E01FF*/
};

uint16* uca900_weight[4352]= {
  uca900_p000, uca900_p001, uca900_p002, uca900_p003, uca900_p004, uca900_p005,
  uca900_p006, uca900_p007, uca900_p008, uca900_p009, uca900_p00A, uca900_p00B,
  uca900_p00C, uca900_p00D, uca900_p00E, uca900_p00F, uca900_p010, uca900_p011,
  uca900_p012, uca900_p013, uca900_p014, uca900_p015, uca900_p016, uca900_p017,
  uca900_p018, uca900_p019, uca900_p01A, uca900_p01B, uca900_p01C, uca900_p01D,
  uca900_p01E, uca900_p01F, uca900_p020, uca900_p021, uca900_p022, uca900_p023,
  uca900_p024, uca900_p025, uca900_p026, uca900_p027, uca900_p028, uca900_p029,
  uca900_p02A, uca900_p02B, uca900_p02C, uca900_p02D, uca900_p02E, uca900_p02F,
  uca900_p030, uca900_p031, uca900_p032, uca900_p033,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL, uca900_p04D,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL, uca900_p0A0, uca900_p0A1,
  uca900_p0A2, uca900_p0A3, uca900_p0A4, uca900_p0A5, uca900_p0A6, uca900_p0A7,
  uca900_p0A8, uca900_p0A9, uca900_p0AA, uca900_p0AB,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL, uca900_p0D7,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL, uca900_p0F9, uca900_p0FA, uca900_p0FB,
  uca900_p0FC, uca900_p0FD, uca900_p0FE, uca900_p0FF, uca900_p100, uca900_p101,
  uca900_p102, uca900_p103, uca900_p104, uca900_p105, uca900_p106, uca900_p107,
  uca900_p108, uca900_p109, uca900_p10A, uca900_p10B, uca900_p10C,        NULL,
  uca900_p10E,        NULL, uca900_p110, uca900_p111, uca900_p112, uca900_p113,
  uca900_p114, uca900_p115, uca900_p116, uca900_p117, uca900_p118,        NULL,
  uca900_p11A,        NULL, uca900_p11C,        NULL,        NULL,        NULL,
  uca900_p120, uca900_p121, uca900_p122, uca900_p123, uca900_p124, uca900_p125,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL, uca900_p130, uca900_p131,
  uca900_p132, uca900_p133, uca900_p134,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
  uca900_p144, uca900_p145, uca900_p146,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
  uca900_p168, uca900_p169, uca900_p16A, uca900_p16B,        NULL,        NULL,
         NULL, uca900_p16F,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
  uca900_p1B0,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
  uca900_p1BC,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL, uca900_p1D0, uca900_p1D1, uca900_p1D2, uca900_p1D3,
  uca900_p1D4, uca900_p1D5, uca900_p1D6, uca900_p1D7, uca900_p1D8, uca900_p1D9,
  uca900_p1DA,        NULL,        NULL,        NULL,        NULL,        NULL,
  uca900_p1E0,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL, uca900_p1E8, uca900_p1E9,        NULL,        NULL,
         NULL,        NULL, uca900_p1EE,        NULL, uca900_p1F0, uca900_p1F1,
  uca900_p1F2, uca900_p1F3, uca900_p1F4, uca900_p1F5, uca900_p1F6, uca900_p1F7,
  uca900_p1F8, uca900_p1F9,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL, uca900_p2F8, uca900_p2F9,
  uca900_p2FA,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL, uca900_pE00, uca900_pE01,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL,        NULL,        NULL,        NULL,        NULL,
         NULL,        NULL
};

uchar uca900_length[4352]= {
  10, 13, 10, 10,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7, 10,
  10,  4,  7,  7,  4,  4, 13,  7,  7, 10, 10,  7,  7, 10, 10, 13,
  13, 13, 10,  7, 13,  4,  4,  7,  4,  4, 13,  7, 10,  7, 10,  7,
   7,  7, 22, 25,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  7,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   4,  4,  4,  4,  7,  4,  7, 10,  7,  7,  7,  7,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0, 10,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  7,  7, 10,  7, 25, 10,  7,
   7,  7,  7,  7,  7,  7,  4,  7,  7,  7,  7,  7,  7,  0,  7,  0,
   7,  7,  7,  7,  7,  7,  7,  7,  7,  0,  7,  0,  7,  0,  0,  0,
   4,  4,  4,  7,  7,  7,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   4,  4,  4,  4,  7,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  4,  4,  7,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  4,  4,  7,  7,  0,  0,  0,  7,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   7,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  7,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   7, 10,  7,  7,  7,  7,  7,  7,  4,  4,  7,  0,  0,  0,  0,  0,
   7,  0,  0,  0,  0,  0,  0,  0,  7,  7,  0,  0,  0,  0,  7,  0,
   7, 25, 13,  4,  4,  4,  7,  7,  7,  7,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  7,  7,  7,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   7,  7,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
};

static const MY_UNICASE_CHARACTER u900p000[]= {
  {0x0000,0x0000,0x0000},  {0x0001,0x0001,0x0001}, /* 0000 */
  {0x0002,0x0002,0x0002},  {0x0003,0x0003,0x0003}, /* 0002 */
  {0x0004,0x0004,0x0004},  {0x0005,0x0005,0x0005}, /* 0004 */
  {0x0006,0x0006,0x0006},  {0x0007,0x0007,0x0007}, /* 0006 */
  {0x0008,0x0008,0x0008},  {0x0009,0x0009,0x0009}, /* 0008 */
  {0x000A,0x000A,0x000A},  {0x000B,0x000B,0x000B}, /* 000A */
  {0x000C,0x000C,0x000C},  {0x000D,0x000D,0x000D}, /* 000C */
  {0x000E,0x000E,0x000E},  {0x000F,0x000F,0x000F}, /* 000E */
  {0x0010,0x0010,0x0010},  {0x0011,0x0011,0x0011}, /* 0010 */
  {0x0012,0x0012,0x0012},  {0x0013,0x0013,0x0013}, /* 0012 */
  {0x0014,0x0014,0x0014},  {0x0015,0x0015,0x0015}, /* 0014 */
  {0x0016,0x0016,0x0016},  {0x0017,0x0017,0x0017}, /* 0016 */
  {0x0018,0x0018,0x0018},  {0x0019,0x0019,0x0019}, /* 0018 */
  {0x001A,0x001A,0x001A},  {0x001B,0x001B,0x001B}, /* 001A */
  {0x001C,0x001C,0x001C},  {0x001D,0x001D,0x001D}, /* 001C */
  {0x001E,0x001E,0x001E},  {0x001F,0x001F,0x001F}, /* 001E */
  {0x0020,0x0020,0x0020},  {0x0021,0x0021,0x0021}, /* 0020 */
  {0x0022,0x0022,0x0022},  {0x0023,0x0023,0x0023}, /* 0022 */
  {0x0024,0x0024,0x0024},  {0x0025,0x0025,0x0025}, /* 0024 */
  {0x0026,0x0026,0x0026},  {0x0027,0x0027,0x0027}, /* 0026 */
  {0x0028,0x0028,0x0028},  {0x0029,0x0029,0x0029}, /* 0028 */
  {0x002A,0x002A,0x002A},  {0x002B,0x002B,0x002B}, /* 002A */
  {0x002C,0x002C,0x002C},  {0x002D,0x002D,0x002D}, /* 002C */
  {0x002E,0x002E,0x002E},  {0x002F,0x002F,0x002F}, /* 002E */
  {0x0030,0x0030,0x0030},  {0x0031,0x0031,0x0031}, /* 0030 */
  {0x0032,0x0032,0x0032},  {0x0033,0x0033,0x0033}, /* 0032 */
  {0x0034,0x0034,0x0034},  {0x0035,0x0035,0x0035}, /* 0034 */
  {0x0036,0x0036,0x0036},  {0x0037,0x0037,0x0037}, /* 0036 */
  {0x0038,0x0038,0x0038},  {0x0039,0x0039,0x0039}, /* 0038 */
  {0x003A,0x003A,0x003A},  {0x003B,0x003B,0x003B}, /* 003A */
  {0x003C,0x003C,0x003C},  {0x003D,0x003D,0x003D}, /* 003C */
  {0x003E,0x003E,0x003E},  {0x003F,0x003F,0x003F}, /* 003E */
  {0x0040,0x0040,0x0040},  {0x0041,0x0061,0x0041}, /* 0040 */
  {0x0042,0x0062,0x0042},  {0x0043,0x0063,0x0043}, /* 0042 */
  {0x0044,0x0064,0x0044},  {0x0045,0x0065,0x0045}, /* 0044 */
  {0x0046,0x0066,0x0046},  {0x0047,0x0067,0x0047}, /* 0046 */
  {0x0048,0x0068,0x0048},  {0x0049,0x0069,0x0049}, /* 0048 */
  {0x004A,0x006A,0x004A},  {0x004B,0x006B,0x004B}, /* 004A */
  {0x004C,0x006C,0x004C},  {0x004D,0x006D,0x004D}, /* 004C */
  {0x004E,0x006E,0x004E},  {0x004F,0x006F,0x004F}, /* 004E */
  {0x0050,0x0070,0x0050},  {0x0051,0x0071,0x0051}, /* 0050 */
  {0x0052,0x0072,0x0052},  {0x0053,0x0073,0x0053}, /* 0052 */
  {0x0054,0x0074,0x0054},  {0x0055,0x0075,0x0055}, /* 0054 */
  {0x0056,0x0076,0x0056},  {0x0057,0x0077,0x0057}, /* 0056 */
  {0x0058,0x0078,0x0058},  {0x0059,0x0079,0x0059}, /* 0058 */
  {0x005A,0x007A,0x005A},  {0x005B,0x005B,0x005B}, /* 005A */
  {0x005C,0x005C,0x005C},  {0x005D,0x005D,0x005D}, /* 005C */
  {0x005E,0x005E,0x005E},  {0x005F,0x005F,0x005F}, /* 005E */
  {0x0060,0x0060,0x0060},  {0x0041,0x0061,0x0041}, /* 0060 */
  {0x0042,0x0062,0x0042},  {0x0043,0x0063,0x0043}, /* 0062 */
  {0x0044,0x0064,0x0044},  {0x0045,0x0065,0x0045}, /* 0064 */
  {0x0046,0x0066,0x0046},  {0x0047,0x0067,0x0047}, /* 0066 */
  {0x0048,0x0068,0x0048},  {0x0049,0x0069,0x0049}, /* 0068 */
  {0x004A,0x006A,0x004A},  {0x004B,0x006B,0x004B}, /* 006A */
  {0x004C,0x006C,0x004C},  {0x004D,0x006D,0x004D}, /* 006C */
  {0x004E,0x006E,0x004E},  {0x004F,0x006F,0x004F}, /* 006E */
  {0x0050,0x0070,0x0050},  {0x0051,0x0071,0x0051}, /* 0070 */
  {0x0052,0x0072,0x0052},  {0x0053,0x0073,0x0053}, /* 0072 */
  {0x0054,0x0074,0x0054},  {0x0055,0x0075,0x0055}, /* 0074 */
  {0x0056,0x0076,0x0056},  {0x0057,0x0077,0x0057}, /* 0076 */
  {0x0058,0x0078,0x0058},  {0x0059,0x0079,0x0059}, /* 0078 */
  {0x005A,0x007A,0x005A},  {0x007B,0x007B,0x007B}, /* 007A */
  {0x007C,0x007C,0x007C},  {0x007D,0x007D,0x007D}, /* 007C */
  {0x007E,0x007E,0x007E},  {0x007F,0x007F,0x007F}, /* 007E */
  {0x0080,0x0080,0x0080},  {0x0081,0x0081,0x0081}, /* 0080 */
  {0x0082,0x0082,0x0082},  {0x0083,0x0083,0x0083}, /* 0082 */
  {0x0084,0x0084,0x0084},  {0x0085,0x0085,0x0085}, /* 0084 */
  {0x0086,0x0086,0x0086},  {0x0087,0x0087,0x0087}, /* 0086 */
  {0x0088,0x0088,0x0088},  {0x0089,0x0089,0x0089}, /* 0088 */
  {0x008A,0x008A,0x008A},  {0x008B,0x008B,0x008B}, /* 008A */
  {0x008C,0x008C,0x008C},  {0x008D,0x008D,0x008D}, /* 008C */
  {0x008E,0x008E,0x008E},  {0x008F,0x008F,0x008F}, /* 008E */
  {0x0090,0x0090,0x0090},  {0x0091,0x0091,0x0091}, /* 0090 */
  {0x0092,0x0092,0x0092},  {0x0093,0x0093,0x0093}, /* 0092 */
  {0x0094,0x0094,0x0094},  {0x0095,0x0095,0x0095}, /* 0094 */
  {0x0096,0x0096,0x0096},  {0x0097,0x0097,0x0097}, /* 0096 */
  {0x0098,0x0098,0x0098},  {0x0099,0x0099,0x0099}, /* 0098 */
  {0x009A,0x009A,0x009A},  {0x009B,0x009B,0x009B}, /* 009A */
  {0x009C,0x009C,0x009C},  {0x009D,0x009D,0x009D}, /* 009C */
  {0x009E,0x009E,0x009E},  {0x009F,0x009F,0x009F}, /* 009E */
  {0x00A0,0x00A0,0x00A0},  {0x00A1,0x00A1,0x00A1}, /* 00A0 */
  {0x00A2,0x00A2,0x00A2},  {0x00A3,0x00A3,0x00A3}, /* 00A2 */
  {0x00A4,0x00A4,0x00A4},  {0x00A5,0x00A5,0x00A5}, /* 00A4 */
  {0x00A6,0x00A6,0x00A6},  {0x00A7,0x00A7,0x00A7}, /* 00A6 */
  {0x00A8,0x00A8,0x00A8},  {0x00A9,0x00A9,0x00A9}, /* 00A8 */
  {0x00AA,0x00AA,0x00AA},  {0x00AB,0x00AB,0x00AB}, /* 00AA */
  {0x00AC,0x00AC,0x00AC},  {0x00AD,0x00AD,0x00AD}, /* 00AC */
  {0x00AE,0x00AE,0x00AE},  {0x00AF,0x00AF,0x00AF}, /* 00AE */
  {0x00B0,0x00B0,0x00B0},  {0x00B1,0x00B1,0x00B1}, /* 00B0 */
  {0x00B2,0x00B2,0x00B2},  {0x00B3,0x00B3,0x00B3}, /* 00B2 */
  {0x00B4,0x00B4,0x00B4},  {0x039C,0x00B5,0x039C}, /* 00B4 */
  {0x00B6,0x00B6,0x00B6},  {0x00B7,0x00B7,0x00B7}, /* 00B6 */
  {0x00B8,0x00B8,0x00B8},  {0x00B9,0x00B9,0x00B9}, /* 00B8 */
  {0x00BA,0x00BA,0x00BA},  {0x00BB,0x00BB,0x00BB}, /* 00BA */
  {0x00BC,0x00BC,0x00BC},  {0x00BD,0x00BD,0x00BD}, /* 00BC */
  {0x00BE,0x00BE,0x00BE},  {0x00BF,0x00BF,0x00BF}, /* 00BE */
  {0x00C0,0x00E0,0x00C0},  {0x00C1,0x00E1,0x00C1}, /* 00C0 */
  {0x00C2,0x00E2,0x00C2},  {0x00C3,0x00E3,0x00C3}, /* 00C2 */
  {0x00C4,0x00E4,0x00C4},  {0x00C5,0x00E5,0x00C5}, /* 00C4 */
  {0x00C6,0x00E6,0x00C6},  {0x00C7,0x00E7,0x00C7}, /* 00C6 */
  {0x00C8,0x00E8,0x00C8},  {0x00C9,0x00E9,0x00C9}, /* 00C8 */
  {0x00CA,0x00EA,0x00CA},  {0x00CB,0x00EB,0x00CB}, /* 00CA */
  {0x00CC,0x00EC,0x00CC},  {0x00CD,0x00ED,0x00CD}, /* 00CC */
  {0x00CE,0x00EE,0x00CE},  {0x00CF,0x00EF,0x00CF}, /* 00CE */
  {0x00D0,0x00F0,0x00D0},  {0x00D1,0x00F1,0x00D1}, /* 00D0 */
  {0x00D2,0x00F2,0x00D2},  {0x00D3,0x00F3,0x00D3}, /* 00D2 */
  {0x00D4,0x00F4,0x00D4},  {0x00D5,0x00F5,0x00D5}, /* 00D4 */
  {0x00D6,0x00F6,0x00D6},  {0x00D7,0x00D7,0x00D7}, /* 00D6 */
  {0x00D8,0x00F8,0x00D8},  {0x00D9,0x00F9,0x00D9}, /* 00D8 */
  {0x00DA,0x00FA,0x00DA},  {0x00DB,0x00FB,0x00DB}, /* 00DA */
  {0x00DC,0x00FC,0x00DC},  {0x00DD,0x00FD,0x00DD}, /* 00DC */
  {0x00DE,0x00FE,0x00DE},  {0x00DF,0x00DF,0x00DF}, /* 00DE */
  {0x00C0,0x00E0,0x00C0},  {0x00C1,0x00E1,0x00C1}, /* 00E0 */
  {0x00C2,0x00E2,0x00C2},  {0x00C3,0x00E3,0x00C3}, /* 00E2 */
  {0x00C4,0x00E4,0x00C4},  {0x00C5,0x00E5,0x00C5}, /* 00E4 */
  {0x00C6,0x00E6,0x00C6},  {0x00C7,0x00E7,0x00C7}, /* 00E6 */
  {0x00C8,0x00E8,0x00C8},  {0x00C9,0x00E9,0x00C9}, /* 00E8 */
  {0x00CA,0x00EA,0x00CA},  {0x00CB,0x00EB,0x00CB}, /* 00EA */
  {0x00CC,0x00EC,0x00CC},  {0x00CD,0x00ED,0x00CD}, /* 00EC */
  {0x00CE,0x00EE,0x00CE},  {0x00CF,0x00EF,0x00CF}, /* 00EE */
  {0x00D0,0x00F0,0x00D0},  {0x00D1,0x00F1,0x00D1}, /* 00F0 */
  {0x00D2,0x00F2,0x00D2},  {0x00D3,0x00F3,0x00D3}, /* 00F2 */
  {0x00D4,0x00F4,0x00D4},  {0x00D5,0x00F5,0x00D5}, /* 00F4 */
  {0x00D6,0x00F6,0x00D6},  {0x00F7,0x00F7,0x00F7}, /* 00F6 */
  {0x00D8,0x00F8,0x00D8},  {0x00D9,0x00F9,0x00D9}, /* 00F8 */
  {0x00DA,0x00FA,0x00DA},  {0x00DB,0x00FB,0x00DB}, /* 00FA */
  {0x00DC,0x00FC,0x00DC},  {0x00DD,0x00FD,0x00DD}, /* 00FC */
  {0x00DE,0x00FE,0x00DE},  {0x0178,0x00FF,0x0178}  /* 00FE */
};

static const MY_UNICASE_CHARACTER u900p001[]= {
  {0x0100,0x0101,0x0100},  {0x0100,0x0101,0x0100}, /* 0100 */
  {0x0102,0x0103,0x0102},  {0x0102,0x0103,0x0102}, /* 0102 */
  {0x0104,0x0105,0x0104},  {0x0104,0x0105,0x0104}, /* 0104 */
  {0x0106,0x0107,0x0106},  {0x0106,0x0107,0x0106}, /* 0106 */
  {0x0108,0x0109,0x0108},  {0x0108,0x0109,0x0108}, /* 0108 */
  {0x010A,0x010B,0x010A},  {0x010A,0x010B,0x010A}, /* 010A */
  {0x010C,0x010D,0x010C},  {0x010C,0x010D,0x010C}, /* 010C */
  {0x010E,0x010F,0x010E},  {0x010E,0x010F,0x010E}, /* 010E */
  {0x0110,0x0111,0x0110},  {0x0110,0x0111,0x0110}, /* 0110 */
  {0x0112,0x0113,0x0112},  {0x0112,0x0113,0x0112}, /* 0112 */
  {0x0114,0x0115,0x0114},  {0x0114,0x0115,0x0114}, /* 0114 */
  {0x0116,0x0117,0x0116},  {0x0116,0x0117,0x0116}, /* 0116 */
  {0x0118,0x0119,0x0118},  {0x0118,0x0119,0x0118}, /* 0118 */
  {0x011A,0x011B,0x011A},  {0x011A,0x011B,0x011A}, /* 011A */
  {0x011C,0x011D,0x011C},  {0x011C,0x011D,0x011C}, /* 011C */
  {0x011E,0x011F,0x011E},  {0x011E,0x011F,0x011E}, /* 011E */
  {0x0120,0x0121,0x0120},  {0x0120,0x0121,0x0120}, /* 0120 */
  {0x0122,0x0123,0x0122},  {0x0122,0x0123,0x0122}, /* 0122 */
  {0x0124,0x0125,0x0124},  {0x0124,0x0125,0x0124}, /* 0124 */
  {0x0126,0x0127,0x0126},  {0x0126,0x0127,0x0126}, /* 0126 */
  {0x0128,0x0129,0x0128},  {0x0128,0x0129,0x0128}, /* 0128 */
  {0x012A,0x012B,0x012A},  {0x012A,0x012B,0x012A}, /* 012A */
  {0x012C,0x012D,0x012C},  {0x012C,0x012D,0x012C}, /* 012C */
  {0x012E,0x012F,0x012E},  {0x012E,0x012F,0x012E}, /* 012E */
  {0x0130,0x0069,0x0130},  {0x0049,0x0131,0x0049}, /* 0130 */
  {0x0132,0x0133,0x0132},  {0x0132,0x0133,0x0132}, /* 0132 */
  {0x0134,0x0135,0x0134},  {0x0134,0x0135,0x0134}, /* 0134 */
  {0x0136,0x0137,0x0136},  {0x0136,0x0137,0x0136}, /* 0136 */
  {0x0138,0x0138,0x0138},  {0x0139,0x013A,0x0139}, /* 0138 */
  {0x0139,0x013A,0x0139},  {0x013B,0x013C,0x013B}, /* 013A */
  {0x013B,0x013C,0x013B},  {0x013D,0x013E,0x013D}, /* 013C */
  {0x013D,0x013E,0x013D},  {0x013F,0x0140,0x013F}, /* 013E */
  {0x013F,0x0140,0x013F},  {0x0141,0x0142,0x0141}, /* 0140 */
  {0x0141,0x0142,0x0141},  {0x0143,0x0144,0x0143}, /* 0142 */
  {0x0143,0x0144,0x0143},  {0x0145,0x0146,0x0145}, /* 0144 */
  {0x0145,0x0146,0x0145},  {0x0147,0x0148,0x0147}, /* 0146 */
  {0x0147,0x0148,0x0147},  {0x0149,0x0149,0x0149}, /* 0148 */
  {0x014A,0x014B,0x014A},  {0x014A,0x014B,0x014A}, /* 014A */
  {0x014C,0x014D,0x014C},  {0x014C,0x014D,0x014C}, /* 014C */
  {0x014E,0x014F,0x014E},  {0x014E,0x014F,0x014E}, /* 014E */
  {0x0150,0x0151,0x0150},  {0x0150,0x0151,0x0150}, /* 0150 */
  {0x0152,0x0153,0x0152},  {0x0152,0x0153,0x0152}, /* 0152 */
  {0x0154,0x0155,0x0154},  {0x0154,0x0155,0x0154}, /* 0154 */
  {0x0156,0x0157,0x0156},  {0x0156,0x0157,0x0156}, /* 0156 */
  {0x0158,0x0159,0x0158},  {0x0158,0x0159,0x0158}, /* 0158 */
  {0x015A,0x015B,0x015A},  {0x015A,0x015B,0x015A}, /* 015A */
  {0x015C,0x015D,0x015C},  {0x015C,0x015D,0x015C}, /* 015C */
  {0x015E,0x015F,0x015E},  {0x015E,0x015F,0x015E}, /* 015E */
  {0x0160,0x0161,0x0160},  {0x0160,0x0161,0x0160}, /* 0160 */
  {0x0162,0x0163,0x0162},  {0x0162,0x0163,0x0162}, /* 0162 */
  {0x0164,0x0165,0x0164},  {0x0164,0x0165,0x0164}, /* 0164 */
  {0x0166,0x0167,0x0166},  {0x0166,0x0167,0x0166}, /* 0166 */
  {0x0168,0x0169,0x0168},  {0x0168,0x0169,0x0168}, /* 0168 */
  {0x016A,0x016B,0x016A},  {0x016A,0x016B,0x016A}, /* 016A */
  {0x016C,0x016D,0x016C},  {0x016C,0x016D,0x016C}, /* 016C */
  {0x016E,0x016F,0x016E},  {0x016E,0x016F,0x016E}, /* 016E */
  {0x0170,0x0171,0x0170},  {0x0170,0x0171,0x0170}, /* 0170 */
  {0x0172,0x0173,0x0172},  {0x0172,0x0173,0x0172}, /* 0172 */
  {0x0174,0x0175,0x0174},  {0x0174,0x0175,0x0174}, /* 0174 */
  {0x0176,0x0177,0x0176},  {0x0176,0x0177,0x0176}, /* 0176 */
  {0x0178,0x00FF,0x0178},  {0x0179,0x017A,0x0179}, /* 0178 */
  {0x0179,0x017A,0x0179},  {0x017B,0x017C,0x017B}, /* 017A */
  {0x017B,0x017C,0x017B},  {0x017D,0x017E,0x017D}, /* 017C */
  {0x017D,0x017E,0x017D},  {0x0053,0x017F,0x0053}, /* 017E */
  {0x0243,0x0180,0x0243},  {0x0181,0x0253,0x0181}, /* 0180 */
  {0x0182,0x0183,0x0182},  {0x0182,0x0183,0x0182}, /* 0182 */
  {0x0184,0x0185,0x0184},  {0x0184,0x0185,0x0184}, /* 0184 */
  {0x0186,0x0254,0x0186},  {0x0187,0x0188,0x0187}, /* 0186 */
  {0x0187,0x0188,0x0187},  {0x0189,0x0256,0x0189}, /* 0188 */
  {0x018A,0x0257,0x018A},  {0x018B,0x018C,0x018B}, /* 018A */
  {0x018B,0x018C,0x018B},  {0x018D,0x018D,0x018D}, /* 018C */
  {0x018E,0x01DD,0x018E},  {0x018F,0x0259,0x018F}, /* 018E */
  {0x0190,0x025B,0x0190},  {0x0191,0x0192,0x0191}, /* 0190 */
  {0x0191,0x0192,0x0191},  {0x0193,0x0260,0x0193}, /* 0192 */
  {0x0194,0x0263,0x0194},  {0x01F6,0x0195,0x01F6}, /* 0194 */
  {0x0196,0x0269,0x0196},  {0x0197,0x0268,0x0197}, /* 0196 */
  {0x0198,0x0199,0x0198},  {0x0198,0x0199,0x0198}, /* 0198 */
  {0x023D,0x019A,0x023D},  {0x019B,0x019B,0x019B}, /* 019A */
  {0x019C,0x026F,0x019C},  {0x019D,0x0272,0x019D}, /* 019C */
  {0x0220,0x019E,0x0220},  {0x019F,0x0275,0x019F}, /* 019E */
  {0x01A0,0x01A1,0x01A0},  {0x01A0,0x01A1,0x01A0}, /* 01A0 */
  {0x01A2,0x01A3,0x01A2},  {0x01A2,0x01A3,0x01A2}, /* 01A2 */
  {0x01A4,0x01A5,0x01A4},  {0x01A4,0x01A5,0x01A4}, /* 01A4 */
  {0x01A6,0x0280,0x01A6},  {0x01A7,0x01A8,0x01A7}, /* 01A6 */
  {0x01A7,0x01A8,0x01A7},  {0x01A9,0x0283,0x01A9}, /* 01A8 */
  {0x01AA,0x01AA,0x01AA},  {0x01AB,0x01AB,0x01AB}, /* 01AA */
  {0x01AC,0x01AD,0x01AC},  {0x01AC,0x01AD,0x01AC}, /* 01AC */
  {0x01AE,0x0288,0x01AE},  {0x01AF,0x01B0,0x01AF}, /* 01AE */
  {0x01AF,0x01B0,0x01AF},  {0x01B1,0x028A,0x01B1}, /* 01B0 */
  {0x01B2,0x028B,0x01B2},  {0x01B3,0x01B4,0x01B3}, /* 01B2 */
  {0x01B3,0x01B4,0x01B3},  {0x01B5,0x01B6,0x01B5}, /* 01B4 */
  {0x01B5,0x01B6,0x01B5},  {0x01B7,0x0292,0x01B7}, /* 01B6 */
  {0x01B8,0x01B9,0x01B8},  {0x01B8,0x01B9,0x01B8}, /* 01B8 */
  {0x01BA,0x01BA,0x01BA},  {0x01BB,0x01BB,0x01BB}, /* 01BA */
  {0x01BC,0x01BD,0x01BC},  {0x01BC,0x01BD,0x01BC}, /* 01BC */
  {0x01BE,0x01BE,0x01BE},  {0x01F7,0x01BF,0x01F7}, /* 01BE */
  {0x01C0,0x01C0,0x01C0},  {0x01C1,0x01C1,0x01C1}, /* 01C0 */
  {0x01C2,0x01C2,0x01C2},  {0x01C3,0x01C3,0x01C3}, /* 01C2 */
  {0x01C4,0x01C6,0x01C5},  {0x01C4,0x01C6,0x01C5}, /* 01C4 */
  {0x01C4,0x01C6,0x01C5},  {0x01C7,0x01C9,0x01C8}, /* 01C6 */
  {0x01C7,0x01C9,0x01C8},  {0x01C7,0x01C9,0x01C8}, /* 01C8 */
  {0x01CA,0x01CC,0x01CB},  {0x01CA,0x01CC,0x01CB}, /* 01CA */
  {0x01CA,0x01CC,0x01CB},  {0x01CD,0x01CE,0x01CD}, /* 01CC */
  {0x01CD,0x01CE,0x01CD},  {0x01CF,0x01D0,0x01CF}, /* 01CE */
  {0x01CF,0x01D0,0x01CF},  {0x01D1,0x01D2,0x01D1}, /* 01D0 */
  {0x01D1,0x01D2,0x01D1},  {0x01D3,0x01D4,0x01D3}, /* 01D2 */
  {0x01D3,0x01D4,0x01D3},  {0x01D5,0x01D6,0x01D5}, /* 01D4 */
  {0x01D5,0x01D6,0x01D5},  {0x01D7,0x01D8,0x01D7}, /* 01D6 */
  {0x01D7,0x01D8,0x01D7},  {0x01D9,0x01DA,0x01D9}, /* 01D8 */
  {0x01D9,0x01DA,0x01D9},  {0x01DB,0x01DC,0x01DB}, /* 01DA */
  {0x01DB,0x01DC,0x01DB},  {0x018E,0x01DD,0x018E}, /* 01DC */
  {0x01DE,0x01DF,0x01DE},  {0x01DE,0x01DF,0x01DE}, /* 01DE */
  {0x01E0,0x01E1,0x01E0},  {0x01E0,0x01E1,0x01E0}, /* 01E0 */
  {0x01E2,0x01E3,0x01E2},  {0x01E2,0x01E3,0x01E2}, /* 01E2 */
  {0x01E4,0x01E5,0x01E4},  {0x01E4,0x01E5,0x01E4}, /* 01E4 */
  {0x01E6,0x01E7,0x01E6},  {0x01E6,0x01E7,0x01E6}, /* 01E6 */
  {0x01E8,0x01E9,0x01E8},  {0x01E8,0x01E9,0x01E8}, /* 01E8 */
  {0x01EA,0x01EB,0x01EA},  {0x01EA,0x01EB,0x01EA}, /* 01EA */
  {0x01EC,0x01ED,0x01EC},  {0x01EC,0x01ED,0x01EC}, /* 01EC */
  {0x01EE,0x01EF,0x01EE},  {0x01EE,0x01EF,0x01EE}, /* 01EE */
  {0x01F0,0x01F0,0x01F0},  {0x01F1,0x01F3,0x01F2}, /* 01F0 */
  {0x01F1,0x01F3,0x01F2},  {0x01F1,0x01F3,0x01F2}, /* 01F2 */
  {0x01F4,0x01F5,0x01F4},  {0x01F4,0x01F5,0x01F4}, /* 01F4 */
  {0x01F6,0x0195,0x01F6},  {0x01F7,0x01BF,0x01F7}, /* 01F6 */
  {0x01F8,0x01F9,0x01F8},  {0x01F8,0x01F9,0x01F8}, /* 01F8 */
  {0x01FA,0x01FB,0x01FA},  {0x01FA,0x01FB,0x01FA}, /* 01FA */
  {0x01FC,0x01FD,0x01FC},  {0x01FC,0x01FD,0x01FC}, /* 01FC */
  {0x01FE,0x01FF,0x01FE},  {0x01FE,0x01FF,0x01FE}  /* 01FE */
};

static const MY_UNICASE_CHARACTER u900p002[]= {
  {0x0200,0x0201,0x0200},  {0x0200,0x0201,0x0200}, /* 0200 */
  {0x0202,0x0203,0x0202},  {0x0202,0x0203,0x0202}, /* 0202 */
  {0x0204,0x0205,0x0204},  {0x0204,0x0205,0x0204}, /* 0204 */
  {0x0206,0x0207,0x0206},  {0x0206,0x0207,0x0206}, /* 0206 */
  {0x0208,0x0209,0x0208},  {0x0208,0x0209,0x0208}, /* 0208 */
  {0x020A,0x020B,0x020A},  {0x020A,0x020B,0x020A}, /* 020A */
  {0x020C,0x020D,0x020C},  {0x020C,0x020D,0x020C}, /* 020C */
  {0x020E,0x020F,0x020E},  {0x020E,0x020F,0x020E}, /* 020E */
  {0x0210,0x0211,0x0210},  {0x0210,0x0211,0x0210}, /* 0210 */
  {0x0212,0x0213,0x0212},  {0x0212,0x0213,0x0212}, /* 0212 */
  {0x0214,0x0215,0x0214},  {0x0214,0x0215,0x0214}, /* 0214 */
  {0x0216,0x0217,0x0216},  {0x0216,0x0217,0x0216}, /* 0216 */
  {0x0218,0x0219,0x0218},  {0x0218,0x0219,0x0218}, /* 0218 */
  {0x021A,0x021B,0x021A},  {0x021A,0x021B,0x021A}, /* 021A */
  {0x021C,0x021D,0x021C},  {0x021C,0x021D,0x021C}, /* 021C */
  {0x021E,0x021F,0x021E},  {0x021E,0x021F,0x021E}, /* 021E */
  {0x0220,0x019E,0x0220},  {0x0221,0x0221,0x0221}, /* 0220 */
  {0x0222,0x0223,0x0222},  {0x0222,0x0223,0x0222}, /* 0222 */
  {0x0224,0x0225,0x0224},  {0x0224,0x0225,0x0224}, /* 0224 */
  {0x0226,0x0227,0x0226},  {0x0226,0x0227,0x0226}, /* 0226 */
  {0x0228,0x0229,0x0228},  {0x0228,0x0229,0x0228}, /* 0228 */
  {0x022A,0x022B,0x022A},  {0x022A,0x022B,0x022A}, /* 022A */
  {0x022C,0x022D,0x022C},  {0x022C,0x022D,0x022C}, /* 022C */
  {0x022E,0x022F,0x022E},  {0x022E,0x022F,0x022E}, /* 022E */
  {0x0230,0x0231,0x0230},  {0x0230,0x0231,0x0230}, /* 0230 */
  {0x0232,0x0233,0x0232},  {0x0232,0x0233,0x0232}, /* 0232 */
  {0x0234,0x0234,0x0234},  {0x0235,0x0235,0x0235}, /* 0234 */
  {0x0236,0x0236,0x0236},  {0x0237,0x0237,0x0237}, /* 0236 */
  {0x0238,0x0238,0x0238},  {0x0239,0x0239,0x0239}, /* 0238 */
  {0x023A,0x2C65,0x023A},  {0x023B,0x023C,0x023B}, /* 023A */
  {0x023B,0x023C,0x023B},  {0x023D,0x019A,0x023D}, /* 023C */
  {0x023E,0x2C66,0x023E},  {0x2C7E,0x023F,0x2C7E}, /* 023E */
  {0x2C7F,0x0240,0x2C7F},  {0x0241,0x0242,0x0241}, /* 0240 */
  {0x0241,0x0242,0x0241},  {0x0243,0x0180,0x0243}, /* 0242 */
  {0x0244,0x0289,0x0244},  {0x0245,0x028C,0x0245}, /* 0244 */
  {0x0246,0x0247,0x0246},  {0x0246,0x0247,0x0246}, /* 0246 */
  {0x0248,0x0249,0x0248},  {0x0248,0x0249,0x0248}, /* 0248 */
  {0x024A,0x024B,0x024A},  {0x024A,0x024B,0x024A}, /* 024A */
  {0x024C,0x024D,0x024C},  {0x024C,0x024D,0x024C}, /* 024C */
  {0x024E,0x024F,0x024E},  {0x024E,0x024F,0x024E}, /* 024E */
  {0x2C6F,0x0250,0x2C6F},  {0x2C6D,0x0251,0x2C6D}, /* 0250 */
  {0x2C70,0x0252,0x2C70},  {0x0181,0x0253,0x0181}, /* 0252 */
  {0x0186,0x0254,0x0186},  {0x0255,0x0255,0x0255}, /* 0254 */
  {0x0189,0x0256,0x0189},  {0x018A,0x0257,0x018A}, /* 0256 */
  {0x0258,0x0258,0x0258},  {0x018F,0x0259,0x018F}, /* 0258 */
  {0x025A,0x025A,0x025A},  {0x0190,0x025B,0x0190}, /* 025A */
  {0xA7AB,0x025C,0xA7AB},  {0x025D,0x025D,0x025D}, /* 025C */
  {0x025E,0x025E,0x025E},  {0x025F,0x025F,0x025F}, /* 025E */
  {0x0193,0x0260,0x0193},  {0xA7AC,0x0261,0xA7AC}, /* 0260 */
  {0x0262,0x0262,0x0262},  {0x0194,0x0263,0x0194}, /* 0262 */
  {0x0264,0x0264,0x0264},  {0xA78D,0x0265,0xA78D}, /* 0264 */
  {0xA7AA,0x0266,0xA7AA},  {0x0267,0x0267,0x0267}, /* 0266 */
  {0x0197,0x0268,0x0197},  {0x0196,0x0269,0x0196}, /* 0268 */
  {0xA7AE,0x026A,0xA7AE},  {0x2C62,0x026B,0x2C62}, /* 026A */
  {0xA7AD,0x026C,0xA7AD},  {0x026D,0x026D,0x026D}, /* 026C */
  {0x026E,0x026E,0x026E},  {0x019C,0x026F,0x019C}, /* 026E */
  {0x0270,0x0270,0x0270},  {0x2C6E,0x0271,0x2C6E}, /* 0270 */
  {0x019D,0x0272,0x019D},  {0x0273,0x0273,0x0273}, /* 0272 */
  {0x0274,0x0274,0x0274},  {0x019F,0x0275,0x019F}, /* 0274 */
  {0x0276,0x0276,0x0276},  {0x0277,0x0277,0x0277}, /* 0276 */
  {0x0278,0x0278,0x0278},  {0x0279,0x0279,0x0279}, /* 0278 */
  {0x027A,0x027A,0x027A},  {0x027B,0x027B,0x027B}, /* 027A */
  {0x027C,0x027C,0x027C},  {0x2C64,0x027D,0x2C64}, /* 027C */
  {0x027E,0x027E,0x027E},  {0x027F,0x027F,0x027F}, /* 027E */
  {0x01A6,0x0280,0x01A6},  {0x0281,0x0281,0x0281}, /* 0280 */
  {0x0282,0x0282,0x0282},  {0x01A9,0x0283,0x01A9}, /* 0282 */
  {0x0284,0x0284,0x0284},  {0x0285,0x0285,0x0285}, /* 0284 */
  {0x0286,0x0286,0x0286},  {0xA7B1,0x0287,0xA7B1}, /* 0286 */
  {0x01AE,0x0288,0x01AE},  {0x0244,0x0289,0x0244}, /* 0288 */
  {0x01B1,0x028A,0x01B1},  {0x01B2,0x028B,0x01B2}, /* 028A */
  {0x0245,0x028C,0x0245},  {0x028D,0x028D,0x028D}, /* 028C */
  {0x028E,0x028E,0x028E},  {0x028F,0x028F,0x028F}, /* 028E */
  {0x0290,0x0290,0x0290},  {0x0291,0x0291,0x0291}, /* 0290 */
  {0x01B7,0x0292,0x01B7},  {0x0293,0x0293,0x0293}, /* 0292 */
  {0x0294,0x0294,0x0294},  {0x0295,0x0295,0x0295}, /* 0294 */
  {0x0296,0x0296,0x0296},  {0x0297,0x0297,0x0297}, /* 0296 */
  {0x0298,0x0298,0x0298},  {0x0299,0x0299,0x0299}, /* 0298 */
  {0x029A,0x029A,0x029A},  {0x029B,0x029B,0x029B}, /* 029A */
  {0x029C,0x029C,0x029C},  {0xA7B2,0x029D,0xA7B2}, /* 029C */
  {0xA7B0,0x029E,0xA7B0},  {0x029F,0x029F,0x029F}, /* 029E */
  {0x02A0,0x02A0,0x02A0},  {0x02A1,0x02A1,0x02A1}, /* 02A0 */
  {0x02A2,0x02A2,0x02A2},  {0x02A3,0x02A3,0x02A3}, /* 02A2 */
  {0x02A4,0x02A4,0x02A4},  {0x02A5,0x02A5,0x02A5}, /* 02A4 */
  {0x02A6,0x02A6,0x02A6},  {0x02A7,0x02A7,0x02A7}, /* 02A6 */
  {0x02A8,0x02A8,0x02A8},  {0x02A9,0x02A9,0x02A9}, /* 02A8 */
  {0x02AA,0x02AA,0x02AA},  {0x02AB,0x02AB,0x02AB}, /* 02AA */
  {0x02AC,0x02AC,0x02AC},  {0x02AD,0x02AD,0x02AD}, /* 02AC */
  {0x02AE,0x02AE,0x02AE},  {0x02AF,0x02AF,0x02AF}, /* 02AE */
  {0x02B0,0x02B0,0x02B0},  {0x02B1,0x02B1,0x02B1}, /* 02B0 */
  {0x02B2,0x02B2,0x02B2},  {0x02B3,0x02B3,0x02B3}, /* 02B2 */
  {0x02B4,0x02B4,0x02B4},  {0x02B5,0x02B5,0x02B5}, /* 02B4 */
  {0x02B6,0x02B6,0x02B6},  {0x02B7,0x02B7,0x02B7}, /* 02B6 */
  {0x02B8,0x02B8,0x02B8},  {0x02B9,0x02B9,0x02B9}, /* 02B8 */
  {0x02BA,0x02BA,0x02BA},  {0x02BB,0x02BB,0x02BB}, /* 02BA */
  {0x02BC,0x02BC,0x02BC},  {0x02BD,0x02BD,0x02BD}, /* 02BC */
  {0x02BE,0x02BE,0x02BE},  {0x02BF,0x02BF,0x02BF}, /* 02BE */
  {0x02C0,0x02C0,0x02C0},  {0x02C1,0x02C1,0x02C1}, /* 02C0 */
  {0x02C2,0x02C2,0x02C2},  {0x02C3,0x02C3,0x02C3}, /* 02C2 */
  {0x02C4,0x02C4,0x02C4},  {0x02C5,0x02C5,0x02C5}, /* 02C4 */
  {0x02C6,0x02C6,0x02C6},  {0x02C7,0x02C7,0x02C7}, /* 02C6 */
  {0x02C8,0x02C8,0x02C8},  {0x02C9,0x02C9,0x02C9}, /* 02C8 */
  {0x02CA,0x02CA,0x02CA},  {0x02CB,0x02CB,0x02CB}, /* 02CA */
  {0x02CC,0x02CC,0x02CC},  {0x02CD,0x02CD,0x02CD}, /* 02CC */
  {0x02CE,0x02CE,0x02CE},  {0x02CF,0x02CF,0x02CF}, /* 02CE */
  {0x02D0,0x02D0,0x02D0},  {0x02D1,0x02D1,0x02D1}, /* 02D0 */
  {0x02D2,0x02D2,0x02D2},  {0x02D3,0x02D3,0x02D3}, /* 02D2 */
  {0x02D4,0x02D4,0x02D4},  {0x02D5,0x02D5,0x02D5}, /* 02D4 */
  {0x02D6,0x02D6,0x02D6},  {0x02D7,0x02D7,0x02D7}, /* 02D6 */
  {0x02D8,0x02D8,0x02D8},  {0x02D9,0x02D9,0x02D9}, /* 02D8 */
  {0x02DA,0x02DA,0x02DA},  {0x02DB,0x02DB,0x02DB}, /* 02DA */
  {0x02DC,0x02DC,0x02DC},  {0x02DD,0x02DD,0x02DD}, /* 02DC */
  {0x02DE,0x02DE,0x02DE},  {0x02DF,0x02DF,0x02DF}, /* 02DE */
  {0x02E0,0x02E0,0x02E0},  {0x02E1,0x02E1,0x02E1}, /* 02E0 */
  {0x02E2,0x02E2,0x02E2},  {0x02E3,0x02E3,0x02E3}, /* 02E2 */
  {0x02E4,0x02E4,0x02E4},  {0x02E5,0x02E5,0x02E5}, /* 02E4 */
  {0x02E6,0x02E6,0x02E6},  {0x02E7,0x02E7,0x02E7}, /* 02E6 */
  {0x02E8,0x02E8,0x02E8},  {0x02E9,0x02E9,0x02E9}, /* 02E8 */
  {0x02EA,0x02EA,0x02EA},  {0x02EB,0x02EB,0x02EB}, /* 02EA */
  {0x02EC,0x02EC,0x02EC},  {0x02ED,0x02ED,0x02ED}, /* 02EC */
  {0x02EE,0x02EE,0x02EE},  {0x02EF,0x02EF,0x02EF}, /* 02EE */
  {0x02F0,0x02F0,0x02F0},  {0x02F1,0x02F1,0x02F1}, /* 02F0 */
  {0x02F2,0x02F2,0x02F2},  {0x02F3,0x02F3,0x02F3}, /* 02F2 */
  {0x02F4,0x02F4,0x02F4},  {0x02F5,0x02F5,0x02F5}, /* 02F4 */
  {0x02F6,0x02F6,0x02F6},  {0x02F7,0x02F7,0x02F7}, /* 02F6 */
  {0x02F8,0x02F8,0x02F8},  {0x02F9,0x02F9,0x02F9}, /* 02F8 */
  {0x02FA,0x02FA,0x02FA},  {0x02FB,0x02FB,0x02FB}, /* 02FA */
  {0x02FC,0x02FC,0x02FC},  {0x02FD,0x02FD,0x02FD}, /* 02FC */
  {0x02FE,0x02FE,0x02FE},  {0x02FF,0x02FF,0x02FF}  /* 02FE */
};

static const MY_UNICASE_CHARACTER u900p003[]= {
  {0x0300,0x0300,0x0300},  {0x0301,0x0301,0x0301}, /* 0300 */
  {0x0302,0x0302,0x0302},  {0x0303,0x0303,0x0303}, /* 0302 */
  {0x0304,0x0304,0x0304},  {0x0305,0x0305,0x0305}, /* 0304 */
  {0x0306,0x0306,0x0306},  {0x0307,0x0307,0x0307}, /* 0306 */
  {0x0308,0x0308,0x0308},  {0x0309,0x0309,0x0309}, /* 0308 */
  {0x030A,0x030A,0x030A},  {0x030B,0x030B,0x030B}, /* 030A */
  {0x030C,0x030C,0x030C},  {0x030D,0x030D,0x030D}, /* 030C */
  {0x030E,0x030E,0x030E},  {0x030F,0x030F,0x030F}, /* 030E */
  {0x0310,0x0310,0x0310},  {0x0311,0x0311,0x0311}, /* 0310 */
  {0x0312,0x0312,0x0312},  {0x0313,0x0313,0x0313}, /* 0312 */
  {0x0314,0x0314,0x0314},  {0x0315,0x0315,0x0315}, /* 0314 */
  {0x0316,0x0316,0x0316},  {0x0317,0x0317,0x0317}, /* 0316 */
  {0x0318,0x0318,0x0318},  {0x0319,0x0319,0x0319}, /* 0318 */
  {0x031A,0x031A,0x031A},  {0x031B,0x031B,0x031B}, /* 031A */
  {0x031C,0x031C,0x031C},  {0x031D,0x031D,0x031D}, /* 031C */
  {0x031E,0x031E,0x031E},  {0x031F,0x031F,0x031F}, /* 031E */
  {0x0320,0x0320,0x0320},  {0x0321,0x0321,0x0321}, /* 0320 */
  {0x0322,0x0322,0x0322},  {0x0323,0x0323,0x0323}, /* 0322 */
  {0x0324,0x0324,0x0324},  {0x0325,0x0325,0x0325}, /* 0324 */
  {0x0326,0x0326,0x0326},  {0x0327,0x0327,0x0327}, /* 0326 */
  {0x0328,0x0328,0x0328},  {0x0329,0x0329,0x0329}, /* 0328 */
  {0x032A,0x032A,0x032A},  {0x032B,0x032B,0x032B}, /* 032A */
  {0x032C,0x032C,0x032C},  {0x032D,0x032D,0x032D}, /* 032C */
  {0x032E,0x032E,0x032E},  {0x032F,0x032F,0x032F}, /* 032E */
  {0x0330,0x0330,0x0330},  {0x0331,0x0331,0x0331}, /* 0330 */
  {0x0332,0x0332,0x0332},  {0x0333,0x0333,0x0333}, /* 0332 */
  {0x0334,0x0334,0x0334},  {0x0335,0x0335,0x0335}, /* 0334 */
  {0x0336,0x0336,0x0336},  {0x0337,0x0337,0x0337}, /* 0336 */
  {0x0338,0x0338,0x0338},  {0x0339,0x0339,0x0339}, /* 0338 */
  {0x033A,0x033A,0x033A},  {0x033B,0x033B,0x033B}, /* 033A */
  {0x033C,0x033C,0x033C},  {0x033D,0x033D,0x033D}, /* 033C */
  {0x033E,0x033E,0x033E},  {0x033F,0x033F,0x033F}, /* 033E */
  {0x0340,0x0340,0x0340},  {0x0341,0x0341,0x0341}, /* 0340 */
  {0x0342,0x0342,0x0342},  {0x0343,0x0343,0x0343}, /* 0342 */
  {0x0344,0x0344,0x0344},  {0x0399,0x0345,0x0399}, /* 0344 */
  {0x0346,0x0346,0x0346},  {0x0347,0x0347,0x0347}, /* 0346 */
  {0x0348,0x0348,0x0348},  {0x0349,0x0349,0x0349}, /* 0348 */
  {0x034A,0x034A,0x034A},  {0x034B,0x034B,0x034B}, /* 034A */
  {0x034C,0x034C,0x034C},  {0x034D,0x034D,0x034D}, /* 034C */
  {0x034E,0x034E,0x034E},  {0x034F,0x034F,0x034F}, /* 034E */
  {0x0350,0x0350,0x0350},  {0x0351,0x0351,0x0351}, /* 0350 */
  {0x0352,0x0352,0x0352},  {0x0353,0x0353,0x0353}, /* 0352 */
  {0x0354,0x0354,0x0354},  {0x0355,0x0355,0x0355}, /* 0354 */
  {0x0356,0x0356,0x0356},  {0x0357,0x0357,0x0357}, /* 0356 */
  {0x0358,0x0358,0x0358},  {0x0359,0x0359,0x0359}, /* 0358 */
  {0x035A,0x035A,0x035A},  {0x035B,0x035B,0x035B}, /* 035A */
  {0x035C,0x035C,0x035C},  {0x035D,0x035D,0x035D}, /* 035C */
  {0x035E,0x035E,0x035E},  {0x035F,0x035F,0x035F}, /* 035E */
  {0x0360,0x0360,0x0360},  {0x0361,0x0361,0x0361}, /* 0360 */
  {0x0362,0x0362,0x0362},  {0x0363,0x0363,0x0363}, /* 0362 */
  {0x0364,0x0364,0x0364},  {0x0365,0x0365,0x0365}, /* 0364 */
  {0x0366,0x0366,0x0366},  {0x0367,0x0367,0x0367}, /* 0366 */
  {0x0368,0x0368,0x0368},  {0x0369,0x0369,0x0369}, /* 0368 */
  {0x036A,0x036A,0x036A},  {0x036B,0x036B,0x036B}, /* 036A */
  {0x036C,0x036C,0x036C},  {0x036D,0x036D,0x036D}, /* 036C */
  {0x036E,0x036E,0x036E},  {0x036F,0x036F,0x036F}, /* 036E */
  {0x0370,0x0371,0x0370},  {0x0370,0x0371,0x0370}, /* 0370 */
  {0x0372,0x0373,0x0372},  {0x0372,0x0373,0x0372}, /* 0372 */
  {0x0374,0x0374,0x0374},  {0x0375,0x0375,0x0375}, /* 0374 */
  {0x0376,0x0377,0x0376},  {0x0376,0x0377,0x0376}, /* 0376 */
  {0x0378,0x0378,0x0378},  {0x0379,0x0379,0x0379}, /* 0378 */
  {0x037A,0x037A,0x037A},  {0x03FD,0x037B,0x03FD}, /* 037A */
  {0x03FE,0x037C,0x03FE},  {0x03FF,0x037D,0x03FF}, /* 037C */
  {0x037E,0x037E,0x037E},  {0x037F,0x03F3,0x037F}, /* 037E */
  {0x0380,0x0380,0x0380},  {0x0381,0x0381,0x0381}, /* 0380 */
  {0x0382,0x0382,0x0382},  {0x0383,0x0383,0x0383}, /* 0382 */
  {0x0384,0x0384,0x0384},  {0x0385,0x0385,0x0385}, /* 0384 */
  {0x0386,0x03AC,0x0386},  {0x0387,0x0387,0x0387}, /* 0386 */
  {0x0388,0x03AD,0x0388},  {0x0389,0x03AE,0x0389}, /* 0388 */
  {0x038A,0x03AF,0x038A},  {0x038B,0x038B,0x038B}, /* 038A */
  {0x038C,0x03CC,0x038C},  {0x038D,0x038D,0x038D}, /* 038C */
  {0x038E,0x03CD,0x038E},  {0x038F,0x03CE,0x038F}, /* 038E */
  {0x0390,0x0390,0x0390},  {0x0391,0x03B1,0x0391}, /* 0390 */
  {0x0392,0x03B2,0x0392},  {0x0393,0x03B3,0x0393}, /* 0392 */
  {0x0394,0x03B4,0x0394},  {0x0395,0x03B5,0x0395}, /* 0394 */
  {0x0396,0x03B6,0x0396},  {0x0397,0x03B7,0x0397}, /* 0396 */
  {0x0398,0x03B8,0x0398},  {0x0399,0x03B9,0x0399}, /* 0398 */
  {0x039A,0x03BA,0x039A},  {0x039B,0x03BB,0x039B}, /* 039A */
  {0x039C,0x03BC,0x039C},  {0x039D,0x03BD,0x039D}, /* 039C */
  {0x039E,0x03BE,0x039E},  {0x039F,0x03BF,0x039F}, /* 039E */
  {0x03A0,0x03C0,0x03A0},  {0x03A1,0x03C1,0x03A1}, /* 03A0 */
  {0x03A2,0x03A2,0x03A2},  {0x03A3,0x03C3,0x03A3}, /* 03A2 */
  {0x03A4,0x03C4,0x03A4},  {0x03A5,0x03C5,0x03A5}, /* 03A4 */
  {0x03A6,0x03C6,0x03A6},  {0x03A7,0x03C7,0x03A7}, /* 03A6 */
  {0x03A8,0x03C8,0x03A8},  {0x03A9,0x03C9,0x03A9}, /* 03A8 */
  {0x03AA,0x03CA,0x03AA},  {0x03AB,0x03CB,0x03AB}, /* 03AA */
  {0x0386,0x03AC,0x0386},  {0x0388,0x03AD,0x0388}, /* 03AC */
  {0x0389,0x03AE,0x0389},  {0x038A,0x03AF,0x038A}, /* 03AE */
  {0x03B0,0x03B0,0x03B0},  {0x0391,0x03B1,0x0391}, /* 03B0 */
  {0x0392,0x03B2,0x0392},  {0x0393,0x03B3,0x0393}, /* 03B2 */
  {0x0394,0x03B4,0x0394},  {0x0395,0x03B5,0x0395}, /* 03B4 */
  {0x0396,0x03B6,0x0396},  {0x0397,0x03B7,0x0397}, /* 03B6 */
  {0x0398,0x03B8,0x0398},  {0x0399,0x03B9,0x0399}, /* 03B8 */
  {0x039A,0x03BA,0x039A},  {0x039B,0x03BB,0x039B}, /* 03BA */
  {0x039C,0x03BC,0x039C},  {0x039D,0x03BD,0x039D}, /* 03BC */
  {0x039E,0x03BE,0x039E},  {0x039F,0x03BF,0x039F}, /* 03BE */
  {0x03A0,0x03C0,0x03A0},  {0x03A1,0x03C1,0x03A1}, /* 03C0 */
  {0x03A3,0x03C2,0x03A3},  {0x03A3,0x03C3,0x03A3}, /* 03C2 */
  {0x03A4,0x03C4,0x03A4},  {0x03A5,0x03C5,0x03A5}, /* 03C4 */
  {0x03A6,0x03C6,0x03A6},  {0x03A7,0x03C7,0x03A7}, /* 03C6 */
  {0x03A8,0x03C8,0x03A8},  {0x03A9,0x03C9,0x03A9}, /* 03C8 */
  {0x03AA,0x03CA,0x03AA},  {0x03AB,0x03CB,0x03AB}, /* 03CA */
  {0x038C,0x03CC,0x038C},  {0x038E,0x03CD,0x038E}, /* 03CC */
  {0x038F,0x03CE,0x038F},  {0x03CF,0x03D7,0x03CF}, /* 03CE */
  {0x0392,0x03D0,0x0392},  {0x0398,0x03D1,0x0398}, /* 03D0 */
  {0x03D2,0x03D2,0x03D2},  {0x03D3,0x03D3,0x03D3}, /* 03D2 */
  {0x03D4,0x03D4,0x03D4},  {0x03A6,0x03D5,0x03A6}, /* 03D4 */
  {0x03A0,0x03D6,0x03A0},  {0x03CF,0x03D7,0x03CF}, /* 03D6 */
  {0x03D8,0x03D9,0x03D8},  {0x03D8,0x03D9,0x03D8}, /* 03D8 */
  {0x03DA,0x03DB,0x03DA},  {0x03DA,0x03DB,0x03DA}, /* 03DA */
  {0x03DC,0x03DD,0x03DC},  {0x03DC,0x03DD,0x03DC}, /* 03DC */
  {0x03DE,0x03DF,0x03DE},  {0x03DE,0x03DF,0x03DE}, /* 03DE */
  {0x03E0,0x03E1,0x03E0},  {0x03E0,0x03E1,0x03E0}, /* 03E0 */
  {0x03E2,0x03E3,0x03E2},  {0x03E2,0x03E3,0x03E2}, /* 03E2 */
  {0x03E4,0x03E5,0x03E4},  {0x03E4,0x03E5,0x03E4}, /* 03E4 */
  {0x03E6,0x03E7,0x03E6},  {0x03E6,0x03E7,0x03E6}, /* 03E6 */
  {0x03E8,0x03E9,0x03E8},  {0x03E8,0x03E9,0x03E8}, /* 03E8 */
  {0x03EA,0x03EB,0x03EA},  {0x03EA,0x03EB,0x03EA}, /* 03EA */
  {0x03EC,0x03ED,0x03EC},  {0x03EC,0x03ED,0x03EC}, /* 03EC */
  {0x03EE,0x03EF,0x03EE},  {0x03EE,0x03EF,0x03EE}, /* 03EE */
  {0x039A,0x03F0,0x039A},  {0x03A1,0x03F1,0x03A1}, /* 03F0 */
  {0x03F9,0x03F2,0x03F9},  {0x037F,0x03F3,0x037F}, /* 03F2 */
  {0x03F4,0x03B8,0x03F4},  {0x0395,0x03F5,0x0395}, /* 03F4 */
  {0x03F6,0x03F6,0x03F6},  {0x03F7,0x03F8,0x03F7}, /* 03F6 */
  {0x03F7,0x03F8,0x03F7},  {0x03F9,0x03F2,0x03F9}, /* 03F8 */
  {0x03FA,0x03FB,0x03FA},  {0x03FA,0x03FB,0x03FA}, /* 03FA */
  {0x03FC,0x03FC,0x03FC},  {0x03FD,0x037B,0x03FD}, /* 03FC */
  {0x03FE,0x037C,0x03FE},  {0x03FF,0x037D,0x03FF}  /* 03FE */
};

static const MY_UNICASE_CHARACTER u900p004[]= {
  {0x0400,0x0450,0x0400},  {0x0401,0x0451,0x0401}, /* 0400 */
  {0x0402,0x0452,0x0402},  {0x0403,0x0453,0x0403}, /* 0402 */
  {0x0404,0x0454,0x0404},  {0x0405,0x0455,0x0405}, /* 0404 */
  {0x0406,0x0456,0x0406},  {0x0407,0x0457,0x0407}, /* 0406 */
  {0x0408,0x0458,0x0408},  {0x0409,0x0459,0x0409}, /* 0408 */
  {0x040A,0x045A,0x040A},  {0x040B,0x045B,0x040B}, /* 040A */
  {0x040C,0x045C,0x040C},  {0x040D,0x045D,0x040D}, /* 040C */
  {0x040E,0x045E,0x040E},  {0x040F,0x045F,0x040F}, /* 040E */
  {0x0410,0x0430,0x0410},  {0x0411,0x0431,0x0411}, /* 0410 */
  {0x0412,0x0432,0x0412},  {0x0413,0x0433,0x0413}, /* 0412 */
  {0x0414,0x0434,0x0414},  {0x0415,0x0435,0x0415}, /* 0414 */
  {0x0416,0x0436,0x0416},  {0x0417,0x0437,0x0417}, /* 0416 */
  {0x0418,0x0438,0x0418},  {0x0419,0x0439,0x0419}, /* 0418 */
  {0x041A,0x043A,0x041A},  {0x041B,0x043B,0x041B}, /* 041A */
  {0x041C,0x043C,0x041C},  {0x041D,0x043D,0x041D}, /* 041C */
  {0x041E,0x043E,0x041E},  {0x041F,0x043F,0x041F}, /* 041E */
  {0x0420,0x0440,0x0420},  {0x0421,0x0441,0x0421}, /* 0420 */
  {0x0422,0x0442,0x0422},  {0x0423,0x0443,0x0423}, /* 0422 */
  {0x0424,0x0444,0x0424},  {0x0425,0x0445,0x0425}, /* 0424 */
  {0x0426,0x0446,0x0426},  {0x0427,0x0447,0x0427}, /* 0426 */
  {0x0428,0x0448,0x0428},  {0x0429,0x0449,0x0429}, /* 0428 */
  {0x042A,0x044A,0x042A},  {0x042B,0x044B,0x042B}, /* 042A */
  {0x042C,0x044C,0x042C},  {0x042D,0x044D,0x042D}, /* 042C */
  {0x042E,0x044E,0x042E},  {0x042F,0x044F,0x042F}, /* 042E */
  {0x0410,0x0430,0x0410},  {0x0411,0x0431,0x0411}, /* 0430 */
  {0x0412,0x0432,0x0412},  {0x0413,0x0433,0x0413}, /* 0432 */
  {0x0414,0x0434,0x0414},  {0x0415,0x0435,0x0415}, /* 0434 */
  {0x0416,0x0436,0x0416},  {0x0417,0x0437,0x0417}, /* 0436 */
  {0x0418,0x0438,0x0418},  {0x0419,0x0439,0x0419}, /* 0438 */
  {0x041A,0x043A,0x041A},  {0x041B,0x043B,0x041B}, /* 043A */
  {0x041C,0x043C,0x041C},  {0x041D,0x043D,0x041D}, /* 043C */
  {0x041E,0x043E,0x041E},  {0x041F,0x043F,0x041F}, /* 043E */
  {0x0420,0x0440,0x0420},  {0x0421,0x0441,0x0421}, /* 0440 */
  {0x0422,0x0442,0x0422},  {0x0423,0x0443,0x0423}, /* 0442 */
  {0x0424,0x0444,0x0424},  {0x0425,0x0445,0x0425}, /* 0444 */
  {0x0426,0x0446,0x0426},  {0x0427,0x0447,0x0427}, /* 0446 */
  {0x0428,0x0448,0x0428},  {0x0429,0x0449,0x0429}, /* 0448 */
  {0x042A,0x044A,0x042A},  {0x042B,0x044B,0x042B}, /* 044A */
  {0x042C,0x044C,0x042C},  {0x042D,0x044D,0x042D}, /* 044C */
  {0x042E,0x044E,0x042E},  {0x042F,0x044F,0x042F}, /* 044E */
  {0x0400,0x0450,0x0400},  {0x0401,0x0451,0x0401}, /* 0450 */
  {0x0402,0x0452,0x0402},  {0x0403,0x0453,0x0403}, /* 0452 */
  {0x0404,0x0454,0x0404},  {0x0405,0x0455,0x0405}, /* 0454 */
  {0x0406,0x0456,0x0406},  {0x0407,0x0457,0x0407}, /* 0456 */
  {0x0408,0x0458,0x0408},  {0x0409,0x0459,0x0409}, /* 0458 */
  {0x040A,0x045A,0x040A},  {0x040B,0x045B,0x040B}, /* 045A */
  {0x040C,0x045C,0x040C},  {0x040D,0x045D,0x040D}, /* 045C */
  {0x040E,0x045E,0x040E},  {0x040F,0x045F,0x040F}, /* 045E */
  {0x0460,0x0461,0x0460},  {0x0460,0x0461,0x0460}, /* 0460 */
  {0x0462,0x0463,0x0462},  {0x0462,0x0463,0x0462}, /* 0462 */
  {0x0464,0x0465,0x0464},  {0x0464,0x0465,0x0464}, /* 0464 */
  {0x0466,0x0467,0x0466},  {0x0466,0x0467,0x0466}, /* 0466 */
  {0x0468,0x0469,0x0468},  {0x0468,0x0469,0x0468}, /* 0468 */
  {0x046A,0x046B,0x046A},  {0x046A,0x046B,0x046A}, /* 046A */
  {0x046C,0x046D,0x046C},  {0x046C,0x046D,0x046C}, /* 046C */
  {0x046E,0x046F,0x046E},  {0x046E,0x046F,0x046E}, /* 046E */
  {0x0470,0x0471,0x0470},  {0x0470,0x0471,0x0470}, /* 0470 */
  {0x0472,0x0473,0x0472},  {0x0472,0x0473,0x0472}, /* 0472 */
  {0x0474,0x0475,0x0474},  {0x0474,0x0475,0x0474}, /* 0474 */
  {0x0476,0x0477,0x0476},  {0x0476,0x0477,0x0476}, /* 0476 */
  {0x0478,0x0479,0x0478},  {0x0478,0x0479,0x0478}, /* 0478 */
  {0x047A,0x047B,0x047A},  {0x047A,0x047B,0x047A}, /* 047A */
  {0x047C,0x047D,0x047C},  {0x047C,0x047D,0x047C}, /* 047C */
  {0x047E,0x047F,0x047E},  {0x047E,0x047F,0x047E}, /* 047E */
  {0x0480,0x0481,0x0480},  {0x0480,0x0481,0x0480}, /* 0480 */
  {0x0482,0x0482,0x0482},  {0x0483,0x0483,0x0483}, /* 0482 */
  {0x0484,0x0484,0x0484},  {0x0485,0x0485,0x0485}, /* 0484 */
  {0x0486,0x0486,0x0486},  {0x0487,0x0487,0x0487}, /* 0486 */
  {0x0488,0x0488,0x0488},  {0x0489,0x0489,0x0489}, /* 0488 */
  {0x048A,0x048B,0x048A},  {0x048A,0x048B,0x048A}, /* 048A */
  {0x048C,0x048D,0x048C},  {0x048C,0x048D,0x048C}, /* 048C */
  {0x048E,0x048F,0x048E},  {0x048E,0x048F,0x048E}, /* 048E */
  {0x0490,0x0491,0x0490},  {0x0490,0x0491,0x0490}, /* 0490 */
  {0x0492,0x0493,0x0492},  {0x0492,0x0493,0x0492}, /* 0492 */
  {0x0494,0x0495,0x0494},  {0x0494,0x0495,0x0494}, /* 0494 */
  {0x0496,0x0497,0x0496},  {0x0496,0x0497,0x0496}, /* 0496 */
  {0x0498,0x0499,0x0498},  {0x0498,0x0499,0x0498}, /* 0498 */
  {0x049A,0x049B,0x049A},  {0x049A,0x049B,0x049A}, /* 049A */
  {0x049C,0x049D,0x049C},  {0x049C,0x049D,0x049C}, /* 049C */
  {0x049E,0x049F,0x049E},  {0x049E,0x049F,0x049E}, /* 049E */
  {0x04A0,0x04A1,0x04A0},  {0x04A0,0x04A1,0x04A0}, /* 04A0 */
  {0x04A2,0x04A3,0x04A2},  {0x04A2,0x04A3,0x04A2}, /* 04A2 */
  {0x04A4,0x04A5,0x04A4},  {0x04A4,0x04A5,0x04A4}, /* 04A4 */
  {0x04A6,0x04A7,0x04A6},  {0x04A6,0x04A7,0x04A6}, /* 04A6 */
  {0x04A8,0x04A9,0x04A8},  {0x04A8,0x04A9,0x04A8}, /* 04A8 */
  {0x04AA,0x04AB,0x04AA},  {0x04AA,0x04AB,0x04AA}, /* 04AA */
  {0x04AC,0x04AD,0x04AC},  {0x04AC,0x04AD,0x04AC}, /* 04AC */
  {0x04AE,0x04AF,0x04AE},  {0x04AE,0x04AF,0x04AE}, /* 04AE */
  {0x04B0,0x04B1,0x04B0},  {0x04B0,0x04B1,0x04B0}, /* 04B0 */
  {0x04B2,0x04B3,0x04B2},  {0x04B2,0x04B3,0x04B2}, /* 04B2 */
  {0x04B4,0x04B5,0x04B4},  {0x04B4,0x04B5,0x04B4}, /* 04B4 */
  {0x04B6,0x04B7,0x04B6},  {0x04B6,0x04B7,0x04B6}, /* 04B6 */
  {0x04B8,0x04B9,0x04B8},  {0x04B8,0x04B9,0x04B8}, /* 04B8 */
  {0x04BA,0x04BB,0x04BA},  {0x04BA,0x04BB,0x04BA}, /* 04BA */
  {0x04BC,0x04BD,0x04BC},  {0x04BC,0x04BD,0x04BC}, /* 04BC */
  {0x04BE,0x04BF,0x04BE},  {0x04BE,0x04BF,0x04BE}, /* 04BE */
  {0x04C0,0x04CF,0x04C0},  {0x04C1,0x04C2,0x04C1}, /* 04C0 */
  {0x04C1,0x04C2,0x04C1},  {0x04C3,0x04C4,0x04C3}, /* 04C2 */
  {0x04C3,0x04C4,0x04C3},  {0x04C5,0x04C6,0x04C5}, /* 04C4 */
  {0x04C5,0x04C6,0x04C5},  {0x04C7,0x04C8,0x04C7}, /* 04C6 */
  {0x04C7,0x04C8,0x04C7},  {0x04C9,0x04CA,0x04C9}, /* 04C8 */
  {0x04C9,0x04CA,0x04C9},  {0x04CB,0x04CC,0x04CB}, /* 04CA */
  {0x04CB,0x04CC,0x04CB},  {0x04CD,0x04CE,0x04CD}, /* 04CC */
  {0x04CD,0x04CE,0x04CD},  {0x04C0,0x04CF,0x04C0}, /* 04CE */
  {0x04D0,0x04D1,0x04D0},  {0x04D0,0x04D1,0x04D0}, /* 04D0 */
  {0x04D2,0x04D3,0x04D2},  {0x04D2,0x04D3,0x04D2}, /* 04D2 */
  {0x04D4,0x04D5,0x04D4},  {0x04D4,0x04D5,0x04D4}, /* 04D4 */
  {0x04D6,0x04D7,0x04D6},  {0x04D6,0x04D7,0x04D6}, /* 04D6 */
  {0x04D8,0x04D9,0x04D8},  {0x04D8,0x04D9,0x04D8}, /* 04D8 */
  {0x04DA,0x04DB,0x04DA},  {0x04DA,0x04DB,0x04DA}, /* 04DA */
  {0x04DC,0x04DD,0x04DC},  {0x04DC,0x04DD,0x04DC}, /* 04DC */
  {0x04DE,0x04DF,0x04DE},  {0x04DE,0x04DF,0x04DE}, /* 04DE */
  {0x04E0,0x04E1,0x04E0},  {0x04E0,0x04E1,0x04E0}, /* 04E0 */
  {0x04E2,0x04E3,0x04E2},  {0x04E2,0x04E3,0x04E2}, /* 04E2 */
  {0x04E4,0x04E5,0x04E4},  {0x04E4,0x04E5,0x04E4}, /* 04E4 */
  {0x04E6,0x04E7,0x04E6},  {0x04E6,0x04E7,0x04E6}, /* 04E6 */
  {0x04E8,0x04E9,0x04E8},  {0x04E8,0x04E9,0x04E8}, /* 04E8 */
  {0x04EA,0x04EB,0x04EA},  {0x04EA,0x04EB,0x04EA}, /* 04EA */
  {0x04EC,0x04ED,0x04EC},  {0x04EC,0x04ED,0x04EC}, /* 04EC */
  {0x04EE,0x04EF,0x04EE},  {0x04EE,0x04EF,0x04EE}, /* 04EE */
  {0x04F0,0x04F1,0x04F0},  {0x04F0,0x04F1,0x04F0}, /* 04F0 */
  {0x04F2,0x04F3,0x04F2},  {0x04F2,0x04F3,0x04F2}, /* 04F2 */
  {0x04F4,0x04F5,0x04F4},  {0x04F4,0x04F5,0x04F4}, /* 04F4 */
  {0x04F6,0x04F7,0x04F6},  {0x04F6,0x04F7,0x04F6}, /* 04F6 */
  {0x04F8,0x04F9,0x04F8},  {0x04F8,0x04F9,0x04F8}, /* 04F8 */
  {0x04FA,0x04FB,0x04FA},  {0x04FA,0x04FB,0x04FA}, /* 04FA */
  {0x04FC,0x04FD,0x04FC},  {0x04FC,0x04FD,0x04FC}, /* 04FC */
  {0x04FE,0x04FF,0x04FE},  {0x04FE,0x04FF,0x04FE}  /* 04FE */
};

static const MY_UNICASE_CHARACTER u900p005[]= {
  {0x0500,0x0501,0x0500},  {0x0500,0x0501,0x0500}, /* 0500 */
  {0x0502,0x0503,0x0502},  {0x0502,0x0503,0x0502}, /* 0502 */
  {0x0504,0x0505,0x0504},  {0x0504,0x0505,0x0504}, /* 0504 */
  {0x0506,0x0507,0x0506},  {0x0506,0x0507,0x0506}, /* 0506 */
  {0x0508,0x0509,0x0508},  {0x0508,0x0509,0x0508}, /* 0508 */
  {0x050A,0x050B,0x050A},  {0x050A,0x050B,0x050A}, /* 050A */
  {0x050C,0x050D,0x050C},  {0x050C,0x050D,0x050C}, /* 050C */
  {0x050E,0x050F,0x050E},  {0x050E,0x050F,0x050E}, /* 050E */
  {0x0510,0x0511,0x0510},  {0x0510,0x0511,0x0510}, /* 0510 */
  {0x0512,0x0513,0x0512},  {0x0512,0x0513,0x0512}, /* 0512 */
  {0x0514,0x0515,0x0514},  {0x0514,0x0515,0x0514}, /* 0514 */
  {0x0516,0x0517,0x0516},  {0x0516,0x0517,0x0516}, /* 0516 */
  {0x0518,0x0519,0x0518},  {0x0518,0x0519,0x0518}, /* 0518 */
  {0x051A,0x051B,0x051A},  {0x051A,0x051B,0x051A}, /* 051A */
  {0x051C,0x051D,0x051C},  {0x051C,0x051D,0x051C}, /* 051C */
  {0x051E,0x051F,0x051E},  {0x051E,0x051F,0x051E}, /* 051E */
  {0x0520,0x0521,0x0520},  {0x0520,0x0521,0x0520}, /* 0520 */
  {0x0522,0x0523,0x0522},  {0x0522,0x0523,0x0522}, /* 0522 */
  {0x0524,0x0525,0x0524},  {0x0524,0x0525,0x0524}, /* 0524 */
  {0x0526,0x0527,0x0526},  {0x0526,0x0527,0x0526}, /* 0526 */
  {0x0528,0x0529,0x0528},  {0x0528,0x0529,0x0528}, /* 0528 */
  {0x052A,0x052B,0x052A},  {0x052A,0x052B,0x052A}, /* 052A */
  {0x052C,0x052D,0x052C},  {0x052C,0x052D,0x052C}, /* 052C */
  {0x052E,0x052F,0x052E},  {0x052E,0x052F,0x052E}, /* 052E */
  {0x0530,0x0530,0x0530},  {0x0531,0x0561,0x0531}, /* 0530 */
  {0x0532,0x0562,0x0532},  {0x0533,0x0563,0x0533}, /* 0532 */
  {0x0534,0x0564,0x0534},  {0x0535,0x0565,0x0535}, /* 0534 */
  {0x0536,0x0566,0x0536},  {0x0537,0x0567,0x0537}, /* 0536 */
  {0x0538,0x0568,0x0538},  {0x0539,0x0569,0x0539}, /* 0538 */
  {0x053A,0x056A,0x053A},  {0x053B,0x056B,0x053B}, /* 053A */
  {0x053C,0x056C,0x053C},  {0x053D,0x056D,0x053D}, /* 053C */
  {0x053E,0x056E,0x053E},  {0x053F,0x056F,0x053F}, /* 053E */
  {0x0540,0x0570,0x0540},  {0x0541,0x0571,0x0541}, /* 0540 */
  {0x0542,0x0572,0x0542},  {0x0543,0x0573,0x0543}, /* 0542 */
  {0x0544,0x0574,0x0544},  {0x0545,0x0575,0x0545}, /* 0544 */
  {0x0546,0x0576,0x0546},  {0x0547,0x0577,0x0547}, /* 0546 */
  {0x0548,0x0578,0x0548},  {0x0549,0x0579,0x0549}, /* 0548 */
  {0x054A,0x057A,0x054A},  {0x054B,0x057B,0x054B}, /* 054A */
  {0x054C,0x057C,0x054C},  {0x054D,0x057D,0x054D}, /* 054C */
  {0x054E,0x057E,0x054E},  {0x054F,0x057F,0x054F}, /* 054E */
  {0x0550,0x0580,0x0550},  {0x0551,0x0581,0x0551}, /* 0550 */
  {0x0552,0x0582,0x0552},  {0x0553,0x0583,0x0553}, /* 0552 */
  {0x0554,0x0584,0x0554},  {0x0555,0x0585,0x0555}, /* 0554 */
  {0x0556,0x0586,0x0556},  {0x0557,0x0557,0x0557}, /* 0556 */
  {0x0558,0x0558,0x0558},  {0x0559,0x0559,0x0559}, /* 0558 */
  {0x055A,0x055A,0x055A},  {0x055B,0x055B,0x055B}, /* 055A */
  {0x055C,0x055C,0x055C},  {0x055D,0x055D,0x055D}, /* 055C */
  {0x055E,0x055E,0x055E},  {0x055F,0x055F,0x055F}, /* 055E */
  {0x0560,0x0560,0x0560},  {0x0531,0x0561,0x0531}, /* 0560 */
  {0x0532,0x0562,0x0532},  {0x0533,0x0563,0x0533}, /* 0562 */
  {0x0534,0x0564,0x0534},  {0x0535,0x0565,0x0535}, /* 0564 */
  {0x0536,0x0566,0x0536},  {0x0537,0x0567,0x0537}, /* 0566 */
  {0x0538,0x0568,0x0538},  {0x0539,0x0569,0x0539}, /* 0568 */
  {0x053A,0x056A,0x053A},  {0x053B,0x056B,0x053B}, /* 056A */
  {0x053C,0x056C,0x053C},  {0x053D,0x056D,0x053D}, /* 056C */
  {0x053E,0x056E,0x053E},  {0x053F,0x056F,0x053F}, /* 056E */
  {0x0540,0x0570,0x0540},  {0x0541,0x0571,0x0541}, /* 0570 */
  {0x0542,0x0572,0x0542},  {0x0543,0x0573,0x0543}, /* 0572 */
  {0x0544,0x0574,0x0544},  {0x0545,0x0575,0x0545}, /* 0574 */
  {0x0546,0x0576,0x0546},  {0x0547,0x0577,0x0547}, /* 0576 */
  {0x0548,0x0578,0x0548},  {0x0549,0x0579,0x0549}, /* 0578 */
  {0x054A,0x057A,0x054A},  {0x054B,0x057B,0x054B}, /* 057A */
  {0x054C,0x057C,0x054C},  {0x054D,0x057D,0x054D}, /* 057C */
  {0x054E,0x057E,0x054E},  {0x054F,0x057F,0x054F}, /* 057E */
  {0x0550,0x0580,0x0550},  {0x0551,0x0581,0x0551}, /* 0580 */
  {0x0552,0x0582,0x0552},  {0x0553,0x0583,0x0553}, /* 0582 */
  {0x0554,0x0584,0x0554},  {0x0555,0x0585,0x0555}, /* 0584 */
  {0x0556,0x0586,0x0556},  {0x0587,0x0587,0x0587}, /* 0586 */
  {0x0588,0x0588,0x0588},  {0x0589,0x0589,0x0589}, /* 0588 */
  {0x058A,0x058A,0x058A},  {0x058B,0x058B,0x058B}, /* 058A */
  {0x058C,0x058C,0x058C},  {0x058D,0x058D,0x058D}, /* 058C */
  {0x058E,0x058E,0x058E},  {0x058F,0x058F,0x058F}, /* 058E */
  {0x0590,0x0590,0x0590},  {0x0591,0x0591,0x0591}, /* 0590 */
  {0x0592,0x0592,0x0592},  {0x0593,0x0593,0x0593}, /* 0592 */
  {0x0594,0x0594,0x0594},  {0x0595,0x0595,0x0595}, /* 0594 */
  {0x0596,0x0596,0x0596},  {0x0597,0x0597,0x0597}, /* 0596 */
  {0x0598,0x0598,0x0598},  {0x0599,0x0599,0x0599}, /* 0598 */
  {0x059A,0x059A,0x059A},  {0x059B,0x059B,0x059B}, /* 059A */
  {0x059C,0x059C,0x059C},  {0x059D,0x059D,0x059D}, /* 059C */
  {0x059E,0x059E,0x059E},  {0x059F,0x059F,0x059F}, /* 059E */
  {0x05A0,0x05A0,0x05A0},  {0x05A1,0x05A1,0x05A1}, /* 05A0 */
  {0x05A2,0x05A2,0x05A2},  {0x05A3,0x05A3,0x05A3}, /* 05A2 */
  {0x05A4,0x05A4,0x05A4},  {0x05A5,0x05A5,0x05A5}, /* 05A4 */
  {0x05A6,0x05A6,0x05A6},  {0x05A7,0x05A7,0x05A7}, /* 05A6 */
  {0x05A8,0x05A8,0x05A8},  {0x05A9,0x05A9,0x05A9}, /* 05A8 */
  {0x05AA,0x05AA,0x05AA},  {0x05AB,0x05AB,0x05AB}, /* 05AA */
  {0x05AC,0x05AC,0x05AC},  {0x05AD,0x05AD,0x05AD}, /* 05AC */
  {0x05AE,0x05AE,0x05AE},  {0x05AF,0x05AF,0x05AF}, /* 05AE */
  {0x05B0,0x05B0,0x05B0},  {0x05B1,0x05B1,0x05B1}, /* 05B0 */
  {0x05B2,0x05B2,0x05B2},  {0x05B3,0x05B3,0x05B3}, /* 05B2 */
  {0x05B4,0x05B4,0x05B4},  {0x05B5,0x05B5,0x05B5}, /* 05B4 */
  {0x05B6,0x05B6,0x05B6},  {0x05B7,0x05B7,0x05B7}, /* 05B6 */
  {0x05B8,0x05B8,0x05B8},  {0x05B9,0x05B9,0x05B9}, /* 05B8 */
  {0x05BA,0x05BA,0x05BA},  {0x05BB,0x05BB,0x05BB}, /* 05BA */
  {0x05BC,0x05BC,0x05BC},  {0x05BD,0x05BD,0x05BD}, /* 05BC */
  {0x05BE,0x05BE,0x05BE},  {0x05BF,0x05BF,0x05BF}, /* 05BE */
  {0x05C0,0x05C0,0x05C0},  {0x05C1,0x05C1,0x05C1}, /* 05C0 */
  {0x05C2,0x05C2,0x05C2},  {0x05C3,0x05C3,0x05C3}, /* 05C2 */
  {0x05C4,0x05C4,0x05C4},  {0x05C5,0x05C5,0x05C5}, /* 05C4 */
  {0x05C6,0x05C6,0x05C6},  {0x05C7,0x05C7,0x05C7}, /* 05C6 */
  {0x05C8,0x05C8,0x05C8},  {0x05C9,0x05C9,0x05C9}, /* 05C8 */
  {0x05CA,0x05CA,0x05CA},  {0x05CB,0x05CB,0x05CB}, /* 05CA */
  {0x05CC,0x05CC,0x05CC},  {0x05CD,0x05CD,0x05CD}, /* 05CC */
  {0x05CE,0x05CE,0x05CE},  {0x05CF,0x05CF,0x05CF}, /* 05CE */
  {0x05D0,0x05D0,0x05D0},  {0x05D1,0x05D1,0x05D1}, /* 05D0 */
  {0x05D2,0x05D2,0x05D2},  {0x05D3,0x05D3,0x05D3}, /* 05D2 */
  {0x05D4,0x05D4,0x05D4},  {0x05D5,0x05D5,0x05D5}, /* 05D4 */
  {0x05D6,0x05D6,0x05D6},  {0x05D7,0x05D7,0x05D7}, /* 05D6 */
  {0x05D8,0x05D8,0x05D8},  {0x05D9,0x05D9,0x05D9}, /* 05D8 */
  {0x05DA,0x05DA,0x05DA},  {0x05DB,0x05DB,0x05DB}, /* 05DA */
  {0x05DC,0x05DC,0x05DC},  {0x05DD,0x05DD,0x05DD}, /* 05DC */
  {0x05DE,0x05DE,0x05DE},  {0x05DF,0x05DF,0x05DF}, /* 05DE */
  {0x05E0,0x05E0,0x05E0},  {0x05E1,0x05E1,0x05E1}, /* 05E0 */
  {0x05E2,0x05E2,0x05E2},  {0x05E3,0x05E3,0x05E3}, /* 05E2 */
  {0x05E4,0x05E4,0x05E4},  {0x05E5,0x05E5,0x05E5}, /* 05E4 */
  {0x05E6,0x05E6,0x05E6},  {0x05E7,0x05E7,0x05E7}, /* 05E6 */
  {0x05E8,0x05E8,0x05E8},  {0x05E9,0x05E9,0x05E9}, /* 05E8 */
  {0x05EA,0x05EA,0x05EA},  {0x05EB,0x05EB,0x05EB}, /* 05EA */
  {0x05EC,0x05EC,0x05EC},  {0x05ED,0x05ED,0x05ED}, /* 05EC */
  {0x05EE,0x05EE,0x05EE},  {0x05EF,0x05EF,0x05EF}, /* 05EE */
  {0x05F0,0x05F0,0x05F0},  {0x05F1,0x05F1,0x05F1}, /* 05F0 */
  {0x05F2,0x05F2,0x05F2},  {0x05F3,0x05F3,0x05F3}, /* 05F2 */
  {0x05F4,0x05F4,0x05F4},  {0x05F5,0x05F5,0x05F5}, /* 05F4 */
  {0x05F6,0x05F6,0x05F6},  {0x05F7,0x05F7,0x05F7}, /* 05F6 */
  {0x05F8,0x05F8,0x05F8},  {0x05F9,0x05F9,0x05F9}, /* 05F8 */
  {0x05FA,0x05FA,0x05FA},  {0x05FB,0x05FB,0x05FB}, /* 05FA */
  {0x05FC,0x05FC,0x05FC},  {0x05FD,0x05FD,0x05FD}, /* 05FC */
  {0x05FE,0x05FE,0x05FE},  {0x05FF,0x05FF,0x05FF}  /* 05FE */
};

static const MY_UNICASE_CHARACTER u900p010[]= {
  {0x1000,0x1000,0x1000},  {0x1001,0x1001,0x1001}, /* 1000 */
  {0x1002,0x1002,0x1002},  {0x1003,0x1003,0x1003}, /* 1002 */
  {0x1004,0x1004,0x1004},  {0x1005,0x1005,0x1005}, /* 1004 */
  {0x1006,0x1006,0x1006},  {0x1007,0x1007,0x1007}, /* 1006 */
  {0x1008,0x1008,0x1008},  {0x1009,0x1009,0x1009}, /* 1008 */
  {0x100A,0x100A,0x100A},  {0x100B,0x100B,0x100B}, /* 100A */
  {0x100C,0x100C,0x100C},  {0x100D,0x100D,0x100D}, /* 100C */
  {0x100E,0x100E,0x100E},  {0x100F,0x100F,0x100F}, /* 100E */
  {0x1010,0x1010,0x1010},  {0x1011,0x1011,0x1011}, /* 1010 */
  {0x1012,0x1012,0x1012},  {0x1013,0x1013,0x1013}, /* 1012 */
  {0x1014,0x1014,0x1014},  {0x1015,0x1015,0x1015}, /* 1014 */
  {0x1016,0x1016,0x1016},  {0x1017,0x1017,0x1017}, /* 1016 */
  {0x1018,0x1018,0x1018},  {0x1019,0x1019,0x1019}, /* 1018 */
  {0x101A,0x101A,0x101A},  {0x101B,0x101B,0x101B}, /* 101A */
  {0x101C,0x101C,0x101C},  {0x101D,0x101D,0x101D}, /* 101C */
  {0x101E,0x101E,0x101E},  {0x101F,0x101F,0x101F}, /* 101E */
  {0x1020,0x1020,0x1020},  {0x1021,0x1021,0x1021}, /* 1020 */
  {0x1022,0x1022,0x1022},  {0x1023,0x1023,0x1023}, /* 1022 */
  {0x1024,0x1024,0x1024},  {0x1025,0x1025,0x1025}, /* 1024 */
  {0x1026,0x1026,0x1026},  {0x1027,0x1027,0x1027}, /* 1026 */
  {0x1028,0x1028,0x1028},  {0x1029,0x1029,0x1029}, /* 1028 */
  {0x102A,0x102A,0x102A},  {0x102B,0x102B,0x102B}, /* 102A */
  {0x102C,0x102C,0x102C},  {0x102D,0x102D,0x102D}, /* 102C */
  {0x102E,0x102E,0x102E},  {0x102F,0x102F,0x102F}, /* 102E */
  {0x1030,0x1030,0x1030},  {0x1031,0x1031,0x1031}, /* 1030 */
  {0x1032,0x1032,0x1032},  {0x1033,0x1033,0x1033}, /* 1032 */
  {0x1034,0x1034,0x1034},  {0x1035,0x1035,0x1035}, /* 1034 */
  {0x1036,0x1036,0x1036},  {0x1037,0x1037,0x1037}, /* 1036 */
  {0x1038,0x1038,0x1038},  {0x1039,0x1039,0x1039}, /* 1038 */
  {0x103A,0x103A,0x103A},  {0x103B,0x103B,0x103B}, /* 103A */
  {0x103C,0x103C,0x103C},  {0x103D,0x103D,0x103D}, /* 103C */
  {0x103E,0x103E,0x103E},  {0x103F,0x103F,0x103F}, /* 103E */
  {0x1040,0x1040,0x1040},  {0x1041,0x1041,0x1041}, /* 1040 */
  {0x1042,0x1042,0x1042},  {0x1043,0x1043,0x1043}, /* 1042 */
  {0x1044,0x1044,0x1044},  {0x1045,0x1045,0x1045}, /* 1044 */
  {0x1046,0x1046,0x1046},  {0x1047,0x1047,0x1047}, /* 1046 */
  {0x1048,0x1048,0x1048},  {0x1049,0x1049,0x1049}, /* 1048 */
  {0x104A,0x104A,0x104A},  {0x104B,0x104B,0x104B}, /* 104A */
  {0x104C,0x104C,0x104C},  {0x104D,0x104D,0x104D}, /* 104C */
  {0x104E,0x104E,0x104E},  {0x104F,0x104F,0x104F}, /* 104E */
  {0x1050,0x1050,0x1050},  {0x1051,0x1051,0x1051}, /* 1050 */
  {0x1052,0x1052,0x1052},  {0x1053,0x1053,0x1053}, /* 1052 */
  {0x1054,0x1054,0x1054},  {0x1055,0x1055,0x1055}, /* 1054 */
  {0x1056,0x1056,0x1056},  {0x1057,0x1057,0x1057}, /* 1056 */
  {0x1058,0x1058,0x1058},  {0x1059,0x1059,0x1059}, /* 1058 */
  {0x105A,0x105A,0x105A},  {0x105B,0x105B,0x105B}, /* 105A */
  {0x105C,0x105C,0x105C},  {0x105D,0x105D,0x105D}, /* 105C */
  {0x105E,0x105E,0x105E},  {0x105F,0x105F,0x105F}, /* 105E */
  {0x1060,0x1060,0x1060},  {0x1061,0x1061,0x1061}, /* 1060 */
  {0x1062,0x1062,0x1062},  {0x1063,0x1063,0x1063}, /* 1062 */
  {0x1064,0x1064,0x1064},  {0x1065,0x1065,0x1065}, /* 1064 */
  {0x1066,0x1066,0x1066},  {0x1067,0x1067,0x1067}, /* 1066 */
  {0x1068,0x1068,0x1068},  {0x1069,0x1069,0x1069}, /* 1068 */
  {0x106A,0x106A,0x106A},  {0x106B,0x106B,0x106B}, /* 106A */
  {0x106C,0x106C,0x106C},  {0x106D,0x106D,0x106D}, /* 106C */
  {0x106E,0x106E,0x106E},  {0x106F,0x106F,0x106F}, /* 106E */
  {0x1070,0x1070,0x1070},  {0x1071,0x1071,0x1071}, /* 1070 */
  {0x1072,0x1072,0x1072},  {0x1073,0x1073,0x1073}, /* 1072 */
  {0x1074,0x1074,0x1074},  {0x1075,0x1075,0x1075}, /* 1074 */
  {0x1076,0x1076,0x1076},  {0x1077,0x1077,0x1077}, /* 1076 */
  {0x1078,0x1078,0x1078},  {0x1079,0x1079,0x1079}, /* 1078 */
  {0x107A,0x107A,0x107A},  {0x107B,0x107B,0x107B}, /* 107A */
  {0x107C,0x107C,0x107C},  {0x107D,0x107D,0x107D}, /* 107C */
  {0x107E,0x107E,0x107E},  {0x107F,0x107F,0x107F}, /* 107E */
  {0x1080,0x1080,0x1080},  {0x1081,0x1081,0x1081}, /* 1080 */
  {0x1082,0x1082,0x1082},  {0x1083,0x1083,0x1083}, /* 1082 */
  {0x1084,0x1084,0x1084},  {0x1085,0x1085,0x1085}, /* 1084 */
  {0x1086,0x1086,0x1086},  {0x1087,0x1087,0x1087}, /* 1086 */
  {0x1088,0x1088,0x1088},  {0x1089,0x1089,0x1089}, /* 1088 */
  {0x108A,0x108A,0x108A},  {0x108B,0x108B,0x108B}, /* 108A */
  {0x108C,0x108C,0x108C},  {0x108D,0x108D,0x108D}, /* 108C */
  {0x108E,0x108E,0x108E},  {0x108F,0x108F,0x108F}, /* 108E */
  {0x1090,0x1090,0x1090},  {0x1091,0x1091,0x1091}, /* 1090 */
  {0x1092,0x1092,0x1092},  {0x1093,0x1093,0x1093}, /* 1092 */
  {0x1094,0x1094,0x1094},  {0x1095,0x1095,0x1095}, /* 1094 */
  {0x1096,0x1096,0x1096},  {0x1097,0x1097,0x1097}, /* 1096 */
  {0x1098,0x1098,0x1098},  {0x1099,0x1099,0x1099}, /* 1098 */
  {0x109A,0x109A,0x109A},  {0x109B,0x109B,0x109B}, /* 109A */
  {0x109C,0x109C,0x109C},  {0x109D,0x109D,0x109D}, /* 109C */
  {0x109E,0x109E,0x109E},  {0x109F,0x109F,0x109F}, /* 109E */
  {0x10A0,0x2D00,0x10A0},  {0x10A1,0x2D01,0x10A1}, /* 10A0 */
  {0x10A2,0x2D02,0x10A2},  {0x10A3,0x2D03,0x10A3}, /* 10A2 */
  {0x10A4,0x2D04,0x10A4},  {0x10A5,0x2D05,0x10A5}, /* 10A4 */
  {0x10A6,0x2D06,0x10A6},  {0x10A7,0x2D07,0x10A7}, /* 10A6 */
  {0x10A8,0x2D08,0x10A8},  {0x10A9,0x2D09,0x10A9}, /* 10A8 */
  {0x10AA,0x2D0A,0x10AA},  {0x10AB,0x2D0B,0x10AB}, /* 10AA */
  {0x10AC,0x2D0C,0x10AC},  {0x10AD,0x2D0D,0x10AD}, /* 10AC */
  {0x10AE,0x2D0E,0x10AE},  {0x10AF,0x2D0F,0x10AF}, /* 10AE */
  {0x10B0,0x2D10,0x10B0},  {0x10B1,0x2D11,0x10B1}, /* 10B0 */
  {0x10B2,0x2D12,0x10B2},  {0x10B3,0x2D13,0x10B3}, /* 10B2 */
  {0x10B4,0x2D14,0x10B4},  {0x10B5,0x2D15,0x10B5}, /* 10B4 */
  {0x10B6,0x2D16,0x10B6},  {0x10B7,0x2D17,0x10B7}, /* 10B6 */
  {0x10B8,0x2D18,0x10B8},  {0x10B9,0x2D19,0x10B9}, /* 10B8 */
  {0x10BA,0x2D1A,0x10BA},  {0x10BB,0x2D1B,0x10BB}, /* 10BA */
  {0x10BC,0x2D1C,0x10BC},  {0x10BD,0x2D1D,0x10BD}, /* 10BC */
  {0x10BE,0x2D1E,0x10BE},  {0x10BF,0x2D1F,0x10BF}, /* 10BE */
  {0x10C0,0x2D20,0x10C0},  {0x10C1,0x2D21,0x10C1}, /* 10C0 */
  {0x10C2,0x2D22,0x10C2},  {0x10C3,0x2D23,0x10C3}, /* 10C2 */
  {0x10C4,0x2D24,0x10C4},  {0x10C5,0x2D25,0x10C5}, /* 10C4 */
  {0x10C6,0x10C6,0x10C6},  {0x10C7,0x2D27,0x10C7}, /* 10C6 */
  {0x10C8,0x10C8,0x10C8},  {0x10C9,0x10C9,0x10C9}, /* 10C8 */
  {0x10CA,0x10CA,0x10CA},  {0x10CB,0x10CB,0x10CB}, /* 10CA */
  {0x10CC,0x10CC,0x10CC},  {0x10CD,0x2D2D,0x10CD}, /* 10CC */
  {0x10CE,0x10CE,0x10CE},  {0x10CF,0x10CF,0x10CF}, /* 10CE */
  {0x10D0,0x10D0,0x10D0},  {0x10D1,0x10D1,0x10D1}, /* 10D0 */
  {0x10D2,0x10D2,0x10D2},  {0x10D3,0x10D3,0x10D3}, /* 10D2 */
  {0x10D4,0x10D4,0x10D4},  {0x10D5,0x10D5,0x10D5}, /* 10D4 */
  {0x10D6,0x10D6,0x10D6},  {0x10D7,0x10D7,0x10D7}, /* 10D6 */
  {0x10D8,0x10D8,0x10D8},  {0x10D9,0x10D9,0x10D9}, /* 10D8 */
  {0x10DA,0x10DA,0x10DA},  {0x10DB,0x10DB,0x10DB}, /* 10DA */
  {0x10DC,0x10DC,0x10DC},  {0x10DD,0x10DD,0x10DD}, /* 10DC */
  {0x10DE,0x10DE,0x10DE},  {0x10DF,0x10DF,0x10DF}, /* 10DE */
  {0x10E0,0x10E0,0x10E0},  {0x10E1,0x10E1,0x10E1}, /* 10E0 */
  {0x10E2,0x10E2,0x10E2},  {0x10E3,0x10E3,0x10E3}, /* 10E2 */
  {0x10E4,0x10E4,0x10E4},  {0x10E5,0x10E5,0x10E5}, /* 10E4 */
  {0x10E6,0x10E6,0x10E6},  {0x10E7,0x10E7,0x10E7}, /* 10E6 */
  {0x10E8,0x10E8,0x10E8},  {0x10E9,0x10E9,0x10E9}, /* 10E8 */
  {0x10EA,0x10EA,0x10EA},  {0x10EB,0x10EB,0x10EB}, /* 10EA */
  {0x10EC,0x10EC,0x10EC},  {0x10ED,0x10ED,0x10ED}, /* 10EC */
  {0x10EE,0x10EE,0x10EE},  {0x10EF,0x10EF,0x10EF}, /* 10EE */
  {0x10F0,0x10F0,0x10F0},  {0x10F1,0x10F1,0x10F1}, /* 10F0 */
  {0x10F2,0x10F2,0x10F2},  {0x10F3,0x10F3,0x10F3}, /* 10F2 */
  {0x10F4,0x10F4,0x10F4},  {0x10F5,0x10F5,0x10F5}, /* 10F4 */
  {0x10F6,0x10F6,0x10F6},  {0x10F7,0x10F7,0x10F7}, /* 10F6 */
  {0x10F8,0x10F8,0x10F8},  {0x10F9,0x10F9,0x10F9}, /* 10F8 */
  {0x10FA,0x10FA,0x10FA},  {0x10FB,0x10FB,0x10FB}, /* 10FA */
  {0x10FC,0x10FC,0x10FC},  {0x10FD,0x10FD,0x10FD}, /* 10FC */
  {0x10FE,0x10FE,0x10FE},  {0x10FF,0x10FF,0x10FF}  /* 10FE */
};

static const MY_UNICASE_CHARACTER u900p013[]= {
  {0x1300,0x1300,0x1300},  {0x1301,0x1301,0x1301}, /* 1300 */
  {0x1302,0x1302,0x1302},  {0x1303,0x1303,0x1303}, /* 1302 */
  {0x1304,0x1304,0x1304},  {0x1305,0x1305,0x1305}, /* 1304 */
  {0x1306,0x1306,0x1306},  {0x1307,0x1307,0x1307}, /* 1306 */
  {0x1308,0x1308,0x1308},  {0x1309,0x1309,0x1309}, /* 1308 */
  {0x130A,0x130A,0x130A},  {0x130B,0x130B,0x130B}, /* 130A */
  {0x130C,0x130C,0x130C},  {0x130D,0x130D,0x130D}, /* 130C */
  {0x130E,0x130E,0x130E},  {0x130F,0x130F,0x130F}, /* 130E */
  {0x1310,0x1310,0x1310},  {0x1311,0x1311,0x1311}, /* 1310 */
  {0x1312,0x1312,0x1312},  {0x1313,0x1313,0x1313}, /* 1312 */
  {0x1314,0x1314,0x1314},  {0x1315,0x1315,0x1315}, /* 1314 */
  {0x1316,0x1316,0x1316},  {0x1317,0x1317,0x1317}, /* 1316 */
  {0x1318,0x1318,0x1318},  {0x1319,0x1319,0x1319}, /* 1318 */
  {0x131A,0x131A,0x131A},  {0x131B,0x131B,0x131B}, /* 131A */
  {0x131C,0x131C,0x131C},  {0x131D,0x131D,0x131D}, /* 131C */
  {0x131E,0x131E,0x131E},  {0x131F,0x131F,0x131F}, /* 131E */
  {0x1320,0x1320,0x1320},  {0x1321,0x1321,0x1321}, /* 1320 */
  {0x1322,0x1322,0x1322},  {0x1323,0x1323,0x1323}, /* 1322 */
  {0x1324,0x1324,0x1324},  {0x1325,0x1325,0x1325}, /* 1324 */
  {0x1326,0x1326,0x1326},  {0x1327,0x1327,0x1327}, /* 1326 */
  {0x1328,0x1328,0x1328},  {0x1329,0x1329,0x1329}, /* 1328 */
  {0x132A,0x132A,0x132A},  {0x132B,0x132B,0x132B}, /* 132A */
  {0x132C,0x132C,0x132C},  {0x132D,0x132D,0x132D}, /* 132C */
  {0x132E,0x132E,0x132E},  {0x132F,0x132F,0x132F}, /* 132E */
  {0x1330,0x1330,0x1330},  {0x1331,0x1331,0x1331}, /* 1330 */
  {0x1332,0x1332,0x1332},  {0x1333,0x1333,0x1333}, /* 1332 */
  {0x1334,0x1334,0x1334},  {0x1335,0x1335,0x1335}, /* 1334 */
  {0x1336,0x1336,0x1336},  {0x1337,0x1337,0x1337}, /* 1336 */
  {0x1338,0x1338,0x1338},  {0x1339,0x1339,0x1339}, /* 1338 */
  {0x133A,0x133A,0x133A},  {0x133B,0x133B,0x133B}, /* 133A */
  {0x133C,0x133C,0x133C},  {0x133D,0x133D,0x133D}, /* 133C */
  {0x133E,0x133E,0x133E},  {0x133F,0x133F,0x133F}, /* 133E */
  {0x1340,0x1340,0x1340},  {0x1341,0x1341,0x1341}, /* 1340 */
  {0x1342,0x1342,0x1342},  {0x1343,0x1343,0x1343}, /* 1342 */
  {0x1344,0x1344,0x1344},  {0x1345,0x1345,0x1345}, /* 1344 */
  {0x1346,0x1346,0x1346},  {0x1347,0x1347,0x1347}, /* 1346 */
  {0x1348,0x1348,0x1348},  {0x1349,0x1349,0x1349}, /* 1348 */
  {0x134A,0x134A,0x134A},  {0x134B,0x134B,0x134B}, /* 134A */
  {0x134C,0x134C,0x134C},  {0x134D,0x134D,0x134D}, /* 134C */
  {0x134E,0x134E,0x134E},  {0x134F,0x134F,0x134F}, /* 134E */
  {0x1350,0x1350,0x1350},  {0x1351,0x1351,0x1351}, /* 1350 */
  {0x1352,0x1352,0x1352},  {0x1353,0x1353,0x1353}, /* 1352 */
  {0x1354,0x1354,0x1354},  {0x1355,0x1355,0x1355}, /* 1354 */
  {0x1356,0x1356,0x1356},  {0x1357,0x1357,0x1357}, /* 1356 */
  {0x1358,0x1358,0x1358},  {0x1359,0x1359,0x1359}, /* 1358 */
  {0x135A,0x135A,0x135A},  {0x135B,0x135B,0x135B}, /* 135A */
  {0x135C,0x135C,0x135C},  {0x135D,0x135D,0x135D}, /* 135C */
  {0x135E,0x135E,0x135E},  {0x135F,0x135F,0x135F}, /* 135E */
  {0x1360,0x1360,0x1360},  {0x1361,0x1361,0x1361}, /* 1360 */
  {0x1362,0x1362,0x1362},  {0x1363,0x1363,0x1363}, /* 1362 */
  {0x1364,0x1364,0x1364},  {0x1365,0x1365,0x1365}, /* 1364 */
  {0x1366,0x1366,0x1366},  {0x1367,0x1367,0x1367}, /* 1366 */
  {0x1368,0x1368,0x1368},  {0x1369,0x1369,0x1369}, /* 1368 */
  {0x136A,0x136A,0x136A},  {0x136B,0x136B,0x136B}, /* 136A */
  {0x136C,0x136C,0x136C},  {0x136D,0x136D,0x136D}, /* 136C */
  {0x136E,0x136E,0x136E},  {0x136F,0x136F,0x136F}, /* 136E */
  {0x1370,0x1370,0x1370},  {0x1371,0x1371,0x1371}, /* 1370 */
  {0x1372,0x1372,0x1372},  {0x1373,0x1373,0x1373}, /* 1372 */
  {0x1374,0x1374,0x1374},  {0x1375,0x1375,0x1375}, /* 1374 */
  {0x1376,0x1376,0x1376},  {0x1377,0x1377,0x1377}, /* 1376 */
  {0x1378,0x1378,0x1378},  {0x1379,0x1379,0x1379}, /* 1378 */
  {0x137A,0x137A,0x137A},  {0x137B,0x137B,0x137B}, /* 137A */
  {0x137C,0x137C,0x137C},  {0x137D,0x137D,0x137D}, /* 137C */
  {0x137E,0x137E,0x137E},  {0x137F,0x137F,0x137F}, /* 137E */
  {0x1380,0x1380,0x1380},  {0x1381,0x1381,0x1381}, /* 1380 */
  {0x1382,0x1382,0x1382},  {0x1383,0x1383,0x1383}, /* 1382 */
  {0x1384,0x1384,0x1384},  {0x1385,0x1385,0x1385}, /* 1384 */
  {0x1386,0x1386,0x1386},  {0x1387,0x1387,0x1387}, /* 1386 */
  {0x1388,0x1388,0x1388},  {0x1389,0x1389,0x1389}, /* 1388 */
  {0x138A,0x138A,0x138A},  {0x138B,0x138B,0x138B}, /* 138A */
  {0x138C,0x138C,0x138C},  {0x138D,0x138D,0x138D}, /* 138C */
  {0x138E,0x138E,0x138E},  {0x138F,0x138F,0x138F}, /* 138E */
  {0x1390,0x1390,0x1390},  {0x1391,0x1391,0x1391}, /* 1390 */
  {0x1392,0x1392,0x1392},  {0x1393,0x1393,0x1393}, /* 1392 */
  {0x1394,0x1394,0x1394},  {0x1395,0x1395,0x1395}, /* 1394 */
  {0x1396,0x1396,0x1396},  {0x1397,0x1397,0x1397}, /* 1396 */
  {0x1398,0x1398,0x1398},  {0x1399,0x1399,0x1399}, /* 1398 */
  {0x139A,0x139A,0x139A},  {0x139B,0x139B,0x139B}, /* 139A */
  {0x139C,0x139C,0x139C},  {0x139D,0x139D,0x139D}, /* 139C */
  {0x139E,0x139E,0x139E},  {0x139F,0x139F,0x139F}, /* 139E */
  {0x13A0,0xAB70,0x13A0},  {0x13A1,0xAB71,0x13A1}, /* 13A0 */
  {0x13A2,0xAB72,0x13A2},  {0x13A3,0xAB73,0x13A3}, /* 13A2 */
  {0x13A4,0xAB74,0x13A4},  {0x13A5,0xAB75,0x13A5}, /* 13A4 */
  {0x13A6,0xAB76,0x13A6},  {0x13A7,0xAB77,0x13A7}, /* 13A6 */
  {0x13A8,0xAB78,0x13A8},  {0x13A9,0xAB79,0x13A9}, /* 13A8 */
  {0x13AA,0xAB7A,0x13AA},  {0x13AB,0xAB7B,0x13AB}, /* 13AA */
  {0x13AC,0xAB7C,0x13AC},  {0x13AD,0xAB7D,0x13AD}, /* 13AC */
  {0x13AE,0xAB7E,0x13AE},  {0x13AF,0xAB7F,0x13AF}, /* 13AE */
  {0x13B0,0xAB80,0x13B0},  {0x13B1,0xAB81,0x13B1}, /* 13B0 */
  {0x13B2,0xAB82,0x13B2},  {0x13B3,0xAB83,0x13B3}, /* 13B2 */
  {0x13B4,0xAB84,0x13B4},  {0x13B5,0xAB85,0x13B5}, /* 13B4 */
  {0x13B6,0xAB86,0x13B6},  {0x13B7,0xAB87,0x13B7}, /* 13B6 */
  {0x13B8,0xAB88,0x13B8},  {0x13B9,0xAB89,0x13B9}, /* 13B8 */
  {0x13BA,0xAB8A,0x13BA},  {0x13BB,0xAB8B,0x13BB}, /* 13BA */
  {0x13BC,0xAB8C,0x13BC},  {0x13BD,0xAB8D,0x13BD}, /* 13BC */
  {0x13BE,0xAB8E,0x13BE},  {0x13BF,0xAB8F,0x13BF}, /* 13BE */
  {0x13C0,0xAB90,0x13C0},  {0x13C1,0xAB91,0x13C1}, /* 13C0 */
  {0x13C2,0xAB92,0x13C2},  {0x13C3,0xAB93,0x13C3}, /* 13C2 */
  {0x13C4,0xAB94,0x13C4},  {0x13C5,0xAB95,0x13C5}, /* 13C4 */
  {0x13C6,0xAB96,0x13C6},  {0x13C7,0xAB97,0x13C7}, /* 13C6 */
  {0x13C8,0xAB98,0x13C8},  {0x13C9,0xAB99,0x13C9}, /* 13C8 */
  {0x13CA,0xAB9A,0x13CA},  {0x13CB,0xAB9B,0x13CB}, /* 13CA */
  {0x13CC,0xAB9C,0x13CC},  {0x13CD,0xAB9D,0x13CD}, /* 13CC */
  {0x13CE,0xAB9E,0x13CE},  {0x13CF,0xAB9F,0x13CF}, /* 13CE */
  {0x13D0,0xABA0,0x13D0},  {0x13D1,0xABA1,0x13D1}, /* 13D0 */
  {0x13D2,0xABA2,0x13D2},  {0x13D3,0xABA3,0x13D3}, /* 13D2 */
  {0x13D4,0xABA4,0x13D4},  {0x13D5,0xABA5,0x13D5}, /* 13D4 */
  {0x13D6,0xABA6,0x13D6},  {0x13D7,0xABA7,0x13D7}, /* 13D6 */
  {0x13D8,0xABA8,0x13D8},  {0x13D9,0xABA9,0x13D9}, /* 13D8 */
  {0x13DA,0xABAA,0x13DA},  {0x13DB,0xABAB,0x13DB}, /* 13DA */
  {0x13DC,0xABAC,0x13DC},  {0x13DD,0xABAD,0x13DD}, /* 13DC */
  {0x13DE,0xABAE,0x13DE},  {0x13DF,0xABAF,0x13DF}, /* 13DE */
  {0x13E0,0xABB0,0x13E0},  {0x13E1,0xABB1,0x13E1}, /* 13E0 */
  {0x13E2,0xABB2,0x13E2},  {0x13E3,0xABB3,0x13E3}, /* 13E2 */
  {0x13E4,0xABB4,0x13E4},  {0x13E5,0xABB5,0x13E5}, /* 13E4 */
  {0x13E6,0xABB6,0x13E6},  {0x13E7,0xABB7,0x13E7}, /* 13E6 */
  {0x13E8,0xABB8,0x13E8},  {0x13E9,0xABB9,0x13E9}, /* 13E8 */
  {0x13EA,0xABBA,0x13EA},  {0x13EB,0xABBB,0x13EB}, /* 13EA */
  {0x13EC,0xABBC,0x13EC},  {0x13ED,0xABBD,0x13ED}, /* 13EC */
  {0x13EE,0xABBE,0x13EE},  {0x13EF,0xABBF,0x13EF}, /* 13EE */
  {0x13F0,0x13F8,0x13F0},  {0x13F1,0x13F9,0x13F1}, /* 13F0 */
  {0x13F2,0x13FA,0x13F2},  {0x13F3,0x13FB,0x13F3}, /* 13F2 */
  {0x13F4,0x13FC,0x13F4},  {0x13F5,0x13FD,0x13F5}, /* 13F4 */
  {0x13F6,0x13F6,0x13F6},  {0x13F7,0x13F7,0x13F7}, /* 13F6 */
  {0x13F0,0x13F8,0x13F0},  {0x13F1,0x13F9,0x13F1}, /* 13F8 */
  {0x13F2,0x13FA,0x13F2},  {0x13F3,0x13FB,0x13F3}, /* 13FA */
  {0x13F4,0x13FC,0x13F4},  {0x13F5,0x13FD,0x13F5}, /* 13FC */
  {0x13FE,0x13FE,0x13FE},  {0x13FF,0x13FF,0x13FF}  /* 13FE */
};

static const MY_UNICASE_CHARACTER u900p01C[]= {
  {0x1C00,0x1C00,0x1C00},  {0x1C01,0x1C01,0x1C01}, /* 1C00 */
  {0x1C02,0x1C02,0x1C02},  {0x1C03,0x1C03,0x1C03}, /* 1C02 */
  {0x1C04,0x1C04,0x1C04},  {0x1C05,0x1C05,0x1C05}, /* 1C04 */
  {0x1C06,0x1C06,0x1C06},  {0x1C07,0x1C07,0x1C07}, /* 1C06 */
  {0x1C08,0x1C08,0x1C08},  {0x1C09,0x1C09,0x1C09}, /* 1C08 */
  {0x1C0A,0x1C0A,0x1C0A},  {0x1C0B,0x1C0B,0x1C0B}, /* 1C0A */
  {0x1C0C,0x1C0C,0x1C0C},  {0x1C0D,0x1C0D,0x1C0D}, /* 1C0C */
  {0x1C0E,0x1C0E,0x1C0E},  {0x1C0F,0x1C0F,0x1C0F}, /* 1C0E */
  {0x1C10,0x1C10,0x1C10},  {0x1C11,0x1C11,0x1C11}, /* 1C10 */
  {0x1C12,0x1C12,0x1C12},  {0x1C13,0x1C13,0x1C13}, /* 1C12 */
  {0x1C14,0x1C14,0x1C14},  {0x1C15,0x1C15,0x1C15}, /* 1C14 */
  {0x1C16,0x1C16,0x1C16},  {0x1C17,0x1C17,0x1C17}, /* 1C16 */
  {0x1C18,0x1C18,0x1C18},  {0x1C19,0x1C19,0x1C19}, /* 1C18 */
  {0x1C1A,0x1C1A,0x1C1A},  {0x1C1B,0x1C1B,0x1C1B}, /* 1C1A */
  {0x1C1C,0x1C1C,0x1C1C},  {0x1C1D,0x1C1D,0x1C1D}, /* 1C1C */
  {0x1C1E,0x1C1E,0x1C1E},  {0x1C1F,0x1C1F,0x1C1F}, /* 1C1E */
  {0x1C20,0x1C20,0x1C20},  {0x1C21,0x1C21,0x1C21}, /* 1C20 */
  {0x1C22,0x1C22,0x1C22},  {0x1C23,0x1C23,0x1C23}, /* 1C22 */
  {0x1C24,0x1C24,0x1C24},  {0x1C25,0x1C25,0x1C25}, /* 1C24 */
  {0x1C26,0x1C26,0x1C26},  {0x1C27,0x1C27,0x1C27}, /* 1C26 */
  {0x1C28,0x1C28,0x1C28},  {0x1C29,0x1C29,0x1C29}, /* 1C28 */
  {0x1C2A,0x1C2A,0x1C2A},  {0x1C2B,0x1C2B,0x1C2B}, /* 1C2A */
  {0x1C2C,0x1C2C,0x1C2C},  {0x1C2D,0x1C2D,0x1C2D}, /* 1C2C */
  {0x1C2E,0x1C2E,0x1C2E},  {0x1C2F,0x1C2F,0x1C2F}, /* 1C2E */
  {0x1C30,0x1C30,0x1C30},  {0x1C31,0x1C31,0x1C31}, /* 1C30 */
  {0x1C32,0x1C32,0x1C32},  {0x1C33,0x1C33,0x1C33}, /* 1C32 */
  {0x1C34,0x1C34,0x1C34},  {0x1C35,0x1C35,0x1C35}, /* 1C34 */
  {0x1C36,0x1C36,0x1C36},  {0x1C37,0x1C37,0x1C37}, /* 1C36 */
  {0x1C38,0x1C38,0x1C38},  {0x1C39,0x1C39,0x1C39}, /* 1C38 */
  {0x1C3A,0x1C3A,0x1C3A},  {0x1C3B,0x1C3B,0x1C3B}, /* 1C3A */
  {0x1C3C,0x1C3C,0x1C3C},  {0x1C3D,0x1C3D,0x1C3D}, /* 1C3C */
  {0x1C3E,0x1C3E,0x1C3E},  {0x1C3F,0x1C3F,0x1C3F}, /* 1C3E */
  {0x1C40,0x1C40,0x1C40},  {0x1C41,0x1C41,0x1C41}, /* 1C40 */
  {0x1C42,0x1C42,0x1C42},  {0x1C43,0x1C43,0x1C43}, /* 1C42 */
  {0x1C44,0x1C44,0x1C44},  {0x1C45,0x1C45,0x1C45}, /* 1C44 */
  {0x1C46,0x1C46,0x1C46},  {0x1C47,0x1C47,0x1C47}, /* 1C46 */
  {0x1C48,0x1C48,0x1C48},  {0x1C49,0x1C49,0x1C49}, /* 1C48 */
  {0x1C4A,0x1C4A,0x1C4A},  {0x1C4B,0x1C4B,0x1C4B}, /* 1C4A */
  {0x1C4C,0x1C4C,0x1C4C},  {0x1C4D,0x1C4D,0x1C4D}, /* 1C4C */
  {0x1C4E,0x1C4E,0x1C4E},  {0x1C4F,0x1C4F,0x1C4F}, /* 1C4E */
  {0x1C50,0x1C50,0x1C50},  {0x1C51,0x1C51,0x1C51}, /* 1C50 */
  {0x1C52,0x1C52,0x1C52},  {0x1C53,0x1C53,0x1C53}, /* 1C52 */
  {0x1C54,0x1C54,0x1C54},  {0x1C55,0x1C55,0x1C55}, /* 1C54 */
  {0x1C56,0x1C56,0x1C56},  {0x1C57,0x1C57,0x1C57}, /* 1C56 */
  {0x1C58,0x1C58,0x1C58},  {0x1C59,0x1C59,0x1C59}, /* 1C58 */
  {0x1C5A,0x1C5A,0x1C5A},  {0x1C5B,0x1C5B,0x1C5B}, /* 1C5A */
  {0x1C5C,0x1C5C,0x1C5C},  {0x1C5D,0x1C5D,0x1C5D}, /* 1C5C */
  {0x1C5E,0x1C5E,0x1C5E},  {0x1C5F,0x1C5F,0x1C5F}, /* 1C5E */
  {0x1C60,0x1C60,0x1C60},  {0x1C61,0x1C61,0x1C61}, /* 1C60 */
  {0x1C62,0x1C62,0x1C62},  {0x1C63,0x1C63,0x1C63}, /* 1C62 */
  {0x1C64,0x1C64,0x1C64},  {0x1C65,0x1C65,0x1C65}, /* 1C64 */
  {0x1C66,0x1C66,0x1C66},  {0x1C67,0x1C67,0x1C67}, /* 1C66 */
  {0x1C68,0x1C68,0x1C68},  {0x1C69,0x1C69,0x1C69}, /* 1C68 */
  {0x1C6A,0x1C6A,0x1C6A},  {0x1C6B,0x1C6B,0x1C6B}, /* 1C6A */
  {0x1C6C,0x1C6C,0x1C6C},  {0x1C6D,0x1C6D,0x1C6D}, /* 1C6C */
  {0x1C6E,0x1C6E,0x1C6E},  {0x1C6F,0x1C6F,0x1C6F}, /* 1C6E */
  {0x1C70,0x1C70,0x1C70},  {0x1C71,0x1C71,0x1C71}, /* 1C70 */
  {0x1C72,0x1C72,0x1C72},  {0x1C73,0x1C73,0x1C73}, /* 1C72 */
  {0x1C74,0x1C74,0x1C74},  {0x1C75,0x1C75,0x1C75}, /* 1C74 */
  {0x1C76,0x1C76,0x1C76},  {0x1C77,0x1C77,0x1C77}, /* 1C76 */
  {0x1C78,0x1C78,0x1C78},  {0x1C79,0x1C79,0x1C79}, /* 1C78 */
  {0x1C7A,0x1C7A,0x1C7A},  {0x1C7B,0x1C7B,0x1C7B}, /* 1C7A */
  {0x1C7C,0x1C7C,0x1C7C},  {0x1C7D,0x1C7D,0x1C7D}, /* 1C7C */
  {0x1C7E,0x1C7E,0x1C7E},  {0x1C7F,0x1C7F,0x1C7F}, /* 1C7E */
  {0x0412,0x1C80,0x0412},  {0x0414,0x1C81,0x0414}, /* 1C80 */
  {0x041E,0x1C82,0x041E},  {0x0421,0x1C83,0x0421}, /* 1C82 */
  {0x0422,0x1C84,0x0422},  {0x0422,0x1C85,0x0422}, /* 1C84 */
  {0x042A,0x1C86,0x042A},  {0x0462,0x1C87,0x0462}, /* 1C86 */
  {0xA64A,0x1C88,0xA64A},  {0x1C89,0x1C89,0x1C89}, /* 1C88 */
  {0x1C8A,0x1C8A,0x1C8A},  {0x1C8B,0x1C8B,0x1C8B}, /* 1C8A */
  {0x1C8C,0x1C8C,0x1C8C},  {0x1C8D,0x1C8D,0x1C8D}, /* 1C8C */
  {0x1C8E,0x1C8E,0x1C8E},  {0x1C8F,0x1C8F,0x1C8F}, /* 1C8E */
  {0x1C90,0x1C90,0x1C90},  {0x1C91,0x1C91,0x1C91}, /* 1C90 */
  {0x1C92,0x1C92,0x1C92},  {0x1C93,0x1C93,0x1C93}, /* 1C92 */
  {0x1C94,0x1C94,0x1C94},  {0x1C95,0x1C95,0x1C95}, /* 1C94 */
  {0x1C96,0x1C96,0x1C96},  {0x1C97,0x1C97,0x1C97}, /* 1C96 */
  {0x1C98,0x1C98,0x1C98},  {0x1C99,0x1C99,0x1C99}, /* 1C98 */
  {0x1C9A,0x1C9A,0x1C9A},  {0x1C9B,0x1C9B,0x1C9B}, /* 1C9A */
  {0x1C9C,0x1C9C,0x1C9C},  {0x1C9D,0x1C9D,0x1C9D}, /* 1C9C */
  {0x1C9E,0x1C9E,0x1C9E},  {0x1C9F,0x1C9F,0x1C9F}, /* 1C9E */
  {0x1CA0,0x1CA0,0x1CA0},  {0x1CA1,0x1CA1,0x1CA1}, /* 1CA0 */
  {0x1CA2,0x1CA2,0x1CA2},  {0x1CA3,0x1CA3,0x1CA3}, /* 1CA2 */
  {0x1CA4,0x1CA4,0x1CA4},  {0x1CA5,0x1CA5,0x1CA5}, /* 1CA4 */
  {0x1CA6,0x1CA6,0x1CA6},  {0x1CA7,0x1CA7,0x1CA7}, /* 1CA6 */
  {0x1CA8,0x1CA8,0x1CA8},  {0x1CA9,0x1CA9,0x1CA9}, /* 1CA8 */
  {0x1CAA,0x1CAA,0x1CAA},  {0x1CAB,0x1CAB,0x1CAB}, /* 1CAA */
  {0x1CAC,0x1CAC,0x1CAC},  {0x1CAD,0x1CAD,0x1CAD}, /* 1CAC */
  {0x1CAE,0x1CAE,0x1CAE},  {0x1CAF,0x1CAF,0x1CAF}, /* 1CAE */
  {0x1CB0,0x1CB0,0x1CB0},  {0x1CB1,0x1CB1,0x1CB1}, /* 1CB0 */
  {0x1CB2,0x1CB2,0x1CB2},  {0x1CB3,0x1CB3,0x1CB3}, /* 1CB2 */
  {0x1CB4,0x1CB4,0x1CB4},  {0x1CB5,0x1CB5,0x1CB5}, /* 1CB4 */
  {0x1CB6,0x1CB6,0x1CB6},  {0x1CB7,0x1CB7,0x1CB7}, /* 1CB6 */
  {0x1CB8,0x1CB8,0x1CB8},  {0x1CB9,0x1CB9,0x1CB9}, /* 1CB8 */
  {0x1CBA,0x1CBA,0x1CBA},  {0x1CBB,0x1CBB,0x1CBB}, /* 1CBA */
  {0x1CBC,0x1CBC,0x1CBC},  {0x1CBD,0x1CBD,0x1CBD}, /* 1CBC */
  {0x1CBE,0x1CBE,0x1CBE},  {0x1CBF,0x1CBF,0x1CBF}, /* 1CBE */
  {0x1CC0,0x1CC0,0x1CC0},  {0x1CC1,0x1CC1,0x1CC1}, /* 1CC0 */
  {0x1CC2,0x1CC2,0x1CC2},  {0x1CC3,0x1CC3,0x1CC3}, /* 1CC2 */
  {0x1CC4,0x1CC4,0x1CC4},  {0x1CC5,0x1CC5,0x1CC5}, /* 1CC4 */
  {0x1CC6,0x1CC6,0x1CC6},  {0x1CC7,0x1CC7,0x1CC7}, /* 1CC6 */
  {0x1CC8,0x1CC8,0x1CC8},  {0x1CC9,0x1CC9,0x1CC9}, /* 1CC8 */
  {0x1CCA,0x1CCA,0x1CCA},  {0x1CCB,0x1CCB,0x1CCB}, /* 1CCA */
  {0x1CCC,0x1CCC,0x1CCC},  {0x1CCD,0x1CCD,0x1CCD}, /* 1CCC */
  {0x1CCE,0x1CCE,0x1CCE},  {0x1CCF,0x1CCF,0x1CCF}, /* 1CCE */
  {0x1CD0,0x1CD0,0x1CD0},  {0x1CD1,0x1CD1,0x1CD1}, /* 1CD0 */
  {0x1CD2,0x1CD2,0x1CD2},  {0x1CD3,0x1CD3,0x1CD3}, /* 1CD2 */
  {0x1CD4,0x1CD4,0x1CD4},  {0x1CD5,0x1CD5,0x1CD5}, /* 1CD4 */
  {0x1CD6,0x1CD6,0x1CD6},  {0x1CD7,0x1CD7,0x1CD7}, /* 1CD6 */
  {0x1CD8,0x1CD8,0x1CD8},  {0x1CD9,0x1CD9,0x1CD9}, /* 1CD8 */
  {0x1CDA,0x1CDA,0x1CDA},  {0x1CDB,0x1CDB,0x1CDB}, /* 1CDA */
  {0x1CDC,0x1CDC,0x1CDC},  {0x1CDD,0x1CDD,0x1CDD}, /* 1CDC */
  {0x1CDE,0x1CDE,0x1CDE},  {0x1CDF,0x1CDF,0x1CDF}, /* 1CDE */
  {0x1CE0,0x1CE0,0x1CE0},  {0x1CE1,0x1CE1,0x1CE1}, /* 1CE0 */
  {0x1CE2,0x1CE2,0x1CE2},  {0x1CE3,0x1CE3,0x1CE3}, /* 1CE2 */
  {0x1CE4,0x1CE4,0x1CE4},  {0x1CE5,0x1CE5,0x1CE5}, /* 1CE4 */
  {0x1CE6,0x1CE6,0x1CE6},  {0x1CE7,0x1CE7,0x1CE7}, /* 1CE6 */
  {0x1CE8,0x1CE8,0x1CE8},  {0x1CE9,0x1CE9,0x1CE9}, /* 1CE8 */
  {0x1CEA,0x1CEA,0x1CEA},  {0x1CEB,0x1CEB,0x1CEB}, /* 1CEA */
  {0x1CEC,0x1CEC,0x1CEC},  {0x1CED,0x1CED,0x1CED}, /* 1CEC */
  {0x1CEE,0x1CEE,0x1CEE},  {0x1CEF,0x1CEF,0x1CEF}, /* 1CEE */
  {0x1CF0,0x1CF0,0x1CF0},  {0x1CF1,0x1CF1,0x1CF1}, /* 1CF0 */
  {0x1CF2,0x1CF2,0x1CF2},  {0x1CF3,0x1CF3,0x1CF3}, /* 1CF2 */
  {0x1CF4,0x1CF4,0x1CF4},  {0x1CF5,0x1CF5,0x1CF5}, /* 1CF4 */
  {0x1CF6,0x1CF6,0x1CF6},  {0x1CF7,0x1CF7,0x1CF7}, /* 1CF6 */
  {0x1CF8,0x1CF8,0x1CF8},  {0x1CF9,0x1CF9,0x1CF9}, /* 1CF8 */
  {0x1CFA,0x1CFA,0x1CFA},  {0x1CFB,0x1CFB,0x1CFB}, /* 1CFA */
  {0x1CFC,0x1CFC,0x1CFC},  {0x1CFD,0x1CFD,0x1CFD}, /* 1CFC */
  {0x1CFE,0x1CFE,0x1CFE},  {0x1CFF,0x1CFF,0x1CFF}  /* 1CFE */
};

static const MY_UNICASE_CHARACTER u900p01D[]= {
  {0x1D00,0x1D00,0x1D00},  {0x1D01,0x1D01,0x1D01}, /* 1D00 */
  {0x1D02,0x1D02,0x1D02},  {0x1D03,0x1D03,0x1D03}, /* 1D02 */
  {0x1D04,0x1D04,0x1D04},  {0x1D05,0x1D05,0x1D05}, /* 1D04 */
  {0x1D06,0x1D06,0x1D06},  {0x1D07,0x1D07,0x1D07}, /* 1D06 */
  {0x1D08,0x1D08,0x1D08},  {0x1D09,0x1D09,0x1D09}, /* 1D08 */
  {0x1D0A,0x1D0A,0x1D0A},  {0x1D0B,0x1D0B,0x1D0B}, /* 1D0A */
  {0x1D0C,0x1D0C,0x1D0C},  {0x1D0D,0x1D0D,0x1D0D}, /* 1D0C */
  {0x1D0E,0x1D0E,0x1D0E},  {0x1D0F,0x1D0F,0x1D0F}, /* 1D0E */
  {0x1D10,0x1D10,0x1D10},  {0x1D11,0x1D11,0x1D11}, /* 1D10 */
  {0x1D12,0x1D12,0x1D12},  {0x1D13,0x1D13,0x1D13}, /* 1D12 */
  {0x1D14,0x1D14,0x1D14},  {0x1D15,0x1D15,0x1D15}, /* 1D14 */
  {0x1D16,0x1D16,0x1D16},  {0x1D17,0x1D17,0x1D17}, /* 1D16 */
  {0x1D18,0x1D18,0x1D18},  {0x1D19,0x1D19,0x1D19}, /* 1D18 */
  {0x1D1A,0x1D1A,0x1D1A},  {0x1D1B,0x1D1B,0x1D1B}, /* 1D1A */
  {0x1D1C,0x1D1C,0x1D1C},  {0x1D1D,0x1D1D,0x1D1D}, /* 1D1C */
  {0x1D1E,0x1D1E,0x1D1E},  {0x1D1F,0x1D1F,0x1D1F}, /* 1D1E */
  {0x1D20,0x1D20,0x1D20},  {0x1D21,0x1D21,0x1D21}, /* 1D20 */
  {0x1D22,0x1D22,0x1D22},  {0x1D23,0x1D23,0x1D23}, /* 1D22 */
  {0x1D24,0x1D24,0x1D24},  {0x1D25,0x1D25,0x1D25}, /* 1D24 */
  {0x1D26,0x1D26,0x1D26},  {0x1D27,0x1D27,0x1D27}, /* 1D26 */
  {0x1D28,0x1D28,0x1D28},  {0x1D29,0x1D29,0x1D29}, /* 1D28 */
  {0x1D2A,0x1D2A,0x1D2A},  {0x1D2B,0x1D2B,0x1D2B}, /* 1D2A */
  {0x1D2C,0x1D2C,0x1D2C},  {0x1D2D,0x1D2D,0x1D2D}, /* 1D2C */
  {0x1D2E,0x1D2E,0x1D2E},  {0x1D2F,0x1D2F,0x1D2F}, /* 1D2E */
  {0x1D30,0x1D30,0x1D30},  {0x1D31,0x1D31,0x1D31}, /* 1D30 */
  {0x1D32,0x1D32,0x1D32},  {0x1D33,0x1D33,0x1D33}, /* 1D32 */
  {0x1D34,0x1D34,0x1D34},  {0x1D35,0x1D35,0x1D35}, /* 1D34 */
  {0x1D36,0x1D36,0x1D36},  {0x1D37,0x1D37,0x1D37}, /* 1D36 */
  {0x1D38,0x1D38,0x1D38},  {0x1D39,0x1D39,0x1D39}, /* 1D38 */
  {0x1D3A,0x1D3A,0x1D3A},  {0x1D3B,0x1D3B,0x1D3B}, /* 1D3A */
  {0x1D3C,0x1D3C,0x1D3C},  {0x1D3D,0x1D3D,0x1D3D}, /* 1D3C */
  {0x1D3E,0x1D3E,0x1D3E},  {0x1D3F,0x1D3F,0x1D3F}, /* 1D3E */
  {0x1D40,0x1D40,0x1D40},  {0x1D41,0x1D41,0x1D41}, /* 1D40 */
  {0x1D42,0x1D42,0x1D42},  {0x1D43,0x1D43,0x1D43}, /* 1D42 */
  {0x1D44,0x1D44,0x1D44},  {0x1D45,0x1D45,0x1D45}, /* 1D44 */
  {0x1D46,0x1D46,0x1D46},  {0x1D47,0x1D47,0x1D47}, /* 1D46 */
  {0x1D48,0x1D48,0x1D48},  {0x1D49,0x1D49,0x1D49}, /* 1D48 */
  {0x1D4A,0x1D4A,0x1D4A},  {0x1D4B,0x1D4B,0x1D4B}, /* 1D4A */
  {0x1D4C,0x1D4C,0x1D4C},  {0x1D4D,0x1D4D,0x1D4D}, /* 1D4C */
  {0x1D4E,0x1D4E,0x1D4E},  {0x1D4F,0x1D4F,0x1D4F}, /* 1D4E */
  {0x1D50,0x1D50,0x1D50},  {0x1D51,0x1D51,0x1D51}, /* 1D50 */
  {0x1D52,0x1D52,0x1D52},  {0x1D53,0x1D53,0x1D53}, /* 1D52 */
  {0x1D54,0x1D54,0x1D54},  {0x1D55,0x1D55,0x1D55}, /* 1D54 */
  {0x1D56,0x1D56,0x1D56},  {0x1D57,0x1D57,0x1D57}, /* 1D56 */
  {0x1D58,0x1D58,0x1D58},  {0x1D59,0x1D59,0x1D59}, /* 1D58 */
  {0x1D5A,0x1D5A,0x1D5A},  {0x1D5B,0x1D5B,0x1D5B}, /* 1D5A */
  {0x1D5C,0x1D5C,0x1D5C},  {0x1D5D,0x1D5D,0x1D5D}, /* 1D5C */
  {0x1D5E,0x1D5E,0x1D5E},  {0x1D5F,0x1D5F,0x1D5F}, /* 1D5E */
  {0x1D60,0x1D60,0x1D60},  {0x1D61,0x1D61,0x1D61}, /* 1D60 */
  {0x1D62,0x1D62,0x1D62},  {0x1D63,0x1D63,0x1D63}, /* 1D62 */
  {0x1D64,0x1D64,0x1D64},  {0x1D65,0x1D65,0x1D65}, /* 1D64 */
  {0x1D66,0x1D66,0x1D66},  {0x1D67,0x1D67,0x1D67}, /* 1D66 */
  {0x1D68,0x1D68,0x1D68},  {0x1D69,0x1D69,0x1D69}, /* 1D68 */
  {0x1D6A,0x1D6A,0x1D6A},  {0x1D6B,0x1D6B,0x1D6B}, /* 1D6A */
  {0x1D6C,0x1D6C,0x1D6C},  {0x1D6D,0x1D6D,0x1D6D}, /* 1D6C */
  {0x1D6E,0x1D6E,0x1D6E},  {0x1D6F,0x1D6F,0x1D6F}, /* 1D6E */
  {0x1D70,0x1D70,0x1D70},  {0x1D71,0x1D71,0x1D71}, /* 1D70 */
  {0x1D72,0x1D72,0x1D72},  {0x1D73,0x1D73,0x1D73}, /* 1D72 */
  {0x1D74,0x1D74,0x1D74},  {0x1D75,0x1D75,0x1D75}, /* 1D74 */
  {0x1D76,0x1D76,0x1D76},  {0x1D77,0x1D77,0x1D77}, /* 1D76 */
  {0x1D78,0x1D78,0x1D78},  {0xA77D,0x1D79,0xA77D}, /* 1D78 */
  {0x1D7A,0x1D7A,0x1D7A},  {0x1D7B,0x1D7B,0x1D7B}, /* 1D7A */
  {0x1D7C,0x1D7C,0x1D7C},  {0x2C63,0x1D7D,0x2C63}, /* 1D7C */
  {0x1D7E,0x1D7E,0x1D7E},  {0x1D7F,0x1D7F,0x1D7F}, /* 1D7E */
  {0x1D80,0x1D80,0x1D80},  {0x1D81,0x1D81,0x1D81}, /* 1D80 */
  {0x1D82,0x1D82,0x1D82},  {0x1D83,0x1D83,0x1D83}, /* 1D82 */
  {0x1D84,0x1D84,0x1D84},  {0x1D85,0x1D85,0x1D85}, /* 1D84 */
  {0x1D86,0x1D86,0x1D86},  {0x1D87,0x1D87,0x1D87}, /* 1D86 */
  {0x1D88,0x1D88,0x1D88},  {0x1D89,0x1D89,0x1D89}, /* 1D88 */
  {0x1D8A,0x1D8A,0x1D8A},  {0x1D8B,0x1D8B,0x1D8B}, /* 1D8A */
  {0x1D8C,0x1D8C,0x1D8C},  {0x1D8D,0x1D8D,0x1D8D}, /* 1D8C */
  {0x1D8E,0x1D8E,0x1D8E},  {0x1D8F,0x1D8F,0x1D8F}, /* 1D8E */
  {0x1D90,0x1D90,0x1D90},  {0x1D91,0x1D91,0x1D91}, /* 1D90 */
  {0x1D92,0x1D92,0x1D92},  {0x1D93,0x1D93,0x1D93}, /* 1D92 */
  {0x1D94,0x1D94,0x1D94},  {0x1D95,0x1D95,0x1D95}, /* 1D94 */
  {0x1D96,0x1D96,0x1D96},  {0x1D97,0x1D97,0x1D97}, /* 1D96 */
  {0x1D98,0x1D98,0x1D98},  {0x1D99,0x1D99,0x1D99}, /* 1D98 */
  {0x1D9A,0x1D9A,0x1D9A},  {0x1D9B,0x1D9B,0x1D9B}, /* 1D9A */
  {0x1D9C,0x1D9C,0x1D9C},  {0x1D9D,0x1D9D,0x1D9D}, /* 1D9C */
  {0x1D9E,0x1D9E,0x1D9E},  {0x1D9F,0x1D9F,0x1D9F}, /* 1D9E */
  {0x1DA0,0x1DA0,0x1DA0},  {0x1DA1,0x1DA1,0x1DA1}, /* 1DA0 */
  {0x1DA2,0x1DA2,0x1DA2},  {0x1DA3,0x1DA3,0x1DA3}, /* 1DA2 */
  {0x1DA4,0x1DA4,0x1DA4},  {0x1DA5,0x1DA5,0x1DA5}, /* 1DA4 */
  {0x1DA6,0x1DA6,0x1DA6},  {0x1DA7,0x1DA7,0x1DA7}, /* 1DA6 */
  {0x1DA8,0x1DA8,0x1DA8},  {0x1DA9,0x1DA9,0x1DA9}, /* 1DA8 */
  {0x1DAA,0x1DAA,0x1DAA},  {0x1DAB,0x1DAB,0x1DAB}, /* 1DAA */
  {0x1DAC,0x1DAC,0x1DAC},  {0x1DAD,0x1DAD,0x1DAD}, /* 1DAC */
  {0x1DAE,0x1DAE,0x1DAE},  {0x1DAF,0x1DAF,0x1DAF}, /* 1DAE */
  {0x1DB0,0x1DB0,0x1DB0},  {0x1DB1,0x1DB1,0x1DB1}, /* 1DB0 */
  {0x1DB2,0x1DB2,0x1DB2},  {0x1DB3,0x1DB3,0x1DB3}, /* 1DB2 */
  {0x1DB4,0x1DB4,0x1DB4},  {0x1DB5,0x1DB5,0x1DB5}, /* 1DB4 */
  {0x1DB6,0x1DB6,0x1DB6},  {0x1DB7,0x1DB7,0x1DB7}, /* 1DB6 */
  {0x1DB8,0x1DB8,0x1DB8},  {0x1DB9,0x1DB9,0x1DB9}, /* 1DB8 */
  {0x1DBA,0x1DBA,0x1DBA},  {0x1DBB,0x1DBB,0x1DBB}, /* 1DBA */
  {0x1DBC,0x1DBC,0x1DBC},  {0x1DBD,0x1DBD,0x1DBD}, /* 1DBC */
  {0x1DBE,0x1DBE,0x1DBE},  {0x1DBF,0x1DBF,0x1DBF}, /* 1DBE */
  {0x1DC0,0x1DC0,0x1DC0},  {0x1DC1,0x1DC1,0x1DC1}, /* 1DC0 */
  {0x1DC2,0x1DC2,0x1DC2},  {0x1DC3,0x1DC3,0x1DC3}, /* 1DC2 */
  {0x1DC4,0x1DC4,0x1DC4},  {0x1DC5,0x1DC5,0x1DC5}, /* 1DC4 */
  {0x1DC6,0x1DC6,0x1DC6},  {0x1DC7,0x1DC7,0x1DC7}, /* 1DC6 */
  {0x1DC8,0x1DC8,0x1DC8},  {0x1DC9,0x1DC9,0x1DC9}, /* 1DC8 */
  {0x1DCA,0x1DCA,0x1DCA},  {0x1DCB,0x1DCB,0x1DCB}, /* 1DCA */
  {0x1DCC,0x1DCC,0x1DCC},  {0x1DCD,0x1DCD,0x1DCD}, /* 1DCC */
  {0x1DCE,0x1DCE,0x1DCE},  {0x1DCF,0x1DCF,0x1DCF}, /* 1DCE */
  {0x1DD0,0x1DD0,0x1DD0},  {0x1DD1,0x1DD1,0x1DD1}, /* 1DD0 */
  {0x1DD2,0x1DD2,0x1DD2},  {0x1DD3,0x1DD3,0x1DD3}, /* 1DD2 */
  {0x1DD4,0x1DD4,0x1DD4},  {0x1DD5,0x1DD5,0x1DD5}, /* 1DD4 */
  {0x1DD6,0x1DD6,0x1DD6},  {0x1DD7,0x1DD7,0x1DD7}, /* 1DD6 */
  {0x1DD8,0x1DD8,0x1DD8},  {0x1DD9,0x1DD9,0x1DD9}, /* 1DD8 */
  {0x1DDA,0x1DDA,0x1DDA},  {0x1DDB,0x1DDB,0x1DDB}, /* 1DDA */
  {0x1DDC,0x1DDC,0x1DDC},  {0x1DDD,0x1DDD,0x1DDD}, /* 1DDC */
  {0x1DDE,0x1DDE,0x1DDE},  {0x1DDF,0x1DDF,0x1DDF}, /* 1DDE */
  {0x1DE0,0x1DE0,0x1DE0},  {0x1DE1,0x1DE1,0x1DE1}, /* 1DE0 */
  {0x1DE2,0x1DE2,0x1DE2},  {0x1DE3,0x1DE3,0x1DE3}, /* 1DE2 */
  {0x1DE4,0x1DE4,0x1DE4},  {0x1DE5,0x1DE5,0x1DE5}, /* 1DE4 */
  {0x1DE6,0x1DE6,0x1DE6},  {0x1DE7,0x1DE7,0x1DE7}, /* 1DE6 */
  {0x1DE8,0x1DE8,0x1DE8},  {0x1DE9,0x1DE9,0x1DE9}, /* 1DE8 */
  {0x1DEA,0x1DEA,0x1DEA},  {0x1DEB,0x1DEB,0x1DEB}, /* 1DEA */
  {0x1DEC,0x1DEC,0x1DEC},  {0x1DED,0x1DED,0x1DED}, /* 1DEC */
  {0x1DEE,0x1DEE,0x1DEE},  {0x1DEF,0x1DEF,0x1DEF}, /* 1DEE */
  {0x1DF0,0x1DF0,0x1DF0},  {0x1DF1,0x1DF1,0x1DF1}, /* 1DF0 */
  {0x1DF2,0x1DF2,0x1DF2},  {0x1DF3,0x1DF3,0x1DF3}, /* 1DF2 */
  {0x1DF4,0x1DF4,0x1DF4},  {0x1DF5,0x1DF5,0x1DF5}, /* 1DF4 */
  {0x1DF6,0x1DF6,0x1DF6},  {0x1DF7,0x1DF7,0x1DF7}, /* 1DF6 */
  {0x1DF8,0x1DF8,0x1DF8},  {0x1DF9,0x1DF9,0x1DF9}, /* 1DF8 */
  {0x1DFA,0x1DFA,0x1DFA},  {0x1DFB,0x1DFB,0x1DFB}, /* 1DFA */
  {0x1DFC,0x1DFC,0x1DFC},  {0x1DFD,0x1DFD,0x1DFD}, /* 1DFC */
  {0x1DFE,0x1DFE,0x1DFE},  {0x1DFF,0x1DFF,0x1DFF}  /* 1DFE */
};

static const MY_UNICASE_CHARACTER u900p01E[]= {
  {0x1E00,0x1E01,0x1E00},  {0x1E00,0x1E01,0x1E00}, /* 1E00 */
  {0x1E02,0x1E03,0x1E02},  {0x1E02,0x1E03,0x1E02}, /* 1E02 */
  {0x1E04,0x1E05,0x1E04},  {0x1E04,0x1E05,0x1E04}, /* 1E04 */
  {0x1E06,0x1E07,0x1E06},  {0x1E06,0x1E07,0x1E06}, /* 1E06 */
  {0x1E08,0x1E09,0x1E08},  {0x1E08,0x1E09,0x1E08}, /* 1E08 */
  {0x1E0A,0x1E0B,0x1E0A},  {0x1E0A,0x1E0B,0x1E0A}, /* 1E0A */
  {0x1E0C,0x1E0D,0x1E0C},  {0x1E0C,0x1E0D,0x1E0C}, /* 1E0C */
  {0x1E0E,0x1E0F,0x1E0E},  {0x1E0E,0x1E0F,0x1E0E}, /* 1E0E */
  {0x1E10,0x1E11,0x1E10},  {0x1E10,0x1E11,0x1E10}, /* 1E10 */
  {0x1E12,0x1E13,0x1E12},  {0x1E12,0x1E13,0x1E12}, /* 1E12 */
  {0x1E14,0x1E15,0x1E14},  {0x1E14,0x1E15,0x1E14}, /* 1E14 */
  {0x1E16,0x1E17,0x1E16},  {0x1E16,0x1E17,0x1E16}, /* 1E16 */
  {0x1E18,0x1E19,0x1E18},  {0x1E18,0x1E19,0x1E18}, /* 1E18 */
  {0x1E1A,0x1E1B,0x1E1A},  {0x1E1A,0x1E1B,0x1E1A}, /* 1E1A */
  {0x1E1C,0x1E1D,0x1E1C},  {0x1E1C,0x1E1D,0x1E1C}, /* 1E1C */
  {0x1E1E,0x1E1F,0x1E1E},  {0x1E1E,0x1E1F,0x1E1E}, /* 1E1E */
  {0x1E20,0x1E21,0x1E20},  {0x1E20,0x1E21,0x1E20}, /* 1E20 */
  {0x1E22,0x1E23,0x1E22},  {0x1E22,0x1E23,0x1E22}, /* 1E22 */
  {0x1E24,0x1E25,0x1E24},  {0x1E24,0x1E25,0x1E24}, /* 1E24 */
  {0x1E26,0x1E27,0x1E26},  {0x1E26,0x1E27,0x1E26}, /* 1E26 */
  {0x1E28,0x1E29,0x1E28},  {0x1E28,0x1E29,0x1E28}, /* 1E28 */
  {0x1E2A,0x1E2B,0x1E2A},  {0x1E2A,0x1E2B,0x1E2A}, /* 1E2A */
  {0x1E2C,0x1E2D,0x1E2C},  {0x1E2C,0x1E2D,0x1E2C}, /* 1E2C */
  {0x1E2E,0x1E2F,0x1E2E},  {0x1E2E,0x1E2F,0x1E2E}, /* 1E2E */
  {0x1E30,0x1E31,0x1E30},  {0x1E30,0x1E31,0x1E30}, /* 1E30 */
  {0x1E32,0x1E33,0x1E32},  {0x1E32,0x1E33,0x1E32}, /* 1E32 */
  {0x1E34,0x1E35,0x1E34},  {0x1E34,0x1E35,0x1E34}, /* 1E34 */
  {0x1E36,0x1E37,0x1E36},  {0x1E36,0x1E37,0x1E36}, /* 1E36 */
  {0x1E38,0x1E39,0x1E38},  {0x1E38,0x1E39,0x1E38}, /* 1E38 */
  {0x1E3A,0x1E3B,0x1E3A},  {0x1E3A,0x1E3B,0x1E3A}, /* 1E3A */
  {0x1E3C,0x1E3D,0x1E3C},  {0x1E3C,0x1E3D,0x1E3C}, /* 1E3C */
  {0x1E3E,0x1E3F,0x1E3E},  {0x1E3E,0x1E3F,0x1E3E}, /* 1E3E */
  {0x1E40,0x1E41,0x1E40},  {0x1E40,0x1E41,0x1E40}, /* 1E40 */
  {0x1E42,0x1E43,0x1E42},  {0x1E42,0x1E43,0x1E42}, /* 1E42 */
  {0x1E44,0x1E45,0x1E44},  {0x1E44,0x1E45,0x1E44}, /* 1E44 */
  {0x1E46,0x1E47,0x1E46},  {0x1E46,0x1E47,0x1E46}, /* 1E46 */
  {0x1E48,0x1E49,0x1E48},  {0x1E48,0x1E49,0x1E48}, /* 1E48 */
  {0x1E4A,0x1E4B,0x1E4A},  {0x1E4A,0x1E4B,0x1E4A}, /* 1E4A */
  {0x1E4C,0x1E4D,0x1E4C},  {0x1E4C,0x1E4D,0x1E4C}, /* 1E4C */
  {0x1E4E,0x1E4F,0x1E4E},  {0x1E4E,0x1E4F,0x1E4E}, /* 1E4E */
  {0x1E50,0x1E51,0x1E50},  {0x1E50,0x1E51,0x1E50}, /* 1E50 */
  {0x1E52,0x1E53,0x1E52},  {0x1E52,0x1E53,0x1E52}, /* 1E52 */
  {0x1E54,0x1E55,0x1E54},  {0x1E54,0x1E55,0x1E54}, /* 1E54 */
  {0x1E56,0x1E57,0x1E56},  {0x1E56,0x1E57,0x1E56}, /* 1E56 */
  {0x1E58,0x1E59,0x1E58},  {0x1E58,0x1E59,0x1E58}, /* 1E58 */
  {0x1E5A,0x1E5B,0x1E5A},  {0x1E5A,0x1E5B,0x1E5A}, /* 1E5A */
  {0x1E5C,0x1E5D,0x1E5C},  {0x1E5C,0x1E5D,0x1E5C}, /* 1E5C */
  {0x1E5E,0x1E5F,0x1E5E},  {0x1E5E,0x1E5F,0x1E5E}, /* 1E5E */
  {0x1E60,0x1E61,0x1E60},  {0x1E60,0x1E61,0x1E60}, /* 1E60 */
  {0x1E62,0x1E63,0x1E62},  {0x1E62,0x1E63,0x1E62}, /* 1E62 */
  {0x1E64,0x1E65,0x1E64},  {0x1E64,0x1E65,0x1E64}, /* 1E64 */
  {0x1E66,0x1E67,0x1E66},  {0x1E66,0x1E67,0x1E66}, /* 1E66 */
  {0x1E68,0x1E69,0x1E68},  {0x1E68,0x1E69,0x1E68}, /* 1E68 */
  {0x1E6A,0x1E6B,0x1E6A},  {0x1E6A,0x1E6B,0x1E6A}, /* 1E6A */
  {0x1E6C,0x1E6D,0x1E6C},  {0x1E6C,0x1E6D,0x1E6C}, /* 1E6C */
  {0x1E6E,0x1E6F,0x1E6E},  {0x1E6E,0x1E6F,0x1E6E}, /* 1E6E */
  {0x1E70,0x1E71,0x1E70},  {0x1E70,0x1E71,0x1E70}, /* 1E70 */
  {0x1E72,0x1E73,0x1E72},  {0x1E72,0x1E73,0x1E72}, /* 1E72 */
  {0x1E74,0x1E75,0x1E74},  {0x1E74,0x1E75,0x1E74}, /* 1E74 */
  {0x1E76,0x1E77,0x1E76},  {0x1E76,0x1E77,0x1E76}, /* 1E76 */
  {0x1E78,0x1E79,0x1E78},  {0x1E78,0x1E79,0x1E78}, /* 1E78 */
  {0x1E7A,0x1E7B,0x1E7A},  {0x1E7A,0x1E7B,0x1E7A}, /* 1E7A */
  {0x1E7C,0x1E7D,0x1E7C},  {0x1E7C,0x1E7D,0x1E7C}, /* 1E7C */
  {0x1E7E,0x1E7F,0x1E7E},  {0x1E7E,0x1E7F,0x1E7E}, /* 1E7E */
  {0x1E80,0x1E81,0x1E80},  {0x1E80,0x1E81,0x1E80}, /* 1E80 */
  {0x1E82,0x1E83,0x1E82},  {0x1E82,0x1E83,0x1E82}, /* 1E82 */
  {0x1E84,0x1E85,0x1E84},  {0x1E84,0x1E85,0x1E84}, /* 1E84 */
  {0x1E86,0x1E87,0x1E86},  {0x1E86,0x1E87,0x1E86}, /* 1E86 */
  {0x1E88,0x1E89,0x1E88},  {0x1E88,0x1E89,0x1E88}, /* 1E88 */
  {0x1E8A,0x1E8B,0x1E8A},  {0x1E8A,0x1E8B,0x1E8A}, /* 1E8A */
  {0x1E8C,0x1E8D,0x1E8C},  {0x1E8C,0x1E8D,0x1E8C}, /* 1E8C */
  {0x1E8E,0x1E8F,0x1E8E},  {0x1E8E,0x1E8F,0x1E8E}, /* 1E8E */
  {0x1E90,0x1E91,0x1E90},  {0x1E90,0x1E91,0x1E90}, /* 1E90 */
  {0x1E92,0x1E93,0x1E92},  {0x1E92,0x1E93,0x1E92}, /* 1E92 */
  {0x1E94,0x1E95,0x1E94},  {0x1E94,0x1E95,0x1E94}, /* 1E94 */
  {0x1E96,0x1E96,0x1E96},  {0x1E97,0x1E97,0x1E97}, /* 1E96 */
  {0x1E98,0x1E98,0x1E98},  {0x1E99,0x1E99,0x1E99}, /* 1E98 */
  {0x1E9A,0x1E9A,0x1E9A},  {0x1E60,0x1E9B,0x1E60}, /* 1E9A */
  {0x1E9C,0x1E9C,0x1E9C},  {0x1E9D,0x1E9D,0x1E9D}, /* 1E9C */
  {0x1E9E,0x00DF,0x1E9E},  {0x1E9F,0x1E9F,0x1E9F}, /* 1E9E */
  {0x1EA0,0x1EA1,0x1EA0},  {0x1EA0,0x1EA1,0x1EA0}, /* 1EA0 */
  {0x1EA2,0x1EA3,0x1EA2},  {0x1EA2,0x1EA3,0x1EA2}, /* 1EA2 */
  {0x1EA4,0x1EA5,0x1EA4},  {0x1EA4,0x1EA5,0x1EA4}, /* 1EA4 */
  {0x1EA6,0x1EA7,0x1EA6},  {0x1EA6,0x1EA7,0x1EA6}, /* 1EA6 */
  {0x1EA8,0x1EA9,0x1EA8},  {0x1EA8,0x1EA9,0x1EA8}, /* 1EA8 */
  {0x1EAA,0x1EAB,0x1EAA},  {0x1EAA,0x1EAB,0x1EAA}, /* 1EAA */
  {0x1EAC,0x1EAD,0x1EAC},  {0x1EAC,0x1EAD,0x1EAC}, /* 1EAC */
  {0x1EAE,0x1EAF,0x1EAE},  {0x1EAE,0x1EAF,0x1EAE}, /* 1EAE */
  {0x1EB0,0x1EB1,0x1EB0},  {0x1EB0,0x1EB1,0x1EB0}, /* 1EB0 */
  {0x1EB2,0x1EB3,0x1EB2},  {0x1EB2,0x1EB3,0x1EB2}, /* 1EB2 */
  {0x1EB4,0x1EB5,0x1EB4},  {0x1EB4,0x1EB5,0x1EB4}, /* 1EB4 */
  {0x1EB6,0x1EB7,0x1EB6},  {0x1EB6,0x1EB7,0x1EB6}, /* 1EB6 */
  {0x1EB8,0x1EB9,0x1EB8},  {0x1EB8,0x1EB9,0x1EB8}, /* 1EB8 */
  {0x1EBA,0x1EBB,0x1EBA},  {0x1EBA,0x1EBB,0x1EBA}, /* 1EBA */
  {0x1EBC,0x1EBD,0x1EBC},  {0x1EBC,0x1EBD,0x1EBC}, /* 1EBC */
  {0x1EBE,0x1EBF,0x1EBE},  {0x1EBE,0x1EBF,0x1EBE}, /* 1EBE */
  {0x1EC0,0x1EC1,0x1EC0},  {0x1EC0,0x1EC1,0x1EC0}, /* 1EC0 */
  {0x1EC2,0x1EC3,0x1EC2},  {0x1EC2,0x1EC3,0x1EC2}, /* 1EC2 */
  {0x1EC4,0x1EC5,0x1EC4},  {0x1EC4,0x1EC5,0x1EC4}, /* 1EC4 */
  {0x1EC6,0x1EC7,0x1EC6},  {0x1EC6,0x1EC7,0x1EC6}, /* 1EC6 */
  {0x1EC8,0x1EC9,0x1EC8},  {0x1EC8,0x1EC9,0x1EC8}, /* 1EC8 */
  {0x1ECA,0x1ECB,0x1ECA},  {0x1ECA,0x1ECB,0x1ECA}, /* 1ECA */
  {0x1ECC,0x1ECD,0x1ECC},  {0x1ECC,0x1ECD,0x1ECC}, /* 1ECC */
  {0x1ECE,0x1ECF,0x1ECE},  {0x1ECE,0x1ECF,0x1ECE}, /* 1ECE */
  {0x1ED0,0x1ED1,0x1ED0},  {0x1ED0,0x1ED1,0x1ED0}, /* 1ED0 */
  {0x1ED2,0x1ED3,0x1ED2},  {0x1ED2,0x1ED3,0x1ED2}, /* 1ED2 */
  {0x1ED4,0x1ED5,0x1ED4},  {0x1ED4,0x1ED5,0x1ED4}, /* 1ED4 */
  {0x1ED6,0x1ED7,0x1ED6},  {0x1ED6,0x1ED7,0x1ED6}, /* 1ED6 */
  {0x1ED8,0x1ED9,0x1ED8},  {0x1ED8,0x1ED9,0x1ED8}, /* 1ED8 */
  {0x1EDA,0x1EDB,0x1EDA},  {0x1EDA,0x1EDB,0x1EDA}, /* 1EDA */
  {0x1EDC,0x1EDD,0x1EDC},  {0x1EDC,0x1EDD,0x1EDC}, /* 1EDC */
  {0x1EDE,0x1EDF,0x1EDE},  {0x1EDE,0x1EDF,0x1EDE}, /* 1EDE */
  {0x1EE0,0x1EE1,0x1EE0},  {0x1EE0,0x1EE1,0x1EE0}, /* 1EE0 */
  {0x1EE2,0x1EE3,0x1EE2},  {0x1EE2,0x1EE3,0x1EE2}, /* 1EE2 */
  {0x1EE4,0x1EE5,0x1EE4},  {0x1EE4,0x1EE5,0x1EE4}, /* 1EE4 */
  {0x1EE6,0x1EE7,0x1EE6},  {0x1EE6,0x1EE7,0x1EE6}, /* 1EE6 */
  {0x1EE8,0x1EE9,0x1EE8},  {0x1EE8,0x1EE9,0x1EE8}, /* 1EE8 */
  {0x1EEA,0x1EEB,0x1EEA},  {0x1EEA,0x1EEB,0x1EEA}, /* 1EEA */
  {0x1EEC,0x1EED,0x1EEC},  {0x1EEC,0x1EED,0x1EEC}, /* 1EEC */
  {0x1EEE,0x1EEF,0x1EEE},  {0x1EEE,0x1EEF,0x1EEE}, /* 1EEE */
  {0x1EF0,0x1EF1,0x1EF0},  {0x1EF0,0x1EF1,0x1EF0}, /* 1EF0 */
  {0x1EF2,0x1EF3,0x1EF2},  {0x1EF2,0x1EF3,0x1EF2}, /* 1EF2 */
  {0x1EF4,0x1EF5,0x1EF4},  {0x1EF4,0x1EF5,0x1EF4}, /* 1EF4 */
  {0x1EF6,0x1EF7,0x1EF6},  {0x1EF6,0x1EF7,0x1EF6}, /* 1EF6 */
  {0x1EF8,0x1EF9,0x1EF8},  {0x1EF8,0x1EF9,0x1EF8}, /* 1EF8 */
  {0x1EFA,0x1EFB,0x1EFA},  {0x1EFA,0x1EFB,0x1EFA}, /* 1EFA */
  {0x1EFC,0x1EFD,0x1EFC},  {0x1EFC,0x1EFD,0x1EFC}, /* 1EFC */
  {0x1EFE,0x1EFF,0x1EFE},  {0x1EFE,0x1EFF,0x1EFE}  /* 1EFE */
};

static const MY_UNICASE_CHARACTER u900p01F[]= {
  {0x1F08,0x1F00,0x1F08},  {0x1F09,0x1F01,0x1F09}, /* 1F00 */
  {0x1F0A,0x1F02,0x1F0A},  {0x1F0B,0x1F03,0x1F0B}, /* 1F02 */
  {0x1F0C,0x1F04,0x1F0C},  {0x1F0D,0x1F05,0x1F0D}, /* 1F04 */
  {0x1F0E,0x1F06,0x1F0E},  {0x1F0F,0x1F07,0x1F0F}, /* 1F06 */
  {0x1F08,0x1F00,0x1F08},  {0x1F09,0x1F01,0x1F09}, /* 1F08 */
  {0x1F0A,0x1F02,0x1F0A},  {0x1F0B,0x1F03,0x1F0B}, /* 1F0A */
  {0x1F0C,0x1F04,0x1F0C},  {0x1F0D,0x1F05,0x1F0D}, /* 1F0C */
  {0x1F0E,0x1F06,0x1F0E},  {0x1F0F,0x1F07,0x1F0F}, /* 1F0E */
  {0x1F18,0x1F10,0x1F18},  {0x1F19,0x1F11,0x1F19}, /* 1F10 */
  {0x1F1A,0x1F12,0x1F1A},  {0x1F1B,0x1F13,0x1F1B}, /* 1F12 */
  {0x1F1C,0x1F14,0x1F1C},  {0x1F1D,0x1F15,0x1F1D}, /* 1F14 */
  {0x1F16,0x1F16,0x1F16},  {0x1F17,0x1F17,0x1F17}, /* 1F16 */
  {0x1F18,0x1F10,0x1F18},  {0x1F19,0x1F11,0x1F19}, /* 1F18 */
  {0x1F1A,0x1F12,0x1F1A},  {0x1F1B,0x1F13,0x1F1B}, /* 1F1A */
  {0x1F1C,0x1F14,0x1F1C},  {0x1F1D,0x1F15,0x1F1D}, /* 1F1C */
  {0x1F1E,0x1F1E,0x1F1E},  {0x1F1F,0x1F1F,0x1F1F}, /* 1F1E */
  {0x1F28,0x1F20,0x1F28},  {0x1F29,0x1F21,0x1F29}, /* 1F20 */
  {0x1F2A,0x1F22,0x1F2A},  {0x1F2B,0x1F23,0x1F2B}, /* 1F22 */
  {0x1F2C,0x1F24,0x1F2C},  {0x1F2D,0x1F25,0x1F2D}, /* 1F24 */
  {0x1F2E,0x1F26,0x1F2E},  {0x1F2F,0x1F27,0x1F2F}, /* 1F26 */
  {0x1F28,0x1F20,0x1F28},  {0x1F29,0x1F21,0x1F29}, /* 1F28 */
  {0x1F2A,0x1F22,0x1F2A},  {0x1F2B,0x1F23,0x1F2B}, /* 1F2A */
  {0x1F2C,0x1F24,0x1F2C},  {0x1F2D,0x1F25,0x1F2D}, /* 1F2C */
  {0x1F2E,0x1F26,0x1F2E},  {0x1F2F,0x1F27,0x1F2F}, /* 1F2E */
  {0x1F38,0x1F30,0x1F38},  {0x1F39,0x1F31,0x1F39}, /* 1F30 */
  {0x1F3A,0x1F32,0x1F3A},  {0x1F3B,0x1F33,0x1F3B}, /* 1F32 */
  {0x1F3C,0x1F34,0x1F3C},  {0x1F3D,0x1F35,0x1F3D}, /* 1F34 */
  {0x1F3E,0x1F36,0x1F3E},  {0x1F3F,0x1F37,0x1F3F}, /* 1F36 */
  {0x1F38,0x1F30,0x1F38},  {0x1F39,0x1F31,0x1F39}, /* 1F38 */
  {0x1F3A,0x1F32,0x1F3A},  {0x1F3B,0x1F33,0x1F3B}, /* 1F3A */
  {0x1F3C,0x1F34,0x1F3C},  {0x1F3D,0x1F35,0x1F3D}, /* 1F3C */
  {0x1F3E,0x1F36,0x1F3E},  {0x1F3F,0x1F37,0x1F3F}, /* 1F3E */
  {0x1F48,0x1F40,0x1F48},  {0x1F49,0x1F41,0x1F49}, /* 1F40 */
  {0x1F4A,0x1F42,0x1F4A},  {0x1F4B,0x1F43,0x1F4B}, /* 1F42 */
  {0x1F4C,0x1F44,0x1F4C},  {0x1F4D,0x1F45,0x1F4D}, /* 1F44 */
  {0x1F46,0x1F46,0x1F46},  {0x1F47,0x1F47,0x1F47}, /* 1F46 */
  {0x1F48,0x1F40,0x1F48},  {0x1F49,0x1F41,0x1F49}, /* 1F48 */
  {0x1F4A,0x1F42,0x1F4A},  {0x1F4B,0x1F43,0x1F4B}, /* 1F4A */
  {0x1F4C,0x1F44,0x1F4C},  {0x1F4D,0x1F45,0x1F4D}, /* 1F4C */
  {0x1F4E,0x1F4E,0x1F4E},  {0x1F4F,0x1F4F,0x1F4F}, /* 1F4E */
  {0x1F50,0x1F50,0x1F50},  {0x1F59,0x1F51,0x1F59}, /* 1F50 */
  {0x1F52,0x1F52,0x1F52},  {0x1F5B,0x1F53,0x1F5B}, /* 1F52 */
  {0x1F54,0x1F54,0x1F54},  {0x1F5D,0x1F55,0x1F5D}, /* 1F54 */
  {0x1F56,0x1F56,0x1F56},  {0x1F5F,0x1F57,0x1F5F}, /* 1F56 */
  {0x1F58,0x1F58,0x1F58},  {0x1F59,0x1F51,0x1F59}, /* 1F58 */
  {0x1F5A,0x1F5A,0x1F5A},  {0x1F5B,0x1F53,0x1F5B}, /* 1F5A */
  {0x1F5C,0x1F5C,0x1F5C},  {0x1F5D,0x1F55,0x1F5D}, /* 1F5C */
  {0x1F5E,0x1F5E,0x1F5E},  {0x1F5F,0x1F57,0x1F5F}, /* 1F5E */
  {0x1F68,0x1F60,0x1F68},  {0x1F69,0x1F61,0x1F69}, /* 1F60 */
  {0x1F6A,0x1F62,0x1F6A},  {0x1F6B,0x1F63,0x1F6B}, /* 1F62 */
  {0x1F6C,0x1F64,0x1F6C},  {0x1F6D,0x1F65,0x1F6D}, /* 1F64 */
  {0x1F6E,0x1F66,0x1F6E},  {0x1F6F,0x1F67,0x1F6F}, /* 1F66 */
  {0x1F68,0x1F60,0x1F68},  {0x1F69,0x1F61,0x1F69}, /* 1F68 */
  {0x1F6A,0x1F62,0x1F6A},  {0x1F6B,0x1F63,0x1F6B}, /* 1F6A */
  {0x1F6C,0x1F64,0x1F6C},  {0x1F6D,0x1F65,0x1F6D}, /* 1F6C */
  {0x1F6E,0x1F66,0x1F6E},  {0x1F6F,0x1F67,0x1F6F}, /* 1F6E */
  {0x1FBA,0x1F70,0x1FBA},  {0x1FBB,0x1F71,0x1FBB}, /* 1F70 */
  {0x1FC8,0x1F72,0x1FC8},  {0x1FC9,0x1F73,0x1FC9}, /* 1F72 */
  {0x1FCA,0x1F74,0x1FCA},  {0x1FCB,0x1F75,0x1FCB}, /* 1F74 */
  {0x1FDA,0x1F76,0x1FDA},  {0x1FDB,0x1F77,0x1FDB}, /* 1F76 */
  {0x1FF8,0x1F78,0x1FF8},  {0x1FF9,0x1F79,0x1FF9}, /* 1F78 */
  {0x1FEA,0x1F7A,0x1FEA},  {0x1FEB,0x1F7B,0x1FEB}, /* 1F7A */
  {0x1FFA,0x1F7C,0x1FFA},  {0x1FFB,0x1F7D,0x1FFB}, /* 1F7C */
  {0x1F7E,0x1F7E,0x1F7E},  {0x1F7F,0x1F7F,0x1F7F}, /* 1F7E */
  {0x1F88,0x1F80,0x1F88},  {0x1F89,0x1F81,0x1F89}, /* 1F80 */
  {0x1F8A,0x1F82,0x1F8A},  {0x1F8B,0x1F83,0x1F8B}, /* 1F82 */
  {0x1F8C,0x1F84,0x1F8C},  {0x1F8D,0x1F85,0x1F8D}, /* 1F84 */
  {0x1F8E,0x1F86,0x1F8E},  {0x1F8F,0x1F87,0x1F8F}, /* 1F86 */
  {0x1F88,0x1F80,0x1F88},  {0x1F89,0x1F81,0x1F89}, /* 1F88 */
  {0x1F8A,0x1F82,0x1F8A},  {0x1F8B,0x1F83,0x1F8B}, /* 1F8A */
  {0x1F8C,0x1F84,0x1F8C},  {0x1F8D,0x1F85,0x1F8D}, /* 1F8C */
  {0x1F8E,0x1F86,0x1F8E},  {0x1F8F,0x1F87,0x1F8F}, /* 1F8E */
  {0x1F98,0x1F90,0x1F98},  {0x1F99,0x1F91,0x1F99}, /* 1F90 */
  {0x1F9A,0x1F92,0x1F9A},  {0x1F9B,0x1F93,0x1F9B}, /* 1F92 */
  {0x1F9C,0x1F94,0x1F9C},  {0x1F9D,0x1F95,0x1F9D}, /* 1F94 */
  {0x1F9E,0x1F96,0x1F9E},  {0x1F9F,0x1F97,0x1F9F}, /* 1F96 */
  {0x1F98,0x1F90,0x1F98},  {0x1F99,0x1F91,0x1F99}, /* 1F98 */
  {0x1F9A,0x1F92,0x1F9A},  {0x1F9B,0x1F93,0x1F9B}, /* 1F9A */
  {0x1F9C,0x1F94,0x1F9C},  {0x1F9D,0x1F95,0x1F9D}, /* 1F9C */
  {0x1F9E,0x1F96,0x1F9E},  {0x1F9F,0x1F97,0x1F9F}, /* 1F9E */
  {0x1FA8,0x1FA0,0x1FA8},  {0x1FA9,0x1FA1,0x1FA9}, /* 1FA0 */
  {0x1FAA,0x1FA2,0x1FAA},  {0x1FAB,0x1FA3,0x1FAB}, /* 1FA2 */
  {0x1FAC,0x1FA4,0x1FAC},  {0x1FAD,0x1FA5,0x1FAD}, /* 1FA4 */
  {0x1FAE,0x1FA6,0x1FAE},  {0x1FAF,0x1FA7,0x1FAF}, /* 1FA6 */
  {0x1FA8,0x1FA0,0x1FA8},  {0x1FA9,0x1FA1,0x1FA9}, /* 1FA8 */
  {0x1FAA,0x1FA2,0x1FAA},  {0x1FAB,0x1FA3,0x1FAB}, /* 1FAA */
  {0x1FAC,0x1FA4,0x1FAC},  {0x1FAD,0x1FA5,0x1FAD}, /* 1FAC */
  {0x1FAE,0x1FA6,0x1FAE},  {0x1FAF,0x1FA7,0x1FAF}, /* 1FAE */
  {0x1FB8,0x1FB0,0x1FB8},  {0x1FB9,0x1FB1,0x1FB9}, /* 1FB0 */
  {0x1FB2,0x1FB2,0x1FB2},  {0x1FBC,0x1FB3,0x1FBC}, /* 1FB2 */
  {0x1FB4,0x1FB4,0x1FB4},  {0x1FB5,0x1FB5,0x1FB5}, /* 1FB4 */
  {0x1FB6,0x1FB6,0x1FB6},  {0x1FB7,0x1FB7,0x1FB7}, /* 1FB6 */
  {0x1FB8,0x1FB0,0x1FB8},  {0x1FB9,0x1FB1,0x1FB9}, /* 1FB8 */
  {0x1FBA,0x1F70,0x1FBA},  {0x1FBB,0x1F71,0x1FBB}, /* 1FBA */
  {0x1FBC,0x1FB3,0x1FBC},  {0x1FBD,0x1FBD,0x1FBD}, /* 1FBC */
  {0x0399,0x1FBE,0x0399},  {0x1FBF,0x1FBF,0x1FBF}, /* 1FBE */
  {0x1FC0,0x1FC0,0x1FC0},  {0x1FC1,0x1FC1,0x1FC1}, /* 1FC0 */
  {0x1FC2,0x1FC2,0x1FC2},  {0x1FCC,0x1FC3,0x1FCC}, /* 1FC2 */
  {0x1FC4,0x1FC4,0x1FC4},  {0x1FC5,0x1FC5,0x1FC5}, /* 1FC4 */
  {0x1FC6,0x1FC6,0x1FC6},  {0x1FC7,0x1FC7,0x1FC7}, /* 1FC6 */
  {0x1FC8,0x1F72,0x1FC8},  {0x1FC9,0x1F73,0x1FC9}, /* 1FC8 */
  {0x1FCA,0x1F74,0x1FCA},  {0x1FCB,0x1F75,0x1FCB}, /* 1FCA */
  {0x1FCC,0x1FC3,0x1FCC},  {0x1FCD,0x1FCD,0x1FCD}, /* 1FCC */
  {0x1FCE,0x1FCE,0x1FCE},  {0x1FCF,0x1FCF,0x1FCF}, /* 1FCE */
  {0x1FD8,0x1FD0,0x1FD8},  {0x1FD9,0x1FD1,0x1FD9}, /* 1FD0 */
  {0x1FD2,0x1FD2,0x1FD2},  {0x1FD3,0x1FD3,0x1FD3}, /* 1FD2 */
  {0x1FD4,0x1FD4,0x1FD4},  {0x1FD5,0x1FD5,0x1FD5}, /* 1FD4 */
  {0x1FD6,0x1FD6,0x1FD6},  {0x1FD7,0x1FD7,0x1FD7}, /* 1FD6 */
  {0x1FD8,0x1FD0,0x1FD8},  {0x1FD9,0x1FD1,0x1FD9}, /* 1FD8 */
  {0x1FDA,0x1F76,0x1FDA},  {0x1FDB,0x1F77,0x1FDB}, /* 1FDA */
  {0x1FDC,0x1FDC,0x1FDC},  {0x1FDD,0x1FDD,0x1FDD}, /* 1FDC */
  {0x1FDE,0x1FDE,0x1FDE},  {0x1FDF,0x1FDF,0x1FDF}, /* 1FDE */
  {0x1FE8,0x1FE0,0x1FE8},  {0x1FE9,0x1FE1,0x1FE9}, /* 1FE0 */
  {0x1FE2,0x1FE2,0x1FE2},  {0x1FE3,0x1FE3,0x1FE3}, /* 1FE2 */
  {0x1FE4,0x1FE4,0x1FE4},  {0x1FEC,0x1FE5,0x1FEC}, /* 1FE4 */
  {0x1FE6,0x1FE6,0x1FE6},  {0x1FE7,0x1FE7,0x1FE7}, /* 1FE6 */
  {0x1FE8,0x1FE0,0x1FE8},  {0x1FE9,0x1FE1,0x1FE9}, /* 1FE8 */
  {0x1FEA,0x1F7A,0x1FEA},  {0x1FEB,0x1F7B,0x1FEB}, /* 1FEA */
  {0x1FEC,0x1FE5,0x1FEC},  {0x1FED,0x1FED,0x1FED}, /* 1FEC */
  {0x1FEE,0x1FEE,0x1FEE},  {0x1FEF,0x1FEF,0x1FEF}, /* 1FEE */
  {0x1FF0,0x1FF0,0x1FF0},  {0x1FF1,0x1FF1,0x1FF1}, /* 1FF0 */
  {0x1FF2,0x1FF2,0x1FF2},  {0x1FFC,0x1FF3,0x1FFC}, /* 1FF2 */
  {0x1FF4,0x1FF4,0x1FF4},  {0x1FF5,0x1FF5,0x1FF5}, /* 1FF4 */
  {0x1FF6,0x1FF6,0x1FF6},  {0x1FF7,0x1FF7,0x1FF7}, /* 1FF6 */
  {0x1FF8,0x1F78,0x1FF8},  {0x1FF9,0x1F79,0x1FF9}, /* 1FF8 */
  {0x1FFA,0x1F7C,0x1FFA},  {0x1FFB,0x1F7D,0x1FFB}, /* 1FFA */
  {0x1FFC,0x1FF3,0x1FFC},  {0x1FFD,0x1FFD,0x1FFD}, /* 1FFC */
  {0x1FFE,0x1FFE,0x1FFE},  {0x1FFF,0x1FFF,0x1FFF}  /* 1FFE */
};

static const MY_UNICASE_CHARACTER u900p021[]= {
  {0x2100,0x2100,0x2100},  {0x2101,0x2101,0x2101}, /* 2100 */
  {0x2102,0x2102,0x2102},  {0x2103,0x2103,0x2103}, /* 2102 */
  {0x2104,0x2104,0x2104},  {0x2105,0x2105,0x2105}, /* 2104 */
  {0x2106,0x2106,0x2106},  {0x2107,0x2107,0x2107}, /* 2106 */
  {0x2108,0x2108,0x2108},  {0x2109,0x2109,0x2109}, /* 2108 */
  {0x210A,0x210A,0x210A},  {0x210B,0x210B,0x210B}, /* 210A */
  {0x210C,0x210C,0x210C},  {0x210D,0x210D,0x210D}, /* 210C */
  {0x210E,0x210E,0x210E},  {0x210F,0x210F,0x210F}, /* 210E */
  {0x2110,0x2110,0x2110},  {0x2111,0x2111,0x2111}, /* 2110 */
  {0x2112,0x2112,0x2112},  {0x2113,0x2113,0x2113}, /* 2112 */
  {0x2114,0x2114,0x2114},  {0x2115,0x2115,0x2115}, /* 2114 */
  {0x2116,0x2116,0x2116},  {0x2117,0x2117,0x2117}, /* 2116 */
  {0x2118,0x2118,0x2118},  {0x2119,0x2119,0x2119}, /* 2118 */
  {0x211A,0x211A,0x211A},  {0x211B,0x211B,0x211B}, /* 211A */
  {0x211C,0x211C,0x211C},  {0x211D,0x211D,0x211D}, /* 211C */
  {0x211E,0x211E,0x211E},  {0x211F,0x211F,0x211F}, /* 211E */
  {0x2120,0x2120,0x2120},  {0x2121,0x2121,0x2121}, /* 2120 */
  {0x2122,0x2122,0x2122},  {0x2123,0x2123,0x2123}, /* 2122 */
  {0x2124,0x2124,0x2124},  {0x2125,0x2125,0x2125}, /* 2124 */
  {0x2126,0x03C9,0x2126},  {0x2127,0x2127,0x2127}, /* 2126 */
  {0x2128,0x2128,0x2128},  {0x2129,0x2129,0x2129}, /* 2128 */
  {0x212A,0x006B,0x212A},  {0x212B,0x00E5,0x212B}, /* 212A */
  {0x212C,0x212C,0x212C},  {0x212D,0x212D,0x212D}, /* 212C */
  {0x212E,0x212E,0x212E},  {0x212F,0x212F,0x212F}, /* 212E */
  {0x2130,0x2130,0x2130},  {0x2131,0x2131,0x2131}, /* 2130 */
  {0x2132,0x214E,0x2132},  {0x2133,0x2133,0x2133}, /* 2132 */
  {0x2134,0x2134,0x2134},  {0x2135,0x2135,0x2135}, /* 2134 */
  {0x2136,0x2136,0x2136},  {0x2137,0x2137,0x2137}, /* 2136 */
  {0x2138,0x2138,0x2138},  {0x2139,0x2139,0x2139}, /* 2138 */
  {0x213A,0x213A,0x213A},  {0x213B,0x213B,0x213B}, /* 213A */
  {0x213C,0x213C,0x213C},  {0x213D,0x213D,0x213D}, /* 213C */
  {0x213E,0x213E,0x213E},  {0x213F,0x213F,0x213F}, /* 213E */
  {0x2140,0x2140,0x2140},  {0x2141,0x2141,0x2141}, /* 2140 */
  {0x2142,0x2142,0x2142},  {0x2143,0x2143,0x2143}, /* 2142 */
  {0x2144,0x2144,0x2144},  {0x2145,0x2145,0x2145}, /* 2144 */
  {0x2146,0x2146,0x2146},  {0x2147,0x2147,0x2147}, /* 2146 */
  {0x2148,0x2148,0x2148},  {0x2149,0x2149,0x2149}, /* 2148 */
  {0x214A,0x214A,0x214A},  {0x214B,0x214B,0x214B}, /* 214A */
  {0x214C,0x214C,0x214C},  {0x214D,0x214D,0x214D}, /* 214C */
  {0x2132,0x214E,0x2132},  {0x214F,0x214F,0x214F}, /* 214E */
  {0x2150,0x2150,0x2150},  {0x2151,0x2151,0x2151}, /* 2150 */
  {0x2152,0x2152,0x2152},  {0x2153,0x2153,0x2153}, /* 2152 */
  {0x2154,0x2154,0x2154},  {0x2155,0x2155,0x2155}, /* 2154 */
  {0x2156,0x2156,0x2156},  {0x2157,0x2157,0x2157}, /* 2156 */
  {0x2158,0x2158,0x2158},  {0x2159,0x2159,0x2159}, /* 2158 */
  {0x215A,0x215A,0x215A},  {0x215B,0x215B,0x215B}, /* 215A */
  {0x215C,0x215C,0x215C},  {0x215D,0x215D,0x215D}, /* 215C */
  {0x215E,0x215E,0x215E},  {0x215F,0x215F,0x215F}, /* 215E */
  {0x2160,0x2170,0x2160},  {0x2161,0x2171,0x2161}, /* 2160 */
  {0x2162,0x2172,0x2162},  {0x2163,0x2173,0x2163}, /* 2162 */
  {0x2164,0x2174,0x2164},  {0x2165,0x2175,0x2165}, /* 2164 */
  {0x2166,0x2176,0x2166},  {0x2167,0x2177,0x2167}, /* 2166 */
  {0x2168,0x2178,0x2168},  {0x2169,0x2179,0x2169}, /* 2168 */
  {0x216A,0x217A,0x216A},  {0x216B,0x217B,0x216B}, /* 216A */
  {0x216C,0x217C,0x216C},  {0x216D,0x217D,0x216D}, /* 216C */
  {0x216E,0x217E,0x216E},  {0x216F,0x217F,0x216F}, /* 216E */
  {0x2160,0x2170,0x2160},  {0x2161,0x2171,0x2161}, /* 2170 */
  {0x2162,0x2172,0x2162},  {0x2163,0x2173,0x2163}, /* 2172 */
  {0x2164,0x2174,0x2164},  {0x2165,0x2175,0x2165}, /* 2174 */
  {0x2166,0x2176,0x2166},  {0x2167,0x2177,0x2167}, /* 2176 */
  {0x2168,0x2178,0x2168},  {0x2169,0x2179,0x2169}, /* 2178 */
  {0x216A,0x217A,0x216A},  {0x216B,0x217B,0x216B}, /* 217A */
  {0x216C,0x217C,0x216C},  {0x216D,0x217D,0x216D}, /* 217C */
  {0x216E,0x217E,0x216E},  {0x216F,0x217F,0x216F}, /* 217E */
  {0x2180,0x2180,0x2180},  {0x2181,0x2181,0x2181}, /* 2180 */
  {0x2182,0x2182,0x2182},  {0x2183,0x2184,0x2183}, /* 2182 */
  {0x2183,0x2184,0x2183},  {0x2185,0x2185,0x2185}, /* 2184 */
  {0x2186,0x2186,0x2186},  {0x2187,0x2187,0x2187}, /* 2186 */
  {0x2188,0x2188,0x2188},  {0x2189,0x2189,0x2189}, /* 2188 */
  {0x218A,0x218A,0x218A},  {0x218B,0x218B,0x218B}, /* 218A */
  {0x218C,0x218C,0x218C},  {0x218D,0x218D,0x218D}, /* 218C */
  {0x218E,0x218E,0x218E},  {0x218F,0x218F,0x218F}, /* 218E */
  {0x2190,0x2190,0x2190},  {0x2191,0x2191,0x2191}, /* 2190 */
  {0x2192,0x2192,0x2192},  {0x2193,0x2193,0x2193}, /* 2192 */
  {0x2194,0x2194,0x2194},  {0x2195,0x2195,0x2195}, /* 2194 */
  {0x2196,0x2196,0x2196},  {0x2197,0x2197,0x2197}, /* 2196 */
  {0x2198,0x2198,0x2198},  {0x2199,0x2199,0x2199}, /* 2198 */
  {0x219A,0x219A,0x219A},  {0x219B,0x219B,0x219B}, /* 219A */
  {0x219C,0x219C,0x219C},  {0x219D,0x219D,0x219D}, /* 219C */
  {0x219E,0x219E,0x219E},  {0x219F,0x219F,0x219F}, /* 219E */
  {0x21A0,0x21A0,0x21A0},  {0x21A1,0x21A1,0x21A1}, /* 21A0 */
  {0x21A2,0x21A2,0x21A2},  {0x21A3,0x21A3,0x21A3}, /* 21A2 */
  {0x21A4,0x21A4,0x21A4},  {0x21A5,0x21A5,0x21A5}, /* 21A4 */
  {0x21A6,0x21A6,0x21A6},  {0x21A7,0x21A7,0x21A7}, /* 21A6 */
  {0x21A8,0x21A8,0x21A8},  {0x21A9,0x21A9,0x21A9}, /* 21A8 */
  {0x21AA,0x21AA,0x21AA},  {0x21AB,0x21AB,0x21AB}, /* 21AA */
  {0x21AC,0x21AC,0x21AC},  {0x21AD,0x21AD,0x21AD}, /* 21AC */
  {0x21AE,0x21AE,0x21AE},  {0x21AF,0x21AF,0x21AF}, /* 21AE */
  {0x21B0,0x21B0,0x21B0},  {0x21B1,0x21B1,0x21B1}, /* 21B0 */
  {0x21B2,0x21B2,0x21B2},  {0x21B3,0x21B3,0x21B3}, /* 21B2 */
  {0x21B4,0x21B4,0x21B4},  {0x21B5,0x21B5,0x21B5}, /* 21B4 */
  {0x21B6,0x21B6,0x21B6},  {0x21B7,0x21B7,0x21B7}, /* 21B6 */
  {0x21B8,0x21B8,0x21B8},  {0x21B9,0x21B9,0x21B9}, /* 21B8 */
  {0x21BA,0x21BA,0x21BA},  {0x21BB,0x21BB,0x21BB}, /* 21BA */
  {0x21BC,0x21BC,0x21BC},  {0x21BD,0x21BD,0x21BD}, /* 21BC */
  {0x21BE,0x21BE,0x21BE},  {0x21BF,0x21BF,0x21BF}, /* 21BE */
  {0x21C0,0x21C0,0x21C0},  {0x21C1,0x21C1,0x21C1}, /* 21C0 */
  {0x21C2,0x21C2,0x21C2},  {0x21C3,0x21C3,0x21C3}, /* 21C2 */
  {0x21C4,0x21C4,0x21C4},  {0x21C5,0x21C5,0x21C5}, /* 21C4 */
  {0x21C6,0x21C6,0x21C6},  {0x21C7,0x21C7,0x21C7}, /* 21C6 */
  {0x21C8,0x21C8,0x21C8},  {0x21C9,0x21C9,0x21C9}, /* 21C8 */
  {0x21CA,0x21CA,0x21CA},  {0x21CB,0x21CB,0x21CB}, /* 21CA */
  {0x21CC,0x21CC,0x21CC},  {0x21CD,0x21CD,0x21CD}, /* 21CC */
  {0x21CE,0x21CE,0x21CE},  {0x21CF,0x21CF,0x21CF}, /* 21CE */
  {0x21D0,0x21D0,0x21D0},  {0x21D1,0x21D1,0x21D1}, /* 21D0 */
  {0x21D2,0x21D2,0x21D2},  {0x21D3,0x21D3,0x21D3}, /* 21D2 */
  {0x21D4,0x21D4,0x21D4},  {0x21D5,0x21D5,0x21D5}, /* 21D4 */
  {0x21D6,0x21D6,0x21D6},  {0x21D7,0x21D7,0x21D7}, /* 21D6 */
  {0x21D8,0x21D8,0x21D8},  {0x21D9,0x21D9,0x21D9}, /* 21D8 */
  {0x21DA,0x21DA,0x21DA},  {0x21DB,0x21DB,0x21DB}, /* 21DA */
  {0x21DC,0x21DC,0x21DC},  {0x21DD,0x21DD,0x21DD}, /* 21DC */
  {0x21DE,0x21DE,0x21DE},  {0x21DF,0x21DF,0x21DF}, /* 21DE */
  {0x21E0,0x21E0,0x21E0},  {0x21E1,0x21E1,0x21E1}, /* 21E0 */
  {0x21E2,0x21E2,0x21E2},  {0x21E3,0x21E3,0x21E3}, /* 21E2 */
  {0x21E4,0x21E4,0x21E4},  {0x21E5,0x21E5,0x21E5}, /* 21E4 */
  {0x21E6,0x21E6,0x21E6},  {0x21E7,0x21E7,0x21E7}, /* 21E6 */
  {0x21E8,0x21E8,0x21E8},  {0x21E9,0x21E9,0x21E9}, /* 21E8 */
  {0x21EA,0x21EA,0x21EA},  {0x21EB,0x21EB,0x21EB}, /* 21EA */
  {0x21EC,0x21EC,0x21EC},  {0x21ED,0x21ED,0x21ED}, /* 21EC */
  {0x21EE,0x21EE,0x21EE},  {0x21EF,0x21EF,0x21EF}, /* 21EE */
  {0x21F0,0x21F0,0x21F0},  {0x21F1,0x21F1,0x21F1}, /* 21F0 */
  {0x21F2,0x21F2,0x21F2},  {0x21F3,0x21F3,0x21F3}, /* 21F2 */
  {0x21F4,0x21F4,0x21F4},  {0x21F5,0x21F5,0x21F5}, /* 21F4 */
  {0x21F6,0x21F6,0x21F6},  {0x21F7,0x21F7,0x21F7}, /* 21F6 */
  {0x21F8,0x21F8,0x21F8},  {0x21F9,0x21F9,0x21F9}, /* 21F8 */
  {0x21FA,0x21FA,0x21FA},  {0x21FB,0x21FB,0x21FB}, /* 21FA */
  {0x21FC,0x21FC,0x21FC},  {0x21FD,0x21FD,0x21FD}, /* 21FC */
  {0x21FE,0x21FE,0x21FE},  {0x21FF,0x21FF,0x21FF}  /* 21FE */
};

static const MY_UNICASE_CHARACTER u900p024[]= {
  {0x2400,0x2400,0x2400},  {0x2401,0x2401,0x2401}, /* 2400 */
  {0x2402,0x2402,0x2402},  {0x2403,0x2403,0x2403}, /* 2402 */
  {0x2404,0x2404,0x2404},  {0x2405,0x2405,0x2405}, /* 2404 */
  {0x2406,0x2406,0x2406},  {0x2407,0x2407,0x2407}, /* 2406 */
  {0x2408,0x2408,0x2408},  {0x2409,0x2409,0x2409}, /* 2408 */
  {0x240A,0x240A,0x240A},  {0x240B,0x240B,0x240B}, /* 240A */
  {0x240C,0x240C,0x240C},  {0x240D,0x240D,0x240D}, /* 240C */
  {0x240E,0x240E,0x240E},  {0x240F,0x240F,0x240F}, /* 240E */
  {0x2410,0x2410,0x2410},  {0x2411,0x2411,0x2411}, /* 2410 */
  {0x2412,0x2412,0x2412},  {0x2413,0x2413,0x2413}, /* 2412 */
  {0x2414,0x2414,0x2414},  {0x2415,0x2415,0x2415}, /* 2414 */
  {0x2416,0x2416,0x2416},  {0x2417,0x2417,0x2417}, /* 2416 */
  {0x2418,0x2418,0x2418},  {0x2419,0x2419,0x2419}, /* 2418 */
  {0x241A,0x241A,0x241A},  {0x241B,0x241B,0x241B}, /* 241A */
  {0x241C,0x241C,0x241C},  {0x241D,0x241D,0x241D}, /* 241C */
  {0x241E,0x241E,0x241E},  {0x241F,0x241F,0x241F}, /* 241E */
  {0x2420,0x2420,0x2420},  {0x2421,0x2421,0x2421}, /* 2420 */
  {0x2422,0x2422,0x2422},  {0x2423,0x2423,0x2423}, /* 2422 */
  {0x2424,0x2424,0x2424},  {0x2425,0x2425,0x2425}, /* 2424 */
  {0x2426,0x2426,0x2426},  {0x2427,0x2427,0x2427}, /* 2426 */
  {0x2428,0x2428,0x2428},  {0x2429,0x2429,0x2429}, /* 2428 */
  {0x242A,0x242A,0x242A},  {0x242B,0x242B,0x242B}, /* 242A */
  {0x242C,0x242C,0x242C},  {0x242D,0x242D,0x242D}, /* 242C */
  {0x242E,0x242E,0x242E},  {0x242F,0x242F,0x242F}, /* 242E */
  {0x2430,0x2430,0x2430},  {0x2431,0x2431,0x2431}, /* 2430 */
  {0x2432,0x2432,0x2432},  {0x2433,0x2433,0x2433}, /* 2432 */
  {0x2434,0x2434,0x2434},  {0x2435,0x2435,0x2435}, /* 2434 */
  {0x2436,0x2436,0x2436},  {0x2437,0x2437,0x2437}, /* 2436 */
  {0x2438,0x2438,0x2438},  {0x2439,0x2439,0x2439}, /* 2438 */
  {0x243A,0x243A,0x243A},  {0x243B,0x243B,0x243B}, /* 243A */
  {0x243C,0x243C,0x243C},  {0x243D,0x243D,0x243D}, /* 243C */
  {0x243E,0x243E,0x243E},  {0x243F,0x243F,0x243F}, /* 243E */
  {0x2440,0x2440,0x2440},  {0x2441,0x2441,0x2441}, /* 2440 */
  {0x2442,0x2442,0x2442},  {0x2443,0x2443,0x2443}, /* 2442 */
  {0x2444,0x2444,0x2444},  {0x2445,0x2445,0x2445}, /* 2444 */
  {0x2446,0x2446,0x2446},  {0x2447,0x2447,0x2447}, /* 2446 */
  {0x2448,0x2448,0x2448},  {0x2449,0x2449,0x2449}, /* 2448 */
  {0x244A,0x244A,0x244A},  {0x244B,0x244B,0x244B}, /* 244A */
  {0x244C,0x244C,0x244C},  {0x244D,0x244D,0x244D}, /* 244C */
  {0x244E,0x244E,0x244E},  {0x244F,0x244F,0x244F}, /* 244E */
  {0x2450,0x2450,0x2450},  {0x2451,0x2451,0x2451}, /* 2450 */
  {0x2452,0x2452,0x2452},  {0x2453,0x2453,0x2453}, /* 2452 */
  {0x2454,0x2454,0x2454},  {0x2455,0x2455,0x2455}, /* 2454 */
  {0x2456,0x2456,0x2456},  {0x2457,0x2457,0x2457}, /* 2456 */
  {0x2458,0x2458,0x2458},  {0x2459,0x2459,0x2459}, /* 2458 */
  {0x245A,0x245A,0x245A},  {0x245B,0x245B,0x245B}, /* 245A */
  {0x245C,0x245C,0x245C},  {0x245D,0x245D,0x245D}, /* 245C */
  {0x245E,0x245E,0x245E},  {0x245F,0x245F,0x245F}, /* 245E */
  {0x2460,0x2460,0x2460},  {0x2461,0x2461,0x2461}, /* 2460 */
  {0x2462,0x2462,0x2462},  {0x2463,0x2463,0x2463}, /* 2462 */
  {0x2464,0x2464,0x2464},  {0x2465,0x2465,0x2465}, /* 2464 */
  {0x2466,0x2466,0x2466},  {0x2467,0x2467,0x2467}, /* 2466 */
  {0x2468,0x2468,0x2468},  {0x2469,0x2469,0x2469}, /* 2468 */
  {0x246A,0x246A,0x246A},  {0x246B,0x246B,0x246B}, /* 246A */
  {0x246C,0x246C,0x246C},  {0x246D,0x246D,0x246D}, /* 246C */
  {0x246E,0x246E,0x246E},  {0x246F,0x246F,0x246F}, /* 246E */
  {0x2470,0x2470,0x2470},  {0x2471,0x2471,0x2471}, /* 2470 */
  {0x2472,0x2472,0x2472},  {0x2473,0x2473,0x2473}, /* 2472 */
  {0x2474,0x2474,0x2474},  {0x2475,0x2475,0x2475}, /* 2474 */
  {0x2476,0x2476,0x2476},  {0x2477,0x2477,0x2477}, /* 2476 */
  {0x2478,0x2478,0x2478},  {0x2479,0x2479,0x2479}, /* 2478 */
  {0x247A,0x247A,0x247A},  {0x247B,0x247B,0x247B}, /* 247A */
  {0x247C,0x247C,0x247C},  {0x247D,0x247D,0x247D}, /* 247C */
  {0x247E,0x247E,0x247E},  {0x247F,0x247F,0x247F}, /* 247E */
  {0x2480,0x2480,0x2480},  {0x2481,0x2481,0x2481}, /* 2480 */
  {0x2482,0x2482,0x2482},  {0x2483,0x2483,0x2483}, /* 2482 */
  {0x2484,0x2484,0x2484},  {0x2485,0x2485,0x2485}, /* 2484 */
  {0x2486,0x2486,0x2486},  {0x2487,0x2487,0x2487}, /* 2486 */
  {0x2488,0x2488,0x2488},  {0x2489,0x2489,0x2489}, /* 2488 */
  {0x248A,0x248A,0x248A},  {0x248B,0x248B,0x248B}, /* 248A */
  {0x248C,0x248C,0x248C},  {0x248D,0x248D,0x248D}, /* 248C */
  {0x248E,0x248E,0x248E},  {0x248F,0x248F,0x248F}, /* 248E */
  {0x2490,0x2490,0x2490},  {0x2491,0x2491,0x2491}, /* 2490 */
  {0x2492,0x2492,0x2492},  {0x2493,0x2493,0x2493}, /* 2492 */
  {0x2494,0x2494,0x2494},  {0x2495,0x2495,0x2495}, /* 2494 */
  {0x2496,0x2496,0x2496},  {0x2497,0x2497,0x2497}, /* 2496 */
  {0x2498,0x2498,0x2498},  {0x2499,0x2499,0x2499}, /* 2498 */
  {0x249A,0x249A,0x249A},  {0x249B,0x249B,0x249B}, /* 249A */
  {0x249C,0x249C,0x249C},  {0x249D,0x249D,0x249D}, /* 249C */
  {0x249E,0x249E,0x249E},  {0x249F,0x249F,0x249F}, /* 249E */
  {0x24A0,0x24A0,0x24A0},  {0x24A1,0x24A1,0x24A1}, /* 24A0 */
  {0x24A2,0x24A2,0x24A2},  {0x24A3,0x24A3,0x24A3}, /* 24A2 */
  {0x24A4,0x24A4,0x24A4},  {0x24A5,0x24A5,0x24A5}, /* 24A4 */
  {0x24A6,0x24A6,0x24A6},  {0x24A7,0x24A7,0x24A7}, /* 24A6 */
  {0x24A8,0x24A8,0x24A8},  {0x24A9,0x24A9,0x24A9}, /* 24A8 */
  {0x24AA,0x24AA,0x24AA},  {0x24AB,0x24AB,0x24AB}, /* 24AA */
  {0x24AC,0x24AC,0x24AC},  {0x24AD,0x24AD,0x24AD}, /* 24AC */
  {0x24AE,0x24AE,0x24AE},  {0x24AF,0x24AF,0x24AF}, /* 24AE */
  {0x24B0,0x24B0,0x24B0},  {0x24B1,0x24B1,0x24B1}, /* 24B0 */
  {0x24B2,0x24B2,0x24B2},  {0x24B3,0x24B3,0x24B3}, /* 24B2 */
  {0x24B4,0x24B4,0x24B4},  {0x24B5,0x24B5,0x24B5}, /* 24B4 */
  {0x24B6,0x24D0,0x24B6},  {0x24B7,0x24D1,0x24B7}, /* 24B6 */
  {0x24B8,0x24D2,0x24B8},  {0x24B9,0x24D3,0x24B9}, /* 24B8 */
  {0x24BA,0x24D4,0x24BA},  {0x24BB,0x24D5,0x24BB}, /* 24BA */
  {0x24BC,0x24D6,0x24BC},  {0x24BD,0x24D7,0x24BD}, /* 24BC */
  {0x24BE,0x24D8,0x24BE},  {0x24BF,0x24D9,0x24BF}, /* 24BE */
  {0x24C0,0x24DA,0x24C0},  {0x24C1,0x24DB,0x24C1}, /* 24C0 */
  {0x24C2,0x24DC,0x24C2},  {0x24C3,0x24DD,0x24C3}, /* 24C2 */
  {0x24C4,0x24DE,0x24C4},  {0x24C5,0x24DF,0x24C5}, /* 24C4 */
  {0x24C6,0x24E0,0x24C6},  {0x24C7,0x24E1,0x24C7}, /* 24C6 */
  {0x24C8,0x24E2,0x24C8},  {0x24C9,0x24E3,0x24C9}, /* 24C8 */
  {0x24CA,0x24E4,0x24CA},  {0x24CB,0x24E5,0x24CB}, /* 24CA */
  {0x24CC,0x24E6,0x24CC},  {0x24CD,0x24E7,0x24CD}, /* 24CC */
  {0x24CE,0x24E8,0x24CE},  {0x24CF,0x24E9,0x24CF}, /* 24CE */
  {0x24B6,0x24D0,0x24B6},  {0x24B7,0x24D1,0x24B7}, /* 24D0 */
  {0x24B8,0x24D2,0x24B8},  {0x24B9,0x24D3,0x24B9}, /* 24D2 */
  {0x24BA,0x24D4,0x24BA},  {0x24BB,0x24D5,0x24BB}, /* 24D4 */
  {0x24BC,0x24D6,0x24BC},  {0x24BD,0x24D7,0x24BD}, /* 24D6 */
  {0x24BE,0x24D8,0x24BE},  {0x24BF,0x24D9,0x24BF}, /* 24D8 */
  {0x24C0,0x24DA,0x24C0},  {0x24C1,0x24DB,0x24C1}, /* 24DA */
  {0x24C2,0x24DC,0x24C2},  {0x24C3,0x24DD,0x24C3}, /* 24DC */
  {0x24C4,0x24DE,0x24C4},  {0x24C5,0x24DF,0x24C5}, /* 24DE */
  {0x24C6,0x24E0,0x24C6},  {0x24C7,0x24E1,0x24C7}, /* 24E0 */
  {0x24C8,0x24E2,0x24C8},  {0x24C9,0x24E3,0x24C9}, /* 24E2 */
  {0x24CA,0x24E4,0x24CA},  {0x24CB,0x24E5,0x24CB}, /* 24E4 */
  {0x24CC,0x24E6,0x24CC},  {0x24CD,0x24E7,0x24CD}, /* 24E6 */
  {0x24CE,0x24E8,0x24CE},  {0x24CF,0x24E9,0x24CF}, /* 24E8 */
  {0x24EA,0x24EA,0x24EA},  {0x24EB,0x24EB,0x24EB}, /* 24EA */
  {0x24EC,0x24EC,0x24EC},  {0x24ED,0x24ED,0x24ED}, /* 24EC */
  {0x24EE,0x24EE,0x24EE},  {0x24EF,0x24EF,0x24EF}, /* 24EE */
  {0x24F0,0x24F0,0x24F0},  {0x24F1,0x24F1,0x24F1}, /* 24F0 */
  {0x24F2,0x24F2,0x24F2},  {0x24F3,0x24F3,0x24F3}, /* 24F2 */
  {0x24F4,0x24F4,0x24F4},  {0x24F5,0x24F5,0x24F5}, /* 24F4 */
  {0x24F6,0x24F6,0x24F6},  {0x24F7,0x24F7,0x24F7}, /* 24F6 */
  {0x24F8,0x24F8,0x24F8},  {0x24F9,0x24F9,0x24F9}, /* 24F8 */
  {0x24FA,0x24FA,0x24FA},  {0x24FB,0x24FB,0x24FB}, /* 24FA */
  {0x24FC,0x24FC,0x24FC},  {0x24FD,0x24FD,0x24FD}, /* 24FC */
  {0x24FE,0x24FE,0x24FE},  {0x24FF,0x24FF,0x24FF}  /* 24FE */
};

static const MY_UNICASE_CHARACTER u900p02C[]= {
  {0x2C00,0x2C30,0x2C00},  {0x2C01,0x2C31,0x2C01}, /* 2C00 */
  {0x2C02,0x2C32,0x2C02},  {0x2C03,0x2C33,0x2C03}, /* 2C02 */
  {0x2C04,0x2C34,0x2C04},  {0x2C05,0x2C35,0x2C05}, /* 2C04 */
  {0x2C06,0x2C36,0x2C06},  {0x2C07,0x2C37,0x2C07}, /* 2C06 */
  {0x2C08,0x2C38,0x2C08},  {0x2C09,0x2C39,0x2C09}, /* 2C08 */
  {0x2C0A,0x2C3A,0x2C0A},  {0x2C0B,0x2C3B,0x2C0B}, /* 2C0A */
  {0x2C0C,0x2C3C,0x2C0C},  {0x2C0D,0x2C3D,0x2C0D}, /* 2C0C */
  {0x2C0E,0x2C3E,0x2C0E},  {0x2C0F,0x2C3F,0x2C0F}, /* 2C0E */
  {0x2C10,0x2C40,0x2C10},  {0x2C11,0x2C41,0x2C11}, /* 2C10 */
  {0x2C12,0x2C42,0x2C12},  {0x2C13,0x2C43,0x2C13}, /* 2C12 */
  {0x2C14,0x2C44,0x2C14},  {0x2C15,0x2C45,0x2C15}, /* 2C14 */
  {0x2C16,0x2C46,0x2C16},  {0x2C17,0x2C47,0x2C17}, /* 2C16 */
  {0x2C18,0x2C48,0x2C18},  {0x2C19,0x2C49,0x2C19}, /* 2C18 */
  {0x2C1A,0x2C4A,0x2C1A},  {0x2C1B,0x2C4B,0x2C1B}, /* 2C1A */
  {0x2C1C,0x2C4C,0x2C1C},  {0x2C1D,0x2C4D,0x2C1D}, /* 2C1C */
  {0x2C1E,0x2C4E,0x2C1E},  {0x2C1F,0x2C4F,0x2C1F}, /* 2C1E */
  {0x2C20,0x2C50,0x2C20},  {0x2C21,0x2C51,0x2C21}, /* 2C20 */
  {0x2C22,0x2C52,0x2C22},  {0x2C23,0x2C53,0x2C23}, /* 2C22 */
  {0x2C24,0x2C54,0x2C24},  {0x2C25,0x2C55,0x2C25}, /* 2C24 */
  {0x2C26,0x2C56,0x2C26},  {0x2C27,0x2C57,0x2C27}, /* 2C26 */
  {0x2C28,0x2C58,0x2C28},  {0x2C29,0x2C59,0x2C29}, /* 2C28 */
  {0x2C2A,0x2C5A,0x2C2A},  {0x2C2B,0x2C5B,0x2C2B}, /* 2C2A */
  {0x2C2C,0x2C5C,0x2C2C},  {0x2C2D,0x2C5D,0x2C2D}, /* 2C2C */
  {0x2C2E,0x2C5E,0x2C2E},  {0x2C2F,0x2C2F,0x2C2F}, /* 2C2E */
  {0x2C00,0x2C30,0x2C00},  {0x2C01,0x2C31,0x2C01}, /* 2C30 */
  {0x2C02,0x2C32,0x2C02},  {0x2C03,0x2C33,0x2C03}, /* 2C32 */
  {0x2C04,0x2C34,0x2C04},  {0x2C05,0x2C35,0x2C05}, /* 2C34 */
  {0x2C06,0x2C36,0x2C06},  {0x2C07,0x2C37,0x2C07}, /* 2C36 */
  {0x2C08,0x2C38,0x2C08},  {0x2C09,0x2C39,0x2C09}, /* 2C38 */
  {0x2C0A,0x2C3A,0x2C0A},  {0x2C0B,0x2C3B,0x2C0B}, /* 2C3A */
  {0x2C0C,0x2C3C,0x2C0C},  {0x2C0D,0x2C3D,0x2C0D}, /* 2C3C */
  {0x2C0E,0x2C3E,0x2C0E},  {0x2C0F,0x2C3F,0x2C0F}, /* 2C3E */
  {0x2C10,0x2C40,0x2C10},  {0x2C11,0x2C41,0x2C11}, /* 2C40 */
  {0x2C12,0x2C42,0x2C12},  {0x2C13,0x2C43,0x2C13}, /* 2C42 */
  {0x2C14,0x2C44,0x2C14},  {0x2C15,0x2C45,0x2C15}, /* 2C44 */
  {0x2C16,0x2C46,0x2C16},  {0x2C17,0x2C47,0x2C17}, /* 2C46 */
  {0x2C18,0x2C48,0x2C18},  {0x2C19,0x2C49,0x2C19}, /* 2C48 */
  {0x2C1A,0x2C4A,0x2C1A},  {0x2C1B,0x2C4B,0x2C1B}, /* 2C4A */
  {0x2C1C,0x2C4C,0x2C1C},  {0x2C1D,0x2C4D,0x2C1D}, /* 2C4C */
  {0x2C1E,0x2C4E,0x2C1E},  {0x2C1F,0x2C4F,0x2C1F}, /* 2C4E */
  {0x2C20,0x2C50,0x2C20},  {0x2C21,0x2C51,0x2C21}, /* 2C50 */
  {0x2C22,0x2C52,0x2C22},  {0x2C23,0x2C53,0x2C23}, /* 2C52 */
  {0x2C24,0x2C54,0x2C24},  {0x2C25,0x2C55,0x2C25}, /* 2C54 */
  {0x2C26,0x2C56,0x2C26},  {0x2C27,0x2C57,0x2C27}, /* 2C56 */
  {0x2C28,0x2C58,0x2C28},  {0x2C29,0x2C59,0x2C29}, /* 2C58 */
  {0x2C2A,0x2C5A,0x2C2A},  {0x2C2B,0x2C5B,0x2C2B}, /* 2C5A */
  {0x2C2C,0x2C5C,0x2C2C},  {0x2C2D,0x2C5D,0x2C2D}, /* 2C5C */
  {0x2C2E,0x2C5E,0x2C2E},  {0x2C5F,0x2C5F,0x2C5F}, /* 2C5E */
  {0x2C60,0x2C61,0x2C60},  {0x2C60,0x2C61,0x2C60}, /* 2C60 */
  {0x2C62,0x026B,0x2C62},  {0x2C63,0x1D7D,0x2C63}, /* 2C62 */
  {0x2C64,0x027D,0x2C64},  {0x023A,0x2C65,0x023A}, /* 2C64 */
  {0x023E,0x2C66,0x023E},  {0x2C67,0x2C68,0x2C67}, /* 2C66 */
  {0x2C67,0x2C68,0x2C67},  {0x2C69,0x2C6A,0x2C69}, /* 2C68 */
  {0x2C69,0x2C6A,0x2C69},  {0x2C6B,0x2C6C,0x2C6B}, /* 2C6A */
  {0x2C6B,0x2C6C,0x2C6B},  {0x2C6D,0x0251,0x2C6D}, /* 2C6C */
  {0x2C6E,0x0271,0x2C6E},  {0x2C6F,0x0250,0x2C6F}, /* 2C6E */
  {0x2C70,0x0252,0x2C70},  {0x2C71,0x2C71,0x2C71}, /* 2C70 */
  {0x2C72,0x2C73,0x2C72},  {0x2C72,0x2C73,0x2C72}, /* 2C72 */
  {0x2C74,0x2C74,0x2C74},  {0x2C75,0x2C76,0x2C75}, /* 2C74 */
  {0x2C75,0x2C76,0x2C75},  {0x2C77,0x2C77,0x2C77}, /* 2C76 */
  {0x2C78,0x2C78,0x2C78},  {0x2C79,0x2C79,0x2C79}, /* 2C78 */
  {0x2C7A,0x2C7A,0x2C7A},  {0x2C7B,0x2C7B,0x2C7B}, /* 2C7A */
  {0x2C7C,0x2C7C,0x2C7C},  {0x2C7D,0x2C7D,0x2C7D}, /* 2C7C */
  {0x2C7E,0x023F,0x2C7E},  {0x2C7F,0x0240,0x2C7F}, /* 2C7E */
  {0x2C80,0x2C81,0x2C80},  {0x2C80,0x2C81,0x2C80}, /* 2C80 */
  {0x2C82,0x2C83,0x2C82},  {0x2C82,0x2C83,0x2C82}, /* 2C82 */
  {0x2C84,0x2C85,0x2C84},  {0x2C84,0x2C85,0x2C84}, /* 2C84 */
  {0x2C86,0x2C87,0x2C86},  {0x2C86,0x2C87,0x2C86}, /* 2C86 */
  {0x2C88,0x2C89,0x2C88},  {0x2C88,0x2C89,0x2C88}, /* 2C88 */
  {0x2C8A,0x2C8B,0x2C8A},  {0x2C8A,0x2C8B,0x2C8A}, /* 2C8A */
  {0x2C8C,0x2C8D,0x2C8C},  {0x2C8C,0x2C8D,0x2C8C}, /* 2C8C */
  {0x2C8E,0x2C8F,0x2C8E},  {0x2C8E,0x2C8F,0x2C8E}, /* 2C8E */
  {0x2C90,0x2C91,0x2C90},  {0x2C90,0x2C91,0x2C90}, /* 2C90 */
  {0x2C92,0x2C93,0x2C92},  {0x2C92,0x2C93,0x2C92}, /* 2C92 */
  {0x2C94,0x2C95,0x2C94},  {0x2C94,0x2C95,0x2C94}, /* 2C94 */
  {0x2C96,0x2C97,0x2C96},  {0x2C96,0x2C97,0x2C96}, /* 2C96 */
  {0x2C98,0x2C99,0x2C98},  {0x2C98,0x2C99,0x2C98}, /* 2C98 */
  {0x2C9A,0x2C9B,0x2C9A},  {0x2C9A,0x2C9B,0x2C9A}, /* 2C9A */
  {0x2C9C,0x2C9D,0x2C9C},  {0x2C9C,0x2C9D,0x2C9C}, /* 2C9C */
  {0x2C9E,0x2C9F,0x2C9E},  {0x2C9E,0x2C9F,0x2C9E}, /* 2C9E */
  {0x2CA0,0x2CA1,0x2CA0},  {0x2CA0,0x2CA1,0x2CA0}, /* 2CA0 */
  {0x2CA2,0x2CA3,0x2CA2},  {0x2CA2,0x2CA3,0x2CA2}, /* 2CA2 */
  {0x2CA4,0x2CA5,0x2CA4},  {0x2CA4,0x2CA5,0x2CA4}, /* 2CA4 */
  {0x2CA6,0x2CA7,0x2CA6},  {0x2CA6,0x2CA7,0x2CA6}, /* 2CA6 */
  {0x2CA8,0x2CA9,0x2CA8},  {0x2CA8,0x2CA9,0x2CA8}, /* 2CA8 */
  {0x2CAA,0x2CAB,0x2CAA},  {0x2CAA,0x2CAB,0x2CAA}, /* 2CAA */
  {0x2CAC,0x2CAD,0x2CAC},  {0x2CAC,0x2CAD,0x2CAC}, /* 2CAC */
  {0x2CAE,0x2CAF,0x2CAE},  {0x2CAE,0x2CAF,0x2CAE}, /* 2CAE */
  {0x2CB0,0x2CB1,0x2CB0},  {0x2CB0,0x2CB1,0x2CB0}, /* 2CB0 */
  {0x2CB2,0x2CB3,0x2CB2},  {0x2CB2,0x2CB3,0x2CB2}, /* 2CB2 */
  {0x2CB4,0x2CB5,0x2CB4},  {0x2CB4,0x2CB5,0x2CB4}, /* 2CB4 */
  {0x2CB6,0x2CB7,0x2CB6},  {0x2CB6,0x2CB7,0x2CB6}, /* 2CB6 */
  {0x2CB8,0x2CB9,0x2CB8},  {0x2CB8,0x2CB9,0x2CB8}, /* 2CB8 */
  {0x2CBA,0x2CBB,0x2CBA},  {0x2CBA,0x2CBB,0x2CBA}, /* 2CBA */
  {0x2CBC,0x2CBD,0x2CBC},  {0x2CBC,0x2CBD,0x2CBC}, /* 2CBC */
  {0x2CBE,0x2CBF,0x2CBE},  {0x2CBE,0x2CBF,0x2CBE}, /* 2CBE */
  {0x2CC0,0x2CC1,0x2CC0},  {0x2CC0,0x2CC1,0x2CC0}, /* 2CC0 */
  {0x2CC2,0x2CC3,0x2CC2},  {0x2CC2,0x2CC3,0x2CC2}, /* 2CC2 */
  {0x2CC4,0x2CC5,0x2CC4},  {0x2CC4,0x2CC5,0x2CC4}, /* 2CC4 */
  {0x2CC6,0x2CC7,0x2CC6},  {0x2CC6,0x2CC7,0x2CC6}, /* 2CC6 */
  {0x2CC8,0x2CC9,0x2CC8},  {0x2CC8,0x2CC9,0x2CC8}, /* 2CC8 */
  {0x2CCA,0x2CCB,0x2CCA},  {0x2CCA,0x2CCB,0x2CCA}, /* 2CCA */
  {0x2CCC,0x2CCD,0x2CCC},  {0x2CCC,0x2CCD,0x2CCC}, /* 2CCC */
  {0x2CCE,0x2CCF,0x2CCE},  {0x2CCE,0x2CCF,0x2CCE}, /* 2CCE */
  {0x2CD0,0x2CD1,0x2CD0},  {0x2CD0,0x2CD1,0x2CD0}, /* 2CD0 */
  {0x2CD2,0x2CD3,0x2CD2},  {0x2CD2,0x2CD3,0x2CD2}, /* 2CD2 */
  {0x2CD4,0x2CD5,0x2CD4},  {0x2CD4,0x2CD5,0x2CD4}, /* 2CD4 */
  {0x2CD6,0x2CD7,0x2CD6},  {0x2CD6,0x2CD7,0x2CD6}, /* 2CD6 */
  {0x2CD8,0x2CD9,0x2CD8},  {0x2CD8,0x2CD9,0x2CD8}, /* 2CD8 */
  {0x2CDA,0x2CDB,0x2CDA},  {0x2CDA,0x2CDB,0x2CDA}, /* 2CDA */
  {0x2CDC,0x2CDD,0x2CDC},  {0x2CDC,0x2CDD,0x2CDC}, /* 2CDC */
  {0x2CDE,0x2CDF,0x2CDE},  {0x2CDE,0x2CDF,0x2CDE}, /* 2CDE */
  {0x2CE0,0x2CE1,0x2CE0},  {0x2CE0,0x2CE1,0x2CE0}, /* 2CE0 */
  {0x2CE2,0x2CE3,0x2CE2},  {0x2CE2,0x2CE3,0x2CE2}, /* 2CE2 */
  {0x2CE4,0x2CE4,0x2CE4},  {0x2CE5,0x2CE5,0x2CE5}, /* 2CE4 */
  {0x2CE6,0x2CE6,0x2CE6},  {0x2CE7,0x2CE7,0x2CE7}, /* 2CE6 */
  {0x2CE8,0x2CE8,0x2CE8},  {0x2CE9,0x2CE9,0x2CE9}, /* 2CE8 */
  {0x2CEA,0x2CEA,0x2CEA},  {0x2CEB,0x2CEC,0x2CEB}, /* 2CEA */
  {0x2CEB,0x2CEC,0x2CEB},  {0x2CED,0x2CEE,0x2CED}, /* 2CEC */
  {0x2CED,0x2CEE,0x2CED},  {0x2CEF,0x2CEF,0x2CEF}, /* 2CEE */
  {0x2CF0,0x2CF0,0x2CF0},  {0x2CF1,0x2CF1,0x2CF1}, /* 2CF0 */
  {0x2CF2,0x2CF3,0x2CF2},  {0x2CF2,0x2CF3,0x2CF2}, /* 2CF2 */
  {0x2CF4,0x2CF4,0x2CF4},  {0x2CF5,0x2CF5,0x2CF5}, /* 2CF4 */
  {0x2CF6,0x2CF6,0x2CF6},  {0x2CF7,0x2CF7,0x2CF7}, /* 2CF6 */
  {0x2CF8,0x2CF8,0x2CF8},  {0x2CF9,0x2CF9,0x2CF9}, /* 2CF8 */
  {0x2CFA,0x2CFA,0x2CFA},  {0x2CFB,0x2CFB,0x2CFB}, /* 2CFA */
  {0x2CFC,0x2CFC,0x2CFC},  {0x2CFD,0x2CFD,0x2CFD}, /* 2CFC */
  {0x2CFE,0x2CFE,0x2CFE},  {0x2CFF,0x2CFF,0x2CFF}  /* 2CFE */
};

static const MY_UNICASE_CHARACTER u900p02D[]= {
  {0x10A0,0x2D00,0x10A0},  {0x10A1,0x2D01,0x10A1}, /* 2D00 */
  {0x10A2,0x2D02,0x10A2},  {0x10A3,0x2D03,0x10A3}, /* 2D02 */
  {0x10A4,0x2D04,0x10A4},  {0x10A5,0x2D05,0x10A5}, /* 2D04 */
  {0x10A6,0x2D06,0x10A6},  {0x10A7,0x2D07,0x10A7}, /* 2D06 */
  {0x10A8,0x2D08,0x10A8},  {0x10A9,0x2D09,0x10A9}, /* 2D08 */
  {0x10AA,0x2D0A,0x10AA},  {0x10AB,0x2D0B,0x10AB}, /* 2D0A */
  {0x10AC,0x2D0C,0x10AC},  {0x10AD,0x2D0D,0x10AD}, /* 2D0C */
  {0x10AE,0x2D0E,0x10AE},  {0x10AF,0x2D0F,0x10AF}, /* 2D0E */
  {0x10B0,0x2D10,0x10B0},  {0x10B1,0x2D11,0x10B1}, /* 2D10 */
  {0x10B2,0x2D12,0x10B2},  {0x10B3,0x2D13,0x10B3}, /* 2D12 */
  {0x10B4,0x2D14,0x10B4},  {0x10B5,0x2D15,0x10B5}, /* 2D14 */
  {0x10B6,0x2D16,0x10B6},  {0x10B7,0x2D17,0x10B7}, /* 2D16 */
  {0x10B8,0x2D18,0x10B8},  {0x10B9,0x2D19,0x10B9}, /* 2D18 */
  {0x10BA,0x2D1A,0x10BA},  {0x10BB,0x2D1B,0x10BB}, /* 2D1A */
  {0x10BC,0x2D1C,0x10BC},  {0x10BD,0x2D1D,0x10BD}, /* 2D1C */
  {0x10BE,0x2D1E,0x10BE},  {0x10BF,0x2D1F,0x10BF}, /* 2D1E */
  {0x10C0,0x2D20,0x10C0},  {0x10C1,0x2D21,0x10C1}, /* 2D20 */
  {0x10C2,0x2D22,0x10C2},  {0x10C3,0x2D23,0x10C3}, /* 2D22 */
  {0x10C4,0x2D24,0x10C4},  {0x10C5,0x2D25,0x10C5}, /* 2D24 */
  {0x2D26,0x2D26,0x2D26},  {0x10C7,0x2D27,0x10C7}, /* 2D26 */
  {0x2D28,0x2D28,0x2D28},  {0x2D29,0x2D29,0x2D29}, /* 2D28 */
  {0x2D2A,0x2D2A,0x2D2A},  {0x2D2B,0x2D2B,0x2D2B}, /* 2D2A */
  {0x2D2C,0x2D2C,0x2D2C},  {0x10CD,0x2D2D,0x10CD}, /* 2D2C */
  {0x2D2E,0x2D2E,0x2D2E},  {0x2D2F,0x2D2F,0x2D2F}, /* 2D2E */
  {0x2D30,0x2D30,0x2D30},  {0x2D31,0x2D31,0x2D31}, /* 2D30 */
  {0x2D32,0x2D32,0x2D32},  {0x2D33,0x2D33,0x2D33}, /* 2D32 */
  {0x2D34,0x2D34,0x2D34},  {0x2D35,0x2D35,0x2D35}, /* 2D34 */
  {0x2D36,0x2D36,0x2D36},  {0x2D37,0x2D37,0x2D37}, /* 2D36 */
  {0x2D38,0x2D38,0x2D38},  {0x2D39,0x2D39,0x2D39}, /* 2D38 */
  {0x2D3A,0x2D3A,0x2D3A},  {0x2D3B,0x2D3B,0x2D3B}, /* 2D3A */
  {0x2D3C,0x2D3C,0x2D3C},  {0x2D3D,0x2D3D,0x2D3D}, /* 2D3C */
  {0x2D3E,0x2D3E,0x2D3E},  {0x2D3F,0x2D3F,0x2D3F}, /* 2D3E */
  {0x2D40,0x2D40,0x2D40},  {0x2D41,0x2D41,0x2D41}, /* 2D40 */
  {0x2D42,0x2D42,0x2D42},  {0x2D43,0x2D43,0x2D43}, /* 2D42 */
  {0x2D44,0x2D44,0x2D44},  {0x2D45,0x2D45,0x2D45}, /* 2D44 */
  {0x2D46,0x2D46,0x2D46},  {0x2D47,0x2D47,0x2D47}, /* 2D46 */
  {0x2D48,0x2D48,0x2D48},  {0x2D49,0x2D49,0x2D49}, /* 2D48 */
  {0x2D4A,0x2D4A,0x2D4A},  {0x2D4B,0x2D4B,0x2D4B}, /* 2D4A */
  {0x2D4C,0x2D4C,0x2D4C},  {0x2D4D,0x2D4D,0x2D4D}, /* 2D4C */
  {0x2D4E,0x2D4E,0x2D4E},  {0x2D4F,0x2D4F,0x2D4F}, /* 2D4E */
  {0x2D50,0x2D50,0x2D50},  {0x2D51,0x2D51,0x2D51}, /* 2D50 */
  {0x2D52,0x2D52,0x2D52},  {0x2D53,0x2D53,0x2D53}, /* 2D52 */
  {0x2D54,0x2D54,0x2D54},  {0x2D55,0x2D55,0x2D55}, /* 2D54 */
  {0x2D56,0x2D56,0x2D56},  {0x2D57,0x2D57,0x2D57}, /* 2D56 */
  {0x2D58,0x2D58,0x2D58},  {0x2D59,0x2D59,0x2D59}, /* 2D58 */
  {0x2D5A,0x2D5A,0x2D5A},  {0x2D5B,0x2D5B,0x2D5B}, /* 2D5A */
  {0x2D5C,0x2D5C,0x2D5C},  {0x2D5D,0x2D5D,0x2D5D}, /* 2D5C */
  {0x2D5E,0x2D5E,0x2D5E},  {0x2D5F,0x2D5F,0x2D5F}, /* 2D5E */
  {0x2D60,0x2D60,0x2D60},  {0x2D61,0x2D61,0x2D61}, /* 2D60 */
  {0x2D62,0x2D62,0x2D62},  {0x2D63,0x2D63,0x2D63}, /* 2D62 */
  {0x2D64,0x2D64,0x2D64},  {0x2D65,0x2D65,0x2D65}, /* 2D64 */
  {0x2D66,0x2D66,0x2D66},  {0x2D67,0x2D67,0x2D67}, /* 2D66 */
  {0x2D68,0x2D68,0x2D68},  {0x2D69,0x2D69,0x2D69}, /* 2D68 */
  {0x2D6A,0x2D6A,0x2D6A},  {0x2D6B,0x2D6B,0x2D6B}, /* 2D6A */
  {0x2D6C,0x2D6C,0x2D6C},  {0x2D6D,0x2D6D,0x2D6D}, /* 2D6C */
  {0x2D6E,0x2D6E,0x2D6E},  {0x2D6F,0x2D6F,0x2D6F}, /* 2D6E */
  {0x2D70,0x2D70,0x2D70},  {0x2D71,0x2D71,0x2D71}, /* 2D70 */
  {0x2D72,0x2D72,0x2D72},  {0x2D73,0x2D73,0x2D73}, /* 2D72 */
  {0x2D74,0x2D74,0x2D74},  {0x2D75,0x2D75,0x2D75}, /* 2D74 */
  {0x2D76,0x2D76,0x2D76},  {0x2D77,0x2D77,0x2D77}, /* 2D76 */
  {0x2D78,0x2D78,0x2D78},  {0x2D79,0x2D79,0x2D79}, /* 2D78 */
  {0x2D7A,0x2D7A,0x2D7A},  {0x2D7B,0x2D7B,0x2D7B}, /* 2D7A */
  {0x2D7C,0x2D7C,0x2D7C},  {0x2D7D,0x2D7D,0x2D7D}, /* 2D7C */
  {0x2D7E,0x2D7E,0x2D7E},  {0x2D7F,0x2D7F,0x2D7F}, /* 2D7E */
  {0x2D80,0x2D80,0x2D80},  {0x2D81,0x2D81,0x2D81}, /* 2D80 */
  {0x2D82,0x2D82,0x2D82},  {0x2D83,0x2D83,0x2D83}, /* 2D82 */
  {0x2D84,0x2D84,0x2D84},  {0x2D85,0x2D85,0x2D85}, /* 2D84 */
  {0x2D86,0x2D86,0x2D86},  {0x2D87,0x2D87,0x2D87}, /* 2D86 */
  {0x2D88,0x2D88,0x2D88},  {0x2D89,0x2D89,0x2D89}, /* 2D88 */
  {0x2D8A,0x2D8A,0x2D8A},  {0x2D8B,0x2D8B,0x2D8B}, /* 2D8A */
  {0x2D8C,0x2D8C,0x2D8C},  {0x2D8D,0x2D8D,0x2D8D}, /* 2D8C */
  {0x2D8E,0x2D8E,0x2D8E},  {0x2D8F,0x2D8F,0x2D8F}, /* 2D8E */
  {0x2D90,0x2D90,0x2D90},  {0x2D91,0x2D91,0x2D91}, /* 2D90 */
  {0x2D92,0x2D92,0x2D92},  {0x2D93,0x2D93,0x2D93}, /* 2D92 */
  {0x2D94,0x2D94,0x2D94},  {0x2D95,0x2D95,0x2D95}, /* 2D94 */
  {0x2D96,0x2D96,0x2D96},  {0x2D97,0x2D97,0x2D97}, /* 2D96 */
  {0x2D98,0x2D98,0x2D98},  {0x2D99,0x2D99,0x2D99}, /* 2D98 */
  {0x2D9A,0x2D9A,0x2D9A},  {0x2D9B,0x2D9B,0x2D9B}, /* 2D9A */
  {0x2D9C,0x2D9C,0x2D9C},  {0x2D9D,0x2D9D,0x2D9D}, /* 2D9C */
  {0x2D9E,0x2D9E,0x2D9E},  {0x2D9F,0x2D9F,0x2D9F}, /* 2D9E */
  {0x2DA0,0x2DA0,0x2DA0},  {0x2DA1,0x2DA1,0x2DA1}, /* 2DA0 */
  {0x2DA2,0x2DA2,0x2DA2},  {0x2DA3,0x2DA3,0x2DA3}, /* 2DA2 */
  {0x2DA4,0x2DA4,0x2DA4},  {0x2DA5,0x2DA5,0x2DA5}, /* 2DA4 */
  {0x2DA6,0x2DA6,0x2DA6},  {0x2DA7,0x2DA7,0x2DA7}, /* 2DA6 */
  {0x2DA8,0x2DA8,0x2DA8},  {0x2DA9,0x2DA9,0x2DA9}, /* 2DA8 */
  {0x2DAA,0x2DAA,0x2DAA},  {0x2DAB,0x2DAB,0x2DAB}, /* 2DAA */
  {0x2DAC,0x2DAC,0x2DAC},  {0x2DAD,0x2DAD,0x2DAD}, /* 2DAC */
  {0x2DAE,0x2DAE,0x2DAE},  {0x2DAF,0x2DAF,0x2DAF}, /* 2DAE */
  {0x2DB0,0x2DB0,0x2DB0},  {0x2DB1,0x2DB1,0x2DB1}, /* 2DB0 */
  {0x2DB2,0x2DB2,0x2DB2},  {0x2DB3,0x2DB3,0x2DB3}, /* 2DB2 */
  {0x2DB4,0x2DB4,0x2DB4},  {0x2DB5,0x2DB5,0x2DB5}, /* 2DB4 */
  {0x2DB6,0x2DB6,0x2DB6},  {0x2DB7,0x2DB7,0x2DB7}, /* 2DB6 */
  {0x2DB8,0x2DB8,0x2DB8},  {0x2DB9,0x2DB9,0x2DB9}, /* 2DB8 */
  {0x2DBA,0x2DBA,0x2DBA},  {0x2DBB,0x2DBB,0x2DBB}, /* 2DBA */
  {0x2DBC,0x2DBC,0x2DBC},  {0x2DBD,0x2DBD,0x2DBD}, /* 2DBC */
  {0x2DBE,0x2DBE,0x2DBE},  {0x2DBF,0x2DBF,0x2DBF}, /* 2DBE */
  {0x2DC0,0x2DC0,0x2DC0},  {0x2DC1,0x2DC1,0x2DC1}, /* 2DC0 */
  {0x2DC2,0x2DC2,0x2DC2},  {0x2DC3,0x2DC3,0x2DC3}, /* 2DC2 */
  {0x2DC4,0x2DC4,0x2DC4},  {0x2DC5,0x2DC5,0x2DC5}, /* 2DC4 */
  {0x2DC6,0x2DC6,0x2DC6},  {0x2DC7,0x2DC7,0x2DC7}, /* 2DC6 */
  {0x2DC8,0x2DC8,0x2DC8},  {0x2DC9,0x2DC9,0x2DC9}, /* 2DC8 */
  {0x2DCA,0x2DCA,0x2DCA},  {0x2DCB,0x2DCB,0x2DCB}, /* 2DCA */
  {0x2DCC,0x2DCC,0x2DCC},  {0x2DCD,0x2DCD,0x2DCD}, /* 2DCC */
  {0x2DCE,0x2DCE,0x2DCE},  {0x2DCF,0x2DCF,0x2DCF}, /* 2DCE */
  {0x2DD0,0x2DD0,0x2DD0},  {0x2DD1,0x2DD1,0x2DD1}, /* 2DD0 */
  {0x2DD2,0x2DD2,0x2DD2},  {0x2DD3,0x2DD3,0x2DD3}, /* 2DD2 */
  {0x2DD4,0x2DD4,0x2DD4},  {0x2DD5,0x2DD5,0x2DD5}, /* 2DD4 */
  {0x2DD6,0x2DD6,0x2DD6},  {0x2DD7,0x2DD7,0x2DD7}, /* 2DD6 */
  {0x2DD8,0x2DD8,0x2DD8},  {0x2DD9,0x2DD9,0x2DD9}, /* 2DD8 */
  {0x2DDA,0x2DDA,0x2DDA},  {0x2DDB,0x2DDB,0x2DDB}, /* 2DDA */
  {0x2DDC,0x2DDC,0x2DDC},  {0x2DDD,0x2DDD,0x2DDD}, /* 2DDC */
  {0x2DDE,0x2DDE,0x2DDE},  {0x2DDF,0x2DDF,0x2DDF}, /* 2DDE */
  {0x2DE0,0x2DE0,0x2DE0},  {0x2DE1,0x2DE1,0x2DE1}, /* 2DE0 */
  {0x2DE2,0x2DE2,0x2DE2},  {0x2DE3,0x2DE3,0x2DE3}, /* 2DE2 */
  {0x2DE4,0x2DE4,0x2DE4},  {0x2DE5,0x2DE5,0x2DE5}, /* 2DE4 */
  {0x2DE6,0x2DE6,0x2DE6},  {0x2DE7,0x2DE7,0x2DE7}, /* 2DE6 */
  {0x2DE8,0x2DE8,0x2DE8},  {0x2DE9,0x2DE9,0x2DE9}, /* 2DE8 */
  {0x2DEA,0x2DEA,0x2DEA},  {0x2DEB,0x2DEB,0x2DEB}, /* 2DEA */
  {0x2DEC,0x2DEC,0x2DEC},  {0x2DED,0x2DED,0x2DED}, /* 2DEC */
  {0x2DEE,0x2DEE,0x2DEE},  {0x2DEF,0x2DEF,0x2DEF}, /* 2DEE */
  {0x2DF0,0x2DF0,0x2DF0},  {0x2DF1,0x2DF1,0x2DF1}, /* 2DF0 */
  {0x2DF2,0x2DF2,0x2DF2},  {0x2DF3,0x2DF3,0x2DF3}, /* 2DF2 */
  {0x2DF4,0x2DF4,0x2DF4},  {0x2DF5,0x2DF5,0x2DF5}, /* 2DF4 */
  {0x2DF6,0x2DF6,0x2DF6},  {0x2DF7,0x2DF7,0x2DF7}, /* 2DF6 */
  {0x2DF8,0x2DF8,0x2DF8},  {0x2DF9,0x2DF9,0x2DF9}, /* 2DF8 */
  {0x2DFA,0x2DFA,0x2DFA},  {0x2DFB,0x2DFB,0x2DFB}, /* 2DFA */
  {0x2DFC,0x2DFC,0x2DFC},  {0x2DFD,0x2DFD,0x2DFD}, /* 2DFC */
  {0x2DFE,0x2DFE,0x2DFE},  {0x2DFF,0x2DFF,0x2DFF}  /* 2DFE */
};

static const MY_UNICASE_CHARACTER u900p0A6[]= {
  {0xA600,0xA600,0xA600},  {0xA601,0xA601,0xA601}, /* A600 */
  {0xA602,0xA602,0xA602},  {0xA603,0xA603,0xA603}, /* A602 */
  {0xA604,0xA604,0xA604},  {0xA605,0xA605,0xA605}, /* A604 */
  {0xA606,0xA606,0xA606},  {0xA607,0xA607,0xA607}, /* A606 */
  {0xA608,0xA608,0xA608},  {0xA609,0xA609,0xA609}, /* A608 */
  {0xA60A,0xA60A,0xA60A},  {0xA60B,0xA60B,0xA60B}, /* A60A */
  {0xA60C,0xA60C,0xA60C},  {0xA60D,0xA60D,0xA60D}, /* A60C */
  {0xA60E,0xA60E,0xA60E},  {0xA60F,0xA60F,0xA60F}, /* A60E */
  {0xA610,0xA610,0xA610},  {0xA611,0xA611,0xA611}, /* A610 */
  {0xA612,0xA612,0xA612},  {0xA613,0xA613,0xA613}, /* A612 */
  {0xA614,0xA614,0xA614},  {0xA615,0xA615,0xA615}, /* A614 */
  {0xA616,0xA616,0xA616},  {0xA617,0xA617,0xA617}, /* A616 */
  {0xA618,0xA618,0xA618},  {0xA619,0xA619,0xA619}, /* A618 */
  {0xA61A,0xA61A,0xA61A},  {0xA61B,0xA61B,0xA61B}, /* A61A */
  {0xA61C,0xA61C,0xA61C},  {0xA61D,0xA61D,0xA61D}, /* A61C */
  {0xA61E,0xA61E,0xA61E},  {0xA61F,0xA61F,0xA61F}, /* A61E */
  {0xA620,0xA620,0xA620},  {0xA621,0xA621,0xA621}, /* A620 */
  {0xA622,0xA622,0xA622},  {0xA623,0xA623,0xA623}, /* A622 */
  {0xA624,0xA624,0xA624},  {0xA625,0xA625,0xA625}, /* A624 */
  {0xA626,0xA626,0xA626},  {0xA627,0xA627,0xA627}, /* A626 */
  {0xA628,0xA628,0xA628},  {0xA629,0xA629,0xA629}, /* A628 */
  {0xA62A,0xA62A,0xA62A},  {0xA62B,0xA62B,0xA62B}, /* A62A */
  {0xA62C,0xA62C,0xA62C},  {0xA62D,0xA62D,0xA62D}, /* A62C */
  {0xA62E,0xA62E,0xA62E},  {0xA62F,0xA62F,0xA62F}, /* A62E */
  {0xA630,0xA630,0xA630},  {0xA631,0xA631,0xA631}, /* A630 */
  {0xA632,0xA632,0xA632},  {0xA633,0xA633,0xA633}, /* A632 */
  {0xA634,0xA634,0xA634},  {0xA635,0xA635,0xA635}, /* A634 */
  {0xA636,0xA636,0xA636},  {0xA637,0xA637,0xA637}, /* A636 */
  {0xA638,0xA638,0xA638},  {0xA639,0xA639,0xA639}, /* A638 */
  {0xA63A,0xA63A,0xA63A},  {0xA63B,0xA63B,0xA63B}, /* A63A */
  {0xA63C,0xA63C,0xA63C},  {0xA63D,0xA63D,0xA63D}, /* A63C */
  {0xA63E,0xA63E,0xA63E},  {0xA63F,0xA63F,0xA63F}, /* A63E */
  {0xA640,0xA641,0xA640},  {0xA640,0xA641,0xA640}, /* A640 */
  {0xA642,0xA643,0xA642},  {0xA642,0xA643,0xA642}, /* A642 */
  {0xA644,0xA645,0xA644},  {0xA644,0xA645,0xA644}, /* A644 */
  {0xA646,0xA647,0xA646},  {0xA646,0xA647,0xA646}, /* A646 */
  {0xA648,0xA649,0xA648},  {0xA648,0xA649,0xA648}, /* A648 */
  {0xA64A,0xA64B,0xA64A},  {0xA64A,0xA64B,0xA64A}, /* A64A */
  {0xA64C,0xA64D,0xA64C},  {0xA64C,0xA64D,0xA64C}, /* A64C */
  {0xA64E,0xA64F,0xA64E},  {0xA64E,0xA64F,0xA64E}, /* A64E */
  {0xA650,0xA651,0xA650},  {0xA650,0xA651,0xA650}, /* A650 */
  {0xA652,0xA653,0xA652},  {0xA652,0xA653,0xA652}, /* A652 */
  {0xA654,0xA655,0xA654},  {0xA654,0xA655,0xA654}, /* A654 */
  {0xA656,0xA657,0xA656},  {0xA656,0xA657,0xA656}, /* A656 */
  {0xA658,0xA659,0xA658},  {0xA658,0xA659,0xA658}, /* A658 */
  {0xA65A,0xA65B,0xA65A},  {0xA65A,0xA65B,0xA65A}, /* A65A */
  {0xA65C,0xA65D,0xA65C},  {0xA65C,0xA65D,0xA65C}, /* A65C */
  {0xA65E,0xA65F,0xA65E},  {0xA65E,0xA65F,0xA65E}, /* A65E */
  {0xA660,0xA661,0xA660},  {0xA660,0xA661,0xA660}, /* A660 */
  {0xA662,0xA663,0xA662},  {0xA662,0xA663,0xA662}, /* A662 */
  {0xA664,0xA665,0xA664},  {0xA664,0xA665,0xA664}, /* A664 */
  {0xA666,0xA667,0xA666},  {0xA666,0xA667,0xA666}, /* A666 */
  {0xA668,0xA669,0xA668},  {0xA668,0xA669,0xA668}, /* A668 */
  {0xA66A,0xA66B,0xA66A},  {0xA66A,0xA66B,0xA66A}, /* A66A */
  {0xA66C,0xA66D,0xA66C},  {0xA66C,0xA66D,0xA66C}, /* A66C */
  {0xA66E,0xA66E,0xA66E},  {0xA66F,0xA66F,0xA66F}, /* A66E */
  {0xA670,0xA670,0xA670},  {0xA671,0xA671,0xA671}, /* A670 */
  {0xA672,0xA672,0xA672},  {0xA673,0xA673,0xA673}, /* A672 */
  {0xA674,0xA674,0xA674},  {0xA675,0xA675,0xA675}, /* A674 */
  {0xA676,0xA676,0xA676},  {0xA677,0xA677,0xA677}, /* A676 */
  {0xA678,0xA678,0xA678},  {0xA679,0xA679,0xA679}, /* A678 */
  {0xA67A,0xA67A,0xA67A},  {0xA67B,0xA67B,0xA67B}, /* A67A */
  {0xA67C,0xA67C,0xA67C},  {0xA67D,0xA67D,0xA67D}, /* A67C */
  {0xA67E,0xA67E,0xA67E},  {0xA67F,0xA67F,0xA67F}, /* A67E */
  {0xA680,0xA681,0xA680},  {0xA680,0xA681,0xA680}, /* A680 */
  {0xA682,0xA683,0xA682},  {0xA682,0xA683,0xA682}, /* A682 */
  {0xA684,0xA685,0xA684},  {0xA684,0xA685,0xA684}, /* A684 */
  {0xA686,0xA687,0xA686},  {0xA686,0xA687,0xA686}, /* A686 */
  {0xA688,0xA689,0xA688},  {0xA688,0xA689,0xA688}, /* A688 */
  {0xA68A,0xA68B,0xA68A},  {0xA68A,0xA68B,0xA68A}, /* A68A */
  {0xA68C,0xA68D,0xA68C},  {0xA68C,0xA68D,0xA68C}, /* A68C */
  {0xA68E,0xA68F,0xA68E},  {0xA68E,0xA68F,0xA68E}, /* A68E */
  {0xA690,0xA691,0xA690},  {0xA690,0xA691,0xA690}, /* A690 */
  {0xA692,0xA693,0xA692},  {0xA692,0xA693,0xA692}, /* A692 */
  {0xA694,0xA695,0xA694},  {0xA694,0xA695,0xA694}, /* A694 */
  {0xA696,0xA697,0xA696},  {0xA696,0xA697,0xA696}, /* A696 */
  {0xA698,0xA699,0xA698},  {0xA698,0xA699,0xA698}, /* A698 */
  {0xA69A,0xA69B,0xA69A},  {0xA69A,0xA69B,0xA69A}, /* A69A */
  {0xA69C,0xA69C,0xA69C},  {0xA69D,0xA69D,0xA69D}, /* A69C */
  {0xA69E,0xA69E,0xA69E},  {0xA69F,0xA69F,0xA69F}, /* A69E */
  {0xA6A0,0xA6A0,0xA6A0},  {0xA6A1,0xA6A1,0xA6A1}, /* A6A0 */
  {0xA6A2,0xA6A2,0xA6A2},  {0xA6A3,0xA6A3,0xA6A3}, /* A6A2 */
  {0xA6A4,0xA6A4,0xA6A4},  {0xA6A5,0xA6A5,0xA6A5}, /* A6A4 */
  {0xA6A6,0xA6A6,0xA6A6},  {0xA6A7,0xA6A7,0xA6A7}, /* A6A6 */
  {0xA6A8,0xA6A8,0xA6A8},  {0xA6A9,0xA6A9,0xA6A9}, /* A6A8 */
  {0xA6AA,0xA6AA,0xA6AA},  {0xA6AB,0xA6AB,0xA6AB}, /* A6AA */
  {0xA6AC,0xA6AC,0xA6AC},  {0xA6AD,0xA6AD,0xA6AD}, /* A6AC */
  {0xA6AE,0xA6AE,0xA6AE},  {0xA6AF,0xA6AF,0xA6AF}, /* A6AE */
  {0xA6B0,0xA6B0,0xA6B0},  {0xA6B1,0xA6B1,0xA6B1}, /* A6B0 */
  {0xA6B2,0xA6B2,0xA6B2},  {0xA6B3,0xA6B3,0xA6B3}, /* A6B2 */
  {0xA6B4,0xA6B4,0xA6B4},  {0xA6B5,0xA6B5,0xA6B5}, /* A6B4 */
  {0xA6B6,0xA6B6,0xA6B6},  {0xA6B7,0xA6B7,0xA6B7}, /* A6B6 */
  {0xA6B8,0xA6B8,0xA6B8},  {0xA6B9,0xA6B9,0xA6B9}, /* A6B8 */
  {0xA6BA,0xA6BA,0xA6BA},  {0xA6BB,0xA6BB,0xA6BB}, /* A6BA */
  {0xA6BC,0xA6BC,0xA6BC},  {0xA6BD,0xA6BD,0xA6BD}, /* A6BC */
  {0xA6BE,0xA6BE,0xA6BE},  {0xA6BF,0xA6BF,0xA6BF}, /* A6BE */
  {0xA6C0,0xA6C0,0xA6C0},  {0xA6C1,0xA6C1,0xA6C1}, /* A6C0 */
  {0xA6C2,0xA6C2,0xA6C2},  {0xA6C3,0xA6C3,0xA6C3}, /* A6C2 */
  {0xA6C4,0xA6C4,0xA6C4},  {0xA6C5,0xA6C5,0xA6C5}, /* A6C4 */
  {0xA6C6,0xA6C6,0xA6C6},  {0xA6C7,0xA6C7,0xA6C7}, /* A6C6 */
  {0xA6C8,0xA6C8,0xA6C8},  {0xA6C9,0xA6C9,0xA6C9}, /* A6C8 */
  {0xA6CA,0xA6CA,0xA6CA},  {0xA6CB,0xA6CB,0xA6CB}, /* A6CA */
  {0xA6CC,0xA6CC,0xA6CC},  {0xA6CD,0xA6CD,0xA6CD}, /* A6CC */
  {0xA6CE,0xA6CE,0xA6CE},  {0xA6CF,0xA6CF,0xA6CF}, /* A6CE */
  {0xA6D0,0xA6D0,0xA6D0},  {0xA6D1,0xA6D1,0xA6D1}, /* A6D0 */
  {0xA6D2,0xA6D2,0xA6D2},  {0xA6D3,0xA6D3,0xA6D3}, /* A6D2 */
  {0xA6D4,0xA6D4,0xA6D4},  {0xA6D5,0xA6D5,0xA6D5}, /* A6D4 */
  {0xA6D6,0xA6D6,0xA6D6},  {0xA6D7,0xA6D7,0xA6D7}, /* A6D6 */
  {0xA6D8,0xA6D8,0xA6D8},  {0xA6D9,0xA6D9,0xA6D9}, /* A6D8 */
  {0xA6DA,0xA6DA,0xA6DA},  {0xA6DB,0xA6DB,0xA6DB}, /* A6DA */
  {0xA6DC,0xA6DC,0xA6DC},  {0xA6DD,0xA6DD,0xA6DD}, /* A6DC */
  {0xA6DE,0xA6DE,0xA6DE},  {0xA6DF,0xA6DF,0xA6DF}, /* A6DE */
  {0xA6E0,0xA6E0,0xA6E0},  {0xA6E1,0xA6E1,0xA6E1}, /* A6E0 */
  {0xA6E2,0xA6E2,0xA6E2},  {0xA6E3,0xA6E3,0xA6E3}, /* A6E2 */
  {0xA6E4,0xA6E4,0xA6E4},  {0xA6E5,0xA6E5,0xA6E5}, /* A6E4 */
  {0xA6E6,0xA6E6,0xA6E6},  {0xA6E7,0xA6E7,0xA6E7}, /* A6E6 */
  {0xA6E8,0xA6E8,0xA6E8},  {0xA6E9,0xA6E9,0xA6E9}, /* A6E8 */
  {0xA6EA,0xA6EA,0xA6EA},  {0xA6EB,0xA6EB,0xA6EB}, /* A6EA */
  {0xA6EC,0xA6EC,0xA6EC},  {0xA6ED,0xA6ED,0xA6ED}, /* A6EC */
  {0xA6EE,0xA6EE,0xA6EE},  {0xA6EF,0xA6EF,0xA6EF}, /* A6EE */
  {0xA6F0,0xA6F0,0xA6F0},  {0xA6F1,0xA6F1,0xA6F1}, /* A6F0 */
  {0xA6F2,0xA6F2,0xA6F2},  {0xA6F3,0xA6F3,0xA6F3}, /* A6F2 */
  {0xA6F4,0xA6F4,0xA6F4},  {0xA6F5,0xA6F5,0xA6F5}, /* A6F4 */
  {0xA6F6,0xA6F6,0xA6F6},  {0xA6F7,0xA6F7,0xA6F7}, /* A6F6 */
  {0xA6F8,0xA6F8,0xA6F8},  {0xA6F9,0xA6F9,0xA6F9}, /* A6F8 */
  {0xA6FA,0xA6FA,0xA6FA},  {0xA6FB,0xA6FB,0xA6FB}, /* A6FA */
  {0xA6FC,0xA6FC,0xA6FC},  {0xA6FD,0xA6FD,0xA6FD}, /* A6FC */
  {0xA6FE,0xA6FE,0xA6FE},  {0xA6FF,0xA6FF,0xA6FF}  /* A6FE */
};

static const MY_UNICASE_CHARACTER u900p0A7[]= {
  {0xA700,0xA700,0xA700},  {0xA701,0xA701,0xA701}, /* A700 */
  {0xA702,0xA702,0xA702},  {0xA703,0xA703,0xA703}, /* A702 */
  {0xA704,0xA704,0xA704},  {0xA705,0xA705,0xA705}, /* A704 */
  {0xA706,0xA706,0xA706},  {0xA707,0xA707,0xA707}, /* A706 */
  {0xA708,0xA708,0xA708},  {0xA709,0xA709,0xA709}, /* A708 */
  {0xA70A,0xA70A,0xA70A},  {0xA70B,0xA70B,0xA70B}, /* A70A */
  {0xA70C,0xA70C,0xA70C},  {0xA70D,0xA70D,0xA70D}, /* A70C */
  {0xA70E,0xA70E,0xA70E},  {0xA70F,0xA70F,0xA70F}, /* A70E */
  {0xA710,0xA710,0xA710},  {0xA711,0xA711,0xA711}, /* A710 */
  {0xA712,0xA712,0xA712},  {0xA713,0xA713,0xA713}, /* A712 */
  {0xA714,0xA714,0xA714},  {0xA715,0xA715,0xA715}, /* A714 */
  {0xA716,0xA716,0xA716},  {0xA717,0xA717,0xA717}, /* A716 */
  {0xA718,0xA718,0xA718},  {0xA719,0xA719,0xA719}, /* A718 */
  {0xA71A,0xA71A,0xA71A},  {0xA71B,0xA71B,0xA71B}, /* A71A */
  {0xA71C,0xA71C,0xA71C},  {0xA71D,0xA71D,0xA71D}, /* A71C */
  {0xA71E,0xA71E,0xA71E},  {0xA71F,0xA71F,0xA71F}, /* A71E */
  {0xA720,0xA720,0xA720},  {0xA721,0xA721,0xA721}, /* A720 */
  {0xA722,0xA723,0xA722},  {0xA722,0xA723,0xA722}, /* A722 */
  {0xA724,0xA725,0xA724},  {0xA724,0xA725,0xA724}, /* A724 */
  {0xA726,0xA727,0xA726},  {0xA726,0xA727,0xA726}, /* A726 */
  {0xA728,0xA729,0xA728},  {0xA728,0xA729,0xA728}, /* A728 */
  {0xA72A,0xA72B,0xA72A},  {0xA72A,0xA72B,0xA72A}, /* A72A */
  {0xA72C,0xA72D,0xA72C},  {0xA72C,0xA72D,0xA72C}, /* A72C */
  {0xA72E,0xA72F,0xA72E},  {0xA72E,0xA72F,0xA72E}, /* A72E */
  {0xA730,0xA730,0xA730},  {0xA731,0xA731,0xA731}, /* A730 */
  {0xA732,0xA733,0xA732},  {0xA732,0xA733,0xA732}, /* A732 */
  {0xA734,0xA735,0xA734},  {0xA734,0xA735,0xA734}, /* A734 */
  {0xA736,0xA737,0xA736},  {0xA736,0xA737,0xA736}, /* A736 */
  {0xA738,0xA739,0xA738},  {0xA738,0xA739,0xA738}, /* A738 */
  {0xA73A,0xA73B,0xA73A},  {0xA73A,0xA73B,0xA73A}, /* A73A */
  {0xA73C,0xA73D,0xA73C},  {0xA73C,0xA73D,0xA73C}, /* A73C */
  {0xA73E,0xA73F,0xA73E},  {0xA73E,0xA73F,0xA73E}, /* A73E */
  {0xA740,0xA741,0xA740},  {0xA740,0xA741,0xA740}, /* A740 */
  {0xA742,0xA743,0xA742},  {0xA742,0xA743,0xA742}, /* A742 */
  {0xA744,0xA745,0xA744},  {0xA744,0xA745,0xA744}, /* A744 */
  {0xA746,0xA747,0xA746},  {0xA746,0xA747,0xA746}, /* A746 */
  {0xA748,0xA749,0xA748},  {0xA748,0xA749,0xA748}, /* A748 */
  {0xA74A,0xA74B,0xA74A},  {0xA74A,0xA74B,0xA74A}, /* A74A */
  {0xA74C,0xA74D,0xA74C},  {0xA74C,0xA74D,0xA74C}, /* A74C */
  {0xA74E,0xA74F,0xA74E},  {0xA74E,0xA74F,0xA74E}, /* A74E */
  {0xA750,0xA751,0xA750},  {0xA750,0xA751,0xA750}, /* A750 */
  {0xA752,0xA753,0xA752},  {0xA752,0xA753,0xA752}, /* A752 */
  {0xA754,0xA755,0xA754},  {0xA754,0xA755,0xA754}, /* A754 */
  {0xA756,0xA757,0xA756},  {0xA756,0xA757,0xA756}, /* A756 */
  {0xA758,0xA759,0xA758},  {0xA758,0xA759,0xA758}, /* A758 */
  {0xA75A,0xA75B,0xA75A},  {0xA75A,0xA75B,0xA75A}, /* A75A */
  {0xA75C,0xA75D,0xA75C},  {0xA75C,0xA75D,0xA75C}, /* A75C */
  {0xA75E,0xA75F,0xA75E},  {0xA75E,0xA75F,0xA75E}, /* A75E */
  {0xA760,0xA761,0xA760},  {0xA760,0xA761,0xA760}, /* A760 */
  {0xA762,0xA763,0xA762},  {0xA762,0xA763,0xA762}, /* A762 */
  {0xA764,0xA765,0xA764},  {0xA764,0xA765,0xA764}, /* A764 */
  {0xA766,0xA767,0xA766},  {0xA766,0xA767,0xA766}, /* A766 */
  {0xA768,0xA769,0xA768},  {0xA768,0xA769,0xA768}, /* A768 */
  {0xA76A,0xA76B,0xA76A},  {0xA76A,0xA76B,0xA76A}, /* A76A */
  {0xA76C,0xA76D,0xA76C},  {0xA76C,0xA76D,0xA76C}, /* A76C */
  {0xA76E,0xA76F,0xA76E},  {0xA76E,0xA76F,0xA76E}, /* A76E */
  {0xA770,0xA770,0xA770},  {0xA771,0xA771,0xA771}, /* A770 */
  {0xA772,0xA772,0xA772},  {0xA773,0xA773,0xA773}, /* A772 */
  {0xA774,0xA774,0xA774},  {0xA775,0xA775,0xA775}, /* A774 */
  {0xA776,0xA776,0xA776},  {0xA777,0xA777,0xA777}, /* A776 */
  {0xA778,0xA778,0xA778},  {0xA779,0xA77A,0xA779}, /* A778 */
  {0xA779,0xA77A,0xA779},  {0xA77B,0xA77C,0xA77B}, /* A77A */
  {0xA77B,0xA77C,0xA77B},  {0xA77D,0x1D79,0xA77D}, /* A77C */
  {0xA77E,0xA77F,0xA77E},  {0xA77E,0xA77F,0xA77E}, /* A77E */
  {0xA780,0xA781,0xA780},  {0xA780,0xA781,0xA780}, /* A780 */
  {0xA782,0xA783,0xA782},  {0xA782,0xA783,0xA782}, /* A782 */
  {0xA784,0xA785,0xA784},  {0xA784,0xA785,0xA784}, /* A784 */
  {0xA786,0xA787,0xA786},  {0xA786,0xA787,0xA786}, /* A786 */
  {0xA788,0xA788,0xA788},  {0xA789,0xA789,0xA789}, /* A788 */
  {0xA78A,0xA78A,0xA78A},  {0xA78B,0xA78C,0xA78B}, /* A78A */
  {0xA78B,0xA78C,0xA78B},  {0xA78D,0x0265,0xA78D}, /* A78C */
  {0xA78E,0xA78E,0xA78E},  {0xA78F,0xA78F,0xA78F}, /* A78E */
  {0xA790,0xA791,0xA790},  {0xA790,0xA791,0xA790}, /* A790 */
  {0xA792,0xA793,0xA792},  {0xA792,0xA793,0xA792}, /* A792 */
  {0xA794,0xA794,0xA794},  {0xA795,0xA795,0xA795}, /* A794 */
  {0xA796,0xA797,0xA796},  {0xA796,0xA797,0xA796}, /* A796 */
  {0xA798,0xA799,0xA798},  {0xA798,0xA799,0xA798}, /* A798 */
  {0xA79A,0xA79B,0xA79A},  {0xA79A,0xA79B,0xA79A}, /* A79A */
  {0xA79C,0xA79D,0xA79C},  {0xA79C,0xA79D,0xA79C}, /* A79C */
  {0xA79E,0xA79F,0xA79E},  {0xA79E,0xA79F,0xA79E}, /* A79E */
  {0xA7A0,0xA7A1,0xA7A0},  {0xA7A0,0xA7A1,0xA7A0}, /* A7A0 */
  {0xA7A2,0xA7A3,0xA7A2},  {0xA7A2,0xA7A3,0xA7A2}, /* A7A2 */
  {0xA7A4,0xA7A5,0xA7A4},  {0xA7A4,0xA7A5,0xA7A4}, /* A7A4 */
  {0xA7A6,0xA7A7,0xA7A6},  {0xA7A6,0xA7A7,0xA7A6}, /* A7A6 */
  {0xA7A8,0xA7A9,0xA7A8},  {0xA7A8,0xA7A9,0xA7A8}, /* A7A8 */
  {0xA7AA,0x0266,0xA7AA},  {0xA7AB,0x025C,0xA7AB}, /* A7AA */
  {0xA7AC,0x0261,0xA7AC},  {0xA7AD,0x026C,0xA7AD}, /* A7AC */
  {0xA7AE,0x026A,0xA7AE},  {0xA7AF,0xA7AF,0xA7AF}, /* A7AE */
  {0xA7B0,0x029E,0xA7B0},  {0xA7B1,0x0287,0xA7B1}, /* A7B0 */
  {0xA7B2,0x029D,0xA7B2},  {0xA7B3,0xAB53,0xA7B3}, /* A7B2 */
  {0xA7B4,0xA7B5,0xA7B4},  {0xA7B4,0xA7B5,0xA7B4}, /* A7B4 */
  {0xA7B6,0xA7B7,0xA7B6},  {0xA7B6,0xA7B7,0xA7B6}, /* A7B6 */
  {0xA7B8,0xA7B8,0xA7B8},  {0xA7B9,0xA7B9,0xA7B9}, /* A7B8 */
  {0xA7BA,0xA7BA,0xA7BA},  {0xA7BB,0xA7BB,0xA7BB}, /* A7BA */
  {0xA7BC,0xA7BC,0xA7BC},  {0xA7BD,0xA7BD,0xA7BD}, /* A7BC */
  {0xA7BE,0xA7BE,0xA7BE},  {0xA7BF,0xA7BF,0xA7BF}, /* A7BE */
  {0xA7C0,0xA7C0,0xA7C0},  {0xA7C1,0xA7C1,0xA7C1}, /* A7C0 */
  {0xA7C2,0xA7C2,0xA7C2},  {0xA7C3,0xA7C3,0xA7C3}, /* A7C2 */
  {0xA7C4,0xA7C4,0xA7C4},  {0xA7C5,0xA7C5,0xA7C5}, /* A7C4 */
  {0xA7C6,0xA7C6,0xA7C6},  {0xA7C7,0xA7C7,0xA7C7}, /* A7C6 */
  {0xA7C8,0xA7C8,0xA7C8},  {0xA7C9,0xA7C9,0xA7C9}, /* A7C8 */
  {0xA7CA,0xA7CA,0xA7CA},  {0xA7CB,0xA7CB,0xA7CB}, /* A7CA */
  {0xA7CC,0xA7CC,0xA7CC},  {0xA7CD,0xA7CD,0xA7CD}, /* A7CC */
  {0xA7CE,0xA7CE,0xA7CE},  {0xA7CF,0xA7CF,0xA7CF}, /* A7CE */
  {0xA7D0,0xA7D0,0xA7D0},  {0xA7D1,0xA7D1,0xA7D1}, /* A7D0 */
  {0xA7D2,0xA7D2,0xA7D2},  {0xA7D3,0xA7D3,0xA7D3}, /* A7D2 */
  {0xA7D4,0xA7D4,0xA7D4},  {0xA7D5,0xA7D5,0xA7D5}, /* A7D4 */
  {0xA7D6,0xA7D6,0xA7D6},  {0xA7D7,0xA7D7,0xA7D7}, /* A7D6 */
  {0xA7D8,0xA7D8,0xA7D8},  {0xA7D9,0xA7D9,0xA7D9}, /* A7D8 */
  {0xA7DA,0xA7DA,0xA7DA},  {0xA7DB,0xA7DB,0xA7DB}, /* A7DA */
  {0xA7DC,0xA7DC,0xA7DC},  {0xA7DD,0xA7DD,0xA7DD}, /* A7DC */
  {0xA7DE,0xA7DE,0xA7DE},  {0xA7DF,0xA7DF,0xA7DF}, /* A7DE */
  {0xA7E0,0xA7E0,0xA7E0},  {0xA7E1,0xA7E1,0xA7E1}, /* A7E0 */
  {0xA7E2,0xA7E2,0xA7E2},  {0xA7E3,0xA7E3,0xA7E3}, /* A7E2 */
  {0xA7E4,0xA7E4,0xA7E4},  {0xA7E5,0xA7E5,0xA7E5}, /* A7E4 */
  {0xA7E6,0xA7E6,0xA7E6},  {0xA7E7,0xA7E7,0xA7E7}, /* A7E6 */
  {0xA7E8,0xA7E8,0xA7E8},  {0xA7E9,0xA7E9,0xA7E9}, /* A7E8 */
  {0xA7EA,0xA7EA,0xA7EA},  {0xA7EB,0xA7EB,0xA7EB}, /* A7EA */
  {0xA7EC,0xA7EC,0xA7EC},  {0xA7ED,0xA7ED,0xA7ED}, /* A7EC */
  {0xA7EE,0xA7EE,0xA7EE},  {0xA7EF,0xA7EF,0xA7EF}, /* A7EE */
  {0xA7F0,0xA7F0,0xA7F0},  {0xA7F1,0xA7F1,0xA7F1}, /* A7F0 */
  {0xA7F2,0xA7F2,0xA7F2},  {0xA7F3,0xA7F3,0xA7F3}, /* A7F2 */
  {0xA7F4,0xA7F4,0xA7F4},  {0xA7F5,0xA7F5,0xA7F5}, /* A7F4 */
  {0xA7F6,0xA7F6,0xA7F6},  {0xA7F7,0xA7F7,0xA7F7}, /* A7F6 */
  {0xA7F8,0xA7F8,0xA7F8},  {0xA7F9,0xA7F9,0xA7F9}, /* A7F8 */
  {0xA7FA,0xA7FA,0xA7FA},  {0xA7FB,0xA7FB,0xA7FB}, /* A7FA */
  {0xA7FC,0xA7FC,0xA7FC},  {0xA7FD,0xA7FD,0xA7FD}, /* A7FC */
  {0xA7FE,0xA7FE,0xA7FE},  {0xA7FF,0xA7FF,0xA7FF}  /* A7FE */
};

static const MY_UNICASE_CHARACTER u900p0AB[]= {
  {0xAB00,0xAB00,0xAB00},  {0xAB01,0xAB01,0xAB01}, /* AB00 */
  {0xAB02,0xAB02,0xAB02},  {0xAB03,0xAB03,0xAB03}, /* AB02 */
  {0xAB04,0xAB04,0xAB04},  {0xAB05,0xAB05,0xAB05}, /* AB04 */
  {0xAB06,0xAB06,0xAB06},  {0xAB07,0xAB07,0xAB07}, /* AB06 */
  {0xAB08,0xAB08,0xAB08},  {0xAB09,0xAB09,0xAB09}, /* AB08 */
  {0xAB0A,0xAB0A,0xAB0A},  {0xAB0B,0xAB0B,0xAB0B}, /* AB0A */
  {0xAB0C,0xAB0C,0xAB0C},  {0xAB0D,0xAB0D,0xAB0D}, /* AB0C */
  {0xAB0E,0xAB0E,0xAB0E},  {0xAB0F,0xAB0F,0xAB0F}, /* AB0E */
  {0xAB10,0xAB10,0xAB10},  {0xAB11,0xAB11,0xAB11}, /* AB10 */
  {0xAB12,0xAB12,0xAB12},  {0xAB13,0xAB13,0xAB13}, /* AB12 */
  {0xAB14,0xAB14,0xAB14},  {0xAB15,0xAB15,0xAB15}, /* AB14 */
  {0xAB16,0xAB16,0xAB16},  {0xAB17,0xAB17,0xAB17}, /* AB16 */
  {0xAB18,0xAB18,0xAB18},  {0xAB19,0xAB19,0xAB19}, /* AB18 */
  {0xAB1A,0xAB1A,0xAB1A},  {0xAB1B,0xAB1B,0xAB1B}, /* AB1A */
  {0xAB1C,0xAB1C,0xAB1C},  {0xAB1D,0xAB1D,0xAB1D}, /* AB1C */
  {0xAB1E,0xAB1E,0xAB1E},  {0xAB1F,0xAB1F,0xAB1F}, /* AB1E */
  {0xAB20,0xAB20,0xAB20},  {0xAB21,0xAB21,0xAB21}, /* AB20 */
  {0xAB22,0xAB22,0xAB22},  {0xAB23,0xAB23,0xAB23}, /* AB22 */
  {0xAB24,0xAB24,0xAB24},  {0xAB25,0xAB25,0xAB25}, /* AB24 */
  {0xAB26,0xAB26,0xAB26},  {0xAB27,0xAB27,0xAB27}, /* AB26 */
  {0xAB28,0xAB28,0xAB28},  {0xAB29,0xAB29,0xAB29}, /* AB28 */
  {0xAB2A,0xAB2A,0xAB2A},  {0xAB2B,0xAB2B,0xAB2B}, /* AB2A */
  {0xAB2C,0xAB2C,0xAB2C},  {0xAB2D,0xAB2D,0xAB2D}, /* AB2C */
  {0xAB2E,0xAB2E,0xAB2E},  {0xAB2F,0xAB2F,0xAB2F}, /* AB2E */
  {0xAB30,0xAB30,0xAB30},  {0xAB31,0xAB31,0xAB31}, /* AB30 */
  {0xAB32,0xAB32,0xAB32},  {0xAB33,0xAB33,0xAB33}, /* AB32 */
  {0xAB34,0xAB34,0xAB34},  {0xAB35,0xAB35,0xAB35}, /* AB34 */
  {0xAB36,0xAB36,0xAB36},  {0xAB37,0xAB37,0xAB37}, /* AB36 */
  {0xAB38,0xAB38,0xAB38},  {0xAB39,0xAB39,0xAB39}, /* AB38 */
  {0xAB3A,0xAB3A,0xAB3A},  {0xAB3B,0xAB3B,0xAB3B}, /* AB3A */
  {0xAB3C,0xAB3C,0xAB3C},  {0xAB3D,0xAB3D,0xAB3D}, /* AB3C */
  {0xAB3E,0xAB3E,0xAB3E},  {0xAB3F,0xAB3F,0xAB3F}, /* AB3E */
  {0xAB40,0xAB40,0xAB40},  {0xAB41,0xAB41,0xAB41}, /* AB40 */
  {0xAB42,0xAB42,0xAB42},  {0xAB43,0xAB43,0xAB43}, /* AB42 */
  {0xAB44,0xAB44,0xAB44},  {0xAB45,0xAB45,0xAB45}, /* AB44 */
  {0xAB46,0xAB46,0xAB46},  {0xAB47,0xAB47,0xAB47}, /* AB46 */
  {0xAB48,0xAB48,0xAB48},  {0xAB49,0xAB49,0xAB49}, /* AB48 */
  {0xAB4A,0xAB4A,0xAB4A},  {0xAB4B,0xAB4B,0xAB4B}, /* AB4A */
  {0xAB4C,0xAB4C,0xAB4C},  {0xAB4D,0xAB4D,0xAB4D}, /* AB4C */
  {0xAB4E,0xAB4E,0xAB4E},  {0xAB4F,0xAB4F,0xAB4F}, /* AB4E */
  {0xAB50,0xAB50,0xAB50},  {0xAB51,0xAB51,0xAB51}, /* AB50 */
  {0xAB52,0xAB52,0xAB52},  {0xA7B3,0xAB53,0xA7B3}, /* AB52 */
  {0xAB54,0xAB54,0xAB54},  {0xAB55,0xAB55,0xAB55}, /* AB54 */
  {0xAB56,0xAB56,0xAB56},  {0xAB57,0xAB57,0xAB57}, /* AB56 */
  {0xAB58,0xAB58,0xAB58},  {0xAB59,0xAB59,0xAB59}, /* AB58 */
  {0xAB5A,0xAB5A,0xAB5A},  {0xAB5B,0xAB5B,0xAB5B}, /* AB5A */
  {0xAB5C,0xAB5C,0xAB5C},  {0xAB5D,0xAB5D,0xAB5D}, /* AB5C */
  {0xAB5E,0xAB5E,0xAB5E},  {0xAB5F,0xAB5F,0xAB5F}, /* AB5E */
  {0xAB60,0xAB60,0xAB60},  {0xAB61,0xAB61,0xAB61}, /* AB60 */
  {0xAB62,0xAB62,0xAB62},  {0xAB63,0xAB63,0xAB63}, /* AB62 */
  {0xAB64,0xAB64,0xAB64},  {0xAB65,0xAB65,0xAB65}, /* AB64 */
  {0xAB66,0xAB66,0xAB66},  {0xAB67,0xAB67,0xAB67}, /* AB66 */
  {0xAB68,0xAB68,0xAB68},  {0xAB69,0xAB69,0xAB69}, /* AB68 */
  {0xAB6A,0xAB6A,0xAB6A},  {0xAB6B,0xAB6B,0xAB6B}, /* AB6A */
  {0xAB6C,0xAB6C,0xAB6C},  {0xAB6D,0xAB6D,0xAB6D}, /* AB6C */
  {0xAB6E,0xAB6E,0xAB6E},  {0xAB6F,0xAB6F,0xAB6F}, /* AB6E */
  {0x13A0,0xAB70,0x13A0},  {0x13A1,0xAB71,0x13A1}, /* AB70 */
  {0x13A2,0xAB72,0x13A2},  {0x13A3,0xAB73,0x13A3}, /* AB72 */
  {0x13A4,0xAB74,0x13A4},  {0x13A5,0xAB75,0x13A5}, /* AB74 */
  {0x13A6,0xAB76,0x13A6},  {0x13A7,0xAB77,0x13A7}, /* AB76 */
  {0x13A8,0xAB78,0x13A8},  {0x13A9,0xAB79,0x13A9}, /* AB78 */
  {0x13AA,0xAB7A,0x13AA},  {0x13AB,0xAB7B,0x13AB}, /* AB7A */
  {0x13AC,0xAB7C,0x13AC},  {0x13AD,0xAB7D,0x13AD}, /* AB7C */
  {0x13AE,0xAB7E,0x13AE},  {0x13AF,0xAB7F,0x13AF}, /* AB7E */
  {0x13B0,0xAB80,0x13B0},  {0x13B1,0xAB81,0x13B1}, /* AB80 */
  {0x13B2,0xAB82,0x13B2},  {0x13B3,0xAB83,0x13B3}, /* AB82 */
  {0x13B4,0xAB84,0x13B4},  {0x13B5,0xAB85,0x13B5}, /* AB84 */
  {0x13B6,0xAB86,0x13B6},  {0x13B7,0xAB87,0x13B7}, /* AB86 */
  {0x13B8,0xAB88,0x13B8},  {0x13B9,0xAB89,0x13B9}, /* AB88 */
  {0x13BA,0xAB8A,0x13BA},  {0x13BB,0xAB8B,0x13BB}, /* AB8A */
  {0x13BC,0xAB8C,0x13BC},  {0x13BD,0xAB8D,0x13BD}, /* AB8C */
  {0x13BE,0xAB8E,0x13BE},  {0x13BF,0xAB8F,0x13BF}, /* AB8E */
  {0x13C0,0xAB90,0x13C0},  {0x13C1,0xAB91,0x13C1}, /* AB90 */
  {0x13C2,0xAB92,0x13C2},  {0x13C3,0xAB93,0x13C3}, /* AB92 */
  {0x13C4,0xAB94,0x13C4},  {0x13C5,0xAB95,0x13C5}, /* AB94 */
  {0x13C6,0xAB96,0x13C6},  {0x13C7,0xAB97,0x13C7}, /* AB96 */
  {0x13C8,0xAB98,0x13C8},  {0x13C9,0xAB99,0x13C9}, /* AB98 */
  {0x13CA,0xAB9A,0x13CA},  {0x13CB,0xAB9B,0x13CB}, /* AB9A */
  {0x13CC,0xAB9C,0x13CC},  {0x13CD,0xAB9D,0x13CD}, /* AB9C */
  {0x13CE,0xAB9E,0x13CE},  {0x13CF,0xAB9F,0x13CF}, /* AB9E */
  {0x13D0,0xABA0,0x13D0},  {0x13D1,0xABA1,0x13D1}, /* ABA0 */
  {0x13D2,0xABA2,0x13D2},  {0x13D3,0xABA3,0x13D3}, /* ABA2 */
  {0x13D4,0xABA4,0x13D4},  {0x13D5,0xABA5,0x13D5}, /* ABA4 */
  {0x13D6,0xABA6,0x13D6},  {0x13D7,0xABA7,0x13D7}, /* ABA6 */
  {0x13D8,0xABA8,0x13D8},  {0x13D9,0xABA9,0x13D9}, /* ABA8 */
  {0x13DA,0xABAA,0x13DA},  {0x13DB,0xABAB,0x13DB}, /* ABAA */
  {0x13DC,0xABAC,0x13DC},  {0x13DD,0xABAD,0x13DD}, /* ABAC */
  {0x13DE,0xABAE,0x13DE},  {0x13DF,0xABAF,0x13DF}, /* ABAE */
  {0x13E0,0xABB0,0x13E0},  {0x13E1,0xABB1,0x13E1}, /* ABB0 */
  {0x13E2,0xABB2,0x13E2},  {0x13E3,0xABB3,0x13E3}, /* ABB2 */
  {0x13E4,0xABB4,0x13E4},  {0x13E5,0xABB5,0x13E5}, /* ABB4 */
  {0x13E6,0xABB6,0x13E6},  {0x13E7,0xABB7,0x13E7}, /* ABB6 */
  {0x13E8,0xABB8,0x13E8},  {0x13E9,0xABB9,0x13E9}, /* ABB8 */
  {0x13EA,0xABBA,0x13EA},  {0x13EB,0xABBB,0x13EB}, /* ABBA */
  {0x13EC,0xABBC,0x13EC},  {0x13ED,0xABBD,0x13ED}, /* ABBC */
  {0x13EE,0xABBE,0x13EE},  {0x13EF,0xABBF,0x13EF}, /* ABBE */
  {0xABC0,0xABC0,0xABC0},  {0xABC1,0xABC1,0xABC1}, /* ABC0 */
  {0xABC2,0xABC2,0xABC2},  {0xABC3,0xABC3,0xABC3}, /* ABC2 */
  {0xABC4,0xABC4,0xABC4},  {0xABC5,0xABC5,0xABC5}, /* ABC4 */
  {0xABC6,0xABC6,0xABC6},  {0xABC7,0xABC7,0xABC7}, /* ABC6 */
  {0xABC8,0xABC8,0xABC8},  {0xABC9,0xABC9,0xABC9}, /* ABC8 */
  {0xABCA,0xABCA,0xABCA},  {0xABCB,0xABCB,0xABCB}, /* ABCA */
  {0xABCC,0xABCC,0xABCC},  {0xABCD,0xABCD,0xABCD}, /* ABCC */
  {0xABCE,0xABCE,0xABCE},  {0xABCF,0xABCF,0xABCF}, /* ABCE */
  {0xABD0,0xABD0,0xABD0},  {0xABD1,0xABD1,0xABD1}, /* ABD0 */
  {0xABD2,0xABD2,0xABD2},  {0xABD3,0xABD3,0xABD3}, /* ABD2 */
  {0xABD4,0xABD4,0xABD4},  {0xABD5,0xABD5,0xABD5}, /* ABD4 */
  {0xABD6,0xABD6,0xABD6},  {0xABD7,0xABD7,0xABD7}, /* ABD6 */
  {0xABD8,0xABD8,0xABD8},  {0xABD9,0xABD9,0xABD9}, /* ABD8 */
  {0xABDA,0xABDA,0xABDA},  {0xABDB,0xABDB,0xABDB}, /* ABDA */
  {0xABDC,0xABDC,0xABDC},  {0xABDD,0xABDD,0xABDD}, /* ABDC */
  {0xABDE,0xABDE,0xABDE},  {0xABDF,0xABDF,0xABDF}, /* ABDE */
  {0xABE0,0xABE0,0xABE0},  {0xABE1,0xABE1,0xABE1}, /* ABE0 */
  {0xABE2,0xABE2,0xABE2},  {0xABE3,0xABE3,0xABE3}, /* ABE2 */
  {0xABE4,0xABE4,0xABE4},  {0xABE5,0xABE5,0xABE5}, /* ABE4 */
  {0xABE6,0xABE6,0xABE6},  {0xABE7,0xABE7,0xABE7}, /* ABE6 */
  {0xABE8,0xABE8,0xABE8},  {0xABE9,0xABE9,0xABE9}, /* ABE8 */
  {0xABEA,0xABEA,0xABEA},  {0xABEB,0xABEB,0xABEB}, /* ABEA */
  {0xABEC,0xABEC,0xABEC},  {0xABED,0xABED,0xABED}, /* ABEC */
  {0xABEE,0xABEE,0xABEE},  {0xABEF,0xABEF,0xABEF}, /* ABEE */
  {0xABF0,0xABF0,0xABF0},  {0xABF1,0xABF1,0xABF1}, /* ABF0 */
  {0xABF2,0xABF2,0xABF2},  {0xABF3,0xABF3,0xABF3}, /* ABF2 */
  {0xABF4,0xABF4,0xABF4},  {0xABF5,0xABF5,0xABF5}, /* ABF4 */
  {0xABF6,0xABF6,0xABF6},  {0xABF7,0xABF7,0xABF7}, /* ABF6 */
  {0xABF8,0xABF8,0xABF8},  {0xABF9,0xABF9,0xABF9}, /* ABF8 */
  {0xABFA,0xABFA,0xABFA},  {0xABFB,0xABFB,0xABFB}, /* ABFA */
  {0xABFC,0xABFC,0xABFC},  {0xABFD,0xABFD,0xABFD}, /* ABFC */
  {0xABFE,0xABFE,0xABFE},  {0xABFF,0xABFF,0xABFF}  /* ABFE */
};

static const MY_UNICASE_CHARACTER u900p0FF[]= {
  {0xFF00,0xFF00,0xFF00},  {0xFF01,0xFF01,0xFF01}, /* FF00 */
  {0xFF02,0xFF02,0xFF02},  {0xFF03,0xFF03,0xFF03}, /* FF02 */
  {0xFF04,0xFF04,0xFF04},  {0xFF05,0xFF05,0xFF05}, /* FF04 */
  {0xFF06,0xFF06,0xFF06},  {0xFF07,0xFF07,0xFF07}, /* FF06 */
  {0xFF08,0xFF08,0xFF08},  {0xFF09,0xFF09,0xFF09}, /* FF08 */
  {0xFF0A,0xFF0A,0xFF0A},  {0xFF0B,0xFF0B,0xFF0B}, /* FF0A */
  {0xFF0C,0xFF0C,0xFF0C},  {0xFF0D,0xFF0D,0xFF0D}, /* FF0C */
  {0xFF0E,0xFF0E,0xFF0E},  {0xFF0F,0xFF0F,0xFF0F}, /* FF0E */
  {0xFF10,0xFF10,0xFF10},  {0xFF11,0xFF11,0xFF11}, /* FF10 */
  {0xFF12,0xFF12,0xFF12},  {0xFF13,0xFF13,0xFF13}, /* FF12 */
  {0xFF14,0xFF14,0xFF14},  {0xFF15,0xFF15,0xFF15}, /* FF14 */
  {0xFF16,0xFF16,0xFF16},  {0xFF17,0xFF17,0xFF17}, /* FF16 */
  {0xFF18,0xFF18,0xFF18},  {0xFF19,0xFF19,0xFF19}, /* FF18 */
  {0xFF1A,0xFF1A,0xFF1A},  {0xFF1B,0xFF1B,0xFF1B}, /* FF1A */
  {0xFF1C,0xFF1C,0xFF1C},  {0xFF1D,0xFF1D,0xFF1D}, /* FF1C */
  {0xFF1E,0xFF1E,0xFF1E},  {0xFF1F,0xFF1F,0xFF1F}, /* FF1E */
  {0xFF20,0xFF20,0xFF20},  {0xFF21,0xFF41,0xFF21}, /* FF20 */
  {0xFF22,0xFF42,0xFF22},  {0xFF23,0xFF43,0xFF23}, /* FF22 */
  {0xFF24,0xFF44,0xFF24},  {0xFF25,0xFF45,0xFF25}, /* FF24 */
  {0xFF26,0xFF46,0xFF26},  {0xFF27,0xFF47,0xFF27}, /* FF26 */
  {0xFF28,0xFF48,0xFF28},  {0xFF29,0xFF49,0xFF29}, /* FF28 */
  {0xFF2A,0xFF4A,0xFF2A},  {0xFF2B,0xFF4B,0xFF2B}, /* FF2A */
  {0xFF2C,0xFF4C,0xFF2C},  {0xFF2D,0xFF4D,0xFF2D}, /* FF2C */
  {0xFF2E,0xFF4E,0xFF2E},  {0xFF2F,0xFF4F,0xFF2F}, /* FF2E */
  {0xFF30,0xFF50,0xFF30},  {0xFF31,0xFF51,0xFF31}, /* FF30 */
  {0xFF32,0xFF52,0xFF32},  {0xFF33,0xFF53,0xFF33}, /* FF32 */
  {0xFF34,0xFF54,0xFF34},  {0xFF35,0xFF55,0xFF35}, /* FF34 */
  {0xFF36,0xFF56,0xFF36},  {0xFF37,0xFF57,0xFF37}, /* FF36 */
  {0xFF38,0xFF58,0xFF38},  {0xFF39,0xFF59,0xFF39}, /* FF38 */
  {0xFF3A,0xFF5A,0xFF3A},  {0xFF3B,0xFF3B,0xFF3B}, /* FF3A */
  {0xFF3C,0xFF3C,0xFF3C},  {0xFF3D,0xFF3D,0xFF3D}, /* FF3C */
  {0xFF3E,0xFF3E,0xFF3E},  {0xFF3F,0xFF3F,0xFF3F}, /* FF3E */
  {0xFF40,0xFF40,0xFF40},  {0xFF21,0xFF41,0xFF21}, /* FF40 */
  {0xFF22,0xFF42,0xFF22},  {0xFF23,0xFF43,0xFF23}, /* FF42 */
  {0xFF24,0xFF44,0xFF24},  {0xFF25,0xFF45,0xFF25}, /* FF44 */
  {0xFF26,0xFF46,0xFF26},  {0xFF27,0xFF47,0xFF27}, /* FF46 */
  {0xFF28,0xFF48,0xFF28},  {0xFF29,0xFF49,0xFF29}, /* FF48 */
  {0xFF2A,0xFF4A,0xFF2A},  {0xFF2B,0xFF4B,0xFF2B}, /* FF4A */
  {0xFF2C,0xFF4C,0xFF2C},  {0xFF2D,0xFF4D,0xFF2D}, /* FF4C */
  {0xFF2E,0xFF4E,0xFF2E},  {0xFF2F,0xFF4F,0xFF2F}, /* FF4E */
  {0xFF30,0xFF50,0xFF30},  {0xFF31,0xFF51,0xFF31}, /* FF50 */
  {0xFF32,0xFF52,0xFF32},  {0xFF33,0xFF53,0xFF33}, /* FF52 */
  {0xFF34,0xFF54,0xFF34},  {0xFF35,0xFF55,0xFF35}, /* FF54 */
  {0xFF36,0xFF56,0xFF36},  {0xFF37,0xFF57,0xFF37}, /* FF56 */
  {0xFF38,0xFF58,0xFF38},  {0xFF39,0xFF59,0xFF39}, /* FF58 */
  {0xFF3A,0xFF5A,0xFF3A},  {0xFF5B,0xFF5B,0xFF5B}, /* FF5A */
  {0xFF5C,0xFF5C,0xFF5C},  {0xFF5D,0xFF5D,0xFF5D}, /* FF5C */
  {0xFF5E,0xFF5E,0xFF5E},  {0xFF5F,0xFF5F,0xFF5F}, /* FF5E */
  {0xFF60,0xFF60,0xFF60},  {0xFF61,0xFF61,0xFF61}, /* FF60 */
  {0xFF62,0xFF62,0xFF62},  {0xFF63,0xFF63,0xFF63}, /* FF62 */
  {0xFF64,0xFF64,0xFF64},  {0xFF65,0xFF65,0xFF65}, /* FF64 */
  {0xFF66,0xFF66,0xFF66},  {0xFF67,0xFF67,0xFF67}, /* FF66 */
  {0xFF68,0xFF68,0xFF68},  {0xFF69,0xFF69,0xFF69}, /* FF68 */
  {0xFF6A,0xFF6A,0xFF6A},  {0xFF6B,0xFF6B,0xFF6B}, /* FF6A */
  {0xFF6C,0xFF6C,0xFF6C},  {0xFF6D,0xFF6D,0xFF6D}, /* FF6C */
  {0xFF6E,0xFF6E,0xFF6E},  {0xFF6F,0xFF6F,0xFF6F}, /* FF6E */
  {0xFF70,0xFF70,0xFF70},  {0xFF71,0xFF71,0xFF71}, /* FF70 */
  {0xFF72,0xFF72,0xFF72},  {0xFF73,0xFF73,0xFF73}, /* FF72 */
  {0xFF74,0xFF74,0xFF74},  {0xFF75,0xFF75,0xFF75}, /* FF74 */
  {0xFF76,0xFF76,0xFF76},  {0xFF77,0xFF77,0xFF77}, /* FF76 */
  {0xFF78,0xFF78,0xFF78},  {0xFF79,0xFF79,0xFF79}, /* FF78 */
  {0xFF7A,0xFF7A,0xFF7A},  {0xFF7B,0xFF7B,0xFF7B}, /* FF7A */
  {0xFF7C,0xFF7C,0xFF7C},  {0xFF7D,0xFF7D,0xFF7D}, /* FF7C */
  {0xFF7E,0xFF7E,0xFF7E},  {0xFF7F,0xFF7F,0xFF7F}, /* FF7E */
  {0xFF80,0xFF80,0xFF80},  {0xFF81,0xFF81,0xFF81}, /* FF80 */
  {0xFF82,0xFF82,0xFF82},  {0xFF83,0xFF83,0xFF83}, /* FF82 */
  {0xFF84,0xFF84,0xFF84},  {0xFF85,0xFF85,0xFF85}, /* FF84 */
  {0xFF86,0xFF86,0xFF86},  {0xFF87,0xFF87,0xFF87}, /* FF86 */
  {0xFF88,0xFF88,0xFF88},  {0xFF89,0xFF89,0xFF89}, /* FF88 */
  {0xFF8A,0xFF8A,0xFF8A},  {0xFF8B,0xFF8B,0xFF8B}, /* FF8A */
  {0xFF8C,0xFF8C,0xFF8C},  {0xFF8D,0xFF8D,0xFF8D}, /* FF8C */
  {0xFF8E,0xFF8E,0xFF8E},  {0xFF8F,0xFF8F,0xFF8F}, /* FF8E */
  {0xFF90,0xFF90,0xFF90},  {0xFF91,0xFF91,0xFF91}, /* FF90 */
  {0xFF92,0xFF92,0xFF92},  {0xFF93,0xFF93,0xFF93}, /* FF92 */
  {0xFF94,0xFF94,0xFF94},  {0xFF95,0xFF95,0xFF95}, /* FF94 */
  {0xFF96,0xFF96,0xFF96},  {0xFF97,0xFF97,0xFF97}, /* FF96 */
  {0xFF98,0xFF98,0xFF98},  {0xFF99,0xFF99,0xFF99}, /* FF98 */
  {0xFF9A,0xFF9A,0xFF9A},  {0xFF9B,0xFF9B,0xFF9B}, /* FF9A */
  {0xFF9C,0xFF9C,0xFF9C},  {0xFF9D,0xFF9D,0xFF9D}, /* FF9C */
  {0xFF9E,0xFF9E,0xFF9E},  {0xFF9F,0xFF9F,0xFF9F}, /* FF9E */
  {0xFFA0,0xFFA0,0xFFA0},  {0xFFA1,0xFFA1,0xFFA1}, /* FFA0 */
  {0xFFA2,0xFFA2,0xFFA2},  {0xFFA3,0xFFA3,0xFFA3}, /* FFA2 */
  {0xFFA4,0xFFA4,0xFFA4},  {0xFFA5,0xFFA5,0xFFA5}, /* FFA4 */
  {0xFFA6,0xFFA6,0xFFA6},  {0xFFA7,0xFFA7,0xFFA7}, /* FFA6 */
  {0xFFA8,0xFFA8,0xFFA8},  {0xFFA9,0xFFA9,0xFFA9}, /* FFA8 */
  {0xFFAA,0xFFAA,0xFFAA},  {0xFFAB,0xFFAB,0xFFAB}, /* FFAA */
  {0xFFAC,0xFFAC,0xFFAC},  {0xFFAD,0xFFAD,0xFFAD}, /* FFAC */
  {0xFFAE,0xFFAE,0xFFAE},  {0xFFAF,0xFFAF,0xFFAF}, /* FFAE */
  {0xFFB0,0xFFB0,0xFFB0},  {0xFFB1,0xFFB1,0xFFB1}, /* FFB0 */
  {0xFFB2,0xFFB2,0xFFB2},  {0xFFB3,0xFFB3,0xFFB3}, /* FFB2 */
  {0xFFB4,0xFFB4,0xFFB4},  {0xFFB5,0xFFB5,0xFFB5}, /* FFB4 */
  {0xFFB6,0xFFB6,0xFFB6},  {0xFFB7,0xFFB7,0xFFB7}, /* FFB6 */
  {0xFFB8,0xFFB8,0xFFB8},  {0xFFB9,0xFFB9,0xFFB9}, /* FFB8 */
  {0xFFBA,0xFFBA,0xFFBA},  {0xFFBB,0xFFBB,0xFFBB}, /* FFBA */
  {0xFFBC,0xFFBC,0xFFBC},  {0xFFBD,0xFFBD,0xFFBD}, /* FFBC */
  {0xFFBE,0xFFBE,0xFFBE},  {0xFFBF,0xFFBF,0xFFBF}, /* FFBE */
  {0xFFC0,0xFFC0,0xFFC0},  {0xFFC1,0xFFC1,0xFFC1}, /* FFC0 */
  {0xFFC2,0xFFC2,0xFFC2},  {0xFFC3,0xFFC3,0xFFC3}, /* FFC2 */
  {0xFFC4,0xFFC4,0xFFC4},  {0xFFC5,0xFFC5,0xFFC5}, /* FFC4 */
  {0xFFC6,0xFFC6,0xFFC6},  {0xFFC7,0xFFC7,0xFFC7}, /* FFC6 */
  {0xFFC8,0xFFC8,0xFFC8},  {0xFFC9,0xFFC9,0xFFC9}, /* FFC8 */
  {0xFFCA,0xFFCA,0xFFCA},  {0xFFCB,0xFFCB,0xFFCB}, /* FFCA */
  {0xFFCC,0xFFCC,0xFFCC},  {0xFFCD,0xFFCD,0xFFCD}, /* FFCC */
  {0xFFCE,0xFFCE,0xFFCE},  {0xFFCF,0xFFCF,0xFFCF}, /* FFCE */
  {0xFFD0,0xFFD0,0xFFD0},  {0xFFD1,0xFFD1,0xFFD1}, /* FFD0 */
  {0xFFD2,0xFFD2,0xFFD2},  {0xFFD3,0xFFD3,0xFFD3}, /* FFD2 */
  {0xFFD4,0xFFD4,0xFFD4},  {0xFFD5,0xFFD5,0xFFD5}, /* FFD4 */
  {0xFFD6,0xFFD6,0xFFD6},  {0xFFD7,0xFFD7,0xFFD7}, /* FFD6 */
  {0xFFD8,0xFFD8,0xFFD8},  {0xFFD9,0xFFD9,0xFFD9}, /* FFD8 */
  {0xFFDA,0xFFDA,0xFFDA},  {0xFFDB,0xFFDB,0xFFDB}, /* FFDA */
  {0xFFDC,0xFFDC,0xFFDC},  {0xFFDD,0xFFDD,0xFFDD}, /* FFDC */
  {0xFFDE,0xFFDE,0xFFDE},  {0xFFDF,0xFFDF,0xFFDF}, /* FFDE */
  {0xFFE0,0xFFE0,0xFFE0},  {0xFFE1,0xFFE1,0xFFE1}, /* FFE0 */
  {0xFFE2,0xFFE2,0xFFE2},  {0xFFE3,0xFFE3,0xFFE3}, /* FFE2 */
  {0xFFE4,0xFFE4,0xFFE4},  {0xFFE5,0xFFE5,0xFFE5}, /* FFE4 */
  {0xFFE6,0xFFE6,0xFFE6},  {0xFFE7,0xFFE7,0xFFE7}, /* FFE6 */
  {0xFFE8,0xFFE8,0xFFE8},  {0xFFE9,0xFFE9,0xFFE9}, /* FFE8 */
  {0xFFEA,0xFFEA,0xFFEA},  {0xFFEB,0xFFEB,0xFFEB}, /* FFEA */
  {0xFFEC,0xFFEC,0xFFEC},  {0xFFED,0xFFED,0xFFED}, /* FFEC */
  {0xFFEE,0xFFEE,0xFFEE},  {0xFFEF,0xFFEF,0xFFEF}, /* FFEE */
  {0xFFF0,0xFFF0,0xFFF0},  {0xFFF1,0xFFF1,0xFFF1}, /* FFF0 */
  {0xFFF2,0xFFF2,0xFFF2},  {0xFFF3,0xFFF3,0xFFF3}, /* FFF2 */
  {0xFFF4,0xFFF4,0xFFF4},  {0xFFF5,0xFFF5,0xFFF5}, /* FFF4 */
  {0xFFF6,0xFFF6,0xFFF6},  {0xFFF7,0xFFF7,0xFFF7}, /* FFF6 */
  {0xFFF8,0xFFF8,0xFFF8},  {0xFFF9,0xFFF9,0xFFF9}, /* FFF8 */
  {0xFFFA,0xFFFA,0xFFFA},  {0xFFFB,0xFFFB,0xFFFB}, /* FFFA */
  {0xFFFC,0xFFFC,0xFFFC},  {0xFFFD,0xFFFD,0xFFFD}, /* FFFC */
  {0xFFFE,0xFFFE,0xFFFE},  {0xFFFF,0xFFFF,0xFFFF}  /* FFFE */
};

static const MY_UNICASE_CHARACTER u900p104[]= {
  {0x10400,0x10428,0x10400},  {0x10401,0x10429,0x10401}, /* 10400 */
  {0x10402,0x1042A,0x10402},  {0x10403,0x1042B,0x10403}, /* 10402 */
  {0x10404,0x1042C,0x10404},  {0x10405,0x1042D,0x10405}, /* 10404 */
  {0x10406,0x1042E,0x10406},  {0x10407,0x1042F,0x10407}, /* 10406 */
  {0x10408,0x10430,0x10408},  {0x10409,0x10431,0x10409}, /* 10408 */
  {0x1040A,0x10432,0x1040A},  {0x1040B,0x10433,0x1040B}, /* 1040A */
  {0x1040C,0x10434,0x1040C},  {0x1040D,0x10435,0x1040D}, /* 1040C */
  {0x1040E,0x10436,0x1040E},  {0x1040F,0x10437,0x1040F}, /* 1040E */
  {0x10410,0x10438,0x10410},  {0x10411,0x10439,0x10411}, /* 10410 */
  {0x10412,0x1043A,0x10412},  {0x10413,0x1043B,0x10413}, /* 10412 */
  {0x10414,0x1043C,0x10414},  {0x10415,0x1043D,0x10415}, /* 10414 */
  {0x10416,0x1043E,0x10416},  {0x10417,0x1043F,0x10417}, /* 10416 */
  {0x10418,0x10440,0x10418},  {0x10419,0x10441,0x10419}, /* 10418 */
  {0x1041A,0x10442,0x1041A},  {0x1041B,0x10443,0x1041B}, /* 1041A */
  {0x1041C,0x10444,0x1041C},  {0x1041D,0x10445,0x1041D}, /* 1041C */
  {0x1041E,0x10446,0x1041E},  {0x1041F,0x10447,0x1041F}, /* 1041E */
  {0x10420,0x10448,0x10420},  {0x10421,0x10449,0x10421}, /* 10420 */
  {0x10422,0x1044A,0x10422},  {0x10423,0x1044B,0x10423}, /* 10422 */
  {0x10424,0x1044C,0x10424},  {0x10425,0x1044D,0x10425}, /* 10424 */
  {0x10426,0x1044E,0x10426},  {0x10427,0x1044F,0x10427}, /* 10426 */
  {0x10400,0x10428,0x10400},  {0x10401,0x10429,0x10401}, /* 10428 */
  {0x10402,0x1042A,0x10402},  {0x10403,0x1042B,0x10403}, /* 1042A */
  {0x10404,0x1042C,0x10404},  {0x10405,0x1042D,0x10405}, /* 1042C */
  {0x10406,0x1042E,0x10406},  {0x10407,0x1042F,0x10407}, /* 1042E */
  {0x10408,0x10430,0x10408},  {0x10409,0x10431,0x10409}, /* 10430 */
  {0x1040A,0x10432,0x1040A},  {0x1040B,0x10433,0x1040B}, /* 10432 */
  {0x1040C,0x10434,0x1040C},  {0x1040D,0x10435,0x1040D}, /* 10434 */
  {0x1040E,0x10436,0x1040E},  {0x1040F,0x10437,0x1040F}, /* 10436 */
  {0x10410,0x10438,0x10410},  {0x10411,0x10439,0x10411}, /* 10438 */
  {0x10412,0x1043A,0x10412},  {0x10413,0x1043B,0x10413}, /* 1043A */
  {0x10414,0x1043C,0x10414},  {0x10415,0x1043D,0x10415}, /* 1043C */
  {0x10416,0x1043E,0x10416},  {0x10417,0x1043F,0x10417}, /* 1043E */
  {0x10418,0x10440,0x10418},  {0x10419,0x10441,0x10419}, /* 10440 */
  {0x1041A,0x10442,0x1041A},  {0x1041B,0x10443,0x1041B}, /* 10442 */
  {0x1041C,0x10444,0x1041C},  {0x1041D,0x10445,0x1041D}, /* 10444 */
  {0x1041E,0x10446,0x1041E},  {0x1041F,0x10447,0x1041F}, /* 10446 */
  {0x10420,0x10448,0x10420},  {0x10421,0x10449,0x10421}, /* 10448 */
  {0x10422,0x1044A,0x10422},  {0x10423,0x1044B,0x10423}, /* 1044A */
  {0x10424,0x1044C,0x10424},  {0x10425,0x1044D,0x10425}, /* 1044C */
  {0x10426,0x1044E,0x10426},  {0x10427,0x1044F,0x10427}, /* 1044E */
  {0x10450,0x10450,0x10450},  {0x10451,0x10451,0x10451}, /* 10450 */
  {0x10452,0x10452,0x10452},  {0x10453,0x10453,0x10453}, /* 10452 */
  {0x10454,0x10454,0x10454},  {0x10455,0x10455,0x10455}, /* 10454 */
  {0x10456,0x10456,0x10456},  {0x10457,0x10457,0x10457}, /* 10456 */
  {0x10458,0x10458,0x10458},  {0x10459,0x10459,0x10459}, /* 10458 */
  {0x1045A,0x1045A,0x1045A},  {0x1045B,0x1045B,0x1045B}, /* 1045A */
  {0x1045C,0x1045C,0x1045C},  {0x1045D,0x1045D,0x1045D}, /* 1045C */
  {0x1045E,0x1045E,0x1045E},  {0x1045F,0x1045F,0x1045F}, /* 1045E */
  {0x10460,0x10460,0x10460},  {0x10461,0x10461,0x10461}, /* 10460 */
  {0x10462,0x10462,0x10462},  {0x10463,0x10463,0x10463}, /* 10462 */
  {0x10464,0x10464,0x10464},  {0x10465,0x10465,0x10465}, /* 10464 */
  {0x10466,0x10466,0x10466},  {0x10467,0x10467,0x10467}, /* 10466 */
  {0x10468,0x10468,0x10468},  {0x10469,0x10469,0x10469}, /* 10468 */
  {0x1046A,0x1046A,0x1046A},  {0x1046B,0x1046B,0x1046B}, /* 1046A */
  {0x1046C,0x1046C,0x1046C},  {0x1046D,0x1046D,0x1046D}, /* 1046C */
  {0x1046E,0x1046E,0x1046E},  {0x1046F,0x1046F,0x1046F}, /* 1046E */
  {0x10470,0x10470,0x10470},  {0x10471,0x10471,0x10471}, /* 10470 */
  {0x10472,0x10472,0x10472},  {0x10473,0x10473,0x10473}, /* 10472 */
  {0x10474,0x10474,0x10474},  {0x10475,0x10475,0x10475}, /* 10474 */
  {0x10476,0x10476,0x10476},  {0x10477,0x10477,0x10477}, /* 10476 */
  {0x10478,0x10478,0x10478},  {0x10479,0x10479,0x10479}, /* 10478 */
  {0x1047A,0x1047A,0x1047A},  {0x1047B,0x1047B,0x1047B}, /* 1047A */
  {0x1047C,0x1047C,0x1047C},  {0x1047D,0x1047D,0x1047D}, /* 1047C */
  {0x1047E,0x1047E,0x1047E},  {0x1047F,0x1047F,0x1047F}, /* 1047E */
  {0x10480,0x10480,0x10480},  {0x10481,0x10481,0x10481}, /* 10480 */
  {0x10482,0x10482,0x10482},  {0x10483,0x10483,0x10483}, /* 10482 */
  {0x10484,0x10484,0x10484},  {0x10485,0x10485,0x10485}, /* 10484 */
  {0x10486,0x10486,0x10486},  {0x10487,0x10487,0x10487}, /* 10486 */
  {0x10488,0x10488,0x10488},  {0x10489,0x10489,0x10489}, /* 10488 */
  {0x1048A,0x1048A,0x1048A},  {0x1048B,0x1048B,0x1048B}, /* 1048A */
  {0x1048C,0x1048C,0x1048C},  {0x1048D,0x1048D,0x1048D}, /* 1048C */
  {0x1048E,0x1048E,0x1048E},  {0x1048F,0x1048F,0x1048F}, /* 1048E */
  {0x10490,0x10490,0x10490},  {0x10491,0x10491,0x10491}, /* 10490 */
  {0x10492,0x10492,0x10492},  {0x10493,0x10493,0x10493}, /* 10492 */
  {0x10494,0x10494,0x10494},  {0x10495,0x10495,0x10495}, /* 10494 */
  {0x10496,0x10496,0x10496},  {0x10497,0x10497,0x10497}, /* 10496 */
  {0x10498,0x10498,0x10498},  {0x10499,0x10499,0x10499}, /* 10498 */
  {0x1049A,0x1049A,0x1049A},  {0x1049B,0x1049B,0x1049B}, /* 1049A */
  {0x1049C,0x1049C,0x1049C},  {0x1049D,0x1049D,0x1049D}, /* 1049C */
  {0x1049E,0x1049E,0x1049E},  {0x1049F,0x1049F,0x1049F}, /* 1049E */
  {0x104A0,0x104A0,0x104A0},  {0x104A1,0x104A1,0x104A1}, /* 104A0 */
  {0x104A2,0x104A2,0x104A2},  {0x104A3,0x104A3,0x104A3}, /* 104A2 */
  {0x104A4,0x104A4,0x104A4},  {0x104A5,0x104A5,0x104A5}, /* 104A4 */
  {0x104A6,0x104A6,0x104A6},  {0x104A7,0x104A7,0x104A7}, /* 104A6 */
  {0x104A8,0x104A8,0x104A8},  {0x104A9,0x104A9,0x104A9}, /* 104A8 */
  {0x104AA,0x104AA,0x104AA},  {0x104AB,0x104AB,0x104AB}, /* 104AA */
  {0x104AC,0x104AC,0x104AC},  {0x104AD,0x104AD,0x104AD}, /* 104AC */
  {0x104AE,0x104AE,0x104AE},  {0x104AF,0x104AF,0x104AF}, /* 104AE */
  {0x104B0,0x104D8,0x104B0},  {0x104B1,0x104D9,0x104B1}, /* 104B0 */
  {0x104B2,0x104DA,0x104B2},  {0x104B3,0x104DB,0x104B3}, /* 104B2 */
  {0x104B4,0x104DC,0x104B4},  {0x104B5,0x104DD,0x104B5}, /* 104B4 */
  {0x104B6,0x104DE,0x104B6},  {0x104B7,0x104DF,0x104B7}, /* 104B6 */
  {0x104B8,0x104E0,0x104B8},  {0x104B9,0x104E1,0x104B9}, /* 104B8 */
  {0x104BA,0x104E2,0x104BA},  {0x104BB,0x104E3,0x104BB}, /* 104BA */
  {0x104BC,0x104E4,0x104BC},  {0x104BD,0x104E5,0x104BD}, /* 104BC */
  {0x104BE,0x104E6,0x104BE},  {0x104BF,0x104E7,0x104BF}, /* 104BE */
  {0x104C0,0x104E8,0x104C0},  {0x104C1,0x104E9,0x104C1}, /* 104C0 */
  {0x104C2,0x104EA,0x104C2},  {0x104C3,0x104EB,0x104C3}, /* 104C2 */
  {0x104C4,0x104EC,0x104C4},  {0x104C5,0x104ED,0x104C5}, /* 104C4 */
  {0x104C6,0x104EE,0x104C6},  {0x104C7,0x104EF,0x104C7}, /* 104C6 */
  {0x104C8,0x104F0,0x104C8},  {0x104C9,0x104F1,0x104C9}, /* 104C8 */
  {0x104CA,0x104F2,0x104CA},  {0x104CB,0x104F3,0x104CB}, /* 104CA */
  {0x104CC,0x104F4,0x104CC},  {0x104CD,0x104F5,0x104CD}, /* 104CC */
  {0x104CE,0x104F6,0x104CE},  {0x104CF,0x104F7,0x104CF}, /* 104CE */
  {0x104D0,0x104F8,0x104D0},  {0x104D1,0x104F9,0x104D1}, /* 104D0 */
  {0x104D2,0x104FA,0x104D2},  {0x104D3,0x104FB,0x104D3}, /* 104D2 */
  {0x104D4,0x104D4,0x104D4},  {0x104D5,0x104D5,0x104D5}, /* 104D4 */
  {0x104D6,0x104D6,0x104D6},  {0x104D7,0x104D7,0x104D7}, /* 104D6 */
  {0x104B0,0x104D8,0x104B0},  {0x104B1,0x104D9,0x104B1}, /* 104D8 */
  {0x104B2,0x104DA,0x104B2},  {0x104B3,0x104DB,0x104B3}, /* 104DA */
  {0x104B4,0x104DC,0x104B4},  {0x104B5,0x104DD,0x104B5}, /* 104DC */
  {0x104B6,0x104DE,0x104B6},  {0x104B7,0x104DF,0x104B7}, /* 104DE */
  {0x104B8,0x104E0,0x104B8},  {0x104B9,0x104E1,0x104B9}, /* 104E0 */
  {0x104BA,0x104E2,0x104BA},  {0x104BB,0x104E3,0x104BB}, /* 104E2 */
  {0x104BC,0x104E4,0x104BC},  {0x104BD,0x104E5,0x104BD}, /* 104E4 */
  {0x104BE,0x104E6,0x104BE},  {0x104BF,0x104E7,0x104BF}, /* 104E6 */
  {0x104C0,0x104E8,0x104C0},  {0x104C1,0x104E9,0x104C1}, /* 104E8 */
  {0x104C2,0x104EA,0x104C2},  {0x104C3,0x104EB,0x104C3}, /* 104EA */
  {0x104C4,0x104EC,0x104C4},  {0x104C5,0x104ED,0x104C5}, /* 104EC */
  {0x104C6,0x104EE,0x104C6},  {0x104C7,0x104EF,0x104C7}, /* 104EE */
  {0x104C8,0x104F0,0x104C8},  {0x104C9,0x104F1,0x104C9}, /* 104F0 */
  {0x104CA,0x104F2,0x104CA},  {0x104CB,0x104F3,0x104CB}, /* 104F2 */
  {0x104CC,0x104F4,0x104CC},  {0x104CD,0x104F5,0x104CD}, /* 104F4 */
  {0x104CE,0x104F6,0x104CE},  {0x104CF,0x104F7,0x104CF}, /* 104F6 */
  {0x104D0,0x104F8,0x104D0},  {0x104D1,0x104F9,0x104D1}, /* 104F8 */
  {0x104D2,0x104FA,0x104D2},  {0x104D3,0x104FB,0x104D3}, /* 104FA */
  {0x104FC,0x104FC,0x104FC},  {0x104FD,0x104FD,0x104FD}, /* 104FC */
  {0x104FE,0x104FE,0x104FE},  {0x104FF,0x104FF,0x104FF}  /* 104FE */
};

static const MY_UNICASE_CHARACTER u900p10C[]= {
  {0x10C00,0x10C00,0x10C00},  {0x10C01,0x10C01,0x10C01}, /* 10C00 */
  {0x10C02,0x10C02,0x10C02},  {0x10C03,0x10C03,0x10C03}, /* 10C02 */
  {0x10C04,0x10C04,0x10C04},  {0x10C05,0x10C05,0x10C05}, /* 10C04 */
  {0x10C06,0x10C06,0x10C06},  {0x10C07,0x10C07,0x10C07}, /* 10C06 */
  {0x10C08,0x10C08,0x10C08},  {0x10C09,0x10C09,0x10C09}, /* 10C08 */
  {0x10C0A,0x10C0A,0x10C0A},  {0x10C0B,0x10C0B,0x10C0B}, /* 10C0A */
  {0x10C0C,0x10C0C,0x10C0C},  {0x10C0D,0x10C0D,0x10C0D}, /* 10C0C */
  {0x10C0E,0x10C0E,0x10C0E},  {0x10C0F,0x10C0F,0x10C0F}, /* 10C0E */
  {0x10C10,0x10C10,0x10C10},  {0x10C11,0x10C11,0x10C11}, /* 10C10 */
  {0x10C12,0x10C12,0x10C12},  {0x10C13,0x10C13,0x10C13}, /* 10C12 */
  {0x10C14,0x10C14,0x10C14},  {0x10C15,0x10C15,0x10C15}, /* 10C14 */
  {0x10C16,0x10C16,0x10C16},  {0x10C17,0x10C17,0x10C17}, /* 10C16 */
  {0x10C18,0x10C18,0x10C18},  {0x10C19,0x10C19,0x10C19}, /* 10C18 */
  {0x10C1A,0x10C1A,0x10C1A},  {0x10C1B,0x10C1B,0x10C1B}, /* 10C1A */
  {0x10C1C,0x10C1C,0x10C1C},  {0x10C1D,0x10C1D,0x10C1D}, /* 10C1C */
  {0x10C1E,0x10C1E,0x10C1E},  {0x10C1F,0x10C1F,0x10C1F}, /* 10C1E */
  {0x10C20,0x10C20,0x10C20},  {0x10C21,0x10C21,0x10C21}, /* 10C20 */
  {0x10C22,0x10C22,0x10C22},  {0x10C23,0x10C23,0x10C23}, /* 10C22 */
  {0x10C24,0x10C24,0x10C24},  {0x10C25,0x10C25,0x10C25}, /* 10C24 */
  {0x10C26,0x10C26,0x10C26},  {0x10C27,0x10C27,0x10C27}, /* 10C26 */
  {0x10C28,0x10C28,0x10C28},  {0x10C29,0x10C29,0x10C29}, /* 10C28 */
  {0x10C2A,0x10C2A,0x10C2A},  {0x10C2B,0x10C2B,0x10C2B}, /* 10C2A */
  {0x10C2C,0x10C2C,0x10C2C},  {0x10C2D,0x10C2D,0x10C2D}, /* 10C2C */
  {0x10C2E,0x10C2E,0x10C2E},  {0x10C2F,0x10C2F,0x10C2F}, /* 10C2E */
  {0x10C30,0x10C30,0x10C30},  {0x10C31,0x10C31,0x10C31}, /* 10C30 */
  {0x10C32,0x10C32,0x10C32},  {0x10C33,0x10C33,0x10C33}, /* 10C32 */
  {0x10C34,0x10C34,0x10C34},  {0x10C35,0x10C35,0x10C35}, /* 10C34 */
  {0x10C36,0x10C36,0x10C36},  {0x10C37,0x10C37,0x10C37}, /* 10C36 */
  {0x10C38,0x10C38,0x10C38},  {0x10C39,0x10C39,0x10C39}, /* 10C38 */
  {0x10C3A,0x10C3A,0x10C3A},  {0x10C3B,0x10C3B,0x10C3B}, /* 10C3A */
  {0x10C3C,0x10C3C,0x10C3C},  {0x10C3D,0x10C3D,0x10C3D}, /* 10C3C */
  {0x10C3E,0x10C3E,0x10C3E},  {0x10C3F,0x10C3F,0x10C3F}, /* 10C3E */
  {0x10C40,0x10C40,0x10C40},  {0x10C41,0x10C41,0x10C41}, /* 10C40 */
  {0x10C42,0x10C42,0x10C42},  {0x10C43,0x10C43,0x10C43}, /* 10C42 */
  {0x10C44,0x10C44,0x10C44},  {0x10C45,0x10C45,0x10C45}, /* 10C44 */
  {0x10C46,0x10C46,0x10C46},  {0x10C47,0x10C47,0x10C47}, /* 10C46 */
  {0x10C48,0x10C48,0x10C48},  {0x10C49,0x10C49,0x10C49}, /* 10C48 */
  {0x10C4A,0x10C4A,0x10C4A},  {0x10C4B,0x10C4B,0x10C4B}, /* 10C4A */
  {0x10C4C,0x10C4C,0x10C4C},  {0x10C4D,0x10C4D,0x10C4D}, /* 10C4C */
  {0x10C4E,0x10C4E,0x10C4E},  {0x10C4F,0x10C4F,0x10C4F}, /* 10C4E */
  {0x10C50,0x10C50,0x10C50},  {0x10C51,0x10C51,0x10C51}, /* 10C50 */
  {0x10C52,0x10C52,0x10C52},  {0x10C53,0x10C53,0x10C53}, /* 10C52 */
  {0x10C54,0x10C54,0x10C54},  {0x10C55,0x10C55,0x10C55}, /* 10C54 */
  {0x10C56,0x10C56,0x10C56},  {0x10C57,0x10C57,0x10C57}, /* 10C56 */
  {0x10C58,0x10C58,0x10C58},  {0x10C59,0x10C59,0x10C59}, /* 10C58 */
  {0x10C5A,0x10C5A,0x10C5A},  {0x10C5B,0x10C5B,0x10C5B}, /* 10C5A */
  {0x10C5C,0x10C5C,0x10C5C},  {0x10C5D,0x10C5D,0x10C5D}, /* 10C5C */
  {0x10C5E,0x10C5E,0x10C5E},  {0x10C5F,0x10C5F,0x10C5F}, /* 10C5E */
  {0x10C60,0x10C60,0x10C60},  {0x10C61,0x10C61,0x10C61}, /* 10C60 */
  {0x10C62,0x10C62,0x10C62},  {0x10C63,0x10C63,0x10C63}, /* 10C62 */
  {0x10C64,0x10C64,0x10C64},  {0x10C65,0x10C65,0x10C65}, /* 10C64 */
  {0x10C66,0x10C66,0x10C66},  {0x10C67,0x10C67,0x10C67}, /* 10C66 */
  {0x10C68,0x10C68,0x10C68},  {0x10C69,0x10C69,0x10C69}, /* 10C68 */
  {0x10C6A,0x10C6A,0x10C6A},  {0x10C6B,0x10C6B,0x10C6B}, /* 10C6A */
  {0x10C6C,0x10C6C,0x10C6C},  {0x10C6D,0x10C6D,0x10C6D}, /* 10C6C */
  {0x10C6E,0x10C6E,0x10C6E},  {0x10C6F,0x10C6F,0x10C6F}, /* 10C6E */
  {0x10C70,0x10C70,0x10C70},  {0x10C71,0x10C71,0x10C71}, /* 10C70 */
  {0x10C72,0x10C72,0x10C72},  {0x10C73,0x10C73,0x10C73}, /* 10C72 */
  {0x10C74,0x10C74,0x10C74},  {0x10C75,0x10C75,0x10C75}, /* 10C74 */
  {0x10C76,0x10C76,0x10C76},  {0x10C77,0x10C77,0x10C77}, /* 10C76 */
  {0x10C78,0x10C78,0x10C78},  {0x10C79,0x10C79,0x10C79}, /* 10C78 */
  {0x10C7A,0x10C7A,0x10C7A},  {0x10C7B,0x10C7B,0x10C7B}, /* 10C7A */
  {0x10C7C,0x10C7C,0x10C7C},  {0x10C7D,0x10C7D,0x10C7D}, /* 10C7C */
  {0x10C7E,0x10C7E,0x10C7E},  {0x10C7F,0x10C7F,0x10C7F}, /* 10C7E */
  {0x10C80,0x10CC0,0x10C80},  {0x10C81,0x10CC1,0x10C81}, /* 10C80 */
  {0x10C82,0x10CC2,0x10C82},  {0x10C83,0x10CC3,0x10C83}, /* 10C82 */
  {0x10C84,0x10CC4,0x10C84},  {0x10C85,0x10CC5,0x10C85}, /* 10C84 */
  {0x10C86,0x10CC6,0x10C86},  {0x10C87,0x10CC7,0x10C87}, /* 10C86 */
  {0x10C88,0x10CC8,0x10C88},  {0x10C89,0x10CC9,0x10C89}, /* 10C88 */
  {0x10C8A,0x10CCA,0x10C8A},  {0x10C8B,0x10CCB,0x10C8B}, /* 10C8A */
  {0x10C8C,0x10CCC,0x10C8C},  {0x10C8D,0x10CCD,0x10C8D}, /* 10C8C */
  {0x10C8E,0x10CCE,0x10C8E},  {0x10C8F,0x10CCF,0x10C8F}, /* 10C8E */
  {0x10C90,0x10CD0,0x10C90},  {0x10C91,0x10CD1,0x10C91}, /* 10C90 */
  {0x10C92,0x10CD2,0x10C92},  {0x10C93,0x10CD3,0x10C93}, /* 10C92 */
  {0x10C94,0x10CD4,0x10C94},  {0x10C95,0x10CD5,0x10C95}, /* 10C94 */
  {0x10C96,0x10CD6,0x10C96},  {0x10C97,0x10CD7,0x10C97}, /* 10C96 */
  {0x10C98,0x10CD8,0x10C98},  {0x10C99,0x10CD9,0x10C99}, /* 10C98 */
  {0x10C9A,0x10CDA,0x10C9A},  {0x10C9B,0x10CDB,0x10C9B}, /* 10C9A */
  {0x10C9C,0x10CDC,0x10C9C},  {0x10C9D,0x10CDD,0x10C9D}, /* 10C9C */
  {0x10C9E,0x10CDE,0x10C9E},  {0x10C9F,0x10CDF,0x10C9F}, /* 10C9E */
  {0x10CA0,0x10CE0,0x10CA0},  {0x10CA1,0x10CE1,0x10CA1}, /* 10CA0 */
  {0x10CA2,0x10CE2,0x10CA2},  {0x10CA3,0x10CE3,0x10CA3}, /* 10CA2 */
  {0x10CA4,0x10CE4,0x10CA4},  {0x10CA5,0x10CE5,0x10CA5}, /* 10CA4 */
  {0x10CA6,0x10CE6,0x10CA6},  {0x10CA7,0x10CE7,0x10CA7}, /* 10CA6 */
  {0x10CA8,0x10CE8,0x10CA8},  {0x10CA9,0x10CE9,0x10CA9}, /* 10CA8 */
  {0x10CAA,0x10CEA,0x10CAA},  {0x10CAB,0x10CEB,0x10CAB}, /* 10CAA */
  {0x10CAC,0x10CEC,0x10CAC},  {0x10CAD,0x10CED,0x10CAD}, /* 10CAC */
  {0x10CAE,0x10CEE,0x10CAE},  {0x10CAF,0x10CEF,0x10CAF}, /* 10CAE */
  {0x10CB0,0x10CF0,0x10CB0},  {0x10CB1,0x10CF1,0x10CB1}, /* 10CB0 */
  {0x10CB2,0x10CF2,0x10CB2},  {0x10CB3,0x10CB3,0x10CB3}, /* 10CB2 */
  {0x10CB4,0x10CB4,0x10CB4},  {0x10CB5,0x10CB5,0x10CB5}, /* 10CB4 */
  {0x10CB6,0x10CB6,0x10CB6},  {0x10CB7,0x10CB7,0x10CB7}, /* 10CB6 */
  {0x10CB8,0x10CB8,0x10CB8},  {0x10CB9,0x10CB9,0x10CB9}, /* 10CB8 */
  {0x10CBA,0x10CBA,0x10CBA},  {0x10CBB,0x10CBB,0x10CBB}, /* 10CBA */
  {0x10CBC,0x10CBC,0x10CBC},  {0x10CBD,0x10CBD,0x10CBD}, /* 10CBC */
  {0x10CBE,0x10CBE,0x10CBE},  {0x10CBF,0x10CBF,0x10CBF}, /* 10CBE */
  {0x10C80,0x10CC0,0x10C80},  {0x10C81,0x10CC1,0x10C81}, /* 10CC0 */
  {0x10C82,0x10CC2,0x10C82},  {0x10C83,0x10CC3,0x10C83}, /* 10CC2 */
  {0x10C84,0x10CC4,0x10C84},  {0x10C85,0x10CC5,0x10C85}, /* 10CC4 */
  {0x10C86,0x10CC6,0x10C86},  {0x10C87,0x10CC7,0x10C87}, /* 10CC6 */
  {0x10C88,0x10CC8,0x10C88},  {0x10C89,0x10CC9,0x10C89}, /* 10CC8 */
  {0x10C8A,0x10CCA,0x10C8A},  {0x10C8B,0x10CCB,0x10C8B}, /* 10CCA */
  {0x10C8C,0x10CCC,0x10C8C},  {0x10C8D,0x10CCD,0x10C8D}, /* 10CCC */
  {0x10C8E,0x10CCE,0x10C8E},  {0x10C8F,0x10CCF,0x10C8F}, /* 10CCE */
  {0x10C90,0x10CD0,0x10C90},  {0x10C91,0x10CD1,0x10C91}, /* 10CD0 */
  {0x10C92,0x10CD2,0x10C92},  {0x10C93,0x10CD3,0x10C93}, /* 10CD2 */
  {0x10C94,0x10CD4,0x10C94},  {0x10C95,0x10CD5,0x10C95}, /* 10CD4 */
  {0x10C96,0x10CD6,0x10C96},  {0x10C97,0x10CD7,0x10C97}, /* 10CD6 */
  {0x10C98,0x10CD8,0x10C98},  {0x10C99,0x10CD9,0x10C99}, /* 10CD8 */
  {0x10C9A,0x10CDA,0x10C9A},  {0x10C9B,0x10CDB,0x10C9B}, /* 10CDA */
  {0x10C9C,0x10CDC,0x10C9C},  {0x10C9D,0x10CDD,0x10C9D}, /* 10CDC */
  {0x10C9E,0x10CDE,0x10C9E},  {0x10C9F,0x10CDF,0x10C9F}, /* 10CDE */
  {0x10CA0,0x10CE0,0x10CA0},  {0x10CA1,0x10CE1,0x10CA1}, /* 10CE0 */
  {0x10CA2,0x10CE2,0x10CA2},  {0x10CA3,0x10CE3,0x10CA3}, /* 10CE2 */
  {0x10CA4,0x10CE4,0x10CA4},  {0x10CA5,0x10CE5,0x10CA5}, /* 10CE4 */
  {0x10CA6,0x10CE6,0x10CA6},  {0x10CA7,0x10CE7,0x10CA7}, /* 10CE6 */
  {0x10CA8,0x10CE8,0x10CA8},  {0x10CA9,0x10CE9,0x10CA9}, /* 10CE8 */
  {0x10CAA,0x10CEA,0x10CAA},  {0x10CAB,0x10CEB,0x10CAB}, /* 10CEA */
  {0x10CAC,0x10CEC,0x10CAC},  {0x10CAD,0x10CED,0x10CAD}, /* 10CEC */
  {0x10CAE,0x10CEE,0x10CAE},  {0x10CAF,0x10CEF,0x10CAF}, /* 10CEE */
  {0x10CB0,0x10CF0,0x10CB0},  {0x10CB1,0x10CF1,0x10CB1}, /* 10CF0 */
  {0x10CB2,0x10CF2,0x10CB2},  {0x10CF3,0x10CF3,0x10CF3}, /* 10CF2 */
  {0x10CF4,0x10CF4,0x10CF4},  {0x10CF5,0x10CF5,0x10CF5}, /* 10CF4 */
  {0x10CF6,0x10CF6,0x10CF6},  {0x10CF7,0x10CF7,0x10CF7}, /* 10CF6 */
  {0x10CF8,0x10CF8,0x10CF8},  {0x10CF9,0x10CF9,0x10CF9}, /* 10CF8 */
  {0x10CFA,0x10CFA,0x10CFA},  {0x10CFB,0x10CFB,0x10CFB}, /* 10CFA */
  {0x10CFC,0x10CFC,0x10CFC},  {0x10CFD,0x10CFD,0x10CFD}, /* 10CFC */
  {0x10CFE,0x10CFE,0x10CFE},  {0x10CFF,0x10CFF,0x10CFF}  /* 10CFE */
};

static const MY_UNICASE_CHARACTER u900p118[]= {
  {0x11800,0x11800,0x11800},  {0x11801,0x11801,0x11801}, /* 11800 */
  {0x11802,0x11802,0x11802},  {0x11803,0x11803,0x11803}, /* 11802 */
  {0x11804,0x11804,0x11804},  {0x11805,0x11805,0x11805}, /* 11804 */
  {0x11806,0x11806,0x11806},  {0x11807,0x11807,0x11807}, /* 11806 */
  {0x11808,0x11808,0x11808},  {0x11809,0x11809,0x11809}, /* 11808 */
  {0x1180A,0x1180A,0x1180A},  {0x1180B,0x1180B,0x1180B}, /* 1180A */
  {0x1180C,0x1180C,0x1180C},  {0x1180D,0x1180D,0x1180D}, /* 1180C */
  {0x1180E,0x1180E,0x1180E},  {0x1180F,0x1180F,0x1180F}, /* 1180E */
  {0x11810,0x11810,0x11810},  {0x11811,0x11811,0x11811}, /* 11810 */
  {0x11812,0x11812,0x11812},  {0x11813,0x11813,0x11813}, /* 11812 */
  {0x11814,0x11814,0x11814},  {0x11815,0x11815,0x11815}, /* 11814 */
  {0x11816,0x11816,0x11816},  {0x11817,0x11817,0x11817}, /* 11816 */
  {0x11818,0x11818,0x11818},  {0x11819,0x11819,0x11819}, /* 11818 */
  {0x1181A,0x1181A,0x1181A},  {0x1181B,0x1181B,0x1181B}, /* 1181A */
  {0x1181C,0x1181C,0x1181C},  {0x1181D,0x1181D,0x1181D}, /* 1181C */
  {0x1181E,0x1181E,0x1181E},  {0x1181F,0x1181F,0x1181F}, /* 1181E */
  {0x11820,0x11820,0x11820},  {0x11821,0x11821,0x11821}, /* 11820 */
  {0x11822,0x11822,0x11822},  {0x11823,0x11823,0x11823}, /* 11822 */
  {0x11824,0x11824,0x11824},  {0x11825,0x11825,0x11825}, /* 11824 */
  {0x11826,0x11826,0x11826},  {0x11827,0x11827,0x11827}, /* 11826 */
  {0x11828,0x11828,0x11828},  {0x11829,0x11829,0x11829}, /* 11828 */
  {0x1182A,0x1182A,0x1182A},  {0x1182B,0x1182B,0x1182B}, /* 1182A */
  {0x1182C,0x1182C,0x1182C},  {0x1182D,0x1182D,0x1182D}, /* 1182C */
  {0x1182E,0x1182E,0x1182E},  {0x1182F,0x1182F,0x1182F}, /* 1182E */
  {0x11830,0x11830,0x11830},  {0x11831,0x11831,0x11831}, /* 11830 */
  {0x11832,0x11832,0x11832},  {0x11833,0x11833,0x11833}, /* 11832 */
  {0x11834,0x11834,0x11834},  {0x11835,0x11835,0x11835}, /* 11834 */
  {0x11836,0x11836,0x11836},  {0x11837,0x11837,0x11837}, /* 11836 */
  {0x11838,0x11838,0x11838},  {0x11839,0x11839,0x11839}, /* 11838 */
  {0x1183A,0x1183A,0x1183A},  {0x1183B,0x1183B,0x1183B}, /* 1183A */
  {0x1183C,0x1183C,0x1183C},  {0x1183D,0x1183D,0x1183D}, /* 1183C */
  {0x1183E,0x1183E,0x1183E},  {0x1183F,0x1183F,0x1183F}, /* 1183E */
  {0x11840,0x11840,0x11840},  {0x11841,0x11841,0x11841}, /* 11840 */
  {0x11842,0x11842,0x11842},  {0x11843,0x11843,0x11843}, /* 11842 */
  {0x11844,0x11844,0x11844},  {0x11845,0x11845,0x11845}, /* 11844 */
  {0x11846,0x11846,0x11846},  {0x11847,0x11847,0x11847}, /* 11846 */
  {0x11848,0x11848,0x11848},  {0x11849,0x11849,0x11849}, /* 11848 */
  {0x1184A,0x1184A,0x1184A},  {0x1184B,0x1184B,0x1184B}, /* 1184A */
  {0x1184C,0x1184C,0x1184C},  {0x1184D,0x1184D,0x1184D}, /* 1184C */
  {0x1184E,0x1184E,0x1184E},  {0x1184F,0x1184F,0x1184F}, /* 1184E */
  {0x11850,0x11850,0x11850},  {0x11851,0x11851,0x11851}, /* 11850 */
  {0x11852,0x11852,0x11852},  {0x11853,0x11853,0x11853}, /* 11852 */
  {0x11854,0x11854,0x11854},  {0x11855,0x11855,0x11855}, /* 11854 */
  {0x11856,0x11856,0x11856},  {0x11857,0x11857,0x11857}, /* 11856 */
  {0x11858,0x11858,0x11858},  {0x11859,0x11859,0x11859}, /* 11858 */
  {0x1185A,0x1185A,0x1185A},  {0x1185B,0x1185B,0x1185B}, /* 1185A */
  {0x1185C,0x1185C,0x1185C},  {0x1185D,0x1185D,0x1185D}, /* 1185C */
  {0x1185E,0x1185E,0x1185E},  {0x1185F,0x1185F,0x1185F}, /* 1185E */
  {0x11860,0x11860,0x11860},  {0x11861,0x11861,0x11861}, /* 11860 */
  {0x11862,0x11862,0x11862},  {0x11863,0x11863,0x11863}, /* 11862 */
  {0x11864,0x11864,0x11864},  {0x11865,0x11865,0x11865}, /* 11864 */
  {0x11866,0x11866,0x11866},  {0x11867,0x11867,0x11867}, /* 11866 */
  {0x11868,0x11868,0x11868},  {0x11869,0x11869,0x11869}, /* 11868 */
  {0x1186A,0x1186A,0x1186A},  {0x1186B,0x1186B,0x1186B}, /* 1186A */
  {0x1186C,0x1186C,0x1186C},  {0x1186D,0x1186D,0x1186D}, /* 1186C */
  {0x1186E,0x1186E,0x1186E},  {0x1186F,0x1186F,0x1186F}, /* 1186E */
  {0x11870,0x11870,0x11870},  {0x11871,0x11871,0x11871}, /* 11870 */
  {0x11872,0x11872,0x11872},  {0x11873,0x11873,0x11873}, /* 11872 */
  {0x11874,0x11874,0x11874},  {0x11875,0x11875,0x11875}, /* 11874 */
  {0x11876,0x11876,0x11876},  {0x11877,0x11877,0x11877}, /* 11876 */
  {0x11878,0x11878,0x11878},  {0x11879,0x11879,0x11879}, /* 11878 */
  {0x1187A,0x1187A,0x1187A},  {0x1187B,0x1187B,0x1187B}, /* 1187A */
  {0x1187C,0x1187C,0x1187C},  {0x1187D,0x1187D,0x1187D}, /* 1187C */
  {0x1187E,0x1187E,0x1187E},  {0x1187F,0x1187F,0x1187F}, /* 1187E */
  {0x11880,0x11880,0x11880},  {0x11881,0x11881,0x11881}, /* 11880 */
  {0x11882,0x11882,0x11882},  {0x11883,0x11883,0x11883}, /* 11882 */
  {0x11884,0x11884,0x11884},  {0x11885,0x11885,0x11885}, /* 11884 */
  {0x11886,0x11886,0x11886},  {0x11887,0x11887,0x11887}, /* 11886 */
  {0x11888,0x11888,0x11888},  {0x11889,0x11889,0x11889}, /* 11888 */
  {0x1188A,0x1188A,0x1188A},  {0x1188B,0x1188B,0x1188B}, /* 1188A */
  {0x1188C,0x1188C,0x1188C},  {0x1188D,0x1188D,0x1188D}, /* 1188C */
  {0x1188E,0x1188E,0x1188E},  {0x1188F,0x1188F,0x1188F}, /* 1188E */
  {0x11890,0x11890,0x11890},  {0x11891,0x11891,0x11891}, /* 11890 */
  {0x11892,0x11892,0x11892},  {0x11893,0x11893,0x11893}, /* 11892 */
  {0x11894,0x11894,0x11894},  {0x11895,0x11895,0x11895}, /* 11894 */
  {0x11896,0x11896,0x11896},  {0x11897,0x11897,0x11897}, /* 11896 */
  {0x11898,0x11898,0x11898},  {0x11899,0x11899,0x11899}, /* 11898 */
  {0x1189A,0x1189A,0x1189A},  {0x1189B,0x1189B,0x1189B}, /* 1189A */
  {0x1189C,0x1189C,0x1189C},  {0x1189D,0x1189D,0x1189D}, /* 1189C */
  {0x1189E,0x1189E,0x1189E},  {0x1189F,0x1189F,0x1189F}, /* 1189E */
  {0x118A0,0x118C0,0x118A0},  {0x118A1,0x118C1,0x118A1}, /* 118A0 */
  {0x118A2,0x118C2,0x118A2},  {0x118A3,0x118C3,0x118A3}, /* 118A2 */
  {0x118A4,0x118C4,0x118A4},  {0x118A5,0x118C5,0x118A5}, /* 118A4 */
  {0x118A6,0x118C6,0x118A6},  {0x118A7,0x118C7,0x118A7}, /* 118A6 */
  {0x118A8,0x118C8,0x118A8},  {0x118A9,0x118C9,0x118A9}, /* 118A8 */
  {0x118AA,0x118CA,0x118AA},  {0x118AB,0x118CB,0x118AB}, /* 118AA */
  {0x118AC,0x118CC,0x118AC},  {0x118AD,0x118CD,0x118AD}, /* 118AC */
  {0x118AE,0x118CE,0x118AE},  {0x118AF,0x118CF,0x118AF}, /* 118AE */
  {0x118B0,0x118D0,0x118B0},  {0x118B1,0x118D1,0x118B1}, /* 118B0 */
  {0x118B2,0x118D2,0x118B2},  {0x118B3,0x118D3,0x118B3}, /* 118B2 */
  {0x118B4,0x118D4,0x118B4},  {0x118B5,0x118D5,0x118B5}, /* 118B4 */
  {0x118B6,0x118D6,0x118B6},  {0x118B7,0x118D7,0x118B7}, /* 118B6 */
  {0x118B8,0x118D8,0x118B8},  {0x118B9,0x118D9,0x118B9}, /* 118B8 */
  {0x118BA,0x118DA,0x118BA},  {0x118BB,0x118DB,0x118BB}, /* 118BA */
  {0x118BC,0x118DC,0x118BC},  {0x118BD,0x118DD,0x118BD}, /* 118BC */
  {0x118BE,0x118DE,0x118BE},  {0x118BF,0x118DF,0x118BF}, /* 118BE */
  {0x118A0,0x118C0,0x118A0},  {0x118A1,0x118C1,0x118A1}, /* 118C0 */
  {0x118A2,0x118C2,0x118A2},  {0x118A3,0x118C3,0x118A3}, /* 118C2 */
  {0x118A4,0x118C4,0x118A4},  {0x118A5,0x118C5,0x118A5}, /* 118C4 */
  {0x118A6,0x118C6,0x118A6},  {0x118A7,0x118C7,0x118A7}, /* 118C6 */
  {0x118A8,0x118C8,0x118A8},  {0x118A9,0x118C9,0x118A9}, /* 118C8 */
  {0x118AA,0x118CA,0x118AA},  {0x118AB,0x118CB,0x118AB}, /* 118CA */
  {0x118AC,0x118CC,0x118AC},  {0x118AD,0x118CD,0x118AD}, /* 118CC */
  {0x118AE,0x118CE,0x118AE},  {0x118AF,0x118CF,0x118AF}, /* 118CE */
  {0x118B0,0x118D0,0x118B0},  {0x118B1,0x118D1,0x118B1}, /* 118D0 */
  {0x118B2,0x118D2,0x118B2},  {0x118B3,0x118D3,0x118B3}, /* 118D2 */
  {0x118B4,0x118D4,0x118B4},  {0x118B5,0x118D5,0x118B5}, /* 118D4 */
  {0x118B6,0x118D6,0x118B6},  {0x118B7,0x118D7,0x118B7}, /* 118D6 */
  {0x118B8,0x118D8,0x118B8},  {0x118B9,0x118D9,0x118B9}, /* 118D8 */
  {0x118BA,0x118DA,0x118BA},  {0x118BB,0x118DB,0x118BB}, /* 118DA */
  {0x118BC,0x118DC,0x118BC},  {0x118BD,0x118DD,0x118BD}, /* 118DC */
  {0x118BE,0x118DE,0x118BE},  {0x118BF,0x118DF,0x118BF}, /* 118DE */
  {0x118E0,0x118E0,0x118E0},  {0x118E1,0x118E1,0x118E1}, /* 118E0 */
  {0x118E2,0x118E2,0x118E2},  {0x118E3,0x118E3,0x118E3}, /* 118E2 */
  {0x118E4,0x118E4,0x118E4},  {0x118E5,0x118E5,0x118E5}, /* 118E4 */
  {0x118E6,0x118E6,0x118E6},  {0x118E7,0x118E7,0x118E7}, /* 118E6 */
  {0x118E8,0x118E8,0x118E8},  {0x118E9,0x118E9,0x118E9}, /* 118E8 */
  {0x118EA,0x118EA,0x118EA},  {0x118EB,0x118EB,0x118EB}, /* 118EA */
  {0x118EC,0x118EC,0x118EC},  {0x118ED,0x118ED,0x118ED}, /* 118EC */
  {0x118EE,0x118EE,0x118EE},  {0x118EF,0x118EF,0x118EF}, /* 118EE */
  {0x118F0,0x118F0,0x118F0},  {0x118F1,0x118F1,0x118F1}, /* 118F0 */
  {0x118F2,0x118F2,0x118F2},  {0x118F3,0x118F3,0x118F3}, /* 118F2 */
  {0x118F4,0x118F4,0x118F4},  {0x118F5,0x118F5,0x118F5}, /* 118F4 */
  {0x118F6,0x118F6,0x118F6},  {0x118F7,0x118F7,0x118F7}, /* 118F6 */
  {0x118F8,0x118F8,0x118F8},  {0x118F9,0x118F9,0x118F9}, /* 118F8 */
  {0x118FA,0x118FA,0x118FA},  {0x118FB,0x118FB,0x118FB}, /* 118FA */
  {0x118FC,0x118FC,0x118FC},  {0x118FD,0x118FD,0x118FD}, /* 118FC */
  {0x118FE,0x118FE,0x118FE},  {0x118FF,0x118FF,0x118FF}  /* 118FE */
};

static const MY_UNICASE_CHARACTER u900p1E9[]= {
  {0x1E900,0x1E922,0x1E900},  {0x1E901,0x1E923,0x1E901}, /* 1E900 */
  {0x1E902,0x1E924,0x1E902},  {0x1E903,0x1E925,0x1E903}, /* 1E902 */
  {0x1E904,0x1E926,0x1E904},  {0x1E905,0x1E927,0x1E905}, /* 1E904 */
  {0x1E906,0x1E928,0x1E906},  {0x1E907,0x1E929,0x1E907}, /* 1E906 */
  {0x1E908,0x1E92A,0x1E908},  {0x1E909,0x1E92B,0x1E909}, /* 1E908 */
  {0x1E90A,0x1E92C,0x1E90A},  {0x1E90B,0x1E92D,0x1E90B}, /* 1E90A */
  {0x1E90C,0x1E92E,0x1E90C},  {0x1E90D,0x1E92F,0x1E90D}, /* 1E90C */
  {0x1E90E,0x1E930,0x1E90E},  {0x1E90F,0x1E931,0x1E90F}, /* 1E90E */
  {0x1E910,0x1E932,0x1E910},  {0x1E911,0x1E933,0x1E911}, /* 1E910 */
  {0x1E912,0x1E934,0x1E912},  {0x1E913,0x1E935,0x1E913}, /* 1E912 */
  {0x1E914,0x1E936,0x1E914},  {0x1E915,0x1E937,0x1E915}, /* 1E914 */
  {0x1E916,0x1E938,0x1E916},  {0x1E917,0x1E939,0x1E917}, /* 1E916 */
  {0x1E918,0x1E93A,0x1E918},  {0x1E919,0x1E93B,0x1E919}, /* 1E918 */
  {0x1E91A,0x1E93C,0x1E91A},  {0x1E91B,0x1E93D,0x1E91B}, /* 1E91A */
  {0x1E91C,0x1E93E,0x1E91C},  {0x1E91D,0x1E93F,0x1E91D}, /* 1E91C */
  {0x1E91E,0x1E940,0x1E91E},  {0x1E91F,0x1E941,0x1E91F}, /* 1E91E */
  {0x1E920,0x1E942,0x1E920},  {0x1E921,0x1E943,0x1E921}, /* 1E920 */
  {0x1E900,0x1E922,0x1E900},  {0x1E901,0x1E923,0x1E901}, /* 1E922 */
  {0x1E902,0x1E924,0x1E902},  {0x1E903,0x1E925,0x1E903}, /* 1E924 */
  {0x1E904,0x1E926,0x1E904},  {0x1E905,0x1E927,0x1E905}, /* 1E926 */
  {0x1E906,0x1E928,0x1E906},  {0x1E907,0x1E929,0x1E907}, /* 1E928 */
  {0x1E908,0x1E92A,0x1E908},  {0x1E909,0x1E92B,0x1E909}, /* 1E92A */
  {0x1E90A,0x1E92C,0x1E90A},  {0x1E90B,0x1E92D,0x1E90B}, /* 1E92C */
  {0x1E90C,0x1E92E,0x1E90C},  {0x1E90D,0x1E92F,0x1E90D}, /* 1E92E */
  {0x1E90E,0x1E930,0x1E90E},  {0x1E90F,0x1E931,0x1E90F}, /* 1E930 */
  {0x1E910,0x1E932,0x1E910},  {0x1E911,0x1E933,0x1E911}, /* 1E932 */
  {0x1E912,0x1E934,0x1E912},  {0x1E913,0x1E935,0x1E913}, /* 1E934 */
  {0x1E914,0x1E936,0x1E914},  {0x1E915,0x1E937,0x1E915}, /* 1E936 */
  {0x1E916,0x1E938,0x1E916},  {0x1E917,0x1E939,0x1E917}, /* 1E938 */
  {0x1E918,0x1E93A,0x1E918},  {0x1E919,0x1E93B,0x1E919}, /* 1E93A */
  {0x1E91A,0x1E93C,0x1E91A},  {0x1E91B,0x1E93D,0x1E91B}, /* 1E93C */
  {0x1E91C,0x1E93E,0x1E91C},  {0x1E91D,0x1E93F,0x1E91D}, /* 1E93E */
  {0x1E91E,0x1E940,0x1E91E},  {0x1E91F,0x1E941,0x1E91F}, /* 1E940 */
  {0x1E920,0x1E942,0x1E920},  {0x1E921,0x1E943,0x1E921}, /* 1E942 */
  {0x1E944,0x1E944,0x1E944},  {0x1E945,0x1E945,0x1E945}, /* 1E944 */
  {0x1E946,0x1E946,0x1E946},  {0x1E947,0x1E947,0x1E947}, /* 1E946 */
  {0x1E948,0x1E948,0x1E948},  {0x1E949,0x1E949,0x1E949}, /* 1E948 */
  {0x1E94A,0x1E94A,0x1E94A},  {0x1E94B,0x1E94B,0x1E94B}, /* 1E94A */
  {0x1E94C,0x1E94C,0x1E94C},  {0x1E94D,0x1E94D,0x1E94D}, /* 1E94C */
  {0x1E94E,0x1E94E,0x1E94E},  {0x1E94F,0x1E94F,0x1E94F}, /* 1E94E */
  {0x1E950,0x1E950,0x1E950},  {0x1E951,0x1E951,0x1E951}, /* 1E950 */
  {0x1E952,0x1E952,0x1E952},  {0x1E953,0x1E953,0x1E953}, /* 1E952 */
  {0x1E954,0x1E954,0x1E954},  {0x1E955,0x1E955,0x1E955}, /* 1E954 */
  {0x1E956,0x1E956,0x1E956},  {0x1E957,0x1E957,0x1E957}, /* 1E956 */
  {0x1E958,0x1E958,0x1E958},  {0x1E959,0x1E959,0x1E959}, /* 1E958 */
  {0x1E95A,0x1E95A,0x1E95A},  {0x1E95B,0x1E95B,0x1E95B}, /* 1E95A */
  {0x1E95C,0x1E95C,0x1E95C},  {0x1E95D,0x1E95D,0x1E95D}, /* 1E95C */
  {0x1E95E,0x1E95E,0x1E95E},  {0x1E95F,0x1E95F,0x1E95F}, /* 1E95E */
  {0x1E960,0x1E960,0x1E960},  {0x1E961,0x1E961,0x1E961}, /* 1E960 */
  {0x1E962,0x1E962,0x1E962},  {0x1E963,0x1E963,0x1E963}, /* 1E962 */
  {0x1E964,0x1E964,0x1E964},  {0x1E965,0x1E965,0x1E965}, /* 1E964 */
  {0x1E966,0x1E966,0x1E966},  {0x1E967,0x1E967,0x1E967}, /* 1E966 */
  {0x1E968,0x1E968,0x1E968},  {0x1E969,0x1E969,0x1E969}, /* 1E968 */
  {0x1E96A,0x1E96A,0x1E96A},  {0x1E96B,0x1E96B,0x1E96B}, /* 1E96A */
  {0x1E96C,0x1E96C,0x1E96C},  {0x1E96D,0x1E96D,0x1E96D}, /* 1E96C */
  {0x1E96E,0x1E96E,0x1E96E},  {0x1E96F,0x1E96F,0x1E96F}, /* 1E96E */
  {0x1E970,0x1E970,0x1E970},  {0x1E971,0x1E971,0x1E971}, /* 1E970 */
  {0x1E972,0x1E972,0x1E972},  {0x1E973,0x1E973,0x1E973}, /* 1E972 */
  {0x1E974,0x1E974,0x1E974},  {0x1E975,0x1E975,0x1E975}, /* 1E974 */
  {0x1E976,0x1E976,0x1E976},  {0x1E977,0x1E977,0x1E977}, /* 1E976 */
  {0x1E978,0x1E978,0x1E978},  {0x1E979,0x1E979,0x1E979}, /* 1E978 */
  {0x1E97A,0x1E97A,0x1E97A},  {0x1E97B,0x1E97B,0x1E97B}, /* 1E97A */
  {0x1E97C,0x1E97C,0x1E97C},  {0x1E97D,0x1E97D,0x1E97D}, /* 1E97C */
  {0x1E97E,0x1E97E,0x1E97E},  {0x1E97F,0x1E97F,0x1E97F}, /* 1E97E */
  {0x1E980,0x1E980,0x1E980},  {0x1E981,0x1E981,0x1E981}, /* 1E980 */
  {0x1E982,0x1E982,0x1E982},  {0x1E983,0x1E983,0x1E983}, /* 1E982 */
  {0x1E984,0x1E984,0x1E984},  {0x1E985,0x1E985,0x1E985}, /* 1E984 */
  {0x1E986,0x1E986,0x1E986},  {0x1E987,0x1E987,0x1E987}, /* 1E986 */
  {0x1E988,0x1E988,0x1E988},  {0x1E989,0x1E989,0x1E989}, /* 1E988 */
  {0x1E98A,0x1E98A,0x1E98A},  {0x1E98B,0x1E98B,0x1E98B}, /* 1E98A */
  {0x1E98C,0x1E98C,0x1E98C},  {0x1E98D,0x1E98D,0x1E98D}, /* 1E98C */
  {0x1E98E,0x1E98E,0x1E98E},  {0x1E98F,0x1E98F,0x1E98F}, /* 1E98E */
  {0x1E990,0x1E990,0x1E990},  {0x1E991,0x1E991,0x1E991}, /* 1E990 */
  {0x1E992,0x1E992,0x1E992},  {0x1E993,0x1E993,0x1E993}, /* 1E992 */
  {0x1E994,0x1E994,0x1E994},  {0x1E995,0x1E995,0x1E995}, /* 1E994 */
  {0x1E996,0x1E996,0x1E996},  {0x1E997,0x1E997,0x1E997}, /* 1E996 */
  {0x1E998,0x1E998,0x1E998},  {0x1E999,0x1E999,0x1E999}, /* 1E998 */
  {0x1E99A,0x1E99A,0x1E99A},  {0x1E99B,0x1E99B,0x1E99B}, /* 1E99A */
  {0x1E99C,0x1E99C,0x1E99C},  {0x1E99D,0x1E99D,0x1E99D}, /* 1E99C */
  {0x1E99E,0x1E99E,0x1E99E},  {0x1E99F,0x1E99F,0x1E99F}, /* 1E99E */
  {0x1E9A0,0x1E9A0,0x1E9A0},  {0x1E9A1,0x1E9A1,0x1E9A1}, /* 1E9A0 */
  {0x1E9A2,0x1E9A2,0x1E9A2},  {0x1E9A3,0x1E9A3,0x1E9A3}, /* 1E9A2 */
  {0x1E9A4,0x1E9A4,0x1E9A4},  {0x1E9A5,0x1E9A5,0x1E9A5}, /* 1E9A4 */
  {0x1E9A6,0x1E9A6,0x1E9A6},  {0x1E9A7,0x1E9A7,0x1E9A7}, /* 1E9A6 */
  {0x1E9A8,0x1E9A8,0x1E9A8},  {0x1E9A9,0x1E9A9,0x1E9A9}, /* 1E9A8 */
  {0x1E9AA,0x1E9AA,0x1E9AA},  {0x1E9AB,0x1E9AB,0x1E9AB}, /* 1E9AA */
  {0x1E9AC,0x1E9AC,0x1E9AC},  {0x1E9AD,0x1E9AD,0x1E9AD}, /* 1E9AC */
  {0x1E9AE,0x1E9AE,0x1E9AE},  {0x1E9AF,0x1E9AF,0x1E9AF}, /* 1E9AE */
  {0x1E9B0,0x1E9B0,0x1E9B0},  {0x1E9B1,0x1E9B1,0x1E9B1}, /* 1E9B0 */
  {0x1E9B2,0x1E9B2,0x1E9B2},  {0x1E9B3,0x1E9B3,0x1E9B3}, /* 1E9B2 */
  {0x1E9B4,0x1E9B4,0x1E9B4},  {0x1E9B5,0x1E9B5,0x1E9B5}, /* 1E9B4 */
  {0x1E9B6,0x1E9B6,0x1E9B6},  {0x1E9B7,0x1E9B7,0x1E9B7}, /* 1E9B6 */
  {0x1E9B8,0x1E9B8,0x1E9B8},  {0x1E9B9,0x1E9B9,0x1E9B9}, /* 1E9B8 */
  {0x1E9BA,0x1E9BA,0x1E9BA},  {0x1E9BB,0x1E9BB,0x1E9BB}, /* 1E9BA */
  {0x1E9BC,0x1E9BC,0x1E9BC},  {0x1E9BD,0x1E9BD,0x1E9BD}, /* 1E9BC */
  {0x1E9BE,0x1E9BE,0x1E9BE},  {0x1E9BF,0x1E9BF,0x1E9BF}, /* 1E9BE */
  {0x1E9C0,0x1E9C0,0x1E9C0},  {0x1E9C1,0x1E9C1,0x1E9C1}, /* 1E9C0 */
  {0x1E9C2,0x1E9C2,0x1E9C2},  {0x1E9C3,0x1E9C3,0x1E9C3}, /* 1E9C2 */
  {0x1E9C4,0x1E9C4,0x1E9C4},  {0x1E9C5,0x1E9C5,0x1E9C5}, /* 1E9C4 */
  {0x1E9C6,0x1E9C6,0x1E9C6},  {0x1E9C7,0x1E9C7,0x1E9C7}, /* 1E9C6 */
  {0x1E9C8,0x1E9C8,0x1E9C8},  {0x1E9C9,0x1E9C9,0x1E9C9}, /* 1E9C8 */
  {0x1E9CA,0x1E9CA,0x1E9CA},  {0x1E9CB,0x1E9CB,0x1E9CB}, /* 1E9CA */
  {0x1E9CC,0x1E9CC,0x1E9CC},  {0x1E9CD,0x1E9CD,0x1E9CD}, /* 1E9CC */
  {0x1E9CE,0x1E9CE,0x1E9CE},  {0x1E9CF,0x1E9CF,0x1E9CF}, /* 1E9CE */
  {0x1E9D0,0x1E9D0,0x1E9D0},  {0x1E9D1,0x1E9D1,0x1E9D1}, /* 1E9D0 */
  {0x1E9D2,0x1E9D2,0x1E9D2},  {0x1E9D3,0x1E9D3,0x1E9D3}, /* 1E9D2 */
  {0x1E9D4,0x1E9D4,0x1E9D4},  {0x1E9D5,0x1E9D5,0x1E9D5}, /* 1E9D4 */
  {0x1E9D6,0x1E9D6,0x1E9D6},  {0x1E9D7,0x1E9D7,0x1E9D7}, /* 1E9D6 */
  {0x1E9D8,0x1E9D8,0x1E9D8},  {0x1E9D9,0x1E9D9,0x1E9D9}, /* 1E9D8 */
  {0x1E9DA,0x1E9DA,0x1E9DA},  {0x1E9DB,0x1E9DB,0x1E9DB}, /* 1E9DA */
  {0x1E9DC,0x1E9DC,0x1E9DC},  {0x1E9DD,0x1E9DD,0x1E9DD}, /* 1E9DC */
  {0x1E9DE,0x1E9DE,0x1E9DE},  {0x1E9DF,0x1E9DF,0x1E9DF}, /* 1E9DE */
  {0x1E9E0,0x1E9E0,0x1E9E0},  {0x1E9E1,0x1E9E1,0x1E9E1}, /* 1E9E0 */
  {0x1E9E2,0x1E9E2,0x1E9E2},  {0x1E9E3,0x1E9E3,0x1E9E3}, /* 1E9E2 */
  {0x1E9E4,0x1E9E4,0x1E9E4},  {0x1E9E5,0x1E9E5,0x1E9E5}, /* 1E9E4 */
  {0x1E9E6,0x1E9E6,0x1E9E6},  {0x1E9E7,0x1E9E7,0x1E9E7}, /* 1E9E6 */
  {0x1E9E8,0x1E9E8,0x1E9E8},  {0x1E9E9,0x1E9E9,0x1E9E9}, /* 1E9E8 */
  {0x1E9EA,0x1E9EA,0x1E9EA},  {0x1E9EB,0x1E9EB,0x1E9EB}, /* 1E9EA */
  {0x1E9EC,0x1E9EC,0x1E9EC},  {0x1E9ED,0x1E9ED,0x1E9ED}, /* 1E9EC */
  {0x1E9EE,0x1E9EE,0x1E9EE},  {0x1E9EF,0x1E9EF,0x1E9EF}, /* 1E9EE */
  {0x1E9F0,0x1E9F0,0x1E9F0},  {0x1E9F1,0x1E9F1,0x1E9F1}, /* 1E9F0 */
  {0x1E9F2,0x1E9F2,0x1E9F2},  {0x1E9F3,0x1E9F3,0x1E9F3}, /* 1E9F2 */
  {0x1E9F4,0x1E9F4,0x1E9F4},  {0x1E9F5,0x1E9F5,0x1E9F5}, /* 1E9F4 */
  {0x1E9F6,0x1E9F6,0x1E9F6},  {0x1E9F7,0x1E9F7,0x1E9F7}, /* 1E9F6 */
  {0x1E9F8,0x1E9F8,0x1E9F8},  {0x1E9F9,0x1E9F9,0x1E9F9}, /* 1E9F8 */
  {0x1E9FA,0x1E9FA,0x1E9FA},  {0x1E9FB,0x1E9FB,0x1E9FB}, /* 1E9FA */
  {0x1E9FC,0x1E9FC,0x1E9FC},  {0x1E9FD,0x1E9FD,0x1E9FD}, /* 1E9FC */
  {0x1E9FE,0x1E9FE,0x1E9FE},  {0x1E9FF,0x1E9FF,0x1E9FF}  /* 1E9FE */
};

const MY_UNICASE_CHARACTER *my_unicase_pages_unicode900[4352]= {
 u900p000, u900p001, u900p002, u900p003, u900p004, u900p005,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
 u900p010,     NULL,     NULL, u900p013,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL, u900p01C, u900p01D, u900p01E, u900p01F,
     NULL, u900p021,     NULL,     NULL, u900p024,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL, u900p02C, u900p02D,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL, u900p0A6, u900p0A7,
     NULL,     NULL,     NULL, u900p0AB,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL, u900p0FF,
     NULL,     NULL,     NULL,     NULL, u900p104,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL, u900p10C,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
 u900p118,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL, u900p1E9,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,
};

MY_UNICASE_INFO my_unicase_unicode900=
{
  0x10FFFF,
  my_unicase_pages_unicode900
};
#endif
